Package: wrMisc
Version: 2.1.0
Title: Analyze Experimental High-Throughput (Omics) Data
Authors@R: person("Wolfgang", "Raffelsberger", email = "w.raffelsberger@gmail.com", role = c("aut", "cre"))
Author: Wolfgang Raffelsberger [aut, cre]
Maintainer: Wolfgang Raffelsberger <w.raffelsberger@gmail.com>
Description: The efficient treatment and convenient analysis of experimental high-throughput (omics) data gets facilitated through this collection of diverse functions. 
  Several functions address advanced object-conversions, like manipulating lists of lists or lists of arrays, reorganizing lists to arrays or into separate vectors, merging of multiple entries, etc.  
  Another set of functions provides speed-optimized calculation of standard deviation (sd), coefficient of variance (CV) or standard error of the mean (SEM)  
  for data in matrixes or means per line with respect to additional grouping (eg n groups of replicates). 
  A group of functions facilitate dealing with non-redundant information, by indexing unique, adding counters to redundant or eliminating lines with respect redundancy in a given reference-column, etc. 
  Help is provided to identify very closely matching numeric values to generate (partial) distance matrixes for very big data in a memory efficient manner or to reduce the complexity of large data-sets by combining very close values. 
  Other functions help aligning a matrix or data.frame to a reference using partial matching or to mine an experimental setup to extract patterns of replicate samples.
  Many times large experimental datasets need some additional filtering, adequate functions are provided. 
  Convenient data normalization is supported in various different modes, parameter estimation via permutations or boot-strap as well as flexible testing of multiple pair-wise combinations using the framework of 'limma' is provided, too.
  Batch reading (or writing) of sets of files and combining data to arrays is supported, too. 
VignetteBuilder: knitr
Depends: R (>= 3.1.0)
Imports: grDevices, graphics, MASS, stats, utils
Suggests: BBmisc, boot, coin, data.table, data.tree, fdrtool,
        flexclust, knitr, limma, markdown, mixdist, NbClust,
        preprocessCore, qvalue, Rcpp, RColorBrewer, readxl, rmarkdown,
        som, stringi, VGAM, vsn, wrGraph
License: GPL-3
Encoding: UTF-8
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-06-08 09:54:54 UTC; wraff
Repository: CRAN
Date/Publication: 2026-06-08 19:10:02 UTC
Built: R 4.5.2; ; 2026-06-08 21:56:07 UTC; unix

# Generated by roxygen2: do not edit by hand

export(.addLetterWoLast)
export(.allRatioMatr1to2)
export(.allRatios)
export(.arrLstMean)
export(.arrLstSEM)
export(.asDF2)
export(.breakInSer)
export(.bringToCtr)
export(.checkArgNa)
export(.checkConsistentArrList)
export(.checkConvt2Vect)
export(.checkFactor)
export(.checkFileNameExtensions)
export(.checkLegendLoc)
export(.checkLmConfInt)
export(.checkRegrArguments)
export(.chooseGrpCol)
export(.combineListAnnot)
export(.compareByDiff)
export(.compareByLogRatio)
export(.compareByPPM)
export(.complCols)
export(.composeCallName)
export(.convertMatrToNum)
export(.convertNa)
export(.corDuplItemsByIncrem)
export(.cutAtSearch)
export(.cutStr)
export(.datSlope)
export(.doubleExt)
export(.extrNAneighb)
export(.extrNumHeadingCap)
export(.extrNumHeadingSepChar)
export(.filtSize)
export(.filterNetw)
export(.filterSw)
export(.findBorderOverlaps)
export(.firstMin)
export(.fuse2ArrBy2ndDim)
export(.getAmean)
export(.getAmean2)
export(.getMvalue2)
export(.getPWseparator)
export(.growTree)
export(.insp1dimByClustering)
export(.inspectHeader)
export(.keepCenter1d)
export(.keepFiniteCol)
export(.mayBeNum)
export(.medianSpecGrp)
export(.mergeMatrices)
export(.minDif)
export(.neigbDis)
export(.normConstSlope)
export(.normalize)
export(.pasteCols)
export(.plotCountPie)
export(.plusLowerCaps)
export(.predRes)
export(.raiseColLowest)
export(.removeCol)
export(.removeEmptyCol)
export(.replSpecChar)
export(.retain1stPart)
export(.rowGrpCV)
export(.rowGrpMeans)
export(.rowGrpSds)
export(.rowGrpSums)
export(.rowNorm)
export(.rowNormFact)
export(.scale01)
export(.scaleSpecGrp)
export(.scaleXY)
export(.seqCutStr)
export(.setLowestTo)
export(.sortMid)
export(.stackArray)
export(.summarizeCols)
export(.trimFromEnd)
export(.trimFromStart)
export(.trimLeft)
export(.trimRight)
export(.uniqueWName)
export(.vector2Matr)
export(XYToDiffPpm)
export(addBeforFileExtension)
export(addBeforeFileExtension)
export(adjBy2ptReg)
export(adjustUnitPrefix)
export(appendNR)
export(arrayCV)
export(asSepList)
export(blockNormalize)
export(buildTree)
export(cbindNR)
export(checkAvSd)
export(checkFilePath)
export(checkGrpOrder)
export(checkGrpOrderSEM)
export(checkSimValueInSer)
export(checkStrictOrder)
export(checkUnitPrefix)
export(checkVectLength)
export(chooseGroupNames)
export(cleanReplicates)
export(closeMatchMatrix)
export(coinPermTest)
export(colCVs)
export(colMedSds)
export(colSds)
export(colorAccording2)
export(combinatIntTable)
export(combineAsN)
export(combineByEitherFactor)
export(combineOverlapInfo)
export(combineRedBasedOnCol)
export(combineRedundLinesInList)
export(combineRedundLinesInListAcRef)
export(combineReplFromListToMatr)
export(combineSingleT)
export(completeArrLst)
export(concatMatch)
export(confInt)
export(contribToContigPerFrag)
export(conv01toColNa)
export(convColorToTransp)
export(convMatr2df)
export(convPairwiseSetup)
export(convToNum)
export(coordOfFilt)
export(correctToUnique)
export(correctWinPath)
export(countCloseToLimits)
export(countSameStartEnd)
export(cutArrayInCluLike)
export(cutAtMultSites)
export(cutToNgrp)
export(diffCombin)
export(diffPPM)
export(elimCloseCoord)
export(equLenNumber)
export(exclExtrValues)
export(exponNormalize)
export(extr1chan)
export(extrColsDeX)
export(extrNumericFromMatr)
export(extrSpcText)
export(extractLast2numericParts)
export(filt3dimArr)
export(filtSizeUniq)
export(filterLiColDeList)
export(filterList)
export(filterNetw)
export(findCloseMatch)
export(findHeadAndTail)
export(findRepeated)
export(findSimilFrom2sets)
export(findUsableGroupRange)
export(firstLineOfDat)
export(firstOfRepLines)
export(firstOfRepeated)
export(fuseAnnotMatr)
export(fuseCommonListElem)
export(fusePairs)
export(get1stOfRepeatedByCol)
export(getPWseparator)
export(getPairwiseSetup)
export(getValuesByUnique)
export(gitDataUrl)
export(htmlSpecCharConv)
export(indexGroupsFromPW)
export(keepCommonText)
export(levIndex)
export(linModelSelect)
export(linRegrParamAndPVal)
export(listBatchReplace)
export(listGroupsByNames)
export(lmSelClu)
export(lrbind)
export(makeMAList)
export(makeNRedMatr)
export(matchMatrixLinesToRef)
export(matchNamesWithReverseParts)
export(matchSampToPairw)
export(matr2list)
export(mergeMatrices)
export(mergeMatrixList)
export(mergeSelCol)
export(mergeSelCol3)
export(mergeVectors)
export(mergeW2)
export(minDiff)
export(moderTest2grp)
export(moderTestXgrp)
export(multiCharReplace)
export(multiMatch)
export(nFragments)
export(nFragments0)
export(nNonNumChar)
export(naOmit)
export(nonAmbiguousMat)
export(nonAmbiguousNum)
export(nonRedundLines)
export(nonredDataFrame)
export(normalizeThis)
export(numPairDeColNames)
export(orderMatrToRef)
export(organizeAsListOfRepl)
export(pVal2lfdr)
export(packageDownloadStat)
export(pairsAsPropensMatr)
export(partUnlist)
export(partialDist)
export(pasteC)
export(presenceFilt)
export(presenceGrpFilt)
export(protectSpecChar)
export(pwSeparatorList)
export(randIndFx)
export(rankToContigTab)
export(ratioAllComb)
export(ratioToPpm)
export(readCsvBatch)
export(readTabulatedBatch)
export(readVarColumns)
export(readXlsxBatch)
export(reduceTable)
export(regrBy1or2point)
export(regrMultBy1or2point)
export(renameColumns)
export(reorgByCluNo)
export(replNAbyLow)
export(replPlateCV)
export(replacePWseparator)
export(replicateStructure)
export(rmDupl2colMatr)
export(rmEnumeratorName)
export(rmOrphans)
export(rmSharedWords)
export(rnormW)
export(rowCVs)
export(rowGrpCV)
export(rowGrpMeans)
export(rowGrpNA)
export(rowGrpSds)
export(rowGrpSums)
export(rowMedSds)
export(rowNormalize)
export(rowSEMs)
export(rowSds)
export(sampNoDeMArrayLM)
export(scaleXY)
export(searchDataPairs)
export(searchLinesAtGivenSlope)
export(simpleFragFig)
export(singleLineAnova)
export(sortBy2CategorAnd1IntCol)
export(sortByNRepeated)
export(stableMode)
export(standardW)
export(stdErrMedBoot)
export(sumNAperGroup)
export(summarizeCols)
export(sysDate)
export(tTestAllVal)
export(tableToPlot)
export(test2factLimma)
export(top3mean)
export(transpGraySca)
export(treatTxtDuplicates)
export(triCoord)
export(trimRedundText)
export(trimmedMean)
export(unifyEnumerator)
export(uniqCountReport)
export(upperMaCoord)
export(vectorDistAngle)
export(volcanoFilter)
export(withinRefRange)
export(writeCsv)
importFrom(graphics,par)

## ----setup0, include=FALSE, echo=FALSE, messages=FALSE, warnings=FALSE--------
suppressPackageStartupMessages({
    library(wrMisc)
})

## ----install, echo=TRUE, eval=FALSE-------------------------------------------
# ## If not already installed, you'll have to install the package first.
# ## This is the basic installation commande in R
# install.packages("wrMisc")

## ----install2, echo=TRUE, eval=FALSE------------------------------------------
# packages <- c("knitr", "rmarkdown", "BiocManager", "kableExtra", "boot", "data.tree", "data.table",
#   "fdrtool", "RColorBrewer", "Rcpp", "wrMisc", "wrGraph", "wrProteo")
# checkInstallPkg <- function(pkg) {       # install function
#   if(!requireNamespace(pkg, quietly=TRUE)) install.packages(pkg) }
# 
# ## install if not yet present
# sapply(packages, checkInstallPkg)

## ----install3, echo=TRUE, eval=FALSE------------------------------------------
# ## Installation of limma
# BiocManager::install("limma")

## ----install4, echo=TRUE, eval=FALSE------------------------------------------
# ## Now you can open this vignette out of R:
# vignette("wrMiscVignette1", package="wrMisc")

## ----setup1-------------------------------------------------------------------
library("wrMisc")

## This is 'wrMisc' version number :
packageVersion("wrMisc")

## ----basicVariability, echo=TRUE----------------------------------------------
grp1 <- rep(LETTERS[1:3], c(3,4,3))
sampNa1 <- paste0(grp1, c(1:3,1:4,1:3))
set.seed(2016); dat1 <- matrix(round(c(runif(50000) +rep(1:1000,50)),3), 
  ncol=10, dimnames=list(NULL,sampNa1))
dim(dat1)
head(dat1)

## ----sdForEachRow, echo=TRUE--------------------------------------------------
head(rowSds(dat1))
system.time(sd1 <- rowSds(dat1))
system.time(sd2 <- apply(dat1, 1, sd))

## ----usingApply, echo=TRUE----------------------------------------------------
table(round(sd1, 13)==round(sd2, 13))

## ----calculateRowCV, echo=TRUE------------------------------------------------
system.time(cv1 <- rowCVs(dat1))
system.time(cv2 <- apply(dat1, 1, sd) / rowMeans(dat1))
# typically the calculation using rowCVs is much faster
head(cv1)
# results from the 'conventional' way
head(cv2)

## ----rowGrpMeans1, echo=TRUE--------------------------------------------------
# we already defined the grouping :
grp1

## the mean for each group and row
system.time(mean1Gr <- rowGrpMeans(dat1, grp1))

## ----sdOrCVbyGrp, echo=TRUE---------------------------------------------------
## Now the sd for each row and group
system.time(sd1Gr <- rowGrpSds(dat1, grp1))
# will give us a matrix with the sd for each group & line 
head(sd1Gr)

# Let's check the results of the first line :
sd1Gr[1,] == c(sd(dat1[1,1:3]), sd(dat1[1,4:7]), sd(dat1[1,8:10]))

# The CV :
system.time(cv1Gr <- rowGrpCV(dat1, grp1))
head(cv1Gr)

## ----rowGrpNA1, echo=TRUE-----------------------------------------------------
mat2 <- matrix(1, nrow=200, ncol=40)
set.seed(2026); mat2[sample.int(prod(dim(mat2)), nrow(mat2), replace=FALSE)] <- NA
set.seed(2025); grp2 <- as.factor(sample.int(n=8, size=ncol(mat2), replace=TRUE))
  table(grp2)

## ----rowGrpNA2, echo=TRUE-----------------------------------------------------
## number of NAs per row and group
head(rowGrpNA(mat2, grp2))

## ----rowGrpNA3, echo=TRUE-----------------------------------------------------
head(rowGrpNA(mat2, grp2, mode="ratio"))
head(rowGrpNA(mat2, grp2, mode="complete"))

## ----rowGrpNA4, echo=TRUE-----------------------------------------------------
## mimick output from testing from package wrProteo
dat1 <- list(isNA =mat2, setup=list(grp=grp2))
head(rowGrpNA(dat1, mode="simple"))
head(rowGrpNA(dat1, mode="complete"))
head(rowGrpNA(dat1, mode="ratioCollapse"))

## ----naOmit, echo=TRUE--------------------------------------------------------
aA <- c(11:13,NA,10,NA)
 
str(naOmit(aA))

# the 'classical' na.omit also stores which elements were NA
str(na.omit(aA))

## ----minDiff, echo=TRUE-------------------------------------------------------
set.seed(2017); aa <- 10 *c(0.1 +round(runif(20),2), 0.53, 0.53)
head(aa)

minDiff(aa,ppm=FALSE)

## ----partUnlist_1, echo=TRUE--------------------------------------------------
bb <- list(fa=gl(2,2), ve=31:33, L2=matrix(21:28,ncol=2), li=list(li1=11:14,li2=data.frame(41:44)))
partUnlist(bb)
partUnlist(lapply(bb,.asDF2))

## ----unlist_1, echo=TRUE------------------------------------------------------
head(unlist(bb, recursive=FALSE))

## ----asSepList, echo=TRUE-----------------------------------------------------
bb <- list(fa=gl(2,2), ve=31:33, L2=matrix(21:28,ncol=2), li=list(li1=11:14,li2=data.frame(41:44)))
asSepList(bb)

## ----lappend1, echo=TRUE------------------------------------------------------
li1 <- list(a=1, b=2, c=3)
li2 <- list(A=11, b=2, C=13)
append(li1, li2)

## ----lappend2, echo=TRUE------------------------------------------------------
appendNR(li1, li2)

## ----lrbind, echo=TRUE--------------------------------------------------------
dat2 <- matrix(11:34, ncol=3, dimnames=list(letters[1:8], colnames=LETTERS[1:3]))
lst2 <- by(dat2, rep(1:3,c(3,2,3)), as.matrix)
lst2

# join list-elements (back) into single matrix
lrbind(lst2)

## ----lrbind2, echo=TRUE-------------------------------------------------------
li2 <- list(a=11:13, b=matrix(21:26, nc=2, dimnames=list(letters[11:13],c("B","C"))), data.frame(d=41:43, e=51:53) )

## cbind like (to data.frame) 
as.data.frame(li2) 

## cbind like and convert into matrix (as long as all data are of same type, ie here all numeric)
as.matrix(as.data.frame(li2)) 

## ----mergeMatrixList, echo=TRUE-----------------------------------------------
mat1 <- matrix(11:18, ncol=2, dimnames=list(letters[3:6],LETTERS[1:2]))
mat2 <- matrix(21:28, ncol=2, dimnames=list(letters[2:5],LETTERS[3:4]))
mat3 <- matrix(31:38, ncol=2, dimnames=list(letters[c(1,3:4,3)],LETTERS[4:5]))
#
mergeMatrixList(list(mat1, mat2), useColumn="all")

# with custom names for the individual matrices
mergeMatrixList(list(m1=mat1, m2=mat2, mat3), mode="union", useColumn=2)

## ----mergeMatrices, echo=TRUE-------------------------------------------------
mergeMatrices(mat1, mat2)

mergeMatrices(mat1, mat2, mat3, mode="union", useColumn=2)

## custom names for matrix-origin
mergeMatrices(m1=mat1, m2=mat2, mat3, mode="union", useColumn=2)

## flexible/custom selection of columns
mergeMatrices(m1=mat1, m2=mat2, mat3, mode="union", useColumn=list(1,1:2,2))

## ----fuseCommonListElem, echo=TRUE--------------------------------------------
val1 <- 10 +1:26
names(val1) <- letters
(lst1 <- list(c=val1[3:6], a=val1[1:3], b=val1[2:3] ,a=val1[12], c=val1[13]))

## here the names 'a' and 'c' appear twice :
names(lst1)

## now, let's fuse all 'a' and 'c'
fuseCommonListElem(lst1)

## ----listBatchReplace1, echo=TRUE---------------------------------------------
lst1 <- list(m1=matrix(11:18, ncol=2), m2=matrix(21:30, ncol=2), indR=31:34,
  m3=matrix(c(21:23,NA,25:27,NA), ncol=2))
filterLiColDeList(lst1, useLines=2:3)
filterLiColDeList(lst1, useLines="allNA", ref=3)

## ----replInList1, echo=TRUE---------------------------------------------------
(lst1 <- list(aa=1:4, bb=c("abc","efg","abhh","effge"), cc=c("abdc","efg","efgh")))
listBatchReplace(lst1, search="efg", repl="EFG", silent=FALSE)

## ----listGroupsByNames, echo=TRUE---------------------------------------------
ser1 <- 1:7; names(ser1) <- c("AA","BB","AA.1","CC","AA.b","BB.e","A")

listGroupsByNames(ser1)

## ----listGroupsByNames2, echo=TRUE--------------------------------------------
listGroupsByNames((1:10)/5)

## ----filterList, echo=TRUE----------------------------------------------------
set.seed(2020); dat1 <- round(runif(80),2)
list1 <- list(m1=matrix(dat1[1:40], ncol=8), m2=matrix(dat1[41:80], ncol=8), other=letters[1:8])
rownames(list1$m1) <- rownames(list1$m2) <- paste0("line",1:5)
# Note: the list-element list1$other has a length different to that of filt. Thus, it won't get filtered.
filterList(list1, list1$m1[,1] >0.4)       # filter according to 1st column of $m1 ...
filterList(list1, list1$m1 >0.4) 

## ----matr2list, echo=TRUE-----------------------------------------------------
(mat1 <- matrix(1:12, ncol=3, dimnames=list(letters[1:4],LETTERS[1:3])))
str(matr2list(mat1))

## ----array0, echo=TRUE--------------------------------------------------------
(arr1 <- array(c(6:4,4:24), dim=c(4,3,2), dimnames=list(c(LETTERS[1:4]),
  paste("col",1:3,sep=""),c("ch1","ch2"))))

## ----arrayCV1, echo=TRUE------------------------------------------------------
arrayCV(arr1)

# this is equivalent to
cbind(rowCVs(arr1[,,1]), rowCVs(arr1[,,2]))

## ----arrayCV2, echo=TRUE------------------------------------------------------
arrayCV(arr1, byDim=2)

## ----cutArrayInCluLike, echo=TRUE---------------------------------------------
cutArrayInCluLike(arr1, cluOrg=c(2,1,2,1))

## ----filt3dimArr, echo=TRUE---------------------------------------------------
filt3dimArr(arr1, displCrit=c("col1","col2"), filtCrit="col2", filtVal=7, filtTy=">")

## ----repeated1, echo=TRUE-----------------------------------------------------
## some text toy data
tr <- c("li0","n",NA,NA, rep(c("li2","li3"),2), rep("n",4))

## ----repeated2, echo=TRUE-----------------------------------------------------
table(tr)
unique(tr) 
duplicated(tr, fromLast=FALSE)

## ----repeated3, echo=TRUE-----------------------------------------------------
aa <- c(11:16,NA,14:12,NA,14)
names(aa) <- letters[1:length(aa)]
aa

## ----findRepeated, echo=TRUE--------------------------------------------------
findRepeated(aa) 

## ----firstOfRepeated, echo=TRUE-----------------------------------------------
firstOfRepeated(aa)

aa[firstOfRepeated(aa)$indUniq]          # only unique with their names

unique(aa)                               # unique() does not return any names !

## ----correctToUnique1, echo=TRUE----------------------------------------------
correctToUnique(aa)

correctToUnique(aa, sep=".", NAenum=FALSE)       # keep NAs (ie without transforming to character)

## ----nonAmbiguousNum, echo=TRUE-----------------------------------------------
unique(aa)                                    # names are lost

nonAmbiguousNum(aa)
nonAmbiguousNum(aa, uniq=FALSE, asLi=TRUE)    # separate in list unique and repeated 

## ----sortByNRepeated, echo=TRUE-----------------------------------------------
cities <- c("Bangkok","London","Paris", "Singapore","New York City", "Istambul","Delhi","Rome","Dubai")
sortByNRepeated(x=cities[c(1:4)], y=cities[c(2:3,5:8)])

## or (unlimited) multiple inputs via list
choices1 <- list(Mary=cities[c(1:4)], Olivia=cities[c(2:3,5:8)], Paul=cities[c(5:3,9,5)])    # Note : Paul cited NYC twice !
table(unlist(choices1))
sortByNRepeated(choices1)
sortByNRepeated(choices1, filterIntraRep=FALSE)  # without correcting multiple citation of NYC by Paul

## ----cbindNR, echo=TRUE-------------------------------------------------------
## First we'll make some toy data :
(ma1 <- matrix(1:6, ncol=3, dimnames=list(1:2,LETTERS[3:1])))
(ma2 <- matrix(11:16, ncol=3, dimnames=list(1:2,LETTERS[3:5])))

## now we can join 2 or more matrixes  
cbindNR(ma1, ma2, summarizeAs="mean")       # average of both columns 'C'

## ----firstLineOfDat, echo=TRUE------------------------------------------------
(mat1 <- matrix(c(1:6, rep(1:3,1:3)), ncol=2, dimnames=list(letters[1:6],LETTERS[1:2])))

## ----firstLineOfDat2, echo=TRUE-----------------------------------------------
firstLineOfDat(mat1, refCol=2)

## ----firstOfRepLines, echo=TRUE-----------------------------------------------
mat2 <- matrix(c("e","n","a","n","z","z","n","z","z","b", 
  "","n","c","n","","","n","","","z"), ncol=2)
firstOfRepLines(mat2, out="conc")

# or as index :
firstOfRepLines(mat2)

## ----nonredDataFrame, echo=TRUE-----------------------------------------------
(df1 <- data.frame(cbind(xA=letters[1:5], xB=c("h","h","f","e","f"), xC=LETTERS[1:5])))

## ----nonredDataFrame2, echo=TRUE----------------------------------------------
nonredDataFrame(df1, useCol=c("xB","xC")) 

# without counter or concatenating
df1[which(!duplicated(df1[,2])),]
# or
df1[firstOfRepLines(df1,useCol=2),]

## ----get1stOfRepeatedByCol, echo=TRUE-----------------------------------------
mat2 <- cbind(no=as.character(1:20), seq=sample(LETTERS[1:15], 20, repl=TRUE),
  ty=sample(c("full","Nter","inter"),20,repl=TRUE), ambig=rep(NA,20), seqNa=1:20)
(mat2uniq <- get1stOfRepeatedByCol(mat2, sortBy="seq", sortSupl="ty"))

# the values from column 'seq' are indeed unique
table(mat2uniq[,"seq"])

# This will return all first repeated (may be >1) but without furter sorting 
#  along column 'ty' neither marking in comumn 'ambig').
mat2[which(duplicated(mat2[,2],fromLast=FALSE)),]

## ----nonAmbiguousMat, echo=TRUE-----------------------------------------------
nonAmbiguousMat(mat1,by=2)

## ----nonAmbiguousMat2, echo=TRUE----------------------------------------------
set.seed(2017); mat3 <- matrix(c(1:100,round(rnorm(200),2)), ncol=3,
  dimnames=list(1:100,LETTERS[1:3]));
head(mat3U <- nonAmbiguousMat(mat3, by="B", na="_", uniqO=FALSE), n=15)
head(get1stOfRepeatedByCol(mat3, sortB="B", sortS="B"))

## ----combineReplFromListToMatr, echo=TRUE-------------------------------------
lst2 <- list(aa_1x=matrix(1:12, nrow=4, byrow=TRUE), ab_2x=matrix(24:13, nrow=4, byrow=TRUE))
combineReplFromListToMatr(lst2)

## ----combineRedundLinesInListAcRef, echo=TRUE---------------------------------
x1 <- list(quant=matrix(11:34, ncol=3, dimnames=list(letters[8:1], LETTERS[11:13])), 
  annot=matrix(paste0(LETTERS[c(1:4,6,3:5)],LETTERS[c(1:4,6,3:5)]), ncol=1, 
  dimnames=list(paste(letters[1:8]),"xx")) )
combineRedundLinesInList(lst=x1, refNa="annot", datNa="quant", refColNa="xx")

## ----nonRedundLines, echo=TRUE------------------------------------------------
mat4 <- matrix(rep(c(1,1:3,3,1),2), ncol=2, dimnames=list(letters[1:6],LETTERS[1:2]))
nonRedundLines(mat4)

## ----filtSizeUniq, echo=TRUE--------------------------------------------------
# input: c and dd are repeated  :
filtSizeUniq(list(A="a", B=c("b","bb","c"), D=c("dd","d","ddd","c")), filtUn=TRUE, minSi=NULL)

# here a,b,c and dd are repeated  :
filtSizeUniq(list(A="a", B=c("b","bb","c"), D=c("dd","d","ddd","c")), ref=c(letters[c(1:26,1:3)],
  "dd","dd","bb","ddd"), filtUn=TRUE, minSi=NULL)   

## ----makeNRedMatr, echo=TRUE--------------------------------------------------
t3 <- data.frame(ref=rep(11:15,3), tx=letters[1:15],
  matrix(round(runif(30,-3,2),1), nc=2), stringsAsFactors=FALSE)
  
# First we split the data.frame in list  
by(t3,t3[,1], function(x) x)
t(sapply(by(t3,t3[,1],function(x) x), summarizeCols, me="maxAbsOfRef"))
(xt3 <- makeNRedMatr(t3, summ="mean", iniID="ref"))
(xt3 <- makeNRedMatr(t3, summ=unlist(list(X1="maxAbsOfRef")), iniID="ref"))

## ----makeNRedMatr2, echo=TRUE-------------------------------------------------
set.seed(2024)
df2 <- data.frame(transcrID=paste("TrID", 101:124, sep=""), geneID=paste("geID",rep(201:206,each=4)), 
  geneLe=round(runif(24, min=50, max=1500)), a=101:124, b=224:201)
df2 <- df2[-1*c(1,4:7,12:15),]  

(dfLongest <- makeNRedMatr(df2, summ=unlist(list(X1="maxOfRef")), iniID="geneID"))

 summarizeCols(df2[1:2,c(1:5,3)], me="min")  
 summarizeCols(df2[1:2,c(1:5,3)], me="mean") 
 summarizeCols(df2[1:2,c(1:5,3)], me="maxOfRef") 
 summarizeCols(df2[1:6,c(1:5,3)], me="maxOfRef")

(xt3 <- makeNRedMatr(t3, summ=unlist(list(X1="maxOfRef")), iniID=c("ref")))  

## ----combineRedBasedOnCol, echo=TRUE------------------------------------------
matr <- matrix(c(letters[1:6],"h","h","f","e",LETTERS[1:5]), ncol=3,
  dimnames=list(letters[11:15],c("xA","xB","xC")))
combineRedBasedOnCol(matr, colN="xB")
combineRedBasedOnCol(rbind(matr[1,],matr), colN="xB")

## ----convMatr2df, echo=TRUE---------------------------------------------------
x <- 1
dat1 <- matrix(1:10, ncol=2)
rownames(dat1) <- letters[c(1:3,2,5)]
## as.data.frame(dat1)  ...  would result in an error
convMatr2df(dat1)
convMatr2df(data.frame(a=as.character((1:3)/2), b=LETTERS[1:3], c=1:3))
tmp <- data.frame(a=as.character((1:3)/2), b=LETTERS[1:3], c=1:3, stringsAsFactors=FALSE)
convMatr2df(tmp)
tmp <- data.frame(a=as.character((1:3)/2), b=1:3, stringsAsFactors=FALSE)
convMatr2df(tmp) 

## ----combineOverlapInfo, echo=TRUE--------------------------------------------
set.seed(2013)
datT2 <- matrix(round(rnorm(200)+3,1), ncol=2, dimnames=list(paste("li",1:100,sep=""),
  letters[23:24]))
# (mimick) some short and longer names for each line
inf2 <- cbind(sh=paste(rep(letters[1:4],each=26), rep(letters,4),1:(26*4),sep=""),
  lo=paste(rep(LETTERS[1:4],each=26), rep(LETTERS,4), 1:(26*4), ",", 
  rep(letters[sample.int(26)],4), rep(letters[sample.int(26)],4), sep=""))[1:100,] 
## We'll use this to test :  
head(datT2, n=10)
## let's assign to each pair of x & y values a 'cluster' (column _clu_, the column _combInf_ tells us which lines/indexes are in this cluster)
head(combineOverlapInfo(datT2, disThr=0.03), n=10)
## it is also possible to rather display names (eg gene or protein-names) instead of index values
head(combineOverlapInfo(datT2, suplI=inf2[,2], disThr=0.03), n=10)

## ----getValuesByUnique, echo=TRUE---------------------------------------------
dat <- 11:19
names(dat) <- letters[c(6:3,2:4,8,3)]
## Here the names are not unique.
## Thus, the values can be binned by their (non-unique) names and a representative values calculated.

## Let's make a 'datUniq' with the mean of each group of values :
datUniq <- round(tapply(dat, names(dat), mean),1)
## now we propagate the mean values to the full vector 
getValuesByUnique(dat, datUniq)
cbind(ini=dat,firstOfRep=getValuesByUnique(dat, datUniq),
  indexUniq=getValuesByUnique(dat, datUniq, asIn=TRUE))

## ----combineByEitherFactor, echo=TRUE-----------------------------------------
nn <- rep(c("a","e","b","c","d","g","f"),c(3,1,2,2,1,2,1))
qq <- rep(c("m","n","p","o","q"),c(2,1,1,4,4))
nq <- cbind(nn,qq)[c(4,2,9,11,6,10,7,3,5,1,12,8),]
## Here we consider 2 columns 'nn' and 'qq' whe trying to regroup common values
##  (eg value 'a' from column 'nn' and value 'o' from 'qq') 
combineByEitherFactor(nq, 1, 2, nBy=FALSE)

## ----combineByEitherFactor2, echo=TRUE----------------------------------------
## the same, but including n by group/cluster
combineByEitherFactor(nq, 1, 2, nBy=TRUE)
## Not running further iterations works faster, but you may not reach 'convergence' immediately
combineByEitherFactor(nq,1, 2, nBy=FALSE)

## ----combineByEitherFactor3, echo=TRUE----------------------------------------
##  another example
mm <- rep(c("a","b","c","d","e"), c(3,4,2,3,1))
pp <- rep(c("m","n","o","p","q"), c(2,2,2,2,5))
combineByEitherFactor(cbind(mm,pp), 1, 2, con=FALSE, nBy=TRUE)

## ----multiCharReplace1, echo=TRUE---------------------------------------------
# replace character content
x1 <- c("ab","bc","cd","efg","ghj")
multiCharReplace(x1, cbind(old=c("bc","efg"), new=c("BBCC","EF")))

# works also on matrix and/or to replace numeric content : 
x3 <- matrix(11:16, ncol=2)
multiCharReplace(x3, cbind(12:13,112:113))

## ----multiCharReplace2, echo=TRUE---------------------------------------------
# replace and return logical vactor
x2 <- c("High","n/a","High","High","Low")
multiCharReplace(x2,cbind(old=c("n/a","Low","High"), new=c(NA,FALSE,TRUE)), convTo="logical")

## ----multiMatch1, echo=TRUE---------------------------------------------------
aa <- c("m","k","j; aa","m; aa; bb; o","n; dd","aa","cc")
bb <- c("aa","dd","aa; bb; q","p; cc") 
## result as list of indexes
(bOnA <- multiMatch(aa, bb, method="asIndex"))   # match bb on aa
## more convenient to the human reader
(bOnA <- multiMatch(aa, bb))                     # match bb on aa
(bOnA <- multiMatch(aa, bb, method="matchedL"))  # match bb on aa

## ----compGlobPat1, echo=TRUE--------------------------------------------------
aa <- letters[rep(c(3:1,4), each=2)]
ab <- letters[rep(c(5,8:6), each=2)]        # 'same general' pattern to aa
ac <- letters[c(1:2,1:3,3:4,4)]             # NOT 'same general' pattern to any other
ad <- letters[c(6:8,8:6,7:6)]               # NOT 'same general' pattern to any other

## ----compGlobPat2, echo=TRUE--------------------------------------------------
## get global patterns
cbind(aa= match(aa, unique(aa)),
  ab= match(ab, unique(ab)),
  ac= match(ac, unique(ac)),
  ad= match(ad, unique(ad)) )

## ----compGlobPat3, echo=TRUE--------------------------------------------------
bb <- data.frame(ind=1:length(aa), a=aa, b=ab, c=ac, d=ad)

## ----compGlobPat4, echo=TRUE--------------------------------------------------
replicateStructure(bb)

## ----compGlobPat5, echo=TRUE--------------------------------------------------
replicateStructure(bb, method="combAll")

## ----compGlobPat6, echo=TRUE--------------------------------------------------
replicateStructure(bb, method="combNonOrth")

## ----checkSimValueInSer, echo=TRUE--------------------------------------------
va1 <- c(4:7,7,7,7,7,8:10) + (1:11)/28600
checkSimValueInSer(va1, ppm=5)
data.frame(va=sort(va1), simil=checkSimValueInSer(va1))

## ----findCloseMatch1, echo=TRUE-----------------------------------------------
aA <- c(11:17); bB <- c(12.001,13.999); cC <- c(16.2,8,9,12.5,15.9,13.5,15.7,14.1,5)
(cloMa <- findCloseMatch(x=aA, y=cC, com="diff", lim=0.5, sor=FALSE))       

## ----closeMatchMatrix2, echo=TRUE---------------------------------------------
# all matches (of 2d arg) to/within limit for each of 1st arg ('x'); 'y' ..to 2nd arg = cC
# first let's display only one single closest/best hit
(maAa <- closeMatchMatrix(cloMa, aA, cC, lim=TRUE))  #

## ----closeMatchMatrix3, echo=TRUE---------------------------------------------
(maAa <- closeMatchMatrix(cloMa, aA, cC, lim=FALSE,origN=TRUE))  #
(maAa <- closeMatchMatrix(cloMa, cbind(valA=81:87, aA), cbind(valC=91:99, cC), colM=2,
  colP=2, lim=FALSE))
(maAa <- closeMatchMatrix(cloMa, cbind(aA,valA=81:87), cC, lim=FALSE))  #
a2 <- aA; names(a2) <- letters[1:length(a2)];  c2 <- cC; names(c2) <- letters[10 +1:length(c2)]
(cloM2 <- findCloseMatch(x=a2, y=c2, com="diff", lim=0.5, sor=FALSE)) 
(maA2 <- closeMatchMatrix(cloM2, predM=cbind(valA=81:87, a2),
  measM=cbind(valC=91:99, c2), colM=2, colP=2, lim=FALSE, asData=TRUE))
(maA2 <- closeMatchMatrix(cloM2, cbind(id=names(a2), valA=81:87,a2), cbind(id=names(c2),
  valC=91:99,c2), colM=3, colP=3, lim=FALSE)) 

## ----findSimilFrom2sets, echo=TRUE--------------------------------------------
aA <- c(11:17); bB <- c(12.001,13.999); cC <- c(16.2,8,9,12.5,12.6,15.9,14.1)
aZ <-  matrix(c(aA,aA+20), ncol=2, dimnames=list(letters[1:length(aA)],c("aaA","aZ")))
cZ <-  matrix(c(cC,cC+20), ncol=2, dimnames=list(letters[1:length(cC)],c("ccC","cZ")))
findCloseMatch(cC, aA, com="diff", lim=0.5, sor=FALSE)
findSimilFrom2sets(aA, cC)
findSimilFrom2sets(cC, aA)
findSimilFrom2sets(aA, cC, best=FALSE)
findSimilFrom2sets(aA, cC, comp="ppm", lim=5e4)
findSimilFrom2sets(aA, cC, comp="ppm", lim=9e4, bestO=FALSE)
# below: find fewer 'best matches' since search window larger (ie more good hits compete !)
findSimilFrom2sets(aA, cC, comp="ppm", lim=9e4, bestO=TRUE)

## ----fusePairs, echo=TRUE-----------------------------------------------------
(daPa <- matrix(c(1:5,8,2:6,9), ncol=2))
fusePairs(daPa, maxFuse=4)

## ----elimCloseCoord1, echo=TRUE-----------------------------------------------
da1 <- matrix(c(rep(0:4,5),0.01,1.1,2.04,3.07,4.5), ncol=2); da1[,1] <- da1[,1]*99; head(da1)
elimCloseCoord(da1)

## ----stableMode, echo=TRUE----------------------------------------------------
set.seed(2012); dat <- round(c(rnorm(120,0,1.2), rnorm(80,0.8,0.6), rnorm(25,-0.6,0.05), runif(200)),3)
dat <- dat[which(dat > -2 & dat <2)]
stableMode(dat)

## ----stableMode2, fig.height=8, fig.width=9, fig.align="center",  echo=TRUE----
layout(1:2)
plot(1:length(dat), sort(dat), type="l", main="Sorted Values", xlab="rank", las=1)
abline(h=stableMode(dat, silent=TRUE), lty=2,col=2)
legend("topleft",c("stableMode"), text.col=2, col=2, lty=2, lwd=1, seg.len=1.2, cex=0.8, xjust=0, yjust=0.5) 


plot(density(dat, kernel="gaussian", adjust=0.7), xlab="Value of dat", main="Density Estimate Plot")
useCol <- c("red","green","blue","grey55")
legend("topleft",c("dens","binning","BBmisc","allModes"), text.col=useCol, col=useCol,
  lty=2, lwd=1, seg.len=1.2, cex=0.8, xjust=0, yjust=0.5) 
abline(v=stableMode(dat, method="dens", silent=TRUE), lty=2, col="red", lwd=2)
abline(v=stableMode(dat, method="binning", silent=TRUE), lty=2, col="green")
abline(v=stableMode(dat, method="BBmisc", silent=TRUE), lty=2, col="blue")  
abline(v=stableMode(dat, method="allModes"), lty=2, col="grey55")  

## ----stableMode3, echo=TRUE---------------------------------------------------
set.seed(2021)
x <- sample(letters, 50000, replace=TRUE)
stableMode(dat, method="mode")
stableMode(dat, method="allModes")

## ----protectSpecChar1, echo=TRUE----------------------------------------------
aa <- c("abc","abcde","ab.c","ab.c.e","ab*c","ab\\d")
grepl("b.", aa)             # all TRUE
grepl(protectSpecChar("b."), aa)

## ----trimRedundText1, echo=TRUE-----------------------------------------------
txt1 <- c("abcd","abcde","abcdefg","abcdE",NA,"abcdEF")
trimRedundText(txt1)

## ----rmSharedWords1, echo=TRUE------------------------------------------------
txt2 <- c("abc d","abc de","abc defg","abc dE",NA,"abc dEF")
rmSharedWords(txt2)

## ----rmSharedWords2, echo=TRUE------------------------------------------------
txt3 <- c("ab 100 m","ab 120 m","ab 1000 m")
trimRedundText(txt3)
rmSharedWords(txt3)

## ----keepCommonText1, echo=TRUE-----------------------------------------------
txt1 <- c("abcd","abcde","abcdefg","abcdE",NA,"abcdEF")
trimRedundText(txt1, side="left")         # remove redundant 
keepCommonText(txt1, side="terminal")     # keep redundant
keepCommonText(txt1, side="center")       # computationally easier   

## ----keepCommonText2, echo=TRUE-----------------------------------------------
txt2 <- c("abcd_abc_kjh", "bcd_abc123", "cd_abc_po")
keepCommonText(txt2, side="center")       

## ----rmEnumeratorName1, echo=TRUE---------------------------------------------
xx <- c("hg_Re1","hjRe2_Re2","hk-Re3_Re33")
rmEnumeratorName(xx)
rmEnumeratorName(xx, newSep="--")
rmEnumeratorName(xx, incl="anyCase")

## ----rmEnumeratorName2, echo=TRUE---------------------------------------------
xy <- cbind(a=11:13, b=c("11#11","2_No2","333_samp333"), c=xx)
rmEnumeratorName(xy)
rmEnumeratorName(xy,incl=c("anyCase","trim2","rmEnumL"))

## ----rmEnumeratorName3, echo=TRUE---------------------------------------------
xz <- cbind(a=11:13, b=c("23#11","4#2","567#333"), c=xx)
apply(xz, 2, rmEnumeratorName, sepEnum=c("","_"), newSep="_", silent=TRUE)

## ----unifyEnumerator1, echo=TRUE----------------------------------------------
unifyEnumerator(c("ab-1","ab-2","c-3"))
unifyEnumerator(c("ab-R1","ab-R2","c-R3"))
unifyEnumerator(c("ab-1","c3-2","dR3"), stringentMatch=FALSE)

## ----chooseGroupNames1, echo=TRUE---------------------------------------------
(mat <- cbind(a=letters[1:6], b=paste(rep(c("b","B"), each=3), 1:3), c=rep(1,6), 
  d=gl(3,2), e=rep(c("e","E"),3), f=paste(rep(c("F","f","ff"), each=2), 1:2)))
  
chooseGroupNames(mat, method="median")         # col 2 (b/B)
chooseGroupNames(mat, method="median", fullReport=TRUE) 
chooseGroupNames(mat, method="min")            # col 2 (b/B)
chooseGroupNames(mat, method="max")            # col 6 (F/f/ff)
chooseGroupNames(mat, method="max", asUnique=FALSE) # col 1 (a..)

## ----checkUnitPrefix1, echo=TRUE----------------------------------------------
x1 <- c("10fg WW","xx 10fg 3pW"," 1pg 2.0W")
checkUnitPrefix(x1)

## different separators between digit and prefix:
x2 <- c("10fg WW","xx 8_fg 3pW"," 1 pg-2.0W")
checkUnitPrefix(x2, stringentSearch=TRUE)
checkUnitPrefix(x2, stringentSearch=FALSE)

## ----adjustUnitPrefix1, echo=TRUE---------------------------------------------
adjustUnitPrefix(c("10.psec", "2 fsec"), unit="sec")

## ----adjustUnitPrefix2, echo=TRUE---------------------------------------------
x2 <- c("abCc 500_nmol ABC", "abEe5_umol", "", "abFF_100_nmol_G", "abGg 2_mol", "abH.1 mmol")
rbind( adjustUnitPrefix(x2, unit="mol", returnType="allText") , 
  adjustUnitPrefix(x2, unit="mol", returnType="trim"),
  adjustUnitPrefix(x2, unit="mol", returnType=""))

## ----adjustUnitPrefix3, echo=TRUE---------------------------------------------
x3 <- c("2.psec abc","300 fsec etc", "34 5fsec")
adjustUnitPrefix(x3, unit=checkUnitPrefix(x3))

## ----mergeVectors1, echo=TRUE-------------------------------------------------
x1 <- c(a=1, b=11, c=21)
x2 <- c(b=12, c=22, a=2)
x3 <- c(a=3, d=43)
mergeVectors(vect1=x1, vect2=x2, vect3=x3)

## ----mergeVectors2, echo=TRUE-------------------------------------------------
mergeVectors(vect1=x1, vect2=x2, vect3=x3, inclInfo=TRUE)   # return list with additional info

## ----mergeVectors3, echo=TRUE-------------------------------------------------
x4 <- 41:44            # no names - not conform for merging and will be ignored
mergeVectors(x1, x2, x3, x4)

## ----matchMatrixLinesToRef1, echo=TRUE----------------------------------------
## Note : columns b and e allow non-ambigous match, not all elements of e are present in a
mat0 <- cbind(a=c("mvvk","axxd","bxxd","vv"),b=c("iwwy","iyyu","kvvh","gxx"), c=rep(9,4),
  d=c("hgf","hgf","vxc","nvnn"), e=c("_vv_","_ww_","_xx_","_yy_"))
matchMatrixLinesToRef(mat0[,1:4], ref=mat0[,5])
matchMatrixLinesToRef(mat0[,1:4], ref=mat0[1:3,5], inclInfo=TRUE)

matchMatrixLinesToRef(mat0[,-2], ref=mat0[,2], inclInfo=TRUE)   # needs 'reverse grep'

## ----orderMatrToRef1, echo=TRUE-----------------------------------------------
mat1 <- matrix(paste0("__",letters[rep(c(1,1,2,2,3),3) +rep(0:2,each=5)], rep(1:5)), ncol=3)
orderMatrToRef(mat1, paste0(letters[c(3,4,5,3,4)],c(1,3,5,2,4)))

mat2 <- matrix(paste0("__",letters[rep(c(1,1,2,2,3),3) +rep(0:2,each=5)], c(rep(1:5,2),1,1,3:5 )), ncol=3)
orderMatrToRef(mat2, paste0(letters[c(3,4,5,3,4)],c(1,3,5,1,4)))

mat3 <- matrix(paste0(letters[rep(c(1,1,2,2,3),3) +rep(0:2,each=5)], c(rep(1:5,2),1,1,3,3,5 )), ncol=3)
orderMatrToRef(mat3, paste0("__",letters[c(3,4,5,3,4)],c(1,3,5,1,3)))

## ----concatMatch1, echo=TRUE--------------------------------------------------
## simple example without concatenations or text-extensions
x0 <- c("ZZ","YY","AA","BB","DD","CC","D")
tab0 <- c("AA","BB,E","CC","FF,U")
match(x0, tab0)
concatMatch(x0, tab0)         # same result as match(), but with names

## now let's construct somthing similar but with concatenations and text-extensions
x1 <- c("ZZ","YY","AA","BB-2","DD","CCdef","Dxy")            # modif of single ID (no concat)
tab1 <- c("AA","WW,Vde,BB-5,E","CCab","FF,Uef")
match(x1, tab1)                   # match finds only the 'simplest' case (ie "AA")
concatMatch(x1, tab1)             # finds all hits as in example above

x2 <- c("ZZ,Z","YY,Y","AA,Z,Y","BB-2","DD","X,CCdef","Dxy")  # conatenated in 'x'
tab2 <- c("AA","WW,Vde,BB-5,E","CCab,WW","FF,UU")
concatMatch(x2, tab2)               # concatenation in both 'x' and 'table'

## ----checkStrictOrder1, echo=TRUE---------------------------------------------
set.seed(2005); mat1 <- rbind(matrix(round(runif(40),1),nc=4), rep(1,4))
head(mat1)
checkStrictOrder(mat1); mat1[which(checkStrictOrder(mat1)[,2]==0),]

## ----checkGrpOrder1, echo=TRUE------------------------------------------------
head(mat1)
checkGrpOrder(mat1)
checkGrpOrder(mat1, revRank=FALSE)    # only constant 'up' tested

## ----linModelSelect1, echo=TRUE-----------------------------------------------
li1 <- rep(c(4,3,3:6), each=3) + round(runif(18)/5,2)
names(li1) <- paste0(rep(letters[1:5], each=3), rep(1:3,6))
li2 <- rep(c(6,3:7), each=3) + round(runif(18)/5, 2)
dat2 <- rbind(P1=li1, P2=li2)
exp2 <- rep(c(11:16), each=3)
exp4 <- rep(c(3,10,30,100,300,1000), each=3)

## Check & plot for linear model 
linModelSelect("P1", dat2, expect=exp2)
linModelSelect("P2", dat2, expect=exp2)

## ----plotLinModelCoef1, echo=TRUE---------------------------------------------
set.seed(2020)
x1 <- matrix(rep(c(2,2:5),each=20) + runif(100) +rep(c(0,0.5,2:3,5),20), 
  byrow=FALSE, ncol=10, dimnames=list(LETTERS[1:10],NULL))
## just the 1st regression :
   summary(lm(b~a, data=data.frame(b=x1[,1], a=rep(1:5,each=2))))
## all regressions
x1.lmSum <- t(sapply(lapply(rownames(x1), linModelSelect, dat=x1, 
  expect=rep(1:5,each=2), silent=TRUE, plotGraph=FALSE), 
  function(x) c(x$coef[2,c(4,1)], startFr=x$startLev)))
x1.lmSum <- cbind(x1.lmSum, medQuantity=apply(x1,1,median))
x1.lmSum[,1] <- log10(x1.lmSum[,1])
head(x1.lmSum)

## ----plotLinModelCoef2, echo=TRUE---------------------------------------------
wrGraphOK <- requireNamespace("wrGraph", quietly=TRUE)      # check if package is available
if(wrGraphOK) wrGraph::plotW2Leg(x1.lmSum, useCol=c("Pr(>|t|)","Estimate","medQuantity","startFr"), 
  legendloc="topleft", txtLegend="start at")

## ----ratioAllComb0, echo=TRUE-------------------------------------------------
set.seed(2014); ra1 <- c(rnorm(9,2,1), runif(8,1,2))

## ----ratioAllComb1, echo=TRUE-------------------------------------------------
median(ra1[1:9]) / median(ra1[10:17])

## ----ratioAllComb2, echo=TRUE-------------------------------------------------
summary( ratioAllComb(ra1[1:9], ra1[10:17]))
boxplot(list(norm=ra1[1:9], unif=ra1[10:17], rat=ratioAllComb(ra1[1:9],ra1[10:17])))

## ----combineAsN1, echo=TRUE---------------------------------------------------
tm1 <- list(a1=LETTERS[1:7], a2=LETTERS[3:9], a3=LETTERS[6:10], a4=LETTERS[8:12])
combineAsN(tm1, nCombin=3, lev=gl(1,4))[,1,]

## ----combineAsN2, echo=TRUE---------------------------------------------------
## different levels/groups in list-elements
tm4 <- list(a1=LETTERS[1:15], a2=LETTERS[3:16], a3=LETTERS[6:17], a4=LETTERS[8:19],
  b1=LETTERS[5:19], b2=LETTERS[7:20], b3=LETTERS[11:24], b4=LETTERS[13:25], c1=LETTERS[17:26],
  d1=LETTERS[4:12], d2=LETTERS[5:11], d3=LETTERS[6:12], e1=LETTERS[7:10])
te4 <- combineAsN(tm4, nCombin=4, lev=substr(names(tm4),1,1))
str(te4)
te4[,,1]           # the counts part only

## ----readCsvBatch, echo=TRUE--------------------------------------------------
path1 <- system.file("extdata", package="wrMisc")
fiNa <-  c("pl01_1.csv","pl01_2.csv","pl02_1.csv","pl02_2.csv")
datAll <- readCsvBatch(fiNa, path1, silent=TRUE)
str(datAll)

## ----readCsvBatch2, echo=TRUE-------------------------------------------------
## batch reading of all csv files in specified path :
datAll2 <- readCsvBatch(fileNames=NULL, path=path1, silent=TRUE)
str(datAll2)

## ----readTabulatedBatch1, echo=TRUE-------------------------------------------
path1 <- system.file("extdata", package="wrMisc")
fiNa <-  c("a1.txt","a2.txt")
allTxt <- readTabulatedBatch(fiNa, path1)
str(allTxt)

## ----readVarColumns, echo=TRUE------------------------------------------------
path1 <- system.file("extdata", package="wrMisc")
fiNa <- "Names1.tsv"
datAll <- readVarColumns(fiName=file.path(path1,fiNa), sep="\t")
str(datAll)

## ----readGit1, echo=TRUE------------------------------------------------------
## An example url with tabulated data :
url1 <- "https://github.com/bigbio/sdrf-annotated-datasets/blob/main/datasets/PXD001819/PXD001819.sdrf.tsv"
gitDataUrl(url1)

## ----readGit2, echo=TRUE------------------------------------------------------
dataPxd <- try(read.delim(gitDataUrl(url1), sep='\t', header=TRUE))
str(dataPxd)

## ----vectorDistAngle1, echo=TRUE----------------------------------------------
x <- matrix(c(0, 3, 0.5, 4, 1, 1), ncol=2, byrow=TRUE)
y <- matrix(c(0, 0, 1, 3, 0.5, 5), ncol=2, byrow=TRUE)
(vectDist <- vectorDistAngle(x, y))

## ----vectorDistAngle2, echo=TRUE----------------------------------------------
plot(cbind(x=as.numeric(x), y=as.numeric(y)), main="Example for vectorDistAngle()",las=1)
arrows(x[,1], y[,1], x[,2], y[,2])
text(x[,2], 0.1 +y[,2], paste0("dist=",round(vectDist[,1],2),", angle=",round(vectDist[,2],2)), adj=1)

## ----presenceGrpFilt1, echo=TRUE----------------------------------------------
dat1 <- matrix(1:56,ncol=7)
dat1[c(2,3,4,5,6,10,12,18,19,20,22,23,26,27,28,30,31,34,38,39,50,54)] <- NA
grp1 <- gl(3,3)[-(3:4)]
dat1

## now let's filter
presenceGrpFilt(dat1, grp=grp1, presThr=0.75)  # stringent
presenceGrpFilt(dat1, grp=grp1, presThr=0.25)  # less stringent

presenceGrpFilt(dat1, grp=grp1, presThr=0.25)  # less stringent

## ----presenceFilt, echo=TRUE--------------------------------------------------
grp2 <- factor(rep(letters[4:2],c(2,2,3)))
presenceFilt(dat1, gr=grp2, maxGr=1, ratM=0.1)
presenceFilt(dat1, gr=grp2, maxGr=2, rat=0.5)

## ----presenceFilt2, echo=TRUE-------------------------------------------------
presenceFilt(dat1, gr=grp2, useComp=c("d-c","b-d"), maxGr=0, ratM=0.1)

## ----cleanReplicates, echo=TRUE-----------------------------------------------

(mat3 <- matrix(c(19,20,30,40, 18,19,28,39, 16,14,35,41, 17,20,30,40), ncol=4))
cleanReplicates(mat3, nOutl=1)
cleanReplicates(mat3, nOutl=3)


## ----normalizeThis0, echo=TRUE------------------------------------------------
set.seed(2015); rand1 <- round(runif(300) +rnorm(300,0,2),3)
dat1 <- cbind(ser1=round(100:1 +rand1[1:100]), ser2=round(1.2*(100:1 +rand1[101:200]) -2),
  ser3=round((100:1 +rand1[201:300])^1.2-3))
dat1 <- cbind(dat1, ser4=round(dat1[,1]^seq(2,5,length.out=100) +rand1[11:110],1))
## Let's introduce some NAs
dat1[dat1 <1] <- NA
## Let's get a quick overview of the data
summary(dat1)
## some selected lines (indeed, the 4th column appears always much higher)
dat1[c(1:5,50:54,95:100),]

## ----normalizeThis1, echo=TRUE------------------------------------------------
no1 <- normalizeThis(dat1, refGrp=1:3, meth="mean")
no2 <- normalizeThis(dat1, refGrp=1:3, meth="trimMean", trim=0.4)
no3 <- normalizeThis(dat1, refGrp=1:3, meth="median")
no4 <- normalizeThis(dat1, refGrp=1:3, meth="slope", quantFa=c(0.2,0.8))

## ----normalizeThis_plot1, echo=FALSE,eval=TRUE--------------------------------
boxplot(dat1, main="raw data", las=1)

## ----normalizeThis_plot2, echo=FALSE,eval=TRUE--------------------------------
layout(matrix(1:4, ncol=2))
boxplot(no1, main="mean normalization", las=1)
boxplot(no2, main="trimMean normalization", las=1)
boxplot(no3, main="median normalization", las=1)
boxplot(no4, main="slope normalization", las=1)

## ----rowNormalize1, echo=TRUE-------------------------------------------------
set.seed(2); AA <- matrix(rbinom(110, 10, 0.05), nrow=10)
AA[,4:5] <- AA[,4:5] *rep(4:3, each=nrow(AA))

AA1 <- rowNormalize(AA)
round(AA1, 2)

## ----rowNormalize2, echo=TRUE-------------------------------------------------
AC <- AA
AC[which(AC <1)] <- NA

(AC1 <- rowNormalize(AC))

## ----rowNormalize3, echo=TRUE-------------------------------------------------
(AC3 <- rowNormalize(AC, refLines=1:5, omitNonAlignable=TRUE))

## ----coordOfFilt1, echo=TRUE--------------------------------------------------
set.seed(2021); ma1 <- matrix(sample.int(n=40, size=27, replace=TRUE), ncol=9)
## let's test which values are >37
which(ma1 >37)      # doesn't tell which row & col
coordOfFilt(ma1, ma1 >37)

## ----trimmedMean1, echo=TRUE--------------------------------------------------
x <- c(17:11,27:28)
mean(x)
mean(x, trim=0.15)       # symmetric trimming
mean(x[x < 25])          # manual trimming
trimmedMean(x, trim=c(l=0, u=0.7))   # asymmetric trim

## ----blockNormalize1, echo=TRUE-----------------------------------------------
## Basic usage with vectors 
x <- c(1, 5, 3, 7, 2)
y <- (1:5)*10
## Align distributions (default: x and y will have identical distributions)
norm1 <- blockNormalize(x, y)
table(sort(norm1$x) == sort(norm1$y))  # all TRUE with 'quantile.block'

## ----blockNormalize2, echo=TRUE-----------------------------------------------
## matrix-example (like with omics-data)
set.seed(2026); mat1 <- matrix(rnorm(70), nrow=10, dimnames=list(letters[1:10],LETTERS[1:7])) *5  # 10 lines x 10 samples
set.seed(2025); mat2 <- matrix(rnorm(70, mean=5), nrow=10)^2 -15
mat2[which(mat2 < 1.8)] <- mat2[which(mat2 < 1.8)] + 32

norm2 <- blockNormalize(mat1, mat2, method="rescale", range=c(0.1, 10))
sapply(norm2, range)

norm3 <- blockNormalize(mat1, mat2, method="median")
sapply(norm3, quantile, c(0.25,0.5,0.75), na.rm=TRUE)

norm4 <- blockNormalize(mat1, mat2, method="quantile.block")
sapply(norm4, quantile, c(0.25,0.5,0.75), na.rm=TRUE)

## ----blockNormalize3, echo=TRUE-----------------------------------------------
layout(matrix(1:4, ncol=2))
boxplot(cbind(mat1, NA, mat2), main="initial", las=1)
boxplot(cbind(norm2$x, NA, norm2$y), main="rescale block", las=1)
boxplot(cbind(norm3$x, NA, norm3$y), main="median block", las=1)
boxplot(cbind(norm4$x, NA, norm4$y), main="quantile.block", las=1)

## ----blockNormalize4, echo=TRUE-----------------------------------------------
## the overall distribution of blocks
layout(matrix(1:4, ncol=2))
boxplot(cbind(mat1=as.numeric(mat1), mat2=as.numeric(mat2)), main="initial (overall)",las=1)
boxplot(cbind(mat1=as.numeric(norm2$x), mat2=as.numeric(norm2$x)), 
  main="rescale block norm (overall)",las=1)
boxplot(cbind(mat1=as.numeric(norm3$x), mat2=as.numeric(norm3$x)), 
  main="median block norm (overall)",las=1)
boxplot(cbind(mat1=as.numeric(norm4$x), mat2=as.numeric(norm4$x)), 
  main="quantile.block norm (overall)",las=1)

## ----rnormW1, echo=TRUE-------------------------------------------------------
## some sample data :
x1 <- (11:16)[-5]
mean(x1); sd(x1)

## ----rnormW2, echo=TRUE-------------------------------------------------------
## the standard way for gerenating normal random values
ra1 <- rnorm(n=length(x1), mean=mean(x1), sd=sd(x1))
## In particular with low n, the random values deviate somehow from expected mean and sd :
mean(ra1) -mean(x1) 
sd(ra1) -sd(x1)

## ----rnormW3, echo=TRUE-------------------------------------------------------
## random numbers with close fit to expected mean and sd :
ra2 <- rnormW(length(x1), mean(x1), sd(x1))
mean(ra2) -mean(x1) 
sd(ra2) -sd(x1)   # much closer to expected value

## ----moderTest2grp, echo=TRUE-------------------------------------------------
set.seed(2017); t8 <- matrix(round(rnorm(1600,10,0.4),2), ncol=8,
  dimnames=list(paste("l",1:200), c("AA1","BB1","CC1","DD1","AA2","BB2","CC2","DD2")))
t8[3:6,1:2] <- t8[3:6,1:2]+3     # augment lines 3:6 for AA1&BB1
t8[5:8,5:6] <- t8[5:8,5:6]+3     # augment lines 5:8 for AA2&BB2 (c,d,g,h should be found)
t4 <- log2(t8[,1:4]/t8[,5:8])
fit4 <- moderTest2grp(t4, gl(2,2))
## now we'll use limma's topTable() function to look at the 'best' results
if("list" %in% mode(fit4)) {  # if you have limma installed we can look further
  library(limma)
  topTable(fit4, coef=1,n=5)                      # effect for 3,4,7,8
  fit4in <- moderTest2grp(t4, gl(2,2), testO="<")
  if("list" %in% mode(fit4in)) topTable(fit4in, coef=1,n=5) }

## ----moderTestXgrp, echo=TRUE-------------------------------------------------
grp <- factor(rep(LETTERS[c(3,1,4)], c(2,3,3)))
set.seed(2017); t8 <- matrix(round(rnorm(208*8,10,0.4),2), ncol=8,
  dimnames=list(paste(letters[], rep(1:8,each=26),sep=""), paste(grp,c(1:2,1:3,1:3),sep="")))
t8[3:6,1:2] <- t8[3:6,1:2] +3                    # augment lines 3:6 (c-f) 
t8[5:8,c(1:2,6:8)] <- t8[5:8,c(1:2,6:8)] -1.5    # lower lines 
t8[6:7,3:5] <- t8[6:7,3:5] +2.2                  # augment lines 
## expect to find C/A in c,d,g, (h)
## expect to find C/D in c,d,e,f
## expect to find A/D in f,g,(h)  
test8 <- moderTestXgrp(t8, grp) 
head(test8$p.value, n=8) 

## ----moderTestXgrp2, echo=TRUE------------------------------------------------
test8a <- moderTestXgrp(t8, grp, useComparison="all")
head(test8a$p.value, n=3) 

## ----moderTestXgrp3, echo=TRUE------------------------------------------------
## multiple ways for choosing same comparison(s)
test8b <- moderTestXgrp(t8, grp, useCo=c("A-D"))   # name of comparison(s) 
test8c <- moderTestXgrp(t8, grp, useCo=2)          # the 2nd of all possible
test8d <- moderTestXgrp(t8, grp, useCo=cbind(samp="A", ref="D"))
test8e <- moderTestXgrp(t8, grp, useCo=cbind(samp=1, ref=3))

head(test8b$p.value, n=3) 
head(test8c$p.value, n=3) 
head(test8d$p.value, n=3) 
head(test8e$p.value, n=3) 

identical(test8b, test8c)
identical(test8b, test8d)

## ----moderTestXgrp4, echo=TRUE------------------------------------------------
## all comparisons with more explicite pairwise combined names
test8f <- moderTestXgrp(t8, grp, sep="combine2", useCo="all") 
head(test8f$p.value, n=8)

## ----pVal2lfdr, echo=TRUE-----------------------------------------------------
set.seed(2017); t8 <- matrix(round(rnorm(160,10,0.4),2), ncol=8, dimnames=list(letters[1:20],
  c("AA1","BB1","CC1","DD1","AA2","BB2","CC2","DD2")))
t8[3:6,1:2] <- t8[3:6,1:2] +3   # augment lines 3:6 (c-f) for AA1&BB1
t8[5:8,5:6] <- t8[5:8,5:6] +3   # augment lines 5:8 (e-h) for AA2&BB2 (c,d,g,h should be found)
head(pVal2lfdr(apply(t8, 1, function(x) t.test(x[1:4], x[5:8])$p.value)))

## ----fcCI, echo=TRUE----------------------------------------------------------
set.seed(2022); ran <- rnorm(50)
confInt(ran, alpha=0.05)
## plot points and confindence interval of mean
plot(ran, jitter(rep(1, length(ran))), ylim=c(0.95, 1.05), xlab="random variable 'ran'",main="Points and Confidence Interval of Mean (alpha=0.05)", ylab="", las=1)
points(mean(ran), 0.97, pch=3, col=4)     # mean
lines(mean(ran) +c(-1, 1) *confInt(ran, 0.05), c(0.97, 0.97), lwd=4, col=4)  # CI
legend("topleft","95% conficence interval of mean", text.col=4,col=4,lty=1,lwd=1,seg.len=1.2,cex=0.9,xjust=0,yjust=0.5)

## ----matchSampToPairw, echo=TRUE----------------------------------------------
## make example if limma is not installed
if(!requireNamespace("limma", quietly=TRUE)) test8 <- list(FDR=matrix(1, nrow=2, ncol=3, dimnames=list(NULL,c("A-C","A-D","C-D"))))
matchSampToPairw(unique(grp), colnames(test8$FDR)) 

## ----pairWiseConc1, echo=TRUE-------------------------------------------------
mat1 <- matrix(1:8, nrow=2, dimnames=list(NULL, paste0(1:4,"-",6:9)))
numPairDeColNames(mat1)

## ----replicateStructure1, echo=TRUE-------------------------------------------
## column a is all different, b is groups of 2,
## c & d  are groups of 2 nut NOT 'same general' pattern as b
strX <- data.frame(a=letters[18:11], b=letters[rep(c(3:1,4), each=2)],
 c=letters[rep(c(5,8:6), each=2)], d=letters[c(1:2,1:3,3:4,4)],
 e=letters[rep(c(4,8,4,7),each=2)], f=rep("z",8) )
strX

replicateStructure(strX[,1:2])
replicateStructure(strX[,1:4], method="combAll")
replicateStructure(strX[,1:4], method="combAll", exclNoRepl=FALSE)
replicateStructure(strX[,1:4], method="combNonOrth", exclNoRepl=TRUE)
replicateStructure(strX, method="lowest")

## ----convPairwiseSetup1, echo=TRUE--------------------------------------------
## concatenated names of groups
convPairwiseSetup(c("B-C","D-A"), LETTERS[1:4])

## separate names of groups
convPairwiseSetup(c("B","C"), LETTERS[1:3])
convPairwiseSetup(2:3, LETTERS[1:3])
convPairwiseSetup(2:3, LETTERS[1:3], sep="__")
convPairwiseSetup(matrix(c("B","D","C","A"), ncol=2), LETTERS[1:4])

## ----suggestSeparator1, echo=TRUE---------------------------------------------
getPWseparator(grp=LETTERS[1:3])
getPWseparator(grp=c("B-C","D","E")) 
getPWseparator(grp=c("B-C","D","E"), includeGrp=TRUE) 

## ----suggestSeparator2, echo=TRUE---------------------------------------------
potSep <- c("-", "D", " -compared to- ")
getPWseparator(grp=c("B-C","D","E"), includeGrp=TRUE, potSep=potSep)

## ----suggestSeparator3, echo=TRUE---------------------------------------------
getPWseparator(grp=c("B-C","D","E"), includeGrp=TRUE) 

## ----identifySeparator1, echo=TRUE--------------------------------------------
## Identify separator used
getPWseparator(compNames=c("B-C","B-D","C-D")) 
getPWseparator(compNames=c("B-C","B-D","C-D"), includeGrp=TRUE)

## ----identifySeparator2, echo=TRUE--------------------------------------------
## Identify separator used
findHeadAndTail(LETTERS[4:2], c("D-X","D-C"))

grp1 <- c("C","ACC","CC","B.B","B.BA","B","CA")
pwNa1 <- "B.B-CC" 
findHeadAndTail(grp1, pwNa1)

pwNa2 <- c("B.B-CC", "CA-ACC")
findHeadAndTail(grp1, pwNa2)

## ----std1, echo=TRUE----------------------------------------------------------
dat <- matrix(2*round(runif(100),2), ncol=4)
mean(dat); sd(dat)

datS <- scale(dat)
apply(datS, 2, sd)
# each column was teated separately
mean(datS); sd(datS); range(datS)
# the mean is almost 0.0 and the sd almost 1.0

datB <- scale(dat, center=TRUE, scale=FALSE)
mean(datB); sd(datB); range(datB)              # mean is almost 0

## ----std2, echo=TRUE----------------------------------------------------------
datS2 <- standardW(dat)
apply(datS2, 2, sd)
summary(datS2)
mean(datS2); sd(datS2)

datS3 <- standardW(dat, byColumn=TRUE)
apply(datS3, 2, sd)
summary(datS3)
mean(datS3); sd(datS3)

## ----scale1, echo=TRUE--------------------------------------------------------
datR2 <- apply(dat, 2, scaleXY, 1, 100)
summary(datR2); sd(datR2)

## ----clu01, echo=TRUE---------------------------------------------------------
nGr <- 3
irKm <- stats::kmeans(iris[,1:4], nGr, nstart=nGr*4)             # no need to standardize
   table(irKm$cluster, iris$Species)
   #wrGraph::plotPCAw(t(as.matrix(iris[,1:4])), sampleGrp=irKm,colBase=irKm$cluster,useSymb=as.numeric(as.factor(iris$Species)))

## ----clu02, echo=TRUE---------------------------------------------------------
## sort results by cluster number
head(reorgByCluNo(iris[,-5], irKm$cluster))
tail(reorgByCluNo(iris[,-5], irKm$cluster))

## ----clu03, echo=TRUE---------------------------------------------------------
## median an CV
ir2 <- reorgByCluNo(iris[,-5], irKm$cluster, addInfo=FALSE, retList=TRUE)

## ----clu04, echo=TRUE---------------------------------------------------------
sapply(ir2, function(x) apply(x, 2, median))

## ----clu05, echo=TRUE---------------------------------------------------------
sapply(ir2, colSds)

## ----rmOrphans1, echo=TRUE----------------------------------------------------
x=c(3:1,3:4,4:6,5:3)
cbind(x, def=rmOrphans(x), assign1=rmOrphans(x, reassign=TRUE), 
  assign1=rmOrphans(x, minN=0.2, reassign=TRUE) )

## ----filterNetw0, echo=TRUE---------------------------------------------------

lst2 <- list('121'=data.frame(ID=as.character(c(141,221,228,229,449)),11:15), 
  '131'=data.frame(ID=as.character(c(228,331,332,333,339)),11:15), 
  '141'=data.frame(ID=as.character(c(121,151,229,339,441,442,449)),c(11:17)), 
  '151'=data.frame(ID=as.character(c(449,141,551,552)),11:14),
  '161'=data.frame(ID=as.character(171),11),
  '171'=data.frame(ID=as.character(161),11),
  '181'=data.frame(ID=as.character(881:882),11:12) )

## ----filterNetw1, echo=TRUE---------------------------------------------------
(nw1 <- filterNetw(lst2, limInt=20, sandwLim=NULL, remOrphans=FALSE))

## ----filterNetw2, echo=TRUE---------------------------------------------------
(nw2 <- filterNetw(lst2, limInt=20, sandwLim=NULL, remOrphans=TRUE))

## ----filterNetw3, echo=TRUE---------------------------------------------------
(nw3 <- filterNetw(lst2, limInt=20, sandwLim=14, remOrphans=TRUE))

## ----propMatr1, echo=TRUE-----------------------------------------------------
pairs3L <- matrix(LETTERS[c(1,3,3, 2,2,1)], ncol=2)      # loop of 3
(netw13pr <- pairsAsPropensMatr(pairs3L))                # as prop matr

## ----contribToContigPerFrag, echo=TRUE----------------------------------------
path1 <- matrix(c(17,19,18,17, 4,4,2,3), ncol=2,
  dimnames=list(c("A/B/C/D","A/B/G/D","A/H","A/H/I"), c("sumLen","n")))
contribToContigPerFrag(path1)

## ----simpleFragFig, echo=TRUE-------------------------------------------------
frag1 <- cbind(beg=c(2,3,7,13,13,15,7,9,7, 3,3,5), end=c(6,12,8,18,20,20,19,12,12, 4,5,7))
rownames(frag1) <- letters[1:nrow(frag1)]
simpleFragFig(frag1)

## ----countSameStartEnd, echo=TRUE---------------------------------------------
countSameStartEnd(frag1)

## ----pasteC, echo=TRUE--------------------------------------------------------
pasteC(1:4)
pasteC(letters[1:4],quoteC="'")

## ----color-gradient1, echo=TRUE-----------------------------------------------
set.seed(2015); dat1 <- round(runif(15),2)
plot(1:15, dat1, pch=16, cex=2, las=1, col=colorAccording2(dat1),
  main="Color gradient according to value in y")
# Here we modify the span of the color gradient
plot(1:15, dat1, pch=16, cex=2, las=1, 
  col=colorAccording2(dat1, nStartO=0, nEndO=4, revCol=TRUE), main="blue to red")
# It is also possible to work with scales of transparency
plot(1:9, pch=3, las=1)
points(1:9, 1:9, col=transpGraySca(st=0, en=0.8, nSt=9,trans=0.3), cex=42, pch=16)

## ----convColorToTransp, fig.height=6, fig.width=3, echo=TRUE------------------
col0 <- c("#998FCC","#5AC3BA","#CBD34E","#FF7D73")
col1 <- convColorToTransp(col0,alph=0.7)
layout(1:2)
pie(rep(1,length(col0)), col=col0, main="no transparency")
pie(rep(1,length(col1)), col=col1, main="new transparency")

## ----sysDate1, echo=TRUE------------------------------------------------------
## To get started
Sys.Date()

## Compact English names (in European order), no matter what your local settings are :
sysDate() 

## ----DateTab, echo=TRUE-------------------------------------------------------
tabD <- cbind(paste0("univ",1:6), c(sysDate(style="univ1"), sysDate(style="univ2"), 
    sysDate(style="univ3"), sysDate(style="univ4"), as.character(sysDate(style="univ5")), 
    sysDate(style="univ6")), paste0("   local",1:6), 
  c(sysDate(style="local1"), sysDate(style="local2"), sysDate(style="local3"), 
    sysDate(style="local4"), sysDate(style="local5"), sysDate(style="local6")))   
knitr::kable(tabD, caption="Various ways of writing current date")

## ----sessionInfo, echo=FALSE--------------------------------------------------
sessionInfo()

#' Convert Pairs of Node-Names to Non-Oriented Propensity Matrix 
#'
#' @description
#' Numerous network query tools produce a listing of pairs of nodes (with one pair of nodes per line). 
#' Using this function such a \code{matrix} (or \code{data.frame}) can be combined to this more comprehensive view as propensity-matrix.  
#' 
#' @details
#' Note, this has been primarily developed for undirected interaction networks, the resulting propensity-matrix does not show any orientation any more.   
#' In a number of applications (eg in protein-protein interaction networks, PPI) the resulting matrix may be rather sparse.    
#' 
#' @param mat (matrix) main input, matrix of interaction partners with each line as a separate pair of nodes; 
#'   the first two columns should contain identifiers of the nodes
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging 
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns matrix or data.frame
#' @seealso uses typically input from \code{\link{filterNetw}}
#' @examples
#' pairs3L <- matrix(LETTERS[c(1,3,3, 2,2,1)], ncol=2)    # loop of 3
#' (netw13pr <- pairsAsPropensMatr(pairs3L))              # as prop matr
#' 
#' @export
pairsAsPropensMatr <- function(mat, silent=FALSE, debug=FALSE, callFrom=NULL) {
  ## convert pairs of node-names (non-oriented) to propensity matrix  
  ## sparse matrix solution
  fxNa <- .composeCallName(callFrom, newNa="pairsAsPropensMatr")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  
  if(!is.matrix(mat)) mat <- try(as.matrix(mat), silent=TRUE)
  msg <- " 'mat'; must be matrix or data.frame with min 1 line and 2 col"
  if(inherits(mat, "try-error")) stop("Invalid input in",msg)
  if(any(length(dim(mat)) !=2, dim(mat) < 1:2)) stop("Invalid argument",msg)
  
  nodeNa <- sort(unique(as.character(mat[,1:2])))
  nodeIM <- matrix(match(as.character(mat[,1:2]), nodeNa), ncol=2)    # as matrix
  di <- matrix(0, nrow=length(nodeNa), ncol=length(nodeNa))
  for(i in 1:ncol(di)) { j <- which(nodeIM[,1]==i)
    if(length(j) >0) { di[i,nodeIM[j,2]] <- di[i,nodeIM[j,2]] +1
      di[nodeIM[j,2],i] <- di[nodeIM[j,2],i] +1 }
    }
  dimnames(di) <- list(seq(nrow(di)), seq(nrow(di))) 
  di }
  

#' Read Tabular Content Of Files With Variable Number Of Columns  
#'
#' Reading the content of files where the number of separators (eg tabulation) is variable poses problems with traditional methods for reding files, like  \code{\link[utils]{read.table}}.
#' This function reads each line independently and then parses all separators therein. The first line is assumed to be column-headers.
#' Finally, all data will be returned in a matrix adopted to the line with most separators and if the number of column-headers is insufficient, new (unique) column-headers will be generated.
#' Thus, the lines may contain different number of elements, empty elements (ie tabular fields) will always get added to right of data read 
#' and their content will be as defined by argument \code{emptyFields} (default \code{NA}).
#' 
#' Note, this functions assumes one line of header and at least one line of data !
#' Note, for numeric data the comma is assumed to be US-Style (as '.').
#' Note, that it is assumed, that any missing fields for the complete tabular view are missing on the right (ie at the end of line) !
#'
#' @param fiName (character) file-name
#' @param path (character) optional path
#' @param sep (character) separator (between columns)
#' @param header (logical) indicating whether the file contains the names of the variables as its first line.
#' @param emptyFields (\code{NA} or character) missing headers will be replaced by the content of 'emptyFields', if \code{NA} the last column-name will be re-used and a counter added 
#' @param refCo (integer) for custom choice of column to be used as row-names (default will use 1st text-column)
#' @param supNa (character) base for constructing name for columns wo names (+counter starting at 2), default column-name to left of 1st col wo colname
#' @param silent (logical) suppress messages
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a matrix (character or numeric)
#' @seealso for regular 'complete' data \code{\link[utils]{read.table}} and its argument \code{flush}  
#' @examples
#' path1 <- system.file("extdata",package="wrMisc")
#' fiNa <- "Names1.tsv"
#' datAll <- readVarColumns(fiName=file.path(path1,fiNa))
#' str(datAll)
#' @export
readVarColumns <- function(fiName, path=NULL, sep="\t", header=TRUE, emptyFields=NA, refCo=NULL, supNa=NULL, silent=FALSE, callFrom=NULL) {
  ## slightly slower function for reading variable tabular content of files: This function allows reading variable number of elements per line (via parsing each line separately).
  ##  read content of file using separator 'sep'; 1st line is expectd to caintain some headers, missing headers will be replaced by the content of 'emptyFields'
  ## 'emptyFields'  (NA or character) content of fields
  ## 'refCo'  (integer) for custom choice of column to be used as row-names (default will use 1st text-column)
  ## 'supNa' (character) base for constructing name for columns wo names (+counter starting at 2), default column-name to left of 1st col wo colname
  fxNa <- .composeCallName(callFrom, newNa="readVarColumns")
  fiName <- if(length(path) >0) {if(nchar(path) >0) file.path(path, fiName) else fiName} else fiName
  if(!file.exists(fiName)) { dataOK <- FALSE
    warning("PROBLEM : Can't find file '",fiName,"'")
  } else {
    dataOK <- TRUE
    if(length(header) !=1) header <- FALSE
    if(!is.logical(header)) { dataOK <- FALSE 
      warning(fxNa,"Invalid command: argument 'header' should be logical and of length=1")} }
  if(dataOK) {
    out <- try(scan(fiName, what="character", sep="\n"), silent=TRUE)
    if(inherits(out, "try-error")) { dataOK <- FALSE
      warning(fxNa,"Did NOT succeed reading file '",fiName,"'")}}

  if(dataOK) {  
    ## parse each line since file does/may contain variable number of columns and/or (many) columns wo colnames
    out <- strsplit(out, sep)
    maxCo <- max(sapply(out, length))
    out <- t(sapply(out, function(x,maxC) {if(length(x)==maxC) x else {z <- rep("",maxC); z[1:length(x)] <- x; z}}, maxC=maxCo))
    supPep <- which(out[1,] =="")
    if(length(supNa) !=1) { supNa0 <- min(supPep, na.rm=TRUE)
      supNa <- if(supNa0 >1) out[1, min(supPep, na.rm=TRUE) -1] else ((1:ncol(out))[-supPep])[1]}
    out[1, supPep] <- paste(supNa, 1 +1:length(supPep),sep="_")
    if(nrow(out) <2) { dataOK <- FALSE
      warning("PROBLEM : data seem to be empty (header only ?)")} }

  if(dataOK) {  
    ## extract row- and col-names, prepare for separating numeric from text
    testNum <- function(x) all(grepl("(^([0-9]+)|(^[+-][0-9]+)|(^\\.[0-9]+))((\\.[0-9]+)?)(([eE][+-]?[0-9]+)?)$", x))
    numCol <- apply(out[-1,], 2, testNum)
    if(length(refCo) !=1) {refCo <- min(which(!numCol))
      if(isFALSE(silent)) message(fxNa,"Setting 'refCo' to '",out[1,refCo],"'")}  #
    dupNa <- duplicated(out[1,])  
    ## integrate colnames and rownames ...
    useLi <- if(header) 2:nrow(out) else 1:nrow(out)
    colNa <- if(header) {if(any(dupNa)) correctToUnique(out[1,], callFrom=fxNa) else out[1,]} else NULL
    dupNa <- duplicated(out[useLi,refCo])
    rowNa <- if(any(dupNa)) correctToUnique(out[useLi,refCo], callFrom=fxNa) else out[useLi,refCo]
    if(header) out <- out[-1,]
    if(length(dim(out)) <2) out <- matrix(out, nrow=1, dimnames=list(rowNa, names(out))) else dimnames(out) <- list(rowNa,colNa)  
    out } }
   

#' Read batch of csv-files
#'
#' This function was designed to read screening data split in parts (with common structure) and saved to multiple files,  
#' to extract the numeric columns and to compile all (numeric) data to a single array (or list).  Some screening platforms save results while progressing 
#' through a pile of microtiter-plates separately. The organization of the resultant files is structured through file-names and all files have exactely the same organization of lines and columns/ 
#' European or US-formatted csv files can be read, if argument \code{fileFormat} is \code{NULL} both types will be tested, otherwise it allows to specify a given format.
#' The presence of headers (to be used as column-names) may be tested using \code{checkFormat}.
#'
#' @param fileNames (character) names of files to be read, if \code{NULL} all files fitting 'fileFormat'
#' @param path (character) where files should be read (folders should be written in R-style)
#' @param fileFormat (character) may be \code{NULL} (both US and European formats will be tried), 'Eur' or 'US' 
#' @param checkFormat (logical) if \code{TRUE}: check header, remove empty columns, 1st line if all empmty, set output format for each file to matrix, if rownames are increasing integeres try to use 2nd of 'columns' as rownames
#' @param returnArray (logical) allows switching from array to list-output
#' @param columns (NULL or character) column-headers to be extracted (if specified), 2nd value may be comlumn with rownames (if rownames are encountered as increasing rownames)
#' @param excludeFiles (character) names of files to exclude (only used when reading all files of given directory)
#' @param simpleNames (logical) allows truncating names (from beginning) to get to variable part (using .trimLeft()), but keeping 'minNamesLe'
#' @param minNamesLe (interger) min length of column-names if simpleNames=TRUE
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging 
#' @param callFrom (character) allows easier tracking of messages produced
#' @return This function returns an array (or list if \code{returnArray=FALSE}) of all numeric data read (numerical columns only) from individual files
#' @seealso \code{\link[utils]{read.table}}, \code{\link{writeCsv}}, \code{\link{readXlsxBatch}} 
#' @examples
#' path1 <- system.file("extdata", package="wrMisc")
#' fiNa <-  c("pl01_1.csv","pl01_2.csv","pl02_1.csv","pl02_2.csv")
#' datAll <- readCsvBatch(fiNa, path1)
#' str(datAll)
#' ## batch reading of all csv files in specified path :
#' datAll2 <- readCsvBatch(fileNames=NULL, path=path1, silent=TRUE)
#' @export
readCsvBatch <- function(fileNames=NULL, path=".", fileFormat="Eur", checkFormat=TRUE, returnArray=TRUE,
  columns=c("Plate","Well","StainA"), excludeFiles="All infected plates", simpleNames=TRUE, minNamesLe=4, silent=FALSE, debug=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom, newNa="readCsvBatch")
  if(isTRUE(debug)) silent <- FALSE else { debug <- FALSE
    if(!isTRUE(silent)) silent <- FALSE }
  HeaderInData <- if(is.null(columns)) FALSE else TRUE
  if(length(path) >0) {
    chPath <- dir.exists(path)
    if(!chPath) {path <- "."
      message(fxNa," Cannot find path '",path,"' !  ... Setting to default='.'")}
  } else path <- "." 
  if(is.null(fileNames)) {               # read batch of files
    fileNames <- file.path(path, dir(path=path, pattern="\\.csv$"))
    if(length(fileNames) <1) message(fxNa,"Could not find ANY suitable files !!") else {
      if(!silent) message(fxNa,"Batch mode: Found ",length(fileNames)," files to extract (eg ",pasteC(utils::head(fileNames,3),quoteC="'"),")")}
  } else {                              # read explicit set of files
    if(length(path) >0) { chPath <- grep(paste0("^",path), fileNames)  # check if path already given with filenames
      if(length(chPath) <1) fileNames <- file.path(path,fileNames)}
    checkFi <- file.exists(fileNames)
    if(any(!checkFi)) { 
      if(!silent) message(fxNa,"Could not find ",sum(!checkFi)," files (eg ",pasteC(utils::head(fileNames[which(!checkFi)],3),quoteC="'"),")")
      fileNames <- fileNames[which(checkFi)] }
  if(length(fileNames) >0) {
    ## treat files to exclude
    checkFi <- if(!is.null(excludeFiles)) grepl(excludeFiles, fileNames) else rep(FALSE, length(fileNames))
    if(any(!checkFi))
      if(!silent) message(fxNa,"Based on 'excludeFiles': excluding ",sum(checkFi)," files (out of ",length(fileNames),")")
      fileNames <- fileNames[which(!checkFi)]} 
  }  
  ## check package(s)
  reqPa <- c("utils")
  chPa <- sapply(reqPa, requireNamespace, quietly=TRUE)
  if(any(!chPa)) { if(!silent) message(fxNa,"Package '",reqPa[1],"' missing, please install first from CRAN  (returning NULL)")
    fileNames <- NULL    
  } 

  ## check if US or European format
  if(length(fileNames) >0) {
    if(!silent) message(fxNa,"Ready to start reading ",length(fileNames)," files;  expecting header(s) : ", HeaderInData)
    tmp1 <- if(!identical(fileFormat,"Eur")) try(utils::read.csv(fileNames[1], header=HeaderInData, stringsAsFactors=FALSE), silent=TRUE) else NULL
    tmp2 <- if(!identical(fileFormat,"US")) try(utils::read.csv2(fileNames[1], header=HeaderInData, stringsAsFactors=FALSE), silent=TRUE) else NULL
    cheF <- c(inherits(tmp1, "try-error"), inherits(tmp2, "try-error"))
    if(debug) message(fxNa,"Done with inital try of reading files")
    
    if(all(cheF)) { fileNames <- NULL
      warning(fxNa,"\n Problem with file format, can't read 1st file neither as European nor US-type CSV !!")    
    } else {
      chDim <- prod(dim(tmp1)) > prod(dim(tmp2))
      datDim <- if(chDim) dim(tmp1) else dim(tmp2)
      if(!silent) message(fxNa,"csv-format used ",if(chDim) "'US'" else "'Europe'","  with ",datDim[1]," lines & ",datDim[2]," cols")
    } }

  ## main reading
  if(length(fileNames) >0) {
    datL <- list()
    arrNames <- if(isTRUE(simpleNames)) .trimLeft(fileNames, minNchar=minNamesLe) else fileNames
    if(debug) message(fxNa,"Ready for main reading of  ",pasteC(utils::head(fileNames)))
    for(i in 1:length(fileNames)) {
      if(!silent) message(fxNa," ..reading file ",fileNames[i])
      curFileNa <- fileNames[i]
      tmp <- if(chDim) {
        utils::read.csv(curFileNa, header=HeaderInData, stringsAsFactors=FALSE)
      } else utils::read.csv2(curFileNa, header=HeaderInData, stringsAsFactors=FALSE) 
      if(isTRUE(checkFormat)) {
        tmp <- .inspectHeader(tmp, headNames=columns, silent=i !=1 | silent, callFrom=fxNa)
        tmp <- .removeEmptyCol(tmp, fromBackOnly=FALSE, silent=i !=1 | silent, callFrom=fxNa)
      }
      if(i ==1) {                         # (re)define new format based on 1st file after format-checking (ie remove empty cols, extract col of well-names,...)
        datDim <- dim(tmp)
        dat <- if(isTRUE(returnArray)) array(NA, dim=c(datDim[1],length(fileNames),datDim[2])) else NULL }
      if(!returnArray) datL[[i]] <- as.matrix(tmp) else {
        if(identical(dim(tmp),datDim)) dat[,i,] <- as.matrix(tmp) else {
          message(fxNa, i,"th file ",fileNames[i]," seems not to have format consistent with prev files (",dim(tmp)[1]," lines & ",dim(tmp)[2]," rows)")
      } }
    }
    if(sum(nchar(arrNames) <1) >0) {
      if(!silent) message(fxNa,"Some arrays seem to have no plate-name specified from within data, using file-names")
      arrNames <- substr(fileNames[i],1,nchar(fileNames[i])-4)}
    if(returnArray) {
      dimnames(dat) <- list(rownames(tmp), arrNames, colnames(tmp))
    } else { dat <- datL
      if(length(fileNames) !=length(dat)) message(fxNa,"Problem ?  Got ",length(fileNames)," fileNames  BUT ",length(dat)," list-elements !")
      names(dat) <- substr(fileNames,1,nchar(fileNames)-4)
      if(isTRUE(simpleNames)) names(dat) <- .trimLeft(names(dat), minNchar=minNamesLe) }
  dat } else NULL }


#' Inspect 'matr' and check if 1st line can be used/converted as header
#'
#' This function inspects 'matr' and check if 1st line can be used/converted as header.
#' If colnames of 'matr' are either NULL or 'V1',etc the 1st row will be tested if it contains any of the elements (if not, 1st line won't be used as new colnames)
#' If 'numericCheck'=TRUE, all columns will be tested if they can be converted to numeric
#' 
#' @param matr (matrix or data.frame) main input to be instected
#' @param headNames (character) column-names t look for
#' @param numericCheck (logical) allows reducing complexity by drawing for very long x or y
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a matrix vector or data.frame similar to input
#' @seealso  \code{\link[utils]{head}} for looking at first few lines
#' @examples
#' ma1 <- matrix(letters[1:6], ncol=3, dimnames=list(NULL,c("ab","Plate","Well")))
#' .inspectHeader(ma1) 
#' 
#' @export
.inspectHeader <- function(matr, headNames=c("Plate","Well","StainA"), numericCheck=TRUE, silent=FALSE, debug=FALSE, callFrom=NULL){
  ## inspect 'matr' and check if 1st line can be used/converted as header
  ## if colnames of 'matr' are either NULL or 'V1',etc the 1st row will be tested if it contains any of the elements (if not, 1st line won't be used as new colnames)
  ## if'numericCheck'=TRUE, all columns will be tested if they can be converted to numeric
  extr1stLine <- TRUE
  fxNa <- .composeCallName(callFrom, newNa=".inspectHeader")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE

  msg <- character()
  if(!is.null(colnames(matr))) {if(identical(colnames(matr), paste0("V",1:ncol(matr)))) extr1stLine <- FALSE}
  if(extr1stLine & sum(tolower(headNames) %in% tolower(as.matrix(matr[1,]))) <1) extr1stLine <- FALSE
  if(extr1stLine) {
    chNewNa <- length(matr[1,]) ==length(unique(matr[1,]))
    if(!chNewNa && !silent) message(fxNa,"Transferring text to now column-names, but they are not unique !")
    matr <- as.data.frame(matr)
    colnames(matr) <- matr[1,]
    matr <- matr[-1,] }
  if(numericCheck) {
    ## try to locate 1st column that can be used as rownames (before removing text-content of non-nuleric columns)
    rowNa <- rownames(matr)
    if(is.null(rowNa) || identical(rowNa,as.character(1:nrow(matr)))) {
      chUniq <- apply(matr, 2, function(x) length(unique(x)) ==length(x))
      if(any(chUniq)) rowNa <- matr[,which(chUniq)[1]]
    }
    ## extract numerical values only (text will be converted to NA)
    matr <- apply(matr, 2, convToNum, spaceRemove=TRUE, convert=c(NA,"allChar"), remove=NULL,sciIncl=TRUE,euroStyle=TRUE,callFrom=fxNa,silent=silent)
    rownames(matr) <- rowNa }
  matr }
   

#' Estimate sd Of Median For Each Row By Bootstrap
#'
#' This function determines the stand error (sd) of the median for each row by bootstraping each row of 'dat'.
#' Note: requires package \href{https://CRAN.R-project.org/package=boot}{boot}
#'
#' @param dat (numeric) matix, main input 
#' @param nBoot (integer) number if iterations for bootstrap
#' @param silent (logical) suppress messages
#' @param callFrom (character) allows easier tracking of messages produced
#' @param debug (logical) display additional messages for debugging
#' @return This functions returns a (numeric) vector with estimated sd values
#' @seealso For a more flexible version able to handle lists please look at \code{\link{colMedSds}} ,  based on \code{\link[boot]{boot}}
#' @examples
#' set.seed(2016); dat1 <- matrix(c(runif(200)+rep(1:10,20)), ncol=10)
#' rowMedSds(dat1) ; plot(rowSds(dat1), rowMedSds(dat1))
#' @export
rowMedSds <- function(dat, nBoot=99, silent=FALSE, debug=FALSE, callFrom=NULL) {
  fxNa <- .composeCallName(callFrom, newNa="rowMedSds")
  if(isTRUE(debug)) silent <- FALSE else { debug <- FALSE
    if(!isTRUE(silent)) silent <- FALSE }
  msg <- "'dat' should be matrix or data.frame with "
  if(is.null(ncol(dat))) stop(fxNa,msg,"multiple columns !") else if(ncol(dat) < 2) stop(msg,"at least 2 columns !")
  if(requireNamespace("boot", quietly=TRUE)) {
    median.fun <- function(dat,indices) stats::median(dat[indices],na.rm=TRUE)
    out <- try(apply(dat,1,function(x) stats::sd(boot::boot(data=x, statistic=median.fun, R=nBoot)$t)))
    if(inherits(out, "try-error")) stop(fxNa,"Did not succeed in running boot()")
    out 
  } else if(!silent) message(fxNa, "NOTE : Package 'boot' not found ! Please install first from CRAN")   
  }
   

#' Convert matrix of integer to matrix of x-times repeated column-names
#'
#' \code{conv01toColNa} transforms matrix of integers (eg 0 and 1) to repeated & concatenated text from argument \code{colNa}, 
#' the character string for 0 occurances of argument \code{zeroTex} may be customized.
#' Used eg when specifying (and concatenating) various counted elements (eg properties) along a vector like variable peptide modifications in proteomics. 
#'
#' @param mat input matrix (with integer values)
#' @param colNa alternative (column-)names to the ones from 'mat' (default colnames of 'mat')
#' @param zeroTex text to display if 0 (default "")
#' @param pasteCol (logical) allows to collapse all columns to single chain of characters in output 
#' @return character vector
#' @examples
#' (ma1 <- matrix(sample(0:3,40,repl=TRUE), ncol=4, dimnames=list(NULL, letters[11:14])))
#' conv01toColNa(ma1)
#' conv01toColNa(ma1, colNa=LETTERS[1:4], ze=".")
#' conv01toColNa(ma1, colNa=LETTERS[1:4], pasteCol=TRUE)
#' @export
conv01toColNa <- function(mat,colNa=NULL,zeroTex="",pasteCol=FALSE){
  txt <- "'mat' should be matrix"
  collTx <- ""
  if(length(dim(mat)) !=2) stop(txt) else if(any(dim(mat)==0)) stop(txt," with >0 rows & cols")
  if(is.data.frame(mat)) {ch <- as.numeric(as.matrix(mat))
    if(!is.numeric(ch)) stop(txt," cannot transform to matrix of numeric")
    mat <- matrix(ch, nrow=nrow(mat), dimnames=dimnames(mat))}
  if(is.null(colNa)) colNa <- colnames(mat) else {if(length(colNa) != ncol(mat)) colNa <- colnames(mat)}
  out <- matrix(zeroTex, nrow=nrow(mat), ncol=ncol(mat), dimnames=dimnames(mat))
  for(j in 1:ncol(mat)) { tmp <- which(mat[,j] >0)
    if(length(tmp) >0) out[tmp,j] <- if(max(mat[tmp,j]) >1) sapply(mat[tmp,j], function(x) paste(rep(colNa[j],x),collapse=collTx)) else colNa[j]}
  if(pasteCol & ncol(mat) >1) { sepPa <- ""           
    tmp <- if(ncol(mat)==2) paste(out[,1],out[,2],sep=sepPa) else paste(out[,1],out[,2],out[,3],sep=sepPa)
    if(ncol(mat) >3) for(j in 4:ncol(mat)) tmp <- paste(tmp,out[,j], sep=sepPa)
    out <- as.matrix(tmp)}
  out }
    

#' Select groups within given range
#'
#' This function aims to help finding streches/segments of data with a given maximum number of NA-instances.
#  To do so, it searches independently each line of the input-matrix 'dat' for sretches with a given maximum of NA-instaces (\code{'maxNA'}. 
#' This function is used to inspect/filter each lines of 'dat' for a subset with sufficient presence/absence of NA values (ie limit number of NAs per level of 'grp').
#' Note :  optimal perfomance with n.lines >> n.groups
#' @param dat (matrix or data.frame) main input
#' @param grp (factor) information which column of 'dat' is replicate of whom
#' @param maxNA (interger) max number of tolerated NAs
#' @param callFrom (character) allow easier tracking of message(s) produced
#' @return matrix with boundaries of 1st and last usable column (NA if there were no suitable groups found)
#' @examples
#' dat1 <- matrix(1:56,nc=7)
#' dat1[c(2,3,4,5,6,10,12,18,19,20,22,23,26,27,28,30,31,34,38,39,50,54)] <- NA
#' rownames(dat1) <- letters[1:nrow(dat1)]
#' findUsableGroupRange(dat1,gl(3,3)[-(3:4)])
#' @export
findUsableGroupRange <- function(dat,grp,maxNA=1,callFrom=NULL){
  fxNa <- .composeCallName(callFrom,newNa="findUsableGroupRange")
  msg <- "expecting (2dim) numeric matrix or data.frame with >1 columns and >1 rows"
  if(length(dim(dat)) !=2) stop(msg)
  if(ncol(dat) <2) stop(msg)
  if(is.data.frame(dat)) dat <- as.matrix(dat)
  if(length(grp) != ncol(dat)) stop("Number of columns in 'dat' not matching levels of 'grp'")
  nGrp <- table(grp)
  nGrp <- nGrp[order(unique(grp))]
  if(length(nGrp) <2) stop(" too few levels in 'grp' !")
  if(any(nGrp <= maxNA)) stop(" some levels of 'grp' with too few instances !")
  ## main
  out <- ou2 <- rep(0,nrow(dat))
  for(i in length(levels(grp)):1) {
    tmp <- dat[,which(grp==levels(grp)[i])]
    out[which(rowSums(is.na(tmp)) < maxNA)] <- i
    }
  for(i in 1:length(levels(grp))) {
    tmp <- dat[,which(grp==levels(grp)[i])]
    ou2[which(rowSums(is.na(tmp)) < maxNA)] <- i
    }
  out <- cbind(from=out,to=ou2)
  rownames(out) <- rownames(dat)
  out[which(out <1)] <- NA
  out }

#' Row-Means With Destinction Of Groups (Of Columns, eg Groups Of Replicates)
#'
#' Thus function calculates column-means for matrix with multiple groups of data, ie similar to rowMeans but one mean for each group of data.
#' Groups are specified as columns of 'x' in 'grp' (so length of grp should match number of columns of 'x', NAs are allowed).
#'
#' @param x (numeric matrix or data.frame) main input
#' @param grp (character or factor) defining which columns should be grouped (considered as replicates)
#' @param na.rm (logical) a logical value indicating whether \code{NA}-values should be stripped before the computation proceeds.
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) This function allows easier tracking of messages produced
#' @return This function returns a matrix with mean values
#' @seealso \code{\link{rowSds}}, \code{\link[base]{colSums}} which also shows \code{colMeans} 
#' @examples
#' set.seed(2016); dat1 <- matrix(c(runif(200) +rep(1:10,20)), ncol=10)
#' head(rowGrpMeans(dat1, gr=gl(4, 3, labels=LETTERS[1:4])[2:11]))
#' @export
rowGrpMeans <- function(x, grp, na.rm=TRUE, silent=FALSE, debug=FALSE, callFrom=NULL){
  if(!is.matrix(x) && !is.data.frame(x)) stop(" 'x' should be data.frame or matrix")
  if(length(dim(x)) !=2) stop(" 'x' should be data.frame or matrix of 2 dimensions")
  if(length(grp) != ncol(x)) stop(" 'grp' should be of length of number of cols in 'x'")
  if(length(grp) <1 || all(is.na(grp))) stop(" 'grp' appears to be empty or all NAs")
  if(!is.factor(grp)) grp <- as.factor(grp)
  if(!is.matrix(x)) x <- matrix(as.matrix(x), nrow=nrow(x), dimnames=if(length(dim(x)) >1) dimnames(x) else list(names(x),NULL))
  if(length(na.rm) !=1 || !is.logical(na.rm)) na.rm <- TRUE
  ## main
  out <- .rowGrpMeans(x, grp, na.rm=na.rm) 
  chNan <- is.nan(out)
  if(any(chNan)) out[which(chNan)] <- NA
  out }
 
#' row group mean (main)
#'
#' This function calculates CVs for matrix with multiple groups of data, ie one CV for each group of data. 
#' 
#' @param x numeric matrix where relplicates are organized into separate columns
#' @param grp (factor) defining which columns should be grouped (considered as replicates)
#' @param na.replVa (numeric) value to replace \code{NA} values
#' @param na.rm (logical) remove all \code{NA} values 
#' @return This function returns a matrix of mean values per row and group of replicates
#' @seealso \code{\link{rowGrpCV}}, \code{\link{rowCVs}}, \code{\link{arrayCV}}, \code{\link{replPlateCV}}
#' @examples
#' set.seed(2016); dat1 <- matrix(c(runif(200)+rep(1:10,20)),ncol=10)
#' grp1 <- gl(4,3,labels=LETTERS[1:4])[2:11]
#' head(.rowGrpMeans(dat1, grp1))
#' @export
.rowGrpMeans <- function(x, grp, na.replVa=NULL, na.rm=TRUE){
  ## determine means of rows conditional as (multiple) groups
  ## NAs (eg from counting data) can be replaced by specified value 'na.replVa', eg 0)
  ## 'grp' expected as factor !!
  if(!is.null(na.replVa)) x[is.na(x)] <- na.replVa
  grNa <- unique(naOmit(as.character(grp)))
  out <- matrix(nrow=nrow(x), ncol=length(grNa), dimnames=list(rownames(x),grNa))
  for(i in 1:length(grNa)) {
    useC <- which(as.character(grp)==grNa[i])
    out[,i] <- if(length(useC) >1) base::rowMeans(x[,useC], na.rm=na.rm) else x[,useC] }
  out }
   

#' Find And Combine Points Located Very Close In x/y Space
#'
#' Search points in x,y space that are located very close and thus likely to overlap.
#' In case of points close enough, various options for joining names (and shortening longer descriptions) are available. 
#'
#' @param dat (matrix) matrix or data.frame with 2 cols (used ONLY 1st & 2nd column !), used as x & y coordinates
#' @param suplInfo (NULL or character) when points are considered overlapping the text from 'suplInfo' will be reduced to fragment before 'txtSepChar' and combined (with others from overlapping text) using 'combSym', if NULL $combInf will appear with row-numbers
#' @param disThr (numeric) distance-thrshold for considering as similar via searchDataPairs()
#' @param addNsimil (logical) include number of fused points
#' @param txtSepChar (character) for use with .retain1stPart(): where to cut (& keep 1st part) text from 'suplInfo' to return in out$CombInf; only 1st element used !
#' @param combSym (character) concatenation symbol (character, length=1) for points considered overlaying, see also 'suplInfo'
#' @param maxOverl (integer) if NULL no limit or max limit of group/clu size (avoid condensing too many neighbour points to single cloud)
#' @param debug (logical) additional messages for debugging
#' @param silent (logical) suppres messages
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a matrix with fused (condensed) information for cluster of overapping points
#' @examples
#' set.seed(2013)
#' datT2 <- matrix(round(rnorm(200)+3,1),ncol=2,dimnames=list(paste("li",1:100,sep=""),
#'   letters[23:24]))
#' # (mimick) some short and longer names for each line
#' inf2 <- cbind(sh=paste(rep(letters[1:4],each=26),rep(letters,4),1:(26*4),sep=""),
#'  lo=paste(rep(LETTERS[1:4],each=26),rep(LETTERS,4),1:(26*4),",",rep(letters[sample.int(26)],4),
#'   rep(letters[sample.int(26)],4),sep=""))[1:100,] 
#' head(datT2,n=10)
#' head(combineOverlapInfo(datT2,disThr=0.03),n=10)
#' head(combineOverlapInfo(datT2,suplI=inf2[,2],disThr=0.03),n=10)
#' @export
combineOverlapInfo <- function(dat,suplInfo=NULL,disThr=0.01,addNsimil=TRUE,txtSepChar=",",combSym="+",maxOverl=50,callFrom=NULL,debug=FALSE,silent=FALSE){
  fxNa <- .composeCallName(callFrom, newNa="combineOverlapInfo")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  
  if(length(dim(dat)) !=2) stop("expecting matrix or data.frame with 2 cols as 'dat'")
  if(!is.null(suplInfo)){
    suplInfo <- as.character(suplInfo)
    if(length(suplInfo) != nrow(dat)) { message(fxNa," number of rows of 'dat' (",nrow(dat),
      ") should match length of 'suplInfo' (",length(suplInfo),") .. ignoring")
      suplInfo <- NULL }}
  datOverl <- searchDataPairs(dat[,1:2], disThr=disThr, byColumn=FALSE, realDupsOnly=FALSE, callFrom=fxNa)
  txt <- " overlap-pairs: making condensed groups via fusePairs() will take "
  if(!silent && nrow(datOverl) >1800) message(fxNa,nrow(datOverl),txt, if(nrow(datOverl) >5000) "VERY (!) " else "","MUCH TIME !")
  modDatLi0 <- sort(unique(naOmit( as.character(as.matrix(datOverl[,1:2])))))      # get all line-numbers with (potentially) close points
  modDatLi2 <- unique(convToNum(modDatLi0, remove=NULL, sciIncl=TRUE))
  modDatLi <- if(length(modDatLi2)==length(modDatLi0)) modDatLi2 else match(modDatLi0,rownames(dat))   # convert to numeric if possible, otherwise as rownames
   if(length(modDatLi) <2) {
    if(!silent) message(fxNa,"all entries seem non-overlapping, no need to combine overlapping ones")
    return(cbind(dat, clu=1:nrow(dat), combInf=if(length(suplInfo) <1) 1:nrow(dat) else suplInfo, clu=1:nrow(dat), isComb=FALSE))
  } else {             # aggregate points considered potentially overlapping to 'clusters'
    datOverl[,1] <- as.character(datOverl[,1])             
    datOverl[,2] <- as.character(datOverl[,2])
    if(debug) {message(fxNa,"combOI\n")}
    tmp <- fusePairs(datOverl, refDatNames=rownames(dat), inclRepLst=TRUE, maxFuse=maxOverl, callFrom=fxNa, debug=FALSE)
    if(is.na(txtSepChar)) txtSepChar <- "_" else if(txtSepChar %in% c("+",".")) txtSepChar <- paste0("\\",txtSepChar)
    if(is.null(suplInfo)) suplInfo <- 1:nrow(dat)
    redInfo <- suplInfo
    if(debug) {message(fxNa,"xxComb3\n")}
    if(!all(is.na(suplInfo))) for(i in unique(tmp$clu)) {                # loop along cluster-names
      j <- match(names(tmp$clu)[which(tmp$clu==i)],rownames(dat))        # ie, at these locations
      if(length(j) >1 && length(txtSepChar)==1) redInfo[j] <- paste(.retain1stPart(
        suplInfo[j],sep=txtSepChar,offSet=1), collapse=sub("\\\\","",combSym)) }     
    out <- data.frame(dat, combInf=NA, clu=NA, isComb=FALSE, stringsAsFactors=FALSE)    
    if(length(suplInfo) ==nrow(dat)) { out$combInf <- redInfo }  
    out[,"clu"] <- tmp$clu[match(rownames(dat),names(tmp$clu))]
    if(any(is.na(out[,"clu"]))) out[which(is.na(out[,"clu"])),"clu"] <- max(out[,"clu"],na.rm=TRUE) +(1:sum(is.na(out[,"clu"])))    # in case of NAs, attribue new no
    cluTab <- table(out[,"clu"])
    out[which(!is.na(match(out[,"clu"],names(cluTab)[which(cluTab >1)]))),"isComb"] <- TRUE
    out }}
   

#' Write (And Convert) Csv Files
#'
#' This functions allows for more options when writing data into csv-files compated to  \code{write.csv}.
#' 
#'
#' @details
#' The main input may be gven as R-object or read from file 'input'. Then, one can (re-)write using specified conversions. 
#' An optional filter to select columns (column-name specified via 'filterCol') is available.
#'  
#' The output may be simultaneaously written to multiple formats, as specified in 'expTy', 
#' tabulation characters may be converted to avoid accidentally split/shift text to multiple columns. 
#' Note: Mixing '.' and ',' as comma separators via text-columns or fused text&data may cause problems lateron, though.
#'
#' @param input either matrix or data.frame
#' @param inPutFi (character or \code{NULL}) file-name to be read (format as US or Euro-type may specified via argument \code{imporTy})
#' @param expTy (character) 'US' and/or 'Eur' for sparator and decimal type in output
#' @param imporTy (character) default 'Eur' (otherwise set to 'US')
#' @param filename (character) optional new file name(s)
#' @param quote (logical) will be passed to function \code{write.csv}
#' @param filterCol (integer or character) optionally, to export only the columns specified here
#' @param replMatr optional, matrix (1st line:search, 2nd li:use for replacing) indicating which characters need to be replaced )
#' @param returnOut (logical) return output as object
#' @param SYLKprevent (logical) prevent difficulty when opening file via Excel. In some cases Excel presumes (by error) the SYLK format and produces an error when trying to open files : 
#'  To prevent this, if necessary, the 1st column-name will be changed from 'ID' to 'Id'.
#' @param digits (interger) limit number of signif digits in output (ie file)
#' @param silent (logical) suppress messages
#' @param debug (logical) for bug-tracking: more/enhanced messages  
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function writes a file to disk and returns \code{NULL} unless \code{returnOut=TRUE}
#' @seealso \code{write.csv} in \code{\link[utils]{write.table}}, batch reading using this package \code{\link{readCsvBatch}}
#' @examples
#' dat1 <- data.frame(ini=letters[1:5],x1=1:5,x2=11:15,t1=c("10,10","20.20","11,11","21,21","33.33"),
#'   t2=c("10,11","20.21","kl;kl","az,az","ze.ze"))
#' fiNa <- file.path(tempdir(), paste("test",1:2,".csv",sep=""))
#' writeCsv(dat1, filename=fiNa[1])
#' dir(path=tempdir(), pattern="cs")
#'
#' (writeCsv(dat1, replM=rbind(bad=c(";",","), replBy="__"), expTy=c("Eur"),
#'   returnOut=TRUE, filename=fiNa[2]))
#'
#' @export
writeCsv <- function(input, inPutFi=NULL, expTy=c("Eur","US"), imporTy="Eur", filename=NULL, quote=FALSE, filterCol=NULL, replMatr=NULL, returnOut=FALSE, SYLKprevent=TRUE, digits=22, silent=FALSE, debug=FALSE, callFrom=NULL) {
  fxNa <- .composeCallName(callFrom, newNa="writeCsv")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE

  argN <- deparse(substitute(input))
  doWrite <- TRUE
  if(!requireNamespace("utils", quietly=TRUE)) { doWrite <- FALSE
    warning(fxNa,"Package 'utils' not found ! Please install first from CRAN") 
  }  
  if(length(input) <1) { doWrite <- FALSE; warning(fxNa," 'input'  should be data or filename")}
  
  if(doWrite) {
    if(is.character(input)) {if(file.exists(input)) {inPutFi <- as.character(dat)[1]; dat <- NULL
      if(!silent) message(fxNa,"trying to read  ",inPutFi,"  as format: ",imporTy)
        dat <- if(imporTy=="Eur") try(utils::read.csv2(inPutFi,stringsAsFactors=FALSE),silent=TRUE) else {
          if(imporTy=="US") try(utils::read.csv(inPutFi, stringsAsFactors=FALSE),silent=TRUE) else try(utils::read.table(inPutFi,stringsAsFactors=FALSE),silent=TRUE) }
        if("try-error" %in% class(dat)) {message(fxNa,"PROBLEM when trying to open file '",inPutFi,"' - abandon"); doWrite <- FALSE }
      } else { dat <- input
        if(!silent) message(fxNa," 'input' is character but since no corresponding to existing filename, trying to interpret as data to be written to file")}
    } else  dat <- input
    if(doWrite) {
      if(!is.null(dat)) { if(length(inPutFi) >1) message(fxNa,"Ignoring content of 'inPutFi'")
         inPutFi <- NULL; imporTy <- ""}
      if(length(dim(dat)) <2) dat <- as.matrix(dat)     # fx created for typical case of data.frame or matrix
      datColCl <- rep(NA,ncol(dat))
      for(i in 1:ncol(dat)) datColCl[i] <- class(dat[,i])                            # document class for each column (won't work using apply)
      if(length(expTy) <1) {expTy <- "Eur"; if(!silent) message(fxNa,"Unkown format for 'expTy', setting to 'Eur'")}
      expTy <- sort(stats::na.omit(expTy))
      if(length(expTy) <1) {expTy <- "Eur"; if(!silent) message(fxNa,"Unkown format for 'expTy', setting to 'Eur'")}
      ## FILTERING  (independent to expTy)
      ## check if 'filterCol' in dat
      if(is.list(filterCol) && length(filterCol) >0) {
        useCol <- sapply(filterCol,function(x) x[1])
        useCol <- which(useCol %in% colnames(dat))
        if(!silent && length(useCol)<1) message(fxNa,"None of the columns from 'filterCol' found in ",inPutFi)
        filtThr <- sapply(filterCol[useCol], function(x) if(length(x>1)) x[2] else NA)
        filterCol <- sapply(filterCol[useCol], function(x) x[1])
        }
      if(debug) {message(fxNa,"..xxWriteC0")}
      if(length(filterCol) >0) {
        for(i in 1:length(filterCol)) {                          
          chLogi <- TRUE                                         
          ## this may be further deveoped: check if column has usable logical content
          chLi <- if(is.na(filtThr[i]) && chLogi) which(dat[,filterCol[i]]) else which(dat[,filterCol[i]] < filtThr[i])
          if(length(chLi) <nrow(dat) & length(chLi) >0) dat <- dat[chLi,] else if(length(chLi) <1) {
            message(fxNa,"Filtering for ",filterCol[i]," nothing left !!")
            dat <- NULL; return(NULL) }}
      } else if(!is.character(inPutFi)) expTy <- expTy[which(!expTy  %in% imporTy)]        # no need to re-write same file if no filtering (unless inPutFi is filename)
      ##
      ## treat non-conform characters (only in non-numeric cols, dependent on expTy)
      ##  locate non-conform characters, then subtitute in non-numeric part of dat
      datExp <- list()
      if(!all(datColCl %in% c("numeric","integer"))){                               # nothing to substitute if only numeric data, otherwise :
        chCols <- which(!datColCl %in% c("numeric","integer"))
        dat0 <- as.matrix(dat[,chCols])
        ## first try to locate numeric cols with bad separator (digit+comma+digit)
        zz <- sub("^[[:digit:]]+,[[:digit:]]+$|^[[:digit:]]+\\.[[:digit:]]+$|^,[[:digit:]]+$|^\\.[[:digit:]]+$","", dat0)     # no need to test for digits only wo separators
        chNA <- is.na(zz)
        if(any(chNA)) zz[which(chNA)] <- 0
        toNum <- colSums(nchar(zz)) <1
        if(debug) {message(fxNa,"..xxWriteC1b")}
        if(any(toNum)) {
          if(!silent) message(fxNa,"Adjusting ",sum(toNum)," column(s) with Euro comma-separator")
          dat[,chCols[which(toNum)]] <- as.numeric(sub(",",".", dat[,chCols[which(toNum)]]))
          datColCl[chCols[which(toNum)]] <- "numeric"
          chCols <- which(!datColCl %in% c("numeric","integer")) }                   # refresh
        ## locate & replace 'bad' characters interfering with tabular separation -> need multiple versions
        if(debug) {message(fxNa,"..xxWriteC2")}
        if(length(chCols) >0) for(ty in expTy) {
          dat0 <- as.matrix(dat[,chCols])                                             # refresh
          replMat <- if(is.null(replMatr))  array(c(";"," ", ","," ", "\t"," "),            
             dim=c(2,1,3),dimnames=list(c("bad","subst"),c("sep1"),c("Eur","US","txt"))) else replMatr
          if(length(dim(replMat)) >2) {
            chTy <- ty %in% dimnames(replMat)[[3]]                    
            replMat <- as.matrix(if(!chTy) replMat[,,1] else replMat[,,which(ty==dimnames(replMat)[[3]])])}  # matrix with characters to test for (1st line) & 2nd line for replacing
          locCh <- lapply(replMat[1,], grep, dat0)
          locCh <- locCh[which(sapply(locCh, length) >0)]                                            # indexes where substitution should take place
          if(length(locCh) >0) for(i in 1:length(locCh)) dat0[locCh[[i]]] <- gsub(replMat[1,i],replMat[2,i],dat0[locCh[[i]]])
          if(debug) {message(fxNa,"..xxWriteC3")}
          if(length(expTy) >1) {datExp[[ty]] <- dat; datExp[[ty]][,chCols] <- dat0} else dat[,chCols] <- dat0 }}
      ## idea (future) -make optional non-redundant version ? allowing to replace completely .exportFilteredCSV()
      ##  prevent Excel trying SYLK format : replace 1st col from 'ID' to 'Id'
      if(SYLKprevent) {if(is.list(datExp)) { chID <- grep("^ID",sapply(datExp,function(x) colnames(x)[1])) >0
        if(any(chID)) for(i in which(chID)) colnames(datExp[[i]])[1] <- sub("^ID","Id",colnames(datExp[[i]])[1])
      } else { chID <- grep("^ID",colnames(datExp)[1])
        if(chID) colnames(datExp)[1] <- sub("^ID","Id",colnames(datExp)[1])}}
      ## write to file
      if(length(filename) <1) filename <- paste0(if(is.character(inPutFi)) sub("\\.csv$","",inPutFi) else argN,".",expTy,".csv")
      if(length(filename) < length(expTy)) {
        if(!silent) message(fxNa," type to name(s) of file(s) to be written")
        filename <- paste0(sub("\\.csv$","",filename),".",expTy,".csv")
        if("txt" %in% expTy) filename <- sub("\\.txt.\\csv$",".txt",filename)
        names(filename) <- expTy }
      if(debug) {message(fxNa,"..xxWriteC4")}
      if(!silent && any(file.exists(filename))) message(fxNa,"file(s) ",pasteC(filename[which(file.exists(filename))],quo="'")," will be overwritten !")
      if( "US" %in% expTy) tryW <- try(utils::write.csv(as.matrix(format(if(length(datExp)>0) datExp$US else dat,digits=digits)), filename["US"], row.names=FALSE, quote=quote),silent=silent)
      if("txt" %in% expTy) tryW <- try(utils::write.table(as.matrix(format(if(length(datExp)>0) datExp$txt else dat,digits=digits)), filename["txt"], row.names=FALSE, quote=quote),silent=silent)
      ## idea (relaed to problem when input is fused numeric&text): in case of Eur test all cols if factor/text and then (optional?) convert '.' to ','
      if("Eur" %in% expTy) tryW <- try(utils::write.csv2(as.matrix(format(if(length(datExp)>0) datExp$Eur else dat,digits=digits)), filename[1], row.names=FALSE, quote=quote),silent=silent)
      ## possibility to return values :
      if(returnOut) {if(length(expTy) <2) dat else datExp} } } }
       

#' Fuse annotation matrix to initial matrix
#'
#' In a number of instances experimental measurements and additional information (annotation) are provided by separate objects (matrixes) as they may not be generated the same time.
#' The aim of this function is provide help when matching approprate lines for 2 sets of data (experimental measures in \code{iniTab} and annotation from \code{annotTab}) for fusing.  
#' \code{fuseAnnotMatr} adds suppelmental columns/annotation to an initial matrix \code{iniTab} : using column 'refIniT' as key (in \code{iniTab}) to compare with key 'refAnnotT' (from 'annotTab'). 
#' The columns to be added from \code{annotTab} must be chosen explicitely. 
#' Note: if  non-unique IDs in iniTab : runs slow (but save) due to use of loop for each unique ID.
#' @param iniTab (matrix), that may have lines with multiple (=repeated) key entries 
#' @param annotTab (matrix) containing reference annotation
#' @param refIniT (character) type of reference (eg 'Uniprot')
#' @param refAnnotT (character) column name to use for reference-annotation
#' @param addCol (character) column-namess of 'annotTab' to use/extract (if no matches found, use all)
#' @param debug (logical) for bug-tracking: more/enhanced messages 
#' @param silent (logical) suppress messages
#' @param callFrom (character) allow easier tracking of message(s) produced
#' @return This function returns a combined matrix (elements not found in 'annotTab' are displayed as NA)
#' @seealso \code{\link[base]{merge}}
#' @examples
#' tab0 <- matrix(rep(letters[1:25],8),ncol=10)
#' tab1 <- cbind(Uniprot=paste(tab0[,1],tab0[,2]),col1=paste(tab0[,3],
#'   tab0[,4],tab0[,5]," ",tab0[,7],tab0[,6]))
#' tab2 <- cbind(combName=paste(tab0[,1],tab0[,2]),col2=paste(tab0[,8],tab0[,9],tab0[,10]))
#' fuseAnnotMatr(tab1,tab2[c(20:11,2:5),],refIni="Uniprot",refAnnotT="combName",addCol="col2")
#' fuseAnnotMatr(tab2[c(20:11,2:5),],tab1,refAnnotT="Uniprot",refIni="combName",addCol="col1")
#' @export
fuseAnnotMatr <- function(iniTab, annotTab, refIniT="Uniprot", refAnnotT="combName",
  addCol=c("ensembl_gene_id","description","geneName","combName"), debug=TRUE, silent=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom,newNa="fuseAnnotMatr")
  argN <- c(deparse(substitute(iniTab)), deparse(substitute(annotTab)))
  concSym <- "; "                    # exeption to IDs (IDs from 'iniTab' containing these characters won't be considered)
  chMatr <- list(dim(iniTab), dim(annotTab))
  msg <- c(paste(argN,collapse=" &"),"must be matrix or data.frame","are expected to have >1 rows and >1 columns")
  if(any(sapply(chMatr,length) <1)) stop(msg[1:2])
  if(any(unlist(chMatr) <2) & !silent) message(fxNa,msg[c(1,3)])
  if(length(unique(annotTab[,refAnnotT])) < nrow(annotTab)) annotTab <- combineRedBasedOnCol(annotTab, refAnnotT, callFrom=fxNa)
  chKey <- c(refIniT %in% colnames(iniTab), refAnnotT %in% colnames(annotTab))
  if(any(!chKey)) stop(paste(c(refIniT, refAnnotT)[which(!chKey)],collapse=" & ")," not found in initial matrix(es) !!")
  useCol <- which(colnames(annotTab) %in% addCol)
  if(length(useCol) <1 || identical(refAnnotT,addCol)) {
    if(!silent) message(fxNa,"NONE of 'addCol' found in ",argN[2],"!!  use/return all cols")
    useCol <- 1:ncol(annotTab)
  } else if(length(addCol) < length(useCol)) message(fxNa,length(addCol)-length(useCol)," columns of 'addCol' not found in ",argN[2])
  chKey <- sum(annotTab[,refAnnotT] %in% iniTab[,refIniT],na.rm=TRUE)
  if(chKey <1 && !silent) message(fxNa,"NO matches of keys found between ",msg[1]," !!")
  uniqGe <- unique(iniTab[,refIniT])
  chConcSym <- grep(concSym,uniqGe)
  if(length(chConcSym) >0) uniqGe <- uniqGe[-1*grep(concSym,uniqGe)]                             #
  if(length(uniqGe) == nrow(iniTab)) {
    annotTa2 <- as.data.frame(annotTab[,unique(c(which(colnames(annotTab)==refAnnotT),useCol))],stringsAsFactors=FALSE)
    out <- merge(iniTab,annotTa2,all.x=TRUE,by.x=refIniT,by.y=refAnnotT)
  } else {
    if(length(uniqGe) >2 && !silent) message(fxNa,"Non-unique IDs found :  be patient ! (running loop across all IDs)")
    supAnn <- matrix(NA_real_, nrow(iniTab), length(useCol), dimnames=list(iniTab[,"Uniprot"], colnames(annotTab)[useCol]))  # (empty) for cbind
    for(i in uniqGe) { useLi <- which(rownames(supAnn)==i)
      if(length(useLi) >0) { j <- which(annotTab[,refAnnotT]==i)
      if(length(j) >0) supAnn[useLi,] <- matrix(rep(annotTab[j,useCol], each=length(useLi)), nrow=length(useLi)) }}
    out <- cbind(iniTab, supAnn) }
  as.matrix(out) }
     

#' Report number of unique and redundant elements (optional figure)
#'
#' Make report about number of unique and redundant elements of vector 'dat'.
#' Note : fairly slow for long vectors !!
#' @param dat (charcter or numeric vector) main input where number of unique (and redunant) should be determined
#' @param frL (logical) optional (re-)introducing results from \code{duplicated} to shorten time of execution 
#' @param plotDispl (logical) decide if pie-type plot should be produced
#' @param tit (character) optional title in plot
#' @param col (character) custom colors in pie
#' @param radius (numeric) radius passed to  \code{pie} 
#' @param sizeTo (numeric or charcter) optional reference group for size-population relative adjusting overall surface of pie 
#' @param clockwise (logical) argument passed to pie 
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return vector with counts of n (total), nUnique (wo any repeated), nHasRepeated (first of repeated), nRedundant), optional figure
#' @seealso \code{\link{correctToUnique}}, \code{\link[base]{unique}}
#' @examples
#' layout(1:2)
#' uniqCountReport(rep(1:7,1:7),plot=TRUE)
#' uniqCountReport(rep(1:3,1:3),plot=TRUE,sizeTo=rep(1:7,1:7))      
#' @export
uniqCountReport <- function(dat, frL=NULL, plotDispl=FALSE, tit=NULL, col=NULL, radius=0.9, sizeTo=NULL, clockwise=FALSE, silent=FALSE, debug=FALSE, callFrom=NULL) {
  fxNa <- .composeCallName(callFrom, newNa="uniqCountReport")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE

  if(is.null(frL)) frL <- duplicated(dat,fromLast=TRUE)
  nUniQ <- length(unique(dat[which(frL)]))
  out <- c(n=length(dat), nUnique=length(unique(dat)) -nUniQ, nHasRepeated=nUniQ, nRedundant=NA )
  out[4] <- out[1] -sum(out[2:3],na.rm=TRUE)
  if(plotDispl) .plotCountPie(out,tit=tit,col=col,radius=radius,sizeTo=sizeTo,clockwise=clockwise,silent=silent,callFrom=fxNa)
  out }
  
#' Pie plot for counting results
#'
#' This function allows to inspect results of \code{table} or \code{uniqCountReport} on a pie-plot
#' Note : fairly slow for long vectors !!
#' @param count (integer vector) counting result
#' @param tit (character) optional title in plot
#' @param col (character) custom colors in pie
#' @param radius (numeric) radius passed to  \code{pie} 
#' @param sizeTo (numeric or charcter) optional reference group for size-population relative adjusting overall surface of pie 
#' @param clockwise (logical) argument passed to pie 
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return vector with counts of n (total), nUnique (wo any repeated), nHasRepeated (first of repeated), nRedundant), optional figure
#' @seealso \code{\link{uniqCountReport}}, \code{\link{correctToUnique}}, \code{\link[base]{unique}}
#' @examples
#' .plotCountPie(table(c(1:5,4:2)))
#' @export
.plotCountPie <- function(count, tit=NULL, col=NULL, radius=0.9, sizeTo=NULL, clockwise=FALSE, silent=FALSE, debug=FALSE, callFrom=NULL) {
  ## plotting for .plotCountPie()
  fxNa <- .composeCallName(callFrom, newNa=".plotCountPie")

  if(is.null(col)) col <- c(3,4,2) 
  if(length(sizeTo) >0) {if(is.character(sizeTo) || length(sizeTo) >1) sizeTo <- length(sizeTo)
    radius <- radius*sqrt(count[1]/sizeTo)
    msg <- "Note: reference is soo high that size-adoped pie won't fit into plotting region, consider lowering 'radius' for all plots" 
    if(radius > 1.09 && !silent) message(fxNa,msg)}
  graphics::pie(count[c(2,3,4)],col=col,init.angle=90,radius=radius,main=tit,labels=c("unique","hasRepeated","redundant"),cex=sort(c(0.55,0.3+radius,1))[2],clockwise=clockwise)
  graphics::mtext(paste("n=",count[1]),cex=0.8,adj=0,line=-0.3)
}  
  

#' sd for each column
#'
#' This function is speed optimized \code{sd} per column of a matrix or data.frame and treats each column as independent set of data for sd (equiv to \code{apply(dat,2,sd)}).
#' NAs are ignored from data.  Speed improvements may be seen at more than 100 columns
#' @param dat matrix (or data.frame) with numeric values (may contain NAs which will be ignored)
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allows easier tracking of messages produced
#' @return numeric vector of sd values
#' @seealso \code{\link[stats]{sd}}
#' @examples
#' set.seed(2016); dat1 <- matrix(c(runif(200) +rep(1:10,20)), ncol=10)
#' colSds(dat1)
#' @export
colSds <- function(dat, silent=FALSE, debug=FALSE, callFrom=NULL) {
  fxNa <- .composeCallName(callFrom, newNa="colSds")
  if(isTRUE(debug)) silent <- FALSE else { debug <- FALSE
    if(!isTRUE(silent)) silent <- FALSE }
  if(length(dat) <1) out <- NULL else {  
    msg <- "'dat' should be matrix or data.frame with "
    if(is.null(ncol(dat))) stop(msg,"multiple rows !") else if(nrow(dat) < 2) stop(msg,"at least 2 rows !")
    if(is.data.frame(dat)) dat <- as.matrix(dat)
    out <- sqrt(colSums(matrix(as.numeric(!is.na(t(dat))), ncol=nrow(dat))*(t(dat) - colMeans(dat, na.rm=TRUE))^2, na.rm=TRUE)/(colSums(!is.na(dat)) -1))  # handels NA OK
    out[is.nan(out)] <- NA }        # replace 'NaN' by NA for output
  out }
   

#' Cut character-vector at multiple sites
#'
#' This function cuts character vector after 'cutAt' (ie keep the search subtsting 'cutAt', different to \code{strsplit}). 
#' Used for theoretical enzymatic digestion (eg in proteomics)
#' @param y character vector (better if of length=1, otherwise one won't know which fragment stems from which input)
#' @param cutAt (character) search subtsting, ie 'cutting rule'
#' @return modified (ie cut) character vector
#' @seealso \code{\link[base]{strsplit}}, \code{\link[wrMisc]{nFragments0}}, \code{\link[wrMisc]{nFragments}} 
#' @examples
#' tmp <- "MSVSRTMEDSCELDLVYVTERIIAVSFPSTANEENFRSNLREVAQMLKSKHGGNYLLFNLSERRPDITKLHAKVLEFGWPDLHTPALEKI"
#' cutAtMultSites(c(tmp,"ojioRij"),c("R","K"))
#' @export
cutAtMultSites <- function(y,cutAt) {
  for(i in cutAt) {y <- strsplit(y,i); y <- as.character(unlist(sapply(y, .addLetterWoLast, addChr=i)))}; y}

#' Add letter to all elements but not last
#'
#' This function allows to add 'addChr' to all entries, without the last entry
#' 
#' @param x (character) main input
#' @param addChr (character)
#' @return This function returns a modified character vector
#' @seealso  \code{\link[base]{paste}}; used in \code{\link{cutAtMultSites}}
#' @examples
#' .addLetterWoLast(c("abc","efgh"),"Z")
#' @export  
.addLetterWoLast <- function(x, addChr) paste0(x,rep(c(addChr,""),c(length(x)-1,1)))            # add 'addChr' to all entries wo last entry  
  

#' Match Names To Concatenated Pairs Of Names   
#'
#' @description
#' The column-names of multiple pairwise testing contain the names of the initial groups/conditions tested, plus there is a separator (eg '-' in \code{moderTestXgrp}).  
#' Thus function allows to map back which groups/conditions were used by returning the index of the respective groups used in pair-wise sets. 
#' 
#' @details 
#' There are two modes of operation : 1) Argument \code{sep} is set to \code{NULL} :  The names of initial groups/conditions (\code{grpNa}) 
#' will be tested for exact pattern matching either at beginning or at end of pair-wise names (\code{pairwNa}).
#' This approach has the advantage that it does not need to be known what character(s) were used as separator (or they may change), 
#' but the disadvantage that in case the perfect \code{grpNa} was not given, the longest best match of \code{grpNa} will be returned.
#'
#' 2) The separator \code{sep} is given and exact matches at both sides will be searched.
#' However, if the character(s) from \code{sep} do appear inside \code{grpNa} no matches will be found.
#'
#' If some \code{grpNa} are not found in \code{pairwNa} this will be marked as NA.  
#' 
#' This function checks also if a double version of \code{sep} may fit better to \code{grpNa}
#' and will adjust \code{sep} accordingly (eg adjust sep='-' to '--'). 
#'	 
#' @param grpNa (character) the names of the groups of replicates (ie conditions) used to test
#' @param pairwNa (character) the names of pairwise-testing (ie 'concatenated' \code{sampNa}
#' @param sep (character) if not \code{NULL} the characters given will be used via \code{stringsplit} 
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a matrix of 2 columns with inidices of \code{sampNa} with \code{pairwNa} as rows
#' @seealso (for running multiple pair-wise test) \code{\link{moderTestXgrp}}, \code{\link[base]{grep}}, \code{\link[base]{strsplit}}
#' @examples
#' pairwNa1 <- c("abc-efg","abc-hij","efg-hij")
#' grpNa1 <- c("hij","abc","abcc","efg","klm")
#' matchSampToPairw(grpNa1, pairwNa1) 
#' 
#' pairwNa2 <- c("abc-efg","abcc-hij","abc-hij","abc-hijj","zz-zz","efg-hij")
#' matchSampToPairw(grpNa1, pairwNa2) 
#' @export
matchSampToPairw <- function(grpNa, pairwNa, sep=NULL, silent=FALSE, debug=FALSE, callFrom=NULL) {
  ## return index of grpNa for each pairwNa
  fxNa <- .composeCallName(callFrom, newNa="matchSampToPairw")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  
  chPw <- duplicated(pairwNa)
  if(any(chPw)) {if(!silent) message(fxNa,"Some entries of 'pairwNa' are duplicated, removing")
    pairwNa <- pairwNa[-which(chPw)]} 
  chNa <- duplicated(grpNa)
  if(any(chNa)) {if(!silent) message(fxNa,"Some entries of 'grpNa' are duplicated, removing")
    grpNa <- grpNa[-which(chNa)]} 
  if(length(grpNa) <2) stop("Insufficient 'grpNa' furnished") 
  if(length(pairwNa) <1) stop("Insufficient 'pairwNa' furnished")
  if(length(sep) >1) {sep <- naOmit(as.character(sep))[1]
    if(!silent) message(fxNa,"'sep' should be of length=1, using first entry")} 
  if(any(is.na(sep))) { sep <- NULL
    if(!silent) message(fxNa,"invalid entry for 'sep', setting to NULL")} 
  ## main
  if(length(sep==1)) {
    spl <- strsplit(pairwNa, as.character(sep))
    chLe <- sapply(spl, length)
    if(any(chLe <2)) stop(" Problem: Separator '",sep,"' seems NOT to occur in 'pairwNa' !")
    if(any(chLe >2)) { if(!silent) message(fxNa," separator '",sep,"' seems to occur multiple times in ",sum(chLe >2),"'pairwNa', using 1st and last of strsplit")
      spl[which(chLe >2)] <- lapply(spl[which(chLe >2)], function(x) x[c(1,length(x))]) }
    out <- t(sapply(spl, match, grpNa))
  } else {
    le <- apply(sapply(pairwNa, function(x) {w <- sapply(grpNa, function(y) length(grep(paste0("^",y),x)) >0)
      if(sum(w) >1) {v <- rep(FALSE, sum(w)); v[which.max(nchar(grpNa)[which(w)])] <- TRUE; w[which(w)] <- v}; w} ), 2, which)
    ri <- apply(sapply(pairwNa, function(x) {w <- sapply(grpNa, function(y) length(grep(paste0(y,"$"),x)) >0)
      if(sum(w) >1) {v <- rep(FALSE, sum(w)); v[which.max(nchar(grpNa)[which(w)])] <- TRUE; w[which(w)] <- v}; w} ), 2, which)
    if(is.list(le)) {le[which(sapply(le,length) <1)] <- NA; le <- unlist(le)}
    if(is.list(ri)) {ri[which(sapply(ri,length) <1)] <- NA; ri <- unlist(ri)}
    out <- cbind(le,ri) }
  #if(length(pairwNa)==nrow(out)) names(out) <- pairwNa
  dimnames(out) <- list(pairwNa,c("le","ri"))
  out }
   

#' Make MA-List Object 
#'
#' \code{makeMAList} extracts sets of data-pairs (like R & G series) and makes MA objects as \code{MA-List object} (eg for ratio oriented analysis).
#' The grouping of columns as sets of replicate-measurements is done according to argumnet \code{MAfac}.
#' The output is fully compatible to functions of package \href{https://bioconductor.org/packages/release/bioc/html/limma.html}{limma} (Bioconductor). 
#'
#' @details This function requires Bioconductor package \href{https://bioconductor.org/packages/release/bioc/html/limma.html}{limma} being installed.
#'
#' @param mat main input matrix
#' @param MAfac (factor) factor orgnaizing columns of 'mat' (if \code{useF} contains the default 'R' and 'G', they should also be part of \code{MAfac})
#' @param useF (character) two specific factor-leves of \code{MAfac} that will be used/extracted
#' @param isLog (logical) tell if data is already log2 (will be considered when computing M and A values)
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return limma-type "MAList" containing M and A values
#' @seealso \code{\link{test2factLimma}}, for creating RG-lists within limma: \code{MA.RG} in \code{\link[limma]{normalizeWithinArrays}}
#' @examples
#' set.seed(2017); t4 <- matrix(round(runif(40,1,9),2), ncol=4,
#'   dimnames=list(letters[c(1:5,3:4,6:4)], c("AA1","BB1","AA2","BB2")))
#' makeMAList(t4, gl(2,2,labels=c("R","G")))
#' @export
makeMAList <- function(mat, MAfac, useF=c("R","G"), isLog=TRUE, silent=FALSE, debug=FALSE, callFrom=NULL){
  ## extract sets of data-pairs (like R & G series) and MA objects as MA-List object (eg for ratio oriented analysis) according to 'MAfac'
  ## 'MAfac' .. factor
  ## require(limma)
  fxNa <- .composeCallName(callFrom, newNa="makeMAList")
  if(isTRUE(debug)) silent <- FALSE else { debug <- FALSE
    if(!isTRUE(silent)) silent <- FALSE }
  if(length(mat) >0 && length(dim(mat)) >1) {  
    if(requireNamespace("limma", quietly=TRUE)) {
      if(!(all(useF %in% MAfac) && length(useF) ==2 && length(MAfac) >1)) stop(fxNa,"Bad input;  Argument 'useF' should describe 2 elements of 'MAfac'")
      if(!isLog) {
        if(any(mat <0) && !silent) message(fxNa,"Negative values will create NAs at log2-transformation !") }
      out <- try(limma::MA.RG(if(isLog) list(R=2^mat[,which(MAfac==useF[1])], G=2^mat[,which(MAfac==useF[2])]) else {
        list(R=mat[,which(MAfac==useF[1])], G=mat[,which(MAfac==useF[2])])}, bc.method="subtract", offset=0), silent=TRUE)
      if(inherits(out, "try-error")) {warning(fxNa,"UNABLE to run limma::MA.RG() '!"); out <- NULL}
      out
    } else {
      return(NULL); if(!silent) message(fxNa,"NOTE: Package 'limma' not found ! Please install from Bioconductor  (returning NULL)") }
  } else if(!silent) message(fxNa,"Input is empty or invalid, returning NULL")
    }


#' Search character-string and cut either before or after 
#'
#' This function extracts/cuts text-fragments out of \code{txt} following specific anchors defined by arguments \code{cutFrom} and \code{cutTo}.
#'
#' @param dat (matrix or data.frame) main input
#' @param ty (character) type of ratio (eg 'log2')
#' @param colNaSep (character) separator 
#' @return This function returns a numeric vector
#' @seealso  \code{\link{makeMAList}}, \code{\link[base]{grep}} 
#' @examples
#' .allRatios(matrix(11:14, ncol=2))
#' @export
.allRatios <- function(dat, ty="log2", colNaSep="_") {
  ## calculate all (log2-)ratios between (entire) indiv columns of 'dat' (matrix or data.frame)
  ## return matrix with ratios (betw columns of dat)
  out <- matrix(nrow=nrow(dat), ncol=choose(ncol(dat),2))
  pwCoor <- upperMaCoord(ncol(dat))
  out <- apply(pwCoor, 1, function(x) dat[,x[1]]/dat[,x[2]])
  colnames(out) <- apply(pwCoor,1,function(x) paste(colnames(dat)[x[1]],colnames(dat)[x[2]], sep=colNaSep))
  if(identical(ty,"log2")) out <- log2(out)
  if(identical(ty,"log10")) out <- log10(out)
  if(identical(ty,"log")) out <- log(out)
  out }

#' Calculate ratios for each column to each column of reference-matrix
#'
#' This function calculates ratio(s) for each column of matrix 'x' versus all/each column(s) of matrix 'y' (reference)
#'
#' @param x (matrix or data.frame) main input1
#' @param y (matrix or data.frame) main input2
#' @param asLog2 (logical)
#' @param sumMeth (character) method
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a numeric vector or matrix in dimension of 'x' (so far summarize all ratios from mult division from mult ref cols as mean or median )
#' @seealso  \code{\link{makeMAList}}, \code{\link[base]{grep}} 
#' @examples
#' .allRatioMatr1to2(matrix(11:14, ncol=2), matrix(21:24, ncol=2))
#' @export
.allRatioMatr1to2 <- function(x, y, asLog2=TRUE, sumMeth="mean", callFrom=NULL){
  ## calculate ratio(s) for each column of matrix 'x' versus all/each column(s) of matrix 'y' (reference)
  ## return matrix in dimension of 'x' (so far summarize all ratios from mult division from mult ref cols as mean or median )
  ## 'log2' .. output as log2 format (before summarizing different ratios obtained for given column of 'x')
  ## 'sumMeth' .. for method for summarizing the ratios from diff cols of 'y' (default mean, otherwise median)
  ## variant of .allRatios() but including external reference ('y')
  fxNa <- .composeCallName(callFrom,newNa=".allRatioMatr1to2")
  xDimNa <- dimnames(x)
  yDimNa <- dimnames(y)
  if(length(xDimNa) <1) x <- matrix(x, ncol=1, dimnames=list(rownames(x),"x1"))
  if(length(yDimNa) <1) y <- matrix(y, ncol=1, dimnames=list(rownames(y),"y1"))
  if(nrow(x) != nrow(y)) stop(fxNa,"Number of rows in 'x' and 'y' must be equal !")
  ## main
  out <- apply(x, 2, function(da, re) matrix(rep(da,ncol(re)), ncol=ncol(re))/re, re=y)    # is this really ok ?
  if(!is.list(out)) out <- list(out)
  if(asLog2) out <- lapply(out,log2)
  out <- if(identical(sumMeth,"mean")) lapply(out,rowMeans) else lapply(out,function(x) apply(x, 1, stats::median,na.rm=TRUE))     # summarize
  nLen <- sapply(out,length)
  if(length(unique(nLen)) >1 || any(nLen <1)) message(fxNa,"Strange format of results, length of lists: ",
    paste(nLen, collapse=" "))
  out <- out[which(nLen >0)]
  out <- if(length(out) >1) matrix(unlist(out), nrow=nrow(x), dimnames=xDimNa) else out[[1]]
  out }

#' Get A value for each group of replicates
#'
#' This function calculates the 'A' value (ie group mean) for each group of replicates (eg for MA-plot)
#'
#' @param dat (matrix or data.frame) main input
#' @param grp (factor) grouping of replicates
#' @return This function returns a numeric vector
#' @seealso  \code{\link{makeMAList}}
#' @examples
#' .getAmean(matrix(11:18, ncol=4), gl(2,2))
#' @export
.getAmean <- function(dat, grp) {
  ## get A value (ie group mean) for each group of replicates (eg for MA-plot)
  ## NOTE : this fx is redundant; does about the same as .rowGrpMeans() !
  if(length(levels(grp)) <2) stop("Problem: 'grp' as factor should have at least 2 levels")
  if(length(grp) != ncol(dat)) stop("Problem: length of 'grp' should match numbe of columns in 'dat'")
  if(!is.factor(grp)) grp <- as.factor(grp)
  out <- matrix(nrow=nrow(dat), ncol=length(levels(grp)), dimnames=list(rownames(dat), levels(grp)))
  for(i in 1:length(levels(grp))) out[,i] <- rowMeans(dat[,which(grp==levels(grp)[i])],na.rm=TRUE)
  out }

#' Get A value for each group of replicates based on comp
#'
#' This function calculates the 'A' value (ie group mean) for each group of replicates (eg for MA-plot)
#' \code{comp} is matrix telling which groups to use/compare, assuming that dat are already group-means)
#'
#' @param dat (matrix or data.frame) main input
#' @param comp (matrix) tells which groups to use/compare, assuming that dat are already group-means)
#' @return This function returns a numeric vector
#' @seealso  \code{\link{makeMAList}}
#' @examples
#' .getAmean(matrix(11:18, ncol=4), gl(2,2))
#' @export
.getAmean2 <- function(dat, comp) as.matrix(apply(comp, 1, function(co,x) rowMeans(x[,co],na.rm=TRUE),dat))   # transform dat to A-values for MA-plot (comp is matrix telling which groups to use/compare, assuming that dat are already group-means)


#' Get M value for each group of replicates based on comp
#'
#' This function calculates the 'M' value (ie log-ratio) for each group of replicates based on comp (eg for MA-plot)
#' \code{comp} is matrix telling which groups to use/compare, assuming that dat are already group-means)
#'
#' @param dat (matrix or data.frame) main input
#' @param comp (matrix) tells which groups to use/compare, assuming that dat are already group-means)
#' @return This function returns a numeric vector
#' @seealso  \code{\link{makeMAList}}
#' @examples
#' .getAmean(matrix(11:18, ncol=4), gl(2,2))
#' @export
.getMvalue2 <- function(dat, comp) as.matrix(apply(comp, 1, function(co,x) diff(t(x[,co])),dat))      # transform dat to M-values  ; comp : matrix (2 columns) indicating which columns should be compared
  

#' Unify Enumerators
#' 
#' The aim of this function is to provide help in automatically harmonizing enumerators at the end of sample-names.
#' When data have same grouped setup/design, many times this is reflected in their names, eg 'A_sample1', 'A_sample2' and 'B_sample1'.
#' However, human operators may use multiple similar (but not identical) ways of expressing the same meanin, eg writng 'A_Samp_1'.
#' This function allows testing a panel of different extensions of enumerators and (if recognized) to replace them by a user-defined standard text/enumerator.
#' Please note that the more recent function \code{\link{rmEnumeratorName}} offers better/more flexible options.
#'
#' @details
#' This function has been developed for matching series of the same samples passing in parallel through different evaluation software (see R package wrProteo).
#' The way human operators may name things may easily leave room for surprises and this function allows testing only a limited number of common ways of writing.
#' Thus, in any case, the user is advised to inspect the results by eye and - if needed- to adjust the parameters.
#'
#' Basically enumerator separators can be constructed by combing a base-separator \code{baseSep} (like '-', '_' etc) and an enumerator-abbreviation \code{suplEnu}.
#' Then, all possible combinations will be tested if they occur in the text \code{x}.
#' Furthermore, the text searched has to be followd by on or multiple digts at the end of text-entry (decimal comma-separators etc are not allowed).
#' Thus, if there is other 'free text' following to the right after the enumerator-text this function will not find any enumerators to replace.
#'
#' The argument \code{stringentMatch} allows defining if this text has to be found in all text-entries of \code{x} or just one of them.
#' Whe using \code{stringentMatch=FALSE} there is risk that other text not meant to design enumerators may be picked up and modified.
#'
#' Please note, that with large data-sets (ie many columns) testing/checking a larger panel of enumerator-abreviations may result in slower performance.
#' In cases of larger data-sets it may be more effective to first study the data and then run simple subsitions using sub targeted for this very case.
#'
#' @param x (character) main input
#' @param refSep (character) separator for output
#' @param baseSep (character) basic seprators to test (you have to protect special characters)
#' @param suplEnu (character) additional text
#' @param stringentMatch (logical) decide if enumerator text has to be found in all instances or only once
#' @param silent (logical) suppress messages
#' @param callFrom (character) allow easier tracking of messages produced
#' @param debug (logical) display additional messages for debugging
#' @return This function returns a character vector of same length as input \code{x}, with it's content as adjusted enumerators
#' @seealso \code{\link{rmEnumeratorName}} for better/more flexible options; \code{\link[base]{grep}} or \code{sub()}, etc if exact and consistent patterns are known
#' @examples
#' unifyEnumerator(c("ab-1","ab-2","c-3"))
#' unifyEnumerator(c("ab-R1","ab-R2","c-R3"))
#' unifyEnumerator(c("ab-1","c3-2","dR3"), strin=FALSE);
#'
#' @export
unifyEnumerator <- function(x, refSep="_", baseSep=c("\\-","\\ ","\\."), suplEnu=c("Repl","Rep","R","Number","No","Sample","Samp"), stringentMatch=TRUE, silent=FALSE, debug=FALSE, callFrom=NULL) {
  ## unify Enumerators (towards end, must be followed by terminal digit) to refSep
  ## redSep (character, length=1)
  #example#
  fxNa <- .composeCallName(callFrom, newNa="unifyEnumerator")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  if(debug) {message(fxNa,"uE1")}

  ## need to filter as much as possible t keep combinatorics to check low  (init check without considering combination of baseSep, suplEnu and digits)
  chBa <- sapply(baseSep, function(y) length(grep(y, x)))               # check if they occur at all, no matter if with digit or not..
  chBa <- if(stringentMatch) chBa==length(x) else chBa >0
  chRe <- length(grep(refSep, x))                                                                                    # check if ref occur at all, no matter if with digit or not..
  chRe <- if(stringentMatch) chRe==length(x) else chRe >0
  if(debug) message(fxNa,"uE1b ",pasteC(chBa), "   chRe ",pasteC(chRe))
  if(any(chBa, chRe)) { baseSep <- if(any(chBa)) baseSep[which(chBa)] else NULL
    if(debug) {message(fxNa,"uE2") ; uE2 <- list(x=x,refSep=refSep,baseSep=baseSep,suplEnu=suplEnu,chBa=chBa,chRe=chRe)}
    ## test enumerators
    if(length(suplEnu) >0) {
      suplEnu <- union(suplEnu, tolower(suplEnu))
      chSup <- sapply(suplEnu, function(y) length(grep(y, x)))                                  # check if they occur at all, no matter if with digit or not..
      chSup <- if(stringentMatch) chSup==length(x) else chSup >0
      suplEnu <- if(any(chSup)) suplEnu[which(chSup)] else NULL }
    if(debug) {message(fxNa,"uE3") ; uE3 <- list(x=x,refSep=refSep,baseSep=baseSep,suplEnu=suplEnu,chBa=chBa,chRe=chRe)}
    if(length(suplEnu) >0) {
      ## construct 'complex' enumerator pattern (all combin of separator & enumerator)
      sep <- paste0(c( paste0(rep(baseSep, each=length(suplEnu)), suplEnu), paste0( suplEnu, rep(baseSep, each=length(suplEnu))),
        paste0(rep(baseSep, each=length(suplEnu)*length(baseSep)), rep(suplEnu,each=length(baseSep)), baseSep), suplEnu, baseSep ), "[[:digit:]]+$")
      chS <- sapply(sep, function(y) length(grep(y, x)))   # test which acually oocur
      chS <- if(stringentMatch) chS==length(x) else chS >0
      if(all(chS)) warning(fxNa,".. no pattern found, this should not happen here")
      if(debug) {message(fxNa,"uE3c") }
      sep <- sep[which(chS)]
    } else {  ## only basic enumerators occur, test which ones occur with digits
      baseSep <- paste0(baseSep,"[[:digit:]]+$")
      chBa <- sapply(baseSep, function(y) if(stringentMatch) length(grep(y, x)) == length(x) else length(grep(y, x)) >0)
      sep <- if(any(chBa)) baseSep[chBa] else NULL }
  } else sep <- NULL
  if(length(sep) >0) {
    if(debug) {message(fxNa,"uE4"); uE4 <- list(x=x,sep=sep,refSep=refSep,baseSep=baseSep,suplEnu=suplEnu,chBa=chBa,chRe=chRe)}
    for(i in sep) x <- sub(i, substr(i, 1, nchar(i) -13), x)
    x
  } else  {if(debug) message(fxNa,"uE4b"); x}    # no need to change, baseSep doesn't occur
}
   

#' Batch Reading Of Tabulated Text-Files
#'  
#' @description
#' This function allows batch reading of multiple tabulated text files n batch. 
#' The files can be designed specifically, or, alternatively all files from a given directory can be read.
#' If package \href{https://CRAN.R-project.org/package=data.table}{data.table} is available, faster reading of files will be performed using the function \code{\link[data.table]{fread}}.  
#' 
#' @details 
#' If you want to provide a flexible pattern of ffile-names, this has to be done before calling this usntion, eg using \code{grep} to provide an explicit collection of flles.
#' However, it is possible to read different files from different locations/directories, the length of \code{path} must match the length of \code{query} 
#' 
#' @param query (character) vector of file-names to be read, if \code{"."} all files will be read (no matter what their extension might be)
#' @param path (character) path for reading files, if \code{NULL} or \code{NA} the current directory will be used
#' @param dec (character, length=1) decimals to use, will be passed to \code{\link[data.table]{fread}} or \code{\link[utils]{read.delim}}
#' @param header (character, length=1) path for reading files, if \code{NULL} or \code{NA} the current directory will be used, will be passed to \code{\link[data.table]{fread}} or \code{\link[utils]{read.delim}}
#' @param strip.white (logical, length=1) Strips leading and trailing whitespaces of unquoted fields, will be passed to \code{\link[data.table]{fread}} or \code{\link[utils]{read.delim}}
#' @param blank.lines.skip (logical, length=1)  If \code{TRUE} blank lines in the input are ignored. will be passed to \code{\link[data.table]{fread}} or \code{\link[utils]{read.delim}}
#' @param fill (logical, length=1) If \code{TRUE} then in case the rows have unequal length, blank fields are implicitly filled, will be passed to \code{\link[data.table]{fread}} or \code{\link[utils]{read.delim}}
#' @param filtCol (integer, length=1) which columns should be used for filtering, if \code{NULL} or \code{NA} all data will be returned
#' @param filterAsInf (logical, length=1) filter as inferior or equal (\code{TRUE}) or superior or equal threshold \code{filtVal} 
#' @param filtVal (numeric, length=1) which numeric threshold should be used for filtering, if \code{NULL} or \code{NA} all data will be returned
#' @param silent (logical) suppress messages
#' @param callFrom (character) allow easier tracking of messages produced
#' @param debug (logical) display additional messages for debugging
#' @return This function returns a list of data.frames
#' @seealso  \code{\link[data.table]{fread}}, \code{\link[utils]{read.delim}}, for reading batch of csv files : \code{\link{readCsvBatch}}
#' @examples
#' path1 <- system.file("extdata", package="wrMisc")
#' fiNa <-  c("a1.txt","a2.txt")
#' allTxt <- readTabulatedBatch(fiNa, path1)
#' str(allTxt)
#' 
#' @export
readTabulatedBatch <- function(query, path=NULL, dec=".", header="auto", strip.white=FALSE, blank.lines.skip=TRUE, fill=FALSE, 
  filtCol=2, filterAsInf=TRUE, filtVal=5000, silent=FALSE, callFrom=NULL, debug=FALSE) {
  ## read tabulated data from local download 
  fxNa <- .composeCallName(callFrom, newNa="readTabulatedBatch")
  if(isTRUE(debug)) silent <- FALSE else { debug <- FALSE
    if(!isTRUE(silent)) silent <- FALSE }
  namesXY <- c(deparse(substitute(query)),deparse(substitute(path)))
  
  ## check path & file as valid entries
  msg <- "Invalid entry for 'query'"
  if(length(query) <1) {warning(fxNa,msg," Setting to '.'"); query <- "." } else {
    chNa <- is.na(query)
    if(all(chNa)) stop("All queries are NA !!") else if(any(chNa)) {
      if(!silent) message(fxNa,"Removing ",sum(chNa)," out of ",length(chNa)," queries at NA-status")
      query <- query[which(!is.na(chNa))]
      if(length(path) >1 && length(path)==length(query)) path <- path[which(!is.na(chNa))]
      }}
  ## check for packages  
  reqPa <- c("utils","data.table")
  chPa <- sapply(reqPa, requireNamespace, quietly=TRUE)
  if(all(!chPa)) { query <- NULL
    message(fxNa,"Neither package 'utils' nor 'data.table' available, returning NULL")
  } else {
    if(!chPa[1]) message(fxNa, "NOTE : package '",reqPa[1],"' not found, for please consider installing from CRAN")  
    if(!chPa[2]) { if(!silent) message(fxNa, "NOTE : package '",reqPa[2],"'  not found ! For faster reading of files please install first from CRAN") }
 }  
  if(debug) message(fxNa,"rTB1")

  if(length(query) >0) {
    ## check & remove entries without any characters
    chChar <- nchar(query) >0
    if(debug) {message(fxNa,"rt1")}
    if(all(!chChar)) stop(msg) else if(any(!chChar)) { query <- query[which(chChar)] 
      if(length(path) >1 && length(path)==length(query)) path <- path[which(chChar)]}
    ## check & remove entries with NA
    if(length(path) >0) {
      chNaPa <- is.na(path)
      if(any(chNaPa)) { path <- path[which(!chNaPa)]
        if(length(query) >1 && length(query)==length(path)) query <- query[which(!chNaPa)] }
  } }
  if(debug) message(fxNa,"rTB2")
  
  if(length(query) >0) {
    ## combine file & path, main testing if existing
    if(length(path) >1 && length(path) != length(query)) { path <- NULL
      if(!silent) message(fxNa,"Length of entries for 'path' not matching length of queries !  Ignoring 'path'")}
    if(identical(".", query)) {
      if(debug) message(fxNa,"Reading all files from 'dir' selected")
      chPath <- if(length(path) ==1) { if(is.na(path)) FALSE else dir.exists(path)} else FALSE
      query <- list.files(if(chPath) path=path else ".")
      if(debug) {message(fxNa,"rTB3")}
      fiPa <- if(chPath) file.path(path, query) else query 
    } else {
      if(debug) {message(fxNa,"rTB3b  Reading specific files selected")}  
      chPath <- if(length(path) ==1 || length(path) == length(query)) { if(any(is.na(path))) FALSE else all(dir.exists(path))} else FALSE
      fiPa <- if(chPath) file.path(path, query) else query
      chfiPa <- file.exists(fiPa)
      if(all(!chfiPa)) warning(msg,"Can't find ANY of the files !") else {
        if(any(!chfiPa)) {   
          if(!silent) message(fxNa,"Removing ",sum(!chfiPa)," out of ",length(fiPa), " non-existing queries/files")
          query <- query[which(chfiPa)]
          fiPa <- fiPa[which(chfiPa)]}
    } }
    if(debug) {message(fxNa,"rRB4"); rTB4 <- list(fiPa=fiPa,query=query,chPa=chPa,filtCol=filtCol,filtVal=filtVal,strip.white=strip.white,blank.lines.skip=blank.lines.skip,fill=fill,filterAsInf=filterAsInf)}
  }  
      
  ## main reading of files (& integrated filtering)
  if(length(query) >0) {
    if(debug) message(fxNa,"Ready to start reading ",length(fiPa)," files using ",if(chPa[2]) "data.table::fread()" else "utils::read.delim()")
    if(length(fiPa) >0) {
      if(length(filtCol) ==1) { if(is.na(filtCol)) filtCol <- NULL } else filtCol <- NULL
      if(length(filtVal) ==1) { if(is.na(filtVal) || !is.numeric(filtVal)) {
        filtVal <- NULL; if(debug) message(fxNa,"Invalid 'filtVal'")}} else filtVal <- NULL
      if(identical(header,"auto") && !chPa[2]) header <- FALSE   # option 'auto' only avail with  data.table
      if(!isFALSE(filterAsInf)) filterAsInf <- TRUE
      if(debug) {message(fxNa,"rTB5")}
      datLi <- if(requireNamespace("data.table")) {
        lapply(fiPa, function(x) { 
          y <- as.data.frame(data.table::fread(x, sep="\t", dec=dec, header=header,strip.white=strip.white, blank.lines.skip=blank.lines.skip,fill=fill))
          if(length(filtCol)==1) { if(ncol(y) < filtCol) filtCol <- NULL else if(!is.numeric(y[,filtCol])) filtCol <- NULL}
          if(length(filtVal)==1 && length(filtCol) ==1) y <- y[which( if(filterAsInf) y[,filtCol] <= filtVal else y[,filtCol] >= filtVal),]
          y })
      } else {
        lapply(fiPa, function(x) { 
          y <- utils::read.delim(file=x, dec=dec, header=header, strip.white=strip.white, blank.lines.skip=blank.lines.skip, fill=fill)
          if(length(filtCol)==1) { if(ncol(y) < filtCol) filtCol <- NULL else if(!is.numeric(y[,filtCol])) filtCol <- NULL}
          if(length(filtVal)==1 && length(filtCol) ==1) y <- y[which( if(filterAsInf) y[,filtCol] <= filtVal else y[,filtCol] >= filtVal),]
          y })    
      }
      names(datLi) <- query
      datLi  
    }
  }  
}  
    

#' Extract Last Two Numeric Parts From Character Vector
#'
#' This function extracts last 2 (integer) numeric parts between punctuations out of character vector 'x'.
#' Runs faster than \code{gregexpr} .
#' Note: won't work correctly with decimals or exponential signs !! (such characters will be considered as punctuation, ie as separator)
#' @param x main character input
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging 
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a (numeric) matrix with 2 columns (eg from initial concatenated coordinates) 
#' @seealso \code{gregexpr} from \code{\link[base]{grep}}
#' @examples
#' extractLast2numericParts(c("M01.1-4","M001/2.5","M_0001_03-16","zyx","012","a1.b2.3-7,2"))
#' @export
extractLast2numericParts <- function(x, silent=FALSE, debug=FALSE, callFrom=NULL) {
  fxNa <- .composeCallName(callFrom, newNa="extractLast2numericParts")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  
  z <- gsub("[[:alpha:]]|[[:blank:]]","", x)                                     # simplify: remove any text or blank characters
  aa <- strsplit(z, "[[:punct:]]", fixed=FALSE)
  chLe <- sapply(aa,length) >1
  if(any(!chLe)) {aa <- aa[which(chLe)]; if(!silent) message(fxNa,sum(!chLe)," elements not fitting -> discard")}
  out <- t(sapply(aa, function(y) {le <- length(y); as.numeric(c(y[le -(1:0)]))}))
  rownames(out) <- if(any(chLe)) x[which(chLe)] else x
  out}
    

#' Count number of NAs per row and group of columns
#' 
#' This functions allows easy counting the number of NAs per row in data organized in multiple sub-groups as columns.
#'  
#'	 
#' @param mat (matrix of data.frame) data to count the number of \code{NA}s
#' @param grp (character or factor) defining which columns should be grouped (considered as replicates)
#' @return matrix with number of \code{NA}s per group
#' @seealso \code{\link{rowGrpMeans}}, \code{\link{rowSds}}, \code{\link[base]{colSums}}
#' @examples
#' mat2 <- c(22.2, 22.5, 22.2, 22.2, 21.5, 22.0, 22.1, 21.7, 21.5, 22, 22.2, 22.7,
#'    NA, NA, NA, NA, NA, NA, NA, 21.2,   NA, NA, NA, NA,
#'    NA, 22.6, 23.2, 23.2,  22.4, 22.8, 22.8, NA,  23.3, 23.2, NA, 23.7,
#'    NA, 23.0, 23.1, 23.0,  23.2, 23.2, NA, 23.3,  NA, NA, 23.3, 23.8)
#' mat2 <- matrix(mat2, ncol=12, byrow=TRUE)
#' gr4 <- gl(3, 4, labels=LETTERS[1:3])
#' # overal number of NAs per row
#' rowSums(is.na(mat2)) 
#' # number of NAs per row and group
#' rowGrpNA(mat2, gr4)
#' @export
rowGrpNA <- function(mat, grp) {
  ## get number of NAs per line & group of replicates
  if(any(length(dim(mat)) !=2, dim(mat) < 1)) stop("Invalid argument 'mat'; must be matrix (or data.frame) with min 1 line and 1 column")
  if(length(grp) != ncol(mat)) stop("Length of 'grp' and number of columns of 'mat' do not match !")
  if(is.data.frame(mat)) mat <- as.matrix(mat)
  gr1 <- naOmit(unique(grp))
  nNA <- matrix(nrow=nrow(mat), ncol=length(gr1), dimnames=list(NULL, gr1))
  for(i in 1:length(gr1)) {
    nNA[,i] <- rowSums(is.na(matrix(mat[,which(grp==gr1[i])], nrow=nrow(mat)) )) }
  nNA }
  

#' Reorganize results of search for close (similar) values in matrix-view 
#'
#' \code{closeMatchMatrix} reorganizes/refines results from simple search of similar values of 2 sets of data by \code{\link[wrMisc]{findCloseMatch}} (as list for one-to many relations) to more human friendly/readable matrix.
#' This function returns results combining two sets of data which were initially compared (eg measured and threoretical values) as matrix-view using output of \code{\link[wrMisc]{findCloseMatch}} and both original datastes
#' Additional information (covariables, annotation, ...) may be included as optional columns for either 'predMatr' or 'measMatr'.
#' Note : It is important to run \code{\link[wrMisc]{findCloseMatch}} with \code{sortMatch=FALSE} !
#' Note : Results presented based on view of 'predMatr', so if multiple 'measMatr' are at within tolared distance, lines of 'measMatr' will be repeated;
#' Note : Distances  'disToMeas' and 'ppmToPred' are oriented : neg value if measured is lower than predicted (and pos values if higher than predicted);
#' Note : Returns \code{NULL} when nothing within given limits of comparison;  
#' @param closeMatch (list) output from \code{\link[wrMisc]{findCloseMatch}}, ie list with hits for each 'x' (1st argument) : named vectors of value & x index in name; run with 'sortMatch'=F
#' @param predMatr (vector or matrix) predicted values, the column 'colPred' indicates which column is used for matching from \code{\link[wrMisc]{findCloseMatch}}; if column 'id' present this column will be used as identifier for matching
#' @param measMatr (vector or matrix) measured values, the column 'colMeas' indicates which column is used for matching from \code{\link[wrMisc]{findCloseMatch}}; if column 'id' present this column will be used as identifier for matching
#' @param prefMatch (character, length=2) prefixes ('^x' and/or '^y') thay may have been added by \code{findCloseMatch} 
#' @param colPred (integer or text, length=1) column of 'predMatr' with main values of comparison
#' @param colMeas (integer or text, length=1) column of 'measMatr' with main measures of comparison
#' @param limitToBest (integer) column of 'measMatr' with main measures of comparison
#' @param asDataFrame (logical) convert results to data.frame if non-numeric matrix produced (may slightly slow down big results)
#' @param origNa (logical) will try to use original names of objects 'predMatr','measMatr', if they are not multi-column and not conflicting other output-names (otherwise 'predMatr','measMatr' will appear)
#' @param silent (logical) suppress messages
#' @param callFrom (character) allows easier tracking of message(s) produced
#' @param debug (logical) for bug-tracking: more/enhanced messages
#' @return results as matrix-view based on initial results from \code{\link[wrMisc]{findCloseMatch}}, including optional columns of suppelemental data for both sets of data for comparison. Returns \code{NULL} when nothing within limits
#' @seealso  \code{\link[wrMisc]{findCloseMatch}}, \code{\link[wrMisc]{checkSimValueInSer}}  
#' @examples
#' aA <- c(11:17); bB <- c(12.001,13.999); cC <- c(16.2,8,9,12.5,15.9,13.5,15.7,14.1,5)
#' (cloMa <- findCloseMatch(aA,cC,com="diff",lim=0.5,sor=FALSE))       
#' # all matches (of 2d arg) to/within limit for each of 1st arg ('x'); 'y' ..to 2nd arg = cC
#' (maAa <- closeMatchMatrix(cloMa,aA,cC,lim=TRUE))  #
#' (maAa <- closeMatchMatrix(cloMa,aA,cC,lim=FALSE,origN=TRUE))  #
#' (maAa <- closeMatchMatrix(cloMa,cbind(valA=81:87,aA),cbind(valC=91:99,cC),colM=2,
#'   colP=2,lim=FALSE))
#' (maAa <- closeMatchMatrix(cloMa,cbind(aA,valA=81:87),cC,lim=FALSE,deb=TRUE))  #
#' a2 <- aA; names(a2) <- letters[1:length(a2)];  c2 <- cC; names(c2) <- letters[10+1:length(c2)]
#' (cloM2 <- findCloseMatch(x=a2,y=c2,com="diff",lim=0.5,sor=FALSE)) 
#' (maA2 <- closeMatchMatrix(cloM2,predM=cbind(valA=81:87,a2),measM=cbind(valC=91:99,c2),
#'   colM=2,colP=2,lim=FALSE,asData=TRUE)) 
#' (maA2 <- closeMatchMatrix(cloM2,cbind(id=names(a2),valA=81:87,a2),cbind(id=names(c2),
#'   valC=91:99,c2),colM=3,colP=3,lim=FALSE,deb=FALSE)) 
#' @export
closeMatchMatrix <- function(closeMatch, predMatr, measMatr, prefMatch=c("^x","^y"), colPred=1, colMeas=1, limitToBest=TRUE, asDataFrame=FALSE, origNa=TRUE,silent=FALSE,callFrom=NULL,debug=FALSE){
  fxNa <- .composeCallName(callFrom,newNa="closeMatchMatrix")
  namesMXY <- c(deparse(substitute(closeMatch)), deparse(substitute(predMatr)), deparse(substitute(measMatr)))
  if(debug) silent <- FALSE
  if(debug) {message(fxNa,".. xxidentToMatr2a\n")}
  if(length(closeMatch) <1) message(fxNa," INPUT EMPTY, NOTHING TO DO !!")
  if(length(dim(predMatr)) <2) {
    predMatr <- cbind(id=if(is.null(names(predMatr))) 1:length(predMatr) else names(predMatr), predMatr)
    if(is.character(colPred)) colnames(predMatr)[2] <- colPred
    colPred <- 2
  } else if(!is.matrix(predMatr)) predMatr <- as.matrix(predMatr)
  if(is.character(colPred)) { chCol <- colnames(predMatr)==colPred
    if(sum(chCol) <1) { if(!silent) message(fxNa," PROBLEM : '",colPred,"' not found in ",namesMXY[2]," !!")
      colPred <- 2
    } else colPred <- which(chCol)}
  if("id" %in% colnames(predMatr)) {
    if(colnames(predMatr)[1] != "id") {tmp <- colnames(predMatr)=="id"
      predMatr <- predMatr[,c(which(tmp),which(!tmp))]} 
  } else { colPred <- colPred +1 
    predMatr <- cbind(id=if(is.null(rownames(predMatr))) 1:nrow(predMatr) else rownames(predMatr), predMatr)}  
  if(length(dim(measMatr)) <2) {
    measMatr <- cbind(id=if(is.null(names(measMatr))) 1:length(measMatr) else names(measMatr), measMatr)
    if(is.character(colMeas)) colnames(measMatr)[2] <- colMeas
    colMeas <- 2 
  } else if(!is.matrix(measMatr)) measMatr <- as.matrix(measMatr)
  if(is.character(colMeas)) colMeas <- which(colnames(measMatr)==colMeas)
  if("id" %in% colnames(measMatr)) {
    if(colnames(measMatr)[1] != "id") {tmp <- colnames(measMatr)=="id"
      measMatr <- measMatr[,c(which(tmp),which(!tmp))]} 
  } else { colMeas <- colMeas +1
    measMatr <- cbind(id=if(is.null(rownames(measMatr))) 1:nrow(measMatr) else rownames(measMatr), measMatr)  }
  txt <- c(" TROUBLE AHEAD :  ", "elements of 'closeMatch' have no name !!")
  chCloseM <- sub(prefMatch[1],"", names(closeMatch)) %in% predMatr[,"id"]
  
  if(all(!chCloseM, na.rm=TRUE)) stop("None of (list-) names of 'closeMatch' fits to 'predMatr'")
  if(any(!chCloseM, na.rm=TRUE) && !silent) message(fxNa,txt[1],sum(chCloseM)," out of ",length(chCloseM)," elements from 'closeMatch' not found in 'predMatr'")  
  chCloMaNa <- nchar(names(closeMatch))                                # check for empty names
  if(any(chCloMaNa <1, na.rm=TRUE)) message(fxNa,txt[1],sum(chCloMaNa <1)," out of ",length(closeMatch)," (list-)",txt[2])
  chCloMaN2 <- nchar(unlist(sapply(closeMatch, names))) 
  if(any(chCloMaN2 <1, na.rm=TRUE)) message(fxNa,txt[1],sum(chCloMaN2 <1)," out of ",sum(sapply(closeMatch,length))," indiv ",txt[2])   ##if(is.null(colnames(predMatr))) colnames(predMatr) <- "predMatr"
  nMatch <- sapply(closeMatch,length)
  if(debug) {message(fxNa,".. xxidentToMatr2c\n")}
  ## main joining of predMatr & measMatr
  measNa <- sub(prefMatch[2],"", unlist(lapply(closeMatch, names)))         # names of measured (y) to each 'y'
  if(length(grep("[[:alpha:]]", measNa)) <1) measNa <- as.integer(measNa)        
  prediNa <- sub(prefMatch[1],"", names(closeMatch))                       # ..'x', ie predMatr
  if(length(grep("[[:alpha:]]", prediNa)) <1) prediNa <- as.integer(prediNa)        
  txt <- c(fxNa," TROUBLE ahead, trouble matching names or NAs already in 'closeMatch'  (see ")  
  if(any(is.na(prediNa))) message(txt,"'x'/predicted) !!")
  if(any(is.na(measNa))) message(txt,"'y'/measured) !!")
  chP <- match(rep(prediNa, nMatch), predMatr[,1])
  partP <- if(length(chP)==1) matrix(predMatr[chP,], nrow=1, dimnames=list(rownames(predMatr)[chP], colnames(predMatr))) else predMatr[chP,]
  partM <- if(length(measNa)==1) matrix(measMatr[measNa,], nrow=1, dimnames=list(rownames(measMatr)[measNa], colnames(measMatr))) else measMatr[measNa,]
  if(origNa && length(grep(" = ",namesMXY[2:3])) >0) { origNa <- FALSE
    if(!silent) message(fxNa,"Reset argument 'origNa' to FALSE since names of 'predMatr' and/or 'measMatr' result of formula and would be too long")}
  if(origNa) {
    colnames(partP)[1] <- paste("id",namesMXY[2],sep=".")
    colnames(partM)[1] <- paste("id",namesMXY[3],sep=".")
    if(ncol(partP)==2) colnames(partP)[colPred] <- namesMXY[2] else {
      colnames(partP)[-1] <- paste(namesMXY[2],colnames(partP)[-1],sep=".")}
    if(ncol(partM)==2) colnames(partM)[colMeas] <- namesMXY[3] else {
      colnames(partM)[-1] <- paste(namesMXY[3],colnames(partM)[-1],sep=".")}    
  } else {
    dupNa <- duplicated(c(colnames(partP), colnames(partM)))
    if(any(dupNa)) {
      dupNa <- c(colnames(partP), colnames(partM))[dupNa]
      chNaP <- colnames(partP) %in% dupNa
      chNaM <- colnames(partM) %in% dupNa
      colnames(partP)[which(chNaP)] <- paste(colnames(partP)[which(chNaP)],"pred",sep=".")
      colnames(partM)[which(chNaM)] <- paste(colnames(partM)[which(chNaM)],"meas",sep=".") }
  }
  out <- cbind(partP, partM)
  colMeas <- colMeas + ncol(partP) 
  if(debug) {message(fxNa,".. xxidentToMatr2d\n")}   
  chOrd <- order(as.character(out[,colMeas]))             # grouping done by measured mass; tapply will always sort by this alphabetically (so instead of repeated re-sorting...)
  if(!identical(chOrd, 1:nrow(out))) out <- out[chOrd,]  
  ## add suppelemental info  (eg distance, if dist is closest, ...)
  if(nrow(out) <1) {if(!silent) message(fxNa," nothing left"); return(NULL)} else {
    di <- cbind(me=as.numeric(out[,colMeas]), pr=if(length(colPred) >0) as.numeric(out[,colPred]))
    out <- cbind(out, disToPred=di[,2] -di[,1], ppmToPred=XYToDiffPpm(di[,2], di[,1], nSign=5, callFrom=fxNa), nByGrp=NA, isMin=NA, nBest=NA)    # note : ppm as normalized difference 1e6*(meas - ref)/ref
    if(debug) {message(fxNa,".. xxidentToMatr2e\n")}
    ## inspect groups for multiple values
    ch1 <- table(out[,colPred])
    out[,ncol(out) -2] <- rep(ch1, ch1)                                   # nByGrp
    tmp <- tapply(as.numeric(out[,"disToPred"]), out[,colPred], function(x) {if(length(unique(x))==1) rep(1,length(x)) else 0+(abs(x)==min(abs(x)))} )
    out[,"isMin"] <- unlist(tmp)
    tmp <- tapply(as.integer(out[,"isMin"]), out[,colPred], sum)            # prepare nBest
    out[,ncol(out)] <- rep(tmp, ch1)          # nBest
    ch3 <- out[,ncol(out) -1] =="0"
    if(any(ch3)) out[which(ch3), ncol(out)] <- 0          # reset lines which are not best dist as nBest=0
    chMax <- out[,ncol(out) -1] >0
    if(limitToBest && any(!chMax)) out <- out[which(chMax),]                                                            # filter & add col for nBest      
    if(debug) { message(fxNa,".. xxidentToMatr2f \n")}
    ## re-sort 
    if(nrow(out) >1) {chOrd <- as.integer(order(out[,1]))    #out[,"id.predMatr"]
      out <- out[chOrd,]}   
    if(is.character(out) & asDataFrame) out <- convMatr2df(out)[,-1]
    out }}
   

#' Find Group-Names In Pairwise Combined And Isolate Separator   
#'
#' @description
#' This function allows mapping which group-names are occuring either as head or as tail in pairwise combined terms (by checking from tails).
#' 
#' 
#' @details 
#' This function aims identifying the separator used wo create pairwise comparison labels by looking for given group-names at head and tail of combined labels.
#' If labeles occur inside oyther ones (eg 'AB' in 'ABC') it will always privilidge the longest one.
#' After stripping off all group-names the remaining charachers are considered as the separator used.
#' The function will also check if all pairwise labels returned the same separtator and in case of multiple separators found the most frequent may be reytained.
#' 
#'  
#'	 
#' @param grpNa (character) the names of the groups of replicates (ie conditions) used to test
#' @param pairwNa (character) the names of pairwise-testing (ie 'concatenated' \code{sampNa}
#' @param reportAs (character) default \code{'reportAs=list'} to return list, otherwise a character vector with separator(s) found will be returned 
#' @param sortGrp (logical) by default groups will be sorted (as levels of a factor appear sorted)
#' @param rmAmbig (logical) remove all ambiguous 
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a list containing $sep a character vector of the separator(s) identified 
#' @seealso (for running multiple pair-wise test) \code{\link{moderTestXgrp}}, used underneith \code{\link[base]{grep}}
#' @examples
#' findHeadAndTail(LETTERS[4:2], c("D-X","D-C"))
#' 
#' grp1 <- c("C","ACC","CC","B.B","B.BA","B","CA")
#' pwNa1 <- "B.B-CC" 
#' findHeadAndTail(grp1, pwNa1)
#' 
#' pwNa2 <- c("B.B-CC", "CA-ACC")
#' findHeadAndTail(grp1, pwNa2)
#' @export
findHeadAndTail <- function(grpNa, pairwNa, reportAs="list", sortGrp=TRUE, rmAmbig=TRUE, silent=FALSE, debug=FALSE, callFrom=NULL) {   # trimming strategy !
  ## find which grpNa are occuring either as head or as tail in pairwNa  (by checking from tails)
  ## will find longest of grpNa as head or tail of pairwNa; 
  ## possible artifact : head & tail identified may be overlapping (will give warning)
  ## returns vector with index of grpNa (and grpNa as name); or NULL if not both head and tail identified
  ## NAs will be removed (changes length of grpNa )
  fxNa <- .composeCallName(callFrom, newNa="findHeadAndTail")
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  if(!isTRUE(silent)) silent <- FALSE

  if(any(is.na(pairwNa))) pairwNa <- wrMisc::naOmit(pairwNa)
  if(all(is.na(grpNa))) grpNa <- NULL
  datOK <- length(pairwNa) >0 && length(grpNa) >0
  if(datOK) { 
    if(isTRUE(sortGrp)) grpNa <- sort(grpNa)
    chDup <- duplicated(grpNa)
    if(any(chDup)) grpNa <- grpNa[which(!chDup)]
    grpNaIni <- grpNa    
    grpNaIni2 <- grpNa <- grpNa[order(nchar(grpNa), decreasing=TRUE)]
    grpNaIni3 <- grpNa <- wrMisc::protectSpecChar(grpNa)
    sep <- NULL    
    ## need to sort decreasing by length (no of characters) since some grpNa may be shorter version of others    
    ## main grep from ends
    indL <- paste0("^", grpNa)
    chL <- sapply(indL, grepl, pairwNa)
    indR <- paste0(grpNa, "$")
    chR <- sapply(indR, grepl, pairwNa)
    names(chL) <- names(chR) <- grpNaIni2
    if(debug) {message("fHT1"); fHT1 <- list(grpNa=grpNa,pairwNa=pairwNa,chL=chL,chR=chR,grpNaIni=grpNaIni,grpNaIni2=grpNaIni2,grpNaIni3=grpNaIni3)}
    if(any(chL) && any(chR)) {
      if(length(pairwNa)==1) {
        out <- c(which(chL)[1], which(chR)[1])         
        if(any(c("list","index","all") %in% reportAs)) {    
          outNa <- names(out)
          outI <- match(names(out), grpNaIni)
          names(outI) <- outNa
          if(sum(nchar(outNa)) > nchar(pairwNa)) { warning(fxNa,"Overlapping instance reported", if(!isTRUE(rmAmbig)) " : Reported results may be likely incorrect")
            if(isTRUE(rmAmbig)) {out[2] <- outI[2] <- sep <- NA}  } 
          if(debug) message("fHT1b"); fHT1b <- list(out=out, grpNa=grpNa,pairwNa=pairwNa,chL=chL,chR=chR,grpNaIni=grpNaIni,grpNaIni2=grpNaIni2,grpNaIni3=grpNaIni3)
          if("list" %in% reportAs && !all(is.na(out))) { if(debug) message(fxNa,"Start substr ",nchar(names(out[1])) +1,"  end ", nchar(pairwNa) - nchar(names(out[2])))
             sep <- substr(pairwNa, nchar(names(out[1])) +1, nchar(pairwNa) - nchar(names(out[2])))          
          }
          if(debug) {message("fHT1c"); fHT1c <- list(out=out,outI=outI,sep=sep,grpNa=grpNa,pairwNa=pairwNa,chL=chL,chR=chR,grpNaIni=grpNaIni,grpNaIni2=grpNaIni2,grpNaIni3=grpNaIni3)}
        } else {
          out <- names(out)
          if(sum(nchar(out)) > nchar(pairwNa)) warning(fxNa,"Overlapping instance reported")
        }                                          
      } else {
        ## pairwNa is longer                             
        out <- cbind(head=apply(chL, 1, function(x) which(x)[1]), tail=apply(chR, 1, function(x) which(x)[1]))  # index to grpNaIni2 and NOT to grpNaIni
        if(debug) {message("fHT2"); fHT2 <- list(out=out,grpNa=grpNa,pairwNa=pairwNa,chL=chL,chR=chR,grpNaIni=grpNaIni,grpNaIni2=grpNaIni2,grpNaIni3=grpNaIni3)}
        rownames(out) <- pairwNa
        ## main localization for longer pairwNa (as char)
        out <- matrix(grpNaIni2[match(grpNa[out], grpNaIni3)], ncol=2, dimnames=dimnames(out)) ## correct
        ## supl checks
        chLe <- rowSums(nchar(out))
        chLe1 <- chLe - nchar(pairwNa) 
        chLe2 <- chLe >= nchar(pairwNa)
        if(debug) { message("fHT3"); fHT3 <- list(out=out,chLe=chLe,chLe1=chLe1,chLe2=chLe2)}
        if(any(chLe2, na.rm=TRUE)) { warning(fxNa,sum(chLe2)," overlapping instances reported")
          if(isTRUE(rmAmbig)) { out[which(chLe2),] <- NA 
            pairwNa[which(chLe2)] <- NA
            chLe[which(chLe2)] <- NA
        } }
        ## isolate sep
        if(any(c("li","list","all") %in% reportAs)) {
          chSep <- nchar(pairwNa) -chLe 
          if(length(unique(wrMisc::naOmit(chSep))) >1) warning(fxNa,"It looks like finding non-unique separator")         
          sep <- unique(substr(pairwNa, nchar(out[,1]) +1, nchar(out[,1]) +0 +unique(chSep)))
          if(length(sep) >1) {          
            sep <- wrMisc::naOmit(unique(sub("^ +","", sub(" +$","", sep))) ) # trim heading or tailing spaces
          }
        } 
        ## located index
        if(any(c("ind","index","li","list","all") %in% reportAs)) {
          outI <- matrix(match(out, grpNaIni), ncol=2, dimnames=dimnames(out))
          if(debug) {message("fHT4"); fHT4 <- list(out=out,outI=outI,sep=sep,grpNa=grpNa,pairwNa=pairwNa,chL=chL,chR=chR,grpNaIni=grpNaIni,grpNaIni2=grpNaIni2,grpNaIni3=grpNaIni3)}
        } 
      }
      if("list" %in% reportAs) list(pwGrpNa=out, sep=sep, pwIndex=outI) else {if("index" %in% reportAs) outI else out } 
    } else NULL
  }
}
  

#' Eliminate close (overlapping) points (in x & y space)
#'
#' \code{elimCloseCoord} reduces number of rows in 'dat' by eliminating lines where x & y coordinates (columns of matrix '\code{dat}' defined by '\code{useCol}') are identical (overlay points) or very close.
#' The stringency for 'close' values may be fine-tuned using \code{nDig}), this function uses internally \code{\link{firstOfRepeated}}.
#'
#' @param dat matrix (or data.frame) with main numeric input
#' @param useCol (numeric) index for numeric columns of 'dat' to use/consider
#' @param elimIdentOnly (logical) if TRUE, eliminate real duplicated points only (ie identical values only)
#' @param refine (numeric) allows increasing stringency even further  (higher 'refine' .. more lines considered equal)
#' @param nDig (integer) number of significant digits used for rounding, if two 'similar' values are identical after this rounding the second will be eliminated. 
#' @param silent (logical) suppress messages
#' @param callFrom (character) allows easier tracking of message(s) produced
#' @return resultant matrix/data.frame
#' @seealso \code{\link{findCloseMatch}}, \code{\link{firstOfRepeated}}
#' @examples
#' da1 <- matrix(c(rep(0:4,5),0.01,1.1,2.04,3.07,4.5),nc=2); da1[,1] <- da1[,1]*99; head(da1)
#' elimCloseCoord(da1)
#' @export
elimCloseCoord <- function(dat,useCol=1:2,elimIdentOnly=FALSE,refine=2,nDig=3,callFrom=NULL,silent=FALSE){
  fxNa <- .composeCallName(callFrom,newNa="elimCloseCoord")
  argN <- deparse(substitute(x))
  refine <- 1/refine
  if(is.null(rownames(dat))) rownames(dat) <- 1:nrow(dat)
  firstOfRep <- firstOfRepeated(paste(dat[,useCol[1]],dat[,useCol[2]],sep="_"))$indUniq
  if(!elimIdentOnly) {
    if(nDig <3) message(fxNa," the stringency chosen (of ",nDig," significant digits) is very high, you may consider using 'nDig' values higher than 2 !")
    dat1 <- as.matrix(dat[,useCol])
    if(is.character(dat1)) dat1 <- matrix(as.numeric(dat1),ncol=2)                   
    nDig <- lapply(as.data.frame(dat1), function(x) signif(max(x,na.rm=TRUE),nDig) )      # numb of digits after decimal for max
    nDig <- sapply(nDig,function(x) nchar(x)-nchar(round(x))-1)
    nDig[nDig <0] <- 0
    firstOfRep <- firstOfRepeated(paste(round(refine*dat1[,1],nDig[1]),
      round(refine*dat1[,2],nDig[2]),sep="_"))$indUniq
    if(!silent) message(fxNa," reducing '",argN,"' from ",nrow(dat)," to ",length(firstOfRep)," lines")
    dat <- dat[firstOfRep,] }
  dat }
   

#' Simple Package Download Statistics From CRAN
#'
#' This function allows accessing the most recent counts of package downloads availabale on http://www.datasciencemeta.com/rpackages,
#' obtaining rank quantiles and to compare (multiple) given packages to the bulk data, optionally a plot can be drawn.
#'
#' @details
#' Detailed articles on this subject have been published on R-Hub (https://blog.r-hub.io/2020/05/11/packagerank-intro/) and on
#' R-bloggers (https://www.r-bloggers.com/2020/10/a-cran-downloads-experiment/).
#' The task of checking the number of downloads for a given package has also been addressed by several other packages (eg dlstats, cranlogs, adjustedcranlogs).
#'
#' This function only allows accessing counts as listed on the website of \code{www.datasciencemeta.com} which get updated dayly.
#' Please note, that reading all lines from the website may take a few seconds !!
#' To get a better understanding of the counts read, reference quantiles for download-counts get added by default  (see argument \code{refQuant}).
#' The (optional) figure can be drawn in linear scale (default, with minor zoom to lower number of counts) or in log (necessary for proper display of the entire range of counts), by setting the argument \code{log="y"}.
#'
#' The number of downloads counted by RStudio may not be a perfect measure for the actual usage/popularity of a given package,
#' the articles cited above discuss this in more detail.
#' For example, multiple downloads from the same IP or subsequent downloads of multiple (older) versions of the same package seem to get counted, too.
#'
#' @param queryPackages (character or integer) package names of interest, if \code{integer}, n random packages will be picked by random
#' @param countUrl (character) the url where the dayly counts ara available
#' @param refQuant (numeric) add reference quantile values to output matrix
#' @param options (character) additional seetings : use 'naOmit' to remove NA-lines from output (package-names not found in 'countUrl'); 
#'   'sort' for sorting output by number of downloads
#' @param figure (logical) decide of figure should be printed
#' @param log (character) set count-axis of figure to linear or log-scale (by setting \code{log="y"})
#' @param silent (logical) suppress messages
#' @param callFrom (character) allow easier tracking of messages produced
#' @param debug (logical) additional messages for debugging
#' @return This function retuns a matrix with download counts (or \code{NULL} if the web-site can't be accessed or the query-packages are not found there)
#' @seealso packages \href{https://CRAN.R-project.org/package=cranlogs}{cranlogs} and \href{https://CRAN.R-project.org/package=packageRank}{packageRank}
#' @examples
#' ## Let's try a microscopic test-file (NOT representative for true up-to-date counts !!)
#' pack1 <- c("cif", "bcv", "FinCovRegularization", "wrMisc", "wrProteo")
#' testFi <- file.path(system.file("extdata", package="wrMisc"), "rpackagesMicro.html")
#' packageDownloadStat(pack1, countUrl=testFi, log="y", figure=FALSE)
#' ## For real online counting simply use the argument countUrl in default setting
#'
#' @export
packageDownloadStat <- function(queryPackages=c("wrMisc","wrProteo","cif","bcv","FinCovRegularization"), countUrl="http://www.datasciencemeta.com/rpackages",
  refQuant=(1:10)/10, options=c("naOmit", "sort"), figure=TRUE, log="", silent=FALSE, callFrom=NULL, debug=FALSE) {
  ## get rank & downloads for all 10% tiles as well as queryPackages
  ## return matrix with 1st line as rank and 2nd as n.dowloads, if inclQuant=TRUE 12th col - end for queryPackages
  fxNa <- .composeCallName(callFrom, newNa="packageDownloadStat")
  if(isTRUE(debug)) silent <- FALSE else { debug <- FALSE
    if(!isTRUE(silent)) silent <- FALSE }
  inclQuant <- length(refQuant) >0
  chPa <- length(naOmit(queryPackages)) >0 && all(is.character(queryPackages) | is.numeric(queryPackages))
  datOK <- FALSE
  txt <- paNa <- paRa <- nDownl <- NULL                    # intialize (just in case)  
  if(chPa) {
    if(length(countUrl) <1) countUrl <- "http://www.datasciencemeta.com/rpackages"
    txt <- try(readLines(countUrl, warn=FALSE), silent=TRUE)
    if(inherits(txt, "try-error") || length(txt) <2) { Sys.sleep(30)
      message(fxNa,"First attempt reading data failed, try again after 30 sec ..")
      txt <- try(readLines(countUrl, warn=FALSE), silent=TRUE)
      if(inherits(txt, "try-error") || length(txt) <2) { txt <- try(readLines(countUrl, warn=FALSE, n=9000), silent=TRUE)
        if(!inherits(txt, "try-error")) {
          message(fxNa,"Failed to read entire data-set, but succeeded to read first 9000 lines. Your package(s) of interest may not be included in this list") 
        } 
      }
    }
    datOK <- !inherits(txt, "try-error") && length(txt) >1
     
    ## start exploiting
    if(datOK) {
      ## check argument refQuant
      if(any(length(refQuant) < 1, identical(refQuant,NA), isFALSE(refQuant))) {refQuant <- 1; inclQuant <- FALSE}        # minimal setting
      if(inclQuant && length(txt) <900) { inclQuant <- FALSE
        if(!silent) message(fxNa,"Too few data for adding reference centile values/packages")
      }
      if(!is.numeric(refQuant)) refQuant <- (1:10)/10
      chNa <- is.na(refQuant)
      if(any(chNa)) { if(all(chNa)) { refQuant <- (1:10)/10
        } else {
          refQuant <- refQuant[which(!chNa)]
          if(!silent) message(fxNa,"Removing ",sum(refQuant)," invalid reference quantile entries (must be 0 > x >= 1)") }
      }
      if(debug) {message("pds1"); pds1 <- list(txt=txt,queryPackages=queryPackages,chPa=chPa,chNa=chNa)}      
      ## start parsing page
      iniIn <- grep("<tbody>", txt)[1] +1             # position 54 +1
      endIn <- grep("</tbody>", txt)[1] -1            # position  143044 -1
      datOK <- length(iniIn)==1 && length(endIn)==1
      if(datOK) datOK <- (endIn -iniIn+1) %% 7 == 0 && all(grepl("<td>[[:digit:]]+</td>", txt[iniIn +1 +c(0,7)])) && 
        all(grepl("<td>[0-9,]+</td>", txt[iniIn +3 +c(0,7)]))   # check if series of 7 and if 1st & 2nd line line contain rank as 1st, counts as 3rd
    } else datOK <- FALSE    
    
    if(datOK) {
      tab <- matrix(txt[iniIn:endIn], byrow=TRUE, ncol=7)[,c(2:4)]
      paNa <- sub("[[:print:]]+https://cran.r-project.org/web/packages/", "", sub("/index.html.+", "", tab[,2]))
      nDownl <- try(as.integer(gsub(",", "", sub(" +<td>", "", sub("</td> *", "", tab[,3])))), silent=TRUE)
      paRa <- try(as.integer(sub(" +<td>", "", sub("</td> *", "", tab[,1]))), silent=TRUE)
      if(any(sapply(list(nDownl,paRa), inherits, "try-error"), length(paNa) !=length(nDownl))) datOK <- FALSE
    } else if(!silent) message(fxNa,"Unable to read data (format of data not recognized or maybe the server is not responding)")
  } else if(!silent) message(fxNa,"Invalid 'queryPackages'")
  
  if(any(c(length(txt), length(paNa), length(paRa)) <1)) datOK <- FALSE
  if(datOK) {
    ## assemble; verify 'queryPackages'
    ta2 <- data.frame(name=paNa, rank=paRa, downloads=nDownl)
    ta2[order(ta2$rank),]              # order (just in case)
    if(debug) {message("pds2"); pds2 <- list(txt=txt,ta2=ta2,tab=tab,queryPackages=queryPackages,chPa=chPa,chNa=chNa,iniIn=iniIn,endIn=endIn, paNa=paNa, nDownl=nDownl,paRa=paRa,datOK=datOK)}

    if(is.numeric(queryPackages)) queryPackages <- paNa[sample(1:nrow(tab), as.integer(abs(queryPackages[1])), replace=FALSE)]   # random pick if numeric
    ## search query
    chPa <- match(queryPackages, ta2$name)
    if(any(is.na(chPa))) { if(all(is.na(chPa))) { warning(fxNa,"NONE of queryPackages found; returning NULL"); datOK <- FALSE
      } else {
        queryPackages <- naOmit(queryPackages)
        ta2 <- data.frame(name=paNa, rank=paRa, downloads=nDownl)  # update
        ta2[order(ta2$rank),]   # update order (just in case)
    } }
    if(debug) {message("pds3"); pds3 <- list()}
  } else if(!silent) message(fxNa,"Unable to understand information from '",countUrl,"'")  

  if(datOK) {
    ## add centile (& plot graph)
    maxPa <- max(ta2$rank, na.rm=TRUE)
    out <- ta2[chPa,]
    out$centile <- round(100*out$rank/maxPa,1)
    if(debug) {message("pds4"); pds4 <- list(out=out,txt=txt,tab=tab,ta2=ta2,queryPackages=queryPackages,maxPa=maxPa,chPa=chPa)}

    if(isTRUE(figure)){
      if(!identical(log,"y")) log <- ""
      yMa <- min(ta2$downloads[round(nrow(ta2)*0.05)], ta2$downloads[1]*0.15)
      suppressWarnings(graphics::plot(downloads ~rank, ta2, las=2,
        ylim=if(identical(log,"y")) c(100,nDownl[1]) else c(1,yMa), type="l", cex.axis=0.8,
        xaxs="i", yaxs="i", xlab="Rank",ylab="Number of Downloads", main="CRAN Packages Downloads", log=log))
      graphics::abline(v=pretty(c(rep(100,5), maxPa)), col="grey75", lty=2)
      graphics::abline(h=if(identical(log,"y")) 10^(1:7) else 1:5*20000, col="grey75", lty=2)
      if(FALSE) graphics::points(ta2$rank[chPa], ta2$downloads[chPa], col=2)

      arrBo <- if(identical(log, "y")) 1.04 *ta2$downloads[chPa] else yMa/150  +1.03* ta2$downloads[chPa] 
      arrUp <- if(identical(log, "y")) 2.2* ta2$downloads[chPa]  else arrBo +yMa/18 
      graphics::arrows(x0=ta2$rank[chPa], y0=arrUp, x1=ta2$rank[chPa], y1=arrBo, col=2, length=0.11, lwd=1.8)   # ok
      graphics::text(ta2$rank[chPa], arrUp*1.03, labels=ta2$name[chPa], col=2,las=1,cex=1,srt=90,adj=0)

      date1 <- sub(" .+","",sub(".+Last updated: ","",txt[grep("Last updated",txt)]))
      if(grepl("https://",countUrl)) graphics::mtext(paste("as of ",date1," "), cex=0.9, adj=1)
    }
    if(debug) {message("pds5"); pds5 <- list(out=out,txt=txt,tab=tab,ta2=ta2,queryPackages=queryPackages,chPa=chPa,chNa=chNa,iniIn=iniIn,endIn=endIn, paNa=paNa, nDownl=nDownl,paRa=paRa,datOK=datOK,refQuant=refQuant,inclQuant=inclQuant,maxPa=maxPa)}

    if(inclQuant) {
      ## add quantile (reference-)values
      if(nrow(ta2) <1000) warning(fxNa,"Too few data ! (",nrow(ta2)," lines)  Estimation of reference centile may be very imprecise  ...")
      refQuan2 <- round(maxPa * refQuant)
      refInd <- round(nrow(ta2) *refQuant)
      supl <- ta2$downloads[refInd]
      names(supl) <- paste0("cent_",100*round(refQuant,3))
      out <- rbind(out, data.frame(name=ta2$name[refInd], rank=ta2$rank[refInd], downloads=supl, centile=100*round(refQuant,3)))
      if("sort" %in% options) out <- out[order(out$rank, decreasing=FALSE),] 
    }
    if(length(out) >0 && "naOmit" %in% options) {
      chNa <- grep("^NA(\\.[[:digit:]]+){0,1}", rownames(out))
      if(length(chNa) < nrow(out)) { if(length(chNa) >0) out <- if(length(chNa) +1 < nrow(out)) out[-1*chNa,] else matrix(out[-1*chNa,], 
        nrow=1, dimnames=list(rownames(out)[-1*chNa], colnames(out)))    # remove lines with NA
        if(!silent) message(fxNa,"Removed lines for ",length(chNa)," packages not found") } }
    out                                                                                           
  }  
}
       

#' Filter three-dimensional array of numeric data
#'
#' Filtering of matrix or (3-dim) array \code{x} : filter column according to \code{filtCrit} (eg 'inf') and threshold \code{filtVal}
#'
#' and extract/display all col matching 'displCrit'.
#'
#' @param x array (3-dim) of numeric data
#' @param filtVal (numeric, length=1) for testing inferior/superor/equal condition
#' @param filtTy  (character, length=1) which type of testing to perform (may be 'eq','inf','infeq','sup','supeq', '>', '<', '>=', '<=', '==')
#' @param filtCrit (character, length=1) which column-name consider when filtering filter with 'filtVal' and 'filtTy'
#' @param displCrit (character) column-name(s) to display
#' @param silent (logical) suppress messages
#' @param callFrom (character) allow easier tracking of messages produced
#' @param debug (logical) additional messages for debugging
#' @seealso \code{\link{filterList}}; \code{\link{filterLiColDeList}};
#' @return This function returns a list of filtered matrixes (by 3rd dim)
#' @examples
#' arr1 <- array(11:34, dim=c(4,3,2), dimnames=list(c(LETTERS[1:4]),
#'   paste("col",1:3,sep=""), c("ch1","ch2")))
#' filt3dimArr(arr1,displCrit=c("col1","col2"),filtCrit="col2",filtVal=7)
#' @export
filt3dimArr <- function(x, filtVal, filtTy=">", filtCrit=NULL, displCrit=NULL, silent=FALSE, debug=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom, newNa="filt3dimArr")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  if(length(dim(x)) != 3) stop("Expecting 3-dim array as input for 'x'")
  if(debug) message(fxNa,"f3d1")
  if(is.character(displCrit) && length(displCrit) >0) { chCrit <- match(displCrit, colnames(x))
    if(any(is.na(chCrit))) stop("Can't find (",sum(is.na(chCrit)),") 'displCrit' in colnames of 'x'") else displCrit <- chCrit}
  if(is.character(filtCrit) && length(filtCrit) >0) { chCrit <- match(filtCrit, colnames(x))
    if(any(is.na(chCrit))) stop("Can't find (",sum(is.na(chCrit)),") 'filtCrit' in colnames of 'x'") else filtCrit <- chCrit}
  if(length(filtCrit) <1) filtCrit <- 1:ncol(x)
  if(length(displCrit) <1) displCrit <- 1:ncol(x)
  if(length(displCrit) <1) displCrit <- colnames(x)
  if(length(filtTy) <1) filtTy <- ">" else filtTy <- filtTy[1]
  if(!(filtTy %in% c("inf","infeq","sup","supeq","eq",">",">=","<","<=","=="))) {
    stop("cannot identify type of filter specified") }
  ##
  out <- list()
  out[[1]] <- x[.filterSw(x[,filtCrit,1], fiTy=filtTy, checkVa=filtVal, indexRet=TRUE), displCrit,1]  # start with 1st di
  if(dim(x)[3] >1) for(i in 2:(dim(x)[3])) out[[i]] <- x[.filterSw(x[,filtCrit,i], fiTy=filtTy, checkVa=filtVal, indexRet=TRUE), displCrit, i]
  out }


#' Filter 3-dim array of numeric data (main)
#'
#' Filtering of matrix or array \code{x} (may be 3-dim array) according to \code{fiTy} and \code{checkVa}
#'
#'
#' @param x array (3-dim) of numeric data
#' @param fiTy  (character) which type of testing to perform ('eq','inf','infeq','sup','supeq', '>', '<', '>=', '<=', '==')
#' @param checkVa (logical) s
#' @param indexRet (logical) if \code{TRUE} (default) rather return index numbers than filtered values
#' @seealso  \code{\link{filt3dimArr}}; \code{\link{filterList}}; \code{\link{filterLiColDeList}};
#' @return This function returns either index (position within 'x') or concrete (filtered) result
#' @examples
#' arr1 <- array(11:34, dim=c(4,3,2), dimnames=list(c(LETTERS[1:4]),
#'   paste("col",1:3,sep=""),c("ch1","ch2")))
#' filt3dimArr(arr1,displCrit=c("col1","col2"),filtCrit="col2",filtVal=7)
#' .filterSw(arr1, fiTy="inf", checkVa=7)
#' @export
.filterSw <- function(x, fiTy, checkVa, indexRet=TRUE){
  ## filterswitch function : filter values of 'x' to satisfy 'check' (vector of FALSE or TRUE)
  ## 'indexRet' .. if TRUE rather return index numbers than filtered values
  ## returns either index (position within 'x') or concrete (filtered) result
  if(fiTy == "==") fiTy <- "eq"
  if(fiTy == "<") fiTy <- "inf"
  if(fiTy == "<=") fiTy <- "infeq"
  if(fiTy == ">") fiTy <- "sup"
  if(fiTy == ">=") fiTy <- "supeq"
  if(!(fiTy %in% c("inf","infeq","sup","supeq","eq",">",">=","<","<=","=="))) {
    stop("cannot identify type of filter specified") }
  if(indexRet)  {
    switch(fiTy,
      eq = which(x == checkVa),
      sup = which(x > checkVa),
      supeq = which(x >= checkVa),
      inf = which(x < checkVa),
      infeq = which(x <= checkVa) )
  } else {
    switch(fiTy,
      eq = x[which(x == checkVa)],
      sup = x[which(x > checkVa)],
      supeq = x[which(x >= checkVa)],
      inf = x[which(x < checkVa)],
      infeq = x[which(x <= checkVa)] )
  } }
  

#' Partial distance matrix (focus on closest)
#'
#' \code{partialDist} calculates distance matrix like \code{dist} for 1- or 2-dim data, but only partially, ie only cases of small distances.
#' This function was made for treating very large data-sets where only very close distances to a given point need to be found,
#' it allows to overcome memory-problems with larger data (and faster execution with > 50 rows of 'dat').
#' 
#' @param dat (matrix of numeric values) main input
#' @param groups (factor) to split using \code{cut} or specific custom grouping (length of dat)
#' @param overLap (logical) if TRUE make groups overlapping by 1 value (ie maintain some context-information)
#' @param method 'character' name of method passed to \code{dist}
#' @param silent (logical) suppress messages
#' @param debug (logical) display additional messages for debugging
#' @param callFrom (character) allow easier tracking of message(s) produced
#' @return This function returns a matrix with partial distances (not of class 'dist' object)
#' @seealso \code{\link[stats]{dist}}
#' @examples
#' set.seed(2016); mat3 <- matrix(runif(300),nr=30)
#' round(dist(mat3), 1)
#' round(partialDist(mat3, gr=3), 1)
#' @export
partialDist <- function(dat, groups, overLap=TRUE, method="euclidean", silent=FALSE, debug=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom,newNa="partialDist")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  dim1 <- length(dim(dat)) <2
  msg <- "names of 'dat' must be unique !"
  if(dim1) {
    if(is.null(names(dat))) names(dat) <- 1:length(dat)
    if(length(unique(names(dat))) < length(dat)) stop(msg)
    dis <- matrix(nrow=length(dat),ncol=length(dat))       # prepare for output
  } else {
    if(is.null(rownames(dat))) rownames(dat) <- paste("li",1:nrow(dat),sep="")
    if(length(unique(rownames(dat))) < nrow(dat)) stop(" row",msg)
    dis <- matrix(nrow=nrow(dat),ncol=nrow(dat))           # prepare for output
  }
  if(debug) {message(fxNa," length(groups) ",length(groups), "   pD1")}
  if(length(groups)==1 && is.numeric(groups)) {  if(dim1) {
      dat <- sort(dat)
      daCu <- cut(dat,groups)
    } else {                                   #  (2dim) sorting too complicated ?
      tmp <- apply(dat, 2, function(x) as.numeric(cut(x, groups)))
      daCu <- factor(paste(tmp[,1],tmp[,2],sep="_"))
    }
  } else daCu <- groups
  ## check for orphan-groups & fuse 
  if(any(table(daCu) <2) && !overLap) message(fxNa,sum(table(daCu) <2)," orphan groups (n=1) in grouping ! (overLap=TRUE suggested)")
  dimnames(dis) <- if(dim1) list(names(dat), names(dat)) else list(rownames(dat),rownames(dat)) 
  datL <- list()
  if(debug) {message(fxNa," pD2")}
  if(overLap) {
    for(i in 1:length(unique(daCu))) {
      tmp <- which(daCu==levels(daCu)[i])
      tmp <- sort(c(tmp, range(tmp) +c(-1,1)))
      if(tmp[1] <1) tmp <- tmp[-1]
      if(max(tmp) > nrow(dis)) tmp <- tmp[which(tmp <= nrow(dis))]
      datL[[i]] <- if(dim1) dat[tmp] else dat[tmp,] }
   ## could check if some groups are now all redundant -> adjust datL & useNo
  } else {
    if(dim1) datL <- split(dat, daCu) else {
      useSep <- "_"                   # make sure this sep doesn't appear in rownames ! !!
      tmp <- apply(dat, 1, paste, collapse=useSep)
      names(tmp) <- rownames(dat)
      datL <- split(tmp, daCu)
      datL <- lapply(datL, function(x) {
        y <- matrix(as.numeric(unlist(strsplit(x,useSep))), ncol=ncol(dat))
        rownames(y) <- names(x); y})
  } }
  if(debug) {message(fxNa," pD3")}
  for(i in 1:length(datL)) {
    useLi <- if(dim1) match(names(datL[[i]]), names(dat)) else match(rownames(datL[[i]]), rownames(dat))
    dis[useLi,useLi] <- as.matrix(stats::dist(datL[[i]], method=method)) }
  dis }

#' Raise all values close to lowest value
#'
#' This function aims to raise all values close to lowest value to end up as at value of 'raiseTo'.
#' This is done independently for each col of mat.
#' This function sets all data to common raiseTo (which is min among all cols)
#' @param mat (matrix of numeric values) main input
#' @param raiseTo (numeric) 
#' @param minFa (numeric) minimum factor
#' @param silent (logical) suppress messages
#' @param debug (logical) display additional messages for debugging
#' @param callFrom (character) allow easier tracking of message(s) produced
#' @return This function returns a numeric vector with  numer of non-numeric characters (ie not '.' or 0-9))
#' @seealso \code{\link[base]{nchar}}
#' @examples
#' aa <- 11:15
#' @export
.raiseColLowest <- function(mat, raiseTo=NULL, minFa=0.1, silent=FALSE, debug=FALSE, callFrom=NULL){
  ## independently for each col of mat : raise all values close to lowest value to end up as at value of 'raiseTo'
  ## select all values within range of 'minFa' to determined min (eg 0.1 select until min+10%ofMin)
  ## this version sets all data to common raiseTo (which is min among all cols)
  fxNa <- .composeCallName(callFrom, newNa=".raiseColLowest")
  colMin <- apply(as.matrix(mat), 2, min, na.rm=TRUE)
  if(is.null(raiseTo)) {
    raiseTo <- apply(mat, 2, function(x) {
      y <- sort(unique(signif(x,3)))
      y[which(y > min(y, na.rm=TRUE) +0.1* abs(min(y, na.rm=TRUE))) [1]] })
    raiseTo <- min(raiseTo, na.rm=TRUE)
    if(!silent) message(fxNa," 'raiseTo' was set to ",raiseTo)}
  if(length(raiseTo) >1) raiseTo <- min(raiseTo, na.rm=TRUE)
  raiseF <- matrix(rep(raiseTo -colMin, each=nrow(mat)), nrow=nrow(mat))
  raiseF[mat > raiseTo + minFa*abs(raiseTo)] <- 0
  out <- mat + raiseF
  out }

#' Find overlap instances among range of values in lines
#'
#' This function aims to find overlap instances among range of values in lines of 'x' (typically give just min & max)
#' 
#' 
#' @param x (matrix of numeric values or all-numeric data.frame) main input
#' @param rmRedund (logical) report overlaps only in 1st instance (will show up twice otherwise)
#' @param callFrom (character) allow easier tracking of message(s) produced
#' @return This function returns a matrix with line for each overlap found, cols 'refLi' (line no), 'targLi' (line no), 'targCol' (col no)
#' @seealso \code{\link[base]{nchar}}
#' @examples
#' aa <- 11:15
#' @export
.findBorderOverlaps <- function(x, rmRedund=FALSE, callFrom=NULL){
  ## find overlap instances among range of values in lines of 'x' (typically give just min & max)
  ## 'x' .. matrix (or all-numeric data.frame), inspect by lines for potential overlap
  ## 'rmRedund' .. report overlaps only in 1st instance (will show up twice otherwise)
  ## return matrix with line for each overlap found, cols 'refLi' (line no), 'targLi' (line no), 'targCol' (col no)
  fxNa <- .composeCallName(callFrom, newNa=".findBorderOverlaps")
  if(any(is.na(x))) {
    message(fxNa," NAs detected, remove all lines with NAs only")
    x <- x[which(rowSums(!is.na(x)) ==ncol(x)),]
    if(any(is.na(x))) { NaLi <- which(rowSums(is.na(x)) >0)
      x[NaLi,] <- apply(x[NaLi], 1, min)}}
  tmp <- matrix(NA, nrow=nrow(x), ncol=ncol(x)*(nrow(x)-1))
  for(i in 1:nrow(x)) {             # check each line of x against everything else
    te <- as.numeric(t(x[-i,]))
    tmp[i,] <- max(x[i,]) > te && te > min(x[i,])
  }
  curLi <- 1
  out <- matrix(nrow=sum(tmp), ncol=3, dimnames=list(NULL,c("refLi","targLi","targCol")))
  for(i in which(rowSums(tmp) >0)) {             # extract corresponding line & col info
    ti <- which(tmp[i,])
    if(length(ti) >0)  {
      to <- cbind(ref=rep(i,length(ti)), li=(1:nrow(x))[-i][ti %/% ncol(x)+ ti %% ncol(x)], col=ncol(x)- ti %% ncol(x))
      if(rmRedund && curLi >1) {to <- to[which(!(to[,2] %in% out[,1] & to[,1] %in% out[,2])),]
        if(length(to) >0) {
          if(!is.matrix(to)) to <- matrix(to, ncol=3)
          out[curLi:(curLi +nrow(to)-1),] <- to }
      } else out[curLi:(curLi +nrow(to)-1),] <- to
      curLi <- curLi +length(ti) }
  }
  out <- out[which(rowSums(is.na(out)) <2),]
  out  }
   

#' Filter nodes & edges for extracting networks
#'  
#' This function allows extracting and filtering network-data based on fixed threshold (\code{limInt}) and add sandwich-nodes (nodes inter-connecting initial nodes) out of node-based queries.
#' 
#' 
#' @param lst (list, composed of multiple matrix or data.frames ) main input (each list-element should have same number of columns)
#' @param filtCol (integer, length=1) which column of \code{lst} should be usd to filter using thresholds \code{limInt} and \code{sandwLim}
#' @param limInt (numeric, length=1) filter main edge-scores according to \code{filterAsInf}
#' @param sandwLim (numeric, length=1) filter sandwich connection edge-scores accodring to \code{filterAsInf} 
#' @param filterAsInf (logical) filter as 'inferior or equal' or 'superior or equal'
#' @param outFormat (character) may be 'matrix' for tabular output, 'all' as list with matrix and list of node-names
#' @param remOrphans (logical) remove networks consisting only of 2 connected edges
#' @param remRevPairs (logical) remove duplicate edges due to reverse massping (eg A - B and B - A); NOTE : use only when edges don't have orientation !   
#' @param elemNa (character) used only for messages
#' @param silent (logical) suppress messages
#' @param callFrom (character) allow easier tracking of message(s) produced
#' @param debug (logical) display additional messages for debugging
#' @return This function returns a matrix or data.frame
#' @seealso  in \code{\link[base]{cbind}}
#' @examples
#' 
#' lst2 <- list('121'=data.frame(ID=as.character(c(141,221,228,229,449)),11:15), 
#' 	 '131'=data.frame(ID=as.character(c(228,331,332,333,339)),11:15), 
#'   '141'=data.frame(ID=as.character(c(121,151,229,339,441,442,449)),c(11:17)), 
#'   '151'=data.frame(ID=as.character(c(449,141,551,552)),11:14),
#'   '161'=data.frame(ID=as.character(171),11), '171'=data.frame(ID=as.character(161),11),
#'   '181'=data.frame(ID=as.character(881:882),11:12) )
#'
#' lst2 <- list('121'=data.frame(ID=as.character(c(141,221,228,229,449)),11:15, 21:25), 
#' 	 '131'=data.frame(ID=as.character(c(228,331,332,333,339)),11:15, 21:25), 
#'   '141'=data.frame(ID=as.character(c(121,151,229,339,441,442,449)), c(11:17), 21:27), 
#'   '151'=data.frame(ID=as.character(c(449,141,551,552)), 11:14, 21:24),
#'   '161'=data.frame(ID=as.character(171), 11,21), '171'=data.frame(ID=as.character(161), 11,21),
#'   '181'=data.frame(ID=as.character(881:882), 11:12,21:22) )
#' 
#' (te1 <- filterNetw(lst2, limInt=90, remOrphans=FALSE))
#' (te2 <- filterNetw(lst2, limInt=90, remOrphans=TRUE))
#' 
#' (te3 <- filterNetw(lst2, limInt=13, remOrphans=FALSE))
#' (te4 <- filterNetw(lst2, limInt=13, remOrphans=TRUE))
#' 
#' 
#' @export
filterNetw <- function(lst, filtCol=3, limInt=5000, sandwLim=5000, filterAsInf=TRUE, outFormat="matrix", remOrphans=TRUE, remRevPairs=TRUE, elemNa="genes", silent=FALSE, callFrom=NULL, debug=FALSE) {
  ##
  fxNa <- .composeCallName(callFrom, newNa="filterNetw")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  if(any(sapply(c("mat","matr","matrix","all"), identical, outFormat))) {asMatrix <- TRUE} else {asMatrix <- FALSE}
  if(length(names(lst)) <1) stop("Invalid format of 'lst' : has NO names")

  ## check input list for duplicate queries
  lstNa <- names(lst)
  chDu <- duplicated(lstNa, fromLast=FALSE)
  if(any(chDu)) { lst <- lst[-1*which(chDu)]
    if(!silent) message(fxNa,"Entry has ",sum(chDu)," duplicate entry-names, removing duplicates")
    lstNa <- names(lst) }                       # update
  useID <- list(query=lstNa)
  if(debug) {message(fxNa,"fiNe1 passed ini check")}
  ## check format
  ch2dim <- sapply(lapply(lst, dim), length) ==2
  ## remove any non-matrix or data.frame
  if(any(!ch2dim)) { 
    if(all(!ch2dim)) {lst <- list(); chDim <- NULL; warning(fxNa,"Invalid input, should be list of matrixes or data.frames !!")
    } else { lst <- lst(which(ch2dim))
      chDim <- lapply(lst, dim)            # update
      if(!silent) message(fxNa,"Note: ",sum(!ch2dim)," out of ",length(ch2dim)," list-elements are not matrix or data.frame, removing")
   } } else chDim <- lapply(lst, dim)
  colNa <- colnames(lst[[1]])    
  if(debug) {message(fxNa,"fiNe2,  passed remove any non-matrix or data.frame")}
  
  ## check if add'l column for quality/intensity data available
  chQ <- sapply(lst, function(x) ncol(x) >1)
  ## check filtCol argument  (assume format of lst as checked)
  msg <- "Invalid entry for 'filtCol': should be integer (of length=1) to designate which column to use or column-name"
  if(length(filtCol) <1) {if(ncol(lst[[1]]) >1) {filtCol <- 2; message(fxNa,msg,"; setting to 2")} else stop(msg," -too short")}
  if(length(filtCol) >1) {filtCol <- filtCol[1]; message(fxNa,msg," -use 1st")}
  if(!is.numeric(filtCol)) {filtCol <- which(colnames(lst[[1]])==filtCol)
     if(length(filtCol) <1) {filtCol <- 2; message(fxNa, msg,"; setting to 2")}   
  } else { if(filtCol > ncol(lst[[1]]) | filtCol <2) {
    filtCol <- 2; message(fxNa, msg,"; setting to 2")} }

  ## check input format if filtering can be applied (reset 'limInt' to NULL if can't be used) 
  if(length(limInt)==1 && is.numeric(limInt) && !any(is.na(limInt))) {           # valid threshod for filtering
    chDimQ <- all(sapply(chDim, function(x) x[2]) >1) 
    if(!chDimQ) { limInt <- sandwLim <- NULL
      if(!silent) message(fxNa,"Data have no second column with numeric data for filtering, ignoring 'limInt' and 'sandwLim'")     
  } } else limInt <- NULL 
  if(debug) message(fxNa,"fiNe3,  check input format if filtering can be applied")
  ## check if reverse-(direct)mapping has same score - only if .filterNetw is not launched (.filterNetw does this check, too)
  if(remRevPairs && !asMatrix && all(chQ)) {    # run only if no conversion to matrix, otherwise do this check after lrbind()
    sepCha <- "__"
    chRe <- cbind(pri=rep(names(lst), sapply(lst,nrow)), sec=lrbind(lst) )
    chR2 <- paste(chRe[,2], chRe[,1], sep=sepCha) %in% paste(chRe[,1], chRe[,2], sep=sepCha) 
    if(any(chR2)) { if(!all(chR2)) chRe <- chRe[which(chR2),]     # data with any reverse mapping only
      tmp <- apply(as.matrix(chRe[,1:2]), 1, function(x) paste(sort(x), sep=sepCha))
      tmp <- paste(tmp[1,], tmp[2,], sep=sepCha)
      if(debug) {message(fxNa,"fiNe4,  opional check if reverse-(direct)mapping has same score")}
      tmp2 <- by(as.matrix(chRe), tmp, function(x) length(unique(x[,3])) ==1)
      if(any(!tmp2)) warning(fxNa,"Reverse value for ",sum(!tmp2)," pairs incoherent !! (",
        pasteC(sub(sepCha,"& ", names(tmp2)[utils::head(which(!tmp2))])),")")
  } }
  ## filter all for thresh limInt, subseq filter sandwLim, establish list of all connected nodes
  if(all(chQ) && length(limInt)==1 && is.numeric(limInt) && !any(is.na(limInt))) {           # valid threshod for filtering
    lst <- lapply(lst, function(x) x[which(if(!isFALSE(filterAsInf)) x[,filtCol] <= limInt else x[,filtCol] >= limInt),])
    chDi <- sapply(lst, function(x) length(dim(x)) <2)
    if(any(chDi)) lst[which(chDi)] <- lapply(lst[which(chDi)], function(x) matrix(x, nrow=1, dimnames=list(NULL,colNa)))  # reset to matrix-type if lost at filtering  
    chN <- sapply(lst, length)
    if(all(chN <1)) warning(fxNa,"NOTHING remaining after filtering for 'limInt'=",limInt," !!")
    if(debug) {message(fxNa,"fiNe5,  filter all for thresh limInt")}
    ##  now check for possible sandwLim filter
    if(any(sapply(c("def","default","auto"), identical, sandwLim))) sandwLim <- limInt
    if(length(sandwLim)!=1 | !is.numeric(sandwLim) | any(is.na(sandwLim))) sandwLim <- NULL    
    if(debug) { message(fxNa,"fiNe6,  check for possible sandwLim filter")}
  } else sandwLim <- NULL
  ## extract 2nd nodes, apply sandwLim (if applicable) 
  lst <- lapply(lst, function(x) if(length(sandwLim) <1 || ncol(x) <2) x[which(x[,1] %in% useID$query),] else x[which(x[,1] %in% useID$query | 
      if(!isFALSE(filterAsInf)) x[,filtCol] <= sandwLim else x[,filtCol] >= sandwLim ),])
  chDi <- sapply(lst, function(x) length(x) >0 && length(dim(x)) <2)    # in case of list of matrixes, some may become simple vectors due to indexing
  if(any(chDi)) lst[which(chDi)] <- lapply(lst[which(chDi)], function(x) matrix(x, nrow=1, dimnames=list(NULL,colNa)))  # reset to matrix-type if lost at filtering  
      
  all2nd <- unlist(lapply(lst, function(x) x[,1]), use.names=FALSE)  # extr 2nd node IDs (after add'l filtering)       
  if(debug) {message(fxNa,"fiNe7,  extracting 2nd nodes, apply sandwLim")}
  ## remove empty (due to filtering)
  chLe <- sapply(lst, length) >0
  if(any(!chLe)) { lst <- lst[which(chLe)]
    if(all(!chLe)) { if(!silent) message(fxNa,"NOTHING remaining after filtering !!")
    } else if(!silent) message(fxNa,"Removed ",sum(!chLe)," out of ",length(chLe)," queries without data passing filtering")
    lstNa <- names(lst)                     # update
    useID <- list(query=lstNa)              # update    
  }  

  ## look for Sandwich nodes/genes  
  useID$sandwID <- NULL
  ## a sandwich node/gene has too aoocur at min 2x in query-results (and may not be part of init query)
  chDup <- duplicated(all2nd, fromLast=FALSE) & !(all2nd %in% lstNa)    # needed to be defined as sandwich
  useID$sandwID <- if(any(chDup)) unique(all2nd[which(chDup)]) else NULL  
  if(debug) {message(fxNa,"fiNe8,  sandwich nodes/genes")}; fiNe8 <- list(lst=lst,all2nd=all2nd,useID=useID,chDup=chDup,chLe=chLe,filtCol=filtCol, limInt=limInt,chQ=chQ)

  ## filter to keep connected network only (and reduce all to IDs & filtCol)
  useID2 <- if(length(limInt) ==1) unique(unlist(useID)) else useID$query
  lst <- lapply(lst, function(x) {y <- which(x[,1] %in% useID2); if(length(y) >0) {if(length(y) >1) x[y,c(1,filtCol)] else { 
    z <- data.frame(x[y,1], x[y,filtCol]); colnames(z) <- colnames(x)[c(1,filtCol)]; z} }})  
  chLe <- sapply(lst, length)
  if(any(chLe <1)) { lst <- lst[which(chLe >0)]
    if(!silent) message(fxNa,"",sum(chLe <1)," element(s) had no data remaining after filtering ..." )  
  }
  if(debug) {message(fxNa,"fiNe9,  filter to keep connected network only")}

  if(length(lst) <1) message(fxNa,"NOTE: NOTHING remaining after filtering for connected networks ") else {
    if(!isFALSE(asMatrix)) {
      ## convert to matrix, remove duplicates
      lst <- .filterNetw(lst, remOrphans=remOrphans, reverseCheck=remRevPairs, filtCol=filtCol, callFrom=fxNa, silent=silent, debug=debug)
      isSandw <- lst[,2] %in% useID$sandwID
      lst <- cbind(lst, toSandw=if(is.data.frame(lst)) isSandw else as.numeric(isSandw))
      if(!isFALSE(remRevPairs)) {
        te1x <- apply(as.matrix(lst[,c(1,filtCol)]), 1, sort)      ## need to sort le/ri !!
        lst <- lst[which(!duplicated(paste(te1x[1,], te1x[2,]), fromLast=FALSE)),]      # corrected for duplicate pairs
      }
    } else {
      ## prepare output as list
      lst <- lapply(lst, function(x) {ch1 <- x[,1] %in% unlist(useID)
        if(sum(ch1) >0) { if(sum(ch1) >0) x[which(ch1),] else matrix(x[which(ch1),], nrow=1, dimnames=list(NULL, colnames(x)))} else NULL })
      if(!silent) message(fxNa,"Network of ",sum(useID$query %in% lstNa)," (init) ",elemNa," plus ",
        sum(useID$sandwID %in% lstNa)," sandw-",elemNa )              #  
    }
  }
  if(any(sapply(c("all"), identical, outFormat))) lst <- list(useID=useID, netw=lst) 
  lst }

 
#' Filter nodes & edges for extracting networks (main)
#'  
#' This function allows extracting and filtering network-data based on fixed threshold (\code{limInt}) and add sandwich-nodes (nodes inter-connecting initial nodes) out of node-based queries.
#' 
#' 
#' @param lst (list, composed of multiple matrix or data.frames ) main input (each list-element should have same number of columns)
#' @param remOrphans (logical) remove networks consisting only of 2 connected edges
#' @param reverseCheck (logical) 
#' @param filtCol (integer, length=1) which column of \code{lst} should be usd to filter using thresholds \code{limInt} and \code{sandwLim}
#' @param silent (logical) suppress messages
#' @param callFrom (character) allow easier tracking of message(s) produced
#' @param debug (logical) display additional messages for debugging
#' @return This function returns a matrix or data.frame
#' @seealso \code{\link{filterNetw}} and other CRAN package dedeicated to networks
#' @examples
#' ab <- 1:10  
#' @export  
.filterNetw <- function(lst, remOrphans=TRUE, reverseCheck=TRUE, filtCol=2, callFrom=NULL, silent=FALSE, debug=FALSE) {
  ##
  fxNa <- .composeCallName(callFrom, newNa=".filterNetw")
  sepCha <- "__"           # characters used in paste when fusing 2 nodes
  lstLe <- sapply(lst, nrow)
  names(lstLe) <- names(lst)
  ## transform list to matrix/data.frame
  lst <- lrbind(lst, silent=TRUE)
  if(any(dim(lst) <1) && length(dim(lst)) <2) stop("Invalid input format ! Bizzare, lrbind() did NOT manage to produce a matrix or data.frame")
  colnames(lst)[1] <- "Node2"
  if(ncol(lst) >1) colnames(lst)[2] <- "edgeScore" 
  
  lst <- data.frame(Node1=rep(names(lstLe), lstLe), lst)
  if(debug) {message(fxNa,".fiNe.1 transform list to matrix/data.frame")}
  ## check for (reverse-)pairs
  chR2 <- paste(lst[,filtCol], lst[,1], sep=sepCha) %in% paste(lst[,1], lst[,filtCol], sep=sepCha) 
  if(any(chR2)) { 
    chRe <- if(!all(chR2)) lst[which(chR2),] else lst      # data with any reverse mapping only
    tmp <- apply(as.matrix(chRe[,1:2]), 1, function(x) paste(sort(x), sep=sepCha))     #  pairs of sorted nodes
    tmp <- paste(tmp[1,], tmp[2,], sep=sepCha)
    names(tmp) <- rownames(chRe)
    ## check for consistent score with reverse pairs
    if(reverseCheck && ncol(lst) >2) {
      tmp2 <- by(as.matrix(chRe), tmp, function(x) length(unique(x[,3])) ==1)
      if(any(!tmp2)) warning(fxNa,"Reverse value for ",sum(!tmp2)," pairs incoherent !! (",
        pasteC(sub(sepCha,"& ", names(tmp2)[utils::head(which(!tmp2))])),")")
    }
    if(debug) {message(fxNa,".fiNe.2 check for consistent score with reverse pairs")}
    ## remove duplicates (ie remove 2nd occurance due to reverse pairs)
    dupP <- duplicated(tmp, fromLast=FALSE)  
    if(any(dupP)) {lst <- lst[-1*as.integer(names(tmp)[which(dupP)]),]
      if(!silent) message(fxNa,"Removing ",sum(dupP)," (reverse) redundant mappings")}
  }
  if(debug) {message(fxNa,".fiNe.3 remove duplicates")} 
  ## 
  lstN <- c(as.character(lst[,1]), as.character(lst[,2]))
  if(remOrphans) {
    ## need number of connections to decide on low-connected
    chNo <- duplicated(lstN, fromLast=TRUE)
    if(any(chNo)) { 
      chNo2 <- chNo | duplicated(lstN, fromLast=FALSE)
      if(debug) {message(fxNa,".fiNe.4 number of connections to decide on low-connected")}
      if(!all(chNo2)) { hiNo <- unique(lstN[which(chNo2)] )
        ## which pairs of connections to keep (any node connected to a hiNo ?)
        chLi <- rowSums(matrix(lstN %in% hiNo, ncol=2)) >0
        if(all(!chLi) & !silent) warning(fxNa,"NONE of the nodes has any further connections, argument 'remOrphans' removes everything")
        lst <- lst[which(chLi),]
        if(debug) {message(fxNa,".fiNe.5 which pairs of connections to keep")}
        } }
  }      
  ##
  rownames(lst) <- 1:nrow(lst)
  lstN <- unique(c(as.character(lst[,1]), as.character(lst[,2])))      
  lst
} 
     

#' Exclude extreme values (based on distance to mean)
#'
#' This function aims to identify extreme values (values most distant to mean, thus potential outlyers), mark them as NA or directely exclude them (depending on '\code{showNAs}').
#' Note that every set of non-identical values will have at least one most extreme value. Extreme values are part of many distributions, they are not necessarily true outliers. 
#'
#' @param dat numeric vector, main input
#' @param result (character) may be 'val' for returning data without extreme values or 'pos' for returning position/index of extreme values   
#' @param CVlim (NULL or numeric) allows to retain extreme values only if a certain CV (for all 'dat') is exceeded (to avoid calling extreme values form homogenous data-sets)
#' @param maxExcl (integer) max number of elments to explude
#' @param showNA (logical) will display extrelme values as NA
#' @param goodValues (logical) allows to display rather the good values instead of the extreme values
#' @param silent (logical) suppress messages
#' @param callFrom (character) allow easier tracking of message(s) produced
#' @return numeric vector wo extremle values or index-position of extreme values 
#' @seealso \code{\link{firstOfRepLines}},  \code{\link{get1stOfRepeatedByCol}} for treatment of matrix
#' @examples
#' x <- c(rnorm(30),-6,20)
#' exclExtrValues(x)
#' @export  
exclExtrValues <- function(dat,result="val",CVlim=NULL,maxExcl=1,showNA=FALSE,goodValues=TRUE,silent=FALSE,callFrom=NULL) {
   ## return position of extreme value (potential outlyer)
   ## results may be given as position of extreme values ('pos') or as values ('val') without the extremes identified
  fxNa <- .composeCallName(callFrom,newNa="exclExtrValues")
  extrVal <- which(abs(dat-sum(dat,na.rm=TRUE)/sum(!is.na(dat))) == max(abs(dat-mean(dat,na.rm=TRUE))))
  if(length(extrVal) == length(dat) & !silent) message(" all values are equidistant, not possible to distinguish one/some as 'extreme'")
  if(length(extrVal) > maxExcl & !silent) message(" more extreme values are equidistant than available for display with 'maxExcl' = ",maxExcl)
  if(!is.null(CVlim)) if(!is.finite(as.numeric(CVlim))) {
    warning(" 'CVlim' should be numeric of length 1, ignoring current value of 'CVlim'"); CVlim <- NULL}
  if(!is.null(CVlim)) {
    if(stats::sd(as.numeric(dat),na.rm=TRUE)/mean(as.numeric(dat),na.rm=TRUE) < as.numeric(CVlim)[1]) extrVal <- NA }
  extrVal <- naOmit(extrVal)[1:maxExcl]
  if(tolower(result) %in% c("val","value")) {
    if(showNA) {out <- dat; out[extrVal] <- NA } else out <- dat[-1*extrVal]}
  if(tolower(result) %in% c("pos","position")) out <- if(goodValues) (1:length(dat))[-1*extrVal]  else extrVal
  out }
   

#' Add text before file-extension
#'
#' This function helps changing charater srings like file-names and allows adding the character vector 'add'
#' (length 1) before the extension (defined by last '.') of the input string 'x'. 
#' Used for easily creating variants/additional filenames but keeping current extension.
#' @param x main character vector
#' @param add character vector to be added
#' @param sep (character) separator between 'x' & 'add' (character, length 1)
#' @param silent (logical) suppress messages
#' @param callFrom (character) allow easier tracking of messages produced
#' @param debug (logical) additional messages for debugging
#' @return modified character vector
#' @examples
#' addBeforFileExtension(c("abd.txt","ghg.ijij.txt","kjh"),"new")
#' @export
addBeforFileExtension <- function(x, add, sep="_", silent=FALSE, callFrom=NULL, debug=FALSE) {
  fxNa <- .composeCallName(callFrom, newNa="addBeforFileExtension")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  if(length(add) >1) add <- add[1]
  if(length(grep("\\.",x)) >0) {
    extLoc <- sapply(gregexpr("\\.", x),function(y) y[length(y)])
    if(any(extLoc <0)) extLoc[extLoc <0] <- nchar(x[extLoc <0]) +1
    paste0(substr(x,1,extLoc-1), sep, add, substr(x,extLoc,nchar(x)))
  } else paste(x, add, sep=sep) }


#' Check File Name Extensions 
#' Function for checking file-names.
#' @param fileNa (character) file name to be checked
#' @param ext (character) file extension
#' @return modified character vector
#' @examples
#' .checkFileNameExtensions("testFile.txt","txt")
#' @export
.checkFileNameExtensions <- function(fileNa, ext){
  msg <- " need at least 1 character-string as 'fileNa' and as 'ext'"
  if(any(length(fileNa) <1,length(ext) <1)) stop(msg)
  ext <- sub("^\\.","",ext)           # remove starting point-separator
  ext <- unique(paste0(".",ext))
  che <- nchar(fileNa) > nchar(sub(paste0(ext[1],"$"),"",fileNa)) 
  if(length(ext) >1) for(i in 2:length(ext)) {
    che <- che || nchar(fileNa) > nchar(sub(paste0(ext[i],"$"),"",fileNa))}
  if(sum(!che) >0)fileNa[!che] <- paste0(fileNa[!che],ext[1])
  fileNa }
   

#' Search Points Forming Lines At Given Slope
#'
#' \code{searchLinesAtGivenSlope} searchs among set of points (2-dim) those forming line(s) with user-defined slope ('coeff'),
#'  ie search optimal (slope-) offset parameter(s) for (regression) line(s) with given slope ('coef').
#'  Note: larger data-sets : segment residuals to 'coeff' & select most homogenous
#' 
#' Note: The package MASS is required when using as \code{lmCompare=TRUE}.
#' For larger data the function will try using the package \code{NbClust} (available from CRAN) if installed.
#' 
#' @param dat matrix or data.frame, main input
#' @param coeff (numeric) slope to consider
#' @param filtExtr (integer) lower & upper quantile values, remove points with extreme deviation to offset=0, (if single value: everything up to or after will be used)
#' @param minMaxDistThr (logical) optional minumum and maximum distance threshold
#' @param lmCompare (logical) add'l fitting of linear regression to best results, return offset AND slope based on lm fit
#' @param indexPoints (logical) return results as list with element 'index' specifying retained points
#' @param displHist (logical) display histogram of residues
#' @param displScat (logical) display (simple) scatter plot
#' @param bestCluByDistRat (logical) initial selection of decent clusters based on ratio overallDist/averNeighbDist (or by CV & cor)
#' @param neighbDiLim (numeric) additional threshold for (trimmed mean) neighbour-distance
#' @param silent (logical) suppress messages
#' @param debug (logical) for bug-tracking: more/enhanced messages
#' @param callFrom (character) allow easier tracking of messages produced
#' @seealso \code{\link[stats]{lm}}
#' @return This functions returns a matrix of line-characteristics  (or if indexPoints is \code{TRUE} then list (line-characteristics & index & lm-results)
#' @examples
#' set.seed(2016); ra1 <- runif(300)
#'  dat1 <- cbind(x=round(c(1:100+ra1[1:100]/5,4*ra1[1:50]),1),
#'   y=round(c(1:100+ra1[101:200]/5, 4*ra1[101:150]), 1))
#' (li1 <- searchLinesAtGivenSlope(dat1, coeff=1))
#' @export
searchLinesAtGivenSlope <- function(dat, coeff=1.5, filtExtr=c(0,1), minMaxDistThr=NULL, lmCompare=TRUE, indexPoints=TRUE,
  displHist=FALSE, displScat=FALSE, bestCluByDistRat=TRUE, neighbDiLim=NULL, silent=FALSE, debug=FALSE, callFrom=NULL) {
  fxNa <- .composeCallName(callFrom, newNa="searchLinesAtGivenSlope")
  opar <- list(mfrow=graphics::par("mfrow"), mfcol=graphics::par("mfcol"))
  on.exit(graphics::par(opar$mfrow))
  on.exit(graphics::par(opar$mfcol))
  if(isTRUE(debug)) silent <- FALSE else { debug <- FALSE
    if(!isTRUE(silent)) silent <- FALSE }
  if(isTRUE(lmCompare)) { 
    if(!requireNamespace("MASS", quietly=TRUE)) warning(fxNa,"Package 'MASS' not found ! Please install first from CRAN \n   setting 'lmCompare' to FALSE")
    lmCompare <- FALSE } else lmCompare <- FALSE 
  minNPoints <- 8                            ## min no of points to be included in primary selection of groups/clusters
  cvThr <- c(0.06,0.09)                      ## threshold for 1st flitering of clustering results
  argNa <- deparse(substitute(dat))

  coeff <- convToNum(coeff, spaceRemove=TRUE, convert=c(NA,"sparseChar"), remove=NULL, euroStyle=TRUE, sciIncl=TRUE, callFrom=fxNa)
  if(length(coeff) <1) stop(" 'coeff' should be numeric of length 1") else coeff <- coeff[1]
  offS <- offS0 <- dat[,2] - coeff*dat[,1]
  if(identical(filtExtr, c(0,1))) filtExtr <- NULL
  if(is.numeric(filtExtr)) {                 ## remove extreme points (needed for all appoaches ??)
    if(length(filtExtr) !=2) filtExtr <- sort(c(filtExtr, if(filtExtr[1] <0.5) 1 else 0)[1:2])
    offSL <- stats::quantile(offS, filtExtr, na.rm=TRUE)           #
    filt1 <- which(offS > offSL[1] & offS < offSL[2])
  } else filt1 <- 1:nrow(dat)                # filt1 : index for which lines of 'dat' are actually condsidered ..
  ## exclude lines containing any NA
  if(sum(is.na(dat[,1:2]) >0)) { exclu <- which(rowSums(is.na(dat[,1:2])) >0)
    filt1 <- filt1[-1*which(filt1 %in% exclu)] }
  filt0 <- (1:nrow(dat))[-1*filt1]                         # opposite of filt1, ie index numbers of points(lines of dat) excluded
  ## main : segment - if data sufficiently large
  bestPart <- if(length(filt1) >60 ) { 
    .insp1dimByClustering(offS[filt1], automClu=TRUE, cluChar=TRUE, silent=silent, debug=debug, callFrom=fxNa)
  } else list(cluster=rep(1,length(filt1)), cluChar=matrix(c(length(filt1), stats::median(offS[filt1]),
    abs(stats::sd(offS[filt1]))/mean(offS[filt1])), nrow=1, dimnames=list(1,c("n","center","centerCV"))))
  if(!silent && length(filt1) >50) message(fxNa,"Data (",nrow(dat), " lines) segemented in ",nrow(bestPart$cluChar)," cluster(s)")
  ## note : cluster-names MUST be numeric-like !!  (no letters !)
  ## now add info from 2 dim data
  ## note : problem with using cor: small clusters have tendency for high cor ...
  ## choose base on 'refVa' (1-cor) x (centerCV)/ sqrt(n)   ... lower =better
  cluTab <- table(bestPart$cluster)
  disIni <- as.list(rep(NA,length(cluTab)))
  extrPtDis <- rep(NA,length(cluTab))
  for(j in as.numeric(names(cluTab))[which(cluTab > max(2,minNPoints))]) {
    cluLi <- which(bestPart$cluster==j)
    disIni[[j]] <- .neigbDis(dat[filt1[cluLi],1:2], asSum=FALSE)
    tmp <- dat[filt1[cluLi[order(rowMeans(dat[filt1[cluLi],1:2], na.rm=TRUE))]], 1:2]
    extrPtDis[j] <- sqrt(sum((tmp[1,] -tmp[length(cluLi)])^2 ))
  }
  if(debug) message(fxNa,"   ",sum(!is.na(extrPtDis))," out of ",length(extrPtDis)," distances calultated")
  xyCor <- as.numeric(unlist(by(dat[filt1,1:2], bestPart$cluster, function(x) stats::cor(x[,1],x[,2]) )))
  offSclu <- cbind(bestPart$cluChar, cor=xyCor)
  offSclu <- cbind(offSclu, refVa=(1-abs(offSclu[,"cor"]))*offSclu[,"centerCV"]/sqrt(offSclu[,"n"]))      ## <<== refVa
  offSclu <- cbind(offSclu, meanNeigbDi=sapply(disIni,mean,na.rm=TRUE,trim=0.2), maxSpread=extrPtDis)
  if(debug) message(fxNa,"   'offSclu' columns: ",paste(colnames(offSclu),collapse=" "))
  ## now refine best clusters (either by ratio overallSpread/meanNeighbDist, or by CV & cor)
  if(bestCluByDistRat && sum(cluTab >minNPoints) >1) {
    refRat <- mean(offSclu[,"meanNeigbDi"]/offSclu[,"maxSpread"], trim=0.2, na.rm=TRUE)
    refCluNo <- which(offSclu[,"meanNeigbDi"]/offSclu[,"maxSpread"] < refRat & offSclu[,"n"] >=minNPoints)
    if(length(refCluNo) <1) { message(fxNa,"Cannot find decent clusters with 'bestCluByDistRat' ") }
  } else  {
    refCluNo <- which(offSclu[,"refVa"] < stats::median(offSclu[,"refVa"],na.rm=TRUE) &&
      abs(offSclu[,"cor"]) > 0.95 && offSclu[,"centerCV"] < cvThr[1] & offSclu[,"n"] >=minNPoints)
    if(length(refCluNo) <1) { refCluNo <- which(offSclu[,"refVa"] < stats::median(offSclu[,"refVa"],na.rm=TRUE) &&
      abs(offSclu[,"cor"]) > 0.92 && offSclu[,"centerCV"] < cvThr[2] & offSclu[,"n"] >=minNPoints) }
  }
  if(debug) message(fxNa,"  select clusters ",if(length(refCluNo) >0) paste(refCluNo,collapse=" ") else "(none)"," for refining")
  if(length(refCluNo) >0) { for(j in refCluNo) {       
      newCluNo <- max(bestPart$cluster) +1
       ## this part may be further improved (.keepCenter1d may remove too much = split good clusters) 
      tmp <- .keepCenter1d(offS[filt1[which(bestPart$cluster==j)]], core="veryhigh", keepOnly=FALSE, displPlot=FALSE, silent=silent, callFrom=fxNa)   # want display of hist-refinement?
      if(!silent) message(fxNa,"Split clu no  ",j," in main (n=",length(tmp$keep),") and fragm clu ",
        newCluNo," (n=",length(tmp$drop),")")
      bestPart$cluster[which(bestPart$cluster==j)[tmp$drop]] <- newCluNo }
    if(debug) message("   now at ",newCluNo," groups/clusters")
    cluN <- bestPart$cluster
    offSclu <- cbind(n=table(cluN), center=tapply(offS[filt1], cluN, mean, na.rm=TRUE), centerSd=tapply(offS[filt1], cluN, stats::sd, na.rm=TRUE))
    ## set cor to 0 if cor can't get caluclated due to indentical values in one of the colums
    xyCor <- as.numeric(unlist(by(dat[filt1,1:2],cluN,function(x) {if(length(unique(x[,1])) >1 && length(unique(x[,2])) >1) stats::cor(x[,1],x[,2]) else 0} ))) 
    refVa <- (1 -abs(xyCor)) *offSclu[,"centerSd"]/ (offSclu[,"center"] *sqrt(offSclu[,"n"]))
    offSclu <- cbind(offSclu ,centerCV=abs(offSclu[,"centerSd"]/offSclu[,"center"]), cor=xyCor, refVa=refVa)
    if(debug) message(fxNa,"  updated xyCor,offSclu ...")
  }
  if(length(refCluNo) <1) {                  ## nothing found so far, rather choose big cluster with cor > 50%
    if(debug) message(fxNa,"  nothing found so far, rather choose big cluster with cor > 50%")
    tmp3 <- which(offSclu[,"n"] >3)
    if(nrow(offSclu) ==1){ refCluNo <- 1 } else {                     # single cluster (no choice)
    if(length(tmp3) <2) refCluNo <- which.max(offSclu[,"n"]) else {   # all clusters smaller <4, choose largest
      tmp <- offSclu[which(offSclu[,"n"] >3),]                        # or select amongst clusters >3
     if(length(dim(tmp)) >1){
        tmp2 <- which(abs(tmp[,"cor"]) >= stats::median(abs(tmp[,"cor"]), na.rm=TRUE))
        tmp <- log(tmp[tmp2,"n"])/20 + tmp[tmp2,"cor"] -10*tmp[tmp2,"centerCV"]       # compromise betw n,cor & centerCV
        refCluNo <- as.numeric(names(tmp2)[which.max(tmp2)])
      } else refCluNo <- which(offSclu[,"n"] >3)
    } }
    if(!silent) message(fxNa,"Cannot easily identify clusters for refining, use (large & decent cor) clu: ",refCluNo)
  }
  ## get characteristics (of best clusters)   (no need to refresh refCluNo, OK)
  ## add parameter for checking if points are compact : sort, sum of each dist.to.neighb
  neigbDist <- rep(NA, nrow(offSclu))
  for(j in which(offSclu[,"n"] >1)) {
    neigbDist[j] <- mean(.neigbDis(dat[filt1[which(bestPart$cluster==j)],1:2], asSum=FALSE), trim=0.1, na.rm=TRUE)}  # 10% each trimmed mean(dist)
  offSclu <- cbind(offSclu,neigbDist=neigbDist)
  ## return matrix 'offT' with line-characteristics (for selected/best) or list (line-characteristics & index)
  reportLi <- unique(c(refCluNo, which(offSclu[,"n"] > min(4,minNPoints))))             
  offT <- matrix(nrow=length(reportLi), ncol=10, dimnames=list(reportLi, c("n","medOffS","CVoffS","r","neigbDist",
    "CIlo","CIhi","CIov","grade","grade2")))
  for(j in reportLi){                
    sel <- which(bestPart$cluster==j)
    CI <- sort(stats::t.test(offS[filt1[sel]])$conf.int)
    offT[as.character(j),] <- c(n=length(sel), medOffS=stats::median(offS[filt1[sel]]),
      CVoffS=abs(stats::sd(offS[filt1[sel]])/mean(offS[filt1[sel]])), r=stats::cor(dat[filt1[sel],1],dat[filt1[sel],2]),
      neigbDist=.neigbDis(dat[filt1[which(bestPart$cluster==j)],1:2])/sum(bestPart$cluster==j),
      CIlo=CI[1], CIhi=CI[2], CIov=NA, grade=NA, grade2=NA) }
  offT[,"grade"] <- signif(log(offT[,"n"])/15 + offT[,"r"] -10*offT[,"CVoffS"],3)    # 'grade' .. consider n,r & CVoffS; higher..better
  if(debug) message(fxNa,"  offT created, columns ",paste(colnames(offT),collapse=" "))
  refCluSel <- which(refCluNo %in% rownames(offT)[which(offT[,"neigbDist"] <= neighbDiLim)])
  if(!is.null(neighbDiLim) && length(refCluSel) >0) refCluNo <- refCluNo[refCluSel]
  ## rank2 : rank of 'grade' among top-hits ('refCluNo'), here with 1 for highest=best 'grade'
  offT[match(refCluNo, rownames(offT)),"grade2"] <- (nrow(offT) +1 -rank(offT[,"grade"]))[match(refCluNo, rownames(offT))]
  if(indexPoints) {
    if(length(filt0) >0) bestPart$cluster[filt0] <- NA
    out <- list(offS=offT, clus=bestPart$cluster, index=lapply(refCluNo, function(x) which(bestPart$cluster==x)))
  } else out <- offT
  ## compare to lm fit                                            
  if(debug) message(fxNa,"  ..ready to refine ",length(refCluNo)," groups by lm (conserve = ",is.list(out),")")
  if(lmCompare && requireNamespace("MASS")) {for(i in 1:length(refCluNo)) {
    j <- refCluNo[i]
    dat2 <- as.data.frame(matrix(dat[filt1[which(bestPart$cluster==j)],1:2], ncol=2))
    colnames(dat2) <- c("slope","B")     # LETTERS[1:2]
    tryLm <- try(MASS::rlm(B ~ slope, data=dat2))
    if(debug) message("  ..i=",i," used ",c("MASS::rlm","stats::lm")[1 +as.numeric(inherits(tryLm, "try-error") )])
    if(inherits(tryLm, "try-error")) {
      if(!silent) message("Group ",refCluNo[i],":  problem making robust regression (from package MASS), trying regular regression instead")
      tryLm <- try(stats::lm(B ~ slope, data=dat2))
    }
    out$lm[[i]] <- tryLm                        # needed for lmFilter()
    if(i==1) out$lmSum <- matrix(NA_real_, nrow=length(refCluNo), ncol=6, dimnames=list(refCluNo,
      c("(Intercept)","slope","pInterc","pSlope","residSE","Rsqu")))
    if(inherits(tryLm, "try-error")) message(fxNa,"Problem making regression on group",refCluNo[i],"") else if(is.list(out)) {
      tmp <- if(inherits(tryLm, "rlm")) {
        c(2*stats::pt(-abs(stats::coef(summary(tryLm))[,3]), df=length(stats::residuals(tryLm))-1), stats::cor(tryLm$qr$qr[,1], tryLm$qr$qr[,2])^2)
        } else c(stats::coef(summary(tryLm))[,4], summary(tryLm)$adj.r.squared)
      if(any(tmp[1:2] ==0, na.rm=TRUE)) tmp[which(tmp[1:2]==0)] <- 1e-320            # avoid p=0 ...
      out$lmSum[i,] <- c(stats::coef(tryLm), tmp[1:2], summary(tryLm)$sigma,tmp[3])
    } }
    names(out$lm) <- refCluNo }
  useColPa <- if(nrow(offT) <8) 2:nrow(offT) else grDevices::rainbow(1.2*nrow(offT))[1+(1:nrow(offT))]
  useCol <- useColPa[out$clus]
  ## plot histogram of residues
  if(displHist) {
    if(displScat) if(all(graphics::par()$mfrow <2)) {graphics::layout(matrix(1:2, ncol=2)) ; message("Adjusting layout for 2 images")}  
    cluBor <- matrix(unlist(by(offS,bestPart$cluster,range)), byrow=TRUE, ncol=2)
    his <- try(graphics::hist(offS, breaks="FD", main="hist of residuals to given slope"), silent=TRUE)
    if(inherits(his, "try-error")) message(fxNa,"UNABLE to plot histogram figure !") else {
      graphics::mtext(paste(nrow(cluBor),"(best) clusters shown as grey/color boxes according to mean neighbour-point distance"),cex=0.8)
      yPos <- c(-0.03*max(his$counts),-0.005*max(his$counts))
      tmp <- unique(as.numeric(cluBor))
      graphics::rect(min(cluBor[,1]), mean(yPos), max(cluBor[,2]), yPos[2],col=grDevices::grey(0.95), border=grDevices::grey(0.7))
      graphics::segments(tmp,mean(yPos), tmp, yPos[2],col=grDevices::grey(0.7))
      graphics::rect(cluBor[refCluNo,1], yPos[1], cluBor[refCluNo,2], yPos[2], col=useColPa,border=useColPa[refCluNo]) } 
    }
  ## plot of dat
  if(displScat) {
    if(debug) message(fxNa,"  ..displScat=",displScat)
    grpChar1 <- offT[,"neigbDist"]
    names(grpChar1) <- rownames(offT)   # names gets lost when just 1 line in offT
    tit <- paste("identification of linear groups with slope ",signif(coeff,3))
    if(!all(unique(bestPart$cluster) %in% rownames(offT))) {
      cluNa <- data.frame(clu=unique(bestPart$cluster), xx=NA, stringsAsFactors=FALSE)
      tmp <- data.frame(clu=rownames(offT), offT, stringsAsFactors=FALSE)
      tmp <- merge(tmp, cluNa, by="clu", all=TRUE)
      grpChar1 <- tmp[,"neigbDist"]
      names(grpChar1) <- tmp[,"clu"]
    } 
    if(debug) message(fxNa,"Class dat ",class(dat),"   class grpNo ",class(bestPart$cluster),"   class grpChar ",class(grpChar1),
      "   class grpHighl ",class(rownames(offT)),"   class datNa ",class(argNa))
    useClu <- which.max(offT[,"n"])
    useLi <- which(bestPart$cluster==useClu)
    if(debug) message(fxNa,"  len useLi",length(useLi),"   useCol ",useCol)
    chG <- try(graphics::plot(dat[,1], dat[,2], main=tit, pch=21, bg=useCol, xlab=colnames(dat)[1], ylab=colnames(dat)[2]), silent=TRUE)
    if(inherits(chG, "try-error"))  message(fxNa,"UNABLE to plot scatter figure !") else {
      graphics::legend("topleft",paste0("clu ",rownames(offT)," ,n=",offT[,1]), text.col=1, pch=21, col=1, pt.bg=useColPa,cex=0.8,xjust=0.5,yjust=0.5)}        # as points
  }
  out }

#' Calculate residues of (2-dim) linear model 'lMod'-prediction of/for 'dat'
#'
#' This function calculates residues of (2-dim) linear model 'lMod'-prediction of/for 'dat' (using 2nd col of 'useCol' )
#'  (indexing in 'dat', matrix or data.frame with min 2 cols), using 1st col of 'useCol' as 'x'.
#' It may be used for comparing/identifying data close to regression  (eg re-finding data on autoregression line in FT-ICR)
#' 
#' @param dat matrix or data.frame, main input
#' @param lMod linear model, only used to extract coefficients offset & slope
#' @param regTy (character) type of regression model
#' @param useCol (integer) columns to use
#' @return This function returns a numeric vector of residues (for each line of dat)
#' @seealso \code{\link{searchLinesAtGivenSlope}}
#' @examples
#' set.seed(2016); dat1 <- matrix(c(runif(200)+rep(1:10,20)),ncol=10)
#' @export
.predRes <- function(dat, lMod, regTy="lin", useCol=1:2){
  ## calculate residues of (2-dim) linear model 'lMod'-prediction of/for 'dat' (using 2nd col of 'useCol' )
  ##  (indexing in 'dat', matrix or data.frame with min 2 cols), using 1st col of 'useCol' as 'x'
  ## used for comparing/identifying data close to regression  (eg re-finding data on autoregression line in FT-ICR)
  ## 'lMod' .. linear model, only used to extract coefficients offset & slope
  ##    used with column useCol[1] and then column useCol[2] subtracted
  ##  return numeric vector of residues (for each line of dat)
  dat <- dat[,useCol]
  lMod <- stats::coef(lMod)
  ## lin: y~ x +d+err; inv: y~ 1/x+d+err; res: y~ (x-y)+; nor: y~ x/y+; resY: y-x~ x+; norRes: y~ (x-y)/y+; norResY: (y-x)/x~ x+; sqNorRes: y ~ (x-y)/(y*y)+; sqNorResY: (y-x)/(x*x)~ x+
  switch(regTy,                         # lMod[2] ... slope !!
    lin = lMod[2]*dat[,1] +lMod[1] -dat[,2],                      
    inv = lMod[2]/dat[,1] +lMod[1] -dat[,2],                      
    res = lMod[2]*(dat[,1] -dat[,2]) +lMod[1] -dat[,2],           
    nor = lMod[2]*dat[,1]/dat[,2] +lMod[1] -dat[,2],              
    resY = lMod[2]*dat[,1] +lMod[1] -(dat[,2] -dat[,1]),          
    norY = lMod[2]*dat[,1] +lMod[1] -dat[,2]/dat[,1],             
    norRes = lMod[2]*(dat[,1] -dat[,2])/dat[,2] +lMod[1] -dat[,2],
    norResY = lMod[2]*dat[,1] +lMod[1] -(dat[,2] -dat[,1])/dat[,1],
    sqNorRes = lMod[2]*(dat[,1] -dat[,2])/dat[,2]^2 +lMod[1] -dat[,2],
    sqNorResY = lMod[2]*dat[,1] +lMod[1] -(dat[,2] -dat[,1])/dat[,1]^2,
    other=NULL)
  }

#' Compare 'dat' to confindence interval of linare model 'lMod' (eg from lm())
#'
#' This function allows to compare 'dat' to confindence interval of linare model 'lMod' (eg from lm())
#' 
#' @param dat matrix or data.frame, main input
#' @param lMod linear model, only used to extract coefficients offset & slope
#' @param level (numeric) alpha threshold for linear model
#' @return This function returns a logical vector for each value in 2nd col of 'dat' if INSIDE confid interval
#' @seealso \code{\link{searchLinesAtGivenSlope}}
#' @examples
#' set.seed(2016); dat1 <- matrix(c(runif(200)+rep(1:10,20)),ncol=10)
#' @export
.checkLmConfInt <- function(dat, lMod, level=0.95){
  ## compare 'dat' to confindence interval of linare model 'lMod' (eg from lm())
  ## suppose (y-x)~x  (ie 2nd col estimated by 1st col) : formula(lMod)
  ## return logical vector for each value in 2nd col of 'dat' if INSIDE confid interval
  ##   could be improved by more steps betw various values of confint(lMod,level=level),
  ##   so far too stringent (nothing within confint range)
  ##   or use predict() and exploit pVal
  formPat <- sub("","", stats::formula(lMod)[3])                         # inspect formula if (y-x)~x
  chResForm <- stats::formula(lMod)[2] == paste0("(",formPat," - ",sub(" 1"," 2",formPat),")()")
  mod <- cbind(main=stats::coef(lMod)[2]*dat[,1] +stats::coef(lMod)[1],
    up1=stats::confint(lMod,level=level)[2,1]*dat[,1] +stats::confint(lMod,level=level)[1,1],
    up2=stats::confint(lMod,level=level)[2,1]*dat[,1] +stats::confint(lMod,level=level)[1,2],
    lo1=stats::confint(lMod,level=level)[2,2]*dat[,1] +stats::confint(lMod,level=level)[1,2],
    lo2=stats::confint(lMod,level=level)[2,2]*dat[,1] +stats::confint(lMod,level=level)[1,1])
  moRa <- t(apply(mod, 1, range))
  out <- if(chResForm) dat[,2] -dat[,1] >= moRa[,1] & dat[,2] -dat[,1] <= moRa[,2] else NULL
  out }

#' Segment (1-dim vector) 'dat' into clusters
#'
#' This function allows aegmenting (1-dim vector) 'dat' into clusters.
#' If 'automClu=TRUE ..' first try automatic clustering, if too few clusters, run km with length(dat)^0.3 clusters
#' This function requires the package NbClust to be installed.
#' 
#' @param dat matrix or data.frame, main input
#' @param automClu (logical) run atomatic clustering
#' @param cluChar (logical) to display cluster characteristics
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns clustering (class index) or (if 'cluChar'=TRUE) list with clustering and cluster-characteristics
#' @seealso \code{\link{searchLinesAtGivenSlope}}
#' @examples
#' set.seed(2016); dat1 <- matrix(c(runif(200)+rep(1:10,20)),ncol=10)
#' @export
.insp1dimByClustering <- function(dat, automClu=TRUE, cluChar=TRUE, silent=FALSE, debug=FALSE, callFrom=NULL){
  ## Segment (1-dim vector) 'dat' into clusters
  ## if 'automClu=TRUE ..' first try automatic clustering, if too few clusters, run km with length(dat)^0.3 clusters
  ## 'cluChar' .. to display cluster characteristics
  ## return clustering (class index) or (if 'cluChar'=TRUE) list with clustering and cluster-characteristics
  ## require(NbClust)
  fxNa <- .composeCallName(callFrom, newNa=".insp1dimByClustering")
  if(isTRUE(debug)) silent <- FALSE else { debug <- FALSE
    if(!isTRUE(silent)) silent <- FALSE }
  if(!isFALSE(automClu)) automClu <- TRUE  
  if(!isFALSE(cluChar)) cluChar <- TRUE
  out <- NULL  
  if(automClu) { if(requireNamespace("NbClust", quietly=TRUE)) {
      cluAut <- try(NbClust::NbClust(dat, method="average", index="ccc"),silent=TRUE)     # cluster only best = smallest dist
      if(inherits(cluAut, "try-error")) { warning(fxNa,"Failed to run NbClust::NbClust(); ignoring  'automClu'")
        automClu <- FALSE }
    } else {if(!silent) message(fxNa,"NOTE: Package 'NbClust' not found ! Please install first from CRAN \n   setting 'automClu'=FALSE for using kmeans()")
    automClu <- FALSE }
  }       

  if(automClu) {    
    out <- cluAut$Best.partition
    if(length(table(out)) < ceiling(length(dat)^0.16)) {
      if(!silent) message(fxNa,"Automatic clustering proposed only ",length(table(out))," clusters -> impose ",ceiling(length(dat)^0.3))
      automClu <- FALSE } }   # insufficient number of clusters -> run kmeans
  if(!automClu) {
    nClu <- ceiling(length(dat)^0.4)
    kMe <- stats::kmeans(dat, centers=nClu, iter.max=9)
    out <- kMe$cluster }
  if(cluChar) {
    ctrClu <- tapply(dat, out, mean, na.rm=TRUE)
    sdClu <- tapply(dat, out, stats::sd, na.rm=TRUE)
    out <- list(cluster=out, cluChar=cbind(n=table(out), center=ctrClu, centerSd=sdClu, centerCV=abs(sdClu/ctrClu))) }
  out }


#' Refine/filter 'dat1' (1dim dataset, eg cluster) with aim of keeping center of data
#'
#' This function allows to refine/filter 'dat1' (1dim dataset, eg cluster) with aim of keeping center of data.
#' It is done based on most freq class of histogramm keep/filter data if 'core' (% of volume) are within 'core' (%) range 
#' 
#' @param dat1 simple numeric vector
#' @param core numeric vactor (betw 0 and 1) for fraction of data to keep; if null trimmedMean/max hist occurance will be used, limited within 30-70 perent; may also be 'high' or 'low' for forcing low (20-60percent) or high (75-99) percent of data to retain
#' @param keepOnly (logical) 
#' @param displPlot (logical) show plot of hist & boundaries
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns the index of values retained or if 'keepOnly' return list with 'keep' index and 'drop' index
#' @seealso \code{\link{searchLinesAtGivenSlope}}
#' @examples
#' set.seed(2016); dat1 <- matrix(c(runif(200)+rep(1:10,20)),ncol=10)
#' @export
.keepCenter1d <- function(dat1, core=NULL, keepOnly=TRUE, displPlot=FALSE, silent=TRUE, debug=FALSE, callFrom=NULL) {
  ## refine/filter 'dat1' (1dim dataset, eg cluster) with aim of keeping center of data
  ## based on most freq class of histogramm keep/filter data if 'core' (% of volume) are within 'core' (%) range
  ##  or adopt to non-symmetric if most freq class is close to either end of distribution
  ## 'dat1  .. simple numeric vector
  ## 'core' .. numeric vactor (betw 0 and 1) for fraction of data to keep
  ##    if null trimmedMean/max hist occurance will be used, limited within 30-70%
  ##    may also be 'high' or 'low' for forcing low (20-60%) or high (75-99) % of data to retain
  ## 'displPlot' .. show plot of hist & boundaries
  ## not efficient if center very homogenous and very few values quite off !!
  ## return index of values retained or if 'keepOnly' return list with 'keep' index and 'drop' index
  fxNa <- .composeCallName(callFrom, newNa=".keepCenter1d")
  lim4cor <- c(0.3,0.7)
  if(identical(core, "high")) {
    lim4cor <- c(0.75,0.99)    # upper & lower limits for automatic determination of 'core' (ie % of values to keep)
    core <- NULL }
  if(identical(core, "low")) {
    lim4cor <- c(0.2,0.6)      # upper & lower limits for automatic determination of 'core' (ie % of values to keep)
    core <- NULL }
  dat1 <- signif(naOmit(dat1), 5)
  if(length(unique(dat1)) ==1)  {
    if(!silent) message(fxNa,"All (slightly rounded) values identical ! (keep all)")
    out <- 1:length(dat1)
    return(if(keepOnly) out else list(keep=out, drop=NULL))
  } else {
    his1 <- graphics::hist(dat1, breaks="FD", plot=FALSE)
    maxPo <- which(his1$counts==max(his1$counts, na.rm=TRUE))     # position of peak(s), ie mode based on hist
    maxVa <- if(length(maxPo)==1) his1$mids[maxPo] else sum(his1$mids[maxPo])/length(maxPo)
    if(length(maxPo) >1) {
      if(any(diff(maxPo)) >1) { if(!silent) message(fxNa," (multiple) non-neighbour peaks in hist")
         maxVa <- maxPo[round(stats::median(maxPo))] } }
    ## refine 'mode' in double resolution in +- 2 hist-classes range
    refWi <- 2.5*(his1$mids[2] -his1$mids[1])
    useBr <- seq(maxVa -refWi, maxVa +refWi, length.out=11)
    tab <- table(cut(dat1[dat1 > maxVa -refWi &  dat1 < maxVa +refWi], breaks=useBr))
    useP <- which(tab==max(tab))
    if(length(useP) >1) useP <- if(length(useP) %% 2 >0) stats::median(useP) else stats::median(useP[-1])
    maxVa <- ((useBr[-1] +useBr[-length(useBr)])/2)[useP]          # refined histogram 'mode'
    maxQu <- sum(dat1 <= maxVa)/length(dat1)                       # quantile of mode
    ## note: effort to find mode ('maxVa') using computeMode() from BBmisc was not at all satisfying ...
    ## note: fully tested against NAs
    ## automatic bandwidth ('core'=NULL) : based on ration max histogram class height / trimmed mean of heights
    if(is.null(core)) {
      core <- round(10*mean(his1$counts, trim=0.25)/his1$counts[which.max(his1$counts)], 3)   # sharp peak ->low ratio, small core fraction
      if(!silent) message(fxNa,"Proposed value for core ",core,"    trim mean ",
        signif(mean(his1$counts, trim=0.25), 3),"   max ",signif(his1$counts[which.max(his1$counts)], 3))
      core <- round(sort(c(lim4cor, core))[2],3) }          # keep proposed 'core' within limits
    .chSD <- function(x,y) {med <- stats::median(x,na.rm=TRUE); which(x > med -y*stats::sd(x,na.rm=TRUE) & x < med +y*stats::sd(x,na.rm=TRUE))}
    if(identical(core,"veryhigh")) {  
      core <- 0.9                         # must have some value for later use ..
      out <- .chSD(dat1, y=1)
      if(length(out) < length(dat1)*core) {out <- .chSD(dat1, y=1.5) }
      if(length(out) < length(dat1)*core) {out <- .chSD(dat1, y=2) }
      useQu <- c(sum(dat1 > min(dat1[out]) ,na.rm=TRUE), sum(dat1 > max(dat1[out]), na.rm=TRUE))/length(dat1)
      quaVa <- signif(stats::quantile(dat1, useQu), 4)
    } else {
      centrQu <- which(abs(dat1 -maxVa) ==min(abs(dat1 -maxVa)))
      useQu <- maxQu +c(-1,1)*core/2                           # boundaries of range around mode
      if(core/2 > min(maxQu,1 -maxQu)) {
        useQu <- if(useQu[1] <0) c(0,core) else c(1 -core,1)}
      quaVa <- signif(stats::quantile(dat1, useQu), 4)
      ## examine boders if assymetric (but not due to fact of touching range of data), if too assym bring most far closer
      offSe <- abs(quaVa -maxVa)
      offSe <- offSe > 1.3*sum(offSe)/length(offSe)
      if(any(offSe) && !(any(useQu %in% c(0,1)))) {
        quaVa <- .bringToCtr(quaVa, maxVa)
        useQu[which(offSe)] <- sum(dat1 >= quaVa[which(offSe)])/length(dat1)  }
      quaVa <- signif(quaVa,4)
      if(any(offSe) && !(any(useQu %in% c(0,1))) && !silent)  message(fxNa,
        "  ..new modified borders ",paste(quaVa,collapse=" "))  # result not ideal ?
      out <- which(dat1 >= quaVa[1] & dat1 <= quaVa[2])
    }
    if(length(out) > length(dat1)*core && !silent) message(fxNa,"Keep ",
      round(length(out) -length(dat1)*core)," more elements than ",round(100*core),"%")
    if(displPlot) {
      chG <- try(graphics::plot(his1, col=grDevices::grey(0.8),border=grDevices::grey(0.7)), silent=TRUE)
      if(inherits(chG, "try-error")) message(fxNa,"UNABLE to draw plot !") else {
        graphics::mtext(paste("n=",length(dat1)), cex=0.8)
        graphics::abline(v=sort(c(quaVa,maxVa)), lty=c(2,1,2), col=grDevices::grey(0.4))}}
    if(!keepOnly) out <- list(keep=out, drop=(1:length(dat1))[-out], limits=useQu)
    out }
    }
       

#' Planing for making all multiplicative combinations
#'
#' Provide all combinations for each of n elements of vector 'nMax' (positive integer, eg number of max multiplicative value). 
#' For example, imagine, we have 3 cities and the (maximum) voting participants per city.
#' Results must be read vertically and allow to see all total possible compositons.
#' 
#' @param nMax (positive integer) could be max number of voting participants form different cities, eg Paris max 2 persons, Lyon max 1 person ...
#' @param include0 (logical) include 0 occurances, ie provide al combinations starting from 0 or from 1 up to nMax 
#' @param asList (logical) return result as list or as array
#' @param silent (logical) suppress messages
#' @param callFrom (character) allow easier tracking of messages produced
#' @return list or array (as 2- or 3 dim) with possible number of occurances for each of the 3 elements in nMax. Read results vertical : out[[1]] or out[,,1] .. (multiplicative) table for 1st element of nMax; out[,,2] .. for 2nd
#' @seealso  \code{\link[utils]{combn}} 
#' @examples
#' combinatIntTable(c(1,1,1,2), include0=TRUE, asList=FALSE, silent=TRUE)
#' ## Imagine we have 3 cities and the (maximum) voting participants per city :
#' nMa <- c(Paris=2, Lyon=1, Strasbourg=1)
#' combinatIntTable(nMa, include0=TRUE, asList=TRUE, silent=TRUE) 
#' @export
combinatIntTable <- function(nMax,include0=TRUE,asList=FALSE,callFrom=NULL,silent=TRUE){
  ##  used by .parCombinateAllAndSum(), combinateAllAndSum()
  fxNa <- .composeCallName(callFrom, newNa="combinatIntTable")
  iniNa <- names(nMax)
  ch <- as.integer(nMax)
  msg <- "'nMax' should be positive integer (numeric) of length >1"
  if(!is.integer(ch) | length(nMax) <1) stop(msg) else nMax <- ch
  ch <- is.na(nMax) | nMax <1
  if(any(ch)) {if(sum(!ch) >0) {nMax <- nMax[!ch]; iniNa <- iniNa[!ch]} else stop(msg)
    if(!silent) message(fxNa,"trimming to ",sum(!ch)," positive non-NA")}
  ## main
  k <- if(include0) 0:1 else 1:0
  if(length(nMax) <2) {out <- as.matrix(k[1]:nMax); return(if(!asList) out else list(out))}       # case of single valid nMax (improve ?)
  diM <- c(k[2] +nMax[2], prod(nMax[-2] +k[2]))
  out <- list(matrix(rep(k[1]:nMax[1], each=diM[1]), nrow=diM[1], ncol=diM[2]))
  if(length(nMax) >1) out[[2]] <- matrix((k[1]:nMax[2]), nrow=diM[1], ncol=diM[2])
  if(length(nMax) >2) for(i in 3:length(nMax)) out[[i]] <- matrix(rep(k[1]:nMax[i], each=prod(k[2]+nMax[1:(i-1)])), nrow=diM[1], ncol=diM[2])
  if(!asList) {
    out <- array(unlist(out), dim=c(dim(out[[1]]), length(nMax)))
    dimnames(out)[[3]] <- iniNa } else names(out) <- iniNa
  out }
   

#' Check how multiple groups of data separate or overlap based on mean +/- sd
#'
#' \code{checkAvSd} compares if/how neighbour groups separate/overlap via the 'engineering approach' (+/- 2 standard-deviations is similar to a=0.05 \code{t.test}).
#' This approach may be used as less elegant alternative to (multi-group) logistic regression.
#' The function uses 'daAv' as matrix of means (rows are tested for up/down character/progression) which get compared with boundaries taken from daSd (for Sd values of each mean in 'daAv').
#' @param daAv matrix or data.frame
#' @param daSd matrix or data.frame
#' @param nByGr optinal specifying number of Elements per group, allows rather using SEM (adopt to variable n of different groups)
#' @param multSd (numeric) the factor specifyin how many sd values should be used as margin
#' @param codeConst (character) which term/word to use when specifying 'constant'
#' @param extSearch (logical) if TRUE, extend search to one group further (will call result 'nearUp' or 'nearDw')
#' @param outAsLogical to switch between 2col-output (separate col for 'up' and 'down') or simple categorical vector ('const','okDw','okUp')
#' @param silent (logical) suppress messages
#' @param callFrom (character) allow easier tracking of message(s) produced
#' @return vector describing character as 'const' or 'okUp','okDw' (or if extSearch=TRUE 'nearUp','nearDw')
#' @seealso \code{\link[wrMisc]{rowGrpMeans}}
#' @examples
#' mat1 <- matrix(rep(11:24,3)[1:40],byrow=TRUE,ncol=8)
#' checkGrpOrderSEM(mat1,grp=gl(3,3)[-1])
#' checkAvSd(rowGrpMeans(mat1,gl(3,3)[-1]),rowGrpSds(mat1,gl(3,3)[-1]) )
#' # consider variable n :
#' checkAvSd(rowGrpMeans(mat1,gl(3,3)[-1]),rowGrpSds(mat1,gl(3,3)[-1]),nByGr=c(2,3,3)) 
#' @export
checkAvSd <- function(daAv,daSd,nByGr=NULL,multSd=2,codeConst="const",extSearch=FALSE,outAsLogical=TRUE,silent=FALSE,callFrom=NULL){
  fxNa <- .composeCallName(callFrom,newNa="checkAvSd")
  if(!identical(dim(daAv),dim(daSd))) stop(fxNa," dimensions of 'daAv' and 'daSd' not same !!")
  if(is.null(nByGr)) nByGr <- rep(1,ncol(daAv))
  nGr <- ncol(daAv)
  if(length(nByGr) != nGr & !silent) {
    message(fxNa," 'nByGr' doesn't match number of columns in 'daAv' !!")  }
  incr <- daAv[,-nGr]+ multSd*daSd[,-nGr]/sqrt(nByGr[-nGr]) < daAv[,-1]- multSd*daSd[,-1]/sqrt(nByGr[-1])
  decr <- daAv[,-nGr]- multSd*daSd[,-nGr]/sqrt(nByGr[-nGr]) > daAv[,-1]+ multSd*daSd[,-1]/sqrt(nByGr[-1])
  if(outAsLogical) {
    out <- cbind(up=rowSums(incr,na.rm=TRUE) >= ncol(incr), down=rowSums(decr,na.rm=TRUE) >= ncol(decr))
  } else {
    outInc <- rowSums(incr,na.rm=TRUE)
    outDec <- rowSums(decr,na.rm=TRUE)
    out <- rep(codeConst,nrow(daAv))
    out[which(outInc == nGr-1)] <- "okUp"
    out[which(outDec == nGr-1)] <- "okDw"
    if(extSearch & nGr >3){
      out[which(outInc == nGr-2)] <- "nearUp"
      out[which(outDec == nGr-2)] <- "nearDw" }
  }
  out }
   

#' Normal random number generation with close fit to expected mean and sd
#'
#' This function allows creating a vector of random values similar to \code{rnorm}, but resulting value get recorrected to fit to expected mean and sd.
#' When the number of random values to generate is low, the mean and sd of the resultant values may deviate from the expected mean and sd when using the standard \code{rnorm} function.
#' In such cases the function \code{rnormW} helps getting much closer to the expected mean and sd.
#' 
#' @details
#' For making result reproducible, a seed for generating random numbers can be set via the argument \code{seed}.
#' However, with \code{n=2} the resulting values are 'fixed' since no random component is possible at n <3.
#' 
#' @param n (integer, length=1)  number of observations. If \code{length(n) > 1}, the length is taken to be the number required.
#' @param mean (numeric, length=1) expected mean
#' @param sd (numeric, length=1) expected sd
#' @param seed (integer, length=1) seed for generating random numbers
#' @param digits (integer, length=1 or \code{NULL}) number of significant digits for output, set to \code{NULL} to get all digits   
#' @param silent (logical) suppress messages
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a numeric vector of random values
#' @seealso \code{\link[stats]{Normal}} 
#' @examples
#' x1 <- (11:16)[-5]
#' mean(x1); sd(x1)
#' ## the standard way
#' ra1 <- rnorm(n=length(x1), mean=mean(x1), sd=sd(x1))
#' ## typically the random values deviate (slightly) from expected mean and sd
#' mean(ra1) -mean(x1) 
#' sd(ra1) -sd(x1)
#' ## random numbers with close fit to expected mean and sd :
#' ra2 <- rnormW(length(x1), mean(x1), sd(x1))
#' mean(ra2) -mean(x1) 
#' sd(ra2) -sd(x1)          # much closer to expected value
#' @export
rnormW <- function(n, mean=0, sd=1, seed=NULL, digits=8, silent=FALSE, callFrom=NULL) {
  ## reconstitute orginal values based on summary
  fxNa <- .composeCallName(callFrom, newNa="rnormW")
  if(!isTRUE(silent)) silent <- FALSE
  if(length(n) <1 | !is.numeric(n)) { out <- NULL; if(!silent) message(fxNa,"invalid 'n', returning NULL")
  } else { 
    if(length(n) >1) n <- length(n)
    msg <- c("argument '","' must be positive numeric and of length=1")
    if(!is.numeric(mean)) mean <- try(as.numeric(mean))
    if(length(mean) != 1 | inherits(mean, "try-error")) stop(msg[1],"mean",msg[2])
    if(!is.numeric(sd)) sd <- try(as.numeric(sd))
    ## main
    if(n==1) {out <- mean
      if(isFALSE(silent) & is.finite(sd)) message(fxNa,"Ignoring 'sd' since n=1")   
    } else if(length(sd) != 1 | inherits(sd, "try-error")) stop(msg[1],"sd",msg[2])
    if(n==2) out <- mean + c(-1,1)*sd/sqrt(2) 
    if(n >2) {
      if(length(seed) ==1) try(set.seed(seed), silent=TRUE)
      out <- if(length(digits)==1 & is.finite(digits)) signif(stats::rnorm(n=n, mean=mean, sd=sd), digits) else stats::rnorm(n=n, mean=mean, sd=sd)
      out <- out - mean(out)            # set meant to 0
      out <- mean + out*sd/sd(out)      # adjust sd and reset mean
  }}
  out }
   

#' Scale data to given minimum and maxiumum
#'
#' This is a convenient way to scale data to given minimum and maxiumum without full standarization, ie without deviding by the sd. 
#' 
#' @param x (numeric) vector to rescacle
#' @param min (numeric) minimum value in output 
#' @param max (numeric) maximum value in output  
#' @return vector of rescaled data (in dimensions as input)
#' @seealso  \code{\link[base]{scale}} 
#' @examples
#' dat <- matrix(2*round(runif(100),2), ncol=4)
#' range(dat)
#' dat1 <- scaleXY(dat, 1,100)
#' range(dat1)
#' summary(dat1)
#' 
#' ## scale for each column individually
#' dat2 <- apply(dat, 2, scaleXY, 1, 100)
#' range(dat2)
#' summary(dat2)
#' @export
scaleXY <- function(x, min=0, max=1) {       ## adjust values to range form min to max
  y <- .scale01(x)
  y*(max-min) + min}

#' Scale between 0 and 1 (main)
#'
#' This function rescales between 0 and 1
#' 
#' @param x numeric vector to be re-scalded
#' @return This function returns a numeric vector of same length with re-scaled values
#' @seealso \code{\link{scaleXY}} , \code{\link[base]{scale}} 
#' @examples
#' .scale01(11:15)
#' @export
.scale01 <- function(x){low <- min(x, na.rm=TRUE); (x - low)/(max(x, na.rm=TRUE) - low)}     ## adjust values to range form 0 to 1

#' Scale between min and max value (main)
#'
#' This function rescales between user-defined min and max values
#' 
#' @param x numeric vector to be re-scalded
#' @param minim (numeric) minimum value for resultant vactor
#' @param maxim (numeric) minimum value for resultant vactor
#' @return This function returns a matrix of CV values
#' @seealso \code{\link{scaleXY}} , \code{\link[base]{scale}} 
#' @examples
#' .scaleXY(11:15, min=1, max=100)
#' @export
.scaleXY <- function(x, minim=2, maxim=3) {       ## adjust values to range form min to max
  y <- .scale01(x)
  y*(maxim-minim) +minim}

#' Search duplicated data over multiple columns, ie pairs of data
#'
#' \code{searchDataPairs} searches matrix for columns of similar data, ie 'duplicate' values in separate columns or very similar columns if \code{realDupsOnly=FALSE}.
#' Initial distance measures will be normalized either to diagonale (\code{normRange=TRUE)} of 'window' or to the real max distance observed (equal or less than diagonale).
#' Return data.frame with names for sample-pair, percent of identical values (100 for complete identical pair) and relative (Euclidean) distance (ie max dist observed =1.0).
#' Note, that low distance values do not necessarily imply correlating data.
#'
#' @param dat matrix or data.frame (main input)
#' @param disThr (numeric) threshold to decide when to report similar data (applied on normalized distances, low val fewer reported), applied on normalized distances (norm to diagonale of all data for best relative 'unbiased' view)
#' @param byColumn (logical) rotates main input by 90 degrees (using \code{\link[base]{t}}), thus allows to read by rows instead of by columns
#' @param normRange (logical) normize each columns separately if \code{TRUE}
#' @param altNa (character, default \code{NULL}) vector with alternative names (for display)
#' @param realDupsOnly (logical) if \code{TRUE} will consider equal values only, otherwise will also consider very close values (based on argument \code{disThr})
#' @param silent (logical) suppres messages
#' @param callFrom (character) allows easier tracking of messages produced
#' @return This function returns a data.frame with names of sample-pairs, percent of identical values (100 for complete identical pair) and rel (Euclidean) distance (ie max dist observed =1.0)
#' @seealso \code{\link[base]{duplicated}}
#' @examples
#' mat <- round(matrix(c(11:40,runif(20)+12,11:19,17,runif(20)+18,11:20), nrow=10), 1)
#' colnames(mat) <- 1:9
#' searchDataPairs(mat,disThr=0.05)
#' @export
searchDataPairs <- function(dat, disThr=0.01, byColumn=TRUE, normRange=TRUE, altNa=NULL, realDupsOnly=TRUE, silent=FALSE, callFrom=NULL) {
  fxNa <- .composeCallName(callFrom, newNa="searchDataPairs")
  if(!isTRUE(silent)) silent <- FALSE
  if(byColumn) dat <- t(dat)
  if(!is.null(altNa)){
    msg <- "invalid content of argument 'altNa' (should be non-redudant vector of length of number of columns of 'dat'"
    if(length(altNa) != ncol(dat)) {altNa <- NULL; if(!silent) message(fxNa,msg)}
    if(length(altNa) > length(unique(dat))) {altNa <- NULL; if(!silent) message(fxNa,msg)} }
  if(!is.null(altNa)) rownames(dat) <- altNa else {
    if(is.null(rownames(dat))) rownames(dat) <- 1:nrow(dat) }
  ma0 <- matrix(1:nrow(dat), nrow=nrow(dat), ncol=nrow(dat))
  diCoor <- cbind(li=ma0[lower.tri(ma0)], co=t(ma0)[lower.tri(ma0)])
  di <- stats::dist(dat)
  diNor <- if(normRange) stats::dist(apply(dat, 2, range)) else max(di,na.rm=TRUE) 
  di <- di/diNor
  chk <- diCoor[which(di < disThr),]
  if(!is.matrix(chk)) chk <- matrix(chk, ncol=2)
  chN <- if(nrow(chk) <1) matrix(nrow=0, ncol=4, dimnames=list(NULL,c("samp1","samp2","pcIdent","relDist"))) else {
    data.frame(samp1=rownames(dat)[chk[,2]], samp2=rownames(dat)[chk[,1]],
    pcIdent=round(100*apply(chk, 1, function(x) sum(dat[x[1],]==dat[x[2],],na.rm=TRUE)/ncol(dat)),1),
    relDist=signif(as.numeric(di)[which(di < disThr)],4))}
  if(nrow(chN) >0 && realDupsOnly) chN <- chN[chN$pcIdent ==100,]
  chN }
    

#' Extract Just One Series, ie One Channel, Of List Of Arrays
#'
#' This function was designed for handeling measurements stored as list of multiple arrays, like eg compound-screens using microtiter-plates where multiple parameters ('channels') 
#' were recorded for each well (element).
#' The elements (eg compounds screened) are typcally stored in the 1st dimension of the arrays, the replicated in the secon dimension and different measure types/parameters in the 3rd chanel. 
#' In order to keep the structure of of individual microtiter-plates, typically each plate forms a separate array (of same dimensions) in a list.  
#' The this function allows extracting a single channel of the list of arrays (3rd dim of each array) and return row-appended matrix. 
#' @param arrLst (list) list of arrays (typically 1st and 2nd dim for specific genes/objects, 3rd for different measures associated with)
#' @param cha (integer) channel number
#' @param na.rm (logical) default =TRUE to remove NAs
#' @param rowSep (character) separator for rows
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allows easier tracking of messages produced
#' @return This function returns a list with just single channel extracted
#' @seealso \code{\link[wrMisc]{organizeAsListOfRepl}}
#' @examples
#' arr1 <- array(1:24,dim=c(4,3,2),dimnames=list(c(LETTERS[1:4]),
#'   paste("col",1:3,sep=""),c("ch1","ch2")))
#' arr2 <- array(74:51,dim=c(4,3,2),dimnames=list(c(LETTERS[1:4]),
#'   paste("col",1:3,sep=""),c("ch1","ch2")))
#' arrL1 <- list(pl1=arr1,pl2=arr2)
#' extr1chan(arrL1,ch=2)
#' @export
extr1chan <- function(arrLst, cha, na.rm=TRUE, rowSep="__", silent=FALSE, debug=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom, newNa="extr1chan")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  ## extract single channel of list of arrays (3rd dim of each array) and return row-appended matrix
  msg <- "'cha' should be numeric of length 1"
  for(pl in 1:length(arrLst)) out <- if(pl ==1) arrLst[[pl]][,,cha] else rbind(out,arrLst[[pl]][,,cha])
  plNa <- names(arrLst)
  if(is.null(plNa)) plNa <- as.character(1:length(arrLst))
  if(!is.null(rowSep)) rownames(out) <- paste(rep(plNa, sapply(arrLst,nrow)), rownames(out), sep=rowSep)
  if(na.rm) out <- out[ which(rowSums(is.na(out)) < ncol(arrLst[[1]])),]
  out }
   

#' Run lm on segmented data (from clustering)
#'
#' \code{lmSelClu} runs linear regression on data segmented previously (eg by clustering).  
#' This functio offers various types of (2-coefficient) linear regression on 2 columns of 'dat' (matrix with 3rd col named 'clu' or 'cluID', numeric elements for cluster-number).
#' If argument \code{'clu'} is (default) 'max', the column 'clu' will be inspected to take most frequent value of 'clu', otherwise a numeric entry specifying the cluster to extract is expected.
#' Note: this function was initially made for use with results from diagCheck()
#' Note: this function lacks means of judging godness of fit of the regression preformed & means for plotting
#' @param dat matrix or data.frame
#' @param useCol (integer or charcter) specify which 2 columns of 'dat' to use for linear regression
#' @param clu (character) name of cluster to be extracted and treatad
#' @param regTy (character) change type used for linear regression :  'lin' for 1st col ~ 2nd col, 'res' for residue ~ 2nd col, 'norRes' for residue/2nd col ~2nd col or 'sqNorRes','inv' for 1st col ~ 1/(2nd col), 'invRes' for residue ~ 1/(2nd col)
#' @param filt1 (logical or numerical) filter criteria for 1st of 'useCol' , if numeric then select all lines of dat less than max of filt1
#' @param filt2 (logical or numerical) filter criteria for 2nd of 'useCol' , if numeric then select all lines of dat less than max of filt2
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return lm object (or NULL if no data left)
#' @seealso \code{\link[stats]{lm}}
#' @examples
#' set.seed(2016); ran1 <- runif(220)
#' mat1 <- round(rbind(matrix(c(1:100+ran1[1:100],rep(1,50)),ncol=3), 
#'   matrix(c(1:60,68:9+ran1[101:160],rep(2,60)),nc=3)),1)
#' colnames(mat1) <- c("a","BB","clu")
#' lmSelClu(mat1)
#' plot(mat1[which(mat1[,3]=="2"),1:2],col=grey(0.6))
#' abline(lmSelClu(mat1),lty=2,lwd=2)
#' # 
#' mat2 <- round(rbind(matrix(c(1:100+ran1[1:100],rep(1,50)),ncol=3), 
#'   matrix(c(1:60,(2:61+ran1[101:160])^2,rep(2,60)),nc=3)),1)
#' colnames(mat2) <- c("a","BB","clu")
#' (reg2 <- lmSelClu(mat2,regTy="sqNor"))
#' plot(function(x) coef(reg2)[2]+ (coef(reg2)[2]*x^2),xlim=c(1,70))
#' points(mat2[which(mat2[,3]=="2"),1:2],col=2)
#' @export
lmSelClu <- function(dat, useCol=1:2, clu="max", regTy="lin", filt1=NULL, filt2=NULL, silent=FALSE, debug=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom, newNa="lmSelClu")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  
  regTyOpt <- c("lin","sqNor","res", "inv","invRes","norRes","sqNorRes","parb","parbRes")
  msgE <- " Argument 'dat' should matrix or data.frame with at least 3 numeric columns (including column called 'clu')"
  if(length(dim(dat)) != 2) stop(msgE)
  if(ncol(dat) <3) stop(msgE)
  cluCol <- which(colnames(dat) %in% c("clu","cluID"))
  if(length(cluCol) <1) stop(" Can't find column named 'clu' or 'cluID' (specifying sub-group/cluster)")
  if(length(clu) >1) { message(fxNa," 'clu' should be of length 1 but is ",length(clu),", truncating...")
    clu <- clu[1]}
  if(identical(clu,"max")) {
    clu <- as.numeric(names(which.max(table(dat[,cluCol[1]])))) }
  msg2 <- paste(" Argument 'regTy' should be either ",pasteC(regTyOpt))
  if(length(regTy) !=1 || sum(regTy %in% regTyOpt) !=1) stop(msg2)
  lmColNa <- colnames(dat)[-1*cluCol][useCol]
  if(!is.null(filt1)) dat <-  dat[if(is.logical(filt1)) filt1 else {
    which(dat[,lmColNa[1]] > min(filt1) & dat[,lmColNa[1]] < max(filt1,na.rm=TRUE))},]
  if(!is.null(filt2)) dat <-  dat[if(is.logical(filt2)) filt2 else {
    which(dat[,lmColNa[2]] > min(filt2) & dat[,lmColNa[2]] < max(filt2,na.rm=TRUE))},]
  dat <- dat[which(dat[,cluCol]==clu),]
  if(length(dat) <1) { message(fxNa,":  No data left for linear modelling !")
    return(NULL)                           
  } else {
    message(fxNa,"  ready for linear modeling of ",nrow(dat)," lines of data, ie cluster '",clu,"'")
    dat <- data.frame(dat, res=dat[,lmColNa[1]]-dat[,lmColNa[2]])
    dat$normRes <- dat$res/dat[,lmColNa[1]]
    dat$sqNormRes <- dat$res/(dat[,lmColNa[1]]^2)
    colnames(dat)[which(colnames(dat)==lmColNa[2])] <- "slope"
    dat$invSlope <- 1/dat$slope
    switch(regTy,   
      lin = stats::lm(get(lmColNa[1]) ~ slope, data=as.data.frame(dat)),        
      sqNor = stats::lm(get(lmColNa[1])^2 ~ slope, data=as.data.frame(dat)),    
      inv = stats::lm(get(lmColNa[1]) ~ invSlope, data=as.data.frame(dat)),     
      parb= stats::lm(get(lmColNa[1]) ~ invSlope^2, data=as.data.frame(dat)),   
      res = stats::lm(res ~ slope, data=as.data.frame(dat)),                    
      invRes = stats::lm(res ~ invSlope, data=as.data.frame(dat)),              
      parbRes= stats::lm(normRes ~ invSlope^2, data=as.data.frame(dat)),        
      norRes = stats::lm(normRes ~ slope, data=as.data.frame(dat)),             
      sqNorRes = stats::lm(sqNormRes ~ slope, data=as.data.frame(dat)))         
  }}
   

#' Simple figure showing line from start- to end-sites of edges (or fragments) defined by their start- and end-sites
#'  
#' \code{simpleFragFig} draws figure showing start- and end-sites of edges (or fragments) 
#'
#' @param frag (matrix) 2 columns defining begin- and end-sites (as interger values)
#' @param fullSize (integer) optional max size used for figure (x-axis)
#' @param sortByHead (logical) sort by begin-sites (if \code{TRUE}) or sort by end-sites
#' @param useTit (character) custom title
#' @param useCol (character) specify colors, if numeric vector will be onsidered as score values
#' @param displNa (character) display names of edges (figure may get crowded)
#' @param useCex (numeric) expansion factor, see also \code{\link[graphics]{par}}
#' @return matrix with mean values
#' @seealso \code{\link{buildTree}}, \code{\link{countSameStartEnd}}, \code{\link{contribToContigPerFrag}}, 
#' @examples
#' frag2 <- cbind(beg=c(2,3,7,13,13,15,7,9,7, 3,7,5,7,3),end=c(6,12,8,18,20,20,19,12,12, 4,12,7,12,4)) 
#' rownames(frag2) <- c("A","E","B","C","D","F","H","G","I", "J","K","L","M","N")
#' simpleFragFig(frag2,fullSize=21,sortByHead=TRUE) 
#' buildTree(frag2)
#' @export
simpleFragFig <- function(frag,fullSize=NULL,sortByHead=TRUE,useTit=NULL,useCol=NULL,displNa=TRUE,useCex=0.7){
  useColumn <- c(1,2)
  opar <- list(yaxt=graphics::par("yaxt"))
  on.exit(graphics::par(opar))
  fra <- frag[,useColumn]
  if("score" %in% colnames(fra)) useCol <- fra[,"score"]
  tmp <- table(as.numeric(fra))
  lin <- sort(unique(names(tmp)[which(tmp > stats::median(tmp,na.rm=TRUE))]))  
  ra <- range(as.numeric(fra),na.rm=TRUE)
  if(is.null(fullSize)) fullSize <- ra[2]
  if(is.numeric(useCol) & length(useCol)==nrow(fra)) {
    scoreV <- useCol
    col1 <- grDevices::rgb(red=c(179,131,135,203,253,244,255),green=c(170,204,221,211,174,109,0),blue=c(215,248,164,78,97,67,0),maxColorValue=255)        # pale purple, pale blue, 2x pale green, 2x orange to red
    useCol <- col1[as.numeric(cut(useCol,length(col1)))][]
    }  else scoreV <- NULL
  formDig <- function(x) sprintf(paste("%0",nchar(as.character(round(ra[2]))),"d",sep=""),x)
  reSort <- if(sortByHead) sort.list(paste(formDig(fra[,1]),formDig(fra[,2]))) else sort.list(paste(formDig(fra[,2]),formDig(fra[,1])),decreasing=TRUE)
  fra <- fra[reSort,]
  if(length(useCol) >1) useCol <- useCol[reSort]
  if(is.null(useCol)) useCol <- grDevices::grey(0.8)
  labOff <- round(ra[2]/40,1)  
  graphics::par(yaxt="n")                   
  graphics::plot(c(1,fullSize),c(1,nrow(fra)),type="n",main=useTit,xlab="fragment location",ylab="",las=1)
  if(length(lin) >0) graphics::abline(v=lin,lty=2,col=grDevices::grey(0.7))
  graphics::segments(x0=fra[,1],y0=1:nrow(fra),x1=fra[,2],y1=1:nrow(fra),lty=1,lwd=4,col=useCol)
  if(!is.null(rownames(fra)) & displNa) graphics::text(fra[,1]-labOff,1:nrow(fra),rownames(fra),cex=useCex)
  if(length(useCol) >1) {
    grpMean <- paste(">", signif(seq(min(scoreV),max(scoreV),length.out=2*length(col1)+1)[2*(1:length(col1))-1],3))
    graphics::legend("bottomright",legend=grpMean,text.col="black",fill=col1,cex=0.8,xjust=0.5,yjust=0.5)
    }
}
 

#' Combine Vectors From List And Return Basic Count Statistics
#'
#' The aim of this function is to choose a fixed number (\code{nCombin}) of list-elments from \code{lst} and count the number of common values/words.
#' Furthermore, one can define levels to fine-tune the types of combinations to examine.
#' In case multiple combinations for a given level are possible, some basic summary statistics are provided, too.
#'
#' @details
#' Note of caution :
#' With very long lists and/or high numbers of repeats of given levels, however, the computational effort incerases very much (like it does when using \code{table}).
#' Thus, when exploring all different combinations of large data-sets may easily result in queries consuming many ressources (RAM and processing time) !
#' It is recommended to start testing with test smaller sub-groups.
#'
#' The main idea of this function is to count frequency of terms when combining different drawings.
#' For example, you ask students from different cities which are their preferred hobbies, they may have different preference depending on the city ( defined by \code{lev}).
#' Now, if you want to make groups of 3 students, possibly with one from each city (A ,B and C), you want to count (/estimate) the frequency of different combinations possible.
#' Thus, using this function all combinations of the students from city A with the students from city B and C will be made when counting the number of common hobbies (by \code{nCombin} students).
#' Then, all counting results will be summarized to the average count for the various categories (which hobbies were seen once, twice or 3 times...),
#' sem (standard error of the mean) and CI (95% confidence interval), as well as sd.
#'
#' Of course, the number of potential combinations may quickly get very large. Using the argument \code{remDouble=TRUE} you can limit the search to
#' either finding all students giving the same answer plus all student giving different answers.
#' In this case, when a given level appears multiple times, all possible combinations using one of the respective entries will be be made with the other levels.
#'
#'
#' @param lst (list of character or integer vectors) main input
#' @param lev (character) define groups of \code{lst}
#' @param nCombin (integer) number of list-elements to combine from \code{lst}
#' @param remDouble (logical) remove intra-duplicates (defaults to \code{TRUE})
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns an array with 3 dimensions : i)  ii) the combinations of \code{nCombin} list-elements,
#'   iii) the number of counts (n), sem (standard error of the mean), CI (confidence interval) and sd
#' @seealso  \code{\link[base]{table}},  \code{\link[wrMisc]{replicateStructure}}
#' @examples
#' ## all list-elements are considered equal
#' tm1 <- list(a1=LETTERS[1:17], a2=LETTERS[3:19], a3=LETTERS[6:20], a4=LETTERS[8:22])
#' combineAsN(tm1, lev=gl(1,4))[,1,]
#'
#' ## different levels/groups in list-elements
#' tm4 <- list(a1=LETTERS[1:15], a2=LETTERS[3:16], a3=LETTERS[6:17], a4=LETTERS[8:19],
#'   b1=LETTERS[5:19], b2=LETTERS[7:20], b3=LETTERS[11:24], b4=LETTERS[13:25], c1=LETTERS[17:26],
#'   d1=LETTERS[4:12], d2=LETTERS[5:11], d3=LETTERS[6:12], e1=LETTERS[7:10])
#' te4 <- combineAsN(tm4, nCombin=4, lev=substr(names(tm4),1,1))
#' str(te4)
#' te4[,,1]
#'
#' @export
combineAsN <- function(lst, lev=NULL, nCombin=3, remDouble=TRUE, silent=FALSE, debug=FALSE, callFrom=NULL) {
  ## count combinatorics
  ## 'remDouble' .. idea to remove mixed auto&hetero-combinations (but leave auto.only and hetero.only combinations)
  ## combine 'nCombin' combinations of the (same level of) vectors in list 'lst' and search how may instances of the elements are found/counted on average, as total ('any'), single, double, triple or min2
  ## Example : There are 4 independent drawings of a variable number of letters from the alphabet.
  ## Note double-repeats may be quite frequent and are eliminated by default, triple- (and higher) repeats are kept.
  ## if multiple ways of making sets of 'nCombin' vetors exist, summary statistics will be added in 3rd dimension of resulting array
  fxNa <- .composeCallName(callFrom, newNa="combineAsN")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  reqPa <- c("utils","wrMisc")
  chPa <- sapply(reqPa, requireNamespace, quietly=TRUE)
  if(any(!chPa)) stop("package(s) '",paste(reqPa[which(!chPa)], collapse="','"),"' not found ! Please install first from CRAN")
  ## sanity checks
  if(length(lev) <1) { if(length(names(lst)) >0) { lev <- substr(names(lst),1,1)
      if(!silent) message(fxNa,"Note : Argument 'lev' is missing; trying to recuperate as 1st character of names of 'lst' :  ",pasteC(utils::head(lev,7),quoteC="'"), if(length(lev) >7) "...")
    } else stop("Argument 'lev' (for levels of list-elements) MUST be provided when 'lst' has no names !")}

  if(length(lst) != length(lev)) stop("Length of 'lst' and 'lev' must match !   Was ",length(lst)," and ",length(lev)," !")
  colNaSep <- c("_","-","\\.","\\+","__","--","\\+\\+")        # separators to test/for choice when combining
  inclMono <- TRUE            # complementary to remDouble (realy useful ?)

  if(length(nCombin) <1) { nCombin <- length(levels(as.factor(lev)))
    if(debug) message(fxNa,"Setting nCombin to ",nCombin)
  } else if(length(nCombin) >1) { nCombin <- nCombin[1]
    if(!silent) message(fxNa,"'nCombin' should be of length=1 (using only 1st)")}
  if(any(is.na(nCombin))) { nCombin <- length(levels(as.factor(lev)))
    if(!silent) message(fxNa,"'nCombin' has NA, resetting to default")}

  if(any(sapply(c("all","def"), identical, nCombin))) nCombin <- length(levels(as.factor(lev)))
  if(length(lst) <2 || nCombin <2) { datOK <- FALSE; if(!silent) message(fxNa,"Nothing to do")
  } else datOK <- TRUE

  if(datOK & nCombin ==2 & remDouble) { remDouble <- FALSE
    if(debug) message(fxNa,"Setting 'nCombin=2' AND 'remDouble=TRUE' makes no sense (nothing will match), running as 'remDouble=FALSE' ")}

  if(datOK) {
    ## check levels
    levNo <- try(suppressWarnings(as.numeric(unique(lev))), silent=TRUE)          # level-indexes
    if(inherits(levNo, "try-error") || any(is.na(levNo))) levNo <- unique(suppressWarnings(as.numeric(as.factor(lev))))
    if(debug) message(fxNa," levNo (",class(levNo),")  = ",pasteC(levNo))
    if(nCombin > length(lev) -1) { nCombin <- length(lev) -1
      if(!silent) message(fxNa," Argument 'nCombin' is too high for given data, re-setting to ",nCombin)
    }
    outDiNa <- list( c("sing","doub","trip","min2","any"), "x", c("n","sem","CI","sd"))
  }
    
  if(datOK) {
    if(nCombin ==length(lev)) {
      out <- table(table(unlist(lst)))[c("1","2","3")]
      out <- array(c(out, sum(out[c(1:2)]), sum(out), rep(NA,5*3)), dim=c(5,1,4), dimnames=outDiNa)
    } else {
      ## MAIN, prepare matrix of combin
      ## Setup combinations to try
      if(isTRUE(remDouble) && length(unique(levNo)) >1) { nCombin <- min(nCombin, length(levels(as.factor(lev))), na.rm=TRUE)
        if(!silent) message(fxNa," argument 'nCombin' combined to 'remDouble'=TRUE  is too high for given data, re-setting to ",nCombin)}
      combOfN <- utils::combn((1:length(lst)), nCombin)              # the combinations (all) ..
      ## prepare separator for names of combinations
      chSep <- sapply(colNaSep, function(x) length(grep(x, levNo)) >0)
      sep <- if(any(!chSep)) colNaSep[which(!chSep)[1]] else "_"
      colnames(combOfN) <- apply(combOfN, 2, function(x) paste(sort(lev[x]), collapse=sep))
      if(debug) {message(fxNa,"cA3a")}
      ## FILTER
      ## idea : remove all multi-intra exect all-intra
      if(remDouble) {           # remove mixed repeats (double-same + other) but keep all-same (or higher n-mers)
        chD <- sapply( strsplit(colnames(combOfN), sep), function(x) length(unique(x)))
        chD <- chD %in% c(nCombin, if(inclMono) 1 else NULL)
        if(any(!chD)) { combOfN <- if(sum(chD) >1) combOfN[,which(chD)] else matrix(combOfN[,which(chD)], ncol=1, dimnames=list(NULL, colnames(combOfN)[which(chD)]))
          if(debug) {message(fxNa,"Removing ",sum(!chD)," poly-repeats but not auto-repeats (out of ",ncol(combOfN),
            ") combinations since argument 'remDouble=",remDouble,"'  cA3b")}
        }
      }
      if(debug) { message(fxNa,"cA3c")}

      ## main part, extract elements and count frequencies
      com3c <- apply(combOfN, 2, function(x) sortByNRepeated(lst[x], silent=silent, debug=debug, callFrom=fxNa))   # make list of number of times found: 1x, 2x, 3x ...  takes a few sec
      forceN <- 3                                     # ?? for extracting (no more than) c("sing","doub","trip")
      if(debug) {message(fxNa,"cA3d")}
      maxLev <- max(sapply(com3c, length))
      tmp <- matrix(0, ncol=length(com3c), nrow=max(forceN, maxLev), dimnames=list(1:max(maxLev, forceN), names(com3c)))
      for(i in 1:length(com3c)) {tm2 <- if(is.matrix(com3c[[i]])) rep(nrow(com3c[[i]]), ncol(com3c[[i]])) else sapply(com3c[[i]], length)
        tmp[if(length(names(tm2)) >0) as.integer(names(tm2)) else 1:length(tm2), i] <- tm2  }
      rownames(tmp) <- c("sing","doub","trip", if(nrow(tmp) > 3) paste0("_",c(4:nrow(tmp)),"x"))
      ## adhere to previous structure (sing, doub, min2, any & rest), ie adjust number of indv x-counts to forceN
      comCou2 <- rbind(tmp[1:forceN,], min2=colSums(tmp[1:2,]), any=colSums(tmp))
      dupNa <- duplicated(colnames(comCou2), fromLast=FALSE)
      if(debug) {message(fxNa,"cA3e  Found ",sum(dupNa)," 'redundant' level-combinations  out of ",length(dupNa))}

      ## which contain repeated combinations of groups: calculate sd & derivatives, organize as array
      if(any(dupNa)) {
        grTy <- unique(colnames(comCou2))
        sumMatr <- function(x) { avX <- rowMeans(x); sdX <- rowSds(x); c(nAv=avX, sem=sdX/sqrt(ncol(x)), CI=apply(x, 1, confInt), sd=sdX)}   # seems OK

        comCou3 <- sapply(grTy, function(x) { y <- comCou2[,which(colnames(comCou2) %in% x)]; if(length(dim(y)) >1) sumMatr(y) else c(y, rep(NA,length(y)*3))} )
        if(length(dim(comCou3)) <2) comCou3 <- matrix(comCou3, ncol=1, dimnames=list(NULL, colnames(comCou2)))
        allNa <- colnames(comCou3)
        faRep <- findRepeated(colnames(comCou3), nonRepeated=TRUE)     # correct object (note: all colnames are already different)
        out <- array(NA, dim=c(nrow(comCou2), length(faRep$rep) +length(faRep$nonrep), 4), dimnames=list(rownames(comCou2), allNa,c("n","sem","CI","sd")))
          #isMult <- match(names(faRep$rep), allNa)   # needed ?
        if(debug) {message(fxNa,"cA3g")}
        for(i in 1:(dim(out)[3])) out[,,i] <- comCou3[(i-1)*nrow(out) +(1:nrow(out)),]
      } else {
        out <- array(c(comCou2, rep(NA, nrow(comCou2)*ncol(comCou2)*3)), dim=c(nrow(comCou2),ncol(comCou2),4),
          dimnames=list(rownames(comCou2), colnames(comCou2),c("n","sem","CI","sd")))
      }
    }
  }
  out }
    

#' Complete list of arrays for same dimensions
#'
#' This functions aims to inspect repeating structues of data given as list of arrays and will try to complete 
#' arrays with fewer lines or columns (as this may appear eg with the very last set of high-thourput sceening data 
#' if fewer measures remain in the last set). Thus, the dimensions of the arrays are compared and 
#' cases with fewer (lost) columns (eg fewer experimental replicates) will be adjust/complete  by adding column(s) of NA.
#' Used eg when at reading mircotiterplate data the last set is not complete.
#' @param arrLst (list) list of arrays (typically 1st and 2nd dim for specific genes/objects, 3rd for different measures associated with)
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allows easier tracking of message(s) produced
#' @return This function returns a list of arrays, now with same dimension of arrays 
#' @seealso \code{\link[wrMisc]{organizeAsListOfRepl}}, \code{\link[wrMisc]{extr1chan}}
#' @examples
#' arr1 <- array(1:24,dim=c(4,3,2),dimnames=list(c(LETTERS[1:4]),
#'   paste("col",1:3,sep=""),c("ch1","ch2")))
#' arr3 <- array(81:96,dim=c(4,2,2),dimnames=list(c(LETTERS[1:4]),
#'   paste("col",1:2,sep=""),c("ch1","ch2")))
#' arrL3 <- list(pl1=arr1,pl3=arr3)
#' completeArrLst(arrL3)
#' @export
completeArrLst <- function(arrLst, silent=FALSE, debug=FALSE, callFrom=NULL){
  ## adjust/complete cases with fewer (lost) columns (replicates) by adding column(s) of NA
  ## to obtain same dimension of arrays in list
  fxNa <- .composeCallName(callFrom,newNa="completeArrLst")
  dims <- sapply(arrLst,dim)
  corPos <- apply(dims, 1, function(x) which(x < max(x,na.rm=TRUE)))
  maxDim <- apply(dims, 1, max,na.rm=TRUE)
  if(!silent) message(fxNa,"Max dimensions ",paste(maxDim,collapse=" "))
  if(length(corPos[[1]]) >0) for(i in corPos[[1]]) message("Filling along rows not yet implemented")
  if(length(corPos[[2]]) >0) for(i in corPos[[2]]) {
    if(!silent) message(fxNa," +filling ",names(arrLst)[i],"   from ",paste(dim(arrLst[[i]]),collapse=" "),"  to ",paste(maxDim,collapse=" "))
    add <- array(NA,dim=c(maxDim[1],maxDim[2]-dim(arrLst[[i]])[2],maxDim[3]))
    orgDimNa <- dimnames(arrLst[[i]])
    arrLst[[i]] <- .fuse2ArrBy2ndDim(arrLst[[i]],add)
    dimnames(arrLst[[i]]) <- list(orgDimNa[[1]], c(orgDimNa[[2]], paste("NA",1:(maxDim[2]-length(orgDimNa[[2]])), sep=".")), orgDimNa[[3]])
    }
  if(length(corPos[[3]]) >0) for(i in corPos[[3]]) message(fxNa,"Filling along channels not yet implemented")
  arrLst }
  

#' Html Special Character Conversion
#'
#' Converts 'txt' so that (the most common) special characters (like 'beta','micro','square' etc) will be displayed correctly whe used for display in html (eg at mouse-over).
#' Note : The package \href{https://CRAN.R-project.org/package=stringi}{stringi} is required for the conversions (the input will get returned if \code{stringi} is not available).
#' Currently only the 16 most common special characters are implemented.
#' 
#' @param txt character vector, including special characters
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a corrected character vector adopted for html display
#' @seealso tables on \url{https://www.htmlhelp.com/reference/html40/entities/latin1.html},  
#'   \url{https://www.degraeve.com/reference/specialcharacters.php}, or \code{https://ascii.cl/htmlcodes.htm}
#' @examples
#' ## we'll use the package stringi to generate text including the 'micro'-symbol as input
#' x <- if(requireNamespace("stringi", quietly=TRUE)) {
#'   stringi::stri_unescape_unicode("\\u00b5\\u003d\\u0061\\u0062")} else "\"x=axb\""
#' htmlSpecCharConv(x)
#' @export
htmlSpecCharConv <- function(txt, silent=FALSE, callFrom=NULL, debug=FALSE) {
  fxNa <- .composeCallName(callFrom, newNa="htmlSpecCharConv")
  if(isTRUE(debug)) silent <- FALSE else { debug <- FALSE
    if(!isTRUE(silent)) silent <- FALSE }
  if(length(txt) >0){  
    if(!is.character(txt)) txt <- try(as.character(txt))
    if(inherits(txt, "try-error")) { warning(fxNa,"Invalid inout format (can't transform into text !)"); txt <- NULL }
  } 
  if(length(txt) >0){    
    if(requireNamespace("stringi", quietly=TRUE)) {
      speCh <- c("\\u00b5","\\u00ba","\\u00b9","\\u00b2","\\u00b3",beta="\\u00df","\\u00e0", ced="\\u00e7",
        "\\u00e8","\\u00e9", ae="\\u00e2","\\u00f6","\\u00fc","\\u00f7","\\u00f5",x="\\u00d7")
      conv <- try(stringi::stri_unescape_unicode(speCh), silent=TRUE) 
      if(inherits(conv, "try-error")) { warning(fxNa,": UNABLE to tun stringi::stri_unescape_unicode() !"); txt <- NULL
      } else {
        conv <- matrix(c(conv, "&micro","&ordm","&sup1","&sup2","&sup3","&szlig",          
          "&agrave","&ccedil","&egrave","&eacute","&auml", "&ouml","&uuml","&divide","&otilde","&times"), ncol=2)  
        conv <- rbind(conv, c('"','&quot'))
        for(i in 1:nrow(conv)) { che <- grep(conv[i,1], txt)
          if(length(che) >0) txt[che] <- gsub(conv[i,1], conv[i,2], txt[che]) } } 
    } else {
      if(!silent) message(fxNa,"NOTE: Package 'stringi' needed for conversion not found ! Please install first from CRAN. Returning initial 'txt'  (returning initial txt)")
    }
  }
  txt }

#' sd for each row (fast execution)
#'
#' This function is speed optimized \code{sd} per row of a matrix or data.frame and treats each row as independent set of data for sd (equiv to \code{apply(dat,1,sd)}).
#' NAs are ignored from data unless entire line NA). Speed improvements may be seen at more than 100 lines.
#' Note: NaN instances will be transformed to NA
#' @param dat matrix (or data.frame) with numeric values (may contain NAs which will be ignored)
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allows easier tracking of messages produced
#' @return numeric vector of sd values
#' @seealso \code{\link[stats]{sd}}
#' @examples
#' set.seed(2016); dat1 <- matrix(c(runif(200)+rep(1:10,20)),ncol=10)
#' rowSds(dat1)
#' @export
rowSds <- function(dat, silent=FALSE, debug=FALSE, callFrom=NULL) {
  fxNa <- .composeCallName(callFrom, newNa="rowSds")
  msg <- "'dat' should be matrix or data.frame with "
  if(is.null(ncol(dat))) stop(msg,"multiple columns !") else if(ncol(dat) < 2) stop(msg,"at least 2 columns !")
  if(is.data.frame(dat)) dat <- as.matrix(dat)
  allRowsNA <- rowSums(!is.na(dat)) <1
  out <- sqrt(rowSums(matrix(as.numeric(!is.na(dat)), ncol=ncol(dat))*((dat) - rowMeans(dat,na.rm=TRUE))^2, na.rm=TRUE)/(rowSums(!is.na(dat)) -1)) 
  chNan <- is.nan(out)
  if(any(chNan)) out[which(chNan)] <- NA      
  if(any(allRowsNA)) out[which(allRowsNA)] <- NA              
  out }
   

#' Reduce To First Occurance Of Repeated Lines
#'
#' This function concatenattes all columns of input-matrix and then searches like \code{unique} for unique elements, optionally the indexes of unique elements may get returned.
#' Note: This function reats input as character (thus won't understand \code{10==10.0} ).
#' Returns simplified/non-redundant vector/matrix (ie fewer lines), or respective index.
#' faster than  \code{\link{firstOfRepeated}} 
#' @param mat initial matrix to treat 
#' @param outTy for output type: 'ind'.. index to 1st occurance (non-red),'orig'..non-red lines of mat, 'conc'.. non-red concateneted values, 'num'.. index to which group/category the lines belong
#' @param useCol (integer) custom choice of which columns to paste/concatenate
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a simplified/non-redundant vector/matrix (ie fewer lines for matrix), or respective index
#' @seealso  \code{\link[base]{unique}}, \code{\link{nonAmbiguousNum}}, faster than \code{\link{firstOfRepeated}} which gives more detail in output (lines/elements/indexes of omitted)
#' @examples
#' mat <- matrix(c("e","n","a","n","z","z","n","z","z","b", 
#'   "","n","c","n","","","n","","","z"),ncol=2)
#' firstOfRepLines(mat,out="conc")
#' @export
firstOfRepLines <- function(mat, outTy="ind", useCol=NULL, silent=FALSE, debug=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom,newNa="firstOfRepLines")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE

  if(length(dim(mat)) !=2) stop(" expecting matrix or data.frame with >1 lines")
  if(!outTy %in% c("ind","orig","conc","num","all")) outTy <- "ind"
  if(is.null(useCol)) useCol <- 1:ncol(mat)
  ch <- if(nrow(mat) >1) .pasteCols(mat[,useCol],sep="") else paste(mat,collapse="")
  dup <- duplicated(ch, fromLast=FALSE)
  switch(outTy,
    ind=which(!dup),                                                            # index to 1st occurance
    orig=mat[which(!dup),],                                                     # non-red lines of mat
    conc=ch[!dup],                                                              # 1st occurance of concatenated
    num=if(any(dup)) match(ch, ch[!dup]) else 1:length(ch),
    all=list(ind=which(!dup), conc=ch[!dup], num=if(any(dup)) match(ch,ch[!dup]) else 1:length(ch)))}


#' Extract NA-neighbour values
#'
#' This function allows extracting NA-neighbour value
#' @param x initial matrix to treat 
#' @param grp (factor) grouing of replicates
#' @return This function returns a numeric vector
#' @seealso  \code{\link[base]{unique}}, \code{\link{nonAmbiguousNum}}, faster than \code{\link{firstOfRepeated}} which gives more detail in output (lines/elements/indexes of omitted)
#' @examples
#' .extrNAneighb(c(11:14,NA), rep(1,5))
#' @export
.extrNAneighb <- function(x, grp){
  ## extract values of numeric vector 'x' when NA in same group 'grp'
  ##  (used for estimatating/replacing NA by low values)
  out <- NULL
  y <- 1:length(x)
  NAgr <- (grp)[which(is.na(x))]
  for(i in unique(NAgr)) out <- c(out, naOmit(x[which(grp==i)]))
  out }


#' Paste-concatenate all columns of matrix
#'
#' This function allows paste columns
#' @param mat inital matrix
#' @param sep (character) separator
#' @return This function returns a simplified/non-redundant vector/matrix (ie fewer lines for matrix), or respective index
#' @seealso  \code{\link[base]{unique}}, \code{\link{nonAmbiguousNum}}, faster than \code{\link{firstOfRepeated}} which gives more detail in output (lines/elements/indexes of omitted)
#' @examples
#' .pasteCols(matrix(11:16,ncol=2), sep="_")
#' @export
.pasteCols <- function(mat, sep=""){
  ## paste all columns
  if(!is.matrix(mat)) mat <- as.matrix(mat)
  if(ncol(mat)==1) return(mat)
  if(nrow(mat) > ncol(mat)){
    out <- paste(mat[,1],mat[,2], if(ncol(mat) >2) mat[,3], if(ncol(mat) >3) mat[,4],sep=sep)
    if(ncol(mat) >4) for(i in 5:ncol(mat)) out <- paste(out,mat[,i])
  } else out <- apply(mat, 1, paste, collapse=sep)
  names(out) <- rownames(mat)
  out }
    

#' Identify Separator In Pairwise Group-Names Or Find Separator For Use With Pairwise Group-Names  
#'   
#' This function allows identifing separator used when pairwise groups are presented. 
#'
#' @details  
#' Note : Potential separators \code{potSep} must be at least 1 character long
#'   
#' The character '.' may used as separator (argument \code{potSep}), it will be protected internally for obtaining correct results when using grep().
#' Note : The indexing is relative to the levels of grp, thus to sorted group-names !
#' 
#' Cases of usage : 
#' 1) Identify separator used in concatenated labels, ie concatenated group-names are given, (only argument \code{compNames} is obligatory)
#' 
#' 2) Suggets/Find suitable separator for given group-labels (ie a separator not occurring in any of the group-labels) when names 
#'    of \code{grp} given (thus argument \code{grp} is obligatory), 
#'    the first instance of \code{potSep} fitting will be chosen
#' 
#' @param compNames (character or integer) names of pairwise combined group-names
#' @param grp (character or factor) optional groups (may include group-names that do not occur in \code{compNames})
#' @param potSep (character) potential separators to be checked if occuring in \code{useComp}; the first fitting will used
#' @param includeGrp (logical) if \code{includeGrp=TRUE} a list including also the group-names is returned
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging 
#' @param callFrom (character) allow easier tracking of messages produced
#' @seealso \code{\link{indexGroupsFromPW}}, \code{\link{getPWseparator}}, colnames of testing results from \code{\link{moderTestXgrp}}, used by \code{\link{replacePWseparator}} 
#' @return This function returns a character vector (length=1) of the (first) separator fitting to all data, or a list with $sep and $grpNa if \code{includeGrp=TRUE}
#' @examples
#' ## Suggest separator
#' getPWseparator(grp=LETTERS[1:3])
#' getPWseparator(grp=c("B-C","D","E")) 
#' getPWseparator(grp=c("B-C","D","E"), includeGrp=TRUE) 
#' 
#' ## Identify separator used
#' getPWseparator(compNames=c("B-C","B-D","C-D")) 
#' getPWseparator(compNames=c("B-C","B-D","C-D"), includeGrp=TRUE) 
#' @export
getPWseparator <- function(compNames=NULL, grp=NULL, potSep=c("-","---","_","___",".","-vs-","_vs_","--vs--","__vs__"," ","  ","--","__"), includeGrp=FALSE, silent=FALSE, debug=FALSE, callFrom=NULL) {
  ## find sperator of pairwise groups (knowing names of groups)
  fxNa <- .composeCallName(callFrom, newNa="getPWseparator")
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  if(!isTRUE(silent)) silent <- FALSE
  out <- out2 <- sep <- NULL
  if(length(compNames) >0) compNames <- naOmit(unique(compNames))
  if(length(dim(compNames)) >1) { compNames <- NULL
    warning(fxNa,"Invalid entry of 'compNames', should be factor or (character) vector")}
  if(length(potSep)==0) { potSep <- pwSeparatorList(type="split1", callFrom=fxNa) # c("-","---","_","___",".","-vs-","_vs_","--vs--","__vs__"," ","  ","--","__")  
    if(debug) message(fxNa,"User provided 'potSep' seems empty, setting to default panel for checking") } 
  if(length(compNames) >0) {
    out <- .getPWseparator(compNames, potSep=potSep, includeGrp=TRUE, silent=silent, debug=debug, callFrom=fxNa)
    sep <- out$sep
    if(length(grp) >0) {
      chGrp <- out$grpNa %in% naOmit(unique(grp))
      if(any(!chGrp)) { if(!silent) message(fxNa,"NOTE: ",sum(!chGrp)," elements found based on 'compNames' but not in user-provided 'grp', adding ..")
        grp <- naOmit(unique(grp, out$grpNa)) }
    } else grp <- out$grpNa
  } else {
    if(length(grp) > 0) grp <- naOmit(unique(grp))
    if(length(grp) > 0) {
      if(length(potSep)==1 && any(grepl("(split)|(combine)[[:digit:]]{0,1}", potSep), na.rm=TRUE)) potSep <- pwSeparatorList(type=potSep, silent=silent, debug=debug, callFrom=fxNa)
      potSep2 <- protectSpecChar(potSep)               ## protect '.' etc
      ## check for optimal separator
      chS <- try(sapply(potSep2, grepl, grp), silent=TRUE)
      if(debug) {message(fxNa,"gPWS1"); gPWS1 <- list(compNames=compNames,grp=grp,potSep=potSep,potSep2=potSep2,chS=chS)}
      if(!inherits(chS, "try-error")) {
        chS <- colSums(chS)
        if(any(chS ==0)) sep <- potSep[which(chS==0)[1]]
      } else if(!silent) message(fxNa,"FAILED checking for suitable SEPARATOR (maybe content of 'potSep' contains special characters that need getting protected ?)")
    } else warning(fxNa,"Arguments 'grp' and 'compNames' seems empty, nothing to do")
  }
  if(isFALSE(includeGrp)) sep else list(sep=sep, grpNa=grp)
}  
  
 
#' Find Separator In Vector Of Pairwise Group-Names  
#'   
#' This function allows identifing separator used when pairwise groups are presented. 
#'
#' @details  
#' Note : Potential separators \code{potSep} to be tested must be at least 1 character long
#'   
#' The character '.' may used as \code{potSep}, it will be protected internally for correct results.
#' Note : The indexing is relative to the levels of grp, thus to sorted group-names !
#' 
#' @param compNames (character or integer) names of pairwise combined group-names
#' @param potSep (character) potential separators to be checked if occuring in \code{useComp}; the first fitting will used
#' @param includeGrp (logical) if \code{includeGrp=TRUE} a list including also the group-names is returned
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging 
#' @param callFrom (character) allow easier tracking of messages produced
#' @seealso  \code{\link{indexGroupsFromPW}}, \code{\link{getPWseparator}}, colnames of testing results from \code{\link{moderTestXgrp}}, used by \code{\link{replacePWseparator}}
#' @return This function returns a character vector (length=1) of the (first) separator fitting to all data, or a list with $sep and $grpNa if \code{includeGrp=TRUE}
#' @examples
#' .getPWseparator(compNames=c("B-C","B-D","C-D")) 
#' @export
.getPWseparator <- function(compNames, potSep="split1", includeGrp=FALSE, silent=FALSE, debug=FALSE, callFrom=NULL) {
  ## find sperator of pairwise groups (knowing names of groups) : which potSep occurs only once in all cases
  fxNa <- .composeCallName(callFrom, newNa=".getPWseparator")
  if(length(potSep) !=0 && "" %in% potSep) potSep <- potSep[-which(potSep =="")]
  if(length(potSep) ==0 || any(is.na(potSep))) { potSep <- pwSeparatorList(type="split1", callFrom=fxNa)  #c("-","--","_","__",".","-vs-","_vs_","--vs--","__vs__"," ","  ","--","__") 
    if(!silent) message(fxNa,"Argument 'potSep' appears empty, setting to default") 
  } else potSep <- potSep[1]
  if(grepl("(split)|(combine)[[:digit:]]{0,1}", potSep)) potSep <- pwSeparatorList(potSep, callFrom=fxNa)

  potSep2 <- potSep  # for protected version
  chSepFx <- function(sep, txt) all(nchar(gsub(sep,"",txt)) +nchar(gsub("\\\\","",sep)) ==nchar(txt)) && !any(grepl(paste0(sep,"$"), txt)) && !any(grepl(paste0("^",sep), txt)) # check that candiates for separator if occoring only once .. 
  ## protect \\
  chP <- grepl("\\.", potSep2)
  if(any(chP)) potSep2 <- gsub("\\.","\\\\.", potSep2)      
  ## protect '.'
  chSpc <- potSep %in% '.' 
  if(any(chSpc))  potSep2[which(chSpc)] <- "\\."

  ## check for optimal separator
  chSep <- sapply(potSep2, chSepFx, compNames)
  if(sum(chSep, na.rm=TRUE) >0) groupSep <- potSep[which(chSep)[1]] else stop(fxNa,"Cannot figure out proper separator for elements like  ",pasteC(utils::head(compNames), quoteC="'"))
  if(!isTRUE(includeGrp)) groupSep else list(sep=groupSep, grpNa= sort(unique(unlist(strsplit(compNames,groupSep)))) )
}


#' Replace Separator In Vector Of Pairwise Group-Names  
#'   
#' This function allows identifying and substituting a separator used in a character vector concatenated of pairwise groups.
#'
#' @param compNames (character or integer) names of pairwise combined group-names
#' @param newSep (character of length=1) new separator between group-names for pair of names; if \code{NULL} a suitable separator (among '-','--','_','__','.',' ' and '  ') will be used
#' @param potSep (character) potential separators to be checked if occuring in \code{useComp}; the first fitting will used
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging 
#' @param callFrom (character) allow easier tracking of messages produced
#' @seealso  \code{\link{indexGroupsFromPW}}, \code{\link{getPWseparator}}, \code{\link{.getPWseparator}}
#' @return This function returns a character vector with group-wise substituted separators (or \code{NULL} if conflicts appear) 
#' @examples
#' replacePWseparator(compNames=c("B-C","B-D","C-D"), newSep="-vs-") 
#' @export
replacePWseparator <- function(compNames, newSep, potSep=c("-","---","_","___",".","-vs-","_vs_","--vs--","__vs__"," ","  "), silent=FALSE, debug=FALSE, callFrom=NULL)  {
  ## Replace Separator In Vector Of Pairwise Group-Names  
  fxNa <- .composeCallName(callFrom, newNa="replacePWseparator")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  if(length(newSep)==0) stop("Argument 'newSep' may not be NULL")
  if(length(potSep)==0 || any(is.na(potSep))) potSep <- pwSeparatorList("split1")  #c("-","---","_","___",".","-vs-","_vs_","--vs--","__vs__"," ","  ")
  
  if(length(newSep) >1) {newSep <- naOmit(newSep)[1]; message(fxNa,"Only 1st value of 'newSep' will be used")}
  tmp <- try(.getPWseparator(compNames, potSep=potSep, includeGrp=FALSE, debug=debug, callFrom=fxNa), silent=TRUE)
  ## check for mult subst
  if(!inherits(tmp, "try-error")) {
    ch1 <- grepl(newSep, compNames) 
    if(any(ch1)) warning(fxNa,"Problem : ",sum(ch1)," entries contain already ",newSep," , ie group-names will appear different")
    potSepP <- protectSpecChar(tmp)
    out <- sub(potSepP, newSep, compNames)
    out 
  } else NULL
}    
        
 

#' Per line and per group sd-values
#'
#' \code{rowGrpSds} calculate Sd (standard-deviation) for matrix with multiple groups of data, ie one sd for each group of data. 
#' Groups are specified as columns of 'x' in 'grp' (so length of grp should match number of columns of 'x', NAs are allowed).
#' @param x matrix where relplicates are organized into seprate columns
#' @param grp (character or factor) defining which columns should be grouped (considered as replicates)
#' @return This function returns a matrix of sd values
#' @seealso  \code{\link{rowGrpMeans}}, \code{\link{rowCVs}}, \code{\link{rowSEMs}},\code{\link[stats]{sd}}
#' @examples
#' set.seed(2016); dat1 <- matrix(c(runif(200) +rep(1:10,20)), ncol=10)
#' head(rowGrpSds(dat1, gr=gl(4,3,labels=LETTERS[1:4])[2:11]))
#' @export
rowGrpSds <- function(x, grp){
  ## return Sd for within-row subgroup of data 'x' (matrix or data.frame)
  if(is.null(ncol(x))) stop("Data should be matrix or data.frame with multiple columns !") else {
    if(ncol(x) < 2) stop("Data should be matrix or data.frame with at least 2 columns !")}
  if(length(grp) != ncol(x)) stop("Length of (factor) 'grp' and number of cols in 'x' don't match !")
  .rowGrpSds(x, grp) }

#' row group sd (main)
#'
#' This function calculates sd for matrix with multiple groups of data, ie one sd for each group of data. 
#' 
#' @param x numeric matrix where relplicates are organized into separate columns
#' @param grp (factor) defining which columns should be grouped (considered as replicates)
#' @return This function returns a matrix of sd values per row and group of replicates
#' @seealso \code{\link{rowGrpCV}}, \code{\link{rowCVs}}, \code{\link{arrayCV}}, \code{\link{replPlateCV}}
#' @examples
#' set.seed(2016); dat1 <- matrix(c(runif(200)+rep(1:10,20)),ncol=10)
#' grp1 <- gl(4,3,labels=LETTERS[1:4])[2:11]
#' head(.rowGrpSds(dat1, grp1))
#' @export
.rowGrpSds <- function(x, grp){
  ## return (entire col of) NA if only single col for spec group
  grpM <- unique(naOmit(as.character(grp)))
  out <- matrix(nrow=nrow(x), ncol=length(grpM), dimnames=list(rownames(x),grpM))
  for(i in 1:length(grpM)) {
    useC <- which(grp==unique(naOmit(grp))[i])
    out[,i] <- if(length(useC) >1) rowSds(x[,useC]) else NA }
  out }
   

#' Row Normalize
#'
#' This function was designed for normalizing data that is supposed to be particularly similar, like a collection of technical replicates.
#' Thus, initially for each row an independent normalization factor is calculated and the median or mean across all factors will be finally applied to the data.
#' This function has a special mode of operation with higher content of \code{NA} values (which may pose problems with other normalization approaches). 
#' If the \code{NA}-content is higher than the threshold set in \code{sparseLim},
#' a special procedure for sparse data will be applied (iteratively trating subsets of \code{nCombin} columns that will be combined in a later step).
#'
#' @details
#' Arguments were kept similar with function \code{normalizeThis} as much as possible.
#' In most cases data get normalized by proportional factors. In case of log2-data (very common in omics-data) normalizing by an additive factor is equivalent to a proportional factor.
#'
#' This function has a special mode of operation for sparse data (ie containing a high content of \code{NA} values).
#' 0-values by themselves will be primarily considered as true measurment outcomes and not as missing.
#' However, by using the argument \code{minQuant} all values below a given threshold will be set as \code{NA} and this may possibly trigger the sparse mode of normalizing.
#'
#' Note : Using a small value of \code{nCombin} will give the highest chances of finding sufficient complete combination of columns with sparse data.
#' However, this will also increase (very much) the computational efforts and time required to produce an output.
#'
#' When using default proportional mode a potential division by 0 could occur, when the initial normalization factor turns out as 0.
#' In this case a small value (default the maximum value of \code{dat} / 10 will be added to all data before normalizing.
#' If this also creates 0-vales in the data this factor will be multiplied by 0.03.
#'
#' @param dat matrix or data.frame of data to get normalized
#' @param method (character) may be "mean","median" (plus "NULL","none"); When NULL or 'none' is chosen the input will be returned as is
#' @param refLines (NULL or numeric) allows to consider only specific lines of 'dat' when determining normalization factors (all data will be normalized)
#' @param refGrp (integer) Only the columns indicated will be used as reference, default all columns (integer or colnames)
#' @param proportMode (logical) decide if normalization should be done by multiplicative or additive factor
#' @param minQuant (numeric) optional filter to set all values below given value as \code{NA}
#' @param sparseLim (integer) decide at which min content of  \code{NA} values the function should go in sparse-mode
#' @param nCombin (NULL or integer) used only in sparse-mode (ie if content of \code{NA}s higher than content of \code{sparseLim}): Number of groups of smller matrixes with this number of columns to be inspected initailly;
#'   low values (small groups have higher chances of more common elements)
#' @param omitNonAlignable (logical) allow omitting all columns which can't get aligned due to sparseness
#' @param maxFact (numeric, length=2) max normalization factor
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allows easier tracking of messages produced
#' @return This function returns a matrix of normalized data
#' @seealso \code{\link{normalizeThis}}, \code{\link{exponNormalize}}, \code{\link{adjBy2ptReg}}, \code{\link[vsn]{justvsn}}
#' @examples
#' ## sparse matrix  normalization
#' set.seed(2); AA <- matrix(rbinom(110,10,0.05), nrow=10)
#' AA[,4:5] <- AA[,4:5] *rep(4:3, each=nrow(AA))
#' AA[2,c(2,6,7)] <- 1; AA[3,8] <- 1;
#'
#' (AA1 <- rowNormalize(AA))
#' (AA2 <- rowNormalize(AA, minQuant=1))   # set all 0 as NAs
#' (AA3 <- rowNormalize(AA, refLines=1:6, omitNonAlignable=FALSE, minQuant=1))
#'
#'
#' @export
rowNormalize <- function(dat, method="median", refLines=NULL, refGrp=NULL, proportMode=TRUE, minQuant=NULL, sparseLim=0.4, nCombin=3, omitNonAlignable=FALSE, maxFact=10, silent=FALSE, debug=FALSE, callFrom=NULL) {
  ## integrated to normalizeThis 
  ## method (character) normalization method applied to slection of complete lines from subsets of \code{nCombin} columns, sent to with normalizeThis()
  ##   however, if less than 25 common lines found "average" normalization will be used
  ## proportMode (logical) .. switch between normalizing by multiplicative (proportional) or additive factor
  ## minQuant (numeric) .. intensity threshold; all values below will be considered as missing (ie \code{NA})
  ## sparseLim (numeric) .. the max ratio of content of \code{NA}s in data for regular normalization, if content of \code{NA}s is higher an iterative approch using subsets of content of \code{nCombin} columns will be used
  ## omitNonAlignable (logical) .. decide if columns which can't be 'connected' to the rest (due to missing common measures) will be added without proper normalization or rather omitted
  ## minQuant (numeric) .. intensity threshold; all values below will be considered as missing (ie \code{NA})
  ## maxFact (numeric) .. max normalization factor (values higher will be set to threshold)
  ## nCombin (integer) .. (only if content of \code{NA}s higher than content of \code{sparseLim}) groups of smller matrixes with this number of columns to be inspected initailly; low values (small groups have higher chances of more common elements)
  ##
  ## This function was designed for normalizing rows of sparse matrixes. 'dat' is supposed to represent equivalent (eg replicate measures) of the same series of individuals/elements,
  ##  However, numerous values may be missing ie \code{NA}. Thus, it may occur frequently that a given element has not all measures, ie lines containing some \code{NA} values.
  ## In consequence, chances of finding lines without any \code{NA} are higher when initially fewer columns (argument \code{nCombin}) are normalized first and then combined with other sets of comumns in an iterative way.
  fxNa <- .composeCallName(callFrom, newNa="rowNormalize")
  ## proportional mode : need to add small value to avoid divBy0 !
  if(length(dat) ==0 || any(length(dim(dat)) !=2, dim(dat) < 2:1)) stop("Invalid argument 'dat'; must be matrix or data.frame with min 2 lines and 1 col")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  sep <- "_"                        # separator used internally with column-numbers
  maxFact=10

  if(is.data.frame(dat)) dat <- as.matrix(dat)
  iniColNa <- colnames(dat)
  colnames(dat) <- 1:ncol(dat)

  if(length(refLines) <1) refLines <- 1:nrow(dat) else if(any(refLines > nrow(dat) | refLines < 1)) {
    if(!silent) message(fxNa,"Note: 'refLines' designs (some) lines not present in data, ignoring..")
    refLines <- refLines[which(refLines <= nrow(dat) & refLines >0)] }
  if(length(refGrp) >0) refGrp <- refGrp[which(refGrp %in% 1:ncol(dat))]
  if(length(refGrp) <1) refGrp <- 1:ncol(dat)
  if(length(minQuant)==1 && is.numeric(minQuant)) if(!is.na(minQuant)) {chMin <- dat < minQuant
    if(any(chMin)) dat[which(chMin)] <- NA }
  if(debug) {message(fxNa,"  srn0")}

  refVal <- if(sum(!is.na(dat[refLines, refGrp])) >10) stats::median(dat[refLines, refGrp], na.rm=TRUE) else mean(dat[refLines, refGrp], na.rm=TRUE)
  if(refVal==0) refVal <- 1
  if(debug) {message(fxNa,"  srn0b"); srn0b <- list(dat=dat,method=method,refLines=refLines,refGrp=refGrp,proportMode=proportMode,minQuant=minQuant,sparseLim=sparseLim,nCombin=nCombin,omitNonAlignable=omitNonAlignable, refVal=refVal)}

  ## may be better/faster to random-shuffle column-order of combOfN ? (allow reaching last quicker)
  chNa <- sum(is.na(dat[refLines,])) / length(dat[refLines,])   # ratio of NAs
  if(debug) message(fxNa,"Data contain ",signif(100*chNa, 3),"% NA, using partial/iterative approach for sparse data")
  if(chNa > sparseLim) {
    if(ncol(dat) <= 9 ) {
      combOfN <- utils::combn(ncol(dat), nCombin)              # the combinations (all, ie max 126) ..
      colnames(combOfN) <- apply(combOfN, 2, paste0, collapse="_")
    } else {      ## shortcut for larger data-sets, consecutive sligtly overlapping segments will be considered
      overLap <- 2
      nSeg <- floor(ncol(dat)/(nCombin  -overLap))
      staSto <- cbind(sta=(0:nSeg)*(nCombin -overLap) +1, sto=(1:(nSeg+1))*(nCombin -overLap) +overLap)
      chStaSto <- staSto[,2] > ncol(dat)
      if(any(chStaSto)) staSto[which(chStaSto),] <- rep(ncol(dat) -c((nCombin-1), 0), each=sum(chStaSto))
      combOfN <- apply(staSto, 1, function(x) x[1]:x[2])
      if(debug) {message(fxNa,"  srn0c"); srn0c <- list(dat=dat,method=method,refLines=refLines,refGrp=refGrp,proportMode=proportMode,minQuant=minQuant,sparseLim=sparseLim,nCombin=nCombin,omitNonAlignable=omitNonAlignable, refVal=refVal,combOfN=combOfN)}
      if( TRUE) {  ## large data-sets : complete by few additional cross-references
        suplGrpFac <- round(log2(ncol(dat)))           ##  used for extending consecutive splits by additional non-consecutive sets : split into x subgroups for shuffeling tsart-sites
        stepW <- c(floor(nCombin/2), ceiling(nCombin/2))
        tmp <- suplGrpFac*(1:(floor(ncol(dat)/suplGrpFac)))

        combOfN <- cbind(combOfN, sapply(tmp, function(x) c(1:stepW[1], x:(x +stepW[2] -1))), c(1:stepW[1], ncol(dat) -(stepW[2] -1):0) )
        if(suplGrpFac >2) for(i in 2:suplGrpFac)  combOfN <- cbind(combOfN, sapply(tmp, function(x) c(tmp[i] -1 + 1:stepW[1], x:(x +stepW[2] -1))))
        combOfN <- apply(combOfN, 2, sort)
        combOfN <- cbind(combOfN, ncol(dat) -((nCombin -1):0) )

        ## now filter away replicated/redundant combinations or combinations with intra-repeats (of specific cols) or combiantions suggesting col-numbers out of bound
        chDu <- duplicated(apply(combOfN, 2, function(x) paste0(sort(x), collapse=sep))) | apply(combOfN, 2, function(x) any(duplicated(x))) | apply(combOfN, 2, max) > ncol(dat)
        if(any(chDu)) combOfN <- if(sum(!chDu) >1) combOfN[,-which(chDu)] else as.matrix(combOfN[,-which(chDu)])
      }
      if(debug) {message("srn1"); srn1 <- list(dat=dat,combOfN=combOfN,nSeg=nSeg,sparseLim=sparseLim,refLines=refLines,refGrp=refGrp,refVal=refVal,method=method,proportMode=proportMode,sparseLim=sparseLim,nCombin=nCombin,combOfN=combOfN)}

    }
    if(debug) {message(fxNa,"  srn0c"); srn0c <- list(dat=dat,method=method,refLines=refLines,refGrp=refGrp,proportMode=proportMode,minQuant=minQuant,sparseLim=sparseLim,nCombin=nCombin,omitNonAlignable=omitNonAlignable, refVal=refVal)}
    ## sort dat ??

    comUse <- apply(combOfN[,], 2, function(x) which(rowSums(is.na(dat[refLines,x])) == 0) )   # index of valid/complete lines (no NA) per set of nCombin cols
    if(!is.list(comUse)) comUse <- as.list(as.data.frame(comUse))
    chLe <- sapply(comUse, length)
    if(any(chLe >0, na.rm=TRUE)) {
      ## filter to subsets of complete lines
      combOfN <- combOfN[,which(chLe >0)]
      comUse <- if(is.matrix(comUse)) as.list(as.data.frame(comUse[,which(chLe >0)])) else comUse[which(chLe >0)]
    } else { warning("No complete lines found !! Try reducing 'nCombin' ...")}
    ## check for coverage
    chC <- !(1:ncol(dat) %in% sort(unique(as.integer(combOfN))))
    if(any(chC, na.rm=TRUE)) {
      ## search for add'l columns complementing existing choice
      suplC <- if(sum(chC)==1) as.matrix(.complCols(which(chC), dat, nCombin)) else sapply(which(chC),.complCols, dat, nCombin)
      if(length(suplC) >0) {   ## attach, remove duplicates & update comUse
        combOfN <- cbind(combOfN, suplC)
        chDu <- duplicated(apply(combOfN, 2, function(x) paste0(sort(x), collapse=sep)))
        if(any(chDu)) combOfN <- if(sum(!chDu) >1) combOfN[,-which(chDu)] else as.matrix(combOfN[,-which(chDu)])
        comUse <- apply(combOfN, 2, function(x) which(rowSums(is.na(dat[refLines,x])) == 0) ) }   # index of valid/complete lines (no NA) per set of nCombin cols}
    }

    ##
    if(debug) {message("rn2"); rn2 <- list(dat=dat,combOfN=combOfN,comUse=comUse,nSeg=nSeg,sparseLim=sparseLim,refLines=refLines,refGrp=refGrp,refVal=refVal,method=method,proportMode=proportMode,sparseLim=sparseLim,nCombin=nCombin)}
    fact <- .rowNormFact(dat=dat, combOfN=combOfN,comUse=comUse, method=method, refLi=refLines, refGrp=refGrp, proportMode=proportMode, minQuant=minQuant, maxFact=10, omitNonAlignable=omitNonAlignable, silent=silent, debug=debug, callFrom=fxNa)
    if(debug) {message("rn3")}

    ##  final rendering
    if(debug) {message(fxNa," final rendering  srnG")}
    out <- dat / rep(fact, each=nrow(dat))
  } else {
    if(debug) { message(fxNa,"This data/selection contains only some/few or no NAs, no need for iterative sparse alignment .. srn9a") }
    ch1 <- colSums(!is.na(dat[refLines,]))      # nNA per columns
    ## proportional mode : .rowNorm() adds small value to avoid divBy0 !
    if(any(ch1 ==0) && length(refLines) < nrow(dat) && omitNonAlignable && !silent) message(fxNa,"Note : ",sum(ch1 ==0)," columns are all NA when using ",length(refLines)," refLines, resuting columns will be retured as NaN")
    out <- .rowNorm(dat, refLines, method, proportMode)
    chNaN <- is.nan(out)
    if(any(chNaN)) if(!omitNonAlignable) {
      out[which(chNaN)] <- dat[which(chNaN)]
      if(!silent) message(fxNa,"Note : ",sum(ch1 ==0)," columns are all NA when using ",length(refLines)," refLines, resulting columns are retured as WITHOUT ANY normalization")
    } else if(!silent) message(fxNa,"Note : ",sum(ch1 ==0)," columns are all NA when using ",length(refLines)," refLines, resulting columns are retured as NaN")
  }
  if(length(iniColNa) >0) colnames(out) <- iniColNa
  out }


#' Search (complementing) Columns For Best Coverage Of Non-NA Data For rowNormalization (main)
#'
#' This function was designed to complete the selection of columns of sparse matrix 'dat' with sets of 'nCombin' columns at complete 'coverage'
#' Context : In sparse matrix 'dat' search subsets of columns with some rows as complete (no NA).
#' 
#' @param x (integer, length=1) column number for with other columns to combine & give (some) complete non-NA lines are seeked
#' @param dat (matrix) .. init data, smay be parse matrix with numerous NA
#' @param nCombin (integer) .. number of columns used to make complete subset
#' @return This function returns a matrix of column-indexes complementing (nCombin rows)
#' @seealso \code{\link{rowNormalize}}; \code{\link{normalizeThis}}
#' @examples
#' .complCols(3, dat=matrix(c(NA,12:17,NA,19),ncol=3), nCombin=3)
#' @export
.complCols <- function(x, dat, nCombin) {
  ## search (complementing) columns for best coverage of non-NA data for rowNormalization
  ## Context : In sparse matrix 'dat' search subsets of columns with some rows as complete (no NA).
  ## This function was designed to complete the selection of columns of sparse matrix 'dat' with sets of 'nCombin' columns at complete 'coverage'
  ## x (integer) .. column number for with other columns to combine & give (some) complete non-NA lines are seeked
  ## dat (matrix) .. init data, smay be parse matrix with numerous NA
  ## nCombin(integer) .. number of columns used to make complete subset
  ## returns matrix of column-indexes complementing (nCombin rows)
  if(any(length(dim(dat)) !=2, dim(dat) < 1:2)) stop("Invalid argument 'dat'; must be matrix or data.frame with min 1 line and 2 cols")
  if(is.null(colnames(dat))) colnames(dat) <- 1:ncol(dat)   # colnames are essential !
  msg <- "x must be integer to design column number of 'dat'"
  if(length(x) <0) stop(msg) else {
    if(is.character(x)) x <- which(colnames(x) %in% x)
    if(length(x) >1) x <- x[1]
    if(x > ncol(dat) || x < 1) stop(msg) }
  ## main
  z3 <- !is.na(dat[which(!is.na(dat[,x])), -x])           # remaining cols: !is.na of extract of lines to consider (based on incomplete col)
  z4 <- if(is.matrix(z3)) apply(z3, 1, which) else which(z3)
  if(!is.list(z4)) z4 <- as.list(as.data.frame(z4))
  chLe <- sapply(z4, length)
  chLe <- chLe > nCombin -2
  if(any(!chLe)) z4 <- z4[which(chLe)]
  if(length(z4) >0) { if(length(z4) >1) {
    c(as.integer(names(sort(table(unlist(z4)), decreasing=TRUE))) [1:(nCombin -1)], x)
    } else c(z4[[1]][1:(nCombin -1)], x)} else NULL }


#' Obtain Normalization Factor (main)
#'
#' This function was designed to obtain normalization factors.
#' 
#' @param dat (matrix) .. init data, smay be parse matrix with numerous NA
#' @param combOfN (matrix) ..  # matrix of index for all sub-groups (assumed as sorted)
#' @param comUse (list) .. index of complete lines for each col of combOfN
#' @param method (character) may be "mean","median" (plus "NULL","none"); When NULL or 'none' is chosen the input will be returned as is
#' @param refLi (NULL or numeric) allows to consider only specific lines of 'dat' when determining normalization factors (all data will be normalized)
#' @param refGrp (integer) Only the columns indicated will be used as reference, default all columns (integer or colnames)
#' @param proportMode (logical) decide if normalization should be done by multiplicative or additive factor
#' @param minQuant (numeric) optional filter to set all values below given value as \code{NA}
#' @param maxFact (numeric, length=2) max normalization factor
#' @param omitNonAlignable (logical) allow omitting all columns which can't get aligned due to sparseness
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) This function allows easier tracking of messages produced
#' @return This function returns a matrix of column-indexes complementing (nCombin rows)
#' @seealso \code{\link{rowNormalize}}; \code{\link{normalizeThis}}
#' @examples
#' ma1 <- matrix(11:41, ncol=3)
#' @export
.rowNormFact <- function(dat, combOfN, comUse, method="median", refLi=NULL, refGrp=NULL, proportMode=TRUE, minQuant=NULL, maxFact=10, omitNonAlignable=FALSE, silent=FALSE, debug=FALSE, callFrom=NULL) {
  ## obtain normalization factor
  ## combOfN (matrix) ..  # matrix of index for all sub-groups (assumed as sorted)
  ## comUse (list) .. index of complete lines for each col of combOfN
  fxNa <- .composeCallName(callFrom, newNa=".rowNormFact")
  outL <- list()
  outFa <- matrix(nrow=ncol(combOfN), ncol=ncol(dat), dimnames=list(1:ncol(combOfN), 1:ncol(dat)))
  conti <- list()
  if(debug) message(callFrom, "..start loop") 
  for(i in 1:ncol(combOfN)) {
    if(length(comUse[[i]]) >0) {                        # has sufficent valid data .. check with refLi
      refLiX <- if(length(refLi) >0) comUse[[i]][which(comUse[[i]] %in% refLi)] else comUse[[i]]
      if(length(refLiX) <1) {comUse[[i]] <- NA; if(debug) message(fxNa,"No valid lines remain when considering 'refLi' (of columns ",pasteC(utils::head(combOfN[,i])),"), loop no ",i," !!")}
    }
    if(length(comUse[[i]]) >0 && sum(is.na(comUse[[i]]))==0) {                        # has sufficent valid data ..
      tm2 <- if(length(refLiX ==1)) dat[refLiX,combOfN[,i]]/mean(dat[refLiX,combOfN[,i]], na.rm=TRUE) else .rowNorm(dat[,combOfN[,i]], method=if(length(comUse[[i]]) < 25) "mean" else method, refLi=refLiX, proportMode=proportMode, retFact=TRUE, callFrom=fxNa, silent=silent)
      if(debug && length(dim(tm2)) >1) message(fxNa,"Strange, tm2 is matrix and NOT vector !!")
      outFa[i, combOfN[,i]] <- if(length(dim(tm2)) >1) tm2[1,] else tm2
      ## construct 'contigs'
      if(length(conti) <1) { conti <- list(conti1=matrix(combOfN[,i], nrow=1, dimnames=list(paste0("li",i),NULL)))   # initiate 1st
      } else {
        tm4 <- unlist(lapply(conti, function(x) sum(combOfN[,i] %in% x)))          # how many hits in which contig
        if(any(tm4 >0)) { if(sum(tm4 >0) ==1) { conti[[which(tm4 >0)]] <- rbind(conti[[which(tm4 >0)]], combOfN[,i])  # (simplest case) add to (single) existing contig
            rownames(conti[[which(tm4 >0)]])[nrow(conti[[which(tm4 >0)]])] <- paste0("li",i)
          } else { maxC <- which.max(tm4)                                      # first align to contig with most common elements
            conti[[maxC]] <- rbind(conti[[maxC]], combOfN[,i])                 # history
            rownames(conti[[maxC]])[nrow(conti[[maxC]])] <- paste0("li",i)
            if(debug) message(fxNa,"Adding contig no ",maxC," to main chunk")
            supC <- tm4[-maxC]
            conti[[maxC]] <- rbind(conti[[maxC]], conti[[supC]][nrow(conti[[supC]]):1,])        # add reversed other contig
            conti <- conti[-supC]
          }
        } else { if(debug) message(fxNa,"  i=",i,"  starting new contig..")
          conti[[length(conti) +1]] <- matrix(combOfN[,i], nrow=1, dimnames=list(paste0("li",i),NULL))
          names(conti)[length(conti)] <- paste0("conti",length(conti)) }  # add/start new contig
      }
    }
    if(debug) {message(fxNa," ..end loop",i," out of ",ncol(combOfN)); rnf2b <- list(dat=dat,combOfN=combOfN,comUse=comUse,method=method,outFa=outFa,conti=conti,i=i)}
  }
  if(debug) {message(fxNa," start 2nd part  rnf3")}
  ## now need to adjust factors row by row (if any overlap)
  if(length(conti) >1) conti <- conti[order(sapply(conti, nrow), decreasing=TRUE)]
  ## assume sorted conti, ie 1st element of conti has most data
  liNo <- as.integer(sub("^li","", rownames(conti[[1]])))            # needed for outFa
  if(ncol(combOfN) >1) for(i in 2:nrow(conti[[1]])) {
    ## loop over all lines of contig (after 2nd), ie over all combinations of subsets of columns, to adjst normalization factor
    if(i ==2) {                # 2nd line, ie only single preceeding line to adjust to
      ovLa <- conti[[1]][i, which(conti[[1]][i,] %in% conti[[1]][i -1,])]
      refPo <- conti[[1]][i -1, which(conti[[1]][i -1,] %in% conti[[1]][i,])]      # needed ??
      tm2 <- (outFa[liNo[i],][ovLa]) / (outFa[liNo[i -1],][ovLa])
      if(length(tm2) >1) tm2 <- if("median" %in% method & length(tm2) > 5) stats::median(tm2, na.rm=TRUE) else mean(tm2, na.rm=TRUE)
      if(debug) message(fxNa," outFa[1,]", pasteC(round(outFa[1,],2)),"   add li 2.. tm2 ",pasteC(round(tm2,2)),"   correct li2 to  ", pasteC(round(outFa[i,] / tm2,2)))
      outFa[i,] <- outFa[i,] / tm2
    } else {
      if(debug) message(fxNa," outFa  li=",i-2,"  ",pasteC(round(outFa[i-2,],2)),"\n  li=",i-1,"  ",pasteC(round(outFa[i-1,],2)),"\n  li=",i,"  ",pasteC(round(outFa[i,],2)))
      ovLa <- apply(conti[[1]][1:(i-1),], 1, function(x) x %in% conti[[1]][i,])      # overlap of prev contigs to current i
      refPo <- apply(conti[[1]][1:(i-1),], 1, function(x) conti[[1]][i,] %in% x)     # overlap of current i to prev contigs
      tm2 <- list()
      for(j in 1:ncol(ovLa)) if(any(ovLa[,j], na.rm=TRUE)) {
        tm2[[j]] <- outFa[liNo[i], conti[[1]][i,][refPo[,j]]] / outFa[liNo[i], conti[[1]][i,][which(ovLa[,j])]]
        if(is.null(names(tm2[[j]]))) names(tm2[[j]]) <- conti[[1]][j,][which(ovLa[,j])]
      }
      tm2 <- tm2[sapply(tm2, length) >0]
      fact <- tapply(unlist(tm2), names(unlist(tm2)), function(x)  if(length(x) > 5 & "median" %in% method) stats::median(x) else mean(x))    # summarize over all lines matching up to i (keep factors for cols separate)
      fact <- if(length(fact) > 5 && "median" %in% method) stats::median(fact) else mean(fact)
      chFa <- rbind(fact > maxFact , fact < 1/maxFact)
      if(any(chFa, na.rm=TRUE)) {if(any(chFa[1,])) fact[which(chFa[1,])] <- maxFact; if(any(chFa[2,])) fact[which(chFa[2,])] <- 1/maxFact }
      outFa[liNo[i],] <- outFa[liNo[i],] / fact
      if(debug) { message(fxNa,"Done with loop ",i," rnf3b")}
    }
  }
  ## (future ?) fuse multiple contigs (if necessary)
  ## compress to single norm-factor per column
  finFa <- apply(outFa, 2, function(x) if(sum(!is.na(x)) > 5 && "median" %in% method) stats::median(x, na.rm=TRUE) else mean(x, na.rm=TRUE))
  if(debug) message("rnf4"); rnf4 <- list(finFa=finFa,dat=dat,combOfN=combOfN,comUse=comUse,method=method,outFa=outFa,conti=conti,ovLa=ovLa,refPo=refPo,i=i,outFa=outFa,tm2=tm2,fact=fact,maxFact=maxFact)
  if(length(conti) >1) {
    chCo <- unlist(conti[-1])
    ch2 <- !chCo %in% conti[[1]]
    if(any(ch2)) {
      if(omitNonAlignable) finFa[chCo[which(ch2)]] <- NA
      if(!silent) message(fxNa,"Note: ",sum(ch2)," columns (no ",if(sum(ch2) <7) pasteC(chCo[which(ch2)]) else paste0(paste0(utils::head(chCo[which(ch2)]), collapse=", ")," ..."),
        ") can't get aligned !", if(omitNonAlignable)" Setting respective norm-factors to NA") }
  }
  finFa }


#' Row-normalization Procedure On Matrix Or Data.frame (main)
#'
#' This function performs a row-normalization procedure on matrix or data.frame 'dat'
#' 
#' @param dat (matrix) .. init data, smay be parse matrix with numerous NA
#' @param refLi (NULL or numeric) allows to consider only specific lines of 'dat' when determining normalization factors (all data will be normalized)
#' @param method (character) may be "mean","median" (plus "NULL","none"); When NULL or 'none' is chosen the input will be returned as is
#' @param proportMode (logical) decide if normalization should be done by multiplicative or additive factor
#' @param maxFact (numeric, length=2) max normalization factor
#' @param fact0val (integer) 
#' @param retFact (logical) 
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) This function allows easier tracking of messages produced
#' @return This function returns a matrix of normalized data same dimensions as 'dat'
#' @seealso \code{\link{rowNormalize}}; \code{\link{normalizeThis}} 
#' @examples
#' .rowNorm(matrix(11:31, ncol=3), refLi=1, method="mean", proportMode=TRUE)
#' @export
.rowNorm <- function(dat, refLi, method, proportMode, maxFact=10, fact0val=10, retFact=FALSE, debug=FALSE, silent=FALSE, callFrom=NULL) {
  ## row-normalization procedure on matrix or data.frame 'dat'
  ## return matrix of same dimensions as 'dat'
  ## note : problem with values=0 in proportional mode (div/0) => replace 0 values by maxAbsVal/fact0val (proportMode only, nor replacement needed in additive mode)
  ## maxFact .. max normalization factor
  ## should be resistant to low degree of NAs
  suplVa <- NULL
  if(!"additive" %in% proportMode && any(dat==0, na.rm=TRUE)) {suplVa <- max(abs(dat), na.rm=TRUE)/fact0val; while(any((-1*suplVa) %in% dat, na.rm=TRUE)) suplVa <- suplVa*0.02; dat <- dat + suplVa } # substitute 0 by value 10000x smaller than smallest pos value
  if(length(dim(dat)) <2) dat <- matrix(dat, nrow=1, dimnames=list(NULL, names(dat)))
  rowMe <- if("median" %in% method && length(refLi) >10) {             # per row mean or median (for each line of refLi)
    if(length(refLi) >1) apply(dat[refLi,], 1, stats::median, na.rm=TRUE) else stats::median(dat[refLi,], na.rm=TRUE)
  } else {
    if(length(refLi) >1) rowMeans(dat[refLi,], na.rm=TRUE) else mean(dat[refLi,], na.rm=TRUE)}
  corFa <- if(!"additive" %in% proportMode) dat[refLi,]/ rep(rowMe, ncol(dat)) else dat[refLi,] -rep(rowMe, ncol(dat))  # cor-factors
  if(length(dim(corFa)) >1) corFa <- if("median" %in% method && length(refLi) >10) apply(corFa, 2, stats::median, na.rm=TRUE) else colMeans(corFa, na.rm=TRUE)  # per column cor-factor
  if(retFact) corFa else { corFa <- rep(corFa, each=nrow(dat))
    if(!"additive" %in% proportMode) dat / corFa else dat - corFa }
  }
  

#' Count Same Start- And End- Sites Of Edges (Or Fragments)
#'
#' Suppose a parent sequence/string 'ABCDE' gets cut in various fragments (eg 'ABC','AB' ...).   
#' \code{countSameStartEnd} counts how many (ie re-occuring) start- and end- sites of edges do occur in the input-data. 
#' The input is presented as matrix of/indicating start- and end-sites of edges.
#' The function is used to characterize partially redundant edges and accumulation of cutting/breakage sites.
#' @param frag (matrix) 1st column \code{beg} start-sites, 2nd column \code{end} end-sites of edges, rownames to precise fragment identities are recommended  
#' @param minFreq (integer) min number of accumulated sites for taking into account (allows filtering with large datasets)  
#' @param nDig (integer) rounding: number of digits for columns \code{beg.rat} and \code{end.rat} in output
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allows easier tracking of messages produced#' @return This function returns a matrix of 6 columns: input (beg and end), beg.n, beg.rat, end.n, end.rat
#' @seealso to build initial tree \code{\link{buildTree}}, \code{\link{contribToContigPerFrag}}, \code{\link{simpleFragFig}}
#' @examples
#' frag1 <- cbind(beg=c(2,3,7,13,13,15,7,9,7, 3,3,5), end=c(6,12,8,18,20,20,19,12,12, 4,5,7))
#' rownames(frag1) <- letters[1:nrow(frag1)]
#' countSameStartEnd(frag1)
#' simpleFragFig(frag1)
#' @export
countSameStartEnd <- function(frag, minFreq=2, nDig=4, silent=FALSE, debug=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom, newNa="countSameStartEnd")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE

  if(!is.matrix(frag)) frag <- as.matrix(frag)
  if(is.null(rownames(frag))) rownames(frag) <- paste0("li",1:nrow(frag)) else {
    frag <- frag[sort.list(rownames(frag)),,drop=FALSE]}
  chTab <- function(tab) {chT <- tab > minFreq-1; if(any(chT)) tab[which(chT)] else NULL}
  headT <- chTab(table(frag[,1]))
  tailT <- chTab(table(frag[,2]))
  out <- matrix(nrow=nrow(frag), ncol=2)
  if(length(headT) >0) { tmp <- which(frag[,1] %in% names(headT)); out[tmp,1] <- as.integer(as.character(factor(frag[tmp,1], labels=headT)))}
  if(length(tailT) >0) { tmp <- which(frag[,2] %in% names(tailT)); out[tmp,2] <- as.integer(as.character(factor(frag[tmp,2], labels=tailT)))}
  cbind(frag, beg.n=out[,1], beg.rat=signif(out[,1]/nrow(frag), digits=nDig), end.n=out[,2], end.rat=signif(out[,2]/nrow(frag), digits=nDig))
}
  

#' Standard Error Of Median For Each Column By Bootstrap
#'
#' Determine standard error (sd) of median by bootstraping for multiple sets of data (rows in input matrix 'dat').
#' Note: The package \href{https://CRAN.R-project.org/package=boot}{boot} must be installed from CRAN.
#' @param dat (numeric) matix 
#' @param nBoot  (integer, length=1) number if iterations
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a (numeric) vector with estimated standard errors 
#' @seealso \code{\link[boot]{boot}}
#' @examples
#' set.seed(2016); dat1 <- matrix(c(runif(200) +rep(1:10,20)), ncol=10)
#' colMedSds(dat1) 
#' @export
colMedSds <- function(dat, nBoot=99, silent=FALSE, debug=FALSE, callFrom=NULL) {
  fxNa <- .composeCallName(callFrom, newNa="colMedSds")
  if(isTRUE(debug)) silent <- FALSE else { debug <- FALSE
    if(!isTRUE(silent)) silent <- FALSE }
  out <- NULL
  if(length(dat) <1) { if(!silent) message(fxNa,"Argument 'dat' seems to be empty")
  } else {
    if(requireNamespace("boot", quietly=TRUE)) {
      median.fun <- function(dat,indices) stats::median(dat[indices], na.rm=TRUE)      
      chDi <- dim(dat)
      if(is.numeric(dat) && length(chDi) <1) dat <- list(dat)
      if(debug) {message(fxNa,"cMS2"); cMS2 <- list() }
      if(length(chDi) >1) {
        out <- if(chDi[1] >1) try(apply(dat, 2, function(x) stats::sd(boot::boot(data=x, statistic=median.fun, R=nBoot)$t)), silent=TRUE) else rep(NA, chDi[2])
      } else {
        if(is.list(dat)) { 
          out <- sapply(dat, function(x, nB) try(stats::sd(boot::boot(data=x, statistic=median.fun, R=nB)$t), silent=TRUE), nB=nBoot) 
          chO <- sapply(out, inherits, "try-error")
          if(all(chO)) out <- NULL else if(any(chO)) out[which(chO)] <- NA    
        }
      }
      if(inherits(out, "try-error")) { warning(fxNa,"Unable to run boot::boot()"); out <- NULL } 
    } else { out - NULL
      if(!silent) message(fxNa,"NOTE: Package 'boot' not found (please install first from CRAN), returning NULL")
    }
  }  
  out }


 

#' Transform (factor) levels into index
#'
#' This function helps transforming a numeric or character vector into indexes of levels (of its original values).
#' By default indexes are assigned by order of occurance, ie, the first value of \code{x} will be get the index of 1.
#' Using the argument \code{byOccurance=FALSE} the resultant indexes will follow the sorted values.
#'
#' @param dat (numeric or character vector or factor) main input 
#' @param byOccurance (logical) toogle if lowest index should be based on alphabetical order or on order of input
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return matrix with mean values
#' @seealso \code{\link{rowSds}}, \code{\link[base]{colSums}}
#' @examples
#' x1 <- letters[rep(c(5,2:3),1:3)]
#' levIndex(x1)
#' levIndex(x1, byOccurance=FALSE)
#' ## with factor 
#' fa1 <- factor(letters[rep(c(5,2:3),1:3)], levels=letters[1:6])
#' levIndex(fa1)
#' levIndex(fa1, byOccurance=FALSE)
#' @export
levIndex <- function(dat, byOccurance=TRUE, silent=FALSE, debug=FALSE, callFrom=NULL) {
  ## transform levels into index
  ## based on https://stackoverflow.com/questions/50898623/how-to-replace-multiple-values-at-once
  fxNa <- .composeCallName(callFrom, newNa="levIndex")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE

  if(length(dat) <1) { out <- NULL; if(!silent) message(fxNa, "'dat' seems to be empty")
  } else {
    out <- as.integer(as.factor(dat))
    names(out) <- dat
    if(isTRUE(byOccurance)) {
      levU <- naOmit(unique(out))           # levels in orig order (non-alpahbetical)    
      corsp <- data.frame(old=levU, new=1:length(levU))
      out[out %in% levU] <- (1:length(levU))[match(out, levU, nomatch = 0)] 
      names(out) <- dat } }
  out }  
  

#' Create factor-like column regrouping data regrouping simultaneaously by two factors
#'
#' This function aims to address the situation when two somehow different groupins (of the same data) exist and need to be joined.
#' It is not necessary that both alternative groupings use the same labels, neither.
#' \code{combineByEitherFactor} adds new (last) column named 'grp' to input matrix representing the combined factor 
#' relative to 2 specified columns from input matrix  \code{mat} (via 'refC1','refC2'). Optionally, the output may be 
#' sorted and a column giving n per factor-level may be added. 
#' The function treats selected columns of  \code{mat} 
#' as pairwise combination of 2 elements (that may occur multiple times over all lines of  \code{mat})
#' and sorts/organizes all instances of such combined elements (ie from both selected columns) as repeats of a given group,
#' who's class number is given in output column 'grp', the (total) number of repeats may be displayed in column 'nGrp' ( \code{nByGrp=TRUE}).
#' If groups are overlapping (after re-ordering), an iterative process of max 3x2 passes will be launched after initial matching.
#' Works on numeric as well as character input.  
#'
#' @param mat main input matrix
#' @param refC1 (integer) column-number of 'mat' to use as 1st set
#' @param refC2 (integer) column-number of 'mat' to use as 2nd set
#' @param nByGrp (logical) add last col with n by group
#' @param convergeMax (logical) if \code{TRUE}, run 2 add'l iteartive steps to search convergence to stable result
#' @param silent (logical) suppress messages
#' @param debug (logical) display additional messages for debugging
#' @param callFrom (character) allow easier tracking of message(s) produced
#' @return This function returns a matrix containing both selected columns plus additional column(s) indicating group-number of the pair-wise combination 
#' (and optional the total n by group)
#' @examples
#' nn <- rep(c("a","e","b","c","d","g","f"),c(3,1,2,2,1,2,1))
#' qq <- rep(c("m","n","p","o","q"),c(2,1,1,4,4))
#' nq <- cbind(nn,qq)[c(4,2,9,11,6,10,7,3,5,1,12,8),]
#' combineByEitherFactor(nq,1,2,nBy=TRUE); combineByEitherFactor(nq,1,2,nBy=FALSE)
#' combineByEitherFactor(nq,1,2,conv=FALSE); combineByEitherFactor(nq,1,2,conv=TRUE)
#' ##
#' mm <- rep(c("a","b","c","d","e"),c(3,4,2,3,1)); pp <- rep(c("m","n","o","p","q"),c(2,2,2,2,5))
#' combineByEitherFactor(cbind(mm,pp), 1, 2, con=FALSE, nBy=TRUE)
#' combineByEitherFactor(cbind(mm,pp), 1, 2, con=TRUE, nBy=TRUE) 
#' @export
combineByEitherFactor <- function(mat, refC1, refC2, nByGrp=FALSE, convergeMax=TRUE, callFrom=NULL, debug=FALSE, silent=FALSE) {
  fxNa <- .composeCallName(callFrom,newNa="combineByEitherFactor")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  if(length(mat) <1) stop(fxNa," nothing found in 'mat'")
  if(length(dim(mat)) !=2) stop("'mat' must be matrix with min 2 columns")
  msg <- "Arguments 'refC1' & 'refC1' should be of length=1 (truncating)"
  if(length(refC1) >1) {if(!silent) message(fxNa,msg); refC1 <- refC1[1]}
  if(length(refC2) >1) {if(!silent) message(fxNa,msg); refC2 <- refC1[2]}
  if(nrow(mat) <2) return(cbind(mat, grp=1, nGrp=if(nByGrp) 1 else NULL))
  mat <- cbind(mat, iniOrder=1:nrow(mat))              # add col initial order
  mat <- mat[order(mat[,refC1], mat[,refC2]),]         # reorder by 1st fact & 2nd fact
  ## express 1st index as 2nd :
  mat <- cbind(mat, grp=unlist(tapply(mat[,refC1],mat[,refC2], function(x) rep(x[1],length(x))))[rank(mat[,refC2],ties.method="first")]) 
  sup1 <- unlist(tapply(mat[,ncol(mat)],mat[,refC1], function(x) rep(x[1],length(x))))[order(mat[,refC1])]
  msg <- c(" did not reach convergence at ","2nd","4th"," pass, continuing ..","6th pass (stop iterating)"," pass")
  if(convergeMax & !all(sup1==mat[,ncol(mat)])) {
    if(!silent && nrow(mat) >999) message(fxNa,msg[c(1,2,4)])
    sup2 <- unlist(tapply(sup1,mat[,refC2], function(x) rep(x[1],length(x))))[order(mat[,refC2])]  
    sup3 <- unlist(tapply(sup2,mat[,refC1], function(x) rep(x[1],length(x))))[order(mat[,refC1])]
    if(!all(sup3==sup2) & !silent) { message(fxNa,msg[c(1,2,4)])
      sup2 <- unlist(tapply(sup3,mat[,refC2], function(x) rep(x[1],length(x))))[order(mat[,refC2])]
      sup3 <- unlist(tapply(sup2,mat[,refC1], function(x) rep(x[1],length(x))))[order(mat[,refC1])]
      if(!all(sup3==sup2) && !silent) message(fxNa,msg[c(1,5)]) }
    mat[,ncol(mat)] <- sup3 } else if(!silent && !convergeMax) message(fxNa,msg[c(1,2,6)])
  mat[,ncol(mat)] <- as.numeric(as.factor(mat[,ncol(mat)]))                   # reset grp names
  if(nByGrp) { mTa <- table(mat[,ncol(mat)])
    mat <- cbind(mat,nGrp=mTa[match(mat[,ncol(mat)],names(mTa))]) }
  mat[as.numeric(mat[,ncol(mat) -1 -nByGrp]), -ncol(mat) +1 +nByGrp] }
   

#' rowCVs
#'
#' This function returns CV for values in each column (using speed optimized standard deviation).
#' Note : NaN values get replaced by NA.
#' @param dat (numeric) matix 
#' @param autoconvert (NULL or character) allows converting simple vectors in matrix of 1 row (autoconvert="row")
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allows easier tracking of messages produced
#' @return This function returns a (numeric) vector with CVs for each column of 'dat'
#' @seealso  \code{\link[base]{rowSums}}, \code{\link{rowCVs}}, \code{\link{rowGrpCV}}, \code{\link{colSds}}
#' @examples
#' set.seed(2016); dat1 <- matrix(c(runif(200) +rep(1:10,20)),ncol=10)
#' head(colCVs(dat1))
#' @export
colCVs <- function(dat, autoconvert=NULL, silent=FALSE, debug=FALSE, callFrom=NULL) {
  fxNa <- .composeCallName(callFrom, newNa="colCVs")
  msg <- "Data should be matrix or data.frame with at least 2 rows !"
  if(is.data.frame(dat)) dat <- as.matrix(dat)
  if(is.null(ncol(dat))) {
    if(identical(autoconvert,"col")) dat <- matrix(dat, ncol=1)}
  if(is.null(nrow(dat))) stop(msg) else if(nrow(dat) < 2) stop(msg)
  out <- colSds(dat)/base::colMeans(dat, na.rm=TRUE)
  out[which(is.nan(out))] <- NA  
  out }
    

#' Test Multiple Starting Levels For Linear Regression Model, Select Best And Plot   
#'
#' The aim of this function is to select the data suiting set of levels on the main input data to construct a linear regression model. 
#' 
#' @details
#' In real world measurements one may be confronted to the case of very low level analytes below the detection limit (LOD) and resulting read-outs fluctuate around around a common baseline (instead of \code{NA}). 
#' With such data it may be preferable to omit the read-outs for the lowest concentrations/levels of analytes if they are spread around a base-line value.
#' This function allows trying to omit all starting levels designed in \code{startLev}, then the resulting p-values for the linear regression slopes will be checked and the best p-value chosen. 
#' The input may also be a MArrayLM-type object from package \href{https://bioconductor.org/packages/release/bioc/html/limma.html}{limma} or from \code{\link{moderTestXgrp}} or \code{\link{moderTest2grp}}.
#' In the graphical representation all points assocoated to levels omitted are shown in light green.
#' For the graphical display additional information can be used : If the  \code{dat} is list or MArrayLM-type object, 
#'  the list-elements $raw (according to argument \code{lisNa} will be used to display points initially given as NA ad imputed lateron in grey.
#' Logarithmic (ie log-linear) data can be treated by settting argument \code{logExpect=TRUE}. Then the levels will be taken as exponent of 2 for the regression, while the original values will be displayed in the figure. 
#'
#' @param rowNa (character or numeric, length=1) if \code{character} rowname for line to be extracted from \code{dat}, if  \code{numeric} used as line-number of \code{dat}
#' @param dat (matrix, list or MArrayLM-object from limma) main input of which columns should get re-ordered, may be output from \code{\link{moderTestXgrp}} or \code{\link{moderTest2grp}}.
#' @param expect (numeric of character) the expected levels; if character, constant unit-characters will be stripped away to extact the numeric content
#' @param logExpect (logical) toggle to \code{TRUE} if the main data are logarithmic but \code{expect} is linear
#' @param startLev (integer) specify all starting levels to test for omitting here (multiple start sites for modelling linear regression may be specified to finally pick the best model)
#' @param lisNa (character) in case \code{dat} is list or MArrayLM-type object, the list-elements with these names will be used as $raw (for indicating initial \code{NA}-values,
#'   $datImp (the main quantitation data to use) and $annot for displaying the corresponding value from the "Accession"-column.	
#' @param plotGraph (logical) display figure
#' @param tit (character) optional custom title
#' @param pch (integer) symbols to use n optional plot; 1st for regular values, 2nd for values not used in regression
#' @param cexLeg (numeric) size of text in legend
#' @param cexSub (numeric) text-size for line (as subtitle) giving regression details of best linear model)
#' @param xLab (character) custom x-axis label
#' @param yLab (character) custom y-axis label
#' @param cexXAxis (character) \code{cex}-type for size of text for x-axis labels
#' @param cexYAxis (character) \code{cex}-type for size of text for y-axis labels
#' @param xLabLas (integer) \code{las}-type orientation of x-axis labels (set to 2 for vertical axix-labels)
#' @param cexLab (numeric) \code{cex}-type for size of text in x & y axis labels (will be passed to \code{cex.lab} in \code{plot()})
#' @param suppressWarnings (logical) suppress warnings from lm() that may appear when NAs get introduced
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a list with $coef (coefficients), $name (as/from input \code{rowNa}), $startLev the best starting level) 
#' @seealso \code{\link{moderTestXgrp}} for single comparisons, \code{\link[base]{order}}  
#' @examples
#' ## Construct data
#' li1 <- rep(c(4,3,3:6),each=3) + round(runif(18)/5,2)
#' names(li1) <- paste0(rep(letters[1:5], each=3), rep(1:3,6))
#' li2 <- rep(c(6,3:7), each=3) + round(runif(18)/5, 2)
#' dat2 <- rbind(P1=li1, P2=li2)
#' exp2 <- rep(c(11:16), each=3)
#' 
#' ## Check & plot for linear model 
#' linModelSelect("P2", dat2, expect=exp2)
#'
#' ## Log-Linear data
#' ## Suppose dat2 is result of measures in log2, but exp4 is not
#' exp4 <- rep(c(3,10,30,100,300,1000), each=3)
#' linModelSelect("P2", dat2, expect=exp4, logE=FALSE)    # bad
#' linModelSelect("P2", dat2, expect=exp4, logE=TRUE)
#' 
#' @export
linModelSelect <- function(rowNa, dat, expect, logExpect=FALSE, startLev=NULL, lisNa=c(raw="raw",annot="annot",datImp="datImp"), plotGraph=TRUE, 
  tit=NULL, pch=c(1,3), cexLeg=0.95, cexSub=0.85, xLab=NULL, yLab=NULL, cexXAxis=0.85, cexYAxis=0.9, xLabLas=1, cexLab=1.1, suppressWarnings=TRUE, silent=FALSE, debug=FALSE, callFrom=NULL)  {
  ##  test for linear models with option to start form multiple later levels (ie omitting some of the early levels)
  fxNa <- .composeCallName(callFrom, newNa="linModelSelect")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) {silent <- FALSE; suppressWarnings <- FALSE} else debug <- FALSE

  argNa <- c(deparse(substitute(rowNa)), deparse(substitute(dat)), deparse(substitute(expect)))
  quantCol <- c("blue","tan2","grey")       # figure: 1st for 'used in regression', 2nd for 'not-used', 3rd for 'NA/imputed'
  legLab <- c("used in regresssion","not used","NA/imputed,used","NA/imputed,not used")  # for figure legend ..
  annoColNa <- c("Accession","GeneName")
  doSelect <- TRUE                    # flag to see if input/data are valid
  if(length(rowNa) <1) {warning(fxNa,"Argument 'rowNa=",argNa[1],"' contains emty object, nothing to do !"); doSelect <- FALSE}
  if(length(dat) <1) {warning(fxNa,"Argument 'dat=",argNa[1],"' contains emty object, nothing to do !"); doSelect <- FALSE}
  if(length(expect) <1) {stop(fxNa,"Argument 'expect=",argNa[3],"' contains emty object, nothing to do !"); doSelect <- FALSE
  } else if(is.character(expect)) {
    exp2 <- try(as.numeric(expect), silent=TRUE)
    if(inherits(exp2, "try-error")) {
      exp2 <- try(as.numeric(gsub("[[:alpha:]]|_|\\-","",expect)), silent=TRUE)
      if(inherits(exp2, "try-error")) stop(fxNa,"Argument 'expect=",pasteC(utils::head(expect)),"' contains no valid numeric content !") else {
        if(!silent) message(fxNa,"Trying to convert from 'expect=",pasteC(utils::head(expect)),"' to numeric  ",pasteC(utils::head(exp2))) }
    }
  } 
  if(doSelect) {  
    if(length(rowNa) >1) { message(fxNa," 'rowNa' should have length of 1 (but is ",length(rowNa),"), truncating ...")
      rowNa <- rowNa[1] }
    hasAnn <- subPat <- FALSE
    if(length(startLev) >0) if(!is.numeric(startLev)) stop("Argument 'startLev' must be integer")
    if(debug) {message(fxNa,"lMS0"); lMS0 <- list(rowNa=rowNa,dat=dat,logExpect=logExpect,startLev=startLev,lisNa=lisNa,hasAnn=hasAnn)}
    if(is.list(dat)) {
      ## 'dat' is list
      if(lisNa[2] %in% names(lisNa) && lisNa[2] %in% names(dat)) if(annoColNa[2] %in% colnames(dat[[lisNa[2]]])) hasAnn <- TRUE
      chLi <- lisNa %in% names(dat)
      if(all(!chLi)) stop("None of the list-elements given in 'lisNa' were found in 'dat' !")
      if(any(!chLi)) message(fxNa,"Trouble ahead : The elements ",pasteC(lisNa[which(!chLi)], quoteC="'")," not found !!")
      if(debug) {message(fxNa,"lMS1a"); lMS1a <- list(rowNa=rowNa,dat=dat,logExpect=logExpect,startLev=startLev,lisNa=lisNa,doSelect=doSelect,hasAnn=hasAnn, chLi=chLi)}
      if(is.numeric(rowNa) && grepl("^[[:digit:]]+$", rowNa)) {                     # is index
        linNo <- as.integer(rowNa)
        if(linNo >0 && linNo <= nrow(dat[[lisNa[2]]])) rowNa <- dat[[lisNa[2]]][rowNa,annoColNa[1]] else { doSelect <- FALSE
          if(!silent) message(fxNa,"Invalid line number (",linNo,")") }
      } else {
        linNo <- if(hasAnn) which(dat[[lisNa[2]]][,annoColNa[1]] == rowNa) else which(rownames(dat[[lisNa[1]]]) ==rowNa)  
        if(length(linNo) >1) { if(!silent) message(fxNa,"Name specified in argument 'lisNa' not unique, using first")
          linNo <- linNo[1] } else if(length(linNo) <1) {  # try to find from rownames
            linNo <- which(rownames(dat[[lisNa[2]]]) %in% rowNa)
            if(length(linNo) <1 && all(grepl("^[[:digit:]]+$", rowNa))) linNo <- as.numeric(rowNa)
            if(length(linNo) <1) { doSelect <- FALSE; if(!silent) message(fxNa,"Unable to find row corresponding to ",rowNa)}
          } 
        }
      dat1 <- if(doSelect) dat[[lisNa[3]]][linNo,] else NULL                       # get imputed data
    } else { 
      ## simple matrix data
      linNo <- if(is.numeric(rowNa) && grepl("^[[:digit:]]+$", rowNa)) as.integer(rowNa) else which(rownames(dat) %in% rowNa)
      if(length(linNo) !=1 && !silent) message(fxNa," Note : ",length(linNo)," lines of 'dat' matched to '",rowNa,"' ! (can use only 1st)")
      if(length(linNo) >1) linNo <- linNo[1] else if(length(linNo) <1) {doSelect <- FALSE; if(!silent) message(fxNa,"Could NOT match ",rowNa," to rownames of ",argNa[2])}
      dat1 <- if(doSelect) dat[linNo,] else NULL}
    if(debug) {message(fxNa,"lMS1"); lMS1 <- list(rowNa=rowNa,dat=dat,logExpect=logExpect,startLev=startLev,lisNa=lisNa,doSelect=doSelect, dat1=dat1)}
      
    ## which starting levels to test
    if(doSelect) {
      startLev <- if(length(startLev) <1) 1:floor(length(unique(expect))/2) else as.integer(startLev)
      expect0 <- expect
      if(!is.numeric(expect)) {
        subPat <- "[[:alpha:]]*[[:punct:]]*[[:alpha:]]*"
        subPat <- paste0(c("^",""),subPat,c("","$"))   
        expect <- try(as.numeric( sub(subPat[2], "", sub(subPat[1], "", as.character(expect))))) } 
      if(inherits(expect, "try-error")) stop(fxNa,"Problem extracting the numeric content of 'expect': ",pasteC(expect0,quoteC="'"))  
      if(!is.numeric(expect)) {
        expect <- as.numeric(as.factor(as.character(expect)))
        if(!silent) message(fxNa,"Note: 'expect' is not numeric, transforming to integers")
      }
      if(debug) {message(fxNa,"lMS2"); lMS2 <- list(rowNa=rowNa,dat=dat,logExpect=logExpect,startLev=startLev,lisNa=lisNa, dat1=dat1,expect0=expect0,expect=expect)}
    
      ## MAIN MODEL
      dat1 <- data.frame(conc=if(logExpect) log2(expect) else expect, quant=dat1, concL=expect)            # omics quantitation data is already log2
      lm0 <- lapply(startLev, function(x) { lmX <- if(!isFALSE(suppressWarnings)) suppressWarnings(
        try(stats::lm(quant ~ conc, data=dat1[which(expect >= sort(unique(expect))[x]),]))) else try(stats::lm(quant ~ conc, data=dat1[which(expect >= sort(unique(expect))[x]),])); lmX })
      slopeAndP <- sapply(lm0, function(x)  z <- stats::coef(summary(x))[2,c("Estimate","Pr(>|t|)")])
      bestReg <- which.min(slopeAndP[2,])
      if(!silent) message(fxNa," best slope pVal starting at level no ",bestReg)
      if(debug) {message(fxNa,"lMS3"); lMS3 <- list(rowNa=rowNa,dat=dat,logExpect=logExpect,startLev=startLev,lisNa=lisNa, dat1=dat1,expect0=expect0,expect=expect,lm0=lm0,slopeAndP=slopeAndP,bestReg=bestReg)}
      
      ## PLOT
      if(plotGraph) {
        levExp <- sort(unique(expect))
        usedP <- expect >= levExp[bestReg] 
        useCol <- rep(quantCol[1], nrow(dat1))                    # initialize as used
        pch1 <- rep(pch[1], nrow(dat1))                           # initialize as used
        if(any(!usedP)) { useCol[which(!usedP)] <- quantCol[2]    # color used for not-used  
          pch1[which(!usedP)] <- pch[2] }                         # symbol used for not-used  
        
        chNa <- is.list(dat) && lisNa[1] %in% names(dat)
        if(any(chNa)) chNa <- is.na(dat[[lisNa[1]]][linNo,])
        hasNa <- any(chNa)
        legCol <- quantCol[if(hasNa) c(1:3,3) else 1:2]
        legPch <- pch[if(hasNa) c(1,2,1,2) else 1:2]
        if(hasNa) {useCol[which(chNa)] <- quantCol[3]} else {legLab <- legLab[1:2]}
        if(length(tit) <1) { tit <- if(hasAnn) dat[[lisNa[2]]][linNo, annoColNa[2]] else paste0(rowNa," (from ",argNa[2],")")}
        if(length(yLab) <1) yLab <- "measured"
        if(length(xLab) <1) xLab <- "expected"
        if(logExpect) xLab <- paste0(xLab," (log-scale)")
        ## main plot
        chG <- try(graphics::plot(quant ~ conc, data=dat1, col=useCol, main=tit,ylab=yLab, xlab=xLab, las=1, pch=pch1, cex.lab=cexLab, col.axis="white", cex.axis=0.3,tck=0), silent=TRUE)
        if(inherits(chG, "try-error")) message(fxNa,"UNABLE to draw figure !!") else {       
          graphics::mtext(paste0("best regr starting at level ",bestReg," (ie ",sort(unique(expect))[bestReg],"),  slope=",signif(slopeAndP[1,bestReg],3),
            ",  p=",signif(slopeAndP[2,bestReg],2)), cex=cexSub, col=quantCol[1])
          ## use crt= for rotating text on bottom ? ... crt works only for text()
          graphics::mtext(at=(unique(dat1[,1])), signif(if(logExpect) 2^(unique(dat1[,1])) else unique(dat1[,1]),3), side=1, las=xLabLas,col=1,cex=cexXAxis)    # set las to vertical ?
          graphics::abline(lm0[[bestReg]], lty=2, col=quantCol[1])
          graphics::axis(side=2, las=1, col=1, tick=TRUE, cex.axis=cexYAxis)                                         # left axis
          ptBg <- quantCol
          chPch <- pch %in% c(21:25)
          if(any(chPch)) { quantCol[which(chPch)] <- grDevices::rgb(0.2,0.2,0.2,0.4) }
          if(requireNamespace("wrGraph", quietly=TRUE)) {
            legLoc <- try(wrGraph::checkForLegLoc(dat1, sampleGrp=legLab, showLegend=FALSE)$loc, silent=TRUE)
          } else {
            if(!silent) message(fxNa,"NOTE: Package 'wrGraph' not installed from CRAN for searching optimal placement of legend  (setting to default 'bottomright')")
            legLoc <- "bottomright"
          } 
          if(inherits(legLoc, "try-error")) { legLoc <- "bottomright"
             message(fxNa,"Did not succeed in determining optimal legend location")}
          tmp <- try(graphics::legend(legLoc, legLab, pch=legPch, col=legCol, text.col=legCol, pt.bg=ptBg, cex=cexLeg, xjust=0.5,yjust=0.5), silent=TRUE)        # as points
          if("try-error" %in% class(tmp)) message(fxNa,"NOTE : Unable to add legend !  ",as.character(tmp))
        }  
      }
      list(coef=stats::coef(summary(lm0[[bestReg]])), name=rowNa, startLev=bestReg) } } }
     

#' Rename columns
#'
#' This function renames columns of 'refMatr' using 2-column matrix (or data.frame) indicating old and new names (for replacement). 
#' 
#' @param refMatr matrix (or data.frame) where column-names should be changed
#' @param newName (matrix of character) giving correspondence of old to new names (number of lines must match number of columns of 'refMatr')
#' @param silent (logical) suppres messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allows easier tracking of messages produced
#' @return  This function returns a matrix (or data.frame) with renamed columns
#' @examples
#' ma <- matrix(1:8,ncol=4,dimnames=list(1:2,LETTERS[1:4]))
#' replBy1 <- cbind(new=c("dd","bb","z_"),old=c("D","B","zz"))
#' replBy2 <- matrix(c("D","B","zz","dd","bb","z_"),ncol=2)
#' replBy3 <- matrix(c("X","Y","zz","xx","yy","z_"),ncol=2)
#' renameColumns(ma,replBy1)
#' renameColumns(ma,replBy2)
#' renameColumns(ma,replBy3)
#' @export
renameColumns <- function(refMatr, newName, silent=FALSE, debug=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom, newNa="renameColumns")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  if(length(dim(refMatr)) <2) stop("Expecting matrix or data.frame")
  msg <- "Expecting matrix with 2 cols ('old','new')"
  if(length(dim(newName)) !=2) stop(msg) else if(ncol(newName) <2) stop(msg)
  if(is.null(colnames(newName))) { colNe <- 1:2
  } else { colNe <- match(c("old","new"),colnames(newName)) 
    if(debug)  message(fxNa," colNe ini ",colNe)
    if(is.na(colNe[1])) colNe[1] <- (1:ncol(newName))[if(is.na(colNe[2])) -2 else -1*colNe[2]][1]
    if(is.na(colNe[2])) colNe[2] <- (1:ncol(newName))[-1*colNe[1]][1] }
  newName <- newName[,colNe]
  replLi <- naOmit(match(colnames(refMatr), newName[,1]))
  if(length(replLi) <1) { if(!silent) message(fxNa,"No names matching for replacing dat, nothing to do !")
  } else {
    colnames(refMatr)[match(newName[replLi,1], colnames(refMatr))] <- newName[replLi,2] }
  refMatr }

#' Remove all columns where all data are not finite
#'
#' This function aims to remove all columns where all data are not finite
#' 
#'
#' @param dat (matrix or data.frame) main input 
#' @param msgStart (character) 
#' @param silent (logical) suppres messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allows easier tracking of messages produced
#' @return This function returns a corrected matrix or data.frame
#' @seealso  \code{\link{renameColumns}}; \code{\link[base]{is.finite}}
#' @examples
#' ma1 <- matrix(c(1:5, Inf), ncol=2)
#' .keepFiniteCol(ma1)
#' 
#' @export
.keepFiniteCol <- function(dat, msgStart=NULL, silent=FALSE, debug=FALSE, callFrom=NULL){
  ## remove all columns where all data are not finite
  fxNa <- .composeCallName(callFrom, newNa=".keepFiniteCol")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  tmp <- colSums(is.finite(dat))
  if(any(tmp <1)) {
    if(!silent) message(fxNa," removing ",sum(tmp <1)," columns without valid (finite) numbers")
    chC <- tmp >0
    dat <- if(sum(chC) ==1) matrix(dat[,which(chC)], ncol=1, dimnames=list(rownames(dat),colnames(dat[chC]))) else dat[,which(chC)]}
  dat }

#' Search for (empty) columns conaining only entries defined in 'searchFields' and remove such columns
#'
#' This function aims to search for (empty) columns conaining only entries defined in 'searchFields' and remove such columns.
#' If 'fromBackOnly' =TRUE .. only tailing empty columns will be removed (other columns with "empty" entries in middle will be kept).
#' If ''=TRUE columns containing all NAs will be excluded as well 
#' This function will also remove columns containing (exculsively) mixtures of the various 'searchFields'.
#'
#' @param dat (matrix or data.frame) main input 
#' @param fromBackOnly (logical) 
#' @param searchFields (character)
#' @param silent (logical) suppres messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allows easier tracking of messages produced
#' @return This function returns a corrected matrix or data.frame
#' @seealso   \code{\link{renameColumns}};  \code{\link[base]{is.finite}}
#' @examples
#' ma1 <- matrix(c(1:5, NA), ncol=2)
#' .removeEmptyCol(ma1)
#' @export
.removeEmptyCol <- function(dat, fromBackOnly=TRUE, searchFields=c(""," ","NA.",NA), silent=FALSE, debug=FALSE, callFrom=NULL){
  ## search for (empty) columns conaining only entries defined in 'searchFields' and remove such columns
  ## if 'fromBackOnly' =TRUE .. only tailing empty columns will be removed (other columns with "empty" entries in middle will be kept)
  ## if ''=TRUE columns containing all NAs will be excluded as well
  ## will also remove columns containing (exculsively) mixtures of the various 'searchFields'
  fxNa <- .composeCallName(callFrom,newNa=".removeEmptyCol")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  if(length(dim(dat)) <1) dat <- matrix(dat,ncol=1,dimnames=list(names(dat),NULL))
  iniDimNa <-  dimnames(dat)
  isEmpty <- which(apply(dat, 2, function(x) sum(x %in% searchFields) ==length(x)))
  if(fromBackOnly && length(isEmpty) >0) {
    isEmpty <- if(max(isEmpty) != ncol(dat)) NULL else {
      if(length(isEmpty) >1) .breakInSer(isEmpty) else isEmpty } }
  if(length(isEmpty) >1) {
    dat <- .removeCol(dat,isEmpty)
    if(!silent) message(fxNa,"Columns no ",paste(isEmpty,collapse=", ")," were considered empty and removed")
    }
  dat }
  

#' Transform columns of matrix to list of vectors
#'
#' convert matrix to list of vectors: each column of 'mat' as vector of list 
#' @param mat (matrix) main input
#' @param concSym (character) symbol for concatenating: concatenation of named vectors in list names as colname(s)+'concSym'+rowname
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return matrix or array (1st dim is intraplate-position, 2nd .. plate-group/type, 3rd .. channels)
#' @seealso \code{\link{convToNum}}
#' @examples
#' mat1 <- matrix(1:12,ncol=3,dimnames=list(letters[1:4],LETTERS[1:3]))
#' mat2 <- matrix(LETTERS[11:22],ncol=3,dimnames=list(letters[1:4],LETTERS[1:3]))
#' matr2list(mat1);  matr2list(mat2)
#' @export
matr2list <- function(mat, concSym=".", silent=FALSE, debug=TRUE, callFrom=NULL) {
  if(!is.matrix(mat)) mat <- as.matrix(mat)
  isNum <- is.numeric(mat)
  outNa <- rownames(mat)
  out <- as.list(as.data.frame(rbind(colnames(mat), mat)))
  out <- lapply(out,function(x) { x <- as.character(x); nam <- x[1];     # 1st element was introduced for transmitting name of current column
    useSep <- nam=="" | outNa==""                                        # check if any of name-parts for concatenation empty -> omit concSym
    x <- if(isNum) as.numeric(x[-1]) else x[-1]                          # trim to input & reset to numeric
    names(x) <- paste0(nam, ifelse(useSep,"",concSym), outNa); x})
  out }
   

#' Normalize Data In Various Modes
#'
#' Generic normalization of 'dat' (by columns), multiple methods may be applied.
#' The choice of normalization procedures must be done with care, plotting the data before and after normalization
#' may be critical to understandig the initial data structure and the effect of the procedure applied.
#' Inappropriate methods chosen may render interpretation of (further) results incorrect.
#'
#' @details
#' In most cases of treating 'Omics'-data one works with the hypothesis that there are no global changes in the structure of all data/columns
#' Under this htpothesis it is very common to assume the the median (via the argument \code{method}) of all samples (ie columns) should remain constant.
#' For examples samples/columns with less signal will be considered as having received 'accidentally' less material (eg due to the imprecision when transfering very small amounts of liquid samples).
#' In consequence, a sample having received only 95% of input material is assumed to give only 95% signal intensity of what may have been expected.
#' Thus, all measures will be multiplied by 1/0.95 (apr 1.053) to compensate for supposed lack of staring material.
#'
#' LOG-SCALE : With the analysis of 'Omics'-data it is very common to work with data on log-scale.
#' In this case the argument \code{mode} should be set to \code{additive} (or \code{log} or \code{logarithmic}).
#' At normalization a constant factor will be added (or subtracted) using this mode. This corresponds to a multiplicative factor on regular scale.
#' Please note that (at this point) the methods 'slope', 'exponent', 'twoPointSlope' and 'vsn' don't distinguish between additive and proportional modes, but take take the data 'as is'
#' (you may look at the original documenation for more details, see \code{\link{exponNormalize}}, \code{\link{adjBy2ptReg}}, \code{\link[vsn]{justvsn}}).
#'
#' Normalization using \code{method="rowNormalize"} runs \code{\link{rowNormalize}} from this package.
#' In this case, the working hypothesis is, that all values in each row are expected to be the same.
#' This method could be applied when all series of values (ie columns) are replicate measurements of the same sample.
#' THere is also an option for treating sparse data (see argument \code{sparseLim}), which may, hovere, consume much more comptational ressources,
#' in particular, when the value \code{nCombin} is low (compared to the number of samples/columns).
#' 
#' Normalization using  \code{method="standardize"} runs \code{\link[base]{scale}} resulting in a matrix where column-means are set to 0 (or very close to 0)
#' and sd et 1, standardization by columns. With this method additive or proportional mode give the same output.  
#' Results from this methid should be interpreted with caution, since the absolute values are not any more considered. 
#' Therefor this approach is rather oriented for comparing profiles (no matter how important changes are on an absolute scale)
#'
#' Normalization using  \code{method="vsn"} runs \code{\link[vsn]{justvsn}} from \href{https://bioconductor.org/packages/release/bioc/html/vsn.html}{vsn}
#' (this requires a minimum of 42 rows of input-data and having the Bioconductor package vsn installed).
#' Note : Depending on the procedure chosen, the normalized data may appear on a different scale.
#'
#' @param dat matrix or data.frame of data to get normalized
#' @param method (character) may be "mean","median","NULL","none", "trimMean", "rowNormalize", "slope", "exponent", "twoPointSlope", "vsn"; When \code{NULL} or 'none' is chosen the input will be returned
#' @param refLines (NULL or numeric) allows to consider only specific lines of 'dat' when determining normalization factors (all data will be normalized)
#' @param refGrp Only the columns indicated will be used as reference, default all columns (integer or colnames)
#' @param mode (character) may be "proportional", "additive" (for log2 data);
#'  decide if normalization factors will be applied as multiplicative (proportional) or additive; for log2-omics data \code{mode="additive"} is suggested
#'  besides 'log' or 'logarithmic' may be used instead of 'additive'; and 'lin' or 'linear' may be used instead of 'proportional'
#' @param trimFa (numeric, length=1) additional parameters for trimmed mean
#' @param minQuant (numeric) only used with \code{method='rowNormalize'}: optional filter to set all values below given value as \code{NA}; see also \code{\link{rowNormalize}}
#' @param sparseLim (integer) only used with \code{method='rowNormalize'}: decide at which min content of  \code{NA} values the function should go in sparse-mode; see also \code{\link{rowNormalize}}
#' @param nCombin (NULL or integer) only used with \code{method='rowNormalize'}: used only in sparse-mode (ie if content of \code{NA}s higher than content of \code{sparseLim}):
#'    Number of groups of smller matrixes with this number of columns to be inspected initially;
#'    low values (small groups have higher chances of more common elements); see also \code{\link{rowNormalize}}
#' @param omitNonAlignable (logical) only used with \code{method='rowNormalize'}: allow omitting all columns which can't get aligned due to sparseness; see also \code{\link{rowNormalize}}
#' @param maxFact (numeric, length=2) only used with \code{method='rowNormalize'}:  max normalization factor; see also \code{\link{rowNormalize}}
#' @param quantFa (numeric, length=2) additional parameters for quantiles to use with method='slope'
#' @param expFa (numeric, length=1) additional parameters for method='exponent'
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allows easier tracking of messages produced
#' @return This function returns a matrix of normalized data (same dimensions as input)
#' @seealso  \code{\link{rowNormalize}}, \code{\link{exponNormalize}}, \code{\link{adjBy2ptReg}}, \code{\link[vsn]{justvsn}}
#' @examples
#' set.seed(2015); rand1 <- round(runif(300)+rnorm(300,0,2),3)
#' dat1 <- cbind(ser1=round(100:1+rand1[1:100]), ser2=round(1.2*(100:1+rand1[101:200])-2),
#'   ser3=round((100:1 +rand1[201:300])^1.2-3))
#' dat1 <- cbind(dat1, ser4=round(dat1[,1]^seq(2,5,length.out=100)+rand1[11:110],1))
#' dat1[dat1 <1] <- NA
#'   summary(dat1)
#'   dat1[c(1:5,50:54,95:100),]
#' no1 <- normalizeThis(dat1, refGrp=1:3, meth="mean")
#' no2 <- normalizeThis(dat1, refGrp=1:3, meth="trimMean", trim=0.4)
#' no3 <- normalizeThis(dat1, refGrp=1:3, meth="median")
#' no4 <- normalizeThis(dat1, refGrp=1:3, meth="slope", quantFa=c(0.2,0.8))
#' dat1[c(1:10,91:100),]
#' cor(dat1[,3],rowMeans(dat1[,1:2],na.rm=TRUE), use="complete.obs")             # high
#' cor(dat1[,4],rowMeans(dat1[,1:2],na.rm=TRUE), use="complete.obs")             # bad
#' cor(dat1[c(1:10,91:100),4],rowMeans(dat1[c(1:10,91:100),1:2],na.rm=TRUE),use="complete.obs")
#' cor(dat1[,3],rowMeans(dat1[,1:2],na.rm=TRUE)^ (1/seq(2,5,length.out=100)),use="complete.obs")
#' @export
normalizeThis <- function(dat, method=c("mean","average","median","trimMean","rowNormalize","standardize","slope","twoPointSlope","exponent","vsn","none","NULL"), refLines=NULL, refGrp=NULL, 
  mode=c("proportional","additive","linear","lin","logarithmic","log"), trimFa=NULL, minQuant=NULL, sparseLim=0.4, nCombin=3, omitNonAlignable=FALSE, maxFact=10, quantFa=NULL, expFa=NULL, silent=FALSE, debug=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom, newNa="normalizeThis")
  if(isTRUE(debug)) silent <- FALSE else { debug <- FALSE
    if(!isTRUE(silent)) silent <- FALSE }
  if(!any(sapply(c("additive","add","ad","a"), identical, mode))) mode <- "proportional"
  
  mode <- match.arg(mode) 
  if(any(c("logarithmic","log") %in% mode))  mode <- "additive" else if(any(c("linear","lin") %in% mode)) mode <- "proportional" 
  if(length(mode) >1) mode <- mode[1]
  method <- match.arg(method) 
  if("average" %in% method)  method <- "mean"  
  if("NULL" %in% method)  method <- "none"  
  if(length(method) >1) method <- method[1]
  
  out <- NULL
  chMe <- is.na(method)
  if(sum(!chMe) <1) stop(fxNa,"Argument 'method' seems empty - nothing to do !")
  method <- if(any(chMe)) naOmit(method)[1] else method[1]
  if(length(dim(dat)) !=2) stop(fxNa,"Expecting matrix or data.frame with >= 2 rows as 'dat' !")
  if(!is.matrix(dat)) dat <- as.matrix(dat)
  if(!is.null(refLines)) if(identical(refLines, 1:nrow(dat))) {refLines <- NULL; if(!silent) message(fxNa,"Omit redundant 'refLines'")}
  ## assemble parameters
  params <- list(refLines=refLines, trimFa=trimFa, useQ=quantFa, useExp=expFa)
  
  ## method specific elements
  if(is.null(refGrp)) { refGrp <- 1:ncol(dat)
  } else if(min(refGrp) > ncol(dat) | max(refGrp) < 1) stop(fxNa," 'refGrp' should be integer vector indicating which columns to be used as reference")
  if(debug) {message(fxNa,"Assemble parameters ; nt1 ;  using method=",method,"   mode=",mode,"   params=",pasteC(unlist(params[-1])))}
  
  if(method %in% "trimMean") {
    params$trimFa <- 0.2
    if(length(trimFa) >0) {if(length(trimFa) ==1) params$trimFa <- trimFa else if(!silent) message(fxNa," invalid 'trimFa', use default 0.2")}
  }
  if(any(sapply(c("rowNormalize","rowNorm","rowN","rowProp","rowP","row"), identical, method))) {
    if(debug) message(fxNa,"Recognized as method='rowNormalize'")
    params$minQuant <- minQuant
    params$sparseLim <- sparseLim
    params$nCombin <- nCombin
    params$omitNonAlignable <- omitNonAlignable
    params$maxFact <- maxFact
    params$mode <- mode
    method <- "rowNorm"
  }
  if(method %in% "slope") {
    params$useQ <- c(0.2,0.8)
    if(length(quantFa) >0) {if(length(quantFa)==2) params$useQ <- quantFa else if(!silent) message(fxNa,"Invalid 'quantFa', use default c(0.2,0.8)")}
  }
  if(method %in% "twoPointSlope") {
    if(length(params$useQ) !=1) params$useQ <- list(signif(stats::quantile(dat, c(0.05,0.15), na.rm=TRUE), 3),
      signif(stats::quantile(dat, c(0.05,0.15), na.rm=TRUE), 3))
  }
  if(any(sapply(c("exponent","expo","exp"), identical, method))) {
    if(debug) message(fxNa,"Recognized as method='exponent'")
    if(length(expFa) <1) { useExp <- c(log(c(10:1)), 30, 10, 3)
      params$useExp <- sort(unique(c(round(1 /(1 +abs(useExp -useExp[1])), 4), round(1 +abs(useExp -useExp[1]), 3)))) }
    method <- "exponent"
  }
  if("vsn" %in% method && !requireNamespace("vsn")) {
    method <- "mean"
    if(!silent) message(fxNa,"NOTE: Please install package 'vsn' first from CRAN, resetting argument 'method' to default 'mean'")
  }
  if("vsn" %in% method && (if(length(refLines) >0) length(refLines) < nrow(dat) else FALSE)) {
    if(debug) message(fxNa,"Recognized as method='vsn'")
    params$refLines <- NULL
    if(!silent) message(fxNa,"Ignoring content of 'refLines', since 'vsn' can only normalize considering all data")}
  if(debug) {message(fxNa,"Ready to start .normalize() ; nt2 ;  using method=",method,"   mode=",mode,"   params=",pasteC(unlist(params)))}
  
  ## main normalization
  out <- .normalize(dat, method, mode=mode, param=params, silent=silent, debug=debug, callFrom=fxNa)
  if(inherits(out, "try-error")) { message(fxNa,"Could not run normalization by '",method,"' which gave an error (returning unnormalized)"); out <- dat}
  out }


#' Main Normalization function
#'
#' This function aims to normalize a matrix or data.frame by columns. 
#' It assumes all checks have been done before calling this function.
#' 
#' @param dat matrix or data.frame of data to get normalized
#' @param meth (character) may be "mean","median","NULL","none", "standardize","trimMean", "rowNormalize", "slope", "exponent", "twoPointSlope", "vsn"; When \code{NULL} or 'none' is chosen the input will be returned
#' @param mode (character) may be "proportional", "additive";
#'  decide if normalizatio factors will be applies as multiplicative (proportional) or additive; for log2-omics data \code{mode="additive"} is suggested
#' @param param (list) additional parameters
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allows easier tracking of messages produced
#' @return This function returns a numeric vector
#' @seealso \code{\link{normalizeThis}}
#' @examples
#' aa <- matrix(1:12, ncol=3)
#' .normalize(aa,"median",mode="proportional",param=NULL)
#' @export
.normalize <- function(dat, meth, mode, param, silent=FALSE, debug=FALSE, callFrom=NULL){
  ## 'dat' .. matrix (>1 col, >1 li) to be normalized
  ## 'meth' .. method
  ## 'param' .. list with supl parameters (refLines, certain specific for norm methods)
  fxNa <- .composeCallName(callFrom, newNa=".normalize")
  if(isTRUE(debug)) silent <- FALSE else { debug <- FALSE
    if(!isTRUE(silent)) silent <- FALSE }
  mode <- if(identical(mode,"additive")) "Add" else "Prop"       # reduce to short name
  if(identical(meth,"average")) meth <- "mean"
  asRefL <- (length(param$refLines) < nrow(dat) && !is.null(param$refLines))
  datRef <- if(asRefL) dat[param$refLines,,drop=FALSE] else NULL
  if("vsn" %in% meth) {
    chPa <- requireNamespace("vsn", quietly=TRUE)
    if(!chPa) { meth <- "mean"
      if(!silent) message(fxNa, "NOTE: Package 'vsn' not found ! Please install first from Bioconductor; resetting argument 'meth' to default 'mean'")
    }
    if(nrow(if(asRefL) datRef else dat) <42) message(callFrom," PROBLEM : Too few lines of data to run 'vsn' ! ")}
  ## Some methods have not yet been adopted/declined to additive/proportional mode
  if(meth %in% c("none", "slope", "exponent", "twoPointSlope", "vsn")) mode <- NULL
  meth <- paste0(meth,mode)

  switch(meth,
    none=dat,
    meanAdd= mean(if(asRefL) datRef else dat, na.rm=TRUE) + dat - rep(colMeans(if(asRefL) datRef else dat, na.rm=TRUE), each=nrow(dat)),
    meanProp= mean(if(asRefL) datRef else dat, na.rm=TRUE) * dat / rep(colMeans(if(asRefL) datRef else dat, na.rm=TRUE), each=nrow(dat)),
    trimMeanAdd=mean(if(asRefL) datRef else dat, trim=param$trimFa, na.rm=TRUE) + dat - rep(apply(
      if(asRefL) datRef else dat, 2, mean, trim=param$trimFa, na.rm=TRUE), each=nrow(dat)),
    trimMeanProp=mean(if(asRefL) datRef else dat, trim=param$trimFa, na.rm=TRUE) * dat / rep(apply(
      if(asRefL) datRef else dat, 2, mean, trim=param$trimFa, na.rm=TRUE), each=nrow(dat)),
    medianAdd=stats::median(if(asRefL) datRef else dat, na.rm=TRUE) + dat - rep(apply(
      if(asRefL) datRef else dat, 2, stats::median, na.rm=TRUE), each=nrow(dat)),
    medianProp=stats::median(if(asRefL) datRef else dat,na.rm=TRUE) * dat / rep(apply(
      if(asRefL) datRef else dat, 2, stats::median, na.rm=TRUE), each=nrow(dat)),
    standardizeAdd= scale(if(asRefL) datRef else dat, center=TRUE, scale=TRUE),
    standardizeProp= scale(if(asRefL) datRef else dat, center=TRUE, scale=TRUE),
    rowNorm= rowNormalize(dat=dat, method=meth, refLines=param$refLines, refGrp=param$refGrp, proportMode="Prop" %in% param$mode, minQuant=param$minQuant,
      sparseLim=param$sparseLim, nCombin=param$nCombin, omitNonAlignable=param$omitNonAlignable, maxFact=param$maxFact, silent=silent, debug=debug, callFrom=fxNa),
    slope=.normConstSlope(mat=dat, useQuant=param$useQ, refLines=param$refLines, diagPlot=FALSE),
    exponent=try(exponNormalize(dat, useExpon=param$useExp, refLines=param$refLines)$datNor, silent=TRUE),
    twoPointSlope=try(adjBy2ptReg(dat, lims=param$useQ, refLines=param$refLines), silent=TRUE),
    vsn=if(requireNamespace("vsn")) try(vsn::justvsn(dat), silent=TRUE) else NULL ) 
  }


#' Normalize Columns Of 2-dim Matrix To Common Linear Regression Fit
#'
#' This function aims to normalize columns of 2dim matrix to common linear regression fit within range of 'useQuant'
#' 
#' @param mat matrix or data.frame of data to get normalized
#' @param useQuant (numeric) quantiles to use
#' @param refLines (NULL or numeric) allows to consider only specific lines of 'dat' when determining normalization factors (all data will be normalized)
#' @param diagPlot (logical) draw diagnistic plot
#' @param plotLog (character) indicate which axis shousl be diplayed on log-scale, may be 'x', 'xy' or 'y'
#' @param datName  (character) use as title in diag plot
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allows easier tracking of messages produced
#' @return This function returns a numeric vector
#' @examples
#' aa <- matrix(1:12, ncol=3)
#' @seealso \code{\link{normalizeThis}}
##' @export
.normConstSlope <- function(mat, useQuant=c(0.2,0.8), refLines=NULL, diagPlot=TRUE, plotLog="", datName=NULL, silent=FALSE, debug=FALSE, callFrom=NULL){
  ## normalize columns of 2dim matrix to common linear regression fit within range of 'useQuant'
  ## returns normalized data (2dim matrix)
  ## in case of matrixes, data will be normalized by columns, but only the average of all indiv regression-lines will be shown on graph
  ## 'useQuant' ..defines window of data to be considered for normalizing
  ## 'refLines' ..lines to use for determining normalization factors
  ## 'datName' for use as title in diag plot
  fxNa <- .composeCallName(callFrom,newNa=".normConstSlope")
  if(isTRUE(debug)) silent <- FALSE else { debug <- FALSE
    if(!isTRUE(silent)) silent <- FALSE }
  msg1 <- "'useQuant' should be vector of two numeric values between 0 & 1"
  msg2 <- paste("'mat' should be matrix with two dimensions. Here it appears as ")
  if(!is.numeric(mat)) stop(fxNa,msg2,class(mat)," with ",mode(mat)," data of ",length(dim(mat))," dims")
  if(sum(is.finite(useQuant)) != 2 || length(useQuant) !=2) {message(fxNa,msg1); useQuant=c(0.2,0.8)}
  if(useQuant[1] < 0 || useQuant[1] > 1) {message(msg1); useQuant=c(0.2,0.8)}
  if(plotLog %in% c("x","xy")) {matOri <- mat; mat <- log(mat); message(fxNa," ..setting data to log")}
  quantVal <- apply(mat, 2, stats::quantile, useQuant,na.rm=TRUE)
  tmp <- apply(mat, 2, function(x) {x <- naOmit(x); x[x >= stats::quantile(x,min(useQuant),na.rm=TRUE) & x <= stats::quantile(x,max(useQuant),na.rm=TRUE)]})
  if(!is.list(tmp)) tmp <- as.data.frame(tmp)
  regr <- sapply(if(length(refLines) < nrow(mat) && !is.null(refLines)) tmp[refLines,] else tmp, .datSlope, toNinX=TRUE)
  regr[1,] <- regr[1,] + apply(rbind(quantVal[1,],mat), 2, function(x) sum(x[-1] < x[1], na.rm=TRUE))    # correct intercept for no of quantile-omitted data
  regrM <- rowMeans(regr, na.rm=FALSE)
  normD <- apply(rbind(regr, mat), 2, function(x) (x[-1*(1:2)] -(regrM[1]-x[1])/x[2]) *x[2]/regrM[2])
  if(plotLog %in% c("x","xy")) normD <- exp(normD)
  if(diagPlot) {
    msg2 <- paste(fxNa,"Unknow argument content ('",plotLog,"') for 'plotLog'; resetting to default no log",sep="")
    if(length(plotLog) !=1) { message(fxNa,msg2); plotLog <- ""}
    if(!(plotLog %in% c("","x","y","xy"))) {message(fxNa,msg2); plotLog <- ""}
    yLab <- "number of values"
    xLab <- "sorted values"
    xDat <- sort(normD[which(is.finite(normD[,1])),1])
    if(length(refLines) < nrow(mat) && !is.null(refLines)) { chG <- try(graphics::plot(1:sum(is.finite(normD[refLines,1])) ~ sort(normD[which(is.finite(normD[refLines,1])),1]),
      type="s", main=paste("normalizing ",datName),log=plotLog,col=3,xlab=xLab,ylab=yLab), silent=TRUE)
      if(inherits(chG, "try-error")) message(fxNa,"UNABLE to draw figure !") else { 
        graphics::points(sort(normD[which(is.finite(normD[-1*refLines,1])),1]), 1:sum(is.finite(normD[-1*refLines,1])), type="s",col=grDevices::grey(0.4))}
    } else chG <- try(graphics::plot(1:sum(is.finite(normD[,1])) ~ sort(normD[which(is.finite(normD[,1])),1]), type="s",main=paste("normalizing ",datName), log=plotLog,col=3,xlab=xLab,ylab=yLab), silent=TRUE)
    if(inherits(chG, "try-error")) message(fxNa,"UNABLE to draw figure !") else {    
      for(ii in 2:ncol(normD)) if(length(refLines) < nrow(mat) && !is.null(refLines)) {
        graphics::points(1:sum(is.finite(normD[refLines,ii]))~ sort(normD[which(is.finite(normD[refLines,ii])),ii]),type="s",col=ii+2)
        graphics::points(sort(normD[which(is.finite(normD[-1*refLines,ii])),1]), 1:sum(is.finite(normD[-1*refLines,ii])),type="s",col=grDevices::grey(0.4))
      } else graphics::points(1:sum(is.finite(normD[,ii]))~ sort(normD[which(is.finite(normD[,ii])),ii]), type="s",col=ii+2)
      graphics::abline(h=useQuant*sum(!is.na(normD))/ncol(normD), lty=3, col=grDevices::grey(0.6))
      if(plotLog %in% c("x","xy")) graphics::curve(log(regrM[1])*(regrM[2]) +regrM[1], lty=2,col=2,add=TRUE) else graphics::abline(regrM[1],regrM[2],lty=2,col=2)
    } }
  normD }
  

#' Fuse content of list-elements with redundant (duplicated) names
#'
#' This function fuses (character or numeric) elements of list re-occuring under same name, so that resultant list has unique names.
#' Note : Will not work with list of matrixes
#'
#' @param lst (list) main input, list of numeric vectors
#' @param initOrd (logical) preserve initial order in output (if TRUE) or otherwise sort alphabetically
#' @param removeDuplicates (logical) allow to remove duplicate entries (if vector contains names, both the name and the value need to be identical to be removed; note: all names must have names with more than 0 characters to be considered as names)  
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return  This function returns a fused list (same names as elements of input)
#' @seealso \code{\link[base]{unlist}}
#' @examples
#' val1 <- 10 +1:26
#' names(val1) <- letters
#' lst1 <- list(c=val1[3:6],a=val1[1:3],b=val1[2:3],a=val1[12],c=val1[13])
#' fuseCommonListElem(lst1)
#' @export
fuseCommonListElem <- function(lst, initOrd=TRUE, removeDuplicates=FALSE, silent=FALSE, debug=FALSE, callFrom=NULL) {
  ## fuse (character or numeric) elements of list re-occuring under same name, so that resultant list has unique names
  ## will not work with list of matrixes
  ## return fused list (same names as elements of input)
  fxNa <- .composeCallName(callFrom, newNa="fuseCommonListElem")
  chDim <- sapply(lst, function(x) length(dim(x)) < 2)
  if(any(!chDim)) stop(fxNa, "Need list of numeric vectors for fusing !")
  chDup <- duplicated(names(lst),fromLast=FALSE)
  if(any(chDup)) {
    iniNa <- names(lst)
    chDup <- chDup | duplicated(names(lst), fromLast = TRUE)
    tmp <- lst[which(chDup)]
    tmp <- tmp[order(names(tmp))]
    nMa <- sapply(tmp, length)
    names(nMa) <- names(tmp)
    names(tmp) <- NULL
    tmp2 <- tapply(unlist(tmp), factor(rep(names(nMa), nMa)), function(x) x)
    if(removeDuplicates) {
      hasNa <- sapply(tmp2,function(x) all(nchar(names(x)) >0) )
      chDu <- if(!all(hasNa)) lapply(tmp2, duplicated, fromLast=FALSE) else {
        lapply(tmp2, function(x) duplicated(x) & duplicated(names(x)))}
      hasDu <- sapply(chDu,any)
      if(any(hasDu)) {
        tmp2[which(hasDu)] <- lapply(tmp2[which(hasDu)], function(x) {
          if(all(nchar(names(x)) >0)) x[which(!duplicated(x) & !duplicated(names(x)))] else x[which(!duplicated(x))] }) }} 
    ## now need to re-integrate modified elements
    chDuR <- duplicated(names(lst), fromLast=TRUE)
    lst <- lst[which(!chDup & !chDuR)]
    suplLi <- length(lst) + (1:length(tmp2))
    lst[suplLi] <- tmp2
    names(lst)[suplLi] <- names(tmp2)
    if(initOrd) lst <- lst[match(unique(iniNa),names(lst))]
  }
  lst}
   

#' Assign new transparency to given colors 
#'
#' This function alows (re-)defining a new transparency. A color encoding vector will be transformed to the same color(s) but with new transparency (alpha).   
#' @param color (character) color input
#' @param alph (numeric) transparency value (1 for no transparency, 0 for complete opaqueness), values <1 will be treated as percent-values
#' @return character vector (of same length as input) with color encoding for new transparency
#' @seealso \code{\link[grDevices]{rgb}}, \code{\link[graphics]{par}}
#' @examples
#' col0 <- c("#998FCC","#5AC3BA","#CBD34E","#FF7D73")
#' col1 <- convColorToTransp(col0,alph=0.7)
#' layout(1:2)
#' pie(rep(1,length(col0)),col=col0)
#' pie(rep(1,length(col1)),col=col1,main="new transparency")
#' @export
convColorToTransp <- function(color,alph=1){
  ## set given colors to specific transparency
  ## convert standard color vector to same color but with specific transparency 'alph'
  if(any(alph <1)) alph <- round(alph*100)
  col1 <- grDevices::col2rgb(color,alpha=TRUE)
  col1[4,] <- rep(alph,ncol(col1))[1:ncol(col1)]
  apply(col1,2,function(x) grDevices::rgb(x[1],x[2],x[3],alpha=x[4],maxColorValue=255)) }
    

#' Find similar numeric values from two vectors/matrixes
#'
#' \code{findSimilFrom2sets} compares to vectors or matrixes and returns combined view including only all close (by \code{\link[wrMisc]{findCloseMatch}}).
#' Return matrix (predMatr) with add'l columns for index to and 'grp' (group of similar values (1-to-many)), 'nGrp' (n of grp), 'isBest' or 'nBest', 'disToMeas' 
#' (distance/difference between pair) &  'ppmToPred' (distance in ppm).
#' Note: too wide 'limitComp' will result in large window and many 'good' hits will compete (and be mutually exlcuded) if selection 'bestOnly' is selected
#' @param predMatr (matrix or numeric vector) dataset number 1, referred to as 'predicted', the colum speified in argument \code{colPre} points to the data to be used
#' @param measMatr (matrix or numeric vector) dataset number 2, referred to as 'measured', the colum speified in argument \code{colMeas} points to the data to be used
#' @param colMeas (integer) which column number of 'measMatr' to consider
#' @param colPre (integer) which column number of 'predMatr' to consider
#' @param compareTy (character) 'diff' (difference) 'ppm' (relative difference)
#' @param limitComp (numeric) limit used by 'compareTy'
#' @param bestOnly (logical) allows to filter only hits with min distance (defined by 'compareTy'), 3rd last col will be 'nBest' - otherwise 3rd last col 'isBest'
#' @param silent (logical) suppress messages
#' @param callFrom (character) allow easier tracking of messages produced
#' @param debug (logical) for bug-tracking: more/enhanced messages
#' @return This function returns a matrix (predMatr) with add'l columns for index to and 'grp' (group of similar values (1-to-many)), 'nGrp' (n of grp), 'isBest' or 'nBest', 'disToMeas' (distance/difference between pair) &  'ppmToPred' (distance in ppm)
#' @seealso \code{\link{checkSimValueInSer}} \code{\link{findCloseMatch}} \code{\link{closeMatchMatrix}}  
#' @examples
#' aA <- c(11:17); bB <- c(12.001,13.999); cC <- c(16.2,8,9,12.5,12.6,15.9,14.1)
#' aZ <-  matrix(c(aA,aA+20),ncol=2,dimnames=list(letters[1:length(aA)],c("aaA","aZ")))
#' cZ <-  matrix(c(cC,cC+20),ncol=2,dimnames=list(letters[1:length(cC)],c("ccC","cZ")))
#' findCloseMatch(cC,aA,com="diff",lim=0.5,sor=FALSE)
#' findSimilFrom2sets(aA,cC)
#' findSimilFrom2sets(cC,aA)
#' findSimilFrom2sets(aA,cC,best=FALSE)
#' findSimilFrom2sets(aA,cC,comp="ppm",lim=5e4,deb=TRUE)
#' findSimilFrom2sets(aA,cC,comp="ppm",lim=9e4,bestO=FALSE)
#' # below: find fewer 'best matches' since search window larger (ie more good hits compete !)
#' findSimilFrom2sets(aA,cC,comp="ppm",lim=9e4,bestO=TRUE)      
#' @export
findSimilFrom2sets <- function(predMatr, measMatr, colMeas=1, colPre=1, compareTy="diff", limitComp=0.5,bestOnly=FALSE,silent=FALSE,callFrom=NULL,debug=FALSE){
  fxNa <- .composeCallName(callFrom,newNa="findSimilFrom2sets")
  namesXY <- c(deparse(substitute(predMatr)), deparse(substitute(measMatr)), deparse(substitute(colPre)), deparse(substitute(colMeas)))
  ## check input
  predMatr <- .vector2Matr(predMatr,colNa=if(length(dim(predMatr)) >1) colnames(predMatr) else namesXY[1])
  measMatr <- .vector2Matr(measMatr,colNa=if(length(dim(measMatr)) >1) colnames(measMatr) else namesXY[2])
  badColNa <- c("dif","ppm","nInRange","isBest","nBest")
  fxCh <- function(colN, matr, argN=NULL) { if(is.character(colN)) {      # check argument 'colN' as (character or number) to matrix 'matr', return numeric index
    msg <- c("Problem to ","find colname from ","locate number from "," reset to default=1")
    ch <- which(colN==colnames(matr))
    if(length(ch) >0) ch[1] else {message(msg[1:2],argN,msg[4]); 1}
  } else {colN <- as.integer(colN); if(colN <1 || colN >ncol(matr)) {colN <- 1; message(msg[c(1,3)],argN,msg[4])} else colN}}
  colMeas <- fxCh(colMeas, measMatr, namesXY[4])
  colPre <- fxCh(colPre, predMatr, namesXY[3])
  if(length(badColNa) >0) if(any(badColNa %in% colnames(measMatr))) message(fxNa,"Problem with colnames of ",namesXY[2])
  ## main
  prefMatch <- c("^x","^y")                  # fixed, since findCloseMatch() called wo initial names
  closeMatch <- findCloseMatch(as.numeric(predMatr[,colPre]), as.numeric(measMatr[,colMeas]), compTy=compareTy, limit=limitComp, sortMatch=FALSE, silent=silent, debug=debug, callFrom=fxNa)
  if(debug) {message(fxNa,"xxfindSimilFrom2sets2\n")}
  out <- closeMatchMatrix(closeMatch=closeMatch, predMatr=predMatr, measMatr=measMatr, prefMatch=prefMatch,
    colMeas=colMeas, colPred=colPre, limitToBest=bestOnly, callFrom=fxNa,debug=debug,silent=silent)
  if(!is.null(out)) { 
    colnames(out)[1:ncol(predMatr)] <- colnames(predMatr)
    colnames(out)[1:ncol(measMatr) +ncol(predMatr) +1] <- colnames(measMatr)
    out <- out[order(out[,colPre]),]}             # order by predicted
  out }
  

#' Find Close Numeric Values Between Two Vectors
#'
#' \code{findCloseMatch} finds close matches (similar values) between two numeric vectors ('x','y') based on method 'compTy' and threshold 'limit'. 
#'  Return list with close matches of 'x' to given 'y', the numeric value dependes on 'sortMatch' (if FALSE then always value of 'y' otherwise of longest of x&y).  
#'  Note: Speed & memory improvement if 'sortMatch'=TRUE (but result might be inversed!): adopt search of x->y or y->x to searching matches of each longest to each shorter  (ie flip x &y).
#'  Otherwise, if length of 'x' & 'y' are very different, it may be advantagous to use a long(er) 'x'  and short(er) 'y' (with 'sortMatch'=FALSE).  
#'  Note: Names of 'x' & 'y' or (if no names) prefix letters 'x' & 'y' are always added as names to results.
# ' Note: Takes much RAM if x & y are large 
#'
#' @param x numeric vector for comparison
#' @param y numeric vector for comparison
#' @param compTy (character) may be 'diff' or 'ppm', will be used with threshold from argument 'limit'
#' @param limit (numeric) threshold value for retaining values, used with distace-type specified in argument 'compTy'
#' @param asIndex (logical) optionally rather report index of retained values
#' @param maxFitShort (numeric) limit output to max number of elements (avoid returning high number of results if filtering was not enough stringent)
#' @param sortMatch (logical) if TRUE than matching will be preformed as 'match longer (of x & y) to closer', this may process slightly faster (eg 'x' longer: list for each 'y' all 'x' that are close, otherwise list of each 'x'), 
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a list with close matches of 'x' to given 'y', the numeric value dependes on 'sortMatch' (if FASLE then always value of 'y' otherwise of longest of x&y)
#' @seealso \code{\link{checkSimValueInSer}} and (from this package) \code{.compareByDiff}, for convient output \code{\link{countCloseToLimits}} 
#' @examples
#' aa <- 11:14 ; bb <- c(13.1,11.5,14.3,20:21)
#' findCloseMatch(aa,bb,com="diff",lim=0.6)
#' findCloseMatch(c(a=5,b=11,c=12,d=18),c(G=2,H=11,I=12,J=13)+0.5, comp="diff", lim=2)
#' findCloseMatch(c(4,5,11,12,18),c(2,11,12,13,33)+0.5, comp="diff", lim=2)
#' findCloseMatch(c(4,5,11,12,18),c(2,11,12,13,33)+0.5, comp="diff", lim=2, sort=FALSE)
#' .compareByDiff(list(c(a=10,b=11,c=12,d=13),c(H=11,I=12,J=13,K=33)+0.5),limit=1) #' return matrix
#' 
#' a2 <- c(11:20); names(a2) <- letters[11:20]
#' b2 <- c(25:5)+c(rep(0,5),(1:10)/50000,rep(0,6)); names(b2) <- LETTERS[25:5]
#' which(abs(b2-a2[8]) < a2[8]*1e-6*5)                                     #'  find R=18 : no10
#' findCloseMatch(a2, b2, com="ppm", lim=5)                                #'  find Q,R,S,T
#' findCloseMatch(a2, b2, com="ppm", lim=5,asI=TRUE)                       #'  find Q,R,S,T
#' findCloseMatch(b2, a2, com="ppm", lim=5,asI=TRUE,sort=FALSE)
#' findCloseMatch(a2, b2, com="ratio", lim=1.000005)                       #'  find Q,R,S,T
#' findCloseMatch(a2, b2, com="diff", lim=0.00005)                         #'  find S,T
#' @export
findCloseMatch <- function(x, y, compTy="ppm", limit=5, asIndex=FALSE, maxFitShort=100, sortMatch=FALSE, silent=FALSE, debug=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom, newNa="findCloseMatch")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  compMeth <- c("ppm","diff","ratio")
  msg <- c(paste("Argument 'compTy' may be one of",pasteC(compMeth,qu="'",la=" or "))," , trimming to length=1")
  if(length(compTy) <1) stop(msg[1]) else if(length(compTy)>1) {compTy <- compTy[1]; message(msg)}
  if(!compTy %in% compMeth) stop(msg[1])
  if(is.null(names(x))) names(x) <- paste0("x",equLenNumber(1:length(x)))                   # need default names to know which of 'x' in results !
  if(is.null(names(y))) names(y) <- paste0("y",equLenNumber(1:length(y)))                   #
  if(any(names(x) %in% names(y))) message(fxNa,"Note that some names do overlap (beware not to confuse...) !")
  dat <- list(x,y)
  if(sortMatch) dat <- dat[order(sapply(dat,length), decreasing=FALSE)]
  if(is.character(maxFitShort)) if(length(grep("%$", maxFitShort)) >0) {
    maxFitShort <- ceiling(length(dat[[1]])*as.numeric(sub("%$","",maxFitShort))/100)
    } else stop("Can't interpret 'maxFitShort'")
  tm2 <- switch(compTy,
    ppm=.compareByPPM(dat, limit, distVal=!asIndex),
    diff=.compareByDiff(dat, limit, distVal=!asIndex),
    ratio=.compareByLogRatio(dat, limit, distVal=!asIndex))
  if(debug) {message(fxNa,"fCM1")}  
  cSu <- colSums(!is.na(tm2))
  cPi <- which(cSu >0)
  if(length(cPi) <1) return(NULL) else {
    if(any(cSu > maxFitShort)) {     # case of fitting close to (very) large number of elements -> keep maxFitShort lowest (+ set others to NA)
      if(!silent) message(fxNa,sum(cSu > maxFitShort,na.rm=TRUE)," column elements at too many 'close elements' try to reduce ..")
      for(i in 1:which(cSu > maxFitShort)) {limDis <- sort(tm2[,i], decreasing=TRUE, na.last=TRUE)[maxFitShort]
        tmX <- which(tm2[,i] <= limDis)
        if(nrow(tm2) > maxFitShort && length(tmX) >= min(maxFitShort*1.15, floor(nrow(tm2)*0.96))) tm2[,i] <- NA else tm2[which(tm2[,i] >limDis),i] <- NA}
      cPi <- which(colSums(!is.na(tm2)) >0)        # refresh
    }
    if(debug) {message(fxNa,"fCM2")}  
    rSu <- rowSums(!is.na(tm2))     
    if(any(rSu > maxFitShort)) {               # case of fitting close to (very) large number of elements -> keep maxFitShort lowest (+ set others to NA)
      if(!silent) message(fxNa,sum(rSu > maxFitShort ,na.rm=TRUE)," row elements at too many 'close elements' try to reduce ..")
      for(i in 1:which(rSu > maxFitShort)) {limDis <- sort(tm2[i,],decreasing=TRUE,na.last=TRUE)[maxFitShort]
        tmY <- which(tm2[i,] <= limDis)
        if(ncol(tm2) > maxFitShort && length(tmY) >= min(maxFitShort*1.15,floor(ncol(tm2)*0.96))) tm2[i,] <- NA else tm2[i,which(tm2[i,] >limDis)] <- NA}
      rPi <- which(rowSums(!is.na(tm2)) >0)        # refresh (rows with some distance values)
    } else rPi <- which(rSu >0)
    if(debug) {message(fxNa,"fCM3")}  
    zz <- if(nrow(tm2) >1) as.matrix(tm2[,cPi])[rPi,] else matrix(tm2[,cPi], nrow=1, dimnames=list(rownames(tm2),colnames(tm2)[cPi]))
	if(length(dim(zz)) <2) zz <- matrix(zz, nrow=length(rPi))
    if(is.null(colnames(zz)) || is.null(rownames(zz))) dimnames(zz) <- list(rownames(tm2)[rPi], colnames(tm2)[cPi])
    out <- if(asIndex) {if(length(cPi) >1) apply(!is.na(zz), 2, which) else which(!is.na(zz))         #list of indexes   dat,function(z) which(z))
    } else {if(length(cPi) >1) apply(zz, 2, naOmit) else naOmit(zz)} 
    if(!is.list(out)) {out <- as.list(out); zn <- rownames(zz)[apply(!is.na(zz),2,which)]; for(j in 1:length(out)) names(out[[j]]) <- zn[j]} 
    if(is.null(names(out))) names(out) <- rep(names(cPi), length(out))[1:length(out)]
  out }}

 
#' Compare by PPM
#'
#' This function allows to compare by ppm
#'
#' @param dat list of 2 numerical vectors
#' @param limit (numeric, length=1) threshold value for retaining values, used with distace-type specified in argument 'compTy'
#' @param distVal (logical) to toggle outpout as matrix of numeric (distance values above 'limit', others NA) or matrix of logical 
#' @return This function returns a list with close matches of 'x' to given 'y', the numeric value dependes on 'sortMatch' (if FASLE then always value of 'y' otherwise of longest of x&y)
#' @seealso \code{\link{findCloseMatch}},  \code{\link{checkSimValueInSer}}, and also \code{.compareByDiff}, for convient output \code{\link{countCloseToLimits}} 
#' @examples
#' cc <- list(aa=11:14, bb=c(13.1,11.5,14.3,20:21))
#' .compareByPPM(cc, 1) 
#' @export
.compareByPPM <- function(dat, limit, distVal=FALSE){
  ## compare both vectors from list 'dat' for similar values based on ppm
  ## 'dat' .. list of 2 numerical vectors
  ## 'distVal'.. (logical) to toggle outpout as matrix of numeric (distance values above 'limit', others NA) or matrix of logical
  ## 'limit'.. (numeric, length=1) threshold to be applied
  ## return logical matrix with rows for 1st & cols for 2nd element of dat  (used in findCloseMatch() )
  ref <- matrix(rep(dat[[1]],each=length(dat[[2]])), nrow=length(dat[[2]]), dimnames=list(names(dat[[2]]),names(dat[[1]])))       # matr of short
  que <- matrix(rep(dat[[2]],length(dat[[1]])), nrow=length(dat[[2]]), dimnames=list(names(dat[[2]]),names(dat[[1]])))
  if(distVal) {tm2 <- (2*(que <= ref) -1)*abs(que/ref -1)/1e-6; 
    tm2[which(!abs(que/ref -1) < limit*1e-6)] <- NA
  } else tm2 <- abs(que/ref -1) < limit*1e-6
  tm2 }


#' Compare by log-ratio
#'
#' This function allows to compare by log-ratio
#'
#' @param dat list of 2 numerical vectors
#' @param limit (numeric, length=1) threshold value for retaining values, used with distace-type specified in argument 'compTy'
#' @param distVal (logical) to toggle outpout as matrix of numeric (distance values above 'limit', others NA) or matrix of logical 
#' @return This function returns a list with close matches of 'x' to given 'y', the numeric value dependes on 'sortMatch' (if FASLE then always value of 'y' otherwise of longest of x&y)
#' @seealso  \code{\link{findCloseMatch}},  \code{\link{checkSimValueInSer}}, and also \code{.compareByDiff}, for convient output \code{\link{countCloseToLimits}} 
#' @examples
#' cc <- list(aa=11:14, bb=c(13.1,11.5,14.3,20:21))
#' .compareByLogRatio(cc, 1) 
#' @export
.compareByLogRatio <- function(dat, limit, distVal=FALSE){
  ## compare both vectors from 'dat' for similar values based on (log)ratio
  ## 'dat' .. list of 2 numerical vectors
  ## 'distVal'.. (logical) to toggle outpout as matrix of numeric (distance values above 'limit', others NA) or matrix of logical
  ## 'limit'.. (numeric, length=1) threshold to be applied
  ## return logical matrix of  (used in findCloseMatch() )
  ref <- matrix(rep(dat[[1]], each=length(dat[[2]])), nrow=length(dat[[2]]), dimnames=list(names(dat[[2]]),names(dat[[1]])))       # matr of short
  que <- matrix(rep(dat[[2]], length(dat[[1]])), nrow=length(dat[[2]]), dimnames=list(names(dat[[2]]),names(dat[[1]])))
  if(distVal) {tm2 <- abs(log2(ref/que)); tm2[which(!abs(log2(ref/que)) <= log2(abs(limit)))] <- NA
  } else tm2 <- abs(log2(ref/que)) <= log2(abs(limit))
  tm2 }


#' Compare by distance/difference
#'
#' This function allows to compare by distance/difference
#'
#' @param dat list of 2 numerical vectors
#' @param limit (numeric, length=1) threshold value for retaining values, used with distace-type specified in argument 'compTy'
#' @param distVal (logical) to toggle outpout as matrix of numeric (distance values above 'limit', others NA) or matrix of logical 
#' @return This function returns a list with close matches of 'x' to given 'y', the numeric value dependes on 'sortMatch' (if FASLE then always value of 'y' otherwise of longest of x&y)
#' @seealso  \code{\link{findCloseMatch}},  \code{\link{checkSimValueInSer}}, and also \code{.compareByLogRatio}, for convient output \code{\link{countCloseToLimits}} 
#' @examples
#' cc <- list(aa=11:14, bb=c(13.1,11.5,14.3,20:21))
#' @export
.compareByDiff <- function(dat, limit, distVal=FALSE){
  ## compare both vectors from 'dat' for similar values based on (absolute) difference
  ## 'dat' .. list of 2 numerical vectors
  ## 'distVal'.. (logical) to toggle outpout as matrix of numeric (distance values above 'limit', others NA) or matrix of logical
  ## 'limit'.. (numeric, length=1) threshold to be applied
  ## return logical matrix of  (used in findCloseMatch() )
  ref <- matrix(rep(dat[[1]], each=length(dat[[2]])), nrow=length(dat[[2]]), dimnames=list(names(dat[[2]]),names(dat[[1]])))       # matr of short
  que <- matrix(rep(dat[[2]], length(dat[[1]])), nrow=length(dat[[2]]), dimnames=list(names(dat[[2]]),names(dat[[1]])))
  if(distVal) {
    tm2 <- que-ref
    chLi <- abs(tm2) > abs(limit)
    if(any(chLi)) tm2[which(chLi)] <- NA
  } else tm2 <- abs(ref -que) <= abs(limit)
  tm2 }
   

#' Calculate all ratios between x and y
#'
#' This function calculates all possible pairwise ratios between all individual calues of x and y, or samples up to a maximum number of combinations.
#' 
#' @param x (numeric) vector, numerator for constructing rations
#' @param y (numeric) vector, denominator for constructing rations
#' @param maxLim (integer) allows reducing complexity by drawing for very long x or y
#' @param isLog (logical) adjust ratio calculation to log-data
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @examples
#' set.seed(2014); ra1 <- c(rnorm(9,2,1),runif(8,1,2))
#' ratioAllComb(ra1[1:9],ra1[10:17])
#' boxplot(list(norm=ra1[1:9], unif=ra1[10:17], rat=ratioAllComb(ra1[1:9],ra1[10:17])))
#' @export
ratioAllComb <- function(x, y, maxLim=1e4, isLog=FALSE, silent=FALSE, debug=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom, newNa="ratioAllComb")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  
  namesXY <- c(deparse(substitute(x)), deparse(substitute(y)))
  if(!is.numeric(x)) x <- try(if(is.factor(x)) as.numeric(as.character(x)) else as.numeric(x), silent=TRUE)
  if(!is.numeric(y)) y <- try(if(is.factor(y)) as.numeric(as.character(y)) else as.numeric(y), silent=TRUE)
  msg <- " 'x' and 'y' must be numeric and length >0. Can't convert !"
  if(inherits(x, "try-error")) stop(fxNa,msg,namesXY[1]," to numeric")
  if(inherits(y, "try-error")) stop(fxNa,msg,namesXY[2]," to numeric")
  if(length(x) > maxLim){
    if(!silent) message(fxNa,"Reducing x from to ",length(x)," to user-selectend length ",maxLim)
    x <- sample(x, size=maxLim, replace=FALSE) }
  if(length(y) > maxLim){
    if(!silent) message(fxNa,"Reducing y from to ",length(y)," to user-selectend length ",maxLim)
    y <- sample(y, size=maxLim, replace=FALSE) }
  if(!silent) {nMax <- prod(length(x), length(y))
    if(nMax > 1e8) message(fxNa,"Calculations may take long time ! (",signif(nMax,4)," combinations !)")}
  rat <- if(isLog) rep(x, each=length(y)) -rep(y, length(x)) else rep(x,each=length(y))/rep(y,length(x))
  rat }

#' Bring most extreme to center
#'
#' This function aims to bring most extreme value to center
#'
#' @param aa (numeric) main input
#' @param ctr (numeric) 'control'
#' @param ctrFa (numeric <1) modulate amplitude of effect
#' @return This function returns an adjusted numeric vector
#' @seealso  \code{\link[stats]{dist}}
#' @examples
#' .bringToCtr(11:14, 9)
#' @export
.bringToCtr <- function(aa, ctr, ctrFa=0.75){
  ## bring most extreme value of 'aa' (numeric vector) closer to 'ctr'
  ## 'ctrFa' .. (numeric <1) modulate amplitude of effect
  ## return adjusted numeric vector
  di <- abs(aa-ctr)
  sel <- which(di ==max(di,na.rm=TRUE))
  aa[sel] <- ctr +max(di[if(length(aa) <3) which.min(di) else order(di)[length(aa) -1]],
    ctrFa*sum(di, na.rm=TRUE))*(((aa -ctr)/di)[sel])/sum(!is.na(di))
  aa }

#' Get series of values after last discontinuity
#'
#' This function aims to get series of values after last discontinuity
#'
#' @param x (numeric) main input
#' @param getFrom (character)
#' @return This function returns a numeric vector of reduced length
#' @seealso  \code{\link[stats]{dist}}
#' @examples
#' .breakInSer(c(11:14,16:18))
#' @export
.breakInSer <- function(x, getFrom="last") {
  ## get series of values after last discontinuity
  ## 'x' as vector of numeric values
  ## 'getFrom' to decide if to extract from beginning or form from end (default)
  diffPos <- which(diff(x) >1)
  extrFrom <- if(length(diffPos) >0) {if(getFrom=="last") max(diffPos) +1 else min(diffPos)
  } else if(getFrom=="last") 1 else length(x)
  extrTo <- if(getFrom=="last") length(x) else 1
  x[sort(extrFrom:extrTo)] }

#' Remove columns indicated by col-number
#'
#' This function aims to remove columns indicated by col-number
#'
#' @param matr (matrix or data.frame) main input
#' @param rmCol (integer) column index for removing
#' @return This function returns an matrix or data.frame
#' @seealso  \code{\link[stats]{dist}}
#' @examples
#' aa <- matrix(1:6, ncol=3)
#' .removeCol(aa, 2)
#' @export
.removeCol <- function(matr, rmCol) {
  ## remove columns indicated by col-number
  ## verifies that dimnames don't get lost and that output is not converted to simple vector (if 1 col remains)
  if(is.null(dim(matr))) stop(" expecting matrix or data.frame")
  if(length(rmCol) >0) {
    if(max(rmCol) > ncol(matr)) stop(" 'rmCol' must not be higher than nuber of columns of 'matr'")
    iniDimNa <- dimnames(matr)
    matr <- matr[,-1*rmCol]
    if(is.null(dim(matr)))  matr <- matrix(matr, ncol=1, dimnames=list(iniDimNa[[1]], iniDimNa[[2]][-1*rmCol]))
  }
  matr }

#' Reorganize array by reducing dimension 'byDim'  (similar to stack() for data-frames)
#'
#' This function aims to reorganize an array by reducing dimension 'byDim'  (similar to stack() for data-frames)
#' It returns an array/matrix of 1 dimension less than 'arr', 1st dim has more lines (names as paste with '_')
#'
#' @param arr (array) main input
#' @param byDim (integer)
#' @return This function returns an array/matrix of 1 dimension less than 'arr', 1st dim has more lines (names as paste with '_')
#' @seealso  \code{\link[stats]{dist}}
#' @examples
#' (arr1 <- array(11:37, dim=c(3,3,3)))
#' .stackArray(arr1, 3)
#' @export
.stackArray <- function(arr, byDim=3){
  ## reorganize array by reducing dimension 'byDim'  (similar to stack() for data-frames)
  ## returns array/matrix of 1 dimension less than 'arr', 1st dim has more lines (names as paste with '_')
  if(byDim > length(dim(arr))) stop(" 'byDim' should indicate the dimension-number to be supressed")
  dimNa <- dimnames(arr)
  useDi <- (1:length(dimNa))[-1*byDim]
  arr <- apply(arr,byDim,function(x) x)
  dimnames(arr) <- list(paste(rep(dimNa[[useDi[1]]], length(dimNa[[useDi[2]]])),
    rep(dimNa[[useDi[2]]], each=length(dimNa[[useDi[1]]])), sep="_"), dimNa[[byDim]])
  arr }

#' Check argument for Location of legend
#'
#' This function allows checking an argument for Location of legend,
#' if value provided not found as valid, it returns 'defLoc
#'
#' @param legLoc (character) main input
#' @param defLoc (character)
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a character vector designing the potential location of legend
#' @seealso  \code{\link[graphics]{legend}}
#' @examples
#' .checkLegendLoc("abc")
#' @export
.checkLegendLoc <- function(legLoc, defLoc="topright", silent=FALSE, debug=FALSE, callFrom=NULL){
  ## check argument for Location of legend
  ## if value provided not found as valid, use 'defLoc'
  fxNa <- .composeCallName(callFrom, newNa=".checkLegendLoc")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  if(is.null(legLoc)) legLoc <- defLoc
  if(!is.character(legLoc) || length(legLoc) >1) legLoc <- defLoc
  if(!(tolower(legLoc) %in% c("topleft","topright", "top","bottomright", "bottomleft", "bottom", "left","right","center"))) {
    legLoc <- defLoc
    if(!silent) message(fxNa,"Can't interpret value of 'legLoc', setting to default '",defLoc,"'") }
  tolower(legLoc) }

#' Avoid duplicating items between 'curNa' and 'newNa' by incrementing digits after 'extPref' (in newNa)
#'
#' This function aims to avoid duplicating items between 'curNa' and 'newNa' by incrementing digits after 'extPref' (in newNa)
#' 
#' @param newNa (character) main input 1
#' @param curNa (character) main input 2
#' @param extPref (character) extension
#' @return This function returns the corrected input vector \code{newNa}
#' @seealso  \code{\link[base]{duplicated}}
#' @examples
#' .corDuplItemsByIncrem(letters[1:6], letters[8:4])
#' @export
.corDuplItemsByIncrem <- function(newNa, curNa, extPref="_s") {
  ## avoid duplicating items between 'curNa' and 'newNa' by incrementing digits after 'extPref' (in newNa)
  ## 'newNa' .. character vector for new names (to be adjusted/corrected)
  ## 'curNa' .. character vector for references of names
  ## cannot handle NAs !
  ## return corrected 'newNa' (character vector)
  chSear <- unlist(regexec(paste0(extPref,"[[:digit:]]+$"), newNa))           # for finding extension at very end
  if(all(chSear==-1)) corExt <- newNa else {
    tt <- chSea2 <- unlist(regexec(paste0(extPref,"[[:digit:]]"),newNa))       # for finding extension at first of mult occurance
    whS <- which(chSear >0)
    tt[tt==-1] <- nchar(newNa)[tt==-1] +1
    newExt <- cbind(bas=substr(newNa, 1, tt -1),ext=substr(newNa, tt +2, nchar(newNa)))
    chSear <- unlist(regexec(paste0(extPref,"[[:digit:]]+$"),curNa))     # for finding extension at very end
    chSea2 <- unlist(regexec(paste0(extPref,"[[:digit:]]"),curNa))       # for finding extension at first of mult occurance
    whS <- which(chSear >0)
    curExt <- cbind(bas=substr(curNa[whS], 1, chSea2[whS] -1),
      ext=substr(curNa[whS],chSear[whS]+2,nchar(curNa)[whS]))
    curMax <- tapply(as.numeric(curExt[,2]), curExt[,1], max)
    for(i in 1:length(curMax)) { uu <- which(newExt[,1]==names(curMax)[i])
      newExt[uu,2] <- 1+ curMax[i]:(curMax[i] +length(uu) -1) }
    corExt <- paste(newExt[,1], newExt[,2], sep=extPref)
    if(any(newExt[,2]=="")) corExt[which(newExt[,2]=="")] <- newExt[which(newExt[,2]==""),1] }
  corExt }
   

#' System-date (compressed format)  
#'
#' This function returns current date (based on Sys.Date) in different format options.
#' 
#' @details   
#' Multiple options for formatting exist :
#' 'univ1' or 'wr' ... (default) compact sytle using day, first 3 letters of English name of month (lowercaps) and last 2 letters of year as ddmmmyy, eg 14jun21 
#' 
#' 'univ2' ... as ddMmmyy, eg 14Jun21
#' 
#' 'univ3' ... as ddMonthyyyy, eg 14June2021
#' 
#' 'univ4' ... as ddmonthyyyy, eg 14june2021
#' 
#' 'univ5' ... as yyyy-mm-dd (output of \code{Sys.Date()}), eg 2021-06-14
#' 
#' 'univ6' ... as yyyy-number of day (in year), eg 2021-165
#' 
#' 'local1' ... compact sytle using day, first 3 letters of current locale name of month (not necessarily unique !) and last 2 letters of year as ddmmmyy, eg 14jui21 
#' 
#' 'local2' ... as ddMmmyy, month based on current locale (not necessarily unique !), eg 14Jui21
#' 
#' 'local3' ... as ddMonthyyyy, month based on current locale , eg 14Juin2021
#' 
#' 'local4' ... as ddmonthyyyy, month based on current locale , eg 14juin2021
#'
#' 'local5' ... as dd-month-yyyy, month based on current locale , eg 14-juin-2021
#'
#' 'local6' ... as yyyymonthddd, month based on current locale , eg 2021juin14
#'
#' 
#' 
#' @param style (character) choose style (default 'univ1' for very compact style)  
#' @return character vector with formatted date
#'
#' @seealso \code{\link[base]{date}}, \code{Sys.Date} and \code{\link[base]{Sys.time}},
#' @examples
#' sysDate() 
#' 
#' @export
sysDate <- function(style="univ1") {
  ## return date in compressed format  ('wr-style') day/month/year eg 15sep20
  if(length(style) <1) style <- "univ1" else if(any(is.na(style))) style <- "univ1"
  if("wr" %in% style) style <- "univ1"
  out <- switch(style,
    abc="abc",
    local1=paste0(format(Sys.Date(),"%d"), substr(format(Sys.Date(),"%b"),1,3), substr(format(Sys.Date(),"%Y"),3,4)),  # ddmmmyy
    local2=paste0(format(Sys.Date(),"%d"), gsub("(^[[:alpha:]])", "\\U\\1", substr(format(Sys.Date(),"%b"),1,3), perl=TRUE), substr(format(Sys.Date(),"%Y"),3,4)),  # ddMmmyy
    local3=paste0(format(Sys.Date(),"%d"), gsub("(^[[:alpha:]])", "\\U\\1", format(Sys.Date(),"%b"), perl=TRUE), format(Sys.Date(),"%Y")),           # ddMonthyy
    local4=format(Sys.Date(), "%d%b%Y"),           # ddmonthyyyy
    local5=format(Sys.Date(),"%v"),                # separated by -, dd-month-yyyy
    local6=format(Sys.Date(), "%Y%b%d"),           # yyyymonthdd
    univ1=paste0(format(Sys.Date(),"%d"), tolower(month.abb[as.integer(format(Sys.Date(),"%m"))]), substr(format(Sys.Date(),"%Y"),3,4)),   # ddmmmyy
    univ2=paste0(format(Sys.Date(),"%d"), month.abb[as.integer(format(Sys.Date(),"%m"))], substr(format(Sys.Date(),"%Y"),3,4)),            # ddMmmyy 
    univ3=paste0(format(Sys.Date(),"%d"), month.name[as.integer(format(Sys.Date(),"%m"))], format(Sys.Date(),"%Y")),  # ddmonthyyyy
    univ4=paste0(format(Sys.Date(),"%d"), tolower(month.name[as.integer(format(Sys.Date(),"%m"))]), format(Sys.Date(),"%Y")),  # ddmonthyyyy
    univ5=Sys.Date(),                                         #yyyy-mm-dd
    univ6=format(Sys.Date(), "%Y-%j") )
  out }  
    

#' Choose Column Most Likely For Sample-Names 
#'
#' This function looks at all comumns of  \code{mat} which columns may be likely choices for sample-names and derives then group-names after stripping terminal enumerators.
#' Ideal sample-names should contain some replicates indicates as terminal enumerators.
#'
#' @details
#' The basic idea is that the column containing (good) samples-names contains all different entries and that by stripping terminal enumerators one can understand the grouping of replicates.
#' Note arguments  \code{asUnique} and \code{partEnumerator} influence which columns of \code{mat} will be evaluated/checked
#'
#'
#' @param mat (matrix or data.frame) contains possible choices for sample-names
#' @param useCoNa (character) optional custom choice for columns of \code{mat} to check; if \code{NULL} all columns will be used/checked
#' @param method (character) decide how to choose number of groups as : min, low, med, high, max or mode
#'   Note arguments  \code{asUnique} and \code{partEnumerator} influence which columns of \code{mat} will be evaluated/checked
#' @param sep (character) separators considered when searching and removing common words
#' @param rmTxt (character, length=1) optional removing of custom text (eg variable file-extensions); no obligation that \code{rmTxt} occurs in all instances
#' @param asUnique (logical) requires all (potential) samples-names to be unique (ie no repeats) to be considered for group-names; also removes all candidate columns with all different names
#' @param partEnumerator (logical) when \code{TRUE} allows some instances of (potential) sample-names without numerator: a1, a2, b (ie some wo enumerator)
#' 
#' @param fullReport (logical) if \code{TRUE} returns list with $group, $sampleNames, $col (iondex of column from mat and name of c)
#' 
#' @param silent (logical) suppress messages if \code{TRUE}
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allows easier tracking of messages produced
#' @return This function returns a character vector with grouop-names (and sample-names as names of entries) or 
#'   if \code{fullReport=TRUE} a list with $group, $sampleNames, $col (index of column from \code{mat} and name of column
#' 
#' @seealso \code{\link{rmSharedWords}}, \code{\link{replicateStructure}}, \code{\link{protectSpecChar}}
#' @examples
#'  mat <- cbind(a=letters[1:6], b=paste(rep(c("b","B"), each=3), 1:3), c=rep(1,6), 
#'    d=gl(3,2), e=rep(c("e","E"),3), f=paste(rep(c("F","f","ff"), each=2), 1:2))
#' chooseGroupNames(mat, method="median")         # col 2 (b/B)
#' chooseGroupNames(mat, method="median", fullReport=TRUE) 
#' chooseGroupNames(mat, method="min")            # col 2 (b/B)
#' chooseGroupNames(mat, method="max")            # col 6 (F/f/ff)
#' chooseGroupNames(mat, method="max", asUnique=FALSE) # col 1 (a..)
#' @export
chooseGroupNames <- function(mat, useCoNa=NULL, method="median", sep=c("_","-"," ",".","=",";"), rmTxt=NULL, asUnique=TRUE, partEnumerator=FALSE, fullReport=FALSE, silent=FALSE, debug=FALSE, callFrom=NULL) {
  ## Each column of mat gets inspected if it may contain sample-names from which group-names could be derived by dropping terminal enumerators
  ## returns vector  with group-names, names of vector as sample-names; In case of multiple possible hist the 1st is returned
  ## Aim : from 'mat' matrix (or data.frame) with possible choices (per column) choose most likely to use as sample-names :
  ##   should contain some replicates after stripping terminal enumerators
  ## method : how to choose number of groups lin, low, med, high, max, mode
  ## sep (character) separator for common words and/or terminal enumerators
  ## rmTxt (character) : remove text (no obligation to occur in all cases, eg variable writing of extensions)
  ## asUnique . requires all (potential) samples-names to be unique (ie no repeats) to be considered for group-names; also removes all candidate columns with all different names
  ## partEnumerator =T : allow : a1, a2, b (ie some wo enumerator)
  ## fullReport .. return list with $group, $sampleNames, $col (iondex of column from mat and name of c)
  ## find column containing suitable sample-names out of matrix or data.frame : remove extension .raw, simplify by removing shared words
  fxNa <- .composeCallName(callFrom, newNa="getSampleNames")
  if(isTRUE(debug)) silent <- FALSE
  if(!isTRUE(silent)) silent <- FALSE
  chPat <- "[[:digit:]]+[[:space:]]*$"
  out <- NULL
  ## check input (useCoNa)
  iniColNa <- colnames(mat)
  if(length(mat) <1) warning("Nothing to do ..") else {
    if(length(dim(mat)) <1 && !is.list(mat)) mat <- matrix(mat, ncol=1, dimnames=list(names(mat), "col1"))}
  if(length(useCoNa) >0) {useCoNa <- useCoNa[which(useCoNa %in% colnames(mat))]
    if(length(useCoNa) <1 && !silent) message(fxNa,"None of specified columns found !! (using all available)") }
  if(length(useCoNa) <1 || all(is.na(useCoNa))) useCoNa <- colnames(mat) 
  ## remove file-ending, remove shared words
  saNa <- lapply(useCoNa, function(z) { if(z %in% colnames(mat)) {
    saNa1 <- if(length(rmTxt) ==1) sub(rmTxt,"", mat[,z]) else mat[,z]        # remove text
    rmSharedWords(saNa1, sep=sep, silent=silent, callFrom=fxNa)} })           # simplify
  if(debug) {message("cGN1")}

  ## optional filter for columns with unique (sample-)names only AND no NAs
  if(isTRUE(asUnique)) { chSa <- sapply(saNa, function(z) length(unique(z))==length(z) && !any(is.na(z)) )
    if(all(!chSa)) { saNa <- NULL
      warning(fxNa,"All of the potential columns for sample-names have non-unique sample-names or contains NAs (returning NULL)")
    } else {
      if(any(!chSa)) {chSa <- which(chSa); mat <- mat[, chSa, drop=FALSE]; useCoNa <- useCoNa[chSa]; saNa <- saNa[chSa]}}     
  }

  ## remove enumerator extensions
  if(length(saNa) >0) { 
     rmExt <- function(z, chPat, allEnumerator=TRUE) { out <- if(allEnumerator) {   # remove terminal enumerator: allEnumerator=TRUE : only when everywhere (ie don't allow empty)
        if(all(grepl(chPat, z))) sub(paste0("(",paste0(protectSpecChar(sep), collapse="|"),")*", "[[:digit:]]+[[:space:]]*$"),"", z) else z
      } else sub(paste0("(",paste0(protectSpecChar(sep), collapse="|"),")*", "[[:digit:]]+[[:space:]]*$"),"", z)
      if(any(nchar(out) <1, na.rm=TRUE)) out <- z
      out }
    ls2 <- lapply(saNa, rmExt, chPat, allEnumerator=!isTRUE(partEnumerator))
    if(debug) {message(fxNa,"cGN2")}
    ## check/choose for best human-readable, ie no-of-chars >0, text-numeric combination & containing some 'replicates' (but not all replic of 1st) 
    if(length(ls2) >1) { 
      nGr2 <- nGrp <- sapply(ls2, function(z) length(unique(naOmit(z))))
      if(isTRUE(asUnique)) nGr2[which(nGrp==1 | nGrp==nrow(mat))] <- NA 
      hasNA <- sapply(ls2, function(z) any(is.na(z)))
      nGrS <- sort(naOmit(nGr2))
      nGrX <- sort(unique(naOmit(nGr2)))           # overal possible numbers of groups
      if(TRUE && any(hasNA)) nGr2[which(hasNA)] <- NA
      modeX <- function(z) {y <- table(z); names(y)[which.max(y)]}      
      if(any(c("median","me") %in% method)) method <- "med"
      if(any(c("hi","hig","h","highest") %in% method)) method <- "high"
      if(any(c("lo","l","lowest") %in% method)) method <- "low"
      if(debug) message(fxNa,"Using method '",method,"'")
      useCol <- switch(method, 
        min= which(nGr2==min(nGr2, na.rm=TRUE))[1],
        low= which(nGr2==nGrX[1+floor(length(nGrX)/5)]),
        med= nGrS[floor(length(nGrS)/2)],  #which(nGr2==floor(median(nGr2, na.rm=TRUE)))[1],
        mode=which(nGr2==modeX(nGr2))[1],
        high=which(nGr2==nGrX[min(length(nGrX), 1+ceiling(4*length(nGrX)/5))]),
        max= which(nGr2==max(nGr2, na.rm=TRUE))[1] )
      if(length(useCol) <1) warning(fxNa,"Unable to understand 'method' (returning NULL)") else {
        out <- ls2[[useCol[1]]]
        names(out) <- saNa[[useCol[1]]] 
      } 
    } else {
      out <- ls2[[1]]
      names(out) <- saNa[[1]]  
    }
    if(isTRUE(fullReport)) {out <- list(group=out, sampleNames=names(out), col=which(iniColNa==colnames(mat)[useCol[1]])); names(out$col) <- useCoNa[useCol[1]]; out} else out 
  } else if(isTRUE(fullReport)) list(group=NULL, sampleNames=NULL, col=NULL) else NULL
}  
 
    

#' Summarize columns (as median,mean,min,last or other methods)
#'
#' \code{summarizeCols} summarizes all columns of matrix (or data.frame).
#' In case of text-columns the sorted middle (~median) will be given, unless 'maxAbsLast', 'minAbsLast',
#'  .. consider only last column of 'matr' : choose from all columns the line where (max of) last col is at min;
#'  'medianComplete' or 'meanComplete' consideres only lines/rows where no NA occur (NA have influence other columns !)
#' 
#' @details
#' The argument \code{method} allows options that treat (summarize) all columns independently or to select one line (based on argument \code{refCol})
#' 
#' @param matr data.frame matrix of data to be summarized by comlumn (may do different method for text and numeric comlumns)
#' @param meth (character) summarization method, may be 'mean','aver','median','sd','CV', 'min','max','first','last','maxOfRef','minOfRef','maxAbsLast','minAbsLast',
#'  'medianComplete' or 'meanComplete', 'n' (number of non-\code{NA} elements),'n.NA' (number of \code{NA} elements), 'NULL' (returns \code{NULL})
#' @param refCol (character or integr) column to be used as reference
#' @param nEqu (logical) if \code{TRUE}, add additional column indicating the number of equal lines for choice (only with min or max) 
#' @param supl (numeric) supplemental parameters for the various summarizing functions (eg used with \code{meth="trimmedMean", supl=c(l=0.1,u=0.2)} to pass arguments to \code{\link{trimmedMean}})
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging 
#' @param callFrom (character) allows easier tracking of messages produced
#' @return vector with summary for each column
#' @seealso \code{\link[base]{colSums}}; if data has subgroups to be used in a \code{\link[base]{tapply}}-way please see \code{\link{makeNRedMatr}}
#' @examples
#' t1 <- matrix(round(runif(30,1,9)),nc=3); rownames(t1) <- letters[c(1:5,3:4,6:4)]
#' summarizeCols(t1, me="median")
#' t(sapply(by(t1,rownames(t1), function(x) x), summarizeCols,me="maxAbsLast"))
#' t3 <- data.frame(ref=rep(11:15,3), tx=letters[1:15],
#'   matrix(round(runif(30,-3,2),1), ncol=2), stringsAsFactors=FALSE)
#' by(t3,t3[,1], function(x) x)
#' by(t3,t3[,1], function(x) summarizeCols(x, me="maxAbsLast"))
#' t(sapply(by(t3, t3[,1], function(x) x), summarizeCols, me="maxAbsLast"))
#' @export
summarizeCols <- function(matr, meth="median", refCol=NULL, nEqu=FALSE, supl=NULL, silent=FALSE, debug=FALSE, callFrom=NULL) {
  ## summarize all columns of matrix (or data.frame) 'x' (most methods will call apply)
  ## in case of text-columns the sorted middle (~median) will be given, unless 'maxAbsLast' or 'minLast'
  ##   'maxAbsLast' or 'minLast' .. consider only last column of 'matr' : choose from all columns the line where (max of) last col is at min,max...
  ##   'medianComplete' or 'meanComplete' consideres only lines/rows where no NA occur (NA have influence other columns !)
  ## return vector with summary for each column
  fxNa <- .composeCallName(callFrom, newNa="summarizeCols")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  
  argOpt <- c("median","mean","aver","average","trimmedMean","min","max","maxAbsLast","minAbsLast","maxOfRef","minOfRef","maxAbsOfRef","minAbsOfRef","firstLi","lastLi","first","last","sd","CV")
  argOpt <- c(argOpt, paste0(argOpt,"Complete"), "n","n.NA","Null")  
  txt <- c("Argument '","' should be "," seeting to first/default (meth='median')")
  if(length(dim(matr)) <2) {if(!silent) message(fxNa,txt[1],"matr",txt[2],"matrix or data.frame")
    if(length(dim(matr)) <1) {meth <- "Null"; matr=matrix(NA)} else matr <- as.matrix(matr)}
  if(length(meth) <1 && !silent) { message(fxNa,txt[3]); meth <- "median"}
  if(length(meth) >1) {if(!silent) message(fxNa,txt[1],"meth",txt[2]," of length=1, ",txt[3]); meth <- meth[1]}  
  if(!meth %in% argOpt) stop(fxNa,txt[1]," meth ",txt[2]," either one of : ",pasteC(argOpt,quoteC="'"))
  if(debug) { message(fxNa, "sC1"); sC1 <- list(matr=matr,meth=meth,nEqu=nEqu,refCol=refCol)}
  if(length(nEqu)==1 && isTRUE(nEqu)) nEqu <- grepl("(max)|(min)Of", meth)

  ## main
  if(meth=="Null") out <- NULL else { 
    if(grepl("Ref$", meth) && length(refCol)==1) {
      ## prepare for reference column
      chRe <- try(matr[,refCol], silent=TRUE)
      if(inherits(chRe, "try-error")) {
        meth <- sub("[[:alpha:]]+OfRef","", meth)
        if(!silent) message(fxNa,"Invalid entry for refCol, ignoring, setting meth to ",meth)
        refCol <- NULL 
      } else { matr <- cbind(matr, ref=chRe) }    # add ref for single-column methods
    }
    if(grepl("Last$", meth)) meth <- sub("Last$","OfRef", meth)
    
    ## now summarize
    if(debug) { message(fxNa, "sC2"); sC2 <- list()}
    out <- .summarizeCols(matr, me=meth, nEq=nEqu, supl=supl, silent=silent,debug=debug,callFrom=fxNa)
    if(grepl("Ref$", meth) && length(refCol)==1) out <- out[,-ncol(out)]      # in case of refLi - remove added col
  }    
  out }
  

#' Summarize columns of matrix (or data.frame) 'x' using apply (main)
#'
#' This function summarizes columns of matrix (or data.frame) 'x' using apply
#' In case of character entries the 'median' of sorted values will be returned
#' 
#'  
#' @param x data.frame matrix of data to be summarized by comlumn 
#' @param me (character, length=1) summarization method (eg 'min','max','mean','mean.trim','median','sd','CV', 'medianComplete' or 'meanComplete' etc, see \code{\link{summarizeCols}})
#' @param nEq (logical) if TRUE, add additional column indicating the number of equal lines for choice (only with min or max) 
#' @param vectAs1row (logical) if TRUE will interprete non-matrix 'x' as matrix with 1 row (correct effect of automatic conversion when extracting 1 line) 
#' @param supl (numeric) supplemental parameters for the various summarizing functions (currently used with 'me=mean.trim' to assign upper and lower trimming fraction, passed to )
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging 
#' @param callFrom (character) allow easier tracking of messages produced
#' @return vector with summary for each column (unless 'me=="summary"', in this case a matrix or list will be returned )
#' @seealso \code{\link{summarizeCols}},  \code{\link[base]{table}} \code{\link[base]{table}}
#' @examples
#' m1 <- matrix(c(28,27,11,12,11,12), nrow=2, dimnames=list(1:2,c("y","x","ref")))
#' .summarizeCols(m1, me="median")
#' @export
.summarizeCols <- function(x, me="median", nEq=FALSE, vectAs1row=TRUE, supl=NULL, silent=FALSE, debug=FALSE, callFrom=NULL) {
  ## summarize columns of matrix (or data.frame) 'x' using apply
  ## CANNOT handle character entries !  (all results will be NA)
  ## 'vectAs1row' .. if TRUE will interprete non-matrix 'x' as matrix with 1 row (correct effect of automatic conversion when extracting 1 line)
  ## me='maxOfRef','maxAbsOfRef' or 'minOfRef': return line where last col of 'x' is at (first) max (or min) ...
  ## me may be "median","medianComplete","mean","meanComplete","aver","average","trimmedMean","mean.trim","sd","CV","min","max","maxOfRef","minOfRef", "maxAbsOfRef","lastLi","last","firstComplete","first","firstLi","summary"
  ## me='lastLi' .. return last line of 'x'
  ## any term of me containing 'Complete' (eg 'firstComplete') ..  first filter to lines of 'x' wo any NA
  ## me='medianComplete' .. median only of 'x' where no NA per line
  ## me='summary' will return matrix instead of vector !! (eah col for init cols of 'x')
  fxNa <- .composeCallName(callFrom, newNa=".summarizeCols")
  if(me %in% c("me","med")) me <- "median" 
  if(me %in% c("av","aver","average")) me <- "mean"                                     # synonyms ..
  if(me %in% c("trimmedMean","trimMean")) me <- "mean.trim"                             # synonyms ..
  if(length(dim(x)) <2) {
    x <- if(vectAs1row) matrix(x, nrow=1, dimnames=list(NULL,names(x))) else as.matrix(x)        
    numCol <- 1
    txtCol <- NULL
  } else { if(is.data.frame(x)) {                                   # check for text
      numCol <- which(sapply(x, is.numeric))
      txtCol <- if(length(numCol) == ncol(x)) NULL else (1:ncol(x))[-numCol] 
    } else { numCol <- if(is.numeric(x)) 1:ncol(x) else NULL; txtCol <- if(is.numeric(x)) NULL else 1:ncol(x)}}
  if(length(numCol) <1 && any(c("mean","median") %in% me)) { me <- "first"
    if(!silent) message(fxNa,"Invalid method for all character-data, setting me to 'first'") }  
  
  if(length(grep("Complete", me)) >0) {                                              # reduce x to complete rows only 
    compl <- which(rowSums(is.na(x)) <1)
    me <- sub("Complete","",me)                                                    # term 'Complete' disappears from me ...
    if(length(compl) <1) me <- "Null"                                              # this way output will be NULL if 0 lines wo NAs
    x <- if(length(compl) >1) x[compl,] else if(length(compl) ==1) matrix(x[compl,], nrow=1, dimnames=list(rownames(x)[compl],colnames(x)))}
  if(debug) {message(fxNa,"sCc1   me=",me); sCc1 <- list()}
  
  out <- switch(me,  Null=NULL,
    maxOfRef= if(is.numeric(x[,ncol(x)])) x[which.max(x[,ncol(x)]),] else x[which(x[,ncol(x)]== sort(x[,ncol(x)], decreasing=TRUE)),],   # checks last column only
    minOfRef= if(is.numeric(x[,ncol(x)])) x[which.min(x[,ncol(x)]),] else x[which(x[,ncol(x)]== sort(x[,ncol(x)], decreasing=FALSE)),],
    maxAbsOfRef= if(is.numeric(x[,ncol(x)])) x[which.max(abs(x[,ncol(x)])),] else x[which(x[,ncol(x)]== sort(x[,ncol(x)], decreasing=TRUE)),],   # checks last column only
    minOfRef= if(is.numeric(x[,ncol(x)])) x[which.min(abs(x[,ncol(x)])),] else x[which(x[,ncol(x)]== sort(x[,ncol(x)], decreasing=FALSE)),],
    median=apply(x[,numCol], 2, stats::median, na.rm=TRUE),    #   all columns
    mean=colMeans(x[,numCol], na.rm=TRUE),
    mean.trim=apply(x, 2, trimmedMean, trim=supl, callFrom=fxNa, silent=silent), 
    top3mean=NA,   # apply(x, 2, top3mean, callFrom=fxNa),
    max=apply(x, 2, max, na.rm=TRUE), 
    min=apply(x, 2, min, na.rm=TRUE),
    summary=if(is.matrix(x)) apply(x, 2, summary) else sapply(x, summary),
    sd=colSds(x, callFrom=fxNa),
    CV=colCVs(x, callFrom=fxNa),
    n=colSums(!is.na(x)),               # will count number of non-NA values (per column)
    n.NA=colSums(is.na(x)),             # will count number of NA values (per column)
    lastLi= x[nrow(x),], last= x[nrow(x),], firstLi= x[1,], first=x[1,])
  if(length(dim(out)) <1) out <- if(is.matrix(x)) matrix(out, ncol=ncol(x), dimnames=list(NULL, colnames(x))) else {if(length(numCol)==1) as.data.frame(out) else t(as.data.frame(out))} 
  if(debug) {message(fxNa,"sCc2"); sCc2 <- list(x=x,out=out,me=me,numCol=numCol,txtCol=txtCol,nEq=nEq,silent=silent,debug=debug)}
    
  if(any(c("median","mean") %in% me) && length(txtCol) >0) { #out2 <- apply(as.matrix(x[,txtCol]), 2, function(y) y[which(y==sort(y)[ceiling(length(y)/2)]]))
    out2 <- apply(as.data.frame(x)[,txtCol], 2, function(y) sort(y)[ceiling(length(y)/2)]) 
    out0 <- x[1:nrow(out),]
    if(is.matrix(x) && length(dim(out0)) <1) out0 <- matrix(out0, nrow=nrow(out), dimnames=list(rownames(out), colnames(x)[numCol]))
    out0[,numCol] <- out
    if(is.matrix(x) && length(dim(out0)) <1) out0 <- matrix(out0, nrow=nrow(out), dimnames=list(rownames(out), colnames(x)[numCol]))
    out0[,txtCol] <- out2
    out <- out0; rm(out0,out2)   
  }  
  if(isTRUE(nEq) && grepl("Ref$", me)) { 
    if(!grepl("Ref$", me)) me <- "other"
    out <- cbind(out, nEqu=switch(me, other=NA,
      maxOfRef=sum(x[,ncol(x)]==max(x[,ncol(x)])), 
      minOfRef=sum(x[,ncol(x)]==min(x[,ncol(x)])),
      minAbsOfRef=sum(x[,ncol(x)]==min(abs(x[,ncol(x)]))),
      maxAbsOfRef=sum(x[,ncol(x)]==max(abs(x[,ncol(x)]))) ) ) }
  out }  

#' cbind to non-redundant
#'
#' \code{cbindNR} combines all matrixes given as arguments to non-redundant column names (by ADDING the number of 'duplicated' columns !). 
#' Thus, this function works similar to \code{cbind}, but allows combining multiple matrix-objects containing redundant column-names.
#' Of course, all input-matrixes must have the same number of rows !
#' By default, the output gets sorted by column-names.
#' Note, due to the use of '...' arguments must be given by their full argument-names, lazy evaluation might not recognize properly argument names.
#'
#' @param ... all matrixes to get combined in cbind way
#' @param convertDFtoMatr (logical) decide if output should be converted to matrix
#' @param sortOutput (logical) optional sorting by column-names
#' @param summarizeAs (character) decide of combined values should get summed (default, 'sum') or averaged ('mean')
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a matrix or data.frame (as cbind would return)
#' @seealso \code{\link[base]{cbind}}, \code{\link{nonAmbiguousNum}}, \code{\link{firstOfRepLines}}
#' @examples
#' ma1 <- matrix(1:6, ncol=3, dimnames=list(1:2,LETTERS[3:1]))
#' ma2 <- matrix(11:16, ncol=3, dimnames=list(1:2,LETTERS[3:5]))
#' cbindNR(ma1, ma2)
#' cbindNR(ma1, ma2, summarizeAs="mean")
#' @export
cbindNR <- function(..., convertDFtoMatr=TRUE, sortOutput=TRUE, summarizeAs="sum", silent=FALSE, debug=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom, newNa="cbindNR")
  inpL <- list(...)
  isDatafr <- sapply(inpL, is.data.frame)
  if(convertDFtoMatr && any(isDatafr) >0) {
    inpL[isDatafr] <- lapply(inpL[isDatafr], as.matrix)     
    }
  isMatr <- sapply(inpL,is.matrix)
  if(any(!isMatr) && !silent) {
    message(fxNa,"Removing ",sum(!isMatr, na.rm=TRUE)," entries from input which are apparently not matrixes")
    inpL <- inpL[which(isMatr)]} 
  if(length(inpL) <1) stop(fxNa," It seems the input doesn't contain any (valid) matrixes")
  matrDim <- sapply(inpL,dim)
  if(length(unique(matrDim[1,])) >1) stop(fxNa," Some matrixes don't have the same number of rows (found ",paste(matrDim[1,],collapse=" "),")")
  nrColNa <- unique(unlist(lapply(inpL,colnames)))
  if(!silent) message(fxNa,"Treating ",length(nrColNa)," different (types of) columns : ",paste(utils::head(nrColNa),collapse=" "))
  out <- matrix(0, nrow=nrow(inpL[[1]]), ncol=length(nrColNa), dimnames=list(rownames(inpL[[1]]),nrColNa))
  startCol <- 1
  nByCol <- rep(0, length(nrColNa))
  names(nByCol) <- nrColNa
  for(i in 1:length(inpL)) { useCol <- match(colnames(inpL[[i]]), colnames(out))
    out[,useCol] <- out[,useCol] + inpL[[i]]
    nByCol[useCol] <- nByCol[useCol] +1 
    startCol <- startCol + ncol(inpL[[i]])}  
  if(identical(summarizeAs,"mean") && any(nByCol >1)) { useCol <- which(nByCol >1)
    out[,useCol] <- out[,useCol] / matrix(rep(nByCol[useCol], each=nrow(out)), nrow=nrow(out))
  }
  if(sortOutput) { out <- out[,order(colnames(out))]
    if(!silent) message(fxNa,"  Sorting columns of output") }
  out }
  

#' Check for strict (ascencing or descending) order
#'
#' \code{checkStrictOrder} tests lines of 'dat' (matrix of data.frame) for strict order (ascending, descending or constant), 
#' each col of data is tested relative to the col on its left.
#' @param dat matrix or data.frame
#' @param invertCount (logical) inverse counting (ie return 0 for all elememts in order)
#' @param silent (logical) suppress messages
#' @param debug (logical) display additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return matrix with counts of up pairs, down pairs, equal-pairs, if 'invertCount'=TRUE all non-events are counted, ie a resulting 0 means that all columns are following the described characteristics (with variabale col-numbers easier to count)
#' @seealso \code{\link[base]{order}}, \code{\link{checkGrpOrder}}   
#' @examples
#' set.seed(2005); mat1 <- rbind(matrix(round(runif(40),1),nc=4), rep(1,4))
#' checkStrictOrder(mat1); mat1[which(checkStrictOrder(mat1)[,2]==0),]
#' @export
checkStrictOrder <- function(dat, invertCount=FALSE, silent=FALSE, debug=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom, newNa="checkStrictOrder")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  testO <- array(NA, dim=c(nrow(dat), ncol(dat) -1, 2))
  testId <- matrix(NA, nrow=nrow(dat), ncol=ncol(dat) -1)
  for(i in 1:(ncol(dat)-1)) {
     testO[,i,1] <- dat[,i] > dat[,i+1]
     testO[,i,2] <- dat[,i] < dat[,i+1]
     testId[,i] <- dat[,i] == dat[,i+1]
  }
  if(isTRUE(invertCount)) {testO <- !testO; testId <- !testId}
  out <- cbind(up=rowSums(testO[,,2], na.rm=TRUE), down=rowSums(testO[,,1], na.rm=TRUE), eq=rowSums(testId, na.rm=TRUE))
  rownames(out) <- rownames(dat)
  out }
  

#' Get first minimum
#'
#' This function allows to find the first minimum of a numeric vector
#' 
#' @param x (numeric vector) main input
#' @param positionOnly (logical)
#' @return numeric vector
#' @seealso  \code{\link[base]{which.min}}
#' @examples
#' .firstMin(c(4,3:6))
#' @export
.firstMin <- function(x, positionOnly=FALSE) {
  ## get (first) min of series
  ## for longer series of data rather use getMedOf1stValley()
  minPos <- which.min(x)                # no problem with NA
  if(positionOnly) minPos else x[minPos] }


#' Rescale respective to specific group
#'
#' This function allows to rescale data 'x' so that specific group 'grpNum' gets normalized to predefined value 'grpVal'.
#' In normal case x will be multiplied by 'grpVal' and devided by value obtained from 'grpNum'.
#' If summary of 'grpNum-positions' or 'grpVal' is 0, then grpVal will be attained by subtraction of summary & adding grpVal
#' 
#' @param x (numeric vector) main input
#' @param grpNum (numeric)
#' @param grpVal (numeric)
#' @param sumMeth (character) method for summarizing
#' @param callFrom (character) allow easier tracking of messages produced
#' @return numeric vector
#' @seealso  \code{\link[base]{which.min}}
#' @examples
#' .firstMin(c(4,3:6))
#' @export
.medianSpecGrp <- function(x, grpNum, grpVal, sumMeth="median", callFrom=NULL){
  ## rescale data 'x' so that specific group 'grpNum' gets normalized to predefined value 'grpVal'
  ## in normal case x will be multiplied by 'grpVal' and devided by value obtained from 'grpNum'
  ## if summary of 'grpNum-positions' or 'grpVal' is 0, then grpVal will be attained by subtration of summary & adding grpVal
  fxNa <- .composeCallName(callFrom,newNa=".medianSpecGrp")
  msg1 <- c("argument ","'grpVal' should be numeric; ",
     "'grpNum' should be of length 1 and may be numeric or names of 'x'")
  if(sum(is.na(x))==length(x)) {
    message(fxNa,"argument 'x' seems all empty or NA, nothing to do")
  } else {
    if(length(grpVal) != 1) stop(fxNa,msg1[c(1:2)])
    if(any(sum(is.na(x)) ==length(x), sum(is.na(grpNum)) >0, is.na(grpVal))) stop(msg1[c(1:2,1,3)])
    if(length(grep("^[[:digit:]]+$",grpNum)) <1)  grpNum <- match(grpNum,names(x))
    grpNum <- convToNum(grpNum, remove=NULL)
    grpNum <- grpNum[grpNum <= length(x)]
    msg2 <- "'grpNum' should be either numeric for positions in 'x' or character for names of 'x'"
    if(length(grpNum) <1) stop(fxNa,msg1[1],msg2)
    grp1ini <- if(length(grpNum) >1) {
      if(identical(sumMeth,"median")) stats::median(x[grpNum],na.rm=TRUE) else mean(x[grpNum],na.rm=TRUE)
      } else x[grpNum]
    x <- if(all(c(grpVal,grp1ini) !=0)) grpVal*x/grp1ini else x + grpVal - grp1ini             # set values
  }
  x }

#' Rescale respective to specific group
#'
#' This function allows to rescale data 'x' so that 2 specific groups get normalized to predefined values (and all other values follow proportionally)
#' 'grp1Num' and 'grp2Num' should be either numeric for positions in 'x' or character for names of 'x';
#' if 'grp1Num' and/or 'grp2Num' design mulitple locations: perform median or mean summarization, according to 'sumMeth'
#' 
#' @param x (numeric vector) main input
#' @param grp1Num (numeric)
#' @param grp1Val (numeric)
#' @param grp2Num (numeric)
#' @param grp2Val (numeric)
#' @param sumMeth (character) method for summarizing
#' @param callFrom (character) allow easier tracking of messages produced
#' @return numeric vector
#' @seealso  \code{\link[base]{which.min}}
#' @examples
#' .firstMin(c(4,3:6))
#' @export
.scaleSpecGrp <- function(x, grp1Num, grp1Val, grp2Num=NULL, grp2Val=NULL, sumMeth="mean",callFrom=NULL){
  ## rescale data 'x' so that 2 specific groups get normalized to predefined values (and all other values follow proportionally)
  ##  'grp1Num' and 'grp2Num' should be either numeric for positions in 'x' or character for names of 'x'
  ##   if 'grp1Num' and/or 'grp2Num' design mulitple locations: perform median or mean summarization, according to 'sumMeth'
  ## return object of same dim as 'x'
  fxNa <- .composeCallName(callFrom, newNa=".scaleSpecGrp")
  msg1 <- c("argument ","'grp1Val'"," and ","'grp2Val'"," should be numeric of length 1; ",
     "'grp1Num'","'grp2Num'"," may be numeric or names of 'x'"," .. ignoring")
  if(sum(is.na(x)) ==length(x)) {
    message(fxNa,"Argument 'x' seems all empty or NA, nothing to do")
  } else {
    if(sum(is.na(x))==length(x) || sum(is.na(grp1Num)) >0 || is.na(grp1Val)) stop(msg1[c(1:2,5,6,8)])
    if(length(grp1Val) !=1) stop(msg1[c(1:2,5)])
    if(length(grep("^[[:digit:]]+$", grp1Num)) <1) grp1Num <- match(grp1Num, names(x))
    grp1Num <- convToNum(grp1Num, remove=NULL)
    grp1Num <- grp1Num[grp1Num <= length(x)]
    if(length(grp1Num) <1) stop(fxNa,"Can't find positions/matches for 'grp1Num' in 'x' !")
    grp1ini <- if(length(grp1Num) >1) {
      if(identical(sumMeth,"median")) stats::median(x[grp1Num],na.rm=TRUE) else mean(x[grp1Num],na.rm=TRUE)
      } else x[grp1Num]
    if(identical(grp1Val, grp2Val)) {grp2Num <- NULL; message(fxNa," grp1Val and grp2Val should be different !  ignoring grp2Val")}
    if(length(grp2Num) >0 && length(grp2Val) !=1) {grp2Num <- NULL; message(fxNa,paste(msg1[c(1,4,5,9)]))}
    if(length(grp2Num) >0) {
      if(any(sum(is.na(grp2Num)) >0, is.na(grp2Val))) stop(msg1[c(1:2,1,3)])
      grp2ini <- if(length(grp2Num) >1) {
        if(identical(sumMeth,"median")) stats::median(x[grp2Num],na.rm=TRUE) else mean(x[grp2Num], na.rm=TRUE)
        } else x[grp2Num]
      if(length(grep("^[[:digit:]]+$",grp2Num)) <1)  grp2Num <- match(grp2Num, names(x))
      grp2Num <- convToNum(grp2Num, remove=NULL)
      grp2Num <- grp2Num[grp2Num <= length(x)]
      msg2 <-  c("'grp2Num' should be either numeric for positions in 'x' or character for names of 'x'"," ... ignoring")
      if(length(grp2Num) <1) {grp2Num <- NULL; message(fxNa,msg1[1],msg2)} }
    if(length(grp2Num) <1) {
      x <- if(all(c(grp1Val, grp1ini) !=0)) grp1Val*x/grp1ini else x + grp1Val - grp1ini             # set values
    } else {
      x <- x - grp1ini              # set 1st val to 0
      x <- grp1Val + x*(grp2Val -grp1Val)/grp2ini } }
  x }
  

#' Check Comparison-Choice
#'
#' This function aims to help getting organized with pairwise (statistical) testing since there are multiple ways of expressing how samples 
#' should be grouped for tests. For example this may be either as combined character strings (eg 'A-B') or as separate elements. 
#' This function takes most input-formats and returns multiple converted formats.
#' 
#' @details
#' This function returns a list with $pwGrpNa (matrix with separated elements), $pwIndex (matrix with separated indexes), $grpNa (levels of \code{grp}, ie sorted),
#'  $sep (character vecor of separator found/used or suggested), $concat (pairwise concatenated names, eg 'A-B') 
#' 
#' @param useComparison (numeric or character vector or matrix) the argument allowing to specify the pairwise comparions 
#'   may be character as result of combining two group-names (from argument \code{grp}) (eg 'A-B', beware, the separator may not appear inside group-names) 
#'   or 2-column matrix of group-names (thus, without separator; names combined with separator may be shown as rownames) or 2-column matrix of indexes to sorted group-names;
#'   default setting of \code{useComparison='all'} will select all possible pairwise combinations
#' @param grp (character or factor)  vector defining groups of replicates for additional checking (may also be just the levels of grp)
#' @param experSetup (list) optional esperimental setup (list with $ind, $pwGrpIndex,$sep and $pwGrpNa  as obtained using \code{getPairwiseSetup})
#' @param sep (character) optional custom separator to use for splitting \code{useComparison}
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allows easier tracking of messages produced
#' @return This function returns a list with $data, $nNA, $randParam, $NAneigLst, $seed, $filt, and $annot  
#' @seealso \code{\link{presenceFilt}}, \code{\link{getPWseparator}}, \code{\link{indexGroupsFromPW}}, \code{\link{getPairwiseSetup}}, \code{\link[base]{strsplit}}
#' @examples
#' convPairwiseSetup(c("B-C","D-A"), LETTERS[1:4])
#' convPairwiseSetup(c("B","C"), LETTERS[1:3])
#' convPairwiseSetup("all", LETTERS[1:3])
#' convPairwiseSetup(2:3, LETTERS[1:3])
#' convPairwiseSetup(2:3, LETTERS[1:3], sep="__")
#' convPairwiseSetup(matrix(c("B","D","C","A"), ncol=2), LETTERS[1:4])
#' 
#' @export
convPairwiseSetup <- function(useComparison, grp, experSetup=NULL, sep=NULL, silent=FALSE, debug=FALSE, callFrom=NULL) {
  ## useComparison (character or matrix) list/table of comparisons to be checked/mapped to grp
  ## experSetup (list with $pwGrpNa, $sep, $index, $grpNa) from getPairwiseSetup() ; old name checkUseComparison()
  ## return list with $pwGrpNa, $pwIndex, $grpNa, $sep  
  ## Note! estimation of 'grp' does NOT reflect number of replicateds neither positions
  fxNa <- .composeCallName(callFrom, newNa="convPairwiseSetup")
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  if(!isTRUE(silent)) silent <- FALSE
  pwComb <- grpNa <- concat <- pwIndex <- concatNa <- NULL
  #combineSep=NULL,
  ## use grp as redundant (length of ncol(dat))
  ## use grpNa as levels(grp)
  if(debug) {message(fxNa,"cUC1"); cUC1 <- list(useComparison=useComparison, grp=grp, experSetup=experSetup, sep=sep, pwComb=pwComb, grpNa=grpNa)}

  ## check if experSetup exists (result from getPairwiseSetup() - use to replace empty spots
  if(length(experSetup) !=0 && is.list(experSetup)) {
    if("sep" %in% names(experSetup)) sep <- experSetup$sep 
    if("grpNa" %in% names(experSetup)) grpNa <- experSetup$grpNa 
    if("grp" %in% names(experSetup) && length(grp)==0) { grp <- experSetup$grp
      if(length(grpNa)==0) grpNa <- sort(unique(grp)) else if(identical(unique(sort(as.character(grp))), as.character(grpNa))) warning(fxNa,"BEWARE 'grpNa' and 'grp' seem not to macth !!")}
    if("pwGrpNa" %in% names(experSetup)) pwGrpNa <- experSetup$pwGrpNa
    if("index" %in% names(experSetup)) index <- experSetup$index       ## ? needed ?
  }
  if(length(grp) >0) {
    grp <- as.factor(grp)
    grpNa <- levels(grp)
  }
  if(length(grpNa)==0) grpNa <- unique(sort(naOmit(grp)))
  if(length(grpNa)==0) warning(fxNa,"Potential problem with argument 'grp'  (all empty or NA ?), trying to find out (based on calssical separators, $grpNa possibly incomplete, $index possibly bad)")
  if(debug) {message(fxNa,"cUC2"); cUC2 <- list(useComparison=useComparison, grp=grp, experSetup=experSetup, sep=sep, pwComb=pwComb, grpNa=grpNa)}

  if(identical("all",useComparison)) {
    pwGrpNa <- t(utils::combn(grpNa, 2))
    useComparison <- pwIndex <- t(utils::combn(1:length(grpNa), 2))
    colnames(pwIndex) <- colnames(pwGrpNa) <- c("samp","ref")
    if(debug) message(fxNa,"Setting from useComparison='all' to ",length(grpNa)," specific names & indexes")
  }
  if(debug) {message(fxNa,"cUC2b"); cUC2b <- list(useComparison=useComparison, grp=grp, experSetup=experSetup, sep=sep, pwComb=pwComb, grpNa=grpNa)}

  ## determine sep
  if(length(sep) != 1) {
    ## extract if avail
    if(length(dim(useComparison)) <2 && !all(grepl("^[[:digit:]]+$", useComparison)) && all(nchar(useComparison) >2)) {
       ## assume as pairwise combined text : extract if avail
       sep <- if(length(sep)==0) suppressWarnings(getPWseparator(grp=grpNa, potSep="split1", silent=silent, debug=debug, callFrom=fxNa)) else {
	     suppressWarnings(getPWseparator(grp=grpNa, potSep=sep, silent=silent, debug=debug, callFrom=fxNa)) } 
    } else {
      if(length(grpNa) >0) {
        ## suggest based on grp
        sep <- if(length(sep)==0) getPWseparator(grp=grpNa, potSep="combine1", silent=silent, debug=debug, callFrom=fxNa) else {
		  getPWseparator(grp=grpNa, potSep=sep, silent=silent, debug=debug, callFrom=fxNa)}
      } else stop(fxNa,"Unable to determine or suggest 'sep' ")      
    }
  } else {
    if(grepl("(split)|(combine)[[:digit:]]{0,1}", sep))  sep <- suppressWarnings(getPWseparator(grp=grpNa, potSep=pwSeparatorList(type=sep), silent=silent, debug=debug, callFrom=fxNa))  
    #if(any(grepl("^split[[:digit:]]",sep), grepl("^combine[[:digit:]]",sep))) sep <- suppressWarnings(getPWseparator(grp=grpNa, potSep=pwSeparatorList(type=sep),silent=silent, debug=debug, callFrom=fxNa)) 
    if(length(sep)==0) {if(!silent) message(fxNa,"Encountered unknown 'sep'; setting to default")
      sep <- suppressWarnings(getPWseparator(grp=grpNa, potSep=pwSeparatorList(type="split1"), silent=silent, debug=debug, callFrom=fxNa)) 
    } #else sep <- suppressWarnings(getPWseparator(grp=grpNa, potSep=sep,silent=silent, debug=debug, callFrom=fxNa)) 
  }
  if(debug) {message(fxNa,"cUC3"); cUC3 <- list(useComparison=useComparison, grp=grp, experSetup=experSetup, sep=sep, pwComb=pwComb, grpNa=grpNa)}


  ## main checking/extracting of index & pwGrpNa
  ## check if at least 2 levels appear in useComparison;  convert all to matrix with index of levels (and useComparisonNa as matrix of names)
  if(length(useComparison)==0) {
    ## make 1st of comparisons
    if(debug) message(fxNa,"Note: argument 'useComparison' seems empty setting to 1st of comparisons   chUC3b" )
    if(length(grpNa)==0) stop(fxNa,"Ipmossible to get result since 'grp' is missing !!") else pwGrpNa <- grpNa[1:2]
    pwIndex <- 1:2
    useComparison <- paste(grpNa[1:2], collapse=sep)  # needed ??
  } else {
    ## may be char (rel to grp) OR matrix (index or char) 
    if(debug) message(fxNa," ..length useComparison : ",length(useComparison),"   cUC3b")
    chNum <- all(grepl("^[[:digit:]]+$", as.character(useComparison)), na.rm=TRUE)
    if(length(dim(useComparison))==2) {
      ## 2.0  matrix
      chNA <- is.na(useComparison)   #
      if(all(chNA) || length(useComparison)==0) stop(fxNa,"Removing NAs from (matrix-type) 'useComparison' - NOTHING REMAINS")
      if(any(chNA)) { useComparison <- useComparison[which(rowSums(chNA)==0),, drop=FALSE] 
        if(!silent) message(fxNa,"BEWARE, 'useComparison' contains NAs, removing ...")}
      if(debug) {message(fxNa,"cUC3c"); cUC3c <- list(useComparison=useComparison, grp=grp, experSetup=experSetup, sep=sep, pwComb=pwComb, pwIndex=pwIndex, grpNa=grpNa)}
      
      if(isTRUE(chNum)) {             # presume as index
        ## 2.1 matrix of index
        if(all(as.integer(useComparison) >0) && all(as.integer(useComparison) >0)) {    # useComparison is matrix of index
          pwIndex <- useComparison
          if(length(grpNa) >0) {
            if(any(as.integer(useComparison) > length(grpNa), as.integer(useComparison) <1)) stop(fxNa,"Invalid entry for integer 'useComparison'")
          } else {
            if(!silent) message(fxNa,"NOTE : 'grp' still unknown, impossible to check range of integer 'useComparison'")
          }
          pwGrpNa <- cbind(samp=grpNa[pwIndex[,1]], ref=grpNa[pwIndex[,2]])
        } else warning(fxNa,"Cannot verify 'useComparison' as presumed integers")
      } else {
        ## 2.2 matrix of grpNa
        ## presume as matrix of character - ie names of levels, need to compare to grp (if knonn)
        if(length(grpNa) ==0) {
          warning(fxNa,"'grp' not known, very risky to try determining based on pw-names")         
          grpNa <- unique(sort(as.character(useComparison)))     ## ++ try to extract grpNa ++
        }  ## from now on assume grp as known
        if(debug) {message(fxNa,"cUC4"); cUC4 <- list(useComparison=useComparison, grp=grp, experSetup=experSetup, sep=sep, pwComb=pwComb, grpNa=grpNa)}
  
        pwGrpNa <- as.matrix(useComparison)
        pwIndex <- matrix(match(as.matrix(useComparison), grpNa), ncol=2)   # check if all exist
        chNA <- is.na(pwIndex)
        if(any(chNA)) { if(all(chNA)) stop(fxNa,"UNABLE to match 'useComparison' to 'grp' - nothing remains") else warning(fxNa,
          "BEWARE : Some comparisons do NOTNeed EXIST; need to eliminate ",sum(rowSums(chNA)==0)," comparison(s)")
          pwGrpNa <- pwGrpNa[which(rowSums(chNA)==0),,drop=FALSE]
          pwIndex <- pwIndex[which(rowSums(chNA)==0),,drop=FALSE]
        }
      }           # ... finished for matrix       
      if(debug) {message(fxNa,"cUC3d"); cUC3d <- list(useComparison=useComparison, grp=grp, experSetup=experSetup, sep=sep, pwComb=pwComb, pwIndex=pwIndex, pwGrpNa=pwGrpNa, grpNa=grpNa)}
    } else {
      ## 3.0 (char) vector          
      ## is vector (not matrix ...)
      if(debug) {message(fxNa,"cUC4a"); cUC4a <- list(useComparison=useComparison, grp=grp, experSetup=experSetup, sep=sep, pwComb=pwComb, grpNa=grpNa,chNum=chNum)} # pwIndex=pwIndex, pwGrpNa=pwGrpNa, 
      if(any(is.na(useComparison))) useComparison <- naOmit(useComparison)
      if(length(useComparison) <1) stop(fxNa,"Removing NAs from (presumed text) 'useComparison' - NOTHING REMAINS")

      chGrp <- useComparison %in% grpNa
      if(length(grpNa)==0) grpNa <- naOmit(unique(sort(grp)))     # needed ??
      if(isTRUE(chNum) && all(as.integer(useComparison) >0) && !all(chGrp)) {   # vector of integer => index of comparison numbers
        if(length(useComparison) !=0) {                ## 3.0 vector of comparison numbers
          useComparison <- try(as.integer(useComparison), silent=TRUE)
          if(inherits(useComparison, "try-error")) {
            useComparison <- 1
            warning(fxNa,"Don't understand entry for 'useComparison', setting to default, ie 1st comparison ")
          }
          pwGrpNaAll <- t(utils::combn(grpNa, 2))
		  
          if(length(sep) != 1) { sep <- if(length(sep)==0) getPWseparator(grp=grpNa, potSep="combine1", silent=silent, debug=debug, callFrom=fxNa) else {               ## suggest based on grp
	    	    getPWseparator(grp=grpNa, potSep=sep, silent=silent, debug=debug, callFrom=fxNa)}}
          dimnames(pwGrpNaAll) <- list(paste0(pwGrpNaAll[,1], sep, pwGrpNaAll[,2]), c("samp","ref"))
          chUseComparison <- useComparison >0 & useComparison <= nrow(pwGrpNaAll) & !is.na(useComparison)
          if(any(!chUseComparison)) { if(all(!chUseComparison)) { 
              stop(fxNa,"Invalid entry for 'useComparison', should be matrix of group-names or number of all pairwise-comparisons")
            } else useComparison <- useComparison[which(chUseComparison)]}
          pwGrpNa <- pwGrpNaAll[useComparison,,drop=FALSE]
          pwIndex <- matrix(match(pwGrpNa, grpNa), ncol=2, dimnames=dimnames(pwGrpNa))
        }  #else {  ## 3.1 vector of indexes - not possible any more
        #}
      } else {           
        ## 3.2 (char) vector
        if(debug) {message(fxNa,"cUC4c  3.2 vector") }
        ## check if pair of grpNa (otherwise strsplit)
        if(all(chGrp, na.rm=TRUE)) {        # char vector of GrpNa
          ## 3.2.1 char vector of GrpNa 
          if(debug) {message(fxNa,"cUC4d  3.2.1 char vector of GrpNa") }
          pwGrpNa <- matrix(useComparison, byrow=TRUE, ncol=2)
          if(length(grpNa) !=0) pwIndex <- matrix(match(useComparison, grpNa), byrow=TRUE, ncol=2)           
        } else {     # concatenated : strsplit
          ## 3.2.2 char vector of concatenated : strsplit
          if(debug) {message(fxNa,"cUC4e  3.2.2 char vector of concatenated : strsplit"); cUC4e <- list(useComparison=useComparison, grp=grp, experSetup=experSetup, sep=sep, pwComb=pwComb, grpNa=grpNa,chNum=chNum)} 
          nCha <- nchar(useComparison) >2
          if(any(!nCha, na.rm=TRUE)) stop(fxNa,"Unable to perform strsplit on 'useComparison' since ",sum(nCha, na.rm=TRUE)," element(s) are too short !!")
          if(length(sep)==0) {           # identify sep from  useComparison
            potSep <- pwSeparatorList("split1")
            spl1 <- lapply(protectSpecChar(potSep), function(x) strsplit(useComparison, x))          
            chLe <- sapply(spl1, sapply, length)
            use1 <- which.max(colSums(chLe==2))
            if(all(colSums(chLe !=2) !=0)) {
              msgE <- "Failed to extract 'grpNa' from 'useComparison', none of the potential separators allowed to split all 'useComparison' in 2 parts" 
              ch2 <- try(findHeadAndTail(grpNa=grpNa, pairwNa=useComparison, reportAs="list", silent=silent, debug=debug, callFrom=fxNa))
              if(inherits(ch2, "try-error") || length(ch2) !=1) message(fxNa,msgE) else {sep <- ch2$sep; pwGrpNa <- matrix(ch2$pwGrpNa) } 
            } else {
              sep <- potSep[use1]
              spl1 <- unlist(spl1[[use1]])
              pwGrpNa <- matrix(spl1, byrow=TRUE, ncol=2)              
              if(length(grpNa)==0) grpNa <- sort(unique(spl1))
            }
            pwIndex <- matrix(match(pwGrpNa, grpNa),  ncol=2) 
             
          } else {
            if(length(sep) >1) {warning(fxNa,"Truncating sep to length=1"); sep <- sep[1]}
            spl1 <- strsplit(useComparison, sep)
            ch1 <- sapply(spl1, length)
            if(any(ch1 !=2)) stop(fxNa,"User-provided 'sep' does NOT allow to split all 'useComparison' in exactely 2 parts")
            pwGrpNa <- matrix(unlist(spl1, use.names=FALSE), byrow=TRUE, ncol=2) 
            pwIndex <- matrix(match(pwGrpNa, grpNa), ncol=2 )           
          }          
        }      # finish split of char vector
        
      }        # finish char vector
    }          # finish vector
    rownames(pwGrpNa) <- rownames(pwIndex) <- concat <- paste0(pwGrpNa[,1], sep, pwGrpNa[,2])        

  }
  if(debug) {message(fxNa,"cUC6"); cUC6 <- list(useComparison=useComparison, grp=grp, experSetup=experSetup, sep=sep, pwGrpNa=pwGrpNa, pwIndex=pwIndex, pwComb=pwComb, grpNa=grpNa)}
  if(length(pwGrpNa) >0 && length(dim(pwGrpNa)) !=2) pwGrpNa <- matrix(pwGrpNa, byrow=TRUE, ncol=2) 
  if(length(pwIndex) >0 && length(dim(pwIndex)) !=2) pwIndex <- matrix(pwIndex, byrow=TRUE, ncol=2)
  if(length(pwComb)==0 && length(pwGrpNa) !=0) pwComb <- paste0(pwGrpNa[,1], sep, pwGrpNa[,2])
  dimnames(pwGrpNa) <- dimnames(pwIndex) <- list(pwComb, c("samp","ref"))
  if(debug) {message(fxNa,"cUC7"); cUC7 <- list(useComparison=useComparison, grp=grp, experSetup=experSetup, sep=sep, pwGrpNa=pwGrpNa, pwIndex=pwIndex, pwComb=pwComb, grpNa=grpNa)}
  list(pwGrpNa=pwGrpNa, pwIndex=pwIndex, grpNa=grpNa, sep=sep, concat=pwComb)
} 
  
  

#' Combine/reduce redundant lines based on specified column 
#'
#' This function works similar to \code{unique}, but it takes a matrix as input and considers one specified column to find unique instances.
#' It identifies 'repeated' lines of the input-matrix (or data.frame) 'mat' based on (repeated) elements in/of column with name 'colNa' (or column-number).
#' Redundant lines (ie repeated lines) will disappear in output.
#' Eg used with extracted annotation where 1 gene has many lines for different GO annotation.
#'
#' @param mat input matrix or data.frame
#' @param colNa character vector (length 1) macting 1 column name (if mult only 1st will be used), in case of mult matches only 1st used
#' @param sep (character) separator (default=",")
#' @param silent (logical) suppress messages
#' @param callFrom (character) allow easier tracking of messages produced
#' @return matrix containing the input matrix without lines considered repeated (unique-like)
#' @seealso \code{\link{findRepeated}}, \code{\link{firstOfRepLines}}, \code{\link{organizeAsListOfRepl}}, \code{\link{combineRedundLinesInList}}
#' @examples
#' matr <- matrix(c(letters[1:6],"h","h","f","e",LETTERS[1:5]),ncol=3,
#'   dimnames=list(letters[11:15],c("xA","xB","xC")))
#' combineRedBasedOnCol(matr,colN="xB")
#' combineRedBasedOnCol(rbind(matr[1,],matr),colN="xB")
#' @export
combineRedBasedOnCol <- function(mat, colNa, sep=",", silent=FALSE, callFrom=NULL){
  ## combine lines of matrix (or data.frame) 'mat' based on (repeated) elements in/of column with name 'colNa' (or column-number).
  ## 'colNa' ... 
  ## 'sep' ...  character vector (length 1) as separator when concatenating text-fields
  ## redundant lines (ie repeated lines) will disappear in output
  ## eg used with extracted annotation where 1 gene has many lines for different GO annot
  fxNa <- .composeCallName(callFrom,newNa="combineBasedOnCol")
  argN <- deparse(substitute(mat))
  msg <- paste(" expecting matrix or data.frame with min 2 columns in '",argN,"'")
  if(length(dim(mat)) <2) stop(msg) else if(ncol(mat) <2) stop(msg)
  if(is.data.frame(mat)) mat <- as.matrix(mat)
  msg <- c(fxNa,"argument 'colNa'"," longer than 1"," not found","using only 1st element")
  if(length(colNa) >1) {if(!silent) message(msg[-4]); colNa <- colNa[1]}
  if(is.numeric(colNa)) { if(colNa > ncol(mat)) stop else colNa <- which(colnames(mat) ==colNa)[1]
  } else if(!colNa %in% colnames(mat)) {colNa <- which(colnames(mat) ==colNa)[1]; message(msg[-3])}
  if(length(sep) <1) {sep <- ","} else if(length(sep) >1) {sep <- sep[1]}
  chCol <- which(colnames(mat) %in% colNa)
  if(length(chCol) <0) stop(" no columns in '",argN,"' corresponding to '",colNa,"' found")
  refCo <- chCol[1]
  chRef <- table(mat[,refCo])
  if(any(is.na(mat[,refCo])) &!silent) {message(fxNa," trouble ahead : column ",colnames(mat)[chRef]," in '",argN,"' contains NAs !!")}
  if(length(chRef) <nrow(mat)) {                             # has repeated elements
    matLst <- by(mat,as.numeric(as.factor(mat[,refCo])),as.matrix)
    out <- t(sapply(matLst,apply,2,function(y) paste(unique(y),collapse=",")))
    out <- out[match(unique(mat[,refCo]),unique(out[,refCo])),]       # bring in order of (non-red) initial
    if(length(dim(out)) <2) out <- matrix(out,nrow=1,dimnames=list("1",names(out)))
  } else out <- mat
  out }
   

#' Multiple replacement of entire character elements in simple vector, matrix or data.frame
#'
#' This functions allows multiple types of replacements of entire character elements in simple vector, matrix or data.frame.
#' In addtion, the result may be optionally directly transformed to logical or numeric
#'
#' @param mat (character vector, matrix or data.frame) main data
#' @param repl (matrix or list) tells what to replace by what: If matrix the 1st oolumn will be considered as 'old' and the 2nd as 'replaceBy'; if named list, the names of the list-elements will be consdered as 'replaceBy'
#' @param convTo (character) optional conversion of content to 'numeric' or 'logical'
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns an object of same dimension as input (with replaced content)
#' @seealso  \code{\link[base]{grep}}
#' @examples
#' x1 <- c("ab","bc","cd","efg","ghj")
#' multiCharReplace(x1, cbind(old=c("bc","efg"), new=c("BBCC","EF")))
#'
#' x2 <- c("High","n/a","High","High","Low")
#' multiCharReplace(x2, cbind(old=c("n/a","Low","High"), new=c(NA,FALSE,TRUE)),convTo="logical")
#'
#' # works also to replace numeric content :
#' x3 <- matrix(11:16, ncol=2)
#' multiCharReplace(x3, cbind(12:13,112:113))
#' @export
multiCharReplace <- function(mat, repl, convTo=NULL, silent=FALSE, debug=TRUE, callFrom=NULL) {
  ## multiple replacement of entire character elements in simple vector, matrix or data.frame
  ## repl .. matrix with 2 colums for old and new or list where list-names design new
  fxNa <- .composeCallName(callFrom, newNa="multiCharReplace")
  if(length(mat) <1 || (is.list(mat) && !is.data.frame(mat))) stop("Invalid argument 'mat'; must be simple vector, matrix or data.frame of length >0")
  if(is.data.frame(mat)) mat <- as.matrix(mat)
  if(is.numeric(mat) && length(convTo) <1) convTo <- "numeric"
  if(all(length(dim(repl)) ==2, dim(repl) > 0:1)) {
    ## 'repl' is matrix with 2 cols
    for(i in 1:nrow(repl)) {ch <- which(mat==repl[i,1]); if(length(ch) >0) mat[ch] <- repl[i,2]}
  } else if(is.list(repl) && length(repl) >0 && !is.null(names(repl))) {
    ## 'repl' is named list
    for(i in 1:length(repl)) {
      ch <- which(mat %in% repl[[i]])
      if(length(ch) >0) mat[ch] <- names(repl)[i]}
  }

  msg <- "Unable to convert to "
  if(length(convTo)==1) { if("numeric" %in% convTo && !is.numeric(mat)) {
    tmp <- try(as.numeric(mat), silent=TRUE)
    if(inherits(tmp, "try-error")) {mat <- if(length(dim(mat)) <2) tmp else matrix(tmp, ncol=ncol(mat), dimnames=dimnames(mat))
    } else if(!silent) message(fxNa,msg,"numeric")
  } else if("logical" %in% convTo & !is.logical(mat)) {
    tmp <- try(as.logical(mat), silent=TRUE)
    if(!inherits(tmp, "try-error")) {mat <- if(length(dim(mat)) <2) tmp else matrix(tmp, ncol=ncol(mat), dimnames=dimnames(mat))
    } else if(!silent) message(fxNa,msg,"logical") }}
  mat }

#' Extended version of merge for multiple objects (even without rownames)
#'
#' \code{mergeW2} povides flexible merging out of 'MArrayLM'-object (if found, won't consider any other input-data) or of separate vectors or matrixes.
#' The main idea was to have somthing not adding add'l lines as merge might do, but to stay within the frame of the 1st argument given, even when IDs are repeated, 
#' so the output follows the order of the 1st argument, non-redundant IDs are created (orig IDs as new column).
#' If no 'MArrayLM'-object found: try to combine all elements of input '...', input-names must match predefined variants 'chInp'.
#' IDs given in 1st argument and not found in later arguments will be displayed as NA in the output matrix of data.frame.  
#'  Note : (non-data) arguments must be given with full name (so far no lazy evaluation, may conflict with names in 'inputNamesLst'). 
#'  Note : special characters in colnames bound to give trouble. 
#'  Note : when no names given, \code{mergeW2} will presume order of elements (names) from 'inputNamesLst'. 
#' PROBLEM : error after xxMerg3 when several entries have matching (row)names but some entries match only partially  (what to do : replace with NAs ??)
#' @param ... all data (vectors, matrixes or data.frames) intendes for merge
#' @param nonRedundID (logical) if TRUE, allways add 1st column with non-redundant IDs (add anyway if non-redundant IDs found )
#' @param convertDF (logical) allows converting output in data.frame, add new heading col with non-red rownames & check which cols should be numeric
#' @param selMerg (logical) if FALSE toggle to classic merge() (will give more rows in output in case of redundant names
#' @param inputNamesLst (list) named list with character vectors (should be unique), search these names in input for extracting/merging elements use for 'lazy matching' when checking names of input, default : 7 groups ('Mvalue', 'Avalue','p.value','mouseInfo','Lfdr','link','filt') with common short versions
#' @param noMatchPursue (logical) allows using entries where 0 names match (just as if no names given)
#' @param standColNa (logical) if TRUE return standard colnames as defined in 'inputNamesLst' (ie 'chInp'), otherwise colnames as initially provided
#' @param lastOfMultCols may specify input groups where only last col will be used/extracted
#' @param duplTxtSep (character)  separator for counting/denomiating multiple occurances of same name
#' @param debug (logical) for bug-tracking: more/enhanced messages and intermediate objects written in global name-space 
#' @param silent (logical) suppress messages
#' @param callFrom (character) allows easier tracking of message(s) produced
#' @return matrix or data.frame of fused data
#' @seealso \code{\link[base]{merge}}
#' @examples
#' t1 <- 1:10; names(t1) <- letters[c(1:7,3:4,8)]
#' t2 <- 20:11;  names(t2) <- letters[c(1:7,3:4,8)]
#' t3 <- 101:110; names(t3) <- letters[c(11:20)]
#' t4 <- matrix(100:81,ncol=2,dimnames=list(letters[1:10],c("co1","co2")))
#' t5 <- cbind(t1=t1,t52=t1+20,t53=t1+30)
#'   t1; t2; t3;  cbind(t1,t2)
#' mergeW2(Mval=t1,p.value=t2,debug=FALSE)
#' @export
mergeW2 <- function(..., nonRedundID=TRUE, convertDF=TRUE, selMerg=TRUE, inputNamesLst=NULL, noMatchPursue=TRUE, standColNa=FALSE,
  lastOfMultCols=c("p.value","Lfdr"), duplTxtSep="_", silent=FALSE, debug=FALSE, callFrom=NULL){
  inp <- list(...)
  out = NULL
  chAnnCol <- c("mouseOver","link","uniqUniprot","Uniprot","firstOfMultUnipr","combUnipr","custMouOv","customMouseOver") # colnames in MArrayLM$annot to extract
  chInp <- if(is.list(inputNamesLst) && length(inputNamesLst)>1) inputNamesLst else {    # names of list-elements to allowed to extract
    list(Mvalue=c("Mvalue","Mval","M.value","M.val","Mv","M"),
    Avalue=c("Avalue","Aval","Amean","A.value","A.val","Av","A"),
    Amatr=c("Amatrix","Amatri","Amatr","Amat","Ama","Am","A.matrix","A.matr","A.mat","A.m"),
    p.value=c("pValue","pVal","p.value","p.val","pVa","pV","p"),
    Lfdr=c("Lfdr","lfdr","FDR","fdr","FdrList"), means=c("allMeans","means"), 
    mouseInfo=c("mouseInfo","info","inf","mou","mouse"), link=c("link","www"),filt=c("filtFin","filter","filt","fi"),
    info2=c("shortInfo","info2","suplInfo"), annot=c("annot","annotation"))}
  if(debug) silent <- FALSE
  fxNa <- .composeCallName(callFrom,newNa="mergeW2")
  if(length(unlist(chInp)) > length(unique(unlist(chInp)))) stop(fxNa,"Problem with redundant names in (predefined/custom) 'inputNamesLst'")
  if(length(inp) <1) stop(fxNa," No valid entries !")
  ## note: potential problem with multiple classes per objet
  cheCla <- lapply(inp,class)
  cheCla2 <- sapply(cheCla,paste,collapse=" ")
  if(debug) message(fxNa," class of arguments :  ",pasteC(cheCla2))
  if("MArrayLM" %in% unlist(cheCla)) {
    tmp <- inp[[which(grep("MArrayLM",cheCla2))]]
    if(!silent) message(fxNa,"Found 'MArrayLM'-object with ",length(names(tmp))," types of data for ",length(tmp)," elements/genes")
    tm2 <- .convertNa(names(tmp),chInp)
    if(length(naOmit(tm2)) > length(unique(naOmit(tm2)))) {
      tm2ta <- table(naOmit(tm2))
      msg <- c(fxNa,"PROBLEM : multiple names of MArrayLM-obj do match to (single) type of data searched '")
      for(i in which(tm2ta >1)) {
        if(!silent) message(msg,names(tm2ta)[i],"' : found ",pasteC(names(tmp)[which(tm2==names(tm2ta)[i])],quoteC="'"),
          " -> using last (otherwise try to modify 'chInp')")
        tm3 <- which(tm2==(names(tm2ta)[i]))
        tm2[tm3[-1*length(tm3)]] <- NA }
    }
    if(sum(!is.na(tm2)) >0) {names(tmp)[!is.na(tm2)] <- naOmit(tm2)            # set names of interest in MArray-obj to standard names
      if(!silent) message(fxNa,"Resetting names in 'MArrayLM'-obj to standardized names") }
    if(length(inp) >1) {
      ## regular case (not 'MArrayLM' class)
      chLe <- length(grep("MArrayLM",cheCla2)) <1 & sapply(inp,length) >0 & !is.na(.convertNa(names(inp),chInp))
      inp2 <- if(sum(chLe) >0) inp[which(chLe)] else NULL
    } else inp2 <- NULL
    inp <- list()
    useMaEl <- naOmit(match(names(chInp),names(tmp)))                          # pick only elements of interest out of list with mult options
    for(i in 1:length(useMaEl)) inp[[i]] <- tmp[[useMaEl[i]]]
    names(inp) <- names(tmp)[useMaEl]
    if(length(inp2) >0) { for(i in names(inp2)) inp[[i]] <- inp2[[i]]           # (re)inject information given besides MArrayLM (replacing) 
      useMaEl <- naOmit(match(names(chInp),names(tmp)))                        # update
      if(!silent) message(fxNa,"Adding ",length(inp2)," non MArrayLM objects: ",pasteC(names(inp2),quoteC="'"))}
    ## ++ need to specify which question (MAarray may contain results for multiple comparisons !!)
    ## immediately pick relevant cols and rename p.value  & Lfdr & annot ??
    colNa <- lapply(inp,colnames)
    rmIntercept <- TRUE                                                               ## <<== supl parameter ??
    exclNa  <- "(Intercept)"
    if(any(unlist(colNa) %in% exclNa) && rmIntercept) {                          # ready to remove cols names "(Intercept)"
      useEl <- which(sapply(colNa,function(x) any(x %in% exclNa)))
      if(length(useEl) >0) for(i in useEl) {
        useCo <- which(!colnames(inp[[i]]) %in% exclNa)
        inp[[i]] <- if(length(useCo) <1) NULL else {if(length(useCo) >1) inp[[i]][,useCo] else matrix(
          inp[[i]][,useCo],nrow=nrow(inp[[i]]),dimnames=list(rownames(inp[[i]]),colnames(inp[[i]])[useCo]))}
        if(ncol(inp[[i]])==1) colnames(inp[[i]]) <- names(inp)[i] }             # if only 1 col remaining rename to name of list-element
    }
    ## (MArrayLM-only) filter suitable cols from annot :
    if(any(chInp$annot %in% names(inp)) & length(chAnnCol) >0) {
      tmp <- which.max(names(inp) %in% chInp$annot)                             # use last of names from 'inp' recognized as annot
      if(length(dim(inp[[tmp]])) >0) inp[[tmp]] <- inp[[tmp]][,which(colnames(inp[[tmp]]) %in% chAnnCol)] }    # select specific cols
    ##
    cheCla <- sapply(inp,class)                                                 # update
  } else {
    if(debug) message(fxNa,"No 'MArrayLM'-object; eximine ",length(inp)," elements (",pasteC(names(inp),quoteC="'"),")")
    if(is.list(inp) && length(inp)==1) {inp <- inp[[1]]; cheCla <- sapply(inp,class) }}
  ##
  if(debug) message(fxNa,"Examine ",length(inp)," elements (",pasteC(names(inp),quoteC="'"),")")
  if(any(sapply(inp,length) <1)) {                                             ## remove empty elements
    if(!silent) message(fxNa,"Remove empty element(s) : ",pasteC(names(inp)[which(sapply(inp,length) <1)],quoteC="'"))
    inp <- inp[which(sapply(inp,length) >0)]
    cheCla <- sapply(inp,class) }
  ## no names given : presume 'chInp' ...
  if(is.null(names(inp))) {
    names(inp) <- names(chInp)[1:length(inp)]
    txt <- "CAUTION: arguments are given without names, PRESUME same order of arguments as given in 'inputNamesLst' ! ie "
    if(!silent) message(fxNa,txt,pasteC(names(inp),quoteC="'")) }
  cheCla2 <- sapply(cheCla,paste,collapse=" ")
  chLis <- grep("list",cheCla2)
  if(length(chLis) >0) {                         # if lists (still) present in 'lst': try to bring elements searched down one level
    for(i in chLis) {               # select list elements potentially containing elements to extract
      useEl <- which( sapply(names(inp[[i]]) %in% chInp))
      ## note : bringing multiple elements dow one level (and elim initial)
      if(length(useEl) >0) {
        for(j in 1:length(useEl)) inp[[length(inp)+1]] <- inp[[i]][[j]]
        inp <- inp[-i] }}
    if(debug) message(fxNa,"revised (list) names(inp) ",pasteC(names(inp)))
  }
  ## check for 1st input element matching (Mval essential), in case of merge keep names of 1st (Mval)
  if(!any(names(inp) %in% chInp[[1]]))  stop(" essential argument group (for) '",names(chInp)[1],"' missing !")
  ##
  stdInpNa <- .convertNa(names(inp),chInp)                                      # standardized input-names
  ## remove non-name-conform list-groups
  if(any(is.na(stdInpNa))) {
    if(!silent) message(fxNa,"Remove non-recognized elements/arguments ",pasteC(names(inp)[which(is.na(stdInpNa))],quo="'"))
    inp <- inp[-1*which(is.na(stdInpNa))]
    stdInpNa <- .convertNa(names(inp),chInp)}
  ## check for arguments matching multiple times
  if(length(stdInpNa) > length(unique(stdInpNa))) {
    tmp <- table(names(which()))
    if(!silent) message(fxNa,"Arguments ",pasteC(names(inp)[which(tmp >1)],quoteC="'")," match multiple times in 'inputNamesLst', use only 1st per group")
    inp <- inp[firstOfRepeated(stdInpNa)$indUniq]                               # keep only 1st of mult
    stdInpNa <- .convertNa(names(inp),chInp) }
  ## transform all non-matrix to matrix
  chConDim <- sapply(inp,function(x) length(dim(x)) >1)    # check for matrix
  if(any(!chConDim)) for(i in which(!chConDim)) inp[[i]] <- matrix(inp[[i]],ncol=1,dimnames=list(names(inp[[i]]),names(chConDim)[i]))
  colNa <- lapply(inp,colnames)
  msg <- c("Potential trouble ahead, "," colnames !")
  if(any(nchar(unlist(colNa)) <1) & !silent) message(fxNa,msg[1],sum(nchar(unlist(colNa)) <1)," missing",msg[2])
  n2 <- c(length(unique(unlist(colNa))), length(unlist(colNa)))
  if(n2[2] > n2[1] && !silent) message(fxNa,msg[1], n2[2]-n2[1]," non-unique",msg[2])
  ## option: extract only last of multiple Cols (for p.value, ...)
  renameSingToStd <- TRUE
  lastOfMultCols <- names(inp)[which(stdInpNa %in% .convertNa(lastOfMultCols,chInp))]
  if(length(lastOfMultCols) >0) {
    if(!silent) message(fxNa,"Use only last column for elements ",pasteC(lastOfMultCols,quoteC="'"))
    for(i in which(names(inp) %in% lastOfMultCols)) { xx <- inp[[i]]
      inp[[i]] <- matrix(xx[,ncol(xx)],ncol=1,dimnames=list(rownames(xx),if(renameSingToStd) names(chInp)[i] else colnames(xx)[ncol(xx)]))}  # rename to standard name (defined in 'chInp')
    } else if(debug) message(fxNa,"Argument 'lastOfMultCols' has no matches to groups presented in 'inputNamesLst' !!")
  ## remove special characters from colnames (if reorderig at end of procedure)
  for(i in length(inp)) colnames(inp[[i]]) <- .replSpecChar(colnames(inp[[i]]),replBy="_")
  chInp <- lapply(chInp,.replSpecChar,replBy="_")
  ## check for conflicting (col)-names  -> rename by adding name of group + '.' + orig (col)name (for all cols of group)
  allColNa <- unlist(lapply(inp,colnames))
  if(length(unique(allColNa)) < length(allColNa)) {
    multNa <- names(table(allColNa))[which(table(allColNa)>1)]
    if(!silent) message(fxNa,"Problem with repeated (col)names ",pasteC(multNa,quoteC="'"),", need to rename (by adding group-name)!!")
    while(length(multNa) >0) {
      i <- multNa[1]
      tmp <- which(sapply(lapply(inp,colnames),function(x) i %in% x))
      tmp <- tmp[-1*which.min(sapply(inp[tmp],ncol))]
      for(j in tmp) colnames(inp[[j]]) <- paste(names(inp)[j],colnames(inp[[j]]),sep=".")     # main renaming of repeated colnames
      allColNa <- unlist(lapply(inp,colnames))                                                # need to update
      multNa <- names(table(allColNa))[which(table(allColNa)>1)]
    } }
  ##
  ##  ++  main joining  ++
  chNaLe <- sapply(inp, function(x) c(le=nrow(x), leNa=length(rownames(x)), nrNa=length(unique(rownames(x)))))       # each elem of 'inp' .. separ col
  chNaLe <- rbind(chNaLe, nFound=rep(NA,ncol(chNaLe)))
  for(i in 2:length(inp)) chNaLe[4,i] <- sum(rownames(inp[[i-1]]) %in% rownames(inp[[i]]))    # number of names matching to Mval
  ## look for unique (row)names
  useCol4na <- chNaLe[2,-1]==chNaLe[1,1] && chNaLe[3,-1] >1                    # same length and >1 non-red names
  if(sum(useCol4na) >1) useCol4na[-1*which.max(chNaLe[3, 1+useCol4na])] <- FALSE   # for multiple potent names choose one with highest non-red
  nRedID <- if(chNaLe[1,1] >0) rownames(inp[[1]]) else {
    if(any(useCol4na)) rownames(inp[[which.max(useCol4na) +1]]) else 1:nrow(inp[[1]])}  # if names exist in Mval -> priority, else best of same le
  if(!is.null(nRedID)) nRedID <- treatTxtDuplicates(nRedID, sep=duplTxtSep, onlyCorrectToUnique=TRUE, callFrom=fxNa)          # warning if NULL
  ## check for identical rownames between elements to combine (cbind/merge)
  ## special case : search for annot (last element) with non-matching names but matching 1st col -> rename colnames of annot to allow simple merging
  compLastNa <- if(names(inp)[length(inp)] %in% chInp[[length(chInp)]]) {       # check if last of 'inp' in last group of 'chInp' present, ie annot
     if(!identical(rownames(inp)[[length(inp)]],rownames(inp[[1]])) && any(all(inp[[length(inp)]][,1]==rownames(inp[[1]])),all(
       inp[[length(inp)]][,1] ==gsub("\\+","; ",rownames(inp[[1]]))))) TRUE else FALSE} else FALSE
  if(compLastNa) rownames(inp[[length(inp)]]) <- rownames(inp[[1]])
  inpNa <- lapply(inp,rownames)
  chConNi <- cbind(idNa=sapply(inpNa[-1],function(x) all(x==inpNa[[1]])),
    noNaButEqLi=sapply(inp[-1], function(x) length(rownames(x)) <1 && nrow(x)==nrow(inp[[1]])),
    partNa=sapply(inpNa[-1],function(x) if(length(x) >0) sum(x %in% rownames(inp[[1]]),na.rm=TRUE) >0 else FALSE),
    hasNaButNoMatch=sapply(inpNa[-1],function(x) length(x) >0 & length(x)==nrow(inp[[1]]) && all(!x %in% inpNa[[1]])))
  if(any(chConNi[,2]) && !silent) message(fxNa,"No (row)names found in ",pasteC(names(inp)[which(chConNi[,2]) +1], quoteC="'"),
    " but presume same as '",names(inp)[1],"'")
  if(any(chConNi[,4]) && noMatchPursue) {
    for(j in which(chConNi[,4])) rownames(inp[[j+1]]) <- rownames(inp[[1]])
    chConNi[which(chConNi[,4]),2] <- TRUE
    txt <- c("CAUTION: none of the (row)names in "," matching, since same length presume names of '")
    if(!silent) message(fxNa,txt[1],pasteC(names(inp)[chConNi[,4]],quoteC="'"),txt[2],names(inp)[1],"'")}
  if(debug) {message(fxNa,"Counting of matching names etc 'chConNi' :\n"); print(chConNi); }
  chConNa <- chConNi[,1] | chConNi[,2] | chConNi[,3]                  # fuse for decision cbind : all names identical or (names absent & same nrow)
  ## remove elements wo names AND different length
  if(all(!chConNa)) stop(fxNa," No arguments left for merging !?!   (check input !!)")
  if(any(!chConNa)) {
    if(!silent) message(fxNa,"Cannot figure out how to join, OMIT element(s) ",pasteC(names(inp[1+which(!chConNa)]),quoteC="'"))
    inp <- inp[c(1, 1 +which(chConNa))]
    inpNa <- lapply(inp, rownames)
    chConNi <- cbind(idNa=sapply(inpNa[-1],function(x) identical(x, inpNa[[1]])),
      eqNumLines= sapply(inp[-1], function(x) length(rownames(x)) <1 && nrow(x)==nrow(inp[[1]])),
      eqNuLinesAndNoNames=rep(NA,length(inp[-1])),                                                    
      anyNa=sapply(inpNa[-1],function(x) if(length(x) >0) sum(x %in% rownames(inp[[1]]),na.rm=TRUE) >0 else FALSE),
      partNa=sapply(inpNa[-1],function(x) if(length(x) >0) sum(x %in% rownames(inp[[1]]),na.rm=TRUE) <nrow(inp[[1]]) else FALSE) )
    chConNi[,"eqNuLinesAndNoNames"] <- chConNi[,"eqNumLines"] & !chConNi[,"anyNa"]  
    chConNa <- chConNi[,1] | chConNi[,2] }
  out <- inp[[1]]
  ## main cbind
  chConNa <- chConNi[,1] | chConNi[,2]
  names(chConNa) <- rownames(chConNi)
  nCol <- sapply(inp,function(x) {dm <- dim(x); if(length(dm)>0) ncol(x) else 1})
  if(any(chConNa)) for(i in which(chConNa)) {                           # simple cbind when names are identical or all absent (&same length)
    tmp <- inp[[names(chConNa)[i]]]
    multCol <- FALSE
    out <- cbind(out,tmp)} 
  if(any(!chConNa)) message(fxNa,"Cannot use ",pasteC(names(chConNa)[which(!chConNa)],quoteC="'")," , different (partial?) matching than others !")
  newColNa <- if(standColNa) as.list(.convertNa(names(inp)[chConNa],chInp))  else as.list(names(inp)[chConNa])
  if(any(nCol[c(TRUE,chConNa)] >1)) for(i in which(nCol > 1 & c(TRUE,chConNa))) newColNa[[i]] <- paste(newColNa[[i]],colnames(inp[[i]]),sep=".")   # colnames for multi-col input
  colnames(out) <- unlist(newColNa) 
  ## (could not handle as cbind ->) have to combine via merge-like :
  if(any(!chConNa)) for(i in which(!chConNa)) {                                        # names do differ -> do some sort of 'merge'
    msg <- c(" .. for case of (partially) different ","(row)names ","in '","")
    useEl <- which(names(inp)==names(chConNa[i]))
    if(!silent) message(fxNa,"loop ",i,msg[1:3],names(inp)[useEl],"', prepare for merging")
    tmp <- rownames(inp[[useEl]])
    toMerNa <- rownames(inp[[useEl]])      # at this level all elements should have names
    ## nonunique rownames/IDs : if  in place, merge does all combinations, if used as NULL -> warning message when creating toMer;
    ##  if set how to re-integrate ??
    rownames(inp[[useEl]]) <- 1:nrow(inp[[useEl]])                                   # avoid warning in case of non-unique IDs !!
    toMer <- data.frame(id=toMerNa, inp[[useEl]], stringsAsFactors=FALSE)
    if(selMerg) {
      if(is.null(rownames(inp[[useEl]]))) {if(nrow(toMer)==nrow(out)) { toMer[,1] <- rownames(out)
        } else stop("problem with missing names in '",names(inp)[useEl],"', no ",useEl," and different nrow !!")}
      dim0 <- dim(out)
      out <- cbind(out, matrix(NA, nrow=nrow(out), ncol=ncol(toMer)-1, dimnames=list(NULL,colnames(toMer)[-1])))
      mat <- match(rownames(out),toMer[,1])
      n1 <- c(length(unique(naOmit(mat))), length(naOmit(mat)))
      msg <- c(" : multiple assoc for "," element(s) ","(don't know which one to choose !) ;  ","from ref not found")
      if(n1[1] <nrow(toMer) && !silent) message(fxNa," ",names(chConNa[i]), msg[1],n1[2]-n1[1],msg[2:3], sum(is.na(mat)), msg[c(2,4)])
      out[which(!is.na(mat)),(dim0[2] +1):ncol(out)] <- as.matrix(toMer[naOmit(mat),-1])
    } else {
      ## note when using merge() : does all combinations ! (ie output will have more lines than input !!), may give warning message
      out <- merge(data.frame(id=rownames(out), out),toMer, by="id", all.x=TRUE) }
  }
  ## put in order matching chInp:
  ## OK with single occurance per set of names
  tmp <- .convertNa(names(inp),chInp)
  goodOrd <- 1:ncol(out)
  if(length(tmp) <ncol(out)){ goodOrd <- naOmit(match(names(chInp),tmp))
  } else if(!silent) message(fxNa,"Can't adjust order of cols i output (since some elements have multiple columns)")
  ##
  goodOrd <- if(length(tmp) <ncol(out)) 1:ncol(out) else naOmit(match(names(chInp),tmp))   # if single column entries, set in order of chInp
  out <- out[,goodOrd]                                    # since merging non-complete elements at end changed order ...
  if(convertDF) out <- convMatr2df(out,duplTxtSep=duplTxtSep)                   # convert matrix to df
  if(is.data.frame(out)) for(i in ncol(out)) {if(is.character(out[,i])) out[,i] <- convToNum(out[,i],remove=NULL,sciIncl=TRUE)}          # keep text entries (eg annotation columns !)
  if("filt" %in% names(chInp)) {                          # convert any col type filt to logical
    useCol <- match(chInp[["filt"]],colnames(out))
    if(sum(!is.na(useCol)) >0) {
      tmp <- as.logical(out[,naOmit(useCol)[1]])
      if(sum(is.na(tmp)) <1) out[,naOmit(useCol)[1]] <- tmp else message(
        fxNa,"Cannot figure out how to convert '",colnames(out)[naOmit(useCol)[1]],"' to logical !") }}
  out }
  
#' Convert/standardize names of 'query' to standard names from 'ref' 
#'
#' This function converts/standardizes names of 'query' to standard names from 'ref' (list of possible names (char vect) where names define standardized name).
#' It takes 'query' as character vector and return character vecor (same length as 'query') with 'converted/corrected' names
#' 
#' @param query (matrix or data.frame, min 2 columns) main input
#' @param ref (list) list of multiple possible names associated to given group, reference name for each group is name of list
#' @param partMatch (logical) allows partial matching (ie name of 'ref' must be in head of 'query')
#' @return This function returns a character vector
#' @examples
#' daPa <- matrix(c(1:5,8,2:6,9), ncol=2)
#' @export
.convertNa <- function(query,ref,partMatch=TRUE){
  ## convert/standardize names of 'query' to standard names from 'ref' (list of possible names (char vect) where names define standardized name)
  ## take 'query' as character vector and return character vecor (same length as 'query') with 'converted/corrected' names
  ## names that are not found in 'ref' will be returned as NA
  ## 'ref' .. list of multiple possible names associated to given group, reference name for each group is name of list
  ## 'partMatch' allows partial matching (ie name of 'ref' must be in head of 'query')
  if(!partMatch) { out <- apply(sapply(ref,function(x) query %in% x),1,function(y) if(any(y)) which.max(y) else NA)                    # perfect match
    out[!is.na(out)] <- names(ref)[out[!is.na(out)]]
  } else {
    se <- sapply(ref, function(x) {unique(unlist(sapply(paste("^",x,sep=""), grep, query)))})
    out <- rep(NA,length(query))
     ##cat(" pos ",unlist(se),"  insert ",rep(names(se),sapply(se,length)),"\n")
    out[unlist(se)] <- rep(names(se), sapply(se, length)) }  #unlist(se)  
  out }
                  

#' Check for similar values in series
#'
#' This function checks all values of 'x' for similar neighbour values within (relative) range of 'ppm' (ie parts per milion as measure of distance).
#' By default values will be sorted internally, so if a given value of \code{x} has anywhere in  \code{x}  another value close enough, this will be detected.
#' However, if \code{sortX=FALSE} only the values next to left and right will be considered.
#' Return logical vector : FALSE for each entry of 'x' if value inside of ppm range to neighbour (of sorted values) 
#' @param x numeric vector
#' @param ppm (numeric, length=1) ppm-range for considering as similar
#' @param sortX (logical) allows speeding up function when set to FALSE, for large data that are already sorted
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a logical vector : \code{TRUE} for each entry of \code{x} where at least one neighbour is inside of ppm distance/range
#' @seealso similar with more options \code{\link{withinRefRange}}
#' @examples
#' va1 <- c(4:7,7,7,7,7,8:10)+(1:11)/28600; checkSimValueInSer(va1)
#' data.frame(va=sort(va1),simil=checkSimValueInSer(va1))
#' @export
checkSimValueInSer <- function(x, ppm=5, sortX=TRUE, silent=FALSE, debug=FALSE, callFrom=NULL) {
  fxNa <- .composeCallName(callFrom, newNa="checkSimValueInSer")
  if(isTRUE(debug)) silent <- FALSE
  if(!isTRUE(silent)) silent <- FALSE
  nNA <- sum(is.na(x))
  if(nNA >0 && !silent) message(fxNa,"Found ",nNA," NA values")
  so <- if(!isFALSE(sortX)) sort(x) else x
  di <- diff(so)
  diLim <- so *ppm *1e-6
  out <- rep(FALSE, length(x))
  names(out) <- names(so)
  out[-length(x)] <- abs(di) < abs(diLim[-length(x)])  # close to right neighbour
  out[-1] <- out[-1] | abs(di) < abs(diLim[-1])        # close to left neighbour
  if(sortX) out[rank(x, ties.method="first")] else out }

#' Simple Multi-to-Multi Matching of (Concatenated) Terms
#'
#' This function allows convenient matching of multi-to-multi relationships between two objects/vectors. 
#' It was designed for finding common elements in multiple to multiple matching situations (eg when comparing \code{c("aa; bb", "cc")}  to \code{c("bb; ab","dd")}, 
#' ie to find 'bb' as matching between both objects).
#'  
#' @param x (vector or list) first object to compare; if vector, the (partially) concatenated identifyers (will be split using separator \code{sep}), or list of items to be matched (ie already split) 
#' @param y (vector or list) second object to compare; if vector, the (partially) concatenated identifyers (will be split using separator \code{sep}), or list of items to be matched (ie already split) 
#' @param sep (character, length=1) separator used to split concatenated identifyers (if \code{x} or  \code{y} is vector)
#' @param sep2 (character, length=1) optional separator used when \code{method="matched"} to concatenate all indexes of \code{y} for column \code{y.allInd}
#' @param method (character) mode of operation: 'asIndex' to return index of y (those hwo have matches) with names of x (which x are the correpsonding match)
#' @param silent (logical) suppress messages
#' @param debug (logical) display additional messages for debugging
#' @param callFrom (character) allow easier tracking of message(s) produced
#'
#' @details
#' \code{method='byX'} .. returns data.frame with view oriented towards entries of \code{x}: character column \code{x} for entire content of \code{x}; integer column \code{x.Ind} for index of \code{x}; 
#'  character column \code{TagBest} for most frequent matching isolated tag/ID; integer column \code{y.IndBest} index of most frequent matching \code{y};
#'  character column \code{y.IndAll} index for all \code{y} matching any of the tags;
#'  character column \code{y.Match} for entire content of best matching \code{y}; 
#'  character column \code{y.Adj} for \code{y} adjusted to best matching \code{y} for easier subsequent perfect matching.
#'                                                                                                                                                       
#' \code{method=c("byX","filter")} .. combinded argument to keep only lines with any matches 
#'                                                                                                                                                       
#' \code{method='byTag'} .. returns matrix (of integers) from view of isolated tags from \code{x} (a separate line for each tag from  \code{x} matching to \code{y});
#' 
#' \code{method=c("byTag","filter")} ..if combined as arguments, this will return a data.frame for all unique tags with any matches between \code{x} and \code{y}, with 
#'   additional colunms \code{x.AllInd} for all matching \code{x}-indexes,  \code{y.IndBest} best matching \code{y} index; \code{x.n} for number of different \code{x} conatining this tag;
#' \code{y.AllInd} for all matching \code{y}-indexes 
#' 
#' \code{method='adjustXtoY'} .. returns vector with \code{x} adjusted to \code{y}, ie those elements of \code{x} matching are replace by the exact corresponding term of \code{y}.
#'                                                                                                                                                       
#' \code{method=NULL} .. If no term matching the options shown above is given, another version of 'asIndex' is returned, but indexes to \code{y} _after_ spliting by \code{sep}. 
#' Again, this method can be filtered by using \code{method="filter"} to focus on the best matches to \code{x}.  
#'                                                                                                                                                       
#' @return matrix, data.frame or list with matching results depending on \code{method} chosen
#' @seealso \code{\link[base]{match}}; \code{\link[base]{strsplit}}
#'                                                                                                                                                       
#' @examples
#' aa <- c("m","k", "j; aa", "m; aa; bb; o; ee", "n; dd; cc", "aa", "cc")
#' bb <- c("dd; r", "aa", "ee; bb; q; cc", "p; cc")
#' (match1 <- multiMatch(aa, bb, method=NULL))      # match bb to aa
#' (match2 <- multiMatch(aa, bb, method="byX"))     # match bb to aa
#' (match3 <- multiMatch(aa, bb, method="byTag"))   # match bb to aa
#' (match4 <- multiMatch(aa, bb, method=c("byTag","filter")))   # match bb to aa
#' 
#' @export
multiMatch <- function(x, y, sep="; ", sep2=NULL, method="byX", silent=FALSE, debug=FALSE, callFrom=NULL) {
  ## for finding common in multiple to multiple matching (eg c("aa; bb", "cc")  vs c("bb; ab","dd"))
  ## tells which x (names of output) was found matching (at least by 1 subunit) to which (part of) y, names of y tell which x (index)
  ## note: multiple matches are ignored (only 1st match reported)
  fxNa <- .composeCallName(callFrom, newNa="multiMatch")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  doMa <- TRUE
  if(length(x) <1) {doMa <- FALSE; if(!silent) message(fxNa,"Argument 'x' is empty, nothing to do !")}
  if(length(y) <1) {doMa <- FALSE; if(!silent) message(fxNa,"Argument 'y' is empty, nothing to do !")}
  out <- outF <- NULL
  if(doMa) {
    if(is.list(x)) {xL <- x; x <- sapply(x, paste, collapse=sep)} else xL <- strsplit(x, sep)
    if(is.list(y)) {yL <- y; y <- sapply(y, paste, collapse=sep)} else yL <- strsplit(y, sep)
    names(xL) <- 1:length(x)
    yV <- unlist(yL)
    names(yV) <- rep(1:length(y), sapply(yL, length))
    if(length(sep2) !=1) sep2 <- sep
    if(debug) {message(fxNa," doMa=",doMa,"   mM1")} 
    
    ## 'asIndex' .. return list (length matches x) where values indicate index of which y is matched by which elment of x (after split)
    ## check each x; determine  that in 3rd x 'aa' matches to 1st of 1st y; in 4th x 'aa' 
    out <- lapply(xL, function(z) {w <- match(yV, z); v <- !is.na(w); if(any(!v)) { v <- yV[which(v)]; u <- as.integer(names(v)); names(u) <- v;
      chDu <- duplicated(paste(names(u),u)); if(any(chDu)) u <- u[which(!chDu)]; u } else NULL })
    
    ## filter1 : filter to most frequent (or first of same freq)
    if(length(grep("^filter",method)) >0 && !"byTag" %in% method) out <- outF <- lapply(out, function(z) if(length(z) >1) z[which(z ==names(which.max(table(z))) )] else z)    
    if(debug) {message(fxNa," method=",method,"   mM2")} 

    ## by isolated tag/ID
    if("byTag" %in% method) { nLi <- sapply(out, length); tmp <- rep(nLi,nLi)
      mat <- cbind(x.Ind=as.integer(names(tmp)), yInd=unlist(out))
      rownames(mat) <- as.character(unlist(sapply(out, names)))
      chNa <- is.na(mat[,2])
      if(any(chNa)) mat <- mat[which(!chNa),]
      mat <- mat[order(rownames(mat), mat[,1]),]
      if(length(grep("^filter", method)) >0) {
        xV <- unlist(xL)
        tmX <- tapply(mat[,1], rownames(mat), function(z) c(IndBest=z[which.max(z)], n=length(unique(z)), allInd=paste(unique(z), collapse=sep2)))
        tmY <- tapply(mat[,2], rownames(mat), function(z) c(IndBest=z[which.max(z)], n=length(unique(z)), allInd=paste(unique(z), collapse=sep2)))
        tmp <- cbind(matrix(unlist(tmX), nrow=length(tmX), byrow=TRUE), matrix(unlist(tmY), nrow=length(tmY), byrow=TRUE))
        dimnames(tmp) <- list(sort(unique(rownames(mat))), paste(rep(c("x","y"), each=3),rep(c("IndBest","n","AllInd"),2), sep="."))
        mat <- data.frame(x.IndBest=as.integer(tmp[,1]), x.n=as.integer(tmp[,2]), x.AllInd=tmp[,3], 
          y.IndBest=as.integer(tmp[,4]), x.n=as.integer(tmp[,5]), y.AllInd=tmp[,6])
      }
      out <- mat }
    if(debug) {message(fxNa,"  mM3")} 
    if("byX" %in% method) { 
      df1 <- data.frame(x=x,x.Ind=1:length(x), TagBest=NA, y.IndBest=NA, y.IndAll=NA, y.Match=NA, y.Adj=NA)
      nLi <- sapply(out,length)
      df1[which(nLi >0),"y.IndAll"] <- sapply(out[which(nLi >0)], function(z) paste(z, collapse=sep2))
      ## for IndBest
      outF <- if(length(outF) <1 | any(sapply(outF,length) >1)) lapply(out, 
        function(z) if(length(z) >1) z[which( z ==names(which.max(table(z))) )][1] else z[1]) else outF 
      df1[which(nLi >0),"y.IndBest"] <- as.integer(sapply(outF[which(nLi >0)], function(z) paste(z, collapse=sep2)))
      df1[which(nLi >0),"TagBest"] <- naOmit(sapply(outF, function(z) names(z)[1]))
      df1[which(nLi >0),"y.Match"] <- y[df1[which(nLi >0),"y.IndBest"]]
      df1[which(nLi >0),"y.Adj"] <- df1[which(nLi >0),"x"]
      if(length(grep("^filter", method)) >0) df1 <- df1[which(!is.na(df1[,"TagBest"])),]      
      out <- df1 }
    if(is.list(out) && length(grep("^filter", method)) >0) out <- out[which(sapply(out,length) >0)]  
  }      
  out }   
     

#' Extract specific text
#'
#' This function extracts/cuts text-fragments out of \code{txt} following specific anchors defined by arguments \code{cutFrom} and \code{cutTo}.
#'
#' @details
#' In case \code{cutFrom} is not found \code{missingAs} will be returned.
#' In case \code{cutTo} is not found, text gets extracted with \code{chaMaxEl} characters.
#'
#' @param txt character vector to be treated
#' @param cutFrom (character) text where to start cutting
#' @param cutTo (character) text where to stop cutting
#' @param missingAs (character) specific content of output at line/location of 'exclLi' 
#' @param exclFromTag (logical) to exclude text given in 'cutFrom' from result
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a modified character vector
#' @seealso  \code{\link[base]{substr}} 
#' @examples
#' extrSpcText(c(" ghjg GN=thisText PE=001"," GN=_ PE=", NA, "abcd"))
#' extrSpcText(c("ABCDEF.3-6","05g","bc.4-5"), cutFr="\\.", cutT="-")
#' @export
extrSpcText <- function(txt, cutFrom=" GN=", cutTo=" PE=", missingAs=NA, exclFromTag=TRUE, silent=FALSE, debug=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom, newNa="extrSpcText")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  if(!isTRUE(silent)) silent <- FALSE
  seFr <- gregexpr(cutFrom, txt)
  seTo <- gregexpr(cutTo, txt)
  chOc <- cbind(fr=sapply(seFr, length), to=sapply(seTo, length))
  chaFr <- if(all(chOc[,1]==1)) unlist(seFr) else sapply(seFr, function(x) x[1])   # if search-term found mult times, take 1st
  chaTo <- if(all(chOc[,2]==1)) unlist(seTo) else sapply(seTo, function(x) x[1])   # if search-term found mult times, take 1st
  exclLi <- which(chaFr <0 | is.na(txt))
  if(exclFromTag) chaFr[which(chaFr >0)] <- chaFr[which(chaFr >0)] +nchar(sub("\\\\","",cutFrom)) 
  if(any(chaTo <0)) {                                                             # gregexpr returns -1 if term not found -> not usable index
    if(!silent) message(fxNa,sum(chaTo <0,na.rm=TRUE)," 'cutTo' tags not found !")
    }
  out <- substr(txt, chaFr, chaTo-1)
  if(length(exclLi) >0) out[exclLi] <- missingAs
  out }


#' Search character-string and cut either before or after 
#'
#' This function extracts/cuts text-fragments out of \code{txt} following specific anchors defined by arguments \code{cutFrom} and \code{cutTo}.
#'
#'
#' @param x character vector to be treated
#' @param searchChar (character) text to look for
#' @param after (logical) 
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a modified character vector
#' @seealso  \code{\link[base]{grep}} 
#' @examples
#' .cutAtSearch("abcdefg","de")
#' @export
.cutAtSearch <- function(x, searchChar, after=TRUE, silent=TRUE, debug=FALSE, callFrom=NULL) {
  ## search 'searchChar' within x (character vector) and cut either before or after values of 'x'
  ## for simple cutting before searchCharacter use .retain1stPart()
  ## 'after' =TRUE means that all text BEFORE 'searchChar' will be returned
  searArg <- if(!silent) deparse(substitute(searchChar)) else "searchChar"
  fxNa <- .composeCallName(callFrom, newNa=".cutAtSearch")
  pos <- gregexpr(searchChar, x)
  ch <- pos <0
  if(any(ch)) pos[which(ch)] <- -1*nchar(searchChar)
  if(max(sapply(pos, length),na.rm=TRUE) >1) {
    if(!silent) message(fxNa," multiple occurances of '",searArg,"' found")
    pos <- sapply(pos,function(z) z[1])
  } else pos <- unlist(pos)
  out <- if(after) substr(x, pos +nchar(searchChar), nchar(x)) else substr(x, 1, pos -1)
  if(!after) {ch <- out=="" & pos <0; if(any(ch)) out[which(ch)] <- x[which(ch)]}
  names(out) <- x
  out }

#' Trim character string: keep only text before 'sep'
#'
#' Trim character string: keep only text before 'sep' (length=1 !)
#' 
#' @param chr character vector to be treated
#' @param sep (character) saparator
#' @param offSet (integer) off-set
#' @return This function returns a modified character vector
#' @seealso  \code{\link[base]{substr}} 
#' @examples
#' .retain1stPart("abc = def")
#' @export
.retain1stPart <- function(chr, sep=" = ", offSet=1){
  ## trim character string: keep only text before 'sep' (length=1 !)
  ## offSet allows to cut n characteres before occurance of 'sep' (character of length=1 or neg value for cut after)
  ## error if all 'chr' NA !
  ## attention special characters as sep ('+' or '.') need to be protected !! eg # if(sep %in% c("+",".")) sep <- paste("\\",sep,sep="")
  ## see also volcDat  .cutAtSearch() for cutting after ...
  chrCh <- regexpr(sep[1], chr)
  if(any(naOmit(chrCh >0))) { trimNo <- which(chrCh >0)
    chr[trimNo] <- substr(chr[trimNo], 1, chrCh[trimNo] -offSet)}
  chr }
    

#' Normalize By Adjusting Exponent
#'
#' This function normalizes 'dat' by optimizing exponent function (ie dat ^exp) to fit best to 'ref' (default: average of each line of 'dat').
#'
#' @param dat matrix or data.frame of numeric data to be normalized
#' @param useExpon (numeric vector or matrix) exponent values to be tested
#' @param dynExp (logical) require 'useExpon' as 2 values (matrix), will gradually increase exponent from 1st to 2nd; may be matrix or data.frame for dynamic, 
#' in this case 1st line for exp for lowest data, 2nd line for highest
#' @param nStep (integer) number of exponent variations (steps) when testing range from-to
#' @param startExp (numeric)
#' @param simMeas (character) similarity metric to be used (so far only "cor"), if rSquare=TRRUE, the r-squared will be returned
#' @param refDat (matrix or data.frame) if null average of each line from 'dat' will be used as reference in similarity measure
#' @param refGrp (factor) designing which col of 'ref' should be used with which col of 'dat' (length equal to number of cols in 'dat').  
#'  Note: 'refGrp' not yet coded optimally to extract numeric part of character vector, protential problems when all lines or cols of dat are NA
#' @param refLines (NULL or integer) optional subset of lines to be considered (only) when determining normalization factors
#' @param rSquare (logical) if \code{TRUE}, add r-squared
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allows easier tracking of messages produced
#' @return This functuion returns a matrix of normalized data
#' @seealso more eveolved than \code{\link[wrMisc]{normalizeThis}} with arugment set to 'exponent'
#' @examples
#' set.seed(2016); dat1 <- matrix(c(runif(200)+rep(1:10,20)),nc=10)
#' head(rowGrpCV(dat1,gr=gl(4,3,labels=LETTERS[1:4])[2:11]))
#' set.seed(2016); dat1 <- c(0.1,0.2,0.3,0.5)*rep(c(1,10),each=4)
#' dat1 <- matrix(round(c(sqrt(dat1),dat1^1.5,3*dat1+runif(length(dat1))),2),nc=3)
#' dat2a <- exponNormalize(dat1[,1],useExpon=2,nSte=1,refD=dat1[,3])
#' layout(matrix(1:2,nc=2))
#' plot(dat1[,1],dat1[,3],type="b",main="init",ylab="ref") 
#' plot(dat2a$datNor[,1],dat1[,3],type="b",main="norm",ylab="ref")
#' dat2b <- exponNormalize(dat1[,1],useExpon=c(1.7,2.3),nSte=5,refD=dat1[,3])
#' plot(dat1[,1],dat1[,3],type="b",main="init",ylab="ref")
#' plot(dat2b$datNor[,1],dat1[,3],type="b",main="norm",ylab="ref")
#' 
#' dat2c <- exponNormalize(dat1[,-3],useExpon=matrix(c(1.7,2.3,0.6,0.8),nc=2),nSte=5,refD=dat1[,3]);
#' plot(dat1[,1],dat1[,3],type="b",main="init",ylab="ref ")
#' plot(dat2c$datNor[,1],dat1[,3],type="b",main="norm 1",ylab="ref")
#' plot(dat1[,2],dat1[,3],type="b",main="init",ylab="ref")
#' plot(dat2c$datNor[,2],dat1[,3],type="b",main="norm 2",ylab="ref"); 
#' @export
exponNormalize <- function(dat, useExpon, dynExp=TRUE, nStep=20, startExp=1, simMeas="cor", refDat=NULL, refGrp=NULL, refLines=NULL, rSquare=FALSE, silent=FALSE, callFrom=NULL, debug=FALSE) {
  fxNa <- .composeCallName(callFrom,newNa="exponNormalize")
  dimDat <- dim(dat)
  if(!is.matrix(dat)) dat <- as.matrix(dat)
  if(is.null(refDat)) refDat <- matrix(rowMeans(if(length(refLines) < nrow(dat) && !is.null(refLines)) dat[refLines,, drop=FALSE] else dat, na.rm=TRUE),
    ncol=1,dimnames=list(NULL,colnames(dat))) else { message(fxNa,"adj 'refDat to matrix' \n")
    if(length(dim(refDat)) <2) refDat <- as.matrix(as.numeric(refDat)) }
  if(is.null(refGrp)) refGrp <- rep(1, ncol(dat))
  if(length(refGrp) != ncol(dat)) message(fxNa," 'refGrp' (",length(refGrp),") doesn't fit to 'dat' (",ncol(dat)," cols)")
  if(max(as.numeric(refGrp), na.rm=TRUE) > ncol(dat)) {
    message(fxNa," 'refGrp' suggests columns not found in 'dat' !") }
  useExpon <- unique(naOmit(useExpon))
  ## main
  .sw <- function(useExpon, startExp, nStep) {     # function for defining new series of 'useExpon'
    meth <- paste0("m",0 +(length(useExpon) >1) + 2*all(startExp <1))
    switch(meth,
      m0=seq(useExpon, startExp, length.out=nStep),
      m1=seq(useExpon[1], useExpon[2], length.out=nStep),
      m2=seq(startExp, useExpon, length.out=nStep),
      m3=seq(useExpon[1], useExpon[2], length.out=nStep) ) }
  if(length(dim(useExpon)) >1 & dynExp & nStep >1) {    # diff/indep expon series for each col of data
    useExpon <- apply(useExpon,2,.sw,startExp,nStep)
    if(!silent) message(fxNa," column-specific exponent testing of ",nrow(useExpon)," x ",ncol(useExpon)," exponents")
  } else {
    if(nStep >1) useExpon <- .sw(useExpon,startExp,nStep)
    if(!silent) message(fxNa," static exponent series (length ",length(useExpon),")")}
  if(!is.matrix(useExpon)) useExpon <- matrix(rep(useExpon,ncol(dat)), ncol=ncol(dat))
  expoNor <- list()
  for(i in 1:nrow(useExpon)) expoNor[[i]] <- dat^matrix(rep(useExpon[i,], each=nrow(dat)), ncol=ncol(dat))
  corVal <- if(identical(simMeas,"cor")) sapply(expoNor, function(x) apply(x,2,stats::cor, y=refDat, use="complete.obs")) else {
    corVal <- NULL; message(fxNa,"PROBLEM : unknown similarity measure !!")}
  if(!is.matrix(corVal)) corVal <- matrix(corVal, ncol=ncol(dat))
  if(all(dim(corVal) == dim(t(useExpon)))) corVal <- t(corVal)
  dimnames(corVal) <- list(paste("exp",useExpon[,1],sep="_"), colnames(dat))
  bestExpPo <- apply(corVal, 2, which.max)
  bestExp <- apply(rbind(bestExpPo, useExpon), 2, function(x) x[-1][x[1]])
  datNor <- matrix(nrow=nrow(dat), ncol=ncol(dat), dimnames=dimnames(dat))
  for(i in 1:ncol(dat)) datNor[,i] <- dat[,i]^bestExp[i]
  out <- list(bestExp=bestExp, datNor=datNor, allSimil = if(rSquare) corVal^2 else corVal)
  out }
    

#' Non-Redundant Lines Of Matrix
#'
#' This function reduces complexity of matrix (or data.frame) if multiple consectuive (!) lines with same values.
#' Return matrix (or data.frame) without repeated lines (keep 1st occurance)
#' @param dat (matrix or data.frame) main input
#' @param silent (logical) suppress messages if \code{TRUE}
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allows easier tracking of messages produced
#' @return This function returns a matrix (or data.frame) without repeated lines (keep 1st occurance)..
#' @seealso \code{\link{firstLineOfDat}}, \code{\link{firstOfRepLines}}, \code{\link{findRepeated}}, \code{\link{firstOfRepeated}}, \code{\link{get1stOfRepeatedByCol}}, \code{\link{combineRedBasedOnCol}}, \code{\link{correctToUnique}} 
#' @examples
#' mat2 <- matrix(rep(c(1,1:3,3,1),2),ncol=2,dimnames=list(letters[1:6],LETTERS[1:2]))
#' nonRedundLines(mat2)
#' @export
nonRedundLines <- function(dat, silent=FALSE, debug=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom,newNa="nonRedundLines")
  msg <- c(fxNa," expecting matrix or data.frame with >1 lines and >= 1 column(s)")
  if(length(dim(dat)) <2) stop(fxNa, msg)
  if(any(nrow(dat) <2, ncol(dat) <1)) stop(fxNa, msg)
  if(is.null(rownames(dat))) rownames(dat) <- 1:nrow(dat)
  exclLi <- which(rowSums(dat[-nrow(dat),, drop=FALSE] == dat[-1,, drop=FALSE]) ==ncol(dat)) +1
  out <- if(length(exclLi) >0) dat[-exclLi,, drop=FALSE] else dat
  if(length(dim(out)) <2) out <- matrix(out, ncol=ncol(dat),
    dimnames=list(rownames(dat)[-1*exclLi],colnames(dat)))
  out }
   

#' Print matrix-content as plot
#'
#' When data have repeated elements (defined by names inside the vector), it may be advantageous to run some operations 
#' only on a unique set of the initial data, or somtimes all repeated occurances need to be replaced by a common (summarizing) value.
#' This function allows to re-introduce new values from on second vector with unique names, to return a final vector of initial input-length and order of names (elements) like initial, too.
#' Normally the user would provide 'datUniq' (without repeated names) containing new values which will be expanded to structure of 'dat', 
#' if 'datUniq' is not provided a vector with unique names will be made using the first occurance of repeated value(s).
#' For more complex cases the indexing relative to 'datUniq' can be returned (setting \code{asIndex=TRUE}).
#' Note: If not all names of 'dat' are found in 'datUniq' the missing spots will be returned as \code{NA}.
#' 
#' @param dat (numeric or character) main long input, must have names
#' @param datUniq (numeric or character) will be used to impose values on \code{dat}, must have names that should match names (at least partially) from \code{dat}
#' @param asIndex (logical) if \code{TRUE} index values will be returned instead of replacing values
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return vector of length \code{dat} with imposed values, or index values if \code{asIndex=TRUE}
#' @seealso \code{\link[base]{unique}}, \code{\link{findRepeated}}, \code{\link{correctToUnique}}, \code{\link{treatTxtDuplicates}}
#' @examples
#' dat <- 11:19
#' names(dat) <- letters[c(6:3,2:4,8,3)]
#' ## let's make a 'datUniq' with the mean of repeated values :
#' datUniq <- round(tapply(dat,names(dat),mean),1)
#' ## now propagate the mean values to the full vector
#' getValuesByUnique(dat,datUniq)
#' cbind(ini=dat,firstOfRep=getValuesByUnique(dat,datUniq),
#'   indexUniq=getValuesByUnique(dat,datUniq,asIn=TRUE))
#' @export
getValuesByUnique <- function(dat, datUniq=NULL, asIndex=FALSE, silent=FALSE, debug=FALSE, callFrom=NULL) {
  ## main
  fxNa <- .composeCallName(callFrom, newNa="getValuesByUnique")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE

  if(length(dat) <1 | is.null(names(dat))) stop("'dat' must be vector of length >0 with names; no names nothing to do")
  if(length(datUniq) >0) if(is.null(names(datUniq))) {
    if(!silent) message(fxNa," 'datUniq' has no names; unable to use ! (using default instead)")
    datUniq <- NULL
  }
  if(is.null(datUniq)) datUniq <- dat[!duplicated(names(dat))]
  if(identical(dat,datUniq)) {out <- if(asIndex) 1:length(dat) else dat
  } else {
    out <- rep(NA,length(dat))
    names(out) <- names(dat)
    ## note: this step performs much faster using match than via as.integer(as.factor) !
    ind <- match(names(dat),names(datUniq))
    if(!asIndex) out[which(!is.na(ind))] <- datUniq[naOmit(ind)] else out <- ind
    chNa <- is.na(out)
    if(any(chNa)) {
      if(!silent) message(fxNa," ",sum(chNa)," name(s) of 'dat' not found in 'datUniq' !")
    }
  }
  out}
     

#' Replace Most Distant Values by NA
#'
#' This procedures aims to streighten (clean) the most extreme values of noisy replicates by identifying the most distant points 
#' (among a set of replicates). The input 'x' (matrix or data.frame) is supposed to come from multiple different measures taken 
#' in replicates (eg weight of different individuals as rows taken as multiple replicate measures in subsequent columns).
#' 
#' @details
#' With the argument \code{nOutl} the user chooses the total number of most extreme values to replace by \code{NA}.
#' how many of the most extreme replicates of the whole dataset will replaced by \code{NA}, ie with \code{nOutl=1} 
#' only the single most extreme outlyer will be replaced by \code{NA}. 
#' Outlier points are determined as point(s) with highest distance to (row) center (median and mean choice via argument 'centrMeth').
#' Thus function returns input data with "removed" points set to \code{NA}, or if \code{retOffPos=TRUE} the most extreme/outlier positions.
#'
#' @param x matrix (or data.frame)
#' @param centrMeth (character) method to summarize (mean or median)
#' @param nOutl (integer) determines how many points per line will be set to \code{NA} (with n=1 the worst row of replicates will be 'cleaned')
#' @param retOffPos (logical) if \code{TRUE}, replace the most extreme outlyer only
#' @param silent (logical) suppres messages
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a matrix of same dimensions as input \code{x},  data-points which were tagged/removed are set to \code{NA}, or if \code{retOffPos=TRUE} the most extreme/outlier positions
#' @examples
#' mat3 <- matrix(c(19,20,30, 18,19,28, 16,14,35),ncol=3)
#' cleanReplicates(mat3, nOutl=1)
#' @export
cleanReplicates <- function(x, centrMeth="median", nOutl=2, retOffPos=FALSE, silent=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom, newNa="cleanReplicates")
  if(!any(is.data.frame(x),is.matrix(x))) stop(" 'x' is assumed as matrix or data.frame")
  if(length(dim(x)) !=2) stop(fxNa," Designed for 2dim 'x'  (here ",if(is.null(ncol(x))) "none" else ncol(x),")")
  if(nOutl >= ncol(x)) stop(fxNa," Argument 'nOutl' should not reach or exceed number of columns of 'x'  (here ",ncol(x)," cols)")
  if(is.data.frame(x)) x <- as.matrix(x)
  if(length(table(table(rownames(x)))) != 1) {
    if(!silent) message(fxNa," rownames of 'x' either NULL or not unique, replacing by row-numbers")
    rownames(x) <- 1:nrow(x) }
  xCV <- rowCVs(x)    
  xCV <- xCV[which(rowSums(!is.na(x)) >0)]
  badCV <- which(xCV >= sort(xCV, decreasing=TRUE, na.last=TRUE)[nOutl])
  if(!silent) message(fxNa,"removing ",length(badCV)," entries in lines ",paste(names(badCV),collapse=","))
  centr <- if(identical(centrMeth, "median")) apply(x, 1, stats::median,na.rm=TRUE) else rowMeans(x, na.rm=TRUE)
  di <- abs(x - matrix(rep(centr, ncol(x)), nrow=nrow(x)))
  di[which(rowSums(!is.na(di)) <1), ] <- 0
  if(!silent & sum(is.infinite(di)) >0) message(fxNa," Attention, ",sum(is.infinite(di))," distance values are infinite !")
  for(i in 1:length(badCV)) {y <- badCV[i]
    x[match(names(y), rownames(x)), which(di[y,] ==max(di[y,], na.rm=TRUE))[1]] <- NA }
  x }
    

#' Standardize (scale) data
#'
#' This functions work similar to \code{\link[base]{scale}}, however, it evaluates the entire input and not column-wise (and independeltly as \code{scale} does). 
#' With Standarizing we speak of transforming the data to end up with mean=O and sd=1.
#' Furthermore, in case of 3-dim arrays, this function returns also an object with the same dimensions as the input.
#' 
#' @param mat (matrix, data.frame or array) data that need to get standardized.
#' @param byColumn (logical) if \code{TRUE} the function will be run independently over all columns such as as \code{apply(mat,2,standardW)}  
#' @param na.rm (logical) if \code{NA}s in the data don't get ignored via this argument, the output will be all \code{NA} 
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This functions retruns a vector of rescaled data (in dimensions as input)
#' @seealso \code{\link[base]{scale}} 
#' @examples
#' dat <- matrix(2*round(runif(100),2), ncol=4)
#' mean(dat); sd(dat)
#' 
#' dat2 <- standardW(dat)
#' apply(dat2, 2, sd)
#' summary(dat2)
#' 
#' dat3 <- standardW(dat, byColumn=TRUE)
#' apply(dat2, 2, sd)
#' summary(dat2)
#' mean(dat2); sd(dat2)
#' 
#' @export
standardW <- function(mat, byColumn=FALSE, na.rm=TRUE, silent=FALSE, debug=FALSE, callFrom=NULL) {   
  ## standardize as entire matrix (ie not column-wise, relative differences in row get thus conserved), specific cols may be selected
  ## used to be standEntMatr
  fxNa <- .composeCallName(callFrom, newNa="standardW")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  if(debug) message(fxNa,"sW1")

  if(is.data.frame(mat)) mat <- as.matrix(mat)
  if(byColumn && length(dim(mat) >1)) { std <- function(x) (mat -mean(x, na.rm=na.rm))/stats::sd(x, na.rm=na.rm) 
    if(length(dim(mat)) ==2) apply(mat, 2, std) else {if(length(dim(mat)) ==3) apply(mat, 2:3, std) }
  } else (mat -mean(mat, na.rm=na.rm)) /stats::sd(mat, na.rm=na.rm)
}
  

#' Filter lines(rows) and/or columns from all suitable elements of list
#'
#' Filter all elements of list (or S3-object) according to criteria designed to one selected reference-element of the list.
#' All simple vectors, matrix, data.frames and 3-dimensional arrays will be checked if matching number of rows and/or columns to decide if they should be filtered the same way.
#' If the reference element has same number of rows and columns simple (1-dimensional) vectors won't be filtered since it not clear if this should be done to lines or columns.
#'
#' @details
#' This function is used eg in package wrProteo to simultaneaously filter raw and transformed data.
#'
#' @param lst (list or S3 object) main input
#' @param useLines (integer, logcial or character) vector to assign lines to keep when filtering along lines;
#'   set to \code{NULL} for no filtering; if '\code{allNA}' all lines composed uniquely of \code{NA} values will be removed.
#' @param useCols (integer, logcial or character) vector for filtering columns; set to \code{NULL} for no filtering; if '\code{allNA}' all columns uniquely \code{NA} values will be removed
#' @param ref (integer) index for designing the elment of 'lst' to take as reference for checking which other list-elements have suitable number of rows or columns
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns the correct(ed) input (object of same class, of same length)
#' @seealso \code{\link{moderTest2grp}} for single comparisons, \code{\link[limma]{lmFit}}
#' @examples
#' lst1 <- list(m1=matrix(11:18,ncol=2), m2=matrix(21:30,ncol=2), indR=31:34,
#'   m3=matrix(c(21:23,NA,25:27,NA),ncol=2))
#' ## here $m2 has more lines than $m1, and thus will be ignored when ref=1
#' filterLiColDeList(lst1, useLines=2:3)
#' filterLiColDeList(lst1, useLines="allNA", ref=4)
#'
#' @export
filterLiColDeList <- function(lst, useLines, useCols=NULL, ref=1, silent=FALSE, callFrom=NULL, debug=FALSE) {
  ## filter all list-elements matching number of lines from the 'ref'-th list-element
  ## 'ref' .. designs list-element where number of rows will be compared to other list-elements to decide which list-elements (have same number of rows and thus) will be filtered
  fxNa <- .composeCallName(callFrom, newNa="filterLiColDeList")
  msg <- "invalid argument 'ref' (should be index to existing element of 'lst')"
  doFilt <- TRUE
  chLe <- c(lst=length(lst), useLines=length(useLines), ref=length(ref))
  if(any(chLe <0)) doFilt <- FALSE
  if(doFilt) {
    if(length(ref) >1) { warning("'ref' should be of length=1 (ie only ONE reference) !  Reducing to first .."); ref <- ref[1]}
    if(ref > length(lst)) stop(msg)
    dims <- lapply(lst, dim)
    ## filter lines
    if(length(useLines) >0) {
      if(identical(useLines,"allNA")) {
        chFormat <- length(dim(lst[[ref]])) >1
        if(!chFormat) { lst[[ref]] <- as.matrix(lst[[ref]])
          message(fxNa,"It appears lst[[ref]] is not matrix (or data.frame) ! Trying to reformat ..")
          dims <- lapply(lst, dim)                           # update
        }
        useLines <- rowSums(is.na(lst[[ref]])) < ncol(lst[[ref]]) }
      if(debug) {message(fxNa,"  fLCL1"); fLCL1 <- list(lst=lst,useLines=useLines,useCols=useCols,ref=ref,dims=dims,useLines=useLines) }
      if(is.logical(useLines)) {           # convert logical argument to index
        ch <- identical(length(useLines), dims[[ref]][1])
        if(!ch && !silent) message(fxNa,"Problem/ignoring filtering lines : 'useLines' is of le length ",length(useLines)," but expecting ",dims[[ref]][1]," !")
        useLines <- if(ch) which(useLines) else NULL }
      if(is.numeric(useLines)) {
        useLines <- naOmit(useLines)
        if(min(useLines) <1 || max(useLines) > dims[[ref]][1]) {
          if(!silent) message(fxNa,"'useLines' may not design lines higher than number of lines in ref, neither may not be negative")
          useLines <- NULL }
      }
      if(debug) {message(fxNa,"  fLCL2"); fLCL2 <- list() }
      if(length(useLines) >0 && identical(length(useLines), dims[[ref]][1])) {
        if(!silent) message(fxNa,"'useLines' seems empty, nothing to do ...")
      } else {
        useEl <- sapply(dims, function(x) if(length(x) >1) x[1] else 0) == dims[[ref]][1]   # compare to ref
        if(any(useEl)) for(i in which(useEl)) {lst[[i]] <- if(length(dims[[i]])==2) lst[[i]][useLines,] else lst[[i]][useLines,,]}
        if(!silent) message(fxNa,"successfully filtered ",pasteC(names(lst)[which(useEl)],quoteC="'")," from ",dims[[ref]][1]," to ",length(useLines)," lines") }
      ## check for single vectors matching nrow of ref (as long nrow not equal ncol of ref)
      chV <- sapply(dims, length)==0  & dims[[ref]][1]==dims[[ref]][2]
      if(any(chV)) { chV <- which(chV)
        chL <- sapply(lst[chV], length) == dims[[ref]][1]
        if(any(chL)) for(i in which(chL)) {lst[[i]] <- lst[[i]][useLines]}
      }
      if(debug) {message(fxNa,"  fLCL2b"); fLCL2b <- list() }
    }
    if(debug) {message(fxNa,"  fLCL3"); fLCL3 <- list(lst=lst,useLines=useLines,useCols=useCols,ref=ref,doFilt=doFilt) }
    ## filter columns
    if(length(useCols) >0) {
      if(is.logical(useCols)) {
        ch <- length(useCols) == dims[[ref]][2]
        useLines <- if(ch) which(useCols) else NULL }
      if(debug) {message(fxNa,"  fLCL4a"); fLCL4a <- list(lst=lst,useLines=useLines,useCols=useCols,ref=ref,doFilt=doFilt,useCols=useCols) }
      if(is.numeric(useCols)) {
        useCols <- naOmit(useCols)
        if(min(useCols) <1 || max(useCols) > dims[[ref]][2]) {
          if(!silent) message(fxNa,"'useCols' may not design columns higher than number of columns in ref, neither may not be negative")
          useCols <- NULL }
      }
      if(debug) {message(fxNa,"  fLCL4b"); fLCL4b <- list() }
      if(length(useCols) >0 && length(useCols)==dims[[ref]][2]) {
        if(!silent) message(fxNa,"'useCols' seems empty, nothing to do ...")
      } else {
        useEl <- sapply(dims, function(x) if(length(x) >1) x[2] else 0) == dims[[ref]][2]
        if(any(useEl)) for(i in which(useEl)) {message(fxNa,"i=",i); lst[[i]] <- if(length(dims[[i]])==2) lst[[i]][,useCols] else lst[[i]][,useCols,]}
        if(!silent) message(fxNa,"successfully filtered ",pasteC(names(lst)[which(useEl)],quoteC="'")," from ",dims[[ref]][2]," to ",length(useCols)," columns") }
      if(debug) {message(fxNa,"  fLCL4c"); fLCL4c <- list() }
      ## check for single vectors matching ncol of ref (as long nrow not equal ncol of ref)
      chV <- sapply(dims, length)==0  & dims[[ref]][1]==dims[[ref]][2]
      if(any(chV)) { chV <- which(chV)
        chL <- sapply(lst[chV], length) == dims[[ref]][2]
        if(any(chL)) for(i in which(chL)) {lst[[i]] <- lst[[i]][useCols] }
      }
    } } else if(!silent) message(fxNa,"Incomplete data - nothing to do; either 'lst','useLines' or 'ref' is empty !")
  lst }
  

#' Number Of Fragments After Cut At Specific Character(s)
#'
#' This function tells the number of fragments/entry when cutting after 'cutAt' 
#' @param protSeq (character) text to be cut
#' @param cutAt (integer) position to cut
#' @param silent (logical) suppress messages if \code{TRUE}
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allows easier tracking of messages produced
#' @return This function returns a numeric vector with number of fragments for each entry 'protSeq' (names are 'protSeq')
#' @seealso  more elaborate \code{{nFragments}}; \code{\link{cutAtMultSites}}
#' @examples
#' tmp <- "MSVSRTMEDSCELDLVYVTERIIAVSFPSTANEENFRSNLREVAQMLKSKHGGNYLLFNLSERRPDITKLHAKVLEFGWPDLHTPALEKI" 
#' nFragments0(c(tmp,"ojioRij"),c("R","K"))
#' @export
nFragments0 <- function(protSeq, cutAt, silent=FALSE, debug=FALSE, callFrom=NULL) {
  fxNa <- .composeCallName(callFrom, newNa="nFragments0")  
  sapply(protSeq, function(x) length(cutAtMultSites(x, cutAt)))}

#' Number Of Fragments After Cut At Specific Character(s) Within Size-range
#'
#' This function determines number of fragments /entry within range of 'sizeRa' (numeric,length=2) when cutting after 'cutAt' 
#' @param protSeq (character)  text to be cut
#' @param cutAt (character) position to cut
#' @param sizeRa (numeric,length=2) min and max size to consider
#' @param silent (logical) suppress messages if \code{TRUE}
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allows easier tracking of messages produced
#' @return This function returns a numeric vector with number of fragments for each entry 'protSeq' (names are 'protSeq')
#' @seealso \code{\link{cutAtMultSites}}, simple version \code{{nFragments0}} (no size-range) 
#' @examples
#' tmp <- "MSVSREDSCELDLVYVTERIIAVSFPSTANEENFRSNLREVAQMLKSKHGGNYLLFNLSERRPDITKLHAKVLEFGWPDLHTPALEKI"
#' nFragments(c(tmp,"ojioRij"),c("R","K"),c(4,31))
#' 
#' @export
nFragments <- function(protSeq, cutAt, sizeRa, silent=FALSE, debug=FALSE, callFrom=NULL) {   # number of fragments /entry within range of 'sizeRa' (numeric,length=2) when cutting after 'cutAt' 
  fxNa <- .composeCallName(callFrom, newNa="nFragments")
  if(length(naOmit(unique(sizeRa))) <2) stop(" 'sizeRa' should be numeric of length=2")
  if(length(cutAt) <0) rep(1, length(protSeq)) else {
    sapply(protSeq, function(x) {y <- cutAtMultSites(x, cutAt)
    sum(nchar(y) > min(sizeRa, na.rm=TRUE) & nchar(y) < max(sizeRa,na.rm=TRUE), na.rm=TRUE)})}}
     

#' Order Lines of Matrix According to Reference (Character) Vector
#'
#' @description
#' This function orders lines of matrix \code{mat} according to a (character) reference vector \code{ref}.
#' To do so, all columns of \code{mat} will be considered to use the first column from left with the best (partial) matching results.
#' This function first looks for unambiguous perfect matches, and if not found successive rounds of more elaborte partial matching will be engaged: 
#' In case of no perfect matches found, grep of \code{ref} on all columns of \code{mat} and/or grep of all columns of \code{mat} on \code{ref} (ie 'reverse grep') will be applied (finally a 'two way grep' approach).
#' Until a perfect match is found each element of \code{ref} will be tested on \code{mat} and inversely (for each column) each element of \code{mat} will be tested on \code{ref}.
#' The approach with the best number of (unique) matches will be chosen. In case of one-to-many matches, it will be tried to use most complete lines (see also last example).
#'
#' @param mat (matrix, data.frame) main input of which rows should get re-ordered according to a (character) reference vector \code{ref}
#' @param ref (character) reference imposing new order
#' @param addRef (logical) add \code{ref} to output as new column
#' @param listReturn (logical) allows retrieving more information in form of list
#' @param silent (logical) suppress messages
#' @param debug (logical) display additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns, depending on \code{listReturn}, either the input-matrix in new order or a list with $mat (the input matrix in new order), $grep (matched matrix) and $col indicating the colum of \code{mat} finally used 
#' @seealso for basic ordering see \code{\link[base]{match}}; \code{\link{checkGrpOrder}} for testing each line for expected order, \code{\link{checkStrictOrder}} to check for strict (ascending or descending) order
#' @examples
#' mat1 <- matrix(paste0("__",letters[rep(c(1,1,2,2,3),3) +rep(0:2,each=5)], rep(1:5)), ncol=3)
#' orderMatrToRef(mat1, paste0(letters[c(3,4,5,3,4)],c(1,3,5,2,4)))
#' 
#' mat2 <- matrix(paste0("__",letters[rep(c(1,1,2,2,3),3) +rep(0:2,each=5)], 
#'   c(rep(1:5,2),1,1,3:5 )), ncol=3)
#' orderMatrToRef(mat2, paste0(letters[c(3,4,5,3,4)],c(1,3,5,1,4)))
#' 
#' mat3 <- matrix(paste0(letters[rep(c(1,1,2,2,3),3) +rep(0:2,each=5)], 
#'   c(rep(1:5,2),1,1,3,3,5 )), ncol=3)
#' orderMatrToRef(mat3, paste0("__",letters[c(3,4,5,3,4)],c(1,3,5,1,3)))
#'
#' @export
orderMatrToRef <- function(mat, ref, addRef=TRUE, listReturn=TRUE, silent=FALSE, debug=FALSE, callFrom=NULL) {
  ## order lines of matrix \code{mat} according to (character) vector \code{ref}
  ## in case of no perfect matches, grep of \code{ref} on all columns of \code{mat} and/or grep of all columns of \code{mat} on \code{ref} (ie 'reverse grep') will be applied
  ## find best column for matching mat (matrix) to ref (char vector) via two way grep
  ## return list $grep (matched matrix), $col best colum
  ## idea : also use trimRedundText() ?
  fxNa <- .composeCallName(callFrom, newNa="orderMatrToRef")   # was '.compToRef'
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  out <- NULL              # initialize

  ## check for simple solution
  ch1 <- colSums(matrix(mat %in% ref, ncol=ncol(mat)))==nrow(mat)
  if(any(ch1)) {
    out <- list(by="match", colNo=which(ch1), le=rep(1,nrow(mat)), ord=match(ref, mat[,which(ch1)]))
    out$mat <- mat[out$ord,]
  } else {
    ## need to use grep ..
    leM <- apply(nchar(as.matrix(mat)), 2, stats::median, na.rm=TRUE)          # median number of characters per col (mat)
    leR <- stats::median(nchar(ref), na.rm=TRUE)                               # median number of characters (ref)
    ## grep ref in each col of mat
    gM <- apply(as.matrix(mat), 2, function(x) sapply(ref, grep, x))           # grep each col of mat in ref
    lM <- sapply(gM, sapply, length)                                           # matrix of counts
    ## 'reverse' grep in each col of mat in ref
    gR <- apply(as.matrix(mat), 2, sapply, grep, ref)
    lR <- if(is.list(gR)) { if(is.list(gR[[1]])) sapply(gR, sapply, length) else sapply(gR, length) } else sapply(gR, length)
    #if(is.list(lR)) lT <- sapply(gR, sapply, length) else if(length(dim(lR)) <2) lR <- matrix(lR, nrow=1,dimnames=list(NULL, names(lR)))
    if(is.list(lR)) warning(fxNa, " Trouble ahead, can't make matrix/vector of counts from grep in each col of mat in ref")
    if(debug) {message(fxNa,"..cTR1"); cTR1 <- list(mat=mat,ref=ref,leM=leM,leR=leR,gM=gM,lM=lM,gR=gR,lR=lR)}
    ##                         
    if(any(lM >0, lR >0)) {           # some (perfect or partial) solutions
      chM1 <- apply(lM, 2, function(x) all(x==1))
      chR1 <- apply(lR, 2, function(x) all(x==1))
      if(sum(chM1, chR1) >0) {         # ideal solution exists (direct matching)
         if(any(chM1)) { out <- list(by="mat", colNo=which(chM1), le=if(length(dim(lM)) >1) lM[,which(chM1)] else lM[which(chM1)], ord=gM[[which(chM1)]])} else {
           out <- list(by="ref", colNo=which(chR1), le=if(length(dim(lR)) >1) lR[,which(chR1)] else lR[which(chR1)], ord=gR[[which(chR1)]]) }
      } else {                        # less ideal (multiple hits and/or empty)
        chM1 <- colSums(lM >0)
        chR1 <- colSums(lR >0)
        if(any(chM1==nrow(mat), chR1==nrow(mat))) {   # multiple hits, no empty, need further refinement
           if(any(chM1==nrow(mat))) { out <- list(by="ref", colNo=which(chM1==nrow(mat))[1],        # solution in M
             le=if(length(dim(lM)) >1) lM[,which(chM1==nrow(mat))[1]] else lM[which(chM1==nrow(mat))[1]], ord=gM[[which(chM1==nrow(mat))[1]]])
           } else if(any(chR1==nrow(mat))) { out <- list(by="mat", colNo=which(chR1==nrow(mat))[1],
             le=if(length(dim(lR)) >1) lR[,which(chR1==nrow(mat))[1]] else lR[which(chR1==nrow(mat))[1]], ord=gR[[which(chR1==nrow(mat))[1]]])}
          ## resolve multiple hits in $ord by picking 1st not yet used
          newGr <- sapply(out$ord, function(x) x[1])                    # simply 1st of multiple
          ## try using other instances of repeated : other cols of mat may contain other information
          multX <- unique(names(newGr[which(sapply(out$ord, length) >1)]))   # names of all multi-hit
          for(i in 1:length(multX)) newGr[which(names(newGr)==multX[i])] <- out$ord[[multX[i]]]
          out$ord <- newGr
          if(debug) message(fxNa," newGr ",pasteC(newGr))
        } else {warning(fxNa,"Impossible to find complete solution, returning NULL")}
      }
      if(debug) { message(fxNa,"..cTR2"); cTR2 <- list(mat=mat,ref=ref,leM=leM,leR=leR,gM=gM,lM=lM,gR=gR,lR=lR,out=out)}
    } else warning(fxNa,"Impossible to find any matches, returning NULL")
    if(length(out) >0) {
      if(out$by=="mat")  out$ord <- order(out$ord)
      out$mat <- mat[out$ord,]
      if(!isFALSE(addRef)) out$mat <- if("ref" %in% colnames(out$mat)) cbind(out$mat, ref) else cbind(out$mat, ref=ref)
      if(isFALSE(listReturn) & length(out) >0) out <- out$mat }
  }
  out }
  

#' Replace NAs by low values
#'
#' With several screening techniques used in hight-throughput biology values at/below detection limit are returned as \code{NA}.
#' However, the resultant \code{NA}-values may be difficult to analyse properly, simply ignoring \code{NA}-values mat not be a good choice.
#' When (technical) replicate measurements are available, one can look for cases where one gave an \code{NA} while the other did not 
#' with the aim of investigating such 'NA-neighbours'.  
#' \code{replNAbyLow} locates and replaces \code{NA} values by (random) values from same line & same group 'grp'. 
#' The origin of NAs should be predominantly absence of measure (quantitation) due to signal below limit of detection
#' and not saturation at upper detection limit or other technical problems.  
#' Note, this approach may be not optimal if the number of NA-neighbours is very low.
#' Replacamet is done -depending on agrument 'unif'- by Gaussian random model based on neighbour values (within same group),
#' using their means and sd, or a uniform random model (min and max of neighbour values) .
#' Then numeric matrix (same dim as 'x') with \code{NA} replaced is returned.
#'
#' @param x (numeric matrix or data.frame) main input
#' @param grp (factor) to organize replicate columns of (x)
#' @param quant (numeric) quantile form 'neighbour' values to use as upper limit for random values
#' @param signific number of signif digits for random values
#' @param unif (logical) toggle between uniform and Gaussian random values
#' @param absOnly (logical) if TRUE, make negative NA-replacment values positive as absolute values
#' @param seed (integer) for use with set.seed for reproducible output
#' @param silent (logical) suppress messages
#' @param callFrom (character) allow easier tracking of message(s) produced
#' @return numeric matrix (same dim as 'x') with \code{NA} replaced
#' @seealso \code{\link{naOmit}}, \code{\link[stats]{na.fail}}
#' @examples
#' dat <- matrix(round(rnorm(30),2),ncol=6); grD <- gl(2,3)
#' dat[sort(sample(1:30,9,repl=FALSE))] <- NA
#' dat; replNAbyLow(dat,gr=grD)
#' @export
replNAbyLow <- function(x,grp,quant=0.8,signific=3,unif=TRUE,absOnly=FALSE,seed=NULL,silent=FALSE,callFrom=NULL){
  fxNa <- .composeCallName(callFrom,newNa="replNAbyLow")
  if(length(dim(x))<2) stop("expecting matrix or data.frameas 'x'")
  if(is.data.frame(x)) x <- as.matrix(x)
  if(length(grp) != ncol(x)) stop(" 'grp' does NOT match number of columns in 'x'")
  nNA <- sum(is.na(x))
  if(nNA >0){
    lowVa <- unlist(apply(x,1,.extrNAneighb,grp))           # all low values with NA in same grp
    if(length(lowVa) <1) {
      lowVa <- sort(as.numeric(x))[1:round(sqrt(length(x)))]
      if(!silent) message(fxNa,"no values within lines of NA groups, use all observations")}
    lowVa <- lowVa[which(lowVa < stats::quantile(lowVa,quant))]
    nEstim <- ceiling(nNA*1.2)+5
    if(!silent) message(fxNa,"random model as ",if(unif) c("uniform ",signif(min(lowVa),signific)," to ",
      signif(max(lowVa),signific)) else c("Gaussian mean=",signif(mean(lowVa),signific),sd=signif(stats::sd(lowVa),signific)))
    if(!is.null(seed)) {if(is.numeric(seed)) set.seed(round(seed[1])) else message(fxNa,"ignoring invalid 'seed'")}
    raVa <- if(unif) signif(stats::rnorm(nEstim,mean(lowVa),stats::sd(lowVa)),digits=signific) else {
      signif(stats::runif(nEstim,min(lowVa),max(lowVa)),digits=signific)}
    if(absOnly & any(raVa <0)) {message(fxNa," force ",sum(raVa <0)," to positive"); raVa <- abs(raVa)}
    tmp <- unique(raVa)
    raVa <- if(length(tmp) > nNA) unique(raVa)[1:nNA] else raVa[1:nNA]                              # avoid identical values
    x[which(is.na(x))] <- raVa }
  x }
    

#' Merge Named Vectors
#'
#' This function allows merging for multiple named vectors (each element needs to be named).
#' Basically, all elements carrying the same name across different input-vectors will be aligned in the same column of the output (input-vectors appear as lines).
#' If vectors are not given using a name (see first example below), they will be names 'x.1' etc (see argument \code{namePrefix}).
#' 
#' @details
#' Note : The arguments '\code{namePrefix}', '\code{NAto0}', '\code{callFrom}' and '\code{silent}' must be given with full name to be recognized as such (and not get considered as vector for merging).   
#' 
#' @param ... all vectors that need to be merged 
#' @param namePrefix (character) prefix to numers used when vectors are not given with explicit names (second exammple)
#' @param NAto0 (logical) optional replacemet of \code{NA}s by 0
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a matrix of merged values
#' @seealso \code{\link[base]{merge}} (for two data.frames)
#' @examples
#' x1 <- c(a=1, b=11, c=21)
#' x2 <- c(b=12, c=22, a=2)
#' x3 <- c(a=3, d=43)
#' mergeVectors(vect1=x1, vect2=x2, vect3=x3)
#' x4 <- 41:44     # no names - not conform for merging
#' mergeVectors(x1, x2, x3, x4)
#' @export
mergeVectors <- function(..., namePrefix="x.", NAto0=FALSE, silent=FALSE, debug=FALSE, callFrom=NULL) {
  ## merge for simple named vectors (each element needs to be named)
  fxNa <- .composeCallName(callFrom, newNa="mergeVectors")
  if(isTRUE(debug)) silent <- FALSE
  if(!isTRUE(silent)) silent <- FALSE
  inpL <- list(...)
  chNa <- sapply(inpL, function(x) length(unique(names(x)))==length(x) && length(x) >0)
  if(any(!chNa)) {if(!silent) message(fxNa," Vectors must be longer than 0 and must have names on each element for merging; omit ",sum(!chNa)," (out of ",length(inpL),") vector(s)")
    inpL <- inpL[which(chNa)] }
  if(length(inpL)==1 && is.list(inpL[[1]])) inpL <- inpL[[1]]
  chNa <- names(inpL)
  if(length(names(inpL)) <1) { names(inpL) <- paste0(namePrefix,1:length(inpL))}
  if(length(inpL) >0) {
    if(isTRUE(NAto0)) inpL <- lapply(inpL, function(x) {chNa <- is.na(x); if(any(chNa)) x[which(chNa)] <- 0; x})
    spe <- sort(unique(unlist(lapply(inpL, names))))
    ta3 <- matrix(NA, nrow=length(inpL), ncol=length(spe), dimnames=list(names(inpL), spe))
    for(i in 1:length(inpL)) ta3[i, match(names(inpL[[i]]), spe)] <- inpL[[i]]
    ta3
  } else NULL }
      

#' Fit linear regression, return parameters and p-values
#'
#' This function fits a linear regression and returns the parameters, including p-values from Anova.
#' Here the vector 'y' (scalar response or dependent variable, ie the value that should get estimated) will be estimated according to 'dep' (explanatory or independent variable).
#' Alternatively, 'dep' may me a \code{matrix} where 1st column will be used as 'dep and the 2nd column as 'y'.
#' 
#' @param dep (numeric vector, matrix or data.frame) explanatory or dependent variable, if matrix or data.frame the 1st column will be used, if 'y'=\code{NULL} the 2nd column will be used as 'y'  
#' @param y (numeric vector) independent variable (the value that should get estimated based on 'dep')
#' @param asVect (logical) return numeric vector (Intercept, slope, p.intercept, p.slope) or matrix or results 
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return numeric vector (Intercept, slope, p.intercept, p.slope), or if \code{asVect}==\code{TRUE} as matrix (p.values in 2nd column)
#' @seealso \code{\link[stats]{lm}}
#' @examples
#' linRegrParamAndPVal(c(5,5.1,8,8.2),gl(2,2))
#' @export
linRegrParamAndPVal <- function(dep, y=NULL, asVect=TRUE, silent=FALSE, debug=FALSE, callFrom=NULL) {
  ## fit linear regression and return parameters, including p-values from Anova
  fxNa <- .composeCallName(callFrom, newNa="linRegrParamAndPVal")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  if(debug) message(fxNa,"lRP1")
  
  msg <- "'dep' and 'y' should be numeric vectors of equal length, or 'dep' may be matrix where the 1st column will be used as 'dep' and the secod as 'y'" 
  if(length(dim(dep)) ==2) {                         
    if(ncol(dep) >1 & length(y) <1) {dF <- as.data.frame(dep[,1:2]); colnames(dF) <- c("x","y")
    } else {
      if(length(y) == nrow(dep)) dF <- data.frame(x=dep[,1],y=y) else stop(msg)}
  } else {
    if(length(y) == length(dep)) dF <- data.frame(x=dep,y=y) else stop(msg)}
  ## main
  if(is.factor(dF[,2])) dF[,2] <- as.numeric(as.character(dF[,2])) 
  z <- stats::coef(summary(stats::lm(y~x,data=dF)))[,c("Estimate","Pr(>|t|)")]
  rownames(z)[2] <- "slope"
  if(asVect){ na <- dimnames(z)
    z <- as.numeric(z)
    names(z) <- c(na[[1]],paste("p",na[[1]],sep="."))}
  z }
    

#' Suggestions Of Collections Of Classical Separators For Pairwise Combinations
#'
#' @description
#' This function provides collecions of typical separators used for combined names in pair-wise testing for later choice of the most suited.
#' 
#' @details 
#' Depending if one rather looks for separators for cutting existing names of for suggesting separators for fresh concatenation, two differentsets of collections are available via the argument \code{type}.
#'
#' Using \code{type="split"} will default to \code{type="split1"}, similarly \code{type="combine"} will be used as \code{type="combine1"}.
#'   
#' The option \code{type="split1"} has a default collection of shorter/compact separators for splitting
#' The option \code{type="split2"} allows more extensive search for 38 separators (ie combinations with ' ','-' and '_'), but extensive checking may cost CPU time.
#' The option \code{type="split3"} has a shorter collection of 12 basic commonly used separators.
#' The option \code{type="split4"} to  \code{type="split9"} default to 'split1'
#' 
#' The option \code{type="combine1"} does not suggest '--' (neither '__') since if '-' may not be chosen  ins the first place, using '--' as separator next to '-' occuring inside gourp-names may not enhance visibility.
#' The option \code{type="combine2"} contains a collection of sperators using 'vs' or 'versus'
#' The option \code{type="combine3"} is designed for explicite terms (starts with ' - versus - ', for suggesting compact terms rather use \code{type="combine1"}
#' The option \code{type="combine4"} to  \code{type="combine9"} default to 'combine1'
#' 
#' 
#' @param type (character) allows selecting of different types of collections of separators
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a charcter vector with proposed separators for later choosing the most suited
#' @seealso (for running multiple pair-wise test) \code{\link{moderTestXgrp}}, \code{\link{convPairwiseSetup}}, \code{\link{presenceFilt}}, \code{\link{getPWseparator}}, \code{\link{indexGroupsFromPW}}, 
#'   (used underneith:) \code{\link[base]{grep}}, \code{\link[base]{strsplit}}
#' @examples
#' pwSeparatorList(type="split1")
#' @export
pwSeparatorList <- function(type="split1", silent=FALSE, debug=FALSE, callFrom=NULL) {
  fxNa <- .composeCallName(callFrom, newNa="pwSeparatorList")
  if(isTRUE(debug)) message(fxNa,"With type='",type,"'")
  if(length(type)==0 || any(is.na(type))) type <- "combine1" else type <- type[1]
  if("combine" %in% type) type <- "combine1"
  if("split" %in% type) type <- "split1"
  if(grepl("^split[5-9]$", type)) type <- "split1"
  if(grepl("^combine[5-9]$", type)) type <- "combine1"
  splS <- rep(c("-","_"," ",""), each=4)     # .. used in split2
  splX <- paste0(splS, c("compare","compared"), splS, c("to","to","To","To") )  # .. used in split2
  splY <- paste0(splS, c("vs","versus","to","VERSUS"), splS )                   # .. used in split2 
  switch(type,
    combine1 =c("-","---","_","___",".","-vs-","_vs_","---vs---"," - vs - ","___vs___","  ","--","__"),
    combine2 =c("-vs-","_vs_"," -vs- ","---vs---","___vs___"," - versus - "),
    combine3 =c("  /  "," vs ","-vs-"," - vs - "," - versus - ","___versus__","---vs---","___vs___"),  # for explicit
    combine4 =c("-","---","_","___",".","-vs-","_vs_","---vs-_-","___vs___"," ","  ","--","__"),       # backup 
    split1 =c("-","--","---","_","__","___","-vs-","_vs_"," - vs - ","--vs--","__vs__","---vs---","___vs___","-versus-","_versus_"," versus "," - versus - ","___versus__","-compareTo-","_compareTo_"),
    split2 =unique(c("-","--","---","_","__","___",splY, splX, "__vs__","--vs--"," - vs - ","___vs___","---vs---"," - versus - ","___versus__","   ","  ")),
    split3 =c("-","--","_","__"," - "," -- ","-vs-","_vs_","--vs--","__vs__"," ","  "),
    split4 =c("-","---","_","___",".","-vs-","_vs_","---vs-_-","___vs___"," ","  ","--","__")    # backup 
  )
}

  

#' Verify File-name If Existing (in specified path), If Has Proper Extension Or Select Files With Proper Extension From Given Path
#'
#' This function allows tesing if a given file-name corresponds to an existing file (eg for reading lateron).
#' Indications to the path and file-extensions may be given separately. If no files do match .gz compressed versions may be searched, too.
#' 
#' @details
#' 
#' When the filename given by the user exists but it's file-extension is not matching \code{expectExt} 
#' the argument \code{strictExtension} allows to decide if the filename will still be returned or not.
#' 
#' When \code{expectExt} is given, initial search will look for perfect matches. 
#' However, if nothing is found and \code{strictExtension=FALSE}, a more relaxed and non-case-sensitive search will be performed.
#' 
#' @param fileName (character) name of file to be tested; may also include an absolute or relative path; 
#'   if \code{NULL} and \code{path} as well as \code{expectExt} will take 1st file in given path and proper extension
#' @param path (character, length=1) optional separate entry for path of \code{fileName}
#' @param expectExt (character) file extension (will not be considered if \code{""})
#' @param mode (character) further details if function should give error or warning if no files found
#'   integrates previous argument \code{compressedOption} to also look for look for .gz compressed files; 
#'   \code{strictExtension} to decide if extension (\code{expectExt}) - if given - should be considered obligatory;
#'   \code{stopIfNothing} to stop with error if no files found
#' @param compressedOption deprected  (logical) also look for .gz compressed files
#' @param strictExtension deprected  (logical) decide if extesion (\code{expectExt}) - if given - should be considered obligatory
#' @param stopIfNothing deprected, please use argument \code{mode} instead !
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a character vector with verified file-name(s) (and path), returns \code{NULL} if nothing found - unless \code{mode="stopIfNothing"} 
#' @seealso \code{\link[base]{file.exists}} 
#' @examples
#' (RhomeFi <- list.files(R.home()))
#' file.exists(file.path(R.home(), "bin"))
#' checkFilePath(c("xxx","unins000"), R.home(), expectExt="dat")
#' @export
checkFilePath <- function(fileName=NULL, path="./", expectExt="", mode="compressedOption", compressedOption=NULL, strictExtension=NULL, stopIfNothing=NULL, silent=FALSE, debug=FALSE, callFrom=NULL) {
  ## verify file if existing (in specified path), if has proper extension or select files with proper extension from given path 
  fxNa <- .composeCallName(callFrom, newNa="checkFilePath")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  mode2 <- NULL
  
  if(debug) {message(fxNa,"cFP0"); cFP0 <- list(fileName=fileName,path=path,expectExt=expectExt,mode=mode,compressedOption=compressedOption)}
  if(length(mode) >0) {
    mode <- unique(as.character(mode))
    if(any(c("s","st") %in% mode)) warning(fxNa,"Content for argument 'mode' unclear, ignoring")
    if(any(c("strictExtension","strictExtens","strictExt","strictEx","strictE","strict","stri","str") %in% mode)) mode2 <- "strictExtension"
    if(any(c("compressedOption","compressedOpt","compressed","compr","comp","com","co","c") %in% mode)) mode2 <- c(mode2, "compressedOption")
    if(any(c("stopIfNothing","stopIfNo","stopIfN","stopIf","stopI","stop","sto") %in% mode)) mode2 <- c(mode2, "stopIfNothing")
  } 
  if(isTRUE(strictExtension)) mode2 <- c(mode2, "strictExtension")          # back compatobilty
  if(length(compressedOption) >0 && !isFALSE(compressedOption)) mode2 <- c(mode2, "compressedOption")       # back compatobilty
  mode2 <- unique(as.character(mode2))
  
  comprExt <- if(length(compressedOption) >0 && is.character(compressedOption)) compressedOption else {
    if("compressedOption" %in% mode2) c("gz", "zip", "7z") else NULL } # Supported compressed file extensions

  validExtension <- function(fiNa, extLst, comprExt=c("gz","zip")) {
    ## check if filenames do correspond to expected extensions
    ## if comprExt given, double-extensions due to compression will be recognized/validated, too
    ## return F & T for each file if extLst (& comprExt) correspond
    ##  add name of (compressed/initial) file as name of output
    #example# fiNa <- c("ab","ab.c","ab.c.gz","ab.d","ab.d.zip","ab.e"); extLst <- c("c","d"); validExtension(fiNa, extLst)  # ok
    fiNaExt1 <- tools::file_ext(fiNa)
    out <- fiNaExt1 %in% extLst
    if(length(comprExt) >0 && any(comprExt %in% fiNaExt1)) {
      fiNaExtD <- .doubleExt(fiNa, termExt=comprExt)
      out <- fiNaExtD %in% c(extLst, paste0(rep(extLst, each=length(comprExt)),".",rep(comprExt,length(extLst))))
    } 
    names(out)[which(out)] <- fiNa[which(out)]
  out }

  ## Ensure path exists and replace NA by default './'
  if(length(path) >0) {
    chNA <- is.na(path)
    if(any(chNA)) path[which(chNA)] <- "./"
    if(any(!dir.exists(path))) stop(sum(!dir.exists(path))," Provided 'path' do(es) not exist !")
  } else path <- "./" 
    
  ## Check 'expectExt'
  if(length(expectExt) >0) {
    expectExt <- unique(as.character(expectExt))
    chBad <- nchar(expectExt) <1 | is.na(expectExt)   # '' and NAs get ignored/removed
    if(any(chBad)) expectExt <- expectExt[which(!chBad)]
  } else chBad <- NULL
  if(debug) {message(fxNa,"cFP1"); cFP1 <- list(fileName=fileName,path=path,expectExt=expectExt,mode2=mode2,compressedOption=compressedOption, chBad=chBad)}
  
  ## Construct combined path & fileName
  if(length(fileName) <1) {
    if(length(path) ==1) {
      ## single path : pick all matching
      pat <-  if(length(expectExt) <1) NULL else {if(length(comprExt) <1) paste0("\\.",sub("\\$$","",expectExt),"$") else paste0("\\.",rep(expectExt,each=length(comprExt)), paste0(sub("\\$$","",comprExt),"$") ) }
      pat <- if(length(expectExt) <1) unique(c("", pat))
      files <- list.files(path[1], full.names=TRUE, pattern=pat) 
    } else {                  
      ## multiple paths : pick 1st matching of each path
      files <- sapply(path, function(pa) list.files( pa, 
        pattern= if(length(expectExt) <1) NULL else {if(length(comprExt) >0) paste0("\\.",rep(expectExt, each=length(comprExt)), 
          sub("\\$$","",comprExt),"$") else  paste0("\\.",sub("\\$$","",expectExt),"$")}, full.names=TRUE)[1] )
    }
    paFile <- files
    if(debug) {message(fxNa,"cFP2"); cFP2 <- list(fileName=fileName,path=path,expectExt=expectExt,mode2=mode2,compressedOption=compressedOption,comprExt=comprExt, chBad=chBad, files=files,paFile=paFile)}

  } else {
    ## fileName(s) given
    ## if path is given and meaningful -> add
    paFile <- fileName                                                                           
    if(length(path) >0) {  
      if(length(path)==1) path <- rep(path, length(fileName))
      chPa <- is.na(path) | nchar(path) <1 | path %in% c(".","./")         # identify unusable path: exclude non-given -> meaning current dir
      if(any(!chPa)) paFile[which(!chPa)] <- file.path(path[which(!chPa)], fileName[which(!chPa)])
    }  
  }   
  
  if(debug) {message(fxNa,"cFP3"); cFP3 <- list(fileName=fileName,path=path,expectExt=expectExt,mode2=mode2,compressedOption=compressedOption,comprExt=comprExt, chBad=chBad, paFile=paFile)}
  ## need to integrate "strictExtension"  & "stopIfNothing"

  ### MAIN CHECK
  ## if  length(expectExt) > 1 treat list  : attach ext
  ## if strict & extensions given: check extensions
  if(length(paFile) >0) {
    if(debug) {message(fxNa,"cFP4"); cFP4 <- list(fileName=fileName,path=path,expectExt=expectExt,mode2=mode2,compressedOption=compressedOption,comprExt=comprExt, paFile=paFile)}

    if("strictExtension" %in% mode2) {    # assume each extension should fit to each paFile
      #check length# length(expectExt) ==1 
      if(length(expectExt) >0) {    # standard case, expectExt given -> check & set not-conform to NA (which will be F after file.exists)
        chFi0 <- validExtension(paFile, expectExt, comprExt) 
        if(any(!chFi0)) paFile[which(!chFi0)] <- NA
      } 
      chEx <- file.exists(paFile)   
      out <- paFile
      if(any(!chEx)) out[!chEx] <- NA
    } else {           #  if not strict, pick first of possibly multiple hits
      out <- rep(NA, length(paFile))
      names(out) <- paFile
      fxCh <- function(x) {z <- file.exists(x); if(any(z)) x[which(z)[1]] else NA}
      chFi1 <- file.exists(paFile)     # check which fit right away
      if(any(chFi1)) out[which(chFi1)] <- paFile[which(chFi1)]
      if(any(!chFi1)) {      # cases of no direct hit 
        ext <- NULL
        ## check & remove extension if already in paFi
        fiExt <- tools::file_ext(paFile) %in% expectExt
        if(any(fiExt)) expectExt <- c("",expectExt)    # add 'no extension' in case extension seen somewhere in paFile        
        if(length(expectExt) >0 && length(comprExt) >0) {
          ext <- unique(sub("\\.$","", c(expectExt,paste0( rep(expectExt, each=length(comprExt)),".",rep(comprExt, length(expectExt))) ) ))
        } else if(length(expectExt) >0) ext <- unique(sub("\\.$","", expectExt)) else {
          if(length(comprExt) >0) ext <- unique(sub("\\.$","", comprExt))
        } 
        if(length(ext) >0) { ch2 <- sub("\\.\\.",".", sapply(paFile[which(!chFi1)], paste0, ".", ext))  # append extensions to fileNames
          out[which(!chFi1)] <- if(is.matrix(ch2)) apply(ch2, 2, fxCh) else unlist(sapply(ch2, fxCh))[1:sum(!chFi1)]     # check for presence
        }
      }

      if(any(!is.na(out))) out <- sub("\\.$","", out)      # remove terminal '.' (may be artifact from paste) 
    }
    if(debug) {message(fxNa,"cFP5"); cFP5 <- list(out=out,fileName=fileName,path=path,expectExt=expectExt,mode2=mode2,compressedOption=compressedOption,comprExt=comprExt, paFile=paFile)}
    if("stopIfNothing" %in% mode2 && any(is.na(out))) { stop(sum(is.na(out))," File(s) not found !")}
 
  } else out <- NULL  
  out }



#' Return File-name Extensions Including Double Extensions (eg txt.gz)
#'
#' This function allows retreiving extensions to filenames similar to \code{\link[tools]{file_ext}}, but also alllows to retreive selected double-extensions (txt.gz).
#' The leading dor will ne excluded similar to \code{\link[tools]{file_ext}}
#' 
#' @details
#' 
#' The argument \code{termExt} allows specifying additional (terminal) extensions to be recognized.
#' 
#' @param fiNa (character) name of file to be tested (may include path)
#' @param termExt (character) additional terminal extensions
#' @return This function returns a character vector with file-extensions (excluding the leading dot). (Only purely alphanumeric extensions are recognized.) 
#' @seealso \code{\link[tools]{file_ext}},  \code{\link{checkFilePath}} 
#' @examples
#' fi <- c("ab","ab.c","ab.c.gz","ab.d","ab.d.zip","ab.e")
#' .doubleExt(fi)
#' @export
.doubleExt <- function(fiNa, termExt=c("gz","zip")) {
  ## get file-extension similar to tools::file_ext() BUT allows retreiving double-extensions (like txt.gz)
  lastExt <- tools::file_ext(fiNa)
  if(length(termExt) >0 && all(nchar(termExt) >0)) {
    chDou <- lastExt %in% termExt
    if(any(chDou)) {      #tools::file_ext(tools::file_ext(fiNa)) 
      lastEx2 <- tools::file_ext(mapply(sub, paste0("\\.",lastExt[which(chDou)],"$"), rep("",sum(chDou)), fiNa[which(chDou)])) 
      lastExt[which(chDou)] <- paste0( lastEx2, ".", lastExt[which(chDou)])
    } }            
  lastExt
}
    

#' Rescaling Of Multiple Data-Sets According To Reference Data Using Regression
#'
#' This function regresses each col of matrix according to 'refLst'(describing rownames of inDat). 
#' If 'refLst' conatins 2 groups, the 1st group will be set to the 1st value of 'regrTo' (and the 2nd group of 'refLst' to the 2nd 'regtTo')
#'
#' @param inDat (matrix or data.frame) data to be rescaled
#' @param refLst list of names existing in inDat (one group of names for each value in 'regrTo'), to be transformed in values precised in 'regTo'; if no matches to names of 'inDat' found, the 2 lowest and/or highest highest values will be chosen
#' @param regrTo (numeric,length=2) range (at scale 0-1) of target-values for mean of elements cited in 'refLst'
#' @param silent (logical) suppress messages if \code{TRUE}
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allows easier tracking of messages produced
#' @return This function returns a normalized matrix
#' @seealso \code{\link{adjBy2ptReg}}, \code{\link{regrBy1or2point}}
#' @examples
#' set.seed(2016); dat2 <- round(cbind(1:50 +(1:50)*runif(50),2.2*(1:50) +rnorm(50,0,3)),1)
#' rownames(dat2) <- 1:nrow(dat2)
#' reg1 <- regrBy1or2point(dat2[,1],refLst=list(as.character(5:7),as.character(44:45)))
#' reg2 <- regrMultBy1or2point(dat2,refLst=list(as.character(5:7),as.character(44:45)))
#' plot(dat2[,1],reg2[,1])
#' identical(reg1,reg2[,1])
#' identical(dat2[,1],reg2[,1])
#' @export
regrMultBy1or2point <- function(inDat, refLst, regrTo=c(1,0.5), silent=FALSE, debug=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom,newNa="regrMultBy1or2point")
  checkEntr <- .checkRegrArguments(inDat,refLst,regrTo)
  refLst <- checkEntr$refLst; regrTo <- checkEntr$regrTo
  contrAve <- sapply(lapply(refLst, function(x) as.matrix(inDat[x,])), colMeans, na.rm=TRUE)
  if(is.null(dim(contrAve))) contrAve <- matrix(contrAve, ncol=length(contrAve))                   
  normFact <- if(length(refLst) ==1) (regrTo[1]/contrAve[,1]) else (regrTo[2]-regrTo[1])/(contrAve[,2] -contrAve[,1])   
  normFact <- cbind(k=normFact, d= if(length(refLst) ==1) rep(0, ncol(inDat)) else regrTo[1] -normFact*contrAve[,1])
  matrix(rep(normFact[,1], each=nrow(inDat)), nrow=nrow(inDat))*inDat + matrix(rep(normFact[,2], each=nrow(inDat)), nrow=nrow(inDat)) }
    

#' Connect edges to from tree and extract all possible branches 
#'
#' It is assumed that multiple fragments from a common ancestor bay be charcterized by the their start- and end-sites by integer values.  
#' For example, If 'abcdefg' is the ancestor, the fragments 'bcd' (from position 2 to 4) to  and 'efg' may then be assembled. 
#' To do so, all fragments must be presented as matix specifying all start- and end-sites (and fragment-names).  
#' \code{buildTree} searchs contiguous fragments from columns 'posCo' (start/end) from 'disDat' to build tree & extract path information starting with line 'startFr'.
#' Made for telling if dissociated fragments contribute to long assemblies.
#' This function uses various functions of package \href{https://CRAN.R-project.org/package=data.tree}{data.tree} which must be installed, too.
#'
#' @param disDat (matrix or data.frame) integer values with 1st column, ie start site of fragment, 2nd column as end of fragments, rownames as unique IDs (node-names)
#' @param startFr (integer) index for 1st node (typically =1 if 'disDat' sorted by "beg"), should point to a terminal node for consective growing of branches
#' @param posCo (character) colnames specifying the begin & start sites in 'disDat', if NULL 1st & 2nd col will be used
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a list with $paths (branches as matrix with columns 'sumLen' & 'n'), $usedNodes (character vector of all names used to build tree) and $tree (object from data.tree)
#' @seealso package \href{https://CRAN.R-project.org/package=data.tree}{data.tree} original function used \code{\link[data.tree]{Node}}; in this package : for exploiting edge/tree related issues \code{\link{simpleFragFig}}, \code{\link{countSameStartEnd}} and \code{\link{contribToContigPerFrag}}, 
#' @examples
#' frag2 <- cbind(beg=c(2,3,7,13,13,15,7,9,7,3,7,5,7,3),end=c(6,12,8,18,20,20,19,12,12,4,12,7,12,4)) 
#' rownames(frag2) <- c("A","E","B","C","D","F","H","G","I", "J","K","L","M","N")
#' buildTree(frag2)
#' countSameStartEnd(frag2)
#' @export
buildTree <- function(disDat, startFr=NULL, posCo=c("beg","end"), silent=FALSE, debug=FALSE, callFrom=NULL) {
  fxNa <- .composeCallName(callFrom, newNa="buildTree")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE

  datOK <- length(disDat) >0
  if(length(dim(disDat)) <2) disDat <- matrix(disDat, nrow=1, dimnames=list("1",names(disDat))) # assume as single entry
  rowNa <- rownames(disDat)
  if(is.null(rowNa)) rowNa <- 1:nrow(disDat)
  chCol <- match(colnames(disDat), posCo)
  disDat <- if(sum(is.na(chCol[1:2])) <1) {
    cbind(beg=as.integer(disDat[,chCol[1]]), end=as.numeric(disDat[,chCol[2]]))
  } else disDat <- as.matrix(disDat[,1:2])
  if(debug) {message(fxNa,"bTr1"); bTr1 <- list(disDat=disDat,startFr=startFr,posCo=posCo,datOK=datOK,chCol=chCol,disDat=disDat)}
  rownames(disDat) <- rowNa
  chSlash <- grep("/", rownames(disDat))
  if(length(chSlash) >0) message(fxNa,"TROUBLE ahead, names of nodes should NOT contain '/' !!")
  disDat <- cbind(disDat[,1:2], le=disDat[,2] -disDat[,1] +1)            # add col for length # if(ncol(disDat)==2)   

  if(datOK && requireNamespace("data.tree", quietly=TRUE)) {
  	## check if package is functioning  
    setX <- try(data.tree::Node$new("_Root_"))        # virtual node as generic root,  need to avoid reserved names (see NODE_RESERVED_NAMES_CONST)
    if(inherits(setX, "try-error")) { datOK <- FALSE
      warning(fxNa,"Problem running package data-tree : Can't even create a new generic node")}
  } else { datOK <- FALSE
    if(!silent) message(fxNa,"NOTE: Package 'data.tree' missing ! Please install from CRAN first .. returning NULL)")   
  }   
  if(debug) {message(fxNa,"bTr2"); bTr2 <- list(disDat=disDat,startFr=startFr,posCo=posCo,datOK=datOK,chCol=chCol,disDat=disDat)}
  
  if(datOK && requireNamespace("data.tree", quietly=TRUE)) {  
  	## main  
    ## check for dupl
    chDup <- duplicated(paste(disDat[,1],disDat[,2],sep="_"), fromLast=FALSE)
    names(chDup) <- rownames(disDat)
    if(any(chDup)) {
      chDu2 <- duplicated(paste(disDat[,1],disDat[,2],sep="_"), fromLast=TRUE)
      hasDu <- chDu2 & !chDup                    # the originals (of dupl) to keep
      names(hasDu) <- rownames(disDat)
      ## remove duplicated/redundant
      dupDat <- lapply(which(hasDu), function(x) {z <- paste(disDat[,2],disDat[,1],sep="_"); which(z %in% z[x])})  # index of replicated elements (line-no)
      if(!silent) message(fxNa,": ", sapply(dupDat, function(x) {y <- names(chDup)[x]; paste("  ",y[1]," duplicated by ",paste(y[-1],collapse=" "),"\n  ")}))
      disDat <- disDat[which(!chDup),]           # cleaned main data
    } else dupDat <- NULL
    ## check possible start sites
    nodeWPrev <- sort(unique(names(which(rep(disDat[,1], nrow(disDat)) == rep(disDat[,2] +1, each=nrow(disDat))))))
    rootBaseNa <- if(length(nodeWPrev) >0) rownames(disDat)[which(!rownames(disDat) %in% nodeWPrev)] else rownames(disDat)
    rootBase <- which(rownames(disDat) %in% rootBaseNa)
    names(rootBase) <- rootBaseNa
    ## check startFr
    startFr <- if(is.null(startFr)) rootBase[1] else try(as.integer(startFr))
    if(inherits(startFr, "try-error")) stop(fxNa,": 'startFr' should be NULL or integer (of length 1) !")
    if(!startFr %in% rootBase) { if(!silent) message(fxNa,"Choice of 'startFr' is not close to root, resetting to ",rootBase[1]," ('",names(rootBase)[1],"')")
      startFr <- rootBase[1] } 
    names(startFr) <- rownames(disDat)[startFr]
    tm1 <- disDat[,1] == disDat[startFr,2] +1              # startFr has following
    tm1 <- list(lo=tm1, it=0 +tm1, preN=rep(NA,nrow(disDat)), disDat=disDat, iter=1, start=startFr)     
    ## grow 1st branch
    if(any(tm1$lo)) tm1 <- .growTree(tm1,setX) else {
      x1 <- setX$AddChild(names(startFr), len=disDat[startFr,3])
      tm1$it[startFr] <- 1
      }
    ## look for other starting points (ie nodes not yet used)
    chSup <- rownames(disDat) %in% rootBaseNa[-which(rootBaseNa == names(startFr))]   
    names(chSup) <- rownames(disDat)
    if(any(chSup)) { 
      while(any(chSup)) {
        j <- which(chSup)[1]
        tm3 <- disDat[,1]== disDat[j,2] +1
        tm3 <- list(lo=tm3, it=0 +tm3, preN=rep(NA,nrow(disDat)), disDat=disDat, iter=1, start=j)
        tm3 <- .growTree(tm3,setX)
        chSup[j] <- FALSE }
    } else tm3 <- NULL     
    ## now extract n and cumulated length fo fragments
    setX$sumLen <- setX$len                              # initialize variable for summed length
    traversal <- data.tree::Traverse(setX, filterFun=data.tree::isNotRoot)
    data.tree::Do(traversal, function(node) node$sumLen <- node$parent$sumLen + node$len)
    setX$n <- 1                                          # initialize variable for path length
    data.tree::Do(traversal, function(node) node$n <- node$parent$n + 1)
    out <- data.tree::ToDataFrameTable(setX, "pathString","sumLen","n")
    rownames(out) <- sub("^_Root_\\/","",out[,1])
    out$n <- out$n-1  
    out <- as.matrix(out[,-1])
    if(length(dupDat) >0) {
      ## need list indicating which node(s) is/are duplicate of which node : dupDat
      replOut <- function(x,chDup,out) {   # fetch original of eliminated duplicated elements, reconstruct output with adjusted rownames
        se <- function(v) paste0("/",v,"/")
        y <- grep(se(names(chDup)[x[1]]), se(rownames(out))) 
        z <- se(rownames(out)[y])
        w <- sub("^/","",sub("/$","",sapply(names(chDup)[x[-1]], function(w) sub(se(names(chDup)[x[1]]), se(w),z))))
        matrix(rep(t(out[y,]), length(x)-1), ncol=ncol(out), dimnames=list(w,colnames(out)), byrow=TRUE) }
       replOut(dupDat[[1]], chDup, out)
      supMat <- lapply(dupDat, replOut, chDup, out)
      out2 <- matrix(unlist(sapply(supMat, as.integer)), ncol=ncol(out), byrow=FALSE)
      dimnames(out2) <- list(unlist(sapply(supMat, rownames)), colnames(out))
      out <- rbind(out, out2)  
    }
    list(paths=out, usedNodes=sort(names(chSup)[which(!chSup)]), tree=setX ) 
  } }

#' Grow Tree
#'
#' This function allows growing tree-like structures (data.tree objects)
#' 
#' @param tm (list) main input, $disDat .. matrix with integer start & end sites for fragments; $lo (logical) which fragments may be grown; $start (integer) index for which line of $disDat to start; $it numeric version of $lo; $preN for previous tree objects towards root; $iter for iterator (starting at 1))
#' @param setX .. data.tree object (main obj from root)
#' @param addToObj .. data.tree object (branch on which to add new branches/nodes) 
#' @return This function returns a list
#' @seealso  \code{\link{buildTree}}
#' @examples
#' .datSlope(c(3:6))
#' @export
  .growTree <- function(tm, setX, addToObj=NULL) {
    ## grow tree 'setX' based on 'tm' 
    ## 'tm' .. list ($disDat .. matrix with integer start & end sites for fragments; $lo (logical) which fragments may be grown; $start (integer) index for which line of $disDat to start; $it numeric version of $lo; $preN for previous tree objects towards root; $iter for iterator (starting at 1))
    ## 'setX' .. data.tree object (main obj from root)
    ## 'addToObj' .. data.tree object (branch on which to add new branches/nodes)
    newNodeNa <- paste0("b",0,"_",tm$disDat[tm$start,1])
    namesX <- deparse(substitute(setX))                   # name of tree-object (typically 'setX'  )
    assign(newNodeNa, get(namesX)$AddChild(rownames(tm$disDat)[tm$start], len=tm$disDat[tm$start,3]))      # add new 1st level branch to '_Root_'
    if(any(tm$lo)) tm$preN[which(tm$lo)] <- newNodeNa
    while(any(tm$lo)) {        # need to grow further ..
      tm$iter <- tm$iter +1
      j <- which(tm$lo)[1]
      addToObj <- if(tm$iter==2) newNodeNa else tm$preN[j]    
      assign(paste0("b",tm$it[j],"_",j), get(addToObj)$AddChild(rownames(tm$disDat)[j], len=tm$disDat[j,3])) 
      tm$lo[j] <- FALSE                                          # this one is done ...
      tm0 <- tm$disDat[,1]== tm$disDat[j,2] +1                    # test for potential children
      if(any(tm0)) {z <- which(tm0);                        
        tm$lo[z] <- TRUE                                        # set to-do status for children
        tm$it[z] <- tm$it[j]+1                                  # tree-level 
        tm$preN[z] <- paste0("b",tm$it[j],"_",j)                # report (prev)name of node
        reOrd <- c(z, which(!tm0))                               # need to change order to treat children next (for treatig correctly branched trees)
        tm$lo <- tm$lo[reOrd]
        tm$it <- tm$it[reOrd]
        tm$preN <- tm$preN[reOrd]
        tm$disDat <- tm$disDat[reOrd,] }    
    }
    tm }
     

#' Locate duplicates in text and make non-redundant
#'
#' \code{treatTxtDuplicates} locates duplictes in character-vector 'x' and return list (length=3) : with $init (initial),
#'   $nRed .. non-redundant text by adding number at end or beginning, and $nrLst .. list-version with indexes per unique entry. 
#'  Note : NAs (if multiple) will be renamed to NA_1, NA_2
#' @param x (character) vector with character-entries to identify (and remove) duplicates
#' @param atEnd (logical) decide location of placing the counter (at end or at beginning of ID) (see \code{\link{correctToUnique}})
#' @param sep (character) separator to add before counter when making non-redundant version
#' @param onlyCorrectToUnique (logical) if TRUE, return only vector of non-redundant 
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return list with $init, $nRed, $nrLst
#' @seealso For simple correction use \code{\link{correctToUnique}}
#' @examples
#' treatTxtDuplicates(c("li0",NA,rep(c("li2","li3"),2)))
#' correctToUnique(c("li0",NA,rep(c("li2","li3"),2)))
#' @export
treatTxtDuplicates <- function(x, atEnd=TRUE, sep="_", onlyCorrectToUnique=FALSE, silent=FALSE, debug=FALSE, callFrom=NULL) {
  fxNa <- .composeCallName(callFrom,newNa="treatTxtDuplicates")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  if(debug) message(fxNa, "tTDA1")

  if(length(dim(x)) >1) { if(!silent) message(fxNa," expecting simple (text) vector as 'x' but got class ",class(x))
    x <- if(is.list(x)) unlist(x) else as.character(x) }
  xIni <- x
  if(any(is.na(x))) x[which(is.na(x))] <- "NA"
  out <- correctToUnique(x, sep=sep, atEnd=atEnd)
  if(!onlyCorrectToUnique) {
    anyDu <- duplicated(x, fromL=FALSE) | duplicated(x, fromL=TRUE)
    if(any(anyDu)) { nrLst <- sapply(unique(x),function(z) which(x %in% z))
      if(!is.list(nrLst)) nrLst <- as.list(as.data.frame(nrLst))
      } else {nrLst <- as.list(1:length(x)); names(nrLst) <- x}
  list(init=xIni, nRed=out, nrLst=nrLst)} else out }
       

#' Trim redundant text
#'
#' This function allows trimming/removing redundant text-fragments (redundant from head or tail) out of character vector 'txt'.
#' 
#' 
#' 
#' @param txt character vector to be treated
#' @param minNchar (integer) minumin number of characters that must remain
#' @param side (character) may be be either 'both', 'left' or 'right'
#' @param spaceElim (logical) optional removal of any heading or tailing white space 
#' @param silent (logical) suppress messages
#' @param callFrom (character) allows easier tracking of messages produced
#' @param debug (logical) display additional messages for debugging
#' @return This function returns a modified character vector
#' @seealso  \code{\link{rmSharedWords}}; Inverse search : Find/keep common text \code{\link{keepCommonText}}; \code{\link{checkUnitPrefix}}; 
#'    you may also look for related functions in package \href{https://CRAN.R-project.org/package=stringr}{stringr}
#' @examples
#' txt1 <- c("abcd_ccc","bcd_ccc","cde_ccc")
#' trimRedundText(txt1, side="right")       # trim from right
#' 
#' txt2 <- c("ddd_ab","ddd_bcd","ddd_cde")
#' trimRedundText(txt2, side="left")        # trim from left 
#' @export
trimRedundText <- function(txt, minNchar=1, side="both", spaceElim=FALSE, silent=TRUE, callFrom=NULL, debug=FALSE) {
  ##
  fxNa <- .composeCallName(callFrom, newNa="trimRedundText")
  if(isTRUE(debug)) silent <- FALSE else { debug <- FALSE
    if(!isTRUE(silent)) silent <- FALSE }
  doTrim <- TRUE
  if(length(txt) <1) message(fxNa,"Problem : 'txt' appears empty, nothing to do") else {
    if(!is.character(txt)) { txt <- try(as.character(txt))
      if(inherits(txt, "try-error")) {txt <- NULL; doTrim <- FALSE
        warning(fxNa,"Unable to convert 'txt' into text; nothing to do")}}
  } 
  ## trimming of heading and tailing white space
  if(length(txt) >0 && spaceElim) { 
    txt <- gsub(" $","", gsub("^ ","",txt)) 
  }
  if(doTrim) { chLe <- nchar(txt)
    if(any(naOmit(chLe) ==0)) { doTrim <- FALSE
      if(sum(chLe ==0, na.rm=TRUE) + sum(is.na(chLe)) ==length(txt)) { message(fxNa,"NOTE : all elements appear empty ! Nothing to do ..")
  	  } else if(!silent) message(fxNa,"NOTE : ",sum(chLe==0)," elements appear empty ! Nothing to do ..") }
  	}    
  ## finish checking arguments    
  if(doTrim) {
    msg2 <- "Argument 'minNchar' should be positive integer; setting to default=1"
    if(length(minNchar) !=1 | !is.numeric(minNchar)) { minNchar <- 1
      message(fxNa,msg2)} else minNchar <- abs(as.integer(minNchar)) 
    msg1 <- "Argument 'side' should be either 'both', 'left' or 'right'; setting to default='both'" 
    if(length(side) <1) {side <- "both"; message(fxNa, msg1)}
    if(!is.character(side)) {side <- "both"; message(fxNa, msg1)} 
    ## main
    nChar <- nchar(txt)
    if(debug) {message(fxNa,"Ready tm trim ",pasteC(nChar)," characters"); trL <- list(txt=txt,minNchar=minNchar,nChar=nChar)}
    if(any(c("any","both","left") %in% side)) {      
      txt <- .trimLeft(txt, minNchar=minNchar, silent=TRUE, callFrom=fxNa)
      if(debug) {message(fxNa," .trimLeft reduced to ",pasteC(nchar(txt))," characters")}
    }
    if(any(c("any","both","right") %in% side)) {      
      txt <- .trimRight(txt, minNchar=minNchar, silent=TRUE, callFrom=fxNa)
      if(debug){ message(fxNa," .trimRight reduced to ",pasteC(nchar(txt))," characters")}
    }
    txt } }


#' Trim From Left Side
#'
#' This function allows trimming/removing redundant text-fragments from left side.
#' 
#' @param x character vector to be treated
#' @param minNchar (integer) minumin number of characters that must remain
#' @param silent (logical) suppress messages
#' @param debug (logical) display additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a modified character vector
#' @seealso \code{\link{trimRedundText}}; Inverse : Find/keep common text \code{\link{keepCommonText}};  you may also look for related functions in package \href{https://CRAN.R-project.org/package=stringr}{stringr}
#' @examples
#' txt1 <- c("abcd_ccc","bcd_ccc","cde_ccc")
#' .trimLeft(txt1)
#' @export
.trimLeft <- function(x, minNchar=1, silent=TRUE, debug=FALSE, callFrom=NULL) {
  ## trim redundant starting from left side
  fxNa <- .composeCallName(callFrom, newNa=".trimLeft")
  nChar <- nchar(x)
  msg <- c(fxNa,"Some entries are too short for trimming to min ",minNchar," characters, nothing to do")
  if(all(naOmit(nChar > minNchar))) {
    ch1 <- min(nChar, na.rm=TRUE)
    ch1 <- (ch1 -minNchar) :1
    ch1 <- paste0("^",substr(rep(x[which.min(nChar)], length(ch1)), 1, ch1))
    ch3 <- lapply(gsub("\\.","\\\\.",ch1), grep, x)
    ch3 <- sapply(ch3, length) + sum(is.na(x)) ==length(x)
    if(any(ch3)) x <- substring(x, nchar(ch1[min(which(ch3))]))
  } else if(!silent) message(msg)
  x
}

#' Trim From Right Side
#'
#' This function allows trimming/removing redundant text-fragments from right side.
#' 
#' @param x character vector to be treated
#' @param minNchar (integer) minumin number of characters that must remain
#' @param silent (logical) suppress messages
#' @param debug (logical) display additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a modified character vector
#' @seealso \code{\link{trimRedundText}}; Inverse : Find/keep common text \code{\link{keepCommonText}};  you may also look for related functions in package \href{https://CRAN.R-project.org/package=stringr}{stringr}
#' @examples
#' txt1 <- c("abcd_ccc","bcd_ccc","cde_ccc")
#' .trimRight(txt1)
#' @export
.trimRight <- function(x, minNchar=1, silent=TRUE, debug=FALSE, callFrom=NULL) {
  fxNa <- .composeCallName(callFrom, newNa=".trimRight")
  nChar <- nchar(x)
  msg <- c(fxNa,"Some entries are too short for trimming to min ",minNchar," characters, nothing to do")
  if(all(naOmit(nChar > minNchar))) {
    ch1 <- min(nChar, na.rm=TRUE)
    ch1 <- 1: (ch1 -minNchar) 
    ch2 <- x[which.min(nChar)]
    ch1 <- paste0(substr(paste0(rep(ch2, length(ch1))), nchar(ch2)- ch1 +1, nchar(ch2)),"$")
    ch3 <- lapply(gsub("\\.","\\\\.",ch1), grep, x)
    ch3 <- sapply(ch3, length) + sum(is.na(x)) ==length(x)
    if(any(ch3)) x <- substring(x, 1, nchar(x) - nchar(ch1[which.max(which(ch3))]) +1)
  } else if(!silent) message(msg) 
  x
}
        
#' Trim from start (Deprecated)
#'
#' Deprecated Version - This function allows trimming/removing redundant text-fragments from start
#' 
#' @param x character vector to be treated
#' @param ... more vectors to be treated
#' @param minNchar (integer) minumin number of characters that must remain
#' @param silent (logical) suppress messages
#' @param debug (logical) display additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a modified character vector
#' @seealso \code{\link{trimRedundText}}; Inverse : Find/keep common text \code{\link{keepCommonText}};  you may also look for related functions in package \href{https://CRAN.R-project.org/package=stringr}{stringr}
#' @examples
#' txt1 <- c("abcd_ccc","bcd_ccc","cde_ccc")
#' .trimLeft(txt1)  # replacement
#' @export
.trimFromStart <- function(x,..., minNchar=1, silent=TRUE, debug=FALSE, callFrom=NULL) {
  ## deprecated version
  ## trim, ie remove redundant characters from beginning
  ## 'minNchar' min number of characters that should remain
  y <- list(...)
  fxNa <- .composeCallName(callFrom, newNa=".trimFromStart")
  .Deprecated(new=".trimLeft", package="wrMisc", msg="The function .trimFromStart() has been deprecated and replaced by .trimLeft()") 
  if(length(x) < 1) {if(!silent) message(fxNa," Problem : 'x' appears empty")} else {
    exclLiNa <- c("minNchar","silent","callFrom")
    exclLiNa2 <- c(sapply(nchar(exclLiNa[1]):2, function(z) substr(exclLiNa[1],1,z)),
      sapply(nchar(exclLiNa[2]):2, function(z) substr(exclLiNa[2],1,z)))
    te <- which(names(y) %in% exclLiNa2)
    if(length(naOmit(te)) > 0) y <- y[-1*which(names(y) %in% exclLiNa2)]
    if(sum(sapply(y, length) >0)) {
      x <- c(x, unlist(y)) }
    if(!silent) message(fxNa," initial no of characters  ", paste(nchar(x), collapse=" "))
    while(length(unique(substr(x, 1, 1))) <2 & min(nchar(x),na.rm=TRUE) > minNchar) x <- substr(x, 2, nchar(x)) }
  x }

#' Trim from end (Deprecated)
#'
#' Deprecated Version - This function allows trimming/removing redundant text-fragments from end
#' 
#' @param x character vector to be treated
#' @param ... more vectors to be treated
#' @param silent (logical) suppress messages
#' @param debug (logical) display additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a modified character vector
#' @seealso \code{\link{trimRedundText}}; Inverse : Find/keep common text \code{\link{keepCommonText}};  you may also look for related functions in package \href{https://CRAN.R-project.org/package=stringr}{stringr}
#' @examples
#' txt1 <- c("abcd_ccc","bcd_ccc","cde_ccc")
#' .trimRight(txt1)
#' @export
.trimFromEnd <- function(x,..., callFrom=NULL, debug=FALSE, silent=TRUE) {
  ## trim, ie remove redundant characters from beginning
  ## note: since aruguments collected by
  ## less elaborated than .trimFromStart()
  fxNa <- .composeCallName(callFrom,newNa=".trimFromEnd")
  .Deprecated(new=".trimRight", package="wrMisc", msg="The function .trimFromEnd() has been deprecated and replaced by .trimRight()") 
  y <- list(...)
  if(length(y) >0) {if(any(c("callFrom","silent") %in% names(y))) {
    y <- y[-1*which(names(y) %in% c("callFrom","callFr","sil","silent"))]}}
  if(sum(sapply(y, length) >0)) {   # '...' argument will be added to x
    x <- c(x, unlist(y)) }
  if(!silent) message(fxNa,"Initial no of characters  ", paste(nchar(x), collapse=" "))
  while(length(unique(substr(x,nchar(x),nchar(x)))) <2) x <- substr(x, 1, nchar(x)-1)
  x }
  

#' Correct vector to unique
#'
#' \code{correctToUnique} checks 'x' for unique entries, while maintaining the original length. If necessary a counter will added to non-unique entries. 
#' @param x input character vector
#' @param sep (character) separator used when  counter
#' @param atEnd (logical) decide location of placing the counter (at end or at beginning of initial text)
#' @param maxIter (numeric) max number of iterations
#' @param NAenum (logical) if \code{TRUE} all \code{NA}s will be enumerated (NA_1,NA_2,...)
#' @param silent (logical) suppress messages
#' @param callFrom (character) for better tracking of use of functions
#' @return This function returns a character vector
#' @seealso \code{\link[base]{unique}} will simply remove repeated elements, ie length of 'x' won't remain constant, \code{\link{filtSizeUniq}} is more complex and slower, \code{\link{treatTxtDuplicates}}
#' @examples
#' correctToUnique(c("li0","n",NA,NA,rep(c("li2","li3"),2),rep("n",4))) 
#' @export
correctToUnique <- function(x, sep="_", atEnd=TRUE, maxIter=4, NAenum=TRUE, silent=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom, newNa="correctToUnique")
  chNA <- is.na(x)
  if(length(NAenum) >1) NAenum <- as.logical(NAenum[1])
  if(NAenum && any(chNA)) x[which(chNA)] <- "NA"
  dupR <- duplicated(x, fromLast=FALSE)
  dupL <- duplicated(x, fromLast=TRUE) 
  anyDu <- anyDx <- dupL | dupR
  if(any(anyDu)) { anyDu <- which(anyDu)
    xIni <- x
    x[which(!dupR & dupL)] <- if(atEnd) paste(x[which(!dupR & dupL)],"1",sep=sep) else paste("1",x[which(!dupR & dupL)],sep=sep)
    dupRx0 <- dupR[anyDu]
    iter <- 2
    finished <- FALSE
    while(iter <= maxIter & !finished){
      dupRX <- duplicated(x[anyDu], fromLast=FALSE)
      if(any(dupRX)) {sel <- which(!dupRX & dupRx0); x[anyDu[sel]] <- if(atEnd) paste(x[anyDu[sel]],iter,sep=sep) else paste(iter,x[anyDu[sel]],sep=sep)
        iter <- iter + 1
        dupRx0 <- dupRX
      } else {x[anyDu[which(dupRx0)]] <- if(atEnd) paste(x[anyDu[which(dupRx0)]],iter,sep=sep) else paste(iter,x[anyDu[which(dupRx0)]],sep=sep)
        finished <- TRUE} }
    if(!NAenum) if(any(chNA)) x[which(chNA)] <- NA
    if(!finished) {                                           
      xTab <- table(xIni[anyDu])[rank(unique(xIni[anyDu]))]    
      if(any(xTab >maxIter)) for(i in names(xTab)[which(xTab > maxIter)]) {
        z <- which(x==i); x[z] <- if(atEnd) paste(x[z],(maxIter +1):xTab[which(names(xTab)==i)],sep=sep) else paste((maxIter +1):xTab[i],x[z],sep=sep)}}}
  x }

#' Check regression arguments
#'
#' This function is an enhanced version of \code{unique}, names of elements are maintained
#' 
#' @param x (numeric or character vector) main input
#' @param splitSameName (logical)
#' @param silent (logical) suppress messages
#' @param callFrom (character) allow easier tracking of messages produced
#' @param debug (logical) additional messages for debugging
#' @return vector like input
#' @seealso  \code{\link[base]{unique}}
#' @examples
#' aa <- c(a=11, b=12,a=11,d=14, c=11)
#' .uniqueWName(aa)
#' .uniqueWName(aa[-1]) # value repeated but different name
#' @export
.uniqueWName <- function(x, splitSameName=TRUE, silent=TRUE, debug=FALSE, callFrom=NULL){
  ## enhanced version of unique(): return unique of vector 'x' with names (if multiple names fit to same value of 'x', use 1st of names)
  ## assumes that names of 'x' are redundant to value of 'x'
  ## 'splitSameName' .. allows keeping different names, even if with same value in 'x' (which would disappear with unique(x))
  fxNa <- .composeCallName(callFrom, newNa=".uniqueWName")
  argNa <- deparse(substitute(x))
  inv <- FALSE
  if(length(unique(x)) < length(unique(names(x))))  {
    if(splitSameName){
      if(!silent) message(fxNa,"'",argNa,"' has more names than different values, maintaining different names")
      tmp <- names(x)
      names(tmp) <- x
      x <- tmp
      inv <- TRUE
    } else if(!silent) message(fxNa,"Names of  '",argNa,"' don't fit to its values, ",
      "result not representative, rather use argument 'splitSameName'=TRUE")
  }
  out <- sapply(unique(x), function(z) x[which(x==z)[1]])
  if(is.character(x)) names(out) <- substr(names(out), nchar(out) +2, nchar(names(out)))
  if(inv){
    tmp <- names(out)
    names(tmp) <- out
    out <- tmp }
  out }
   

#' Standard Eror Of Median by Boot-Strap
#'
#' \code{stdErrMedBoot} estimate standard eror of median by bootstrap approach. 
#' Note: requires package \href{https://CRAN.R-project.org/package=boot}{boot}
#'
#' @param x (numeric) vector to estimate median and it's standard error
#' @param nBoot (integer) number for iterations
#' @param silent (logical) suppress messages
#' @param callFrom (character) allows easier tracking of messages produced
#' @param debug (logical) display additional messages for debugging
#' @return This function returns a (numeric) vector with estimated standard error
#' @seealso \code{\link{colMedSds}} and \code{\link{rowMedSds}}; based on \code{\link[boot]{boot}}
#' @examples
#' set.seed(2014); ra1 <- c(rnorm(9,2,1),runif(8,1,2))
#' rat1 <- ratioAllComb(ra1[1:9],ra1[10:17])
#' median(rat1); stdErrMedBoot(rat1)
#' @export
stdErrMedBoot <- function(x, nBoot=9, silent=FALSE, debug=FALSE, callFrom=NULL) { 
  fxNa <- .composeCallName(callFrom, newNa="stdErrMedBoot")
  if(isTRUE(debug)) silent <- FALSE else { debug <- FALSE
    if(!isTRUE(silent)) silent <- FALSE }
   
  ## uses package boot
  if(requireNamespace("boot", quietly=TRUE)) {
    median.fun <- function(dat, indices) stats::median(dat[indices], na.rm=TRUE)
    out <- try(stats::sd(boot::boot(data=x, statistic=median.fun, R=nBoot)$t))
    if(inherits(out, "try-error")) stop("Unable to run bootstrap, check format of data and argument 'nBoot'")
    out
  } else {
    if(!silent) message(fxNa,"NOTE: Package 'boot' not found ! Please install first from CRAN  (returning NULL)")
  }
}
 

#' Find repeated elements
#'
#' \code{findRepeated} gets index of repeated items/values in vector 'x' (will be treated as character). 
#' Return (named) list of indexes for each of the repeated values, or \code{NULL} if all values are unique.
#' This approach is similar but more basic compared to \code{\link{get1stOfRepeatedByCol}}.
#' @param x character vector
#' @param nonRepeated (logical) if \code{=TRUE}, return list with elements \code{$rep} and \code{$nonrep}
#' @param silent (logical) suppress messages
#' @param callFrom (character) allow easier tracking of message(s) produced
#' @return (named) list of indexes for each of the repeated values, or NULL if all values unique
#' @seealso similar approach but more basic than \code{\link{get1stOfRepeatedByCol}}
#' @examples
#' aa <- c(11:16,14:12,14); findRepeated(aa) 
#' @export
findRepeated <- function(x, nonRepeated=FALSE,silent=FALSE,callFrom=NULL) {
  y <- as.character(x)          
  tab <- table(y)
  tab2 <- which(tab >1)
  if(length(tab2) >0){
    out <- lapply(names(tab)[tab2],function(z) which(y %in% z))               # slightly faster than which(!is.na(match(y,z))))
    names(out) <- x[sapply(out,utils::head,1)]                                # use initial value of repeated items as name
  } else out <- NULL
  if(nonRepeated) out <- list(rep=out,nonrep=match(names(tab)[which(tab <2)],y))
  out }
   

#' get coordinates of values/points in matrix according to filtering condition
#'
#' Get coordinates of values/points in matrix according to filtering condition
#'
#' @param mat (matrix or data.frame) matrix or data.frame 
#' @param cond (logical or integer) condition/test to see which values of \code{mat} fulfull test, or integer of index passing  
#' @param sortByRows (logical) optional sorting of results by row-index
#' @param silent (logical) suppress messages
#' @param callFrom (character) allow easier tracking of message(s) produced 
#' @return matrix columns 'row' and 'col'
#' @seealso \code{\link[base]{which}}
#' @examples
#' set.seed(2021); ma1 <- matrix(sample.int(n=40,size=27,replace=TRUE), ncol=9)
#' ## let's test which values are >37
#' which(ma1 >37)      # doesn't tell which row & col
#' coordOfFilt(ma1, ma1 >37)
#' 
#' @export
coordOfFilt <- function(mat, cond, sortByRows=FALSE, silent=FALSE, callFrom=NULL) {
  ## get coordinates of values/points in matrix according to filtering condition
  fxNa <- .composeCallName(callFrom, newNa="coordOfFilt")
  if(any(length(dim(mat)) !=2, dim(mat) < 2:1)) stop("Invalid argument 'mat'; must be matrix or data.frame with min 2 lines and 1 col")
  cond <- if(is.logical(cond)) which(cond) else as.integer(cond) 
  chNa <- is.na(cond)
  if(any(chNa)) cond <- cond[which(!chNa)]
  if(min(cond) <1 | max(cond) > prod(dim(mat))) stop("invalid entry for 'cond'")
  ## main
  out <- cbind(row=cond %% nrow(mat), col=(cond +nrow(mat) -1) %/% nrow(mat))
  ch0 <- out[,1]==0        # need to replace row=0
  if(any(ch0)) out[which(ch0)] <- nrow(mat)
  if(length(unique(names(cond)))==length(cond)) rownames(out) <- names(cond)
  if(sortByRows) out <- out[order(out[,1], decreasing=FALSE), ]
  out }
  

#' Find first of repeated elements
#'
#' This function works similar to \code{unique}, but provides additional information about which elements of original input \code{'x'} are repeatd by providing indexes realtoe to the input.
#' \code{firstOfRepeated} makes list with 3 elements : $indRepeated.. index for first of repeated 'x', $indUniq.. index of all unique + first of repeated, $indRedund.. index of all redundant entries, ie non-unique (wo 1st).
#' Used for reducing data to non-redundant status, however, for large numeric input the function nonAmbiguousNum() may perform better/faster.
#' NAs won't be considered (NAs do not appear in reported  index of results), see also firstOfRepLines() .
#' @param x (charcter or numeric) main input
#' @param silent (logical) suppress messages
#' @param callFrom (character) allow easier tracking of message(s) produced
#' @param debug (logical) display additional messages for debugging
#' @return list with indices: $indRepeated,  $indUniq, $indRedund
#' @seealso \code{\link[base]{duplicated}}, \code{\link{nonAmbiguousNum}}, \code{\link{firstOfRepLines}} gives less detail in output (lines/elements/indexes of omitted not directly accessible) and works fsster
#' @examples
#' x <- c(letters[c(3,2:4,8,NA,3:1,NA,5:4)]); names(x) <- 100+(1:length(x))
#' firstOfRepeated(x)
#' x[firstOfRepeated(x)$indUniq]          # only unique with names
#' @export  
firstOfRepeated <- function(x, silent=FALSE, debug=FALSE, callFrom=NULL) {
  fxNa <- .composeCallName(callFrom,newNa="firstOfRepeated")
  chNA <- is.na(x)
  dupH <- duplicated(x, fromLast=FALSE)
  dupL <- duplicated(x, fromLast=TRUE)
  out <- list()
  out$indRepeated <- which(!dupH & dupL & !chNA)
  names(out$indRepeated) <- x[out$indRepeated]
  out$indUniq <- which(!dupH & !chNA)
  out$indRedund <- which(dupH | chNA)
  out }
  

#' Replacements in list
#'
#' \code{listBatchReplace} replaces in list \code{lst} all entries with value \code{searchValue} by \code{replaceBy}
#' @param lst input-list to be used for replacing
#' @param searchValue (character, length=1) 
#' @param replaceBy (character, length=1)
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a corrected list
#' @seealso basic replacement \code{sub} in \code{\link[base]{grep}}
#' @examples
#' lst1 <- list(aa=1:4, bb=c("abc","efg","abhh","effge"), cc=c("abdc","efg"))
#' listBatchReplace(lst1, search="efg", repl="EFG", sil=FALSE)
#' @export
listBatchReplace <- function(lst, searchValue, replaceBy, silent=FALSE, debug=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom, newNa="listBatchReplace")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  
  msg1 <- " 'searchValue' and 'replaceBy' should be vectors"
  if(length(searchValue) <1 || length(replaceBy) <1) stop(fxNa,msg1)
  if(length(lst) <1 || !inherits(lst, "list")) stop(fxNa," 'lst' should be list with at least 1 element")
  outNa <- names(lst)
  if(length(searchValue) ==1 && length(replaceBy) ==1){
    out <- lapply(lst, function(x) {x[x==searchValue] <- replaceBy; x})
  } else {
    if(length(searchValue) ==length(replaceBy)) {
      out <- lapply(lst,function(x) {for(se in 1:length(searchValue)) x[x==searchValue[se]] <- replaceBy[se]; x})
    } else if(length(replaceBy) ==1) {
      out <- lapply(lst,function(x) {for(se in 1:length(searchValue)) x[x==searchValue[se]] <- replaceBy; x})
      } else { out <- lst
      if(!silent) message(fxNa," number of elments in 'searchValue' doesn't match well those from 'replaceBy'; doing nothing")}
  }
  names(out) <- outNa
  out }
    

#' (re)organize data of (3-dim) array as list of replicates
#'
#' Organize array of all data ('arrIn', long table) into list of (replicate-)arrays (of similar type/layout) based on dimension number 'byDim' of 'arrIn' (eg 2nd or 3rd dim).
#' Argument \code{inspNChar} defines the number of characters to consider, so if the beginning of names is the same they will be separated as list of multiple arrays.
#' Default will search for '_' separator or trim from end if not found in the relevant dimnames
#' @param arrIn (array) main input
#' @param inspNChar (interger) if inspNChar=0 the array-names (2nd dim of 'arrIn') will be cut before last '_'
#' @param byDim (integer, length=1) dimension number along which data will be split in separate elements (considering the first inspNChar characters)
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a list of arrays (typically 1st and 2nd dim for specific genes/objects, 3rd for different measures associated with)
#' @seealso \code{\link[base]{array}}
#' @examples
#' arr1 <- array(1:24,dim=c(4,3,2),dimnames=list(c(LETTERS[1:4]),
#'   paste("col",1:3,sep=""), c("ch1","ch2")))
#' organizeAsListOfRepl(arr1)
#' @export
organizeAsListOfRepl <- function(arrIn, inspNChar=0, byDim=3, silent=TRUE, debug=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom, newNa="organizeAsListOfRepl")
  if(isTRUE(debug)) silent <- FALSE else { debug <- FALSE
    if(!isTRUE(silent)) silent <- FALSE }
  if(length(dim(arrIn)) !=3) stop(fxNa,"Expecting data organized as 3 dims, where last dimension should represent different plates")
  msg <- "Expecting 2nd dim to have names where characters 1 to 6 describe the plate-type- to identify replicates"
  if(sum(nchar(dimnames(arrIn)[[byDim]]) <1) >1) stop(fxNa,msg)
  msg <- "Expecting 'byDim' to be single integer between 1 an 3"
  if(!is.numeric(byDim)) stop(msg) else byDim <- as.integer(byDim)[1]  
  if(byDim <1 || byDim >3) stop(msg) 
  if(length(inspNChar) != ncol(arrIn)) inspNChar <- rep(inspNChar,3)[1:ncol(arrIn)]
  chNch <- inspNChar <1
  if(any(chNch)) inspNChar[which(chNch)] <- as.integer(gregexpr("_",dimnames(arrIn)[[byDim]][which(chNch)])) -1 
  chNch <- inspNChar <0
  if(any(inspNChar <1)) inspNChar[which(chNch)] <- nchar(.trimRight(dimnames(arrIn)[[byDim]])[which(chNch)])
  if(!silent) message(fxNa," inspect ",inspNChar," characters of  ",pasteC(dimnames(arrIn)[[byDim]]))
  plateTy <- substr(dimnames(arrIn)[[byDim]],1,inspNChar)
  if(length(unique(plateTy)) ==dim(arrIn)[byDim]) message(fxNa," names of rd dim of 'arrIn' seem all different")
  replPlaArr <- list()
  if(!silent) message(fxNa," inspect the following ",length(unique(plateTy))," plate-types : ",
    paste(utils::head(unique(plateTy),n=20),collapse=" "),if(length(plateTy) >20) " ...")
  uniPlaTy <- unique(plateTy)
  if(length(uniPlaTy) >0) for(i in 1:length(uniPlaTy)) {
    replPlaArr[[i]] <- array(as.numeric(arrIn[,which(plateTy==uniPlaTy[i]),]), dim=c(dim(arrIn)[1],sum(plateTy %in% uniPlaTy[i],na.rm=TRUE),dim(arrIn)[3]))
    dimnames(replPlaArr[[i]]) <- list(dimnames(arrIn)[[1]],dimnames(arrIn)[[byDim]][which(plateTy==uniPlaTy[i])],dimnames(arrIn)[[3]])
    }
  names(replPlaArr) <- uniPlaTy  
  replPlaArr }

#' fuse 2 instances of 3dim arr as mult cols in 3dim array
#'
#' This function allows fusing 2 instances of 3dim arr as mult cols in 3dim array (ie fuse along 2nd dim, increase cols)
#' 
#' @param arr1 (array) 
#' @param arr2 (array) 
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This functuin returns a numeric vector with  numer of non-numeric characters (ie not '.' or 0-9))
#' @seealso \code{\link[base]{array}}
#' @examples
#' aa <- 11:15
#' @export
.fuse2ArrBy2ndDim <- function(arr1,arr2,silent=FALSE, debug=FALSE, callFrom=NULL){
  ## fuse 2 instances of 3dim arr as mult cols in 3dim array (ie fuse along 2nd dim, increase cols)
  fxNa <- .composeCallName(callFrom,newNa=".fuse2ArrBy2ndDim")
  msg <- "Problem with input : needs to have same number of rows and levels of 3rd dim"
  if(!identical(dim(arr1)[c(1,3)], dim(arr2)[c(1,3)])) stop(fxNa,msg)
  array(rbind(matrix(as.numeric(arr1), ncol=dim(arr1)[3]), matrix(as.numeric(arr2),ncol=dim(arr2)[3])), 
    dim=c(dim(arr1)[1], dim(arr1)[2] +dim(arr2)[2], dim(arr1)[3])) }
     

#' Count number of non-numeric characters
#'
#' \code{nNonNumChar} counts number of non-numeric characters.
#' Made for positive non-scientific values (eg won't count neg-sign, neither Euro comma ',') 
#' @param txt character vector to be treated
#' @return This function returns a numeric vector with  numer of non-numeric characters (ie not '.' or 0-9))
#' @seealso \code{\link[base]{nchar}}
#' @examples
#' nNonNumChar("a1b "); sapply(c("aa","12ab","a1b2","12","0.5"), nNonNumChar)
#' @export
nNonNumChar <- function(txt) {txt <- as.character(txt)
   sum(!sapply(1:nchar(txt), function(x) substr(txt, x, x) %in% c(".",0:9))) } 

#' Extract number(s) before capital character
#'
#' This function aims to extract number(s) before capital character
#' 
#' @param x character vector to be treated
#' @return This function returns a numeric vector
#' @seealso  \code{\link[base]{grep}}, \code{\link[base]{nchar}}
#' @examples
#' .extrNumHeadingCap(" 1B ")
#' @export
.extrNumHeadingCap <- function(x){
  ## extract number(s) before capital character
  tmp <- substr(x, regexpr("[[:digit:]]+[[:upper:]]",x), nchar(x))
  as.numeric(substr(tmp, 0, regexpr("[[:upper:]]",tmp) -1)) }

#' Extract numbers before separator followed by alphabetic character
#'
#' This function aims to extract number(s) before separator followed by alphabetic character (return named numeric vector, NAs when no numeric part found)
#' 
#' @param x character vector to be treated
#' @param sep (character) separator
#' @return This function returns a numeric vector 
#' @seealso \code{\link[base]{nchar}}
#' @examples
#' .extrNumHeadingSepChar(" 1B ")
#' @export
.extrNumHeadingSepChar <- function(x, sep="_"){
  ## extract number(s) before separator followed by alphabetic character (return named numeric vector, NAs when no numeric part found)
  tmp <- substr(x, 1, attributes(regexpr(paste("[[:digit:]]+",sep,"[[:alpha:]]",sep=""),x))[[1]] -2)
  out <- as.numeric(tmp)
  if(!is.null(names(x))) names(out) <- names(x)
  out }
  
#' Set lowest value to given value 
#'
#' This function aims to set lowest value of x to value 'setTo'
#' 
#' @param x (numeric) main vector to be treated
#' @param setTo (numeric) replacement value
#' @return This function returns a numeric vector 
#' @seealso \code{\link[base]{nchar}}
#' @examples
#' .setLowestTo(9:4, 6)
#' @export
.setLowestTo <- function(x, setTo) {
  ## set lowest value of x to value 'setTo'
  mi <- min(x[which(is.finite(x))], na.rm=TRUE); x[which(x==mi)] <- setTo; x}
  

#' Organize Data as Separate List-Entries
#'
#' \code{asSepList} allows reorganizing most types of input into a list with separate numeric vectors. For example, matrixes or data.frames will be split into separate columns
#' (differnt to \code{\link[wrMisc]{partUnlist}} which maintains the original structure). This function also works with lists of lists.
#' This function may be helpful for reorganizing data for plots.
#'
#' @param y list to be separated/split in vectors
#' @param minLen (integer) min length (or number of rows), as add'l element to eliminate arguments given without names when asSepList is called in vioplot2
#' @param asNumeric (logical) to transform all list-elements in simple numeric vectors (won't work if some entries are character)
#' @param exclElem (character) optinal names to exclude if any (lazy matching) matches (to exclude other arguments be misinterpreted as data)
#' @param sep (character) separator when combining name of list-element to colames
#' @param fillNames (logical) add names for list-elements/ series when not given
#' @param silent (logical) suppress messages
#' @param callFrom (character) allow easier tracking of messages produced
#' @param debug (logical) display additional messages for debugging
#' @return This function returns a list, partially unlisted to vectors
#' @seealso \code{\link{partUnlist}}, \code{\link[base]{unlist}}
#' @examples
#' bb <- list(fa=gl(2,2), c=31:33, L2=matrix(21:28,nc=2),
#'   li=list(li1=11:14, li2=data.frame(41:44)))
#' asSepList(bb)
#' ## multi data-frame examples
#' ca <- data.frame(a=11:15, b=21:25, c=31:35)
#' cb <- data.frame(a=51:53, b=61:63)
#' cc <- list(gl(3,2), ca, cb, 91:94, short=81:82, letters[1:5])
#' asSepList(cc)
#' cd <- list(e1=gl(3,2), e2=ca, e3=cb, e4=91:94, short=81:82, e6=letters[1:5])
#' asSepList(cd)
#' @export
asSepList <- function(y, minLen=4, asNumeric=TRUE, exclElem=NULL, sep="_", fillNames=TRUE, silent=FALSE, callFrom=NULL, debug=FALSE) {
  ## convert all data-series of list (ie all list elements or columns) in separate list-elements (OK with list of lists) eg for plots
  ## 'asNumeric'.. to transform all list-elements in simple numeric vectors (won't work if some entries are character)
  ## 'minLen' .. min length (or number of rows), as add'l element to eliminate arguments given wo names when asSepList is called in vioplot2
  ## 'fxArg' .. optinal, names to exclude if any (lazy matching) matches (to exclude other arguments be mis-interpreted as data, used in vioplot2)
  fxNa <- .composeCallName(callFrom, newNa="asSepList")
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  if(!isTRUE(silent)) silent <- FALSE
  namesY <- sub("[[:punct:]].*|[[:space:]].*","",deparse(substitute(y)))   # reduce to alphanum content
   aSL0 <- list(y=y,asNumeric=asNumeric,minLen=minLen,namesY=namesY)
  if(length(y) >0) {
    if(!is.list(y) || is.data.frame(y)) {    # for data.frame
      y <- .asDF2(y)
      chNam <- if(length(colnames(y)) <1) rep(TRUE, ncol(y)) else colnames(y) %in% ""
      if(any(chNam)) colnames(y)[which(chNam)] <- paste0(namesY,sep,which(chNam))
      y <- as.list(y)
      if(debug) {message(fxNa,"aSL1  Non-list concert to list of length ",length(y)); aSL1 <- list(y=y,asNumeric=asNumeric,minLen=minLen) }
    } else {
        aSL0b <- list()
      .matr2List <- function(z) as.list(as.data.frame(z))
      chSubLi <- sapply(y, is.list) & !sapply(y, is.data.frame)
      if(debug) {message(fxNa,"aSL1  list-entry; ini length of 'y' ",length(y)); aSL1 <- list(y=y,asNumeric=asNumeric,chSubLi=chSubLi) }  
      
      w <- NULL
      ## try to separate sub-lists
      if(length(y) >0 & any(chSubLi)) {                        # run partUnlist() on all sub-lists
        if(length(y)==1) { y <- .asDF2(y[[1]])
          if(debug) {message(fxNa,"aSL1b")}
          chNam <- if(length(colnames(y)) <1) rep(TRUE, ncol(y)) else colnames(y) %in% ""
          if(any(chNam)) colnames(y)[which(chNam)] <- paste0(namesY,sep,which(chNam))
          y <- as.list(y)
        } else {
          isLi <- sapply(y, inherits, "list")
          chNam <- if(length(names(y)) >0) names(y) =="" else rep(FALSE, length(y))
          if(any(chNam) & isTRUE(fillNames)) {newNa <- paste(namesY,which(chNam),sep=sep)
            if(any(newNa %in% names(y))) newNa <- paste0(namesY,sep,"_",which(chNam))
              names(y)[which(chNam)] <- newNa }
          if(debug) {message(fxNa,"aSL2  "); aSL2 <- list(isLi=isLi,chNam=chNam)}

          if(any(isLi)) y <- partUnlist(y, silent=silent, debug=debug,callFrom=fxNa)  #[which(chSubLi)])
          ## now need to separate matrix-columns (& check names)
          iniDim <- lapply(y, ncol)
          ch2d <- sapply(iniDim, function(x) length(x)==1)
          if(any(ch2d)) {             ## contains matrix or data.frame, need to separate cols
            w <- lapply(y[which(ch2d)], .matr2List)
            w <- partUnlist(w, silent=silent,debug=debug,callFrom=fxNa)
            names(w) <- paste0(rep(names(y)[which(ch2d)],unlist(iniDim[which(ch2d)])),sep, unlist(lapply(unlist(iniDim[which(ch2d)]), function(x) if(x >1) 1:x else ""))) }
          y <- y[-which(ch2d)]                        # remove matrix parts
          y[length(y) +(1:length(w))] <- w            # attach separated columns
          names(y)[1 +length(y) -(length(w):1)] <- names(w)
          if(debug) {message(fxNa,"aSL3   Length of basic part ",length(y),"   length of matrix-part ",length(w))}

          ## adjust order
          newOr <- as.list(match(names(iniDim), names(y)))
          chNa <- is.na(match(names(iniDim), names(y)))
          if(any(chNa, na.rm=TRUE)) newOr[which(chNa)] <- lapply(paste0("^",names(iniDim)[which(is.na(match(names(iniDim), names(y))))],sep), grep, names(y))
          y <- y[unlist(newOr)]
          names(y) <- sub(paste0(sep,"$"),"", names(y))     # remove tailing sep from single-column matrices
        }
      }
    }
    if(debug) {message(fxNa,"aSL4   Length of list output (befor minLen-filter) ",length(y))}
    ## check length
    chLe <- sapply(y, length) < minLen
    if(any(chLe, na.rm=TRUE)) { y <- y[which(!chLe)]
      if(all(chLe, na.rm=TRUE) & !silent) message(fxNa,"All elements of ',namesY,' below length-limit (",minLen,") - nothing remains") }
    ## convert to numeric
    if(isTRUE(asNumeric)) { chMode <- sapply(y, function(x) "numeric" %in% mode(x))
      if(any(!chMode)) y[which(!chMode)] <- lapply(y, convToNum, callFrom=fxNa,silent=silent) }
    if(debug) {message(fxNa,"aSL5   Length of list output (after minLen-filter) ",length(y))}
  } else if(debug(fxNa,"Empty input, nothing to do"))
  if(debug) message(fxNa,"Returning list of length ",length(y))
  y }


#' Convert anything to data.frame
#'
#' This function allows converting anything to data.frame
#' 
#' @param z (numeric vector, factor, matrix or list) main input
#' @return data.frame
#' @seealso  \code{\link[base]{as.data.frame}}
#' @examples
#' .asDF2(c(3:6))
#' @export
.asDF2 <- function(z) if(is.factor(z)) as.data.frame(as.character(z)) else as.data.frame(z)  # convert anything to data.frame-like
  

#' Extract pair of numeric values from vector or column-names
#'
#' This function extracts a pair of numeric values out of a vector or colnames (from a matrix). 
#' This is useful when pairwise comparisons are concatenated like '10c-100c', return matrix with 'index'=selComp, log2rat and both numeric.
#' Additional white space or character text can be removed via the argument \code{stripTxt}.
#' Of course, the separator \code{sep} needs to be specified and should not be included to 'stripTxt'.
#'
#' @param dat (matrix or data.frame) main input
#' @param selComp (character) the column index selected 
#' @param stripTxt (character, max length=2) text to ignore, if NULL heading letter and punctuation characters will be removed; default will remove all letters (and following spaces)
#' @param sep (character, length=1) separator between pair of numeric values to extract
#' @param columLabel (character) column labels in output
#' @param sortByAbsRatio (logical) optional sorting of output by (absolute) log-ratios (most extreme ratios on top)
#' @param silent (logical) suppress messages
#' @param callFrom (character) allow easier tracking of messages produced
#' @param debug (logical) additional messages for debugging
#' @return This function returns a matrix
#' @seealso \code{\link[base]{strsplit}} and help on regex 
#' @examples
#' ##  composed column names
#' mat1 <- matrix(1:8, nrow=2, dimnames=list(NULL, paste0(1:4,"-",6:9)))
#' numPairDeColNames(mat1)
#' numPairDeColNames(colnames(mat1))
#' ##  works also with simple numeric column names
#' mat2 <- matrix(1:8, nrow=2, dimnames=list(NULL, paste0("a",6:9)))
#' numPairDeColNames(mat2)
#' @export
numPairDeColNames <- function(dat, selComp=NULL, stripTxt=NULL, sep="-", columLabel="conc", sortByAbsRatio=FALSE, silent=FALSE, debug=FALSE, callFrom=NULL) {
  ##  extract pair of numeric content of colnames from pairwise comparisons, return matrix with 'index'=selComp, log2rat and both numeric 
  ## dat (matrix or data.frame) testing results (only the colnames will be used)
  ## selComp (integer) the column index selected
  ## stripTxt (character) text to be removed, if NULL letter and punctuation characters will be removed; default will remove all letters (and following spaces)
  ## sep (character) separator (for separating multiple numeric parts out of character-string)
  ## columLabel (character) used for column labels in output
  ## sortByAbsRatio (logical) sort output by (absolute) log-ratios
  fxNa <- .composeCallName(callFrom, newNa="numPartDeColNames")
  if(length(dim(dat)) >1) dat <- colnames(dat)  
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE

  if(length(dat) <1) stop("Invalid entry of 'dat'")
  if(length(selComp) >0) {
    if(is.numeric(selComp)) dat <- dat[as.integer(selComp)] else {
      if(any(selComp %in% dat)) dat[selComp] else message(fxNa," invalid entry for 'selComp', ignoring")}
  } else selComp <- 1:length(dat)
  if(length(stripTxt) <1) stripTxt <- "[[:alpha:]]+[[:space:]]*[[:alpha:]]*|[[:space:]]+[[:alpha:]]+[[:space:]]*[[:alpha:]]*|[[:space:]]+"
  chSep <- nchar(sub(stripTxt,"",paste0("A",sep,"B")))
  if(chSep !=3) message(fxNa,"PROBLEM ? : 'stripTxt' does REMOVE the separator 'sep' ! Select a different separator or 'stripTxt' strategy to resolve pairwise combinations !") 
  dat <- if(length(stripTxt) >1) sub(stripTxt[2],"",sub(stripTxt[1],"",dat)) else gsub(stripTxt,"",dat)
  chSep <- grep(sep, dat)
  if(length(chSep) <1) {
    if(!silent) message(fxNa,"Could not find any instance of separator 'sep'; (maybe removed with 'stripTxt' ?)")
    logRat <- try(as.integer(dat), silent=TRUE)       # try remove text at start or end
    if(inherits(logRat, "try-error")) stop(fxNa," Did not succed to extract (single) numeric content")
    logRat <- cbind(index=selComp, logRat=NA, logRat)
    colnames(logRat)[3] <- columLabel 
  } else {
    logRat <- try(as.integer( unlist(strsplit(dat, sep))), silent=TRUE)
    if(inherits(logRat, "try-error")) stop(fxNa," Did not succed to extract numeric content (by splitting)")
    logRat <- matrix(logRat, ncol=2, byrow=TRUE, dimnames=list(NULL,paste0(columLabel,1:2)))
    chRat <- logRat[,1] > logRat[,2]
    if(any(chRat)) logRat[which(chRat),] <- logRat[which(chRat), 2:1]
    logRat <- cbind(index=selComp, log2rat=signif(log2(logRat[,2]/logRat[,1]),4), logRat) }
  if(sortByAbsRatio && nrow(logRat) >1) logRat <- logRat[order(if(length(chSep) <1) logRat[,3] else abs(logRat[,2]), decreasing=TRUE),]
  logRat }
   

#' Sort matrix by two categorical and one integer columns 
#'
#' This function sorts matrix 'mat' subsequently by categorical and numerical columns of 'mat',
#'  ie lines with identical values for categor are sorted by numeric value.
#' @param mat matrix (or data.frame) from which by 2 columns will be selected for sorting
#' @param categCol (integer or character) which columns of 'mat' to be used as categorical columns
#' @param numCol (integer or character) which column of 'mat' to be used as integer columns
#' @param findNeighb (logical) if 'findNeighb' neighbour cols according to 'numCol' will be identified as groups & marked in new col 'neiGr', orphans marked as NA
#' @param decreasing (logical) order of sort
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a sorted matrix (same dimensions as 'mat')
#' @examples
#' mat <- cbind(aa=letters[c(3,rep(7:8,3:4),4,4:6,7)],bb=LETTERS[rep(1:5,c(1,3,4,4,1))],
#'   nu=c(23:21,23,21,22,18:12))
#' mat[c(3:5,1:2,6:9,13:10),]
#' sortBy2CategorAnd1IntCol(mat,cate=c("bb","aa"),num="nu",findN=FALSE,decr=TRUE)
#' sortBy2CategorAnd1IntCol(mat,cate=c("bb","aa"),num="nu",findN=TRUE,decr=FALSE)
#' @export
sortBy2CategorAnd1IntCol <- function(mat, categCol, numCol, findNeighb=TRUE, decreasing=FALSE, silent=FALSE, debug=FALSE,callFrom=NULL) {
  fxNa <- .composeCallName(callFrom,newNa="sortBy2CategorAnd1IntCol")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE

  if(length(dim(mat)) <2) stop("'mat' should be matrix (or data.frame) with >2 cols and >1 line")
  if(length(numCol) <1) stop("'numCol' invalid") else if(length(numCol) >1) numCol <- numCol[1]
  if(is.numeric(numCol)) { numCol <- as.integer(numCol)
    if(numCol <1 || numCol >ncol(mat)) stop("'numCol' out of range of 'mat'")
  } else if(numCol %in% colnames(mat)) numCol <- match(numCol,colnames(mat)) else stop("cCan't find 'numCol' in colnames of 'mat'")
  dimIni <- dim(mat)
  ## main
  num <- as.numeric(mat[,numCol[1]])
  mat <-  mat[sort.list(num,decreasing=decreasing),]
  mat <- mat[sort.list(if(length(categCol)==2) paste(mat[,categCol[1]],mat[,categCol[2]]) else mat[,categCol[1]],decreasing=decreasing),] # combined sort for categor
  if(findNeighb){
    num <- as.numeric(mat[,numCol[1]])
    if(length(categCol)==2) {
      tmp <- paste(mat[,categCol[1]],mat[,categCol[2]])
      isNei <- tmp[-1]==tmp[-nrow(mat)]       
    } else isNei <- mat[-1,categCol[1]]==mat[-nrow(mat),categCol[1]]
    isNei <- c(FALSE,isNei) & c(0,num[-1] -num[-nrow(mat)])== (if(decreasing) -1 else 1)
    gr <- cumsum(!isNei)
    dup <- (duplicated(gr, fromLast=TRUE) | duplicated(gr, fromLast=FALSE))
    if(any(!dup)) gr[which(!dup)] <- NA
    mat <- cbind(mat[,1:dimIni[2]],neiGr=gr) }
  mat }
    

#' Filter for unique elements
#' 
#' This function aims to apply a given filter-citerium, a matrix or vector of \code{FALSE/TRUE} which is typically combined with a second layer   
#' which filters for a min content of filer-passing values per line for the first/main criterium.
#' Then all lines concerned will be removed. This will be done for all list-elements (of appropriate size) of the input-list  
#' (while maintaining the list-structure in the output) not matching the filtering criteria.
#' 
#' @param lst (list) main input, each vector, matrix or data.frame in this list will be filtered if its length or number of lines fits to \code{filt}
#' @param filt (logical) vector of \code{FALSE/TRUE} to use for filtering. If this a matrix is given, the value of \code{minLineRatio} will be applied as threshod of min content of \code{TRUE} for each line of \code{filt} 
#' @param minLineRatio (numeric) in case \code{filt} is a matrix of \code{FALSE/TRUE}, this value will be used as threshold of min content of \code{TRUE} for each line of \code{filt} 
#' @param silent (logical) suppress messages
#' @param callFrom (character) allow easier tracking of message(s) produced
#' @return filtered list 
#' @seealso \code{\link{correctToUnique}}, \code{\link[base]{unique}}, \code{\link[base]{duplicated}}, \code{\link{extrColsDeX}} 
#' @examples
#' set.seed(2020); dat1 <- round(runif(80),2)
#' list1 <- list(m1=matrix(dat1[1:40],ncol=8), m2=matrix(dat1[41:80],ncol=8), other=letters[1:8])
#' rownames(list1$m1) <- rownames(list1$m2) <- paste0("line",1:5)
#' filterList(list1, list1$m1[,1] >0.4) 
#' filterList(list1, list1$m1 >0.4) 
#' @export
filterList <- function(lst,filt,minLineRatio=0.5,silent=FALSE,callFrom=NULL) {
  ##  adjust all elements of lst to filtering
  ## minLineRatio (numeric) min ratio of columns where 
  ## assumes that all elements of lst are in correct order !
  fxNa <- .composeCallName(callFrom, newNa="filterList")
  if(length(filt) <1) stop(" 'filt' seems to empty")
  if(length(dim(filt)) >1) { 
    if(length(minLineRatio) <1 | !is.numeric(minLineRatio)) { minLineRatio <- 0.5
      if(!silent) message(fxNa," argument 'minLineRatio' must be numeric !  Setting to default (0.5)")}
    filt <- rowSums(filt) >= ncol(filt)*minLineRatio }
  chFi <- sub("(TRUE)|(FALSE)|T|F","",filt)
  if(any(nchar(chFi) >0)) stop(" 'filt' contains non-logical elements")
  if(is.logical(filt)) filt <- as.logical(filt)
  if(all(!filt)) stop("nothing passes filtering")  
  ## main
  if(any(!filt)) {
    nFilt <- length(filt)
    filt <- which(filt)
    lstDim <- lapply(lst, dim)
    chLst <- sapply(lstDim,length) ==2
    msg <- c(" element '","' : "," not suitable for filter")
    ## filter all matrix & data.frames
    if(any(chLst)) {
      for(i in which(chLst)) if(nrow(lst[[i]]) ==nFilt) { 
        lst[[i]] <- if(length(filt) >1 & length(dim(lst[[i]])) >1) lst[[i]][filt,] else {
          matrix(lst[[i]][filt,],ncol=ncol(lst[[i]]),dimnames=list(rownames(lst[[i]])[filt],colnames(lst[[i]]))) }
      } else {
        if(!silent) message(fxNa,msg[1],names(lst)[i],msg[2],"number of lines",msg[3]) }}
    ## filter all vectors    
    chLst <- sapply(lstDim,length) ==1
    if(any(chLst)) {
      for(i in which(chLst)) if(nrow(lst[[i]]) ==nFilt) lst[[i]] <- lst[[i]][filt] else {
        if(!silent) message(fxNa,msg[1],names(lst)[i],msg[2],"length of vector",msg[3]) }}      
  } else if(!silent) message(fxNa,"all elements pass filter (nothing to remove)")
  lst }
  

#' Convert GitHub Url-Name For Reading In Raw-Mode
#'
#' This functions converts a given urlName so that (tabular) data from GitHub (https://github.com/) can data be read correctly as tabular data.
#' 
#' @details
#'   
#' Thus, this will exchange the beginning of the site to 'raw.githubusercontent.com' and then exchange 'blob' to 'refs/head' .
#' Alternatively, the user may custom define all terms to get exchanged (as matrix with 2 columns 'old' and 'new').          
#' 
#' Note : The default spearator from  \code{\link[utils]{read.delim}} may suit better for reading the data.
#'  
#' @param urlName (charachter) main url-address
#' @param replTxt (NULL or matrix) adjust/ custom-modify search- and replacement items; default settings will exchange the beginning of the site to 'raw.githubusercontent.com' and then exchange 'blob' to 'refs/head'
#'  otherwise should be matrix with 2 columns, the 1st colimn entries will be used as 'search-for' and the 2nd as 'replace by' for each row 
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return corrected urlName
#' @seealso  \code{\link[base]{sub}}; \code{\link[utils]{read.delim}}
#' @examples
#' url1 <- paste0("https://github.com/bigbio/proteomics-metadata-standard/blob/",
#'   "main/datasets/PXD001819/PXD001819.sdrf.tsv") 
#' rbind(ini=url1, new=gitDataUrl(url1))
#' 
#' @export
gitDataUrl <- function(urlName, replTxt=NULL, silent=FALSE, debug=FALSE, callFrom=NULL) {
  ## convert urlName from git that tabular data can be read correctly, ie remove '/blob/' & change start to 'raw.githubusercontent.com'
  fxNa <- .composeCallName(callFrom, newNa="gitDataUrl")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  if(length(urlName) <1) { if(!silent) message(fxNa,"'urlName' is empty, nothing to do")
  } else {
    if(length(replTxt) ==0 || (any(length(dim(replTxt)) !=2, dim(replTxt) < 1:2))) replTxt <- rbind(
      c("^https://github.com/", "https://raw.githubusercontent.com/"),
      c("/blob/", "/refs/heads/"))
    for(i in 1:nrow(replTxt)) {
      ch1 <- grepl(replTxt[i,1], urlName)
      if(isTRUE(ch1)) urlName <- sub(replTxt[i,1], replTxt[i,2], urlName)
    }
  }
  urlName }   
       

#' SEM for each row
#'
#' This function speed optimized SEM (standard error of the mean) for each row.
#' The function takes a matrix or data.frame and treats each row as set of data for SEM; NAs are ignored from data.
#' Note: NaN instances will be transformed to NA 
#' @param dat matrix or data.frame
#' @return  This function returns a numeric vector with SEM values
#' @seealso \code{\link{rowSds}},  \code{\link{colSds}}, \code{\link[base]{colSums}}
#' @examples
#' set.seed(2016); dat1 <- matrix(c(runif(200)+rep(1:10,20)),ncol=10)
#' head(rowSEMs(dat1))
#' @export
rowSEMs <- function(dat) {
  if(is.null(ncol(dat))) stop("Data should be matrix or data.frame with multiple columns !") else if(ncol(dat) < 2) {
    stop("Data should be matrix or data.frame with at least 2 columns !")}
  out <- sqrt(rowSums(matrix(as.numeric(!is.na(dat)),ncol=ncol(dat))*((dat) - rowMeans(dat, na.rm=TRUE))^2,na.rm=TRUE)/(rowSums(!is.na(dat)) -1)) /
    sqrt(rowSums(!is.na(dat)))
  out[is.nan(out)] <- NA      
  out }
  

#' Value Matching with optional reversing of sub-parts of non-matching elements
#'
#' This function provides a variant to \code{\link[base]{match}}, where initially non-matching elements of \code{x}  
#' will be tested by decomposing non-matching elements, reversing the parts in front and after the separator \code{sep} and re-matching.
#' If separator \code{sep} does not occur, a warning will be issued, if it occurs more than once, 
#' the parts before and after the first separartor will be used and a warning issued.
#'
#' @param x (character) first vector for match 
#' @param y (character) second vector for match
#' @param sep (character) separator between elements
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return index for matching (integer) x to y
#' @seealso \code{\link[base]{match}} 
#' @examples
#' tx1 <- c("a-b","a-c","d-a","d-b","b-c","d-c")
#' tmp <- triCoord(4)
#' tx2 <- paste(letters[tmp[,1]],letters[tmp[,2]],sep="-")
#' ## Some matches won't be found, since 'a-d' got reversed to 'd-a', etc... 
#' match(tx1,tx1)  
#' matchNamesWithReverseParts(tx1,tx2)
#' @export
matchNamesWithReverseParts <- function(x, y, sep="-", silent=FALSE, debug=FALSE, callFrom=NULL) {
  fxNa <- .composeCallName(callFrom,newNa="matchNamesWithReverseParts")
  if(length(x) <1) stop(" Nothing to do, 'x' seems empty")
  if(length(y) <1) stop(" Nothing to do, 'y' seems empty")
  ma1 <- match(x,y)
  if(any(is.na(ma1))) {
    sup <- which(is.na(ma1))
    x2 <- strsplit(x[sup],sep)
    chX <- sapply(x2,length)
    if(any(chX <2) && !silent) message(fxNa,"Can't find separator ('",sep,"') in  ",sum(chX <2)," elements of 'x'")
    if(any(chX >2) && !silent) message(fxNa,"BEWARE, results may be different to expected !  Separator ('",
      sep,"') occurs more than once in ",sum(chX >2)," elements of 'x'")
    x2 <- sapply(x2,function(z) if(length(z) >1) paste(z[2],z[1],sep=sep) else z)
    x[sup] <- x2
    ma1 <- match(x,y) } 
  ma1 }
   

#' Filter For UniqueElements
#' 
#' This function aims to identify and remove duplicated elements in a list and maintain the list-structure in the output. 
#' \code{filtSizeUniq}  filters 'lst' (list of character-vectors or character-vector) for elements being unique (to 'ref' or if NULL to all 'lst') and of character length. 
#' In addition, the min- and max- character length may be filtered, too. Eg, in proteomics this helps removing peptide sequences which would not be measured/detected any way.
#'
#' @param lst list of character-vectors or character-vector
#' @param ref (character) optional alternative 'reference', if not \code{NULL} used in addition to 'lst' for considering elements of 'lst' as unique
#' @param minSize (integer) minimum number of characters, if \code{NULL} set to 0
#' @param maxSize (integer) maximum number of characters
#' @param filtUnique (logical) if \code{TRUE} return unique-only character-strings
#' @param byProt (logical) if \code{TRUE} organize output as list (by names of input, eg protein-names) - if 'lst' was named list
#' @param inclEmpty (logical) optional including empty list-elements when all elements have been filtered away - if 'lst' was named list 
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return list of filtered input
#' @seealso \code{\link{correctToUnique}}, \code{\link[base]{unique}}, \code{\link[base]{duplicated}}
#' @examples
#' filtSizeUniq(list(A="a",B=c("b","bb","c"),D=c("dd","d","ddd","c")),filtUn=TRUE,minSi=NULL)
#' # input: c and dd are repeated
#' filtSizeUniq(list(A="a",B=c("b","bb","c"),D=c("dd","d","ddd","c")),ref=c(letters[c(1:26,1:3)],
#'   "dd","dd","bb","ddd"),filtUn=TRUE,minSi=NULL)  # a,b,c,dd repeated 
#' @export
filtSizeUniq <- function(lst, ref=NULL, minSize=6, maxSize=36, filtUnique=TRUE, byProt=TRUE, inclEmpty=TRUE, silent=FALSE,debug=FALSE, callFrom=NULL) {
  ## filter protein sequences for size/length and for unique
  fxNa <- .composeCallName(callFrom,newNa="filtSizeUniq")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE

  chNa <- grep("\\.$", names(utils::head(lst)))                                   # check for attached tailing '.'
  if(!is.list(lst)) {byProt <- FALSE; inclEmpty <- FALSE}
  if(length(chNa) <= min(2,length(lst))) names(lst) <- paste0(names(lst),".")
  pep <- unlist(lst)
  chNa <- max(sapply(lst,length), na.rm=TRUE)
  if(chNa >1) names(pep) <- sub("\\.$","",names(pep))                 # remove tailing '.' of names if list-element has length=1
  nPep <- length(pep)
  nAA <- nchar(pep)
  if(length(minSize) <1) minSize <- 0
  if(length(maxSize) <1) {maxSize <- 40
    if(!silent) message(fxNa," can't understant 'maxSize', setting to default=40")}
  ## filter by size
  chAA <- nAA >= minSize & nAA <= maxSize
  if(any(!chAA)) {pep <- if(all(!chAA)) NULL else pep[which(chAA)]
    if(!silent) message(fxNa,nPep - length(pep)," out of ",nPep," peptides beyond range (",minSize,"-",maxSize,")")}
  ## filter unique /reundant
  if(filtUnique) {
    nPe2 <- length(pep)
    if(length(ref) >0) {pep0 <- pep; pep <- c(pep,unique(unlist(ref))) } else pep0 <- NULL
    chDup <- duplicated(pep,fromLast=FALSE)
    if(any(chDup)) {
      chDu2 <- duplicated(pep,fromLast=TRUE)
      if(length(ref) >0) {pep <- pep0; chDup <- chDup[1:nPe2]; chDu2 <- chDu2[1:nPe2]} 
      pep <- list(unique=pep[which(!chDu2 & !chDup)],allRedund=pep[which(!(!chDu2 & !chDup))], firstOfRed=pep[which(chDu2 & !chDup)])
      if(!silent) message(fxNa,length(pep$allRedund)," out of ",nPe2," peptides redundant")
    } else {if(length(ref) >0) {pep <- pep0; chDup <- chDup[1:nPe2]}}   
  }
  ##  
  if(byProt) { fac <- sub("\\.[[:digit:]]+$","",names(if(filtUnique) pep$unique else pep))
    pep <- tapply(if(filtUnique) pep$unique else pep,fac,function(x) x) 
    if(length(pep) <1) pep <- character() 
    if(inclEmpty) { iniPro <- sub("\\.$","",names(lst)) 
      curPro <- names(pep) 
      newNo <- sum(!iniPro %in% curPro)
      if(newNo >0){ pep[length(curPro) +(1:newNo)] <- lapply(1:newNo,function(x) character())
        names(pep)[length(curPro) +(1:newNo)] <- iniPro[which(!iniPro %in% curPro)]}
    }
  }
  pep }
  
#' Filter for size
#' 
#' This function aims to filter for size
#'
#' @param x main inpuy
#' @param minSize (integer) minimum number of characters, if \code{NULL} set to 0
#' @param maxSize (integer) maximum number of characters
#' @return This function returns a list of filtered input
#' @seealso \code{\link{filtSizeUniq}}; \code{\link{correctToUnique}}, \code{\link[base]{unique}}, \code{\link[base]{duplicated}}
#' @examples
#' aa <- 1:10
#' @export
.filtSize <- function(x, minSize=5, maxSize=36) {nCha <- nchar(x); x[which(nCha >= minSize & nCha <= maxSize)]}      # filter by size (no of characters)
   

#' Count number of NAs per sub-set of columns
#'
#' This function will count the number of \code{NA}s per group (defined by argument \code{grp}) while summing over all lines of a matrix or data.frame.
#' The row-position has no influence on the counting.
#' Using the argument \code{asRelative=TRUE} the result will be given as (average) number of \code{NA}s per row and group.
#' 
#' @param x matrix or data.frame which may contain \code{NA}s
#' @param grp factor describing which column of 'dat' belongs to which group
#' @param asRelative (logical) return as count of \code{NA}s per row and group
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns an integer vector with count of \code{NA}s per group
#' @seealso \code{\link[base]{NA}}, filter \code{NA}s by line \code{\link{presenceFilt}}
#' @examples
#' mat <- matrix(1:25, ncol=5) 
#' mat[lower.tri(mat)] <- NA
#' sumNAperGroup(mat, rep(1:2,c(3,2)))
#' sumNAperGroup(mat, rep(1:2,c(3,2)), asRelative=TRUE)
#' 
#' @export
sumNAperGroup <- function(x, grp, asRelative=FALSE, silent=FALSE, debug=FALSE, callFrom=NULL) {
  ## count number of NAs per set of columns defined by grp
  fxNa <- .composeCallName(callFrom, newNa="sumNAperGroup")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE

  if(length(dim(x)) <2) stop("Argument 'x' should be matrix or data.frame")
  if(length(grp) != ncol(x)) stop("Length of argument 'x' should match number of columns in 'x'")
  if(is.data.frame(x)) x <- as.matrix(x)
  if(debug) message(fxNa,"sNAG1")
  grpLev <- unique(naOmit(grp))
  out <- as.integer(by(t(x), grp, function(y) sum(is.na(y))))[rank(grpLev)]
  names(out) <- grpLev
  if(isTRUE(asRelative)) {
    nGrp <- table(grp)[rank(grpLev)] 
    out <- out/(nrow(x)*nGrp) }
  out }   
  

#' Rescaling According To Reference Data Using Linear Regression.
#'
#' This function does rescaling: linear transform simple vector 'inDat' that (mean of) elements of names cited in 'refLst' will end up as values 'regrTo'. 
#' Regress single vector according to 'refLst' (describing names of inDat).
#' If 'refLst' contains 2 groups, the 1st group will be set to the 1st value of 'regrTo' (and the 2nd group of 'refLst' to the 2nd 'regtTo')
#' @param inDat (matrix or data.frame) data to rescaled 
#' @param refLst list of names existing in inDat (one group of names for each value in 'regrTo'), to be transformed in values precised in 'regTo'; if no matches to names of 'inDat' found, the 2 lowest and/or highest highest values will be chosen
#' @param regrTo (numeric,length=2) range (at scale 0-1) of target-values for mean of elements cited in 'refLst'
#' @param silent (logical) suppress messages if \code{TRUE}
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allows easier tracking of messages produced
#' @return This function returns a normalized matrix
#' @seealso \code{\link{adjBy2ptReg}}, \code{\link{regrMultBy1or2point}}
#' @examples
#' set.seed(2016); dat1 <- 1:50 +(1:50)*round(runif(50),1)
#' names(dat1) <- 1:length(dat1)
#' reg1 <- regrBy1or2point(dat1, refLst=c("2","49"))
#' plot(reg1, dat1) 
#' @export
regrBy1or2point <- function(inDat, refLst, regrTo=c(1,0.5), silent=FALSE, debug=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom,newNa="regrBy1or2point")
  if(isTRUE(debug)) silent <- FALSE
  if(!isTRUE(silent)) silent <- FALSE
  if(length(names(inDat)) <0) {
    names(inDat) <- 1:length(inDat)
    if(!silent) message(fxNa,"No (valid) names found in 'inDat', making default numbering")}
  chLst <- sapply(refLst, function(x) !all(x %in% names(inDat)))
  if(any(chLst)) {
    if(chLst[1]) refLst[[1]] <- names(inDat)[order(inDat)][1:2]
    if(chLst[2]) refLst[[2]] <- names(inDat)[order(inDat)][length(inDat):(length(inDat)-1)]
    if(!silent) message(fxNa," no (valid) names found in 'refLst', choosing noth most extreme values")}
  checkEntr <- .checkRegrArguments(inDat, refLst, regrTo)
  refLst <- checkEntr$refLst
  regrTo <- checkEntr$regrTo
  tmp <- sapply(refLst, function(x) inDat[x])
  contrAve <- if(is.matrix(tmp)) colMeans(tmp) else sapply(tmp, mean, na.rm=TRUE)
  normFact <- if(length(refLst) ==1) (regrTo[1]/contrAve[1]) else (regrTo[2] -regrTo[1])/(contrAve[2]-contrAve[1])  
  normFact <- c(k=normFact, d= if(length(refLst) ==1) 0 else regrTo[1] -normFact*contrAve[1])
  inDat*normFact[1] + normFact[2] }
    

#' checkGrpOrder
#'
#' \code{checkGrpOrder} tests each line of 'x' if expected order appears. 
#' Used for comparing groups of measures with expected profile (simply by mataching expected order)
#' @param x matrix or data.frame
#' @param rankExp (numeric) expected order for values in columns, default 'rankExp' =1:ncol(x)
#' @param revRank (logical) if 'revRank'=TRUE, the initial ranks & reversed ranks will be tested
#' @param silent (logical) suppress messages
#' @param debug (logical) display additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return vector of logical values 
#' @seealso \code{\link[wrMisc]{checkGrpOrderSEM}}
#' @examples
#' set.seed(2005); mat1 <- rbind(matrix(round(runif(40),1),nc=4), rep(1,4))
#' checkGrpOrder(mat1)
#' checkGrpOrder(mat1,c(1,4,3,2))
#' @export
checkGrpOrder <- function(x, rankExp=NULL, revRank=TRUE, silent=FALSE, debug=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom, newNa="checkGrpOrder")
  if(length(x) <1 | length(dim(x)) !=2) stop(" 'x' should be data.frame or matrix of 2 dimensions")
  if(is.null(rankExp)) rankExp <- 1:ncol(x)
  if(length(rankExp) != ncol(x)) stop("Number of elements in 'rankExp' doesn't match number of columns in 'x'")
  ## main
  rankExp <- as.numeric(rankExp)
  out <- if(revRank) {
    apply(x, 1, function(x) {y <- as.numeric(order(x)); identical(rankExp,y) | identical(rev(rankExp),y)})
  } else {
    apply(x, 1, function(x) identical(rankExp, as.numeric(order(x))) )}
  out }
  

#' Transform Numeric Values To Color-Gradient
#'
#' This function helps making color-gradients for plotting a numerical variable.
#' Note : RColorBrewer palettes were not integrated here/yet.
#'
#' @param x (character) color input
#' @param gradTy (character) type of gradeint may be 'rainbow', 'heat.colors', 'terrain.colors', 'topo.colors', 'cm.colors', 'hcl.colors', 'grey.colors', 'gray.colorsW' or 'logGray'
#' @param nStartOmit (integer) omit n steps from begining of gradient range
#' @param nEndOmit (integer or "sep") omit n steps from end of gradient range, if \code{nEndOmit="sep"} 20 percent of initial grades will be removed to obtain 'separate' ie non-closing color-circles/gradients eg with \code{rainbow}
#' @param revCol (logical) reverse order
#' @param alpha (numeric) optional transparency value (1 for no transparency, 0 for complete opaqueness)
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a character vector (of same length as x) with color encoding
#' @seealso \code{\link[base]{cut}} 
#' @examples
#' set.seed(2015); dat1 <- round(runif(15),2)
#' plot(1:15,dat1,pch=16,cex=2,col=colorAccording2(dat1))
#' plot(1:15,dat1,pch=16,cex=2,col=colorAccording2(dat1,nStartO=0,nEndO=4,revCol=TRUE))
#' plot(1:9,pch=3)
#' points(1:9,1:9,col=transpGraySca(st=0,en=0.8,nSt=9,trans=0.3),cex=42,pch=16)
#' @export
colorAccording2 <- function(x, gradTy="rainbow", nStartOmit=NULL, nEndOmit=NULL, revCol=FALSE, alpha=1, silent=FALSE, debug=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom, newNa="colorAccording2")
  if(isTRUE(debug)) silent <- FALSE else { debug <- FALSE
    if(!isTRUE(silent)) silent <- FALSE }
  x <- convToNum(x, remove=NULL, sciIncl=TRUE, callFrom=fxNa)
  if(revCol) x <- -1*x
  ind <- round(1 +(.scale01(x) *(sum(!is.na(x)) -1)) )
  stFro <- if(is.null(nStartOmit)) 1 else nStartOmit +1        # + test/convert as integer
  if(identical(nEndOmit,"sep")) nEndOmit <- floor(length(x) *0.2)
  upTo <- sum(!is.na(x)) +stFro -1
  nCol <- sum(!is.na(x)) +stFro-1 +if(is.null(nEndOmit)) 0 else nEndOmit   # + test/convert as integer
  switch(gradTy,
    rainbow=grDevices::rainbow(nCol, alpha=alpha)[stFro:upTo][ind],
    heat.colors=grDevices::heat.colors(nCol, alpha=alpha)[stFro:upTo][ind],
    terrain.colors=grDevices::terrain.colors(nCol, alpha=alpha)[stFro:upTo][ind],
    topo.colors=grDevices::topo.colors(nCol, alpha=alpha)[stFro:upTo][ind],
    cm.colors=grDevices::cm.colors(nCol, alpha=alpha)[stFro:upTo][ind],
    hcl.colors=grDevices::hcl.colors(nCol, alpha=alpha)[stFro:upTo][ind],
    gray.colors=grDevices::grey.colors(nCol, start=0.2, end=0.9, alpha=alpha)[stFro:upTo][ind],
    gray.colorsW=transpGraySca(startGray=0.9, endGrey=0.2, nSteps=5, transp=alpha) [stFro:upTo][ind],
    logGray=transpGraySca(0, 1, nSteps=sum(!is.na(x)), transp=alpha)[stFro:upTo][ind]
    ) }
    
#' Automatic choice of colors
#'
#' This function allows to do automatic choice of colors: if single-> grey, if few -> RColorBrewer, if many : gradient green -> grey/red
#' 
#' @param nGrp (numeric vector) main input
#' @param paired (logical)
#' @param alph (numeric vector) 
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced#' 
#' @return This function returns a character vector with color codes
#' @seealso  \code{\link[grDevices]{rgb}}; \code{\link{colorAccording2}}
#' @examples
#' .chooseGrpCol(4)	
#' @export
.chooseGrpCol <- function(nGrp, paired=FALSE, alph=0.2, silent=FALSE, debug=FALSE, callFrom=NULL) {
  ## automatic choice of colors: if single-> grey, if few -> RColorBrewer, if many : gradient green -> grey/red
  ## if max groups =8 (or max 2x6 paired groups) choose contrasting colors from RColorBrewer
  ## requires (RColorBrewer)
  fxNa <- .composeCallName(callFrom, newNa=".chooseGrpCol")
  if(isTRUE(debug)) silent <- FALSE else { debug <- FALSE
    if(!isTRUE(silent)) silent <- FALSE }
  finCol <- grDevices::rgb(0.2, 0.2, 0.2, alpha=alph)                                  # dark grey
  if(nGrp >1 && nGrp < (9 +4*paired)) {
    if(requireNamespace("RColorBrewer", quietly=TRUE)) {
      finCol <- if(isTRUE(paired)) RColorBrewer::brewer.pal(12,"Paired")[c(5:6,1:4,7:12)] else RColorBrewer::brewer.pal(8,"Accent")[c(6,5,1:3,4,7:8)]   # red, blue,green,purple,orange,yellow,brown,grey
    } else if(!silent) message(fxNa,"Please install package 'RColorBrewer' from CRAN for nicer color-schemes")
  } else if(nGrp > (8 +4*paired)) {      # gradient bright green -> dark olive -> bright red
    finCol <- grDevices::rgb(seq(0, 0.9, length.out=nGrp), seq(0.9, 0, length.out=nGrp), 0.2, alpha=alph)
  }    
  if(nGrp < length(finCol)) finCol <- finCol[1:nGrp]
  if(alph <1 && alph >0) finCol <- convColorToTransp(finCol, alph=alph)
  finCol }
      

#' Filter for unique elements
#'
#' \code{nonredDataFrame}  filters 'x' (list of char-vectors or char-vector) for elements unique (to 'ref' or if NULL to all 'x') and of character length. 
#' May be used for different 'accession' for same pep sequence (same 'peptide_id').
#' Note : made for treating data.frames, may be slightly slower than matrix equivalent
#' @param dataFr (data.frame) main inpput
#' @param useCol (character,length=2) comlumn names of 'dataFr' to use : 1st value designates where redundant values should be gathered; 2nd value designes column of which information should be concatenated 
#' @param sepCollapse (character) conatenation symbol
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a data.frame of filtered (fewer lines) with additional 2 columns 'nSamePep' (number of redundant entries) and 'concID' (concatenated content)
#' @seealso \code{\link{combineRedBasedOnCol}}, \code{\link{correctToUnique}}, \code{\link[base]{unique}}
#' @examples
#' df1 <- data.frame(cbind(xA=letters[1:5], xB=c("h","h","f","e","f"), xC=LETTERS[1:5])) 
#' nonredDataFrame(df1, useCol=c("xB","xC")) 
#' @export
nonredDataFrame <- function(dataFr, useCol=c(pepID="peptide_id", protID="accession", seq="sequence",mod="modifications"), sepCollapse="//", callFrom=NULL) {
  fxNa <- .composeCallName(callFrom,newNa="nonredDataFrame")
  chColNa <- useCol %in% colnames(dataFr)
  if(any(!chColNa)) message(" TROUBLE AHEAD : can't find ",pasteC(useCol[which(!chColNa)]))
  if(!is.data.frame(dataFr)) dataFr <- as.data.frame(dataFr,stringsAsFactors=FALSE)
  i <- which(colnames(dataFr)==useCol[1]); dataFr[,i] <- as.character(dataFr[,i])
  i <- which(colnames(dataFr)==useCol[2]); dataFr[,i] <- as.character(dataFr[,i])
  dataFr <- cbind(dataFr, nSamePep=1, concID="")
  class(dataFr[,ncol(dataFr)-1]) <- "integer"
  class(dataFr[,ncol(dataFr)]) <- "character"
  dataFr[,ncol(dataFr)] <- as.character(dataFr[,useCol[2]])
  dupL <- duplicated(dataFr[,useCol[1]], fromLast=TRUE)                        # search in col with petide_id
  if(any(dupL)) {
    dupB <- duplicated(dataFr[,useCol[1]], fromLast=FALSE)
    firOfRep <- which(dupL & !dupB)
    anyDup <- which(dupL | dupB)
    out <- matrix(unlist(tapply(dataFr[anyDup,useCol[2]],dataFr[anyDup,useCol[1]], function(x) c(length(x),paste(x,collapse=sepCollapse)))),
      ncol=2, byrow=TRUE, dimnames=list(dataFr[firOfRep,useCol[1]],c("n","protIDs")))  
    dataFr <- dataFr[-1*which(dupB),]                          # remove lines of mult repeated 
     replLi <- match(rownames(out),as.character(dataFr[,useCol[1]]) ) 
    dataFr[match(rownames(out),as.character(dataFr[,useCol[1]]) ),ncol(dataFr)-(1:0)] <- out 
    class(dataFr[,ncol(dataFr)-1]) <- "integer" }
  dataFr }    
     

#' Cut numeric vector to n groups (ie convert to factor) 
#'
#' This function is a more elaborate version of \code{\link[base]{cut}} for cutting a the content of a 
#' numeric vector '\code{x}' into a given number of groups, taken from the length of '\code{lev}'.
#' Besides, this function provides the group borders/limits for convention use with legends.
#'
#' @param x numeric vector
#' @param lev (character or numeric), the length of this argument tells the number of groups to be used for cutting
#' @param NAuse (logical) include NAs as separate group
#' @param callFrom (character) for better tracking of use of functions
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allows easier tracking of messages produced
#' @return This function returns a list with \code{$grouped} telling which element of '\code{x}' goes in which group and \code{$legTxt} with gourp-borders for convenient use with legends
#' @seealso \code{\link[base]{cut}}
#' @examples
#' set.seed(2019); dat <- runif(30) +(1:30)/2
#' cutToNgrp(dat,1:5)
#' plot(dat,col=(1:5)[as.numeric(cutToNgrp(dat,1:5)$grouped)])
#' @export
cutToNgrp <- function (x, lev, NAuse=FALSE, silent=FALSE, debug=FALSE, callFrom=NULL) {
  ##
  fxNa <- .composeCallName(callFrom,newNa="cutToNgrp")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
   ra <- range(x, na.rm=TRUE)
  if(length(lev) <2) message(fxNa," 'lev' indicates to make only ",length(lev)," groups,  do you really want this ?")
  if(diff(ra) > 0) {
    ndig <- round(1.5 +log(length(lev)))
    br <- signif(seq(ra[1] -diff(ra)*0.001, ra[2]+diff(ra)*0.001, length.out=length(lev) +1), digits=ndig)
    if(max(br) < ra[2]) br[length(br)] <- signif(br[length(br)] +(br[2] -br[1])/12, digits=ndig) 
    chBr <- duplicated(br, fromLast=FALSE)
    if(any(chBr)) {
      br <- signif(seq(ra[1] -diff(ra)*0.001, ra[2], length.out=length(lev) +1), digits=ndig +1)
      chBr <- duplicated(br,fromLast=FALSE)
      if(any(chBr)) {
        whRep <- !chBr
        br[which(chBr)] <- br[which(chBr)] +ra[1]/1e6 }}  
    intGrp <- cut(x, br)
  } else intGrp <- rep(1, length(x))
  legTx <- matrix(sub("\\]", "", sub("\\(","", unlist(sapply(levels(intGrp), strsplit, ",")))), ncol=2, byrow=TRUE)
  lowVa <- signif(ra[1], 1)
  if(lowVa <= ra[1]) legTx[1,1] <- lowVa
  legTx <- matrix(signif(as.numeric(legTx), digits=round(0.5+log(length(lev)))), ncol=2, dimnames=list(lev,c("from","to")))
  if(NAuse) if (any(is.na(x))) {
    levels(intGrp) <- c(levels(intGrp), "NA")
    intGrp[which(is.na(intGrp))] <- "NA"
    legTx <- rbind(legTx, c("NA", "NA"))
  }
  list(grouped=intGrp, legTxt=legTx)
}
  

#' Remove or rename enumerator tag/name (or remove entire enumerator) from tailing enumerators
#'
#' @description
#' This function allows indentifying, removing or renaming enumerator tag/name (or remove entire enumerator) from tailing enumerators (eg 'abc_No1' to 'abc_1').
#' A panel of potential candidates as combination of separator-symbols and separtor text/words will be tested to find if one matches all data.
#' In case the main input is a matrix, all columns will be tested independently to find the first column where one specific combination of separator-symbols and separtor text/words is found.
#' Several options exist for the output, the combination of separator-symbols and separtor text/words may be included, too.
#'
#' @details
#' 
#' In case only digit-enumerators are present (ie, without repetitive text), one has to use \code{incl="rmEnum"} to remove terminal enumerators. 
#' This will work, only when all items do contain terminal digits.
#' 
#' Please note, that checking a variety of different separator text-word and separator-symbols may give an important number of combinations to check.
#' In particular, when automatic trimming of separator text-words is added (eg \code{incl="trim2"}), the complexity of associated searches increases quickly.
#' Thus, with large data-sets restricting the content of the arguments \code{nameEnum}, \code{sepEnum} and (in particular) \code{newSep} to the most probable terms/options
#' is suggested to help reducing demands on memory and CPU.
#'
#' In case the input \code{dat} is a matrix and multiple different numerator-types are found, only the first colum (from the left) will be treated.
#' If you which to remove/subsitute mutiple types of enumerators the function \code{rmEnumeratorName} must be run independently, see last example below.
#'
#' @param dat (character vecor or matrix) main input
#' @param nameEnum (character) potential enumerator-names
#' @param sepEnum (character)  potential separators for enumerator-names
#' @param newSep (character) potential enumerator-names
#' @param incl (character) options to include further variants of the enumerator-names, 
#'   use \code{"rmEnum"} for completely removing enumerator tag/name and digits for different options of trimming names/tags from \code{nameEnum};
#'   or one may use \code{anyCase}, 
#'   \code{trim3} (trimming down to max 3 letters),
#'   \code{trim2} (trimming to max 2 letters) or  \code{trim1} (trimming down to single letter); 
#'   \code{trim0} works like \code{trim1} but also includes ' ', ie no enumerator tag/name in front of the digit(s)  
#' @param silent (logical) suppress messages
#' @param debug (logical) display additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a corrected vector (or matrix), or a list if \code{incl="rmEnumL"} containing $dat (corrected data),
#'   $pattern (the combination of separator-symbols and separtor text/words found), and if input is matrix $column (which column of the input was identified and treated)
#' @seealso when the exact pattern is known \code{\link[base]{grep}} and \code{sub} may allow direct manipulations much faster
#' @examples
#' xv <- c("hg_1","hjRe2_2","hk-33")
#' rmEnumeratorName(xv)
#' rmEnumeratorName(xv, incl="rmEnum")
#' 
#' xx <- c("hg_Re1","hjRe2_Re2","hk-Re3_Re33")
#' rmEnumeratorName(xx)
#' rmEnumeratorName(xx, newSep="--")
#' rmEnumeratorName(xx, incl="anyCase")
#'
#' xy <- cbind(a=11:13, b=c("11#11","2_No2","333_samp333"), c=xx)
#' rmEnumeratorName(xy)
#' rmEnumeratorName(xy,incl=c("anyCase","trim2","rmEnumL"))
#'
#' xz <- cbind(a=11:13, b=c("23#11","4#2","567#333"), c=xx)
#' apply(xz, 2, rmEnumeratorName, sepEnum=c("","_"), newSep="_", silent=TRUE)
#'
#' @export
rmEnumeratorName <- function(dat, nameEnum=c("Number","No","#","Replicate","Sample"), sepEnum=c(" ","-","_","/"), newSep="", incl=c("anyCase","trim2"), silent=FALSE, debug=FALSE, callFrom=NULL) {
  ## remove or rename enumerator tag/name (or remove entire enumerator) from tailing enumerators (eg 'abc_No1' to 'abc_1'), only if found present in all instances
  ## dat (character voector or matrix)
  ## return character vector of same length as initial
  ## return-options : 1) repl/no EnumName 2) wo any enum, no digits 3) both

  fxNa <- .composeCallName(callFrom, newNa="rmEnumeratorName")
  if(isTRUE(debug)) silent <- FALSE
  if(!isTRUE(silent)) silent <- FALSE

  out <- dat
  datOK <- length(dat) >0
  if(datOK) {
    ch1 <- if(length(dim(dat)) >0) colSums(!matrix(grepl("..[[:digit:]]+$", as.matrix(dat)), ncol=ncol(dat))) <1  else all(grepl("..[[:digit:]]+$", as.character(dat)))
    datOK <- any(ch1, na.rm=TRUE)
    useCol <- if(length(dim(dat)) >0) which(ch1) else NULL }

  if(length(nameEnum) >0) { nameEnum <- naOmit(nameEnum)}
  if(length(nameEnum) <1) { nameEnum <- c("Number","Replicate")
    if(!silent) message(fxNa,"Empty or Invalid entry for 'sepEnum', setting to default") }
  if(length(sepEnum) >0) { sepEnum <- naOmit(sepEnum)
    chDu <- duplicated(nameEnum)
    if(any(chDu)) nameEnum <- nameEnum[which(!chDu)]
  }
  if(length(sepEnum) <1) { sepEnum <- c(" ","-","_")
    if(!silent) message(fxNa,"Empty or Invalid entry for 'sepEnum', setting to default") }
  if(length(incl) <1)  incl <- NA
  if(length(newSep) >0) { newSep <- naOmit(newSep)[1]}
  if(length(newSep) <1) { newSep <- ""
    if(!silent) message(fxNa,"Empty or Invalid entry for 'newSep', setting to default") }


  if(datOK) {
    ## prepare enumerator-patterns to test
    chDu <- duplicated(nameEnum)
    if(any(chDu)) nameEnum <- nameEnum[which(!chDu)]
    if("anyCase" %in% incl) nameEnum <- unique(c(nameEnum, tolower(nameEnum), toupper(nameEnum)))
    if("trim3" %in% incl) {tmp <- 3:max(nchar(nameEnum)); nameEnum <- unique(substring(rep(nameEnum, each=length(tmp)), 1, rep(3:max(nchar(nameEnum)), length(nameEnum)))) } else {
      if("trim2" %in% incl) {tmp <- 2:max(nchar(nameEnum)); nameEnum <- unique(substring(rep(nameEnum, each=length(tmp)), 1, rep(2:max(nchar(nameEnum)), length(nameEnum)))) } else {
        if("trim1" %in% incl || "trim0" %in% incl) {tmp <- 1:max(nchar(nameEnum)); nameEnum <- unique(substring(rep(nameEnum, each=length(tmp)), 1, rep(1:max(nchar(nameEnum)), length(nameEnum)))) }}}
    if("trim0" %in% incl) {nameEnum <- unique(c(nameEnum, ""))
      sepEnum <- unique(c(sepEnum, ""))}

    chDu <- duplicated(sepEnum)
    if(any(chDu)) sepEnum <- sepEnum[which(!chDu)]
    nameEnum <- paste0(rep(sepEnum, length(nameEnum)), rep(nameEnum, each=length(sepEnum)),"[[:digit:]]+$")
    if(debug) message(fxNa,"Ready to test ",length(nameEnum)," types of enumerators")


    ## main
    if(length(nameEnum) >1) nameEnum <- nameEnum[order(nchar(nameEnum), decreasing=TRUE)]   # sort to prefer using longest version

    chEnum <- if(length(dim(dat)) ==2) apply(dat[,useCol], 2, function(y) sapply(nameEnum, function(x) all(grepl(x, y)))) else {
      sapply(nameEnum, function(x) all(grepl(x, dat)))}
    if(debug) {message(fxNa,"rEN0 .."); rEN0 <- list(dat=dat,out=out,nameEnum=nameEnum,newSep=newSep,sepEnum=sepEnum,chEnum=chEnum,newSep=newSep)}

    if(any(chEnum)) { nameEnumInd <- which(chEnum, arr.ind=length(dim(dat)) ==2)                 # each hit in new line
      if(debug) {message(fxNa,"rEN1 .."); rEN1 <- list(dat=dat,out=out,nameEnum=nameEnum,nameEnumInd=nameEnumInd,chEnum=chEnum,newSep=newSep)}
      if(length(dim(dat)) >0) {      ## input is matrix
        usePat <- rownames(nameEnumInd)[1]               # the (1st) pattern matching all input
        nameEnumInd <- nameEnumInd[1,]
        enu2 <- sub(usePat,"", dat[,useCol[nameEnumInd[2]]])         # wo nameEnumerator
        curSep <-  sepEnum[1+ ((nameEnumInd[1] -1) %/% length(sepEnum))]
        if(debug) {message(fxNa,"rEN2"); rEN2 <- list()}
        out[,useCol[nameEnumInd[2]]] <- if(length(grep("^rmEnum", incl)) >0) enu2 else {
          paste0(enu2, if(length(newSep)==1) newSep else curSep, substr(dat[,useCol[nameEnumInd[2]]], nchar(enu2) +nchar(usePat) -12, max(nchar(dat[,useCol[nameEnumInd[2]]]))))}
        if(debug && any(grepl(".L$", incl))) message("Matched matrix via column '",useCol[nameEnumInd[2]],"'")
        if(any(grepl(".L$", incl))) out <- list(dat=out, column=useCol[nameEnumInd[2]], pattern=substr(usePat, 1, nchar(usePat) -13))                  # optinal return as list including info which col was modified
      } else {                               ## input is vector
        if(length(nameEnumInd) >1) nameEnumInd <- nameEnumInd[1]
        usePat <- names(nameEnumInd)
        maxNch <- max(nchar(dat), na.rm=TRUE)
        enu2 <- sub(names(nameEnumInd),"", dat)     # wo nameEnumerator
        curSep <- sepEnum[1+ ((nameEnumInd -1) %/% length(sepEnum))]
        if(length(grep("^rmEnum", incl)) >0)  out <- enu2 else {
          out <- if(length(grep("^rmEnum", incl)) >0) enu2 else {
            paste0(enu2, if(length(newSep)==1) newSep else curSep, substr(dat, nchar(enu2) +nchar(usePat) -12, maxNch)) }
        }
        if("all" %in% incl) out <- cbind(ini=dat, new=out)
        if(debug && any(grepl(".L$", incl))) message(fxNa,"Matched vector")
        if(any(grepl(".L$", incl))) out <- list(dat=out, pattern=substr(usePat, 1, nchar(usePat) -13))                  # optinal return as list including info which col was modified
      }

    } else {
      ## check for removing enumerators (absence of add'l text)
      if(nchar(newSep) <1 && length(grep("^rmEnum", incl)) >0) {
        if(debug) {message(fxNa,"rEN4 .."); rEN4 <- list(dat=dat,out=out,nameEnum=nameEnum,newSep=newSep,sepEnum=sepEnum,chEnum=chEnum,newSep=newSep)}
        rmTailEnu <- sub(paste0("(",paste(protectSpecChar(sepEnum),collapse="|"),")", "[[:digit:]]+$"), "", dat)  # allows all combin of enumerators
        ch3 <- nchar(dat) > nchar(rmTailEnu)
        ## option for 'strict' , ie enumerator at all instances ?
        
        if(length(dim(dat)) <2 ) out <- rmTailEnu else {       
          ch4 <- if(length(dim(dat)) >1) colSums(ch3)==nrow(dat) 
          if(any(ch4)) out <- rmTailEnu[,which.min(ch4)]
        } 
      } else if(!silent && identical(out, dat)) message(fxNa,"No conistent enumerator+digit combination found; nothing to do ..")
    } 
       
  } else if(debug) message(fxNa,"Invalid or empty input; nothing to do ..")
  out }
  

#' Multiple Moderated Pairwise t-tests From limma
#'
#' This function runs (selected or all) pairwise combinations of moderated t-tests from package 'limma' on each line of data against.
#' Note: This function requires the package \href{https://bioconductor.org/packages/release/bioc/html/limma.html}{limma} from bioconductor. 
#'    The limma contrast-matrix has to be read by column, the lines in the contrast-matrix containing '+1' will be compared to the '-1' lines, eg grpA-grpB .
#' 
#' @details
#' When multiple pairwise comparisons will be run, first a global linear model will be estimated and the particular pairwise comparisons will then be performed using a contrast-matrix.
#' This process is described with the bioconductor package \href{https://bioconductor.org/packages/release/bioc/html/limma.html}{limma} which is used underneith.
#' 
#' By default (\code{useComparison=NULL}), the first of all possible pairwise comparisons will be run.
#' If you would like to run all comparisons, plase set \code{useComparison="all"}.
#' Besides, it is also possible to custom choose which comparisons should be run (and which order of sample/reference) via the argument \code{useComparison} using
#' the character vector '--' as separator for the two groups of samples (referring to argument \code{grp}) to be compared.
#' This will be interpreted as 'sample--reference' (additional space around the separator, if present, will be removed), thus the second element will be used as reference. 
#' Furthermore, the argument \code{useComparison} may be a matrix of 2 columns (for sample and reference) where each line represents a pairwise comparison that should be run.  
#' 
#' As effort for compatibility to previous versions and compatibility to standard writing in limma the group-separator '-' in \code{useComparison} has limited support : 
#' This (single character) separator will be internally converted to '--' and in absence of '-' in argument \code{grp}, output will have the initial  (single character) separator '-'.
#' 
#' Concerning the separator used when reporting results for pairwise comparisons :
#' If argument \code{grp} contains any '-', comparisons of groups will always (!) get reported using '--' as separator to avoid any confusion.
#' In case \code{useComparison} is a matrix, the reported comparisons will have '-' if \code{grp} does not contain as well any '-', otherwise '--' will be used 
#' (to support compatibility with results from limma).
#' 
#' Please note, that in the output of this function the Benjamini-Hochberg (BH) adjusted p-values are called 'FDR' (see argument \code{addResults}).
#' 
#' the output caontains a list-element named \code{setup} summarizing the global setup as captured (names of groups of samples), including the names of the pairwise combinations that were tested.
#'
#' @param dat matrix or data.frame with rows for multiple (independent) tests, use ONLY with 2 groups; assumed as log2-data !!!
#' @param grp (factor) describes column-relationship of 'dat'   (1st factor is considered as reference -> orientation of M-values !!)
#' @param useComparison (numeric, character or matrix) optional way to indicate which pairwise comparisons should be performed :
#'      if \code{useComparison="all"} all pairwise comparisons possible will be performed,
#'      otherise pairwise comparisons may be chosen using indexes (the ith instance of all possible pairwise comparisons);
#'      a character vector citing the concatenated levels of  argument \code{grp}; may be used (in this case it may be necessary to specify the separator used in argument argument \code{sep} ;
#'      please note, it is important to check that a custom selected separator \code{sep} does not occur inside the groups from \code{grp});
#'      finally, the argument \code{useComparison} may be or as matrix of 2 columns where each row specifies a different comparison, 
#'      the matrix mat contain the names of the levels of argument \code{grp} or an index relative to the levels of argument \code{grp};
#'      In all cases where 2 groups are specified this will be read as 'sample-reference' - thus, the 2nd entity will be taken as reference;
#'      Of course this separator should not occur in 'grp' (if present it will be automatically repaced by '__' in 'grp' ).
#' @param limmaOutput (logical) return full (or extended) MArrayLM-object from limma or 'FAlSE' for only the (uncorrected) p.values
#' @param addResults (character) types of results to add besides basic limma-output, data are assumed to be log2 ! (eg "lfdr" using fdrtool-package, "FDR" or "BH" for BH-FDR, "BY" for BY-FDR, 
#'   "bonferroni" for Bonferroni-correction, "qValue" for lfdr by qvalue, "Mval", "means" or "nonMod" for non-moderated test and he equivaent all (other) multiple testing corrections chosen here)
#' @param testOrientation (character) for one-sided test (">","greater" or "<","less"), NOTE : 2nd grp is considered control/reference, '<' will identify grp1 < grp2
#' @param sep (\code{NULL]} or character of length=1) optional custom choice for separator for column-names of pairwise comparisons, otherwise determined as character(-set) not occurring in \code{grp}
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a limma-type MA-object (which can be handeled like a list), or when problems are encountered \code{NULL}
#' @seealso \code{\link{moderTest2grp}} for simplified single comparisons (with fewer options/flexibility), \code{\link[limma]{lmFit}} and the \code{eBayes}-family of functions in package \href{https://bioconductor.org/packages/release/bioc/html/limma.html}{limma}
#' @examples
#' grp3 <- factor(rep(LETTERS[c(3,1,4)],c(2,3,3)))
#' set.seed(2017); t8 <- matrix(round(rnorm(208*8,10,0.4),2), ncol=8,
#'   dimnames=list(paste(letters[],rep(1:8,each=26),sep=""), paste0(grp3, c(1:2,1:3,1:3))))
#' t8[3:6,1:2] <- t8[3:6,1:2] +3                    # augment lines 3:6 (c-f) 
#' t8[5:8,c(1:2,6:8)] <- t8[5:8,c(1:2,6:8)] -1.5    # lower lines 
#' t8[6:7,3:5] <- t8[6:7,3:5] +2.2                  # augment lines 
#' ## expect to find C/A in c,d,g, (h)
#' ## expect to find C/D in c,d,e,f
#' ## expect to find A/D in f,g,(h) 
#' test8 <- moderTestXgrp(t8, grp3) 
#' 
#' ## Custom choice of what to compare
#' test8b <- moderTestXgrp(t8, grp3, useComparison=c("D-A","C-A")) 
#' head(test8b$FDR)
#' head(test8b$Mval)
#' 
#' ## Custom choice as matrix
#' comp4 <- matrix(LETTERS[c(4,3,3,1)], ncol=2)
#' test4 <- moderTestXgrp(t8, grp3, useComparison=comp4)
#' 
#' ## Names of pairwise comparisons performed
#' test4$setup$concat
#'
#' ## One can also use functions from package limma to see more
#' library(limma)
#' topTable(test8b, n=5)
#' @export
moderTestXgrp <- function(dat, grp,  useComparison=NULL, limmaOutput=TRUE, addResults=c("lfdr","FDR","Mval","means"), testOrientation="=", sep=NULL, silent=FALSE, debug=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom, newNa="moderTestXgrp")
  if(isTRUE(debug)) silent <- FALSE else { debug <- FALSE
    if(!isTRUE(silent)) silent <- FALSE }
  if(!isFALSE(limmaOutput)) limmaOutput <- TRUE
  runTest <- TRUE
  pwIndex <- sep1 <- indX <- useComparisonNa <- NULL
  .extrSep2 <- function(comb , indiv) {     ## extract separator by striping indiv elements (iniv) from combined pairwise names (comb)  .. move to wrMisc ??
    fx2 <- function(x, y) sub(paste0(protectSpecChar(x),"$"),"", sub(paste0("^",protectSpecChar(x)),"", y))   # remove x from head and tail   (eg rm C from end)
    for(i in indiv) comb <- fx2(i, comb)
    unique(comb[which(nchar(comb) >0)])   # will/might return NULL when sep==""
  }
  
  if(any(length(dat) <1, length(dim(dat)) !=2, dim(dat) < c(1,3))) { runTest <- FALSE
    warning(fxNa,"Invalid argument 'dat'; must be matrix or data.frame with min 1 lines and 3 columns") }
  if(runTest) {  
    if(length(grp) != ncol(dat)) { runTest <- FALSE
      warning(fxNa,"Check parameters: Number of columns of 'dat' doesn't match to length of 'grp'") }}  
  if(runTest) {  
    if(!is.factor(grp)) grp <- as.factor(grp)
    if(length(levels(grp)) <2) stop("Need at least 2 groups in argument 'grp'")
    ## check packages dependencies 
    packages <- c("limma", "fdrtool", "qvalue")
    checkPkg <- function(pkg) requireNamespace(pkg, quietly=TRUE)
    checkPkgs <- sapply(packages, checkPkg)
    if(!checkPkgs[1]) { runTest <- FALSE; message(fxNa,"NOTE: Package 'limma' not found ! Unable to run tests. Please install first from Bioconductor")} } 
  if(debug) {message(fxNa,"mTX0"); mTX0 <- list(dat=dat,grp=grp,useComparison=useComparison,limmaOutput=limmaOutput,addResults=addResults,testOrientation=testOrientation,runTest=runTest,sep=sep)}
    
  if(runTest && requireNamespace("limma")) {
    if(limmaOutput && length(addResults) >0) if("all" %in% addResults) addResults <- c("BH", "BY","lfdr","qValue","bonferroni","Mval","means","nonMod")
    if(!checkPkgs[2] && any("lfdr" %in% tolower(addResults))) {
      if(!silent) message(fxNa,"Package 'fdrtool' not found, omitting .. Please install from CRAN for enabeling 'lfdr'")
      addResults <- addResults[-which(tolower(addResults) %in% "lfdr")] }
    if(!checkPkgs[3] && any(c("qvalue","qval") %in% tolower(addResults))) {
      if(!silent) message(fxNa,"Package 'qvalue' not found, omitting .. Please install from Bioconductor for enabeling 'qValue'")
      addResults <- addResults[-which(tolower(addResults) %in% c("qvalue","qval"))] }
    if(debug) {message(fxNa,"mTX00")}
    ## treat non-unique row-names ?

  	if(length(rownames(dat)) > 0 && length(unique(rownames(dat))) < nrow(dat)) {
  	  if(!silent) message(fxNa, "Detected ", nrow(dat) - length(unique(rownames(dat))), " non-unique rownames of 'dat' !")
  	}
    altHyp <- "two.sided"                                             # default, change only if explicit sign recognized
    if(length(testOrientation) <1) testOrientation <- altHyp
    if(testOrientation %in% c("<","less","inf")) altHyp <- "less"           
    if(testOrientation %in% c(">","greater","sup")) altHyp <- "greater"
    
    ## prepare modeling - initial design-matrix 
    datDesign <- stats::model.matrix(~ -1 + grp)                  # can't use directly, need contrasts !!
    colnames(datDesign) <- sub("^grp","", colnames(datDesign))
    if(debug) {message(fxNa,"mTX0a"); mTX0a <- list(dat=dat,useComparison=useComparison,grp=grp,datDesign=datDesign,sep=sep)}    # ,compNames=compNames

    ## option 1: no useComparison (NULL) => do all pw combin as before
    ## option 2: useComparison as char vector (using '--' as sep, need to check if index)   
    ## option 3: useComparison as 2-dim (numeric) matrix (need to check if index, indexes should refer to unique(gr) and NOT to levels(gr) !), as from wrProteo

    ## note : sep .. as user-provided; sep1 .. finally used (en from automat determin)    
    if(length(sep) >1) {
      message(fxNa,"Expecting argument 'sep' to be of length=1, trimming to 1st (not NA and min 1 character long)")
      sep <- sep[which(!is.na(sep) & nchar(sep) >0)]
      if(length(sep) >1) sep <- sep[1]
    }
    ## check & convert useComparison
    # useComparison=NULL #  => 1st comparison # if(identical("all",useComparison)) useComparison <- NULL

    chComp <- convPairwiseSetup(useComparison, grp=grp, experSetup=NULL, sep=sep, silent=silent, debug=debug, callFrom=fxNa)
    pwIndex <- chComp$pwIndex
    chComp$grp <- grp
  	names(grp) <- colnames(dat)

    concatNa <- if(length(sep)==0) chComp$concat else { paste(chComp$pwGrpNa[,1], chComp$pwGrpNa[,2], sep=sep)	}
	
    pwGrpNa <- chComp$pwGrpNa
    sep1 <- chComp$sep
    grpNa <- chComp$grpNa
    ## now pwIndex (matrix of index for question-parts) is finished           
    if(debug) {message(fxNa,"mTX2a"); mTX2a <- list(dat=dat,grp=grp,useComparison=useComparison,sep=sep,sep1=sep1,limmaOutput=limmaOutput,addResults=addResults,testOrientation=testOrientation,runTest=runTest,datDesign=datDesign,pwIndex=pwIndex,chComp=chComp,grpNa=grpNa, indX=indX) }   #concatNa=concatNa,
    convSingSep <- FALSE                  # initialize
     
    if(debug) {message(fxNa,"mTX3"); mTX3 <- list(dat=dat,grp=grp,useComparison=useComparison,sep1=sep1,limmaOutput=limmaOutput,addResults=addResults,testOrientation=testOrientation,runTest=runTest,datDesign=datDesign,pwIndex=pwIndex,pwGrpNa=pwGrpNa,grpNa=grpNa )}
  }
    
  ## main testing
  if(runTest && requireNamespace("limma") && length(pwIndex) >0 && length(dat) >0 && all(!is.na(pwIndex))) {
    ## contrast matrix
    ## see eg   https://support.bioconductor.org/p/57268/; https://www.biostars.org/p/157068/
  	contr.matr <- matrix(0, nrow=length(levels(grp)), ncol=nrow(pwIndex), 
  		dimnames=list(levels(grp), rownames(pwIndex)))
  	for(j in 1:nrow(pwIndex)) contr.matr[pwIndex[j, ], j] <- c(1,-1)
  	if(debug) { message(fxNa, "mTX3") }
    globFilt <- 1:nrow(dat)                                       # so far apply testing to all lines
    
    ## main
    fit0 <- try(limma::lmFit(dat[globFilt,], datDesign), silent=TRUE)          # testing part 1
    if(inherits(fit0, "try-error")) { runTest <- FALSE
       warning(fxNa,"Problem running lmFit(), unable to run tests; check if package 'limma' is properly installed !")}
    if(debug) {message(fxNa,"mTX4"); mTX4 <- list(dat=dat,fit0=fit0,datDesign=datDesign,grp=grp,useComparison=useComparison,limmaOutput=limmaOutput,addResults=addResults,testOrientation=testOrientation,runTest=runTest,pwIndex=pwIndex, contr.matr=contr.matr,pwGrpNa=pwGrpNa) }       
  } else runTest <- FALSE
  
  if(runTest && requireNamespace("limma")) {
    fit1 <- limma::eBayes(limma::contrasts.fit(fit0, contrasts=contr.matr))  # variant to run all contrasts at same time
    if(debug) {message(fxNa,"mTX4b"); mTX4b <- list(dat=dat,fit0=fit0,fit1=fit1,datDesign=datDesign,grp=grp,useComparison=useComparison,limmaOutput=limmaOutput,addResults=addResults,testOrientation=testOrientation,runTest=runTest, contr.matr=contr.matr,pwIndex=pwIndex,pwGrpNa=pwGrpNa,grpNa=grpNa,sep1=sep1,altHyp=altHyp) } # concatNa=concatNa,      
    ## note rownames(fit1$contrasts) may not be sufficient (- in case of 'A-B', 'B-A')
    #concatNa <- paste(useComparisonNa[,1],useComparisonNa[,2], sep=sep1)
      #if(debug) message(fxNa," Note: Could not find names of (multiple) comparisons .. reconstructing") }
    
    fit1$means <- rowGrpMeans(dat, grp)  
    
    ## separate running of contrasts, like gxTools, need then to extract & combine all pValues
    chNA <- colSums(is.na(fit1$p.value))
    if(any(chNA==nrow(dat))) fit1$p.value <- fit1$p.value[,-1*which(chNA==nrow(dat))]
    ## modif to adjust for different H0  (29mar18)
    tx <- c("testing alternative hypothesis: true difference in means is "," than 0 (ie focus on "," results with A ",altHyp," than B)")
    if(identical(altHyp,"greater")) {
      ch <- fit1$means[,1] > fit1$means[,2]
      if(!silent) message(fxNa,tx[1],altHyp,tx[2],sum(ch),tx[3:5])
      if(any(ch)) fit1$p.value[which(ch),] <- fit1$p.value[which(ch),]/2
      if(any(!ch)) fit1$p.value[which(!ch),] <- 1- fit1$p.value[which(!ch),]/2        # !(A > B)  .. A <= B
    }
    if(identical(altHyp,"less")){
      ch <- fit1$means[,2] > fit1$means[,1]
      if(!silent) message(fxNa,tx[1],altHyp,tx[2],sum(ch),tx[3:5])
      if(any(ch)) fit1$p.value[which(ch),] <- fit1$p.value[which(ch),]/2
      if(any(!ch)) fit1$p.value[which(!ch),] <- 1- fit1$p.value[which(!ch),]/2        # !(A > B)  .. A <= B
    }
    if(is.null(colnames(fit1$t))) colnames(fit1$t) <- concatNa
    if(is.null(colnames(fit1$p.value))) colnames(fit1$p.value) <- concatNa
    if(any(c("qval","qvalue") %in% tolower(addResults)) && !requireNamespace("qvalue") && !silent) message(fxNa,"NOTE: Package 'qvalue' not installed from CRAN, can't calulate ...")
    if(debug) {message(fxNa,"mTX5"); mTX5 <- list(dat=dat,fit0=fit0,fit1=fit1,datDesign=datDesign,grp=grp,useComparison=useComparison,limmaOutput=limmaOutput,addResults=addResults,testOrientation=testOrientation,runTest=runTest,pwIndex=pwIndex, grpNa=grpNa,contr.matr=contr.matr,pwGrpNa=pwGrpNa)  }       

    ## add various multiple testing corrections
    if(!limmaOutput) out <- fit1$p.value[,2] else { out <- fit1
      ## further inspect & correct values of 'addResults' ?
      if("Mval" %in% addResults) { 
        out$Mval <- apply(pwGrpNa,1, function(x) fit1$means[,which(colnames(fit1$means)==x[1])] - fit1$means[,which(colnames(fit1$means)==x[2])]) }      
      if(any(c("FDR","BH") %in% toupper(addResults))) out$FDR <- if(is.matrix(out$p.value)) {
        apply(out$p.value, 2, stats::p.adjust, meth="BH")} else stats::p.adjust(out$p.value, meth="BH")
      if("BY" %in% toupper(addResults)) out$BY <- if(is.matrix(out$p.value)) {
        apply(out$p.value, 2, stats::p.adjust, meth="BY")} else stats::p.adjust(out$p.value, meth="BY")
      if("lfdr" %in% tolower(addResults) && requireNamespace("fdrtool")) {out$lfdr <- if(is.matrix(out$p.value)) {
        suppressWarnings(apply(out$p.value, 2, pVal2lfdr))} else suppressWarnings(pVal2lfdr(out$p.value)) }    
      if(any(c("qval","qvalue") %in% tolower(addResults)) && requireNamespace("qvalue")) { out$qVal <- if(is.matrix(out$p.value)) {
        try(apply(out$p.value, 2, function(x) qvalue::qvalue(x,lfdr.out=TRUE)$lfdr), silent=TRUE)} else try(qvalue::qvalue(out$p.value,lfdr.out=TRUE)$lfdr, silent=TRUE) 
        if(inherits(out$qVal, "try-error")) { 
          if(!silent) message(fxNa," Problem with pi0 estimation, setting pi0=1 like BH-FDR")
          out$qVal <- if(is.matrix(out$p.value)) { apply(out$p.value, 2, function(x) qvalue::qvalue(x, pi0=1, lfdr.out=TRUE)$lfdr)
            } else qvalue::qvalue(out$p.value, pi0=1, lfdr.out=TRUE)$lfdr }
      }    
      if(any(c("bonferroni","bonf") %in% tolower(addResults))) out$bonf <- if(is.matrix(out$p.value)) {
        apply(out$p.value, 2, stats::p.adjust, meth="bonferroni")} else stats::p.adjust(out$p.value, meth="bonferroni")
    }
    
    if("nonMod" %in% addResults) { leLev <- length(levels(grp))
      grX <- lapply(2:leLev, function(x) which(grp==levels(grp)[x]))
      grX[2:leLev] <- grX[1:(length(levels(grp)) -1)]
      grX[[1]] <- which(grp ==levels(grp)[1])
      names(grX) <- levels(grp)
      altHyp <- testOrientation
      if(altHyp =="=") altHyp <- "two.sided"
      pwIndex <- triCoord(leLev)
      out$nonMod.p <- apply(pwIndex,1,function(y) apply(dat, 1, function(x) stats::t.test(x[which(grp==levels(grp)[y[1]])], x[which(grp==levels(grp)[y[2]])], alternative=altHyp)$p.value) )
      colnames(out$nonMod.p) <- concatNa
      if(any(c("FDR","BH") %in% toupper(addResults))) {
        if(length(dim(out$nonMod.p)) ==2) { out$nonMod.FDR <- apply(out$nonMod.p, 2, stats::p.adjust, method="BH")
        colnames(out$nonMod.FDR)  <- concatNa } else out$nonMod.FDR <- stats::p.adjust(out$nonMod.p, method="BH")
      }
      if(any("BY" %in% toupper(addResults))) {
        if(length(dim(out$nonMod.p)) ==2) { out$nonMod.BY <- apply(out$nonMod.p, 2, stats::p.adjust, method="BY")
        colnames(out$nonMod.BY)  <- concatNa } else out$nonMod.BY <- stats::p.adjust(out$nonMod.p, method="BY")
      }
      if(any("lfdr" %in% tolower(addResults)) && requireNamespace("fdrtool")) {
        if(length(dim(out$nonMod.p)) ==2) { out$nonMod.lfdr <- apply(out$nonMod.p, 2, pVal2lfdr)
        colnames(out$nonMod.lfdr)  <- concatNa } else out$nonMod.lfdr <- suppressWarnings(pVal2lfdr(out$nonMod.p))
      }
      if(any(c("qval","qvalue") %in% tolower(addResults)) && requireNamespace("qvalue")) {
        out$nonMod.qVal <- if(length(dim(out$nonMod.p))==2) try(apply(out$nonMod.p, 2, qvalue::qvalue), silent=TRUE) else try(qvalue::qvalue(out$nonMod.p), silent=TRUE)
        if(inherits(out$nonMod.qVal, "try-error")) { if(!silent) message(fxNa,"Problem with pi0 estimation (non-shrinked p-values) for qValue, setting pi0=1 like BH-FDR")
          out$nonMod.qVal <- if(length(dim(out$nonMod.p))==2) apply(out$nonMod.p, 2, qvalue::qvalue,pi0=1, lfdr.out=TRUE) else qvalue::qvalue(out$nonMod.p,pi0=1, lfdr.out=TRUE)
        }
        if(length(dim(out$nonMod.p)) ==2) colnames(out$nonMod.qVal)  <- concatNa 
      }
      if(any(c("bonferroni","bonf") %in% tolower(addResults))) {
        if(length(dim(out$nonMod.p)) ==2) { out$nonMod.bonf <- apply(out$nonMod.p, 2, stats::p.adjust, method="bonferroni")
        colnames(out$nonMod.bonf)  <- concatNa } else out$nonMod.lfdr <- stats::p.adjust(out$nonMod.p, method="bonferroni")
      }

    }

    out$setup <- chComp  #list(sep=sep1, grpNa=grpNa, pwGrpNa=pwGrpNa, pxIndex=pwIndex)             # document sep used
    if(debug) {message(fxNa,"mTX6"); mTX6 <- list(dat=dat,fit1=fit1,datDesign=datDesign,grp=grp,useComparison=useComparison,out=out,addResults=addResults,limmaOutput=limmaOutput) }       
    
    if(convSingSep) {
      ## reset comparison to initial '-'
      for(i in 1:length(out)) if(length(dim(out[[i]])) >0) { ch1 <- grepl("--", colnames(out[[i]])); if(any(ch1)) colnames(out[[i]])[which(ch1)] <- sub("--","-", colnames(out[[i]])[which(ch1)]) }
    }
    
    ## problem : colnames for Mval,FDR etc not correct
    
    out
  } }  
             

#' Filter matrix to keep only first of repeated lines
#'
#' This function aims to reduce the complexity of a matrix (or data.frame) in case column 'refCol' has multiple lines with same value. 
#' In this case, it reduces the input-data to 1st line of redundant entries and returns a matrix (or data.frame) without lines identified as redundant entries for 'refCol').
#' in sum, this functions works lile useng \code{unique} on a given column, and propagates the same treatment to all other columns.
#' @param dat (matrix or data.frame) main input
#' @param refCol (integer) column number of reference-column
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return matrix (same number of columns as input)
#' @seealso \code{\link{firstOfRepeated}}, \code{\link[base]{unique}}, \code{\link[base]{duplicated}} 
#' @examples
#' (mat1 <- matrix(c(1:6,rep(1:3,1:3)),ncol=2,dimnames=list(letters[1:6],LETTERS[1:2])))
#' firstLineOfDat(mat1)
#' @export
firstLineOfDat <- function(dat, refCol=2, silent=FALSE, debug=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom, newNa="firstLineOfDat")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE

  msg <- " expecting matrix or data.frame with >= 2 columns"
  if(length(dim(dat)) <2) stop(msg)
  if(ncol(dat) < 2) stop(" expecting matrix or data.frame with >= 2 columns")
  if(is.character(refCol)) refCol <- which(refCol==colnames(dat))
  if(refCol > ncol(dat)) {
    if(!silent) message(fxNa," 'refCol' was set too high, reset to last column of 'dat'")
    refCol <- ncol(dat) }
  .getFirst <- function(x) x[1]     # value at 1st position
  useCol <- (1:ncol(dat))[-1*refCol]
  useLi <- tapply(1:nrow(dat), as.factor(dat[,refCol]), .getFirst)
  out <- if(length(useLi) >1) dat[useLi,] else matrix(dat[useLi,], ncol=ncol(dat), dimnames=list(rownames(dat)[useLi],colnames(dat)))
  out }
   

#' Merge selected columns out of 2 matrix or data.frames
#'
#' This function merges selected columns out of 2 matrix or data.frames.
#' 'selCols' will be used to define columns to be used; optionally may be different for 'dat2' : define in 'supCols2'. 
#' Output-cols will get additions specified in newSuff (default '.x' and '.y')
#'
#' @param dat1 matrix or data.frame for fusing
#' @param dat2 matrix or data.frame for fusing
#' @param selCols will be used to define columns to be used; optionally may be different for 'dat2' : define in 'supCols2'
#' @param supCols2 if additional column-names should be extracted form dat2
#' @param byC (character) 'by' value used in \code{\link[base]{merge}}
#' @param useAll (logical) use all lines (will produce NAs when given identifyer not found un 2nd group of data)
#' @param setRownames (logical) if \code{TRUE}, will use values of col used as 'by' as rownames instead of showing as add'l col in output
#' @param newSuff (character) prefix (argument 'suffixes' in \code{merge})
#' @param silent (logical) suppress messages
#' @param debug (logical) display additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a data.frame containing the merged columns
#' @seealso \code{\link[base]{merge}}, merge 3 data.frames using \code{\link{mergeSelCol3}}
#' @examples
#' mat1 <- matrix(c(1:7,letters[1:7],11:17), ncol=3, dimnames=list(LETTERS[1:7],c("x1","x2","x3")))
#' mat2 <- matrix(c(1:6,c("b","a","e","f","g","k"), 31:36),
#'   ncol=3, dimnames=list(LETTERS[11:16],c("y1","x2","x3")))
#' mergeSelCol(mat1, mat2, selC=c("x2","x3"))
#' @export
mergeSelCol <- function(dat1, dat2, selCols, supCols2=NULL, byC=NULL, useAll=FALSE, setRownames=TRUE, newSuff=c(".x",".y"), silent=FALSE, debug=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom,newNa="mergeSelCol")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  msg <- " should be matrix or data.frame (with >0 lines)"
  ms2 <- c(" can't find "," columns specfied in 'selCols' in ")
  if(length(dim(dat1)) !=2) stop(" Argument dat1",msg)
  if(length(dim(dat2)) !=2) stop(" Argument dat2",msg)
  if(is.null(byC)) {byC <- naOmit(colnames(dat1)[match(colnames(dat2),colnames(dat1))])[1]
    message(fxNa,"Automatically selecting '",byC,"' as 'by'")}
  if(length(byC) >1) { byC <- byC[1]; message(fxNa,"reducing 'by' to length 1")}
  if(!(byC %in% colnames(dat1) & byC %in% colnames(dat1))) stop(ms2[1],byC," in both 'dat1' & 'dat2'")
  if(debug) {message(fxNa," byC ",byC)}
  if(length(byC) <1 | is.na(byC)) stop(" Problem with 'by' : no valid column-names")
  if(is.null(supCols2)) supCols2 <- selCols
  if(sum(selCols %in% colnames(dat1), na.rm=TRUE) < length(selCols)) stop(ms2[1],length(selCols),ms2[2],"dat1 !")
  if(sum(supCols2 %in% colnames(dat2), na.rm=TRUE) < length(supCols2)) stop(ms2[1],length(supCols2),ms2[2],"dat2 !")
  sel3 <- which(colnames(dat1) %in% unique(c(byC,selCols)))
  sel4 <- which(colnames(dat2) %in% unique(c(byC,supCols2)))
  if(any(length(sel3) <2, length(sel4) <2)) { out <- dat1; message(fxNa,"Nothing to do")
  } else {
    out <- merge(as.data.frame(dat1[,sel3]),as.data.frame(dat2[,sel4]), by=byC,all=useAll, suffixes=newSuff)}
  if(setRownames) {rownames(out) <- out[,byC]; out <- out[,-1*which(colnames(out) ==byC)]}
  out }

#' mergeSelCol3
#'
#' successive merge of selected columns out of 3 matrix or data.frames.
#' 'selCols' will be used to define columns to be used; optionally may be different for 'dat2' : define in 'supCols2'. 
#' Output-cols will get additions specified in newSuff (default '.x' and '.y')
#'
#' @param dat1 matrix or data.frame for fusing
#' @param dat2 matrix or data.frame for fusing
#' @param dat3 matrix or data.frame for fusing
#' @param selCols will be used to define columns to be used; optionally may be different for 'dat2' : define in 'supCols2'
#' @param supCols2 if additional column-names should be extracted form dat2
#' @param supCols3 if additional column-names should be extracted form dat3
#' @param byC (character) 'by' value used in \code{\link[base]{merge}}
#' @param useAll (logical) use all lines (will produce NAs when given identifyer not found un 2nd group of data)
#' @param setRownames if \code{TRUE}, will use values of col used as 'by' as rownames instead of showing as add'l col in output
#' @param newSuff (character) prefix (argument 'suffixes' in \code{merge})
#' @param silent (logical) suppress messages
#' @param debug (logical) display additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a data.frame containing the merged columns
#' @seealso \code{\link[base]{merge}}, \code{\link{mergeSelCol}}
#' @examples
#' mat1 <- matrix(c(1:7,letters[1:7],11:17),ncol=3,dimnames=list(LETTERS[1:7],c("x1","x2","x3")))
#' mat2 <- matrix(c(1:6,c("b","a","e","f","g","k"),31:36), ncol=3,
#'   dimnames=list(LETTERS[11:16],c("y1","x2","x3")))
#' mat3 <- matrix(c(1:6,c("c","a","e","b","g","k"),51:56), ncol=3,
#'   dimnames=list(LETTERS[11:16],c("z1","x2","x3")))
#' mergeSelCol3(mat1, mat2, mat3, selC=c("x2","x3"))
#' @export
mergeSelCol3 <- function(dat1, dat2, dat3, selCols, supCols2=NULL, supCols3=NULL, byC=NULL, useAll=FALSE, setRownames=TRUE, newSuff=c(".x",".y",".z"), silent=FALSE, debug=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom, newNa="mergeSelCol3")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  msg <- c(" Argument dat"," should be matrix or data.frame (with >0 lines)")
  ms2 <- c(" Can't find "," columns specfied in 'selCols' in dat")
  if(length(dim(dat1)) !=2) stop(msg[1],"1",msg[2])
  if(length(dim(dat2)) !=2) stop(msg[1],"2",msg[2])
  if(length(dim(dat3)) !=2) stop(msg[1],"3",msg[2])
  allCo <- list(colnames(dat1), colnames(dat2), colnames(dat3))
  if(is.null(byC)) {
    byC <- allCo[[1]][which(allCo[[1]] %in% allCo[[2]] & allCo[[1]] %in% allCo[[3]])[1]]
    message(fxNa,"automatically selecting '",byC,"' as 'by'")}
  if(length(byC) >1) { byC <- byC[1]; message(fxNa,"Reducing 'by' to length 1")}
  if(!(byC %in% colnames(dat1) & byC %in% colnames(dat1))) stop(ms2[1],byC," in both 'dat1' & 'dat2'")
  if(length(byC) <1 | is.na(byC)) stop(" Problem with 'by' : no valid column-names")
  if(is.null(supCols2)) supCols2 <- selCols
  if(is.null(supCols3)) supCols3 <- selCols
  if(sum(selCols %in% allCo[[1]], na.rm=TRUE) < length(selCols)) stop(ms2[1],length(selCols),ms2[2],"1 !")
  if(sum(supCols2 %in% allCo[[2]], na.rm=TRUE) < length(supCols2)) stop(ms2[1],length(supCols2),ms2[2],"2 !")
  if(sum(supCols3 %in% allCo[[3]], na.rm=TRUE) < length(supCols3)) stop(ms2[1],length(supCols3),ms2[2],"3 !")
  sel0 <- list(which(allCo[[1]] %in% unique(c(byC,selCols))), which(allCo[[2]] %in% unique(c(byC,supCols2))),
    which(allCo[[3]] %in% unique(c(byC, supCols3))))
  if(any(sapply(sel0,length) <2) ) { out <- dat1; message(fxNa,"Nothing to do ..")
  } else {
    out <- merge(as.data.frame(dat1[,sel0[[1]]]), as.data.frame(dat2[,sel0[[2]]]), by=byC, all=useAll, suffixes=newSuff)
    out <- merge(out, as.data.frame(dat3[,sel0[[3]]]), by=byC, all=useAll)
    co1 <- (1:ncol(out))[-1*unlist(lapply(paste0(newSuff,"$"), grep, colnames(out)))][-1]
    if(length(newSuff) >2) colnames(out)[co1] <- paste0(colnames(out)[co1], newSuff[3])
    }
  if(setRownames) {rownames(out) <- out[,byC]; out <- out[,-1*which(colnames(out) ==byC)]}
  out }
   

#' Protect Special Characters
#'
#' Some characters do have a special meaning when used with regular expressions.
#' This concerns characters like a point, parinthesis, backslash etc.
#' Thus, when using \code{grep} or any related command, shuch special characters must get protected in order to get considered as they are.  
#' 
#' 
#' 
#' 
#' 
#' @param x character vector to be prepared for use in regular expressions
#' @param prot (character) collection of characters that need to be protected 
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a modified character vector
#' @examples
#' aa <- c("abc","abcde","ab.c","ab.c.e","ab*c","ab\\d")
#' grepl("b.", aa)             # all TRUE
#' grepl("b\\.", aa)           # manual prootection
#' grepl(protectSpecChar("b."), aa)
#' @export
protectSpecChar <- function(x, prot=c(".","\\","|","(",")","[","{","^","$","*","+","?"), silent=TRUE, debug=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom, newNa="protectSpecChar")
  if(isTRUE(debug)) silent <- FALSE else { debug <- FALSE
    if(!isTRUE(silent)) silent <- FALSE }
  if(length(x) >0 && length(prot) >0) {
    chSp <- sapply(paste0("\\",prot), function(y) grepl(y, x))
    if(length(dim(chSp)) <2) chSp <- matrix(chSp, ncol=length(prot))
    
    ## special treatment for '\\'
    if("\\" %in% prot & any(grepl("\\\\",x))) {x <- gsub("\\\\","\\\\\\\\",x); prot <- prot[-which(prot %in% "\\")]}
    ## main protecting of sec chars
    if(any(chSp)) {
      z <- if(length(x) >1) which(colSums(chSp) >0) else which(chSp)
      if(debug) {message(fxNa," pSC1"); pSC1 <- list(x=x,prot=prot,chSp=chSp,z=z)}
      for(i in z) {
        if(debug) message(fxNa," i=",i," at ",pasteC(which(chSp[,i])),"  to ",pasteC(x[which(chSp[,i])]) )   
        x[which(chSp[,i])] <- gsub(paste0("\\",prot[i]), paste0("\\\\",prot[i]), x[which(chSp[,i])])
      }
    }
  } else if(debug) message(fxNa," 'x' and 'prot' may not be empty, nothing to do ..")    
  x
}
  

#' Advanced paste-collapse
#'
#' This function is a variant of \code{\link[base]{paste}} for convenient use of paste-collapse and separation of last element to paste (via 'lastCol').
#' This function was mode for more human like enumeriating in output and messages.
#' If multiple arguments are given without names they will all be concatenated, if they contain names lazy evaluation for names will be tried
#' (with preference to longest match to argument names).
#' Note that some special characters (like backslash) may need to be protetected when used with 'collapse' or 'quoteC'.
#' Returns character vector of length 1 (everything pasted together)
#' @param ... (character) main input to be collapsed
#' @param collapse (character,length=1) element to use for collapsing
#' @param lastCol (character) text to use before last item enumerated element
#' @param quoteC character to use for citing with quotations (default "")
#' @return This function returns a character vector of truncated versions of intpup \code{txt}
#' @keywords character
#' @seealso  \code{\link[base]{paste}} for basic paste
#' @examples
#' pasteC(1:4)
#' @export
pasteC <- function(..., collapse=", ", lastCol=" and ", quoteC=""){
  inp <- list(...)
  chArgNa <- c("collapse","lastCol","quoteC")
  chNa0 <- lapply(chArgNa, function(x) rev(substring(x, 1, 2:nchar(x))))
  chNa <- lapply(chNa0, function(x) stats::na.omit(match(x, names(inp)))[1])                 # find location of match to longest variant of argument name
  if(any(!is.na(chNa[[1]]))) {collapse <- inp[[chNa[[1]]]]; inp <- inp[-chNa[[1]]]}
  if(any(!is.na(chNa[[2]]))) {lastCol <- inp[[chNa[[2]]]]; inp <- inp[-chNa[[2]]]}
  if(any(!is.na(chNa[[3]]))) {quoteC <- inp[[chNa[[3]]]]; inp <- inp[-chNa[[3]]]}
  .paQ <- function(x,quo) sapply(x, function(y) paste0(quo, y, quo))
  if(nchar(quoteC) >0) inp <- lapply(inp, .paQ, quoteC)
  if(length(unlist(inp)) >1) {
    out <- if(length(inp) >1) paste(unlist(inp[-length(inp)]), collapse=collapse) else NULL
    tmp <- inp[[length(inp)]]
    tmp <- list(tmp[-length(tmp)], tmp[length(tmp)])
    out <- paste0(paste(c(out, tmp[[1]]), collapse=collapse), lastCol, tmp[[2]])
  } else out <- as.character(unlist(inp))
  out }
  
#' Cut string to get all variants from given start with min and max length
#'
#' This function allows truncating character vector to all variants from given start, with min and optonal max length
#' Used to evaluate argument calls without giving full length of argument
#' 
#' @param txt (character) main input, may be length >1
#' @param startFr (interger) where to start
#' @param minLe (interger) minimum length of output
#' @param maxLe (interger) maximum length of output
#' @param reverse (logical) return longest text-fragments at beginning of vector
#' @return This function returns a character vector
#' @seealso used in \code{\link{pasteC}}; \code{\link[base]{substr}}
#' @examples
#' .cutStr("abcdefg", minLe=2)
#' @export
.cutStr <- function(txt, startFr=1, minLe=1, maxLe=NULL, reverse=TRUE){
  maxChar <- max(nchar(txt), na.rm=TRUE)
  if(length(startFr) >1) {startFr <- startFr[1]}
  if(startFr <1) startFr <- 1 
  if(length(minLe) >1) {minLe <- minLe[1]}
  if(length(maxLe) >0) if(any(is.na(maxLe))) maxLe <- NULL
  if(length(maxLe) >0) maxChar <- min(startFr +maxLe -1, maxChar)
  minStop <- startFr + minLe -1
  out <- if(maxChar - startFr +1 >= minLe && minLe <= minStop) unique(substring(txt, startFr, rep((minStop):maxChar, each=length(txt)))) else NULL 
  nChar <- nchar(out) >= minLe  
  if(any(!nChar)) out <- out[which(nChar)]
  if(isTRUE(reverse) && length(out) >1) out <- rev(out)
  out }


#' Cut string to get all variants from given start with min length, depreciated
#'
#' This function is depreciated, please use \code{/cutStr} instead !
#' This function allows truncating character vector to all variants from given start, with min and optonal max length
#' Used to evaluate argument calls without giving full length of argument
#' 
#' @param txt (character) main input, may be length >1
#' @param startFr (interger) where to start
#' @param minLe (interger) minimum length of output
#' @param reverse (logical) return longest text-fragments at beginning of vector
#' @return This function returns a character vector
#' @seealso \code{\link{pasteC}}; \code{\link[base]{substr}}
#' @examples
#' .seqCutStr("abcdefg", minLe=2)
#' @export
.seqCutStr <- function(txt, startFr=1, minLe=1, reverse=TRUE){
  .Deprecated(new=".cutStr", package="wrMisc", msg="The function .seqCutStr() has been deprecated and replaced by .cutStr()")

  fxNa <- ".seqCutStr" 
  maxChar <- max(nchar(txt), na.rm=TRUE)
  message(fxNa, "Depreciated function, please use .cutStr() instead  (also from package wrMisc)")
  if(length(startFr) >1) {startFr <- startFr[1]}
  out <- unique(substring(txt, 1, rep(max(1, startFr):maxChar, each=length(txt))))   # get all variants from start of min 3 to max 6 letters  (substr() will give only 1st)
  nChar <- nchar(out) >= minLe  
  if(any(!nChar)) out <- out[which(nChar)]
  if(isTRUE(reverse) && length(out) >1) out <- rev(out)
  out }
   

    

#' Transform matrix to non-ambiguous matrix (in respect to given column)
#'
#' \code{nonAmbiguousMat} makes values of matrix 'mat' in col 'byCol' unique. 
#' @param mat numeric or character matrix (or data.frame), column specified by 'byCol' must be/will be used as.numeric, 1st column of 'mat' will be considered like index & used for adding prefix 'nameMod' (unless byCol=1, then 2nd col will be used)
#' @param byCol (character or integer-index) column by which ambiguousity will be tested
#' @param uniqOnly (logical) if =TRUE return unique only, if =FALSE return unique and single representative of non-unique values (with '' added to name), selection of representative of repeated: first (of sorted) or middle if >2 instances
#' @param asList (logical) return result as list
#' @param nameMod (character) prefix added to 1st column of 'mat' (expect 'by') for indicating non-unique/ambiguous values
#' @param debug (logical) for bug-tracking: more/enhanced messages and intermediate objects written in global name-space 
#' @param silent (logical) suppress messages
#' @param callFrom (character) allows easier tracking of message(s) produced
#' @return sorted non-ambigous numeric vector (or list if 'asList'=TRUE and 'uniqOnly'=FALSE)
#' @seealso for non-numeric use \code{\link{firstOfRepeated}} - but 1000x much slower !; \code{\link{get1stOfRepeatedByCol}}
#' @examples
#' set.seed(2017); mat2 <- matrix(c(1:100,round(rnorm(200),2)),ncol=3,
#'   dimnames=list(1:100,LETTERS[1:3]));
#' head(mat2U <- nonAmbiguousMat(mat2,by="B",na="_",uniqO=FALSE),n=15)
#' head(get1stOfRepeatedByCol(mat2,sortB="B",sortS="B"))
#' @export
nonAmbiguousMat <- function(mat, byCol, uniqOnly=FALSE, asList=FALSE, nameMod="amb_", silent=TRUE, debug=FALSE, callFrom=NULL) {
  fxNa <- .composeCallName(callFrom,newNa="nonAmbiguousMat")
  if(is.character(byCol)) byCol <- naOmit(match(byCol, colnames(mat))) else {               # transform 'byCol' to index using match
    if(is.factor(byCol)) byCol <- as.numeric(as.character(byCol))}
  msg <- "Invalid 'byCol': Argument 'byCol' allows selecting one column of numeric data from 'mat', either as column-name or index"
  if(length(byCol) <1) stop(msg) else if(length(byCol) >1) {byCol <- byCol[1]; message(msg)}
  mat <- mat[order(as.numeric(mat[,byCol]), decreasing=FALSE),]
  y <- convToNum(mat[,byCol], remove=NULL, sciIncl=TRUE, callFrom=fxNa)
  ab <- which(diff(y)==0)
  onlySingleLast <- FALSE
  if(length(ab) <1) return(if(asList) list(unique=mat) else mat) else {
    if(onlySingleLast) { ac <- diff(ab)
      ac <- ab[c(which(ac >1), if(ac[length(ac)] >1) length(ab) else NULL)]}   # not yet used: only single (last) instance of repeated
    nrName <- if(is.null(rownames(mat))) rownames(mat) else mat[,if(byCol==1) 2 else 1]
    nrName[c(ab,ab+1)] <- paste(nameMod,nrName[c(ab,ab+1)],sep="")
    if(is.null(rownames(mat))) mat[,if(byCol==1) 2 else 1] <- nrName else rownames(mat) <- nrName
    out <- if(uniqOnly | asList) mat[-1*unique(sort(c(ab, ab +1))),] else NULL
    out <- if(asList) {if(uniqOnly) list(unique=out) else list(unique=out, ambig=mat[ab,])} else {if(uniqOnly) out else mat[-ab,]}
    out } }
         

#' Fuse pairs to generate cluster-names
#'
#' Fuse previously identified pairs to 'clusters', return vector with cluster-numbers. 
#' @param datPair 2-column matrix where each line represents 1 pair
#' @param refDatNames (NULL or character) allows placing selected pairs in context of larger data-set (names to match those of 'datPair')
#' @param inclRepLst (logical) if TRUE, return list with 'clu' (clu-numbers, default output) and 'refLst' (list of clustered elements, only n>1)
#' @param maxFuse (integer, default NULL) maximal number of groups/clusters
#' @param silent (logical) suppress messages
#' @param debug (logical) display additional messages for debugging
#' @param callFrom (character) allow easier tracking of message(s) produced
#' @return This function returns a vector with cluster-numbers
#' @examples
#' daPa <- matrix(c(1:5,8,2:6,9), ncol=2)
#' fusePairs(daPa, maxFuse=4)
#' @export
fusePairs <- function(datPair, refDatNames=NULL, inclRepLst=FALSE, maxFuse=NULL, debug=FALSE, silent=TRUE, callFrom=NULL) {
  fxNa <- .composeCallName(callFrom, newNa="fusePairs")
  if(isTRUE(debug)) silent <- FALSE else { debug <- FALSE
    if(!isTRUE(silent)) silent <- FALSE }
  msg <- " 'datPair' should be matrix of 2 cols with paired rownames"
  if(length(dim(datPair)) !=2) stop(msg) else if(ncol(datPair) <2) stop(msg)
  if(!is.matrix(datPair)) datPair <- if(nrow(datPair) >1) as.matrix(datPair) else matrix(datPair,nrow=1,dimnames=dimnames(datPair))
  if(is.null(maxFuse)) maxFuse <- nrow(datPair)+1 else {
    if(!silent) message(fxNa," use of 'maxFuse' at ",maxFuse," may increase 'artificially' the number of groups/clusters")}
  if(!is.null(refDatNames)) { chRef <- unique(as.character(datPair[,1:2])) %in% as.character(refDatNames)
    if(sum(!chRef,na.rm=TRUE) >0) { if(!silent) message(fxNa,sum(!chRef,na.rm=TRUE)," names from 'datPair' missing in 'refDatNames', ignore")
    refDatNames <- refDatNames } }
  refDatNa <- if(is.null(refDatNames)) unique(as.character(datPair[,1:2])) else refDatNames
  if(is.null(rownames(datPair))) rownames(datPair) <- 1:nrow(datPair)
  nPt <- length(unique(as.character(datPair[,1:2])))
  similDat <- list(which(refDatNa %in% datPair[1,1:2]))          # similDat .. index of refDatNa per cluster
  if(debug) {message(fxNa," dim(datPair) ",nrow(datPair)," ",ncol(datPair))}
  if(nrow(datPair) >1) for(i in 2:nrow(datPair)) {
    basTest <- datPair[i,1:2] %in% as.character(datPair[1:(i-1),1:2])      # see if (any of) new pair already seen before
    chPr <- sapply(similDat, function(x) sum(datPair[i,1:2] %in% refDatNa[x]))   # for each clu : number of common with cur query
    if(debug) message(fxNa," basTest  ",pasteC(basTest))
    if(any(chPr >0) && length(similDat[[which.max(chPr)]]) < maxFuse) {                                     # add to existing cluster
      if(sum(basTest)==1){
        similDat[[which.max(chPr)]] <- unique(sort(c(similDat[[which.max(chPr)]], which(refDatNa %in% datPair[i,1:2]))))
      } else {                                               # fuse clusters instead of adding
        isMax <- which(chPr==max(chPr))
        if(length(isMax) >1) {
          if(debug) message(fxNa," iter no ",i,"  fuse clusters : max ",pasteC(isMax))
          similDat[[isMax[1]]] <- unique(unlist(similDat[which(chPr==max(chPr))]))
        similDat <- similDat[-isMax[2]]
        } else if(debug) message(fxNa," iter no ",i," : loop, already fused")}        # don't make loops circular
    } else {                                                     # create new cluster
      if(debug) message(fxNa,"Create new cluster no ",length(similDat)+1,"  for ",pasteC(which(refDatNa %in% datPair[i,1:2])))
      similDat[[length(similDat)+1]] <- which(refDatNa %in% datPair[i,1:2])
  }  }
  msg <- "Trouble ahead, some elements are indexed more than 1x in 'similDat' !!!"
  if(any(table(unlist(similDat)) >1) && maxFuse < nrow(datPair) && !silent) message(fxNa,msg)
  out <- rep(1:length(similDat), sapply(similDat,length))
  names(out) <- unlist(lapply(similDat, function(x) refDatNa[x]))
  if(debug) {message(fxNa," fP2")}
  if(length(refDatNa) > length(unlist(similDat))) {
    tmp <- refDatNa[which(!(refDatNa %in% as.character(as.matrix(datPair[,1:2]))))]
    out2 <- (1:length(tmp)) +length(out)
    names(out2) <- tmp
    out <- c(out,out2) }
  out <- out[order(convToNum(.trimLeft(names(out)), remove=NULL, sciIncl=TRUE))]
  if(inclRepLst) list(clu=out, repLst=similDat) else out }


#' Distances beteenw sorted points of 2-columns
#'
#' This function returns distances beteenw sorted points of 2-column matrix 'x'
#' 
#' @param x (matrix or data.frame, min 2 columns) main input
#' @param asSum (logical) if \code{TRUE} (default) the sum of all distances will be returned, otherwise the individual distances
#' @return This function returns a numeric vector with distances
#' @examples
#' daPa <- matrix(c(1:5,8,2:6,9), ncol=2)
#' .neigbDis(daPa)
#' @export
.neigbDis <- function(x, asSum=TRUE) {    # return (sum of) distances betw sorted points of 2-column matrix 'x'
  if(nrow(x) ==2) sqrt(sum(abs(x[1,] -x[2,]))) else {
    x <- x[order(rowMeans(x)),]
    out <- sqrt(rowSums((x[-nrow(x),]-x[-1,])^2, na.rm=TRUE))
    if(!isFALSE(asSum)) sum(out, na.rm=TRUE) else out } }
      

#' Difference in ppm between numeric values
#'
#'  This is a \code{diff()}-like function to return difference in ppm between subsequent values. 
#'  Result is oriented, ie neg ppm value means decrease (from higher to lower value). Note that if the absolute difference remains the same the difference in ppm will not remain same.
#'  Any difference to NA is returned as NA, thus a single NA will result in two NAs in output (unless NA is 1st or last).
#'
#' @param dat (numeric) vector for calculating difference to preceeding/following value in ppm
#' @param toPrev (logical) determine oriention
#' @param silent (logical) suppress messages
#' @param callFrom (character) allows easier tracking of messages produced
#' @return This function returns a list with close matches of 'x' to given 'y', the numeric value dependes on 'sortMatch' (if FALSE then always value of 'y' otherwise of longest of x&y)
#' @seealso \code{\link{checkSimValueInSer}} and (from this package) \code{.compareByDiff},  \code{\link[base]{diff}} 
#' @examples
#' aa <- c(1000.01, 1000.02, 1000.05, 1000.08, 1000.09, 1000.08)
#' .compareByPPM(list(aa,aa), 30, TRUE)                    # tabular 'long' version
#' diffPPM(aa)
#' @export
diffPPM <- function(dat, toPrev=FALSE, silent=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom,newNa="diffPPM")
  if(!is.numeric(dat)) dat <- try(as.numeric(if(is.factor(dat)) as.character(dat) else dat), silent=TRUE)
  if(inherits(dat, "try-error")) { if(!silent) message(fxNa,"Can't convert 'dat' to numeric !"); return(NULL)
  } else {
    rat <- if(toPrev) dat[-1]/dat[-length(dat)] else dat[-length(dat)]/dat[-1]
    (2*(dat[-1] > dat[-length(dat)])-1)*abs(rat -1)/1e-6}}
     

#' Locate Sample Index From Index Or Name Of Pair-Wise Comparisons In List Or MArrayLM-Object  
#'
#' When multiple series of data are tested simultaneaously (eg using \code{moderTestXgrp}), multiple pairwise comparisons get performed. 
#' This function helps locating the groups of samples, eg respective mean-columns, corresponding to specific pairwise comparisons.
#' 
#' @details  
#' As main input one gives a list or MArrayLM-object containing testing results from pairwise comparisons,
#' and specific comparisons indicated by \code{useComp} to get located in the elements of mean-columns (\code{lstMeans}).
#' 
#' Note, that that the numeric indexes should be used with caution as they refer to the levels of the groups which are sorted
#' and thus not necessarily in the order of other elements like the group-means. 
#'
#' This function allows finding inverted questions (eg 'D-C' even though 'C-D' was actually run in testing), but the result will always be presented in the initial order
#' 
#' @param MArrayObj (list or MArray-object) main input, object to examine and extract pairwise question information
#' @param useComp (character or integer, length=1) index or name of pairwise-comparison to be addressed
#' @param outTy (character) choose if just vector of indexes (with \code{outTy="index0"}), matrix of indexes (organized by pairwise questions, with \code{outTy="index"}), 
#'    or matrix of group-names (with \code{outTy="names"}) or list with $index0 (osrted unique vector or indexes),  $pwGrpNa, $pwIndex, $concat, $sep and $compIndex will get returned
#' @param groupSep (NULL or character of length=1) separator for pair of names; if \code{NULL} a suitable separator (among '-','--','_','__','.',' ' and '  ') will be used
#' @param combPwSep (character) sequence of separators for building combined names (may be combPwSep='combine1' or 'combine2'); will be used in pwSeparatorList()  
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging 
#' @param callFrom (character) allow easier tracking of messages produced
#' @seealso  \code{\link{moderTestXgrp}}, this function gets used eg in \code{\link[wrGraph]{MAplotW}} or \code{\link[wrGraph]{VolcanoPlotW}}
#' @return This function returns if \code{outTy="names"} a numeric vector with indexes indicating the sorted (!!) columns of (replicate) mean-values corresponding to the comparison specified in \code{useComp};
#'   or if \code{outTy="names"} or  \code{outTy="pwGrpNa"} :  the (separated) names of pairwise groups will be returned as matrix (each pairwise question as separate line) 
#'   or if \code{outTy="list"} a list with $index0 (sorted unique vector or indexes), $pwGrpNa (the separated pairwise group-names), $pwIndex (the ith pairwise comparison), $concat (the concatenated pairwise names), $sep (pairwise separator) and $compIndex 
#' @examples
#' grp3 <- factor(rep(LETTERS[4:2], c(2,3,3)))
#' set.seed(2017); t8 <- matrix(round(rnorm(208*8,10,0.4), 2), ncol=8,
#'   dimnames=list(paste(letters[],rep(1:8,each=26),sep=""), paste0(grp3,c(1:2,1:3,1:3))))
#' if(requireNamespace("limma", quietly=TRUE)) {  # need limma installed...
#'   test8 <- moderTestXgrp(t8, grp3, useComparison="all") 
#'   head(test8$p.value)         # all pairwise comparisons available
#'   sampNoDeMArrayLM(test8, 3)   
#'   levels(grp3)[sampNoDeMArrayLM(test8, 3)]
#'   head(test8$means[,sampNoDeMArrayLM(test8, 3, outTy="names")], 3)      
#'   head(test8$means[,sampNoDeMArrayLM(test8, "D-C", outTy="names")], 3) # inverted question
#' } 
#' @export
sampNoDeMArrayLM <- function(MArrayObj, useComp=NULL, outTy="index", groupSep=NULL, combPwSep="combine1", silent=FALSE, debug=FALSE, callFrom=NULL) {
  ## full list of groups rather in mean-column - if available (seed into indexGroupsFromPW() ) 
  ## locate sample index from index or name of pair-wise comparisons in list or MArrayLM-object
  ## no need to give grp since columns are considered exhaustive ??
  fxNa <- .composeCallName(callFrom, newNa="sampNoDeMArrayLM")
  if(isTRUE(debug)) silent <- FALSE else { debug <- FALSE
    if(!isTRUE(silent)) silent <- FALSE }
  
  invQuestion <- FALSE
  errMsg <- c("Argument 'MArrayObj' is ","empty")
  if(length(MArrayObj) <1) stop(errMsg[1:2])
  if(debug) {message(fxNa,"sNDM1"); sNDM1 <- list(MArrayObj=MArrayObj,useComp=useComp,groupSep=groupSep)}
  
  ## check for complete set of groups from  respective mean-columns
  setup <- if("setup" %in% names(MArrayObj)) MArrayObj$setup else getPairwiseSetup(pwGrpNa=colnames(MArrayObj$coefficients), grpNa=if("means" %in% names(MArrayObj)) colnames(MArrayObj$means) else colnames(MArrayObj$design), sep=groupSep, combPwSep=combPwSep, silent=silent, debug=debug, callFrom=fxNa)
  
  if(length(useComp)==0) useComp <- 1 else {                          # set to default if empty
    chNA <- is.na(useComp)
    if(all(chNA)) useComp <- 1 else if(any(chNA)) useComp <- useComp[which(!chNA)]} 
  chNum <- grepl("^[[:digit:]]+$", useComp)
  if(length(dim(useComp)) ==2 && ncol(useComp)==2) {
    ## matrix
    if(debug) {message(fxNa,"sNDM2"); sNDM2 <- list(MArrayObj=MArrayObj,useComp=useComp,groupSep=groupSep,chNum=chNum,setup=setup)}
    if(all(chNum)) {  useComp <- matrix(as.integer(useComp), ncol=ncol(useComp), dimnames=dimnames(useComp))
      ## need to match pair of indexes (NOTE : interpreted to/as index to SORTED names of groups)
      useComp <- which(paste0(setup$pwIndex[,1],"-",setup$pwIndex[,2]) %in% paste0(useComp[,1],"-",useComp[,2]))
      if(length(useComp)==0) { 
        useComp <- which(paste0(setup$pwIndex[,1],"-",setup$pwIndex[,2]) %in% paste0(useComp[,2],"-",useComp[,1]))
        if(length(useComp)==0) {
          if(!silent) message(fxNa,"Don't understand index-indications for 'useComp' (supposed as matrix of pairwise indexes), setting to default =1  (68)")
          useComp <- 1 } else invQuestion <- TRUE
      }    
    } else {                 ## suppose as matrix of group-names
      if(debug) {message(fxNa,"sNDM2b"); sNDM2b <- list(MArrayObj=MArrayObj,useComp=useComp,groupSep=groupSep,chNum=chNum,setup=setup)}    
      useComp <- matrix(match(useComp, colnames(if("means" %in% names(MArrayObj)) MArrayObj$means else MArrayObj$design)), ncol=2, dimnames=dimnames(useComp))
      chNA <- is.na(useComp)
      useComp <- which(rowSums(useComp)==0)
      if(all(chNA) || length(useComp)) { if(!silent) message(fxNa,"Don't understand index-indications for 'useComp' (supposed as matrix of pairwise group-names), setting to default =1  (76)")
        useComp <- 1
      }
    }                # finish useComp as pairwise matrix
  } else {
    if(debug) {message(fxNa,"sNDM3"); sNDM3 <- list(MArrayObj=MArrayObj,useComp=useComp,groupSep=groupSep,chNum=chNum,setup=setup)}
    ##  useComp as vector
    if(all(chNum)) { useComp <- as.integer(useComp)
      ch1 <- (useComp >0) & (useComp <= nrow(setup$pwGrpNa))
      if(!all(ch1)) {if(!silent) message(fxNa,"Don't understand index-inidications for 'useComp' (values not in range available), setting to default =1  (85)")
        useComp <- 1
      } else if(any(!ch1)) useComp <- useComp[which(ch1)]
    } else {
      ## character vector - try to find in pairwise names
      if(debug) {message(fxNa,"sNDM3b"); sNDM3b <- list(MArrayObj=MArrayObj,useComp=useComp,groupSep=groupSep,chNum=chNum,setup=setup)}
      
      useComp1 <- useComp2 <- NULL
      ch2 <- useComp %in% colnames(if("contrasts" %in% MArrayObj) MArrayObj$contrasts else MArrayObj$coefficients)
      if(any(ch2)) useComp1 <- which(colnames(if("contrasts" %in% MArrayObj) MArrayObj$contrasts else MArrayObj$coefficients) %in% useComp) 
      if(any(!ch2)) {
        ## could contain reversed question(s)
        ch3 <- useComp %in% paste0(setup$pwGrpNa[,2], setup$sep, setup$pwGrpNa[,1])
        if(any(ch3)) { useComp2 <- which(paste0(setup$pwGrpNa[,2], setup$sep, setup$pwGrpNa[,1]) %in% useComp)
          invQuestion <- TRUE}
      }
      useComp <- c(useComp1, useComp2)
      if(length(useComp) ==0) {
        if(!silent) message(fxNa,"Don't understand text-indications for 'useComp' (names not appearing in existing pairwise sets), setting to default =1  (103)")
        useComp2 <- 1 
      }       
    }
  }       # finish determining useComp (as index)
  if(!silent && invQuestion) message(fxNa,"NOTE : Some questions were found as inverted questions")
  
  chTy <- c("names","pwGrpNa","list") %in% outTy
  if(debug) {message(fxNa,"sNDM4"); sNDM4 <- list(MArrayObj=MArrayObj,useComp=useComp,groupSep=groupSep,chNum=chNum,setup=setup,outTy=outTy,chTy=chTy)}
  if(any(chTy, na.rm=TRUE)) {
    ##
    pwNa <- colnames(if("contrasts" %in% MArrayObj) MArrayObj$contrasts else MArrayObj$coefficients)[useComp]
    if(any(chTy[1:2], na.rm=TRUE)) setup$pwGrpNa[useComp,,drop=FALSE] else {
      list(index0= unique(sort(as.integer(setup$pwIndex[useComp,]))), pwGrpNa=setup$pwGrpNa[useComp,,drop=FALSE], pwIndex=setup$pwIndex[useComp,,drop=FALSE], concat=setup$concat[useComp], sep=setup$sep, compIndex=outTy)
    }
  } 
}  
  
 

#' Get First Of Repeated By Column
#'
#' \code{get1stOfRepeatedByCol} sorts matrix 'mat' and extracts only 1st occurance of values in column 'sortBy'.
#' Returns then non-redundant matrix (ie for column 'sortBy', if 'markIfAmbig' specifies existing col, mark ambig there).
#' Note : problem when sortSupl or sortBy not present (or not intended for use)
#'
#' @param mat (matrix or data.frame) numeric vector to be tested
#' @param sortBy (character) column name for which elements should be made unique, numeric or character column; 'sortSupl' .. add'l colname to always select specific 1st)
#' @param sortSupl (character) default="ty"
#' @param asFirstLast (character,length=2) to force specific strings from coluln 'sortSupl' as first and last when selecting 1st of repeated terms, default=c("full","inter")
#' @param markIfAmbig (character,length=2) 1st will be set to 'TRUE' if ambiguous/repeated, 2nd will get (heading) prefix, default=c("ambig","seqNa")
#' @param asList (logical) to return list with non-redundant ('unique') and removed lines ('repeats') 
#' @param abmiPref (character) prefix to note ambiguous entries/terms, default="_"
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns depending on argumnet 'asList' either list with non-redundant ('unique') and removed lines ('repeats') 
#' @seealso \code{\link{firstOfRepeated}} for (more basic) treatment of simple vector, \code{\link{nonAmbiguousNum}} for numeric use (much faster !!!)
#' @examples
#' aa <- cbind(no=as.character(1:20),seq=sample(LETTERS[1:15],20,repl=TRUE),
#'   ty=sample(c("full","Nter","inter"),20,repl=TRUE),ambig=rep(NA,20),seqNa=1:20)
#' get1stOfRepeatedByCol(aa)
#' @export
get1stOfRepeatedByCol <- function(mat, sortBy="seq", sortSupl="ty", asFirstLast=c("full","inter"), markIfAmbig=c("ambig","seqNa"), asList=FALSE, abmiPref="_", silent=FALSE, debug=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom,newNa="get1stOfRepeatedByCol")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  msg <- " 'mat' should be matrix or data.frame with >1 lines & >1 columns"
  if(length(dim(mat)) !=2) stop(fxNa,msg) else if(any(dim(mat) <2)) stop(fxNa,msg)
  if(is.character(sortBy)) if(!sortBy %in% colnames(mat)) stop(" invalid 'sortBy'")     # check better to allow alternative use of vector instead of colname ??
  num <- if(is.numeric(mat[,sortBy])) mat[,sortBy] else as.numeric(as.factor(mat[,sortBy]))
  if(is.data.frame(mat)) mat <- as.matrix(mat)
  reps <- which(mat[,sortBy] %in% names(which(table(mat[,sortBy]) >1)))
  if(length(reps) >0){
    sel <- mat[reps,]  
    supl <- gsub(asFirstLast[1],"a", gsub(asFirstLast[2],"z", sel[,sortSupl]))                  # allow 'full' to come 1st & 'inter' as last
    sel <- sel[order(sel[,sortBy],supl),]
    selFi <- sel[which(diff(c(-1,as.numeric(as.factor(sel[,sortBy])))) >0),]                  # main selection of 1st of repeated
    if(length(dim(selFi))<2) selFi <- matrix(selFi, nrow=1, dimnames=list(NULL,names(selFi)))
    if(markIfAmbig[1] %in% colnames(mat)) selFi[,markIfAmbig[1]] <- TRUE                         # mark ambiguous as 'TRUE'
    if(markIfAmbig[2] %in% colnames(mat)) selFi[,markIfAmbig[2]] <- paste0(abmiPref,selFi[,markIfAmbig[2]])
    out <- rbind(mat[-1*reps,],selFi)
    if(asList) out <- list(unique=out, repeats=mat[reps,])
  } else out <- if(asList) list(unique=out) else mat
  out }
   

#' Pairwise x,y Combinations
#'
#' This function gets pairwise combinations for 'n' elements; returns matrix with x & y coordinates to form all pairwise groups for 1:n elements
#' 
#' @param n (integer) number of elements for making all pair-wise combinations 
#' @param side (character) "upper" or "lower"
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging 
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a 2-column matrix with indexes for all pairwise combinations of 1:n
#' @seealso \code{\link{upperMaCoord}}, \code{\link{getPWseparator}}, \code{\link{indexGroupsFromPW}}, \code{\link{diffCombin}}; \code{\link[utils]{combn}}, \code{\link[base]{lower.tri}} or \code{upper.tri}, simpler version \code{\link{upperMaCoord}}
#' @examples
#' triCoord(4)
#' @export
triCoord <- function(n, side="upper", silent=FALSE, debug=FALSE, callFrom=NULL) {
  ma <- matrix(1:n, ncol=n, nrow=n)
  out <- if(identical(side,"upper")) cbind(x=ma[upper.tri(ma)], y=t(ma)[upper.tri(ma)])  else {
    cbind(x=ma[lower.tri(ma)], y=t(ma)[lower.tri(ma)])}
  out }
     

#' Combine replicates from list to matrix
#'
#' Suppose multiple measures (like multiple chanels) are taken for subjects and these measures are organized as groups in a list, 
#' like muliple parameters (= channels) or types of measurements (typically many paramters are recorded when screeinig compounds in microtiter plates).
#' Within one parameter/channel all replicate-data from separate list-entries ('lst') will get combined according to names of list-elements.
#' The function will trim any redundant text in names of list-elements, try to isolate separator (may vary among replicate-groups, but should be 1 character long).
#' eg names "hct116 1.1.xlsx" & "hct116 1.2.xlsx" will be combined as replicates, "hct116 2.1.xlsx" will be considered as new group.
#' 
#' @param lst (list)  list of arrays (typically multi-parameter measures of micortiterplate data)
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a list of arrays now with same dimension of arrays (but shorter, since replicate-arrays were combined)
#' @seealso \code{\link{extr1chan}}, \code{\link{organizeAsListOfRepl}}
#' @examples
#' lst2 <- list(aa_1x=matrix(1:12,nrow=4,byrow=TRUE),ab_2x=matrix(24:13,nrow=4,byrow=TRUE))
#' combineReplFromListToMatr(lst2)
#' @export
combineReplFromListToMatr <- function(lst, silent=FALSE, debug=FALSE, callFrom=NULL){
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  fxNa <- .composeCallName(callFrom, newNa="combineReplFromListToMatr")
  varNa <- .trimRight(.trimLeft(names(lst)))
  varSep <- gsub("[[:alnum:]]","",varNa)
  if(any(nchar(varSep) <1)) stop(fxNa,"Manually EDIT file-names, can't find separator between group/plate-name and replicate-number !")
  if(any(nchar(varSep) >1)) message(fxNa,"So far designed for single separator character; use gregexpr & unlist",
    "\ Here :",paste(varSep,collapse=" "))
  varSep[which(varSep== ".")] <- "\\."
  varNa2 <- apply(cbind(varNa, varSep), 1, function(x) unlist(strsplit(x[1], x[2])) )
  arrTy <- table(varNa2[1,])[rank(unique(varNa2[1,]))]
  out <- list()
  iniFi <- 0
  for(i in 1:length(arrTy)) {
    check <- sapply(lst[iniFi +(1:arrTy[i])], length)
    if(length(unique(check)) !=1) {
      message(fxNa,"Plate-type ",i," :  Trouble ahead : Dimensions of plates not constant ! files ",paste(names(lst)[iniFi+(1:arrTy[i])],collapse=" "))
    } else {
      out[[i]] <- as.matrix(sapply(lst[iniFi +(1:arrTy[i])], as.numeric))
      dimNa <- dimnames(lst[[1 +iniFi]])
      dimnames(out[[i]]) <- list(paste0(dimNa[[1]], rep(sub("^X","",dimNa[[2]]), each=length(dimNa[[1]]))), varNa2[2, iniFi +(1:arrTy[i])] )
      iniFi <- iniFi + arrTy[i]
   } }
   names(out) <- unique(varNa2[1,])
   out }
   

#' Characterize individual contribution of single edges in tree-structures
#'
#' This function helps investigating tree-like structures with the aim of indicating how much individual tree components contribute 
#' to compose long stretches.
#' \code{contribToContigPerFrag} characterizes individual (isolated) contribution of single edges in tree-structures. 
#' Typically used to process/exploit summarized trees (as matrix) made by \code{\link{buildTree}} which makes use of the package \href{https://CRAN.R-project.org/package=data.tree}{data.tree}.
#' For example if A,B and C can be joined aa well and B +D, this function will check if A+B+C is longer and if A contributes to the longest tree. 
#' @param joinMat (matrix) matrix with concatenated edges as rownames (separated by slashes), column \code{sumLen} for total length and column \code{n} for number of edges
#' @param fullLength (integer) custom total length (useful if the concatenated edges do not cover 100 percent of the original precursor whose fragments are studied)  
#' @param nDig (integer) rounding: number of digits for 3rd column \code{len.rat} in output
#' @return matrix of 3 columns: with length of longest tree-branches where given edge participates (column \code{sumLen}), the (total) number of edges therein (col \code{n.frag}) and a relative value (\code{len.rat})
#' @seealso to build tree \code{\link{buildTree}}
#' @examples
#' path1 <- matrix(c(17,19,18,17, 4,4,2,3),ncol=2,
#'   dimnames=list(c("A/B/C/D","A/B/G/D","A/H","A/H/I"),c("sumLen","n")))
#' contribToContigPerFrag(path1)
#' @export
contribToContigPerFrag <- function(joinMat,fullLength=NULL,nDig=3){
  argN <- deparse(substitute(joinMat))
  msg <- c("Unknown input-format in '",argN,"'")
  if(is.list(joinMat)) {if("paths" %in% names(joinMat)) joinMat <- joinMat$paths else stop(msg)}
  if(length(dim(joinMat)) <2) stop(msg) else if(!"sumLen" %in% colnames(joinMat)) stop("Need column 'sumLen' in ",argN)
  if(is.null(fullLength)) fullLength <- max(joinMat[,"sumLen"])  
  joinMat <- joinMat[sort.list(joinMat[,"sumLen"],decreasing=FALSE),]
  jLi <- lapply(rownames(joinMat),strsplit,"/")
  frags <- sort(unique(unlist(jLi)))
  out <- t(sapply(frags,function(x) {z <- grep(x,rownames(joinMat))
    w <- max(joinMat[z,"sumLen"])
    c(w, min(joinMat[z[which(joinMat[z,"sumLen"]==w)],"n"]) )
  } ))
  colnames(out) <- c("sumLe","n.frag")
  out <- cbind(out,len.rat=round(out[,"sumLe"]/fullLength,nDig))
  out }
      

#' Moderated Pairwise t-test From Limma
#'
#' This function runs moderated t-test from package \code{limma} on each line of data.
#' Note: This function requires the package \href{https://bioconductor.org/packages/release/bioc/html/limma.html}{limma} from bioconductor being installed. 
#' The \code{limma} contrast-matrix has to be read by column, the lines in the contrast-matrix containing '+1' will be compared to the '-1' lines, eg grpA-grpB .
#' Local false discovery rates (lfdr) estimations will be made using the CRAN-package \href{https://CRAN.R-project.org/package=fdrtool}{fdrtool} (if available).
#'
#' @param dat matrix or data.frame with rows for multiple (independent) tests, use ONLY with 2 groups; assumed as log2-data
#' @param grp (factor) describes column-relationship of 'dat'   (1st factor is considered as reference -> orientation of M-values !!)
#' @param limmaOutput (logical) return full (or extended) MArrayLM-object from limma or 'FALSE' for only the (uncorrected) p.values
#' @param addResults (character) types of results to add besides basic limma-output, data are assumed to be log2 ! (eg "lfdr" using fdrtool-package, "FDR" or "BH" for BH-FDR, "BY" for BY-FDR, 
#'   "bonferroni" for Bonferroni-correction, "qValue" for lfdr by qvalue, "Mval", "means" or "nonMod" for non-moderated test and he equivaent all (other) multiple testing corrections chosen here)
#' @param testOrientation (character) for one-sided test (">","greater" or "<","less"), NOTE : 2nd grp is considered control/reference, '<' will identify grp1 < grp2
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a limma-type object of class \code{MArrayLM} (which can be handeled like a list)
#' @seealso \code{\link[limma]{lmFit}} and the \code{eBayes}-family of functions in package \href{https://bioconductor.org/packages/release/bioc/html/limma.html}{limma}, \code{\link[stats]{p.adjust}}
#' @examples
#' set.seed(2017); t8 <- matrix(round(rnorm(1520,10,0.4),2), ncol=8,
#'   dimnames=list(paste("l",1:190),c("AA1","BB1","CC1","DD1","AA2","BB2","CC2","DD2")))
#' t8[3:6,1:2] <- t8[3:6,1:2] +3     # augment lines 3:6 for AA1&BB1
#' t8[5:8,5:6] <- t8[5:8,5:6] +3     # augment lines 5:8 for AA2&BB2 (c,d,g,h should be found)
#' t4 <- log2(t8[,1:4]/t8[,5:8])
#' ## Two-sided testing 
#' fit4 <- moderTest2grp(t4,gl(2,2))
#' # If you have limma installed we can now see further
#' if(requireNamespace("limma")) {
#'   limma::topTable(fit4, coef=1, n=5)}    # effect for 3,4,7,8
#' 
#' ## One-sided testing
#' fit4in <- moderTest2grp(t4,gl(2,2), testO="<")
#' # If you have limma installed we can now see further
#' if(requireNamespace("limma")) {
#'   limma::topTable(fit4in, coef=1, n=5) }
#' @export
moderTest2grp <- function(dat, grp, limmaOutput=TRUE, addResults=c("lfdr","FDR","Mval","means"), testOrientation="=", silent=FALSE, debug=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom, newNa="moderTest2grp")
  runTest <- length(dat) >0
  if(isTRUE(debug)) silent <- FALSE else { debug <- FALSE
    if(!isTRUE(silent)) silent <- FALSE }
  if(any(length(dat) <1, length(dim(dat)) !=2, dim(dat) < c(1,3))) stop("Invalid argument 'dat'; must be matrix or data.frame with min 1 lines and 3 columns")
  if(length(grp) != ncol(dat)) stop("Check parameters: Number of columns of 'dat' doesn't match to length of 'grp'")  
  if(!is.factor(grp)) grp <- as.factor(grp)
  if(length(levels(grp)) <2) stop("Need at least 2 groups in argument 'grp'")
  if(length(levels(grp)) >2) {message(fxNa,"Capable only of treating 2 groups in argument 'grp' (ignore rest)")
    grp[which(grp %in% levels(grp)[-(1:2)])] <- NA }
    
  packages <- c("limma", "fdrtool", "qvalue")
  checkPkg <- function(pkg) requireNamespace(pkg, quietly=TRUE)
  checkPkgs <- sapply(packages, checkPkg)
  if(!checkPkgs[1]) {message(fxNa,"NOTE: Package 'limma' not found ! Please install first from Bioconductor"); runTest <- FALSE}
  if(runTest  && requireNamespace("limma")) {   
    if(limmaOutput && length(addResults) >0) if("all" %in% addResults) addResults <- c("BH", "BY","lfdr","qValue","bonferroni","Mval","means","nonMod")
    if(!checkPkgs[2] && any("lfdr" %in% tolower(addResults))) {
      if(!silent) message(fxNa,"Package 'fdrtool' not found, omitting .. Please install from CRAN for enabeling 'lfdr'")
      addResults <- addResults[-which(tolower(addResults) %in% "lfdr")] }
    if(!checkPkgs[3] && any(c("qvalue","qval") %in% tolower(addResults))) {
      if(!silent) message(fxNa,"Package 'qvalue' not found, omitting .. Please install from Bioconductor for enabeling 'qValue'")
      addResults <- addResults[-which(tolower(addResults) %in% c("qvalue","qval"))] }
    ## treat non-unique row-names ?
    if(length(rownames(dat) >0)) if(length(unique(rownames(dat))) < nrow(dat)) {
      if(!silent) message(fxNa,"Detected ",nrow(dat) -length(unique(rownames(dat)))," non-unique rownames of 'dat' !")}
    altHyp <- "two.sided"                                             # default, change only if explicit sign recognized
    if(length(testOrientation) <1) testOrientation <- altHyp
    if(testOrientation %in% c("<","less","inf")) altHyp <- "less" 
    if(testOrientation %in% c(">","greater","sup")) altHyp <- "greater" 
    datDesign <- stats::model.matrix(~ grp)
    if(nrow(datDesign) < ncol(dat)) datDesign <- rbind(datDesign, matrix(rep(0,length(levels(grp))*(ncol(dat)-nrow(datDesign))), ncol=ncol(datDesign)))
    fit1 <- try(limma::eBayes(limma::lmFit(dat, design=datDesign)), silent=TRUE)                ## Fitting linear model
    if(inherits(fit1, "try-error")) { runTest <- FALSE
      warning(fxNa," UNABLE to run limma::lmFit() and/or limma::eBayes(); having problems running this function !")}
  } else runTest <- FALSE
  
  if(runTest) {  
    fit1$means <- rowGrpMeans(dat, grp) 
    chNA <- colSums(is.na(fit1$p.value))
    if(any(chNA==nrow(dat))) fit1$p.value <- fit1$p.value[,-1*which(chNA==nrow(dat))]
    ## modif to adjust for different H0  (29mar18)
    tx <- c("Testing alternative hypothesis: true difference in means is "," than 0 (ie focus on "," results with A ",altHyp," than B)")
    if(identical(altHyp,"greater")) {
      ch <- fit1$means[,1] > fit1$means[,2]
      if(!silent) message(fxNa,tx[1],altHyp,tx[2],sum(ch),tx[3:5])
      if(any(ch)) fit1$p.value[which(ch),] <- fit1$p.value[which(ch),]/2
      if(any(!ch)) fit1$p.value[which(!ch),] <- 1- fit1$p.value[which(!ch),]/2        # !(A > B)  .. A <= B
    }
    if(identical(altHyp,"less")){
      ## testing for Ho A > B
      ch <- fit1$means[,2] > fit1$means[,1]
      if(!silent) message(fxNa,tx[1],altHyp,tx[2],sum(ch),tx[3:5])
      if(any(ch)) fit1$p.value[which(ch),] <- fit1$p.value[which(ch),]/2
      if(any(!ch)) fit1$p.value[which(!ch),] <- 1- fit1$p.value[which(!ch),]/2        # !(A > B)  .. A <= B
    }
    if(any(c("qval","qvalue") %in% tolower(addResults)) && !requireNamespace("qvalue") && !silent) message(fxNa,"NOTE: package 'qvalue' not installed from CRAN, can't calulate ...")

    if(!limmaOutput) out <- fit1$p.value[,2] else { out <- fit1
      ## further inspect & correct values of 'addResults' ?
      if("Mval" %in% addResults) out$Mval <- (out$means[,1] - out$means[,2]) 
      if(any(c("FDR","BH") %in% toupper(addResults))) out$FDR <- if(is.matrix(out$p.value)) {
        apply(out$p.value, 2, stats::p.adjust, meth="BH")} else stats::p.adjust(out$p.value, meth="BH")
      if("BY" %in% toupper(addResults)) out$BY <- if(is.matrix(out$p.value)) {
        apply(out$p.value, 2, stats::p.adjust, meth="BY")} else stats::p.adjust(out$p.value, meth="BY")
      if("lfdr" %in% tolower(addResults) && requireNamespace("fdrtool")) {
        if(!silent && nrow(dat) <200) message(fxNa,"Please NOTE that fdrtool() (from package fdrtool) considers that there are TOO FEW input test statistics for reliable FDR calculations !   (..originally giving a warning)")
        out$lfdr <- if(is.matrix(out$p.value)) {
        suppressWarnings(apply(out$p.value, 2, pVal2lfdr))} else suppressWarnings(pVal2lfdr(out$p.value)) }    
      if(any(c("qval","qvalue") %in% tolower(addResults)) && requireNamespace("qvalue")) { out$qVal <- if(is.matrix(out$p.value)) {
        try(apply(out$p.value, 2, function(x) qvalue::qvalue(x,lfdr.out=TRUE)$lfdr), silent=TRUE)
          } else try(qvalue::qvalue(out$p.value,lfdr.out=TRUE)$lfdr, silent=TRUE) 
        if(inherits(out$qVal, "try-error")) {  
          if(!silent) message(fxNa,"Problem with pi0 estimation, setting pi0=1 like BH-FDR")
          out$qVal <- if(is.matrix(out$p.value)) { apply(out$p.value, 2, function(x) qvalue::qvalue(x, pi0=1, lfdr.out=TRUE)$lfdr)
            } else qvalue::qvalue(out$p.value, pi0=1, lfdr.out=TRUE)$lfdr }
      }    
      if(any(c("bonferroni","bonf") %in% tolower(addResults))) out$bonf <- if(is.matrix(out$p.value)) {
        apply(out$p.value, 2, stats::p.adjust, meth="bonferroni")} else stats::p.adjust(out$p.value, meth="bonferroni")
    }
    if("nonMod" %in% addResults) { gr1 <- which(grp==levels(grp)[1])
      gr2 <- which(grp==levels(grp)[2])
      altHyp <- testOrientation
      if(altHyp=="=") altHyp <- "two.sided" 
      out$nonMod.p <- apply(dat, 1, function(x) stats::t.test(x[gr1], x[gr2], alternative=altHyp)$p.value)
      if(any(c("FDR","BH") %in% toupper(addResults))) out$nonMod.FDR <- stats::p.adjust(out$nonMod.p, method="BH") 
      if("BY" %in% toupper(addResults)) out$nonMod.BY <- stats::p.adjust(out$nonMod.p, method="BY")
      if(any(c("lfdr") %in% tolower(addResults)) && requireNamespace("fdrtool")) out$nonMod.lfdr <- suppressWarnings(try(pVal2lfdr(out$nonMod.p), silent=TRUE)) 
      if(any(c("qval","qvalue") %in% tolower(addResults)) && requireNamespace("qvalue")) { 
        out$nonMod.qVal <- try(qvalue::qvalue(out$nonMod.p, lfdr.out=TRUE)$lfdr, silent=TRUE)
        if(inherits(out$nonMod.qVal, "try-error")) {
          if(!silent) message(fxNa,"Problem with pi0 estimation (non-shrinked p-values) fro qValue, setting pi0=1 like BH-FDR")
          out$qVal <- if(is.matrix(out$p.value)) qvalue::qvalue(out$nonMod.p, pi0=1, lfdr.out=TRUE)$lfdr }
      }
      if(any(c("bonferroni","bonf") %in% tolower(addResults))) out$nonMod.bonf <- stats::p.adjust(out$nonMod.p, method="bonferroni")     
    }
    out }}
    

#' Organize Values Into List And Sort By Names 
#'
#' Sort values of \code{'x'} by its names and organize as list by common names, the names until \code{'sep'} are used for (re)grouping. 
#' Note that typical spearators occuring the initial names may need protection by '\\' (this is automatically taken care of for the case of the dot ('.') separator).
#' @param x (list) main input
#' @param sep (character) separator (note that typcal separators may need to be protected, only automatically added for '.')
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a matrix or data.frame
#' @seealso \code{rbind} in \code{\link[base]{cbind}}
#' @examples
#' listGroupsByNames((1:10)/5)
#' ser1 <- 1:6; names(ser1) <- c("AA","BB","AA.1","CC","AA.b","BB.e")
#' listGroupsByNames(ser1)
#' @export
listGroupsByNames <- function(x, sep=".", silent=FALSE, debug=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom, newNa="listGroupsByNames")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  if(debug) message(fxNa,"lGNB1")

  if(length(sep) <1) sep <- "\\."
  if(length(sep) >1) {if(!silent) message(fxNa,"'sep' should be character of length 1"); sep <- sep[1]}
  if(sep %in% c(".")) sep <- paste0("\\",sep)
  if(length(names(x)) <1) { message(fxNa," no names found in 'x' !!")
    names(x) <- spl <- sapply(strsplit(as.character(x), sep), function(y) y[1])
  } else spl <- strsplit(names(x),sep)
  ex1st <- sapply(spl, function(x) x[1])
  if(length(ex1st) > length(unique(ex1st))) {
    out <- lapply(unique(ex1st), function(y) which(ex1st==y))
    out <- lapply(out, function(y) c(x[y]))
  } else out <- as.list(x)
  if(length(names(out)) <1) names(out) <- sapply(out, function(y) names(y)[1])
  out }
    

#' Index Names Of Groups From Vector Of Concatenated Pairwise Group-Names  
#' 
#' This function allows matching which groups are cited in a pairwise concatenated manner.
#' This function can also be used to split concatenated group-names (while automatically determining a suitable separator) and display as matrix when \code{includeGrp=TRUE}.
#' Replaces matchSampToPairw() ???
#'
#' @param compNames (vector of character or integer, or matrix) names of pairwise combined group-names; if matrix the rownames of \code{compNames} will be used
#' @param grp (character or factor) optional groups (may include group-names that do not occur in \code{compNames})
#' @param potSep (character) potential separators to be checked if occuring in \code{useComp}; the first fitting will used
#' @param includeGrp (logical) instead of matrix, rather return list with $pwInd, $pwGrpNames (group-names corresponding to index), $sep (separator used to split) and $grpNa
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging 
#' @param callFrom (character) allow easier tracking of messages produced
#' @seealso  \code{\link{moderTestXgrp}} (colnames of testing results from this function may be used); \code{\link{replacePWseparator}}; used internally \code{\link{getPWseparator}},  \code{\link{pwSeparatorList}}, \code{\link{.getPWseparator}}
#' @return This function returns a numeric matrix with indexes corresponding to (sorted) group-names as they appear in \code{compNames};
#'   or if \code{includeGrp=TRUE} a list with $ind (matrix indexes for each \code{compNames}), $grpNames (group-names corresponding to index), $sep (separator used to split) and $grpNa (sorted names of types of groups)
#' @examples
#' indexGroupsFromPW(c("C-B","B-D","C-D")) 
#' indexGroupsFromPW(c("C-B","B-D","C-D"), grp=rep(LETTERS[1:6],2), includeGrp=TRUE) 
#' @export
indexGroupsFromPW <- function(compNames, grp=NULL, potSep=pwSeparatorList("combine1"), includeGrp=FALSE, silent=FALSE, debug=FALSE, callFrom=NULL) {
  ## Index Names Of Groups From Vector Of Concatenated Pairwise Group-Names
  fxNa <- .composeCallName(callFrom, newNa="indexGroupsFromPW")
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  if(!isTRUE(silent)) silent <- FALSE
  out <- out2 <- tmp <- NULL
  if(length(potSep) >0) potSep <- unique(naOmit(potSep))
  if(length(potSep)==0 || any(is.na(potSep))) { potSep <- pwSeparatorList("combine1") # c("-","---","_","___","."," ","  ","--","__")      # set to default
    if(debug) message(fxNa,"Argument 'potSep' appers empty or invalid, using default")
    if(length(compNames) >3) potSep <- unique(c(potSep[1],"--",potSep[2:3],"__",potSep[4:length(potSep)])) }
  ## main  
  if(length(compNames) >0) {
    if(debug) {message(fxNa,"iGFP1"); iGFP1 <- list(compNames=compNames, grp=grp,potSep=potSep)}
      #pwGrpNa, grpNa=
    if(length(potSep) != 1) {
       # getPWseparator(iGFP1$compNames, grp=iGFP1$grp)
       # getPWseparator(iGFP1$compNames, grp=iGFP1$grp, potSep=iGFP1$potSep, includeGrp=TRUE)
      tmp <- try(getPWseparator(compNames=if(length(dim(compNames))==2) rownames(compNames) else compNames, grp=grp, potSep=potSep, includeGrp=TRUE, silent=silent, debug=debug, callFrom=fxNa), silent=TRUE)
      if(debug) {message(fxNa,"iGFP1b"); iGFP1b <- list(compNames=compNames, grp=grp,potSep=potSep,tmp=tmp)}
      if(inherits(tmp, "try-error")) warning(fxNa,"Cannot figure out proper separator (returning NULL)") else potSep <- tmp$sep 
    }
    if(length(potSep) == 1) {
      if(length(grp) >0) {                    # use grp (if valid) instead of automatically determined grpNa
        grp <- unique(naOmit(grp))
        if(length(tmp) >0) {                  # from checking multiple potential sep : compNames already split to show grp
          chGr <- tmp$grpNa %in% grp
          if(any(!chGr)) { warning(fxNa,sum(chGr)," elements found by splitting not present in user-provided 'grp', adding ..")
            grp <- unique(grp, tmp$grpNa)
          } else grp <- tmp$grpNa   
        }
        if(debug) {message(fxNa,"iGFP1c"); iGFP1c <- list(compNames=compNames, grp=grp,potSep=potSep,tmp=tmp)}
      } 
      grp <- as.factor(grp)
      #if(length(tmp) >0 && !is.factor(tmp$grpNa)) tmp$grpNa <- as.factor(tmp$grpNa)
      if(debug) {message(fxNa,"iGFP3"); iGFP3 <- list(compNames=compNames, grp=grp,potSep=potSep,tmp=tmp)}
      
      out2 <- strsplit(if(length(dim(compNames)==2)) rownames(compNames) else compNames, potSep)
      chLe <- sapply(out2, length) !=2
      if(debug) {message(fxNa,"iGFP3b"); iGFP3b <- list(compNames=compNames, grp=grp,potSep=potSep,tmp=tmp,out2=out2,chLe=chLe,includeGrp=includeGrp)}
      if(any(chLe)) out <- NULL else {
          #matrix(match(unlist(iGFP3b$out2), iGFP3b$grp),ncol=2, byrow=TRUE)
        out <- matrix(match(unlist(out2), grp), ncol=2, byrow=TRUE, dimnames=list(if(length(dim(compNames)==2)) rownames(compNames) else compNames, c("samp","ref")))
        if(length(out2) >0) { out2 <- t(as.data.frame(out2)); dimnames(out2) <- dimnames(out)}          
        if(isTRUE(includeGrp)) { 
          out <- list(pwGrpNa=if(length(out2) >0) out2 else matrix(tmp$grpNa[out], ncol=2, dimnames=dimnames(out)), sep=potSep, index=out, grpNa=grp)
        }
      }
    } 
  } else if(debug) message(fxNa,"Argument 'compNames' is empty, nothing to do")
  out                              
}
    

#' Get All Combinations With TRUE From Each Column 
#'
#' This function addresses the case when multiple alternatove ways exit to combine two elements. 
#' \code{combineSingleT} makes combinatory choices : if multiple \code{TRUE} in given column of 'mat' make all multiple selections with always one \code{TRUE} from each column
#' The resultant output contains index for first and second input columns elements to be combined.
#' @param mat 2-column matrix of logical values
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allows easier tracking of messages produced
#' @return This function returns a matrix with indexes of conbinations of \code{TRUE}
#' @examples
#' ## Example: Fist column indicates which boys want to dance and second column  
#' ## which girls want to dance. So if several boys want to dance each of the girls 
#' ## will have the chance to dance with each of them.  
#' matr <- matrix(c(TRUE,FALSE,TRUE,FALSE,TRUE,FALSE),ncol=2)
#' combineSingleT(matr)
#' @export
combineSingleT <- function(mat, silent=FALSE, debug=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom, newNa="combineSingleT")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
 
  msg <- "Expecting matrix of logical entries with 2 columns and at least 1 'TRUE' per column "
  if(length(dim(mat)) !=2) stop(msg) else if(nrow(mat) <1 | ncol(mat)<2) stop(msg)
  if(!is.logical(mat)) msg <- matrix(as.logical(mat), ncol=ncol(mat))
  chMu <- apply(mat, 2, which)
  if(any(sapply(chMu,length)<1)) stop(msg)
  if(is.matrix(chMu)) chMu <- list(chMu[,1],chMu[,2])
  out <- matrix(c(rep(chMu[[1]], sum(mat[,2])), rep(chMu[[2]], each=sum(mat[,1]))), ncol=2)
  colnames(out) <- if(is.null(colnames(mat))) paste0("de",1:2) else colnames(mat)
  out}
   

#' Reorganize matrix according to clustering-output
#'
#' Reorganize input matrix as sorted by cluster numbers (and geometric mean) according to vector with cluster names, and index for sorting per cluster and per geometric mean.
#' In case \code{mat} is an array, the 3rd dimension will be considered as 'column' with arguments \code{useColumn} ( and \code{cluNo}, if it designs a 'column' of mat).
#' 
#' 
#' @param mat (matrix or data.frame) main input
#' @param cluNo (positive integer, length to match nrow(dat) initial cluster numbers for each line of 'mat' (obtained by separate clustering or other segmentation) or may desinn column of \code{mat} to use as cluster-numbers
#' @param useColumn (character or integer) the columns to use from \code{mat} as main data (default will use all, exept \code{cluCol} and/or \code{meanCol} if they design columns))
#' @param meanCol (character or integer) alternative summarizing data for intra-cluster sorting (instead of geometric mean)
#' @param addInfo (logical) allows  of columns 'index', 'geoMean' and 'cluNo'  (or array if \code{FALSE}) 

#' @param retList (logical) return as list of matrixes (or array if \code{FALSE}) 
#' @param silent (logical) suppress messages
#' @param callFrom (character) allow easier tracking of messages produced
#' @param debug (logical) additional messages for debugging 
#' @return This function returns a list or array (as 2- or 3 dim) with possible number of occurances for each of the 3 elements in nMax. Read results vertical : out[[1]] or out[,,1] .. (multiplicative) table for 1st element of nMax; out[,,2] .. for 2nd
#' @seealso pairwise combinations \code{\link[utils]{combn}}, clustering \code{\link[stats]{kmeans}} 
#' @examples
#' dat1 <- matrix(round(runif(24),2), ncol=3, dimnames=list(NULL,letters[1:3]))
#' clu <- stats::kmeans(dat1, 5)$cluster
#' reorgByCluNo(dat1, clu) 
#' 
#' dat2 <- cbind(dat1, clu=clu) 
#' reorgByCluNo(dat2, "clu") 
#' @export
reorgByCluNo <- function(mat, cluNo, useColumn=NULL, meanCol=NULL, addInfo=TRUE, retList=FALSE, silent=FALSE, callFrom=NULL, debug=FALSE) {
  ## reorganize input matrix as sorted by cluster numbers (and geometric mean) according to vector with cluster names, and index for sorting per cluster and per geometric mean
  fxNa <- .composeCallName(callFrom, newNa="reorgByCluNo")
  iniCla <- inherits(mat, "data.frame")
  chDim <- dim(mat)
  dataOK <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  if(!isTRUE(silent)) silent <- FALSE
  if(length(chDim) >1) if(all(chDim >1)) dataOK <- TRUE
  ## extract cluNo
  if(dataOK & length(cluNo)==1) {
    cluN <- if(is.character(cluNo)) which(dimnames(mat)[[length(chDim)]]==cluNo) else cluNo
    if(length(chDim) ==3) { cluNo <- mat[,1,cluN]; mat <- mat[,,-cluN] 
    } else { cluNo <- mat[,cluN]; mat <- mat[,-cluN] } }   
  if(dataOK) if(length(cluNo) != nrow(mat)) dataOK <- FALSE 
  ## extract meanCol
  if(dataOK & length(meanCol)==1) {
    meanC <- if(is.character(meanCol)) which(dimnames(mat)[[length(chDim)]]==meanCol) else meanCol
    if(length(chDim) ==3) { meanCol <- mat[,1,meanC]; mat <- mat[,,-meanC] 
    } else { meanCol <- mat[,meanC]; mat <- mat[,-meanC] } }   
  if(dataOK & length(meanCol) >0) {if(length(meanCol) != nrow(mat)) dataOK <- FALSE }
  if(debug) {message(fxNa,"rbc1")}
  ## main
  if(dataOK){    
    if(length(useColumn) <1) useColumn <- 1:ncol(mat)
    mat1 <- if(length(chDim) ==3) mat[,,useColumn] else mat[,useColumn]
    if(length(dim(mat1)) <1) mat1 <- matrix(mat1, ncol=1, dimnames=list(rownames(mat1), colnames(mat)[useColumn]))
    nClu <- length(unique(naOmit(cluNo)))
    if(!isFALSE(addInfo)) {
      ## construct geometric mean for sorting (if not provided externally)
      geoMe <- if(length(meanCol)==nrow(mat)) meanCol else apply(mat1, 1, function(x) prod(x,na.rm=TRUE)^(1/sum(!is.na(x))))
      if(length(dim(geoMe)) >1) geoM <- rowMeans(geoMe, na.rm=TRUE)
      if(debug) {message(fxNa,"rbc2")}
      ## assemble main data
      mat1 <- cbind(mat1, index=1:nrow(mat), geoMean=geoMe)    
      ## 1: split in list, determine clu median, overall score & sort clusters  
      cluL <- by(mat1, cluNo, as.matrix)
      clTab <- table(cluNo)               
      if(length(clTab) < max(cluNo) & !silent) message(fxNa," Note: Some cluster-names seem to be absent (no-consecutive numbers for names) !")
      ## need to correct when single occurance
      if(any(clTab ==1)) for(i in which(clTab ==1)) cluL[[i]] <- matrix(cluL[[i]], nrow=1, dimnames=list(rownames(mat1)[which(cluNo==i)], colnames(mat1)))
      if(debug) {message(fxNa,"rbc3")}
  
      ## sort intra
      cluL <- lapply(cluL, function(x) if(nrow(x) >1) x[order(x[,ncol(x)], decreasing=TRUE),] else x)
      ## sort inter
      cluL <- cluL[order(sapply(cluL, function(x) stats::median(x[,ncol(x)], na.rm=TRUE)), decreasing=TRUE)]        
      names(cluL) <- 1:length(cluL)
      nByClu  <- sapply(cluL, nrow)
    } else {
      fOrg <- if(is.data.frame(mat1)) as.data.frame else as.matrix
      cluL <- by(mat1, cluNo, fOrg)
    }  
    if(debug) {message(fxNa,"rbc4"); rbc4 <- list(mat=mat,cluNo=cluNo,cluL=cluL,nByClu=nByClu,clTab=clTab,geoMe=geoMe,nClu=nClu)}

    if(isTRUE(retList)) { out <- cluL
      for(i in 1:nClu) out[[i]] <- if(!isFALSE(addInfo)) cbind(out[[i]][,-1], cluNo=i) else out[[i]][,-1]
      if(iniCla) out <- lapply(out, convMatr2df, addIniNa=FALSE, silent=silent, callFrom=fxNa) 
    } else {
      if(debug) {message(fxNa,"rbc5")}
      out <- lrbind(cluL, silent=silent, callFrom=fxNa)
      if(!isFALSE(addInfo)) out <- cbind(out, cluNo=rep(1:nClu, nByClu)) }
  } else {out <- NULL; if(!silent) message(fxNa," invalid input, returning NULL")}
  out }
  

#' Adjust Value With Different Decimal Prefixes To Single Prefix Plus Unit
#'
#' This function provides help converting values with with different unit-prefixes to a single prefix-unit type.
#' This can be used to convert a vector of mixed prefixes like 'p' and 'n'.
#' Any text to the right of the unit will be ignored.
#'
#' @details
#' The aim of this function if to allow adjusting a vector containing '100pMol' and '1nMol' to '100pMol' and '1000pMol' for better downstream analysis.
#' Please note that the current version recognizes and converts only interger values; decimals or scientific writing won't be recognized properly.
#' The resultant numeric vector expresses all values as lowest prefix unit level.
#' In case of invalid entries \code{NA}s will be returned.
#'
#' Please note that decimal/comma digits will not be recognized properly, since the function will consider (by default) the decimal sign as just another separator.
#'
#' To avoid special characters (which may not work on all operating-systems) the letter 'u' is used for 'micro'.
#'
#' @param x (character) vector containing digit uunit-prefix and unit terms
#' @param pref (character) multiplicative unit-prefixes, assumes as increasing factors of 1000
#' @param unit (character) unit name, the numeric part may be sepatated by one space-character
#' @param sep (character) separator characters that may appear between integer numeric value and unit description
#' @param minTrimNChar (integer) min number of text characters when trimming adjacent text on left and right of main numeric+prefix+unit
#' @param returnType (character) set options for retuning results : 'NAifInvalid' .. return NA for invalid parts,'allText' .. return initial text if problem, 'trim'
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a character vector (same length as input) with adjusted unified decimal prefix and adjusted numeric content, the numeric content only is also giben in the names of the output
#' @seealso \code{\link{convToNum}}; \code{\link{checkUnitPrefix}}; \code{\link{trimRedundText}}
#' @examples
#' adjustUnitPrefix(c("2.psec abc","20 fsec etc"), unit="sec")
#' 
#' x1 <- c("50_amol", "5_fmol","250_amol","100_amol", NA, "500_amol", "500_amol", "1_fmol")
#' adjustUnitPrefix(x1, unit="mol")                   
#'
#' x2 <- c("abCc 500_nmol ABC", "abEe5_umol", "", "abFF_100_nmol_G", "abGg 2_mol", "abH.1 mmol")
#' rbind( adjustUnitPrefix(x2, unit="mol", returnType="allText") , 
#'   adjustUnitPrefix(x2, unit="mol", returnType="trim"),
#'   adjustUnitPrefix(x2, unit="mol", returnType=""))
#' @export
adjustUnitPrefix <- function(x, pref=c("z","a","f","p","n","u","m","","k","M","G"), unit="sec", sep=c("_","."," ",""), minTrimNChar=0, returnType=c("NAifInvalid","allText"), silent=FALSE, debug=FALSE, callFrom=NULL) {
  ## fix amol/fmol labels
  fxNa <- .composeCallName(callFrom, newNa="adjustUnitPrefix")
  if(isTRUE(debug)) silent <- FALSE else { debug <- FALSE
    if(!isTRUE(silent)) silent <- FALSE }
  datOK <- all(c(length(x), length(pref), length(unit)) >0)
  if(length(returnType) >0 && is.character(returnType)) returnType <- naOmit(returnType)
  if(length(returnType) <1 || !is.character(returnType)) {returnType <- c("NAifInvalid","allText"); if(!silent) message(fxNa,"Invalid entry for 'returnType', setting ti default ('NAifInvalid','allText')")}
  out <- if("NAifInvalid" %in% returnType) rep(NA, length(x)) else x
  if(datOK) {  
    if(length(unit) >1) { if(!silent) message(fxNa,"Argument 'unit' must be of length=1, truncating...")
       unit <- unit[1]}
     chUnit <- grepl(unit, x)
     if(all(!chUnit)) { datOK <- FALSE
       if(!silent) message(fxNa,"Data do NOT contain '",unit,"' anywhere (nothing to do)") 
     } else if(any(!chUnit)) { x[which(!chUnit)] <- NA                        # set entries wo unit to NA
       if(!silent) message(fxNa,"Ignoring ",sum(!chUnit)," entries not containing '",unit,"' ")} 
  }    
  if(datOK) {  
    pref2 <- paste0(pref, collapse="|")
    sep2 <- if("" %in% sep) sep[-which(sep %in% "")] else sep   
    sep3 <- if(length(sep2) <1) protectSpecChar(sep, callFrom=fxNa) else paste0(protectSpecChar(sep2, callFrom=fxNa),"{0,1}")
    if(length(sep3) >1) sep3 <- paste0(sep3, collapse="|")  #,")")
    if(debug) {message(fxNa,"aUP1"); aUP1 <- list(x=x,pref=pref,unit=unit,sep=sep,sep2=sep2,sep3=sep3,pref2=pref2,minTrimNChar=minTrimNChar,returnType=returnType, sep2=sep2,sep3=sep3)}
    txRi <- sub( paste0(".*[[:digit:]]+(", sep3,")(",pref2,"){0,1}", unit), "", x)    # OK   (keep all AFTER digit+sep+pref +unit) 
    pref3 <- paste0("(",paste(unique(pref2), collapse="|"), ")")       # prefixes
     if(debug) {message(fxNa,"aUP1b"); aUP1b <- list()}      #headingTxt=headingTxt,chPref=chPref,txLe=txLe,
    lePat <- paste0("[[:digit:]]+(",sep3,")",pref3,"{0,1}(",sep3,")",unit,".*") 
    chLe <- grepl(lePat, x)                  # see if pattern (digit+sep +pref+unit) exists
    if(any(isFALSE(chLe))) {
      if(!silent) message(fxNa,"Ignoring ",sum(isFALSE(chLe))," instances for not-recognized pattern (eg no numeric content, or not-recognized unit or separator)")
      x[which(isFALSE(chLe))] <- NA }           # correcting/ignoring for not-recognized pattern
    chUnit <- !is.na(x)
    datOK <- sum(chUnit) >0
  }    
  if(datOK) {        
    txLe <- gsub(lePat,"", x)                   #   (keep all heading=toLeft to digits+sep+pref+unit+etc) 
    chPref <- nchar(x)==nchar(txLe)
     if(debug) {message(fxNa,"aUP1d"); aUP1d <- list(x=x,pref=pref,unit=unit,sep=sep,sep2=sep2,sep3=sep3,pref2=pref2,chPref=chPref,txLe=txLe,txRi=txRi)}      #headingTxt=headingTxt,
    xIni <- x 
    x <- mapply(sub, txRi, "", mapply(sub, txLe, "", x))                   # reduce to num +sep+unit
    num <- substr(x, 1, nchar(x)- nchar(sub("[[:digit:]]+","",x)))         # numeric content/part     
    sepPrefUnit <- mapply(sub, num, "", x)
    prefUnit <- sub(paste0("^",sep3),"", sepPrefUnit)
    allSep <- mapply(sub, prefUnit, "", sepPrefUnit)
    allPref <- sub(unit,"", prefUnit)
    dig <- try(as.numeric(num), silent=TRUE)
    if(debug) {message(fxNa,"-- aUP2"); aUP2 <- list()}     # conVal=conVal,
    if(inherits(dig, "try-error")) { if(!silent) message(fxNa,"Invalid entry for x (can't extract numeric part), returning NA")
    } else {
      freqSep <- names(sort(table(allSep), decreasing=TRUE))[1]     # most frequent separator
      ## now check which prefix used, if all prefix same 
      if(length(unique(naOmit(allPref))) >1) {
        if(debug) {message(fxNa, "aUP3, converting ",length(unique(naOmit(allPref)))," different prefixes")}
        molCh <- match(allPref, pref)
        if(any(is.na(molCh) & !is.na(dig))) warning("Non-recognized prefix encountered :",paste(unique(pref2[which(is.na(molCh))])), "Result might be WRONG !")
        minPref <- pref[min(molCh, na.rm=TRUE)]
        ## need to find most frequent separator : if txRi & txLe -> need to remove
        if(debug) {message(fxNa,"-- aUP3b"); aUP3b <- list(x=x,pref=pref,unit=unit,sep=sep,sep2=sep2,sep3=sep3,pref2=pref2,chPref=chPref,dig=dig,molCh=molCh,txLe=txLe,txRi=txRi,freqSep=freqSep)}
        
        ## convert (main)
        decFact <- 1e3                           # assume factor 1000             
        conVa3 <- dig* (decFact^(molCh - min(molCh, na.rm=TRUE)))    # adjust
        out <- if("allText" %in% returnType) {          ## add remaining text
          paste0(if(any(nchar(txLe) >0, na.rm=TRUE)) paste0(txLe, freqSep), conVa3, freqSep, minPref, unit, if(any(nchar(txRi) >0, na.rm=TRUE)) paste0(freqSep, txRi))
        } else { if("trim" %in% returnType)  paste0(trimRedundText(txLe, minNchar=minTrimNChar, callFrom=fxNa), freqSep, conVa3, freqSep, minPref, unit, 
          freqSep, trimRedundText(txRi, minNchar=minTrimNChar, callFrom=fxNa)) else paste0(conVa3, freqSep, minPref, unit) }
        if(any(!chUnit)) out[which(!chUnit)] <- NA
        names(out) <- conVa3                       # numeric value in name
        if(debug) {message(fxNa,"-- aUP4 .."); aUP4 <- list()}                  
      } else {
        out <- if("trim" %in% returnType) {
          paste0(trimRedundText(txLe, minNchar=minTrimNChar, callFrom=fxNa), dig, freqSep, naOmit(allPref)[1], unit, trimRedundText(txRi, minNchar=minTrimNChar, callFrom=fxNa))
        } else x
        names(out) <- dig    
      }
    } 
  }
  out }      
          

#' Check For Values Within Range Of Reference
#'
#' This function checks which values of numeric vector 'x' are within range +/- 'fa' x 'ref' (ie within range of reference). 
#' @param x matrix or data.frame
#' @param fa (numeric) absolute or relative tolerance value (numeric, length=1), interpreted according to 'absRef' as absolute or relative to 'x'(ie fa*ref)
#' @param ref (numeric) (center) reference value for comparison (numeric, length=1), if not given mean of 'x' (excluding NA or non-finite values) will be used
#' @param absRef (logical) return result as absolute or relative to 'x'(ie fa*ref)
#' @param asInd (logical) if TRUE return index of which values of 'x' are within range, otherwise return values if 'x' within range
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allows easier tracking of messages produced
#' @return This function returns a numeric vector containing only the values within range of reference
#' @examples
#' ## within 2.5 +/- 0.7
#' withinRefRange(-5:6,fa=0.7,ref=2.5)                
#' ## within 2.5 +/- (0.7*2.5)
#' withinRefRange(-5:6,fa=0.7,ref=2.5,absRef=FALSE)
#' @export
withinRefRange <- function(x, fa, ref=NULL, absRef=TRUE, asInd=FALSE, silent=FALSE, debug=FALSE, callFrom=NULL) {
  fxNa <- .composeCallName(callFrom, newNa="withinRefRange")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  
  xIni <- x
  if(any(length(fa) !=1, !is.finite(fa), !is.finite(ref))) stop(fxNa," 'fa' and 'ref' must be finite !")
  chFin <- is.finite(as.numeric(x))
  if(sum(chFin) <1) stop(" no finite values found in 'x' !")
  if(sum(!chFin) >0) x[which(!chFin)] <- NA
  x <- as.numeric(x)
  if(is.null(ref)) ref <- sum(x[which(chFin)],na.rm=TRUE)/sum(chFin)
  ## main
  out <- if(absRef) which(abs(x -ref) < fa) else which(abs(x/ref -1) < fa)
  if(asInd) out else xIni[out] }
   

#' Convert p-values to lfdr
#'
#' This function takes a numeric vector of p-values and returns a vector of lfdr-values (local false discovery) using 
#' the package \href{https://CRAN.R-project.org/package=fdrtool}{fdrtool}.
#' Multiple testing correction should be performed with caution, short series of p-values typically pose problems for transforming to lfdr. 
#' The transformation to lfdr values may give warning messages, in this case the resultant lfdr values may be invalid ! 
#' @param x (numeric) vector of p.values
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a (numeric) vector of lfdr values (or \code{NULL} if data insufficient to run the function 'fdrtool')
#' @seealso lfdr from \code{\link[fdrtool]{fdrtool}}, other p-adjustments (multiple test correction, eg FDR) in \code{\link[stats]{p.adjust}} 
#' @examples
#' ## Note that this example is too small for estimating really meaningful fdr values
#' ## In consequence, a warning will be issued.
#' set.seed(2017); t8 <- matrix(round(rnorm(160,10,0.4),2), ncol=8,
#'   dimnames=list(letters[1:20], c("AA1","BB1","CC1","DD1","AA2","BB2","CC2","DD2")))
#' t8[3:6,1:2] <- t8[3:6,1:2]+3   # augment lines 3:6 (c-f) for AA1&BB1
#' t8[5:8,5:6] <- t8[5:8,5:6]+3   # augment lines 5:8 (e-h) for AA2&BB2 (c,d,g,h should be found)
#' head(pVal2lfdr(apply(t8, 1, function(x) t.test(x[1:4], x[5:8])$p.value)))
#' @export
pVal2lfdr <- function(x, silent=TRUE, debug=FALSE, callFrom=NULL) {    ## take vector of p-values and return vector of lfdr-values
  fxNa <- .composeCallName(callFrom, newNa="pVal2lfdr")
  if(isTRUE(debug)) silent <- FALSE else { debug <- FALSE
    if(!isTRUE(silent)) silent <- FALSE }

  if(length(x) >0) {
    chNa <- is.na(x)
    if(any(chNa)) { if(!silent) message(fxNa,"Omitting ",sum(chNa)," NAs !")
      z <- try(as.numeric(naOmit(x)), silent=TRUE) 
    } else z <- x
    rm(chNa)
    if(inherits(z, "try-error")) stop(fxNa,"FAILED to convert 'x' into numeric data !") else {
      if(requireNamespace("fdrtool", quietly=TRUE)) {
        z <- try(fdrtool::fdrtool(z, statistic="pvalue", plot=FALSE, verbose=!silent)$lfdr, silent=TRUE)
        if(inherits(z, "try-error")) { message(fxNa,"FAILED to calulate lfdr !  Check how to use package 'fdrtool' (data too small ?)")
          return(NULL) 
        } else { z <- as.numeric(z)
          lfdr <- rep(NA, length(x))
          lfdr[!is.na(x)] <- z                          #  for returning NA at place of initial NAs
          if(!is.null(names(x))) names(lfdr) <- names(x)
          lfdr }
      } else message(fxNa,"NOTE: package 'fdrtool' not found ! Please install first from CRAN  ... (returning NULL)")
    }
  }
}
      

#' Express difference as ppm
#'
#' This function transforms offset (pariwise-difference) between 'x' & 'y' to ppm (as normalized difference ppm, parts per million, ie (x-y)/y ).
#' This type of expressiong differences is used eg in mass-spectrometry.
#' @param x (numeric) typically for measured variable
#' @param y (numeric) typically for theoretical/expected value (vector must be of same length as 'x')
#' @param nSign (integer) number of significant digits in output
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a numeric vector of (ratio-) ppm values
#' @seealso \code{\link{ratioToPpm}} for classical ppm
#' @examples
#' set.seed(2017); aa <- runif(10,50,900)
#' cbind(x=aa,y=aa+1e-3,ppm=XYToDiffPpm(aa,aa+1e-3,nSign=4)) 
#' @export
XYToDiffPpm <- function(x, y, nSign=NULL, silent=FALSE, debug=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom,newNa="XYToDiffPpm")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE

  if(any(is.na(x))) {if(!silent) message(fxNa," 'x' contains ",sum(is.na(x))," out of ",length(x)," NA alike, omit !"); x <- naOmit(x)}
  if(length(y) >0) if(any(is.na(y))) {if(!silent) message(fxNa," 'y' contains ",sum(is.na(y))," NA alike, omit !"); y <- naOmit(y)}
  if(any(length(x) <1, length(y) <1, length(x)!= length(y))) stop("Length of 'y' (",length(y),") not corresponding to 'x' (",length(x),")")
  x <- 1e6*(x-y)/y
  if(length(nSign)>0) signif(x,as.numeric(nSign)) else x }
   

#' Print matrix-content as plot
#'
#' This function prints all columns of matrix in plotting region for easier inclusion to reports (default values are set to work for output as A4-sized pdf).
#' It was made for integrating listings of text to graphical output to devices like \code{png}, \code{jpeg} or \code{pdf}. 
#' 
#' @details 
#' This function was initially designed for listings with small/medium 1st col (eg couner or index), 2nd & 3rd col small and long 3rd col (like file paths).
#' Obviously, the final number of lines one can pack and still read correctly into the graphical output depends on the size of the device 
#' (on a pdf of size A4 one can pack up to apr. 11O lines). 
#' Of ourse, \code{\link[utils]{Sweave}}, combined with LaTeX, provides a powerful alternative for wrapping text to pdf-output (and further combining text and graphics).
#' Note: The final result on pdf devices may vary depending on screen-size (ie with of current device), the parameters 'colPos' and 'titOffS' may need some refinements.
#' Note: In view of typical page/figure layouts like A4, the plotting region will be split to avoid too wide spacing between rows with less than 30 rows. 
#' 
#' @param matr (matrix) main (character) matrix to display
#' @param colPos (numeric) postion of columns on x-scale (from 0 to 1)
#' @param useCex (numeric) cex expension factor forsiez of text (may be different for each column)
#' @param useAdj (numeric) left/cneter/right alignment for text (may be different for each column)
#' @param useCol color specification for text (may be different for each column)
#' @param titOffS (numeric) offset for title line (ralive to 'colPos')
#' @param silent (logical) suppress messages
#' @param callFrom (character) allow easier tracking of message(s) produced
#' @return This function returns NULL (no R-object returned), print 'plot' in current device only
#' @seealso \code{\link[utils]{Sweave}} for more flexible framework
#' @importFrom graphics par
#' @examples
#' ## as example let's make a listing of file-names and associated parameters in current directory
#' mat <- dir()
#' mat <- cbind(no=1:length(mat),fileName=mat,mode=file.mode(mat),
#'   si=round(file.size(mat)/1024),path=getwd())
#' ## Now, we wrap all text into a figure (which could be saved as jpg, pdf etc)
#' tableToPlot(mat[,-1],colPos=c(0.01,0.4,0.46,0.6),titOffS=c(0.05,-0.03,-0.01,0.06))
#' tableToPlot(mat,colPos=c(0,0.16,0.36,0.42,0.75),useAdj=0.5,titOffS=c(-0.01,0,-0.01,0,-0.1)) 
#' @export
tableToPlot <- function(matr, colPos=c(0.05,0.35,0.41,0.56), useCex=0.7, useAdj=c(0,1,1,0), titOffS=0, useCol=1, silent=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom, newNa="tableToPlot")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  
  opar <- list(mar=graphics::par("mar"))
  on.exit(graphics::par(opar))
  msg <- " requires matrix (or data.frame) with >0 rows and 2-7 columns"
  msg2 <- " number of columns of 'matr' doesn't match with number of elements in 'colPos'"
  if(length(dim(matr)) <2) stop(msg)
  if(is.data.frame(matr)) matr <- as.matrix(matr)
  if(nrow(matr) <1) stop("nothing to display", msg)
  if(ncol(matr) >7 | ncol(matr) <2) stop(msg)
  if(length(colPos) != ncol(matr)) stop(msg2) 
  if(length(titOffS) < ncol(matr)) titOffS <- c(titOffS,rep(0,ncol(matr)-length(titOffS)))
  if(length(useAdj) < ncol(matr)) useAdj <- c(useAdj,rep(0.5,ncol(matr)-length(useAdj)))
  if(length(useCex) < ncol(matr)) useCex <- rep(useCex,ceiling(ncol(matr)/length(useCex)))
  if(length(useCol) < ncol(matr)) useCol <- rep(useCol,ceiling(ncol(matr)/length(useCol)))
  matr <- matr[nrow(matr):1,]            # reverse order since plot starts at bottom
  graphics::par(mar=c(1,1,1,1))
  graphics::plot.new()
  if(nrow(matr) <30) graphics::layout(matrix(1:2), heights=c(nrow(matr)/2,1))
  j <- 1:nrow(matr) +0.1                                               # y-position in plot
  graphics::plot.window(xlim=c(0,1), ylim=c(1,nrow(matr) +1.7))
  titPos <- colPos + titOffS
  ## add col-heads
  graphics::text(titPos, nrow(matr)+1, colnames(matr), cex=sort(c(0.5,max(useCex,na.rm=TRUE) +0.1,1.2))[2])  
  for(i in 1:ncol(matr)) graphics::text(colPos[i], j, matr[,i], cex=useCex[i], col=useCol[i], adj=useAdj[i])
}
   

#' Flexible extraction of columns
#'
#' This function provides flexible checking if a set of columns may be extracted from a matrix or data.frame 'x'.
#' If argument \code{extrCol} is list of character vectors, this allows to search among given options, the first matching name for each vector will be identified. 
#' 
#' @param x (matrix or data.frame) main input (where data should be extracted from)
#' @param extrCol (character, integer or list) columns to be extracted, may be column-names or column index; if is \code{list} each first-level element will be considered as options for one choice
#' @param doExtractCols (logical) if default \code{FALSE} only the column indexes will be returned
#' @param silent (logical) suppress messages
#' @param callFrom (character) allows easier tracking of message(s) produced
#' @return integer-vector (if\code{doExtractCols=FALSE} return depending on input \code{matrix} or \code{data.frame})  
#' @seealso \code{\link[utils]{read.table}}, \code{\link{filterList}}
#' @examples
#' dFr <- data.frame(a=11:14, b=24:21, cc=LETTERS[1:4], dd=rep(c(TRUE,FALSE),2)) 
#' extrColsDeX(dFr,c("b","cc","notThere")) 
#' extrColsDeX(dFr,c("b","cc","notThere"), doExtractCols=TRUE) 
#' extrColsDeX(dFr, list(c("nn","b","a"), c("cc","a"),"notThere")) 
#' @export
extrColsDeX <- function(x, extrCol, doExtractCols=FALSE, callFrom=NULL, silent=FALSE) {
  ## flexible extraction of columns from x
  ## remove any NA
  fxNa <- .composeCallName(callFrom, newNa="extrColsDeX")
  nameX <- deparse(substitute(x))
  nameCol <- deparse(substitute(extrCol))
  if(length(dim(x)) <2) stop("argument ",nameX," should be matrix or data.frame")
  if(any(dim(x) <1)) stop("nothing to do, ",nameX,"  has no lines or columns")
  chNa <- is.na(extrCol)
  if(any(chNa)) extrCol <- extrCol[which(!chNa)]
  ## check if use as integer (or match)
  chExtrCol <- sub("^[[:digit:]]+", "", extrCol)
  chExtrCol <- nchar(chExtrCol) >0
  if(length(chExtrCol) <1) stop("nothing to do")
  if(all(!chExtrCol)) extrCol <- as.integer(extrCol) else {  ## try to find text in colnames
    if(is.null(colnames(x))) stop(" Problem: ",nameX,"  has no colnames !")
    if(!is.list(extrCol)) {
      ## single choice for each column to extract
      extrCo2 <- match(extrCol, colnames(x))
    } else  {
      ## try to locate first of multi-choices
      extrCo2 <- unlist(lapply(extrCol, function(z) naOmit(match(z,colnames(x)))[1]))      
    }  
    ## check for NA due to match
    chNa <- is.na(extrCo2)
    if(any(chNa)) { 
      if(all(chNa)) stop(fxNa," Did not find ANY of the names given in ",nameCol," for extracting !")
      if(isFALSE(silent)) message(fxNa," Can't find column(s) ",pasteC(extrCol[which(chNa)],quote="'")," in ",nameCol)
      extrCo2 <- extrCo2[which(!chNa)] }
    extrCol <- extrCo2   
    }
  if(isTRUE(doExtractCols)) {
    if(length(extrCol) >1  | is.data.frame(x)) x[,extrCol] else if(length(extrCol)==1) {
      matrix(x[,extrCol], ncol=1, dimnames=list(rownames(x), colnames(x)[extrCol]))} else NULL
  } else extrCol } 
   

#' Convert matrix (eg with redundant) row-names to data.frame 
#'
#' This function provides flexible converting of matrix to data.frame. 
#' For example repeated/redundant rownames are not allowed in data.frame(), thus the corresponding column-names have to be renamed using a counter-suffix.
#' In case of non-redundant rownames, a new column 'addIniNa' will be introduced at beginning to document the initial (redundant) rownames, 
#' non-redundant rownames will be created.
#' Finally, this functions converts the corrected matrix to data.frame and checks/converts columns for transforming character to numeric if possible. 
#' If the input is a data.frame containing factors, they will be converted to character before potential conversion.
#' Note: for simpler version (only text to numeric) see from this package \code{.convertMatrToNum} .
#' @param mat matrix (or data.frame) to be converted
#' @param addIniNa (logical) if \code{TRUE} an additional column ('ID') with rownames will be added at beginning
#' @param duplTxtSep (character) separator for enumerating replicated names
#' @param silent (logical) suppres messages
#' @param callFrom (character) allow easier tracking of message(s) produced
#' @return This functions returns a data.frame equivalent to the input matrix, an additional column named 'ID' will be added for initial rownames
#' @seealso \code{\link[base]{numeric}}, for simpler version (only text to numeric) see from this package \code{.convertMatrToNum}
#' @examples
#' dat1 <- matrix(1:10, ncol=2)
#' rownames(dat1) <- letters[c(1:3,2,5)]
#' ## as.data.frame(dat1)  ...  would result in an error
#' convMatr2df(dat1)
#' 
#' df1 <- data.frame(a=as.character((1:3)/2), b=LETTERS[1:3], c=1:3)
#' str(convMatr2df(df1))
#' 
#' df2 <- df1; df2$b <- as.factor(df2$b)
#' str(convMatr2df(df2))
#' @export
convMatr2df <- function(mat, addIniNa=TRUE, duplTxtSep="_", silent=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom, newNa="convMatr2df")
  iniNa <- rownames(mat)
  chNR <- any(duplicated(iniNa)) & length(iniNa) >0              # non-redundant ?
  out <- NULL
  ## check&treat duplicated rownames
  if(is.matrix(mat)) {
    if(chNR){
      nrNa <- treatTxtDuplicates(iniNa, onlyCorrectToUnique=TRUE, sep=duplTxtSep, silent=silent,callFrom=fxNa)
      rownames(mat) <- NULL
      mat <- data.frame(ID=iniNa, mat, stringsAsFactors=FALSE)
      rownames(mat) <- nrNa
    } else {
      mat <- data.frame(ID=if(is.null(iniNa)) rep(NA,nrow(mat)) else iniNa, mat, stringsAsFactors=FALSE) }
  }
  ##  try converting columns from factor or text to numeric
  if(is.data.frame(mat)) {   
    chFa <- sapply(mat, inherits, "factor")
    if(any(chFa)) for(i in which(chFa)) mat[,i] <- as.character(mat[,i])
    chNum <- sapply(mat, inherits, c("integer","numeric")) 
    chTx <- sapply(mat, inherits, "character")
 
    if(colnames(mat)[1]=="ID") chTx[1] <- FALSE
    ## limit trying to columns conataining digits in all elements
    if(any(chTx)) chTx[which(chTx)] <- if(sum(chTx)==1) all(grepl("[[:digit:]]",mat[,which(chTx)])) else grepl("[[:digit:]]",mat[,which(chTx)]) 
    if(any(chTx)) for(i in which(chTx)) { tmp <- try(suppressWarnings(as.numeric(mat[,i])), silent=TRUE)
      if(!inherits(tmp, "try-error")) if(!all(is.na(tmp))) mat[,i] <- tmp }
  } else { mat <- NULL
    if(!silent) message(fxNa,"Can't convert object of class ",pasteC(class(mat), quoteC="'")," to data.frame, returning NULL") }
  mat }

#' Check if vector may be numeric content
#'
#' This function allows to checking if a given vector may be numeric content
#' 
#' @param x (numeric vector) main input
#' @param pattern (character)  custom pattern to check
#' @return This functions returns a logical/boolean vector for each of the elements of 'x'
#' @seealso  \code{\link[base]{numeric}}; \code{\link{convMatr2df}}
#' @examples
#' .mayBeNum(c(3:6))
#' @export
.mayBeNum <- function(x, pattern=NULL) {
  ## test if values of (simple) char vector may be numeric; return index of suitable values
  ## 'pattern' .. pattern of regular expressions (default for pos & neg values incl '.' as decimal)
  ## won't work with scientif annotations or heading or tailing spaces !!
  if(is.factor(x)) x <- as.character(x)
  if(is.null(pattern)) pattern <- "(^([0-9]+)|(^[+-][0-9]+)|(^\\.[0-9]+)|(^0\\.[0-9]+))((\\.[0-9]+)?)(([eE][+-]?[0-9]+)?)$" #"(^([0-9]+)|(^[+-][0-9]+)|(^\\.[0-9]+))((\\.[0-9]+)?)(([eE][+-]?[0-9]+)?)$"
  all(grepl(pattern, x)) }
   

#' Compare Means Of Two Vectors By Permutation Test
#'
#' Run coin-flipping like permutation tests (to compare difference of 2 means: 'x1' and 'x2') without any distribution-assumptions.
#' This function uses the package \href{https://CRAN.R-project.org/package=coin}{coin}, if not installed, the function will return NULL and give a warning.
#' 
#' @param x1 numeric vector (to be compared with vector 'x2')
#' @param x2 numeric vector (to be compared with vector 'x1')
#' @param orient (character) may be "two.sided","greater" or "less"
#' @param nPerm (integer) number of permutations
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns an object of "MCp" class numeric output with p-values 
#' @seealso \code{oneway_test} in \code{\link[coin]{LocationTests}} 
#' @examples
#' coinPermTest(2, 3, nPerm=200)
#' @export
coinPermTest <- function(x1, x2, orient="two.sided", nPerm=5000, silent=FALSE, debug=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom, newNa="coinPermTest")
  if(isTRUE(debug)) silent <- FALSE else { debug <- FALSE
    if(!isTRUE(silent)) silent <- FALSE }
  if(requireNamespace("coin", quietly=TRUE)) {
    tmp <- if(identical(orient,"greater")) try(coin::oneway_test(c(x1,x2) ~ factor(rep(c("A","B"), c(length(x1), length(x2)))),
      alternative="greater", distribution=coin::approximate(B=nPerm)), silent=TRUE) else if(any(identical(orient, "fewer"), identical(orient, "less"))) {
        try(coin::oneway_test(c(x1, x2) ~ factor(rep(c("A", "B"), c(length(x1), length(x2)))),
          alternative="less", distribution=coin::approximate(B=nPerm)), silent=TRUE) } else {
        try(coin::oneway_test(c(x1, x2) ~ factor(rep(c("A", "B"), c(length(x1), length(x2)))), distribution=coin::approximate(B=nPerm)),silent=TRUE) }
    if(inherits(tmp, "try-error")) { warning(fxNa,"Unable to run coin::oneway_test() !"); out <- NULL    
    } else out <- coin::pvalue(tmp)
    out 
  } else {  
    if(!silent) message(fxNa,"Package 'coin' not found, please install from CRAN; returning NULL")
  }     
}
    

#' (upper) pairwise x,y combinations
#' 
#' \code{upperMaCoord} gets pairwise combinations for 'n' elements; return matrix with x & y coordinates to form all pairwise groups for n elements.
#' But no distinction of 'upper' or 'lower' possible like in \code{\link{triCoord}}
#' @param n (integer) number of elements for making all pair-wise combinations 
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging 
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a 2-column matrix wiyh indexes for all pairwise combinations of 1:n
#' @seealso \code{\link[base]{lower.tri}}, more evolved version \code{\link{triCoord}}
#' @examples
#' upperMaCoord(4)
#' @export
upperMaCoord <- function(n, silent=FALSE, debug=FALSE, callFrom=NULL) {
  ma <- matrix(1:n,ncol=n,nrow=n)
  cbind(x=ma[upper.tri(ma)],y=t(ma)[upper.tri(ma)]) }
    

#' Count From Two Vectors Number Of Values Close Within Given Limits
#'
#' This functions summarizes the serach of similar (or identical) numeric values from 2 initial vectors, it 
#' evaluates the result from initial search run by findCloseMatch(), whose output is a less convenient list.
#' \code{countCloseToLimits} checks furthermore how many results within additional (more stringent)
#' distance-limits may be found and returns the number of distance values within the limits tested.
#' Designed for checking if threshold used with findCloseMatch() may be set more stringent, eg  when searching reasonable FDR limits ...
#'
#' @param closeMatch (list) output from findCloseMatch(), ie list indicating which instances of 2 series of data have close matches
#' @param limitIdent (numeric) max limit or panel of threshold values to test (if single value, in addtion a panel with values below will be tested)
#' @param prefix (character)  prefix for names of output
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns an integer vector with counts for number of list-elements with at least one absolue value below threshold, names
#' @seealso \code{\link[wrMisc]{findCloseMatch}}
#' @examples
#' set.seed(2019); aa <- sample(12:15,20,repl=TRUE) +round(runif(20),2)-0.5
#' bb <- 11:18
#' match1 <- findCloseMatch(aa,bb,com="diff",lim=0.65)
#' head(match1)
#' (tmp3 <- countCloseToLimits(match1,lim=c(0.5,0.35,0.2)))
#' (tmp4 <- countCloseToLimits(match1,lim=0.7))
#' @export
countCloseToLimits <- function(closeMatch, limitIdent=5, prefix="lim_", silent=FALSE, debug=FALSE, callFrom=NULL) {
  fxNa <- .composeCallName(callFrom,newNa="countCloseToLimits")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  limitIdent <- unique(limitIdent)
  if(length(limitIdent) ==1) {x <- floor(log10(signif(limitIdent,1)))
    x <- c(10^c((x-1):x), 10^c((x-1):x)/5, 4 *round(seq(limitIdent/40,limitIdent/4, length.out=20),2), limitIdent)     # default series of limits
    limitIdent <- unique(signif(sort(x), digits=5)) }
  if(length(closeMatch) <1 || length(limitIdent) <1) { out <- rep(NA,length(limitIdent))
  } else {
    out <- rowSums(sapply(closeMatch,function(z) (min(abs(z)) <= limitIdent)))}
  if(is.null(names(out))) names(out) <- paste0(prefix, limitIdent)  
  out }
  

#' Check length of vector
#'
#' \code{checkVectLength} checks argument 'x' for expected length 'expeL' and return either message or error when expectation not met. 
#' May be used for parameter ('sanity') checking in other user front-end functions.
#' @param x (numeric or charcter vector) input to check length
#' @param expeL (numeric) expected length
#' @param stopOnProblem (logical) continue on problems with message or stop (as error message)
#' @param silent (logical) suppress messages
#' @param debug (logical) display additional messages for debugging
#' @param callFrom (character) allow easier tracking of message(s) produced
#' @return This function returns \code{NULL}; it produces either error-message if length is not OK or optional message if length is OK 
#' @examples
#' aa <- 1:5; checkVectLength(aa,exp=3)
#' @export
checkVectLength <- function(x, expeL=1, stopOnProblem=FALSE, silent=FALSE, debug=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom, newNa="checkVectLength")
  argN <- deparse(substitute(x))
  msg <- " argument 'expeL' should be numeric of length 1 ; resetting to default =1"
  if(!is.finite(expeL)) {expeL <- 1; message(fxNa,msg)}
  if(length(x) != expeL) {
    msg <- paste0(" Argument '",argN,"' doesn't fit to expected length of ",expeL)
    if(stopOnProblem) stop(fxNa,msg) else {if(!silent) message(fxNa,msg)} }
  }

#' Convert numeric vector to matrix
#'
#' Take (numeric) vector and return matrix, if 'colNa' given will be used as colname
#' 
#' @param x (numeric or character) main input
#' @param colNa (integer) design the comumn-name to be given
#' @param rowsKeep (logical) is \code{TRUE} make matrix of 1 column, otherwise of 1 row
#' @return matrix
#' @seealso  \code{\link[base]{matrix}}
#' @examples
#' .vector2Matr(c(3:6))
#' @export
.vector2Matr <- function(x, colNa=NULL, rowsKeep=TRUE) {
  ## take (numeric) vector and return matrix, if 'colNa' given will be used as colname
  nameX <- deparse(substitute(x))
  if(is.factor(x)) x <- as.character(x)
  if(length(dim(x)) <2) x <- if(rowsKeep) matrix(x, ncol=1, dimnames=list(names(x), if(length(colNa) >0) colNa[1] else nameX)) else {
   matrix(x, nrow=1, dimnames=list( if(length(colNa) >0) colNa[1] else nameX), names(x))}
  x }

#' Convert numeric matrix to numeric
#'
#' Take matrix and return vector
#' 
#' @param matr (matrix) main input
#' @param useCol (integer) design the comumns to be used
#' @return numeric vector
#' @seealso  \code{\link[base]{matrix}}
#' @examples
#' .convertMatrToNum(matrix(1:6, ncol=2))
#' @export
.convertMatrToNum <- function(matr, useCol=NULL){
  ## convert all (or selected by 'useCol') colums of matrix (or data.frame) to matrix of numeric data
  ## by default ('useCol' as NULL) all columns will be used
  ## columns with text data will be returned as NAs
  ## columns not covered/mentioned in useCol will only be transformed to matrix
  ## for more elaborate function see convMatr2df()
  if(is.null(useCol)) {
    suplMa <- NULL
    useCol <- 1:ncol(matr)
  } else {                 # start with testing validity of values given
    msg1 <- "'useCol' should design the columns of 'matr' to be used for conversion.  Invalid entry"
    if(is.numeric(useCol)) {
      if(sum(is.finite(useCol)) < length(useCol) || max(useCol) > length(matr)) stop(msg1)
    } else {               # if real col-name given, convert to number
      useCol <- naOmit(match(as.character(useCol), colnames(matr)))
      if(length(useCol) <1) stop(" no valid columns-names found") }
    suplMa <- matr[,-1*useCol]
  }
  dimNa <- dimnames(matr)
  out <- matrix(as.numeric(as.character(as.matrix(matr[,useCol]))), nrow=length(dimNa[[1]]), ncol=length(useCol))    # robust to factors
  if(!is.null(suplMa)) out <- cbind(as.matrix(suplMa), out)
  dimnames(out) <- dimNa
  out }

#' Check Factor
#'
#' This function was designed to check a factor object
#' 
#' @param fac (factor) main input
#' @param facNa (character) level-names
#' @param minLev (integer) minium number of levels
#' @param silent (logical) suppress messages
#' @param callFrom (character) allow easier tracking of messages produced
#' @param debug (logical) additional messages for debugging
#' @return This function returns a corrceted/adjusted factor
#' @seealso  \code{\link[base]{factor}}
#' @examples
#' .checkFactor(gl(3,2))
#' @export
.checkFactor <- function(fac, facNa=NULL, minLev=2, silent=FALSE, debug=FALSE, callFrom=NULL){
  ## checking of factors (used for for limma model.matrix)
  objNa <- deparse(substitute(fac))
  fxNa <- .composeCallName(callFrom, newNa=".checkFactor")
  if(is.null(facNa) | length(facNa) <1) facNa <- objNa
  if(length(facNa) >1) facNa <- facNa[1]
  if(!is.factor(fac)) {fac <- as.factor(fac)
    if(!silent) message(fxNa," transforming ",facNa," to ",length(levels(fac))," level-factor  (",
      paste(utils::head(levels(fac)),collapse=", "),if(length(levels(fac)) >6) " ..",")")}
  if(length(levels(fac)) < minLev & !silent) message(fxNa," NOTE : factor ",facNa," contains not required number of levels")
  fac }


#' Add lower caps to character vector
#'
#' This function allows adding all content as lower caps to/of character vector
#' 
#' @param x (character) main input
#' @return This function returns a elongated character vector
#' @seealso  \code{\link[base]{chartr}}
#' @examples
#' .plusLowerCaps(c("Abc","BCD"))
#' @export
.plusLowerCaps <- function(x) unique(c(x,tolower(x)))   # return original content of 'x' plus all in lower caps (non-redundant)


#' Replace Special Characters
#'
#' This function allows replacing special characters
#' Note that (most) special characters must be presented with protection for \code{grep} and \code{sub}.
#' 
#' @param x (character) main input
#' @param findSp (character) special characters to replace (may have to be given as protected)
#' @param replBy (character) replace by
#' @return This function returns a corrceted/adjusted factor
#' @seealso  \code{\link[base]{factor}}
#' @examples
#' .replSpecChar(c("jhjh(ab)","abc"))
#' @export
.replSpecChar <- function(x, findSp=c("\\(","\\)","\\$"), replBy="_") {
  ## in character vector 'x' replace special character 'findSp' and replace by 'replBy'
  ## note that spec characters must pe presented with protection for grep & sub
  if(length(findSp) > 1 & length(replBy)==1) replBy <- rep(replBy, length(findSp))
  for(i in 1:length(findSp)) { xx <- grep(findSp[i],x)
    if(length(xx) >0) x <- gsub(findSp[i],replBy[i],x)}
  x}


#' Check argument names
#'
#' This function allows checking of argument names 
#' 
#' @param x (character) main input
#' @param argNa (character) argument name
#' @param lazyEval (logical) decide if argument should be avaluated with abbreviated names, too
#' @return This function returns a elongated character vector
#' @seealso  \code{\link[base]{chartr}}
#' @examples
#' .checkArgNa("Abc",c("ab","Ab","BCD"))
#' @export
.checkArgNa <- function(x, argNa, lazyEval=TRUE) {
   ## check character vector 'x' if any of its elements may be (lazy avaluation) argument names of vector 'argNa'
   ## potentially calling too many conflicts : eg will call conflict for single character even when saveral 'argNa' start with this letter (ie invalid for calling fx)
   ## return logical vector at length & names of 'x'
   isArgNa <- sapply(x, nchar) <1
   zz <- naOmit(match(argNa, x))
   if(length(zz) >0) isArgNa[zz] <- TRUE
   if(lazyEval) {
     tmp <- .cutStr(argNa[1], startFr=1, reverse=TRUE)[-1]
     tmp <- sapply(x, function(z) z %in% tmp)
     if(any(tmp, na.rm=TRUE)) isArgNa[which(tmp)] <- TRUE
   } else {zz <- naOmit(match(argNa,x)); if(length(zz)>0) isArgNa[zz] <- TRUE}
  isArgNa }
   

#' Mean Of 3 Highest Values
#'
#' This function returns the mean of the top3 highest values.
#' 
#' @details
#' Generally, \code{NA} will be excluded, if all values are \code{NA} this finction will return \code{NULL} ;
#' thus, in case of (entirely) unsuitable data (non-numeric ...)  \code{NULL} will be returned.
#' If data has subgroups you may try a tapply -way. 
#' 
#' @param x (numeric vector) main input
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging 
#' @param callFrom (character) allows easier tracking of messages produced
#' @return This function returns a vector with single numeric value for mean of top3
#'   ( returns \code{NULL} with invalid input or if all input is \code{NA})
#' @seealso \code{\link[base]{mean}}
#' @examples
#' x1 <- c(15:11,NA,16)
#' top3mean(x1)
#' @export
top3mean <- function(x, silent=FALSE, debug=FALSE, callFrom=NULL) {
  ##  mean of the top3 highest values
  ## return vector
  fxNa <- .composeCallName(callFrom, newNa="top3mean")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  datOK <- length(x) >0 
  if(datOK) { if(!is.numeric(x)) x <- try(as.numeric(x), silent=TRUE)
    if(inherits(x, "try-error")) { datOK <- FALSE
      if(debug) message(fxNa,"Main input data are NOT numeric !  (nothing to do)")}
  }
  if(datOK) {
    x <- sort(x, decreasing=TRUE)
    out <- x[1:min(3, length(x))]
    chNA <- is.na(out)
    if(any(chNA) && !silent) message("fxNa","Note : Data contain ",sum(chNA)," NAs")
    out <- if(all(chNA)) NA else mean(out, na.rm=TRUE)
  } else out <- NULL
  out }  
  
 

#' Check For Common Unit-Name in Text
#'
#' This function aims to find a unit abbreviation or name occurring in all elements of a character-vector \code{x}.
#' The unit name may be preceeded by different decimal prefixes (eg 'k','M'), as defined by argument \code{pref} and separators (\code{sep}). 
#' The unit name will be returned (or first of multiple).
#' 
#'
#' @details
#' Basically this function searches the pattern : digit + separator(\code{sep}) + prefix(\code{pref}) + \code{unit} + optional separator2(\code{sep2})
#' and returns the first unit-name/abbreviation found in all elements of \code{x}.
#' 
#' If \code{} if()
#' In case of invalid entries or no common unit-names \code{NULL} will be returned.
#'
#' Please note the 'u' is used for 'micro' since handeling of special characters may not be portal between different operating systems.
#'
#' @param x (character) vector containing digit uunit-prefix and unit terms
#' @param pref (character) multiplicative unit-prefixes, assumes as increasing factors of 1000
#' @param unit (character) unit name, the numeric part may be sepatated by one space-character
#' @param sep (character) separator character(s) that may appear between integer numeric value and unit-prefix
#' @param sep2 (character) separator character(s) after \code{unit}, set to \code{sep2=""} for ignoring characters following \code{unit}
#' @param stringentSearch (logical) if \code{TRUE} only matches with same separators (sep, sep2) pass, otherwise different elements may contain different separators
#' @param na.rm (logical) remove \code{NA} from input
#' @param protSpecChar (logical) protect special characters and use as they are instead of regex-meaning
#' @param inclPat (logical) return list including pattern of successful search
#' @param silent (logical) suppress messages 
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a charcter vector (length=1) with the common unit name, if \code{inclPat=TRUE} it returns a list with $unit and $pattern
#' @seealso \code{\link{convToNum}}; \code{\link{adjustUnitPrefix}}
#' @examples
#' x1 <- c("10fg WW","xx 10fg 3pW"," 1pg 2.0W")
#' checkUnitPrefix(x1)
#' ## different separators between digit and prefix:
#' x2 <- c("10fg WW","xx 8_fg 3pW"," 1 pg-2.0W")
#' checkUnitPrefix(x2, stringentSearch=TRUE)
#' checkUnitPrefix(x2, stringentSearch=FALSE)
#' 
#' x4 <- c("CT_mixture_QY_50_amol_CN_UPS1_CV_Standards_Research_Group",
#'   "CT_mixture_QY_5_fmol_CN_UPS1_CV_Standards_Research_Group")
#'
#' @export
checkUnitPrefix <- function(x, pref=c("a","f","p","n","u","m","","k","M","G","T","P"), unit=c("m","s","sec", "Mol","mol","g","K","cd","A","W","Watt","V","Volt"), sep=c(""," ",";",",","_","."),
  sep2="", stringentSearch=FALSE, na.rm=FALSE, protSpecChar=TRUE, inclPat=FALSE, callFrom=NULL, silent=FALSE, debug=FALSE) {
  ## check if digit + (flexible) prefix + (fixed) unit combination exists in all entries, 
  ## return first 'x' (ie unit) matching (NULL if not any found)
  fxNa <- .composeCallName(callFrom, newNa="checkUnitPrefix")
  if(isTRUE(debug)) silent <- FALSE else { debug <- FALSE
    if(!isTRUE(silent)) silent <- FALSE }
  out <- NULL
  fx1 <- function(y, pr, se, se2) apply(matrix(apply(cbind("[[:digit:]]", rep(se, length(pr)), rep(pr, each=length(se)*length(se2)),  y, rep(rep(se2, length(pr)), each=length(se))),
    1, paste0, collapse=""), ncol=length(pr)), 1, paste, collapse="|" )     # (stringent : fixed) dig + sep + pref + unit(y) + sep2  
  fx3 <- function(y, pr, se, se2) paste0(unique(apply(cbind( "[[:digit:]]", if("" %in% se) { paste0("(",paste(se[-which(se=="")], collapse="|"),"){0,1}")
    } else paste(se, collapse="|"), pr, y, if("" %in% se2) NULL else paste(se2, collapse="|")), 1, paste0, collapse="")), collapse="|")  # pattern less stringent, (ie variable) sep & sep2  

  ## check input & prepare
  if(!isFALSE(na.rm) && length(x) >0) x <- naOmit(unique(x))
  if(length(pref) >0) pref <- naOmit(unique(pref))
  if(length(sep) >0) sep <- naOmit(unique(sep))
  datOK <- all(c(length(x), length(pref), length(unit), length(sep)) >0)
  if(!datOK && !silent) message(fxNa,"Invalid entries: 'x','pref','unit' and/or 'sep' may not be all NA or NULL")
  if(debug) {message(fxNa,"cUPF1"); cUPF1 <- list(x=x,pref=pref,unit=unit,sep=sep,sep2=sep2,na.rm=na.rm,protSpecChar=protSpecChar,inclPat=inclPat)}
  if(datOK) {     
    if("." %in% c(pref, unit)) { datOK <- FALSE
      if(!silent) message(fxNa,"'pref' or 'unit' may NOT contain '.' !") } }
  if(debug) {message(fxNa,"cUPF2"); cUPF2 <- list(x=x,pref=pref,unit=unit,sep=sep,sep2=sep2,na.rm=na.rm,protSpecChar=protSpecChar,inclPat=inclPat)}
  if(datOK) {   
    if(protSpecChar) sep <- protectSpecChar(sep, silent=!debug, callFrom=fxNa) 
    if(length(sep2) <1) sep2 <- sep else {
      sep2 <- if("" %in% sep2) "" else {
        if(protSpecChar) protectSpecChar(unique(sep2), silent=!debug, callFrom=fxNa) else unique(sep2)} } 
  if(debug) {message(fxNa,"cUPF3"); cUPF3 <- list(x=x,pref=pref,unit=unit,sep=sep,sep2=sep2,na.rm=na.rm,protSpecChar=protSpecChar,inclPat=inclPat)}

    ## MAIN :check pattern per unit
    chPat <- lapply(unit, fx1, pr=pref, se=sep, se2=sep2)
    names(chPat) <- unit
    chPa2 <- lapply(chPat, sapply, grepl, x)
    if(debug) {message(fxNa,"cUPF4"); cUPF4 <- list(x=x,pref=pref,unit=unit,sep=sep,sep2=sep2,na.rm=na.rm,protSpecChar=protSpecChar,inclPat=inclPat,chPat=chPat,chPa2=chPa2)}
    fxCh <- function(z) if(length(dim(z)) >1) any(colSums(z)==nrow(z)) else {if(length(x) >1) all(z) else any(z)}
    chUnit <- unlist(lapply(chPa2, fxCh))
    if(debug) { message(fxNa,"colSu : ", paste(if(length(dim(chUnit)) >1) colSums(chUnit) else sum(chUnit), collapse=" "), "  cUP5")
      cUP5 <- list(x=x, pref=pref,unit=unit,sep=sep,sep2=sep2,stringentSearch=stringentSearch,na.rm=na.rm,protSpecChar=protSpecChar, chUnit=chUnit,fx1=fx1,fx3=fx3, out=out) }
    if(any(chUnit, na.rm=TRUE)) {
       ii <- which.max(nchar(names(chUnit))[which(chUnit)])    # in case of multiple hits choose (1st of) longest (supposedly entire unit-name)
       out <- if(isTRUE(inclPat)) list(unit=names(chUnit)[which(chUnit)][ii], 
       pattern=colnames(chPa2[[which(chUnit)]])[which(colSums(chPa2[[which(chUnit)]])==length(x))]) else names(chUnit)[which(chUnit)][ii]} 
    if(length(out) <1 && isFALSE(stringentSearch)) {
      ## now check less stringent, ie any combination of separators
      if(debug) message(fxNa,"Entering less stringent search ..  cUP3")
      chPat <- sapply(unit, fx3, pr=pref,se=sep,se2=sep2)
      chUnit <- colSums(sapply(chPat, grepl, x)) ==length(x)
      if(debug) message(fxNa,"Checking less stringent (variable sep & sep2) : ", any(chUnit))
      if(any(chUnit, na.rm=TRUE)) { out <- names(chPat)[which(chUnit)][1]
        if(isTRUE(inclPat)) out <- list(unit=out, pattern=chPat[which(chUnit)])} 
    }
  } else if(!silent) message(fxNa,"Invalid entries - nothing to do")
  out } 
     

#' Cut 3-dim array in list of matrixes (or arrays) similar to organizing into clusters
#'
#' \code{cutArrayInCluLike} cuts 'dat' (matrix,data.frame or 3-dim array) in list (of appended lines) according to 'cluOrg', 
#'  which serves as instruction which line of 'dat' should be placed in which list-element (like sorting according to cluster-numbers). 
#' @param dat array (3 dim)
#' @param cluOrg (factor) organization of lines to clusters
#' @param silent (logical) suppress messages
#' @param debug (logical) display additional messages for debugging
#' @param callFrom (character) allow easier tracking of message(s) produced
#' @return This function retruns a list of matrixes (or arrays)
#' @examples
#' mat1 <- matrix(1:30,nc=3,dimnames=list(letters[1:10],1:3))
#' cutArrayInCluLike(mat1,cluOrg=factor(c(2,rep(1:4,2),5)))
#' @export
cutArrayInCluLike <- function(dat, cluOrg, silent=FALSE, debug=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom, newNa="cutArrayInCluLike")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  msg2 <- " 'cluOrg' should be factor or numeric vector corresponding to lines of 'dat', ie instruction which line of 'dat' to put in which list-element (cluster)"
  che <- dim(dat)
  if(!length(che) %in% c(2,3)) stop(fxNa,"Expecting matrix or data.frame of 2 or array of 3 dimensions")
  if(is.null(cluOrg)) stop(fxNa,msg2)
  if(length(cluOrg) != nrow(dat)) stop(fxNa,msg2)
  uniqNa <- unique(naOmit(cluOrg))
  out <- list()
  for(i in 1:length(uniqNa)) out[[i]] <- if(length(che) ==3){
    dat[which(cluOrg==uniqNa[i]),,]
  } else {
    dat[which(cluOrg==uniqNa[i]),] }
  names(out) <- uniqNa
  out }
  
#' Combine annotation information from list of matrixes
#'
#' This function allows to combine information (annotation) from list of matrixes (ie replace when NA), using always the columns specified in 'useCol' (numeric)
#' 
#' @param lst (list) main input
#' @param useCol (numeric vector) which columns should be used
#' @param silent (logical) suppress messages
#' @param callFrom (character) allow easier tracking of messages produced
#' @param debug (logical) additional messages for debugging
#' @return This function returns a single matrix of combined (non-redundant) info
#' @seealso  used in \code{\link{cutArrayInCluLike}}
#' @examples
#' .datSlope(c(3:6))
#' @export
.combineListAnnot <- function(lst, useCol=1:2, silent=FALSE, debug=FALSE, callFrom=NULL){
  ## combine information (annotation) from list of matrixes (ie replace when NA), using always the columns specified in 'useCol' (numeric)
  ## the 1st elment of 'useCol' will be used as to select column used as key for merging
  ## return single matrix of combined (non-redundant) info
  fxNa <- .composeCallName(callFrom, newNa=".combineListAnnot")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  if(length(useCol) <2) stop(fxNa,"Need at least 2 columns numbers in 'useCol'  (1st col specified used as key)")
  cheDim <- sapply(lst, dim)
  if(is.list(cheDim)) {
    lst <- lst[sapply(cheDim, length) >1]
    cheDim <- sapply(lst, dim)}
  if(is.list(cheDim)) {
    cheDim3 <- sapply(cheDim, length)
    if(any(cheDim3 >2)) message(fxNa,"So far not coded for arrays with >2 dims")
    lst <- lst[sapply(cheDim,length) <3] }
  if(length(lst) <1) stop(fxNa,"No (valuable) matrixes or data.frames found in 'lst'")
  cheDim <- sapply(lst, dim)
  if(any(cheDim[1,] <0)) {message(fxNa,"Omiting 0-row elements"); lst <- lst[cheDim[1,] >0]}
  if(any(max(useCol) > cheDim[2,])) stop(fxNa,"Column no specified in 'useCol' seems not to exist")
  ## main
  colNa1 <- colnames(lst[[1]])[useCol]
  if(!silent) message(fxNa," .combineListAnnot() .. checking ",length(lst)," list-elements;   '",
    colNa1[1],"' will be used as key")
  out <- as.matrix(lst[[1]][,useCol])
  if(length(lst) >1) for(i in 2:length(lst)) {
    compColNa <- sort(colnames(lst[[i]])[useCol]) == sort(colNa1)    # compare col-names (too strict ??)
    if(all(compColNa)) {              # for optional testing if col-names do indeed match
      out <- as.matrix(merge(out,lst[[i]][,useCol], by=colNa1[1],all=TRUE))
      NaLi <- which(is.na(out[,2]))
      for(j in 2:length(useCol)) out[NaLi,j] <- out[NaLi,length(useCol)+j-1] }
    out <- out[,1:length(useCol)] }
  stripColNa <- sub(".x$","",colnames(out))
  if(length(unique(stripColNa)) ==ncol(out)) colnames(out) <- stripColNa
  out }

#' Check list of arrays for consistent dimensions of all arrays
#'
#' This function allows to check list of arrays for consistent dimensions of all arrays
#' 
#' @param arrLst (list) main input
#' @param arrNDim (integer) number of dimensions for arrays
#' @param fxName (character) this name will be given in message
#' @param varName (character) 
#' @param silent (logical) suppress messages
#' @param callFrom (character) allow easier tracking of messages produced
#' @param debug (logical) additional messages for debugging
#' @return list
#' @seealso  used in \code{\link{cutArrayInCluLike}}
#' @examples
#' .datSlope(c(3:6))
#' @export
.checkConsistentArrList <- function(arrLst, arrNDim=3, fxName=NULL, varName=NULL, silent=FALSE, debug=FALSE, callFrom=NULL) {
  ## check list of arrays for consistent dimensions of all arrays
  ## arrNDim .. number of dimensions for arrays
  ## fxName will be given in message
  fxNa <- .composeCallName(callFrom, newNa=".checkConsistentArrList")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  che <- sapply(arrLst, dim)
  if(!is.null(fxName)) fxName <- paste(fxName,": ")
  if(!is.null(varName)) varName <- "'arrLst'"
  if(is.list(che)) stop(fxName,"Format of '",varName,"' seems not consistent")
  if(any(nrow(che) != arrNDim)) stop(fxName,"Expecting list of ",paste(arrNDim,collapse=", ")," dim arrays in ",varName)
  if(any(apply(che, 1, function(x) length(unique(x)))) !=1) message(fxName,"Format of ",varName," seems not consistent")
  }

#' Summarize along columns of multiple arrays in list
#'
#' This function allows summarizing along columns of multiple arrays in list
#' 
#' @param arrLst (list) main input
#' @param sumType (character)
#' @param arrOutp (logical)
#' @param signifDig (integer)
#' @param formatCheck (logical)
#' @param silent (logical) suppress messages
#' @param callFrom (character) allow easier tracking of messages produced
#' @param debug (logical) additional messages for debugging
#' @return array (1st dim will be summary along cols, rows will be layers of 3rd array-dim 
#' @seealso used in \code{\link{cutArrayInCluLike}}
#' @examples
#' .datSlope(c(3:6))
#' @export
.arrLstMean <- function(arrLst, sumType="mean", arrOutp=FALSE, signifDig=3,formatCheck=FALSE,silent=FALSE, debug=FALSE, callFrom=NULL){
  ## summarize along columns of mult arrays in list
  ##  can summarize as median or mean
  ## resultant 1st dim will be summary along cols, rows will be layers of 3rd array-dim ie dim(arrLst[[1]])[3]
  ## eg expr data as clusters (list-elements)
  fxNa <- .composeCallName(callFrom, newNa=".arrLstMean")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  if(formatCheck) .checkConsistentArrList(arrLst, fxName=fxNa)   
  diNa <- dimnames(arrLst[[1]])
  out <- if(sumType=="median") {
    lapply(arrLst, function(x) t(signif(apply(x,c(2,3), stats::median, na.rm=TRUE), signifDig)) )
   } else {
    lapply(arrLst, function(x) t(signif(apply(x, 3, colMeans, signifDig))) ) }
  if(is.list(out)) names(out) <- names(arrLst)
  if(arrOutp) out <- aperm(array(unlist(lapply(out,t)), dim=c(dim(arrLst[[1]])[-1], length(arrLst)),
    dimnames=list(diNa[[2]], diNa[[3]], names(arrLst))), c(2,1,3))  
  out }

#' Summarize along columns of mult arrays in list
#'
#' This function allows summarizing along columns of mult arrays in list
#' 
#' @param arrLst (list) main input
#' @param arrOutp (logical)
#' @param signifDig (integer)
#' @param formatCheck (logical)
#' @param silent (logical) suppress messages
#' @param callFrom (character) allow easier tracking of messages produced
#' @param debug (logical) additional messages for debugging
#' @return array (1st dim will be summary along cols, rows will be layers of 3rd array-dim ie dim(arrLst[[1]])[3])
#' @seealso used in \code{\link{cutArrayInCluLike}}
#' @examples
#' .datSlope(c(3:6))
#' @export
.arrLstSEM <- function(arrLst, arrOutp=FALSE, signifDig=3, formatCheck=FALSE, silent=FALSE, debug=FALSE, callFrom=NULL){
  ## summarize along columns of mult arrays in list
  ##  can summarize as median or mean
  ## resultant 1st dim will be summary along cols, rows will be layers of 3rd array-dim ie dim(arrLst[[1]])[3]
  ## eg expr data as clusters (list-elements)
  fxNa <- .composeCallName(callFrom, newNa=".arrLstSEM")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  if(formatCheck) .checkConsistentArrList(arrLst, fxName=fxNa)
  diNa <- dimnames(arrLst[[1]])
  out <- lapply(arrLst, function(x) t(apply(aperm(x,c(2,1,3)), 3, rowSEMs)) )
  if(is.list(out)) names(out) <- names(arrLst)
  if(arrOutp) out <- aperm(array(unlist(lapply(out,t)), dim=c(dim(arrLst[[1]])[-1],length(arrLst)),
    dimnames=list(diNa[[2]], diNa[[3]], names(arrLst))), c(2,1,3))  
  out }
  

#' Extract Pairwise Testing Setup  
#'
#' @description
#' This function allows extacting, checking and combinig elements defining labels for pairwise comparisons.
#' 
#'  
#'  
#' @details 
#' This function was designed to support functions exploiting pairwise testing results like plotting functions for Volcano-Plots etc.
#' 
#' Initially the function will check if the object given as argument \code{pwGrpNa} is an MAarrayLM-objects object and/or was created using \code{moderTestXgrp}. 
#' If yes, the function will try to extract names of groups, pairwise combinations and separator used from $setup, if present.
#' If no $setup found this function will check through several elements in MAarrayLM-objects to gather all elements needed.
#' 
#' 2) The separator \code{sep} is given and exact matches at both sides will be searched.
#' However, if the character(s) from \code{sep} do appear inside \code{pwGrpNa} no matches will be found.
#'
#' If some \code{pwGrpNa} are not found in \code{grpNa} this will be marked as NA.  
#' 
#' Finally, if no testing result is given as input, basic parameters group-names, pairwise names and the separator to be used can be combined into a list.
#' 
#' @param pwGrpNa (character) the names of pairwise-testing (ie 'concatenated' \code{pwGrpNa} using the separator \code{sep} ) 
#' @param grpNa (character) the names of the groups of replicates (ie conditions) used to test
#' @param sep (NULL or character, length=1) if not \code{NULL} the character-vector (of length=1) given will be used to split \code{grpNa} using \code{stringsplit()} 
#' @param combPwSep (character) sequence of separators for building combined names; may be combPwSep='combine1' or 'combine2'; will be used in pwSeparatorList()  
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a list with $ind with matrix of indexes to (sorted) \code{pwGrpNa} for each pairwise comparison; 
#'   $pwGrpNames with matrix of \code{pwGrpNa} for each pairwise comparison;  
#'   $sep for the separator found ; $pwGrpNa which recapitulates (sorted) \code{pwGrpNa} as factor
#' @seealso (for running multiple pair-wise test) \code{\link{moderTestXgrp}}, \code{\link{convPairwiseSetup}}, \code{\link{presenceFilt}}, \code{\link{getPWseparator}}, \code{\link{indexGroupsFromPW}}, 
#'   (used underneith:) \code{\link[base]{grep}}, \code{\link[base]{strsplit}}
#' @examples
#' grp3 <- factor(rep(LETTERS[c(3,1,4)],c(2,3,3)))
#' set.seed(2017); t8 <- matrix(round(rnorm(208*8,10,0.4),2), ncol=8,
#'   dimnames=list(paste(letters[],rep(1:8,each=26),sep=""), paste0(grp3, c(1:2,1:3,1:3))))
#' test1 <- moderTestXgrp(t8, grp3)  
#' getPairwiseSetup(test1)
#' 
#' 
#' 
#' @export
getPairwiseSetup <- function(pwGrpNa, grpNa=NULL, sep=NULL, combPwSep="combine1", silent=FALSE, debug=FALSE, callFrom=NULL) {
  ##  Understand pairwise experim setup - to be used for exploiting testing-results (eg Volcano-Plot, ...)
  ##
  fxNa <- .composeCallName(callFrom, newNa="getPairwiseSetup")
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  if(!isTRUE(silent)) silent <- FALSE

  if(all(is.na(pwGrpNa))) pwGrpNa <- NULL
  if(any(is.na(grpNa))) grpNa <- naOmit(grpNa)
  out <- useComparisonNa <- pwGrpNa1 <- grpNa1 <- NULL    # initialize
  singleCompSetup <- FALSE
  
  datOK <- length(pwGrpNa) >0 
  if(datOK) {
    ## pwGrpNa may also be obj from testing
    if(any(c("list","MArrayLM") %in% class(pwGrpNa))) {

      ## A) sep :  check for present predefined sep  (from wrProteo::testRobustToNAimputation() or wrMisc::moderTestXgrp()    
      ch1 <- length(pwGrpNa$grpSep) ==1      ## need to create $grpSep for  wrProteo::testRobustToNAimputation() or wrMisc::moderTestXgrp()
      ch2 <- length(pwGrpNa$setup$sep) ==1   ## from importFx
      ## other places to check ?
      if(any(ch1, ch2)) {
        sep1 <- if(ch1) pwGrpNa$grpSep else {if(ch2) pwGrpNa$setup$sep}
        if(length(sep) >0 && !identical(sep,sep1)) {message(fxNa,"Note : 'sep' given by user seems to  DIFFER  from 'sep' found in MArrayLM-object")}
        if(length(sep)==0) sep <- sep1
      }
      if(debug) {message(fxNa,"yy0"); yy0 <- list(pwGrpNa=pwGrpNa,grpNa=grpNa,sep=sep)}
      
      ## B) pwGrpNa1 : look for 2dim table of comparisons(names)  in  $setup$useComparisonNa
      if("pwGrpNa" %in% names(pwGrpNa) && length(pwGrpNa$pwGrpNa) >0) pwGrpNa1 <- pwGrpNa$pwGrpNa else {
        if("setup" %in% names(pwGrpNa) && "pwGrpNa" %in% names(pwGrpNa$setup) && length(pwGrpNa$setup$pwGrpNa) >0) pwGrpNa1 <- pwGrpNa$setup$pwGrpNa else {
          ## otherwise, try extracting colnames from testing
          ## B1) test for special case -  single comparison : may not have pw combin in colnames
          if("t" %in% names(pwGrpNa) && ncol(pwGrpNa$t)==2 && colnames(pwGrpNa)[1] =="(Intercept)") {
            ## single comparison case (colnames of pwGrpNa$design are fixed); determine optimal sep and create pairwise name(s)
            ## note that pwGrpNa$design is not reflecting sample-names as usual !!
            singleCompSetup <- TRUE
            if("means" %in% names(pwGrpNa)) sampGrNa <- colnames(pwGrpNa$means) else {
              warning(fxNa,"Unabale obtain names of groups of repliaces, assuming as '1' and '2'")
              sampGrNa <- 1:2
            }
            if(length(sep) !=1) sep <- getPWseparator(compNames=NULL, grp=sampGrNa, potSep=sep, includeGrp=FALSE, silent=silent, debug=debug, callFrom=fxNa)
            useGrp <- sampGrNa[which(colSums(pwGrpNa$design) >0)]
            if(length(useGrp) <2) stop(fxNa,"Unable to find out which sammples were compared in this single-comparison case !")
            if(length(useGrp) >2) warning(fxNa,"Having difficulty understanding SETUP of this single-comparison data ! .. possibly associating wrong labels to data ?")
            pwGrpNa1 <- paste(useGrp, collapse=sep)
          } else {
            ## B2, regular extraction of pwGrpNa1 (multiple pw comparisons)
            if("t" %in% names(pwGrpNa) && length(colnames(pwGrpNa$t)) >0) pwGrpNa1 <- colnames(pwGrpNa$t) else {
              if("p.value" %in% names(pwGrpNa) && length(colnames(pwGrpNa$p.value)) >0) pwGrpNa1 <- colnames(pwGrpNa$p.value) else {
                ## also check $design (would need to concatenate later) ? (gives no of samples & groups) note : $contrasts NOT present in result from wrMisc::moderTest2grp() !!
                warning(fxNa,"Unable to locate 'pwGrpNa' in input")
              }
            }  
          }
        }
      } 
      if(debug) {message(fxNa,"yy1"); yy1 <- list(pwGrpNa=pwGrpNa,grpNa=grpNa,sep=sep,pwGrpNa1=pwGrpNa1)}

      ## C) grpNa : look names of groups (grpNa)
      if("grpNa" %in% names(pwGrpNa) && length(pwGrpNa$grpNa) >0) grpNa1 <- pwGrpNa$grpNa else {
        if("setup" %in% names(pwGrpNa) && "grpNa" %in% names(pwGrpNa$setup) && length(pwGrpNa$setup$grpNa) >0) grpNa1 <- pwGrpNa$setup$grpNa else {
          if("design" %in% names(pwGrpNa) && length(pwGrpNa$design) >0) grpNa1 <- colnames(pwGrpNa$design) else {    #
            if("means" %in% names(pwGrpNa) && length(pwGrpNa$means) >0) grpNa1 <- colnames(pwGrpNa$means) else {
              warning(fxNa,"Unable to locate information for 'grpNa'") }
          }  
        }
      }
      if(length(grpNa1) >0 && length(grpNa) >0) {
        if(identical( unique(sort(as.character(grpNa1))), unique(sort(as.character(grpNa))))) grpNa <- grpNa1 else {
          if(!silent) message(fxNa,"Names of groups extracted from MArrayLM-object  DIFFERENT  to user provided 'grpNa', using initial")
        }
      } else if(length(grpNa1) >0) grpNa <- grpNa1
      if(length(grpNa)==0) warning(fxNa,"Unable to locate/extract 'grpNa' in input")

      ## D) useComparisonNa : recuperate useComparisonNa
      if(length(pwGrpNa$setup$useComparisonNa) >0) useComparisonNa <- pwGrpNa$setup$useComparisonNa     
      if(debug) {message(fxNa,"yy2"); yy2 <- list(pwGrpNa=pwGrpNa,grpNa=grpNa,sep=sep,pwGrpNa1=pwGrpNa1,useComparisonNa=useComparisonNa)}
      
      ## pwGrpNa D) recuperate /restore (core) pwGrpNa  (and dismiss other info attached)
      pwGrpNa <- pwGrpNa1             # dismiss rest of MArrayLM-object 

      if(length(sep)==0) sep <- getPWseparator(grp=pwGrpNa, potSep=pwSeparatorList(type=combPwSep, silent=silent, debug=debug, callFrom=fxNa), includeGrp=FALSE, silent=silent, debug=debug, callFrom=fxNa) 
      #if(length(sep)==0) sep <- getPWseparator(grp=pwGrpNa, potSep=pwSeparatorList(type="combine1"), includeGrp=FALSE, silent=silent, debug=debug, callFrom=fxNa) 
      if(debug) {message(fxNa,"Finished extracting MArrayLM obj   yy3"); yy3 <- list(pwGrpNa=pwGrpNa,grpNa=grpNa,sep=sep,pwGrpNa1=pwGrpNa1,useComparisonNa=useComparisonNa)}
      
      ## finished recuperating info from MArrayLM-obj
    } else { datOK <- length(grpNa) >0 }
    if(length(pwGrpNa) >0) grpNa <- unique(sort(grpNa)) else warning(fxNa,"Unable to isolate 'grpNa' from MArrayLM-object -  nothing to do  ")
    #pwGrpNa <- unique(sort(pwGrpNa))   #else warning(fxNa,"Found 'pwGrpNa' but seems to be of length=0 - nothing to do  ")
    if(debug) {message(fxNa,"yy4"); yy4 <- list(pwGrpNa=pwGrpNa,grpNa=grpNa,sep=sep)}
  }  

  if(datOK) { if(length(sep)==1 && nchar(sep) >0) {  
      ## OK for strspl (sep seems valid) 
      
      if(length(pwGrpNa)==0 && length(dim(pwGrpNa)) !=2) {
        ## suppose combined grp -vector given - need to split

      }
      
      if(length(grpNa) >0) {
        pwGrpNa2 <- if(length(dim(pwGrpNa))==2) apply(pwGrpNa, 1, paste, collapse=sep) else paste(pwGrpNa, collapse=sep)
        if(debug) {message(fxNa,"yy4b"); yy4b <- list(pwGrpNa=pwGrpNa,grpNa=grpNa,sep=sep,pwGrpNa2=pwGrpNa2)}
        
        out <- indexGroupsFromPW(compNames=pwGrpNa2, grp=grpNa, includeGrp=TRUE, potSep=if(length(sep) ==1) sep else NULL, silent=silent, debug=debug, callFrom=fxNa) # uses strspl ; grp may be NULL
        if(all(is.na(out$index)) && ncol(out$index) ==2) out$index <- matrix(1:2, ncol=2, dimnames=list(paste(out$pwGrpNa, collapse=out$sep), 1:2))
      }
      if(debug) {message(fxNa,"yy5"); yy5 <- list(pwGrpNa=pwGrpNa,grpNa=grpNa,sep=sep, out=out)}           
        
    } else {
      ## check for intra-repeating grp
      chIntra <- sapply(grpNa, function(x) grepl(x, pwGrpNa[-1*which(pwGrpNa == x)]))  #
      chIntra2 <- if(length(sep)==1) {any(chIntra) && any(grepl(sep, pwGrpNa))} else any(chIntra)
      if(chIntra2 && !silent) message(fxNa," ",sum(rowSums(chIntra) >0)," elements of grpNa appear inside others (- may pose problems identifying groups from tails")    
      ## try by trimming                                                                                                         
      out <- findHeadAndTail(grpNa, pwGrpNa, silent=silent, debug=debug, callFrom=fxNa)
      if(length(out) !=1) {               ## if failed so far, go for strspl
        out <- indexGroupsFromPW(grpNa, grp=grpNa, includeGrp=TRUE, silent=silent, debug=debug, callFrom=fxNa)   # uses strspl        
      }    
    } 
  }      
  out }                
       

#' Merge Multiple Matrices
#'
#' This function allows merging of multiple matrix-like objects.
#' The matix-rownames will be used to align common elements, either be returning all common elements \code{mode='intersect'} or containg all elements \code{mode='union'} (the result may contains additional \code{NA}s).
#'
#'
#' @details
#' Custom column-names can be given by entering matrices like named arguments (see examples below).
#' The choice of columns tu use may be adopted to each matrix entered, in this case the argument \code{useColumn} may be a list with matrix-names to use or a list of indexes (see examples below).
#'
#' Note, that matrices may contain repeated rownames (see examples, \code{mat3}). In this case only the first of repeated rownames will be considered (and lines of repeated names ignored).
#'
#' @param ... (matrix or data.frame) multiple matrix or data.frame objects may be entered
#' @param mode (character) allows choosing restricting to all common elements (\code{mode='intersect'}) or union  (\code{mode='union'})
#' @param useColumn (integer, character or list) the column(s) to consider, may be \code{'all'} to use all, integer to select specific indexes or list of indexes or colnames for cutom-selection per matrix
#' @param na.rm (logical) suppress \code{NA}s
#' @param extrRowNames (logical) decide whether columns with all values different (ie no replicates or max divergency) should be excluded
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a matrix containing all selected columns of the input matrices to fuse
#' @seealso  \code{\link[base]{merge}},  \code{\link{mergeMatrixList}}
#' @examples
#' mat1 <- matrix(11:18, ncol=2, dimnames=list(letters[3:6],LETTERS[1:2]))
#' mat2 <- matrix(21:28, ncol=2, dimnames=list(letters[2:5],LETTERS[3:4]))
#' mat3 <- matrix(31:38, ncol=2, dimnames=list(letters[c(1,3:4,3)],LETTERS[4:5]))
#'
#' mergeMatrices(mat1, mat2)
#' mergeMatrices(mat1, mat2, mat3, mode="union", useCol=2)
#' ## custom names for matrix-origin
#' mergeMatrices(m1=mat1, m2=mat2, mat3, mode="union", useCol=2)
#' ## flexible/custom selection of columns
#' mergeMatrices(m1=mat1, m2=mat2, mat3, mode="union", useCol=list(1,1:2,2))
#' @export
mergeMatrices <- function(..., mode="intersect", useColumn=1, na.rm=TRUE, extrRowNames=FALSE, silent=FALSE, debug=FALSE, callFrom=NULL) {
  ## merge n matrix (or data.frame) entries, either as all shared lines or as all united/get common lines
  ## 'useColumn' (integer) column used for merging
  inpL <- list(...)
  out <- useColNo <- NULL
  fxNa <- " -> mergeMatrices"    # temporary
  ## separate spectific arguments from all-input (lazy fitting)
  fixArg <- c("mode","useColumn","extrRowNames","silent","debug","callFrom")    # fixed argument names to check (and adjust)
  argL <- match.call(expand.dots = FALSE)$...            # extr arg names, based on https://stackoverflow.com/questions/55019441/deparse-substitute-with-three-dots-arguments
  pMa <- pmatch(names(argL), fixArg)                     # will only find if unique (partial) match
  pMa[which(nchar(names(argL)) <3)] <- NA                # limit to min 3 chars length
  if(any(!is.na(pMa))) for(i in which(!is.na(pMa))) {assign(fixArg[pMa[i]], inpL[[i]]); names(inpL)[i] <- "replaceReplace"}
  chRepl <- names(inpL) %in% "replaceReplace"
  if(any(chRepl)) {inpL <- inpL[-which(chRepl)]; argL <- argL[-which(chRepl)]}
  if(debug) {message(fxNa,"mM0")}

  ## more tests
  if(!isFALSE(na.rm)) silent <- TRUE
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  fxNa <- .composeCallName(callFrom, newNa="mergeMatrices")
  extrRowNames <- try(as.logical(extrRowNames, silent=TRUE))
  if(inherits(extrRowNames, "try-error")) {message(fxNa,"Invalid entry for 'extrRowNames', reset to default =FALSE"); extrRowNames <- FALSE}

  ## adjust names of matrices
  if(length(inpL) >1) {replNa <- if(length(names(inpL)) >0) names(inpL) %in% "" else rep(TRUE, length(inpL))
     if(any(replNa)) names(inpL)[which(replNa)] <- sub("[[:punct:]].*|[[:space:]].*","",as.character(unlist(argL))[which(replNa)]) }

    if(debug) {message(fxNa,"mM1"); mM1 <- list(inpL=inpL, out=out,useColumn=useColumn,extrRowNames=extrRowNames,fixArg=fixArg,argL=argL,pMa=pMa, debug=debug,silent=silent)}
  ## main
  .mergeMatrices(inpL, mode=mode, useColumn=useColumn, extrRowNames=extrRowNames, na.rm=na.rm, argL=argL, silent=silent, debug=debug, callFrom=fxNa)
}


#' Merge Multiple Matrices from List
#'
#' This function allows merging of multiple matrix-like objects from an initial list.
#' The matix-rownames will be used to align common elements, either be returning all common elements \code{mode='intersect'} or containg all elements \code{mode='union'} (the result may contains additional \code{NA}s).
#'
#'
#' @details
#' Custom column-names can be given by entering matrices like named arguments (see examples below).
#' The choice of columns tu use may be adopted to each matrix entered, in this case the argument \code{useColumn} may be a list with matrix-names to use or a list of indexes (see examples below).
#'
#' Note, that matrices may contain repeated rownames (see examples, \code{mat3}). In this case only the first of repeated rownames will be considered (and lines of repeated names ignored).
#'
#' @param matLst (list containing matrices or data.frames) main input (multiple matrix or data.frame objects)
#' @param mode (character) allows choosing restricting to all common elements (\code{mode='intersect'}) or union  (\code{mode='union'})
#' @param useColumn (integer, character or list) the column(s) to consider, may be \code{'all'} to use all, integer to select specific indexes or list of indexes or colnames for cutom-selection per matrix
#' @param na.rm (logical) suppress \code{NA}s
#' @param extrRowNames (logical) decide whether columns with all values different (ie no replicates or max divergency) should be excluded
#' @param silent (logical) suppress messages
#' @param callFrom (character) allow easier tracking of messages produced
#' @param debug (logical) additional messages for debugging
#' @return This function returns a matrix containing all selected columns of the input matrices to fuse
#' @seealso  \code{\link[base]{merge}},  \code{\link{mergeMatrices}} for separate entries
#' @examples
#' mat1 <- matrix(11:18, ncol=2, dimnames=list(letters[3:6],LETTERS[1:2]))
#' mat2 <- matrix(21:28, ncol=2, dimnames=list(letters[2:5],LETTERS[3:4]))
#' mat3 <- matrix(31:38, ncol=2, dimnames=list(letters[c(1,3:4,3)],LETTERS[4:5]))
#'
#' mergeMatrixList(list(mat1, mat2))
#'
#' mergeMatrixList(list(m1=mat1, m2=mat2, mat3), mode="union", useCol=2)
#' @export
mergeMatrixList <- function(matLst, mode="intersect", useColumn=1, na.rm=TRUE, extrRowNames=FALSE, silent=FALSE, debug=FALSE, callFrom=NULL) {
  ## merge list of matrices
  if(!isFALSE(na.rm)) silent <- TRUE
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  fxNa <- .composeCallName(callFrom, newNa="mergeMatrixList")
  if(!is.list(matLst) | length(matLst) <1) stop("Invalid entry for 'matLst'")
  namesX <- deparse(substitute(matLst))
  if(debug) {message(fxNa,"mML0"); mML0 <- list(matLst=matLst, useColumn=useColumn,extrRowNames=extrRowNames,namesX=namesX, debug=debug,silent=silent)}

  ## check names (and provide default names)
  lstNa <- names(matLst)
  if(length(lstNa) <1) {
    namesX2 <- unlist(strsplit(sub("\\)$","",sub("^list\\(","",namesX)), ", "))
    names(matLst) <- if(length(namesX2)==length(matLst)) namesX2 else paste0(sub("[[:punct:]].*|[[:space:]].*","",namesX),"_", 1:length(matLst))}
  if(any("" %in% lstNa)) {
    names(matLst)[which(lstNa %in% "")] <- paste0(sub("[[:punct:]].*|[[:space:]].*","",namesX),"_", which(lstNa %in% ""))
  }
  argL <- names(matLst)
  if(debug) {message(fxNa,"mML1")}

  ## main
  .mergeMatrices(matLst, mode=mode, useColumn=useColumn, extrRowNames=extrRowNames, na.rm=na.rm, argL=argL, silent=silent, debug=debug, callFrom=fxNa)
}


#' Merge Multiple Matrices (main)
#'
#' This function allows merging of multiple matrix-like objects from an initial list.
#'
#' @param inpL (list containing matrices or data.frames) main input (multiple matrix or data.frame objects)
#' @param mode (character) allows choosing restricting to all common elements (\code{mode='intersect'}) or union  (\code{mode='union'})
#' @param useColumn (integer, character or list) the column(s) to consider, may be \code{'all'} to use all, integer to select specific indexes or list of indexes or colnames for cutom-selection per matrix
#' @param extrRowNames (logical) decide whether columns with all values different (ie no replicates or max divergency) should be excluded
#' @param na.rm (logical) suppress \code{NA}s
#' @param argL (list of arguments)
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a matrix containing all selected columns of the input matrices to fuse
#' @seealso \code{\link{mergeMatrixList}},  \code{\link[base]{merge}},  \code{\link{mergeMatrices}} for separate entries
#' @examples
#' mat1 <- matrix(11:18, ncol=2, dimnames=list(letters[3:6],LETTERS[1:2]))
#' @export
.mergeMatrices <- function(inpL, mode="intersect", useColumn=1, extrRowNames=FALSE, na.rm=TRUE, argL=NULL, silent=FALSE, debug=FALSE, callFrom=NULL) {
  ## merge n matrix (or data.frame) entries, either as all shared lines or as all united/get common lines
  ## inpL (list) main input : list of matrices
  fxNa <- .composeCallName(callFrom, newNa=".mergeMatrices")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  
  out <- NULL
  chLe <- sapply(inpL, length)
  if(all(chLe >0)) {
    chDi <- sapply(inpL, ncol)
    if(is.list(chDi)) { useInp <- sapply(chDi, length) >0
      inpL <- inpL[which(useInp)]
      if(!silent) message(fxNa,"Removing ",sum(useInp)," empty elements from input")
      chDi <- sapply(inpL, ncol)}
    if(any(chDi) <1) {
      if(!silent) message(fxNa,"Removing ",sum(chDi <1)," non-matrix like elements from input")
      inpL <- inpL[which(chDi >0)]
      chDi <- sapply(inpL, ncol) }
    chLe <- sapply(inpL, length)       # update
    }

    if(debug) {message(fxNa,"mNM1"); mNM1 <- list(inpL=inpL, out=out,useColumn=useColumn,extrRowNames=extrRowNames,chLe=chLe,chDi=chDi,inpL=inpL)}
    if(all(chLe >0) && sum(duplicated(chDi), na.rm=TRUE)==length(inpL) -1)  {   # all valid entries

    ## final column-names
    useColNa <- NULL
    if(length(inpL) >1) {
      ## names of matrices
      chName <- nchar(names(inpL)) <1
      if(any(chName)) {names(inpL)[which(chName)] <- argL[which(chName)]
        if(debug) message(fxNa,"Adding object-names to ",sum(chName)," matrices as ",pasteC(argL[which(chName)], quoteC="'")) }
      chName <- nchar(names(inpL)) <1                           # update
      ## colnames
      chCoNa <- lapply(inpL, colnames)
      chCoNa2 <- sapply(chCoNa, length) <1
      if(any(chCoNa2)) for(i in which(chCoNa2)) colnames(inpL[[which(chCoNa2)]]) <- 1:ncol(inpL[[which(chCoNa2)]])     # fill empty colnames
    }
    ## check for redundant rownames
    chRNa <- sapply(inpL, function(x) sum(duplicated(rownames(x))) )
    if(any(chRNa >0))  {if(!silent) message(fxNa,"Note : The matrices ",pasteC(names(inpL)[which(chRNa >0)], quoteC="'")," contain redundant rownames (which will be ignored)")}

    ##
    if(identical(useColumn,"all")) { useColumn <- lapply(inpL, function(x) 1:ncol(x))}

    if(debug) {message(fxNa,"mNM2"); mNM2 <- list(inpL=inpL, out=out,useColumn=useColumn,useColNa=useColNa,extrRowNames=extrRowNames,chLe=chLe,chDi=chDi,inpL=inpL)}

    ## prepare empty matrix for results
    if(!identical(mode,"union")) {      # suppose mode is 'intersect'
      ## get common rownames
      useLi <- if(na.rm) naOmit(rownames(inpL[[1]])) else rownames(inpL[[1]])
      if(length(inpL) >1) for(i in 2:length(inpL)) useLi <- intersect(useLi, if(na.rm) naOmit(rownames(inpL[[i]])) else rownames(inpL[[i]]))
      useLi <- sort(useLi)             # multiple results may get easier to compare

      ## start extracting data (mode='intersect'):  extract of 1st matrix
      if(!isTRUE(extrRowNames)) {
        uCol <- if(is.list(useColumn)) useColumn[[1]] else useColumn
        out <- if(length(inpL) >1) inpL[[1]][match(useLi, rownames(inpL[[1]])), uCol] else inpL[[1]][,uCol]
        if(length(dim(out)) <2) out <- matrix(out, ncol=1, dimnames=list(useLi, if(is.numeric(useColumn)) colnames(inpL[[1]])[useColumn] else uCol))
        if(debug) {message(fxNa,"mNM3"); mNM3 <- list(inpL=inpL, out=out,useLi=useLi,useColumn=useColumn,chDi=chDi,chDi=chDi,useColNa=useColNa)}

        ## the remaining matrices (mode='intersect')
        if(length(inpL) >1) {
          ##  get colnames concerned
          uCol <- if(is.list(useColumn)) {lapply(1:length(useColumn), function(x) {if(is.numeric(x)) colnames(inpL[[x]])[useColumn[[x]]] else x})
            } else { if(is.numeric(useColumn)) lapply(inpL, function(x) colnames(x)[useColumn]) else useColumn }
          if(is.list(uCol)) {uCol <- unlist(uCol)}
          names(uCol) <- rep(names(useColumn), sapply(useColumn, length))
          chNa <- is.na(uCol)
          if(any(chNa)) { stop("Invalid content of argument 'useColumn'") }
          chDu <- duplicated(uCol, fromLast=FALSE)
          if(any(chDu)) uCol <- if(length(useColumn) >1) paste(trimRedundText(names(uCol), side="right"), trimRedundText(uCol), sep=".") else trimRedundText(uCol)
          ## finish extracting
          for(i in 2:length(inpL)) {
            uCo2 <- if(is.list(useColumn)) useColumn[[i]] else useColumn
            out <- cbind(out, inpL[[i]][match(useLi, rownames(inpL[[i]])), uCo2])
          }
          if(length(useColumn) >1) colnames(out) <- uCol     # adjust colnames as composed
        }
      } else out <- useLi

    } else {        ## suppose mode='union'
      doSort=TRUE
        if(debug) {message("mNM4"); mNM4 <- list(inpL=inpL, out=out, useColumn=useColumn,chDi=chDi,chDi=chDi,useColNa=useColNa)}
      ## get all rownames
      useLi <- unlist(lapply(inpL, rownames))
      useLi <- if(doSort) unique(sort(useLi)) else unique(useLi)
      ## get colnames concerned
      uCol <- if(is.list(useColumn)) {lapply(1:length(useColumn), function(x) {if(is.numeric(x)) colnames(inpL[[x]])[useColumn[[x]]] else x})
        } else { if(is.numeric(useColumn)) lapply(inpL, function(x) colnames(x)[useColumn]) else useColumn }
      if(is.list(uCol)) uCol <- unlist(uCol)
      chNa <- is.na(uCol)
      if(any(chNa)) { stop("Invalid content of argument 'useColumn'") }
      if(length(useColumn) >1) names(uCol) <- if(is.list(useColumn)) rep(names(inpL), sapply(useColumn, length)) else rep(names(inpL), each=length(useColumn))
        if(debug) {message("mNM4b"); mNM4b <- list(inpL=inpL, out=out, useColumn=useColumn,chDi=chDi,chDi=chDi,useColNa=useColNa)}

      out <- matrix(NA, nrow=length(useLi), ncol=length(uCol), dimnames=list(useLi, paste0(names(uCol),".",uCol)))
      ## 1st matr
      out[match(rownames(inpL[[1]]), useLi), which(names(uCol) %in% names(inpL)[1])] <- as.matrix(inpL[[1]][,if(is.list(useColumn)) useColumn[[1]] else useColumn])
        if(debug) {message("mNM4c"); mNM4c <- list(inpL=inpL, out=out, useColumn=useColumn, chDi=chDi,chDi=chDi,useColNa=useColNa)}

      ## remaining matrices
      if(length(inpL) >1) {
        incCol <- if(is.list(useColumn)) cumsum(sapply(useColumn, length)) else seq_along(inpL)*length(useColumn)            # cumsum() gives last of series
        for(i in 2:length(inpL)) {
          uCo2 <- if(is.list(useColumn)) useColumn[[i]] else useColumn
          out[match(rownames(inpL[[i]]), useLi), which(names(uCol) %in% names(inpL)[i])] <- inpL[[i]][,uCo2] }
      }
    }
  } else if(!silent) message(fxNa,"Invalid entries, all ",length(inpL)," elements must have 2 dimensions")
  out }
    

#' Read Batch of Excel xlsx-Files
#'
#' \code{readXlsxBatch} reads data out of multiple xlsx files, the sheet indicated by 'sheetInd' will be considered. 
#' All files must have the same organization of data, as this is typically the case when high-throughput measurements are automatically saved while experiments progress.
#' In particular, the first file read is used to structure the output.
#' 
#' @details
#' By default all columns with text-content may be eliminated to keep the numeric part only, which may then get organized to a 3-dim numeric array 
#'  (where the additional files will be used as 2nd dimension and multiple columns per file shown as 3rd dimension). 
#' 
#' NOTE : (starting from version wrMisc-1.5.5) requires packages \href{https://CRAN.R-project.org/package=readxl}{readxl} and 
#' \href{https://CRAN.R-project.org/package=Rcpp}{Rcpp} being installed ! 
#' (This allows much faster and memory efficient processing than previous use of package '\code{xlsx}') 
#' 
#' @param fileNames (character) provide either explicit list of file-names to be read or leave \code{NULL} for reading all files ending with 'xlsx' in path specified with argument \code{path}
#' @param path (character) there may be a different path for each file
#' @param fileExtension (character) extension of files (default='\code{xlsx}')
#' @param excludeFiles (character) names of files to exclude (only used when reading all files of given directory)
#' @param sheetInd (character or integer) specify which sheet to extract (as exact name of sheed or sheet-number, eg \code{sheetInd=2} will extract always the 2nd sheet (no matter the name); if given as sheet-name but nor present in file an empty list-elements wil be returned
#' @param checkFormat (logical) if \code{TRUE}: check header, remove empty columns, if rownames are increasing integeres it will searh for fisrt column with different entries to use as rownames
#' @param returnArray (logical) allows switching from array to list-output
#' @param columns (NULL or character) column-headers to be extracted (if specified, otherwise all columns will be extracted)
#' @param simpleNames (integer), if \code{NULL} all characters of fileNames will be maintained, otherwise allows truncating names (from beginning) to get to variable part (using .trimLeft()), but keeping at least the number of charcters indicated by this argument
#' @param silent (logical) suppress messages
#' @param callFrom (character) allows easier tracking of messages produced
#' @param debug (logical) display additional messages for debugging
#' @return This function returns a list of data.frames
#' @seealso \code{\link[readxl]{read_excel}}; for simple reading of (older) xls-files under 32-bit R one may also see the package \href{https://CRAN.R-project.org/package=RODBC}{RODBC} 
#' @examples
#' path1 <- system.file("extdata", package="wrMisc")
#' fiNa <- c("pl01_1.xlsx","pl01_2.xlsx","pl02_1.xlsx","pl02_2.xlsx")
#' datAll <- readXlsxBatch(fiNa, path1)
#' str(datAll)
#' ## Now let's read all xlsx files of directory
#' datAll2 <- readXlsxBatch(path=path1, silent=TRUE)
#' identical(datAll, datAll2)
#' @export
readXlsxBatch <- function(fileNames=NULL, path=".", fileExtension="xlsx", excludeFiles=NULL, sheetInd=1, checkFormat=TRUE,
  returnArray=TRUE, columns=c("Plate","Well","StainA"), simpleNames=3, silent=FALSE, debug=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom, newNa="readXlsxBatch")
  if(isTRUE(debug)) silent <- FALSE else { debug <- FALSE
    if(!isTRUE(silent)) silent <- FALSE }
  datOK <- TRUE 
  out <- NULL 
  packages <- c("readxl", "Rcpp")
  checkPkg <- function(pkg) requireNamespace(pkg, quietly=TRUE)
  checkPkgs <- sapply(packages, checkPkg)
  if(isTRUE(debug)) silent <- FALSE else { debug <- FALSE
    if(!isTRUE(silent)) silent <- FALSE }
  if(any(!checkPkgs)) {
    if(!silent) message(fxNa,"NOTE: Package 'readxl' and/or 'Rcpp' not found ! Please install first from CRAN (returning NULL)")
    datOK <- FALSE
  } 
  if(datOK) {
    ## prepare
    if(is.null(path)) path <- "."
    chPath <- dir.exists(path)
    if(!chPath) {message(fxNa,"Cannot find path '",path,"' !  ... Setting to default='.'")}
    if(is.null(fileNames)) {
      ## automatic reading of all files in directory
      fileNames <- dir(path=path,pattern=paste0(fileExtension,"$"))
      if(length(fileNames) <1) message(fxNa,"Could not find ANY suitable files !!") else {
        if(silent) message(fxNa,"Found ",length(fileNames)," files to extract (eg ",pasteC(utils::head(fileNames,3),quoteC="'"),")")}
      useFi <- file.path(path,fileNames)  
    } else {
      ## reading of specfied files in directory
      douPath <- grep(path,fileNames)
      useFi <- if(length(douPath) <1) file.path(path, fileNames) else fileNames
      checkFi <- file.exists(useFi)
      if(sum(!checkFi) >0) { 
        if(silent) message(fxNa,"Could not find ",sum(!checkFi)," files out of ",length(useFi),
          "  (eg ",pasteC(utils::head(fileNames[which(!checkFi)],3),quoteC="'"),")")
        useFi <- fileNames[which(checkFi)] 
        fileNames <- fileNames[which(checkFi)] }}
    ## files to omit from reading, ie exclude
    checkFi <- if(!is.null(excludeFiles)) grep(excludeFiles, fileNames) else NULL
    if(!any("try-error" %in% checkFi)) useFi <- NULL
    if(length(checkFi) >0) {
      if(silent) message(fxNa,"Based on 'excludeFiles': excluding ",length(checkFi)," files (out of ",length(fileNames),")")
      useFi <- useFi[-1*checkFi]
      fileNames <- fileNames[-1*checkFi] }

    ## main reading
    outL <- list()
    if(length(useFi) >0 && requireNamespace("readxl")) {
      for(i in 1:length(useFi)) {
        sheets <- try(readxl::excel_sheets(useFi[i]))
        if("try-error" %in% class(sheets)) { sheetInd <- NULL
          warning(fxNa,"Unable to read '",fileNames[i],"' Check if you have sufficient rights to open the file !?!")
        } else {
          ## inspect for sheet to load
          if(is.numeric(sheetInd)) { sheetInd <- as.integer(sheetInd)
            if(length(sheets) < sheetInd || sheetInd <1) {  sheetInd <- NULL
          }} else sheetInd <- naOmit(match(sheetInd,sheets))
          if(silent && length(sheetInd) <1) message(fxNa,"Unable to read '",fileNames[i],"', the sheet '",sheetInd,
            "' was not found (existing: ",pasteC(sheets,quoteC="'"),")")
        }  
        if(silent && length(sheetInd) >1) message(fxNa,"Only '",sheets[sheetInd],"', ie first match of 'sheetInd' will be read !")
        ## read xls and xlsx
        tmp <- if(length(sheetInd)==1) readxl::read_excel(useFi[i], sheet=sheets[sheetInd]) else NULL   # may also use readxl::read_xlsx    
        if(length(tmp) >0) {
          outL[[i]] <- as.data.frame(tmp)  
          if(isTRUE(checkFormat)) {
            ## display messages only for first file (others are presumed to repeat...)
            silent2 <- isTRUE(silent) || i !=1
            outL[[i]] <- .inspectHeader(outL[[i]], headNames=columns,silent=silent2, callFrom=fxNa)
            outL[[i]] <- .removeEmptyCol(outL[[i]], fromBackOnly=FALSE,silent=silent2, callFrom=fxNa)
          }
          ## for case array-output : define object out with dimensions based on 1st file
          if(i ==1) {                         # (re)define new format based on 1st file after format-checking (ie remove empty cols, extract col of well-names,...)
            outDim <- dim(outL[[i]])       
            out <- if(returnArray) array(NA, dim=c(outDim[1],length(fileNames),outDim[2]), 
              dimnames=list(rownames(outL[[i]]), basename(fileNames),colnames(outL[[i]]))) else NULL }
          ## for case array-output : fill directly into object out
          if(returnArray) outL[[i]] <- as.matrix(outL[[i]]) else {
            if(identical(dim(outL[[i]]),outDim)) out[,i,] <- as.matrix(outL[[i]]) else {
              message(fxNa,"Omit ",i,"th file ''",fileNames[i],"': format (",dim(outL[[i]])[1]," rows & ",dim(outL[[i]])[2]," cols) NOT consistent with previous files - omitting")
          } }
      }
      }
      names(outL) <- fileNames
      if(returnArray) {
        ## refine column-names in array
        chColNa <- length(unique(colnames(out)))==1
        arrNames <- if(chColNa) paste(colnames(out),sub("\\.xlsx$","",if(length(simpleNames) >1) .trimLeft(fileNames, minNchar=simpleNames, callFrom=fxNa) else fileNames),sep=".") else colnames(out)
      } else { out <- outL
        ## refine names in list-output
        if(length(fileNames) !=length(out)) message(fxNa,"Problem ?  Got ",length(fileNames)," fileNames  BUT ",length(out)," list-elements !")
        names(out) <- substr(basename(fileNames), 1, nchar(basename(fileNames)) -4)   # remove extesions
        if(simpleNames) names(out) <- if(length(simpleNames) >1) .trimLeft(fileNames, minNchar=simpleNames, callFrom=fxNa) else fileNames } 
    }      
  }  
  out }
      

#' Remove or Reassign Orphan Indexes
#'
#' This function allows detecting terminal orphans of a vector of (cluster-) indexes and removing (ie marking as \code{NA})
#'   or re-assigning them to the neighbour class towrds the center.
#'
#' @details
#' All input of \code{ind} is supposed to be interger values as (cluster-) indexes.
#' This function will look if the lowest and/or highest (cluster-) indexes appear at very low frequency so that they may be considered orphans. 
#' The argument \code{minN} assigns the threshold of when the frquency of terminal values may be considered as 'orphan',
#' either as absolute threshold or if less than 1 as ratio (0.1 => 10%tile of length of \code{ind}) .
#' 
#' The argument \code{side} may be 'both', 'b', 'upper', 'u', 'lower' or 'l', to decide if lower and/or upper end indexes should be treated.
#' 
#' @param ind (integer) main input of (cluster-) indexes
#' @param minN (numeric, length=1) the min frequency to consider as orphans, if less than 1 it will be interpreted as ratio compared to length of \code{index}
#' @param reassign (logical) if \code{TRUE} orphan indexes will be replaced by neighbour class indexes (towrds the center)
#' @param side (character) may be 'both', 'b', 'upper', 'u', 'lower' or 'l' to decide if lower and/or upper end indexes should be treated.
#' @param callFrom (character) allows easier tracking of messages produced
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @return This function returns an integer vector of adjusted indexes
#' @seealso \code{\link[base]{table}}
#' @examples
#'  x=c(3:1,3:4,4:6,5:3); rmOrphans(x)
#' rmOrphans(x, minN=0.2) 
#' ## reassign orphans to neighbour center class
#' cbind(x,  x=x, def=rmOrphans(x, reassign=TRUE), 
#'   minN=rmOrphans(x, minN=0.2, reassign=TRUE) )
#' @export
rmOrphans <- function(ind, minN=1, reassign=FALSE, side="both", silent=FALSE, debug=FALSE, callFrom=NULL) {
  ## reassign/remove orphans (groups with too few members) from both ends of vector if indexes 'ind'
  ## minN (numeric) if 0 > minN > 1 will be interpreded as ratio (0.1 => 10%tile of length of ind) otherwise as interger threshold for min number of values in terminal groups
  fxNa <- .composeCallName(callFrom, newNa="rmOrphans")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  if(length(ind) >0 && is.character(ind)) ind <- as.integer(factor(ind))
  if(length(ind) <1 || all(is.na(ind))) {
    if(!silent) message(fxNa,"Nothing to do ..")
  } else {
    minN <- if(length(minN) <1 || !is.numeric(minN)) 1 else {if(minN >0 && minN <1) round(length(ind)*minN) else as.integer(minN)}
    tab <- table(ind)
    begOrph <- cumsum(tab <= minN) ==1:length(tab)
    if(any(begOrph) && is.character(side) && any(c("both","lower","b","l") %in% side)) { 
      rmVal <- names(begOrph)[which(begOrph)]       
      ind[which(ind %in% rmVal)] <- if(length(reassign)==1 && isTRUE(reassign)) sum(begOrph) +1 else NA
      tab <- table(ind) }
    endOrph <- cumsum(rev(tab) <= minN) ==1:length(tab)
    if(any(endOrph) && is.character(side) && any(c("both","upper","b","u") %in% side)) { rmVal <- names(endOrph)[which(endOrph)]
      ind[which(ind %in% rmVal)] <- if(length(reassign)==1 && isTRUE(reassign)) max(as.numeric(names(tab))) -sum(endOrph) else NA} }
  ind }

#' Equal character-length number 
#'
#' \code{equLenNumber} convert numeric entry 'x' to text, with all elements getting the same number of characters (ie  by adding preceeding or tailing 0s, if needed).
#' So far, this function cannot handle scientific annotations. 
#'
#' @param x (caracter) input vector
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a character vector formated as equal number of characters per value
#' @seealso \code{\link[base]{sprintf}}
#' @examples
#' equLenNumber(c(12,-3,321))
#' equLenNumber(c(12,-3.3,321))
#' @export
equLenNumber <- function(x, silent=FALSE, callFrom=NULL, debug=FALSE){
  fxNa <- .composeCallName(callFrom, newNa="equLenNumber")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE

  x <- sub("^ +","",sub(" +$","",x))              # remove heading or tailing spaces
  check1 <- grep("^[[:digit:]]+$|^\\-[[:digit:]]+$",x)                          # use as integer
  check2 <- grep("^[[:digit:]]+\\.[[:digit:]]+$|^[[:digit:]]+$|^-[[:digit:]]+$|^-[[:digit:]]+\\.[[:digit:]]+$",x)   # other numeric (wo exponent)
  if(length(check2) < length(x) && !silent) message(fxNa," ",length(x)-length(check2)," out of ",length(x)," entries can't be transformed to numeric")
  x <- as.numeric(x)
  if(length(check1) ==length(x)) sprintf(paste0("%0",max(nchar(as.character(x))),"d"),x) else {
    sprintf(paste0("%0",max(nchar(as.character(x))+1),".",max(nchar(gsub("\\.","",gsub("^[[:digit:]]+","",gsub("^-","",x))))),"f"),x)}
}

#' Convert To Simple Vector (similar to unlist)
#'
#' This function allows converting 'dat' (may be list, data.frame etc) to simple vector, more elaborate than unlist()
#' 
#' @param dat (list, data.frame) main input
#' @param toNumeric (logical)
#' @return This function returns a character (or numeric) vector
#' @seealso  \code{\link[base]{unlist}}; used in  \code{\link{equLenNumber}}
#' @examples
#' aa <- matrix(11:14, ncol=2)
#' .checkConvt2Vect(aa)
#' @export
.checkConvt2Vect <- function(dat, toNumeric=TRUE) {
  ## convert 'dat' (may be list, data.frame etc) to simple vector, more elaborate unlist()
  ## if 'toNumeric'=FALSE only the 1st column of a matrix or data.frame will be extracted !!
  ## list names will be passed on (list-elements longer than 1 will get number-extesions)
  if(is.data.frame(dat)) dat <- as.matrix(dat)
  if(is.list(dat)) {
    datNa <- names(dat)
    dat <- unlist(dat)
    if(length(dat) ==length(datNa)) names(dat) <- datNa}
  if(is.matrix(dat)) {
    rowNa <- rownames(dat)
    dat <- if(toNumeric) as.numeric(dat) else dat[,1]
    if(!is.null(rowNa)) names(dat) <- rowNa }
  dat }

#' Compose Sequence Of (Function-)Calls
#'
#' This function was designed for tracing the hierarchy of function-calls.
#' It allows to remove any tailing space or ': ' from 'callFrom' (character vector) and return with added 'newNa' (+ 'add2Tail')
#' 
#' @param newNa (character vector) main input
#' @param add2Head (character)
#' @param add2Tail (character)
#' @param callFrom (character) may also contain multiple separate names (ie length >1), will be concatenated using ' -> '
#' @return This function returns a character vector (history of who called whom)
#' @seealso  \code{\link[base]{paste}}
#' @examples
#' .composeCallName("newFunction", callFrom="initFunction")
#' @export
.composeCallName <- function(newNa, add2Head="", add2Tail=" : ", callFrom=NULL) {
  ## remove any tailing space or ': ' from 'callFrom' (character vector) and return with added 'newNa' (+ 'add2Tail')
  ## 'callFrom' may also contain multiple separate names (ie length >1), will be concatenated using ' -> '
  ## used for building history of who called whom ...
  if(length(callFrom) >0) {
    callFrom <- sub("[[:blank:]]+$","", callFrom)      
    callFrom <- sub("^[[:blank:]]+","", callFrom)
    callFrom <- sub("[[:blank:]]:+$","", callFrom)  
    paste(callFrom, collapse=" -> ")}
  fxNa <- paste(c(add2Head, callFrom, if(!is.null(callFrom)) " -> ", newNa, add2Tail), collapse="")
  fxNa }
       

#' 2-factorial Anova on single line of data
#'
#' This function runs 2-factorial Anova on a single line of data (using \code{\link[stats]{aov}} from package \code{stats}) 
#' using a model with two factors (without factor-interaction) and extracts the correpsonding p-value. 
#'
#' @param dat numeric vector
#' @param fac1 (character or factor) vector describing grouping elements of dat for first factor, must be of same langth as fac2
#' @param fac2 (character or factor) vector describing grouping elements of dat for second factor, must be of same langth as fac1
#' @param inclInteraction (logical) decide if factor-interactions (eg synergy) should be included to model
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced

#' @return This function returns the (uncorrected) p for factor 'Pr(>F)' (see \code{\link[stats]{aov}}) 
#' @seealso \code{\link[stats]{aov}}, \code{\link[stats]{anova}}; for repeated tests using the  package \href{https://bioconductor.org/packages/release/bioc/html/limma.html}{limma} including \code{\link[limma]{lmFit}} and \code{eBayes} see \code{\link{test2factLimma}}
#' @examples
#' set.seed(2012); dat <- round(runif(8),1)
#' singleLineAnova(dat, gl(2,4),rep(1:2,4))
#' @export
singleLineAnova <- function(dat, fac1, fac2, inclInteraction=TRUE, silent=FALSE, debug=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom, newNa="singleLineAnova")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  if(debug) message(fxNa,"sLA1")
  
  if(!identical(length(fac1),length(fac2))) stop("Arguments 'fac1' & 'fac2' should be of same length !")
  tmp <- data.frame(dat=as.numeric(dat), fac1=fac1, fac2=fac2)
  tmp.aov <- if(inclInteraction) stats::aov(dat ~ fac1 + fac2 + fac1:fac2, data=tmp) else {
    stats::aov(dat ~ fac1 + fac2, data=tmp)}
  tmp <- stats::anova(tmp.aov)[["Pr(>F)"]]
  tmp[-1*length(tmp)] }
    

#' Combine Redundant Lines In List
#'
#' This function provides help for combining/summarizing lines of numeric data which may be summaried according to reference vector or matrix of annotation (part of the same input-list).
#' The data and reference will be aligned and data corresponding to redundant information be combined/summarized.
#'  
#' 
#' @details 
#' All input data should be in a list, ie one or multipl matrix or data.frame for numeric data (see argument \code{datNa}), as well as the reference (see argument \code{refNa}). 
#' The refgerence may be a named character vecor or a matrix for which the column to be used should be specified using the argument \code{refColNa}.
#' In case the annotation is a matrix, the rownames will be used as unique/independent identifyers to adjust potentially different order of numeric data and annotation.
#' In absence of rownames, an additional column  \code{supRefColNa} of the annotation may be designed for adjusting the order of annotation and numeric data.
#'  
#' The numeric list may contain multiple matrixes or data.frames which will all be summarized by the same procedure as long as they have the same initial dimensions and are specified by \code{refNa}. 
#'
#' Please note that all other list elements from input not specified by \code{refNa} (or \code{datNa}) will be maintained in the output just as they are.
#'
#' @param lst (list) main input, containing matrix or data.frame of numeric data (see \code{datNa} and annotation (see \code{refNa}) and possibly unrelated stuff
#' @param refNa (character) name of list-element containing annotation 
#' @param datNa (character) name(s) of list-element(s) containing numeric/quantitation data 
#' @param refColNa (character) in case the list-element to be used as reference is \code{matrix} or \code{data.frame}, the column to be used must be specified here
#' @param supRefColNa (character) in case the \code{lst$refNa} has no rownames, the content of column \code{lst$supRefColNa} will be used instead
#' @param summarizeType (character) the summarization method gets specified here; so far 'sum','av','med','first' and 'last' are implemented
#' @param NA.rm (logical) pass to summarizing functions order to omit \code{NA}s, defaults to \code{TRUE} 
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' 
#' @seealso \code{\link{findRepeated}}, \code{\link{firstOfRepLines}}, \code{\link{organizeAsListOfRepl}}, \code{\link{combineRedBasedOnCol}}
#' @return This function returns a list of same length as input
#' @examples
#' x1 <- list(quant=matrix(11:34, ncol=3, dimnames=list(letters[8:1], LETTERS[11:13])), 
#'   annot=matrix(paste0(LETTERS[c(1:4,6,3:5)],LETTERS[c(1:4,6,3:5)]), ncol=1, 
#'   dimnames=list(paste(letters[1:8]),"xx")) )
#' combineRedundLinesInList(lst=x1, refNa="annot", datNa="quant", refColNa="xx")
#' @export
combineRedundLinesInList <- function(lst, refNa="ref", datNa="quant", refColNa="GeneName", supRefColNa=NULL, summarizeType="av", 
  NA.rm=TRUE, silent=FALSE, debug=FALSE, callFrom=NULL) {
  ## combine lines of (matrix) list-elements according to (line-) grouping given by reference (also part of list x); 
  ## name of reference should be in refNa; in case reference is matrix, a specific column (refColNa) may be chosen 
  fxNa <- .composeCallName(callFrom, newNa="combineRedundLinesInList")
  if(isTRUE(debug)) silent <- FALSE else { debug <- FALSE
    if(!isTRUE(silent)) silent <- FALSE }
  namesXY <- deparse(substitute(lst))
  out <- NULL
  datOK <- length(lst) >0
  if(debug) message(fxNa, "cRLL0")
   
  if(length(refNa) >1) {refNa <- refNa[1]} else if(length(refNa) <1) {datOK <- FALSE
    stop("Argument 'refNa' should design list-element of '",namesXY,"' (index or name)") }
  if(length(datNa) >1) {datNa <- datNa[1]} else if(length(datNa) <1) {datOK <- FALSE
    stop("Argument 'datNa' should design list-element of '",namesXY,"' (index or name)") }

  .chLstNa <- function(x, lst, colName=NULL, lstArgNa, datOK, fxNa, silent) {
    ## check if x (refNa, etc) is valid element of lst,  optional check if including column named colName (for 'refNa')
    ## returns $lstMatch ie position of x in names of lst 
    if(is.numeric(x)) { if(x >= 1 && x <= length(lst)) { lstMatch <- as.integer(x)
      } else { if(!silent) message(fxNa,"Invalid numeric entry for list-element"); datOK <- FALSE}
    } else {  ## check if given name exists in lst
      lstMatch <- which(names(lst) %in% x)
      if(length(lstMatch) >0) {
        names(lstMatch) <- x         
      } else { datOK <- FALSE
        if(!silent) message(fxNa,"Can't find ",x," in names of ",lstArgNa)}
    }
    ## check if lstMatch corresponds to matrix or data.frame with named columns
    if(datOK) { datOK <- length(colnames(lst[[lstMatch]])) ==ncol(lst[[lstMatch]])
      if(!datOK) { if(!silent) message(fxNa," ",lstArgNa,"$",names(lst)[refNa]," does not seem to be suitable matrix or data.frame") }}
    if(datOK  && length(colName)==1) {    ## check if colName part of colnames
      if(is.integer(colName)) { datOK <- colName <= ncol(lst[[lstMatch]]) && colName >=1
        if(!datOK && !silent) message(fxNa,"Invalid numeric entry for 'colName")
      } else {
        chRefCol <- colName %in% colnames(lst[[x]])
        if(!chRefCol) { datOK <- FALSE
          if(!silent) message(fxNa,"Can't find '",colName,"'  in names of ",lstArgNa)
      } } 
    }
    list(datOK=datOK, lstMatch=lstMatch)
  }
  
  if(length(refColNa) < 1) refColNa <- "GeneName"
  if(length(refColNa) > 1) { refColNa <- refColNa[1]
    if(!silent) message(fxNa, "Argument 'refColNa' should be of length 1, trimming ..") }
  xx <- .chLstNa(x=refNa, lst, colName=refColNa, lstArgNa=namesXY[1], datOK=datOK, fxNa, silent)
  datOK <- xx$datOK
  refNa <- xx$lstMatch     # as index (+name)
  xx <- .chLstNa(x=datNa, lst, colName=NULL, lstArgNa=namesXY[1], datOK=datOK, fxNa, silent)
  datOK <- xx$datOK
  datNa <- xx$lstMatch     # as index (+name)
  
  if(debug) { message(fxNa, "cRLL1"); cRLL1 <- list(lst=lst, refNa=refNa,datNa=datNa,refColNa=refColNa, NA.rm=NA.rm, summarizeType=summarizeType,supRefColNa=supRefColNa,datOK=datOK) }

  ## check if redundant data
  if(datOK) {
    datOK <- any(duplicated(if(length(dim(lst[[refNa]])) > 1) lst[[refNa]][,refColNa] else lst[[refNa]]), na.rm=TRUE)
    if(!datOK && debug) message(fxNa,"No redundant values foound, nothing to do ...")    
  }
  
  ## check rownames of annot 
  ## when annotation (ie lst[[refNa]]) has no (row)names, supRefColNa will be used instead (must give unique IDs)
  if(datOK) {
    if(length(dim(lst[[refNa]])) > 1) {
       if(length(rownames(lst[[refNa]])) <1) {
        chSupTRef <- supRefColNa[1] %in% colnames(lst[[refNa]]) 
        if(isTRUE(chSupTRef)) chSupTRef <- isTRUE(sum(duplicated(lst[[refNa]][,chSupTRef])) <1)
        if(!chSupTRef) { datOK <- FALSE
          if(!silent) message(fxNa,"'supRefColNa' either not in annotation-data or not unique !") }       
       }       
    } else {
      if(length(names(lst[[refNa]])) <1) { datOK <- FALSE
        if(!silent) message(fxNa,"Annotation has no names, can't use 'supRefColNa' (annotation is not matrix) !") }
  } }
  if(debug) { message(fxNa, "cRLL1a"); cRLL1a <- list() }
  
  if(datOK) {
    ## pick annotation concerned : if ..
    ## problem : lst[[refNa]] may be matrix wo rownames !!! (proteomics example)
    ## strategy : option to specify 2nd column (supRefColNa) to use instead of rownames for collapsing all names or picking 1st
    annS <- if(length(dim(lst[[refNa]])) > 1) lst[[refNa]][,refColNa] else lst[[refNa]]
    out <- lst    # keep also non-concerned data, instead of : list()
    newNa <- if(length(dim(annS)) > 1) tapply(rownames(annS), annS[, refColNa], paste, collapse=",") else tapply(names(annS), annS, paste, collapse=",")
    newNFi <- if(length(dim(annS)) > 1) tapply(rownames(annS), annS[, refColNa], function(z) z[1]) else tapply(names(annS), annS, function(z) z[1])
    ## check if any duplicated items exist
    chDu <- duplicated(if(length(dim(annS)) >1) annS[, refColNa] else annS)
    if(all(!chDu, na.rm=TRUE)) datOK <- FALSE
  }   
  if(debug) { message(fxNa, "cRLL2"); cRLL2 <- list(lst=lst, out=out,refNa=refNa,datNa=datNa,refColNa=refColNa, NA.rm=NA.rm, summarizeType=summarizeType,datOK=datOK)  }

    
  if(datOK) {
    ## prepare for multiple summarizing options/functions via argument summarizeType
    sumFx <- function(z, meth) {
      if(length(dim(z)) < 2) stop("Invalid entry 'z' (should be matrix)")
      if(any(c("aver", "average", "mean") %in% meth))  meth <- "av"
      if(any(c("median") %in% meth)) meth <- "med"
      out <- try(switch(meth, 
        sum=colSums(z, na.rm=NA.rm), 
        av=colMeans(z, na.rm=NA.rm), 
        med=apply(z, 2, stats::median, na.rm=NA.rm), 
        first=z[1,], last=z[nrow(z), ]), silent=TRUE)
      if(inherits(out, "try-error")) out <- NULL
      out }
    if(debug) { message(fxNa, "cRLL2a"); cRLL2a <- list()  }
          
    ## prepare $annot
    if(length(dim(annS)) > 1) {
      out[[refNa]] <- annS[which(!duplicated(annS[, refColNa])), ]
      out[[refNa]] <- out[[refNa]][order(out[[refNa]][, refColNa]), ]
    } else out[[refNa]] <- sort(annS[which(!duplicated(annS))])          # , fromLast=TRUE
    if(debug) { message(fxNa, "cRLL2b") }
    
    out[[refNa]] <- by(lst[[refNa]], if(length(dim(lst[[refNa]])) > 1) lst[[refNa]][, refColNa] else names(lst[[refNa]]), 
      function(x) if(length(dim(x)) > 1) cbind(x[1, ], allIniNames=paste(rownames(x), collapse=",")) else cbind(x[1], allIniNames=paste(names(x), collapse=",")))
    if(debug) { message(fxNa, "cRLL2c") }
    out[[refNa]] <- matrix(unlist(out[[refNa]]), nrow=length(out[[refNa]]), byrow=TRUE)
    colNa <- colnames(lst[[refNa]])
    if(length(colNa) < 1) colNa <- paste0("X.", 1:ncol(out[[refNa]]))
    dimnames(out[[refNa]]) <- list(rownames(out[[refNa]]), if(length(colNa) ==ncol(out[[refNa]])) colNa else c(colNa[1], paste0(colNa[1], 2:(ncol(out[[refNa]])))))
    if(debug) { message(fxNa, "cRLL2d") }
    if(is.null(colnames(out[[refNa]]))) {
      colnames(out[[refNa]]) <- c(if(length(refColNa) ==1) refColNa else "ref", paste("x", 1:(ncol(out[[refNa]] -1)))) } else if(colnames(out[[refNa]])[1] == "") 
    colnames(lst[[refNa]])[1] <- if(length(refColNa) ==1) refColNa else "ref"
        
    if(debug) { message(fxNa, "cRLL3; ready to combine data of ", length(datNa)," table(s) (element(s) of 'lst') as ", summarizeType)
      cRLL3 <- list(lst=lst, out=out,sumFx=sumFx,annS=annS,refNa=refNa,datNa=datNa,refColNa=refColNa,NA.rm=NA.rm,summarizeType=summarizeType) }        
    ## now summarize                                                            
    for(i in datNa) { yy <- lst[[i]]
      yz <- unlist(by(yy, if(length(dim(annS)) > 1) annS[, refColNa] else annS, sumFx, summarizeType))
      if(length(yz) > 0) out[[i]] <- matrix(yz, byrow=TRUE, ncol=ncol(yy), dimnames=list(sort(unique(if(length(dim(annS)) >1) annS[, refColNa] else annS)), colnames(yy)))
    }
    if(debug) { message(fxNa, "cRLL3b");  cRLL3b <- list()}
    names(out) <- names(lst)
    out
  } else { if(debug) message(fxNa, "Invalid entry OR no redundant ref/annnot found, nothing to do ...")
    lst
  }
}


#' Combine Redundant Lines In List, Deprecated
#'
#' The function combineRedundLinesInListAcRef() has been deprecated and replaced by combineRedundLinesInList() from the same package
#'
#' @param lst (list) main input
#' @param listNa (character)   names of list-elements containing quantitation data (1st position) and protein/line annotation (2nd position)
#' @param refColNa (character) in case the list-element to be used as reference is \code{matrix} or \code{data.frame}, the column to be used must be specified here
#' @param summarizeType (character) the summarization method gets specified here; so far 'sum','av','med','first' and 'last' are implemented
#' @param NA.rm (logical) pass to summarizing functions order to omit \code{NA}s, defaults to \code{TRUE} 
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @seealso \code{\link{combineRedundLinesInList}}
#' @return This function returns a list of same length as input
#' @examples
#' x1 <- list(quant=matrix(11:34, ncol=3, dimnames=list(letters[8:1], LETTERS[11:13])), 
#'   annot=matrix(paste0(LETTERS[c(1:4,6,3:5)],LETTERS[c(1:4,6,3:5)]), ncol=1, 
#'   dimnames=list(paste(letters[1:8]),"xx")) )
#' ## please use combineRedundLinesInList()
#' combineRedundLinesInList(lst=x1, refNa="annot", datNa="quant", refColNa="xx")
#' @export
combineRedundLinesInListAcRef <- function(lst, listNa=c("ref","quant"), refColNa="xx", summarizeType="av", NA.rm=TRUE, silent=FALSE, debug=FALSE, callFrom=NULL) {
  .Deprecated(new="combineRedundLinesInList", package="wrMisc", msg="The function combineRedundLinesInListAcRef() has been deprecated and replaced by combineRedundLinesInList()") 
  fxNa <- .composeCallName(callFrom, newNa="combineRedundLinesInList")
  if(isTRUE(debug)) silent <- FALSE else { debug <- FALSE
    if(!isTRUE(silent)) silent <- FALSE }
  namesXY <- deparse(substitute(lst))
  out <- NULL
  datOK <- TRUE
  if(debug) message(fxNa, "cRLL0")
  msg1 <- c(" entry for 'listNa' (must design at least 2 list-elements of '", namesXY, "', 1st sould be annotation, others numeric matrix)")
  if(length(listNa) > 1) {
   if(is.numeric(listNa)) {
      listNa <- as.integer(listNa)
      if(any(listNa < 1 | listNa > length(lst))) 
          stop(fxNa, "Invalid (numeric)", msg1)
    } else {
      lstMatch <- match(listNa, names(lst))
      chNA <- is.na(lstMatch)
      if(sum(!chNA) < 2 || chNA[1]) stop(fxNa, msg1)
      if(any(chNA)) { refColNa <- naOmit(refColNa)
          if(!silent) message(fxNa, "Removing ", sum(chNA), " element(s) not found in 'refColNa'") }
    }
  } else stop(fxNa, "Insufficient", msg1)
  
  chUni <- unique(lst[[listNa[1]]])
  if(length(chUni) < 2) datOK <- FALSE
  if(datOK) {
    if(length(refColNa) < 1) refColNa <- "GeneName"
    if(length(refColNa) > 1) { refColNa <- refColNa[1]
      if(!silent) message(fxNa, "Argument 'refColNa' should be of length 1, trimming ..") }
    ## sort listNa to have annot at beginning
    if(length(names(listNa)) == length(listNa)) {
      chO <- naOmit(match(c("ref", "reference", "annot"), tolower(names(listNa))))
      if(length(chO) > 0) {
        if(chO[1] != 1) {
          if(debug) message(fxNa, "Changing order of 'listNa' based on names of 'listNa'")
          listNa <- listNa[c(chO[1], (1:length(listNa))[-chO[1]])] }
      }
    }
    if(debug) { message(fxNa, "cRLL1"); cRLL1 <- list(lst=lst, listNa=listNa, refColNa=refColNa, NA.rm=NA.rm, summarizeType=summarizeType) }
    
    ## re-order listNa to have annotation as 1st
    nuI <- naOmit(match(listNa[-1], names(lst)))
    
    ## check presence of 'refColNa' in lst
    if(length(dim(lst[[listNa[1]]])) > 1) {
      ch1 <- refColNa %in% colnames(lst[[listNa[1]]])
      if(!any(ch1)) stop(fxNa, "Missing column '", refColNa, "' in ", namesXY[1], "$", listNa[1])
    }
    chDuRef <- duplicated(if(length(dim(lst[[listNa[1]]])) > 1) lst[[listNa[1]]][, refColNa] else lst[[listNa[1]]])
    datOK <- any(chDuRef, na.rm=TRUE)
    if(debug) { message(fxNa, "cRLL1b"); cRLL1b <- list()
    }
  }
  if(datOK) {
    chNa <- is.na(nuI)
    if(length(listNa) < 2 || all(chNa)) stop("Invalid entry for 'listNa' (must design at least one numeric matrix of 'lst')")
    if(any(chNa))  message(fxNa, "Elements ", pasteC(listNa[which(chNa)]),"missing")
    xAnn <- which(names(lst) == listNa[1])
    annS <- if(length(dim(lst[[xAnn]])) > 1) lst[[xAnn]][match(rownames(lst[[xAnn]]), rownames(lst[[nuI[1]]])), ] else lst[[xAnn]][match(names(lst[[xAnn]]), rownames(lst[[nuI[1]]]))]
    out <- list()
    newNa <- if(length(dim(annS)) > 1) tapply(rownames(annS), annS[, refColNa], paste, collapse=",") else tapply(names(annS), annS, paste, collapse=",")
    newNFi <- if(length(dim(annS)) > 1) tapply(rownames(annS), annS[, refColNa], function(z) z[1]) else tapply(names(annS), annS, function(z) z[1])
    ## prepare for multiple summarizing options/functions via argument summarizeType
    sumFx <- function(z, meth) {
      if(length(dim(z)) < 2) stop("Invalid entry 'z' (should be matrix)")
      if(any(c("aver", "average", "mean") %in% meth))  meth <- "av"
      if(any(c("median") %in% meth)) meth <- "med"
      out <- try(switch(meth, 
        sum=colSums(z, na.rm=NA.rm), 
        av=colMeans(z, na.rm=NA.rm), 
        med=apply(z, 2, stats::median, na.rm=NA.rm), 
        first=z[1,], last=z[nrow(z), ]), silent=TRUE)
      if(inherits(out, "try-error")) out <- NULL
      out }
    if(debug) { message(fxNa, "cRLL2") }
    
    ## prepare $annot
    if(length(dim(annS)) > 1) {
      out[[xAnn]] <- annS[which(!duplicated(annS[, refColNa], fromLast=TRUE)), ]
      out[[xAnn]] <- out[[xAnn]][order(out[[xAnn]][, refColNa]), ]
    } else out[[xAnn]] <- sort(annS[which(!duplicated(annS, fromLast=TRUE))])
    if(debug) { message(fxNa, "cRLL2b"); cRLL2b <- list() }
    out[[xAnn]] <- by(lst[[xAnn]], if(length(dim(lst[[xAnn]])) > 1) lst[[xAnn]][, refColNa] else names(lst[[xAnn]]), 
      function(x) if(length(dim(x)) > 1) cbind(x[1, ], allIniNames=paste(rownames(x), collapse=",")) else cbind(x[1], allIniNames=paste(names(x), collapse=",")))
    if(debug) { message(fxNa, "cRLL2c") }
    out[[xAnn]] <- matrix(unlist(out[[xAnn]]), nrow=length(out[[xAnn]]), byrow=TRUE)
    colNa <- colnames(lst[[xAnn]])
    if(length(colNa) < 1) colNa <- paste0("X.", 1:ncol(out[[xAnn]]))
    dimnames(out[[xAnn]]) <- list(names(out[[xAnn]]), if(length(colNa) ==ncol(out[[xAnn]])) colNa else c(colNa[1], paste0(colNa[1], 2:(ncol(out[[xAnn]])))))
    if(debug) { message(fxNa, "cRLL2d") }
    if(is.null(colnames(out[[xAnn]]))) {
      colnames(out[[xAnn]]) <- c(if(length(refColNa) ==1) refColNa else "ref", paste("x", 1:(ncol(out[[xAnn]] -1)))) } else if(colnames(out[[xAnn]])[1] == "") 
    colnames(lst[[xAnn]])[1] <- if(length(refColNa) ==1) refColNa else "ref"    
    if(debug) { message(fxNa, "cRLL3; Summarizing ", length(nuI)," elements as ", summarizeType)
      cRLL3 <- list(lst=lst, out=out,sumFx=sumFx,annS=annS, xAnn=xAnn,listNa=listNa,refColNa=refColNa,NA.rm=NA.rm,summarizeType=summarizeType) }
        
    ## now summarize                                                            
    for(i in nuI) { yy <- lst[[i]]
      yz <- unlist(by(yy, if(length(dim(annS)) > 1) annS[, refColNa] else annS, sumFx, summarizeType))
      if(length(yz) > 0) out[[i]] <- matrix(yz, byrow=TRUE, ncol=ncol(yy), dimnames=list(sort(unique(if(length(dim(annS)) >1) annS[, refColNa] else annS)), colnames(yy)))
    }
    names(out) <- names(lst)
    rownames(out[[xAnn]]) <- rownames(out[[nuI[1]]])
    out
  } else { if(debug) message(fxNa, "No redundant ref/annnot found, nothing to do ...")
    lst
  }
}  
   

#' t.test on all individual values against all other values
#'
#' Run t.test on each indiv value of x against all its neighbours (=remaining values of same vector) in order to test if tis value is likely to belong to vector x. 
#' This represents a repeated leave-one-out testing. Mutiple choices for multiple testing correction are available.
#'
#' @param x matrix or data.frame
#' @param alph (numeric) threshold alpha (passed to \code{t.test})
#' @param alternative (character) will be passed to \code{t.test} as argument 'alternative', may be "two.sided",..
#' @param p.adj (character) multiple test correction : may be NULL (no correction), "BH","BY","holm","hochberg" or "bonferroni"  (but not 'fdr' since this may be confounded with local false discovery rate), see \code{\link[stats]{p.adjust}}
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a numeric vector with p-values or FDR (depending on argument \code{p.adj}) 
#' @seealso \code{\link[stats]{t.test}}, \code{\link[stats]{p.adjust}}
#' @examples
#' set.seed(2016); x1 <- rnorm(100)
#' allTests1 <- tTestAllVal(x1)
#' hist(allTests1,breaks="FD")
#' @export
tTestAllVal <- function(x, alph=0.05, alternative="two.sided", p.adj=NULL, silent=FALSE, debug=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom, newNa="tTestAllVal")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  if(debug) message(fxNa,"tTA1")

  x <- cbind(x=as.numeric(x), no=1:length(x))
  pVal <- sapply(1:length(x), function(z) stats::t.test(x[-z], x[z], conf.level=alph, alternative=alternative, var.equal=TRUE)$p.value )
  if(!is.null(p.adj)) if(any(p.adj %in% c("BH","BY","holm","hochberg","bonferroni"))) pVal <- stats::p.adjust(pVal,method=p.adj)
  pVal }
   

#' Make non-redundant matrix
#'
#' This function takes matrix or data.frame 'dat' to summarize redundant lines (column argument \code{iniID}) along method specified in \code{summarizeRedAs} 
#' to treat all lines with redundant \code{iniID} by same approach (ie for all columns the line where specified column is at eg max = 'maxOfRef' ).
#' If no name given, the function will take the last numeric (factors may be used - they will be read as levels).
#'
#' @details
#' When using for selection of single initial line give the character-string of argument \code{summarizeRedAs} a name (eg  \code{summ=c(X1="minOfRef")}
#'  so that the function will use ONLY the column specified via the name for determining which line should be used/kept.    
#' 
#' It is possible to base the choice from 'redundant' lines on a single reference-column.  
#' For example, when \code{summarizeRedAs='maxOfRef'} summarizing of all (numeric) columns will be performed according to one single column (ie the line where the last numeric column is at its max).
#' Otherwiser, a name can be assigned as reference column to be used (eg see last example using \code{summarizeRedAs=c(x1='maxOfRef')})
#' 
#' @param dat (matrix or data.frame) main input for making non-redundant
#' @param summarizeRedAs (character) summarization method(s), typical choices 'median','mean','min' or 'maxOfRef';
#'    basic usage like \code{summarizeRedAs='mean'} will pick independently the mean for each (numeric) column; 
#'    it is also possible to specify different methods for each of columnw (length of \code{summarizeRedAs} should be equal number of numeric columns);
#'    special methods look at a single reference column to decide which line should be picked and their values reported (not compatible with specifying different methods for different columns), 
#' @param iniID (character) column-name used as reference for determining groups of redundant lines (default="iniID")
#' @param retDataFrame (logical) if \code{TRUE}, check if text-columns may be converted to data.frame with numeric
#' @param nEqu (logical) if \code{TRUE}, add additional column indicating the number of equal lines for choice (only with min or max) 
#' @param callFrom (character) allows easier tracking of messages produced
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @return This function returns a (numeric) matrix or data.frame with summarized data and add'l col with number of initial redundant lines
#' @seealso simple/partial functionality in \code{\link{summarizeCols}},  \code{\link{checkSimValueInSer}}
#' @examples
#' t3 <- data.frame(ref=rep(11:15,3),tx=letters[1:15],
#'   matrix(round(runif(30,-3,2),1),nc=2),stringsAsFactors=FALSE)
#' by(t3,t3[,1],function(x) x)
#' t(sapply(by(t3,t3[,1],function(x) x), summarizeCols, me="maxAbsOfRef"))
#' # calculate mean for lines concerened of all columns :
#' (xt3 <- makeNRedMatr(t3, summ="mean", iniID="ref"))
#' # choose lines based only on content of column 'X1' (here: max):
#' (xt3 <- makeNRedMatr(t3, summ=c(X1="maxOfRef"), iniID="ref")) 
#' @export
makeNRedMatr <- function(dat, summarizeRedAs, iniID="iniID", retDataFrame=TRUE, nEqu=FALSE, silent=FALSE, debug=FALSE, callFrom=NULL) {
  fxNa <- .composeCallName(callFrom, newNa="makeNRedMatr")
  maxLaArg <- c("maxOfRef","minOfRef","maxAbsOfRef")    
  summOpt <- c("median","mean","min","max","first","last",maxLaArg)   
  summOpt <- c(summOpt, paste0(summOpt,"Complete"))
  chSuMeth <- summarizeRedAs %in% summOpt
  txt <- " argument 'dat' must be matrix or data.frame with >1 line"
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  if(length(dim(dat)) <1) stop(txt) else if(nrow(dat) <2) return(dat)
  if(all(!chSuMeth)) stop(fxNa,"Value(s) ",pasteC(utils::head(chSuMeth),quoteC="'")," in argument 'summarizeRedAs' not valid")  
  if(any(!chSuMeth) && !silent) message(fxNa,"TROUBLE ahead ? Some methods specified in 'summarizeRedAs' seem not valid")  
  iniColNa <- colnames(dat)
  txt <- "'iniID' must be single column name of 'dat' !"
  if(length(iniID) !=1) stop(fxNa,txt) else if(is.na(iniID)) stop(fxNa,txt)
  iniIDcol <- colnames(dat) ==iniID
  if(sum(iniIDcol) <1) stop(fxNa,txt,"  Cannot identify column '",iniID,"' in input !")
  refID <- dat[,iniID]
  refDat <- refCol <- NULL

  ## check/correct for NAs in refID ?(ID for grouping) ??
  sumRedMode <- rep("",sum(!iniIDcol))                                        # need to know which col is numeric when treating each col individually
  for(i in which(!iniIDcol)) sumRedMode[i] <- mode(dat[,i])  
  if(any(summarizeRedAs %in% maxLaArg, na.rm=TRUE)) {                          # has special methods (limit to those)
    if(!silent && length(summarizeRedAs) >1) message(fxNa,"Canot use all ",length(summarizeRedAs),
      " methods specified in 'summarizeRedAs', only 1 method can be applied, using 1st of special methods") 
    summarizeRedAs <- summarizeRedAs[which(summarizeRedAs %in% maxLaArg)]
    if(all(names(summarizeRedAs) %in% colnames(dat),na.rm=TRUE)) {            # (col) names identified
      summarizeRedAs <- summarizeRedAs[which(names(summarizeRedAs) %in% colnames(dat))[1]]
    } else {
      summarizeRedAs <- summarizeRedAs[1]
      lastNum <- sumRedMode %in% c("numeric","integer")
      lastNum <- which(lastNum)[sum(lastNum,na.rm=TRUE)]      
      if(!silent) message(fxNa,"Which column to use with '",summarizeRedAs,"' not specified, using last numeric '",colnames(dat)[lastNum],"'")
      names(summarizeRedAs) <- colnames(dat)[lastNum] }
  } else if(length(summarizeRedAs) < ncol(dat)) {         
    if(length(summarizeRedAs) >1 && !silent) message(fxNa,"Too few (",length(summarizeRedAs),") methods specified, recycling methods")
    summarizeRedAs <- rep(summarizeRedAs, ncol(dat))[1:(ncol(dat))]
  }
  if(debug) {message(fxNa,"mNR1"); mNR1 <- list(dat=dat,summarizeRedAs=summarizeRedAs,iniID=iniID,retDataFrame=retDataFrame,sumRedMode=sumRedMode,iniIDcol=iniIDcol,nEqu=nEqu)}

  out <- matrix(NA, nrow=length(unique(refID)), ncol=ncol(dat), dimnames=list(unique(refID),colnames(dat)))       # initalize (for all summariz w/o special meth)
  refID <- dat[,which(iniIDcol)]
  ## prepare argument "summarizeRedAs"
  if(any(summarizeRedAs %in% maxLaArg)) {
    if(any(names(summarizeRedAs) %in% colnames(dat), na.rm=TRUE)) {
      ## has name to use as ref
      if(length(summarizeRedAs) >1) {  if(!silent) message(fxNa,"Only a single value of argument 'summarizeRedAs' can be used when reference-approach is chosen !")
        summarizeRedAs <- summarizeRedAs[which.min(!names(summarizeRedAs) %in% colnames(dat)[refID])]}     # reduce 'summarizeRedAs' to 1st instance
    } else {
      lastNum <- sumRedMode %in% c("numeric","integer")
      lastNum <- which(lastNum)[sum(lastNum,na.rm=TRUE)]
      if(!silent) message(fxNa,"Which column to use with '",summarizeRedAs,"' not specified, using last numeric '",colnames(dat)[lastNum],"'")
      names(summarizeRedAs) <- colnames(dat)[lastNum] 
    }
    refCol <- which.min(!colnames(dat) %in% names(summarizeRedAs))       # which col of data as key
    refDat <- dat[,refCol]
    names(refCol) <- colnames(dat)[refCol] 
    if(debug) message(fxNa,"refCol = '",names(refCol),"'")                                    
  } else {
    if(length(summarizeRedAs) < sum(!iniIDcol)) {
      if(!silent) message(fxNa,"Argument 'summarizeRedAs' has to few elements, extening by ",sum(!iniIDcol) -length(summarizeRedAs)," elements")
      summarizeRedAs <- rep(summarizeRedAs, ceiling(sum(!iniIDcol)/length(summarizeRedAs)))[1:sum(!iniIDcol)]
      names(summarizeRedAs) <- colnames(dat)[which(!iniIDcol)] }    
  }

  ##  summarize all cols together (based on summarizeRedAs) 
  if(debug) {message(fxNa,"mNR2a"); mNR2a <- list(dat=dat,summarizeRedAs=summarizeRedAs,iniID=iniID,refDat=refDat,retDataFrame=retDataFrame,out=out,sumRedMode=sumRedMode,refID=refID,iniIDcol=iniIDcol,nEqu=nEqu,maxLaArg=maxLaArg)}
  if(any(summarizeRedAs %in% maxLaArg)) {                            
    tmp <- cbind(dat, ref=refDat) 
    tmp <- by(tmp, dat[,which(iniIDcol)], summarizeCols, meth=summarizeRedAs, refCol=refCol, silent=silent, debug=debug, callFrom=fxNa)
    out <- data.frame(do.call("rbind", tmp))[,1:ncol(dat)]    
    colnames(out) <- colnames(dat) 
  } else { if(length(unique(summarizeRedAs)) >1) { for(i in which(!iniIDcol)) {              # various summarization methods defined for each col ...
      ## key for grouping : iniIDcol    key for summarizing : which(colnames(dat)==names(summarizeRedAs)) 
      if(debug) {message(fxNa,"mNR2b"); mNR2b <- list()}
      tmp <- tapply(dat[,i], refID, function(x) if(is.numeric(x) && !all(is.na(x))) .summarizeCols(x,me=summarizeRedAs[i]) else .sortMid(x))
      out[,i] <- tmp }
    if(!silent) message(fxNa,"Various summarization methods appear, run col by col")      
    } else {                                                     # all summerization by same mode, run batch for numeric & batch for character
      if(!silent) message(fxNa,"Common summarization method '",unique(summarizeRedAs),"', run as batch")      
      if(any(sumRedMode != "numeric")) {
        i <- which(sumRedMode != "numeric")   # & colnames(dat) !=iniID
        out[,i] <- tmp <- t(sapply(by(dat[,i], refID,function(y) y), function(x) apply(x, 2, .sortMid))) }
      if(debug) {message(fxNa,"mNR2c"); mNR2c <- list()}
      if(any(sumRedMode == "numeric")) {
        i <- which(sumRedMode == "numeric")   # & colnames(dat) !=iniID
        out[,i] <- tmp <- t(sapply(by(dat[,i], refID, function(y) y), function(x) .summarizeCols(x, me=summarizeRedAs[i][1]))) }
  } }
  if(debug) {message(fxNa,"mNR3"); mNR3 <- list()}
  if(!silent) message(fxNa,"Summarize redundant based on col '",iniID,"'  using method(s) : ",pasteC(summarizeRedAs,quoteC="'"),
    if(any(summarizeRedAs %in% maxLaArg)) c(" and col '",names(summarizeRedAs)[1],"'")," yielding ",ncol(out)," cols")
  if(debug) {message(fxNa," mNR4")}
  if(length(dim(tmp)) >1) { if(length(rownames(tmp)) >0) rownames(out) <- rownames(tmp)
    } else if(length(names(tmp)) >0) rownames(out) <- names(tmp) 
  out <- cbind(out, tapply(dat[,1], dat[,which(iniIDcol)], length))
  colnames(out)[ncol(out)] <- if("nRedLi" %in% colnames(out)) paste(colnames(out)[rev(grep("^nRedLi",colnames(out)))[1]],"X",sep=".") else "nRedLi"
  if(retDataFrame && "character" %in% mode(out)) out <- convMatr2df(out, silent=silent)
  out }


#' Choose most frequent or middle of sorted vector
#'
#' This function chooses the (first) most frequent or  middle of sorted vector, similar to the concept of \code{mode}
#'
#' @param x (numeric) main input 
#' @param retVal (logical) return value of most frequent, if \code{FALSE} return index of (1st) 'x' for most frequent
#' @return This function returns a numeric verctor
#' @seealso simple/partial functionality in \code{\link{summarizeCols}},  \code{\link{checkSimValueInSer}}
#' @examples
#' .sortMid(11:14)
#' .sortMid(rep("b",3))
#' @export
.sortMid <- function(x, retVal=TRUE) {
  ##  choose (1st) most frequent or (if all equal) middle of sorted vector 
  ## 'x' .. character vector; return single 'representative' element
  ## 'retVal'.. return value of most frequent, if FALSE return index of (1st) 'x' for most frequent
  ## note : in  case of equally frequent only one is chose, no warning
  x <- naOmit(x); if(length(x) >0) { y <- table(x)
    out <- if(sum(duplicated(y))==length(y)-1)  sort(x)[ceiling(length(x)/2)] else names(y)[which.max(y)]} else NA 
  if(!retVal) out <- which(x==out[1])[1]
  out }
    

#' Estimate Mode (Most Frequent Value)   
#' 
#' Estimate mode, ie most frequent value. In case of continuous numeric data, the most frequent values may not be the most frequently repeated exact term.
#' This function offers various approches to estimate the mode of a numeric vector. 
#' Besides, it can also be used to identify the most frequentexact term (in this case also from character vectors). 
#'
#' @details
#' The argument \code{method} allows to choose among (so far) 4 different methods available.
#' If "density" is chosen, the most dense region of sqrt(n) values will be chosen; 
#' if "binning", the data will be binned (like in histograms) via rounding to a user-defined number of significant values ("rangeSign").
#' If \code{method} is set to "BBmisc", the function \code{computeMode()} from package \href{https://CRAN.R-project.org/package=BBmisc}{BBmisc} will be used.
#' If "mode" is chosen, the first most frequently occuring (exact) value will be returned, if "allModes", all ties will be returned. This last mode also works with character input. 
#'	 
#' @param x (numeric, or character if 'method='mode') data to find/estimate most frequent value
#' @param method (character) There are 3 options : BBmisc, binning and density (default). If "binning" the function will search context dependent, ie like most frequent class of histogram.
#'  Using "binning" mode the search will be refined if either 80 percent of values in single class or >50 percent in single class.
#' @param finiteOnly (logical) suppress non-finite values; allows avoiding \code{NULL} as result in presence of some \code{Inf} values; \code{NA} will be ignored in any case
#' @param bandw (integer) only used when \code{method="binning"} or  \code{method="density"} : defines the number of points to look for density or number of classes used; 
#'  very "critical" parameter, may change results in strong way. Note: with \code{method="binning"}: At higher values for "bandw" you will finally loose advantage of histLike-type search of mode !
#' @param rangeSign (integer) only used when \code{method="binning"}: range of numbers used as number of significant values

#' @param silent (logical) suppress messages
#' @param callFrom (character) allows easier tracking of messages produced
#' @param debug (logical) additional messages for debugging 
#' @return This function returns a numeric vector with value of mode, the name of the value indicates it's position
#' @seealso \code{computeMode()} in package \href{https://CRAN.R-project.org/package=BBmisc}{BBmisc}
#' @examples
#' set.seed(2012); dat <- round(c(rnorm(50), runif(100)),3)
#' stableMode(dat)
#' @export
stableMode <- function(x, method="density", finiteOnly=TRUE, bandw=NULL, rangeSign=1:6, silent=FALSE, callFrom=NULL, debug=FALSE) {
  ## stable mode  
  fxNa <- .composeCallName(callFrom, newNa="stableMode")
  if(isTRUE(debug)) silent <- FALSE else { debug <- FALSE
    if(!isTRUE(silent)) silent <- FALSE }
  ## prepare data: treat NA or non-finite values
  if(finiteOnly) { chFin <- is.finite(x)
    if(all(!chFin)) x <- NULL else if(any(!chFin)) {x <- x[which(chFin)]
      if(!silent) message(fxNa,"Removing ",sum(!chFin)," (out of ",length(chFin),") non-finite values") }
  } else x <- naOmit(x)
  if(length(unique(x)) < 0) {
    method <- NULL
    return(NULL)
    if(!silent)  message(fxNa, "NO numeric values (nothing to do)") 
  } else if(length(unique(x)) == 1) {
    method <- NULL
    return(x[1])
    if(!silent && length(x) >1) message(fxNa, "All values are the same (= mode)") }
  if(identical(method, "dens"))  method <- "density"
  if(identical(method, "bin")) method <- "binning"
  if(identical(method, "histLike")) {
    method <- "binning"
    if(!silent) message(fxNa, "Note: argument option 'histLike' has been depreciated and replaced by 'binning'") }
  out <- NULL
  isNum <- is.numeric(x)
  ## check type of input
  if(any(sapply(c("BBmisc","density","binning"), identical, method)) && !isNum) {    
    chNum <- try(as.numeric(if(is.factor(x)) as.character(x) else x), silent=TRUE)
    if(inherits(chNum, "try-error")) {
      if(!silent) message(fxNa,"Note : Input is NOT numeric, not compatible with method chosen, thus setting method='mode' !")
     method <- "mode"
  }}
  ## find simply most frequent exact value(s)
  if(any(sapply(c("allModes","mode","asIs"), identical, method))) {
    if(!is.factor(x)) x <- factor(as.character(x))
    tabX <- tabulate(x)
    out <- if(identical(method, "allModes")) {
      levels(x)[which(tabX == max(tabX))]
    } else levels(x)[which.max(tabX)]
    if(isNum) out <- as.numeric(out)
    if(debug) {message(fxNa," stM1"); stM1 <- list(method=method,bandw=bandw,x=x,tabX=tabX) }
  } else  chDu <- sum(duplicated(x)) 
  ## BBmisc
  if(identical(method, "BBmisc")) {
    if(requireNamespace("BBmisc")) {
      mo <- try(sapply(rangeSign, function(y) BBmisc::computeMode(signif(x, y))), silent=TRUE)
      if(inherits(mo, "try-error")) { method <- "density"
        warning(fxNa,"UNABLE to calulate BBmisc::computeMode(),  setting 'method' to 'density'")
      } else  { posi <- .firstMin(diff(mo)/mo[-length(mo)], positionOnly = TRUE)
        out <- mo[posi] }
      if(debug) {message(fxNa," stM2"); stM2 <- list(method=method,bandw=bandw,x=x,mo=mo) }
    } else { method <- "density"
      if(!silent) message(fxNa,"NOTE: Package 'BBmisc' not found ! Please install first from CRAN \n   setting 'method' to 'density'")
    }  
  }  
  ## density 
  if(identical(method, "density")) {
    if(length(bandw) <1) {bandw <- round(1.4* sqrt(length(x)))
      if(!silent) message(fxNa,"Method='density',  length of x =",length(x),", 'bandw' has been set to ",bandw)}
    x <- sort(x)
    raX <- max(x) - min(x)
    nExt <- bandw - 1
    x <- c(min(x) - raX * (nExt:1)/nExt, x, max(x) + raX * (1:nExt)/nExt)
    dif <- x[(bandw + 1):length(x)] - x[1:(length(x) - bandw)]
    maxDi <- which(dif == min(dif) & is.finite(dif))
    if(length(maxDi) > 1) maxDi <- maxDi[round(length(maxDi)/2)]
    out <- x[maxDi +nExt]
    names(out) <- maxDi
    if(debug) {message(fxNa," stM3"); stM3 <- list(method=method,bandw=bandw,x=x,raX=raX,nExt=nExt,maxDi=maxDi,dif=dif) }
  }
  ## binning
  if(identical(method, "binning")) {
    if(!all(length(bandw) >0, is.numeric(bandw))) bandw <- ceiling(sqrt(length(x)))
    if(70 * bandw > length(x) && !silent)  message(fxNa,"Method='binning', value of 'bandw'=", bandw, " may be too high for good functioning !")
    xRa <- range(x[which(is.finite(x))])
    frq <- table(cut(x, breaks=seq(xRa[1], xRa[2], length.out=bandw)))
    che <- max(frq, na.rm=TRUE) > c(0.5, 0.8) * length(x)
    if (che[2]) {
      if (!silent)  message(fxNa, ">80% of values in class no ", which.max(frq), ", refining mode estimation")
      mxF <- which.max(frq)
      mxF <- signif(seq(xRa[1], xRa[2], length.out = bandw)[c(max(mxF -3, 1), min(mxF +3, bandw))], 4)
      frq <- table(cut(x, breaks=seq(mxF[1], mxF[2], length.out=bandw)))
    } else {
      if(che[1] && sum(frq < length(x)/5000) > 0.5 * bandw) {
        if(!silent) message(fxNa, ">50% of values in class no ", 
          which.max(frq), " & >50% of other classes almost empty, refining result")
        useBr <- range(which(frq > 0.05 * length(x))) +c(-1, 1)
        useBr <- c(max(useBr[1], 1), min(useBr[2], bandw))
        mxF <- signif(seq(xRa[1], xRa[2], length.out=bandw)[useBr], 4)
        frq <- table(cut(x, breaks=seq(mxF[1], mxF[2], length.out=bandw)))
      }
    }
    out <- which.max(frq)
    out <- names(frq)[which.max(frq)]
    out <- as.numeric(unlist(strsplit(substr(out, 2, nchar(out) -1), ",")))
    out <- sum(out)/length(out)
  }
  out
}
  

#' Filter For Each Group Of Columns For Sufficient Data As Non-NA
#'
#' The aim of this function is to filter for each group of columns for sufficient data as non-NA.
#' 
#'
#' @details
#' This function allows to identify lines with an \code{NA}-content above the threshold \code{presThr} per group as defined by the levels of factor \code{grp}.  
#' With different types of projects/questions different threshold  \code{presThr} levels may be useful.
#' For example, if one would like to keep the degree of threshold  \code{presThr}s per group rather low, one could use a value of 0.75 (ie >= 75% values non-NA. 
#' 
#' 
#'
#' @param dat matrix or data.frame (abundance or expression-values which may contain some \code{NA}s).
#' @param grp factor of min 2 levels describing which column of 'dat' belongs to which group (levels 1 & 2 will be used)
#' @param presThr (numeric) min ratio of non- \code{NA} values (per group) for returning a given line & group as  \code{TRUE} 
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return logical matrix (with on column for each level of \code{grp})
#' @seealso  \code{\link{presenceFilt}}, there are also other packages totaly dedicated to filtering on CRAN and Bioconductor
#' @examples
#'  mat <- matrix(NA, nrow=11, ncol=6)
#'  mat[lower.tri(mat)] <- 1
#'  mat <- cbind(mat, mat[,1:4])
#'  colnames(mat) <- c(paste0("re",1:6), paste0("x",1:4))
#'  mat[6:8,7:10] <- mat[1:3,7:10]  # ref
#'  mat[9:11,1:6] <- mat[2:4,1:6]
#'
#' ##  accept 1 NA out of 4, 2 NA out of 6   (ie certainly present)
#'  (filt0a <- presenceGrpFilt(mat, rep(1:2, c(6,4)), pres=0.66))
#' ##  accept 2 NA out of 4, 2 NA out of 6   (ie min 50% present)
#'  (filt0b <- presenceGrpFilt(mat, rep(1:2, c(6,4)), pres=0.5))
#' ##  accept 3 NA out of 4, 4 NA out of 6   (ie possibly present)
#'  (filt0c <- presenceGrpFilt(mat, rep(1:2, c(6,4)), pres=0.19))
#'
#' @export
presenceGrpFilt <- function(dat, grp, presThr=0.75, silent=FALSE, debug=FALSE, callFrom=NULL) {
  ## dat(matrix or data.frame)
  ## filter for each group of columns for sufficient data as non-NA
  fxNa <- .composeCallName(callFrom, newNa="presenceGrpFilt")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE

  if(length(grp) != ncol(dat)) stop("Number of columns of 'dat' (",ncol(dat),") does NOT match length of 'grp' (",length(grp),") !")
  if(presThr >1) { presThr <- 1; if(!silent) message(fxNa, "Argument 'presThr' may not be higher tan 1 (ie 100%), correcting")}
  if(debug) {message(fxNa," pGF1"); pGF1 <- list(dat=dat,grp=grp,presThr=presThr)}
  ## construct unique group-names to use
  chDu <-  duplicated(grp, fromLast=FALSE)
  grpU <- if(any(chDu)) as.character(grp)[which(!chDu)] else as.character(grp)
  ## main filter
  out <- matrix(nrow=nrow(dat), ncol=length(grpU), dimnames=list(rownames(dat), grpU))
  for(i in 1:length(grpU)) { useCol <- which(grp ==grpU[i])
    out[,i] <- if(length(useCol) >0) {if(length(useCol) ==1) !is.na(dat[,useCol]) else rowSums(!is.na(dat[,useCol]))/length(useCol) >= presThr} else FALSE }
  out }
  

#' Extract numeric part of matrix or data.frame
#'
#' This function extracts numeric part of matrix or data.frame, removing remaining non-numeric elements if \code{trimToData} is set to \code{TRUE}.
#' Note, that cropping entire lines where a (single) text element appeared may quickly reduce the overal content of the input data.
#'
#' @param dat matrix (or data.frame) for extracting numeric parts
#' @param trimToData (logical) default to remove (crop) lines and cols contributing to NA, non-numeric data is transfomed to NA
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of message(s) produced
#' @return This function reurns a matrix of numeric data
#' @examples
#' mat <- matrix(c(letters[1:7],14:16,LETTERS[1:6]),nrow=4,dimnames=list(1:4,letters[1:4]))
#' mat; extrNumericFromMatr(mat)
#' mat <- matrix(c(letters[1:4],1,"e",12:19,LETTERS[1:6]),nr=5,dimnames=list(11:15,letters[1:4]))
#' mat; extrNumericFromMatr(mat)
#' @export
extrNumericFromMatr <- function(dat, trimToData=TRUE, silent=FALSE, debug=FALSE, callFrom=NULL) {
  fxNa <- .composeCallName(callFrom, newNa="extrNumericFromMatr")
  dimNa <- dimnames(dat)
  chDat <- is.data.frame(dat)
  if(is.data.frame(dat)) {
    facCol <- sapply(1:ncol(dat), function(x) is.factor(dat[,x]))
    if(sum(facCol,na.rm=TRUE) >0) for(i in which(facCol)) dat[,i] <- as.character(dat[,i])  
    dat <- gsub("\x5E[[:blank:]]+","",as.matrix(dat)) }  
  out <- matrix(NA_real_, nrow=nrow(dat), ncol=ncol(dat))
  for(i in 1:ncol(dat)) {x <- .mayBeNum(dat[,i]); if(length(x)>0) out[x,i] <- dat[x,i]}
  out <- matrix(as.numeric(out), nrow=nrow(dat), ncol=ncol(dat), dimnames=dimnames(dat))
  if(trimToData) {
    useLi <- table(unlist(apply(out, 2, function(x) (1:length(x))[!is.na(x)])))
    useLi <- which(useLi==max(useLi))
    useCo <- table(unlist(apply(out, 1, function(x) (1:length(x))[!is.na(x)])))
    useCo <- which(useCo==max(useCo))
    ## primary cropping
    ou2 <- out[useLi,useCo]                                                      
    if(length(dim(ou2)) <1) ou2 <- matrix(ou2, nrow=length(useLi), dimnames=list(rownames(dat)[useLi],colnames(dat)[useCo]))
    ## final/add'l cropping of lines with NAs
    out <- ou2[which(rowSums(is.na(ou2)) <1),]                                    
    if(length(dim(out)) <1) out <- matrix(out, ncol=ncol(ou2), dimnames=list(rownames(ou2)[which(rowSums(is.na(ou2)) <1)], colnames(ou2)))
    txt <- c("Nothing remains when cropping for numeric part !","Removed "," out of "," columns and "," rows")
    if(!silent) message(fxNa, if(length(out) <1) txt[1] else c(txt[2], ncol(dat)-ncol(out),
      txt[3], ncol(dat), txt[4], nrow(dat)-nrow(out), txt[3], nrow(dat), txt[5]))
    }         
  out }    

#' Value Matching With Option For Concatenated Terms
#'
#' This is a _match()_-like function allowing to serach among concatenated terms/IDs, additional options to remove text pattern like terminal lowercase extesion are available.
#' The function returns a named vector indicating the positions of (first) matches similar to \code{\link[base]{match}}.
#'
#' @details
#' The main motivation to create this function was to be able to treat concatenated entries and to look if \code{any} of the concatenated values match to 'x'.
#' This function offers additional options for trimming values before running the main comparison.
#'
#' Of course, the concatenation strategy must be known and only a single concatenation separator (which may be multiple characters long) may be used for both \code{x} and \code{match}.
#' Thus result will only indicate that at least one of the concatenated terms had a match, but not which one.
#' Finally, both vectors \code{x} and \code{table} may contain concatenated terms.
#' In this case this function will require much more computational ressources due to the increased combinatorics when comparing larger vectors.
#'
#' Please note, that in case of multiple to multiple matches, only the first hit gets reported.
#'
#' The argument \code{globalPat="digitExtension"} allows eg reducing 'A1234-4' to 'A1234'.
#'
#' @param x (vector) the values to be matched
#' @param table (vector) the values to be matched against (ie reference)
#' @param sep (character) separator character in case concatenation of entries is tested
#' @param sepPattern (character or \code{NULL}) optional custom  pattern for splitting concatenations of \code{x}) and \code{table}) (in case \code{NULL}) is not sufficient)
#' @param globalPat (character) pattern for additional trimming of serach-terms. If \code{globalPat="digitExtension"} all terminal digits will not be considered when matching
#' @param nomatch (vector) similar to \code{\link[base]{match}} the value to be returned in the case when no match is found
#' @param incomparables (vector) similar to \code{\link[base]{match}}, a vector of values that cannot be matched. Any value in x matching a value in this vector is assigned the nomatch value.
#' @param extensPat (logical) similar to \code{\link[base]{match}} the value to be returned in the case when no match is found
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a character vector with verified path and file-name(s), returns \code{NULL} if nothing
#' @seealso \code{\link[base]{match}} (for two simple vectors without concatenated terms), \code{\link[base]{grep}}
#' @examples
#' tab1 <- c("AA","BB-5","CCab","FF")
#' tab2 <- c("AA","WW,Vde,BB-5,E","CCab","FF,Uef")
#' x1 <- c("ZZ","YY","AA","BB-2","DD","CCdef","Dxy")            # modif of single ID (no concat)
#' concatMatch(x1, tab2)
#' x2 <- c("ZZ,Z","YY,Y","AA,Z,Y","BB-2","DD","X,CCdef","Dxy")  # conatenated in 'x'
#' concatMatch(x2, tab2)
#' tab1 <- c("AA","BB-5","CCab","FF")              # no conatenated in 'table'
#' concatMatch(x2, tab1)                          # simple case of no concat anywhere
#' concatMatch(x1, tab1)
#' @export
concatMatch <- function(x, table, sep=",", sepPattern=NULL, globalPat="digitExtension", nomatch=NA_integer_, incomparables=NULL, extensPat=TRUE, silent=FALSE, debug=FALSE, callFrom=NULL) {
  ## move to wrMisc ?
  ## idea : find where x (eg fasta) matche(s) IN table (MQresult, has concat)
  ## both \code{x} and \code{table} may contain concatenated IDs
  ## match-like function including each of concatenated terms/IDss, add'l option to remove add'l pattern like terminal lowercase extension
  ## strategy A : few concatenations : run match() on 1st set of table, 2nd set etc
  ## strategy B : many concat & short x : grep each x in table (as heading, sep+x+sep, as term)
  ##  globalPat (character, 'digitExtension') regular expression for searching eg reduce 'A1234-4' to 'A1234'
  ##  extensPat (charcter) \code{TRUE} for removing terminal lower case eg ('AB1234ups')
  ## need to find if any of concatenated 'x' in 'table'
  ## prepare
  fxNa <- .composeCallName(callFrom, newNa="concatMatch")
  if(isTRUE(debug)) silent <- FALSE else { debug <- FALSE
    if(!isTRUE(silent)) silent <- FALSE }
  xIni <- x; tableIni <- table           # initialize backup
  chDux <- duplicated(x)
  if(length(x) <1) stop("Invalid entry, 'x' must be at least of length=1")
  if(length(table) <1) stop("Invalid entry, 'table' must be at least of length=1")
  if(!any(c("numeric", "character", "logical") %in% class(x)))  stop("Invalid entry, 'x' must be numeric, character or logical vector")
  if(!any(c("numeric", "character", "logical") %in% class(table)))  stop("Invalid entry, 'table' must be numeric, character or logical vector")
  if(any(chDux) && !silent) message(fxNa,"Note, some entries of 'x' are not unique !")
  chDut <- duplicated(table)
  if(any(chDut) && !silent) message(fxNa,"Note, some entries of 'table' are not unique !")
  if(debug) {message(fxNa,"cMa1"); cMa1 <- list(x=x,table=table, sep=sep,sepPattern=sepPattern,globalPat=globalPat,extensPat=extensPat)}
  ## rm term sep
  chSep <- grep(paste0(sep,"$"), x)
  if(length(chSep) >0) {x <- sub(paste0(sep,"$"), "", x)}
  chSep <- grep(paste0(sep,"$"), table)
  if(length(chSep) >0)  {table <- sub(paste0(sep,"$"), "", table)}
  ## however - possibility that x or table become non-unique due to manipulation !!
  if(length(sepPattern) <1) sepPattern <- sep
  ## remove terinal extensions (eg AB1234ups)
  if(length(extensPat) ==1) {
    if(isTRUE(extensPat)) {     # remove terminal lower case eg ('AB1234ups')
      chPatXt <- grep("[A-Z0-9][[:lower:]]+$", x)                   # tailing pattern
      chPatXc <- grep(paste0("[A-Z0-9][[:lower:]]+",sep,"."), x)    # center pattern
      chPatTt <- grep("[A-Z0-9][[:lower:]]+$", table)
      chPatTc <- grep(paste0("[A-Z0-9][[:lower:]]+",sep,"."), table)
      if(length(chPatXt) >0) x[chPatXt] <- sub("[[:lower:]]+$","", x[chPatXt])
      if(length(chPatXc) >0) x[chPatXc] <- sub(paste0("[[:lower:]]+",sep), sep, x[chPatXc])
      if(length(chPatTt) >0) table[chPatTt] <- sub("[[:lower:]]+$","", table[chPatTt])
      if(length(chPatTc) >0) table[chPatTc] <- sub(paste0("[[:lower:]]+",sep), sep, table[chPatTc])
    } else {
      chExt <- grep(extensPat, x)
      if(length(chExt) >0) x <- sub(extensPat, "", x)}
    chDux <- duplicated(x)
    if(any(chDux) && !silent) message(fxNa,"Note, some entries of 'x' are not unique ! (after cleaning by 'extensPat')")
    chDut<- duplicated(table)
    if(any(chDut) && !silent) message(fxNa,"Note, some entries of 'table' are not unique ! (after cleaning by 'extensPat')")
  }
  ## global pattern modif
  if(length(globalPat) >0) {
    ## however - possibility that x or table become non-unique due to manipulation !!
    if(identical(globalPat, "digitExtension")) {                          ## remove all after '-' eg ('P1234-5')
      chPatXt <- grep("[A-Z0-9]\\-[[:digit:]]{1,4}$", x)                   # tailing pattern
      chPatXc <- grep(paste0("[A-Z0-9]\\-[[:digit:]]{1,4}",sep,"."), x)    # center pattern
      chPatTt <- grep("[A-Z0-9]\\-[[:digit:]]{1,4}$", table)
      chPatTc <- grep(paste0("[A-Z0-9]\\-[[:digit:]]{1,4}",sep,"."), table)
      if(length(chPatXt) >0) x[chPatXt] <- sub("\\-[[:digit:]]{1,4}$","", x[chPatXt])
      if(length(chPatXc) >0) x[chPatXc] <- sub(paste0("\\-[[:digit:]]{1,4}",sep), sep, x[chPatXc])
      if(length(chPatTt) >0) table[chPatTt] <- sub("\\-[[:digit:]]{1,4}$","", table[chPatTt])
      if(length(chPatTc) >0) table[chPatTc] <- sub(paste0("\\-[[:digit:]]{1,4}",sep), sep, table[chPatTc])
    } else {
      chPat <- grep(globalPat, x)
      if(length(chPat) >0) x[chPat] <- sub(globalPat, sep, x[chPat])
      chPat <- grep(globalPat, table)
      if(length(chPat) >0) table[chPat] <- sub(globalPat, sep, table[chPat])}
    chDux <- duplicated(x)
    if(any(chDux) && !silent) message(fxNa,"Note, some entries of 'x' are not unique ! (after cleaning by 'globalPat')")
    chDut<- duplicated(table)
    if(any(chDut) && !silent) message(fxNa,"Note, some entries of 'table' are not unique ! (after cleaning by 'globalPat')")
  }

  if(debug) { message(fxNa,"cMa2"); cMa2 <- list(x=x,table=table, sep=sep,sepPattern=sepPattern,globalPat=globalPat,extensPat=extensPat,chSep=chSep)}

  ## MAIN :
  ## start with regular match
  out <- match(x, table, nomatch=nomatch, incomparables=incomparables)
  x1 <- x; table1 <- table
  names(out) <- x
  chMa <- is.na(out)

  if(any(chMa)) {
    ## not all items of 'x' found in 'table' ==> try complementing missing
    ##  now work only on x not prevusouly found !
    if(debug) {message(fxNa,sum(chMa)," ID/term of 'x' not found so far : ", pasteC(utils::head(x[which(chMa)], quoteC="'")))}
    ## check for separators
    chSepT <- grep(sepPattern, table)             # which 'table' are concatented => useful for splitting
    chSepX <- grep(sepPattern, x[which(chMa)])    # don't use truncation (strat A) when x has concatenation
    if(debug) { message(fxNa,"cMa3"); cMa3 <- list(x=x,table=table,out=out,chMa=chMa,chSepT=chSepT,chSepX=chSepX, sep=sep,sepPattern=sepPattern,globalPat=globalPat,extensPat=extensPat,chSep=chSep)}
    if(length(chSepT) >0 || length(chSepX) >0) {
      if(debug) message(fxNa,"Found ",length(chSepT)," cases of conatenated IDs in 'table' -> split;  ",length(chSepX)," cases of conatenated 'x'")
      ## has separators
      ## now devide strat A and B
      nCharX <- nchar(x)
      nSepX <- (nCharX - nchar(gsub(sepPattern,"", x))) /nchar(sep)
      nCharTab <- nchar(table)
      nSepTab <- (nCharTab - nchar(gsub(sepPattern,"", table))) /nchar(sep)       # number of concatenated terms for each 'table'
      if(length(chSepX) <1 && max(nSepTab, na.rm=TRUE) / length(x) < 0.5 && max(nSepTab, na.rm=TRUE) < 15) {
        ## strategy A :
        ##   few concatenations : run match() on 1st set of table, 2nd set etc
        if(debug) message(fxNa,"Choose strategy A : few concatenations : run match() on 1st set of table, 2nd set etc")
        if(debug) { message(fxNa,"cMa4"); cMa4 <- list(x=x,table=table,out=out,chSepT=chSepT,chMa=chMa,chSepT=chSepT, sep=sep,sepPattern=sepPattern,globalPat=globalPat,extensPat=extensPat,chSep=chSep)}
        ##
        for(i in 0:max(nSepTab, na.rm=TRUE)) {
          if(length(chSepT) >0 && any(is.na(out))) {                   # yes, need to go further and split
            if(i >0) {chSepT <- grep(sepPattern, table)                # update (which 'table' are concatented => use for splitting)
               table <- substring(table, nchar(tabT) +2) }             # removve 1st element from left (already treated)
            tabT <- sub(paste0(sep,".+"), "", table[chSepT])           # truncate from right, ie leave only 1st
            outNa <- which(is.na(out))
            maT <- match(x[outNa], tabT, nomatch=nomatch, incomparables=incomparables)
              names(maT) <- x[which(is.na(out))]
            chNa <- !is.na(maT)
            if(any(chNa)) { out[outNa[which(chNa)]] <- chSepT[maT[which(chNa)]]
              if(debug) message(fxNa,"Found ",sum(chNa)," add'l matches in i=",i+1) }
          }
        }
      } else {
        ## strategy B :
        ##    many concat & short x : grep each 'x' in table (as heading, sep+x+sep, as term) ==> may be slow when 'x' is long
        ## also works if 'x' has concatenations
        if(debug) message(fxNa," ++ Strategy B : use grep for terminal 'x' or 'x' combined with 'sep'")
        .sepGrep <- function(xx, sep, table) grep(paste0("^",xx,"$|","^",xx,sep,"|",sep,xx,sep,"|",sep,xx,"$"), table)    # terminal & separator-grep
        if(length(chSepX) <1) {
          out2 <- lapply(x[which(chMa)], .sepGrep, sep, table)
          out2 <- sapply(out2, function(y) if(length(y) >0) y[1] else NA)
        } else {
          ## double concatenated (x & table) => strsplit x for grep of each !
          if(debug) { message(fxNa,"cMa5"); cMa5 <- list(x=x,table=table,out=out,chSepT=chSepT,chMa=chMa,chSepT=chSepT, sep=sep,sepPattern=sepPattern,globalPat=globalPat,extensPat=extensPat,chSep=chSep)}
          spl <- strsplit(x[which(chMa)], sep)
          naOm3 <- function(zz, sep, table) {zz <- unlist(lapply(zz, .sepGrep, sep, table)); isNa <- is.na(zz); if(any(!isNa)) zz[which(!isNa)[1]] else NA }  # return 1st non-NA or NA from sepGrep search
          out2 <- sapply(spl, naOm3, sep, table)
        }
        chLe <- sapply(out2, length)
        if(any(chLe) >1) {
          if(!silent) message(fxNa,"Found ",sum(chLe >1)," cased of multiple matching after split & grep, using only first") }
        out[which(chMa)] <- unlist(out2)
      }
    } else {if(debug) message(fxNa,"Remain with match-approach (no concatenations found)")
    }
  } else if(debug) message(fxNa,"All initial 'x' were found in 'table'")
out }

#' rbind on lists
#'
#' rbind-like function to append list-elements containing matrixes (or data.frames) and return one long table. 
#' All list-elements must have same number of columns (and same types of classes in case of data.frames.
#' Simple vectors (as list-elements) will be considered as sigle lines for attaching.
#' @param lst (list, composed of multiple matrix or data.frames or simple vectors) main input (each list-element should have same number of columns, numeric vectors will be converted to number of columns of other columns/elements)
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns (depending on input) a matrix or data.frame
#' @seealso \code{rbind} in \code{\link[base]{cbind}}
#' @examples
#' lst1 <- list(matrix(1:9, ncol=3, dimnames=list(letters[1:3],c("AA","BB","CC"))),
#'   11:13, matrix(51:56, ncol=3))
#' lrbind(lst1)
#' @export
lrbind <- function(lst, silent=FALSE, debug=FALSE, callFrom=NULL) {
  fxNa <- .composeCallName(callFrom, newNa="lrbind")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  if(debug) message(fxNa,"lrb1")
    
  argN <- deparse(substitute(lst))
  if(!isTRUE(silent)) silent <- FALSE
  chDf <- sapply(lst, inherits, "data.frame")
  if(any(chDf)) {
    iniCla <- sapply(lst[[which(chDf)[1]]], class)    # read types of classes of 1st data.frame
    for(i in which(chDf)) lst[[i]] <- as.matrix(lst[[i]])  
  }
  chNum <- sapply(lst, inherits, c("numeric","integer"))
  nCol <- if(any(!chNum)) ncol(lst[[which(!chNum)[1]]]) else min(sapply(lst, length), na.rm=TRUE)
  if(any(chNum)) for (i in which(chNum)) lst[[i]] <- matrix(lst[[i]], ncol=nCol)
  ldim <- sapply(lst, dim)
  if(length(unique(ldim[2,])) >1) stop("Bad dimension(s): each list-element should have same number of columns")
  outm <- matrix(nrow=sum(ldim[1,]), ncol=ldim[2,1])
  ldim <- rbind(ldim, cumsum(ldim[1,]))
  ldim <- rbind(ldim, c(1, ldim[3, -1*ncol(ldim)] +1))
  for(i in 1:ncol(ldim)) outm[ldim[4,i]:ldim[3,i],] <- lst[[i]]
  lstColn <- unlist(sapply(lst, colnames))
  if(!is.null(lstColn)) if(length(unique(lstColn)) ==ldim[2,1] && !silent) message(fxNa,"Col-names not homogenous or partially absent in ",argN)
  colnames(outm) <- colnames(lst[[1]])
  lstRown <- unlist(sapply(lst, rownames))
  if(!is.null(lstRown)) if(length(unique(lstRown)) ==ldim[3, ncol(ldim)]) {
    rownames(outm) <- unique(lstRown)} else if(!silent) message(fxNa,"Row-names not complete")
  if(any(chDf)) { outm <- as.data.frame(outm)
    chCla <- sapply(outm, class) ==iniCla
    if(any(!chCla)) for(i in which(!chCla)) class(outm[,i]) <- iniCla[i]}
  outm }   
   

#' Match All Lines of Matrix To Reference Note
#'
#' This function allows adjusting the order of lines of a matrix \code{mat} to a reference character-vector \code{ref},
#' even when initial direct matching of character-strings using \code{match} is not possible/successful.
#' In this case, various variants of using \code{grep} will be used to see if unambiguous matching is possible of characteristic parts of the text.
#' All columns of \code{mat} will be tested an the column giving the bes resuts will be used.
#'
#' @details
#' This function tests all columns of \code{mat} to find perfect matching results to the reference \code{ref}.
#' In case of multiple results the
#' In case no direct matching is possible, \code{grep} will be used to find the best partial matching.
#' The orderof the rows of input \code{mat} will be adjusted according to the matching results.
#'
#' If \code{addRef=TRUE}, the reference will be included as additional column to the results, too.
#'
#' @param mat (matrix or data.frame) main input, all columns of \code{mat} will be tested for (partial) matching of \code{ref}
#' @param ref (character, length must match ) reference for trying to match each of the columns of  \code{mat}
#' @param exclCol (character or integer) column-name or -index of column to ignore/exclude when looking for matches
#' @param addRef (logical), if \code{TRUE} the content of \code{ref} will be added to  \code{mat} as additional column
#' @param inclInfo (logical) allows returning list with new matrix and additional information
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allows easier tracking of messages produced
#' @return This function returns the input matrix in an adjusted order (plus an optional additional column showing the reference)
#'  or if \code{inclInfo=TRUE} a list with $mat (adjusted matrix), $byColumn, $newOrder and $method;
#'  the reference can bee added as additional last column if \code{addRef=TRUE}
#' @seealso \code{\link[base]{match}},  \code{\link[base]{grep}}, \code{\link{trimRedundText}}, \code{\link{replicateStructure}}
#' @examples
#' ## Note : columns b and e allow non-ambigous match, not all elements of e are present in a
#' mat0 <- cbind(a=c("mvvk","axxd","bxxd","vv"),b=c("iwwy","iyyu","kvvh","gxx"), c=rep(9,4),
#'   d=c("hgf","hgf","vxc","nvnn"), e=c("_vv_","_ww_","_xx_","_yy_"))
#' matchMatrixLinesToRef(mat0[,1:4], ref=mat0[,5])
#' matchMatrixLinesToRef(mat0[,1:4], ref=mat0[1:3,5], inclInfo=TRUE)
#'
#' matchMatrixLinesToRef(mat0[,-2], ref=mat0[,2], inclInfo=TRUE)   # needs 'reverse grep'
#' @export
matchMatrixLinesToRef <- function(mat, ref, exclCol=NULL, addRef=TRUE, inclInfo=FALSE, silent=FALSE, debug=FALSE, callFrom=NULL) {
  ## find best column for matching lines of mat (matrix) to ref (char vector) via two way grep
  ## return list $grep (matched matrix), $col best column
  fxNa <- .composeCallName(callFrom, newNa="matchMatrixLinesToRef")
  if(isTRUE(debug)) silent <- FALSE else { debug <- FALSE
    if(!isTRUE(silent)) silent <- FALSE }
  byCol <- out <- matElim <- msg <- msgM <- newOr <- chIdenCol <- NULL              # initialize
  datOK <- length(mat) >0
  namesXY <- sub("\\[.*","",c(deparse(substitute(mat)), deparse(substitute(ref))))
  .applyOrder <- function(mat,  ref, newOr, goodCol=NULL, matElim=NULL, chIdenCol=NULL, addRef=TRUE) {
    ## set rows of matrix 'mat' into new order 'newOr'   #; if 'matAlt' give, use instead of 'mat'
    ## 'ref' .. (character); add'l column to add to output
    ## 'newOr' ..(integer) new order
    ## 'chIdenCol' ..(logical) remove all TRUE; has names of very orig mat ; 
    ## 'matElim' ..(matrix) add'l matrix to add to output (order will be adjusted like mat)
    ## 'goodCol' - not used any more #from counting NAs (eg from match), has names of colnames of mat
    if(length(mat) <1 || length(newOr) <1 || length(ref) <1) { out <- mat       # empty or invalid entry - nothing to do
    } else {
      if(is.logical(newOr)) { if(length(newOr)==nrow(mat)) newOr <- which(newOr) else newOr <- NULL } }
    if(length(mat) >0 && length(newOr) >0) {  
      if(length(ref) != length(newOr)) addRef <- FALSE
      out <- if(length(newOr) ==1 || ncol(mat)==1) matrix(mat[newOr,], nrow=length(newOr), dimnames=list(rownames(mat)[newOr], colnames(mat))) else mat[newOr,]
      if(length(matElim) >0) {
        if(debug) {message(fxNa, "mMLR1"); mMLR1 <- list(out=out,mat=mat,ref=ref,newOr=newOr,goodCol=goodCol,matElim=matElim,chIdenCol=chIdenCol,addRef=addRef)}
        outDiNa <- list(rownames(mat), colnames(out)[-1*which(colnames(out) %in% colnames(matElim))])
        rmCol <- which(colnames(out) %in% colnames(matElim))
        out <- if(length(rmCol) < ncol(out) -1) out[,-1*rmCol] else matrix( out[,-1*rmCol], nrow=nrow(mat), dimnames=list(outDiNa[[1]], outDiNa[[1]][-1*rmCol]))
        if(length(chIdenCol) >0) {
          iniColNa <- colnames(out)
          if(is.logical(chIdenCol) && length(names(chIdenCol)) >0) out <- out[,-1*which(iniColNa %in% names(chIdenCol)[which(chIdenCol)])]
          if(length(out) >0 && length(dim(out)) <1) out <- matrix(out, nrow=nrow(mat), dimnames=list(names(out), iniColNa[-1*which(iniColNa %in% names(chIdenCol)[which(chIdenCol)])]))  
        }
      } 
      if(isTRUE(addRef)) out <- cbind(out, ref=ref)}
    out }

  if(length(mat)==1) { out <- mat; if(!silent) message(fxNa,"length of '",namesXY[1],"'(mat) ==1, nothing to do - return as input"); datOK <- FALSE}
  if(datOK) { if(length(dim(mat)) <2) mat <- matrix(mat, nrow=length(mat), dimnames=list(names(mat), namesXY[1]))
    if(length(ref) <1) { datOK <- FALSE
      stop(fxNa,"Argument '",namesXY[2],"' has incorrect length (found ",length(ref),") !")}}
  if(datOK) if(nrow(mat)==1) {out <- mat; if(!silent) message(fxNa,"'",namesXY[1],"' has single row, nothing to do - return as input"); datOK <- FALSE}

  if(datOK) {
    ## remove all columns of mat with identical values
    fidentVal <- function(x) length(unique(x)) ==1
    chIdenCol <- apply(mat, 2, fidentVal)        # designate columns not useful (all identical values); has also init colnames (as names)

    if(all(chIdenCol, na.rm=TRUE)) { datOK <- FALSE
      if(!silent) message(fxNa,"All lines of '",namesXY[1],"' seem identical, nothing to do for best matching !  (returning NULL)")
    } else {
      if(any(chIdenCol, na.rm=TRUE)) {
        ## split& remove non-informative
        matElim <- mat[,which(chIdenCol), drop=FALSE] 
        matUse <-  mat[,which(!chIdenCol), drop=FALSE]
        if(debug) message(fxNa,"Removing ",sum(chIdenCol), "columns of all identical values (have no value for distinguishing lines)")
      } else matUse <- mat
      if(debug) {message(fxNa,"mMLR2"); mMLR2 <- list(mat=mat,matUse=matUse,ref=ref,matElim=matElim,chIdenCol=chIdenCol,out=out )}
      ## exclude specified column(s) : exclCol
      if(length(exclCol) >0) {
        exclCol <- if(is.character(exclCol)) naOmit(colnames(matUse) %in% exclCol) else as.integer(naOmit(exclCol[which(exclCol >0 & exclCol <= ncol(mat))] ))
        if(length(exclCol) >0) {
          exclCol <- sort(unique(exclCol)) 
          if(length(exclCol) ==ncol(matUse)) {
            msg <-"All non-uniform columns get excluded via 'exclCol', nothing remains"
            matUse <- NULL
          } else {
            matUse <- if(length(exclCol) +1 ==ncol(matUse)) matrix(matUse[,-exclCol], ncol=1, dimnames=list(rownames(matUse), colnames(matUse)[-exclCol])) else matUse[,-exclCol] }  
        }
      }
      if(length(matUse) >0) {  
        ## try simple match
        sMa <- apply(if(length(matUse) >0) matUse else mat, 2, function(x) match(ref, x))
        chNa <- colSums(is.na(sMa))
        if(any(chNa==0)) { newOr <- sMa[,which(chNa==0)[1]]
          out <- .applyOrder(mat=mat, ref=ref, newOr=newOr, goodCol=NULL, matElim=matElim, chIdenCol=NULL, addRef=addRef)  #  goodCol=chNa, matElim=matElim, chIdenCol=chIdenCol,
          byCol <- which(chNa==0)[1]
          msgM <- "direct match"
        } else {
          if(debug) {message(fxNa,"mMLR3"); mMLR3 <- list(mat=mat,ref=ref,matElim=matElim,chIdenCol=chIdenCol,matUse=matUse,out=out)}
          ## trim redundant text, re-try match
          matIni <- mat        ## needed ?
          ## note : '\\' (from win paths) gave problems ..
          matUse <- apply(if(length(matUse) >0) matUse else mat, 2, function(x) gsub("\\\\","/", x))
          ref <- gsub("\\\\","/", ref)
          ## used previously  trimRedundText(.., minNchar=1, side="both") insead of rmSharedWords()
          mat2 <- apply(matUse, 2, rmSharedWords, sep=c("_"," ","/","-","."), minLe=1, silent=silent,callFrom=fxNa,debug=debug)
          ref2 <- rmSharedWords(ref, sep=c("_"," ","/","-","."), minLe=1, silent=silent,callFrom=fxNa,debug=debug)
          sMa <- apply(mat2, 2, function(x) match(ref2, x))
          chNa <- colSums(is.na(sMa))
          if(debug) {message(fxNa,"mMLR3b"); mMLR3b <- list(mat=mat,ref=ref,matElim=matElim,chIdenCol=chIdenCol,out=out,chNa=chNa,mat2=mat2,ref2=ref2,sMa=sMa )}
          if(any(chNa==0)) {     ## should be sufficient for ProteomeDiscoverer file-list
            newOr <- sMa[,which(chNa==0)[1]]
            out <- .applyOrder(mat=mat, ref=ref, newOr=newOr, addRef=addRef)  #
            #old#out <- .applyOrder(mat=mat, ref=ref, newOr=newOr, goodCol=chNa, matElim=matElim, chIdenCol=chIdenCol, addRef=addRef)  #
            byCol <- which(chNa==0)[1]
            msgM <- "direct match after trimming redundant text"
            if(debug) {message(fxNa,"mMLR3c"); mMLR3c <- list(mat=mat,ref=ref,matElim=matElim,chIdenCol=chIdenCol,out=out,chNa=chNa,mat2=mat2,ref2=ref2,sMa=sMa)}
          } else {
            ## direct matching not successful, check check for all ref starting with X, retry direct matching (automatic conversion of colnames starting with digits)
            if(all(grepl("^X[[:digit:]]", ref))) { ref2 <- sub("^X","", ref2); ref <- sub("^X","", ref)}
            sMa <- apply(mat2, 2, function(x) match(ref2, x))
            chNa <- colSums(is.na(sMa))
            if(any(chNa==0)) {     ## should be sufficient for ProteomeDiscoverer file-list
              newOr <- sMa[,which(chNa==0)[1]]
              out <- .applyOrder(mat=mat, ref=ref, newOr=newOr, addRef=addRef)  #
              #old#out <- .applyOrder(mat=mat, ref=ref, newOr=newOr, goodCol=chNa, matElim=matElim, chIdenCol=chIdenCol, addRef=addRef)  #
              byCol <- which(chNa==0)[1]
              msgM <- "direct match after trimming redundant text (removed 'X' from ref)"
            }
          }
          if(debug) {message(fxNa,"mMLR4"); mMLR4 <- list(mat=mat,ref=ref,matElim=matElim,chIdenCol=chIdenCol,out=out,chNa=chNa,mat2=mat2,ref2=ref2,sMa=sMa )}

          if(length(out)==0) { 
            ## direct matching not successful, check if grep possible (only when pattern not longer than x)
            leM <- nchar(as.matrix(mat2))
            leR <- nchar(ref2)
            refPoss <- apply(leM, 2, function(x) all(sort(x, decreasing=TRUE)[1:length(leR)] >= sort(leR, decreasing=TRUE)))
            if(any(refPoss)) {                           # grep each ref to each col
              chGre <- apply(mat2, 2, function(x) sapply(ref2, grep, x))
              chDL <- sapply(chGre, sapply, length)      # number of grp hits
              ch1 <- colSums(chDL ==1) ==length(ref)
              if(any(ch1)) { newOr <- chGre[[which(ch1)[1]]]
                out <- .applyOrder(mat=mat, ref=ref, newOr=newOr, goodCol=chNa, matElim=matElim, chIdenCol=chIdenCol, addRef=addRef)  #
                byCol <- which(ch1)[1]
                msgM <- "grep of ref after trimming redundant text"
              } else { refPoss <- FALSE
              msg <- c("grep matching not successful (",c("no","ambiguous hits")[1 +any(colSums(chDL >1) >0)],")") }}
            if(debug) {message(fxNa,"mMLR4c"); mMLR4c <- list(mat=mat,ref=ref,matElim=matElim,chIdenCol=chIdenCol,out=out,chNa=chNa,mat2=mat2,ref2=ref2,sMa=sMa,refPoss=refPoss,leM=leM,leR=leR )}
            
            if(any(!refPoss) && length(out) ==0) {
              ## check by harmonizing/trimming enumerators
              mat3 <- apply(mat2, 2, rmEnumeratorName, nameEnum=c("Number","No","#","Replicate","Sample","Speciem"), sepEnum=c(" ","-","_"), newSep="_No", incl=c("anyCase","trim1"), silent=debug, debug=debug, callFrom=fxNa)
              chCol <- colSums(mat2 ==mat3) < nrow(mat2)      # see if change in all elements in a given column
              if(debug) {message(fxNa,"mMLR5"); mMLR5 <- list(mat=mat,ref=ref,matElim=matElim,chIdenCol=chIdenCol,out=out,chNa=chNa,mat2=mat2,ref2=ref2,chCol=chCol,refPoss=refPoss) }

              if(any(chCol)) {
                if(debug) message(fxNa,"Enumerators exist, try matching after harmonizing style ..")
                if(!all(chCol)) mat3 <- mat3[,which(chCol), drop=FALSE]   # trim
                ref3 <- rmEnumeratorName(ref, nameEnum=c("","Number","No","#","Replicate","Sample","Speciem"), sepEnum=c(" ","-","_"), newSep="_No", incl=c("anyCase","trim1"), silent=debug, debug=debug, callFrom=fxNa)
                chDir <- apply(mat3, 2, match, ref3)
                chMa <- apply(chDir, 2, function(x) all(1:nrow(mat) %in% x, na.rm=TRUE))
                if(debug) {message(fxNa,"mMLR5b"); mMLR5b <- list(mat=mat,ref=ref,matElim=matElim,chIdenCol=chIdenCol,out=out,chNa=chNa,mat3=mat2,chCol=chCol) }
                if(any(chMa)) {
                  goodCol <- which(chMa)[1]
                  if(debug) message(fxNa,"Found good hit by using column '",names(goodCol),"'  ie ",pasteC(utils::head(mat2[,goodCol]))," ...")
                  out <- if(addRef) cbind(mat[chDir[,goodCol],, drop=FALSE], ref=ref) else mat[chDir[,goodCol],, drop=FALSE]
                  msgM <- "Match after harmonizing enumerators (& trimming redundant text)"
                  if(debug) {message(fxNa,"mMLR5c"); mMLR5c <- list(mat=mat,ref=ref,matElim=matElim,chIdenCol=chIdenCol,out=out,chNa=chNa,mat3=mat2,chCol=chCol) }
                } else {
                  if(debug) message(fxNa,"Try matching all after trimming enumerators")
                  mat3 <- apply(mat2, 2, rmEnumeratorName, nameEnum=c("Number","No","#","Replicate","Sample","Speciem"), sepEnum=c(" ","-","_"), newSep="", incl=c("anyCase","trim1","rmEnum"), silent=debug, debug=debug, callFrom=fxNa)
                  ref3 <- rmEnumeratorName(ref, nameEnum=c("","Number","No","#","Replicate","Sample","Speciem"), sepEnum=c(" ","-","_"), newSep="", incl=c("anyCase","trim1","rmEnum"), silent=debug, debug=debug, callFrom=fxNa)
                  chDir <- apply(mat3, 2, match, ref3)
                  chNa <- colSums(is.na(chDir)) <1
                  if(any(chMa)) {
                    ## need to attribute multiple hits
                    warning(fxNa,"Attribute multiple hits NOT FINISHED !!")
                  } else {
                    if(!silent) message(fxNa," STILL NO MATCH FOUND, try reverse grep of terms wo enumerators ?")
                    invePoss <- apply(leM, 2, function(x) all(sort(x, decreasing=TRUE)[1:length(leR)] <= sort(leR, decreasing=TRUE)))
                  }
                }
              } else {
                ## grep each ref to each col
                ## need to try reverse matching
                chRev <- apply(mat2, 2, function(x) sapply(x, grep, ref2))
                chRL <- sapply(chRev, sapply, length)       # number of grp hits
                if(length(dim(chRL)) !=2) {
                  useEl <- sapply(chRev, function(x) length(dim(x))!=2) & sapply(chRev, function(x) !is.list(x))  # select list-element with single hits
                  if(any(useEl)) mat <- mat[,which(useEl)[1], drop=FALSE] else {
                    useEl <- sapply(chRev, function(x) length(dim(x))==2) & sapply(chRev, function(x) !is.list(x))
                    if(any(useEl)) mat <- mat[,which(useEl)[1], drop=FALSE] else stop(fxNa,"Unable to match  mMLR5d")
                  }
                  mat2 <- as.matrix(mat)
                  chRev <- apply(mat2, 2, function(x) sapply(x, grep, ref2))
                  chRL <- sapply(chRev, sapply, length)       # number of grp hits
                }

                ch1 <- (if(length(dim(chRL)) >1) colSums(chRL) ==1 else sum(chRL==1)) ==length(ref)
                if(any(ch1)) { newOr <- if(is.list(chRev)) order(unlist(chRev[[which(ch1)[1]]], use.names=FALSE)) else chRev[,which(ch1)[1]]         # new order for mat
                  if(debug) {message(fxNa,"mMLR6"); mMLR6 <- list(mat=mat,ref=ref,matElim=matElim,chIdenCol=chIdenCol,out=out,chRev=chRev,chRL=chRL,ch1=ch1) }
                  out <- cbind(if(any(dim(mat)==1, length(newOr)==1)) matrix(mat[newOr,], ncol=ncol(mat), dimnames=list(rownames(mat)[newOr], colnames(mat))) else mat[newOr,, drop=FALSE],
                    if(any(dim(matElim)==1, length(newOr)==1)) matrix(matElim[newOr,], ncol=ncol(matElim), dimnames=list(rownames(matElim)[newOr], colnames(matElim))) else matElim[newOr,, drop=FALSE])
                  if(length(matElim) >0) out <- out[,match(names(chIdenCol), colnames(out)), drop=FALSE]   # adjust init col-order
                  if(addRef) out <- cbind(out, ref=ref)
                  byCol <- which(ch1)[1]
                  msgM <- "Reverse grep after trimming redundant text"
                } else msg <- c(msg,"Tried reverse grep matching, but impossible to find full set of non-ambiguous matches")
              }
            }
          }
        }
      }  
      if(debug) {message(fxNa,"mMLR7"); mMLR7 <- list(mat=mat,ref=ref,matElim=matElim,chIdenCol=chIdenCol,out=out )}
      if(length(msg) >0 && !silent) message(fxNa, msg,"!  Returning NULL")
      if(debug && length(newOr) >0) message(fxNa,"Successfully found new order by ",msgM," : ",pasteC(newOr, quoteC="'"))
    }
  }
  if(isTRUE(inclInfo)) list(mat=out, byColumn=match(names(byCol), names(chIdenCol)), newOrder=newOr, method=msgM) else out }
     

#' Search and Select Groups of Replicates
#'
#' This function was designed for mining annotation information organized in multiple columns to identify the (potential) grouping of multiple samples, ie to determine factor levels.
#' The argument \code{method} allows further finetuning if high or low number of groups should be preferred, if multiple columns may be combined, or to choose a particular custom column for desiganting factor levels.
#'
#'
#' @details
#' Statistical tests require specifying which samples should be considered as replicates of whom.
#' In some cases, like the Sdrf-format, automatic mining of such annotation to indentify an experiment's underlying structure of replicates
#' may be challanging, since the key information may not always be found in the same column.
#' For this reason this function allows inspecting all columns of a matrix of data.frame to identify which colmns may serve describing groups of replicates.
#'
#' The argument \code{exclNoRepl=TRUE} allows excluding all columns with different content for each line (like line-numbers), ie information without any replicates.
#' It is set by default to \code{TRUE} to exclude such columns, since statistical tests usually do require some replicates.
#'
#' When using as \code{method="combAll"}, there is risk all lines (samples) will be be considered different and no replicates remain.
#' To avoid this situation the argument can be set to \code{method="combNonOrth"}.
#' Using this mode it will be checked if  more columns will lead to complete loss of replicates, and -if so- concerned columns omitted.
#'
#'
#'
#' @param x (matrix or data.frame) the annotation to inspect; each column is supposed to describe another set of annoation/metadata for the rows of \code{x} (min 1 row and 1 column),
#' @param method (character, length=1) the procedure to choose column(s) with properties of information, may be \code{highest} or \code{max} (max number of levels)
#'   \code{lowest} or \code{min} (min number of levels), \code{median} (median of all options for number of levels),
#'   \code{combAll} (combine all columns of \code{x}) or \code{combNonOrth} (combine only non-orthogonal columns of \code{x}, to avoid avoid n lines with n levels);
#'   lazy evluation of the argument is possible
#' @param sep (character) separator used when a method combining multiple columns (eg combAll, combNonOrth)  is chosen (should not appear anywhere in \code{x})
#' @param exclNoRepl (logical) decide whether columns with all values different (ie no replicates or max divergency) should be excluded
#' @param trimNames (logical) optional trimming of names in \code{x} by removing redundant heading and tailing text
#' @param includeOther (logical) include $allCols with pattern of (all) other columns
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a list with $col (column index relativ to \code{x}), $lev (abstract labels of level),
#'   $meth (note of method finally used) and $allCols with general replicate structure of all columns of  \code{x}
#' @seealso  \code{\link[base]{duplicated}}, uses \code{\link{trimRedundText}}; for chosing single most likely column describing sample-names and groups of replicates \code{\link{chooseGroupNames}}
#' @examples
#' ## a is all different, b is groups of 2,
#' ## c & d  are groups of 2 nut NOT 'same general' pattern as b
#' strX <- data.frame(a=letters[18:11], b=letters[rep(c(3:1,4), each=2)],
#'  c=letters[rep(c(5,8:6), each=2)], d=letters[c(1:2,1:3,3:4,4)],
#'  e=letters[rep(c(4,8,4,7),each=2)], f=rep("z",8) )
#' strX
#' replicateStructure(strX[,1:2])
#' replicateStructure(strX[,1:4], method="combAll")
#' replicateStructure(strX[,1:4], method="combAll", exclNoRepl=FALSE)
#' replicateStructure(strX[,1:4], method="combNonOrth", exclNoRepl=TRUE)
#' replicateStructure(strX, method="lowest")
#' replicateStructure(strX, method=3, includeOther=TRUE)   # custom choice of 3rd column
#'
#'
#'
#' @export
replicateStructure <- function(x, method="median", sep="__", exclNoRepl=TRUE, trimNames=FALSE, includeOther=FALSE, silent=FALSE, callFrom=NULL, debug=FALSE) {
  ##
  fxNa <- .composeCallName(callFrom, newNa="replicateStructure")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  out <- ou3 <- str1 <- NULL              # initialize

  datOK <- all(length(x) >0 & length(dim(x)) >0)
  if(debug) message(fxNa,"ini datOK  ",datOK)
  ## check for unique colnames
  if(datOK) {
    chMe <- TRUE
    if(length(method) <1) { chMe <- FALSE; method <- "median"} else if(length(method) >1)  method <- method[1]
    if(is.na(method)) { chMe <- FALSE; method <- "median" }
    if(!chMe && !silent) message(fxNa, "Invalid entry for 'method', setting to default, ie 'median'")
    iniColNa <- colnames(x)
    chColN <- duplicated(colnames(x), fromLast=FALSE)
    if(any(chColN)) { if(!silent) message(fxNa,"Need to correct redundant colnames !!!")}
  } else if(!silent) message(fxNa,"Invalid entry for 'x' .. should be matrix or data.frame (min 1 row, 1 column) .. returning NULL")
  ## single available/given .. shortcut
  if(datOK) { if(any(dim(x) ==1)) { datOK <- FALSE
      if(debug) message(fxNa,"Single column or single line available/relevant .. shortcut")
      if(nrow(x) ==1) {
        out <- list(col=1, lev=1, meth="single row")
        if(!silent) message(fxNa,"Note : Only single line with levels, ie single group wo any replicates")
      } else {
        nn <- match(as.character(x), unique(as.character(x)))
        names(nn) <- if(ncol(x)==1) x[,1] else x[1,]
        coln <- 1
        names(coln) <- colnames(x)
        out <- list(col=coln, lev=nn, meth="single informative") } }
  }

  ## check argument 'method' : check for costom choice of column
  if(datOK && length(out) <1 && grepl("^[[:digit:]]+$", method)) {
    colNo <- as.integer(method)
    out <- list(col=colNo, lev=match(x[,colNo], unique(x[,colNo])), meth="custom column") }
  if(debug) {message(fxNa,"fd0"); fd0 <- list(x=x,method=method,sep=sep,exclNoRepl=exclNoRepl,datOK=datOK,out=out)}

  ## check argument 'method'
  if(datOK && length(out) <1) {
    methOpt <- c("highest","lowest","min","median","combAll","combNonOrth")
  	msg <- c("  'method' should designate one of the following options ",pasteC(methOpt,quoteC="'"),"; setting to default")
    if(identical(method,"min")) method <- "lowest"
    if(identical(method,"max")) method <- "highest"
    if(length(method) <1) { method <- "median"
      warning(fxNa,"Argument 'method' seems empty !",msg)}
    chNa <- is.na(method)
    if(any(chNa)) { method <- "median"
      warning(fxNa,"Argument 'method' may not contain NA !",msg)}
    if(length(method) >1) { method <- method[1]
      if(!silent) message(fxNa,"Argument 'method' should be of length=1, using first")}
    chMeth <- pmatch(method, methOpt)
    chMe2 <- any(is.na(chMeth))
    if(chMe2) { method <- "median"; chMeth <- 3
      warning(fxNa,"Unable to match argument 'method' to any of the options ",pasteC(methOpt,quoteC="'"),msg)}
    method <- methOpt[chMeth]

    ## check separator
    if(length(sep) != 1) { sep <- "__"
      warning(fxNa," Argument 'sep' must be character and of length =1; resetting to default")}
    if(nchar(sep) <1) { sep <- "__"
      warning(fxNa," Argument sep=\"\" may lead to incorrect comparisons, resetting to default")}
    chSep <- grep(sep, x)
    if(length(chSep) >0) warning(fxNa," BEWARE, the annotation data do contain ",length(chSep)," instances of '",
      sep,"' ! The results of combined searches may be incorrect. Please consider using a different separator")

    ##
    if(debug) {message(fxNa,"fd1"); fd1 <- list(x=x,ou3=ou3,method=method,sep=sep,exclNoRepl=exclNoRepl,datOK=datOK,out=out)}

    ## utility functions
    chStru <- function(y, debug) {   # check duplication structure of vector
      dup <- duplicated(y)
      out <- -1
      if(sum(dup) <1) out <- if(isTRUE(exclNoRepl)) 0 else 1:length(y)     # all unique
      if(sum(dup) == length(y) -1) out <- 1                                # all duplic
      if(any(out < 0)) out <- match(y, unique(y))
      if(debug) message(fxNa," -> chStru : ",pasteC(out))
      out }
    fMin <- function(y, ref=y) { coln <- which.min(apply(y, 2, max, na.rm=TRUE));
      if(debug) {message(fxNa," -> fMin   colnames y", colnames(y),"\n"); fMi <- list(y=y,coln=coln, ref=ref)}
      refn <- if(identical(y, ref)) coln else which(colnames(y)[coln]==colnames(ref))
      nn <- y[,coln]; names(nn) <- ref[,refn];
      names(refn) <- colnames(y)[coln]
      list(col=refn, lev=nn, meth="single min col")}
    fMax <- function(y, ref=y) { coln <- which.max(apply(y, 2, max, na.rm=TRUE));
      if(debug) {message(fxNa," -> fMax   colnames y", colnames(y),"\n"); fMa <- list(y=y,coln=coln, ref=ref)}
      refn <- if(identical(y, ref)) coln else which(colnames(y)[coln]==colnames(ref))
      nn <- y[,coln]; names(nn) <- ref[,refn];
      names(refn) <- colnames(y)[coln]
      list(col=refn, lev=nn, meth="single max col")}
    fMed <- function(y, ref=y) { zz <- apply(y, 2, max, na.rm=TRUE);
      coln <- which(zz==stats::median(if((ncol(y) %% 2) ==0) c(zz[1],zz) else zz))[1];
      if(debug) {message(fxNa," -> fMed   colnames y", colnames(y),"\n"); fMe <- list(y=y,zz=zz,coln=coln, ref=ref)}
      refn <- if(identical(y, ref)) coln else which(colnames(y)[coln]==colnames(ref))
      names(refn) <- colnames(y)[coln]
      nn <- y[,coln]; names(nn) <- ref[,refn]
      list(col=refn, lev=nn, meth="single median col")}
    fCombAll <- function(y, ref=y) {   # uses debug
      refn <- if(identical(y, ref)) 1:ncol(y) else match(colnames(y), colnames(ref))
      names(refn) <- colnames(y)
      tm <- if(ncol(y)==2) paste(y[,1],y[,2]) else apply(y, 1, paste, collapse=" ");
      nn <- match(tm, unique(tm))
      if(debug) {message(fxNa," -> fCombAll"); fComb <- list(y=y,ref=ref,refn=refn,tm=tm,nn=nn)}
      names(nn) <- if(ncol(y)==1) ref[,refn] else apply(ref[,refn], 1, paste0, collapse="_")
      list(col=refn, lev=nn, meth="comb all col") }

    fComNonOrth <- function(y, oup="min", ref=y) { if(debug) message(fxNa," -> fComNonOrth")
      tm <- if(ncol(y)==2) as.matrix(paste(y[,1], y[,2])) else {
        if(ncol(y)==3) cbind(paste(y[,1],y[,2]), paste(y[,1],y[,3])) else apply(y[,-1], 2, paste,y[,1])}      # pairwise groups/combin
      st2 <- as.matrix(apply(tm, 2, function(z) match(z, unique(z))))                # levels of pairwise combin
      colnames(st2) <- paste(colnames(y)[1], colnames(y)[2:ncol(y)], sep=sep)
      maxL <- apply(st2, 2, max)
      if(debug) {message(fxNa," -> fComNonOrt"); fComNonOrt <- list(y=y,oup=oup,tm=tm, st2=st2,maxL=maxL,x=x)}
      if(any(maxL ==nrow(y))) {useCo <- which(maxL < nrow(y)); st2 <- matrix(st2[,useCo], ncol=length(useCo), dimnames=list(rownames(st2), colnames(st2)[useCo]))}
      if(length(st2) >0) { if(TRUE) {maxP <- which.max(apply(st2, 2, max));
        coln <- which(colnames(y) %in% unlist(strsplit(names(maxP), split=sep)))
        refn <- if(identical(y, ref)) coln else match(colnames(y)[coln], colnames(ref))
        names(refn) <- colnames(y)[coln]
        nn <- st2[,maxP]; names(nn) <- if(length(refn)==1) ref[,refn] else {
          if(length(refn)==2) paste(ref[,refn[1]],ref[,refn[2]],sep="_") else apply(ref[,refn], 1, paste0, collapse="_")}
        out <- list(col=refn, lev=nn, meth="combNonOrth col" )}
      } else out <- fMax(y)
      out }

    ## get generalized pattern of replicates
    str1 <- apply(x, 2, chStru, debug=debug)
    chStr <- if(is.matrix(str1)) rep(nrow(str1) <nrow(x), ncol(x)) else {sapply(str1, length) < nrow(x)}
    ## develop more to recuperate replicates after enumerarators ?

    if(debug) {message(fxNa,"fd2"); fd2 <- list(x=x,method=method,ou3=ou3,str1=str1,sep=sep,exclNoRepl=exclNoRepl,datOK=datOK)}
    if(all(chStr)) {
      if(exclNoRepl) {
        if(debug) message(fxNa,"None of the columns have any replicates, setting exclNoRepl=FALSE")
        exclNoRepl <- FALSE
        str1 <- apply(x, 2, chStru, debug=debug)
        chStr <- sapply(str1, length) < nrow(x)
      } else warning(fxNa,"Can't find replicates for any of the columns !! ")
    }
    if(debug) {message(fxNa,"fd3"); fd3 <- list(x=x,method=method,ou3=ou3,str1=str1,sep=sep,exclNoRepl=exclNoRepl,datOK=datOK,chStr=chStr)}
    ## had removing all replicated instances, but this will create list if str1 is matrix with uneven number of replicates per column !
    if(length(dim(str1)) <1 & any(chStr, na.rm=TRUE) & !all(chStr, na.rm=TRUE)) str1 <- str1[which(!chStr)]

    if(is.list(str1)) str1 <- as.matrix(as.data.frame(str1))
    if(length(dim(str1)) <2 & length(str1) >0) str1 <- matrix(unlist(str1), ncol=ncol(x), dimnames=dimnames(x))
    if(debug) {message(fxNa,"fd4"); fd4 <- list(x=x,method=method,ou3=ou3,str1=str1,sep=sep,exclNoRepl=exclNoRepl,datOK=datOK,chStr=chStr)}
    if(!isTRUE(exclNoRepl)) {              ## check if single col at max levels may serve as shortcut
      chMax <- sapply(str1, max, na.rm=TRUE) ==nrow(x)
      if(any(chMax)) { datOK <- FALSE
        coln <- which.min(chMax)
        names(coln) <- colnames(str1)[coln]
        nn <- 1:nrow(x)
        names(nn) <- str1[,coln]
        out <- list(col=coln, lev=nn, meth="(shortcut, first) single col at max divergence", allCols=str1) }
    }
  }

  if(datOK && length(str1) >0) {
    ## remove non-useful, obtain matrix of indexes
    if(is.list(str1)) { chLe <- sapply(str1, length)
      if(any(chLe) <2) if(sum(duplicated(chLe[which(chLe >1)])) == sum(chLe >1) -1) {
        ou3 <- as.matrix(as.data.frame(str1[which(chLe >1)]))  # all remaining of same length
      } else { ou3 <- NULL; message(fxNa,"Trouble ahead ?  Different length when trying to extract patterns")  #
      }
    } else { ou3 <- if(length(dim(str1)) >1) str1 else NULL }
    if(length(ou3) >0) {  # now look for replicates of these patterns (paste-collapse series internal) & remove redundant columns (keep 1st instance from left)
      tm2 <- apply(ou3, 2, paste0, collapse="")
      chDu <- duplicated(tm2, fromLast=FALSE)
      if(any(chDu)) {tm2 <- tm2[which(!chDu)];
        ou3 <- matrix(ou3[,which(!chDu)], nrow=nrow(x), dimnames=list(NULL,colnames(ou3)[which(!chDu)]))}
      if(debug) {message(fxNa,"fd5 .."); fd5 <- list(x=x,method=method,out=out,ou3=ou3,tm2=tm2,str1=str1,chDu=chDu,sep=sep,exclNoRepl=exclNoRepl)}

      out <- if(ncol(ou3) <2) {
        if(debug) message(fxNa," single column for choice ..")
        co <- which(colnames(x) ==colnames(ou3))
        names(co) <- colnames(ou3)
        nn <- as.integer(ou3)
        names(nn) <- x[,co]
        list(col=co, lev=nn, meth="single informative col")
      } else switch(method,                     ## return column index
        highest =fMax(ou3, ref=x),
        lowest =fMin(ou3, ref=x),
        median =fMed(ou3, ref=x),
        combAll=fCombAll(ou3, ref=x),
        combNonOrth=fComNonOrth(ou3, ref=x) )
    }
    if(debug) {message(fxNa,"fd6"); fd6 <- list(x=x,method=method,out=out,ou3=ou3,tm2=tm2,str1=str1,chDu=chDu,sep=sep,exclNoRepl=exclNoRepl)}
  }

  ## optional include all patterns
  if(datOK && length(out) >0 && isTRUE(includeOther) && !"allCols" %in% names(out)) out$allCols <- apply(x, 2, function(y) match(y, unique(y)))
  ## optional trim names to ou3$lev
  if(datOK && length(out) >0 && isTRUE(trimNames)) names(out$lev) <- trimRedundText(names(out$lev), spaceElim=TRUE, silent=silent, callFrom=fxNa, debug=debug)
  out }
  

#' Linear rescaling of numeric vertor or matrix
#'
#' \code{adjBy2ptReg} takes data within window defined by 'lims' and determines linear transformation so that these points get the regression characteristics 'regrTo', 
#' all other points (ie beyond the limits) will follow the same transformation.
#' In other words, this function performs 'linear rescaling', by adjusting (normalizing) the vector 'dat' by linear regression so that points falling in 'lims'
#' (list with upper & lower boundaries) will end up as 'regrTo'.
#'
#' @param dat numeric vector, matrix or data.frame
#' @param lims (list, length=2) should be list giving limits (list(lo=c(min,max),hi=c(min,max)) in data allowing identifying which points will be used for determining slope & offset
#' @param regrTo (numeric, length=2) to which characteristics data should be regressed
#' @param refLines (NULL or integer) optional subselection of lines of dat (will be used internal as refDat)
#' @param silent (logical) suppress messages
#' @param debug (logical) display additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a matrix (of same dimensions as inlut matrix) with normalized values
#' @seealso \code{\link{normalizeThis}}
#' @examples
#' set.seed(2016); dat1 <- round(runif(50,0,100),1)
#' ## extreme values will be further away :
#' adjBy2ptReg(dat1,lims=list(c(5,9), c(60,90)))
#' plot(dat1, adjBy2ptReg(dat1, lims=list(c(5,9),c(60,90))))
#' @export
adjBy2ptReg <- function(dat, lims, regrTo=c(0.1,0.9), refLines=NULL, silent=FALSE, debug=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom, newNa="adjBy2ptReg")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  if(length(refLines) >0) {
    if(is.character(refLines) && !is.null(rownames(dat))) refLines <- match(refLines, names(dat)) else {
    refLines <- as.integer(refLines)
    refLines <- refLines[which(refLines >0 & refLines < length(dat))]}}
  dat1 <- if(length(refLines) >0) dat[refLines] else dat
  if(is.null(names(dat1))) names(dat1) <- 1:length(dat1)
  msg <- "'lims' should be list giving limits (list(lo=c(min,max), hi=c(min,max))"
  if(!is.list(lims)) lims <- as.list(lims)
  if(length(lims) != 2) stop(msg)
  lim2 <- if(all(sapply(lims, length) ==1)) {
    list(lo=which(dat1 == lims[[1]]), hi=which(dat1 == lims[[1]]))
  } else {
    if(all(sapply(lims, length) ==2)) { list(
      lo=which(dat1 >= min(lims[[1]],na.rm=TRUE) & dat1 <= max(lims[[1]],na.rm=TRUE)),
      hi=which(dat1 >= min(lims[[2]],na.rm=TRUE) & dat1 <= max(lims[[2]],na.rm=TRUE)))
    } else stop(" cannot figure out 'lims' !\n ",msg)}
  chLe <- sapply(lim2, length)
  if(any(chLe <1)) message(fxNa,"Limits seem too tight : lo ",chLe[1],"  & hi ",chLe[2],"  entries found !")
  lim3 <- sapply(lim2, function(x) sum(dat1[x], na.rm=TRUE)/length(x))               # mean value at lo & hi indexing
  normFact <- if(length(lims) ==1) (regrTo[1]/lim3[1]) else (regrTo[2] -regrTo[1])/ (lim3[2] -lim3[1])       # slope
  normFact <- c(k=normFact, d= if(length(lims) ==1) 0 else regrTo[1] -normFact*lim3[1])
  norD <- dat1*normFact[1] + normFact[2]
  dat[which(is.finite(dat))] <- as.numeric(norD)
  dat }
 
#' Model linear regression and optional plot
#'
#' This function allows to model a linear regression and optionally to plot the results
#' 
#' @param dat (vector or matrix) main input
#' @param typeOfPlot (character)
#' @param toNinX (logical) 
#' @param plotData (logical) 
#' @param silent (logical) suppress messages
#' @param debug (logical) display additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return numeric vector with intercept and slope, optional plot
#' @seealso  \code{\link[base]{append}}; \code{\link{lrbind}}
#' @examples
#' .datSlope(c(3:6))
#' @export
.datSlope <- function(dat, typeOfPlot="sort", toNinX=FALSE, plotData=FALSE, silent=FALSE, debug=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom, newNa=".datSlope")
  if(length(typeOfPlot) >1) {
    if(!silent) message(fxNa,"Invalid entry for typeOfPlot ",paste(typeOfPlot, collapse=", ")," (use 1st)")
    typeOfPlot <- typeOfPlot[1] }
  dat <- as.numeric(naOmit(sort(dat)))
  y <- if(tolower(typeOfPlot) %in% "cumsum") cumsum(sort(dat)) else sort(dat)
  y <- data.frame(out=y, n=1:length(dat))
  out <- if(toNinX) stats::lm(n ~ out, data=y)$coefficients else stats::lm(out ~ n, data=y)$coefficients
  if(plotData) { if(toNinX) {
      chF <- try(graphics::plot(y, type="s", ylab=if("cumsum" %in% tolower(typeOfPlot)) "cumsum" else "sorted"), silent=TRUE)
      if(!inherits(chF, "try-error")) { for(i in 2:ncol(out)) graphics::points(y); graphics::abline(out, lty=2, col=2)
      } else {
        if(!silent) message(fxNa,"UNABLE to plot data !")}
    } else {
      chF <- try(graphics::plot(1:nrow(y), y[,1], type="s", ylab=if("cumsum" %in% tolower(typeOfPlot)) "cumsum" else "sorted"), silent=TRUE)
      if(!inherits(chF, "try-error")) {graphics::points(1:nrow(y), y[,1]); graphics::abline(out, lty=2, col=2)} else { 
        if(!silent) message(fxNa,"UNABLE to plot data !")}
    } 
  }
  names(out) <- c("intercept","slope")
  out }

#' Check regression arguments
#'
#' This function allows to check arguments for linear regression. Used as argument checking for \code{regrBy1or2point} and \code{regrMultBy1or2point}
#' 
#' @param inData (numeric vector) main input
#' @param refList (list)
#' @param regreTo (numeric vector) 
#' @param callFrom (character) allow easier tracking of messages produced
#' @return list
#' @seealso  \code{\link[base]{append}}; \code{\link{lrbind}}
#' @examples
#' .datSlope(c(3:6))
#' @export
.checkRegrArguments <- function(inData, refList, regreTo, callFrom=NULL){
  ## argument checking for regrBy1or2point() & regrMultBy1or2point()
  ## 'inData' should be
  ## returns corrected/adjusted 'refList' and 'regreTo' (as list)
  fxNa <- .composeCallName(callFrom, newNa=".checkRegrArguments")
  if(!is.list(refList)) refList <- list(refList)
  if(is.list(inData) && !is.data.frame(inData)) message(fxNa,"Potential problem : not expecting list (length=",length(inData),") at place of 'inDat' !!")
  if(min(sapply(refList, length)) <2) {
    warning(fxNa,sum(sapply(refList, length) <2)," entries of 'refLst' contain less than 2 array-positions (well-names)")
    refList <- refList[which(sapply(refList, length) >1)] }
  datIsMa <- !is.null(dim(inData)) 
  checkRefFields <- unlist(refList) %in% (if(datIsMa) rownames(inData) else names(inData))
  if(sum(checkRefFields) <1) {
    message(fxNa," 'refList'  ",paste(unlist(refList),collapse=" "))
    message("Head(names(inDat) ",paste(if(datIsMa) utils::head(rownames(inData)) else utils::head(names(inData)),collapse=" ")," ...")
    stop(" none of the fields given in 'refList' appear in names of 'inDat' !") }
  if(sum(checkRefFields) < sum(sapply(refList, length))) message(fxNa,"Some of the fields given in 'refLst' do not appear in names of 'inDat' !")
  if(length(refList) > length(regreTo)) message(fxNa,"Need as many values 'regrTo' as types (of samples) given in 'refLst' !")
  list(refLst=refList, regrTo=regreTo) }
      

#' Confidence Interval To Given Alpha  
#'
#' This little function returns the confidence interval associated to a given significance level \code{alpha} under the hypothesis of the Normal distribution is valid.	
#' 
#' @param x (numeric) main input 
#' @param alpha (numeric) significance level, accepted type I error 
#' @param distrib (character) distribution, so far only \code{Normal} is implemented 
#' @param silent (logical) suppress messages
#' @param callFrom (character) allow easier tracking of message(s) produced
#' @return This function returns the confidence interval to a given \code{alpha} under the hypothesis of the Normal distribution. 	
#' @seealso \code{\link[stats]{TDist}}; \code{\link[stats]{confint}}  
#' @examples
#' 
#' confInt(c(5,2:6))
#' 
#' @export
confInt <- function(x, alpha=0.05, distrib="Normal", silent=FALSE, callFrom=NULL) {            ## return Confidence Interval (CI) (range) based on sd ('x')
  fxNa <- .composeCallName(callFrom, newNa="confInt")
  if(!"normal" %in% tolower(distrib)) warning(fxNa,"So far only distrib='Normal' has been implemented, all data will be considered as originating from Normal distribution")
  tVal <- stats::qt((1 -alpha)/2 + 0.5, length(x) -1)    
  tVal*stats::sd(x, na.rm=TRUE) / sqrt(length(x))}
  

#' Reduce table by aggregating smaller groups 
#'
#' \code{reduceTable} treats/reduces results from \code{\link[base]{table}} to 'nGrp' groups, 
#' optional indiv resolution of 'separFirst' (numeric or NULL). 
#' Mainly made for reducing the number of classes for betters plots with \code{\link[graphics]{pie}}
#'
#' @param tab output of \code{\link[base]{table}}
#' @param separFirst (integer or NULL) optinal separartion of n 'separFirst' groups (value <2 or NULL 
#'  will priviledge more uniform size of groups, higher values will cause small inital and larger tailing groups) 
#' @param nGrp (integer) number of groups expected
#' @return This function returns a numeric vector with number of counts and class-borders as names (like table).
#' @seealso \code{\link[base]{table}}
#' @examples
#' set.seed(2018); dat <- sample(11:60,200,repl=TRUE)
#' pie(table(dat))
#' pie(reduceTable(table(dat), sep=NULL))
#' pie(reduceTable(table(dat), sep=NULL), init.angle=90,
#'   clockwise=TRUE, col=rainbow(20)[1:15], cex=0.8)
#' @export
reduceTable <- function(tab, separFirst=4, nGrp=15) {
   if(!inherits(tab, "table")) message(" Note: Expecting class 'table' for argument 'tab' for proper functioning !!")
   cuSu <- cumsum(tab)
   if(length(separFirst) >0) {ch <- grep("^[[:digit:]]$", separFirst[1])
     separFirst <- if(length(ch)==1) as.numeric(separFirst[1]) else NULL}
   if(length(separFirst) >0) {
      nGrp <- nGrp -separFirst +1
      if(nGrp <1) message("correcting nGrp -> 1 ") else message(" nGrp=",nGrp)
      if(nGrp <1) nGrp <- 1 
      cu <- as.numeric(cut(cuSu,nGrp)) +separFirst -1
      cu[1:separFirst] <- 1:separFirst
   } else cu <- as.numeric(cut(cuSu, nGrp))
   ## idea : modify to to priviledge round numbers (of names(tab))
   out <- tapply(tab, cu, sum)
   names(out) <- tapply(as.numeric(names(tab)), cu, function(x) if(length(x) >1) paste(range(x),collapse="-") else x)
   out }
      

#' Remove Lines Of Matrix Redundant /Duplicated For 1st And 2nd Column
#'
#' This function removes lines of matrix that are redundant /duplicated for 1st and 2nd column (irrespective of content of their columns).
#' The first occurance of redundant /duplicated elements is kept.
#'
#' @param mat (matrix or data.frame) main input
#' @param useCol (integer, length=2) columns to consider/use when looking for duplicated entries
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a matrix where duplictaed lines are removed
#' @seealso \code{\link[base]{unlist}}
#' @examples
#' mat <- matrix(1:12,ncol=3)
#' mat[3,1:2] <- mat[1,1:2]
#' rmDupl2colMatr(mat)
#' @export
rmDupl2colMatr <- function(mat,useCol=c(1,2), silent=FALSE, debug=FALSE, callFrom=NULL) {
  ## remove entries with same value in 1st & 2nd col of mat
  fxNa <- .composeCallName(callFrom, newNa="rmDupl2colMatr")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  msg <- " 'mat' should be matrix with at least 2 columns"
  if(length(dim(mat)) <2) stop(fxNa, msg) else if(ncol(mat) <2) stop(fxNa, msg)
  tx <- paste(mat[,useCol[1]], mat[,useCol[2]], sep="_")
  dup <- duplicated(tx, fromLast=FALSE)
  if(any(dup)) {mat[which(!dup),, drop=FALSE]} else mat }
   

#' CV of array
#'
#' \code{arrayCV} gets CVs for replicates in 2 or 3 dim array and returns CVs as matrix.
#' This function may be used to calculate CVs from replicate microtiter plates (eg 8x12) where replicates are typically done as multiple plates,
#' ie initial matrixes that are the organized into arrays.
#' @param arr (3-dim) array of numeric data like where replicates are along one dimesion of the array
#' @param byDim (integer) over which dimension repliates are found
#' @param silent (logical) suppres messages
#' @param callFrom (character) allow easier tracking of message produced
#' @return matrix of CV values
#' @seealso \code{\link{rowCVs}}, \code{\link{rowGrpCV}}, \code{\link{replPlateCV}}
#' @examples
#' set.seed(2016); dat1 <- matrix(c(runif(200) +rep(1:10,20)), ncol=10)
#' head(arrayCV(dat1,byDim=2))
#' @export
arrayCV <- function(arr, byDim=3, silent=TRUE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom, newNa="arrayCV")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  curDiNa <- dimnames(arr)
  if(byDim > length(dim(arr))) { byDim <- length(dim(arr))
    message(fxNa," dimension number for argument 'byDim' too high, adjusting to ",byDim) }
  basDim <- if(byDim != 1) 1 else 2
  arrCV <- matrix(nrow=dim(arr)[basDim], ncol=dim(arr)[byDim], dimnames=curDiNa[c(basDim,byDim)] )           # (1:3)[!(1:3 %in% c(basDim,byDim))]
  if(!silent) message(fxNa," CV output in matrix/array of ",nrow(arrCV)," x ",ncol(arrCV)," ")
  if(length(dim(arr)) ==3) {
    for(i in 1:dim(arr)[byDim]) {arrCV[,i] <- if(byDim==3) rowCVs(arr[,,i]) else {
      if(byDim ==2) rowCVs(arr[,i,]) else rowCVs(arr[i,,]) }}
  } else arrCV <- matrix(rowCVs(arr), dimnames=list(rownames(arr), NULL))
  arrCV }
 

#' Contingenty tables for fit of ranking 
#'
#' Count the number of instances where the corresponding columns of 'dat' have a value matching the group number as specified by 'grp'.
#' Counting will be performed/repeated independently for each line of 'dat'.
#' Returns array  (1st dim is rows of dat, 2nd is unique(grp), 3rd dim is ok/bad), these results may be tested using eg \code{\link[stats]{fisher.test}}.
#' This function was made for prearing to test the ranking of multiple features (lines in 'mat') including replicates (levels of 'grp'). 
#' 
#' @param dat (matrix or data.frame of integer values) ranking of multiple features (lines), equal ranks may occur   
#' @param grp (integer) expected ranking
#' @return array (1st dim is rows of dat, 2nd is unique(grp), 3rd dim is ok/bad)
#' @seealso \code{\link[stats]{lm}}
#' @examples
#' # Let's create a matrix with ranks (equal ranks do occur)
#' ma0 <- matrix(rep(1:3,each=6), ncol=6, dimnames=list(
#'   c("li1","li2","ref"), letters[1:6]))
#' ma0[1,6] <- 1                       # create item not matching correctly
#' ma0[2,] <- c(3:1,2,1,3)             # create items not matching correctly
#' gr0 <- gl(3,2)                      # the expected ranking (as duplicates)
#' (count0 <- rankToContigTab(ma0,gr0))
#' cTab <- t(apply(count0, c(1,3) ,sum))
#' # Now we can compare the ranking of line1 to ref ...
#' fisher.test(cTab[,c(3,1)])          # test li1 against ref
#' fisher.test(cTab[,c(3,2)])          # test li2 against ref
#' @export
rankToContigTab <- function(dat,grp) {
  ## It is supposed that the columns of dat can be cut in multiple groups according to (sorted) grp
  ## Then the number of instances where the corresponding columns of dat have a value matching the group number is counted
  ## returns array  (1st dim is rows of dat, 2nd is unique(grp), 3rd dim is ok/bad)
  grUnq <- unique(grp)
  chSo <- sort(grUnq)==grUnq 
  if(length(dim(dat)) <2) dat <- matrix(as.numeric(dat),nrow=1,dimnames=list(NULL,names(dat)))
  if(any(!chSo)) { message("need to re-sort groups !")
    newOrd <- order(grp)
    oldRa <- rank(grp)          # for setting back in old order
    dat <- dat[,newOrd]
    grp <- grp[newOrd]
    grUnq <- unique(grp)  
  } else newOrd <- NULL
  lims <- sapply(grUnq,function(x) range(which(grp==x)))
  out <- array(dim=c(nrow(dat),length(grUnq),2),dimnames=list(rownames(dat),grUnq,c("ok","bad")))
  for(i in 1:length(grUnq)) { z <- dat[,lims[1,i]:lims[2,i]]
    out[,i,] <- if(nrow(z) >1) cbind(ok=rowSums(z ==i,na.rm=TRUE),bad=rowSums(z !=i,na.rm=TRUE)) else c(ok=sum(z ==i,na.rm=TRUE),bad=sum(z !=i,na.rm=TRUE)) }
  out }  
  

#' Convert ratio to ppm 
#'
#' This function transforms ratio 'x' to ppm (parts per million). 
#' If 'y' not given (or different length as 'x'), then 'x' is assumed as ratio otherise rations are constructed as x/y is used lateron.
#' Does additional checking : negative values not expected - will be made absolute !
#' 
#' @param x (numeric) main input
#' @param y (numeric) optional value to construct ratios (x/y). If NULL (or different length as 'x'), then 'x' will be considered as ratio.
#' @param nSign (numeric) number of significan digits
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a numeric vector of ppm values
#' @seealso \code{\link{XYToDiffPpm}} for ppm of difference as used in mass spectrometrie
#' @examples
#' set.seed(2017); aa <- c(1.000001,0.999999,1+rnorm(10,0,0.001))
#' cbind(x=aa,ppm=ratioToPpm(aa,nSign=4))
#' @export
ratioToPpm <- function(x, y=NULL, nSign=NULL, silent=FALSE, debug=FALSE, callFrom=NULL){
  fxNa <- .composeCallName(callFrom, newNa="ratioToPpm")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  if(any(is.na(x))) {if(!silent) message(fxNa," 'x' contains ",sum(is.na(x))," out of ",length(x)," NA alike, omit !"); x <- naOmit(x)}
  if(length(y) >0) if(any(is.na(y))) {if(!silent) message(fxNa," 'y' contains ",sum(is.na(y))," NA alike, omit !"); y <- naOmit(y)}
  if(length(y) >1 && length(x) != length(x)) {
    if(!silent) message(fxNa,"Length of 'y' (",length(y),") not corresponding to 'x' (",length(x),"), ignore 'y'")
    y <- NULL}
  if(length(y) == length(x) || length(y) ==1) {                      # get ration of  x / Y
   if(any(y==0, na.rm=TRUE) && !silent) message(fxNa,"Division by 0 !")
    x <- x/y }
  if(any(x <0)) {if(!silent) message(fxNa,"Ngative values not possible"); x <- abs(x)}
  whNeg <- which(x <1)
  x[whNeg] <- 1/x[whNeg]
  x <- 1e6*(1-x)
  x[-1*whNeg] <- abs(x[-1*whNeg])
  if(length(nSign) >0) signif(x, as.numeric(nSign)) else x }
   

#' Partial unlist of lists of lists
#'
#' \code{partUnlist} does partial unlist for treating list of lists : New (returned) list has one level less of hierarchy 
#' (Highest level list will be appended). In case of conflicting (non-null) listnames a prefix will be added. 
#' Behaviour different to \code{\link[base]{unlist}} when unlisting list of matrixes.
#' @param lst (list) main input, list to be partially unlisted
#' @param sep (character, length=1) separator for names
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a list with partially reduced nested structure
#' @seealso \code{\link[base]{unlist}}, \code{\link{asSepList}}
#' @examples
#' partUnlist(list(list(a=11:12,b=21:24), list(c=101:101,d=201:204)))
#' li4 <- list(c=1:3, M2=matrix(1:4,ncol=2), L3=list(L1=11:12, M3=matrix(21:26,ncol=2)))
#' partUnlist(li4)
#' unlist(li4, rec=FALSE)
#' @export
partUnlist <- function(lst, sep="_", silent=FALSE, debug=FALSE, callFrom=NULL) {
  fxNa <- .composeCallName(callFrom, newNa="partUnlist")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  notL <- !sapply(lst, inherits, "list")  
  if(debug) message(fxNa," pU1")
  if(all(notL)) { if(!silent) message(fxNa,"Input is not list of lists, nothing to do")
    return(lst)
  } else {
    ## lst is list of list(s)
    out <- if(any(notL)) lst[which(notL)] else list()   # the non-list elements
    for(i in which(!notL)) {
      iniLe <- length(out)
      newNa <- names(lst[[i]])      
      if(any(nchar(newNa) >0)) {
        newNa <- if(any(newNa %in% names(out))) paste(names(lst)[i], newNa, sep=sep) else newNa}
      out[length(out) +1:length(lst[[i]])] <- lst[[i]] 
      if(any(nchar(newNa) >0)) names(out)[iniLe + 1:length(lst[[i]])] <- newNa }
    out }}  
     

#' Make numeric vector non-ambiguous (ie unique)
#'
#' Thios function transfomrmes a vector of (named) numeric values \code{x} into unique. 
#' Note: for non-numeric use the function \code{\link{firstOfRepeated}} - but 1000x slower !
#' Return sorted non-ambigous numeric vector (or list if 'asList'=TRUE and 'uniqOnly'=FASLSE)
#' @param x (numeric) main input
#' @param uniqOnly (logical) if=TRUE return unique only, if =FALSE return unique and single representative of non-unique values (with '' added to name), selection of representative of repeated: first (of sorted) or middle if >2 instances
#' @param asList (logical) return list
#' @param nameMod (character) text to add in case on ambiguous values, default="amb_"
#' @param callFrom (character) allows easier tracking of messages produced
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @return sorted non-ambigous numeric vector (or list if 'asList'=TRUE and 'uniqOnly'=FALSE)
#' @seealso \code{\link{firstOfRepeated}} for non-numeric use (much slower !!!), \code{\link[base]{duplicated}} 
#' @examples
#' set.seed(2017); aa <- round(rnorm(100),2); names(aa) <- 1:length(aa)
#' str(nonAmbiguousNum(aa))
#' str(nonAmbiguousNum(aa,uniq=FALSE,asLi=TRUE))
#' @export
nonAmbiguousNum <- function(x,uniqOnly=FALSE,asList=FALSE,nameMod="amb_", silent=FALSE, debug=FALSE, callFrom=NULL) {
  fxNa <- .composeCallName(callFrom,newNa="nonAmbiguousNum")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  y <- sort(x)
  ab <- which(diff(y)==0)
  if(length(ab) <1) return(if(asList) list(unique=x) else x) else {
    ac <- diff(ab)
    ac <- ab[c(which(ac >1), if(ac[length(ac)] >1) length(ab) else NULL)]   # index : single instance of repeated
    red <- y[ac]
    names(red) <- paste(nameMod, names(red), sep="")
    out <- y[-1*unique(sort(c(ab,ab+1)))]
    if(!uniqOnly) out <- if(asList) list(unique=out, ambig=red) else c(out, red)
    out} }
     

#' Row group CV
#'
#' This function calculates CVs for matrix with multiple groups of data, ie one CV for each group of data. 
#' Groups are specified as columns of 'x' in 'grp' (so length of grp should match number of columns of 'x', NAs are allowed)
#' @param x numeric matrix where relplicates are organized into separate columns
#' @param grp (factor) defining which columns should be grouped (considered as replicates)
#' @param means (numeric) alternative values instead of means by .rowGrpMeans()
#' @param listOutp (logical) if TRUE, provide output as list with $CV, $mean and $n
#' @return This function returns a matrix of CV values
#' @seealso \code{\link{rowCVs}}, \code{\link{arrayCV}},  \code{\link{replPlateCV}}
#' @examples
#' set.seed(2016); dat1 <- matrix(c(runif(200)+rep(1:10,20)),ncol=10)
#' head(rowGrpCV(dat1, gr=gl(4,3,labels=LETTERS[1:4])[2:11]))
#' @export
rowGrpCV <- function(x, grp, means=NULL, listOutp=FALSE){
  if(!is.matrix(x) && !is.data.frame(x)) stop(" 'x' should be data.frame or matrix")
  if(length(dim(x)) !=2) stop(" 'x' should be data.frame or matrix of 2 dimensions")
  if(length(grp) != ncol(x)) stop(" 'grp' should be of length of number of cols in 'x'")
  if(length(grp) <1 || all(is.na(grp))) stop(" 'grp' appears to be empty or all NAs")
  if(!is.factor(grp)) grp <- as.factor(grp)
  if(is.null(means)) means <- .rowGrpMeans(x, grp)
  ## main
  out <- .rowGrpCV(x, grp, means)
  if(listOutp) out <- list(CV=out, mean=means, n=summary(grp))
  out }

#' row group CV (main)
#'
#' This function calculates CVs for matrix with multiple groups of data, ie one CV for each group of data. 
#' 
#' @param x numeric matrix where relplicates are organized into separate columns
#' @param grp (factor) defining which columns should be grouped (considered as replicates)
#' @param means (numeric) alternative values instead of means by .rowGrpMeans()
#' @return This function returns a matrix of CV values
#' @seealso \code{\link{rowGrpCV}}, \code{\link{rowCVs}}, \code{\link{arrayCV}}, \code{\link{replPlateCV}}
#' @examples
#' set.seed(2016); dat1 <- matrix(c(runif(200)+rep(1:10,20)),ncol=10)
#' grp1 <- gl(4,3,labels=LETTERS[1:4])[2:11]
#' head(.rowGrpCV(dat1, grp1,  .rowGrpMeans(dat1, grp1)))
#' @export
.rowGrpCV <- function(x, grp, means){
  .rowGrpSds(x, grp)/means
  }
    

#' Distance of categorical data (Jaccard, Rand and adjusted Rand index) 
#'
#' \code{randIndFx} calculates distance of categorical data (as Rand Index, Adjusted Rand Index or Jaccard Index). 
#' Note: uses/requires package \href{https://CRAN.R-project.org/package=flexclust}{flexclust}
#' Methods so far available (via flexclust): "ARI" .. adjusted Rand Index, "RI" .. Rand index, "J" .. Jaccard, "FM" .. Fowlkes-Mallows.
#' @param ma (matrix) main input for distance calulation
#' @param method (character) name of distance method (eg "ARI","RI","J","FM")
#' @param adjSense (logical) allows introducing correlation/anticorrelation (interprete neg distance results as anti)
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a distance matrix
#' @seealso \code{comPart} in \code{\link[flexclust]{randIndex}}
#' @examples
#' set.seed(2016); tab2 <- matrix(sample(1:2, size=42, replace=TRUE), ncol=7)
#' if(requireNamespace("flexclust")) { flexclust::comPart(tab2[1,], tab2[2,])
#'   flexclust::comPart(tab2[1,], tab2[3,])
#'   flexclust::comPart(tab2[1,], tab2[4,]) }
#' ## via randIndFx():
#'   randIndFx(tab2, adjSense=FALSE)
#'   cor(t(tab2))
#'   randIndFx(tab2, adjSense=TRUE)
#' @export
randIndFx <- function(ma, method="ARI", adjSense=TRUE, silent=FALSE, debug=FALSE, callFrom=NULL){
  ## calculate distance for categorical data (using Rand Index, Adjusted Rand Index or Jaccard Index)
  ## method : "ARI" .. adjusted Rand Index, "RI" .. Rand index, "J" .. Jaccard, "FM" .. Fowlkes-Mallows
  ## 'adjSense' allows introducing corretaltion/anticorrelation (interprete neg distance results as anti)
  ## uses package flexclust
  ## wr 29jan15, cor 23mar16
  ## require(flexclust)
  fxNa <- .composeCallName(callFrom, newNa="randIndFx")
  if(isTRUE(debug)) silent <- FALSE else { debug <- FALSE
    if(!isTRUE(silent)) silent <- FALSE }
  if(requireNamespace("flexclust", quietly=TRUE)) {
    if(!is.matrix(ma) && !silent) message(fxNa,"Caution : data-frames with factors may cause problems !!")
    if(is.logical(ma)) stop(fxNa,"Expecting matrix with integer values")
    maCo <- matrix(1:nrow(ma), ncol=nrow(ma), nrow=nrow(ma))
    maCo <- cbind(x=maCo[upper.tri(maCo)], y=t(maCo)[upper.tri(maCo)])
    maCo <- maCo[order(maCo[,1], maCo[,2]),]                                 # need proper order for upper.tri
    di <- try(apply(maCo, 1, function(x) flexclust::comPart(ma[x[1],],ma[x[2],], type=method)), silent=TRUE)
    if(inherits(di, "try-error")) message(fxNa,"Problem running flexclust::comPart !   class ",class(di)," mode ",mode(di)," ",di)
    out <- matrix(NA_real_, nrow=nrow(ma), ncol=nrow(ma), dimnames=list(rownames(ma),colnames(ma)))
    out[upper.tri(out)] <- rev(di)                    # not used any more by as.dist()
    out[lower.tri(out)] <- di
    if(adjSense){
      out <- .scaleXY(out, minim=0, maxim=max(out,na.rm=TRUE))                 # re-scale from 0 to max
      mi <- min(out, na.rm=TRUE)
      maX <- max(out, na.rm=TRUE)
      if(mi <0) out <- (out -mi)*maX/(maX -mi)
      orient <- try(stats::cor(t(ma)) >0)
      if("try-error" %in% class(orient)) { out <- NA
        warning(fxNa,"PROBLEM with calulating cor(), returning 0 ")
      } else out <- as.matrix(out)*(-1 +2*orient) }
    stats::as.dist(out)
  } else { 
    if(!silent) message(fxNa,"NOTE: Package 'flexclust' not found ! Please install first from CRAN  (returning NULL)")  
  }}
   

#' Correct Mixed Slash And Backslash In File-Path
#'
#' @description
#' This function corrects paths character strings for mixed slash and backslash in file path.
#' In Windows the function \code{tempdir()} will use double backslashes as separator while \code{file.path()} uses regular slashes.
#' So when combining these two one might encounter a mix of slashes and double backslashes which may cause trouble, unless this is streightened out to a single separator used.
#' When pointig to given files inside html-files, paths need to have a prefix, this can be added using the argument \code{asHtml}.
#'	 
#' @param x (character) input path to test and correct
#' @param asHtml (logical) option for use in html : add prefix "file:/" 
#' @param anyPlatf (logical, length=1) if \code{TRUE}, checking will only be performed in Windows environement
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allows easier tracking of messages produced
#' @return This function returns a character vector with corrected path
#' @seealso \code{\link[base]{tempfile}}, \code{\link[base]{file.path}}
#' @examples
#' path1 <- 'D:\\temp\\Rtmp6X8/working_dir\\RtmpKC/example.txt'
#' (path1b <- correctWinPath(path1, anyPlatf=TRUE)) 
#' (path1h <- correctWinPath(path1, anyPlatf=TRUE, asHtml=TRUE)) 
#' @export
correctWinPath <- function(x, asHtml=FALSE, anyPlatf=FALSE, silent=FALSE, debug=FALSE, callFrom=NULL) {
  ## correct mixed slash and backslash in file path
  fxNa <- .composeCallName(callFrom,newNa="correctWinPath")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  if(anyPlatf || length(grep("ming.32", R.Version()$platform)) >0) {
    x <- gsub("\\\\","/",x)      #"
    if(asHtml && length(grep("[[:upper:]]:", substr(x,1,2))) >0) {
      x <- paste0("file:///",x) }
  } 
  if(asHtml && length(grep("^/",x)) >0) x <- paste0("file:///", x,)
  x } 
  

#' Filter Lines Of Matrix For Max Number Of NAs 
#'
#' This function produces a logical matrix to be used as filter for lines of 'dat' for sufficient presence of non-\code{NA} values (ie limit number of NAs per line). 
#' Filter abundance/expression data for min number and/or ratio of non-\code{NA} values in at east 1 of multiple groups.
#' This type of procedure is common in proteomics and tanscriptomics, where a \code{NA} can many times be assocoaued with quantitation below detetction limit.
#'
#' @param dat matrix or data.frame (abundance or expression-values which may contain some \code{NA}s).
#' @param grp factor of min 2 levels describing which column of 'dat' belongs to which group (levels 1 & 2 will be used)
#' @param useComparison (character or matrix) optional argument allowing to specify which pairwise comparions sould be performed, default \code{useComparison=NULL} will run all pairwise comparisons;
#'   may be character combining two group-names (from argument \code{grp}) separated by a '-' (eg 'A-B') or matrix where the rownames design the elements to be compared as pairwise;
#'   Note : the names of the groups may not contain any '-' to avoid confucing them with pairwise separators !
#' @param maxGrpMiss (numeric) at least 1 group has not more than this number of NAs (otherwise marke line as bad)
#' @param ratMaxNA (numeric) at least 1 group below this content of \code{NA} values
#' @param minVal (default NULL or numeric), any value below will be treated like \code{NA}
#' @param sep (character) in case \code{useComparison} is not given all pairwise comparisons will be done, the separator to be used when combining names of groups can be given using this argument
#' @param experSetup (list) optional experimental setup
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a logical matrix (with separate col for each pairwise combination of 'grp' levels) indicating if line of 'dat' acceptable based on \code{NA}s (and values minVal)
#' @seealso  \code{\link{presenceGrpFilt}},   there are also other packages on CRAN and Bioconductor dedicated to filtering
#' @examples
#' mat <- matrix(rep(8,150), ncol=15, dimnames=list(NULL,
#'   paste0(rep(LETTERS[4:2],each=6),1:6)[c(1:5,7:16)]))
#' mat[lower.tri(mat)] <- NA
#' mat[,15] <- NA
#' mat[c(2:3,9),14:15] <- NA
#' mat[c(1,10),13:15] <- NA
#' mat
#' # default : at least 1 group with max 1 NA and one group below 80% NA
#' presenceFilt(mat, substr(colnames(mat),1,1))
#' # custom 2 groups
#' presenceFilt(mat, rep(1:2,c(9,6)))         # D1- C4, C5 - B4
#' 
#' # one more example 
#' dat1 <- matrix(1:56, ncol=7)
#' dat1[c(2:6,10,12,18,19,20,22,23,26:28,30,31,34,38,39,50,54)] <- NA
#' grp3 <- letters[c(3,3,2,2,1,1,1)]
#' colnames(dat1) <- correctToUnique(grp3, sep="") 
#' dat1
#' ## At least one group wo any NAs
#' presenceFilt(dat1, grp3, maxGr=0)
#' presenceFilt(dat1, gr=gl(2,4)[-1], maxGr=1, ratM=0.1)
#' presenceFilt(dat1, gr=gl(2,4)[-1], maxGr=2, rat=0.5)
#' @export
presenceFilt <- function(dat, grp, useComparison=NULL, maxGrpMiss=1, ratMaxNA=0.8, minVal=NULL, sep=NULL, experSetup=NULL, silent=FALSE, debug=FALSE, callFrom=NULL){           
  fxNa <- .composeCallName(callFrom, newNa="presenceFilt")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  
  msg <- "Expecting (2dim) numeric matrix or data.frame with >1 columns and >1 rows"
  if(length(dim(dat)) !=2) stop(msg)
  if(ncol(dat) <2) stop(msg)
  if(is.data.frame(dat)) dat <- as.matrix(dat)
  if(length(experSetup) >1 && is.list(experSetup)) {
    if(length(grp)==0 && "grp" %in% names(useComparison)) grp <- experSetup$grpNa
    if(length(useComparison)==0 && "pwGrpNa" %in% names(useComparison)) useComparison <- experSetup$pwGrpNa
    if(length(sep)==0 && "sep" %in% names(useComparison)) sep <- experSetup$sep
  }
  if(!is.factor(grp)) grp <- as.factor(grp)

  ## prepare 
  nGrp <- table(grp)
  if(length(nGrp) <2) stop(" too few levels (",length(nGrp),") in 'grp' !")
  nGrp <- nGrp[order(unique(grp))]
  if(length(maxGrpMiss) !=length(nGrp)) {
    if(length(maxGrpMiss)==1) maxGrpMiss <- rep(maxGrpMiss, length(nGrp)) else {
      if(length(maxGrpMiss) < nGrp) {stop(fxNa,"Length of 'maxGrpMiss' too short")} else maxGrpMiss <- maxGrpMiss[1:length(nGrp)]}
  }
  if(debug) {message(fxNa,"pFi1"); pFi1 <- list(dat=dat,grp=grp,useComparison=useComparison,maxGrpMiss=maxGrpMiss,ratMaxNA=ratMaxNA,minVal=minVal,nGrp=nGrp)}

  ## main
  ## establish by group
  #byGrp <- matrix(TRUE, nrow=nrow(dat), ncol=length(nGrp), dimnames=list(rownames(dat), names(nGrp)))
  ## minVal : set mat to NA if below ...
  if(length(minVal)==1 && is.numeric(minVal)) dat[which(dat <minVal)] <- NA
  ## count NAs
  nNa <- matrix(unlist(by(t(dat), grp, function(x) colSums(is.na(x)))), nrow=nrow(dat))[,order(unique(grp))]  # no of NAs per group & line
  #nNa <- nNa[,order(unique(grp))]
  dimnames(nNa) <- list(rownames(dat), names(nGrp))
  #ch1 <- nNa > maxGrpMiss
  byGrp <- nNa <= maxGrpMiss  # so far those which pass on absol count
  
  #if(any(ch1)) byGrp[which(ch1)] <- FALSE
  if(debug) {message(fxNa,"pFi1b"); pFi1b <- list(dat=dat,grp=grp,useComparison=useComparison,maxGrpMiss=maxGrpMiss,ratMaxNA=ratMaxNA,nGrp=nGrp,byGrp=byGrp,nNa=nNa)}
  ## filter for ratio
  NaRat <- nNa/matrix(rep(nGrp, each=nrow(dat)), nrow=nrow(dat))
  ch1 <- NaRat > ratMaxNA & byGrp
  
  if(any(ch1)) byGrp[which(ch1)] <- FALSE
  if(debug) {message(fxNa,"pFi2"); pFi2 <- list(dat=dat,grp=grp,useComparison=useComparison,maxGrpMiss=maxGrpMiss,ratMaxNA=ratMaxNA,nGrp=nGrp,byGrp=byGrp,nNa=nNa,NaRat=NaRat,sep=sep,experSetup=experSetup)}

  ## filter for absol count
  ## combine groups
  if(length(useComparison)==0) {
    if(length(sep)==0) sep <- getPWseparator(grp=grp, silent=silent, debug=debug, callFrom=fxNa)   
    useComparison <- t(utils::combn(1:length(levels(grp)), 2, simplify=TRUE))               # relatoiv to LEVELS !!
    if(ncol(useComparison)==1) useComparison <- t(useComparison)
    dimnames(useComparison) <- list(utils::combn(levels(grp), 2, paste, collapse=sep), c("samp","ref")) 
  } else if(length(dim(useComparison)) <2) {
    if(debug) {message(fxNa,"Trying to convert 'useComparison' to matrix of indexes")}
    sep <- getPWseparator(grp=grp, silent=silent, debug=debug, callFrom=fxNa)
    experSetup <- convPairwiseSetup(useComparison, grp=grp, experSetup=experSetup, sep=sep, silent=silent, debug=debug, callFrom=fxNa)
    useComparison <- experSetup$pwIndex 

    #bad# useComparison <- try(indexGroupsFromPW(compNames=useComparison, grp=unique(grp), includeGrp=FALSE, silent=silent, debug=debug, callFrom=fxNa))
    if(inherits(useComparison, "try-error")) {useComparison <- NULL; message(fxNa,"Failed to convert 'useComparison")}
  }   
  if(debug) {message(fxNa,"pFi3"); pFi3 <- list(dat=dat,grp=grp,useComparison=useComparison,maxGrpMiss=maxGrpMiss,ratMaxNA=ratMaxNA,nGrp=nGrp,byGrp=byGrp,nNa=nNa,NaRat=NaRat,sep=sep)}
  
  if(length(dim(useComparison)) !=2 && all(dim(useComparison) >= c(1,2))) {
    if(!silent) message(fxNa,"Invalid 'useComparison', must be matrix or data.fame with 2 columns;  returning NULL")
  } else {
    out <- matrix(TRUE, nrow=nrow(dat), ncol=nrow(useComparison), dimnames=list(rownames(dat), rownames(useComparison)))  
    for(i in 1:nrow(useComparison)) out[,i] <- byGrp[,useComparison[i,1]] | byGrp[,useComparison[i,2]]
    out
  } 
}
  

#' Compute Matrix Of Differences For All Pairwise Combinations Of Numeric Vector
#'
#' \code{diffCombin} returns matrix of differences (eg resulting from subsititution) for all pairwise combinations of numeric vector 'x'.
#'
#' @param x numeric vector to compute differences for all combinations
#' @param diagAsNA (logical) return all self-self combinations as NA (otherwise 0)
#' @param prefix (logical) if TRUE, dimnames of output will specify orientation (prefix='from.' and 'to.')
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allows easier tracking of messages produced
#' @return This function returns a numeric matrix of all pairwise differences
#' @seealso \code{\link[base]{diff}} for simple differences
#' @examples
#' diffCombin(c(10,11.1,13.3,16.6))
#' @export
diffCombin <- function(x, diagAsNA=FALSE, prefix=TRUE, silent=FALSE, debug=FALSE, callFrom=NULL) {
  fxNa <- .composeCallName(callFrom,newNa="diffCombin")
  fxNa <- .composeCallName(callFrom, newNa="combineOverlapInfo")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE

  if(!is.numeric(x)) x <- convToNum(x, remove=NULL, sciIncl=TRUE, callFrom=fxNa, silent=silent)
  if(is.null(names(x))) names(x) <- sprintf(paste0("%0",nchar(length(x)),"d"),1:length(x))
  diNa <- if(prefix) list(paste("to",names(x),sep="."),paste("from",names(x),sep=".")) else list(names(x),names(x))
  out <- matrix(rep(x,length(x)), nrow=length(x), dimnames=diNa) -matrix(rep(x,each=length(x)), nrow=length(x))
  if(diagAsNA) diag(out) <- NA
  out }
   

#' 2-Factorial Limma-Style t-Test
#'
#' The aim of this function is to provide convenient acces to two-factorial (linear) testing withing the framework of \code{\link{makeMAList}} including the emprical Bayes shrinkage. 
#' The input data 'datMatr' which should already be organized as limma-type MAList, eg using using \code{\link{makeMAList}}. 
#' Note: This function uses the Bioconductor package \href{https://bioconductor.org/packages/release/bioc/html/limma.html}{limma} (which must be installed).
#'
#' @param datMatr matrix or data.frame with lines as indenpendent series of measures (eg different genes)
#' @param fac1 (character or factor) vector describing grouping elements of each line of 'datMatr' for first factor, must be of same langth as fac2
#' @param fac2 (character or factor) vector describing grouping elements of each line of 'datMatr' for second factor, must be of same langth as fac1
#' @param testSynerg (logical) decide if factor-interactions (eg synergy) should be tested in model, otherwise additive factors are supposed
#' @param testOrientation (character) default (or any non-recignized input) '=', otherwise either '>','gerater','sup','upper' or '<','inf','lower'
#' @param addResults (character) vector defining which types of information should be included to output, may be 'lfdr','FDR' (for BY correction), 'Mval' (M values), 'means' (matrix with mean values for each group of replicates)
#' @param addGenes (matrix or data.frame) additional information to add to output
#' @param silent (logical) suppress messages
#' @param callFrom (character) allow easier tracking of messages produced
#' @param debug (logical) additional messages for debugging
#' @return This function returns an object of class "MArrayLM" (from limma) containing/enriched by the testing results
#' @seealso \code{\link{makeMAList}}, single line testing \code{\link[limma]{lmFit}} and the \code{eBayes}-family of functions in package \href{https://bioconductor.org/packages/release/bioc/html/limma.html}{limma}
#' @examples
#' set.seed(2014)
#' dat0 <- rnorm(30) + rep(c(10,15,19,20),c(9,8,7,6))
#' fa <- factor(rep(letters[1:4],c(9,8,7,6)))
#' dat2 <- data.frame(facA=rep(c("-","A","-","A"), c(9,8,7,6)),
#'   facB= rep(c("-","-","B","B"), c(9,8,7,6)), dat1=dat0, dat2=runif(30))
#' grpNa <- sub("-","",sub("\\.","", apply(dat2[,1:2], 1, paste, collapse="")))    
#' test2f <- test2factLimma(t(dat2[,3:4]), dat2$facA, dat2$facB)  
#' test2f     # just the p-values
#' # Similarly, you can easily summarize results using topTable from limma
#' if(requireNamespace("limma", quietly=TRUE)) {
#'   test2g <- test2factLimma(t(dat2[,3:4]), dat2$facA, dat2$facB, addR=FALSE)  
#'   library(limma)
#'   topTable(test2g, coef=1, n=5) 
#'   topTable(test2g, coef=2, n=5) } 
#' @export
test2factLimma <- function(datMatr, fac1, fac2, testSynerg=FALSE, testOrientation="=", addResults=c("lfdr","FDR","Mval","means"), addGenes=NULL, silent=FALSE, callFrom=NULL, debug=FALSE){
  fxNa <- .composeCallName(callFrom, newNa="test2factLimma")
  if(isTRUE(debug)) silent <- FALSE else { debug <- FALSE
    if(!isTRUE(silent)) silent <- FALSE }
  doTest <- TRUE
  if(!requireNamespace("limma", quietly=TRUE)) { doTest <- FALSE
    message(fxNa,"NOTE : You need to install first the package 'limma' from Bioconductor")}
  if(doTest) {    
    msg1 <- " 'datMatr' should have the same number of cols as length of fac1 & fac2 !"
    if(ncol(datMatr) != length(fac1) || ncol(datMatr) != length(fac2)) stop(msg1)
    datDesign <- if(isTRUE(testSynerg)) try(stats::model.matrix(~ fac1 * fac2), silent=TRUE) else try(stats::model.matrix(~ fac1 + fac2), silent=TRUE)
    if(inherits(datDesign, "try-error")) { doTest <- FALSE; message(fxNa,"Problem with model.matrix(), please check your factors !")
    } else if(debug) message(fxNa,"Design matrix has ",nrow(datDesign)," rows and ",ncol(datDesign),"  cols")
  }
  if(doTest && requireNamespace("limma", quietly=TRUE)) {    
    datFit <- try(limma::lmFit(datMatr, design=datDesign), silent=TRUE)                ## Fitting linear models
    if(inherits(datFit, "try-error")) {warning(fxNa," PROBLEM with lmFit(); check if package 'limma' is correctly installed, it seems not to be working properly !?!")
      doTest <- FALSE
    } else if(debug) message(fxNa,"Sucessfully run lmFit()") }
    
  if(doTest && requireNamespace("limma", quietly=TRUE)) {    
    datFit <- limma::eBayes(datFit)                                  ## Adjusting using empirical Bayes
    altHyp <- "two.sided"                                             # default, change only if explicit sign recognized
    if(length(testOrientation) <1) testOrientation <- altHyp
    if(testOrientation %in% c("<","less","inf","lower")) altHyp <- "less"           
    if(testOrientation %in% c(">","greater","sup","upper")) altHyp <- "greater"
    chNAp <- colSums(!is.na(datFit$p.value))
    if(any(chNAp <1)) {message(fxNa,"Problem with redundant factors ?  Some cols of p.values are all NA !! (remove)")
      datFit$p.value <- datFit$p.value[,which(chNAp >0)]}  
    if(length(addResults) >0) if("lfdr" %in% tolower(addResults) && !requireNamespace("fdrtool", quietly=TRUE)) {
      if(!silent) message(fxNa,"Package 'fdrtool' not found ! Please install package fdrtool from CRAN for enabeling 'lfdr' estimations") 
      addResults <- addResults[-which("lfdr" %in% tolower(addResults))] }    
    tx <- c("testing alternative hypothesis: true difference in means is "," than 0 (ie focus on "," results with A ",altHyp," than B)")
    if(debug) message(fxNa,"altHyp = '",altHyp,"'")
    if(identical(altHyp,"greater")) {
      ch <- datFit$means[,1] > datFit$means[,2]
      if(!silent) message(fxNa,tx[1],altHyp,tx[2],sum(ch),tx[3:5])
      if(any(ch)) datFit$p.value[which(ch),] <- datFit$p.value[which(ch),]/2
      if(any(!ch)) datFit$p.value[which(!ch),] <- 1- datFit$p.value[which(!ch),]/2        # !(A > B)  .. A <= B
    }
    if(identical(altHyp,"less")){
      ch <- datFit$means[,2] > datFit$means[,1]
      if(!silent) message(fxNa,tx[1],altHyp,tx[2],sum(ch),tx[3:5])
      if(any(ch)) datFit$p.value[which(ch),] <- datFit$p.value[which(ch),]/2
      if(any(!ch)) datFit$p.value[which(!ch),] <- 1- datFit$p.value[which(!ch),]/2        # !(A > B)  .. A <= B
    }
    if(length(addResults) <1) out <- datFit$p.value[,2] else { out <- datFit
      if(debug) message(fxNa,"addResults",addResults)
      ## further inspect & correct values of 'addResults' ?
      if("Mval" %in% addResults) out$Mval <- (out$means[,1] - out$means[,2]) 
      if("FDR" %in% toupper(addResults)) out$FDR <- if(length(dim(out$p.value)) >1) {
        apply(out$p.value, 2, stats::p.adjust, meth="BH")} else stats::p.adjust(out$p.value, meth="BH")
        if("lfdr" %in% tolower(addResults)) {
          if(requireNamespace("fdrtool", quietly=TRUE)) {
            out$lfdr <- if(is.matrix(out$p.value)) {
              apply(out$p.value,2,pVal2lfdr)} else pVal2lfdr(out$p.value)   
            if(inherits(out$lfdr, "try-error")) { if(!silent) message(fxNa,"PROBLEM with calulating lfdr ! ")
            } else if(debug) message(fxNa,"Sucessfully calculated lfdr ...")
          } 
        }
      if("BY" %in% toupper(addResults)) { datFit$BY <- if(length(dim(out$p.value)) >1) {
        apply(datFit$p.value, 2, stats::p.adjust, meth="BY")} else stats::p.adjust(out$p.value, meth="BY")
      }
      for(i in c("FDR","lfdr","BY")) {if(length(dim(out[[i]])) >1) rownames(out[[i]]) <- rownames(datMatr)}
    }
    out 
  } else if(debug) message(fxNa,"ATTENTION, no two-factorial was calculated, returning NULL") 
  }
         

#' Minimum distance/difference between values
#'
#' This function aims to find the min distance (ie closest point) to any other x (numeric value), ie intra 'x' and
#'  returns matrix with 'index','value','dif','ppm','ncur','nbest','best'.
#' At equal distance to lower & upper neighbour point, the upper (following) point is chosen (as single best).
#' In case of multiple ex-aequo distance returns 1st of multiple, may be different at various repeats.
#'
#' @param x (numeric) vector to search minimum difference
#' @param digSig number of significant digits, used for ratio or ppm column
#' @param ppm (logical) display distance as ppm (1e6*diff/refValue, ie normalized difference eg as used in mass spectrometry), otherwise the ratio is given as : value(from 'x') / closestValue (from 'x')
#' @param initOrder (logical) return matrix so that 'x' matches exactely 2nd col of output
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a matrix
#' @seealso  \code{\link[base]{diff}}
#' @examples
#' set.seed(2017); aa <- 100*c(0.1 +round(runif(20),2),0.53,0.53)
#' minDiff(aa); 
#' minDiff(aa,initO=TRUE,ppm=FALSE); .minDif(unique(aa))
#' @export
minDiff <- function(x, digSig=3, ppm=TRUE, initOrder=TRUE, silent=FALSE, debug=FALSE, callFrom=NULL) {
  ## for comparison of 2 vectors see .clostestNumByRat()  or
  fxNa <- .composeCallName(callFrom, newNa="minDiff")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  y <- try(as.numeric(x), silent=TRUE)
  if(inherits(y, "try-error")) stop(fxNa,"Invalid entry : Unable to transform 'x' into numeric vector")
  if(debug) message(fxNa,"mD0")

  so <- .minDif(y, initOrder=FALSE, rat=TRUE)
  chNegDif <- which(so[,5] <1)    # for orientation
  if(length(chNegDif) >0) so[chNegDif, c(3)] <- -1*so[chNegDif, c(3)]       # neg 'dif' if 'value' is lower than best/closest(~ref)
  so <- cbind(so, ncur=1, nbest=1)
  if(length(digSig) >0 && !ppm) if(is.numeric(digSig)) so[,"rat"] <- signif(so[,"rat"], digits=digSig)  # finalize ratio result
  repInd <- which(c(diff(so[,2]),NA) ==0)                                  # index of repeated vals exept last of series
  if(length(repInd) >0) {                                                 # repeated values, need to correct 'so'
    firOf <- repInd[c(1,1 +which(diff(repInd) >1))]                        # first of repeated (values for diff incorrect)  ,length(repVa)
    repIn2 <- sort(unique(c(repInd, repInd +1)))                            # index of all repeated
    tab <- table(so[repIn2,2])
    lasOf <- repIn2[cumsum(tab)]                                          # index of last of rep; correct dist is in last of series !
    z2 <- so[firOf -1, 1:2]
    z3 <- so[lasOf +1, 1:2]
    if(length(dim(z2)) <2) z2 <- matrix(z2, nrow=1, dimnames=list(NULL,names(z2)))
    if(length(dim(z3)) <2) z3 <- matrix(z3, nrow=1, dimnames=list(NULL,names(z3)))
    zz <- rbind(so[firOf,1:2], z2[which(!z2[,2] %in% so[firOf,2]),])
    zz <- rbind(zz,z3[which(!z3[,2] %in% zz[,2]),])
    if(debug) message(fxNa,"mD1")
    s3 <- .minDif(zz[,2], initOrder=FALSE)
    s3[,1] <- zz[s3[,1],1]                                                # correct to full range indexes
    s3[,4] <- zz[s3[,4],1]                                                # correct to full range indexes
    s3 <- s3[which(s3[,1] %in% so[repIn2,1]),]
    if(length(dim(s3)) <2) s3 <- matrix(s3, nrow=1, dimnames=list(NULL,names(s3)))
    so[repIn2,3:5] <- matrix(rep(s3[,3:5], rep(tab,3)), ncol=3)
    so[repIn2,6] <- rep(tab,tab) }
  so[,7] <- so[match(so[,4], so[,1]),6]                                                 # change from n.current to n of best
  chNA <- is.na(so[,2])
  if(any(chNA)) so[which(chNA),3:7] <- NA
  if(ppm) {colnames(so)[5] <- "ppm"
    so[,5] <- 1e6*so[,3]/so[,2] }
  if(length(digSig) >0) if(is.numeric(digSig)) {
    so[,3] <- signif(so[,3], digits=digSig)
    so[,5] <- signif(so[,5], digits=digSig) }
  so[if(initOrder) order(so[,1]) else 1:nrow(so), c(1:3,5:7,4)] }
  
#' find closest neighbour to numeric vector
#'
#' This function aims to find closest neighbour to numeric vector
#'
#' @param z (numeric) vector to search minimum difference
#' @param initOrder (logical) return matrix so that 'x' matches exactely 2nd col of output
#' @param rat (logical) express result as ratio
#' @return This function returns a matrix with index,value,dif,best
#' @seealso  \code{\link[stats]{dist}}
#' @examples
#' .minDif(c(11:15,17))
#' @export
.minDif <- function(z, initOrder=TRUE, rat=TRUE){
  ## find closest neighbour to numeric vector 'z', return matrix with index,value,dif,best
  ##  note that 'dif' is reported as absolule !
  ## 'sortRet' .. if TRUE return in orig order of values in 2nd col of dat
  ## however, repeated values will NOT be evalualted correctly !!
  z <- cbind(index=1:length(z), value=z)[order(z),]
  if(rat)
  so <- cbind(z, dif=c(diff(z[,2]),Inf), best=c(z[-1,1],NA))
  sw <- cbind(diA=so[,3], diB=c(Inf, so[-nrow(so),3]))
  alt <- which(sw[,1] > sw[,2])
  if(length(alt) >0) {so[alt,4] <- so[alt -1,1]
    so[alt,3] <- so[alt -1,3]}
  if(rat) {so <- cbind(so, rat=so[,2]/so[match(so[,4], so[,1]),2]) }  
  if(initOrder) so[order(so[,1]),] else so }
     

#' CV of replicate plates (list of matrixes)
#'
#' This function calculates CVs of replicates from list of 2 or 3-dim arrays (where 2nd dim is replicates, 3rd dim may be channel). 
#' Note : all list-elements of must MUST have SAME dimensions !
#' When treating data from microtiter plates (eg 8x12) data are typically spread over multiple plates, ie initial matrixes that are the organized into arrays.
#' Returns matrix or array (1st dim is intraplate-position, 2nd .. plate-group/type, 3rd .. channels)
#' @param lst list of matrixes : suppose lines are independent elements, colums are replicates of the 1st column. All matrixes must have same dimensions
#' @param silent (logical) suppress messages if \code{TRUE}
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allows easier tracking of messages produced
#' @return This function returns a matrix or array (1st dim is intraplate-position, 2nd .. plate-group/type, 3rd .. channels)
#' @seealso \code{\link{rowCVs}}, @seealso \code{\link{arrayCV}}
#' @examples
#' set.seed(2016); ra1 <- matrix(rnorm(3*96),nrow=8)
#' pla1 <- list(ra1[,1:12],ra1[,13:24],ra1[,25:36])
#' replPlateCV(pla1)
#' arrL1 <- list(a=array(as.numeric(ra1)[1:192],dim=c(8,12,2)),
#'   b=array(as.numeric(ra1)[97:288],dim=c(8,12,2)))
#' replPlateCV(arrL1)
#' @export
replPlateCV <- function(lst, silent=FALSE, debug=FALSE, callFrom=NULL) {
  fxNa <- .composeCallName(callFrom,newNa="replPlateCV")
  if(isTRUE(debug)) silent <- FALSE
  if(!isTRUE(silent)) silent <- FALSE
  chDim <- sapply(lst, dim)
  msg <- "ALL list-elements of must MUST have SAME dimensions"
  if(is.matrix(chDim)) {if(any(apply(chDim,1,function(x) length(unique(x))>1))) stop(fxNa,msg)} else stop(fxNa,msg)
  if(length(dim(lst[[1]])) >2){
    out <- matrix(sapply(lst, arrayCV), nrow=nrow(lst[[1]]))
    tmp <- 1:(ncol(out)/2)
    out <- out[,c((2*tmp-1),2*tmp)]
    out <- array(out, dim=c(nrow(lst[[1]]), length(lst), dim(lst[[1]])[3]), dimnames=list(
      rownames(lst[[1]]), names(lst), dimnames(lst[[1]])[[3]]))
  } else {        
    out <- matrix(sapply(lst, arrayCV), nrow=nrow(lst[[1]]), dimnames=list(
      rownames(lst[[1]]), names(lst))) }
  out }
   

#' Make a list of common occurances sorted by number of repeats  
#'
#' The aim of this function is to count the number of occurances of words when comaring separate vectors (\code{x}, \code{y} and \code{z}) or from a list (given as \code{x})
#' and to give an output sorted by their frequency.
#' The output lists the various values/words by their frequency, the names of the resulting list-elements indicate number of times the values/words were found repeated.	
#' 
#' @details
#' 
#' In order to compare the frquency of values/words between separate vectors or vectors within a list, it is necessary that these have been made unique before calling this function or using \code{filterIntraRep=TRUE}. 
#' 
#' In case the input is given as list (in \code{x}), there is no restriction to the number of vectors to be compared. 
#' With very long lists, however, the computational effort incerases (like it does when using \code{table})
#' 
#' @param x (list, character or integer) main input, if list, arguments \code{y} and \code{z} will not be used 
#' @param y (character or integer) supplemental vector to comare with \code{x} 
#' @param z (character or integer) supplemental vector to comare with \code{x} 
#' @param filterIntraRep (logical) allow making vectors \code{x}, \code{y} and \code{z} unique before comparing (defaults to \code{TRUE})  
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging 
#' @param callFrom (character) allow easier tracking of messages produced
#' @return This function returns a list sorted by number of occurances. The names of the list indicate the number of repeats. 	
#' @seealso  \code{\link[base]{table}},  \code{\link[wrMisc]{replicateStructure}} 
#' @examples
#' sortByNRepeated(x=LETTERS[1:11], y=LETTERS[3:13], z=LETTERS[6:12])
#' sortByNRepeated(x=LETTERS[1:11], y=LETTERS[c(3:13,5:4)], z=LETTERS[6:12])
#' 
#' @export
sortByNRepeated <- function(x, y=NULL, z=NULL, filterIntraRep=TRUE, silent=TRUE, debug=FALSE, callFrom=NULL) {
  ## move to wrMisc with combineAsN()
  ## make list of common occurances sorted by number of repeats based on list of character-entries/words (eg peptide sequ) or up to 3 separate character vectors
  ## the name of output indicates number onf times all elements of the vector are repeated
  ## for 4 sets of data provide 'x' in form of list conatining all data
  fxNa <- .composeCallName(callFrom, newNa="sortByNRepeated")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  if(length(x) >1 && is.list(x)) {
    if(length(y) >0) x[length(x) +1] <- unlist(y)
    if(length(z) >0) x[length(x) +1] <- unlist(z)
  } else {x <- list(x=x, y=y, z=z); rm(y,z)}
  chLe <- sapply(x, length) >0
  if(debug) { message(fxNa,"sBYN1   list-elements >0 ",pasteC(chLe))}

  ## remove empty list-elements
  if(!all(!chLe) && any(!chLe)) { x <- x[which(chLe)]
    if(debug) message(fxNa,"Removing ",sum(!chLe)," empty entries (out of ",length(x),")")
    chLe <- sapply(x, length) >0 }
  if(debug) {message(fxNa,"sBYN2   length of list-elements >0 ",pasteC(sapply(x,length))) }
  if(length(x) <2) { if(!silent) message(fxNa,"Only ",length(x)," set(s) of data provided", if(length(x) <1 | filterIntraRep)" nothing to do !")
    if(length(x) ==1) {             ## simple case : single vector to treat
      chDu <- duplicated(x[[1]])
      if(any(chDu) & filterIntraRep) x[[1]] <- unique(naOmit(x[[1]]))
      out2 <- table(table(x[[1]]))
      out <- rep(0, max(as.integer(names(out2))))
      out[match(as.integer(names(out2)), 1:max(as.integer(names(out2))) )] <- out2
      if(length(out) >1) {
        out <- c(out[1:3], min2=sum(out[2:3],na.rm=TRUE), any=sum(out), if(length(out) >3) out[4:length(out)])
      } else out <- c(out,0,0,0,out)
      out <- array(c(out, rep(NA, length(out)*3)), dim=c(length(out),1,4), dimnames=list(
        c("sing","doub","trip","min2","any", if(length(out) >5) paste0("x",4:(length(out) +2))),
        names(x[[1]]), c("n","sem","CI","sd")) )
    } else out <- NULL
  } else {
    ## check/remove for internal repeats
    if(filterIntraRep) {
      chDu <- lapply(x, duplicated)
      chDu2 <- sapply(chDu, any)
      if(any(chDu2)) { if(debug) message(fxNa,sum(chDu2)," list elements contain (intra-) duplicated elements, need to remove for correct inter-comparison")
        for(i in which(chDu2)) x[[i]] <- x[[i]][which(!chDu[[i]])] }
    }
    if(debug) {message(fxNa,"sBYN3   length of list-elements ",pasteC(sapply(x,length))) }

    ## sort according to level of duplication
    x <- table(unlist(x, use.names=FALSE))
    ta2 <- table(x)
    out <- sapply(names(ta2), function(y) names(x)[which(x==as.integer(y))]) }
  out }
    

#' Make single vector gray-gradient
#'
#' This function helps making gray-gradients.
#' Note : The resulting color gradient does not seem linear to the human eye, you may try \code{\link[grDevices]{gray.colors}} instead
#' @param startGray (numeric) gray shade at start
#' @param endGrey (numeric)  gray shade at end 
#' @param nSteps (integer) number of levels
#' @param transp (numeric) transparency alpha
#' @return character vector (of same length as x) with color encoding
#' @seealso \code{\link[grDevices]{gray.colors}}
#' @examples
#' layout(1:2)
#' col1 <- transpGraySca(0.8,0.3,7,0.9) 
#' pie(rep(1,length(col1)), col=col1, main="from transpGraySca")
#' col2 <- gray.colors(7,0.9,0.3,alph=0.9)
#' pie(rep(1,length(col2)), col=col2, main="from gray.colors")
#' @export
transpGraySca <- function(startGray=0.2, endGrey=0.8, nSteps=5, transp=0.3){
  ## make single vector gray-gradient
  ## 'transp' should be numeric (betw 0 and 1) or NULL for no transparency (high val .. no transparency)
  ## default startGray (low) -> endGrey (high) goes from dense to light
  ## NOTE : color gradient NOT LINEAR to EYE !! use rather grey.colors
  grSeq <- seq(startGray, endGrey, len=nSteps)
  useCol <- grDevices::rgb(r=grSeq, g=grSeq, b=grSeq, alpha=if(is.null(transp)) 1 else transp)
  useCol }

#' Trimmed Mean
#'
#' This function allows more flexible options for calculating a trimmed mean compared to \code{mean} (from the base-package).
#' For example, the trimming may be asymmetric to the median of the data.
#' 
#' 
#' @details
#' If the second value of trim is <0.5 it is supposed that this indicates a fraction from the upper end the vector-dat as \code{\link[base]{mean}} does.
#' Otherwise, \code{trim=c(l=0.2,u=0.7)} will be interpreted indication to use the 20th percentile to 70th percentile of \code{dat}.
#' 
#' Please note, that trimmed means - and in particular asymmetric trimmed means - should be used with caution as there is also a risk of introducing bias.
#' 
#' 
#' @param dat (numeric) numeric vector
#' @param trim (numeric, length=2) specifies how data should get trimmed, lower and upper fraction(s) to exclude have to be assigned separately.
#'   The lower and upper fraction may be named 'l' and 'u'.
#'   The value 0 means that all (sorted) data on a given side will be used. 
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allows easier tracking of messages produced
#' @return This function returns a (numeric) vector with the trimmed mean
#' @seealso  \code{\link[base]{mean}} (symmetric trimming only)
#' @examples
#' x <- c(17:11,27:28)
#' mean(x); mean(x, trim=0.15)
#' trimmedMean(x, trim=c(l=0, u=0.7))   # asymmetric trim
#' 
#' @export
trimmedMean <- function(dat, trim=c(l=0.2,u=0.2), silent=FALSE, debug=FALSE, callFrom=NULL) {
  ##
  fxNa <- .composeCallName(callFrom, newNa="trimmedMean")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  out <- NULL
  datOK <- is.numeric(dat) && length(dat) >0 
  if(length(trim) <1) trim <- c(0, 0)
  if(datOK) {
    if(length(trim) <2) {trim <- rep(trim, 2); 
      if(debug) message(fxNa,"Argument 'trim' is only of length=1, setting given value as symmetric") }
  } else { if(!silent) message(fxNa,"Invalid 'dat', returning NULL")}
  if(datOK) if(all(is.na(trim)) || all(is.na(dat))) {datOK <- FALSE; out <- NA}
  if(datOK) datOK <- all(trim >= 0 & trim <= 1)

  ## main
  if(datOK) if(sum(duplicated(trim[1:2]))==1) { out <- mean(dat, trim=trim[1], na.rm=TRUE)
    } else {
      if(trim[1]==trim[2]) out <- mean(dat, trim=trim[1], na.rm=TRUE) else {
        if(all(c("l","u") %in% names(trim))) trim <- trim[match(c("l","u"), names(trim))]
        if(trim[2] < 0.5) trim[2] <- 1 - trim[2]
        trim <- round(trim[1:2]*length(dat))
        if(debug) message(fxNa,"Using indexes ",trim[1]," to ",trim[2],"  out of 1:",length(dat))
        out <- if(trim[2] != trim[1]) mean(sort(dat)[trim[1]:trim[2]], na.rm=TRUE) else dat[trim[1]]}
    }   
  out }
    

#' Append vectors or lists, without duplcating common elements
#'
#' This function allows combining two vectors or lists without duplicating common content (definded by name of list-elements). 
#' 
#' @details When setting the argument \code{rmDuplicate=FALSE} the function will behave like \code{append}. 
#' 
#' @param x (vector or list) must have names to allow checking for duplicate names in y
#' @param y (vector or list) must have names to allow checking for duplicate names in x
#' @param rmDuplicate (logical) avoid duplicating liste-elements present in both x and y (based on names of list-elements)
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allow easier tracking of message(s) produced
#' @return If both \code{x} and \code{y} are vectors, the output will be a vector, otherwise it will be a list
#' @seealso  \code{\link[base]{append}}; \code{\link{lrbind}}
#' @examples
#' li1 <- list(a=1, b=2, c=3)
#' li2 <- list(A=11, B=12, c=3)
#' appendNR(li1, li2)
#' append(li1, li2) 
#' @export
appendNR <- function(x, y, rmDuplicate=TRUE, silent=FALSE, debug=FALSE, callFrom=NULL) {
  fxNa <- .composeCallName(callFrom, newNa="appendNR")
  if(!rmDuplicate | length(x) <1 | length(y) <1) { x <- append(x, y)
  } else {
    chY <- !names(y) %in% names(x)         #  names of y not in x (ie new)
    if(any(chY)) {
      xLe <- length(x)
      x <- append(x, y[which(chY)])
      if(!silent) message(fxNa,"Adding ",sum(chY)," new names/elements (",sum(!chY)," already present)")
      names(x)[xLe + seq(sum(chY))] <- names(y[which(chY)]) } }
  x }  
  

#' Trim/Remove Redundant Words
#'
#' This function allows removing shared words, ie triming to non-redundant words.
#' 
#'
#' @details
#' Heading separators will be removed in any case (even if not followed by a 'word').
#'
#' Special characters will be automatically protected.
#' When looking for repeated words, the order of such words does NOT matter, multiple repeats will be removed, too.
#'  
#' #' 
#' @param x (character) main input for making non-redundant
#' @param sep (character) separator(s) to be used
#' @param anySep (logical) if \code{TRUE}, will consider all separators at one time (), thus combinations with different separators won't be distinguished
#' @param newSep (character) new (uniform) separator between words, if \code{NULL} the first value/separator of if \code{sep} will be used
#' @param minLe (integer) minimum length for allowing being recognised as 'word'
#' @param na.omit (logical) if \code{TRUE NA}s will be removed from output 
#' @param fixed (logical) will be transmitted to argument \code{fixed} of \code{strsplit()};  if \code{TRUE} regular expressions are allowed/used
#' @param callFrom (character) allows easier tracking of messages produced
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' @return This function returns character vector of same length (unless \code{na.omit=TRUE}), simply with modified text-content
#' @seealso \code{\link{trimRedundText}}
#' @examples
#' x1 <- c("aa_A1 yy_zz.txt", NA, "B2 yy_aa_aa_zz.txt")
#' rmSharedWords(x1)
#'  
#' @export
rmSharedWords <- function(x, sep=c("_"," ","."), anySep=TRUE, newSep=NULL, minLe=2, na.omit=FALSE, fixed=TRUE, silent=FALSE, debug=FALSE, callFrom=NULL) {
  ## function to trim redundant words (@separator) similar to trimRedundText()
  ## remove common/repeated words as occuring in each instance of x; words are separarated by sep (ignoring NAs); separators must be consistent (no mixing of separators allowed)
  ## special characters will be automatically protected, order does NOT matter, multiple repeats will be removed, too
  ## note : anySep=TRUE will consider all separators at one time (), thus combinations with different separators won't be distinguished
  ## note : heading separators will be removed in any case
  ## move to wrMisc ??
  #example#   x1 <- c("aa_A1 yy_zz.txt", NA, "B2 yy_aa_aa_zz.txt"); rmSharedWords(x1)
  fxNa <- .composeCallName(callFrom, newNa="rmSharedWords")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  chNA <- NULL
  datOK <- length(x) >0 && length(sep) >0
  if(datOK) {
    chNA <- is.na(x)
    x2 <- if(any(chNA)) x[-which(chNA)] else x
    redSep <- which(colSums(sapply(protectSpecChar(sep), function(y) nchar(x2) > nchar(sub(y,"", x2))))==length(x2))
  } else x2 <- x
  if(datOK && length(redSep) >0) {
    se2 <- sep[redSep]                                  # reduce to separators appearing in all cases
    if(length(newSep) >1) newSep <- newSep[redSep]
    if(isTRUE(anySep)) {    # combine all separators
      iP <- paste(protectSpecChar(sep), collapse="|")
      se2 <- paste(sep, collapse="|")
      fixed <- FALSE
    } else  { se2 <- sep
      iP <- protectSpecChar(sep) }
    for(i in 1:length(se2)) {
      i2 <- iP[i]
      chSep <- if(i==1) TRUE else all(nchar(x2) > nchar(sub(i2,"", x2)))            # check current separator 'se2' if occuring in each instance
      if(chSep) {
        spl <- strsplit(x2, split=if(isTRUE(fixed)) se2[i] else iP[i], fixed=fixed)        
        rmW <- Reduce(intersect, spl)                              # get words common in all instances
        if(length(rmW) >0) { 
          nCh <- nchar(rmW)
          chL <- nCh >= minLe | nCh ==0
          if(any(chL)) {
            spl <- lapply(spl, function(y) y[-which(y %in% unique(c(rmW[which(chL)], if(TRUE) "")))])       # remove repeated/common 'words'
            newSe <- if(length(newSep) >0) {if(length(newSep) < length(sep)) rep(newSep,i)[i] else newSep[i]} else sep[1]
            x2 <- sapply(spl, paste, collapse=newSe) } }                      # paste collapse
      }
    } }
  if(!any(chNA) || isTRUE(na.omit)) x2 else {out <- x; out[-which(chNA)] <- x2; out}
}
   

#' Fast na.omit 
#'
#' This function removes NAs from input vector, in contrast to \code{na.omit} this function has no slot for removed elements. 
#' @details
#' Resulting objects from \code{naOmit} are smaller in size and subsequent code execution (on large vectors) may be faster (in particular if many NAs get encountered).
#' Note : This function behaves differently to \code{na.omit} with input other than plain vectors. Will not work with data.frames !
#' @param x (vector or matrix) data to check & remove \code{NA}s 
#' @param silent (logical) suppress messages if \code{TRUE}
#' @param debug (logical) additional messages for debugging
#' @param callFrom (character) allows easier tracking of messages produced
#' @return This function returns a vector without NAs (matrix input will be transformed to vector). Returns NULL if input consists only of NAs. 
#' @seealso \code{\link[stats]{na.fail}}, \code{na.omit}   
#' @examples
#' aA <- c(11:13,NA,10,NA); 
#' naOmit(aA) 
#' @export
naOmit <- function(x, silent=FALSE, debug=FALSE, callFrom=NULL) {
  fxNa <- .composeCallName(callFrom, newNa="naOmit")
  chNa <- is.na(x)
  if(all(chNa)) NULL else x[which(!chNa)] }
   

#' Check order of multiple groups including non-overlapping SEM-margins
#'
#' \code{checkGrpOrderSEM} tests each line of 'x' if expected order of (replicate-) groups (defined in 'grp') appears intact,
#'  while inluding SEM of groups (replicates) via a proportional weight 'sdFact' as (avGr1-gr1SEM) < (avGr1+gr1SEM) < (avGr2-gr2SEM) < (avGr2+gr2SEM).  
#' Used for comparing groups of measures with expected profile (by matching expected order)
#' to check if data in 'x' represting groups ('grp') as lines follow.  
#' Groups of size=1: The sd (and SEM) can't be estimated directly without any replicates, however, an estimate can be given by shrinking if 'shrink1sampSd'=TRUE 
#' under the hypothesis that the overall mechanisms determining the variances is constant across all samples.
#' @param x matrix or data.frame
#' @param grp (factor) to organize replicate columns of (x)
#' @param sdFact (numeric) is proportional factor how many units of SEM will be used for defining lower & upper bounds of each group
#' @param revRank (logical) optionally revert ranks
#' @param shrink1sampSd (logical)
#' @param silent (logical) suppress messages
#' @param callFrom (character) allow easier tracking of message(s) produced
#' @return logical vector if order correct (as expected based on ranks)
#' @seealso takes only 10% more time than \code{\link[wrMisc]{checkGrpOrder}} wo considering intra-group sd 
#' @examples
#' mat1 <- matrix(rep(11:24,3)[1:40],byrow=TRUE,ncol=8)
#' checkGrpOrderSEM(mat1,grp=gl(3,3)[-1])
#' @export
checkGrpOrderSEM <- function(x,grp,sdFact=1,revRank=TRUE,shrink1sampSd=TRUE,silent=FALSE,callFrom=NULL){
  fxNa <- .composeCallName(callFrom,newNa="checkGrpOrderSEM")
  if(length(dim(x)) !=2) stop(" 'x' should be data.frame or matrix of 2 dimensions")
  if(length(grp) != ncol(x)) stop(" 'grp' should be of length of number of cols in 'x'")
  if(length(grp) <1 | sum(is.na(grp)) == length(grp)) stop(" 'grp' appears to be empty or all NAs")
  if(!is.factor(grp)) grp <- as.factor(grp)
  ## main
  avs <- .rowGrpMeans(x,grp)
  sds <- .rowGrpSds(x,grp)
  sdsNaCol <- colSums(is.na(sds)) ==nrow(x)
  if(shrink1sampSd & sum(sdsNaCol) >0) {
    if(!silent) message(fxNa,sum(sdsNaCol)," single-column groups : estimating possible sd based on other samples/groups")
    sds[,which(sdsNaCol)] <- stats::median(naOmit(as.numeric(sds)),na.rm=TRUE) }
  nMa <- matrix(rep(naOmit(as.numeric(table(grp))[order(unique(grp))]),each=nrow(x)),nrow=nrow(x))
  lims <- list(low=avs - sdFact*sds, hi=avs + sdFact*sds)
  out <- matrix(nrow=nrow(x),ncol=2*ncol(avs))
  for(i in 1:ncol(avs)) out[,(2*i-1):(2*i)] <- cbind(lims[[1]][,i],lims[[2]][,i])
  checkGrpOrder(out,rankExp=1:ncol(out),revRank=revRank) }
   

#' Convert Vector To Numeric
#'
#' This function checks if input vector/character string contains numbers (with or without comma) and attempts converting to numeric.
#' This functions was designed for extracting the numeric part of character-vectors (or matrix) containing both numbers and character-elements.
#' Depending on the parameters \code{convert} and \code{remove} text-entries can be converted to NA (in resulting numeric objects) or removed (the number of elements/lines gets reduced, in consequece). 
#' Note: if 'x' is a matrix, its matrix-dimensions & -names will be preserved.
#' Note: so far Inf  and -Inf do not get recognized as numeric.
#'
#' @param x vector to be converted
#' @param autoConv (logical) simple automatic conversion based on function \code{as.numeric}; if \code{TRUE} all other arguments exept  \code{spaceRemove} will not be considered
#' @param spaceRemove (logical) to remove all heading and trailing (white) space (until first non-space character)
#' @param convert (character) define which type of non-conform entries to convert to NAs. Note, if \code{remove} is selected to eliminate character-entries they cannot be converted any more. Use 'allChar' for all character-entries; 'sparseChar' sparse (ie rare) character entries; \code{NA} for converting 'Na' or 'na' to \code{NA}; if 'none' or \code{NULL} no conversions at all. 
#' @param remove (character) define which type of non-conform entries to remove, removed items cannot be converted to \code{NA} any more. Use 'allChar' for removing all character entries; \code{NA} for removing all instances of \code{NA} (execept thise created by converting text); all elements will be kept if 'none' or \code{NULL}. 
#' @param euroStyle (logical) if \code{TRUE} will convert all ',' (eg used as European decimal-separator) to '.' (as internally used by R as decimal-separator), thus allowing converting the European decimal format. 
#' @param sciIncl (logical) include recognizing scientific notation (eg 2e-4)
#' @param callFrom (character) allow easier tracking of messages produced
#' @param silent (logical) suppress messages
#' @param debug (logical) additional messages for debugging
#' 
#' @details This function may be used in two modes, depening if argument \code{autoConv} is \code{TRUE} or \code{FALSE}.
#' The first options allows accessing an automatic mode based on \code{as.numeric}, 
#' while the second options investigates all characters if they may belong to numeric expressions and allows removing specific text-elements. 
#' 
#' @return This function returns a numeric vector (or matrix (if 'x' is matrix))
#' @seealso \code{\link[base]{numeric}} and \code{as.numeric} (on same help-page)
#' @examples
#' x1 <- c("+4"," + 5","6","bb","Na","-7") 
#' convToNum(x1) 
#' convToNum(x1, autoConv=FALSE, convert=c("allChar"))
#' convToNum(x1, autoConv=FALSE)      # too many non-numeric instances for 'sparseChar'
#' 
#' x2 <- c("+4"," + 5","6","-7"," - 8","1e6","+ 2.3e4","-3E4","- 4E5") 
#' convToNum(x2) 
#' convToNum(x2, autoConv=FALSE, convert=NA,remove=c("allChar",NA))
#' convToNum(x2, autoConv=FALSE, convert=NA,remove=c("allChar",NA),sciIncl=FALSE)
#' @export 
convToNum <- function(x, autoConv=TRUE, spaceRemove=TRUE, convert=c(NA,"sparseChar"),remove=NULL,euroStyle=TRUE,sciIncl=TRUE,callFrom=NULL,silent=TRUE, debug=FALSE) {
  fxNa <- .composeCallName(callFrom,newNa="convToNum")
  if(isTRUE(debug)) silent <- FALSE else { debug <- FALSE
    if(!isTRUE(silent)) silent <- FALSE }
  if(!is.numeric(x)) {
    if(isTRUE(autoConv)) {
      if(isTRUE(spaceRemove)) x <- gsub(" ","",x)
      ini <- list(class=class(x), mode=mode(x), len=length(x), dim=dim(x))
      if(length(ini$dim) ==2) ini$dimnames=dimnames(x)
      if(!any(c("numeric","integer") %in% ini$class)) {     # thus non-numeric
        if(isFALSE(silent)) message(fxNa,"length(x) ",ini$len,"   class(x) ",class(x),"    mode(x) ",  mode(x),
          "   head ",paste(utils::head(x),collapse=" "))
        if("data.frame" %in% ini$class | "list" %in% ini$mode) x <- unlist(x)
        x <- try(suppressWarnings(as.numeric(if(inherits(x, "factor")) as.character(x) else x)), silent=TRUE)
        if(inherits(x, "try-error")) x <- rep(NA,ini$len)
        if(length(ini$dim) ==2) {x <- matrix(x, nrow=ini$dim[1], ncol=ini$dim[2], dimnames=ini$dimnames)
          if("data.frame" %in% ini$class) x <- as.data.frame(x)}         
      }    
    } else {   
      ## old version of this function
      if(is.factor(x)) x <- as.character(x)
      if(isTRUE(spaceRemove)) x <- sub("^ +","",sub(" +$","",x))
      if(isTRUE(euroStyle)) x <- sub(",",".",x)
      if("none" %in% convert) convert <- NULL
      if("none" %in% remove) remove <- NULL
      ## convert NA-like text (if specified via 'convert')
      if(length(convert) >0) if("na" %in% tolower(convert)) convert[which(tolower(convert) %in% "na")] <- NA
      if(length(remove) >0) if("na" %in% tolower(remove)) convert[which(tolower(remove) %in% "na")] <- NA
      if(any(is.na(convert))) { naCh <- tolower(x) %in% "na" | tolower(x) %in% "nan"
        if(any(naCh)) x[which(naCh)] <- NA}
      if("NA" %in% remove | any(is.na(remove))) {chNa <- is.na(x); if(any(chNa)) x <- x[-1*which(chNa)]}
      aa <- c("[[:digit:]]+\\.[[:digit:]]+$","[[:digit:]]+$")
      ab <- paste0("^",rep(c("","\\+ *","\\- *"),each=2),rep(aa,3))
      check <- grep(paste(ab,collapse="|"),x)
      if(sciIncl) {
        ac <- c("[[:digit:]]+","[[:digit:]]+\\.[[:digit:]]+")
        ac <- matrix(c(rep("^",24),rep(c("","\\+ *","\\- *"),8),rep(ac,12),rep(c("E","e"),each=12),rep(c("","","-","-"),6),rep(ac[1],24),rep("$",24)),nrow=24)
        ac <- apply(ac,1,paste,collapse="")
        check2 <- unlist(sapply(ac,grep,x))
        check <- sort(unique(c(check,check2))) }
      ## include NAs to check (unless specified to remove)
      chNa <- is.na(x)
      if(!any(is.na(remove)) & any(chNa)) check <- sort(unique(c(check,which(chNa))))    
      ## option remove all character entries
      if(length(remove) >0) {
        msg <- NULL
        if(any(is.na(remove))) { chNa <- is.na(x)
          if(any(chNa)) {x <- x[-1*which(chNa)]
          msg <- c("  remove ",sum(chNa)," NAs !")
          check <- check[-1*which(check %in% which(chNa))]} }
        if("allChar" %in% remove) { x <- if(length(check) >0) x[check] else NULL
          if(length(x) -length(check) >0) msg <- c(if(length(msg) >0) c(msg," ;"),"  remove 'allChar': ",
            length(x) -length(check)," text-elements removed allowing conversion to numeric !")
          check <- if(length(x) >0) 1:length(x) else NULL }
        if(isFALSE(silent) & length(msg) >0) if(paste(nchar(msg),collapse="") >3) message(fxNa,msg)
      }
      ## option: convert (sparse) character entries to NA
      if(length(x) >0) {if(any(c("allChar","sparseChar") %in% convert) & !"allChar" %in% remove) { 
        msg <- conv <- NULL
        if("allChar" %in% convert) { conv <- if(length(check) >0) (1:length(x))[-1*check] else 1:length(x)
          if(length(x) >length(check)) msg <- c(" option convert='allChar': replacing ",length(x)-length(check)," character-entries by NA !")
        } else if("sparseChar" %in% convert & length(check) > ceiling(length(x)*0.7)) {
          conv <- (1:length(x))[-1*check] }
        if(length(conv) >0) {
          msg <- c(" replacing ",length(conv)," 'sparse' character-entries by NA !")
          x[conv] <- NA; check <- sort(unique(c(check,conv))) }
        if(length(msg) >0 & isFALSE(silent)) if(paste(nchar(msg),collapse="") >3) message(fxNa,msg)  
      }
      if(length(check) >0) x[check] <- sub(" ","",x[check])
      chNa <- is.na(x) 
      x <- if(length(x) <1) NULL else { if(identical(1:length(x),check)) as.numeric(x) else {
        if(identical(1:length(x), sort(unique(check,which(chNa))))) as.numeric(x) else x}}}      
  } }
  x }     
  

#' Extract Longest Common Text Out Of Character Vector
#'
#' @description 
#' This function allows recovering the single longest common text-fragments (from center, head or tail) out of character vector \code{txt}.
#' Only the first of all of the longest solutions will be returned.
#'
#' @details
#' Please note, that finding common parts between chains of characters is not a completely trivial task. 
#' This topic still has ongoing research for the application of sequence-alignments, where chains of characters to be compared get very long.
#' This function uses a k-mer inspirated approach.
#' The initial aim with this function was allowing to treat smaller chains of characters (and finding shorter strteches of common text), like eg with column-names.
#' 
#' Important : This function identifies only the first best hit, ie other shared/common character-chains of the same length will not be found !
#' 
#' Using the argument \code{hiResol=FALSE} it is possible to accelerate the search aprox 3x (with larger character-vectors), however, frequently the very best solution may not be found. 
#' This means, that in this case the result should rather be considered a 'seed', allowing check if further extension may improve the result, 
#' ie for identifying a (slightly) longer chain of common characters.
#' 
#' With longer vectors and longer character chains this may get demanding on computational reesources, the argument \code{hiResol=FALSE} allows reducing this at the price of missing the best solution.
#' With this argument single common/matching characters will not be searched if all text-elements are longer than 500 characters, an empty character vector will be returned. 
#' 
#' When argument \code{side} is either \code{left}, \code{right} or \code{terminal} only terminal common text may be found (a potentially even longer internal text will be lost).
#' Of course, choosing this option makes searches much faster.  
#' 
#' This function does not return the position of the shared/common characters within the text, you may use \code{gregexpr} or \code{regexec} to locate them.
#' 
#' 
#' @param txt character vector to be treated
#' @param minNchar (integer) minumin number of characters that must remain
#' @param side (character) may be be either 'center', 'any', 'terminal', 'left' or 'right'; only with \code{side='center'} or \code{'any'} internal text-segments may be found
#' 
#' @param hiResol (logical) find best solution, but at much higher comptational cost (eg 3x slower, however \code{hiResol=FALSE} rather finds anchor which may need to get extended)
#' @param silent (logical) suppress messages
#' @param callFrom (character) allow easier tracking of messages produced
#' @param debug (logical) display additional messages for debugging
#' @return This function returns a character vector of length=1, ie only one (normally the longest) common sequence of characters is identified.
#'  If nothing is found common/shared an empty character-vector is returned
#' @seealso Use \code{gregexpr} or \code{regexec} in \code{\link[base]{grep}} for locating the identified common characters in the initial query. 
#' @seealso Inverse : Trim redundant text (from either side) to keep only varaible part using \code{\link{trimRedundText}}; 
#'  you may also look for related functions in package \href{https://CRAN.R-project.org/package=stringr}{stringr}
#' @examples
#' txt1 <- c("abcd_abc_kjh", "bcd_abc123", "cd_abc_po")
#' keepCommonText(txt1, side="center")       # trim from right
#' 
#' txt2 <- c("ddd_ab","ddd_bcd","ddd_cde")
#' trimRedundText(txt2, side="left")          #  
#' keepCommonText(txt2, side="center")        # 
#' @export
keepCommonText <- function(txt, minNchar=1, side="center", hiResol=TRUE, silent=TRUE, callFrom=NULL, debug=FALSE) {
  ##
  ##
  fxNa <- .composeCallName(callFrom, newNa="keepCommonText")
  if(!isTRUE(silent)) silent <- FALSE
  if(isTRUE(debug)) silent <- FALSE else debug <- FALSE
  if(!isFALSE(hiResol)) hiResol <- TRUE
  side <- try(as.character(side), silent=TRUE)
  if(inherits(side, "try-error")) stop("Invalid entry for 'side', must be character, 'center','any', 'terminal', 'left' or 'right'")
  out <- lastOK <- NULL                      # initialize
  txOK <- TRUE
  ## check of input
  if(length(txt) <1 || all(is.na(txt))) { txOK <- FALSE; out <- NA
    if(!silent) message(fxNa,"Empty or all NA input, no common text possible")
  }
  if(txOK) {
    chNa <- is.na(txt)
    if(any(chNa)) txt <- txt[which(!chNa)]
    if(length(txt)==1) { out <- txt                          # length=1 .. full text
      if(debug) message(fxNa,"Single (valid) vector-element")}
    if(length(txt) >0 && length(out) <1) { chSame <- all(sapply(txt[-1], function(x) x==txt[1]))     # all the same no need for extensive testing
      if(all(chSame)) { out <- out[1]
        if(!silent) message(fxNa,"All text identical") } }
    nChar <- nchar(txt)
    if(length(out) <1 && any(nChar <1)) { out <- ""
      if(!silent) message(fxNa,"Some text elements are empty, no common text possible")}   
    if(debug) {message(fxNa,"kCT1"); kCT1 <- list(txt=txt,side=side,minNchar=minNchar,out=out,chSame=chSame,nChar=nChar,hiResol=hiResol)}  

    ## search at terminal positions
    if(length(txt) >0 && any(c("left","terminal") %in% side)) {
      txTr <- .trimRight(txt, minNchar=minNchar, silent=silent, callFrom=fxNa)
      if(nchar(txt[1]) > nchar(txTr[1])) { out <- txt <- sub(txTr[1],"",txt[1])
        nChar <- nchar(txt)}             # update
      if(debug) {message(fxNa,"kCT1a")}
    }
    if(length(txt) >0 && any(c("right","terminal") %in% side)) {
      txTr <- .trimLeft(txt, minNchar=minNchar, silent=silent, callFrom=fxNa)
      if(nchar(txt[1]) > nchar(txTr[1])) out <- sub(txTr[1],"",txt[1])    #
      nChar <- nchar(txt)                # update
    }
  }
  
  if(debug) {message(fxNa,"kCT2"); kCT2 <- list(txt=txt,side=side,minNchar=minNchar,out=out,chSame=chSame,nChar=nChar,hiResol=hiResol)}  
  
  ## search anywhere (incl center)
  if(length(txt) >0 && any(c("center","any") %in% side)) {
    ch1 <- min(nChar, na.rm=TRUE)
    if(ch1 -1 > minNchar) {                       # sufficient characters in all instances
      if(!hiResol && !silent) message(fxNa,"Please note that using the argument hiResol=FALSE, the optimal solution may not be found, you may check if the result can be further extended") 
      ch2 <- txt[which.min(nChar)]
      kMer <- 2
      kMerIni <- kMer    ## first round
      stSpl <- seq(1, ch1 -kMer +1, by=if(hiResol) 1 else kMer)
      words <- unique(substr(rep(ch2, length(stSpl)), stSpl, stSpl+kMer-1))
      ch3 <- sapply(lapply(words, grep, txt[-which.min(nChar)]), length) == length(txt) -1
      lastOK <- if(any(ch3)) list(kMer=kMer,words=words, ch3=ch3) else NULL
      ## now searh for longer if any matches found above
      while(any(ch3) && kMer < ch1 -2) { kMer <- kMer +2
        stSpl <- seq(1, ch1 -kMer +1, by=if(hiResol) 1 else kMer)
        words <- unique(substr(rep(ch2, length(stSpl)), stSpl, stSpl +kMer -1))
        if(debug) message(fxNa,"Increase kMer from ",kMer -2," to ",kMer,";  testing ",length(words)," character-chains")
        ch3 <- sapply(lapply(words, grep, txt[-which.min(nChar)]), length) == length(txt) -1
        if(any(ch3)) lastOK <- list(kMer=kMer,words=words, ch3=ch3)
      }
      if(debug) {message(fxNa,"kCT3"); kCT3 <- list(txt=txt,side=side,minNchar=minNchar,out=out,chSame=chSame,nChar=nChar,hiResol=hiResol,lastOK=lastOK, words=words,stSpl=stSpl,kMer=kMer,ch2=ch2,ch3=ch3)}  
      ## have reached too long words try step down, 1 less
      if(all(kMer > 1, !ch3, kMer > lastOK$kMer +1)) { 
        if(kMer==2) {    # nothing found at kMer=2, try split in single characters & look for common character
            if(minNchar==1 & any(all(nChar <500), hiResol)) {            # run only if hiResol=TRUE or all text entries are less than 500 chars
              txtSn <- lapply(strsplit(txt, ""), unique)
              ch3 <- table(unlist(txtSn))
              if(any(ch3==length(txt))) out <- names(ch3)[which(ch3==length(txt))[1]] }
        } else {
          kMer <- kMer -1
          stSpl <- seq(1, ch1 -kMer +1, by=if(hiResol) 1 else kMer)
          words <- unique(substr(rep(ch2, length(stSpl)), stSpl, stSpl+kMer-1))
          if(debug) message(fxNa,"Decrease kMer from ",kMer +1," to ",kMer,";  testing ",length(words)," character-chains")
          ch3 <- sapply(lapply(words, grep, txt[-which.min(nChar)]), length) == length(txt) -1
          if(any(ch3)) lastOK <- list(kMer=kMer,words=words, ch3=ch3)
        }
      }
      if(debug) {message(fxNa,"kCT4")}  
      if(length(lastOK) >0) out <- lastOK$words[which(lastOK$ch3)[1]]
    }
  }
  if(length(out) >0) out else ""}
  

---
title: "Getting started with wrMisc"
author: Wolfgang Raffelsberger
date: '`r Sys.Date()`'
encoding: UTF-8  
output:
  knitr:::html_vignette:
    toc: true
    fig_caption: yes
  pdf_document:
    highlight: null
    number_sections: no  
vignette: >
  %\VignetteIndexEntry{wrMiscVignette1}
  %\VignetteEngine{knitr::rmarkdown}
---


## Introduction

This package contains a collection of various (low-level) tools which may be of general interest, in particular in the context of working with 'omics'-data. 
These functions were accumulated over a number of years of data-wrangling when treating high-throughput data from biomedical applications.
Besides, these functions are further used/integrated in more specialized functions dedicated to specific applications in the packages [wrProteo](https://CRAN.R-project.org/package=wrProteo), [wrGraph](https://CRAN.R-project.org/package=wrGraph) or [wrTopDownFrag](https://CRAN.R-project.org/package=wrTopDownFrag). 
All these packages are available on [CRAN](https://cran.r-project.org/).

If you are not familiar with [R](https://www.r-project.org/) you may find many introductory documents on the official R-site 
in [contributed documents](https://cran.r-project.org/other-docs.html) or under [Documentation/Manuals](https://cran.r-project.org/manuals.html).
Of course, numerous other documents/videos/sites with tutorials and courses exist, too.


### Dependencies and Compilation
One of the aims was to write a package easy to install, with low system requirements and few obligatory dependencies.  
All code is written in pure R and does not need any special compilers.
The number of obligatory dependencies was kept to a minumum.

Most of additional packages used in some of the functions were declared as 'suggested' (ie not obligatory), to allow installation of _wrMisc_ even if some these additional packages can't be installed/compiled at the user's side.
When a feature/function of one of the 'suggested' packages is about to be used, its presence/installation will be checked and,
only if found as missing, the user will be prompted a message inviting to install specific package(s) before using these specific functions.


### Installation And Loading
To get started, we need to install (if not yet installed) and load the package "[wrMisc](https://CRAN.R-project.org/package=wrMisc)" available from [CRAN](https://cran.r-project.org/).


```{r setup0, include=FALSE, echo=FALSE, messages=FALSE, warnings=FALSE}
suppressPackageStartupMessages({
    library(wrMisc)
})
```

```{r install, echo=TRUE, eval=FALSE}
## If not already installed, you'll have to install the package first.
## This is the basic installation commande in R
install.packages("wrMisc")
```

Since the functions illustrated in this vignette require a number of the _suggested_ packages,
let's check if they are installed and add them (via a small function), if not yet installed.
```{r install2, echo=TRUE, eval=FALSE}
packages <- c("knitr", "rmarkdown", "BiocManager", "kableExtra", "boot", "data.tree", "data.table", 
  "fdrtool", "RColorBrewer", "Rcpp", "wrMisc", "wrGraph", "wrProteo")
checkInstallPkg <- function(pkg) {       # install function
  if(!requireNamespace(pkg, quietly=TRUE)) install.packages(pkg) }

## install if not yet present
sapply(packages, checkInstallPkg)
```

Finally, this package also uses the Bioconductor package [limma](https://bioconductor.org/packages/release/bioc/html/limma.html) 
which has to be installed differently (see also help on [Bioconductor](https://bioconductor.org)):
```{r install3, echo=TRUE, eval=FALSE}
## Installation of limma 
BiocManager::install("limma")
```

This vignette is also accessible from R command-line or on CRAN at [wrMisc](https://CRAN.R-project.org/package=wrMisc):
```{r install4, echo=TRUE, eval=FALSE}
## Now you can open this vignette out of R:
vignette("wrMiscVignette1", package="wrMisc")
```

Before using the functions of this package, we actually need to load the package first (best on a fresh R-session):

```{r setup1}
library("wrMisc")

## This is 'wrMisc' version number :
packageVersion("wrMisc")
```



## Speed Optimized Functions In The Package wrMisc

In high-throughput experiments in biology (like transcriptomics, proteomics etc...) many different features get measured a number if times (different samples like patients or evolution of a disease). The resulting data typically contain many (independent) rows (eg >1000 different genes or proteins who's abundance was measured) and much fewer columns that may get further organized in groups of replicates. 
As R is a versatile language, multiple options exist for assessing the global characteristics of such data, some are more efficient on a computational point of view.
In order to allow fast treatment of very large data-sets some tools have been re-designed for optimal performance.


### Assessing Basic Information About Variability (for matrix)

Many measurement techniques applied in high throughput manner suffer from precision.
This means, the same measurements taken twice in a row (ie repeated on the same subject) will very likely not give an identical result. 
For this reason it is common practice to make replicate measurements to i) estimate mean (ie representative) values and ii) asses the factors contributing to the variablity observed.
Briefly, technical replicates represent the case where multiple read-outs of the very same sample are generated and the resulting variability is associated to technical issues during the process of taking measures. Biological replicates represent independant samples and reflect therefore the varibility a given parameter may have in a certain population of individuals.
With the tools presented here, both technical and biological replicates can be dealt with. 
In several cases the interpretation of the resulting numbers should consider the experimental setup, though.

Let's make a simple matrix as toy data:
```{r basicVariability, echo=TRUE}
grp1 <- rep(LETTERS[1:3], c(3,4,3))
sampNa1 <- paste0(grp1, c(1:3,1:4,1:3))
set.seed(2016); dat1 <- matrix(round(c(runif(50000) +rep(1:1000,50)),3), 
  ncol=10, dimnames=list(NULL,sampNa1))
dim(dat1)
head(dat1)
```

Now lets estimate the standard deviation _(sd)_ for every row:
```{r sdForEachRow, echo=TRUE}
head(rowSds(dat1))
system.time(sd1 <- rowSds(dat1))
system.time(sd2 <- apply(dat1, 1, sd))
```

On most systems the equivalent calculation using *apply()* will run much slower compared to `rowSds`.

Note, there is a minor issue with rounding :
```{r usingApply, echo=TRUE}
table(round(sd1, 13)==round(sd2, 13))
```

Similarly we can easily calculate the CV (coefficient of variation, ie sd / mean, see also [CV](https://en.wikipedia.org/wiki/Coefficient_of_variation)) for every row using `rowCVs` :

```{r calculateRowCV, echo=TRUE}
system.time(cv1 <- rowCVs(dat1))
system.time(cv2 <- apply(dat1, 1, sd) / rowMeans(dat1))
# typically the calculation using rowCVs is much faster
head(cv1)
# results from the 'conventional' way
head(cv2)
```

Note, these calculations will be very efficient as long as the number of rows is much higher (>>) than the number of columns.


### Data Organized In (Sub-)Groups As Sets Of Columns

Now, let's assume our data is contains 3 initial samples measured as several replicates (already defined in _grp1_).
Similarly, we can also calculate the sd or CV for each line while splitting into groups of replicates (functions `rowGrpMeans`, `rowGrpSds` and `rowGrpCV`): 

```{r rowGrpMeans1, echo=TRUE}
# we already defined the grouping :
grp1

## the mean for each group and row
system.time(mean1Gr <- rowGrpMeans(dat1, grp1))
```

```{r sdOrCVbyGrp, echo=TRUE}
## Now the sd for each row and group
system.time(sd1Gr <- rowGrpSds(dat1, grp1))
# will give us a matrix with the sd for each group & line 
head(sd1Gr)

# Let's check the results of the first line :
sd1Gr[1,] == c(sd(dat1[1,1:3]), sd(dat1[1,4:7]), sd(dat1[1,8:10]))

# The CV :
system.time(cv1Gr <- rowGrpCV(dat1, grp1))
head(cv1Gr)
```


#### Counting Number Of NAs Per Row And Group Of Columns

Some data, like with quantitative proteomics measures, may contain an elevated number of _NAs_ (see also the package [wrProteo](https://CRAN.R-project.org/package=wrProteo) for further options for dealing with such data). 
Furthermore, many other packages on CRAN and Bioconductor cover this topic, see also the [missing data task-view](https://CRAN.R-project.org/view=MissingData) on CRAN.
Similar as above there is an easy way to count the number of _NAs_ to get an overview how NAs are distributed. 

Let's assume we have measures for 200 features (eg gene-abundaces) from 8 groups of samples :
```{r rowGrpNA1, echo=TRUE}
mat2 <- matrix(1, nrow=200, ncol=40)
set.seed(2026); mat2[sample.int(prod(dim(mat2)), nrow(mat2), replace=FALSE)] <- NA
set.seed(2025); grp2 <- as.factor(sample.int(n=8, size=ncol(mat2), replace=TRUE))
  table(grp2)
```

Now we can easily count the number of NAs per row and set of replicates.
```{r rowGrpNA2, echo=TRUE}
## number of NAs per row and group
head(rowGrpNA(mat2, grp2))
```

It is also possible to return the ratio of NA values or both the number of _NAs_ and the size of the sub-groups.
```{r rowGrpNA3, echo=TRUE}
head(rowGrpNA(mat2, grp2, mode="ratio"))
head(rowGrpNA(mat2, grp2, mode="complete"))
```

This function allows also treating objects created by testing from package [wrProteo](https://CRAN.R-project.org/package=wrProteo), the group-information is typically already available in such objects.
Finally, all results may get collapsed in single column/vector when using any of the _..Collapse_ modes.
```{r rowGrpNA4, echo=TRUE}
## mimick output from testing from package wrProteo
dat1 <- list(isNA =mat2, setup=list(grp=grp2))
head(rowGrpNA(dat1, mode="simple"))
head(rowGrpNA(dat1, mode="complete"))
head(rowGrpNA(dat1, mode="ratioCollapse"))
```



### Fast NA-omit For Very Large Objects

The function _na.omit()_ from the package _stats_ also keeps a trace of all omitted instances. 
This can be penalizing in terms of memory usage when handling very large vectors with a high content of NAs (eg >10000 NAs). 
If you don't need to document precisely which elements got eliminated, the function `naOmit()` may offer 
smoother functioning for very large objects. 

```{r naOmit, echo=TRUE}
aA <- c(11:13,NA,10,NA)
 
str(naOmit(aA))

# the 'classical' na.omit also stores which elements were NA
str(na.omit(aA))
```

  
### Minimum Distance/Difference Between Values

If you need to find the closest neighbour(s) of a numeric vector, the function `minDiff()` will tell you the 
distance ("dif","ppm" or "ratio") and index ("best") of the closest neighbour. 
In case of multiple shortest distances the index if the first one is reported, and the column "nbest" will display a value of >1.

```{r minDiff, echo=TRUE}
set.seed(2017); aa <- 10 *c(0.1 +round(runif(20),2), 0.53, 0.53)
head(aa)

minDiff(aa,ppm=FALSE)
```

When you look at the first line, the value of 10.2 has one single closest value which is 10.4,
which is located in line number 19 (the column 'best' gives the index of the best).
Line number 19 points back to line number 1. 
You can see, that some elements (like 5.7) occure multiple times (line no 3 and 9), multiple occurences are counted in the column _ncur_.
This is why column _nbest_ for line 15 (_value_ =6.0) indicates that it appears twice as closest value _nbest_.



## Working With Lists (And Lists Of Lists)   {#WorkingWithLists}

### Partial unlist
When input from different places gets collected and combined into a list, this may give a collection of different types of data.
The function `partUnlist()` will to preserve multi-column elements as they are (and just bring down one level):
```{r partUnlist_1, echo=TRUE}
bb <- list(fa=gl(2,2), ve=31:33, L2=matrix(21:28,ncol=2), li=list(li1=11:14,li2=data.frame(41:44)))
partUnlist(bb)
partUnlist(lapply(bb,.asDF2))
```

This won't be possible using _unlist()_.
```{r unlist_1, echo=TRUE}
head(unlist(bb, recursive=FALSE))
```

To uniform such data to obtain a list with one column only for each list-element, the function `asSepList()` provides help : 
```{r asSepList, echo=TRUE}
bb <- list(fa=gl(2,2), ve=31:33, L2=matrix(21:28,ncol=2), li=list(li1=11:14,li2=data.frame(41:44)))
asSepList(bb)
```

### Appending/Combining Lists

Separate lists may be combined using the _append()_ command, which also allows treating simple vectors.

```{r lappend1, echo=TRUE}
li1 <- list(a=1, b=2, c=3)
li2 <- list(A=11, b=2, C=13)
append(li1, li2)
```

However, this way there is no checking if some of the list-elements are present in both lists and thus will appear twice.
The function `appendNR()` allows to checking if some list-elements will appear twice, and thus avoid such duplicate entries.


```{r lappend2, echo=TRUE}
appendNR(li1, li2)
```


### rbind On Lists

When a matrix (or data.frame) gets split into a list, like in the example using _by()_, as a reverse-function such lists can get joined using `lrbind()` in an _rbind_-like fashion.

```{r lrbind, echo=TRUE}
dat2 <- matrix(11:34, ncol=3, dimnames=list(letters[1:8], colnames=LETTERS[1:3]))
lst2 <- by(dat2, rep(1:3,c(3,2,3)), as.matrix)
lst2

# join list-elements (back) into single matrix
lrbind(lst2)
```


#### cbind On Lists

In order to combine list-elements into a data.frame (or matrix), base R can do the job - as long as there is no problem with the number of rows.

```{r lrbind2, echo=TRUE}
li2 <- list(a=11:13, b=matrix(21:26, nc=2, dimnames=list(letters[11:13],c("B","C"))), data.frame(d=41:43, e=51:53) )

## cbind like (to data.frame) 
as.data.frame(li2) 

## cbind like and convert into matrix (as long as all data are of same type, ie here all numeric)
as.matrix(as.data.frame(li2)) 
```


### Merge Multiple Matrices From List

When combining different datasets the function  `mergeMatrixList()` allows merging multiple matrices (or data.frames) into a single matrix.
Two types of mode of operation are available : i) Returning only the common/shared elements (as defined by the rownames), this is default _mode='intersect'_ ;
alternatively one may ii) fuse/merge all matrices together without any loss of data (using _mode='union'_, additional _NA_s may appear when a given rowname is absent in one of the input matrices).

Furthermore, one may specifically select which columns should be used for fusing using the argument _useColumn_.


```{r mergeMatrixList, echo=TRUE}
mat1 <- matrix(11:18, ncol=2, dimnames=list(letters[3:6],LETTERS[1:2]))
mat2 <- matrix(21:28, ncol=2, dimnames=list(letters[2:5],LETTERS[3:4]))
mat3 <- matrix(31:38, ncol=2, dimnames=list(letters[c(1,3:4,3)],LETTERS[4:5]))
#
mergeMatrixList(list(mat1, mat2), useColumn="all")

# with custom names for the individual matrices
mergeMatrixList(list(m1=mat1, m2=mat2, mat3), mode="union", useColumn=2)
```

Similarly, separate entries may be merged using  `mergeMatrices()` :

```{r mergeMatrices, echo=TRUE}
mergeMatrices(mat1, mat2)

mergeMatrices(mat1, mat2, mat3, mode="union", useColumn=2)

## custom names for matrix-origin
mergeMatrices(m1=mat1, m2=mat2, mat3, mode="union", useColumn=2)

## flexible/custom selection of columns
mergeMatrices(m1=mat1, m2=mat2, mat3, mode="union", useColumn=list(1,1:2,2))
```



### Fuse Content Of List-Elements With Redundant (Duplicated) Names

When list-elements have the same name, their content (of named numeric or character vectors) 
may get fused using `fuseCommonListElem()` according to the names of the list-elements :

```{r fuseCommonListElem, echo=TRUE}
val1 <- 10 +1:26
names(val1) <- letters
(lst1 <- list(c=val1[3:6], a=val1[1:3], b=val1[2:3] ,a=val1[12], c=val1[13]))

## here the names 'a' and 'c' appear twice :
names(lst1)

## now, let's fuse all 'a' and 'c'
fuseCommonListElem(lst1)
```


### Filtering Lines And/Or Columns For All List-Elements Of Same Size

In a number of cases the information in various list-elements is somehow related.
Eg, in S3-objects produced by [limma](https://bioconductor.org/packages/release/bioc/html/limma.html), or data produced using [wrProteo](https://CRAN.R-project.org/package=wrProteo) several instances of matrix or data.frame refer to data that are related. 
Some matrixes may conatain abundance data (or weights, etc) while another matrix or data.frame may contain the annotation information related to each line of the abundance data.
So if one wants to filter the data, ie remove some lines, this should be done in the same way with all related list-elements. 
This way one may maintain a conventient 1:1 matching of lines.

The function `filterLiColDeList()` searches if other list-elements have suitable dimensions and will then run the same filtering as in the 'target' list-element.
In consequence this can be used with the output of wrProteo to remove simultaneously the same lines and/or columns.

```{r listBatchReplace1, echo=TRUE}
lst1 <- list(m1=matrix(11:18, ncol=2), m2=matrix(21:30, ncol=2), indR=31:34,
  m3=matrix(c(21:23,NA,25:27,NA), ncol=2))
filterLiColDeList(lst1, useLines=2:3)
filterLiColDeList(lst1, useLines="allNA", ref=3)
```


### Replacements In List

The function `listBatchReplace()` works similar to _sub()_ and allows to search & replace exact matches to a character string along all elements of a list. 

```{r replInList1, echo=TRUE}
(lst1 <- list(aa=1:4, bb=c("abc","efg","abhh","effge"), cc=c("abdc","efg","efgh")))
listBatchReplace(lst1, search="efg", repl="EFG", silent=FALSE)
```


### Organize Values Into List And Sort By Names 

Named numeric or character vectors can be organized into lists using `listGroupsByNames()`, 
based on their names (only the part before any extensions starting with a point gets considered).
Of course, other separators may be defined using the argument _sep_. 

```{r listGroupsByNames, echo=TRUE}
ser1 <- 1:7; names(ser1) <- c("AA","BB","AA.1","CC","AA.b","BB.e","A")

listGroupsByNames(ser1)
```
If no names are present, the content of the vector itself will be used as name :
```{r listGroupsByNames2, echo=TRUE}
listGroupsByNames((1:10)/5)
```

### Batch-filter List-Elements

In the view of object-oriented programming several methods produce results integrated into lists or S3-objects (eg 
[limma](https://bioconductor.org/packages/release/bioc/html/limma.html)).
The function `filterList()` aims facilitating the filtering of all elements of lists or S3-objects. 
List-elements with inappropriate number of lines will be ignored.

```{r filterList, echo=TRUE}
set.seed(2020); dat1 <- round(runif(80),2)
list1 <- list(m1=matrix(dat1[1:40], ncol=8), m2=matrix(dat1[41:80], ncol=8), other=letters[1:8])
rownames(list1$m1) <- rownames(list1$m2) <- paste0("line",1:5)
# Note: the list-element list1$other has a length different to that of filt. Thus, it won't get filtered.
filterList(list1, list1$m1[,1] >0.4)       # filter according to 1st column of $m1 ...
filterList(list1, list1$m1 >0.4) 
```

### Transform Columns Of Matrix To List Of Vectors

At some occasions it may be useful separate columns of a matrix into separate vectors inside a list.
This can be done using `matr2list()`:

```{r matr2list, echo=TRUE}
(mat1 <- matrix(1:12, ncol=3, dimnames=list(letters[1:4],LETTERS[1:3])))
str(matr2list(mat1))
```




## Working With Arrays  {#WorkingWithArrays}

Let's get stared with a little toy-array:
```{r array0, echo=TRUE}
(arr1 <- array(c(6:4,4:24), dim=c(4,3,2), dimnames=list(c(LETTERS[1:4]),
  paste("col",1:3,sep=""),c("ch1","ch2"))))
```

### CV (Coefficient Of Variance) With Arrays

Now we can obtain the CV (coefficient of variance) by splitting along 3rd dimesion (ie this is equivalent to an _apply_ along the 3rd dimension) using `arrayCV()`:

```{r arrayCV1, echo=TRUE}
arrayCV(arr1)

# this is equivalent to
cbind(rowCVs(arr1[,,1]), rowCVs(arr1[,,2]))
```
  
Similarly we can split along any other dimension, eg the 2nd dimension : 
```{r arrayCV2, echo=TRUE}
arrayCV(arr1, byDim=2)
```

### Slice 3-dim Array In List Of Matrixes (Or Arrays)

This procedure is similar to (re-)organizing an initial array into clusters, here we split along a user-defined factor/vector.
If a clustering-algorithm produces the cluster assignments, this function can be used to organize the input data accordingly using `cutArrayInCluLike()`.

```{r cutArrayInCluLike, echo=TRUE}
cutArrayInCluLike(arr1, cluOrg=c(2,1,2,1))
```

Let's cut by filtering along the 3rd dimension for all lines where column 'col2' is >7, and then display only the content of columns 'col1' and 'col2' (using `filt3dimArr()`):

```{r filt3dimArr, echo=TRUE}
filt3dimArr(arr1, displCrit=c("col1","col2"), filtCrit="col2", filtVal=7, filtTy=">")
```



## Working With Redundant Data  {#WorkingWithRedundantData}

$_Semantics_$ : Please note, that there are two ways of interpreting the term '**unique**' :

* In regular understanding one describes this way an event which occurs only once, and thus does not occur/happen anywhere else.

* The command `unique()` will eliminate redundant entries to obtain a shorter 'unique' output vector, ie in the resultant vector all values/content (values) occur only once. 
However, from the result of _unique()_ you cannot tell any more which ones were not unique initially ! 

In some applications (eg proteomics) initial identifiers (IDs) may occur multiple times in the data and we frequently need to identify events/values that occur only once, as the first meaning of '_unique_'. 
This package provides (additional) functions to easily distinguish values occurring just once (ie _unique_) from those occurring multiple times. Furthermore, there are functions to rename/remove/combine replicated elements, eg `correctToUnique()` or `nonAmbiguousNum()`, so that no elements or lines of data get lost.


### Identify What Is Repeated (and Where Repeated Do Occur)

```{r repeated1, echo=TRUE}
## some text toy data
tr <- c("li0","n",NA,NA, rep(c("li2","li3"),2), rep("n",4))
```
The function _table()_ (from the package _base_) is very useful get some insights when working with smaller objects, but may be slow to handle very large objects.
As mentioned, _unique()_ will make everything unique, and afterwards you won't know any more who was unique in the first place !
The function `duplicated()` (also from package base) helps us getting the information who is repeated.

```{r repeated2, echo=TRUE}
table(tr)
unique(tr) 
duplicated(tr, fromLast=FALSE)
```

```{r repeated3, echo=TRUE}
aa <- c(11:16,NA,14:12,NA,14)
names(aa) <- letters[1:length(aa)]
aa
```

`findRepeated()` (from this package) will return the position/index (and content/value) of repeated elements. However, the output in form of a list is not very convenient to the human reader.

```{r findRepeated, echo=TRUE}
findRepeated(aa) 
```


`firstOfRepeated()` tells the index of the first instance of repeated elements, which elements you need to make the vector 'unique', and which elements get stripped off when making unique.
Please note, that _NA_ (no matter if they occure once or more times) are automatically in the part suggested to be removed.
```{r firstOfRepeated, echo=TRUE}
firstOfRepeated(aa)

aa[firstOfRepeated(aa)$indUniq]          # only unique with their names

unique(aa)                               # unique() does not return any names !
```


### Correct Vector To Unique (While Maintaining The Original Vector Length)

If necessary, a counter can be added to non-unique entries, thus no individual values get eliminated and the length and order of the resultant object maintains the same using `correctToUnique()`.

This is of importance when assigning rownames to a data.frame : Assigning redundant values/text as rownames of a data.frame will result in an error !

```{r correctToUnique1, echo=TRUE}
correctToUnique(aa)

correctToUnique(aa, sep=".", NAenum=FALSE)       # keep NAs (ie without transforming to character)
```
You see from the last example above, that this function has an argument for controlling enumerating elements.

### Mark Any Duplicated (ie Ambiguous) Elements by Changing Their Names (and Separate from Unqiue)

First, the truly unique values are reported and then the first occurance of repeated elements is given, _NA_ instances get ignored.
This can be done using `nonAmbiguousNum()` which maintains the length of the initial character vector.

```{r nonAmbiguousNum, echo=TRUE}
unique(aa)                                    # names are lost

nonAmbiguousNum(aa)
nonAmbiguousNum(aa, uniq=FALSE, asLi=TRUE)    # separate in list unique and repeated 
```



### Compare Multiple Vectors And Sort By Number Of Common/Repeated Values/Words

The main aim of the function `sortByNRepeated()` is allowing to compare multiple vectors for common values/words and providing an output sorted by number of repeats.

Suppose 3 persons are asked which cities they wanted to visit. 
Then we would like to make a counting of the most frequently cited cities.
Here we consider individual choices as equally ranked.
By default intra-repeats are eliminated.

```{r sortByNRepeated, echo=TRUE}
cities <- c("Bangkok","London","Paris", "Singapore","New York City", "Istambul","Delhi","Rome","Dubai")
sortByNRepeated(x=cities[c(1:4)], y=cities[c(2:3,5:8)])

## or (unlimited) multiple inputs via list
choices1 <- list(Mary=cities[c(1:4)], Olivia=cities[c(2:3,5:8)], Paul=cities[c(5:3,9,5)])    # Note : Paul cited NYC twice !
table(unlist(choices1))
sortByNRepeated(choices1)
sortByNRepeated(choices1, filterIntraRep=FALSE)  # without correcting multiple citation of NYC by Paul
```


### Combine Multiple Matrixes Where Some Column-Names Are The Same

Here, it is supposed that you want to join 2 or more matrixes describing different properties of the same collection of individuals (as rows).
Common column-names are interpreted that their respective information should be combined (either as average or as sum). 
This can be done using `cbindNR()` :

```{r cbindNR, echo=TRUE}
## First we'll make some toy data :
(ma1 <- matrix(1:6, ncol=3, dimnames=list(1:2,LETTERS[3:1])))
(ma2 <- matrix(11:16, ncol=3, dimnames=list(1:2,LETTERS[3:5])))

## now we can join 2 or more matrixes  
cbindNR(ma1, ma2, summarizeAs="mean")       # average of both columns 'C'
```


### Filter Matrix To Keep Only First Of Repeated Lines

This ressembles to the functioning of _unique()_, but applies to a user-specified column of the matrix.

```{r firstLineOfDat, echo=TRUE}
(mat1 <- matrix(c(1:6, rep(1:3,1:3)), ncol=2, dimnames=list(letters[1:6],LETTERS[1:2])))
```

The function `firstLineOfDat()` allows to access/extract the first line of repeated instances.

```{r firstLineOfDat2, echo=TRUE}
firstLineOfDat(mat1, refCol=2)
```

This function was rather designed for dealing with character input, it allows concatenating all columns and to remove redundant.

```{r firstOfRepLines, echo=TRUE}
mat2 <- matrix(c("e","n","a","n","z","z","n","z","z","b", 
  "","n","c","n","","","n","","","z"), ncol=2)
firstOfRepLines(mat2, out="conc")

# or as index :
firstOfRepLines(mat2)
```


### Filter To Unique Column-Content Of Matrix, Add Counter And Concatenated Information

```{r nonredDataFrame, echo=TRUE}
(df1 <- data.frame(cbind(xA=letters[1:5], xB=c("h","h","f","e","f"), xC=LETTERS[1:5])))
```
The function `nonredDataFrame()` offers to include a counter of redundant instances encountered (for 1st column specified) :

```{r nonredDataFrame2, echo=TRUE}
nonredDataFrame(df1, useCol=c("xB","xC")) 

# without counter or concatenating
df1[which(!duplicated(df1[,2])),]
# or
df1[firstOfRepLines(df1,useCol=2),]
```



### Get First Of Repeated By Column

```{r get1stOfRepeatedByCol, echo=TRUE}
mat2 <- cbind(no=as.character(1:20), seq=sample(LETTERS[1:15], 20, repl=TRUE),
  ty=sample(c("full","Nter","inter"),20,repl=TRUE), ambig=rep(NA,20), seqNa=1:20)
(mat2uniq <- get1stOfRepeatedByCol(mat2, sortBy="seq", sortSupl="ty"))

# the values from column 'seq' are indeed unique
table(mat2uniq[,"seq"])

# This will return all first repeated (may be >1) but without furter sorting 
#  along column 'ty' neither marking in comumn 'ambig').
mat2[which(duplicated(mat2[,2],fromLast=FALSE)),]
```

### Transform (ambigous) Matrix To Non-ambiguous Matrix (In Respect To Given Column)

```{r nonAmbiguousMat, echo=TRUE}
nonAmbiguousMat(mat1,by=2)
```
Here another example, ambiguous will be marked by an '_' :
```{r nonAmbiguousMat2, echo=TRUE}
set.seed(2017); mat3 <- matrix(c(1:100,round(rnorm(200),2)), ncol=3,
  dimnames=list(1:100,LETTERS[1:3]));
head(mat3U <- nonAmbiguousMat(mat3, by="B", na="_", uniqO=FALSE), n=15)
head(get1stOfRepeatedByCol(mat3, sortB="B", sortS="B"))
```

### Combine Replicates From List To Matrix

```{r combineReplFromListToMatr, echo=TRUE}
lst2 <- list(aa_1x=matrix(1:12, nrow=4, byrow=TRUE), ab_2x=matrix(24:13, nrow=4, byrow=TRUE))
combineReplFromListToMatr(lst2)
```


### Combine Redundant Lines From List with (Multiple) Matrix According to Reference

The function `combineRedundLinesInList()` provides help for combining/summarizing lines of numeric data which may be summaried according to reference vector or matrix (part of the same input-list).
Initial data and reference will be aligned based on rownames and the content of reference (or the column specified by \code{refColNa}).
 
```{r combineRedundLinesInListAcRef, echo=TRUE}
x1 <- list(quant=matrix(11:34, ncol=3, dimnames=list(letters[8:1], LETTERS[11:13])), 
  annot=matrix(paste0(LETTERS[c(1:4,6,3:5)],LETTERS[c(1:4,6,3:5)]), ncol=1, 
  dimnames=list(paste(letters[1:8]),"xx")) )
combineRedundLinesInList(lst=x1, refNa="annot", datNa="quant", refColNa="xx")
```



### Non-redundant Lines Of Matrix

```{r nonRedundLines, echo=TRUE}
mat4 <- matrix(rep(c(1,1:3,3,1),2), ncol=2, dimnames=list(letters[1:6],LETTERS[1:2]))
nonRedundLines(mat4)
```


### Filter For Unique Elements /2

```{r filtSizeUniq, echo=TRUE}
# input: c and dd are repeated  :
filtSizeUniq(list(A="a", B=c("b","bb","c"), D=c("dd","d","ddd","c")), filtUn=TRUE, minSi=NULL)

# here a,b,c and dd are repeated  :
filtSizeUniq(list(A="a", B=c("b","bb","c"), D=c("dd","d","ddd","c")), ref=c(letters[c(1:26,1:3)],
  "dd","dd","bb","ddd"), filtUn=TRUE, minSi=NULL)   
```


### Make Non-redundant Matrix 

```{r makeNRedMatr, echo=TRUE}
t3 <- data.frame(ref=rep(11:15,3), tx=letters[1:15],
  matrix(round(runif(30,-3,2),1), nc=2), stringsAsFactors=FALSE)
  
# First we split the data.frame in list  
by(t3,t3[,1], function(x) x)
t(sapply(by(t3,t3[,1],function(x) x), summarizeCols, me="maxAbsOfRef"))
(xt3 <- makeNRedMatr(t3, summ="mean", iniID="ref"))
(xt3 <- makeNRedMatr(t3, summ=unlist(list(X1="maxAbsOfRef")), iniID="ref"))
```


#### Example : Summarize table for longest of transcripts

In the previous example for each subgroup a summarization was calculated.
In other cases you may just want to select a line according to values from a single column.
Suppose you want to select the line corresponding to the longest transcript.


```{r makeNRedMatr2, echo=TRUE}
set.seed(2024)
df2 <- data.frame(transcrID=paste("TrID", 101:124, sep=""), geneID=paste("geID",rep(201:206,each=4)), 
  geneLe=round(runif(24, min=50, max=1500)), a=101:124, b=224:201)
df2 <- df2[-1*c(1,4:7,12:15),]  

(dfLongest <- makeNRedMatr(df2, summ=unlist(list(X1="maxOfRef")), iniID="geneID"))

 summarizeCols(df2[1:2,c(1:5,3)], me="min")  
 summarizeCols(df2[1:2,c(1:5,3)], me="mean") 
 summarizeCols(df2[1:2,c(1:5,3)], me="maxOfRef") 
 summarizeCols(df2[1:6,c(1:5,3)], me="maxOfRef")

(xt3 <- makeNRedMatr(t3, summ=unlist(list(X1="maxOfRef")), iniID=c("ref")))  
```


### Combine/Reduce Redundant Lines Based On Specified Column

```{r combineRedBasedOnCol, echo=TRUE}
matr <- matrix(c(letters[1:6],"h","h","f","e",LETTERS[1:5]), ncol=3,
  dimnames=list(letters[11:15],c("xA","xB","xC")))
combineRedBasedOnCol(matr, colN="xB")
combineRedBasedOnCol(rbind(matr[1,],matr), colN="xB")
```


### Convert Matrix (eg With Redundant) Row-Names To data.frame

```{r convMatr2df, echo=TRUE}
x <- 1
dat1 <- matrix(1:10, ncol=2)
rownames(dat1) <- letters[c(1:3,2,5)]
## as.data.frame(dat1)  ...  would result in an error
convMatr2df(dat1)
convMatr2df(data.frame(a=as.character((1:3)/2), b=LETTERS[1:3], c=1:3))
tmp <- data.frame(a=as.character((1:3)/2), b=LETTERS[1:3], c=1:3, stringsAsFactors=FALSE)
convMatr2df(tmp)
tmp <- data.frame(a=as.character((1:3)/2), b=1:3, stringsAsFactors=FALSE)
convMatr2df(tmp) 
```


### Find And Combine Points Located Very Close In X/Y Space

```{r combineOverlapInfo, echo=TRUE}
set.seed(2013)
datT2 <- matrix(round(rnorm(200)+3,1), ncol=2, dimnames=list(paste("li",1:100,sep=""),
  letters[23:24]))
# (mimick) some short and longer names for each line
inf2 <- cbind(sh=paste(rep(letters[1:4],each=26), rep(letters,4),1:(26*4),sep=""),
  lo=paste(rep(LETTERS[1:4],each=26), rep(LETTERS,4), 1:(26*4), ",", 
  rep(letters[sample.int(26)],4), rep(letters[sample.int(26)],4), sep=""))[1:100,] 
## We'll use this to test :  
head(datT2, n=10)
## let's assign to each pair of x & y values a 'cluster' (column _clu_, the column _combInf_ tells us which lines/indexes are in this cluster)
head(combineOverlapInfo(datT2, disThr=0.03), n=10)
## it is also possible to rather display names (eg gene or protein-names) instead of index values
head(combineOverlapInfo(datT2, suplI=inf2[,2], disThr=0.03), n=10)
```


### Bin And Summarize Values According To Their Names
```{r getValuesByUnique, echo=TRUE}
dat <- 11:19
names(dat) <- letters[c(6:3,2:4,8,3)]
## Here the names are not unique.
## Thus, the values can be binned by their (non-unique) names and a representative values calculated.

## Let's make a 'datUniq' with the mean of each group of values :
datUniq <- round(tapply(dat, names(dat), mean),1)
## now we propagate the mean values to the full vector 
getValuesByUnique(dat, datUniq)
cbind(ini=dat,firstOfRep=getValuesByUnique(dat, datUniq),
  indexUniq=getValuesByUnique(dat, datUniq, asIn=TRUE))
```


### Regrouping Simultaneaously by Two Factors

For example, if you wish to create group-labels considering the eye- and hair-color of a small group students (supposed a sort of controlled vocabulary was used),
the function `combineByEitherFactor()` will help. So basically, this is an empiric segmentation-approach for two categorical variables.
Please note, that with large data-sets and very disperse data this approach will not provide great results.
In the example below we'll attempt to 'cluster' according to columns _nn_ and _qq_, the resultant cluster number can be found in column _grp_.

```{r combineByEitherFactor, echo=TRUE}
nn <- rep(c("a","e","b","c","d","g","f"),c(3,1,2,2,1,2,1))
qq <- rep(c("m","n","p","o","q"),c(2,1,1,4,4))
nq <- cbind(nn,qq)[c(4,2,9,11,6,10,7,3,5,1,12,8),]
## Here we consider 2 columns 'nn' and 'qq' whe trying to regroup common values
##  (eg value 'a' from column 'nn' and value 'o' from 'qq') 
combineByEitherFactor(nq, 1, 2, nBy=FALSE)
```
The argument _nBy_ simply allows adding an additional column with the group/cluster-number.

```{r combineByEitherFactor2, echo=TRUE}
## the same, but including n by group/cluster
combineByEitherFactor(nq, 1, 2, nBy=TRUE)
## Not running further iterations works faster, but you may not reach 'convergence' immediately
combineByEitherFactor(nq,1, 2, nBy=FALSE)
```

```{r combineByEitherFactor3, echo=TRUE}
##  another example
mm <- rep(c("a","b","c","d","e"), c(3,4,2,3,1))
pp <- rep(c("m","n","o","p","q"), c(2,2,2,2,5))
combineByEitherFactor(cbind(mm,pp), 1, 2, con=FALSE, nBy=TRUE)
```


### Batch Replacing Of Values Or Character-Strings

The function `multiCharReplace()` facilitates multiple replacements in a vector, matrix or data.frame.

```{r multiCharReplace1, echo=TRUE}
# replace character content
x1 <- c("ab","bc","cd","efg","ghj")
multiCharReplace(x1, cbind(old=c("bc","efg"), new=c("BBCC","EF")))

# works also on matrix and/or to replace numeric content : 
x3 <- matrix(11:16, ncol=2)
multiCharReplace(x3, cbind(12:13,112:113))
```

Sometimes data get imported using different encoding for what should be interpreted as _FALSE_ and _TRUE_ :

```{r multiCharReplace2, echo=TRUE}
# replace and return logical vactor
x2 <- c("High","n/a","High","High","Low")
multiCharReplace(x2,cbind(old=c("n/a","Low","High"), new=c(NA,FALSE,TRUE)), convTo="logical")
```


### Multi-to-multi Matching Of (Concatenated) Terms

The function allows to split (if necessary, using _strsplit()_) two vectors and compare each isolated tag (eg identifyer) from the 1st vector/object against each isolated tag from the second vector/object. This runs like a loop of one to many comparisons. The basic output is a list with indexes of which element of the 1st vector/object has matches in the 2nd vector/object. Since this is not convenient to the human reader, tabular output can be created, too.

```{r multiMatch1, echo=TRUE}
aa <- c("m","k","j; aa","m; aa; bb; o","n; dd","aa","cc")
bb <- c("aa","dd","aa; bb; q","p; cc") 
## result as list of indexes
(bOnA <- multiMatch(aa, bb, method="asIndex"))   # match bb on aa
## more convenient to the human reader
(bOnA <- multiMatch(aa, bb))                     # match bb on aa
(bOnA <- multiMatch(aa, bb, method="matchedL"))  # match bb on aa
```


### Comparing Global Patterns        {#ComparingGlobalPatterns}

In most programming languages it is fairly easy to compare _exact_ content of character vectors or factors with unordered levels.
However, sometimes - due to semantic issues - some people may call a color 'purple' while others call it 'violet'.
Thus, without using controled vocabulary the _exact_ terms may vary.

Here, let's address the case, where no dictionaries of controled vocabulary are available for substituting equivalent terms.
Thus, we'll compare 4 vectors of equal length and check if the words/letters used could be substituted to result in the first vector.
Vectors _aa_ and _ab_ have the same global pattern, ie after repeating a word twice it moves to another word.
Vectors _ac_ and _ad_ have different general patterns, either with alternating words or falling back to a word previsously used.

Based and extended on a post on stackoverflow [https://stackoverflow.com/questions/71353218/extracting-flexible-general-patterns/](https://stackoverflow.com/questions/71353218/extracting-flexible-general-patterns/) :


```{r compGlobPat1, echo=TRUE}
aa <- letters[rep(c(3:1,4), each=2)]
ab <- letters[rep(c(5,8:6), each=2)]        # 'same general' pattern to aa
ac <- letters[c(1:2,1:3,3:4,4)]             # NOT 'same general' pattern to any other
ad <- letters[c(6:8,8:6,7:6)]               # NOT 'same general' pattern to any other
```

The basic pattern can be extracted combining match() and unique():
```{r compGlobPat2, echo=TRUE}
## get global patterns
cbind(aa= match(aa, unique(aa)),
  ab= match(ab, unique(ab)),
  ac= match(ac, unique(ac)),
  ad= match(ad, unique(ad)) )
```

Let's make a data.frame with the annotation toy-data from above. 
Each line is supposed to represent a sample, and the columns show different aspects of annotation.

```{r compGlobPat3, echo=TRUE}
bb <- data.frame(ind=1:length(aa), a=aa, b=ab, c=ac, d=ad)
```
Via the function `replicateStructure()` is it possible to compare annotation as different columns for equivalent global patterns.

By default, this function excludes all columns not designating any replicates, like the numbers in the first column ($ind).
Also it will try to find the column with the median number of levels, when comparing to all other columns.

The output is a list with  *\$col* inidicating which column(s) may be used, *\$lev* for the correpsonding global pattern, *\$meth* for the method finally used and 
_\$allCols_ for documenting the global pattern in each column (whether it was selected or not).


```{r compGlobPat4, echo=TRUE}
replicateStructure(bb)
```

Besides, it is also possible to combine all columns if one considers they contribute complementary substructures of the overal annotation.

```{r compGlobPat5, echo=TRUE}
replicateStructure(bb, method="combAll")
```
However, when combining multiple columns it may happen -like in the example above- that finally no more lines remain being considered as replicates.

This can also be found when one column describes the groups and another gives the order of the replicates therein.
However, for calling a (standard) statistical test it may be necessary exclude these replicate-numbers to designate the groups of replicates.

To overcome the problem of loosing the understanding of replicate-structure when combining all factors, it is possible to look for non-orthogonal structures,
ie to try excluding columns which (after combining) would suggest no replicates after combining all columns.
See the example below :

```{r compGlobPat6, echo=TRUE}
replicateStructure(bb, method="combNonOrth")
```

Please see also the function _chooseGroupNames()_ which may help choosing among different types of patterns (ie low or high number of groups of replicates) (see section [Working With Redundant Data] (#WorkingWithRedundantData)).



## Search For Similar (Numeric) Values  {#SearchForSimilarNumericValues}

This section addresses values that are not truly _identical_ but may differ only in the very last digit(s) 
and thus may be in a pragmatic view get considered and treated as 'about the same'.
The simplest approach would be to round values and then look for identical values. 
The functions presented here (like `checkSimValueInSer()`) offer this type of search in a convenient way.

Of course the user must define a threshold for how similar may retained as positive (in the the logical vector returned).
With the function _checkSimValueInSer()_ this threshod must be given as [ppm](https://simple.wikipedia.org/wiki/Parts_per_million) (parts per million).

```{r checkSimValueInSer, echo=TRUE}
va1 <- c(4:7,7,7,7,7,8:10) + (1:11)/28600
checkSimValueInSer(va1, ppm=5)
data.frame(va=sort(va1), simil=checkSimValueInSer(va1))
```


### Find Similar Numeric Values Of Two Columns Of A Matrix

The search for similar values may be preformed as absolute distance or as 'ppm' (as it is eg usual in proteomics when comparing measured and theoretically expected mass).

```{r findCloseMatch1, echo=TRUE}
aA <- c(11:17); bB <- c(12.001,13.999); cC <- c(16.2,8,9,12.5,15.9,13.5,15.7,14.1,5)
(cloMa <- findCloseMatch(x=aA, y=cC, com="diff", lim=0.5, sor=FALSE))       
```
The result of _findCloseMatch()_ is a list organized by each 'x', telling all instances of 'y' found within the distance tolerance given by _lim_.
Using `closeMatchMatrix()` the result obtained above, can be presented in a more convenient format for the human eye.

```{r closeMatchMatrix2, echo=TRUE}
# all matches (of 2d arg) to/within limit for each of 1st arg ('x'); 'y' ..to 2nd arg = cC
# first let's display only one single closest/best hit
(maAa <- closeMatchMatrix(cloMa, aA, cC, lim=TRUE))  #
```

Using the argument _limitToBest=FALSE_ we can display all distances within the limits imposed, some values/points may occur multiple times.
For example, value number 4 of 'cC' (=12.5) or value number 3 of 'aA' (=13) now occur multiple times...

```{r closeMatchMatrix3, echo=TRUE}
(maAa <- closeMatchMatrix(cloMa, aA, cC, lim=FALSE,origN=TRUE))  #
(maAa <- closeMatchMatrix(cloMa, cbind(valA=81:87, aA), cbind(valC=91:99, cC), colM=2,
  colP=2, lim=FALSE))
(maAa <- closeMatchMatrix(cloMa, cbind(aA,valA=81:87), cC, lim=FALSE))  #
a2 <- aA; names(a2) <- letters[1:length(a2)];  c2 <- cC; names(c2) <- letters[10 +1:length(c2)]
(cloM2 <- findCloseMatch(x=a2, y=c2, com="diff", lim=0.5, sor=FALSE)) 
(maA2 <- closeMatchMatrix(cloM2, predM=cbind(valA=81:87, a2),
  measM=cbind(valC=91:99, c2), colM=2, colP=2, lim=FALSE, asData=TRUE))
(maA2 <- closeMatchMatrix(cloM2, cbind(id=names(a2), valA=81:87,a2), cbind(id=names(c2),
  valC=91:99,c2), colM=3, colP=3, lim=FALSE)) 
```


### Find Similar Numeric Values From Two Vectors/Matrixes

For comparing two sets of data one may use `findSimilarFrom2sets()`.

```{r findSimilFrom2sets, echo=TRUE}
aA <- c(11:17); bB <- c(12.001,13.999); cC <- c(16.2,8,9,12.5,12.6,15.9,14.1)
aZ <-  matrix(c(aA,aA+20), ncol=2, dimnames=list(letters[1:length(aA)],c("aaA","aZ")))
cZ <-  matrix(c(cC,cC+20), ncol=2, dimnames=list(letters[1:length(cC)],c("ccC","cZ")))
findCloseMatch(cC, aA, com="diff", lim=0.5, sor=FALSE)
findSimilFrom2sets(aA, cC)
findSimilFrom2sets(cC, aA)
findSimilFrom2sets(aA, cC, best=FALSE)
findSimilFrom2sets(aA, cC, comp="ppm", lim=5e4)
findSimilFrom2sets(aA, cC, comp="ppm", lim=9e4, bestO=FALSE)
# below: find fewer 'best matches' since search window larger (ie more good hits compete !)
findSimilFrom2sets(aA, cC, comp="ppm", lim=9e4, bestO=TRUE)
```


### Fuse Previously Identified Pairs To 'Clusters'

When you have already identified the closest neighbour of a set of values, you may want to 
re-organize/fuse such pairs to a given number of total clusters (using `fusePairs()`).

```{r fusePairs, echo=TRUE}
(daPa <- matrix(c(1:5,8,2:6,9), ncol=2))
fusePairs(daPa, maxFuse=4)
```

### Eliminate Close (Overlapping) Points (In Bivariate x & y Space)

When visualizing larger data-sets in an x&y space one may find many points overlapping when their values are almost the same.  
The function `elimCloseCoord()` aims to do reduce a bivariate data-set to 'non-overlapping' points, somehow similar to human perception.

```{r elimCloseCoord1, echo=TRUE}
da1 <- matrix(c(rep(0:4,5),0.01,1.1,2.04,3.07,4.5), ncol=2); da1[,1] <- da1[,1]*99; head(da1)
elimCloseCoord(da1)
```


### Mode Of (Continuous) Data

Looking for the _mode_ is rather easy with counting data, the result of _table()_ will get you there quickly.
However, with continuous data the mode may be more tricky to defne and identify. 
Intuitively most people consider the mode asthe peak of a density estimation (which remains to be defined and estimated).
With continuous data most frequent (precise) value may be quite different/distant to the most dense region of data.
The function `stableMode()` presented here has different modes of operation, at this point there is no clear rule which mode may perform most satisfactory in different situations. 

```{r stableMode, echo=TRUE}
set.seed(2012); dat <- round(c(rnorm(120,0,1.2), rnorm(80,0.8,0.6), rnorm(25,-0.6,0.05), runif(200)),3)
dat <- dat[which(dat > -2 & dat <2)]
stableMode(dat)
```

Now we can try to show on a plot :
```{r stableMode2, fig.height=8, fig.width=9, fig.align="center",  echo=TRUE}
layout(1:2)
plot(1:length(dat), sort(dat), type="l", main="Sorted Values", xlab="rank", las=1)
abline(h=stableMode(dat, silent=TRUE), lty=2,col=2)
legend("topleft",c("stableMode"), text.col=2, col=2, lty=2, lwd=1, seg.len=1.2, cex=0.8, xjust=0, yjust=0.5) 


plot(density(dat, kernel="gaussian", adjust=0.7), xlab="Value of dat", main="Density Estimate Plot")
useCol <- c("red","green","blue","grey55")
legend("topleft",c("dens","binning","BBmisc","allModes"), text.col=useCol, col=useCol,
  lty=2, lwd=1, seg.len=1.2, cex=0.8, xjust=0, yjust=0.5) 
abline(v=stableMode(dat, method="dens", silent=TRUE), lty=2, col="red", lwd=2)
abline(v=stableMode(dat, method="binning", silent=TRUE), lty=2, col="green")
abline(v=stableMode(dat, method="BBmisc", silent=TRUE), lty=2, col="blue")  
abline(v=stableMode(dat, method="allModes"), lty=2, col="grey55")  
```

Please note, that plotting data modelled via a Kernell function (as above) also relies on strong hypothesis which may not be well justified in a number of cases !
For this reason, the _sorted values_ were plotted, too.

As you can see from this example above, looking for the most frequent exact value may not be a perfect choice for continous data. 
In this example the method _'allModes'_ (ie the multiple instances of most frequent exact values) gave partially usable results (dashed grey lines), due to the rounding to 3 digits. 
As you can see in the example above, the method _'allModes'_ may give multiple ties !
More rounding will make to data more discrete and ultimately ressemble cunting data. However, with rounding some of the finer resolution/details will get lost. 



### Most Frequently Occuring Value  (traditional mode)

The function  `stableMode()` can also be used to locate _the_ most frequently occuring exact value of numeric or character vectors.
As we just saw at the end of the previous example, the argument _method="allModes"_ allows finding all ties (if present).

```{r stableMode3, echo=TRUE}
set.seed(2021)
x <- sample(letters, 50000, replace=TRUE)
stableMode(dat, method="mode")
stableMode(dat, method="allModes")
```



## Text-Manipulations    {#Text-Manipulations}

There are several packages offering interesting functions for manipulating text. Here are a few functions to complement these.

### Protect Special Characters
The function `protectSpecChar()` allows protecting the majority of special characters which may influence the outcome of _grep()_ and related functions so thay they can be used easier in searches using _grep()_ or alike.

```{r protectSpecChar1, echo=TRUE}
aa <- c("abc","abcde","ab.c","ab.c.e","ab*c","ab\\d")
grepl("b.", aa)             # all TRUE
grepl(protectSpecChar("b."), aa)
```


### Trimming Redundant Text

Automatic annotation has the tendency to concatenate many parameters into a single names.
The function `trimRedundText()` was designed to allow trimming redundant text from left and/or right side of a character-vector 
(when the same portion of text appears in _each_ element). 
However, as in some cases (like the first element of the example below) nothing would remain, it is possible to define a _minimum_ width for the remaining/resulting text.

```{r trimRedundText1, echo=TRUE}
txt1 <- c("abcd","abcde","abcdefg","abcdE",NA,"abcdEF")
trimRedundText(txt1)
```


### Removing Redundant/Shared Words

A more gentle way to make text non-redundant consists in removing redundant text isporposed by the function `rmSharedWords()`.
Multiple separators may be used (and combined) to specify 'words', a minimum length of what mat be considered a word can be defined, too (default at min 2 characters).
For example, this can be used to make column-names shorter and more precise.

```{r rmSharedWords1, echo=TRUE}
txt2 <- c("abc d","abc de","abc defg","abc dE",NA,"abc dEF")
rmSharedWords(txt2)
```

In contrast to trimRedundText() numeric parts may remain 'intact', see example below.
```{r rmSharedWords2, echo=TRUE}
txt3 <- c("ab 100 m","ab 120 m","ab 1000 m")
trimRedundText(txt3)
rmSharedWords(txt3)
```


### Extract Common Part Of Text

The original idea was to do something resembling the inverse process of trimming redundant text (example above), but this time to discard the variable text.

In the end this is not as trivial when 'common' or 'redundant' text is not at the beginning or end of a chain of characters. 
In particular with very large text this is an active field of research (eg for sequence alignment). 
The function presented here is a very light-weight solution designed for smaller and simple settings, like inspecting column-names.
Furthermore, the function `keepCommonText()` only reports the first (longest) hit. 
So, when there are multiple conserved 'words' of equal length, only the first of them will be identified.

When setting the argument 'hiResol=FALSE' this function has an option to decrease the resulution of searching, which in turn increases the speed, howevere, at cost of missing the optimal solution.
In this case the resultant chain of characters should be inspected if it can be further extended/optimized.

With terminal common text :
```{r keepCommonText1, echo=TRUE}
txt1 <- c("abcd","abcde","abcdefg","abcdE",NA,"abcdEF")
trimRedundText(txt1, side="left")         # remove redundant 
keepCommonText(txt1, side="terminal")     # keep redundant
keepCommonText(txt1, side="center")       # computationally easier   
```

With internal coomon text:
```{r keepCommonText2, echo=TRUE}
txt2 <- c("abcd_abc_kjh", "bcd_abc123", "cd_abc_po")
keepCommonText(txt2, side="center")       
```


### Manipulating Enumerator-Extensions

Human operators may have many ways to write enumerators like 'xx_sample_1', 'xx_Sample_2', 'xx_s3', 'xx_4', etc.
Many times you may find such text as names or column-names for measures underneith.

The functions presented below will work only if _consistent numerators_, ie (text +) digit-character(s) are at the end of all character-strings to be treated.

Please note, that with large vectors testing/checking a larger panel of enumerator-abreviations may result in slower performance.
In cases of such larger data-sets it may be more effective to first study the data and then run simple subsitions using _sub()_ targeted for this very case.


#### Remove/Modify Enumerators

The aim of this function consists in identifying a _common_ pattern for terminal enumeratos (ie at end of words/character strings) and to subsequently modify or remove them.
As separator-symbols and separator-words are given indedently all combinations thereof may be tested.
Furthermore the user has the choice to (automatically) all truncated versions of separator-words (eg _Sam_ instead of _Sample_).

As basic setting `rmEnumeratorName()` allows to identify and then modify a _common_ terminal enumerator from all elements of a character string :
```{r rmEnumeratorName1, echo=TRUE}
xx <- c("hg_Re1","hjRe2_Re2","hk-Re3_Re33")
rmEnumeratorName(xx)
rmEnumeratorName(xx, newSep="--")
rmEnumeratorName(xx, incl="anyCase")
```

Furthermore, this function allows scanning a matrix of text-data and to perform similar operations to the _first_ column found containing a  _common_ terminal enumerator.

```{r rmEnumeratorName2, echo=TRUE}
xy <- cbind(a=11:13, b=c("11#11","2_No2","333_samp333"), c=xx)
rmEnumeratorName(xy)
rmEnumeratorName(xy,incl=c("anyCase","trim2","rmEnumL"))
```

If you which to remove/subsitute mutiple types of enumerators the function \code{rmEnumeratorName} must be run independently, see last example below.

```{r rmEnumeratorName3, echo=TRUE}
xz <- cbind(a=11:13, b=c("23#11","4#2","567#333"), c=xx)
apply(xz, 2, rmEnumeratorName, sepEnum=c("","_"), newSep="_", silent=TRUE)
```


#### Unify Enumerators

The (slightly older) function `unifyEnumerator()` offers less options, in particular the potential separator-words must be given explicitly, only lower/upper-case may be kept flexible.

```{r unifyEnumerator1, echo=TRUE}
unifyEnumerator(c("ab-1","ab-2","c-3"))
unifyEnumerator(c("ab-R1","ab-R2","c-R3"))
unifyEnumerator(c("ab-1","c3-2","dR3"), stringentMatch=FALSE)
```


### Find/Choose Column Most Likely Containing Sample-Names

When information about a collection of samples is given in a table, the function  `unifyEnumerator()` was designed to find the column most likely containg sample-names 
and it derives then the structure of groups of replicates by removing terminal enumerators from sample-names.
Initial tables (as frequently found in transcriptomics and proteomics) may not have consistent schemes of naming columns (due to human annotators not using controlled vocabulary) 
and may contain additional columns with redundant information (elements/conditions that were kept constant for all samples.    

Internally this function uses _rmSharedWords()_ to further simplify resultant names.

```{r chooseGroupNames1, echo=TRUE}
(mat <- cbind(a=letters[1:6], b=paste(rep(c("b","B"), each=3), 1:3), c=rep(1,6), 
  d=gl(3,2), e=rep(c("e","E"),3), f=paste(rep(c("F","f","ff"), each=2), 1:2)))
  
chooseGroupNames(mat, method="median")         # col 2 (b/B)
chooseGroupNames(mat, method="median", fullReport=TRUE) 
chooseGroupNames(mat, method="min")            # col 2 (b/B)
chooseGroupNames(mat, method="max")            # col 6 (F/f/ff)
chooseGroupNames(mat, method="max", asUnique=FALSE) # col 1 (a..)
```

Please see also the function _replicateStructure()_ which allows inspecting patterns of replicating elements (see section [Comparing Global Patterns] (#ComparingGlobalPatterns)).


### Find Common Unit

The function `checkUnitPrefix` aims to find a unit-abbreviation or -name occurring in all elements of a character-vector.
The unit name may be preceeded by different decimal prefixes (eg 'k','M', see argument _pref_) which may vary within the vector.
By default some common SI-units will be searched, see argument _unit_.

```{r checkUnitPrefix1, echo=TRUE}
x1 <- c("10fg WW","xx 10fg 3pW"," 1pg 2.0W")
checkUnitPrefix(x1)

## different separators between digit and prefix:
x2 <- c("10fg WW","xx 8_fg 3pW"," 1 pg-2.0W")
checkUnitPrefix(x2, stringentSearch=TRUE)
checkUnitPrefix(x2, stringentSearch=FALSE)
```



### Adjust Decimal Prefixes And Extact Numeric+Unit Part

The function `adjustUnitPrefix()` provides help extracting the numeric part of character vectors and allows adjusting to a single million-unit type.
This can be used to convert a vector of mixed prefixes like  'z','a','f','p','n','u' and 'm' (note: the 'u' is used for 'micro').
The output is a character vector with all prefix+unit expressions adjusted to use the same prefix everywhere (numeric+separator+prefix+unit). 
Redundant additional text may get (optionally) trimmed (see argument _returnType="trim"_), the numeric part names + unit is give in the names.

Please note that decimal/comma digits will not be recognized properly, the function may/will consider the decimal sign as just another separator 
(as '.' is part af the default selection of separators).

```{r adjustUnitPrefix1, echo=TRUE}
adjustUnitPrefix(c("10.psec", "2 fsec"), unit="sec")
```

Using the argument _returnType_ you can choose how much of the remaining text should be shown.
```{r adjustUnitPrefix2, echo=TRUE}
x2 <- c("abCc 500_nmol ABC", "abEe5_umol", "", "abFF_100_nmol_G", "abGg 2_mol", "abH.1 mmol")
rbind( adjustUnitPrefix(x2, unit="mol", returnType="allText") , 
  adjustUnitPrefix(x2, unit="mol", returnType="trim"),
  adjustUnitPrefix(x2, unit="mol", returnType=""))
```

If the _unit_ -name is not knonw in advance, you can try to figure out. In the example it is shown that the unit-name can be determined using the function _checkUnitPrefix()_.
```{r adjustUnitPrefix3, echo=TRUE}
x3 <- c("2.psec abc","300 fsec etc", "34 5fsec")
adjustUnitPrefix(x3, unit=checkUnitPrefix(x3))
```


### Merging Multiple Named Vectors To Matrix

The function `mergeVectors()` allows merging of multiple named vectors (each element needs to be named !).
Basically, all elements carrying the same name across different input-vectors will be aligned in the same column of the output (input-vectors appear as lines).
Different to _merge()_ which allows merging only 2 data.frames, here multiple vectors may be merge at once.

```{r mergeVectors1, echo=TRUE}
x1 <- c(a=1, b=11, c=21)
x2 <- c(b=12, c=22, a=2)
x3 <- c(a=3, d=43)
mergeVectors(vect1=x1, vect2=x2, vect3=x3)
```

```{r mergeVectors2, echo=TRUE}
mergeVectors(vect1=x1, vect2=x2, vect3=x3, inclInfo=TRUE)   # return list with additional info
```

In the example below we'll add another vector, nuty this time _without_ named elements.
In the absence of names there are no anchors and in consequence the last vector cannot get merged to the rest.
You can see a message telling that this vector was ignored/omitted.

```{r mergeVectors3, echo=TRUE}
x4 <- 41:44            # no names - not conform for merging and will be ignored
mergeVectors(x1, x2, x3, x4)
```



### Match All Lines of Matrix To Reference Note

This function allows adjusting the order of lines of a matrix \code{mat} to a reference character-vector \code{ref},
even when initial direct matching of character-strings using \code{match} is not possible/successful.
In this case, various variants of using \code{grep} will be used to see if unambiguous matching is possible of characteristic parts of the text.
All columns of \code{mat} will be tested an the column giving the best results will be used.

```{r matchMatrixLinesToRef1, echo=TRUE}
## Note : columns b and e allow non-ambigous match, not all elements of e are present in a
mat0 <- cbind(a=c("mvvk","axxd","bxxd","vv"),b=c("iwwy","iyyu","kvvh","gxx"), c=rep(9,4),
  d=c("hgf","hgf","vxc","nvnn"), e=c("_vv_","_ww_","_xx_","_yy_"))
matchMatrixLinesToRef(mat0[,1:4], ref=mat0[,5])
matchMatrixLinesToRef(mat0[,1:4], ref=mat0[1:3,5], inclInfo=TRUE)

matchMatrixLinesToRef(mat0[,-2], ref=mat0[,2], inclInfo=TRUE)   # needs 'reverse grep'
```



### Order Matrix According To Reference

The function `orderMatrToRef()` has the aim of facilitating bringing a matrix of text/data in the order of a given reference (character vector).
This function will try all columns of the input-matrix to see which gives the best coverage/ highest number of matches to the reference.
If no hits are found, this function will try by partial matching (using _grep()_) all entries of the reference and vice-versa all entries of the matrix.

```{r orderMatrToRef1, echo=TRUE}
mat1 <- matrix(paste0("__",letters[rep(c(1,1,2,2,3),3) +rep(0:2,each=5)], rep(1:5)), ncol=3)
orderMatrToRef(mat1, paste0(letters[c(3,4,5,3,4)],c(1,3,5,2,4)))

mat2 <- matrix(paste0("__",letters[rep(c(1,1,2,2,3),3) +rep(0:2,each=5)], c(rep(1:5,2),1,1,3:5 )), ncol=3)
orderMatrToRef(mat2, paste0(letters[c(3,4,5,3,4)],c(1,3,5,1,4)))

mat3 <- matrix(paste0(letters[rep(c(1,1,2,2,3),3) +rep(0:2,each=5)], c(rep(1:5,2),1,1,3,3,5 )), ncol=3)
orderMatrToRef(mat3, paste0("__",letters[c(3,4,5,3,4)],c(1,3,5,1,3)))
```


### Value Matching With Option For Concatenated Terms

Sometimes we need to match terms in concatenated tables.
The function `concatMatch()` was designed to behave similar to _match()_  but also allowing to serach among concatenated terms and some further text-simplifications.

```{r concatMatch1, echo=TRUE}
## simple example without concatenations or text-extensions
x0 <- c("ZZ","YY","AA","BB","DD","CC","D")
tab0 <- c("AA","BB,E","CC","FF,U")
match(x0, tab0)
concatMatch(x0, tab0)         # same result as match(), but with names

## now let's construct somthing similar but with concatenations and text-extensions
x1 <- c("ZZ","YY","AA","BB-2","DD","CCdef","Dxy")            # modif of single ID (no concat)
tab1 <- c("AA","WW,Vde,BB-5,E","CCab","FF,Uef")
match(x1, tab1)                   # match finds only the 'simplest' case (ie "AA")
concatMatch(x1, tab1)             # finds all hits as in example above

x2 <- c("ZZ,Z","YY,Y","AA,Z,Y","BB-2","DD","X,CCdef","Dxy")  # conatenated in 'x'
tab2 <- c("AA","WW,Vde,BB-5,E","CCab,WW","FF,UU")
concatMatch(x2, tab2)               # concatenation in both 'x' and 'table'
```



### Check for (Strict) Order

Thi function `checkStrictOrder()` was designed to scan each line of an (numeric) input matrix for up- down- or equal-development, ie the chang to the next value on the right.
For example when working with a matrix of with 4 columns one can look 3 times a the neighbour value following to the right (in the same line), thus the output will mention 3 events (for each line).
If _all counts_ are 'up' and 0 counts are 'down' or 'eq', the line follows a permanently increase (not necessarily linear), etc.

In some automated procedures (where the numer of columns of initial input may vary) it may be easier to test if any 0 occur.
For this reason the argument _invertCount_ was introduced, in this case a line with a '0' occurring characterizes a constant behaviour (for the respective column).

```{r checkStrictOrder1, echo=TRUE}
set.seed(2005); mat1 <- rbind(matrix(round(runif(40),1),nc=4), rep(1,4))
head(mat1)
checkStrictOrder(mat1); mat1[which(checkStrictOrder(mat1)[,2]==0),]
```

A slightly more general way of testing can be done using  `checkGrpOrder()`. Here, simlpy a logical value will produced for each line of input indicating if there is constant behaviour.
When the argument _revRank=TRUE_ (default) constant up- or constant down-characteristics will be tested

```{r checkGrpOrder1, echo=TRUE}
head(mat1)
checkGrpOrder(mat1)
checkGrpOrder(mat1, revRank=FALSE)    # only constant 'up' tested
```




## Working With Regressions    {#WorkingWithRegressions}

### Best Starting Point For Linear Regressions (Start of linearity)

In many types of measurments the very low level measures are delicate.
Especially when the readout starts with a baseline signal before increasing amounts of the analyte start producing a linear relationship.
In such cases some of the very lowest levels of the analyte are masked by the (random) baseline signal.
The function `linModelSelect()` presented here allows omitting some of the lowest analyte measures to focus on the linear part of the dose-response relationship.

```{r linModelSelect1, echo=TRUE}
li1 <- rep(c(4,3,3:6), each=3) + round(runif(18)/5,2)
names(li1) <- paste0(rep(letters[1:5], each=3), rep(1:3,6))
li2 <- rep(c(6,3:7), each=3) + round(runif(18)/5, 2)
dat2 <- rbind(P1=li1, P2=li2)
exp2 <- rep(c(11:16), each=3)
exp4 <- rep(c(3,10,30,100,300,1000), each=3)

## Check & plot for linear model 
linModelSelect("P1", dat2, expect=exp2)
linModelSelect("P2", dat2, expect=exp2)
```

This function was designed for use with rather small data-sets with no (or very few) measures of base-line.
When larger panels of data ara available, it may be better to first define a confidence interval for the base-line measurement
and then only to consider points outside this confidence interval for regressing dose-response relationships
(see also [Detection limit](https://en.wikipedia.org/wiki/Detection_limit)).



### High Throughput Testing For Linear Regressions

Once we have run multiple linear regressions on differt parts of the data we might wat to compare them in a single plot.
Below, we construct 10 series of data that get modeled the same way, ideally one would obtain a slope close to 1.0.
We still allow omitting some starting points, if the resulting model would fit better.

```{r plotLinModelCoef1, echo=TRUE}
set.seed(2020)
x1 <- matrix(rep(c(2,2:5),each=20) + runif(100) +rep(c(0,0.5,2:3,5),20), 
  byrow=FALSE, ncol=10, dimnames=list(LETTERS[1:10],NULL))
## just the 1st regression :
   summary(lm(b~a, data=data.frame(b=x1[,1], a=rep(1:5,each=2))))
## all regressions
x1.lmSum <- t(sapply(lapply(rownames(x1), linModelSelect, dat=x1, 
  expect=rep(1:5,each=2), silent=TRUE, plotGraph=FALSE), 
  function(x) c(x$coef[2,c(4,1)], startFr=x$startLev)))
x1.lmSum <- cbind(x1.lmSum, medQuantity=apply(x1,1,median))
x1.lmSum[,1] <- log10(x1.lmSum[,1])
head(x1.lmSum)
```

Now we can try to plot :
```{r plotLinModelCoef2, echo=TRUE}
wrGraphOK <- requireNamespace("wrGraph", quietly=TRUE)      # check if package is available
if(wrGraphOK) wrGraph::plotW2Leg(x1.lmSum, useCol=c("Pr(>|t|)","Estimate","medQuantity","startFr"), 
  legendloc="topleft", txtLegend="start at")
```



## Combinatorics Issues   {#CombinatoricsIssues}

### All Pairwise Ratios

`ratioAllComb()` calculates all possible pairwise ratios between all individual calues of x and y.

```{r ratioAllComb0, echo=TRUE}
set.seed(2014); ra1 <- c(rnorm(9,2,1), runif(8,1,2))
```

Let's assume there are 2 parts of 'x' for which we would like to know the representative ratio :
The ratio of medians does not well reflect the typical ratio (if each element has the same chance to be picked).

```{r ratioAllComb1, echo=TRUE}
median(ra1[1:9]) / median(ra1[10:17])
```

Instead, we'll build all possible ratios and summarize then.

```{r ratioAllComb2, echo=TRUE}
summary( ratioAllComb(ra1[1:9], ra1[10:17]))
boxplot(list(norm=ra1[1:9], unif=ra1[10:17], rat=ratioAllComb(ra1[1:9],ra1[10:17])))
```


###  Count Frequency Of Terms Combined From Different Drawings (combineAsN)

The main idea of this function is to count frequency of terms when combining different drawings.
Suppose, you are asking students for their prefered hobbies.
Now, you want to know how many terms will occur in common in groups of 3 students.
In the example below, simple letters are shown instead of names of hobbies ...

In the simplest way of using `combineAsN()` does something similar to _table_ :
Here we're looking at the full combinatorics of making groups of _nCombin_ students and let's count the frequency of terms found 3 times identical, 2 times or only once (ie not cited by the others).
In case multiple groups of _nCombin_ students can be formed, the average of the counts, standard error of the mean (sem), 95% confidence interval (CI) and sd aregiven to resume the results.

```{r combineAsN1, echo=TRUE}
tm1 <- list(a1=LETTERS[1:7], a2=LETTERS[3:9], a3=LETTERS[6:10], a4=LETTERS[8:12])
combineAsN(tm1, nCombin=3, lev=gl(1,4))[,1,]
```

One may imagine that different locations/coties/countries will give different results.
Thus, we'll declare the different origins/location using the _lev_ argument.
Now, this function focusses (by default) on combinations of students from _nCombin_ different origins/location and
counts how many hobbies were mentioned as all different ('sing', ie number of hobbies only one student mentioned),
single repeat ('doub') or three times repeated ('trip'), plus minumum twice or 'any' (ie number of hobies citied no matter how many repeats).
The output is an array, the 3rd dimension contains the counts, fllowed by sem, CI and sd.

```{r combineAsN2, echo=TRUE}
## different levels/groups in list-elements
tm4 <- list(a1=LETTERS[1:15], a2=LETTERS[3:16], a3=LETTERS[6:17], a4=LETTERS[8:19],
  b1=LETTERS[5:19], b2=LETTERS[7:20], b3=LETTERS[11:24], b4=LETTERS[13:25], c1=LETTERS[17:26],
  d1=LETTERS[4:12], d2=LETTERS[5:11], d3=LETTERS[6:12], e1=LETTERS[7:10])
te4 <- combineAsN(tm4, nCombin=4, lev=substr(names(tm4),1,1))
str(te4)
te4[,,1]           # the counts part only
```


## Import/Export

### Batch-Reading Of CSV Files

Some software do produce a series of csv files, where a large experiment/data-set get recorded as multiple files.
The function `readCsvBatch()` was designed for reading multiple csv files of exactly the same layout and to join their content.
As output a list with the content of each file can be produced (one matrix per file), or the data may be fused into an array, as shown below.  


```{r readCsvBatch, echo=TRUE}
path1 <- system.file("extdata", package="wrMisc")
fiNa <-  c("pl01_1.csv","pl01_2.csv","pl02_1.csv","pl02_2.csv")
datAll <- readCsvBatch(fiNa, path1, silent=TRUE)
str(datAll)
```

When setting the first argument _fileNames_ to _NULL_, you can read all files of a given path.
```{r readCsvBatch2, echo=TRUE}
## batch reading of all csv files in specified path :
datAll2 <- readCsvBatch(fileNames=NULL, path=path1, silent=TRUE)
str(datAll2)
```


### Batch-Reading Of Tabulated Files

The function `readTabulatedBatch()` allows fast batch reading of tabulated files. 
All files specified (or all files from a given directory) will be read into separate data.frames of a list.
Default options are US-style comma, automatic testing for head in case the package _data.table_ is available (otheriwse : no header).
Furthermore it is possible to design a given (numeric) column and directly filter for all lines passing a given threshold, allowing to get smaller objects.


```{r readTabulatedBatch1, echo=TRUE}
path1 <- system.file("extdata", package="wrMisc")
fiNa <-  c("a1.txt","a2.txt")
allTxt <- readTabulatedBatch(fiNa, path1)
str(allTxt)
```


### Reading/Importing Incomplete Tables

Sometimes were may get confronted with data which look like 'incomplete' tables. 
In such cases some rows do not contain as many elements/columns as other columns. 
Files with this type of data may pose a problem for `read.table()` (from the _utils_ package).
In some cases using the argument _fill=TRUE_ may allow to overcome this problem.
The function _readVarColumns()_ (from this package) was designed to provide better help in such odd cases.
Basically, each line is read and parsed separately, the user should check/decide on the separator to be used.

The example below lists people's names in different locations, some locations have more persons ... 
Sometimes exporting such data will generate shorter lines in locations with fewer elements (here 'London') and no additional separators will get added (to mark all empty fields) towards the end.
The function `readVarColumns()` (from this package) provides help to read such data, if the content (and separators) of the last columns are missing.

```{r readVarColumns, echo=TRUE}
path1 <- system.file("extdata", package="wrMisc")
fiNa <- "Names1.tsv"
datAll <- readVarColumns(fiName=file.path(path1,fiNa), sep="\t")
str(datAll)
```

In this example _readVarColumns()_ would give a warning (and column-names are not recognized), if you use the argument _header=TRUE_ you'll get an error and nothing gets read.



### Converting Url For Reading Tabulated Data From GitHub

GitHub (https://github.com) allows sharing code and data.
In order to properly read tabulated (txt, tsv or csv) data directly from a given url from the GitHub pages, the user typically needs switch to the 'Raw' view.
The function `gitDataUrl()` allows to conventiently switch any url (on GitHub) to the format from 'Raw view', suitable for directly reading the data using _read.delim()_ , _read.table()_ or _read.csv()_ etc ...).

```{r readGit1, echo=TRUE}
## An example url with tabulated data :
url1 <- "https://github.com/bigbio/sdrf-annotated-datasets/blob/main/datasets/PXD001819/PXD001819.sdrf.tsv"
gitDataUrl(url1)
```

The example below shows how this is used in the function _readSampleMetaData()_ in [wrProteo](https://CRAN.R-project.org/package=wrProteo).
```{r readGit2, echo=TRUE}
dataPxd <- try(read.delim(gitDataUrl(url1), sep='\t', header=TRUE))
str(dataPxd)
```

---




## Data Vectorization   {#Vectorization}

In many scientific domains raw data often come as coordinates in form of pairs of start/end points. 
While correlation measures linear association, vector-based metricsâ€”distance and angleâ€”capture both magnitude and direction of change.
The  `vectorDistAngle()` function helps by transforming paired coordinates into polar form (distance and angle).


```{r vectorDistAngle1, echo=TRUE}
x <- matrix(c(0, 3, 0.5, 4, 1, 1), ncol=2, byrow=TRUE)
y <- matrix(c(0, 0, 1, 3, 0.5, 5), ncol=2, byrow=TRUE)
(vectDist <- vectorDistAngle(x, y))
```
Below, a graphical representation of the the example-vectors (which were initially defined by their start- and end-points).
```{r vectorDistAngle2, echo=TRUE}
plot(cbind(x=as.numeric(x), y=as.numeric(y)), main="Example for vectorDistAngle()",las=1)
arrows(x[,1], y[,1], x[,2], y[,2])
text(x[,2], 0.1 +y[,2], paste0("dist=",round(vectDist[,1],2),", angle=",round(vectDist[,2],2)), adj=1)
```

---


## Normalization 

The main reason of normalization is to remove variability in the data which is not directly linked to the (original/biological) concept of a given experiment. 
High throughput data from real world measurements may easily contain various deformations due to technical reasons, eg slight temperature variations, electromagnetic interference, instability of reagents etc. 
In particular, transferring constant amounts of liquids/reagents in highly repeated steps over large experiments is often also very challenging, small variations of the amounts of liquid (or similar) are typically addressed by normalization. However, applying aggressive normalization to the data also brings considerable risk of starting to loose some of the effects one intended to study.
At some point it may rather be better to eliminate a few samples or branches of an experiment to avoid too invasive intervention. This shows that quality control can be tightly linked to decisions about data-normalization.                      {#Normalization}
In conclusion, normalization may be far more challenging than simply running some algorithms.. 

In general, the use has to assume/define some hypothesis to justify intervention.
Sometimes specific elements of an experiment are known to be not affected and can therefore be used to normalize the rest.
Eg, if you observe growth of trees in a forest, big blocks of rock on the floor are assumed no to change their location. 
So one could use them as alignment-marks to superpose pictures taken at slightly different positions.

The hypothesis of no global changes is very common : During the course of many biological experiments (eg change of nutrient) one 
assumes that only a small portion of the elements measured (eg the abundance of all different gene-products) do change,
since many processes of a living cell like growth, replication and interaction with neighbour-cells are assumed not to be affected.
So, if one assumes that there are no global changes one normalizes the input-data in a way that the average or median across each experiment will give the same value.
In analogy, if one takes photographs on a partially cloudy day, most cameras will adjust light settings (sun r clouds) so that global luminosity stays the same. 
However, if too many of the measured elements are affected, this normalization approach will lead to (additional) loss of information.

It is _essential_ to understand the type of deformation(s) data may suffer from in order to choose the appropriate approacges for normalization. 
Of course, graphical representations ([PCA](https://en.wikipedia.org/wiki/Principal_component_analysis), [MA-plots](https://en.wikipedia.org/wiki/MA_plot), etc) are extremely important to identifying abnormalities and potential problems. 
The package [wrGraph](https://CRAN.R-project.org/package=wrGraph) offers also complementary options useful in the context of normalization.
Again, graphical representation(s) of the data help to visualize how different normalization procedures affect outcomes.

Before jumping into normalization it may be quite useful to _filter_ the data first.
The overall idea is, that most high-throughput experiments do produce some non-meaningful data (artefacts) and it may be wise to remove such 'bad' data
first, as they may effect normalization (in particular _extreme values_). 
A special case of problematic data concerns _NA_-values.


### Filter Lines Of Matrix To Reduce Content Of NAs

Frequent _NA_-values may represent another potential issue. Unfortunately, there is no general optimal advice for handeling NA-values.
To get started, you should try to investigate how and why NA-values occurred to check if there is a special 'meaning' to them.
For example, on some measurement systems values below detection limit may be simply reported as NAs.

High number of NAs in specific columns may point to technical problems or very different samples.
If the second case doesn't seem to apply, one might consider eliminiating such columns of low-quality data. 

If the lines of your data represent different features quantified (eg proteins, or transcripts), then lines almost entirely composed of NA-values represent features
that may not be well exploited anyway. Therefore many times one tries to filter away lines of 'bad' data (ie too high content of NAs).
 
                                     
Please note, that imputing _NA_-values represents another option instead of filtering and removing, multiple other packages address this in detail, too.
All decisions of which approach to choose should be data-driven.


#### Filter For Each Group Of Columns For Sufficient Data As Non-NA

Filter for each group of columns for sufficient data as non-NA
The function `presenceGrpFilt()` allows to

```{r presenceGrpFilt1, echo=TRUE}
dat1 <- matrix(1:56,ncol=7)
dat1[c(2,3,4,5,6,10,12,18,19,20,22,23,26,27,28,30,31,34,38,39,50,54)] <- NA
grp1 <- gl(3,3)[-(3:4)]
dat1

## now let's filter
presenceGrpFilt(dat1, grp=grp1, presThr=0.75)  # stringent
presenceGrpFilt(dat1, grp=grp1, presThr=0.25)  # less stringent

presenceGrpFilt(dat1, grp=grp1, presThr=0.25)  # less stringent
```


#### Filter As Separate Pairwise Groups Of Samples

If you want to use your data in a pair-wise view (like running t-tests on each line) the function `presenceFilt()`
allows to eliminate lines containing too many _NA_-values for each pair-wise combination of the groups/levles.

```{r presenceFilt, echo=TRUE}
grp2 <- factor(rep(letters[4:2],c(2,2,3)))
presenceFilt(dat1, gr=grp2, maxGr=1, ratM=0.1)
presenceFilt(dat1, gr=grp2, maxGr=2, rat=0.5)
```

Here it is also possible to custom select only a few pairwise comparisons 
(when an elevated number of groups will trigger a rather high number of pairwise comparisons) :

```{r presenceFilt2, echo=TRUE}
presenceFilt(dat1, gr=grp2, useComp=c("d-c","b-d"), maxGr=0, ratM=0.1)
```


#### Cleaning Replicates

This procedures aims to remove (by setting to as _NA_) the most extreme of noisy replicates.
Thus, it is assumed that all columns of the input matrix (or data.frame) are replicates of the other columns.
The _nOutl_ most distant points are identified and will be set to _NA_.

```{r cleanReplicates, echo=TRUE}

(mat3 <- matrix(c(19,20,30,40, 18,19,28,39, 16,14,35,41, 17,20,30,40), ncol=4))
cleanReplicates(mat3, nOutl=1)
cleanReplicates(mat3, nOutl=3)

```


### The Function normalizeThis()      {#NormalizeThis}

In biological high-throughput data columns typically represent different samples, which may be organized as replicates. 
During high-throughput experiments thousands of (independent) elements are measured (eg abundance of gene-products), they are represented by rows.
As real-world experiments are not always as perfect as we may think, small changes in the signal measured may easily happen.
Thus, the aim of normalizing is to remove or reduce any trace/variability in the data not related to the original experiement but due to imperfections during detection.

Note, that some experiments may produce a considerable amount of missing data (NAs) which require special attention (dedicated developments exist in other R-packages eg in [wrProteo](https://CRAN.R-project.org/package=wrProteo)). 
My general advice is to first carefully look where such missing data is observed and to pay attention to replicate measurements 
where a given element once was measured with a real numeric value and once as missing information (NA). 


```{r normalizeThis0, echo=TRUE}
set.seed(2015); rand1 <- round(runif(300) +rnorm(300,0,2),3)
dat1 <- cbind(ser1=round(100:1 +rand1[1:100]), ser2=round(1.2*(100:1 +rand1[101:200]) -2),
  ser3=round((100:1 +rand1[201:300])^1.2-3))
dat1 <- cbind(dat1, ser4=round(dat1[,1]^seq(2,5,length.out=100) +rand1[11:110],1))
## Let's introduce some NAs
dat1[dat1 <1] <- NA
## Let's get a quick overview of the data
summary(dat1)
## some selected lines (indeed, the 4th column appears always much higher)
dat1[c(1:5,50:54,95:100),]
```

Our toy data may be normalized by a number of different criteria. 
In real applications the nature of the data and the type of deformation detected/expected will largely help 
deciding which normalization might be the 'best' choice. Here we'll try first normalizing by the mean,
ie all columns will be forced to end up with the same column-mean. 
The trimmed mean does not consider values at extremes (as outliers are frequently artefacts and display extreme values).
When restricting even stronger which values to consider one will eventually end up with the median (3rd method used below).

```{r normalizeThis1, echo=TRUE}
no1 <- normalizeThis(dat1, refGrp=1:3, meth="mean")
no2 <- normalizeThis(dat1, refGrp=1:3, meth="trimMean", trim=0.4)
no3 <- normalizeThis(dat1, refGrp=1:3, meth="median")
no4 <- normalizeThis(dat1, refGrp=1:3, meth="slope", quantFa=c(0.2,0.8))
```

It is suggested to verify normalization results by plots. 
Note, that  [Box plots](https://en.wikipedia.org/wiki/Box_plot) may not be appropriate in some cases (eg multimodal distributions),
for displaying more details you may consider using [Violin-Plots](https://en.wikipedia.org/wiki/Violin_plot) from packages [vioplot](https://CRAN.R-project.org/package=vioplot) or [wrGraph](https://CRAN.R-project.org/package=wrGraph), another option might be a (cumulated) frequency plot (eg in package [wrGraph](https://CRAN.R-project.org/package=wrGraph)).

```{r normalizeThis_plot1, echo=FALSE,eval=TRUE}
boxplot(dat1, main="raw data", las=1)
```

You can see clearly, that the 4th data-set has a problem of range. So we'll see if some proportional normalization
may help to make it more comparable to the other ones.
 
```{r normalizeThis_plot2, echo=FALSE,eval=TRUE}
layout(matrix(1:4, ncol=2))
boxplot(no1, main="mean normalization", las=1)
boxplot(no2, main="trimMean normalization", las=1)
boxplot(no3, main="median normalization", las=1)
boxplot(no4, main="slope normalization", las=1)
```


### Normalize By Rows        {#RowNormalize}

The standard approach for normalizing relies on consisting all columns as collections of data who's distribution is not supposed to change.
In some cases/projects we may want to formulate a much more 'engaged' hypothesis : We consider the content of all columns strictly as the same.
For example this may be the case when comparing a dataset consisting of technical replicates only (the data may have many 'holes', ie may have sparse character).
In such cases one may use the function `rowNormalize()` which tries to find the average or mean optimal within-line normalization factor.

Besides, an additional mode of operation for _sparse data_ has been added :
Basically, once a row contains just one NA, this row can't be used any more to derive a normalization factor for all rows.
Thus, with many NA-values the number of 'complete' rows will be low or even 0 redering this approach inefficient or impossible.
Once the content of NA-values is above a customizable threshold, the data will be broken in smaller subsets with fewer groups of fewer columns,
 thus increasing the chances of finding 'complete' subsets of data which will be normalized first and added to other subsets in later steps.

This approach relies on the **hypothesis** that *all data in a given line should be (aproximately) the same value* !
Thus, this procedure is particularly well adopted to the case when _all_ samples are multiple replicate measurements of the _same_ sample.


```{r rowNormalize1, echo=TRUE}
set.seed(2); AA <- matrix(rbinom(110, 10, 0.05), nrow=10)
AA[,4:5] <- AA[,4:5] *rep(4:3, each=nrow(AA))

AA1 <- rowNormalize(AA)
round(AA1, 2)
```

Now, let's make this sparse and try normalizing:

```{r rowNormalize2, echo=TRUE}
AC <- AA
AC[which(AC <1)] <- NA

(AC1 <- rowNormalize(AC))
```

Like with _normalizeThis()_ we can define some reference-lines (only these lines will be considered to determine normalization-factors)

```{r rowNormalize3, echo=TRUE}
(AC3 <- rowNormalize(AC, refLines=1:5, omitNonAlignable=TRUE))
```

Please note, that the iterative procedure for _sparse data_ may consume large amounts of computational resources, in particular when
a small number of subgroups has been selected.



### Matrix Coordinates Of Values/Points According To Filtering

Sometimes one needs to obtain the coordinates of values/points of a matrix according to a given filtering condition.
The standard approach using _which()_ gives only a _linearized_ index but not row/column, which is sufficient for replacing indexed values.
If you need to know the true row/column indexes, you may use `coordOfFilt()`.

```{r coordOfFilt1, echo=TRUE}
set.seed(2021); ma1 <- matrix(sample.int(n=40, size=27, replace=TRUE), ncol=9)
## let's test which values are >37
which(ma1 >37)      # doesn't tell which row & col
coordOfFilt(ma1, ma1 >37)
```


### Trimmed Mean

Under certain circumstances using the trimmed mean may be more stable towards outlyer values.
The base function  `mean()` allows to preform symmetric trimming, thus with more trimming the result will start converging to the _mean_.
The function `trimmedMean()` (from this package) gives more flexible options for assinging different upper- and lower fractions of the initial data to be trimmed.
When outlyer values always appear at the high (or low) end of data, such proceeding may be useful.
However, the user is encouraged to use this with caution since there is also a risk of introducing bias.


```{r trimmedMean1, echo=TRUE}
x <- c(17:11,27:28)
mean(x)
mean(x, trim=0.15)       # symmetric trimming
mean(x[x < 25])          # manual trimming
trimmedMean(x, trim=c(l=0, u=0.7))   # asymmetric trim
```


###  Blockwise Normalization Of 2 or 3 Data-Sets   {#blockNormalize}

In some instances you may want to make data-sets globally comparable without completely "normamlizing" all columns, in order to preseve some of the inherent structure of each of the data-sets.
Such procedures typically consiste in rescaling each data-set to common boundaries or to have the same overall median (without median-normalizing each column).

Instead of normalizing all columns of joined data-sets the alternative of blockwise procedures treat each initial data-set as a given block without addressing each column independently.
The function `blockNormalize()` performs block quantile normalization on 2 or 3 datasets (vectors, matrices or data.frames) to align their overall distributions while preserving the internal rank structure within each block.
This function was designed to make 2 or 3 data-sets globally comparable without intervening too much to the internal structures.

As result, all output blocks (up to 2 blocks) will have identical distributions (the mean of their sorted values)
The relative ordering (ie ranks) within each original block is preserved.
This function resembles somehow to the function normalizeBetweenArrays() from package limma, which does quantile normalization - but not for arbitrary blocks (using entire arrays only).


```{r blockNormalize1, echo=TRUE}
## Basic usage with vectors 
x <- c(1, 5, 3, 7, 2)
y <- (1:5)*10
## Align distributions (default: x and y will have identical distributions)
norm1 <- blockNormalize(x, y)
table(sort(norm1$x) == sort(norm1$y))  # all TRUE with 'quantile.block'
```


The primary us of this function will likely consist in normalizing matrixes or data.frames
```{r blockNormalize2, echo=TRUE}
## matrix-example (like with omics-data)
set.seed(2026); mat1 <- matrix(rnorm(70), nrow=10, dimnames=list(letters[1:10],LETTERS[1:7])) *5  # 10 lines x 10 samples
set.seed(2025); mat2 <- matrix(rnorm(70, mean=5), nrow=10)^2 -15
mat2[which(mat2 < 1.8)] <- mat2[which(mat2 < 1.8)] + 32

norm2 <- blockNormalize(mat1, mat2, method="rescale", range=c(0.1, 10))
sapply(norm2, range)

norm3 <- blockNormalize(mat1, mat2, method="median")
sapply(norm3, quantile, c(0.25,0.5,0.75), na.rm=TRUE)

norm4 <- blockNormalize(mat1, mat2, method="quantile.block")
sapply(norm4, quantile, c(0.25,0.5,0.75), na.rm=TRUE)
```

The resulting distribution :
```{r blockNormalize3, echo=TRUE}
layout(matrix(1:4, ncol=2))
boxplot(cbind(mat1, NA, mat2), main="initial", las=1)
boxplot(cbind(norm2$x, NA, norm2$y), main="rescale block", las=1)
boxplot(cbind(norm3$x, NA, norm3$y), main="median block", las=1)
boxplot(cbind(norm4$x, NA, norm4$y), main="quantile.block", las=1)
```

You can see that relative variability inside each dataset will show up differently depending on the normalization procedure chosen.

Under some circumstances you may want to define different now ranges for each data-set.
This might be of interest if the data-sets follow somehow different distributions, in particuar in respect to (rare) extreme values.
In some cases it my be of interest to not provide absolute boundaries but given quantiles for lower and/or upper boundaries. 

In some cases it my be of interest not defining absolute boundaries but rather use quantiles for lower and/or upper boundaries. 

the overal distributions look like this:
```{r blockNormalize4, echo=TRUE}
## the overall distribution of blocks
layout(matrix(1:4, ncol=2))
boxplot(cbind(mat1=as.numeric(mat1), mat2=as.numeric(mat2)), main="initial (overall)",las=1)
boxplot(cbind(mat1=as.numeric(norm2$x), mat2=as.numeric(norm2$x)), 
  main="rescale block norm (overall)",las=1)
boxplot(cbind(mat1=as.numeric(norm3$x), mat2=as.numeric(norm3$x)), 
  main="median block norm (overall)",las=1)
boxplot(cbind(mat1=as.numeric(norm4$x), mat2=as.numeric(norm4$x)), 
  main="quantile.block norm (overall)",las=1)
```

In some cases (rare) extreme values may have strong impact on the outcome of re-scaling.
In such cases you can rather define two quantiles of the data to get normalized to predefined values, and the remaining data will get adjusted in a linear fashion.
This way it is poissible to align the major part of 2 datasets while extreme values live their 'own lives' (if you choose to conserve their extreme character).
Otherwise the  _method="quantile.block"_ will help bringing extreme values closer to the main bulk of data.





## Statistical Testing  {#StatisticalTesting}

### Normal Random Number Generation with Close Fit to Expected mean and sd

When creating random values to an expected _mean_ and _sd_, the results ontained using the standard function `rnorm()` 
may deviate somehow from the expected mean and sd, in particular with low _n_. 
To still produce random values fitting closely to the expected _mean_ and _sd_ you may use the function `rnormW()`. 
The case of _n=2_ is quite simple with one possible results. 
In other cases (_n>2_), there will be a random initiation which can be fixed using the argument _seed_.


```{r rnormW1, echo=TRUE}
## some sample data :
x1 <- (11:16)[-5]
mean(x1); sd(x1)
```

```{r rnormW2, echo=TRUE}
## the standard way for gerenating normal random values
ra1 <- rnorm(n=length(x1), mean=mean(x1), sd=sd(x1))
## In particular with low n, the random values deviate somehow from expected mean and sd :
mean(ra1) -mean(x1) 
sd(ra1) -sd(x1)
```

```{r rnormW3, echo=TRUE}
## random numbers with close fit to expected mean and sd :
ra2 <- rnormW(length(x1), mean(x1), sd(x1))
mean(ra2) -mean(x1) 
sd(ra2) -sd(x1)   # much closer to expected value
```

Thus, the second data-sets fits even with few _n_ very well to the global characteristics defined/expected.


### Moderated Pair-Wise t-Test from limma

If you are not familiar with the way data is handled in the Bioconductor package [limma](https://bioconductor.org/packages/release/bioc/html/limma.html) 
and you would like to use some of the tools for running moderated t-tests therein, this will provide easy access using `moderTest2grp()` :
```{r moderTest2grp, echo=TRUE}
set.seed(2017); t8 <- matrix(round(rnorm(1600,10,0.4),2), ncol=8,
  dimnames=list(paste("l",1:200), c("AA1","BB1","CC1","DD1","AA2","BB2","CC2","DD2")))
t8[3:6,1:2] <- t8[3:6,1:2]+3     # augment lines 3:6 for AA1&BB1
t8[5:8,5:6] <- t8[5:8,5:6]+3     # augment lines 5:8 for AA2&BB2 (c,d,g,h should be found)
t4 <- log2(t8[,1:4]/t8[,5:8])
fit4 <- moderTest2grp(t4, gl(2,2))
## now we'll use limma's topTable() function to look at the 'best' results
if("list" %in% mode(fit4)) {  # if you have limma installed we can look further
  library(limma)
  topTable(fit4, coef=1,n=5)                      # effect for 3,4,7,8
  fit4in <- moderTest2grp(t4, gl(2,2), testO="<")
  if("list" %in% mode(fit4in)) topTable(fit4in, coef=1,n=5) }
```

### Multiple Moderated Pair-Wise t-Tests From limma

If you want to make multiple pair-wise comparisons using `moderTestXgrp()` :
```{r moderTestXgrp, echo=TRUE}
grp <- factor(rep(LETTERS[c(3,1,4)], c(2,3,3)))
set.seed(2017); t8 <- matrix(round(rnorm(208*8,10,0.4),2), ncol=8,
  dimnames=list(paste(letters[], rep(1:8,each=26),sep=""), paste(grp,c(1:2,1:3,1:3),sep="")))
t8[3:6,1:2] <- t8[3:6,1:2] +3                    # augment lines 3:6 (c-f) 
t8[5:8,c(1:2,6:8)] <- t8[5:8,c(1:2,6:8)] -1.5    # lower lines 
t8[6:7,3:5] <- t8[6:7,3:5] +2.2                  # augment lines 
## expect to find C/A in c,d,g, (h)
## expect to find C/D in c,d,e,f
## expect to find A/D in f,g,(h)  
test8 <- moderTestXgrp(t8, grp) 
head(test8$p.value, n=8) 
```

#### Choosing amongst multiple possible pairwise comparisons

If you would like to run all possible pairwise comparisons you can set argument _useComparison_ to 'all'.

```{r moderTestXgrp2, echo=TRUE}
test8a <- moderTestXgrp(t8, grp, useComparison="all")
head(test8a$p.value, n=3) 
```

Besides, there are multiple ways to select which pairwise comparison(s) you would like to perform.

```{r moderTestXgrp3, echo=TRUE}
## multiple ways for choosing same comparison(s)
test8b <- moderTestXgrp(t8, grp, useCo=c("A-D"))   # name of comparison(s) 
test8c <- moderTestXgrp(t8, grp, useCo=2)          # the 2nd of all possible
test8d <- moderTestXgrp(t8, grp, useCo=cbind(samp="A", ref="D"))
test8e <- moderTestXgrp(t8, grp, useCo=cbind(samp=1, ref=3))

head(test8b$p.value, n=3) 
head(test8c$p.value, n=3) 
head(test8d$p.value, n=3) 
head(test8e$p.value, n=3) 

identical(test8b, test8c)
identical(test8b, test8d)
```


In some cases one might like a more explicite name of pairwise combined groups.
Using the argument _sep_ a specific separator may be given or searched from groups like _'combine2'_ .

```{r moderTestXgrp4, echo=TRUE}
## all comparisons with more explicite pairwise combined names
test8f <- moderTestXgrp(t8, grp, sep="combine2", useCo="all") 
head(test8f$p.value, n=8)
```



### Transform p-values To Local False Discovery Rate (lfdr)

To get an introduction into local false discovery rate estimations you may read [Strimmer 2008](https://doi.org/10.1093/bioinformatics/btn209). 
A convenient way to get lfdr values calculated by the package [fdrtool](https://CRAN.R-project.org/package=fdrtool) is available via the function `pVal2lfdr()`.

Note, that the toy-example used below is too small for estimating meaningful lfdr values.
For this reason the function _fdrtool()_ from package [fdrtool](https://CRAN.R-project.org/package=fdrtool) will issue warnings.

```{r pVal2lfdr, echo=TRUE}
set.seed(2017); t8 <- matrix(round(rnorm(160,10,0.4),2), ncol=8, dimnames=list(letters[1:20],
  c("AA1","BB1","CC1","DD1","AA2","BB2","CC2","DD2")))
t8[3:6,1:2] <- t8[3:6,1:2] +3   # augment lines 3:6 (c-f) for AA1&BB1
t8[5:8,5:6] <- t8[5:8,5:6] +3   # augment lines 5:8 (e-h) for AA2&BB2 (c,d,g,h should be found)
head(pVal2lfdr(apply(t8, 1, function(x) t.test(x[1:4], x[5:8])$p.value)))
```


### Confindence Intervals (under Normal Distribution)

The [confindence interval (CI)](https://en.wikipedia.org/wiki/Confidence_interval) is a common way of describing the uncertainity of measured or estimated values. 
The function `confInt()` allows calculating the confidence interval of the mean (using the functions _qt()_ and _sd()_) under 
a given [significance level (alpha)](https://en.wikipedia.org/wiki/Statistical_significance).
assuming that the [Normal distribution](https://en.wikipedia.org/wiki/Normal_distribution) is valid.

```{r fcCI, echo=TRUE}
set.seed(2022); ran <- rnorm(50)
confInt(ran, alpha=0.05)
## plot points and confindence interval of mean
plot(ran, jitter(rep(1, length(ran))), ylim=c(0.95, 1.05), xlab="random variable 'ran'",main="Points and Confidence Interval of Mean (alpha=0.05)", ylab="", las=1)
points(mean(ran), 0.97, pch=3, col=4)     # mean
lines(mean(ran) +c(-1, 1) *confInt(ran, 0.05), c(0.97, 0.97), lwd=4, col=4)  # CI
legend("topleft","95% conficence interval of mean", text.col=4,col=4,lty=1,lwd=1,seg.len=1.2,cex=0.9,xjust=0,yjust=0.5)
```


### Extract Groups Of Replicates From Pair-Wise Column-Names

When running multiple pairwise tests (using *moderTestXgrp()*) the column-names are concatenated group-names. 
To get the index of which group has been used in which pair-wise set you may use the function `matchSampToPairw()`, as shown below.

```{r matchSampToPairw, echo=TRUE}
## make example if limma is not installed
if(!requireNamespace("limma", quietly=TRUE)) test8 <- list(FDR=matrix(1, nrow=2, ncol=3, dimnames=list(NULL,c("A-C","A-D","C-D"))))
matchSampToPairw(unique(grp), colnames(test8$FDR)) 
```


### Extract Numeric Part Of Column-Names

When running multiple pairwise tests (using *moderTestXgrp()*) the results will be in adjacent columns and the group-names reflected in the column-names.
In the case measurements from multiple levels of a given variable are compared it is useful to extract the numeric part, the function `numPairDeColNames()` provides support to do so. 
When extracting just the numeric part, unit names will get lost, though. Note, if units used are not constant (eg seconds and milliseconds mixed) the extracted numeric values do not reflect the real quantitative context any more.

```{r pairWiseConc1, echo=TRUE}
mat1 <- matrix(1:8, nrow=2, dimnames=list(NULL, paste0(1:4,"-",6:9)))
numPairDeColNames(mat1)
```


### Automatic Determination Of Replicate Structure Based On Meta-Data

In order to run statistical testing the user must know which sample should be considered replicate of whom.
The function `replicateStructure()` aims to provide help by checking all column of a matrix of meta-data with the aim of identifying the replicate-status.

To do so, all columns are examined how many groups of replicats they may design. Depending on the argumen _method_ various options for choosing automatically exist :
The default _method="combAll"_ will select the column with the median number of groups (not counting all-different or all-same columns)).
When using as _method="combAll"_ (ie combine all columns that are neither all-different nor all-same), there is risk all lines (samples) will be be considered different and no replicates remain.
To avoid this situation the argument -method_ can be set to _"combNonOrth"_.
Then, it will be checked if adding more columns will lead to complete loss of replicates, and -if so- concerned columns omitted.


```{r replicateStructure1, echo=TRUE}
## column a is all different, b is groups of 2,
## c & d  are groups of 2 nut NOT 'same general' pattern as b
strX <- data.frame(a=letters[18:11], b=letters[rep(c(3:1,4), each=2)],
 c=letters[rep(c(5,8:6), each=2)], d=letters[c(1:2,1:3,3:4,4)],
 e=letters[rep(c(4,8,4,7),each=2)], f=rep("z",8) )
strX

replicateStructure(strX[,1:2])
replicateStructure(strX[,1:4], method="combAll")
replicateStructure(strX[,1:4], method="combAll", exclNoRepl=FALSE)
replicateStructure(strX[,1:4], method="combNonOrth", exclNoRepl=TRUE)
replicateStructure(strX, method="lowest")
```


### Addressing Pairwise Comparisons

There are multiple ways of specifying pairwise comparions (eg 'A-B', or c('A','B'), or using indexes), the function `convPairwiseSetup()`  allows converting between different types 
and returns a list with multiple formats for addressing specific quesions. 
If the separator for concantenating is known this may be helpful, but not obligatory. In many cases the function will manage to identify independently which separator has been used.


```{r convPairwiseSetup1, echo=TRUE}
## concatenated names of groups
convPairwiseSetup(c("B-C","D-A"), LETTERS[1:4])

## separate names of groups
convPairwiseSetup(c("B","C"), LETTERS[1:3])
convPairwiseSetup(2:3, LETTERS[1:3])
convPairwiseSetup(2:3, LETTERS[1:3], sep="__")
convPairwiseSetup(matrix(c("B","D","C","A"), ncol=2), LETTERS[1:4])
```

#### Suggesting A Separator For Pairwise Combinations

In many cases the user doesn't choose a specific separator to be used for concatenating group-names.  
The function `convPairwiseSetup()` allows checking a panel of separators to choose a 'unique' one which does not occur in any of the group-names. 

```{r suggestSeparator1, echo=TRUE}
getPWseparator(grp=LETTERS[1:3])
getPWseparator(grp=c("B-C","D","E")) 
getPWseparator(grp=c("B-C","D","E"), includeGrp=TRUE) 
```

Of course you can also check your preferred separators, the first found compatible (ie unique) will be reported.
```{r suggestSeparator2, echo=TRUE}
potSep <- c("-", "D", " -compared to- ")
getPWseparator(grp=c("B-C","D","E"), includeGrp=TRUE, potSep=potSep)
```


By using the argument _includeGrp_ the output will be a list including the input instead of simply the suggested separator.
```{r suggestSeparator3, echo=TRUE}
getPWseparator(grp=c("B-C","D","E"), includeGrp=TRUE) 
```


#### Identify The Separator Used In Concatenated Text

The same function may be used to identify which separator has been used, by checking a panel of classical candidates for separators
```{r identifySeparator1, echo=TRUE}
## Identify separator used
getPWseparator(compNames=c("B-C","B-D","C-D")) 
getPWseparator(compNames=c("B-C","B-D","C-D"), includeGrp=TRUE)
```

Besides, the function  `findHeadAndTail()` allows (unbiased) identifying separators without having to define candidates to check,
as it proceeds by removing names of groups from beginning and end of concatenated terms. 
However, there is a slight risk that some special characters may not allow finding the proper solution.

```{r identifySeparator2, echo=TRUE}
## Identify separator used
findHeadAndTail(LETTERS[4:2], c("D-X","D-C"))

grp1 <- c("C","ACC","CC","B.B","B.BA","B","CA")
pwNa1 <- "B.B-CC" 
findHeadAndTail(grp1, pwNa1)

pwNa2 <- c("B.B-CC", "CA-ACC")
findHeadAndTail(grp1, pwNa2)
```



## Working With Clustering  {#WorkingWithClustering}

Multiple concepts for clustering have been deeveloped, most of them allow extracting a vector with the cluster-numbers.
Here some functions helping to work with the output of such clustering results are presented.


### Prepare Data For Clustering

The way how to prepare data for clustering may be as important as the choice of the actual clustering-algorithm ...

Many clustering algorithms are available in R (eg see also [CRAN Task View: Cluster Analysis & Finite Mixture Models](https://CRAN.R-project.org/view=Cluster)), many of them require the input data to be standardized.
The regular way of standardizing sets all elements to mean=0 and sd=1.
To do so, the function `scale()` may be used. 

```{r std1, echo=TRUE}
dat <- matrix(2*round(runif(100),2), ncol=4)
mean(dat); sd(dat)

datS <- scale(dat)
apply(datS, 2, sd)
# each column was teated separately
mean(datS); sd(datS); range(datS)
# the mean is almost 0.0 and the sd almost 1.0

datB <- scale(dat, center=TRUE, scale=FALSE)
mean(datB); sd(datB); range(datB)              # mean is almost 0
```

However, if you want the entire data-set and not each column sparately, you may use `standardW()`.
Thus, relative differences visible within a line will be conserved.
Furthermore, in case of 3-dim arrays, this function returns also the same dimensions as the input.

```{r std2, echo=TRUE}
datS2 <- standardW(dat)
apply(datS2, 2, sd)
summary(datS2)
mean(datS2); sd(datS2)

datS3 <- standardW(dat, byColumn=TRUE)
apply(datS3, 2, sd)
summary(datS3)
mean(datS3); sd(datS3)
```


Sometimes it is sufficient to only set the minimum and maximum to a given range.

```{r scale1, echo=TRUE}
datR2 <- apply(dat, 2, scaleXY, 1, 100)
summary(datR2); sd(datR2)
```

Please note that this function shares some overlap with _adjBy2ptReg()_ (which has less options and will get deprecated in the future)


### Characterize Clustering Results

Here a very basic clustering example...

```{r clu01, echo=TRUE}
nGr <- 3
irKm <- stats::kmeans(iris[,1:4], nGr, nstart=nGr*4)             # no need to standardize
   table(irKm$cluster, iris$Species)
   #wrGraph::plotPCAw(t(as.matrix(iris[,1:4])), sampleGrp=irKm,colBase=irKm$cluster,useSymb=as.numeric(as.factor(iris$Species)))
```

Using the function `reorgByCluNo()` we can now 'apply' the clustering result to the initial data to obtain other information.

```{r clu02, echo=TRUE}
## sort results by cluster number
head(reorgByCluNo(iris[,-5], irKm$cluster))
tail(reorgByCluNo(iris[,-5], irKm$cluster))
```

Let's calculate the median and sd values for each cluster:

```{r clu03, echo=TRUE}
## median an CV
ir2 <- reorgByCluNo(iris[,-5], irKm$cluster, addInfo=FALSE, retList=TRUE)
```


```{r clu04, echo=TRUE}
sapply(ir2, function(x) apply(x, 2, median))
```

```{r clu05, echo=TRUE}
sapply(ir2, colSds)
```

Besides, we have already seen the function `cutArrayInCluLike()` in section [Working with Arrays](#WorkingWithArrays) 'Working with Arrays'.


### Remove or Reassign Orphans

In some some circumstances clusters/groups with very vew individuals are not productive during further evalualtions (in particular in the context of interaction-netwoks).
The function `rmOrphans()` allows identifying and modifying cluster- or group-assignments of such very small groups ('orphans').

In the example below a vector of cluster-assignments ('x') is treated by different options to remove orphans :

```{r rmOrphans1, echo=TRUE}
x=c(3:1,3:4,4:6,5:3)
cbind(x, def=rmOrphans(x), assign1=rmOrphans(x, reassign=TRUE), 
  assign1=rmOrphans(x, minN=0.2, reassign=TRUE) )
```


## Tree-Like Structures   {#TreeLikeStructures}


### Filter Lists Of Connected Nodes, Extension Of Networks As 'Sandwich'

When interogating network-databases (like String for proteins or the coexpressionDB for gene co-expression) typically a (semi-)quantitatve
value is supplied with the connection of node 'A' to node 'B'.  
In many cases, it may be useful to filter the initial query-output to retain only strong interactions.
Furthermore, it may be of interest to expand such networks by nodes allowing to (further) inter-connect initial query-nodes 
(so called 'Sandwich' nodes as they are in the middle of initial nodes), for such nodes a separate (eg even more stringent) threshold can be applied.

Here let's suppose nodes have 3-digit names (ie numbers). 7 nodes of an initial query gave 1 to 7 conected nodes, 
the results are presented as list of data.frames where the 1st column is the connected node and the 2nd column the quality score of the connection (edge).
Furthemore, let's assume that here lower scores are better.

```{r filterNetw0, echo=TRUE}

lst2 <- list('121'=data.frame(ID=as.character(c(141,221,228,229,449)),11:15), 
  '131'=data.frame(ID=as.character(c(228,331,332,333,339)),11:15), 
  '141'=data.frame(ID=as.character(c(121,151,229,339,441,442,449)),c(11:17)), 
  '151'=data.frame(ID=as.character(c(449,141,551,552)),11:14),
  '161'=data.frame(ID=as.character(171),11),
  '171'=data.frame(ID=as.character(161),11),
  '181'=data.frame(ID=as.character(881:882),11:12) )
```

Now, we'd like to keep the core network consisting of all (dirctly) interconnected nodes with scores below 20 :

```{r filterNetw1, echo=TRUE}
(nw1 <- filterNetw(lst2, limInt=20, sandwLim=NULL, remOrphans=FALSE))
```

In the resulting output the 1st column now represents the query-nodes, the 2nd column all connected nodes based on filtering scores for edges,
and the 3rd colum the score for the edges.

Let's also remove all nodes not connected to a backbone at least 3 nodes long, ie remove orphan pairs of nodes :

```{r filterNetw2, echo=TRUE}
(nw2 <- filterNetw(lst2, limInt=20, sandwLim=NULL, remOrphans=TRUE))
```

If you want to expand this network by nodes allowing to further interconnect the nodes from above, 
we can add all 'sandwich' nodes (let's use a threshold of inferior/equal to 14 which will use only the better 'sandwich'-edges) :

```{r filterNetw3, echo=TRUE}
(nw3 <- filterNetw(lst2, limInt=20, sandwLim=14, remOrphans=TRUE))
```


### Convert Collection Of Pairs Of Nodes To Propensity Matrix

Many times networks get created from pairs of nodes. One way to represent the full network is via propensisty matrixes.
Several advanced tools and packages rather accept such propensisty matrixes as input.
Here, it is assumed that each line of the input represents a separate pair of nodes connected by an edge.

```{r propMatr1, echo=TRUE}
pairs3L <- matrix(LETTERS[c(1,3,3, 2,2,1)], ncol=2)      # loop of 3
(netw13pr <- pairsAsPropensMatr(pairs3L))                # as prop matr
```



### Characterize Individual Contribution Of Single Edges In Tree-Structures

```{r contribToContigPerFrag, echo=TRUE}
path1 <- matrix(c(17,19,18,17, 4,4,2,3), ncol=2,
  dimnames=list(c("A/B/C/D","A/B/G/D","A/H","A/H/I"), c("sumLen","n")))
contribToContigPerFrag(path1)
```


### Count Same Start- And End- Sites Of Edges (Or Fragments)

If you have a set of fragments from a common ancestor and the fragment's start- and end-sites 
are marked by index-positions (integers), you can make a simple graphical display :

```{r simpleFragFig, echo=TRUE}
frag1 <- cbind(beg=c(2,3,7,13,13,15,7,9,7, 3,3,5), end=c(6,12,8,18,20,20,19,12,12, 4,5,7))
rownames(frag1) <- letters[1:nrow(frag1)]
simpleFragFig(frag1)
```

Now we can make a matrix telling if some fragments do start or end at exactely the same position. 
```{r countSameStartEnd, echo=TRUE}
countSameStartEnd(frag1)
```


## Support for Graphical Output   {#SupportForGraphicalOutput}

### Convenient Paste-Collapse

The function `pasteC()` allows adding quotes and separating the last element by specific text (eg 'and').
```{r pasteC, echo=TRUE}
pasteC(1:4)
pasteC(letters[1:4],quoteC="'")
```


### Transform Numeric Values to Color-Gradient

By default most color-gradients end with a color very close to the beginning.
```{r color-gradient1, echo=TRUE}
set.seed(2015); dat1 <- round(runif(15),2)
plot(1:15, dat1, pch=16, cex=2, las=1, col=colorAccording2(dat1),
  main="Color gradient according to value in y")
# Here we modify the span of the color gradient
plot(1:15, dat1, pch=16, cex=2, las=1, 
  col=colorAccording2(dat1, nStartO=0, nEndO=4, revCol=TRUE), main="blue to red")
# It is also possible to work with scales of transparency
plot(1:9, pch=3, las=1)
points(1:9, 1:9, col=transpGraySca(st=0, en=0.8, nSt=9,trans=0.3), cex=42, pch=16)
```


### Assign New Transparency To Given Colors
 
For this purpose you may use `convColorToTransp`.
```{r convColorToTransp, fig.height=6, fig.width=3, echo=TRUE}
col0 <- c("#998FCC","#5AC3BA","#CBD34E","#FF7D73")
col1 <- convColorToTransp(col0,alph=0.7)
layout(1:2)
pie(rep(1,length(col0)), col=col0, main="no transparency")
pie(rep(1,length(col1)), col=col1, main="new transparency")
```


### Print Matrix-Content As Plot

There are many ways of creating reports. If you want simply to combine a few plots into a pdf, the function `tableToPlot()` 
may be helpful to add a small table (eg overview of points/samples/files used in other plots of the same pdf). 
This function prints tables in the current graphical output/window (which may by a pdf-device).


## Other Convenience Functions  {#OtherConvenienceFunctions}

### Writing Compact Dates (more options ...)

Many times it may be useful to add the date to filenames when saving data or plots as files.
The built-in functions _date()_, _Sys.Date()_ and _Sys.Time()_ are a good way to start.

Generally I like to use abbreviated month-names since the order of writing the month is different in Europe compared to the USA.
So, this may help avoiding mis-interpreting dates instead of writing the number of the Month.
For example, 2021-03-05 means in Europe March 5th while in other places it means May 3rd.

You may also look at the standardized format for *numeric dates* [norm ISO 8601](https://en.wikipedia.org/wiki/ISO_8601), 
which matches to _Sys.Date()_ (from package _base_) and _sysDate(style="univ5")_ (package _wrMisc_) .

The R-functions mentioned above (_date()_, _Sys.Date()_, from the _base_ package) use local language settings.
For using English languange names you may use the function `sysDate` from this package (ie wrProteo). 
It allows producing compact versions of current the date, **independent to local language settings** (or not -if you prefer), ie locale-specific, 
(yes, in some languages - like French - the first 3 letters of the month may give ambiguous results !) 
and to avoid white space ' ' (which I prefer to avoid in file-names).
Please look at the function's help-page for all available options.

```{r sysDate1, echo=TRUE}
## To get started
Sys.Date()

## Compact English names (in European order), no matter what your local settings are :
sysDate() 
```

The table below shows a number of options to write the date in English or using local month-names :

```{r DateTab, echo=TRUE}
tabD <- cbind(paste0("univ",1:6), c(sysDate(style="univ1"), sysDate(style="univ2"), 
    sysDate(style="univ3"), sysDate(style="univ4"), as.character(sysDate(style="univ5")), 
    sysDate(style="univ6")), paste0("   local",1:6), 
  c(sysDate(style="local1"), sysDate(style="local2"), sysDate(style="local3"), 
    sysDate(style="local4"), sysDate(style="local5"), sysDate(style="local6")))   
knitr::kable(tabD, caption="Various ways of writing current date")
```


## Acknowledgements

The author would like to acknowledge the support by the [IGBMC](https://www.igbmc.fr) (CNRS UMR 7104, Inserm U 1258, UdS), [CNRS](http://www.cnrs.fr/en), 
[Universit&eacute; de Strasbourg (UdS)](https://www.unistra.fr/fr) and [Inserm](https://www.inserm.fr).



## Session-Info

```{r sessionInfo, echo=FALSE}
sessionInfo()
```

---
title: "Getting started with wrMisc"
author: Wolfgang Raffelsberger
date: '`r Sys.Date()`'
output:
  knitr:::html_vignette:
    toc: true
    fig_caption: yes
  pdf_document:
    highlight: null
    number_sections: no
vignette: >
  %\VignetteIndexEntry{wrMiscVignette1}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---


## Introduction

This package contains a collection of various (low-level) tools which may be of general interest. 
These functions were accumulated over a number of years of data-wrangling when treating high-throughput data from biomedical applications.
Besides, these functions are further used/integrated in more specialized functions dedicated to specific applications in the packages [wrProteo](https://CRAN.R-project.org/package=wrProteo), [wrGraph](https://CRAN.R-project.org/package=wrGraph) or [wrTopDownFrag](https://CRAN.R-project.org/package=wrTopDownFrag). 
All these packages are available on [CRAN](https://cran.r-project.org/).

If you are not familiar with [R](https://www.r-project.org/) you may find many introductory documents on the official R-site 
in [contributed documents](https://cran.r-project.org/other-docs.html) or under [Documentation/Manuals](https://cran.r-project.org/manuals.html).
Of course, numerous other documents/sites with tutorials and courses exist, too.


### Dependencies and Compilation
One of the aims was to write a package easy to install, with low system requirements and few obligatory dependencies.  
All code is written in pure R and does not need any special compilers.
The number of obligatory dependencies was kept to a minumum.

Most of additional packages used in some of the functions were declared as 'suggested' (ie not obligatory), to allow installation of _wrMisc_ even if some these additional packages can't be installed/compiled by the user's instance.
When a feature/function of one of the 'suggested' packages is about to be used, its presence/installation will be checked and,
only if found as missing, the user will be prompted a message inviting to install specific package(s) before using these specific functions.
This helps to avoid not being able installing this package at all if some dependencies may fail to get installed themselves.


### Installation And Loading
To get started, we need to install (if not yet installed) and load the package "[wrMisc](https://CRAN.R-project.org/package=wrMisc)" available from [CRAN](https://cran.r-project.org/).


```{r setup0, include=FALSE, echo=FALSE, messages=FALSE, warnings=FALSE}
suppressPackageStartupMessages({
    library(wrMisc)
})
```

```{r install, echo=TRUE, eval=FALSE}
## If not already installed, you'll have to install the package first.
## This is the basic installation commande in R
install.packages("wrMisc")
```

Since the functions illustrated in this vignette require a number of the _suggested_ packages,
let's check if they are installed and add them (via a small function), if not yet installed.
```{r install2, echo=TRUE, eval=FALSE}
packages <- c("knitr", "rmarkdown", "BiocManager", "kableExtra", "boot", "data.tree", "data.table", 
  "fdrtool", "RColorBrewer", "Rcpp", "wrMisc", "wrGraph", "wrProteo")
checkInstallPkg <- function(pkg) {       # install function
  if(!requireNamespace(pkg, quietly=TRUE)) install.packages(pkg) }

## install if not yet present
sapply(packages, checkInstallPkg)
```

Finally, this package also uses the Bioconductor package [limma](https://bioconductor.org/packages/release/bioc/html/limma.html) 
which has to be installed differently (see also help on [Bioconductor](https://bioconductor.org)):
```{r install3, echo=TRUE, eval=FALSE}
## Installation of limma 
BiocManager::install("limma")
```

This vignette is also accessible from R command-line or on CRAN at [wrMisc](https://CRAN.R-project.org/package=wrMisc):
```{r install4, echo=TRUE, eval=FALSE}
## Now you can open this vignette out of R:
vignette("wrMiscVignette1", package="wrMisc")
```

Before using the functions of this package, we actually need to load the package first (best on a fresh R-session):

```{r setup1}
library("wrMisc")
library("knitr")

## This is 'wrMisc' version number :
packageVersion("wrMisc")
```



## Speed Optimized Functions In The Package wrMisc

In high-throughput experiments in biology (like transcriptomics, proteomics etc...) many different features get measured a number if times (different samples like patients or evolution of a disease). The resulting data typically contain many (independent) rows (eg >1000 different genes or proteins who's abundance was measured) and much fewer columns that may get further organized in groups of replicates. 
As R is a versatile language, multiple options exist for assessing the global characteristics of such data, some are more efficient on a computational point of view.
In order to allow fast treatment of very large data-sets some tools have been re-designed for optimal performance.


### Assessing Basic Information About Variability (for matrix)

Many measurement techniques applied in high throughput manner suffer from precision.
This means, the same measurements taken twice in a row (ie repeated on the same subject) will very likely not give an identical result. 
For this reason it is common practice to make replicate measurements to i) estimate mean (ie representative) values and ii) asses the factors contributing to the variablity observed.
Briefly, technical replicates represent the case where multiple read-outs of the very same sample are generated and the resulting variability is associated to technical issues during the process of taking measures. Biological replicates represent independant samples and reflect therefore the varibility a given parameter may have in a certain population of individuals.
With the tools presented here, both technical and biological replicates can be dealt with. 
In several cases the interpretation of the resulting numbers should consider the experimental setup, though.

Let's make a simple matrix as toy data:
```{r basicVariability, echo=TRUE}
grp1 <- rep(LETTERS[1:3], c(3,4,3))
sampNa1 <- paste0(grp1, c(1:3,1:4,1:3))
set.seed(2016); dat1 <- matrix(round(c(runif(50000) +rep(1:1000,50)),3), 
  ncol=10, dimnames=list(NULL,sampNa1))
dim(dat1)
head(dat1)
```

Now lets estimate the standard deviation _(sd)_ for every row:
```{r sdForEachRow, echo=TRUE}
head(rowSds(dat1))
system.time(sd1 <- rowSds(dat1))
system.time(sd2 <- apply(dat1, 1, sd))
```

On most systems the equivalent calculation using *apply()* will run much slower compared to `rowSds`.

Note, there is a minor issue with rounding :
```{r usingApply, echo=TRUE}
table(round(sd1, 13)==round(sd2, 13))
```

Similarly we can easily calculate the CV (coefficient of variation, ie sd / mean, see also [CV](https://en.wikipedia.org/wiki/Coefficient_of_variation)) for every row using `rowCVs` :

```{r calculateRowCV, echo=TRUE}
system.time(cv1 <- rowCVs(dat1))
system.time(cv2 <- apply(dat1, 1, sd) / rowMeans(dat1))
# typically the calculation using rowCVs is much faster
head(cv1)
# results from the 'conventional' way
head(cv2)
```

Note, these calculations will be very efficient as long as the number of rows is much higher (>>) than the number of columns.


### Data Organized In (Sub-)Groups As Sets Of Columns

Now, let's assume our data is contains 3 initial samples measured as several replicates (already defined in _grp1_).
Similarly, we can also calculate the sd or CV for each line while splitting into groups of replicates (functions `rowGrpMeans`, `rowGrpSds` and `rowGrpCV`): 

```{r rowGrpMeans1, echo=TRUE}
# we already defined the grouping :
grp1

## the mean for each group and row
system.time(mean1Gr <- rowGrpMeans(dat1, grp1))
```

```{r sdOrCVbyGrp, echo=TRUE}
## Now the sd for each row and group
system.time(sd1Gr <- rowGrpSds(dat1, grp1))
# will give us a matrix with the sd for each group & line 
head(sd1Gr)

# Let's check the results of the first line :
sd1Gr[1,] == c(sd(dat1[1,1:3]), sd(dat1[1,4:7]), sd(dat1[1,8:10]))

# The CV :
system.time(cv1Gr <- rowGrpCV(dat1, grp1))
head(cv1Gr)
```


#### Counting Number Of NAs Per Row And Group Of Columns

Some data, like with quantitative proteomics measures, may contain an elevated number of _NAs_ (see also the package [wrProteo](https://CRAN.R-project.org/package=wrProteo) for further options for dealing with such data). 
Furthermore, many other packages on CRAN and Bioconductor cover this topic, see also the [missing data task-view](https://CRAN.R-project.org/view=MissingData) on CRAN.
Similar as above there is an easy way to count the number of _NAs_ to get an overview how NAs are distributed. 

Let's assume we have measures from 3 groups/samples with 4 replicates each :
```{r rowGrpNA1, echo=TRUE}
mat2 <- c(22.2, 22.5, 22.2, 22.2, 21.5, 22.0, 22.1, 21.7, 21.5, 22, 22.2, 22.7,
   NA, NA, NA, NA, NA, NA, NA, 21.2,   NA, NA, NA, NA,
   NA, 22.6, 23.2, 23.2,  22.4, 22.8, 22.8, NA,  23.3, 23.2, NA, 23.7,
   NA, 23.0, 23.1, 23.0,  23.2, 23.2, NA, 23.3,  NA, NA, 23.3, 23.8)
mat2 <- matrix(mat2, ncol=12, byrow=TRUE)
## The definition of the groups (ie replicates)
gr4 <- gl(3, 4, labels=LETTERS[1:3])
```

Now we can easily count the number of NAs per row and set of replicates.
```{r rowGrpNA2, echo=TRUE}
rowGrpNA(mat2,gr4)
```


### Fast NA-omit For Very Large Objects

The function _na.omit()_ from the package _stats_ also keeps a trace of all omitted instances. 
This can be penalizing in terms of memory usage when handling very large vectors with a high content of NAs (eg >10000 NAs). 
If you don't need to document precisely which elements got eliminated, the function `naOmit()` may offer 
smoother functioning for very large objects. 

```{r naOmit, echo=TRUE}
aA <- c(11:13,NA,10,NA)
 
str(naOmit(aA))

# the 'classical' na.omit also stores which elements were NA
str(na.omit(aA))
```

  
### Minimum Distance/Difference Between Values

If you need to find the closest neighbour(s) of a numeric vector, the function `minDiff()` will tell you the 
distance ("dif","ppm" or "ratio") and index ("best") of the closest neighbour. 
In case of multiple shortest distances the index if the first one is reported, and the column "nbest" will display a value of >1.

```{r minDiff, echo=TRUE}
set.seed(2017); aa <- 10 *c(0.1 +round(runif(20),2), 0.53, 0.53)
head(aa)

minDiff(aa,ppm=FALSE)
```

When you look at the first line, the value of 10.2 has one single closest value which is 10.4,
which is located in line number 19 (the column 'best' gives the index of the best).
Line number 19 points back to line number 1. 
You can see, that some elements (like 5.7) occure multiple times (line no 3 and 9), multiple occurences are counted in the column _ncur_.
This is why column _nbest_ for line 15 (_value_ =6.0) indicates that it appears twice as closest value _nbest_.



## Working With Lists (And Lists Of Lists)   {#WorkingWithLists}

### Partial unlist
When input from different places gets collected and combined into a list, this may give a collection of different types of data.
The function `partUnlist()` will to preserve multi-column elements as they are (and just bring down one level):
```{r partUnlist_1, echo=TRUE}
bb <- list(fa=gl(2,2), ve=31:33, L2=matrix(21:28,ncol=2), li=list(li1=11:14,li2=data.frame(41:44)))
partUnlist(bb)
partUnlist(lapply(bb,.asDF2))
```

This won't be possible using _unlist()_.
```{r unlist_1, echo=TRUE}
head(unlist(bb, recursive=FALSE))
```

To uniform such data to obtain a list with one column only for each list-element, the function `asSepList()` provides help : 
```{r asSepList, echo=TRUE}
bb <- list(fa=gl(2,2), ve=31:33, L2=matrix(21:28,ncol=2), li=list(li1=11:14,li2=data.frame(41:44)))
asSepList(bb)
```

### Appending/Combining Lists

Separate lists may be combined using the _append()_ command, which also allows treating simple vectors.

```{r lappend1, echo=TRUE}
li1 <- list(a=1, b=2, c=3)
li2 <- list(A=11, b=2, C=13)
append(li1, li2)
```

However, this way there is no checking if some of the list-elements are present in both lists and thus will appear twice.
The function `appendNR()` allows to checking if some list-elements will appear twice, and thus avoid such duplicate entries.


```{r lappend2, echo=TRUE}
appendNR(li1, li2)
```


### rbind On Lists

When a matrix (or data.frame) gets split into a list, like in the example using _by()_, as a reverse-function such lists can get joined using `lrbind()` in an _rbind_-like fashion.

```{r lrbind, echo=TRUE}
dat2 <- matrix(11:34, ncol=3, dimnames=list(letters[1:8], colnames=LETTERS[1:3]))
lst2 <- by(dat2, rep(1:3,c(3,2,3)), as.matrix)
lst2

# join list-elements (back) into single matrix
lrbind(lst2)
```


#### cbind On Lists

In order to combine list-elements into a data.frame (or matrix), base R can do the job - as long as there is no problem with the number of rows.

```{r lrbind2, echo=TRUE}
li2 <- list(a=11:13, b=matrix(21:26, nc=2, dimnames=list(letters[11:13],c("B","C"))), data.frame(d=41:43, e=51:53) )

## cbind like (to data.frame) 
as.data.frame(li2) 

## cbind like and convert into matrix (as long as all data are of same type, ie here all numeric)
as.matrix(as.data.frame(li2)) 
```


### Merge Multiple Matrices From List

When combining different datasets the function  `mergeMatrixList()` allows merging multiple matrices (or data.frames) into a single matrix.
Two types of mode of operation are available : i) Returning only the common/shared elements (as defined by the rownames), this is default _mode='intersect'_ ;
alternatively one may ii) fuse/merge all matrices together without any loss of data (using _mode='union'_, additional _NA_s may appear when a given rowname is absent in one of the input matrices).

Furthermore, one may specifically select which columns should be used for fusing using the argument _useColumn_.


```{r mergeMatrixList, echo=TRUE}
mat1 <- matrix(11:18, ncol=2, dimnames=list(letters[3:6],LETTERS[1:2]))
mat2 <- matrix(21:28, ncol=2, dimnames=list(letters[2:5],LETTERS[3:4]))
mat3 <- matrix(31:38, ncol=2, dimnames=list(letters[c(1,3:4,3)],LETTERS[4:5]))
#
mergeMatrixList(list(mat1, mat2), useColumn="all")

# with custom names for the individual matrices
mergeMatrixList(list(m1=mat1, m2=mat2, mat3), mode="union", useColumn=2)
```

Similarly, separate entries may be merged using  `mergeMatrices()` :

```{r mergeMatrices, echo=TRUE}
mergeMatrices(mat1, mat2)

mergeMatrices(mat1, mat2, mat3, mode="union", useColumn=2)

## custom names for matrix-origin
mergeMatrices(m1=mat1, m2=mat2, mat3, mode="union", useColumn=2)

## flexible/custom selection of columns
mergeMatrices(m1=mat1, m2=mat2, mat3, mode="union", useColumn=list(1,1:2,2))
```



### Fuse Content Of List-Elements With Redundant (Duplicated) Names

When list-elements have the same name, their content (of named numeric or character vectors) 
may get fused using `fuseCommonListElem()` according to the names of the list-elements :

```{r fuseCommonListElem, echo=TRUE}
val1 <- 10 +1:26
names(val1) <- letters
(lst1 <- list(c=val1[3:6], a=val1[1:3], b=val1[2:3] ,a=val1[12], c=val1[13]))

## here the names 'a' and 'c' appear twice :
names(lst1)

## now, let's fuse all 'a' and 'c'
fuseCommonListElem(lst1)
```


### Filtering Lines And/Or Columns For All List-Elements Of Same Size

In a number of cases the information in various list-elements is somehow related.
Eg, in S3-objects produced by [limma](https://bioconductor.org/packages/release/bioc/html/limma.html), or data produced using [wrProteo](https://CRAN.R-project.org/package=wrProteo) several instances of matrix or data.frame refer to data that are related. 
Some matrixes may conatain abundance data (or weights, etc) while another matrix or data.frame may contain the annotation information related to each line of the abundance data.
So if one wants to filter the data, ie remove some lines, this should be done in the same way with all related list-elements. 
This way one may maintain a conventient 1:1 matching of lines.

The function `filterLiColDeList()` searches if other list-elements have suitable dimensions and will then run the same filtering as in the 'target' list-element.
In consequence this can be used with the output of wrProteo to remove simultaneously the same lines and/or columns.

```{r listBatchReplace1, echo=TRUE}
lst1 <- list(m1=matrix(11:18, ncol=2), m2=matrix(21:30, ncol=2), indR=31:34,
  m3=matrix(c(21:23,NA,25:27,NA), ncol=2))
filterLiColDeList(lst1, useLines=2:3)
filterLiColDeList(lst1, useLines="allNA", ref=3)
```


### Replacements In List

The function `listBatchReplace()` works similar to _sub()_ and allows to search & replace exact matches to a character string along all elements of a list. 

```{r replInList1, echo=TRUE}
(lst1 <- list(aa=1:4, bb=c("abc","efg","abhh","effge"), cc=c("abdc","efg","efgh")))
listBatchReplace(lst1, search="efg", repl="EFG", silent=FALSE)
```


### Organize Values Into list and Sort By Names 

Named numeric or character vectors can be organized into lists using `listGroupsByNames()`, 
based on their names (only the part before any extensions starting with a point gets considered).
Of course, other separators may be defined using the argument _sep_. 

```{r listGroupsByNames, echo=TRUE}
ser1 <- 1:7; names(ser1) <- c("AA","BB","AA.1","CC","AA.b","BB.e","A")

listGroupsByNames(ser1)
```
If no names are present, the content of the vector itself will be used as name :
```{r listGroupsByNames2, echo=TRUE}
listGroupsByNames((1:10)/5)
```

### Batch-filter List-Elements

In the view of object-oriented programming several methods produce results integrated into lists or S3-objects (eg 
[limma](https://bioconductor.org/packages/release/bioc/html/limma.html)).
The function `filterList()` aims facilitating the filtering of all elements of lists or S3-objects. 
List-elements with inappropriate number of lines will be ignored.

```{r filterList, echo=TRUE}
set.seed(2020); dat1 <- round(runif(80),2)
list1 <- list(m1=matrix(dat1[1:40], ncol=8), m2=matrix(dat1[41:80], ncol=8), other=letters[1:8])
rownames(list1$m1) <- rownames(list1$m2) <- paste0("line",1:5)
# Note: the list-element list1$other has a length different to that of filt. Thus, it won't get filtered.
filterList(list1, list1$m1[,1] >0.4)       # filter according to 1st column of $m1 ...
filterList(list1, list1$m1 >0.4) 
```

### Transform Columns Of Matrix To List Of Vectors

At some occasions it may be useful separate columns of a matrix into separate vectors inside a list.
This can be done using `matr2list()`:

```{r matr2list, echo=TRUE}
(mat1 <- matrix(1:12, ncol=3, dimnames=list(letters[1:4],LETTERS[1:3])))
str(matr2list(mat1))
```




## Working With Arrays  {#WorkingWithArrays}

Let's get stared with a little toy-array:
```{r array0, echo=TRUE}
(arr1 <- array(c(6:4,4:24), dim=c(4,3,2), dimnames=list(c(LETTERS[1:4]),
  paste("col",1:3,sep=""),c("ch1","ch2"))))
```

### CV (Coefficient Of Variance) With Arrays

Now we can obtain the CV (coefficient of variance) by splitting along 3rd dimesion (ie this is equivalent to an _apply_ along the 3rd dimension) using `arrayCV()`:

```{r arrayCV1, echo=TRUE}
arrayCV(arr1)

# this is equivalent to
cbind(rowCVs(arr1[,,1]), rowCVs(arr1[,,2]))
```
  
Similarly we can split along any other dimension, eg the 2nd dimension : 
```{r arrayCV2, echo=TRUE}
arrayCV(arr1, byDim=2)
```

### Slice 3-dim Array In List Of Matrixes (Or Arrays)

This procedure is similar to (re-)organizing an initial array into clusters, here we split along a user-defined factor/vector.
If a clustering-algorithm produces the cluster assignments, this function can be used to organize the input data accordingly using `cutArrayInCluLike()`.

```{r cutArrayInCluLike, echo=TRUE}
cutArrayInCluLike(arr1, cluOrg=c(2,1,2,1))
```

Let's cut by filtering along the 3rd dimension for all lines where column 'col2' is >7, and then display only the content of columns 'col1' and 'col2' (using `filt3dimArr()`):

```{r filt3dimArr, echo=TRUE}
filt3dimArr(arr1, displCrit=c("col1","col2"), filtCrit="col2", filtVal=7, filtTy=">")
```



## Working With Redundant Data  {#WorkingWithRedundantData}

$_Semantics_$ : Please note, that there are two ways of interpreting the term '**unique**' :

* In regular understanding one describes this way an event which occurs only once, and thus does not occur/happen anywhere else.

* The command `unique()` will eliminate redundant entries to obtain a shorter 'unique' output vector, ie in the resultant vector all values/content (values) occur only once. 
However, from the result of _unique()_ you cannot tell any more which ones were not unique initially ! 

In some applications (eg proteomics) initial identifiers (IDs) may occur multiple times in the data and we frequently need to identify events/values that occur only once, as the first meaning of '_unique_'. 
This package provides (additional) functions to easily distinguish values occurring just once (ie _unique_) from those occurring multiple times. Furthermore, there are functions to rename/remove/combine replicated elements, eg `correctToUnique()` or `nonAmbiguousNum()`, so that no elements or lines of data get lost.


### Identify What Is Repeated (and Where Repeated Do Occur)

```{r repeated1, echo=TRUE}
## some text toy data
tr <- c("li0","n",NA,NA, rep(c("li2","li3"),2), rep("n",4))
```
The function _table()_ (from the package _base_) is very useful get some insights when working with smaller objects, but may be slow to handle very large objects.
As mentioned, _unique()_ will make everything unique, and afterwards you won't know any more who was unique in the first place !
The function `duplicated()` (also from package base) helps us getting the information who is repeated.

```{r repeated2, echo=TRUE}
table(tr)
unique(tr) 
duplicated(tr, fromLast=FALSE)
```

```{r repeated3, echo=TRUE}
aa <- c(11:16,NA,14:12,NA,14)
names(aa) <- letters[1:length(aa)]
aa
```

`findRepeated()` (from this package) will return the position/index (and content/value) of repeated elements. However, the output in form of a list is not very convenient to the human reader.

```{r findRepeated, echo=TRUE}
findRepeated(aa) 
```


`firstOfRepeated()` tells the index of the first instance of repeated elements, which elements you need to make the vector 'unique', and which elements get stripped off when making unique.
Please note, that _NA_ (no matter if they occure once or more times) are automatically in the part suggested to be removed.
```{r firstOfRepeated, echo=TRUE}
firstOfRepeated(aa)

aa[firstOfRepeated(aa)$indUniq]          # only unique with their names

unique(aa)                               # unique() does not return any names !
```


### Correct Vector To Unique (While Maintaining The Original Vector Length)

If necessary, a counter can be added to non-unique entries, thus no individual values get eliminated and the length and order of the resultant object maintains the same using `correctToUnique()`.

This is of importance when assigning rownames to a data.frame : Assigning redundant values/text as rownames of a data.frame will result in an error !

```{r correctToUnique1, echo=TRUE}
correctToUnique(aa)

correctToUnique(aa, sep=".", NAenum=FALSE)       # keep NAs (ie without transforming to character)
```
You see from the last example above, that this function has an argument for controlling enumerating elements.

### Mark Any Duplicated (ie Ambiguous) Elements by Changing Their Names (and Separate from Unqiue)

First, the truly unique values are reported and then the first occurance of repeated elements is given, _NA_ instances get ignored.
This can be done using `nonAmbiguousNum()` which maintains the length of the initial character vector.

```{r nonAmbiguousNum, echo=TRUE}
unique(aa)                                    # names are lost

nonAmbiguousNum(aa)
nonAmbiguousNum(aa, uniq=FALSE, asLi=TRUE)    # separate in list unique and repeated 
```



### Compare Multiple Vectors And Sort By Number Of Common/Repeated Values/Words

The main aim of the function `sortByNRepeated()` is allowing to compare multiple vectors for common values/words and providing an output sorted by number of repeats.

Suppose 3 persons are asked which cities they wanted to visit. 
Then we would like to make a counting of the most frequently cited cities.
Here we consider individual choices as equally ranked.
By default intra-repeats are eliminated.

```{r sortByNRepeated, echo=TRUE}
cities <- c("Bangkok","London","Paris", "Singapore","New York City", "Istambul","Delhi","Rome","Dubai")
sortByNRepeated(x=cities[c(1:4)], y=cities[c(2:3,5:8)])

## or (unlimited) multiple inputs via list
choices1 <- list(Mary=cities[c(1:4)], Olivia=cities[c(2:3,5:8)], Paul=cities[c(5:3,9,5)])    # Note : Paul cited NYC twice !
table(unlist(choices1))
sortByNRepeated(choices1)
sortByNRepeated(choices1, filterIntraRep=FALSE)  # without correcting multiple citation of NYC by Paul
```


### Combine Multiple Matrixes Where Some Column-Names Are The Same

Here, it is supposed that you want to join 2 or more matrixes describing different properties of the same collection of individuals (as rows).
Common column-names are interpreted that their respective information should be combined (either as average or as sum). 
This can be done using `cbindNR()` :

```{r cbindNR, echo=TRUE}
## First we'll make some toy data :
(ma1 <- matrix(1:6, ncol=3, dimnames=list(1:2,LETTERS[3:1])))
(ma2 <- matrix(11:16, ncol=3, dimnames=list(1:2,LETTERS[3:5])))

## now we can join 2 or more matrixes  
cbindNR(ma1, ma2, summarizeAs="mean")       # average of both columns 'C'
```


### Filter Matrix To Keep Only First Of Repeated Lines

This ressembles to the functioning of _unique()_, but applies to a user-specified column of the matrix.

```{r firstLineOfDat, echo=TRUE}
(mat1 <- matrix(c(1:6, rep(1:3,1:3)), ncol=2, dimnames=list(letters[1:6],LETTERS[1:2])))
```

The function `firstLineOfDat()` allows to access/extract the first line of repeated instances.

```{r firstLineOfDat2, echo=TRUE}
firstLineOfDat(mat1, refCol=2)
```

This function was rather designed for dealing with character input, it allows concatenating all columns and to remove redundant.

```{r firstOfRepLines, echo=TRUE}
mat2 <- matrix(c("e","n","a","n","z","z","n","z","z","b", 
  "","n","c","n","","","n","","","z"), ncol=2)
firstOfRepLines(mat2, out="conc")

# or as index :
firstOfRepLines(mat2)
```


### Filter To Unique Column-Content Of Matrix, Add Counter And Concatenated Information

```{r nonredDataFrame, echo=TRUE}
(df1 <- data.frame(cbind(xA=letters[1:5], xB=c("h","h","f","e","f"), xC=LETTERS[1:5])))
```
The function `nonredDataFrame()` offers to include a counter of redundant instances encountered (for 1st column specified) :

```{r nonredDataFrame2, echo=TRUE}
nonredDataFrame(df1, useCol=c("xB","xC")) 

# without counter or concatenating
df1[which(!duplicated(df1[,2])),]
# or
df1[firstOfRepLines(df1,useCol=2),]
```



### Get First Of Repeated By Column

```{r get1stOfRepeatedByCol, echo=TRUE}
mat2 <- cbind(no=as.character(1:20), seq=sample(LETTERS[1:15], 20, repl=TRUE),
  ty=sample(c("full","Nter","inter"),20,repl=TRUE), ambig=rep(NA,20), seqNa=1:20)
(mat2uniq <- get1stOfRepeatedByCol(mat2, sortBy="seq", sortSupl="ty"))

# the values from column 'seq' are indeed unique
table(mat2uniq[,"seq"])

# This will return all first repeated (may be >1) but without furter sorting 
#  along column 'ty' neither marking in comumn 'ambig').
mat2[which(duplicated(mat2[,2],fromLast=FALSE)),]
```

### Transform (ambigous) Matrix To Non-ambiguous Matrix (In Respect To Given Column)

```{r nonAmbiguousMat, echo=TRUE}
nonAmbiguousMat(mat1,by=2)
```
Here another example, ambiguous will be marked by an '_' :
```{r nonAmbiguousMat2, echo=TRUE}
set.seed(2017); mat3 <- matrix(c(1:100,round(rnorm(200),2)), ncol=3,
  dimnames=list(1:100,LETTERS[1:3]));
head(mat3U <- nonAmbiguousMat(mat3, by="B", na="_", uniqO=FALSE), n=15)
head(get1stOfRepeatedByCol(mat3, sortB="B", sortS="B"))
```

### Combine Replicates From List To Matrix

```{r combineReplFromListToMatr, echo=TRUE}
lst2 <- list(aa_1x=matrix(1:12, nrow=4, byrow=TRUE), ab_2x=matrix(24:13, nrow=4, byrow=TRUE))
combineReplFromListToMatr(lst2)
```


### Combine Redundant Lines From List with (Multiple) Matrix According to Reference

The function `combineRedundLinesInList()` provides help for combining/summarizing lines of numeric data which may be summaried according to reference vector or matrix (part of the same input-list).
Initial data and reference will be aligned based on rownames and the content of reference (or the column specified by \code{refColNa}).
 
```{r combineRedundLinesInListAcRef, echo=TRUE}
x1 <- list(quant=matrix(11:34, ncol=3, dimnames=list(letters[8:1], LETTERS[11:13])), 
  annot=matrix(paste0(LETTERS[c(1:4,6,3:5)],LETTERS[c(1:4,6,3:5)]), ncol=1, 
  dimnames=list(paste(letters[1:8]),"xx")) )
combineRedundLinesInList(lst=x1, refNa="annot", datNa="quant", refColNa="xx")
```



### Non-redundant Lines Of Matrix

```{r nonRedundLines, echo=TRUE}
mat4 <- matrix(rep(c(1,1:3,3,1),2), ncol=2, dimnames=list(letters[1:6],LETTERS[1:2]))
nonRedundLines(mat4)
```


### Filter For Unique Elements /2

```{r filtSizeUniq, echo=TRUE}
# input: c and dd are repeated  :
filtSizeUniq(list(A="a", B=c("b","bb","c"), D=c("dd","d","ddd","c")), filtUn=TRUE, minSi=NULL)

# here a,b,c and dd are repeated  :
filtSizeUniq(list(A="a", B=c("b","bb","c"), D=c("dd","d","ddd","c")), ref=c(letters[c(1:26,1:3)],
  "dd","dd","bb","ddd"), filtUn=TRUE, minSi=NULL)   
```


### Make Non-redundant Matrix 

```{r makeNRedMatr, echo=TRUE}
t3 <- data.frame(ref=rep(11:15,3), tx=letters[1:15],
  matrix(round(runif(30,-3,2),1), nc=2), stringsAsFactors=FALSE)
  
# First we split the data.frame in list  
by(t3,t3[,1], function(x) x)
t(sapply(by(t3,t3[,1],function(x) x), summarizeCols, me="maxAbsOfRef"))
(xt3 <- makeNRedMatr(t3, summ="mean", iniID="ref"))
(xt3 <- makeNRedMatr(t3, summ=unlist(list(X1="maxAbsOfRef")), iniID="ref"))
```


#### Example : Summarize table for longest of transcripts

In the previous example for each subgroup a summarization was calculated.
In other cases you may just want to select a line according to values from a single column.
Suppose you want to select the line corresponding to the longest transcript.


```{r makeNRedMatr2, echo=TRUE}
set.seed(2024)
df2 <- data.frame(transcrID=paste("TrID", 101:124, sep=""), geneID=paste("geID",rep(201:206,each=4)), 
  geneLe=round(runif(24, min=50, max=1500)), a=101:124, b=224:201)
df2 <- df2[-1*c(1,4:7,12:15),]  

(dfLongest <- makeNRedMatr(df2, summ=unlist(list(X1="maxOfRef")), iniID="geneID"))

 summarizeCols(df2[1:2,c(1:5,3)], me="min")  
 summarizeCols(df2[1:2,c(1:5,3)], me="mean") 
 summarizeCols(df2[1:2,c(1:5,3)], me="maxOfRef") 
 summarizeCols(df2[1:6,c(1:5,3)], me="maxOfRef")

(xt3 <- makeNRedMatr(t3, summ=unlist(list(X1="maxOfRef")), iniID=c("ref")))  
```


### Combine/Reduce Redundant Lines Based On Specified Column

```{r combineRedBasedOnCol, echo=TRUE}
matr <- matrix(c(letters[1:6],"h","h","f","e",LETTERS[1:5]), ncol=3,
  dimnames=list(letters[11:15],c("xA","xB","xC")))
combineRedBasedOnCol(matr, colN="xB")
combineRedBasedOnCol(rbind(matr[1,],matr), colN="xB")
```


### Convert Matrix (eg With Redundant) Row-Names To data.frame

```{r convMatr2df, echo=TRUE}
x <- 1
dat1 <- matrix(1:10, ncol=2)
rownames(dat1) <- letters[c(1:3,2,5)]
## as.data.frame(dat1)  ...  would result in an error
convMatr2df(dat1)
convMatr2df(data.frame(a=as.character((1:3)/2), b=LETTERS[1:3], c=1:3))
tmp <- data.frame(a=as.character((1:3)/2), b=LETTERS[1:3], c=1:3, stringsAsFactors=FALSE)
convMatr2df(tmp)
tmp <- data.frame(a=as.character((1:3)/2), b=1:3, stringsAsFactors=FALSE)
convMatr2df(tmp) 
```


### Find And Combine Points Located Very Close In X/Y Space

```{r combineOverlapInfo, echo=TRUE}
set.seed(2013)
datT2 <- matrix(round(rnorm(200)+3,1), ncol=2, dimnames=list(paste("li",1:100,sep=""),
  letters[23:24]))
# (mimick) some short and longer names for each line
inf2 <- cbind(sh=paste(rep(letters[1:4],each=26), rep(letters,4),1:(26*4),sep=""),
  lo=paste(rep(LETTERS[1:4],each=26), rep(LETTERS,4), 1:(26*4), ",", 
  rep(letters[sample.int(26)],4), rep(letters[sample.int(26)],4), sep=""))[1:100,] 
## We'll use this to test :  
head(datT2, n=10)
## let's assign to each pair of x & y values a 'cluster' (column _clu_, the column _combInf_ tells us which lines/indexes are in this cluster)
head(combineOverlapInfo(datT2, disThr=0.03), n=10)
## it is also possible to rather display names (eg gene or protein-names) instead of index values
head(combineOverlapInfo(datT2, suplI=inf2[,2], disThr=0.03), n=10)
```


### Bin And Summarize Values According To Their Names
```{r getValuesByUnique, echo=TRUE}
dat <- 11:19
names(dat) <- letters[c(6:3,2:4,8,3)]
## Here the names are not unique.
## Thus, the values can be binned by their (non-unique) names and a representative values calculated.

## Let's make a 'datUniq' with the mean of each group of values :
datUniq <- round(tapply(dat, names(dat), mean),1)
## now we propagate the mean values to the full vector 
getValuesByUnique(dat, datUniq)
cbind(ini=dat,firstOfRep=getValuesByUnique(dat, datUniq),
  indexUniq=getValuesByUnique(dat, datUniq, asIn=TRUE))
```


### Regrouping Simultaneaously by Two Factors

For example, if you wish to create group-labels considering the eye- and hair-color of a small group students (supposed a sort of controlled vocabulary was used),
the function `combineByEitherFactor()` will help. So basically, this is an empiric segmentation-approach for two categorical variables.
Please note, that with large data-sets and very disperse data this approach will not provide great results.
In the example below we'll attempt to 'cluster' according to columns _nn_ and _qq_, the resultant cluster number can be found in column _grp_.

```{r combineByEitherFactor, echo=TRUE}
nn <- rep(c("a","e","b","c","d","g","f"),c(3,1,2,2,1,2,1))
qq <- rep(c("m","n","p","o","q"),c(2,1,1,4,4))
nq <- cbind(nn,qq)[c(4,2,9,11,6,10,7,3,5,1,12,8),]
## Here we consider 2 columns 'nn' and 'qq' whe trying to regroup common values
##  (eg value 'a' from column 'nn' and value 'o' from 'qq') 
combineByEitherFactor(nq, 1, 2, nBy=FALSE)
```
The argument _nBy_ simply allows adding an additional column with the group/cluster-number.

```{r combineByEitherFactor2, echo=TRUE}
## the same, but including n by group/cluster
combineByEitherFactor(nq, 1, 2, nBy=TRUE)
## Not running further iterations works faster, but you may not reach 'convergence' immediately
combineByEitherFactor(nq,1, 2, nBy=FALSE)
```

```{r combineByEitherFactor3, echo=TRUE}
##  another example
mm <- rep(c("a","b","c","d","e"), c(3,4,2,3,1))
pp <- rep(c("m","n","o","p","q"), c(2,2,2,2,5))
combineByEitherFactor(cbind(mm,pp), 1, 2, con=FALSE, nBy=TRUE)
```


### Batch Replacing Of Values Or Character-Strings

The function `multiCharReplace()` facilitates multiple replacements in a vector, matrix or data.frame.

```{r multiCharReplace1, echo=TRUE}
# replace character content
x1 <- c("ab","bc","cd","efg","ghj")
multiCharReplace(x1, cbind(old=c("bc","efg"), new=c("BBCC","EF")))

# works also on matrix and/or to replace numeric content : 
x3 <- matrix(11:16, ncol=2)
multiCharReplace(x3, cbind(12:13,112:113))
```

Sometimes data get imported using different encoding for what should be interpreted as _FALSE_ and _TRUE_ :

```{r multiCharReplace2, echo=TRUE}
# replace and return logical vactor
x2 <- c("High","n/a","High","High","Low")
multiCharReplace(x2,cbind(old=c("n/a","Low","High"), new=c(NA,FALSE,TRUE)), convTo="logical")
```


### Multi-to-multi Matching Of (Concatenated) Terms

The function allows to split (if necessary, using _strsplit()_) two vectors and compare each isolated tag (eg identifyer) from the 1st vector/object against each isolated tag from the second vector/object. This runs like a loop of one to many comparisons. The basic output is a list with indexes of which element of the 1st vector/object has matches in the 2nd vector/object. Since this is not convenient to the human reader, tabular output can be created, too.

```{r multiMatch1, echo=TRUE}
aa <- c("m","k","j; aa","m; aa; bb; o","n; dd","aa","cc")
bb <- c("aa","dd","aa; bb; q","p; cc") 
## result as list of indexes
(bOnA <- multiMatch(aa, bb, method="asIndex"))   # match bb on aa
## more convenient to the human reader
(bOnA <- multiMatch(aa, bb))                     # match bb on aa
(bOnA <- multiMatch(aa, bb, method="matchedL"))  # match bb on aa
```


### Comparing Global Patterns        {#ComparingGlobalPatterns}

In most programming languages it is fairly easy to compare _exact_ content of character vectors or factors with unordered levels.
However, sometimes - due to semantic issues - some people may call a color 'purple' while others call it 'violet'.
Thus, without using controled vocabulary the _exact_ terms may vary.

Here, let's address the case, where no dictionaries of controled vocabulary are available for substituting equivalent terms.
Thus, we'll compare 4 vectors of equal length and check if the words/letters used could be substituted to result in the first vector.
Vectors _aa_ and _ab_ have the same global pattern, ie after repeating a word twice it moves to another word.
Vectors _ac_ and _ad_ have different general patterns, either with alternating words or falling back to a word previsously used.

Based and extended on a post on stackoverflow [https://stackoverflow.com/questions/71353218/extracting-flexible-general-patterns/](https://stackoverflow.com/questions/71353218/extracting-flexible-general-patterns/) :


```{r compGlobPat1, echo=TRUE}
aa <- letters[rep(c(3:1,4), each=2)]
ab <- letters[rep(c(5,8:6), each=2)]        # 'same general' pattern to aa
ac <- letters[c(1:2,1:3,3:4,4)]             # NOT 'same general' pattern to any other
ad <- letters[c(6:8,8:6,7:6)]               # NOT 'same general' pattern to any other
```

The basic pattern can be extracted combining match() and unique():
```{r compGlobPat2, echo=TRUE}
## get global patterns
cbind(aa= match(aa, unique(aa)),
  ab= match(ab, unique(ab)),
  ac= match(ac, unique(ac)),
  ad= match(ad, unique(ad)) )
```

Let's make a data.frame with the annotation toy-data from above. 
Each line is supposed to represent a sample, and the columns show different aspects of annotation.

```{r compGlobPat3, echo=TRUE}
bb <- data.frame(ind=1:length(aa), a=aa, b=ab, c=ac, d=ad)
```
Via the function `replicateStructure()` is it possible to compare annotation as different columns for equivalent global patterns.

By default, this function excludes all columns not designating any replicates, like the numbers in the first column ($ind).
Also it will try to find the column with the median number of levels, when comparing to all other columns.

The output is a list with  *\$col* inidicating which column(s) may be used, *\$lev* for the correpsonding global pattern, *\$meth* for the method finally used and 
_\$allCols_ for documenting the global pattern in each column (whether it was selected or not).


```{r compGlobPat4, echo=TRUE}
replicateStructure(bb)
```

Besides, it is also possible to combine all columns if one considers they contribute complementary substructures of the overal annotation.

```{r compGlobPat5, echo=TRUE}
replicateStructure(bb, method="combAll")
```
However, when combining multiple columns it may happen -like in the example above- that finally no more lines remain being considered as replicates.

This can also be found when one column describes the groups and another gives the order of the replicates therein.
However, for calling a (standard) statistical test it may be necessary exclude these replicate-numbers to designate the groups of replicates.

To overcome the problem of loosing the understanding of replicate-structure when combining all factors, it is possible to look for non-orthogonal structures,
ie to try excluding columns which (after combining) would suggest no replicates after combining all columns.
See the example below :

```{r compGlobPat6, echo=TRUE}
replicateStructure(bb, method="combNonOrth")
```

Please see also the function _chooseGroupNames()_ which may help choosing among different types of patterns (ie low or high number of groups of replicates) (see section [Working With Redundant Data] (#WorkingWithRedundantData)).



## Search For Similar (Numeric) Values  {#SearchForSimilarNumericValues}

This section addresses values that are not truly _identical_ but may differ only in the very last digit(s) 
and thus may be in a pragmatic view get considered and treated as 'about the same'.
The simplest approach would be to round values and then look for identical values. 
The functions presented here (like `checkSimValueInSer()`) offer this type of search in a convenient way.

Of course the user must define a threshold for how similar may retained as positive (in the the logical vector returned).
With the function _checkSimValueInSer()_ this threshod must be given as [ppm](https://simple.wikipedia.org/wiki/Parts_per_million) (parts per million).

```{r checkSimValueInSer, echo=TRUE}
va1 <- c(4:7,7,7,7,7,8:10) + (1:11)/28600
checkSimValueInSer(va1, ppm=5)
data.frame(va=sort(va1), simil=checkSimValueInSer(va1))
```


### Find Similar Numeric Values Of Two Columns Of A Matrix

The search for similar values may be preformed as absolute distance or as 'ppm' (as it is eg usual in proteomics when comparing measured and theoretically expected mass).

```{r findCloseMatch1, echo=TRUE}
aA <- c(11:17); bB <- c(12.001,13.999); cC <- c(16.2,8,9,12.5,15.9,13.5,15.7,14.1,5)
(cloMa <- findCloseMatch(x=aA, y=cC, com="diff", lim=0.5, sor=FALSE))       
```
The result of _findCloseMatch()_ is a list organized by each 'x', telling all instances of 'y' found within the distance tolerance given by _lim_.
Using `closeMatchMatrix()` the result obtained above, can be presented in a more convenient format for the human eye.

```{r closeMatchMatrix2, echo=TRUE}
# all matches (of 2d arg) to/within limit for each of 1st arg ('x'); 'y' ..to 2nd arg = cC
# first let's display only one single closest/best hit
(maAa <- closeMatchMatrix(cloMa, aA, cC, lim=TRUE))  #
```

Using the argument _limitToBest=FALSE_ we can display all distances within the limits imposed, some values/points may occur multiple times.
For example, value number 4 of 'cC' (=12.5) or value number 3 of 'aA' (=13) now occur multiple times...

```{r closeMatchMatrix3, echo=TRUE}
(maAa <- closeMatchMatrix(cloMa, aA, cC, lim=FALSE,origN=TRUE))  #
(maAa <- closeMatchMatrix(cloMa, cbind(valA=81:87, aA), cbind(valC=91:99, cC), colM=2,
  colP=2, lim=FALSE))
(maAa <- closeMatchMatrix(cloMa, cbind(aA,valA=81:87), cC, lim=FALSE))  #
a2 <- aA; names(a2) <- letters[1:length(a2)];  c2 <- cC; names(c2) <- letters[10 +1:length(c2)]
(cloM2 <- findCloseMatch(x=a2, y=c2, com="diff", lim=0.5, sor=FALSE)) 
(maA2 <- closeMatchMatrix(cloM2, predM=cbind(valA=81:87, a2),
  measM=cbind(valC=91:99, c2), colM=2, colP=2, lim=FALSE, asData=TRUE))
(maA2 <- closeMatchMatrix(cloM2, cbind(id=names(a2), valA=81:87,a2), cbind(id=names(c2),
  valC=91:99,c2), colM=3, colP=3, lim=FALSE)) 
```


### Find Similar Numeric Values From Two Vectors/Matrixes

For comparing two sets of data one may use `findSimilarFrom2sets()`.

```{r findSimilFrom2sets, echo=TRUE}
aA <- c(11:17); bB <- c(12.001,13.999); cC <- c(16.2,8,9,12.5,12.6,15.9,14.1)
aZ <-  matrix(c(aA,aA+20), ncol=2, dimnames=list(letters[1:length(aA)],c("aaA","aZ")))
cZ <-  matrix(c(cC,cC+20), ncol=2, dimnames=list(letters[1:length(cC)],c("ccC","cZ")))
findCloseMatch(cC, aA, com="diff", lim=0.5, sor=FALSE)
findSimilFrom2sets(aA, cC)
findSimilFrom2sets(cC, aA)
findSimilFrom2sets(aA, cC, best=FALSE)
findSimilFrom2sets(aA, cC, comp="ppm", lim=5e4)
findSimilFrom2sets(aA, cC, comp="ppm", lim=9e4, bestO=FALSE)
# below: find fewer 'best matches' since search window larger (ie more good hits compete !)
findSimilFrom2sets(aA, cC, comp="ppm", lim=9e4, bestO=TRUE)
```


### Fuse Previously Identified Pairs To 'Clusters'

When you have already identified the closest neighbour of a set of values, you may want to 
re-organize/fuse such pairs to a given number of total clusters (using `fusePairs()`).

```{r fusePairs, echo=TRUE}
(daPa <- matrix(c(1:5,8,2:6,9), ncol=2))
fusePairs(daPa, maxFuse=4)
```

### Eliminate Close (Overlapping) Points (In Bivariate x & y Space)

When visualizing larger data-sets in an x&y space one may find many points overlapping when their values are almost the same.  
The function `elimCloseCoord()` aims to do reduce a bivariate data-set to 'non-overlapping' points, somehow similar to human perception.

```{r elimCloseCoord1, echo=TRUE}
da1 <- matrix(c(rep(0:4,5),0.01,1.1,2.04,3.07,4.5), ncol=2); da1[,1] <- da1[,1]*99; head(da1)
elimCloseCoord(da1)
```


### Mode Of (Continuous) Data

Looking for the _mode_ is rather easy with counting data, the result of _table()_ will get you there quickly.
However, with continuous data the mode may be more tricky to defne and identify. 
Intuitively most people consider the mode asthe peak of a density estimation (which remains to be defined and estimated).
With continuous data most frequent (precise) value may be quite different/distant to the most dense region of data.
The function `stableMode()` presented here has different modes of operation, at this point there is no clear rule which mode may perform most satisfactory in different situations. 

```{r stableMode, echo=TRUE}
set.seed(2012); dat <- round(c(rnorm(120,0,1.2), rnorm(80,0.8,0.6), rnorm(25,-0.6,0.05), runif(200)),3)
dat <- dat[which(dat > -2 & dat <2)]
stableMode(dat)
```

Now we can try to show on a plot :
```{r stableMode2, fig.height=8, fig.width=9, fig.align="center",  echo=TRUE}
layout(1:2)
plot(1:length(dat), sort(dat), type="l", main="Sorted Values", xlab="rank", las=1)
abline(h=stableMode(dat, silent=TRUE), lty=2,col=2)
legend("topleft",c("stableMode"), text.col=2, col=2, lty=2, lwd=1, seg.len=1.2, cex=0.8, xjust=0, yjust=0.5) 


plot(density(dat, kernel="gaussian", adjust=0.7), xlab="Value of dat", main="Density Estimate Plot")
useCol <- c("red","green","blue","grey55")
legend("topleft",c("dens","binning","BBmisc","allModes"), text.col=useCol, col=useCol,
  lty=2, lwd=1, seg.len=1.2, cex=0.8, xjust=0, yjust=0.5) 
abline(v=stableMode(dat, method="dens", silent=TRUE), lty=2, col="red", lwd=2)
abline(v=stableMode(dat, method="binning", silent=TRUE), lty=2, col="green")
abline(v=stableMode(dat, method="BBmisc", silent=TRUE), lty=2, col="blue")  
abline(v=stableMode(dat, method="allModes"), lty=2, col="grey55")  
```

Please note, that plotting data modelled via a Kernell function (as above) also relies on strong hypothesis which may not be well justified in a number of cases !
For this reason, the _sorted values_ were plotted, too.

As you can see from this example above, looking for the most frequent exact value may not be a perfect choice for continous data. 
In this example the method _'allModes'_ (ie the multiple instances of most frequent exact values) gave partially usable results (dashed grey lines), due to the rounding to 3 digits. 
As you can see in the example above, the method _'allModes'_ may give multiple ties !
More rounding will make to data more discrete and ultimately ressemble cunting data. However, with rounding some of the finer resolution/details will get lost. 



### Most Frequently Occuring Value  (traditional mode)

The function  `stableMode()` can also be used to locate _the_ most frequently occuring exact value of numeric or character vectors.
As we just saw at the end of the previous example, the argument _method="allModes"_ allows finding all ties (if present).

```{r stableMode3, echo=TRUE}
set.seed(2021)
x <- sample(letters, 50000, replace=TRUE)
stableMode(dat, method="mode")
stableMode(dat, method="allModes")
```



## Text-Manipulations    {#Text-Manipulations}

There are several packages offering interesting functions for manipulating text. Here are a few functions to complement these.

### Protect Special Characters
The function `protectSpecChar()` allows protecting the majority of special characters which may influence the outcome of _grep()_ and related functions so thay they can be used easier in searches using _grep()_ or alike.

```{r protectSpecChar1, echo=TRUE}
aa <- c("abc","abcde","ab.c","ab.c.e","ab*c","ab\\d")
grepl("b.", aa)             # all TRUE
grepl(protectSpecChar("b."), aa)
```


### Trimming Redundant Text

Automatic annotation has the tendency to concatenate many parameters into a single names.
The function `trimRedundText()` was designed to allow trimming redundant text from left and/or right side of a character-vector 
(when the same portion of text appears in _each_ element). 
However, as in some cases (like the first element of the example below) nothing would remain, it is possible to define a _minimum_ width for the remaining/resulting text.

```{r trimRedundText1, echo=TRUE}
txt1 <- c("abcd","abcde","abcdefg","abcdE",NA,"abcdEF")
trimRedundText(txt1)
```


### Removing Redundant/Shared Words

A more gentle way to make text non-redundant consists in removing redundant text isporposed by the function `rmSharedWords()`.
Multiple separators may be used (and combined) to specify 'words', a minimum length of what mat be considered a word can be defined, too (default at min 2 characters).
For example, this can be used to make column-names shorter and more precise.

```{r rmSharedWords1, echo=TRUE}
txt2 <- c("abc d","abc de","abc defg","abc dE",NA,"abc dEF")
rmSharedWords(txt2)
```

In contrast to trimRedundText() numeric parts may remain 'intact', see example below.
```{r rmSharedWords2, echo=TRUE}
txt3 <- c("ab 100 m","ab 120 m","ab 1000 m")
trimRedundText(txt3)
rmSharedWords(txt3)
```


### Extract Common Part Of Text

The original idea was to do something resembling the inverse process of trimming redundant text (example above), but this time to discard the variable text.

In the end this is not as trivial when 'common' or 'redundant' text is not at the beginning or end of a chain of characters. 
In particular with very large text this is an active field of research (eg for sequence alignment). 
The function presented here is a very light-weight solution designed for smaller and simple settings, like inspecting column-names.
Furthermore, the function `keepCommonText()` only reports the first (longest) hit. 
So, when there are multiple conserved 'words' of equal length, only the first of them will be identified.

When setting the argument 'hiResol=FALSE' this function has an option to decrease the resulution of searching, which in turn increases the speed, howevere, at cost of missing the optimal solution.
In this case the resultant chain of characters should be inspected if it can be further extended/optimized.

With terminal common text :
```{r keepCommonText1, echo=TRUE}
txt1 <- c("abcd","abcde","abcdefg","abcdE",NA,"abcdEF")
trimRedundText(txt1, side="left")         # remove redundant 
keepCommonText(txt1, side="terminal")     # keep redundant
keepCommonText(txt1, side="center")       # computationally easier   
```

With internal coomon text:
```{r keepCommonText2, echo=TRUE}
txt2 <- c("abcd_abc_kjh", "bcd_abc123", "cd_abc_po")
keepCommonText(txt2, side="center")       
```


### Manipulating Enumerator-Extensions

Human operators may have many ways to write enumerators like 'xx_sample_1', 'xx_Sample_2', 'xx_s3', 'xx_4', etc.
Many times you may find such text as names or column-names for measures underneith.

The functions presented below will work only if _consistent numerators_, ie (text +) digit-character(s) are at the end of all character-strings to be treated.

Please note, that with large vectors testing/checking a larger panel of enumerator-abreviations may result in slower performance.
In cases of such larger data-sets it may be more effective to first study the data and then run simple subsitions using _sub()_ targeted for this very case.


#### Remove/Modify Enumerators

The aim of this function consists in identifying a _common_ pattern for terminal enumeratos (ie at end of words/character strings) and to subsequently modify or remove them.
As separator-symbols and separator-words are given indedently all combinations thereof may be tested.
Furthermore the user has the choice to (automatically) all truncated versions of separator-words (eg _Sam_ instead of _Sample_).

As basic setting `rmEnumeratorName()` allows to identify and then modify a _common_ terminal enumerator from all elements of a character string :
```{r rmEnumeratorName1, echo=TRUE}
xx <- c("hg_Re1","hjRe2_Re2","hk-Re3_Re33")
rmEnumeratorName(xx)
rmEnumeratorName(xx, newSep="--")
rmEnumeratorName(xx, incl="anyCase")
```

Furthermore, this function allows scanning a matrix of text-data and to perform similar operations to the _first_ column found containing a  _common_ terminal enumerator.

```{r rmEnumeratorName2, echo=TRUE}
xy <- cbind(a=11:13, b=c("11#11","2_No2","333_samp333"), c=xx)
rmEnumeratorName(xy)
rmEnumeratorName(xy,incl=c("anyCase","trim2","rmEnumL"))
```

If you which to remove/subsitute mutiple types of enumerators the function \code{rmEnumeratorName} must be run independently, see last example below.

```{r rmEnumeratorName3, echo=TRUE}
xz <- cbind(a=11:13, b=c("23#11","4#2","567#333"), c=xx)
apply(xz, 2, rmEnumeratorName, sepEnum=c("","_"), newSep="_", silent=TRUE)
```


#### Unify Enumerators

The (slightly older) function `unifyEnumerator()` offers less options, in particular the potential separator-words must be given explicitly, only lower/upper-case may be kept flexible.

```{r unifyEnumerator1, echo=TRUE}
unifyEnumerator(c("ab-1","ab-2","c-3"))
unifyEnumerator(c("ab-R1","ab-R2","c-R3"))
unifyEnumerator(c("ab-1","c3-2","dR3"), stringentMatch=FALSE)
```


### Find/Choose Column Most Likely Containing Sample-Names

When information about a collection of samples is given in a table, the function  `unifyEnumerator()` was designed to find the column most likely containg sample-names 
and it derives then the structure of groups of replicates by removing terminal enumerators from sample-names.
Initial tables (as frequently found in transcriptomics and proteomics) may not have consistent schemes of naming columns (due to human annotators not using controlled vocabulary) 
and may contain additional columns with redundant information (elements/conditions that were kept constant for all samples.    

Internally this function uses _rmSharedWords()_ to further simplify resultant names.

```{r chooseGroupNames1, echo=TRUE}
(mat <- cbind(a=letters[1:6], b=paste(rep(c("b","B"), each=3), 1:3), c=rep(1,6), 
  d=gl(3,2), e=rep(c("e","E"),3), f=paste(rep(c("F","f","ff"), each=2), 1:2)))
  
chooseGroupNames(mat, method="median")         # col 2 (b/B)
chooseGroupNames(mat, method="median", fullReport=TRUE) 
chooseGroupNames(mat, method="min")            # col 2 (b/B)
chooseGroupNames(mat, method="max")            # col 6 (F/f/ff)
chooseGroupNames(mat, method="max", asUnique=FALSE) # col 1 (a..)
```

Please see also the function _replicateStructure()_ which allows inspecting patterns of replicating elements (see section [Comparing Global Patterns] (#ComparingGlobalPatterns)).


### Find Common Unit

The function `checkUnitPrefix` aims to find a unit-abbreviation or -name occurring in all elements of a character-vector.
The unit name may be preceeded by different decimal prefixes (eg 'k','M', see argument _pref_) which may vary within the vector.
By default some common SI-units will be searched, see argument _unit_.

```{r checkUnitPrefix1, echo=TRUE}
x1 <- c("10fg WW","xx 10fg 3pW"," 1pg 2.0W")
checkUnitPrefix(x1)

## different separators between digit and prefix:
x2 <- c("10fg WW","xx 8_fg 3pW"," 1 pg-2.0W")
checkUnitPrefix(x2, stringentSearch=TRUE)
checkUnitPrefix(x2, stringentSearch=FALSE)
```



### Adjust Decimal Prefixes And Extact Numeric+Unit Part

The function `adjustUnitPrefix()` provides help extracting the numeric part of character vectors and allows adjusting to a single million-unit type.
This can be used to convert a vector of mixed prefixes like  'z','a','f','p','n','u' and 'm' (note: the 'u' is used for 'micro').
The output is a character vector with all prefix+unit expressions adjusted to use the same prefix everywhere (numeric+separator+prefix+unit). 
Redundant additional text may get (optionally) trimmed (see argument _returnType="trim"_), the numeric part names + unit is give in the names.

Please note that decimal/comma digits will not be recognized properly, the function may/will consider the decimal sign as just another separator 
(as '.' is part af the default selection of separators).

```{r adjustUnitPrefix1, echo=TRUE}
adjustUnitPrefix(c("10.psec", "2 fsec"), unit="sec")
```

Using the argument _returnType_ you can choose how much of the remaining text should be shown.
```{r adjustUnitPrefix2, echo=TRUE}
x2 <- c("abCc 500_nmol ABC", "abEe5_umol", "", "abFF_100_nmol_G", "abGg 2_mol", "abH.1 mmol")
rbind( adjustUnitPrefix(x2, unit="mol", returnType="allText") , 
  adjustUnitPrefix(x2, unit="mol", returnType="trim"),
  adjustUnitPrefix(x2, unit="mol", returnType=""))
```

If the _unit_ -name is not knonw in advance, you can try to figure out. In the example it is shown that the unit-name can be determined using the function _checkUnitPrefix()_.
```{r adjustUnitPrefix3, echo=TRUE}
x3 <- c("2.psec abc","300 fsec etc", "34 5fsec")
adjustUnitPrefix(x3, unit=checkUnitPrefix(x3))
```


### Merging Multiple Named Vectors To Matrix

The function `mergeVectors()` allows merging for multiple named vectors (each element needs to be named).
Basically, all elements carrying the same name across different input-vectors will be aligned in the same column of the output (input-vectors appear as lines).
Different to _merge()_ which allows merging only 2 data.frames, here multiple vectors may be merge at once.

```{r mergeVectors1, echo=TRUE}
x1 <- c(a=1, b=11, c=21)
x2 <- c(b=12, c=22, a=2)
x3 <- c(a=3, d=43)
mergeVectors(vect1=x1, vect2=x2, vect3=x3)
```

```{r mergeVectors2, echo=TRUE}
mergeVectors(vect1=x1, vect2=x2, vect3=x3, inclInfo=TRUE)   # return list with additional info
```

In the example below we'll add another vector _without_ named elements.
As you can see a message tells the this vector was been ignored/omitted.

```{r mergeVectors3, echo=TRUE}
x4 <- 41:44            # no names - not conform for merging and will be ignored
mergeVectors(x1, x2, x3, x4)
```



### Match All Lines of Matrix To Reference Note

This function allows adjusting the order of lines of a matrix \code{mat} to a reference character-vector \code{ref},
even when initial direct matching of character-strings using \code{match} is not possible/successful.
In this case, various variants of using \code{grep} will be used to see if unambiguous matching is possible of characteristic parts of the text.
All columns of \code{mat} will be tested an the column giving the best results will be used.

```{r matchMatrixLinesToRef1, echo=TRUE}
## Note : columns b and e allow non-ambigous match, not all elements of e are present in a
mat0 <- cbind(a=c("mvvk","axxd","bxxd","vv"),b=c("iwwy","iyyu","kvvh","gxx"), c=rep(9,4),
  d=c("hgf","hgf","vxc","nvnn"), e=c("_vv_","_ww_","_xx_","_yy_"))
matchMatrixLinesToRef(mat0[,1:4], ref=mat0[,5])
matchMatrixLinesToRef(mat0[,1:4], ref=mat0[1:3,5], inclInfo=TRUE)

matchMatrixLinesToRef(mat0[,-2], ref=mat0[,2], inclInfo=TRUE)   # needs 'reverse grep'
```



### Order Matrix According To Reference

The function `orderMatrToRef()` has the aim of facilitating brining a matrix of text/data in the order of a given reference (character vector).
This function will try all columns of the input-matrix to see which gives the best coverage/ highest number of matches to the reference.
If no hits are found, this function will try by partial matching (using _grep()_) all entries of the reference and vice-versa all entries of the matrix.

```{r orderMatrToRef1, echo=TRUE}
mat1 <- matrix(paste0("__",letters[rep(c(1,1,2,2,3),3) +rep(0:2,each=5)], rep(1:5)), ncol=3)
orderMatrToRef(mat1, paste0(letters[c(3,4,5,3,4)],c(1,3,5,2,4)))

mat2 <- matrix(paste0("__",letters[rep(c(1,1,2,2,3),3) +rep(0:2,each=5)], c(rep(1:5,2),1,1,3:5 )), ncol=3)
orderMatrToRef(mat2, paste0(letters[c(3,4,5,3,4)],c(1,3,5,1,4)))

mat3 <- matrix(paste0(letters[rep(c(1,1,2,2,3),3) +rep(0:2,each=5)], c(rep(1:5,2),1,1,3,3,5 )), ncol=3)
orderMatrToRef(mat3, paste0("__",letters[c(3,4,5,3,4)],c(1,3,5,1,3)))
```


### Value Matching With Option For Concatenated Terms

Sometimes we need to match terms in concatenated tables.
The function `concatMatch()` was designed to behave similar to _match()_  but also allowing to serach among concatenated terms and some further text-simplifications.

```{r concatMatch1, echo=TRUE}
## simple example without concatenations or text-extensions
x0 <- c("ZZ","YY","AA","BB","DD","CC","D")
tab0 <- c("AA","BB,E","CC","FF,U")
match(x0, tab0)
concatMatch(x0, tab0)         # same result as match(), but with names

## now let's construct somthing similar but with concatenations and text-extensions
x1 <- c("ZZ","YY","AA","BB-2","DD","CCdef","Dxy")            # modif of single ID (no concat)
tab1 <- c("AA","WW,Vde,BB-5,E","CCab","FF,Uef")
match(x1, tab1)                   # match finds only the 'simplest' case (ie "AA")
concatMatch(x1, tab1)             # finds all hits as in example above

x2 <- c("ZZ,Z","YY,Y","AA,Z,Y","BB-2","DD","X,CCdef","Dxy")  # conatenated in 'x'
tab2 <- c("AA","WW,Vde,BB-5,E","CCab,WW","FF,UU")
concatMatch(x2, tab2)               # concatenation in both 'x' and 'table'
```



### Check for (Strict) Order

Thi function `checkStrictOrder()` was designed to scan each line of an (numeric) input matrix for up- down- or equal-development, ie the chang to the next value on the right.
For example when working with a matrix of with 4 columns one can look 3 times a the neighbour value following to the right (in the same line), thus the output will mention 3 events (for each line).
If _all counts_ are 'up' and 0 counts are 'down' or 'eq', the line follows a permanently increase (not necessarily linear), etc.

In some automated procedures (where the numer of columns of initial input may vary) it may be easier to test if any 0 occur.
For this reason the argument _invertCount_ was introduced, in this case a line with a '0' occurring characterizes a constant behaviour (for the respective column).

```{r checkStrictOrder1, echo=TRUE}
set.seed(2005); mat1 <- rbind(matrix(round(runif(40),1),nc=4), rep(1,4))
head(mat1)
checkStrictOrder(mat1); mat1[which(checkStrictOrder(mat1)[,2]==0),]
```

A slightly more general way of testing can be done using  `checkGrpOrder()`. Here, simlpy a logical value will produced for each line of input indicating if there is constant behaviour.
When the argument _revRank=TRUE_ (default) constant up- or constant down-characteristics will be tested

```{r checkGrpOrder1, echo=TRUE}
head(mat1)
checkGrpOrder(mat1)
checkGrpOrder(mat1, revRank=FALSE)    # only constant 'up' tested
```




## Working With Regressions    {#WorkingWithRegressions}

### Best Starting Point For Linear Regressions (Start of linearity)

In many types of measurments the very low level measures are delicate.
Especially when the readout starts with a baseline signal before increasing amounts of the analyte start producing a linear relationship.
In such cases some of the very lowest levels of the analyte are masked by the (random) baseline signal.
The function `linModelSelect()` presented here allows omitting some of the lowest analyte measures to focus on the linear part of the dose-response relationship.

```{r linModelSelect1, echo=TRUE}
li1 <- rep(c(4,3,3:6), each=3) + round(runif(18)/5,2)
names(li1) <- paste0(rep(letters[1:5], each=3), rep(1:3,6))
li2 <- rep(c(6,3:7), each=3) + round(runif(18)/5, 2)
dat2 <- rbind(P1=li1, P2=li2)
exp2 <- rep(c(11:16), each=3)
exp4 <- rep(c(3,10,30,100,300,1000), each=3)

## Check & plot for linear model 
linModelSelect("P1", dat2, expect=exp2)
linModelSelect("P2", dat2, expect=exp2)
```

This function was designed for use with rather small data-sets with no (or very few) measures of base-line.
When larger panels of data ara available, it may be better to first define a confidence interval for the base-line measurement
and then only to consider points outside this confidence interval for regressing dose-response relationships
(see also [Detection limit](https://en.wikipedia.org/wiki/Detection_limit)).



### High Throughput Testing For Linear Regressions

Once we have run multiple linear regressions on differt parts of the data we might wat to compare them in a single plot.
Below, we construct 10 series of data that get modeled the same way, ideally one would obtain a slope close to 1.0.
We still allow omitting some starting points, if the resulting model would fit better.

```{r plotLinModelCoef1, echo=TRUE}
set.seed(2020)
x1 <- matrix(rep(c(2,2:5),each=20) + runif(100) +rep(c(0,0.5,2:3,5),20), 
  byrow=FALSE, ncol=10, dimnames=list(LETTERS[1:10],NULL))
## just the 1st regression :
   summary(lm(b~a, data=data.frame(b=x1[,1], a=rep(1:5,each=2))))
## all regressions
x1.lmSum <- t(sapply(lapply(rownames(x1), linModelSelect, dat=x1, 
  expect=rep(1:5,each=2), silent=TRUE, plotGraph=FALSE), 
  function(x) c(x$coef[2,c(4,1)], startFr=x$startLev)))
x1.lmSum <- cbind(x1.lmSum, medQuantity=apply(x1,1,median))
x1.lmSum[,1] <- log10(x1.lmSum[,1])
head(x1.lmSum)
```

Now we can try to plot :
```{r plotLinModelCoef2, echo=TRUE}
wrGraphOK <- requireNamespace("wrGraph", quietly=TRUE)      # check if package is available
if(wrGraphOK) wrGraph::plotW2Leg(x1.lmSum, useCol=c("Pr(>|t|)","Estimate","medQuantity","startFr"), 
  legendloc="topleft", txtLegend="start at")
```



## Combinatorics Issues   {#CombinatoricsIssues}

### All Pairwise Ratios

`ratioAllComb()` calculates all possible pairwise ratios between all individual calues of x and y.

```{r ratioAllComb0, echo=TRUE}
set.seed(2014); ra1 <- c(rnorm(9,2,1), runif(8,1,2))
```

Let's assume there are 2 parts of 'x' for which we would like to know the representative ratio :
The ratio of medians does not well reflect the typical ratio (if each element has the same chance to be picked).

```{r ratioAllComb1, echo=TRUE}
median(ra1[1:9]) / median(ra1[10:17])
```

Instead, we'll build all possible ratios and summarize then.

```{r ratioAllComb2, echo=TRUE}
summary( ratioAllComb(ra1[1:9], ra1[10:17]))
boxplot(list(norm=ra1[1:9], unif=ra1[10:17], rat=ratioAllComb(ra1[1:9],ra1[10:17])))
```


###  Count Frequency Of Terms Combined From Different Drawings (combineAsN)

The main idea of this function is to count frequency of terms when combining different drawings.
Suppose, you are asking students for their prefered hobbies.
Now, you want to know how many terms will occur in common in groups of 3 students.
In the example below, simple letters are shown instead of names of hobbies ...

In the simplest way of using `combineAsN()` does something similar to _table_ :
Here we're looking at the full combinatorics of making groups of _nCombin_ students and let's count the frequency of terms found 3 times identical, 2 times or only once (ie not cited by the others).
In case multiple groups of _nCombin_ students can be formed, the average of the counts, standard error of the mean (sem), 95% confidence interval (CI) and sd aregiven to resume the results.

```{r combineAsN1, echo=TRUE}
tm1 <- list(a1=LETTERS[1:7], a2=LETTERS[3:9], a3=LETTERS[6:10], a4=LETTERS[8:12])
combineAsN(tm1, nCombin=3, lev=gl(1,4))[,1,]
```

One may imagine that different locations/coties/countries will give different results.
Thus, we'll declare the different origins/location using the _lev_ argument.
Now, this function focusses (by default) on combinations of students from _nCombin_ different origins/location and
counts how many hobbies were mentioned as all different ('sing', ie number of hobbies only one student mentioned),
single repeat ('doub') or three times repeated ('trip'), plus minumum twice or 'any' (ie number of hobies citied no matter how many repeats).
The output is an array, the 3rd dimension contains the counts, fllowed by sem, CI and sd.

```{r combineAsN2, echo=TRUE}
## different levels/groups in list-elements
tm4 <- list(a1=LETTERS[1:15], a2=LETTERS[3:16], a3=LETTERS[6:17], a4=LETTERS[8:19],
  b1=LETTERS[5:19], b2=LETTERS[7:20], b3=LETTERS[11:24], b4=LETTERS[13:25], c1=LETTERS[17:26],
  d1=LETTERS[4:12], d2=LETTERS[5:11], d3=LETTERS[6:12], e1=LETTERS[7:10])
te4 <- combineAsN(tm4, nCombin=4, lev=substr(names(tm4),1,1))
str(te4)
te4[,,1]           # the counts part only
```


## Import/Export

### Batch-Reading Of CSV Files

Some software do produce a series of csv files, where a large experiment/data-set get recorded as multiple files.
The function `readCsvBatch()` was designed for reading multiple csv files of exactly the same layout and to join their content.
As output a list with the content of each file can be produced (one matrix per file), or the data may be fused into an array, as shown below.  


```{r readCsvBatch, echo=TRUE}
path1 <- system.file("extdata", package="wrMisc")
fiNa <-  c("pl01_1.csv","pl01_2.csv","pl02_1.csv","pl02_2.csv")
datAll <- readCsvBatch(fiNa, path1, silent=TRUE)
str(datAll)
```

When setting the first argument _fileNames_ to _NULL_, you can read all files of a given path.
```{r readCsvBatch2, echo=TRUE}
## batch reading of all csv files in specified path :
datAll2 <- readCsvBatch(fileNames=NULL, path=path1, silent=TRUE)
str(datAll2)
```


### Batch-Reading Of Tabulated Files

The function `readTabulatedBatch()` allows fast batch reading of tabulated files. 
All files specified (or all files from a given directory) will be read into separate data.frames of a list.
Default options are US-style comma, automatic testing for head in case the package _data.table_ is available (otheriwse : no header).
Furthermore it is possible to design a given (numeric) column and directly filter for all lines passing a given threshold, allowing to get smaller objects.


```{r readTabulatedBatch1, echo=TRUE}
path1 <- system.file("extdata", package="wrMisc")
fiNa <-  c("a1.txt","a2.txt")
allTxt <- readTabulatedBatch(fiNa, path1)
str(allTxt)
```

### Reading Incomplete Tables

Sometimes were may get confronted with data which look like 'incomplete' tables. 
In such cases some rows do not contain as many elements/columns as other columns. 
Files with this type of data may pose a problem for `read.table()` (from the _utils_ package).
In some cases using the argument _fill=TRUE_ may allow to overcome this problem.
The function _readVarColumns()_ (from this package) was designed to provide better help in such odd cases.
Basically, each line is read and parsed separately, the user should check/decide on the separator to be used.

The example below lists people's names in different locations, some locations have more persons ... 
Sometimes exporting such data will generate shorter lines in locations with fewer elements (here 'London') and no additional separators will get added (to mark all empty fields) towards the end.
The function `readVarColumns()` (from this package) provides help to read such data, if the content (and separators) of the last columns are missing.

```{r readVarColumns, echo=TRUE}
path1 <- system.file("extdata", package="wrMisc")
fiNa <- "Names1.tsv"
datAll <- readVarColumns(fiName=file.path(path1,fiNa), sep="\t")
str(datAll)
```

In this example _readVarColumns()_ would give a warning (and column-names are not recognized), if you use the argument _header=TRUE_ you'll get an error and nothing gets read.



### Converting Url For Reading Tabulated Data From GitHub

GitHub (https://github.com) allows sharing code and data.
In order to properly read tabulated (txt, tsv or csv) data directly from a given url from the GitHub pages, the user typically needs switch to the 'Raw' view.
The function `gitDataUrl()` allows to conventiently switch any url (on GitHub) to the format from 'Raw view', suitable for directly reading the data using _read.delim()_ , _read.table()_ or _read.csv()_ etc ...).

```{r readGit1, echo=TRUE}
## An example url with tabulated data :
url1 <- "https://github.com/bigbio/sdrf-annotated-datasets/blob/main/datasets/PXD001819/PXD001819.sdrf.tsv"
gitDataUrl(url1)
```

The example below shows how this is used in the function _readSampleMetaData()_ in [wrProteo](https://CRAN.R-project.org/package=wrProteo).
```{r readGit2, echo=TRUE}
dataPxd <- try(read.delim(gitDataUrl(url1), sep='\t', header=TRUE))
str(dataPxd)
```

---



## Normalization   {#Normalization}

The main reason of normalization is to remove variability in the data which is not directly linked to the (original/biological) concept of a given experiment. 
High throughput data from real world measurements may easily contain various deformations due to technical reasons, eg slight temperature variations, electromagnetic interference, instability of reagents etc. 
In particular, transferring constant amounts of liquids/reagents in highly repeated steps over large experiments is often also very challenging, small variations of the amounts of liquid (or similar) are typically addressed by normalization. However, applying aggressive normalization to the data also brings considerable risk of starting to loose some of the effects one intended to study.
At some point it may rather be better to eliminate a few samples or branches of an experiment to avoid too invasive intervention. This shows that quality control can be tightly linked to decisions about data-normalization. 
In conclusion, normalization may be far more challenging than simply running some algorithms.. 

In general, the use has to assume/define some hypothesis to justify intervention.
Sometimes specific elements of an experiment are known to be not affected and can therefore be used to normalize the rest.
Eg, if you observe growth of trees in a forest, big blocks of rock on the floor are assumed no to change their location. 
So one could use them as alignment-marks to superpose pictures taken at slightly different positions.

The hypothesis of no global changes is very common : During the course of many biological experiments (eg change of nutrient) one 
assumes that only a small portion of the elements measured (eg the abundance of all different gene-products) do change,
since many processes of a living cell like growth, replication and interaction with neighbour-cells are assumed not to be affected.
So, if one assumes that there are no global changes one normalizes the input-data in a way that the average or median across each experiment will give the same value.
In analogy, if one takes photographs on a partially cloudy day, most cameras will adjust light settings (sun r clouds) so that global luminosity stays the same. 
However, if too many of the measured elements are affected, this normalization approach will lead to (additional) loss of information.

It is _essential_ to understand the type of deformation(s) data may suffer from in order to choose the appropriate approacges for normalization. 
Of course, graphical representations ([PCA](https://en.wikipedia.org/wiki/Principal_component_analysis), [MA-plots](https://en.wikipedia.org/wiki/MA_plot), etc) are extremely important to identifying abnormalities and potential problems. 
The package [wrGraph](https://CRAN.R-project.org/package=wrGraph) offers also complementary options useful in the context of normalization.
Again, graphical representation(s) of the data help to visualize how different normalization procedures affect outcomes.

Before jumping into normalization it may be quite useful to _filter_ the data first.
The overall idea is, that most high-throughput experiments do produce some non-meaningful data (artefacts) and it may be wise to remove such 'bad' data
first, as they may effect normalization (in particular _extreme values_). 
A special case of problematic data concerns _NA_-values.


### Filter Lines Of Matrix To Reduce Content Of NAs

Frequent _NA_-values may represent another potential issue. With NA-values there is no general optimal advice.
To get started, you should try to investigate how and why NA-values occurred to check if there is a special 'meaning' to them.
For example, on some measurement systems values below detection limit may be simply reported as NAs.
If the lines of your data represent different features quantified (eg proteins), than lines with mostly NA-values represent features
that may not be well exploited anyway. Therefore many times one tries to filter away lines of 'bad' data. 
Of course, if there is a column (sample) with an extremely high content of NAs, one should also investigate what might be particular 
with this column (sample), to see if one might be better of to eliminate the entire column.


Please note, that imputing _NA_-values represents another option instead of filtering and removing, multiple other packages address this in detail, too.
All decisions of which approach to use should be data-driven.


#### Filter For Each Group Of Columns For Sufficient Data As Non-NA

Filter for each group of columns for sufficient data as non-NA
The function `presenceGrpFilt()` allows to

```{r presenceGrpFilt1, echo=TRUE}
dat1 <- matrix(1:56,ncol=7)
dat1[c(2,3,4,5,6,10,12,18,19,20,22,23,26,27,28,30,31,34,38,39,50,54)] <- NA
grp1 <- gl(3,3)[-(3:4)]
dat1

## now let's filter
presenceGrpFilt(dat1, grp=grp1, presThr=0.75)  # stringent
presenceGrpFilt(dat1, grp=grp1, presThr=0.25)  # less stringent

presenceGrpFilt(dat1, grp=grp1, presThr=0.25)  # less stringent
```


#### Filter As Separate Pairwise Groups Of Samples

If you want to use your data in a pair-wise view (like running t-tests on each line) the function `presenceFilt()`
allows to eliminate lines containing too many _NA_-values for each pair-wise combination of the groups/levles.

```{r presenceFilt, echo=TRUE}
grp2 <- factor(rep(letters[4:2],c(2,2,3)))
presenceFilt(dat1, gr=grp2, maxGr=1, ratM=0.1)
presenceFilt(dat1, gr=grp2, maxGr=2, rat=0.5)
```

Here it is also possible to custom select only a few pairwise comparisons 
(when an elevated number of groups will trigger a rather high number of pairwise comparisons) :

```{r presenceFilt2, echo=TRUE}
presenceFilt(dat1, gr=grp2, useComp=c("d-c","b-d"), maxGr=0, ratM=0.1)
```


#### Cleaning Replicates

This procedures aims to remove (by setting to as _NA_) the most extreme of noisy replicates.
Thus, it is assumed that all columns of the input matrix (or data.frame) are replicates of the other columns.
The _nOutl_ most distant points are identified and will be set to _NA_.

```{r cleanReplicates, echo=TRUE}

(mat3 <- matrix(c(19,20,30,40, 18,19,28,39, 16,14,35,41, 17,20,30,40), ncol=4))
cleanReplicates(mat3, nOutl=1)
cleanReplicates(mat3, nOutl=3)

```


### The Function normalizeThis()

In biological high-throughput data columns typically represent different samples, which may be organized as replicates. 
During high-throughput experiments thousands of (independent) elements are measured (eg abundance of gene-products), they are represented by rows.
As real-world experiments are not always as perfect as we may think, small changes in the signal measured may easily happen.
Thus, the aim of normalizing is to remove or reduce any trace/variability in the data not related to the original experiement but due to imperfections during detection.

Note, that some experiments may produce a considerable amount of missing data (NAs) which require special attention (dedicated developments exist in other R-packages eg in [wrProteo](https://CRAN.R-project.org/package=wrProteo)). 
My general advice is to first carefully look where such missing data is observed and to pay attention to replicate measurements 
where a given element once was measured with a real numeric value and once as missing information (NA). 


```{r normalizeThis0, echo=TRUE}
set.seed(2015); rand1 <- round(runif(300) +rnorm(300,0,2),3)
dat1 <- cbind(ser1=round(100:1 +rand1[1:100]), ser2=round(1.2*(100:1 +rand1[101:200]) -2),
  ser3=round((100:1 +rand1[201:300])^1.2-3))
dat1 <- cbind(dat1, ser4=round(dat1[,1]^seq(2,5,length.out=100) +rand1[11:110],1))
## Let's introduce some NAs
dat1[dat1 <1] <- NA
## Let's get a quick overview of the data
summary(dat1)
## some selected lines (indeed, the 4th column appears always much higher)
dat1[c(1:5,50:54,95:100),]
```

Our toy data may be normalized by a number of different criteria. 
In real applications the nature of the data and the type of deformation detected/expected will largely help 
deciding which normalization might be the 'best' choice. Here we'll try first normalizing by the mean,
ie all columns will be forced to end up with the same column-mean. 
The trimmed mean does not consider values at extremes (as outliers are frequently artefacts and display extreme values).
When restricting even stronger which values to consider one will eventually end up with the median (3rd method used below).

```{r normalizeThis1, echo=TRUE}
no1 <- normalizeThis(dat1, refGrp=1:3, meth="mean")
no2 <- normalizeThis(dat1, refGrp=1:3, meth="trimMean", trim=0.4)
no3 <- normalizeThis(dat1, refGrp=1:3, meth="median")
no4 <- normalizeThis(dat1, refGrp=1:3, meth="slope", quantFa=c(0.2,0.8))
```

It is suggested to verify normalization results by plots. 
Note, that  [Box plots](https://en.wikipedia.org/wiki/Box_plot) may not be appropriate in some cases (eg multimodal distributions),
for displaying more details you may consider using [Violin-Plots](https://en.wikipedia.org/wiki/Violin_plot) from packages [vioplot](https://CRAN.R-project.org/package=vioplot) or [wrGraph](https://CRAN.R-project.org/package=wrGraph), another option might be a (cumulated) frequency plot (eg in package [wrGraph](https://CRAN.R-project.org/package=wrGraph)).

```{r normalizeThis_plot1, echo=FALSE,eval=TRUE}
boxplot(dat1, main="raw data", las=1)
```

You can see clearly, that the 4th data-set has a problem of range. So we'll see if some proportional normalization
may help to make it more comparable to the other ones.
 
```{r normalizeThis_plot2, echo=FALSE,eval=TRUE}
layout(matrix(1:4, ncol=2))
boxplot(no1, main="mean normalization", las=1)
boxplot(no2, main="trimMean normalization", las=1)
boxplot(no3, main="median normalization", las=1)
boxplot(no4, main="slope normalization", las=1)
```


### Normalize By Rows

The standard approach for normalizing relies on consisting all columns as collections of data who's distribution is not supposed to change.
In some cases/projects we may want to formulate a much more 'aggressive' hypothesis : We consider the content of all columns strictly as the same.
For example this may be the case when comparing with technical replicates only.
In such cases one may use the function `rowNormalize()` which tries to find the average or mean optimal within-line normalization factor.

Besides, an additional mode of operation for _sparse data_ has been added :
Basically, once a row contains just one NA, this row can't be used any more to derive a normalization factor for all rows.
Thus, with many NA-values the number of 'complete' rows will be low or even 0 redering this approach inefficient or impossible.
Once the content of NA-values is above a customizable threshold, the data will be broken in smaller subsets with fewer groups of fewer columns,
 thus increasing the chances of finding 'complete' subsets of data which will be normalized first and added to other subsets in later steps.

This approach relies on the **hypothesis** that *all data in a given line should be (aproximately) the same value* !
Thus, this procedure is particularly well adopted to the case when _all_ samples are multiple replicate measurements of the _same_ sample.


```{r rowNormalize1, echo=TRUE}
set.seed(2); AA <- matrix(rbinom(110, 10, 0.05), nrow=10)
AA[,4:5] <- AA[,4:5] *rep(4:3, each=nrow(AA))

AA1 <- rowNormalize(AA)
round(AA1, 2)
```

Now, let's make this sparse and try normalizing:

```{r rowNormalize2, echo=TRUE}
AC <- AA
AC[which(AC <1)] <- NA

(AC1 <- rowNormalize(AC))
```

Like with _normalizeThis()_ we can define some reference-lines (only these lines will be considered to determine normalization-factors)

```{r rowNormalize3, echo=TRUE}
(AC3 <- rowNormalize(AC, refLines=1:5, omitNonAlignable=TRUE))
```

Please note, that the iterative procedure for _sparse data_ may consume large amounts of computational resources, in particular when
a small number of subgroups has been selected.



### Matrix Coordinates Of Values/Points According To Filtering

Sometimes one needs to obtain the coordinates of values/points of a matrix according to a given filtering condition.
The standard approach using _which()_ gives only a _linearized_ index but not row/column, which is sufficient for replacing indexed values.
If you need to know the true row/column indexes, you may use `coordOfFilt()`.

```{r coordOfFilt1, echo=TRUE}
set.seed(2021); ma1 <- matrix(sample.int(n=40, size=27, replace=TRUE), ncol=9)
## let's test which values are >37
which(ma1 >37)      # doesn't tell which row & col
coordOfFilt(ma1, ma1 >37)
```


### Trimmed Mean

Under certain circumstances using the trimmed mean may be more stable towards outlyer values.
The base function  `mean()` allows to preform symmetric trimming, thus with more trimming the result will start converging to the _mean_.
The function `trimmedMean()` gives more flexible options for assinging different upper- and lower fractions of the initial data to be trimmed.
When outlyer values always appear at the high (or low) end of data, such proceeding may be useful.
However, the user is encouraged to use this with caution since there is also a risk of introducing bias.


```{r trimmedMean1, echo=TRUE}
x <- c(17:11,27:28)
mean(x)
mean(x, trim=0.15)       # symmetric trimming
mean(x[x < 25])          # manual trimming
trimmedMean(x, trim=c(l=0, u=0.7))   # asymmetric trim
```



## Statistical Testing  {#StatisticalTesting}

### Normal Random Number Generation with Close Fit to Expected mean and sd

When creating random values to an expected _mean_ and _sd_, the results ontained using the standard function `rnorm()` 
may deviate somehow from the expected mean and sd, in particular with low _n_. 
To still produce random values fitting closely to the expected _mean_ and _sd_ you may use the function `rnormW()`. 
The case of _n=2_ is quite simple with one possible results. 
In other cases (_n>2_), there will be a random initiation which can be fixed using the argument _seed_.


```{r rnormW1, echo=TRUE}
## some sample data :
x1 <- (11:16)[-5]
mean(x1); sd(x1)
```

```{r rnormW2, echo=TRUE}
## the standard way for gerenating normal random values
ra1 <- rnorm(n=length(x1), mean=mean(x1), sd=sd(x1))
## In particular with low n, the random values deviate somehow from expected mean and sd :
mean(ra1) -mean(x1) 
sd(ra1) -sd(x1)
```

```{r rnormW3, echo=TRUE}
## random numbers with close fit to expected mean and sd :
ra2 <- rnormW(length(x1), mean(x1), sd(x1))
mean(ra2) -mean(x1) 
sd(ra2) -sd(x1)   # much closer to expected value
```

Thus, the second data-sets fits even with few _n_ very well to the global characteristics defined/expected.


### Moderated Pair-Wise t-Test from limma

If you are not familiar with the way data is handled in the Bioconductor package [limma](https://bioconductor.org/packages/release/bioc/html/limma.html) 
and you would like to use some of the tools for running moderated t-tests therein, this will provide easy access using `moderTest2grp()` :
```{r moderTest2grp, echo=TRUE}
set.seed(2017); t8 <- matrix(round(rnorm(1600,10,0.4),2), ncol=8,
  dimnames=list(paste("l",1:200), c("AA1","BB1","CC1","DD1","AA2","BB2","CC2","DD2")))
t8[3:6,1:2] <- t8[3:6,1:2]+3     # augment lines 3:6 for AA1&BB1
t8[5:8,5:6] <- t8[5:8,5:6]+3     # augment lines 5:8 for AA2&BB2 (c,d,g,h should be found)
t4 <- log2(t8[,1:4]/t8[,5:8])
fit4 <- moderTest2grp(t4, gl(2,2))
## now we'll use limma's topTable() function to look at the 'best' results
if("list" %in% mode(fit4)) {  # if you have limma installed we can look further
  library(limma)
  topTable(fit4, coef=1,n=5)                      # effect for 3,4,7,8
  fit4in <- moderTest2grp(t4, gl(2,2), testO="<")
  if("list" %in% mode(fit4in)) topTable(fit4in, coef=1,n=5) }
```

### Multiple Moderated Pair-Wise t-Tests From limma

If you want to make multiple pair-wise comparisons using `moderTestXgrp()` :
```{r moderTestXgrp, echo=TRUE}
grp <- factor(rep(LETTERS[c(3,1,4)], c(2,3,3)))
set.seed(2017); t8 <- matrix(round(rnorm(208*8,10,0.4),2), ncol=8,
  dimnames=list(paste(letters[], rep(1:8,each=26),sep=""), paste(grp,c(1:2,1:3,1:3),sep="")))
t8[3:6,1:2] <- t8[3:6,1:2] +3                    # augment lines 3:6 (c-f) 
t8[5:8,c(1:2,6:8)] <- t8[5:8,c(1:2,6:8)] -1.5    # lower lines 
t8[6:7,3:5] <- t8[6:7,3:5] +2.2                  # augment lines 
## expect to find C/A in c,d,g, (h)
## expect to find C/D in c,d,e,f
## expect to find A/D in f,g,(h)  
test8 <- moderTestXgrp(t8, grp) 
head(test8$p.value, n=8) 
```

#### Choosing amongst multiple possible pairwise comparisons

If you would like to run all possible pairwise comparisons you can set argument _useComparison_ to 'all'.

```{r moderTestXgrp2, echo=TRUE}
test8a <- moderTestXgrp(t8, grp, useComparison="all")
head(test8a$p.value, n=3) 
```

Besides, there are multiple ways to select which pairwise comparison(s) you would like to perform.

```{r moderTestXgrp3, echo=TRUE}
## multiple ways for choosing same comparison(s)
test8b <- moderTestXgrp(t8, grp, useCo=c("A-D"))   # name of comparison(s) 
test8c <- moderTestXgrp(t8, grp, useCo=2)          # the 2nd of all possible
test8d <- moderTestXgrp(t8, grp, useCo=cbind(samp="A", ref="D"))
test8e <- moderTestXgrp(t8, grp, useCo=cbind(samp=1, ref=3))

head(test8b$p.value, n=3) 
head(test8c$p.value, n=3) 
head(test8d$p.value, n=3) 
head(test8e$p.value, n=3) 

identical(test8b, test8c)
identical(test8b, test8d)
```


In some cases one might like a more explicite name of pairwise combined groups.
Using the argument _sep_ a specific separator may be given or searched from groups like _'combine2'_ .

```{r moderTestXgrp4, echo=TRUE}
## all comparisons with more explicite pairwise combined names
test8f <- moderTestXgrp(t8, grp, sep="combine2", useCo="all") 
head(test8f$p.value, n=8)
```



### Transform p-values To Local False Discovery Rate (lfdr)

To get an introduction into local false discovery rate estimations you may read [Strimmer 2008](https://doi.org/10.1093/bioinformatics/btn209). 
A convenient way to get lfdr values calculated by the package [fdrtool](https://CRAN.R-project.org/package=fdrtool) is available via the function `pVal2lfdr()`.

Note, that the toy-example used below is too small for estimating meaningful lfdr values.
For this reason the function _fdrtool()_ from package [fdrtool](https://CRAN.R-project.org/package=fdrtool) will issue warnings.

```{r pVal2lfdr, echo=TRUE}
set.seed(2017); t8 <- matrix(round(rnorm(160,10,0.4),2), ncol=8, dimnames=list(letters[1:20],
  c("AA1","BB1","CC1","DD1","AA2","BB2","CC2","DD2")))
t8[3:6,1:2] <- t8[3:6,1:2] +3   # augment lines 3:6 (c-f) for AA1&BB1
t8[5:8,5:6] <- t8[5:8,5:6] +3   # augment lines 5:8 (e-h) for AA2&BB2 (c,d,g,h should be found)
head(pVal2lfdr(apply(t8, 1, function(x) t.test(x[1:4], x[5:8])$p.value)))
```


### Confindence Intervals (under Normal Distribution)

The [confindence interval (CI)](https://en.wikipedia.org/wiki/Confidence_interval) is a common way of describing the uncertainity of measured or estimated values. 
The function `confInt()` allows calculating the confidence interval of the mean (using the functions _qt()_ and _sd()_) under 
a given [significance level (alpha)](https://en.wikipedia.org/wiki/Statistical_significance).
assuming that the [Normal distribution](https://en.wikipedia.org/wiki/Normal_distribution) is valid.

```{r fcCI, echo=TRUE}
set.seed(2022); ran <- rnorm(50)
confInt(ran, alpha=0.05)
## plot points and confindence interval of mean
plot(ran, jitter(rep(1, length(ran))), ylim=c(0.95, 1.05), xlab="random variable 'ran'",main="Points and Confidence Interval of Mean (alpha=0.05)", ylab="", las=1)
points(mean(ran), 0.97, pch=3, col=4)     # mean
lines(mean(ran) +c(-1, 1) *confInt(ran, 0.05), c(0.97, 0.97), lwd=4, col=4)  # CI
legend("topleft","95% conficence interval of mean", text.col=4,col=4,lty=1,lwd=1,seg.len=1.2,cex=0.9,xjust=0,yjust=0.5)
```


### Extract Groups Of Replicates From Pair-Wise Column-Names

When running multiple pairwise tests (using *moderTestXgrp()*) the column-names are concatenated group-names. 
To get the index of which group has been used in which pair-wise set you may use the function `matchSampToPairw()`, as shown below.

```{r matchSampToPairw, echo=TRUE}
## make example if limma is not installed
if(!requireNamespace("limma", quietly=TRUE)) test8 <- list(FDR=matrix(1, nrow=2, ncol=3, dimnames=list(NULL,c("A-C","A-D","C-D"))))
matchSampToPairw(unique(grp), colnames(test8$FDR)) 
```


### Extract Numeric Part Of Column-Names

When running multiple pairwise tests (using *moderTestXgrp()*) the results will be in adjacent columns and the group-names reflected in the column-names.
In the case measurements from multiple levels of a given variable are compared it is useful to extract the numeric part, the function `numPairDeColNames()` provides support to do so. 
When extracting just the numeric part, unit names will get lost, though. Note, if units used are not constant (eg seconds and milliseconds mixed) the extracted numeric values do not reflect the real quantitative context any more.

```{r pairWiseConc1, echo=TRUE}
mat1 <- matrix(1:8, nrow=2, dimnames=list(NULL, paste0(1:4,"-",6:9)))
numPairDeColNames(mat1)
```


### Automatic Determination Of Replicate Structure Based On Meta-Data

In order to run statistical testing the user must know which sample should be considered replicate of whom.
The function `replicateStructure()` aims to provide help by checking all column of a matrix of meta-data with the aim of identifying the replicate-status.

To do so, all columns are examined how many groups of replicats they may design. Depending on the argumen _method_ various options for choosing automatically exist :
The default _method="combAll"_ will select the column with the median number of groups (not counting all-different or all-same columns)).
When using as _method="combAll"_ (ie combine all columns that are neither all-different nor all-same), there is risk all lines (samples) will be be considered different and no replicates remain.
To avoid this situation the argument -method_ can be set to _"combNonOrth"_.
Then, it will be checked if adding more columns will lead to complete loss of replicates, and -if so- concerned columns omitted.


```{r replicateStructure1, echo=TRUE}
## column a is all different, b is groups of 2,
## c & d  are groups of 2 nut NOT 'same general' pattern as b
strX <- data.frame(a=letters[18:11], b=letters[rep(c(3:1,4), each=2)],
 c=letters[rep(c(5,8:6), each=2)], d=letters[c(1:2,1:3,3:4,4)],
 e=letters[rep(c(4,8,4,7),each=2)], f=rep("z",8) )
strX

replicateStructure(strX[,1:2])
replicateStructure(strX[,1:4], method="combAll")
replicateStructure(strX[,1:4], method="combAll", exclNoRepl=FALSE)
replicateStructure(strX[,1:4], method="combNonOrth", exclNoRepl=TRUE)
replicateStructure(strX, method="lowest")
```


### Addressing Pairwise Comparisons

There are multiple ways of specifying pairwise comparions (eg 'A-B', or c('A','B'), or using indexes), the function `convPairwiseSetup()`  allows converting between different types 
and returns a list with multiple formats for addressing specific quesions. 
If the separator for concantenating is known this may be helpful, but not obligatory. In many cases the function will manage to identify independently which separator has been used.


```{r convPairwiseSetup1, echo=TRUE}
## concatenated names of groups
convPairwiseSetup(c("B-C","D-A"), LETTERS[1:4])

## separate names of groups
convPairwiseSetup(c("B","C"), LETTERS[1:3])
convPairwiseSetup(2:3, LETTERS[1:3])
convPairwiseSetup(2:3, LETTERS[1:3], sep="__")
convPairwiseSetup(matrix(c("B","D","C","A"), ncol=2), LETTERS[1:4])
```

#### Suggesting A Separator For Pairwise Combinations

In many cases the user doesn't choose a specific separator to be used for concatenating group-names.  
The function `convPairwiseSetup()` allows checking a panel of separators to choose a 'unique' one which does not occur in any of the group-names. 

```{r suggestSeparator1, echo=TRUE}
getPWseparator(grp=LETTERS[1:3])
getPWseparator(grp=c("B-C","D","E")) 
getPWseparator(grp=c("B-C","D","E"), includeGrp=TRUE) 
```

Of course you can also check your preferred separators, the first found compatible (ie unique) will be reported.
```{r suggestSeparator2, echo=TRUE}
potSep <- c("-", "D", " -compared to- ")
getPWseparator(grp=c("B-C","D","E"), includeGrp=TRUE, potSep=potSep)
```


By using the argument _includeGrp_ the output will be a list including the input instead of simply the suggested separator.
```{r suggestSeparator3, echo=TRUE}
getPWseparator(grp=c("B-C","D","E"), includeGrp=TRUE) 
```


#### Identify The Separator Used In Concatenated Text

The same function may be used to identify which separator has been used, by checking a panel of classical candidates for separators
```{r identifySeparator1, echo=TRUE}
## Identify separator used
getPWseparator(compNames=c("B-C","B-D","C-D")) 
getPWseparator(compNames=c("B-C","B-D","C-D"), includeGrp=TRUE)
```

Besides, the function  `findHeadAndTail()` allows (unbiased) identifying separators without having to define candidates to check,
as it proceeds by removing names of groups from beginning and end of concatenated terms. 
However, there is a slight risk that some special characters may not allow finding the proper solution.

```{r identifySeparator2, echo=TRUE}
## Identify separator used
findHeadAndTail(LETTERS[4:2], c("D-X","D-C"))

grp1 <- c("C","ACC","CC","B.B","B.BA","B","CA")
pwNa1 <- "B.B-CC" 
findHeadAndTail(grp1, pwNa1)

pwNa2 <- c("B.B-CC", "CA-ACC")
findHeadAndTail(grp1, pwNa2)
```



## Working With Clustering  {#WorkingWithClustering}

Multiple concepts for clustering have been deeveloped, most of them allow extracting a vector with the cluster-numbers.
Here some functions helping to work with the output of such clustering results are presented.


### Prepare Data For Clustering

The way how to prepare data for clustering may be as important as the choice of the actual clustering-algorithm ...

Many clustering algorithms are available in R (eg see also [CRAN Task View: Cluster Analysis & Finite Mixture Models](https://CRAN.R-project.org/view=Cluster)), many of them require the input data to be standardized.
The regular way of standardizing sets all elements to mean=0 and sd=1.
To do so, the function `scale()` may be used. 

```{r std1, echo=TRUE}
dat <- matrix(2*round(runif(100),2), ncol=4)
mean(dat); sd(dat)

datS <- scale(dat)
apply(datS, 2, sd)
# each column was teated separately
mean(datS); sd(datS); range(datS)
# the mean is almost 0.0 and the sd almost 1.0

datB <- scale(dat, center=TRUE, scale=FALSE)
mean(datB); sd(datB); range(datB)              # mean is almost 0
```

However, if you want the entire data-set and not each column sparately, you may use `standardW()`.
Thus, relative differences visible within a line will be conserved.
Furthermore, in case of 3-dim arrays, this function returns also the same dimensions as the input.

```{r std2, echo=TRUE}
datS2 <- standardW(dat)
apply(datS2, 2, sd)
summary(datS2)
mean(datS2); sd(datS2)

datS3 <- standardW(dat, byColumn=TRUE)
apply(datS3, 2, sd)
summary(datS3)
mean(datS3); sd(datS3)
```


Sometimes it is sufficient to only set the minimum and maximum to a given range.

```{r scale1, echo=TRUE}
datR2 <- apply(dat, 2, scaleXY, 1, 100)
summary(datR2); sd(datR2)
```


### Characterize Clustering Results

Here a very basic clustering example...

```{r clu01, echo=TRUE}
nGr <- 3
irKm <- stats::kmeans(iris[,1:4], nGr, nstart=nGr*4)             # no need to standardize
   table(irKm$cluster, iris$Species)
   #wrGraph::plotPCAw(t(as.matrix(iris[,1:4])), sampleGrp=irKm,colBase=irKm$cluster,useSymb=as.numeric(as.factor(iris$Species)))
```

Using the function `reorgByCluNo()` we can now 'apply' the clustering result to the initial data to obtain other information.

```{r clu02, echo=TRUE}
## sort results by cluster number
head(reorgByCluNo(iris[,-5], irKm$cluster))
tail(reorgByCluNo(iris[,-5], irKm$cluster))
```

Let's calculate the median and sd values for each cluster:

```{r clu03, echo=TRUE}
## median an CV
ir2 <- reorgByCluNo(iris[,-5], irKm$cluster, addInfo=FALSE, retList=TRUE)
```


```{r clu04, echo=TRUE}
sapply(ir2, function(x) apply(x, 2, median))
```

```{r clu05, echo=TRUE}
sapply(ir2, colSds)
```

Besides, we have already seen the function `cutArrayInCluLike()` in section [Working with Arrays](#WorkingWithArrays) 'Working with Arrays'.


### Remove or Reassign Orphans

In some some circumstances clusters/groups with very vew individuals are not productive during further evalualtions (in particular in the context of interaction-netwoks).
The function `rmOrphans()` allows identifying and modifying cluster- or group-assignments of such very small groups ('orphans').

In the example below a vector of cluster-assignments ('x') is treated by different options to remove orphans :

```{r rmOrphans1, echo=TRUE}
x=c(3:1,3:4,4:6,5:3)
cbind(x, def=rmOrphans(x), assign1=rmOrphans(x, reassign=TRUE), 
  assign1=rmOrphans(x, minN=0.2, reassign=TRUE) )
```


## Tree-Like Structures   {#TreeLikeStructures}


### Filter Lists Of Connected Nodes, Extension Of Networks As 'Sandwich'

When interogating network-databases (like String for proteins or the coexpressionDB for gene co-expression) typically a (semi-)quantitatve
value is supplied with the connection of node 'A' to node 'B'.  
In many cases, it may be useful to filter the initial query-output to retain only strong interactions.
Furthermore, it may be of interest to expand such networks by nodes allowing to (further) inter-connect initial query-nodes 
(so called 'Sandwich' nodes as they are in the middle of initial nodes), for such nodes a separate (eg even more stringent) threshold can be applied.

Here let's suppose nodes have 3-digit names (ie numbers). 7 nodes of an initial query gave 1 to 7 conected nodes, 
the results are presented as list of data.frames where the 1st column is the connected node and the 2nd column the quality score of the connection (edge).
Furthemore, let's assume that here lower scores are better.

```{r filterNetw0, echo=TRUE}

lst2 <- list('121'=data.frame(ID=as.character(c(141,221,228,229,449)),11:15), 
  '131'=data.frame(ID=as.character(c(228,331,332,333,339)),11:15), 
  '141'=data.frame(ID=as.character(c(121,151,229,339,441,442,449)),c(11:17)), 
  '151'=data.frame(ID=as.character(c(449,141,551,552)),11:14),
  '161'=data.frame(ID=as.character(171),11),
  '171'=data.frame(ID=as.character(161),11),
  '181'=data.frame(ID=as.character(881:882),11:12) )
```

Now, we'd like to keep the core network consisting of all (dirctly) interconnected nodes with scores below 20 :

```{r filterNetw1, echo=TRUE}
(nw1 <- filterNetw(lst2, limInt=20, sandwLim=NULL, remOrphans=FALSE))
```

In the resulting output the 1st column now represents the query-nodes, the 2nd column all connected nodes based on filtering scores for edges,
and the 3rd colum the score for the edges.

Let's also remove all nodes not connected to a backbone at least 3 nodes long, ie remove orphan pairs of nodes :

```{r filterNetw2, echo=TRUE}
(nw2 <- filterNetw(lst2, limInt=20, sandwLim=NULL, remOrphans=TRUE))
```

If you want to expand this network by nodes allowing to further interconnect the nodes from above, 
we can add all 'sandwich' nodes (let's use a threshold of inferior/equal to 14 which will use only the better 'sandwich'-edges) :

```{r filterNetw3, echo=TRUE}
(nw3 <- filterNetw(lst2, limInt=20, sandwLim=14, remOrphans=TRUE))
```


### Convert Collection Of Pairs Of Nodes To Propensity Matrix

Many times networks get created from pairs of nodes. One way to represent the full network is via propensisty matrixes.
Several advanced tools and packages rather accept such propensisty matrixes as input.
Here, it is assumed that each line of the input represents a separate pair of nodes connected by an edge.

```{r propMatr1, echo=TRUE}
pairs3L <- matrix(LETTERS[c(1,3,3, 2,2,1)], ncol=2)      # loop of 3
(netw13pr <- pairsAsPropensMatr(pairs3L))                # as prop matr
```



### Characterize Individual Contribution Of Single Edges In Tree-Structures

```{r contribToContigPerFrag, echo=TRUE}
path1 <- matrix(c(17,19,18,17, 4,4,2,3), ncol=2,
  dimnames=list(c("A/B/C/D","A/B/G/D","A/H","A/H/I"), c("sumLen","n")))
contribToContigPerFrag(path1)
```


### Count Same Start- And End- Sites Of Edges (Or Fragments)

If you have a set of fragments from a common ancestor and the fragment's start- and end-sites 
are marked by index-positions (integers), you can make a simple graphical display :

```{r simpleFragFig, echo=TRUE}
frag1 <- cbind(beg=c(2,3,7,13,13,15,7,9,7, 3,3,5), end=c(6,12,8,18,20,20,19,12,12, 4,5,7))
rownames(frag1) <- letters[1:nrow(frag1)]
simpleFragFig(frag1)
```

Now we can make a matrix telling if some fragments do start or end at exactely the same position. 
```{r countSameStartEnd, echo=TRUE}
countSameStartEnd(frag1)
```


## Support for Graphical Output   {#SupportForGraphicalOutput}

### Convenient Paste-Collapse

The function `pasteC()` allows adding quotes and separating the last element by specific text (eg 'and').
```{r pasteC, echo=TRUE}
pasteC(1:4)
pasteC(letters[1:4],quoteC="'")
```


### Transform Numeric Values to Color-Gradient

By default most color-gradients end with a color very close to the beginning.
```{r color-gradient1, echo=TRUE}
set.seed(2015); dat1 <- round(runif(15),2)
plot(1:15, dat1, pch=16, cex=2, las=1, col=colorAccording2(dat1),
  main="Color gradient according to value in y")
# Here we modify the span of the color gradient
plot(1:15, dat1, pch=16, cex=2, las=1, 
  col=colorAccording2(dat1, nStartO=0, nEndO=4, revCol=TRUE), main="blue to red")
# It is also possible to work with scales of transparency
plot(1:9, pch=3, las=1)
points(1:9, 1:9, col=transpGraySca(st=0, en=0.8, nSt=9,trans=0.3), cex=42, pch=16)
```


### Assign New Transparency To Given Colors
 
For this purpose you may use `convColorToTransp`.
```{r convColorToTransp, fig.height=6, fig.width=3, echo=TRUE}
col0 <- c("#998FCC","#5AC3BA","#CBD34E","#FF7D73")
col1 <- convColorToTransp(col0,alph=0.7)
layout(1:2)
pie(rep(1,length(col0)), col=col0, main="no transparency")
pie(rep(1,length(col1)), col=col1, main="new transparency")
```


### Print Matrix-Content As Plot

There are many ways of creating reports. If you want simply to combine a few plots into a pdf, the function `tableToPlot()` 
may be helpful to add a small table (eg overview of points/samples/files used in other plots of the same pdf). 
This function prints tables in the current graphical output/window (which may by a pdf-device).


## Other Convenience Functions  {#OtherConvenienceFunctions}

### Writing Compact Dates (more options ...)

Many times it may be useful to add the date to filenames when saving data or plots as files.
The built-in functions _date()_, _Sys.Date()_ and _Sys.Time()_ are a good way to start.

Generally I like to use abbreviated month-names since the order of writing the month is different in Europe compared to the USA.
So, this may help avoiding mis-interpreting dates instead of writing the number of the Month.
For example, 2021-03-05 means in Europe March 5th while in other places it means May 3rd.

You may also look at the standardized format for *numeric dates* [norm ISO 8601](https://en.wikipedia.org/wiki/ISO_8601), 
which matches to _Sys.Date()_ (from package _base_) and _sysDate(style="univ5")_ (package _wrMisc_) .

The R-functions mentioned above (_date()_, _Sys.Date()_, from the _base_ package) use local language settings.
For using English languange names you may use the function `sysDate` from this package (ie wrProteo). 
It allows producing compact versions of current the date, **independent to local language settings** (or not -if you prefer), ie locale-specific, 
(yes, in some languages - like French - the first 3 letters of the month may give ambiguous results !) 
and to avoid white space ' ' (which I prefer to avoid in file-names).
Please look at the function's help-page for all available options.

```{r sysDate1, echo=TRUE}
## To get started
Sys.Date()

## Compact English names (in European order), no matter what your local settings are :
sysDate() 
```

The table below shows a number of options to write the date in English or using local month-names :

```{r DateTab, echo=TRUE}
tabD <- cbind(paste0("univ",1:6), c(sysDate(style="univ1"), sysDate(style="univ2"), 
    sysDate(style="univ3"), sysDate(style="univ4"), as.character(sysDate(style="univ5")), 
    sysDate(style="univ6")), paste0("   local",1:6), 
  c(sysDate(style="local1"), sysDate(style="local2"), sysDate(style="local3"), 
    sysDate(style="local4"), sysDate(style="local5"), sysDate(style="local6")))   
knitr::kable(tabD, caption="Various ways of writing current date")
```


## Acknowledgements

The author would like to acknowledge the support by the [IGBMC](https://www.igbmc.fr) (CNRS UMR 7104, Inserm U 1258, UdS), [CNRS](http://www.cnrs.fr/en), 
[Universit&eacute; de Strasbourg (UdS)](https://www.unistra.fr/fr) and [Inserm](https://www.inserm.fr).



## Session-Info

```{r sessionInfo, echo=FALSE}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mergeVectors.R
\name{mergeVectors}
\alias{mergeVectors}
\title{Merge Named Vectors}
\usage{
mergeVectors(
  ...,
  namePrefix = "x.",
  NAto0 = FALSE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{...}{all vectors that need to be merged}

\item{namePrefix}{(character) prefix to numers used when vectors are not given with explicit names (second exammple)}

\item{NAto0}{(logical) optional replacemet of \code{NA}s by 0}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a matrix of merged values
}
\description{
This function allows merging for multiple named vectors (each element needs to be named).
Basically, all elements carrying the same name across different input-vectors will be aligned in the same column of the output (input-vectors appear as lines).
If vectors are not given using a name (see first example below), they will be names 'x.1' etc (see argument \code{namePrefix}).
}
\details{
Note : The arguments '\code{namePrefix}', '\code{NAto0}', '\code{callFrom}' and '\code{silent}' must be given with full name to be recognized as such (and not get considered as vector for merging).
}
\examples{
x1 <- c(a=1, b=11, c=21)
x2 <- c(b=12, c=22, a=2)
x3 <- c(a=3, d=43)
mergeVectors(vect1=x1, vect2=x2, vect3=x3)
x4 <- 41:44     # no names - not conform for merging
mergeVectors(x1, x2, x3, x4)
}
\seealso{
\code{\link[base]{merge}} (for two data.frames)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/moderTest2grp.R
\name{moderTest2grp}
\alias{moderTest2grp}
\title{Moderated Pairwise t-test From Limma}
\usage{
moderTest2grp(
  dat,
  grp,
  limmaOutput = TRUE,
  addResults = c("lfdr", "FDR", "Mval", "means"),
  testOrientation = "=",
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{dat}{matrix or data.frame with rows for multiple (independent) tests, use ONLY with 2 groups; assumed as log2-data}

\item{grp}{(factor) describes column-relationship of 'dat'   (1st factor is considered as reference -> orientation of M-values !!)}

\item{limmaOutput}{(logical) return full (or extended) MArrayLM-object from limma or 'FALSE' for only the (uncorrected) p.values}

\item{addResults}{(character) types of results to add besides basic limma-output, data are assumed to be log2 ! (eg "lfdr" using fdrtool-package, "FDR" or "BH" for BH-FDR, "BY" for BY-FDR, 
"bonferroni" for Bonferroni-correction, "qValue" for lfdr by qvalue, "Mval", "means" or "nonMod" for non-moderated test and he equivaent all (other) multiple testing corrections chosen here)}

\item{testOrientation}{(character) for one-sided test (">","greater" or "<","less"), NOTE : 2nd grp is considered control/reference, '<' will identify grp1 < grp2}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a limma-type object of class \code{MArrayLM} (which can be handeled like a list)
}
\description{
This function runs moderated t-test from package \code{limma} on each line of data.
Note: This function requires the package \href{https://bioconductor.org/packages/release/bioc/html/limma.html}{limma} from bioconductor being installed. 
The \code{limma} contrast-matrix has to be read by column, the lines in the contrast-matrix containing '+1' will be compared to the '-1' lines, eg grpA-grpB .
Local false discovery rates (lfdr) estimations will be made using the CRAN-package \href{https://CRAN.R-project.org/package=fdrtool}{fdrtool} (if available).
}
\examples{
set.seed(2017); t8 <- matrix(round(rnorm(1520,10,0.4),2), ncol=8,
  dimnames=list(paste("l",1:190),c("AA1","BB1","CC1","DD1","AA2","BB2","CC2","DD2")))
t8[3:6,1:2] <- t8[3:6,1:2] +3     # augment lines 3:6 for AA1&BB1
t8[5:8,5:6] <- t8[5:8,5:6] +3     # augment lines 5:8 for AA2&BB2 (c,d,g,h should be found)
t4 <- log2(t8[,1:4]/t8[,5:8])
## Two-sided testing 
fit4 <- moderTest2grp(t4,gl(2,2))
# If you have limma installed we can now see further
if(requireNamespace("limma")) {
  limma::topTable(fit4, coef=1, n=5)}    # effect for 3,4,7,8

## One-sided testing
fit4in <- moderTest2grp(t4,gl(2,2), testO="<")
# If you have limma installed we can now see further
if(requireNamespace("limma")) {
  limma::topTable(fit4in, coef=1, n=5) }
}
\seealso{
\code{\link[limma]{lmFit}} and the \code{eBayes}-family of functions in package \href{https://bioconductor.org/packages/release/bioc/html/limma.html}{limma}, \code{\link[stats]{p.adjust}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/equLenNumber.R
\name{.checkConvt2Vect}
\alias{.checkConvt2Vect}
\title{Convert To Simple Vector (similar to unlist)}
\usage{
.checkConvt2Vect(dat, toNumeric = TRUE)
}
\arguments{
\item{dat}{(list, data.frame) main input}

\item{toNumeric}{(logical)}
}
\value{
This function returns a character (or numeric) vector
}
\description{
This function allows converting 'dat' (may be list, data.frame etc) to simple vector, more elaborate than unlist()
}
\examples{
aa <- matrix(11:14, ncol=2)
.checkConvt2Vect(aa)
}
\seealso{
\code{\link[base]{unlist}}; used in  \code{\link{equLenNumber}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extract1chan.R
\name{extr1chan}
\alias{extr1chan}
\title{Extract Just One Series, ie One Channel, Of List Of Arrays}
\usage{
extr1chan(
  arrLst,
  cha,
  na.rm = TRUE,
  rowSep = "__",
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{arrLst}{(list) list of arrays (typically 1st and 2nd dim for specific genes/objects, 3rd for different measures associated with)}

\item{cha}{(integer) channel number}

\item{na.rm}{(logical) default =TRUE to remove NAs}

\item{rowSep}{(character) separator for rows}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allows easier tracking of messages produced}
}
\value{
This function returns a list with just single channel extracted
}
\description{
This function was designed for handeling measurements stored as list of multiple arrays, like eg compound-screens using microtiter-plates where multiple parameters ('channels') 
were recorded for each well (element).
The elements (eg compounds screened) are typcally stored in the 1st dimension of the arrays, the replicated in the secon dimension and different measure types/parameters in the 3rd chanel. 
In order to keep the structure of of individual microtiter-plates, typically each plate forms a separate array (of same dimensions) in a list.  
The this function allows extracting a single channel of the list of arrays (3rd dim of each array) and return row-appended matrix.
}
\examples{
arr1 <- array(1:24,dim=c(4,3,2),dimnames=list(c(LETTERS[1:4]),
  paste("col",1:3,sep=""),c("ch1","ch2")))
arr2 <- array(74:51,dim=c(4,3,2),dimnames=list(c(LETTERS[1:4]),
  paste("col",1:3,sep=""),c("ch1","ch2")))
arrL1 <- list(pl1=arr1,pl2=arr2)
extr1chan(arrL1,ch=2)
}
\seealso{
\code{\link[wrMisc]{organizeAsListOfRepl}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extrColsDeX.R
\name{extrColsDeX}
\alias{extrColsDeX}
\title{Flexible extraction of columns}
\usage{
extrColsDeX(x, extrCol, doExtractCols = FALSE, callFrom = NULL, silent = FALSE)
}
\arguments{
\item{x}{(matrix or data.frame) main input (where data should be extracted from)}

\item{extrCol}{(character, integer or list) columns to be extracted, may be column-names or column index; if is \code{list} each first-level element will be considered as options for one choice}

\item{doExtractCols}{(logical) if default \code{FALSE} only the column indexes will be returned}

\item{callFrom}{(character) allows easier tracking of message(s) produced}

\item{silent}{(logical) suppress messages}
}
\value{
integer-vector (if\code{doExtractCols=FALSE} return depending on input \code{matrix} or \code{data.frame})
}
\description{
This function provides flexible checking if a set of columns may be extracted from a matrix or data.frame 'x'.
If argument \code{extrCol} is list of character vectors, this allows to search among given options, the first matching name for each vector will be identified.
}
\examples{
dFr <- data.frame(a=11:14, b=24:21, cc=LETTERS[1:4], dd=rep(c(TRUE,FALSE),2)) 
extrColsDeX(dFr,c("b","cc","notThere")) 
extrColsDeX(dFr,c("b","cc","notThere"), doExtractCols=TRUE) 
extrColsDeX(dFr, list(c("nn","b","a"), c("cc","a"),"notThere")) 
}
\seealso{
\code{\link[utils]{read.table}}, \code{\link{filterList}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/replPlateCV.R
\name{replPlateCV}
\alias{replPlateCV}
\title{CV of replicate plates (list of matrixes)}
\usage{
replPlateCV(lst, silent = FALSE, debug = FALSE, callFrom = NULL)
}
\arguments{
\item{lst}{list of matrixes : suppose lines are independent elements, colums are replicates of the 1st column. All matrixes must have same dimensions}

\item{silent}{(logical) suppress messages if \code{TRUE}}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allows easier tracking of messages produced}
}
\value{
This function returns a matrix or array (1st dim is intraplate-position, 2nd .. plate-group/type, 3rd .. channels)
}
\description{
This function calculates CVs of replicates from list of 2 or 3-dim arrays (where 2nd dim is replicates, 3rd dim may be channel). 
Note : all list-elements of must MUST have SAME dimensions !
When treating data from microtiter plates (eg 8x12) data are typically spread over multiple plates, ie initial matrixes that are the organized into arrays.
Returns matrix or array (1st dim is intraplate-position, 2nd .. plate-group/type, 3rd .. channels)
}
\examples{
set.seed(2016); ra1 <- matrix(rnorm(3*96),nrow=8)
pla1 <- list(ra1[,1:12],ra1[,13:24],ra1[,25:36])
replPlateCV(pla1)
arrL1 <- list(a=array(as.numeric(ra1)[1:192],dim=c(8,12,2)),
  b=array(as.numeric(ra1)[97:288],dim=c(8,12,2)))
replPlateCV(arrL1)
}
\seealso{
\code{\link{rowCVs}}, @seealso \code{\link{arrayCV}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stableMode.R
\name{stableMode}
\alias{stableMode}
\title{Estimate Mode (Most Frequent Value)}
\usage{
stableMode(
  x,
  method = "density",
  finiteOnly = TRUE,
  bandw = NULL,
  rangeSign = 1:6,
  silent = FALSE,
  callFrom = NULL,
  debug = FALSE
)
}
\arguments{
\item{x}{(numeric, or character if 'method='mode') data to find/estimate most frequent value}

\item{method}{(character) There are 3 options : BBmisc, binning and density (default). If "binning" the function will search context dependent, ie like most frequent class of histogram.
Using "binning" mode the search will be refined if either 80 percent of values in single class or >50 percent in single class.}

\item{finiteOnly}{(logical) suppress non-finite values; allows avoiding \code{NULL} as result in presence of some \code{Inf} values; \code{NA} will be ignored in any case}

\item{bandw}{(integer) only used when \code{method="binning"} or  \code{method="density"} : defines the number of points to look for density or number of classes used; 
very "critical" parameter, may change results in strong way. Note: with \code{method="binning"}: At higher values for "bandw" you will finally loose advantage of histLike-type search of mode !}

\item{rangeSign}{(integer) only used when \code{method="binning"}: range of numbers used as number of significant values}

\item{silent}{(logical) suppress messages}

\item{callFrom}{(character) allows easier tracking of messages produced}

\item{debug}{(logical) additional messages for debugging}
}
\value{
This function returns a numeric vector with value of mode, the name of the value indicates it's position
}
\description{
Estimate mode, ie most frequent value. In case of continuous numeric data, the most frequent values may not be the most frequently repeated exact term.
This function offers various approches to estimate the mode of a numeric vector. 
Besides, it can also be used to identify the most frequentexact term (in this case also from character vectors).
}
\details{
The argument \code{method} allows to choose among (so far) 4 different methods available.
If "density" is chosen, the most dense region of sqrt(n) values will be chosen; 
if "binning", the data will be binned (like in histograms) via rounding to a user-defined number of significant values ("rangeSign").
If \code{method} is set to "BBmisc", the function \code{computeMode()} from package \href{https://CRAN.R-project.org/package=BBmisc}{BBmisc} will be used.
If "mode" is chosen, the first most frequently occuring (exact) value will be returned, if "allModes", all ties will be returned. This last mode also works with character input.
}
\examples{
set.seed(2012); dat <- round(c(rnorm(50), runif(100)),3)
stableMode(dat)
}
\seealso{
\code{computeMode()} in package \href{https://CRAN.R-project.org/package=BBmisc}{BBmisc}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mergeMatrices.R
\name{.mergeMatrices}
\alias{.mergeMatrices}
\title{Merge Multiple Matrices (main)}
\usage{
.mergeMatrices(
  inpL,
  mode = "intersect",
  useColumn = 1,
  extrRowNames = FALSE,
  na.rm = TRUE,
  argL = NULL,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{inpL}{(list containing matrices or data.frames) main input (multiple matrix or data.frame objects)}

\item{mode}{(character) allows choosing restricting to all common elements (\code{mode='intersect'}) or union  (\code{mode='union'})}

\item{useColumn}{(integer, character or list) the column(s) to consider, may be \code{'all'} to use all, integer to select specific indexes or list of indexes or colnames for cutom-selection per matrix}

\item{extrRowNames}{(logical) decide whether columns with all values different (ie no replicates or max divergency) should be excluded}

\item{na.rm}{(logical) suppress \code{NA}s}

\item{argL}{(list of arguments)}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a matrix containing all selected columns of the input matrices to fuse
}
\description{
This function allows merging of multiple matrix-like objects from an initial list.
}
\examples{
mat1 <- matrix(11:18, ncol=2, dimnames=list(letters[3:6],LETTERS[1:2]))
}
\seealso{
\code{\link{mergeMatrixList}},  \code{\link[base]{merge}},  \code{\link{mergeMatrices}} for separate entries
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/indexGroupsFromPW.R
\name{indexGroupsFromPW}
\alias{indexGroupsFromPW}
\title{Index Names Of Groups From Vector Of Concatenated Pairwise Group-Names}
\usage{
indexGroupsFromPW(
  compNames,
  grp = NULL,
  potSep = pwSeparatorList("combine1"),
  includeGrp = FALSE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{compNames}{(vector of character or integer, or matrix) names of pairwise combined group-names; if matrix the rownames of \code{compNames} will be used}

\item{grp}{(character or factor) optional groups (may include group-names that do not occur in \code{compNames})}

\item{potSep}{(character) potential separators to be checked if occuring in \code{useComp}; the first fitting will used}

\item{includeGrp}{(logical) instead of matrix, rather return list with $pwInd, $pwGrpNames (group-names corresponding to index), $sep (separator used to split) and $grpNa}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a numeric matrix with indexes corresponding to (sorted) group-names as they appear in \code{compNames};
  or if \code{includeGrp=TRUE} a list with $ind (matrix indexes for each \code{compNames}), $grpNames (group-names corresponding to index), $sep (separator used to split) and $grpNa (sorted names of types of groups)
}
\description{
This function allows matching which groups are cited in a pairwise concatenated manner.
This function can also be used to split concatenated group-names (while automatically determining a suitable separator) and display as matrix when \code{includeGrp=TRUE}.
Replaces matchSampToPairw() ???
}
\examples{
indexGroupsFromPW(c("C-B","B-D","C-D")) 
indexGroupsFromPW(c("C-B","B-D","C-D"), grp=rep(LETTERS[1:6],2), includeGrp=TRUE) 
}
\seealso{
\code{\link{moderTestXgrp}} (colnames of testing results from this function may be used); \code{\link{replacePWseparator}}; used internally \code{\link{getPWseparator}},  \code{\link{pwSeparatorList}}, \code{\link{.getPWseparator}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rowNormalize.R
\name{.complCols}
\alias{.complCols}
\title{Search (complementing) Columns For Best Coverage Of Non-NA Data For rowNormalization (main)}
\usage{
.complCols(x, dat, nCombin)
}
\arguments{
\item{x}{(integer, length=1) column number for with other columns to combine & give (some) complete non-NA lines are seeked}

\item{dat}{(matrix) .. init data, smay be parse matrix with numerous NA}

\item{nCombin}{(integer) .. number of columns used to make complete subset}
}
\value{
This function returns a matrix of column-indexes complementing (nCombin rows)
}
\description{
This function was designed to complete the selection of columns of sparse matrix 'dat' with sets of 'nCombin' columns at complete 'coverage'
Context : In sparse matrix 'dat' search subsets of columns with some rows as complete (no NA).
}
\examples{
.complCols(3, dat=matrix(c(NA,12:17,NA,19),ncol=3), nCombin=3)
}
\seealso{
\code{\link{rowNormalize}}; \code{\link{normalizeThis}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkVectLength.R
\name{checkVectLength}
\alias{checkVectLength}
\title{Check length of vector}
\usage{
checkVectLength(
  x,
  expeL = 1,
  stopOnProblem = FALSE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{x}{(numeric or charcter vector) input to check length}

\item{expeL}{(numeric) expected length}

\item{stopOnProblem}{(logical) continue on problems with message or stop (as error message)}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) display additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of message(s) produced}
}
\value{
This function returns \code{NULL}; it produces either error-message if length is not OK or optional message if length is OK
}
\description{
\code{checkVectLength} checks argument 'x' for expected length 'expeL' and return either message or error when expectation not met. 
May be used for parameter ('sanity') checking in other user front-end functions.
}
\examples{
aa <- 1:5; checkVectLength(aa,exp=3)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filt3dimArr.R
\name{filt3dimArr}
\alias{filt3dimArr}
\title{Filter three-dimensional array of numeric data}
\usage{
filt3dimArr(
  x,
  filtVal,
  filtTy = ">",
  filtCrit = NULL,
  displCrit = NULL,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{x}{array (3-dim) of numeric data}

\item{filtVal}{(numeric, length=1) for testing inferior/superor/equal condition}

\item{filtTy}{(character, length=1) which type of testing to perform (may be 'eq','inf','infeq','sup','supeq', '>', '<', '>=', '<=', '==')}

\item{filtCrit}{(character, length=1) which column-name consider when filtering filter with 'filtVal' and 'filtTy'}

\item{displCrit}{(character) column-name(s) to display}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a list of filtered matrixes (by 3rd dim)
}
\description{
Filtering of matrix or (3-dim) array \code{x} : filter column according to \code{filtCrit} (eg 'inf') and threshold \code{filtVal}
}
\details{
and extract/display all col matching 'displCrit'.
}
\examples{
arr1 <- array(11:34, dim=c(4,3,2), dimnames=list(c(LETTERS[1:4]),
  paste("col",1:3,sep=""), c("ch1","ch2")))
filt3dimArr(arr1,displCrit=c("col1","col2"),filtCrit="col2",filtVal=7)
}
\seealso{
\code{\link{filterList}}; \code{\link{filterLiColDeList}};
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/buildTree.R
\name{buildTree}
\alias{buildTree}
\title{Connect edges to from tree and extract all possible branches}
\usage{
buildTree(
  disDat,
  startFr = NULL,
  posCo = c("beg", "end"),
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{disDat}{(matrix or data.frame) integer values with 1st column, ie start site of fragment, 2nd column as end of fragments, rownames as unique IDs (node-names)}

\item{startFr}{(integer) index for 1st node (typically =1 if 'disDat' sorted by "beg"), should point to a terminal node for consective growing of branches}

\item{posCo}{(character) colnames specifying the begin & start sites in 'disDat', if NULL 1st & 2nd col will be used}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a list with $paths (branches as matrix with columns 'sumLen' & 'n'), $usedNodes (character vector of all names used to build tree) and $tree (object from data.tree)
}
\description{
It is assumed that multiple fragments from a common ancestor bay be charcterized by the their start- and end-sites by integer values.  
For example, If 'abcdefg' is the ancestor, the fragments 'bcd' (from position 2 to 4) to  and 'efg' may then be assembled. 
To do so, all fragments must be presented as matix specifying all start- and end-sites (and fragment-names).  
\code{buildTree} searchs contiguous fragments from columns 'posCo' (start/end) from 'disDat' to build tree & extract path information starting with line 'startFr'.
Made for telling if dissociated fragments contribute to long assemblies.
This function uses various functions of package \href{https://CRAN.R-project.org/package=data.tree}{data.tree} which must be installed, too.
}
\examples{
frag2 <- cbind(beg=c(2,3,7,13,13,15,7,9,7,3,7,5,7,3),end=c(6,12,8,18,20,20,19,12,12,4,12,7,12,4)) 
rownames(frag2) <- c("A","E","B","C","D","F","H","G","I", "J","K","L","M","N")
buildTree(frag2)
countSameStartEnd(frag2)
}
\seealso{
package \href{https://CRAN.R-project.org/package=data.tree}{data.tree} original function used \code{\link[data.tree]{Node}}; in this package : for exploiting edge/tree related issues \code{\link{simpleFragFig}}, \code{\link{countSameStartEnd}} and \code{\link{contribToContigPerFrag}},
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/contribToContigPerFrag.R
\name{contribToContigPerFrag}
\alias{contribToContigPerFrag}
\title{Characterize individual contribution of single edges in tree-structures}
\usage{
contribToContigPerFrag(joinMat, fullLength = NULL, nDig = 3)
}
\arguments{
\item{joinMat}{(matrix) matrix with concatenated edges as rownames (separated by slashes), column \code{sumLen} for total length and column \code{n} for number of edges}

\item{fullLength}{(integer) custom total length (useful if the concatenated edges do not cover 100 percent of the original precursor whose fragments are studied)}

\item{nDig}{(integer) rounding: number of digits for 3rd column \code{len.rat} in output}
}
\value{
matrix of 3 columns: with length of longest tree-branches where given edge participates (column \code{sumLen}), the (total) number of edges therein (col \code{n.frag}) and a relative value (\code{len.rat})
}
\description{
This function helps investigating tree-like structures with the aim of indicating how much individual tree components contribute 
to compose long stretches.
\code{contribToContigPerFrag} characterizes individual (isolated) contribution of single edges in tree-structures. 
Typically used to process/exploit summarized trees (as matrix) made by \code{\link{buildTree}} which makes use of the package \href{https://CRAN.R-project.org/package=data.tree}{data.tree}.
For example if A,B and C can be joined aa well and B +D, this function will check if A+B+C is longer and if A contributes to the longest tree.
}
\examples{
path1 <- matrix(c(17,19,18,17, 4,4,2,3),ncol=2,
  dimnames=list(c("A/B/C/D","A/B/G/D","A/H","A/H/I"),c("sumLen","n")))
contribToContigPerFrag(path1)
}
\seealso{
to build tree \code{\link{buildTree}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lrbind.R
\name{lrbind}
\alias{lrbind}
\title{rbind on lists}
\usage{
lrbind(lst, silent = FALSE, debug = FALSE, callFrom = NULL)
}
\arguments{
\item{lst}{(list, composed of multiple matrix or data.frames or simple vectors) main input (each list-element should have same number of columns, numeric vectors will be converted to number of columns of other columns/elements)}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns (depending on input) a matrix or data.frame
}
\description{
rbind-like function to append list-elements containing matrixes (or data.frames) and return one long table. 
All list-elements must have same number of columns (and same types of classes in case of data.frames.
Simple vectors (as list-elements) will be considered as sigle lines for attaching.
}
\examples{
lst1 <- list(matrix(1:9, ncol=3, dimnames=list(letters[1:3],c("AA","BB","CC"))),
  11:13, matrix(51:56, ncol=3))
lrbind(lst1)
}
\seealso{
\code{rbind} in \code{\link[base]{cbind}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filterNetw.R
\name{filterNetw}
\alias{filterNetw}
\title{Filter nodes & edges for extracting networks
 
This function allows extracting and filtering network-data based on fixed threshold (\code{limInt}) and add sandwich-nodes (nodes inter-connecting initial nodes) out of node-based queries.}
\usage{
filterNetw(
  lst,
  filtCol = 3,
  limInt = 5000,
  sandwLim = 5000,
  filterAsInf = TRUE,
  outFormat = "matrix",
  remOrphans = TRUE,
  remRevPairs = TRUE,
  elemNa = "genes",
  silent = FALSE,
  callFrom = NULL,
  debug = FALSE
)
}
\arguments{
\item{lst}{(list, composed of multiple matrix or data.frames ) main input (each list-element should have same number of columns)}

\item{filtCol}{(integer, length=1) which column of \code{lst} should be usd to filter using thresholds \code{limInt} and \code{sandwLim}}

\item{limInt}{(numeric, length=1) filter main edge-scores according to \code{filterAsInf}}

\item{sandwLim}{(numeric, length=1) filter sandwich connection edge-scores accodring to \code{filterAsInf}}

\item{filterAsInf}{(logical) filter as 'inferior or equal' or 'superior or equal'}

\item{outFormat}{(character) may be 'matrix' for tabular output, 'all' as list with matrix and list of node-names}

\item{remOrphans}{(logical) remove networks consisting only of 2 connected edges}

\item{remRevPairs}{(logical) remove duplicate edges due to reverse massping (eg A - B and B - A); NOTE : use only when edges don't have orientation !}

\item{elemNa}{(character) used only for messages}

\item{silent}{(logical) suppress messages}

\item{callFrom}{(character) allow easier tracking of message(s) produced}

\item{debug}{(logical) display additional messages for debugging}
}
\value{
This function returns a matrix or data.frame
}
\description{
Filter nodes & edges for extracting networks
 
This function allows extracting and filtering network-data based on fixed threshold (\code{limInt}) and add sandwich-nodes (nodes inter-connecting initial nodes) out of node-based queries.
}
\examples{

lst2 <- list('121'=data.frame(ID=as.character(c(141,221,228,229,449)),11:15), 
	 '131'=data.frame(ID=as.character(c(228,331,332,333,339)),11:15), 
  '141'=data.frame(ID=as.character(c(121,151,229,339,441,442,449)),c(11:17)), 
  '151'=data.frame(ID=as.character(c(449,141,551,552)),11:14),
  '161'=data.frame(ID=as.character(171),11), '171'=data.frame(ID=as.character(161),11),
  '181'=data.frame(ID=as.character(881:882),11:12) )

lst2 <- list('121'=data.frame(ID=as.character(c(141,221,228,229,449)),11:15, 21:25), 
	 '131'=data.frame(ID=as.character(c(228,331,332,333,339)),11:15, 21:25), 
  '141'=data.frame(ID=as.character(c(121,151,229,339,441,442,449)), c(11:17), 21:27), 
  '151'=data.frame(ID=as.character(c(449,141,551,552)), 11:14, 21:24),
  '161'=data.frame(ID=as.character(171), 11,21), '171'=data.frame(ID=as.character(161), 11,21),
  '181'=data.frame(ID=as.character(881:882), 11:12,21:22) )

(te1 <- filterNetw(lst2, limInt=90, remOrphans=FALSE))
(te2 <- filterNetw(lst2, limInt=90, remOrphans=TRUE))

(te3 <- filterNetw(lst2, limInt=13, remOrphans=FALSE))
(te4 <- filterNetw(lst2, limInt=13, remOrphans=TRUE))


}
\seealso{
in \code{\link[base]{cbind}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nNonNumChar.R
\name{.extrNumHeadingSepChar}
\alias{.extrNumHeadingSepChar}
\title{Extract numbers before separator followed by alphabetic character}
\usage{
.extrNumHeadingSepChar(x, sep = "_")
}
\arguments{
\item{x}{character vector to be treated}

\item{sep}{(character) separator}
}
\value{
This function returns a numeric vector
}
\description{
This function aims to extract number(s) before separator followed by alphabetic character (return named numeric vector, NAs when no numeric part found)
}
\examples{
.extrNumHeadingSepChar(" 1B ")
}
\seealso{
\code{\link[base]{nchar}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/combineReplFromListToMatr.R
\name{combineReplFromListToMatr}
\alias{combineReplFromListToMatr}
\title{Combine replicates from list to matrix}
\usage{
combineReplFromListToMatr(lst, silent = FALSE, debug = FALSE, callFrom = NULL)
}
\arguments{
\item{lst}{(list)  list of arrays (typically multi-parameter measures of micortiterplate data)}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a list of arrays now with same dimension of arrays (but shorter, since replicate-arrays were combined)
}
\description{
Suppose multiple measures (like multiple chanels) are taken for subjects and these measures are organized as groups in a list, 
like muliple parameters (= channels) or types of measurements (typically many paramters are recorded when screeinig compounds in microtiter plates).
Within one parameter/channel all replicate-data from separate list-entries ('lst') will get combined according to names of list-elements.
The function will trim any redundant text in names of list-elements, try to isolate separator (may vary among replicate-groups, but should be 1 character long).
eg names "hct116 1.1.xlsx" & "hct116 1.2.xlsx" will be combined as replicates, "hct116 2.1.xlsx" will be considered as new group.
}
\examples{
lst2 <- list(aa_1x=matrix(1:12,nrow=4,byrow=TRUE),ab_2x=matrix(24:13,nrow=4,byrow=TRUE))
combineReplFromListToMatr(lst2)
}
\seealso{
\code{\link{extr1chan}}, \code{\link{organizeAsListOfRepl}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/findCloseMatch.R
\name{.compareByPPM}
\alias{.compareByPPM}
\title{Compare by PPM}
\usage{
.compareByPPM(dat, limit, distVal = FALSE)
}
\arguments{
\item{dat}{list of 2 numerical vectors}

\item{limit}{(numeric, length=1) threshold value for retaining values, used with distace-type specified in argument 'compTy'}

\item{distVal}{(logical) to toggle outpout as matrix of numeric (distance values above 'limit', others NA) or matrix of logical}
}
\value{
This function returns a list with close matches of 'x' to given 'y', the numeric value dependes on 'sortMatch' (if FASLE then always value of 'y' otherwise of longest of x&y)
}
\description{
This function allows to compare by ppm
}
\examples{
cc <- list(aa=11:14, bb=c(13.1,11.5,14.3,20:21))
.compareByPPM(cc, 1) 
}
\seealso{
\code{\link{findCloseMatch}},  \code{\link{checkSimValueInSer}}, and also \code{.compareByDiff}, for convient output \code{\link{countCloseToLimits}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/naOmit.R
\name{naOmit}
\alias{naOmit}
\title{Fast na.omit}
\usage{
naOmit(x, silent = FALSE, debug = FALSE, callFrom = NULL)
}
\arguments{
\item{x}{(vector or matrix) data to check & remove \code{NA}s}

\item{silent}{(logical) suppress messages if \code{TRUE}}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allows easier tracking of messages produced}
}
\value{
This function returns a vector without NAs (matrix input will be transformed to vector). Returns NULL if input consists only of NAs.
}
\description{
This function removes NAs from input vector, in contrast to \code{na.omit} this function has no slot for removed elements.
}
\details{
Resulting objects from \code{naOmit} are smaller in size and subsequent code execution (on large vectors) may be faster (in particular if many NAs get encountered).
Note : This function behaves differently to \code{na.omit} with input other than plain vectors. Will not work with data.frames !
}
\examples{
aA <- c(11:13,NA,10,NA); 
naOmit(aA) 
}
\seealso{
\code{\link[stats]{na.fail}}, \code{na.omit}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/colCVs.R
\name{colCVs}
\alias{colCVs}
\title{rowCVs}
\usage{
colCVs(dat, autoconvert = NULL, silent = FALSE, debug = FALSE, callFrom = NULL)
}
\arguments{
\item{dat}{(numeric) matix}

\item{autoconvert}{(NULL or character) allows converting simple vectors in matrix of 1 row (autoconvert="row")}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allows easier tracking of messages produced}
}
\value{
This function returns a (numeric) vector with CVs for each column of 'dat'
}
\description{
This function returns CV for values in each column (using speed optimized standard deviation).
Note : NaN values get replaced by NA.
}
\examples{
set.seed(2016); dat1 <- matrix(c(runif(200) +rep(1:10,20)),ncol=10)
head(colCVs(dat1))
}
\seealso{
\code{\link[base]{rowSums}}, \code{\link{rowCVs}}, \code{\link{rowGrpCV}}, \code{\link{colSds}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fusePairs.R
\name{fusePairs}
\alias{fusePairs}
\title{Fuse pairs to generate cluster-names}
\usage{
fusePairs(
  datPair,
  refDatNames = NULL,
  inclRepLst = FALSE,
  maxFuse = NULL,
  debug = FALSE,
  silent = TRUE,
  callFrom = NULL
)
}
\arguments{
\item{datPair}{2-column matrix where each line represents 1 pair}

\item{refDatNames}{(NULL or character) allows placing selected pairs in context of larger data-set (names to match those of 'datPair')}

\item{inclRepLst}{(logical) if TRUE, return list with 'clu' (clu-numbers, default output) and 'refLst' (list of clustered elements, only n>1)}

\item{maxFuse}{(integer, default NULL) maximal number of groups/clusters}

\item{debug}{(logical) display additional messages for debugging}

\item{silent}{(logical) suppress messages}

\item{callFrom}{(character) allow easier tracking of message(s) produced}
}
\value{
This function returns a vector with cluster-numbers
}
\description{
Fuse previously identified pairs to 'clusters', return vector with cluster-numbers.
}
\examples{
daPa <- matrix(c(1:5,8,2:6,9), ncol=2)
fusePairs(daPa, maxFuse=4)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cutToNgrp.R
\name{cutToNgrp}
\alias{cutToNgrp}
\title{Cut numeric vector to n groups (ie convert to factor)}
\usage{
cutToNgrp(
  x,
  lev,
  NAuse = FALSE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{x}{numeric vector}

\item{lev}{(character or numeric), the length of this argument tells the number of groups to be used for cutting}

\item{NAuse}{(logical) include NAs as separate group}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allows easier tracking of messages produced}
}
\value{
This function returns a list with \code{$grouped} telling which element of '\code{x}' goes in which group and \code{$legTxt} with gourp-borders for convenient use with legends
}
\description{
This function is a more elaborate version of \code{\link[base]{cut}} for cutting a the content of a 
numeric vector '\code{x}' into a given number of groups, taken from the length of '\code{lev}'.
Besides, this function provides the group borders/limits for convention use with legends.
}
\examples{
set.seed(2019); dat <- runif(30) +(1:30)/2
cutToNgrp(dat,1:5)
plot(dat,col=(1:5)[as.numeric(cutToNgrp(dat,1:5)$grouped)])
}
\seealso{
\code{\link[base]{cut}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convPairwiseSetup.R
\name{convPairwiseSetup}
\alias{convPairwiseSetup}
\title{Check Comparison-Choice}
\usage{
convPairwiseSetup(
  useComparison,
  grp,
  experSetup = NULL,
  sep = NULL,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{useComparison}{(numeric or character vector or matrix) the argument allowing to specify the pairwise comparions 
may be character as result of combining two group-names (from argument \code{grp}) (eg 'A-B', beware, the separator may not appear inside group-names) 
or 2-column matrix of group-names (thus, without separator; names combined with separator may be shown as rownames) or 2-column matrix of indexes to sorted group-names;
default setting of \code{useComparison='all'} will select all possible pairwise combinations}

\item{grp}{(character or factor)  vector defining groups of replicates for additional checking (may also be just the levels of grp)}

\item{experSetup}{(list) optional esperimental setup (list with $ind, $pwGrpIndex,$sep and $pwGrpNa  as obtained using \code{getPairwiseSetup})}

\item{sep}{(character) optional custom separator to use for splitting \code{useComparison}}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allows easier tracking of messages produced}
}
\value{
This function returns a list with $data, $nNA, $randParam, $NAneigLst, $seed, $filt, and $annot
}
\description{
This function aims to help getting organized with pairwise (statistical) testing since there are multiple ways of expressing how samples 
should be grouped for tests. For example this may be either as combined character strings (eg 'A-B') or as separate elements. 
This function takes most input-formats and returns multiple converted formats.
}
\details{
This function returns a list with $pwGrpNa (matrix with separated elements), $pwIndex (matrix with separated indexes), $grpNa (levels of \code{grp}, ie sorted),
 $sep (character vecor of separator found/used or suggested), $concat (pairwise concatenated names, eg 'A-B')
}
\examples{
convPairwiseSetup(c("B-C","D-A"), LETTERS[1:4])
convPairwiseSetup(c("B","C"), LETTERS[1:3])
convPairwiseSetup("all", LETTERS[1:3])
convPairwiseSetup(2:3, LETTERS[1:3])
convPairwiseSetup(2:3, LETTERS[1:3], sep="__")
convPairwiseSetup(matrix(c("B","D","C","A"), ncol=2), LETTERS[1:4])

}
\seealso{
\code{\link{presenceFilt}}, \code{\link{getPWseparator}}, \code{\link{indexGroupsFromPW}}, \code{\link{getPairwiseSetup}}, \code{\link[base]{strsplit}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/exclExtrValues.R
\name{exclExtrValues}
\alias{exclExtrValues}
\title{Exclude extreme values (based on distance to mean)}
\usage{
exclExtrValues(
  dat,
  result = "val",
  CVlim = NULL,
  maxExcl = 1,
  showNA = FALSE,
  goodValues = TRUE,
  silent = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{dat}{numeric vector, main input}

\item{result}{(character) may be 'val' for returning data without extreme values or 'pos' for returning position/index of extreme values}

\item{CVlim}{(NULL or numeric) allows to retain extreme values only if a certain CV (for all 'dat') is exceeded (to avoid calling extreme values form homogenous data-sets)}

\item{maxExcl}{(integer) max number of elments to explude}

\item{showNA}{(logical) will display extrelme values as NA}

\item{goodValues}{(logical) allows to display rather the good values instead of the extreme values}

\item{silent}{(logical) suppress messages}

\item{callFrom}{(character) allow easier tracking of message(s) produced}
}
\value{
numeric vector wo extremle values or index-position of extreme values
}
\description{
This function aims to identify extreme values (values most distant to mean, thus potential outlyers), mark them as NA or directely exclude them (depending on '\code{showNAs}').
Note that every set of non-identical values will have at least one most extreme value. Extreme values are part of many distributions, they are not necessarily true outliers.
}
\examples{
x <- c(rnorm(30),-6,20)
exclExtrValues(x)
}
\seealso{
\code{\link{firstOfRepLines}},  \code{\link{get1stOfRepeatedByCol}} for treatment of matrix
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rowMedSds.R
\name{rowMedSds}
\alias{rowMedSds}
\title{Estimate sd Of Median For Each Row By Bootstrap}
\usage{
rowMedSds(dat, nBoot = 99, silent = FALSE, debug = FALSE, callFrom = NULL)
}
\arguments{
\item{dat}{(numeric) matix, main input}

\item{nBoot}{(integer) number if iterations for bootstrap}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) display additional messages for debugging}

\item{callFrom}{(character) allows easier tracking of messages produced}
}
\value{
This functions returns a (numeric) vector with estimated sd values
}
\description{
This function determines the stand error (sd) of the median for each row by bootstraping each row of 'dat'.
Note: requires package \href{https://CRAN.R-project.org/package=boot}{boot}
}
\examples{
set.seed(2016); dat1 <- matrix(c(runif(200)+rep(1:10,20)), ncol=10)
rowMedSds(dat1) ; plot(rowSds(dat1), rowMedSds(dat1))
}
\seealso{
For a more flexible version able to handle lists please look at \code{\link{colMedSds}} ,  based on \code{\link[boot]{boot}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkFilePath.R
\name{.doubleExt}
\alias{.doubleExt}
\title{Return File-name Extensions Including Double Extensions (eg txt.gz)}
\usage{
.doubleExt(fiNa, termExt = c("gz", "zip"))
}
\arguments{
\item{fiNa}{(character) name of file to be tested (may include path)}

\item{termExt}{(character) additional terminal extensions}
}
\value{
This function returns a character vector with file-extensions (excluding the leading dot). (Only purely alphanumeric extensions are recognized.)
}
\description{
This function allows retreiving extensions to filenames similar to \code{\link[tools]{file_ext}}, but also alllows to retreive selected double-extensions (txt.gz).
The leading dor will ne excluded similar to \code{\link[tools]{file_ext}}
}
\details{
The argument \code{termExt} allows specifying additional (terminal) extensions to be recognized.
}
\examples{
fi <- c("ab","ab.c","ab.c.gz","ab.d","ab.d.zip","ab.e")
.doubleExt(fi)
}
\seealso{
\code{\link[tools]{file_ext}},  \code{\link{checkFilePath}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/confInt.R
\name{confInt}
\alias{confInt}
\title{Confidence Interval To Given Alpha}
\usage{
confInt(x, alpha = 0.05, distrib = "Normal", silent = FALSE, callFrom = NULL)
}
\arguments{
\item{x}{(numeric) main input}

\item{alpha}{(numeric) significance level, accepted type I error}

\item{distrib}{(character) distribution, so far only \code{Normal} is implemented}

\item{silent}{(logical) suppress messages}

\item{callFrom}{(character) allow easier tracking of message(s) produced}
}
\value{
This function returns the confidence interval to a given \code{alpha} under the hypothesis of the Normal distribution.
}
\description{
This little function returns the confidence interval associated to a given significance level \code{alpha} under the hypothesis of the Normal distribution is valid.
}
\examples{

confInt(c(5,2:6))

}
\seealso{
\code{\link[stats]{TDist}}; \code{\link[stats]{confint}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ratioToPpm.R
\name{ratioToPpm}
\alias{ratioToPpm}
\title{Convert ratio to ppm}
\usage{
ratioToPpm(
  x,
  y = NULL,
  nSign = NULL,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{x}{(numeric) main input}

\item{y}{(numeric) optional value to construct ratios (x/y). If NULL (or different length as 'x'), then 'x' will be considered as ratio.}

\item{nSign}{(numeric) number of significan digits}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a numeric vector of ppm values
}
\description{
This function transforms ratio 'x' to ppm (parts per million). 
If 'y' not given (or different length as 'x'), then 'x' is assumed as ratio otherise rations are constructed as x/y is used lateron.
Does additional checking : negative values not expected - will be made absolute !
}
\examples{
set.seed(2017); aa <- c(1.000001,0.999999,1+rnorm(10,0,0.001))
cbind(x=aa,ppm=ratioToPpm(aa,nSign=4))
}
\seealso{
\code{\link{XYToDiffPpm}} for ppm of difference as used in mass spectrometrie
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cutAtMultSites.R
\name{cutAtMultSites}
\alias{cutAtMultSites}
\title{Cut character-vector at multiple sites}
\usage{
cutAtMultSites(y, cutAt)
}
\arguments{
\item{y}{character vector (better if of length=1, otherwise one won't know which fragment stems from which input)}

\item{cutAt}{(character) search subtsting, ie 'cutting rule'}
}
\value{
modified (ie cut) character vector
}
\description{
This function cuts character vector after 'cutAt' (ie keep the search subtsting 'cutAt', different to \code{strsplit}). 
Used for theoretical enzymatic digestion (eg in proteomics)
}
\examples{
tmp <- "MSVSRTMEDSCELDLVYVTERIIAVSFPSTANEENFRSNLREVAQMLKSKHGGNYLLFNLSERRPDITKLHAKVLEFGWPDLHTPALEKI"
cutAtMultSites(c(tmp,"ojioRij"),c("R","K"))
}
\seealso{
\code{\link[base]{strsplit}}, \code{\link[wrMisc]{nFragments0}}, \code{\link[wrMisc]{nFragments}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/asSepList.R
\name{asSepList}
\alias{asSepList}
\title{Organize Data as Separate List-Entries}
\usage{
asSepList(
  y,
  minLen = 4,
  asNumeric = TRUE,
  exclElem = NULL,
  sep = "_",
  fillNames = TRUE,
  silent = FALSE,
  callFrom = NULL,
  debug = FALSE
)
}
\arguments{
\item{y}{list to be separated/split in vectors}

\item{minLen}{(integer) min length (or number of rows), as add'l element to eliminate arguments given without names when asSepList is called in vioplot2}

\item{asNumeric}{(logical) to transform all list-elements in simple numeric vectors (won't work if some entries are character)}

\item{exclElem}{(character) optinal names to exclude if any (lazy matching) matches (to exclude other arguments be misinterpreted as data)}

\item{sep}{(character) separator when combining name of list-element to colames}

\item{fillNames}{(logical) add names for list-elements/ series when not given}

\item{silent}{(logical) suppress messages}

\item{callFrom}{(character) allow easier tracking of messages produced}

\item{debug}{(logical) display additional messages for debugging}
}
\value{
This function returns a list, partially unlisted to vectors
}
\description{
\code{asSepList} allows reorganizing most types of input into a list with separate numeric vectors. For example, matrixes or data.frames will be split into separate columns
(differnt to \code{\link[wrMisc]{partUnlist}} which maintains the original structure). This function also works with lists of lists.
This function may be helpful for reorganizing data for plots.
}
\examples{
bb <- list(fa=gl(2,2), c=31:33, L2=matrix(21:28,nc=2),
  li=list(li1=11:14, li2=data.frame(41:44)))
asSepList(bb)
## multi data-frame examples
ca <- data.frame(a=11:15, b=21:25, c=31:35)
cb <- data.frame(a=51:53, b=61:63)
cc <- list(gl(3,2), ca, cb, 91:94, short=81:82, letters[1:5])
asSepList(cc)
cd <- list(e1=gl(3,2), e2=ca, e3=cb, e4=91:94, short=81:82, e6=letters[1:5])
asSepList(cd)
}
\seealso{
\code{\link{partUnlist}}, \code{\link[base]{unlist}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coinPermTest.R
\name{coinPermTest}
\alias{coinPermTest}
\title{Compare Means Of Two Vectors By Permutation Test}
\usage{
coinPermTest(
  x1,
  x2,
  orient = "two.sided",
  nPerm = 5000,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{x1}{numeric vector (to be compared with vector 'x2')}

\item{x2}{numeric vector (to be compared with vector 'x1')}

\item{orient}{(character) may be "two.sided","greater" or "less"}

\item{nPerm}{(integer) number of permutations}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns an object of "MCp" class numeric output with p-values
}
\description{
Run coin-flipping like permutation tests (to compare difference of 2 means: 'x1' and 'x2') without any distribution-assumptions.
This function uses the package \href{https://CRAN.R-project.org/package=coin}{coin}, if not installed, the function will return NULL and give a warning.
}
\examples{
coinPermTest(2, 3, nPerm=200)
}
\seealso{
\code{oneway_test} in \code{\link[coin]{LocationTests}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makeNRedMatr.R
\name{.sortMid}
\alias{.sortMid}
\title{Choose most frequent or middle of sorted vector}
\usage{
.sortMid(x, retVal = TRUE)
}
\arguments{
\item{x}{(numeric) main input}

\item{retVal}{(logical) return value of most frequent, if \code{FALSE} return index of (1st) 'x' for most frequent}
}
\value{
This function returns a numeric verctor
}
\description{
This function chooses the (first) most frequent or  middle of sorted vector, similar to the concept of \code{mode}
}
\examples{
.sortMid(11:14)
.sortMid(rep("b",3))
}
\seealso{
simple/partial functionality in \code{\link{summarizeCols}},  \code{\link{checkSimValueInSer}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pasteC.R
\name{.seqCutStr}
\alias{.seqCutStr}
\title{Cut string to get all variants from given start with min length, depreciated}
\usage{
.seqCutStr(txt, startFr = 1, minLe = 1, reverse = TRUE)
}
\arguments{
\item{txt}{(character) main input, may be length >1}

\item{startFr}{(interger) where to start}

\item{minLe}{(interger) minimum length of output}

\item{reverse}{(logical) return longest text-fragments at beginning of vector}
}
\value{
This function returns a character vector
}
\description{
This function is depreciated, please use \code{/cutStr} instead !
This function allows truncating character vector to all variants from given start, with min and optonal max length
Used to evaluate argument calls without giving full length of argument
}
\examples{
.seqCutStr("abcdefg", minLe=2)
}
\seealso{
\code{\link{pasteC}}; \code{\link[base]{substr}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matr2list.R
\name{matr2list}
\alias{matr2list}
\title{Transform columns of matrix to list of vectors}
\usage{
matr2list(mat, concSym = ".", silent = FALSE, debug = TRUE, callFrom = NULL)
}
\arguments{
\item{mat}{(matrix) main input}

\item{concSym}{(character) symbol for concatenating: concatenation of named vectors in list names as colname(s)+'concSym'+rowname}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
matrix or array (1st dim is intraplate-position, 2nd .. plate-group/type, 3rd .. channels)
}
\description{
convert matrix to list of vectors: each column of 'mat' as vector of list
}
\examples{
mat1 <- matrix(1:12,ncol=3,dimnames=list(letters[1:4],LETTERS[1:3]))
mat2 <- matrix(LETTERS[11:22],ncol=3,dimnames=list(letters[1:4],LETTERS[1:3]))
matr2list(mat1);  matr2list(mat2)
}
\seealso{
\code{\link{convToNum}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transpGraySca.R
\name{transpGraySca}
\alias{transpGraySca}
\title{Make single vector gray-gradient}
\usage{
transpGraySca(startGray = 0.2, endGrey = 0.8, nSteps = 5, transp = 0.3)
}
\arguments{
\item{startGray}{(numeric) gray shade at start}

\item{endGrey}{(numeric)  gray shade at end}

\item{nSteps}{(integer) number of levels}

\item{transp}{(numeric) transparency alpha}
}
\value{
character vector (of same length as x) with color encoding
}
\description{
This function helps making gray-gradients.
Note : The resulting color gradient does not seem linear to the human eye, you may try \code{\link[grDevices]{gray.colors}} instead
}
\examples{
layout(1:2)
col1 <- transpGraySca(0.8,0.3,7,0.9) 
pie(rep(1,length(col1)), col=col1, main="from transpGraySca")
col2 <- gray.colors(7,0.9,0.3,alph=0.9)
pie(rep(1,length(col2)), col=col2, main="from gray.colors")
}
\seealso{
\code{\link[grDevices]{gray.colors}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nNonNumChar.R
\name{.setLowestTo}
\alias{.setLowestTo}
\title{Set lowest value to given value}
\usage{
.setLowestTo(x, setTo)
}
\arguments{
\item{x}{(numeric) main vector to be treated}

\item{setTo}{(numeric) replacement value}
}
\value{
This function returns a numeric vector
}
\description{
This function aims to set lowest value of x to value 'setTo'
}
\examples{
.setLowestTo(9:4, 6)
}
\seealso{
\code{\link[base]{nchar}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mergeSelCol.R
\name{mergeSelCol}
\alias{mergeSelCol}
\title{Merge selected columns out of 2 matrix or data.frames}
\usage{
mergeSelCol(
  dat1,
  dat2,
  selCols,
  supCols2 = NULL,
  byC = NULL,
  useAll = FALSE,
  setRownames = TRUE,
  newSuff = c(".x", ".y"),
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{dat1}{matrix or data.frame for fusing}

\item{dat2}{matrix or data.frame for fusing}

\item{selCols}{will be used to define columns to be used; optionally may be different for 'dat2' : define in 'supCols2'}

\item{supCols2}{if additional column-names should be extracted form dat2}

\item{byC}{(character) 'by' value used in \code{\link[base]{merge}}}

\item{useAll}{(logical) use all lines (will produce NAs when given identifyer not found un 2nd group of data)}

\item{setRownames}{(logical) if \code{TRUE}, will use values of col used as 'by' as rownames instead of showing as add'l col in output}

\item{newSuff}{(character) prefix (argument 'suffixes' in \code{merge})}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) display additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a data.frame containing the merged columns
}
\description{
This function merges selected columns out of 2 matrix or data.frames.
'selCols' will be used to define columns to be used; optionally may be different for 'dat2' : define in 'supCols2'. 
Output-cols will get additions specified in newSuff (default '.x' and '.y')
}
\examples{
mat1 <- matrix(c(1:7,letters[1:7],11:17), ncol=3, dimnames=list(LETTERS[1:7],c("x1","x2","x3")))
mat2 <- matrix(c(1:6,c("b","a","e","f","g","k"), 31:36),
  ncol=3, dimnames=list(LETTERS[11:16],c("y1","x2","x3")))
mergeSelCol(mat1, mat2, selC=c("x2","x3"))
}
\seealso{
\code{\link[base]{merge}}, merge 3 data.frames using \code{\link{mergeSelCol3}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/protectSpecChar.R
\name{protectSpecChar}
\alias{protectSpecChar}
\title{Protect Special Characters}
\usage{
protectSpecChar(
  x,
  prot = c(".", "\\\\", "|", "(", ")", "[", "{", "^", "$", "*", "+", "?"),
  silent = TRUE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{x}{character vector to be prepared for use in regular expressions}

\item{prot}{(character) collection of characters that need to be protected}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a modified character vector
}
\description{
Some characters do have a special meaning when used with regular expressions.
This concerns characters like a point, parinthesis, backslash etc.
Thus, when using \code{grep} or any related command, shuch special characters must get protected in order to get considered as they are.
}
\examples{
aa <- c("abc","abcde","ab.c","ab.c.e","ab*c","ab\\\\d")
grepl("b.", aa)             # all TRUE
grepl("b\\\\.", aa)           # manual prootection
grepl(protectSpecChar("b."), aa)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/searchLinesAtGivenSlope.R
\name{.checkLmConfInt}
\alias{.checkLmConfInt}
\title{Compare 'dat' to confindence interval of linare model 'lMod' (eg from lm())}
\usage{
.checkLmConfInt(dat, lMod, level = 0.95)
}
\arguments{
\item{dat}{matrix or data.frame, main input}

\item{lMod}{linear model, only used to extract coefficients offset & slope}

\item{level}{(numeric) alpha threshold for linear model}
}
\value{
This function returns a logical vector for each value in 2nd col of 'dat' if INSIDE confid interval
}
\description{
This function allows to compare 'dat' to confindence interval of linare model 'lMod' (eg from lm())
}
\examples{
set.seed(2016); dat1 <- matrix(c(runif(200)+rep(1:10,20)),ncol=10)
}
\seealso{
\code{\link{searchLinesAtGivenSlope}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sortBy2CategorAnd1intCol.R
\name{sortBy2CategorAnd1IntCol}
\alias{sortBy2CategorAnd1IntCol}
\title{Sort matrix by two categorical and one integer columns}
\usage{
sortBy2CategorAnd1IntCol(
  mat,
  categCol,
  numCol,
  findNeighb = TRUE,
  decreasing = FALSE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{mat}{matrix (or data.frame) from which by 2 columns will be selected for sorting}

\item{categCol}{(integer or character) which columns of 'mat' to be used as categorical columns}

\item{numCol}{(integer or character) which column of 'mat' to be used as integer columns}

\item{findNeighb}{(logical) if 'findNeighb' neighbour cols according to 'numCol' will be identified as groups & marked in new col 'neiGr', orphans marked as NA}

\item{decreasing}{(logical) order of sort}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a sorted matrix (same dimensions as 'mat')
}
\description{
This function sorts matrix 'mat' subsequently by categorical and numerical columns of 'mat',
 ie lines with identical values for categor are sorted by numeric value.
}
\examples{
mat <- cbind(aa=letters[c(3,rep(7:8,3:4),4,4:6,7)],bb=LETTERS[rep(1:5,c(1,3,4,4,1))],
  nu=c(23:21,23,21,22,18:12))
mat[c(3:5,1:2,6:9,13:10),]
sortBy2CategorAnd1IntCol(mat,cate=c("bb","aa"),num="nu",findN=FALSE,decr=TRUE)
sortBy2CategorAnd1IntCol(mat,cate=c("bb","aa"),num="nu",findN=TRUE,decr=FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkVectLength.R
\name{.replSpecChar}
\alias{.replSpecChar}
\title{Replace Special Characters}
\usage{
.replSpecChar(x, findSp = c("\\\\(", "\\\\)", "\\\\$"), replBy = "_")
}
\arguments{
\item{x}{(character) main input}

\item{findSp}{(character) special characters to replace (may have to be given as protected)}

\item{replBy}{(character) replace by}
}
\value{
This function returns a corrceted/adjusted factor
}
\description{
This function allows replacing special characters
Note that (most) special characters must be presented with protection for \code{grep} and \code{sub}.
}
\examples{
.replSpecChar(c("jhjh(ab)","abc"))
}
\seealso{
\code{\link[base]{factor}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/appendNR.R
\name{appendNR}
\alias{appendNR}
\title{Append vectors or lists, without duplcating common elements}
\usage{
appendNR(
  x,
  y,
  rmDuplicate = TRUE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{x}{(vector or list) must have names to allow checking for duplicate names in y}

\item{y}{(vector or list) must have names to allow checking for duplicate names in x}

\item{rmDuplicate}{(logical) avoid duplicating liste-elements present in both x and y (based on names of list-elements)}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of message(s) produced}
}
\value{
If both \code{x} and \code{y} are vectors, the output will be a vector, otherwise it will be a list
}
\description{
This function allows combining two vectors or lists without duplicating common content (definded by name of list-elements).
}
\details{
When setting the argument \code{rmDuplicate=FALSE} the function will behave like \code{append}.
}
\examples{
li1 <- list(a=1, b=2, c=3)
li2 <- list(A=11, B=12, c=3)
appendNR(li1, li2)
append(li1, li2) 
}
\seealso{
\code{\link[base]{append}}; \code{\link{lrbind}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/findRepeated.R
\name{findRepeated}
\alias{findRepeated}
\title{Find repeated elements}
\usage{
findRepeated(x, nonRepeated = FALSE, silent = FALSE, callFrom = NULL)
}
\arguments{
\item{x}{character vector}

\item{nonRepeated}{(logical) if \code{=TRUE}, return list with elements \code{$rep} and \code{$nonrep}}

\item{silent}{(logical) suppress messages}

\item{callFrom}{(character) allow easier tracking of message(s) produced}
}
\value{
(named) list of indexes for each of the repeated values, or NULL if all values unique
}
\description{
\code{findRepeated} gets index of repeated items/values in vector 'x' (will be treated as character). 
Return (named) list of indexes for each of the repeated values, or \code{NULL} if all values are unique.
This approach is similar but more basic compared to \code{\link{get1stOfRepeatedByCol}}.
}
\examples{
aa <- c(11:16,14:12,14); findRepeated(aa) 
}
\seealso{
similar approach but more basic than \code{\link{get1stOfRepeatedByCol}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/partialDist.R
\name{partialDist}
\alias{partialDist}
\title{Partial distance matrix (focus on closest)}
\usage{
partialDist(
  dat,
  groups,
  overLap = TRUE,
  method = "euclidean",
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{dat}{(matrix of numeric values) main input}

\item{groups}{(factor) to split using \code{cut} or specific custom grouping (length of dat)}

\item{overLap}{(logical) if TRUE make groups overlapping by 1 value (ie maintain some context-information)}

\item{method}{'character' name of method passed to \code{dist}}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) display additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of message(s) produced}
}
\value{
This function returns a matrix with partial distances (not of class 'dist' object)
}
\description{
\code{partialDist} calculates distance matrix like \code{dist} for 1- or 2-dim data, but only partially, ie only cases of small distances.
This function was made for treating very large data-sets where only very close distances to a given point need to be found,
it allows to overcome memory-problems with larger data (and faster execution with > 50 rows of 'dat').
}
\examples{
set.seed(2016); mat3 <- matrix(runif(300),nr=30)
round(dist(mat3), 1)
round(partialDist(mat3, gr=3), 1)
}
\seealso{
\code{\link[stats]{dist}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/findUsableGroupRange.R
\name{findUsableGroupRange}
\alias{findUsableGroupRange}
\title{Select groups within given range}
\usage{
findUsableGroupRange(dat, grp, maxNA = 1, callFrom = NULL)
}
\arguments{
\item{dat}{(matrix or data.frame) main input}

\item{grp}{(factor) information which column of 'dat' is replicate of whom}

\item{maxNA}{(interger) max number of tolerated NAs}

\item{callFrom}{(character) allow easier tracking of message(s) produced}
}
\value{
matrix with boundaries of 1st and last usable column (NA if there were no suitable groups found)
}
\description{
This function aims to help finding streches/segments of data with a given maximum number of NA-instances.
This function is used to inspect/filter each lines of 'dat' for a subset with sufficient presence/absence of NA values (ie limit number of NAs per level of 'grp').
Note :  optimal perfomance with n.lines >> n.groups
}
\examples{
dat1 <- matrix(1:56,nc=7)
dat1[c(2,3,4,5,6,10,12,18,19,20,22,23,26,27,28,30,31,34,38,39,50,54)] <- NA
rownames(dat1) <- letters[1:nrow(dat1)]
findUsableGroupRange(dat1,gl(3,3)[-(3:4)])
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makeMAList.R
\name{.allRatios}
\alias{.allRatios}
\title{Search character-string and cut either before or after}
\usage{
.allRatios(dat, ty = "log2", colNaSep = "_")
}
\arguments{
\item{dat}{(matrix or data.frame) main input}

\item{ty}{(character) type of ratio (eg 'log2')}

\item{colNaSep}{(character) separator}
}
\value{
This function returns a numeric vector
}
\description{
This function extracts/cuts text-fragments out of \code{txt} following specific anchors defined by arguments \code{cutFrom} and \code{cutTo}.
}
\examples{
.allRatios(matrix(11:14, ncol=2))
}
\seealso{
\code{\link{makeMAList}}, \code{\link[base]{grep}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/combineAsN.R
\name{combineAsN}
\alias{combineAsN}
\title{Combine Vectors From List And Return Basic Count Statistics}
\usage{
combineAsN(
  lst,
  lev = NULL,
  nCombin = 3,
  remDouble = TRUE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{lst}{(list of character or integer vectors) main input}

\item{lev}{(character) define groups of \code{lst}}

\item{nCombin}{(integer) number of list-elements to combine from \code{lst}}

\item{remDouble}{(logical) remove intra-duplicates (defaults to \code{TRUE})}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns an array with 3 dimensions : i)  ii) the combinations of \code{nCombin} list-elements,
  iii) the number of counts (n), sem (standard error of the mean), CI (confidence interval) and sd
}
\description{
The aim of this function is to choose a fixed number (\code{nCombin}) of list-elments from \code{lst} and count the number of common values/words.
Furthermore, one can define levels to fine-tune the types of combinations to examine.
In case multiple combinations for a given level are possible, some basic summary statistics are provided, too.
}
\details{
Note of caution :
With very long lists and/or high numbers of repeats of given levels, however, the computational effort incerases very much (like it does when using \code{table}).
Thus, when exploring all different combinations of large data-sets may easily result in queries consuming many ressources (RAM and processing time) !
It is recommended to start testing with test smaller sub-groups.

The main idea of this function is to count frequency of terms when combining different drawings.
For example, you ask students from different cities which are their preferred hobbies, they may have different preference depending on the city ( defined by \code{lev}).
Now, if you want to make groups of 3 students, possibly with one from each city (A ,B and C), you want to count (/estimate) the frequency of different combinations possible.
Thus, using this function all combinations of the students from city A with the students from city B and C will be made when counting the number of common hobbies (by \code{nCombin} students).
Then, all counting results will be summarized to the average count for the various categories (which hobbies were seen once, twice or 3 times...),
sem (standard error of the mean) and CI (95% confidence interval), as well as sd.

Of course, the number of potential combinations may quickly get very large. Using the argument \code{remDouble=TRUE} you can limit the search to
either finding all students giving the same answer plus all student giving different answers.
In this case, when a given level appears multiple times, all possible combinations using one of the respective entries will be be made with the other levels.
}
\examples{
## all list-elements are considered equal
tm1 <- list(a1=LETTERS[1:17], a2=LETTERS[3:19], a3=LETTERS[6:20], a4=LETTERS[8:22])
combineAsN(tm1, lev=gl(1,4))[,1,]

## different levels/groups in list-elements
tm4 <- list(a1=LETTERS[1:15], a2=LETTERS[3:16], a3=LETTERS[6:17], a4=LETTERS[8:19],
  b1=LETTERS[5:19], b2=LETTERS[7:20], b3=LETTERS[11:24], b4=LETTERS[13:25], c1=LETTERS[17:26],
  d1=LETTERS[4:12], d2=LETTERS[5:11], d3=LETTERS[6:12], e1=LETTERS[7:10])
te4 <- combineAsN(tm4, nCombin=4, lev=substr(names(tm4),1,1))
str(te4)
te4[,,1]

}
\seealso{
\code{\link[base]{table}},  \code{\link[wrMisc]{replicateStructure}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nFragments.R
\name{nFragments}
\alias{nFragments}
\title{Number Of Fragments After Cut At Specific Character(s) Within Size-range}
\usage{
nFragments(
  protSeq,
  cutAt,
  sizeRa,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{protSeq}{(character)  text to be cut}

\item{cutAt}{(character) position to cut}

\item{sizeRa}{(numeric,length=2) min and max size to consider}

\item{silent}{(logical) suppress messages if \code{TRUE}}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allows easier tracking of messages produced}
}
\value{
This function returns a numeric vector with number of fragments for each entry 'protSeq' (names are 'protSeq')
}
\description{
This function determines number of fragments /entry within range of 'sizeRa' (numeric,length=2) when cutting after 'cutAt'
}
\examples{
tmp <- "MSVSREDSCELDLVYVTERIIAVSFPSTANEENFRSNLREVAQMLKSKHGGNYLLFNLSERRPDITKLHAKVLEFGWPDLHTPALEKI"
nFragments(c(tmp,"ojioRij"),c("R","K"),c(4,31))

}
\seealso{
\code{\link{cutAtMultSites}}, simple version \code{{nFragments0}} (no size-range)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extractLast2numericParts.R
\name{extractLast2numericParts}
\alias{extractLast2numericParts}
\title{Extract Last Two Numeric Parts From Character Vector}
\usage{
extractLast2numericParts(x, silent = FALSE, debug = FALSE, callFrom = NULL)
}
\arguments{
\item{x}{main character input}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a (numeric) matrix with 2 columns (eg from initial concatenated coordinates)
}
\description{
This function extracts last 2 (integer) numeric parts between punctuations out of character vector 'x'.
Runs faster than \code{gregexpr} .
Note: won't work correctly with decimals or exponential signs !! (such characters will be considered as punctuation, ie as separator)
}
\examples{
extractLast2numericParts(c("M01.1-4","M001/2.5","M_0001_03-16","zyx","012","a1.b2.3-7,2"))
}
\seealso{
\code{gregexpr} from \code{\link[base]{grep}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/firstOfRepeated.R
\name{firstOfRepeated}
\alias{firstOfRepeated}
\title{Find first of repeated elements}
\usage{
firstOfRepeated(x, silent = FALSE, debug = FALSE, callFrom = NULL)
}
\arguments{
\item{x}{(charcter or numeric) main input}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) display additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of message(s) produced}
}
\value{
list with indices: $indRepeated,  $indUniq, $indRedund
}
\description{
This function works similar to \code{unique}, but provides additional information about which elements of original input \code{'x'} are repeatd by providing indexes realtoe to the input.
\code{firstOfRepeated} makes list with 3 elements : $indRepeated.. index for first of repeated 'x', $indUniq.. index of all unique + first of repeated, $indRedund.. index of all redundant entries, ie non-unique (wo 1st).
Used for reducing data to non-redundant status, however, for large numeric input the function nonAmbiguousNum() may perform better/faster.
NAs won't be considered (NAs do not appear in reported  index of results), see also firstOfRepLines() .
}
\examples{
x <- c(letters[c(3,2:4,8,NA,3:1,NA,5:4)]); names(x) <- 100+(1:length(x))
firstOfRepeated(x)
x[firstOfRepeated(x)$indUniq]          # only unique with names
}
\seealso{
\code{\link[base]{duplicated}}, \code{\link{nonAmbiguousNum}}, \code{\link{firstOfRepLines}} gives less detail in output (lines/elements/indexes of omitted not directly accessible) and works fsster
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/colSds.R
\name{colSds}
\alias{colSds}
\title{sd for each column}
\usage{
colSds(dat, silent = FALSE, debug = FALSE, callFrom = NULL)
}
\arguments{
\item{dat}{matrix (or data.frame) with numeric values (may contain NAs which will be ignored)}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allows easier tracking of messages produced}
}
\value{
numeric vector of sd values
}
\description{
This function is speed optimized \code{sd} per column of a matrix or data.frame and treats each column as independent set of data for sd (equiv to \code{apply(dat,2,sd)}).
NAs are ignored from data.  Speed improvements may be seen at more than 100 columns
}
\examples{
set.seed(2016); dat1 <- matrix(c(runif(200) +rep(1:10,20)), ncol=10)
colSds(dat1)
}
\seealso{
\code{\link[stats]{sd}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/firstOfRepLines.R
\name{firstOfRepLines}
\alias{firstOfRepLines}
\title{Reduce To First Occurance Of Repeated Lines}
\usage{
firstOfRepLines(
  mat,
  outTy = "ind",
  useCol = NULL,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{mat}{initial matrix to treat}

\item{outTy}{for output type: 'ind'.. index to 1st occurance (non-red),'orig'..non-red lines of mat, 'conc'.. non-red concateneted values, 'num'.. index to which group/category the lines belong}

\item{useCol}{(integer) custom choice of which columns to paste/concatenate}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a simplified/non-redundant vector/matrix (ie fewer lines for matrix), or respective index
}
\description{
This function concatenattes all columns of input-matrix and then searches like \code{unique} for unique elements, optionally the indexes of unique elements may get returned.
Note: This function reats input as character (thus won't understand \code{10==10.0} ).
Returns simplified/non-redundant vector/matrix (ie fewer lines), or respective index.
faster than  \code{\link{firstOfRepeated}}
}
\examples{
mat <- matrix(c("e","n","a","n","z","z","n","z","z","b", 
  "","n","c","n","","","n","","","z"),ncol=2)
firstOfRepLines(mat,out="conc")
}
\seealso{
\code{\link[base]{unique}}, \code{\link{nonAmbiguousNum}}, faster than \code{\link{firstOfRepeated}} which gives more detail in output (lines/elements/indexes of omitted)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filterLiColDeList.R
\name{filterLiColDeList}
\alias{filterLiColDeList}
\title{Filter lines(rows) and/or columns from all suitable elements of list}
\usage{
filterLiColDeList(
  lst,
  useLines,
  useCols = NULL,
  ref = 1,
  silent = FALSE,
  callFrom = NULL,
  debug = FALSE
)
}
\arguments{
\item{lst}{(list or S3 object) main input}

\item{useLines}{(integer, logcial or character) vector to assign lines to keep when filtering along lines;
set to \code{NULL} for no filtering; if '\code{allNA}' all lines composed uniquely of \code{NA} values will be removed.}

\item{useCols}{(integer, logcial or character) vector for filtering columns; set to \code{NULL} for no filtering; if '\code{allNA}' all columns uniquely \code{NA} values will be removed}

\item{ref}{(integer) index for designing the elment of 'lst' to take as reference for checking which other list-elements have suitable number of rows or columns}

\item{silent}{(logical) suppress messages}

\item{callFrom}{(character) allow easier tracking of messages produced}

\item{debug}{(logical) additional messages for debugging}
}
\value{
This function returns the correct(ed) input (object of same class, of same length)
}
\description{
Filter all elements of list (or S3-object) according to criteria designed to one selected reference-element of the list.
All simple vectors, matrix, data.frames and 3-dimensional arrays will be checked if matching number of rows and/or columns to decide if they should be filtered the same way.
If the reference element has same number of rows and columns simple (1-dimensional) vectors won't be filtered since it not clear if this should be done to lines or columns.
}
\details{
This function is used eg in package wrProteo to simultaneaously filter raw and transformed data.
}
\examples{
lst1 <- list(m1=matrix(11:18,ncol=2), m2=matrix(21:30,ncol=2), indR=31:34,
  m3=matrix(c(21:23,NA,25:27,NA),ncol=2))
## here $m2 has more lines than $m1, and thus will be ignored when ref=1
filterLiColDeList(lst1, useLines=2:3)
filterLiColDeList(lst1, useLines="allNA", ref=4)

}
\seealso{
\code{\link{moderTest2grp}} for single comparisons, \code{\link[limma]{lmFit}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/treatTxtDuplicates.R
\name{treatTxtDuplicates}
\alias{treatTxtDuplicates}
\title{Locate duplicates in text and make non-redundant}
\usage{
treatTxtDuplicates(
  x,
  atEnd = TRUE,
  sep = "_",
  onlyCorrectToUnique = FALSE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{x}{(character) vector with character-entries to identify (and remove) duplicates}

\item{atEnd}{(logical) decide location of placing the counter (at end or at beginning of ID) (see \code{\link{correctToUnique}})}

\item{sep}{(character) separator to add before counter when making non-redundant version}

\item{onlyCorrectToUnique}{(logical) if TRUE, return only vector of non-redundant}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
list with $init, $nRed, $nrLst
}
\description{
\code{treatTxtDuplicates} locates duplictes in character-vector 'x' and return list (length=3) : with $init (initial),
  $nRed .. non-redundant text by adding number at end or beginning, and $nrLst .. list-version with indexes per unique entry. 
 Note : NAs (if multiple) will be renamed to NA_1, NA_2
}
\examples{
treatTxtDuplicates(c("li0",NA,rep(c("li2","li3"),2)))
correctToUnique(c("li0",NA,rep(c("li2","li3"),2)))
}
\seealso{
For simple correction use \code{\link{correctToUnique}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fuseCommonListElem.R
\name{fuseCommonListElem}
\alias{fuseCommonListElem}
\title{Fuse content of list-elements with redundant (duplicated) names}
\usage{
fuseCommonListElem(
  lst,
  initOrd = TRUE,
  removeDuplicates = FALSE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{lst}{(list) main input, list of numeric vectors}

\item{initOrd}{(logical) preserve initial order in output (if TRUE) or otherwise sort alphabetically}

\item{removeDuplicates}{(logical) allow to remove duplicate entries (if vector contains names, both the name and the value need to be identical to be removed; note: all names must have names with more than 0 characters to be considered as names)}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a fused list (same names as elements of input)
}
\description{
This function fuses (character or numeric) elements of list re-occuring under same name, so that resultant list has unique names.
Note : Will not work with list of matrixes
}
\examples{
val1 <- 10 +1:26
names(val1) <- letters
lst1 <- list(c=val1[3:6],a=val1[1:3],b=val1[2:3],a=val1[12],c=val1[13])
fuseCommonListElem(lst1)
}
\seealso{
\code{\link[base]{unlist}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/combinatIntTable.R
\name{combinatIntTable}
\alias{combinatIntTable}
\title{Planing for making all multiplicative combinations}
\usage{
combinatIntTable(
  nMax,
  include0 = TRUE,
  asList = FALSE,
  callFrom = NULL,
  silent = TRUE
)
}
\arguments{
\item{nMax}{(positive integer) could be max number of voting participants form different cities, eg Paris max 2 persons, Lyon max 1 person ...}

\item{include0}{(logical) include 0 occurances, ie provide al combinations starting from 0 or from 1 up to nMax}

\item{asList}{(logical) return result as list or as array}

\item{callFrom}{(character) allow easier tracking of messages produced}

\item{silent}{(logical) suppress messages}
}
\value{
list or array (as 2- or 3 dim) with possible number of occurances for each of the 3 elements in nMax. Read results vertical : out[[1]] or out[,,1] .. (multiplicative) table for 1st element of nMax; out[,,2] .. for 2nd
}
\description{
Provide all combinations for each of n elements of vector 'nMax' (positive integer, eg number of max multiplicative value). 
For example, imagine, we have 3 cities and the (maximum) voting participants per city.
Results must be read vertically and allow to see all total possible compositons.
}
\examples{
combinatIntTable(c(1,1,1,2), include0=TRUE, asList=FALSE, silent=TRUE)
## Imagine we have 3 cities and the (maximum) voting participants per city :
nMa <- c(Paris=2, Lyon=1, Strasbourg=1)
combinatIntTable(nMa, include0=TRUE, asList=TRUE, silent=TRUE) 
}
\seealso{
\code{\link[utils]{combn}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filtSizeUniq.R
\name{filtSizeUniq}
\alias{filtSizeUniq}
\title{Filter For UniqueElements}
\usage{
filtSizeUniq(
  lst,
  ref = NULL,
  minSize = 6,
  maxSize = 36,
  filtUnique = TRUE,
  byProt = TRUE,
  inclEmpty = TRUE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{lst}{list of character-vectors or character-vector}

\item{ref}{(character) optional alternative 'reference', if not \code{NULL} used in addition to 'lst' for considering elements of 'lst' as unique}

\item{minSize}{(integer) minimum number of characters, if \code{NULL} set to 0}

\item{maxSize}{(integer) maximum number of characters}

\item{filtUnique}{(logical) if \code{TRUE} return unique-only character-strings}

\item{byProt}{(logical) if \code{TRUE} organize output as list (by names of input, eg protein-names) - if 'lst' was named list}

\item{inclEmpty}{(logical) optional including empty list-elements when all elements have been filtered away - if 'lst' was named list}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
list of filtered input
}
\description{
This function aims to identify and remove duplicated elements in a list and maintain the list-structure in the output. 
\code{filtSizeUniq}  filters 'lst' (list of character-vectors or character-vector) for elements being unique (to 'ref' or if NULL to all 'lst') and of character length. 
In addition, the min- and max- character length may be filtered, too. Eg, in proteomics this helps removing peptide sequences which would not be measured/detected any way.
}
\examples{
filtSizeUniq(list(A="a",B=c("b","bb","c"),D=c("dd","d","ddd","c")),filtUn=TRUE,minSi=NULL)
# input: c and dd are repeated
filtSizeUniq(list(A="a",B=c("b","bb","c"),D=c("dd","d","ddd","c")),ref=c(letters[c(1:26,1:3)],
  "dd","dd","bb","ddd"),filtUn=TRUE,minSi=NULL)  # a,b,c,dd repeated 
}
\seealso{
\code{\link{correctToUnique}}, \code{\link[base]{unique}}, \code{\link[base]{duplicated}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ratioAllComb.R
\name{.checkLegendLoc}
\alias{.checkLegendLoc}
\title{Check argument for Location of legend}
\usage{
.checkLegendLoc(
  legLoc,
  defLoc = "topright",
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{legLoc}{(character) main input}

\item{defLoc}{(character)}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a character vector designing the potential location of legend
}
\description{
This function allows checking an argument for Location of legend,
if value provided not found as valid, it returns 'defLoc
}
\examples{
.checkLegendLoc("abc")
}
\seealso{
\code{\link[graphics]{legend}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sumNAperGroup.R
\name{sumNAperGroup}
\alias{sumNAperGroup}
\title{Count number of NAs per sub-set of columns}
\usage{
sumNAperGroup(
  x,
  grp,
  asRelative = FALSE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{x}{matrix or data.frame which may contain \code{NA}s}

\item{grp}{factor describing which column of 'dat' belongs to which group}

\item{asRelative}{(logical) return as count of \code{NA}s per row and group}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns an integer vector with count of \code{NA}s per group
}
\description{
This function will count the number of \code{NA}s per group (defined by argument \code{grp}) while summing over all lines of a matrix or data.frame.
The row-position has no influence on the counting.
Using the argument \code{asRelative=TRUE} the result will be given as (average) number of \code{NA}s per row and group.
}
\examples{
mat <- matrix(1:25, ncol=5) 
mat[lower.tri(mat)] <- NA
sumNAperGroup(mat, rep(1:2,c(3,2)))
sumNAperGroup(mat, rep(1:2,c(3,2)), asRelative=TRUE)

}
\seealso{
\code{\link[base]{NA}}, filter \code{NA}s by line \code{\link{presenceFilt}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkGroupOrderSEM.R
\name{checkGrpOrderSEM}
\alias{checkGrpOrderSEM}
\title{Check order of multiple groups including non-overlapping SEM-margins}
\usage{
checkGrpOrderSEM(
  x,
  grp,
  sdFact = 1,
  revRank = TRUE,
  shrink1sampSd = TRUE,
  silent = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{x}{matrix or data.frame}

\item{grp}{(factor) to organize replicate columns of (x)}

\item{sdFact}{(numeric) is proportional factor how many units of SEM will be used for defining lower & upper bounds of each group}

\item{revRank}{(logical) optionally revert ranks}

\item{shrink1sampSd}{(logical)}

\item{silent}{(logical) suppress messages}

\item{callFrom}{(character) allow easier tracking of message(s) produced}
}
\value{
logical vector if order correct (as expected based on ranks)
}
\description{
\code{checkGrpOrderSEM} tests each line of 'x' if expected order of (replicate-) groups (defined in 'grp') appears intact,
 while inluding SEM of groups (replicates) via a proportional weight 'sdFact' as (avGr1-gr1SEM) < (avGr1+gr1SEM) < (avGr2-gr2SEM) < (avGr2+gr2SEM).  
Used for comparing groups of measures with expected profile (by matching expected order)
to check if data in 'x' represting groups ('grp') as lines follow.  
Groups of size=1: The sd (and SEM) can't be estimated directly without any replicates, however, an estimate can be given by shrinking if 'shrink1sampSd'=TRUE 
under the hypothesis that the overall mechanisms determining the variances is constant across all samples.
}
\examples{
mat1 <- matrix(rep(11:24,3)[1:40],byrow=TRUE,ncol=8)
checkGrpOrderSEM(mat1,grp=gl(3,3)[-1])
}
\seealso{
takes only 10% more time than \code{\link[wrMisc]{checkGrpOrder}} wo considering intra-group sd
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/top3mean.R
\name{top3mean}
\alias{top3mean}
\title{Mean Of 3 Highest Values}
\usage{
top3mean(x, silent = FALSE, debug = FALSE, callFrom = NULL)
}
\arguments{
\item{x}{(numeric vector) main input}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allows easier tracking of messages produced}
}
\value{
This function returns a vector with single numeric value for mean of top3
  ( returns \code{NULL} with invalid input or if all input is \code{NA})
}
\description{
This function returns the mean of the top3 highest values.
}
\details{
Generally, \code{NA} will be excluded, if all values are \code{NA} this finction will return \code{NULL} ;
thus, in case of (entirely) unsuitable data (non-numeric ...)  \code{NULL} will be returned.
If data has subgroups you may try a tapply -way.
}
\examples{
x1 <- c(15:11,NA,16)
top3mean(x1)
}
\seealso{
\code{\link[base]{mean}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/writeCsv.R
\name{writeCsv}
\alias{writeCsv}
\title{Write (And Convert) Csv Files}
\usage{
writeCsv(
  input,
  inPutFi = NULL,
  expTy = c("Eur", "US"),
  imporTy = "Eur",
  filename = NULL,
  quote = FALSE,
  filterCol = NULL,
  replMatr = NULL,
  returnOut = FALSE,
  SYLKprevent = TRUE,
  digits = 22,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{input}{either matrix or data.frame}

\item{inPutFi}{(character or \code{NULL}) file-name to be read (format as US or Euro-type may specified via argument \code{imporTy})}

\item{expTy}{(character) 'US' and/or 'Eur' for sparator and decimal type in output}

\item{imporTy}{(character) default 'Eur' (otherwise set to 'US')}

\item{filename}{(character) optional new file name(s)}

\item{quote}{(logical) will be passed to function \code{write.csv}}

\item{filterCol}{(integer or character) optionally, to export only the columns specified here}

\item{replMatr}{optional, matrix (1st line:search, 2nd li:use for replacing) indicating which characters need to be replaced )}

\item{returnOut}{(logical) return output as object}

\item{SYLKprevent}{(logical) prevent difficulty when opening file via Excel. In some cases Excel presumes (by error) the SYLK format and produces an error when trying to open files : 
To prevent this, if necessary, the 1st column-name will be changed from 'ID' to 'Id'.}

\item{digits}{(interger) limit number of signif digits in output (ie file)}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) for bug-tracking: more/enhanced messages}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function writes a file to disk and returns \code{NULL} unless \code{returnOut=TRUE}
}
\description{
This functions allows for more options when writing data into csv-files compated to  \code{write.csv}.
}
\details{
The main input may be gven as R-object or read from file 'input'. Then, one can (re-)write using specified conversions. 
An optional filter to select columns (column-name specified via 'filterCol') is available.
 
The output may be simultaneaously written to multiple formats, as specified in 'expTy', 
tabulation characters may be converted to avoid accidentally split/shift text to multiple columns. 
Note: Mixing '.' and ',' as comma separators via text-columns or fused text&data may cause problems lateron, though.
}
\examples{
dat1 <- data.frame(ini=letters[1:5],x1=1:5,x2=11:15,t1=c("10,10","20.20","11,11","21,21","33.33"),
  t2=c("10,11","20.21","kl;kl","az,az","ze.ze"))
fiNa <- file.path(tempdir(), paste("test",1:2,".csv",sep=""))
writeCsv(dat1, filename=fiNa[1])
dir(path=tempdir(), pattern="cs")

(writeCsv(dat1, replM=rbind(bad=c(";",","), replBy="__"), expTy=c("Eur"),
  returnOut=TRUE, filename=fiNa[2]))

}
\seealso{
\code{write.csv} in \code{\link[utils]{write.table}}, batch reading using this package \code{\link{readCsvBatch}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pasteC.R
\name{pasteC}
\alias{pasteC}
\title{Advanced paste-collapse}
\usage{
pasteC(..., collapse = ", ", lastCol = " and ", quoteC = "")
}
\arguments{
\item{...}{(character) main input to be collapsed}

\item{collapse}{(character,length=1) element to use for collapsing}

\item{lastCol}{(character) text to use before last item enumerated element}

\item{quoteC}{character to use for citing with quotations (default "")}
}
\value{
This function returns a character vector of truncated versions of intpup \code{txt}
}
\description{
This function is a variant of \code{\link[base]{paste}} for convenient use of paste-collapse and separation of last element to paste (via 'lastCol').
This function was mode for more human like enumeriating in output and messages.
If multiple arguments are given without names they will all be concatenated, if they contain names lazy evaluation for names will be tried
(with preference to longest match to argument names).
Note that some special characters (like backslash) may need to be protetected when used with 'collapse' or 'quoteC'.
Returns character vector of length 1 (everything pasted together)
}
\examples{
pasteC(1:4)
}
\seealso{
\code{\link[base]{paste}} for basic paste
}
\keyword{character}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/XYToDiffPpm.R
\name{XYToDiffPpm}
\alias{XYToDiffPpm}
\title{Express difference as ppm}
\usage{
XYToDiffPpm(x, y, nSign = NULL, silent = FALSE, debug = FALSE, callFrom = NULL)
}
\arguments{
\item{x}{(numeric) typically for measured variable}

\item{y}{(numeric) typically for theoretical/expected value (vector must be of same length as 'x')}

\item{nSign}{(integer) number of significant digits in output}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a numeric vector of (ratio-) ppm values
}
\description{
This function transforms offset (pariwise-difference) between 'x' & 'y' to ppm (as normalized difference ppm, parts per million, ie (x-y)/y ).
This type of expressiong differences is used eg in mass-spectrometry.
}
\examples{
set.seed(2017); aa <- runif(10,50,900)
cbind(x=aa,y=aa+1e-3,ppm=XYToDiffPpm(aa,aa+1e-3,nSign=4)) 
}
\seealso{
\code{\link{ratioToPpm}} for classical ppm
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ratioAllComb.R
\name{.stackArray}
\alias{.stackArray}
\title{Reorganize array by reducing dimension 'byDim'  (similar to stack() for data-frames)}
\usage{
.stackArray(arr, byDim = 3)
}
\arguments{
\item{arr}{(array) main input}

\item{byDim}{(integer)}
}
\value{
This function returns an array/matrix of 1 dimension less than 'arr', 1st dim has more lines (names as paste with '_')
}
\description{
This function aims to reorganize an array by reducing dimension 'byDim'  (similar to stack() for data-frames)
It returns an array/matrix of 1 dimension less than 'arr', 1st dim has more lines (names as paste with '_')
}
\examples{
(arr1 <- array(11:37, dim=c(3,3,3)))
.stackArray(arr1, 3)
}
\seealso{
\code{\link[stats]{dist}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convColorToTransp.R
\name{convColorToTransp}
\alias{convColorToTransp}
\title{Assign new transparency to given colors}
\usage{
convColorToTransp(color, alph = 1)
}
\arguments{
\item{color}{(character) color input}

\item{alph}{(numeric) transparency value (1 for no transparency, 0 for complete opaqueness), values <1 will be treated as percent-values}
}
\value{
character vector (of same length as input) with color encoding for new transparency
}
\description{
This function alows (re-)defining a new transparency. A color encoding vector will be transformed to the same color(s) but with new transparency (alpha).
}
\examples{
col0 <- c("#998FCC","#5AC3BA","#CBD34E","#FF7D73")
col1 <- convColorToTransp(col0,alph=0.7)
layout(1:2)
pie(rep(1,length(col0)),col=col0)
pie(rep(1,length(col1)),col=col1,main="new transparency")
}
\seealso{
\code{\link[grDevices]{rgb}}, \code{\link[graphics]{par}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rowGrpMeans.R
\name{rowGrpMeans}
\alias{rowGrpMeans}
\title{Row-Means With Destinction Of Groups (Of Columns, eg Groups Of Replicates)}
\usage{
rowGrpMeans(
  x,
  grp,
  na.rm = TRUE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{x}{(numeric matrix or data.frame) main input}

\item{grp}{(character or factor) defining which columns should be grouped (considered as replicates)}

\item{na.rm}{(logical) a logical value indicating whether \code{NA}-values should be stripped before the computation proceeds.}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) This function allows easier tracking of messages produced}
}
\value{
This function returns a matrix with mean values
}
\description{
Thus function calculates column-means for matrix with multiple groups of data, ie similar to rowMeans but one mean for each group of data.
Groups are specified as columns of 'x' in 'grp' (so length of grp should match number of columns of 'x', NAs are allowed).
}
\examples{
set.seed(2016); dat1 <- matrix(c(runif(200) +rep(1:10,20)), ncol=10)
head(rowGrpMeans(dat1, gr=gl(4, 3, labels=LETTERS[1:4])[2:11]))
}
\seealso{
\code{\link{rowSds}}, \code{\link[base]{colSums}} which also shows \code{colMeans}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkStrictOrder.R
\name{.firstMin}
\alias{.firstMin}
\title{Get first minimum}
\usage{
.firstMin(x, positionOnly = FALSE)
}
\arguments{
\item{x}{(numeric vector) main input}

\item{positionOnly}{(logical)}
}
\value{
numeric vector
}
\description{
This function allows to find the first minimum of a numeric vector
}
\examples{
.firstMin(c(4,3:6))
}
\seealso{
\code{\link[base]{which.min}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/correctToUnique.R
\name{.uniqueWName}
\alias{.uniqueWName}
\title{Check regression arguments}
\usage{
.uniqueWName(
  x,
  splitSameName = TRUE,
  silent = TRUE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{x}{(numeric or character vector) main input}

\item{splitSameName}{(logical)}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
vector like input
}
\description{
This function is an enhanced version of \code{unique}, names of elements are maintained
}
\examples{
aa <- c(a=11, b=12,a=11,d=14, c=11)
.uniqueWName(aa)
.uniqueWName(aa[-1]) # value repeated but different name
}
\seealso{
\code{\link[base]{unique}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/keepCommonText.R
\name{keepCommonText}
\alias{keepCommonText}
\title{Extract Longest Common Text Out Of Character Vector}
\usage{
keepCommonText(
  txt,
  minNchar = 1,
  side = "center",
  hiResol = TRUE,
  silent = TRUE,
  callFrom = NULL,
  debug = FALSE
)
}
\arguments{
\item{txt}{character vector to be treated}

\item{minNchar}{(integer) minumin number of characters that must remain}

\item{side}{(character) may be be either 'center', 'any', 'terminal', 'left' or 'right'; only with \code{side='center'} or \code{'any'} internal text-segments may be found}

\item{hiResol}{(logical) find best solution, but at much higher comptational cost (eg 3x slower, however \code{hiResol=FALSE} rather finds anchor which may need to get extended)}

\item{silent}{(logical) suppress messages}

\item{callFrom}{(character) allow easier tracking of messages produced}

\item{debug}{(logical) display additional messages for debugging}
}
\value{
This function returns a character vector of length=1, ie only one (normally the longest) common sequence of characters is identified.
 If nothing is found common/shared an empty character-vector is returned
}
\description{
This function allows recovering the single longest common text-fragments (from center, head or tail) out of character vector \code{txt}.
Only the first of all of the longest solutions will be returned.
}
\details{
Please note, that finding common parts between chains of characters is not a completely trivial task. 
This topic still has ongoing research for the application of sequence-alignments, where chains of characters to be compared get very long.
This function uses a k-mer inspirated approach.
The initial aim with this function was allowing to treat smaller chains of characters (and finding shorter strteches of common text), like eg with column-names.

Important : This function identifies only the first best hit, ie other shared/common character-chains of the same length will not be found !

Using the argument \code{hiResol=FALSE} it is possible to accelerate the search aprox 3x (with larger character-vectors), however, frequently the very best solution may not be found. 
This means, that in this case the result should rather be considered a 'seed', allowing check if further extension may improve the result, 
ie for identifying a (slightly) longer chain of common characters.

With longer vectors and longer character chains this may get demanding on computational reesources, the argument \code{hiResol=FALSE} allows reducing this at the price of missing the best solution.
With this argument single common/matching characters will not be searched if all text-elements are longer than 500 characters, an empty character vector will be returned. 

When argument \code{side} is either \code{left}, \code{right} or \code{terminal} only terminal common text may be found (a potentially even longer internal text will be lost).
Of course, choosing this option makes searches much faster.  

This function does not return the position of the shared/common characters within the text, you may use \code{gregexpr} or \code{regexec} to locate them.
}
\examples{
txt1 <- c("abcd_abc_kjh", "bcd_abc123", "cd_abc_po")
keepCommonText(txt1, side="center")       # trim from right

txt2 <- c("ddd_ab","ddd_bcd","ddd_cde")
trimRedundText(txt2, side="left")          #  
keepCommonText(txt2, side="center")        # 
}
\seealso{
Use \code{gregexpr} or \code{regexec} in \code{\link[base]{grep}} for locating the identified common characters in the initial query.

Inverse : Trim redundant text (from either side) to keep only varaible part using \code{\link{trimRedundText}}; 
 you may also look for related functions in package \href{https://CRAN.R-project.org/package=stringr}{stringr}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readCsvBatch.R
\name{.inspectHeader}
\alias{.inspectHeader}
\title{Inspect 'matr' and check if 1st line can be used/converted as header}
\usage{
.inspectHeader(
  matr,
  headNames = c("Plate", "Well", "StainA"),
  numericCheck = TRUE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{matr}{(matrix or data.frame) main input to be instected}

\item{headNames}{(character) column-names t look for}

\item{numericCheck}{(logical) allows reducing complexity by drawing for very long x or y}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a matrix vector or data.frame similar to input
}
\description{
This function inspects 'matr' and check if 1st line can be used/converted as header.
If colnames of 'matr' are either NULL or 'V1',etc the 1st row will be tested if it contains any of the elements (if not, 1st line won't be used as new colnames)
If 'numericCheck'=TRUE, all columns will be tested if they can be converted to numeric
}
\examples{
ma1 <- matrix(letters[1:6], ncol=3, dimnames=list(NULL,c("ab","Plate","Well")))
.inspectHeader(ma1) 

}
\seealso{
\code{\link[utils]{head}} for looking at first few lines
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/colorAccording2.R
\name{colorAccording2}
\alias{colorAccording2}
\title{Transform Numeric Values To Color-Gradient}
\usage{
colorAccording2(
  x,
  gradTy = "rainbow",
  nStartOmit = NULL,
  nEndOmit = NULL,
  revCol = FALSE,
  alpha = 1,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{x}{(character) color input}

\item{gradTy}{(character) type of gradeint may be 'rainbow', 'heat.colors', 'terrain.colors', 'topo.colors', 'cm.colors', 'hcl.colors', 'grey.colors', 'gray.colorsW' or 'logGray'}

\item{nStartOmit}{(integer) omit n steps from begining of gradient range}

\item{nEndOmit}{(integer or "sep") omit n steps from end of gradient range, if \code{nEndOmit="sep"} 20 percent of initial grades will be removed to obtain 'separate' ie non-closing color-circles/gradients eg with \code{rainbow}}

\item{revCol}{(logical) reverse order}

\item{alpha}{(numeric) optional transparency value (1 for no transparency, 0 for complete opaqueness)}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a character vector (of same length as x) with color encoding
}
\description{
This function helps making color-gradients for plotting a numerical variable.
Note : RColorBrewer palettes were not integrated here/yet.
}
\examples{
set.seed(2015); dat1 <- round(runif(15),2)
plot(1:15,dat1,pch=16,cex=2,col=colorAccording2(dat1))
plot(1:15,dat1,pch=16,cex=2,col=colorAccording2(dat1,nStartO=0,nEndO=4,revCol=TRUE))
plot(1:9,pch=3)
points(1:9,1:9,col=transpGraySca(st=0,en=0.8,nSt=9,trans=0.3),cex=42,pch=16)
}
\seealso{
\code{\link[base]{cut}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cutArrayInCluLike.R
\name{cutArrayInCluLike}
\alias{cutArrayInCluLike}
\title{Cut 3-dim array in list of matrixes (or arrays) similar to organizing into clusters}
\usage{
cutArrayInCluLike(dat, cluOrg, silent = FALSE, debug = FALSE, callFrom = NULL)
}
\arguments{
\item{dat}{array (3 dim)}

\item{cluOrg}{(factor) organization of lines to clusters}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) display additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of message(s) produced}
}
\value{
This function retruns a list of matrixes (or arrays)
}
\description{
\code{cutArrayInCluLike} cuts 'dat' (matrix,data.frame or 3-dim array) in list (of appended lines) according to 'cluOrg', 
 which serves as instruction which line of 'dat' should be placed in which list-element (like sorting according to cluster-numbers).
}
\examples{
mat1 <- matrix(1:30,nc=3,dimnames=list(letters[1:10],1:3))
cutArrayInCluLike(mat1,cluOrg=factor(c(2,rep(1:4,2),5)))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rowNormalize.R
\name{.rowNorm}
\alias{.rowNorm}
\title{Row-normalization Procedure On Matrix Or Data.frame (main)}
\usage{
.rowNorm(
  dat,
  refLi,
  method,
  proportMode,
  maxFact = 10,
  fact0val = 10,
  retFact = FALSE,
  debug = FALSE,
  silent = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{dat}{(matrix) .. init data, smay be parse matrix with numerous NA}

\item{refLi}{(NULL or numeric) allows to consider only specific lines of 'dat' when determining normalization factors (all data will be normalized)}

\item{method}{(character) may be "mean","median" (plus "NULL","none"); When NULL or 'none' is chosen the input will be returned as is}

\item{proportMode}{(logical) decide if normalization should be done by multiplicative or additive factor}

\item{maxFact}{(numeric, length=2) max normalization factor}

\item{fact0val}{(integer)}

\item{retFact}{(logical)}

\item{debug}{(logical) additional messages for debugging}

\item{silent}{(logical) suppress messages}

\item{callFrom}{(character) This function allows easier tracking of messages produced}
}
\value{
This function returns a matrix of normalized data same dimensions as 'dat'
}
\description{
This function performs a row-normalization procedure on matrix or data.frame 'dat'
}
\examples{
.rowNorm(matrix(11:31, ncol=3), refLi=1, method="mean", proportMode=TRUE)
}
\seealso{
\code{\link{rowNormalize}}; \code{\link{normalizeThis}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getPWseparator.R
\name{.getPWseparator}
\alias{.getPWseparator}
\title{Find Separator In Vector Of Pairwise Group-Names  
  
This function allows identifing separator used when pairwise groups are presented.}
\usage{
.getPWseparator(
  compNames,
  potSep = "split1",
  includeGrp = FALSE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{compNames}{(character or integer) names of pairwise combined group-names}

\item{potSep}{(character) potential separators to be checked if occuring in \code{useComp}; the first fitting will used}

\item{includeGrp}{(logical) if \code{includeGrp=TRUE} a list including also the group-names is returned}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a character vector (length=1) of the (first) separator fitting to all data, or a list with $sep and $grpNa if \code{includeGrp=TRUE}
}
\description{
Find Separator In Vector Of Pairwise Group-Names  
  
This function allows identifing separator used when pairwise groups are presented.
}
\details{
Note : Potential separators \code{potSep} to be tested must be at least 1 character long
  
The character '.' may used as \code{potSep}, it will be protected internally for correct results.
Note : The indexing is relative to the levels of grp, thus to sorted group-names !
}
\examples{
.getPWseparator(compNames=c("B-C","B-D","C-D")) 
}
\seealso{
\code{\link{indexGroupsFromPW}}, \code{\link{getPWseparator}}, colnames of testing results from \code{\link{moderTestXgrp}}, used by \code{\link{replacePWseparator}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkVectLength.R
\name{.plusLowerCaps}
\alias{.plusLowerCaps}
\title{Add lower caps to character vector}
\usage{
.plusLowerCaps(x)
}
\arguments{
\item{x}{(character) main input}
}
\value{
This function returns a elongated character vector
}
\description{
This function allows adding all content as lower caps to/of character vector
}
\examples{
.plusLowerCaps(c("Abc","BCD"))
}
\seealso{
\code{\link[base]{chartr}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matchSampToPairw.R
\name{matchSampToPairw}
\alias{matchSampToPairw}
\title{Match Names To Concatenated Pairs Of Names}
\usage{
matchSampToPairw(
  grpNa,
  pairwNa,
  sep = NULL,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{grpNa}{(character) the names of the groups of replicates (ie conditions) used to test}

\item{pairwNa}{(character) the names of pairwise-testing (ie 'concatenated' \code{sampNa}}

\item{sep}{(character) if not \code{NULL} the characters given will be used via \code{stringsplit}}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a matrix of 2 columns with inidices of \code{sampNa} with \code{pairwNa} as rows
}
\description{
The column-names of multiple pairwise testing contain the names of the initial groups/conditions tested, plus there is a separator (eg '-' in \code{moderTestXgrp}).  
Thus function allows to map back which groups/conditions were used by returning the index of the respective groups used in pair-wise sets.
}
\details{
There are two modes of operation : 1) Argument \code{sep} is set to \code{NULL} :  The names of initial groups/conditions (\code{grpNa}) 
will be tested for exact pattern matching either at beginning or at end of pair-wise names (\code{pairwNa}).
This approach has the advantage that it does not need to be known what character(s) were used as separator (or they may change), 
but the disadvantage that in case the perfect \code{grpNa} was not given, the longest best match of \code{grpNa} will be returned.

2) The separator \code{sep} is given and exact matches at both sides will be searched.
However, if the character(s) from \code{sep} do appear inside \code{grpNa} no matches will be found.

If some \code{grpNa} are not found in \code{pairwNa} this will be marked as NA.  

This function checks also if a double version of \code{sep} may fit better to \code{grpNa}
and will adjust \code{sep} accordingly (eg adjust sep='-' to '--').
}
\examples{
pairwNa1 <- c("abc-efg","abc-hij","efg-hij")
grpNa1 <- c("hij","abc","abcc","efg","klm")
matchSampToPairw(grpNa1, pairwNa1) 

pairwNa2 <- c("abc-efg","abcc-hij","abc-hij","abc-hijj","zz-zz","efg-hij")
matchSampToPairw(grpNa1, pairwNa2) 
}
\seealso{
(for running multiple pair-wise test) \code{\link{moderTestXgrp}}, \code{\link[base]{grep}}, \code{\link[base]{strsplit}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/organizeAsListOfRepl.R
\name{organizeAsListOfRepl}
\alias{organizeAsListOfRepl}
\title{(re)organize data of (3-dim) array as list of replicates}
\usage{
organizeAsListOfRepl(
  arrIn,
  inspNChar = 0,
  byDim = 3,
  silent = TRUE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{arrIn}{(array) main input}

\item{inspNChar}{(interger) if inspNChar=0 the array-names (2nd dim of 'arrIn') will be cut before last '_'}

\item{byDim}{(integer, length=1) dimension number along which data will be split in separate elements (considering the first inspNChar characters)}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a list of arrays (typically 1st and 2nd dim for specific genes/objects, 3rd for different measures associated with)
}
\description{
Organize array of all data ('arrIn', long table) into list of (replicate-)arrays (of similar type/layout) based on dimension number 'byDim' of 'arrIn' (eg 2nd or 3rd dim).
Argument \code{inspNChar} defines the number of characters to consider, so if the beginning of names is the same they will be separated as list of multiple arrays.
Default will search for '_' separator or trim from end if not found in the relevant dimnames
}
\examples{
arr1 <- array(1:24,dim=c(4,3,2),dimnames=list(c(LETTERS[1:4]),
  paste("col",1:3,sep=""), c("ch1","ch2")))
organizeAsListOfRepl(arr1)
}
\seealso{
\code{\link[base]{array}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pVal2lfdr.R
\name{pVal2lfdr}
\alias{pVal2lfdr}
\title{Convert p-values to lfdr}
\usage{
pVal2lfdr(x, silent = TRUE, debug = FALSE, callFrom = NULL)
}
\arguments{
\item{x}{(numeric) vector of p.values}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a (numeric) vector of lfdr values (or \code{NULL} if data insufficient to run the function 'fdrtool')
}
\description{
This function takes a numeric vector of p-values and returns a vector of lfdr-values (local false discovery) using 
the package \href{https://CRAN.R-project.org/package=fdrtool}{fdrtool}.
Multiple testing correction should be performed with caution, short series of p-values typically pose problems for transforming to lfdr. 
The transformation to lfdr values may give warning messages, in this case the resultant lfdr values may be invalid !
}
\examples{
## Note that this example is too small for estimating really meaningful fdr values
## In consequence, a warning will be issued.
set.seed(2017); t8 <- matrix(round(rnorm(160,10,0.4),2), ncol=8,
  dimnames=list(letters[1:20], c("AA1","BB1","CC1","DD1","AA2","BB2","CC2","DD2")))
t8[3:6,1:2] <- t8[3:6,1:2]+3   # augment lines 3:6 (c-f) for AA1&BB1
t8[5:8,5:6] <- t8[5:8,5:6]+3   # augment lines 5:8 (e-h) for AA2&BB2 (c,d,g,h should be found)
head(pVal2lfdr(apply(t8, 1, function(x) t.test(x[1:4], x[5:8])$p.value)))
}
\seealso{
lfdr from \code{\link[fdrtool]{fdrtool}}, other p-adjustments (multiple test correction, eg FDR) in \code{\link[stats]{p.adjust}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pwSeparatorList.R
\name{pwSeparatorList}
\alias{pwSeparatorList}
\title{Suggestions Of Collections Of Classical Separators For Pairwise Combinations}
\usage{
pwSeparatorList(
  type = "split1",
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{type}{(character) allows selecting of different types of collections of separators}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a charcter vector with proposed separators for later choosing the most suited
}
\description{
This function provides collecions of typical separators used for combined names in pair-wise testing for later choice of the most suited.
}
\details{
Depending if one rather looks for separators for cutting existing names of for suggesting separators for fresh concatenation, two differentsets of collections are available via the argument \code{type}.

Using \code{type="split"} will default to \code{type="split1"}, similarly \code{type="combine"} will be used as \code{type="combine1"}.
  
The option \code{type="split1"} has a default collection of shorter/compact separators for splitting
The option \code{type="split2"} allows more extensive search for 38 separators (ie combinations with ' ','-' and '_'), but extensive checking may cost CPU time.
The option \code{type="split3"} has a shorter collection of 12 basic commonly used separators.
The option \code{type="split4"} to  \code{type="split9"} default to 'split1'

The option \code{type="combine1"} does not suggest '--' (neither '__') since if '-' may not be chosen  ins the first place, using '--' as separator next to '-' occuring inside gourp-names may not enhance visibility.
The option \code{type="combine2"} contains a collection of sperators using 'vs' or 'versus'
The option \code{type="combine3"} is designed for explicite terms (starts with ' - versus - ', for suggesting compact terms rather use \code{type="combine1"}
The option \code{type="combine4"} to  \code{type="combine9"} default to 'combine1'
}
\examples{
pwSeparatorList(type="split1")
}
\seealso{
(for running multiple pair-wise test) \code{\link{moderTestXgrp}}, \code{\link{convPairwiseSetup}}, \code{\link{presenceFilt}}, \code{\link{getPWseparator}}, \code{\link{indexGroupsFromPW}}, 
  (used underneith:) \code{\link[base]{grep}}, \code{\link[base]{strsplit}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/colMedSds.R
\name{colMedSds}
\alias{colMedSds}
\title{Standard Error Of Median For Each Column By Bootstrap}
\usage{
colMedSds(dat, nBoot = 99, silent = FALSE, debug = FALSE, callFrom = NULL)
}
\arguments{
\item{dat}{(numeric) matix}

\item{nBoot}{(integer, length=1) number if iterations}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a (numeric) vector with estimated standard errors
}
\description{
Determine standard error (sd) of median by bootstraping for multiple sets of data (rows in input matrix 'dat').
Note: The package \href{https://CRAN.R-project.org/package=boot}{boot} must be installed from CRAN.
}
\examples{
set.seed(2016); dat1 <- matrix(c(runif(200) +rep(1:10,20)), ncol=10)
colMedSds(dat1) 
}
\seealso{
\code{\link[boot]{boot}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/presenceFilt.R
\name{presenceFilt}
\alias{presenceFilt}
\title{Filter Lines Of Matrix For Max Number Of NAs}
\usage{
presenceFilt(
  dat,
  grp,
  useComparison = NULL,
  maxGrpMiss = 1,
  ratMaxNA = 0.8,
  minVal = NULL,
  sep = NULL,
  experSetup = NULL,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{dat}{matrix or data.frame (abundance or expression-values which may contain some \code{NA}s).}

\item{grp}{factor of min 2 levels describing which column of 'dat' belongs to which group (levels 1 & 2 will be used)}

\item{useComparison}{(character or matrix) optional argument allowing to specify which pairwise comparions sould be performed, default \code{useComparison=NULL} will run all pairwise comparisons;
may be character combining two group-names (from argument \code{grp}) separated by a '-' (eg 'A-B') or matrix where the rownames design the elements to be compared as pairwise;
Note : the names of the groups may not contain any '-' to avoid confucing them with pairwise separators !}

\item{maxGrpMiss}{(numeric) at least 1 group has not more than this number of NAs (otherwise marke line as bad)}

\item{ratMaxNA}{(numeric) at least 1 group below this content of \code{NA} values}

\item{minVal}{(default NULL or numeric), any value below will be treated like \code{NA}}

\item{sep}{(character) in case \code{useComparison} is not given all pairwise comparisons will be done, the separator to be used when combining names of groups can be given using this argument}

\item{experSetup}{(list) optional experimental setup}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a logical matrix (with separate col for each pairwise combination of 'grp' levels) indicating if line of 'dat' acceptable based on \code{NA}s (and values minVal)
}
\description{
This function produces a logical matrix to be used as filter for lines of 'dat' for sufficient presence of non-\code{NA} values (ie limit number of NAs per line). 
Filter abundance/expression data for min number and/or ratio of non-\code{NA} values in at east 1 of multiple groups.
This type of procedure is common in proteomics and tanscriptomics, where a \code{NA} can many times be assocoaued with quantitation below detetction limit.
}
\examples{
mat <- matrix(rep(8,150), ncol=15, dimnames=list(NULL,
  paste0(rep(LETTERS[4:2],each=6),1:6)[c(1:5,7:16)]))
mat[lower.tri(mat)] <- NA
mat[,15] <- NA
mat[c(2:3,9),14:15] <- NA
mat[c(1,10),13:15] <- NA
mat
# default : at least 1 group with max 1 NA and one group below 80\% NA
presenceFilt(mat, substr(colnames(mat),1,1))
# custom 2 groups
presenceFilt(mat, rep(1:2,c(9,6)))         # D1- C4, C5 - B4

# one more example 
dat1 <- matrix(1:56, ncol=7)
dat1[c(2:6,10,12,18,19,20,22,23,26:28,30,31,34,38,39,50,54)] <- NA
grp3 <- letters[c(3,3,2,2,1,1,1)]
colnames(dat1) <- correctToUnique(grp3, sep="") 
dat1
## At least one group wo any NAs
presenceFilt(dat1, grp3, maxGr=0)
presenceFilt(dat1, gr=gl(2,4)[-1], maxGr=1, ratM=0.1)
presenceFilt(dat1, gr=gl(2,4)[-1], maxGr=2, rat=0.5)
}
\seealso{
\code{\link{presenceGrpFilt}},   there are also other packages on CRAN and Bioconductor dedicated to filtering
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/combineRedBasedOnCol.R
\name{combineRedBasedOnCol}
\alias{combineRedBasedOnCol}
\title{Combine/reduce redundant lines based on specified column}
\usage{
combineRedBasedOnCol(mat, colNa, sep = ",", silent = FALSE, callFrom = NULL)
}
\arguments{
\item{mat}{input matrix or data.frame}

\item{colNa}{character vector (length 1) macting 1 column name (if mult only 1st will be used), in case of mult matches only 1st used}

\item{sep}{(character) separator (default=",")}

\item{silent}{(logical) suppress messages}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
matrix containing the input matrix without lines considered repeated (unique-like)
}
\description{
This function works similar to \code{unique}, but it takes a matrix as input and considers one specified column to find unique instances.
It identifies 'repeated' lines of the input-matrix (or data.frame) 'mat' based on (repeated) elements in/of column with name 'colNa' (or column-number).
Redundant lines (ie repeated lines) will disappear in output.
Eg used with extracted annotation where 1 gene has many lines for different GO annotation.
}
\examples{
matr <- matrix(c(letters[1:6],"h","h","f","e",LETTERS[1:5]),ncol=3,
  dimnames=list(letters[11:15],c("xA","xB","xC")))
combineRedBasedOnCol(matr,colN="xB")
combineRedBasedOnCol(rbind(matr[1,],matr),colN="xB")
}
\seealso{
\code{\link{findRepeated}}, \code{\link{firstOfRepLines}}, \code{\link{organizeAsListOfRepl}}, \code{\link{combineRedundLinesInList}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nNonNumChar.R
\name{nNonNumChar}
\alias{nNonNumChar}
\title{Count number of non-numeric characters}
\usage{
nNonNumChar(txt)
}
\arguments{
\item{txt}{character vector to be treated}
}
\value{
This function returns a numeric vector with  numer of non-numeric characters (ie not '.' or 0-9))
}
\description{
\code{nNonNumChar} counts number of non-numeric characters.
Made for positive non-scientific values (eg won't count neg-sign, neither Euro comma ',')
}
\examples{
nNonNumChar("a1b "); sapply(c("aa","12ab","a1b2","12","0.5"), nNonNumChar)
}
\seealso{
\code{\link[base]{nchar}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rmSharedWords.R
\name{rmSharedWords}
\alias{rmSharedWords}
\title{Trim/Remove Redundant Words}
\usage{
rmSharedWords(
  x,
  sep = c("_", " ", "."),
  anySep = TRUE,
  newSep = NULL,
  minLe = 2,
  na.omit = FALSE,
  fixed = TRUE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{x}{(character) main input for making non-redundant}

\item{sep}{(character) separator(s) to be used}

\item{anySep}{(logical) if \code{TRUE}, will consider all separators at one time (), thus combinations with different separators won't be distinguished}

\item{newSep}{(character) new (uniform) separator between words, if \code{NULL} the first value/separator of if \code{sep} will be used}

\item{minLe}{(integer) minimum length for allowing being recognised as 'word'}

\item{na.omit}{(logical) if \code{TRUE NA}s will be removed from output}

\item{fixed}{(logical) will be transmitted to argument \code{fixed} of \code{strsplit()};  if \code{TRUE} regular expressions are allowed/used}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allows easier tracking of messages produced}
}
\value{
This function returns character vector of same length (unless \code{na.omit=TRUE}), simply with modified text-content
}
\description{
This function allows removing shared words, ie triming to non-redundant words.
}
\details{
Heading separators will be removed in any case (even if not followed by a 'word').

Special characters will be automatically protected.
When looking for repeated words, the order of such words does NOT matter, multiple repeats will be removed, too.
 
#'
}
\examples{
x1 <- c("aa_A1 yy_zz.txt", NA, "B2 yy_aa_aa_zz.txt")
rmSharedWords(x1)
 
}
\seealso{
\code{\link{trimRedundText}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cutArrayInCluLike.R
\name{.combineListAnnot}
\alias{.combineListAnnot}
\title{Combine annotation information from list of matrixes}
\usage{
.combineListAnnot(
  lst,
  useCol = 1:2,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{lst}{(list) main input}

\item{useCol}{(numeric vector) which columns should be used}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a single matrix of combined (non-redundant) info
}
\description{
This function allows to combine information (annotation) from list of matrixes (ie replace when NA), using always the columns specified in 'useCol' (numeric)
}
\examples{
.datSlope(c(3:6))
}
\seealso{
used in \code{\link{cutArrayInCluLike}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readCsvBatch.R
\name{readCsvBatch}
\alias{readCsvBatch}
\title{Read batch of csv-files}
\usage{
readCsvBatch(
  fileNames = NULL,
  path = ".",
  fileFormat = "Eur",
  checkFormat = TRUE,
  returnArray = TRUE,
  columns = c("Plate", "Well", "StainA"),
  excludeFiles = "All infected plates",
  simpleNames = TRUE,
  minNamesLe = 4,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{fileNames}{(character) names of files to be read, if \code{NULL} all files fitting 'fileFormat'}

\item{path}{(character) where files should be read (folders should be written in R-style)}

\item{fileFormat}{(character) may be \code{NULL} (both US and European formats will be tried), 'Eur' or 'US'}

\item{checkFormat}{(logical) if \code{TRUE}: check header, remove empty columns, 1st line if all empmty, set output format for each file to matrix, if rownames are increasing integeres try to use 2nd of 'columns' as rownames}

\item{returnArray}{(logical) allows switching from array to list-output}

\item{columns}{(NULL or character) column-headers to be extracted (if specified), 2nd value may be comlumn with rownames (if rownames are encountered as increasing rownames)}

\item{excludeFiles}{(character) names of files to exclude (only used when reading all files of given directory)}

\item{simpleNames}{(logical) allows truncating names (from beginning) to get to variable part (using .trimLeft()), but keeping 'minNamesLe'}

\item{minNamesLe}{(interger) min length of column-names if simpleNames=TRUE}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allows easier tracking of messages produced}
}
\value{
This function returns an array (or list if \code{returnArray=FALSE}) of all numeric data read (numerical columns only) from individual files
}
\description{
This function was designed to read screening data split in parts (with common structure) and saved to multiple files,  
to extract the numeric columns and to compile all (numeric) data to a single array (or list).  Some screening platforms save results while progressing 
through a pile of microtiter-plates separately. The organization of the resultant files is structured through file-names and all files have exactely the same organization of lines and columns/ 
European or US-formatted csv files can be read, if argument \code{fileFormat} is \code{NULL} both types will be tested, otherwise it allows to specify a given format.
The presence of headers (to be used as column-names) may be tested using \code{checkFormat}.
}
\examples{
path1 <- system.file("extdata", package="wrMisc")
fiNa <-  c("pl01_1.csv","pl01_2.csv","pl02_1.csv","pl02_2.csv")
datAll <- readCsvBatch(fiNa, path1)
str(datAll)
## batch reading of all csv files in specified path :
datAll2 <- readCsvBatch(fileNames=NULL, path=path1, silent=TRUE)
}
\seealso{
\code{\link[utils]{read.table}}, \code{\link{writeCsv}}, \code{\link{readXlsxBatch}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/addBeforFileExtension.R
\name{.checkFileNameExtensions}
\alias{.checkFileNameExtensions}
\title{Check File Name Extensions 
Function for checking file-names.}
\usage{
.checkFileNameExtensions(fileNa, ext)
}
\arguments{
\item{fileNa}{(character) file name to be checked}

\item{ext}{(character) file extension}
}
\value{
modified character vector
}
\description{
Check File Name Extensions 
Function for checking file-names.
}
\examples{
.checkFileNameExtensions("testFile.txt","txt")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sysDate.R
\name{sysDate}
\alias{sysDate}
\title{System-date (compressed format)}
\usage{
sysDate(style = "univ1")
}
\arguments{
\item{style}{(character) choose style (default 'univ1' for very compact style)}
}
\value{
character vector with formatted date
}
\description{
This function returns current date (based on Sys.Date) in different format options.
}
\details{
Multiple options for formatting exist :
'univ1' or 'wr' ... (default) compact sytle using day, first 3 letters of English name of month (lowercaps) and last 2 letters of year as ddmmmyy, eg 14jun21 

'univ2' ... as ddMmmyy, eg 14Jun21

'univ3' ... as ddMonthyyyy, eg 14June2021

'univ4' ... as ddmonthyyyy, eg 14june2021

'univ5' ... as yyyy-mm-dd (output of \code{Sys.Date()}), eg 2021-06-14

'univ6' ... as yyyy-number of day (in year), eg 2021-165

'local1' ... compact sytle using day, first 3 letters of current locale name of month (not necessarily unique !) and last 2 letters of year as ddmmmyy, eg 14jui21 

'local2' ... as ddMmmyy, month based on current locale (not necessarily unique !), eg 14Jui21

'local3' ... as ddMonthyyyy, month based on current locale , eg 14Juin2021

'local4' ... as ddmonthyyyy, month based on current locale , eg 14juin2021

'local5' ... as dd-month-yyyy, month based on current locale , eg 14-juin-2021

'local6' ... as yyyymonthddd, month based on current locale , eg 2021juin14
}
\examples{
sysDate() 

}
\seealso{
\code{\link[base]{date}}, \code{Sys.Date} and \code{\link[base]{Sys.time}},
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fuseAnnotMatr.R
\name{fuseAnnotMatr}
\alias{fuseAnnotMatr}
\title{Fuse annotation matrix to initial matrix}
\usage{
fuseAnnotMatr(
  iniTab,
  annotTab,
  refIniT = "Uniprot",
  refAnnotT = "combName",
  addCol = c("ensembl_gene_id", "description", "geneName", "combName"),
  debug = TRUE,
  silent = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{iniTab}{(matrix), that may have lines with multiple (=repeated) key entries}

\item{annotTab}{(matrix) containing reference annotation}

\item{refIniT}{(character) type of reference (eg 'Uniprot')}

\item{refAnnotT}{(character) column name to use for reference-annotation}

\item{addCol}{(character) column-namess of 'annotTab' to use/extract (if no matches found, use all)}

\item{debug}{(logical) for bug-tracking: more/enhanced messages}

\item{silent}{(logical) suppress messages}

\item{callFrom}{(character) allow easier tracking of message(s) produced}
}
\value{
This function returns a combined matrix (elements not found in 'annotTab' are displayed as NA)
}
\description{
In a number of instances experimental measurements and additional information (annotation) are provided by separate objects (matrixes) as they may not be generated the same time.
The aim of this function is provide help when matching approprate lines for 2 sets of data (experimental measures in \code{iniTab} and annotation from \code{annotTab}) for fusing.  
\code{fuseAnnotMatr} adds suppelmental columns/annotation to an initial matrix \code{iniTab} : using column 'refIniT' as key (in \code{iniTab}) to compare with key 'refAnnotT' (from 'annotTab'). 
The columns to be added from \code{annotTab} must be chosen explicitely. 
Note: if  non-unique IDs in iniTab : runs slow (but save) due to use of loop for each unique ID.
}
\examples{
tab0 <- matrix(rep(letters[1:25],8),ncol=10)
tab1 <- cbind(Uniprot=paste(tab0[,1],tab0[,2]),col1=paste(tab0[,3],
  tab0[,4],tab0[,5]," ",tab0[,7],tab0[,6]))
tab2 <- cbind(combName=paste(tab0[,1],tab0[,2]),col2=paste(tab0[,8],tab0[,9],tab0[,10]))
fuseAnnotMatr(tab1,tab2[c(20:11,2:5),],refIni="Uniprot",refAnnotT="combName",addCol="col2")
fuseAnnotMatr(tab2[c(20:11,2:5),],tab1,refAnnotT="Uniprot",refIni="combName",addCol="col1")
}
\seealso{
\code{\link[base]{merge}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convMatr2df.R
\name{convMatr2df}
\alias{convMatr2df}
\title{Convert matrix (eg with redundant) row-names to data.frame}
\usage{
convMatr2df(
  mat,
  addIniNa = TRUE,
  duplTxtSep = "_",
  silent = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{mat}{matrix (or data.frame) to be converted}

\item{addIniNa}{(logical) if \code{TRUE} an additional column ('ID') with rownames will be added at beginning}

\item{duplTxtSep}{(character) separator for enumerating replicated names}

\item{silent}{(logical) suppres messages}

\item{callFrom}{(character) allow easier tracking of message(s) produced}
}
\value{
This functions returns a data.frame equivalent to the input matrix, an additional column named 'ID' will be added for initial rownames
}
\description{
This function provides flexible converting of matrix to data.frame. 
For example repeated/redundant rownames are not allowed in data.frame(), thus the corresponding column-names have to be renamed using a counter-suffix.
In case of non-redundant rownames, a new column 'addIniNa' will be introduced at beginning to document the initial (redundant) rownames, 
non-redundant rownames will be created.
Finally, this functions converts the corrected matrix to data.frame and checks/converts columns for transforming character to numeric if possible. 
If the input is a data.frame containing factors, they will be converted to character before potential conversion.
Note: for simpler version (only text to numeric) see from this package \code{.convertMatrToNum} .
}
\examples{
dat1 <- matrix(1:10, ncol=2)
rownames(dat1) <- letters[c(1:3,2,5)]
## as.data.frame(dat1)  ...  would result in an error
convMatr2df(dat1)

df1 <- data.frame(a=as.character((1:3)/2), b=LETTERS[1:3], c=1:3)
str(convMatr2df(df1))

df2 <- df1; df2$b <- as.factor(df2$b)
str(convMatr2df(df2))
}
\seealso{
\code{\link[base]{numeric}}, for simpler version (only text to numeric) see from this package \code{.convertMatrToNum}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trimRedundText.R
\name{trimRedundText}
\alias{trimRedundText}
\title{Trim redundant text}
\usage{
trimRedundText(
  txt,
  minNchar = 1,
  side = "both",
  spaceElim = FALSE,
  silent = TRUE,
  callFrom = NULL,
  debug = FALSE
)
}
\arguments{
\item{txt}{character vector to be treated}

\item{minNchar}{(integer) minumin number of characters that must remain}

\item{side}{(character) may be be either 'both', 'left' or 'right'}

\item{spaceElim}{(logical) optional removal of any heading or tailing white space}

\item{silent}{(logical) suppress messages}

\item{callFrom}{(character) allows easier tracking of messages produced}

\item{debug}{(logical) display additional messages for debugging}
}
\value{
This function returns a modified character vector
}
\description{
This function allows trimming/removing redundant text-fragments (redundant from head or tail) out of character vector 'txt'.
}
\examples{
txt1 <- c("abcd_ccc","bcd_ccc","cde_ccc")
trimRedundText(txt1, side="right")       # trim from right

txt2 <- c("ddd_ab","ddd_bcd","ddd_cde")
trimRedundText(txt2, side="left")        # trim from left 
}
\seealso{
\code{\link{rmSharedWords}}; Inverse search : Find/keep common text \code{\link{keepCommonText}}; \code{\link{checkUnitPrefix}}; 
   you may also look for related functions in package \href{https://CRAN.R-project.org/package=stringr}{stringr}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convToNum.R
\name{convToNum}
\alias{convToNum}
\title{Convert Vector To Numeric}
\usage{
convToNum(
  x,
  autoConv = TRUE,
  spaceRemove = TRUE,
  convert = c(NA, "sparseChar"),
  remove = NULL,
  euroStyle = TRUE,
  sciIncl = TRUE,
  callFrom = NULL,
  silent = TRUE,
  debug = FALSE
)
}
\arguments{
\item{x}{vector to be converted}

\item{autoConv}{(logical) simple automatic conversion based on function \code{as.numeric}; if \code{TRUE} all other arguments exept  \code{spaceRemove} will not be considered}

\item{spaceRemove}{(logical) to remove all heading and trailing (white) space (until first non-space character)}

\item{convert}{(character) define which type of non-conform entries to convert to NAs. Note, if \code{remove} is selected to eliminate character-entries they cannot be converted any more. Use 'allChar' for all character-entries; 'sparseChar' sparse (ie rare) character entries; \code{NA} for converting 'Na' or 'na' to \code{NA}; if 'none' or \code{NULL} no conversions at all.}

\item{remove}{(character) define which type of non-conform entries to remove, removed items cannot be converted to \code{NA} any more. Use 'allChar' for removing all character entries; \code{NA} for removing all instances of \code{NA} (execept thise created by converting text); all elements will be kept if 'none' or \code{NULL}.}

\item{euroStyle}{(logical) if \code{TRUE} will convert all ',' (eg used as European decimal-separator) to '.' (as internally used by R as decimal-separator), thus allowing converting the European decimal format.}

\item{sciIncl}{(logical) include recognizing scientific notation (eg 2e-4)}

\item{callFrom}{(character) allow easier tracking of messages produced}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}
}
\value{
This function returns a numeric vector (or matrix (if 'x' is matrix))
}
\description{
This function checks if input vector/character string contains numbers (with or without comma) and attempts converting to numeric.
This functions was designed for extracting the numeric part of character-vectors (or matrix) containing both numbers and character-elements.
Depending on the parameters \code{convert} and \code{remove} text-entries can be converted to NA (in resulting numeric objects) or removed (the number of elements/lines gets reduced, in consequece). 
Note: if 'x' is a matrix, its matrix-dimensions & -names will be preserved.
Note: so far Inf  and -Inf do not get recognized as numeric.
}
\details{
This function may be used in two modes, depening if argument \code{autoConv} is \code{TRUE} or \code{FALSE}.
The first options allows accessing an automatic mode based on \code{as.numeric}, 
while the second options investigates all characters if they may belong to numeric expressions and allows removing specific text-elements.
}
\examples{
x1 <- c("+4"," + 5","6","bb","Na","-7") 
convToNum(x1) 
convToNum(x1, autoConv=FALSE, convert=c("allChar"))
convToNum(x1, autoConv=FALSE)      # too many non-numeric instances for 'sparseChar'

x2 <- c("+4"," + 5","6","-7"," - 8","1e6","+ 2.3e4","-3E4","- 4E5") 
convToNum(x2) 
convToNum(x2, autoConv=FALSE, convert=NA,remove=c("allChar",NA))
convToNum(x2, autoConv=FALSE, convert=NA,remove=c("allChar",NA),sciIncl=FALSE)
}
\seealso{
\code{\link[base]{numeric}} and \code{as.numeric} (on same help-page)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pairsAsPropensMatr.R
\name{pairsAsPropensMatr}
\alias{pairsAsPropensMatr}
\title{Convert Pairs of Node-Names to Non-Oriented Propensity Matrix}
\usage{
pairsAsPropensMatr(mat, silent = FALSE, debug = FALSE, callFrom = NULL)
}
\arguments{
\item{mat}{(matrix) main input, matrix of interaction partners with each line as a separate pair of nodes; 
the first two columns should contain identifiers of the nodes}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns matrix or data.frame
}
\description{
Numerous network query tools produce a listing of pairs of nodes (with one pair of nodes per line). 
Using this function such a \code{matrix} (or \code{data.frame}) can be combined to this more comprehensive view as propensity-matrix.
}
\details{
Note, this has been primarily developed for undirected interaction networks, the resulting propensity-matrix does not show any orientation any more.   
In a number of applications (eg in protein-protein interaction networks, PPI) the resulting matrix may be rather sparse.
}
\examples{
pairs3L <- matrix(LETTERS[c(1,3,3, 2,2,1)], ncol=2)    # loop of 3
(netw13pr <- pairsAsPropensMatr(pairs3L))              # as prop matr

}
\seealso{
uses typically input from \code{\link{filterNetw}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/normalizeThis.R
\name{.normConstSlope}
\alias{.normConstSlope}
\title{Normalize Columns Of 2-dim Matrix To Common Linear Regression Fit}
\usage{
.normConstSlope(
  mat,
  useQuant = c(0.2, 0.8),
  refLines = NULL,
  diagPlot = TRUE,
  plotLog = "",
  datName = NULL,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{mat}{matrix or data.frame of data to get normalized}

\item{useQuant}{(numeric) quantiles to use}

\item{refLines}{(NULL or numeric) allows to consider only specific lines of 'dat' when determining normalization factors (all data will be normalized)}

\item{diagPlot}{(logical) draw diagnistic plot}

\item{plotLog}{(character) indicate which axis shousl be diplayed on log-scale, may be 'x', 'xy' or 'y'}

\item{datName}{(character) use as title in diag plot}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allows easier tracking of messages produced}
}
\value{
This function returns a numeric vector
}
\description{
This function aims to normalize columns of 2dim matrix to common linear regression fit within range of 'useQuant'
}
\examples{
aa <- matrix(1:12, ncol=3)
}
\seealso{
\code{\link{normalizeThis}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/regrMultBy1or2point.R
\name{regrMultBy1or2point}
\alias{regrMultBy1or2point}
\title{Rescaling Of Multiple Data-Sets According To Reference Data Using Regression}
\usage{
regrMultBy1or2point(
  inDat,
  refLst,
  regrTo = c(1, 0.5),
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{inDat}{(matrix or data.frame) data to be rescaled}

\item{refLst}{list of names existing in inDat (one group of names for each value in 'regrTo'), to be transformed in values precised in 'regTo'; if no matches to names of 'inDat' found, the 2 lowest and/or highest highest values will be chosen}

\item{regrTo}{(numeric,length=2) range (at scale 0-1) of target-values for mean of elements cited in 'refLst'}

\item{silent}{(logical) suppress messages if \code{TRUE}}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allows easier tracking of messages produced}
}
\value{
This function returns a normalized matrix
}
\description{
This function regresses each col of matrix according to 'refLst'(describing rownames of inDat). 
If 'refLst' conatins 2 groups, the 1st group will be set to the 1st value of 'regrTo' (and the 2nd group of 'refLst' to the 2nd 'regtTo')
}
\examples{
set.seed(2016); dat2 <- round(cbind(1:50 +(1:50)*runif(50),2.2*(1:50) +rnorm(50,0,3)),1)
rownames(dat2) <- 1:nrow(dat2)
reg1 <- regrBy1or2point(dat2[,1],refLst=list(as.character(5:7),as.character(44:45)))
reg2 <- regrMultBy1or2point(dat2,refLst=list(as.character(5:7),as.character(44:45)))
plot(dat2[,1],reg2[,1])
identical(reg1,reg2[,1])
identical(dat2[,1],reg2[,1])
}
\seealso{
\code{\link{adjBy2ptReg}}, \code{\link{regrBy1or2point}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/combineRedundLinesInList.R
\name{combineRedundLinesInList}
\alias{combineRedundLinesInList}
\title{Combine Redundant Lines In List}
\usage{
combineRedundLinesInList(
  lst,
  refNa = "ref",
  datNa = "quant",
  refColNa = "GeneName",
  supRefColNa = NULL,
  summarizeType = "av",
  NA.rm = TRUE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{lst}{(list) main input, containing matrix or data.frame of numeric data (see \code{datNa} and annotation (see \code{refNa}) and possibly unrelated stuff}

\item{refNa}{(character) name of list-element containing annotation}

\item{datNa}{(character) name(s) of list-element(s) containing numeric/quantitation data}

\item{refColNa}{(character) in case the list-element to be used as reference is \code{matrix} or \code{data.frame}, the column to be used must be specified here}

\item{supRefColNa}{(character) in case the \code{lst$refNa} has no rownames, the content of column \code{lst$supRefColNa} will be used instead}

\item{summarizeType}{(character) the summarization method gets specified here; so far 'sum','av','med','first' and 'last' are implemented}

\item{NA.rm}{(logical) pass to summarizing functions order to omit \code{NA}s, defaults to \code{TRUE}}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a list of same length as input
}
\description{
This function provides help for combining/summarizing lines of numeric data which may be summaried according to reference vector or matrix of annotation (part of the same input-list).
The data and reference will be aligned and data corresponding to redundant information be combined/summarized.
}
\details{
All input data should be in a list, ie one or multipl matrix or data.frame for numeric data (see argument \code{datNa}), as well as the reference (see argument \code{refNa}). 
The refgerence may be a named character vecor or a matrix for which the column to be used should be specified using the argument \code{refColNa}.
In case the annotation is a matrix, the rownames will be used as unique/independent identifyers to adjust potentially different order of numeric data and annotation.
In absence of rownames, an additional column  \code{supRefColNa} of the annotation may be designed for adjusting the order of annotation and numeric data.
 
The numeric list may contain multiple matrixes or data.frames which will all be summarized by the same procedure as long as they have the same initial dimensions and are specified by \code{refNa}. 

Please note that all other list elements from input not specified by \code{refNa} (or \code{datNa}) will be maintained in the output just as they are.
}
\examples{
x1 <- list(quant=matrix(11:34, ncol=3, dimnames=list(letters[8:1], LETTERS[11:13])), 
  annot=matrix(paste0(LETTERS[c(1:4,6,3:5)],LETTERS[c(1:4,6,3:5)]), ncol=1, 
  dimnames=list(paste(letters[1:8]),"xx")) )
combineRedundLinesInList(lst=x1, refNa="annot", datNa="quant", refColNa="xx")
}
\seealso{
\code{\link{findRepeated}}, \code{\link{firstOfRepLines}}, \code{\link{organizeAsListOfRepl}}, \code{\link{combineRedBasedOnCol}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ratioAllComb.R
\name{.corDuplItemsByIncrem}
\alias{.corDuplItemsByIncrem}
\title{Avoid duplicating items between 'curNa' and 'newNa' by incrementing digits after 'extPref' (in newNa)}
\usage{
.corDuplItemsByIncrem(newNa, curNa, extPref = "_s")
}
\arguments{
\item{newNa}{(character) main input 1}

\item{curNa}{(character) main input 2}

\item{extPref}{(character) extension}
}
\value{
This function returns the corrected input vector \code{newNa}
}
\description{
This function aims to avoid duplicating items between 'curNa' and 'newNa' by incrementing digits after 'extPref' (in newNa)
}
\examples{
.corDuplItemsByIncrem(letters[1:6], letters[8:4])
}
\seealso{
\code{\link[base]{duplicated}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rowGrpCV.R
\name{.rowGrpCV}
\alias{.rowGrpCV}
\title{row group CV (main)}
\usage{
.rowGrpCV(x, grp, means)
}
\arguments{
\item{x}{numeric matrix where relplicates are organized into separate columns}

\item{grp}{(factor) defining which columns should be grouped (considered as replicates)}

\item{means}{(numeric) alternative values instead of means by .rowGrpMeans()}
}
\value{
This function returns a matrix of CV values
}
\description{
This function calculates CVs for matrix with multiple groups of data, ie one CV for each group of data.
}
\examples{
set.seed(2016); dat1 <- matrix(c(runif(200)+rep(1:10,20)),ncol=10)
grp1 <- gl(4,3,labels=LETTERS[1:4])[2:11]
head(.rowGrpCV(dat1, grp1,  .rowGrpMeans(dat1, grp1)))
}
\seealso{
\code{\link{rowGrpCV}}, \code{\link{rowCVs}}, \code{\link{arrayCV}}, \code{\link{replPlateCV}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/correctWinPath.R
\name{correctWinPath}
\alias{correctWinPath}
\title{Correct Mixed Slash And Backslash In File-Path}
\usage{
correctWinPath(
  x,
  asHtml = FALSE,
  anyPlatf = FALSE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{x}{(character) input path to test and correct}

\item{asHtml}{(logical) option for use in html : add prefix "file:/"}

\item{anyPlatf}{(logical, length=1) if \code{TRUE}, checking will only be performed in Windows environement}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allows easier tracking of messages produced}
}
\value{
This function returns a character vector with corrected path
}
\description{
This function corrects paths character strings for mixed slash and backslash in file path.
In Windows the function \code{tempdir()} will use double backslashes as separator while \code{file.path()} uses regular slashes.
So when combining these two one might encounter a mix of slashes and double backslashes which may cause trouble, unless this is streightened out to a single separator used.
When pointig to given files inside html-files, paths need to have a prefix, this can be added using the argument \code{asHtml}.
}
\examples{
path1 <- 'D:\\\\temp\\\\Rtmp6X8/working_dir\\\\RtmpKC/example.txt'
(path1b <- correctWinPath(path1, anyPlatf=TRUE)) 
(path1h <- correctWinPath(path1, anyPlatf=TRUE, asHtml=TRUE)) 
}
\seealso{
\code{\link[base]{tempfile}}, \code{\link[base]{file.path}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/searchLinesAtGivenSlope.R
\name{.insp1dimByClustering}
\alias{.insp1dimByClustering}
\title{Segment (1-dim vector) 'dat' into clusters}
\usage{
.insp1dimByClustering(
  dat,
  automClu = TRUE,
  cluChar = TRUE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{dat}{matrix or data.frame, main input}

\item{automClu}{(logical) run atomatic clustering}

\item{cluChar}{(logical) to display cluster characteristics}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns clustering (class index) or (if 'cluChar'=TRUE) list with clustering and cluster-characteristics
}
\description{
This function allows aegmenting (1-dim vector) 'dat' into clusters.
If 'automClu=TRUE ..' first try automatic clustering, if too few clusters, run km with length(dat)^0.3 clusters
This function requires the package NbClust to be installed.
}
\examples{
set.seed(2016); dat1 <- matrix(c(runif(200)+rep(1:10,20)),ncol=10)
}
\seealso{
\code{\link{searchLinesAtGivenSlope}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkVectLength.R
\name{.checkFactor}
\alias{.checkFactor}
\title{Check Factor}
\usage{
.checkFactor(
  fac,
  facNa = NULL,
  minLev = 2,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{fac}{(factor) main input}

\item{facNa}{(character) level-names}

\item{minLev}{(integer) minium number of levels}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a corrceted/adjusted factor
}
\description{
This function was designed to check a factor object
}
\examples{
.checkFactor(gl(3,2))
}
\seealso{
\code{\link[base]{factor}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rmDupl2colMatr.R
\name{rmDupl2colMatr}
\alias{rmDupl2colMatr}
\title{Remove Lines Of Matrix Redundant /Duplicated For 1st And 2nd Column}
\usage{
rmDupl2colMatr(
  mat,
  useCol = c(1, 2),
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{mat}{(matrix or data.frame) main input}

\item{useCol}{(integer, length=2) columns to consider/use when looking for duplicated entries}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a matrix where duplictaed lines are removed
}
\description{
This function removes lines of matrix that are redundant /duplicated for 1st and 2nd column (irrespective of content of their columns).
The first occurance of redundant /duplicated elements is kept.
}
\examples{
mat <- matrix(1:12,ncol=3)
mat[3,1:2] <- mat[1,1:2]
rmDupl2colMatr(mat)
}
\seealso{
\code{\link[base]{unlist}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/standardW.R
\name{standardW}
\alias{standardW}
\title{Standardize (scale) data}
\usage{
standardW(
  mat,
  byColumn = FALSE,
  na.rm = TRUE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{mat}{(matrix, data.frame or array) data that need to get standardized.}

\item{byColumn}{(logical) if \code{TRUE} the function will be run independently over all columns such as as \code{apply(mat,2,standardW)}}

\item{na.rm}{(logical) if \code{NA}s in the data don't get ignored via this argument, the output will be all \code{NA}}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This functions retruns a vector of rescaled data (in dimensions as input)
}
\description{
This functions work similar to \code{\link[base]{scale}}, however, it evaluates the entire input and not column-wise (and independeltly as \code{scale} does). 
With Standarizing we speak of transforming the data to end up with mean=O and sd=1.
Furthermore, in case of 3-dim arrays, this function returns also an object with the same dimensions as the input.
}
\examples{
dat <- matrix(2*round(runif(100),2), ncol=4)
mean(dat); sd(dat)

dat2 <- standardW(dat)
apply(dat2, 2, sd)
summary(dat2)

dat3 <- standardW(dat, byColumn=TRUE)
apply(dat2, 2, sd)
summary(dat2)
mean(dat2); sd(dat2)

}
\seealso{
\code{\link[base]{scale}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/combineRedundLinesInList.R
\name{combineRedundLinesInListAcRef}
\alias{combineRedundLinesInListAcRef}
\title{Combine Redundant Lines In List, Deprecated}
\usage{
combineRedundLinesInListAcRef(
  lst,
  listNa = c("ref", "quant"),
  refColNa = "xx",
  summarizeType = "av",
  NA.rm = TRUE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{lst}{(list) main input}

\item{listNa}{(character)   names of list-elements containing quantitation data (1st position) and protein/line annotation (2nd position)}

\item{refColNa}{(character) in case the list-element to be used as reference is \code{matrix} or \code{data.frame}, the column to be used must be specified here}

\item{summarizeType}{(character) the summarization method gets specified here; so far 'sum','av','med','first' and 'last' are implemented}

\item{NA.rm}{(logical) pass to summarizing functions order to omit \code{NA}s, defaults to \code{TRUE}}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a list of same length as input
}
\description{
The function combineRedundLinesInListAcRef() has been deprecated and replaced by combineRedundLinesInList() from the same package
}
\examples{
x1 <- list(quant=matrix(11:34, ncol=3, dimnames=list(letters[8:1], LETTERS[11:13])), 
  annot=matrix(paste0(LETTERS[c(1:4,6,3:5)],LETTERS[c(1:4,6,3:5)]), ncol=1, 
  dimnames=list(paste(letters[1:8]),"xx")) )
## please use combineRedundLinesInList()
combineRedundLinesInList(lst=x1, refNa="annot", datNa="quant", refColNa="xx")
}
\seealso{
\code{\link{combineRedundLinesInList}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simpleFragFig.R
\name{simpleFragFig}
\alias{simpleFragFig}
\title{Simple figure showing line from start- to end-sites of edges (or fragments) defined by their start- and end-sites
 
\code{simpleFragFig} draws figure showing start- and end-sites of edges (or fragments)}
\usage{
simpleFragFig(
  frag,
  fullSize = NULL,
  sortByHead = TRUE,
  useTit = NULL,
  useCol = NULL,
  displNa = TRUE,
  useCex = 0.7
)
}
\arguments{
\item{frag}{(matrix) 2 columns defining begin- and end-sites (as interger values)}

\item{fullSize}{(integer) optional max size used for figure (x-axis)}

\item{sortByHead}{(logical) sort by begin-sites (if \code{TRUE}) or sort by end-sites}

\item{useTit}{(character) custom title}

\item{useCol}{(character) specify colors, if numeric vector will be onsidered as score values}

\item{displNa}{(character) display names of edges (figure may get crowded)}

\item{useCex}{(numeric) expansion factor, see also \code{\link[graphics]{par}}}
}
\value{
matrix with mean values
}
\description{
Simple figure showing line from start- to end-sites of edges (or fragments) defined by their start- and end-sites
 
\code{simpleFragFig} draws figure showing start- and end-sites of edges (or fragments)
}
\examples{
frag2 <- cbind(beg=c(2,3,7,13,13,15,7,9,7, 3,7,5,7,3),end=c(6,12,8,18,20,20,19,12,12, 4,12,7,12,4)) 
rownames(frag2) <- c("A","E","B","C","D","F","H","G","I", "J","K","L","M","N")
simpleFragFig(frag2,fullSize=21,sortByHead=TRUE) 
buildTree(frag2)
}
\seealso{
\code{\link{buildTree}}, \code{\link{countSameStartEnd}}, \code{\link{contribToContigPerFrag}},
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trimRedundText.R
\name{.trimFromStart}
\alias{.trimFromStart}
\title{Trim from start (Deprecated)}
\usage{
.trimFromStart(
  x,
  ...,
  minNchar = 1,
  silent = TRUE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{x}{character vector to be treated}

\item{...}{more vectors to be treated}

\item{minNchar}{(integer) minumin number of characters that must remain}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) display additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a modified character vector
}
\description{
Deprecated Version - This function allows trimming/removing redundant text-fragments from start
}
\examples{
txt1 <- c("abcd_ccc","bcd_ccc","cde_ccc")
.trimLeft(txt1)  # replacement
}
\seealso{
\code{\link{trimRedundText}}; Inverse : Find/keep common text \code{\link{keepCommonText}};  you may also look for related functions in package \href{https://CRAN.R-project.org/package=stringr}{stringr}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/partUnlist.R
\name{partUnlist}
\alias{partUnlist}
\title{Partial unlist of lists of lists}
\usage{
partUnlist(lst, sep = "_", silent = FALSE, debug = FALSE, callFrom = NULL)
}
\arguments{
\item{lst}{(list) main input, list to be partially unlisted}

\item{sep}{(character, length=1) separator for names}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a list with partially reduced nested structure
}
\description{
\code{partUnlist} does partial unlist for treating list of lists : New (returned) list has one level less of hierarchy 
(Highest level list will be appended). In case of conflicting (non-null) listnames a prefix will be added. 
Behaviour different to \code{\link[base]{unlist}} when unlisting list of matrixes.
}
\examples{
partUnlist(list(list(a=11:12,b=21:24), list(c=101:101,d=201:204)))
li4 <- list(c=1:3, M2=matrix(1:4,ncol=2), L3=list(L1=11:12, M3=matrix(21:26,ncol=2)))
partUnlist(li4)
unlist(li4, rec=FALSE)
}
\seealso{
\code{\link[base]{unlist}}, \code{\link{asSepList}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getPWseparator.R
\name{replacePWseparator}
\alias{replacePWseparator}
\title{Replace Separator In Vector Of Pairwise Group-Names  
  
This function allows identifying and substituting a separator used in a character vector concatenated of pairwise groups.}
\usage{
replacePWseparator(
  compNames,
  newSep,
  potSep = c("-", "---", "_", "___", ".", "-vs-", "_vs_", "--vs--", "__vs__", " ", "  "),
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{compNames}{(character or integer) names of pairwise combined group-names}

\item{newSep}{(character of length=1) new separator between group-names for pair of names; if \code{NULL} a suitable separator (among '-','--','_','__','.',' ' and '  ') will be used}

\item{potSep}{(character) potential separators to be checked if occuring in \code{useComp}; the first fitting will used}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a character vector with group-wise substituted separators (or \code{NULL} if conflicts appear)
}
\description{
Replace Separator In Vector Of Pairwise Group-Names  
  
This function allows identifying and substituting a separator used in a character vector concatenated of pairwise groups.
}
\examples{
replacePWseparator(compNames=c("B-C","B-D","C-D"), newSep="-vs-") 
}
\seealso{
\code{\link{indexGroupsFromPW}}, \code{\link{getPWseparator}}, \code{\link{.getPWseparator}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conv01toColNa.R
\name{conv01toColNa}
\alias{conv01toColNa}
\title{Convert matrix of integer to matrix of x-times repeated column-names}
\usage{
conv01toColNa(mat, colNa = NULL, zeroTex = "", pasteCol = FALSE)
}
\arguments{
\item{mat}{input matrix (with integer values)}

\item{colNa}{alternative (column-)names to the ones from 'mat' (default colnames of 'mat')}

\item{zeroTex}{text to display if 0 (default "")}

\item{pasteCol}{(logical) allows to collapse all columns to single chain of characters in output}
}
\value{
character vector
}
\description{
\code{conv01toColNa} transforms matrix of integers (eg 0 and 1) to repeated & concatenated text from argument \code{colNa}, 
the character string for 0 occurances of argument \code{zeroTex} may be customized.
Used eg when specifying (and concatenating) various counted elements (eg properties) along a vector like variable peptide modifications in proteomics.
}
\examples{
(ma1 <- matrix(sample(0:3,40,repl=TRUE), ncol=4, dimnames=list(NULL, letters[11:14])))
conv01toColNa(ma1)
conv01toColNa(ma1, colNa=LETTERS[1:4], ze=".")
conv01toColNa(ma1, colNa=LETTERS[1:4], pasteCol=TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkStrictOrder.R
\name{.medianSpecGrp}
\alias{.medianSpecGrp}
\title{Rescale respective to specific group}
\usage{
.medianSpecGrp(x, grpNum, grpVal, sumMeth = "median", callFrom = NULL)
}
\arguments{
\item{x}{(numeric vector) main input}

\item{grpNum}{(numeric)}

\item{grpVal}{(numeric)}

\item{sumMeth}{(character) method for summarizing}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
numeric vector
}
\description{
This function allows to rescale data 'x' so that specific group 'grpNum' gets normalized to predefined value 'grpVal'.
In normal case x will be multiplied by 'grpVal' and devided by value obtained from 'grpNum'.
If summary of 'grpNum-positions' or 'grpVal' is 0, then grpVal will be attained by subtraction of summary & adding grpVal
}
\examples{
.firstMin(c(4,3:6))
}
\seealso{
\code{\link[base]{which.min}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scaleXY.R
\name{.scale01}
\alias{.scale01}
\title{Scale between 0 and 1 (main)}
\usage{
.scale01(x)
}
\arguments{
\item{x}{numeric vector to be re-scalded}
}
\value{
This function returns a numeric vector of same length with re-scaled values
}
\description{
This function rescales between 0 and 1
}
\examples{
.scale01(11:15)
}
\seealso{
\code{\link{scaleXY}} , \code{\link[base]{scale}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/exponNormalize.R
\name{exponNormalize}
\alias{exponNormalize}
\title{Normalize By Adjusting Exponent}
\usage{
exponNormalize(
  dat,
  useExpon,
  dynExp = TRUE,
  nStep = 20,
  startExp = 1,
  simMeas = "cor",
  refDat = NULL,
  refGrp = NULL,
  refLines = NULL,
  rSquare = FALSE,
  silent = FALSE,
  callFrom = NULL,
  debug = FALSE
)
}
\arguments{
\item{dat}{matrix or data.frame of numeric data to be normalized}

\item{useExpon}{(numeric vector or matrix) exponent values to be tested}

\item{dynExp}{(logical) require 'useExpon' as 2 values (matrix), will gradually increase exponent from 1st to 2nd; may be matrix or data.frame for dynamic, 
in this case 1st line for exp for lowest data, 2nd line for highest}

\item{nStep}{(integer) number of exponent variations (steps) when testing range from-to}

\item{startExp}{(numeric)}

\item{simMeas}{(character) similarity metric to be used (so far only "cor"), if rSquare=TRRUE, the r-squared will be returned}

\item{refDat}{(matrix or data.frame) if null average of each line from 'dat' will be used as reference in similarity measure}

\item{refGrp}{(factor) designing which col of 'ref' should be used with which col of 'dat' (length equal to number of cols in 'dat').
Note: 'refGrp' not yet coded optimally to extract numeric part of character vector, protential problems when all lines or cols of dat are NA}

\item{refLines}{(NULL or integer) optional subset of lines to be considered (only) when determining normalization factors}

\item{rSquare}{(logical) if \code{TRUE}, add r-squared}

\item{silent}{(logical) suppress messages}

\item{callFrom}{(character) allows easier tracking of messages produced}

\item{debug}{(logical) additional messages for debugging}
}
\value{
This functuion returns a matrix of normalized data
}
\description{
This function normalizes 'dat' by optimizing exponent function (ie dat ^exp) to fit best to 'ref' (default: average of each line of 'dat').
}
\examples{
set.seed(2016); dat1 <- matrix(c(runif(200)+rep(1:10,20)),nc=10)
head(rowGrpCV(dat1,gr=gl(4,3,labels=LETTERS[1:4])[2:11]))
set.seed(2016); dat1 <- c(0.1,0.2,0.3,0.5)*rep(c(1,10),each=4)
dat1 <- matrix(round(c(sqrt(dat1),dat1^1.5,3*dat1+runif(length(dat1))),2),nc=3)
dat2a <- exponNormalize(dat1[,1],useExpon=2,nSte=1,refD=dat1[,3])
layout(matrix(1:2,nc=2))
plot(dat1[,1],dat1[,3],type="b",main="init",ylab="ref") 
plot(dat2a$datNor[,1],dat1[,3],type="b",main="norm",ylab="ref")
dat2b <- exponNormalize(dat1[,1],useExpon=c(1.7,2.3),nSte=5,refD=dat1[,3])
plot(dat1[,1],dat1[,3],type="b",main="init",ylab="ref")
plot(dat2b$datNor[,1],dat1[,3],type="b",main="norm",ylab="ref")

dat2c <- exponNormalize(dat1[,-3],useExpon=matrix(c(1.7,2.3,0.6,0.8),nc=2),nSte=5,refD=dat1[,3]);
plot(dat1[,1],dat1[,3],type="b",main="init",ylab="ref ")
plot(dat2c$datNor[,1],dat1[,3],type="b",main="norm 1",ylab="ref")
plot(dat1[,2],dat1[,3],type="b",main="init",ylab="ref")
plot(dat2c$datNor[,2],dat1[,3],type="b",main="norm 2",ylab="ref"); 
}
\seealso{
more eveolved than \code{\link[wrMisc]{normalizeThis}} with arugment set to 'exponent'
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/adjBy2ptReg.R
\name{.datSlope}
\alias{.datSlope}
\title{Model linear regression and optional plot}
\usage{
.datSlope(
  dat,
  typeOfPlot = "sort",
  toNinX = FALSE,
  plotData = FALSE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{dat}{(vector or matrix) main input}

\item{typeOfPlot}{(character)}

\item{toNinX}{(logical)}

\item{plotData}{(logical)}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) display additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
numeric vector with intercept and slope, optional plot
}
\description{
This function allows to model a linear regression and optionally to plot the results
}
\examples{
.datSlope(c(3:6))
}
\seealso{
\code{\link[base]{append}}; \code{\link{lrbind}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nNonNumChar.R
\name{.extrNumHeadingCap}
\alias{.extrNumHeadingCap}
\title{Extract number(s) before capital character}
\usage{
.extrNumHeadingCap(x)
}
\arguments{
\item{x}{character vector to be treated}
}
\value{
This function returns a numeric vector
}
\description{
This function aims to extract number(s) before capital character
}
\examples{
.extrNumHeadingCap(" 1B ")
}
\seealso{
\code{\link[base]{grep}}, \code{\link[base]{nchar}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readTabulatedBatch.R
\name{readTabulatedBatch}
\alias{readTabulatedBatch}
\title{Batch Reading Of Tabulated Text-Files}
\usage{
readTabulatedBatch(
  query,
  path = NULL,
  dec = ".",
  header = "auto",
  strip.white = FALSE,
  blank.lines.skip = TRUE,
  fill = FALSE,
  filtCol = 2,
  filterAsInf = TRUE,
  filtVal = 5000,
  silent = FALSE,
  callFrom = NULL,
  debug = FALSE
)
}
\arguments{
\item{query}{(character) vector of file-names to be read, if \code{"."} all files will be read (no matter what their extension might be)}

\item{path}{(character) path for reading files, if \code{NULL} or \code{NA} the current directory will be used}

\item{dec}{(character, length=1) decimals to use, will be passed to \code{\link[data.table]{fread}} or \code{\link[utils]{read.delim}}}

\item{header}{(character, length=1) path for reading files, if \code{NULL} or \code{NA} the current directory will be used, will be passed to \code{\link[data.table]{fread}} or \code{\link[utils]{read.delim}}}

\item{strip.white}{(logical, length=1) Strips leading and trailing whitespaces of unquoted fields, will be passed to \code{\link[data.table]{fread}} or \code{\link[utils]{read.delim}}}

\item{blank.lines.skip}{(logical, length=1)  If \code{TRUE} blank lines in the input are ignored. will be passed to \code{\link[data.table]{fread}} or \code{\link[utils]{read.delim}}}

\item{fill}{(logical, length=1) If \code{TRUE} then in case the rows have unequal length, blank fields are implicitly filled, will be passed to \code{\link[data.table]{fread}} or \code{\link[utils]{read.delim}}}

\item{filtCol}{(integer, length=1) which columns should be used for filtering, if \code{NULL} or \code{NA} all data will be returned}

\item{filterAsInf}{(logical, length=1) filter as inferior or equal (\code{TRUE}) or superior or equal threshold \code{filtVal}}

\item{filtVal}{(numeric, length=1) which numeric threshold should be used for filtering, if \code{NULL} or \code{NA} all data will be returned}

\item{silent}{(logical) suppress messages}

\item{callFrom}{(character) allow easier tracking of messages produced}

\item{debug}{(logical) display additional messages for debugging}
}
\value{
This function returns a list of data.frames
}
\description{
This function allows batch reading of multiple tabulated text files n batch. 
The files can be designed specifically, or, alternatively all files from a given directory can be read.
If package \href{https://CRAN.R-project.org/package=data.table}{data.table} is available, faster reading of files will be performed using the function \code{\link[data.table]{fread}}.
}
\details{
If you want to provide a flexible pattern of ffile-names, this has to be done before calling this usntion, eg using \code{grep} to provide an explicit collection of flles.
However, it is possible to read different files from different locations/directories, the length of \code{path} must match the length of \code{query}
}
\examples{
path1 <- system.file("extdata", package="wrMisc")
fiNa <-  c("a1.txt","a2.txt")
allTxt <- readTabulatedBatch(fiNa, path1)
str(allTxt)

}
\seealso{
\code{\link[data.table]{fread}}, \code{\link[utils]{read.delim}}, for reading batch of csv files : \code{\link{readCsvBatch}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filtSizeUniq.R
\name{.filtSize}
\alias{.filtSize}
\title{Filter for size}
\usage{
.filtSize(x, minSize = 5, maxSize = 36)
}
\arguments{
\item{x}{main inpuy}

\item{minSize}{(integer) minimum number of characters, if \code{NULL} set to 0}

\item{maxSize}{(integer) maximum number of characters}
}
\value{
This function returns a list of filtered input
}
\description{
This function aims to filter for size
}
\examples{
aa <- 1:10
}
\seealso{
\code{\link{filtSizeUniq}}; \code{\link{correctToUnique}}, \code{\link[base]{unique}}, \code{\link[base]{duplicated}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coordOfFilt.R
\name{coordOfFilt}
\alias{coordOfFilt}
\title{get coordinates of values/points in matrix according to filtering condition}
\usage{
coordOfFilt(mat, cond, sortByRows = FALSE, silent = FALSE, callFrom = NULL)
}
\arguments{
\item{mat}{(matrix or data.frame) matrix or data.frame}

\item{cond}{(logical or integer) condition/test to see which values of \code{mat} fulfull test, or integer of index passing}

\item{sortByRows}{(logical) optional sorting of results by row-index}

\item{silent}{(logical) suppress messages}

\item{callFrom}{(character) allow easier tracking of message(s) produced}
}
\value{
matrix columns 'row' and 'col'
}
\description{
Get coordinates of values/points in matrix according to filtering condition
}
\examples{
set.seed(2021); ma1 <- matrix(sample.int(n=40,size=27,replace=TRUE), ncol=9)
## let's test which values are >37
which(ma1 >37)      # doesn't tell which row & col
coordOfFilt(ma1, ma1 >37)

}
\seealso{
\code{\link[base]{which}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reorgByCluNo.R
\name{reorgByCluNo}
\alias{reorgByCluNo}
\title{Reorganize matrix according to clustering-output}
\usage{
reorgByCluNo(
  mat,
  cluNo,
  useColumn = NULL,
  meanCol = NULL,
  addInfo = TRUE,
  retList = FALSE,
  silent = FALSE,
  callFrom = NULL,
  debug = FALSE
)
}
\arguments{
\item{mat}{(matrix or data.frame) main input}

\item{cluNo}{(positive integer, length to match nrow(dat) initial cluster numbers for each line of 'mat' (obtained by separate clustering or other segmentation) or may desinn column of \code{mat} to use as cluster-numbers}

\item{useColumn}{(character or integer) the columns to use from \code{mat} as main data (default will use all, exept \code{cluCol} and/or \code{meanCol} if they design columns))}

\item{meanCol}{(character or integer) alternative summarizing data for intra-cluster sorting (instead of geometric mean)}

\item{addInfo}{(logical) allows  of columns 'index', 'geoMean' and 'cluNo'  (or array if \code{FALSE})}

\item{retList}{(logical) return as list of matrixes (or array if \code{FALSE})}

\item{silent}{(logical) suppress messages}

\item{callFrom}{(character) allow easier tracking of messages produced}

\item{debug}{(logical) additional messages for debugging}
}
\value{
This function returns a list or array (as 2- or 3 dim) with possible number of occurances for each of the 3 elements in nMax. Read results vertical : out[[1]] or out[,,1] .. (multiplicative) table for 1st element of nMax; out[,,2] .. for 2nd
}
\description{
Reorganize input matrix as sorted by cluster numbers (and geometric mean) according to vector with cluster names, and index for sorting per cluster and per geometric mean.
In case \code{mat} is an array, the 3rd dimension will be considered as 'column' with arguments \code{useColumn} ( and \code{cluNo}, if it designs a 'column' of mat).
}
\examples{
dat1 <- matrix(round(runif(24),2), ncol=3, dimnames=list(NULL,letters[1:3]))
clu <- stats::kmeans(dat1, 5)$cluster
reorgByCluNo(dat1, clu) 

dat2 <- cbind(dat1, clu=clu) 
reorgByCluNo(dat2, "clu") 
}
\seealso{
pairwise combinations \code{\link[utils]{combn}}, clustering \code{\link[stats]{kmeans}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cbindNR.R
\name{cbindNR}
\alias{cbindNR}
\title{cbind to non-redundant}
\usage{
cbindNR(
  ...,
  convertDFtoMatr = TRUE,
  sortOutput = TRUE,
  summarizeAs = "sum",
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{...}{all matrixes to get combined in cbind way}

\item{convertDFtoMatr}{(logical) decide if output should be converted to matrix}

\item{sortOutput}{(logical) optional sorting by column-names}

\item{summarizeAs}{(character) decide of combined values should get summed (default, 'sum') or averaged ('mean')}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a matrix or data.frame (as cbind would return)
}
\description{
\code{cbindNR} combines all matrixes given as arguments to non-redundant column names (by ADDING the number of 'duplicated' columns !). 
Thus, this function works similar to \code{cbind}, but allows combining multiple matrix-objects containing redundant column-names.
Of course, all input-matrixes must have the same number of rows !
By default, the output gets sorted by column-names.
Note, due to the use of '...' arguments must be given by their full argument-names, lazy evaluation might not recognize properly argument names.
}
\examples{
ma1 <- matrix(1:6, ncol=3, dimnames=list(1:2,LETTERS[3:1]))
ma2 <- matrix(11:16, ncol=3, dimnames=list(1:2,LETTERS[3:5]))
cbindNR(ma1, ma2)
cbindNR(ma1, ma2, summarizeAs="mean")
}
\seealso{
\code{\link[base]{cbind}}, \code{\link{nonAmbiguousNum}}, \code{\link{firstOfRepLines}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/combineByEitherFactor.R
\name{combineByEitherFactor}
\alias{combineByEitherFactor}
\title{Create factor-like column regrouping data regrouping simultaneaously by two factors}
\usage{
combineByEitherFactor(
  mat,
  refC1,
  refC2,
  nByGrp = FALSE,
  convergeMax = TRUE,
  callFrom = NULL,
  debug = FALSE,
  silent = FALSE
)
}
\arguments{
\item{mat}{main input matrix}

\item{refC1}{(integer) column-number of 'mat' to use as 1st set}

\item{refC2}{(integer) column-number of 'mat' to use as 2nd set}

\item{nByGrp}{(logical) add last col with n by group}

\item{convergeMax}{(logical) if \code{TRUE}, run 2 add'l iteartive steps to search convergence to stable result}

\item{callFrom}{(character) allow easier tracking of message(s) produced}

\item{debug}{(logical) display additional messages for debugging}

\item{silent}{(logical) suppress messages}
}
\value{
This function returns a matrix containing both selected columns plus additional column(s) indicating group-number of the pair-wise combination 
(and optional the total n by group)
}
\description{
This function aims to address the situation when two somehow different groupins (of the same data) exist and need to be joined.
It is not necessary that both alternative groupings use the same labels, neither.
\code{combineByEitherFactor} adds new (last) column named 'grp' to input matrix representing the combined factor 
relative to 2 specified columns from input matrix  \code{mat} (via 'refC1','refC2'). Optionally, the output may be 
sorted and a column giving n per factor-level may be added. 
The function treats selected columns of  \code{mat} 
as pairwise combination of 2 elements (that may occur multiple times over all lines of  \code{mat})
and sorts/organizes all instances of such combined elements (ie from both selected columns) as repeats of a given group,
who's class number is given in output column 'grp', the (total) number of repeats may be displayed in column 'nGrp' ( \code{nByGrp=TRUE}).
If groups are overlapping (after re-ordering), an iterative process of max 3x2 passes will be launched after initial matching.
Works on numeric as well as character input.
}
\examples{
nn <- rep(c("a","e","b","c","d","g","f"),c(3,1,2,2,1,2,1))
qq <- rep(c("m","n","p","o","q"),c(2,1,1,4,4))
nq <- cbind(nn,qq)[c(4,2,9,11,6,10,7,3,5,1,12,8),]
combineByEitherFactor(nq,1,2,nBy=TRUE); combineByEitherFactor(nq,1,2,nBy=FALSE)
combineByEitherFactor(nq,1,2,conv=FALSE); combineByEitherFactor(nq,1,2,conv=TRUE)
##
mm <- rep(c("a","b","c","d","e"),c(3,4,2,3,1)); pp <- rep(c("m","n","o","p","q"),c(2,2,2,2,5))
combineByEitherFactor(cbind(mm,pp), 1, 2, con=FALSE, nBy=TRUE)
combineByEitherFactor(cbind(mm,pp), 1, 2, con=TRUE, nBy=TRUE) 
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/partialDist.R
\name{.findBorderOverlaps}
\alias{.findBorderOverlaps}
\title{Find overlap instances among range of values in lines}
\usage{
.findBorderOverlaps(x, rmRedund = FALSE, callFrom = NULL)
}
\arguments{
\item{x}{(matrix of numeric values or all-numeric data.frame) main input}

\item{rmRedund}{(logical) report overlaps only in 1st instance (will show up twice otherwise)}

\item{callFrom}{(character) allow easier tracking of message(s) produced}
}
\value{
This function returns a matrix with line for each overlap found, cols 'refLi' (line no), 'targLi' (line no), 'targCol' (col no)
}
\description{
This function aims to find overlap instances among range of values in lines of 'x' (typically give just min & max)
}
\examples{
aa <- 11:15
}
\seealso{
\code{\link[base]{nchar}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/normalizeThis.R
\name{normalizeThis}
\alias{normalizeThis}
\title{Normalize Data In Various Modes}
\usage{
normalizeThis(
  dat,
  method = c("mean", "average", "median", "trimMean", "rowNormalize", "standardize",
    "slope", "twoPointSlope", "exponent", "vsn", "none", "NULL"),
  refLines = NULL,
  refGrp = NULL,
  mode = c("proportional", "additive", "linear", "lin", "logarithmic", "log"),
  trimFa = NULL,
  minQuant = NULL,
  sparseLim = 0.4,
  nCombin = 3,
  omitNonAlignable = FALSE,
  maxFact = 10,
  quantFa = NULL,
  expFa = NULL,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{dat}{matrix or data.frame of data to get normalized}

\item{method}{(character) may be "mean","median","NULL","none", "trimMean", "rowNormalize", "slope", "exponent", "twoPointSlope", "vsn"; When \code{NULL} or 'none' is chosen the input will be returned}

\item{refLines}{(NULL or numeric) allows to consider only specific lines of 'dat' when determining normalization factors (all data will be normalized)}

\item{refGrp}{Only the columns indicated will be used as reference, default all columns (integer or colnames)}

\item{mode}{(character) may be "proportional", "additive" (for log2 data);
decide if normalization factors will be applied as multiplicative (proportional) or additive; for log2-omics data \code{mode="additive"} is suggested
besides 'log' or 'logarithmic' may be used instead of 'additive'; and 'lin' or 'linear' may be used instead of 'proportional'}

\item{trimFa}{(numeric, length=1) additional parameters for trimmed mean}

\item{minQuant}{(numeric) only used with \code{method='rowNormalize'}: optional filter to set all values below given value as \code{NA}; see also \code{\link{rowNormalize}}}

\item{sparseLim}{(integer) only used with \code{method='rowNormalize'}: decide at which min content of  \code{NA} values the function should go in sparse-mode; see also \code{\link{rowNormalize}}}

\item{nCombin}{(NULL or integer) only used with \code{method='rowNormalize'}: used only in sparse-mode (ie if content of \code{NA}s higher than content of \code{sparseLim}):
Number of groups of smller matrixes with this number of columns to be inspected initially;
low values (small groups have higher chances of more common elements); see also \code{\link{rowNormalize}}}

\item{omitNonAlignable}{(logical) only used with \code{method='rowNormalize'}: allow omitting all columns which can't get aligned due to sparseness; see also \code{\link{rowNormalize}}}

\item{maxFact}{(numeric, length=2) only used with \code{method='rowNormalize'}:  max normalization factor; see also \code{\link{rowNormalize}}}

\item{quantFa}{(numeric, length=2) additional parameters for quantiles to use with method='slope'}

\item{expFa}{(numeric, length=1) additional parameters for method='exponent'}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allows easier tracking of messages produced}
}
\value{
This function returns a matrix of normalized data (same dimensions as input)
}
\description{
Generic normalization of 'dat' (by columns), multiple methods may be applied.
The choice of normalization procedures must be done with care, plotting the data before and after normalization
may be critical to understandig the initial data structure and the effect of the procedure applied.
Inappropriate methods chosen may render interpretation of (further) results incorrect.
}
\details{
In most cases of treating 'Omics'-data one works with the hypothesis that there are no global changes in the structure of all data/columns
Under this htpothesis it is very common to assume the the median (via the argument \code{method}) of all samples (ie columns) should remain constant.
For examples samples/columns with less signal will be considered as having received 'accidentally' less material (eg due to the imprecision when transfering very small amounts of liquid samples).
In consequence, a sample having received only 95% of input material is assumed to give only 95% signal intensity of what may have been expected.
Thus, all measures will be multiplied by 1/0.95 (apr 1.053) to compensate for supposed lack of staring material.

LOG-SCALE : With the analysis of 'Omics'-data it is very common to work with data on log-scale.
In this case the argument \code{mode} should be set to \code{additive} (or \code{log} or \code{logarithmic}).
At normalization a constant factor will be added (or subtracted) using this mode. This corresponds to a multiplicative factor on regular scale.
Please note that (at this point) the methods 'slope', 'exponent', 'twoPointSlope' and 'vsn' don't distinguish between additive and proportional modes, but take take the data 'as is'
(you may look at the original documenation for more details, see \code{\link{exponNormalize}}, \code{\link{adjBy2ptReg}}, \code{\link[vsn]{justvsn}}).

Normalization using \code{method="rowNormalize"} runs \code{\link{rowNormalize}} from this package.
In this case, the working hypothesis is, that all values in each row are expected to be the same.
This method could be applied when all series of values (ie columns) are replicate measurements of the same sample.
THere is also an option for treating sparse data (see argument \code{sparseLim}), which may, hovere, consume much more comptational ressources,
in particular, when the value \code{nCombin} is low (compared to the number of samples/columns).

Normalization using  \code{method="standardize"} runs \code{\link[base]{scale}} resulting in a matrix where column-means are set to 0 (or very close to 0)
and sd et 1, standardization by columns. With this method additive or proportional mode give the same output.  
Results from this methid should be interpreted with caution, since the absolute values are not any more considered. 
Therefor this approach is rather oriented for comparing profiles (no matter how important changes are on an absolute scale)

Normalization using  \code{method="vsn"} runs \code{\link[vsn]{justvsn}} from \href{https://bioconductor.org/packages/release/bioc/html/vsn.html}{vsn}
(this requires a minimum of 42 rows of input-data and having the Bioconductor package vsn installed).
Note : Depending on the procedure chosen, the normalized data may appear on a different scale.
}
\examples{
set.seed(2015); rand1 <- round(runif(300)+rnorm(300,0,2),3)
dat1 <- cbind(ser1=round(100:1+rand1[1:100]), ser2=round(1.2*(100:1+rand1[101:200])-2),
  ser3=round((100:1 +rand1[201:300])^1.2-3))
dat1 <- cbind(dat1, ser4=round(dat1[,1]^seq(2,5,length.out=100)+rand1[11:110],1))
dat1[dat1 <1] <- NA
  summary(dat1)
  dat1[c(1:5,50:54,95:100),]
no1 <- normalizeThis(dat1, refGrp=1:3, meth="mean")
no2 <- normalizeThis(dat1, refGrp=1:3, meth="trimMean", trim=0.4)
no3 <- normalizeThis(dat1, refGrp=1:3, meth="median")
no4 <- normalizeThis(dat1, refGrp=1:3, meth="slope", quantFa=c(0.2,0.8))
dat1[c(1:10,91:100),]
cor(dat1[,3],rowMeans(dat1[,1:2],na.rm=TRUE), use="complete.obs")             # high
cor(dat1[,4],rowMeans(dat1[,1:2],na.rm=TRUE), use="complete.obs")             # bad
cor(dat1[c(1:10,91:100),4],rowMeans(dat1[c(1:10,91:100),1:2],na.rm=TRUE),use="complete.obs")
cor(dat1[,3],rowMeans(dat1[,1:2],na.rm=TRUE)^ (1/seq(2,5,length.out=100)),use="complete.obs")
}
\seealso{
\code{\link{rowNormalize}}, \code{\link{exponNormalize}}, \code{\link{adjBy2ptReg}}, \code{\link[vsn]{justvsn}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scaleXY.R
\name{.scaleXY}
\alias{.scaleXY}
\title{Scale between min and max value (main)}
\usage{
.scaleXY(x, minim = 2, maxim = 3)
}
\arguments{
\item{x}{numeric vector to be re-scalded}

\item{minim}{(numeric) minimum value for resultant vactor}

\item{maxim}{(numeric) minimum value for resultant vactor}
}
\value{
This function returns a matrix of CV values
}
\description{
This function rescales between user-defined min and max values
}
\examples{
.scaleXY(11:15, min=1, max=100)
}
\seealso{
\code{\link{scaleXY}} , \code{\link[base]{scale}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tableToPlot.R
\name{tableToPlot}
\alias{tableToPlot}
\title{Print matrix-content as plot}
\usage{
tableToPlot(
  matr,
  colPos = c(0.05, 0.35, 0.41, 0.56),
  useCex = 0.7,
  useAdj = c(0, 1, 1, 0),
  titOffS = 0,
  useCol = 1,
  silent = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{matr}{(matrix) main (character) matrix to display}

\item{colPos}{(numeric) postion of columns on x-scale (from 0 to 1)}

\item{useCex}{(numeric) cex expension factor forsiez of text (may be different for each column)}

\item{useAdj}{(numeric) left/cneter/right alignment for text (may be different for each column)}

\item{titOffS}{(numeric) offset for title line (ralive to 'colPos')}

\item{useCol}{color specification for text (may be different for each column)}

\item{silent}{(logical) suppress messages}

\item{callFrom}{(character) allow easier tracking of message(s) produced}
}
\value{
This function returns NULL (no R-object returned), print 'plot' in current device only
}
\description{
This function prints all columns of matrix in plotting region for easier inclusion to reports (default values are set to work for output as A4-sized pdf).
It was made for integrating listings of text to graphical output to devices like \code{png}, \code{jpeg} or \code{pdf}.
}
\details{
This function was initially designed for listings with small/medium 1st col (eg couner or index), 2nd & 3rd col small and long 3rd col (like file paths).
Obviously, the final number of lines one can pack and still read correctly into the graphical output depends on the size of the device 
(on a pdf of size A4 one can pack up to apr. 11O lines). 
Of ourse, \code{\link[utils]{Sweave}}, combined with LaTeX, provides a powerful alternative for wrapping text to pdf-output (and further combining text and graphics).
Note: The final result on pdf devices may vary depending on screen-size (ie with of current device), the parameters 'colPos' and 'titOffS' may need some refinements.
Note: In view of typical page/figure layouts like A4, the plotting region will be split to avoid too wide spacing between rows with less than 30 rows.
}
\examples{
## as example let's make a listing of file-names and associated parameters in current directory
mat <- dir()
mat <- cbind(no=1:length(mat),fileName=mat,mode=file.mode(mat),
  si=round(file.size(mat)/1024),path=getwd())
## Now, we wrap all text into a figure (which could be saved as jpg, pdf etc)
tableToPlot(mat[,-1],colPos=c(0.01,0.4,0.46,0.6),titOffS=c(0.05,-0.03,-0.01,0.06))
tableToPlot(mat,colPos=c(0,0.16,0.36,0.42,0.75),useAdj=0.5,titOffS=c(-0.01,0,-0.01,0,-0.1)) 
}
\seealso{
\code{\link[utils]{Sweave}} for more flexible framework
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/moderTestXgrp.R
\name{moderTestXgrp}
\alias{moderTestXgrp}
\title{Multiple Moderated Pairwise t-tests From limma}
\usage{
moderTestXgrp(
  dat,
  grp,
  useComparison = NULL,
  limmaOutput = TRUE,
  addResults = c("lfdr", "FDR", "Mval", "means"),
  testOrientation = "=",
  sep = NULL,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{dat}{matrix or data.frame with rows for multiple (independent) tests, use ONLY with 2 groups; assumed as log2-data !!!}

\item{grp}{(factor) describes column-relationship of 'dat'   (1st factor is considered as reference -> orientation of M-values !!)}

\item{useComparison}{(numeric, character or matrix) optional way to indicate which pairwise comparisons should be performed :
if \code{useComparison="all"} all pairwise comparisons possible will be performed,
otherise pairwise comparisons may be chosen using indexes (the ith instance of all possible pairwise comparisons);
a character vector citing the concatenated levels of  argument \code{grp}; may be used (in this case it may be necessary to specify the separator used in argument argument \code{sep} ;
please note, it is important to check that a custom selected separator \code{sep} does not occur inside the groups from \code{grp});
finally, the argument \code{useComparison} may be or as matrix of 2 columns where each row specifies a different comparison, 
the matrix mat contain the names of the levels of argument \code{grp} or an index relative to the levels of argument \code{grp};
In all cases where 2 groups are specified this will be read as 'sample-reference' - thus, the 2nd entity will be taken as reference;
Of course this separator should not occur in 'grp' (if present it will be automatically repaced by '__' in 'grp' ).}

\item{limmaOutput}{(logical) return full (or extended) MArrayLM-object from limma or 'FAlSE' for only the (uncorrected) p.values}

\item{addResults}{(character) types of results to add besides basic limma-output, data are assumed to be log2 ! (eg "lfdr" using fdrtool-package, "FDR" or "BH" for BH-FDR, "BY" for BY-FDR, 
"bonferroni" for Bonferroni-correction, "qValue" for lfdr by qvalue, "Mval", "means" or "nonMod" for non-moderated test and he equivaent all (other) multiple testing corrections chosen here)}

\item{testOrientation}{(character) for one-sided test (">","greater" or "<","less"), NOTE : 2nd grp is considered control/reference, '<' will identify grp1 < grp2}

\item{sep}{(\code{NULL]} or character of length=1) optional custom choice for separator for column-names of pairwise comparisons, otherwise determined as character(-set) not occurring in \code{grp}}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a limma-type MA-object (which can be handeled like a list), or when problems are encountered \code{NULL}
}
\description{
This function runs (selected or all) pairwise combinations of moderated t-tests from package 'limma' on each line of data against.
Note: This function requires the package \href{https://bioconductor.org/packages/release/bioc/html/limma.html}{limma} from bioconductor. 
   The limma contrast-matrix has to be read by column, the lines in the contrast-matrix containing '+1' will be compared to the '-1' lines, eg grpA-grpB .
}
\details{
When multiple pairwise comparisons will be run, first a global linear model will be estimated and the particular pairwise comparisons will then be performed using a contrast-matrix.
This process is described with the bioconductor package \href{https://bioconductor.org/packages/release/bioc/html/limma.html}{limma} which is used underneith.

By default (\code{useComparison=NULL}), the first of all possible pairwise comparisons will be run.
If you would like to run all comparisons, plase set \code{useComparison="all"}.
Besides, it is also possible to custom choose which comparisons should be run (and which order of sample/reference) via the argument \code{useComparison} using
the character vector '--' as separator for the two groups of samples (referring to argument \code{grp}) to be compared.
This will be interpreted as 'sample--reference' (additional space around the separator, if present, will be removed), thus the second element will be used as reference. 
Furthermore, the argument \code{useComparison} may be a matrix of 2 columns (for sample and reference) where each line represents a pairwise comparison that should be run.  

As effort for compatibility to previous versions and compatibility to standard writing in limma the group-separator '-' in \code{useComparison} has limited support : 
This (single character) separator will be internally converted to '--' and in absence of '-' in argument \code{grp}, output will have the initial  (single character) separator '-'.

Concerning the separator used when reporting results for pairwise comparisons :
If argument \code{grp} contains any '-', comparisons of groups will always (!) get reported using '--' as separator to avoid any confusion.
In case \code{useComparison} is a matrix, the reported comparisons will have '-' if \code{grp} does not contain as well any '-', otherwise '--' will be used 
(to support compatibility with results from limma).

Please note, that in the output of this function the Benjamini-Hochberg (BH) adjusted p-values are called 'FDR' (see argument \code{addResults}).

the output caontains a list-element named \code{setup} summarizing the global setup as captured (names of groups of samples), including the names of the pairwise combinations that were tested.
}
\examples{
grp3 <- factor(rep(LETTERS[c(3,1,4)],c(2,3,3)))
set.seed(2017); t8 <- matrix(round(rnorm(208*8,10,0.4),2), ncol=8,
  dimnames=list(paste(letters[],rep(1:8,each=26),sep=""), paste0(grp3, c(1:2,1:3,1:3))))
t8[3:6,1:2] <- t8[3:6,1:2] +3                    # augment lines 3:6 (c-f) 
t8[5:8,c(1:2,6:8)] <- t8[5:8,c(1:2,6:8)] -1.5    # lower lines 
t8[6:7,3:5] <- t8[6:7,3:5] +2.2                  # augment lines 
## expect to find C/A in c,d,g, (h)
## expect to find C/D in c,d,e,f
## expect to find A/D in f,g,(h) 
test8 <- moderTestXgrp(t8, grp3) 

## Custom choice of what to compare
test8b <- moderTestXgrp(t8, grp3, useComparison=c("D-A","C-A")) 
head(test8b$FDR)
head(test8b$Mval)

## Custom choice as matrix
comp4 <- matrix(LETTERS[c(4,3,3,1)], ncol=2)
test4 <- moderTestXgrp(t8, grp3, useComparison=comp4)

## Names of pairwise comparisons performed
test4$setup$concat

## One can also use functions from package limma to see more
library(limma)
topTable(test8b, n=5)
}
\seealso{
\code{\link{moderTest2grp}} for simplified single comparisons (with fewer options/flexibility), \code{\link[limma]{lmFit}} and the \code{eBayes}-family of functions in package \href{https://bioconductor.org/packages/release/bioc/html/limma.html}{limma}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/organizeAsListOfRepl.R
\name{.fuse2ArrBy2ndDim}
\alias{.fuse2ArrBy2ndDim}
\title{fuse 2 instances of 3dim arr as mult cols in 3dim array}
\usage{
.fuse2ArrBy2ndDim(arr1, arr2, silent = FALSE, debug = FALSE, callFrom = NULL)
}
\arguments{
\item{arr1}{(array)}

\item{arr2}{(array)}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This functuin returns a numeric vector with  numer of non-numeric characters (ie not '.' or 0-9))
}
\description{
This function allows fusing 2 instances of 3dim arr as mult cols in 3dim array (ie fuse along 2nd dim, increase cols)
}
\examples{
aa <- 11:15
}
\seealso{
\code{\link[base]{array}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/searchDataPairs.R
\name{searchDataPairs}
\alias{searchDataPairs}
\title{Search duplicated data over multiple columns, ie pairs of data}
\usage{
searchDataPairs(
  dat,
  disThr = 0.01,
  byColumn = TRUE,
  normRange = TRUE,
  altNa = NULL,
  realDupsOnly = TRUE,
  silent = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{dat}{matrix or data.frame (main input)}

\item{disThr}{(numeric) threshold to decide when to report similar data (applied on normalized distances, low val fewer reported), applied on normalized distances (norm to diagonale of all data for best relative 'unbiased' view)}

\item{byColumn}{(logical) rotates main input by 90 degrees (using \code{\link[base]{t}}), thus allows to read by rows instead of by columns}

\item{normRange}{(logical) normize each columns separately if \code{TRUE}}

\item{altNa}{(character, default \code{NULL}) vector with alternative names (for display)}

\item{realDupsOnly}{(logical) if \code{TRUE} will consider equal values only, otherwise will also consider very close values (based on argument \code{disThr})}

\item{silent}{(logical) suppres messages}

\item{callFrom}{(character) allows easier tracking of messages produced}
}
\value{
This function returns a data.frame with names of sample-pairs, percent of identical values (100 for complete identical pair) and rel (Euclidean) distance (ie max dist observed =1.0)
}
\description{
\code{searchDataPairs} searches matrix for columns of similar data, ie 'duplicate' values in separate columns or very similar columns if \code{realDupsOnly=FALSE}.
Initial distance measures will be normalized either to diagonale (\code{normRange=TRUE)} of 'window' or to the real max distance observed (equal or less than diagonale).
Return data.frame with names for sample-pair, percent of identical values (100 for complete identical pair) and relative (Euclidean) distance (ie max dist observed =1.0).
Note, that low distance values do not necessarily imply correlating data.
}
\examples{
mat <- round(matrix(c(11:40,runif(20)+12,11:19,17,runif(20)+18,11:20), nrow=10), 1)
colnames(mat) <- 1:9
searchDataPairs(mat,disThr=0.05)
}
\seealso{
\code{\link[base]{duplicated}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffPPM.R
\name{diffPPM}
\alias{diffPPM}
\title{Difference in ppm between numeric values}
\usage{
diffPPM(dat, toPrev = FALSE, silent = FALSE, callFrom = NULL)
}
\arguments{
\item{dat}{(numeric) vector for calculating difference to preceeding/following value in ppm}

\item{toPrev}{(logical) determine oriention}

\item{silent}{(logical) suppress messages}

\item{callFrom}{(character) allows easier tracking of messages produced}
}
\value{
This function returns a list with close matches of 'x' to given 'y', the numeric value dependes on 'sortMatch' (if FALSE then always value of 'y' otherwise of longest of x&y)
}
\description{
This is a \code{diff()}-like function to return difference in ppm between subsequent values. 
 Result is oriented, ie neg ppm value means decrease (from higher to lower value). Note that if the absolute difference remains the same the difference in ppm will not remain same.
 Any difference to NA is returned as NA, thus a single NA will result in two NAs in output (unless NA is 1st or last).
}
\examples{
aa <- c(1000.01, 1000.02, 1000.05, 1000.08, 1000.09, 1000.08)
.compareByPPM(list(aa,aa), 30, TRUE)                    # tabular 'long' version
diffPPM(aa)
}
\seealso{
\code{\link{checkSimValueInSer}} and (from this package) \code{.compareByDiff},  \code{\link[base]{diff}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/concatMatch.R
\name{concatMatch}
\alias{concatMatch}
\title{Value Matching With Option For Concatenated Terms}
\usage{
concatMatch(
  x,
  table,
  sep = ",",
  sepPattern = NULL,
  globalPat = "digitExtension",
  nomatch = NA_integer_,
  incomparables = NULL,
  extensPat = TRUE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{x}{(vector) the values to be matched}

\item{table}{(vector) the values to be matched against (ie reference)}

\item{sep}{(character) separator character in case concatenation of entries is tested}

\item{sepPattern}{(character or \code{NULL}) optional custom  pattern for splitting concatenations of \code{x}) and \code{table}) (in case \code{NULL}) is not sufficient)}

\item{globalPat}{(character) pattern for additional trimming of serach-terms. If \code{globalPat="digitExtension"} all terminal digits will not be considered when matching}

\item{nomatch}{(vector) similar to \code{\link[base]{match}} the value to be returned in the case when no match is found}

\item{incomparables}{(vector) similar to \code{\link[base]{match}}, a vector of values that cannot be matched. Any value in x matching a value in this vector is assigned the nomatch value.}

\item{extensPat}{(logical) similar to \code{\link[base]{match}} the value to be returned in the case when no match is found}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a character vector with verified path and file-name(s), returns \code{NULL} if nothing
}
\description{
This is a _match()_-like function allowing to serach among concatenated terms/IDs, additional options to remove text pattern like terminal lowercase extesion are available.
The function returns a named vector indicating the positions of (first) matches similar to \code{\link[base]{match}}.
}
\details{
The main motivation to create this function was to be able to treat concatenated entries and to look if \code{any} of the concatenated values match to 'x'.
This function offers additional options for trimming values before running the main comparison.

Of course, the concatenation strategy must be known and only a single concatenation separator (which may be multiple characters long) may be used for both \code{x} and \code{match}.
Thus result will only indicate that at least one of the concatenated terms had a match, but not which one.
Finally, both vectors \code{x} and \code{table} may contain concatenated terms.
In this case this function will require much more computational ressources due to the increased combinatorics when comparing larger vectors.

Please note, that in case of multiple to multiple matches, only the first hit gets reported.

The argument \code{globalPat="digitExtension"} allows eg reducing 'A1234-4' to 'A1234'.
}
\examples{
tab1 <- c("AA","BB-5","CCab","FF")
tab2 <- c("AA","WW,Vde,BB-5,E","CCab","FF,Uef")
x1 <- c("ZZ","YY","AA","BB-2","DD","CCdef","Dxy")            # modif of single ID (no concat)
concatMatch(x1, tab2)
x2 <- c("ZZ,Z","YY,Y","AA,Z,Y","BB-2","DD","X,CCdef","Dxy")  # conatenated in 'x'
concatMatch(x2, tab2)
tab1 <- c("AA","BB-5","CCab","FF")              # no conatenated in 'table'
concatMatch(x2, tab1)                          # simple case of no concat anywhere
concatMatch(x1, tab1)
}
\seealso{
\code{\link[base]{match}} (for two simple vectors without concatenated terms), \code{\link[base]{grep}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getValuesByUnique.R
\name{getValuesByUnique}
\alias{getValuesByUnique}
\title{Print matrix-content as plot}
\usage{
getValuesByUnique(
  dat,
  datUniq = NULL,
  asIndex = FALSE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{dat}{(numeric or character) main long input, must have names}

\item{datUniq}{(numeric or character) will be used to impose values on \code{dat}, must have names that should match names (at least partially) from \code{dat}}

\item{asIndex}{(logical) if \code{TRUE} index values will be returned instead of replacing values}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
vector of length \code{dat} with imposed values, or index values if \code{asIndex=TRUE}
}
\description{
When data have repeated elements (defined by names inside the vector), it may be advantageous to run some operations 
only on a unique set of the initial data, or somtimes all repeated occurances need to be replaced by a common (summarizing) value.
This function allows to re-introduce new values from on second vector with unique names, to return a final vector of initial input-length and order of names (elements) like initial, too.
Normally the user would provide 'datUniq' (without repeated names) containing new values which will be expanded to structure of 'dat', 
if 'datUniq' is not provided a vector with unique names will be made using the first occurance of repeated value(s).
For more complex cases the indexing relative to 'datUniq' can be returned (setting \code{asIndex=TRUE}).
Note: If not all names of 'dat' are found in 'datUniq' the missing spots will be returned as \code{NA}.
}
\examples{
dat <- 11:19
names(dat) <- letters[c(6:3,2:4,8,3)]
## let's make a 'datUniq' with the mean of repeated values :
datUniq <- round(tapply(dat,names(dat),mean),1)
## now propagate the mean values to the full vector
getValuesByUnique(dat,datUniq)
cbind(ini=dat,firstOfRep=getValuesByUnique(dat,datUniq),
  indexUniq=getValuesByUnique(dat,datUniq,asIn=TRUE))
}
\seealso{
\code{\link[base]{unique}}, \code{\link{findRepeated}}, \code{\link{correctToUnique}}, \code{\link{treatTxtDuplicates}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matchMatrixLinesToRef.R
\name{matchMatrixLinesToRef}
\alias{matchMatrixLinesToRef}
\title{Match All Lines of Matrix To Reference Note}
\usage{
matchMatrixLinesToRef(
  mat,
  ref,
  exclCol = NULL,
  addRef = TRUE,
  inclInfo = FALSE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{mat}{(matrix or data.frame) main input, all columns of \code{mat} will be tested for (partial) matching of \code{ref}}

\item{ref}{(character, length must match ) reference for trying to match each of the columns of  \code{mat}}

\item{exclCol}{(character or integer) column-name or -index of column to ignore/exclude when looking for matches}

\item{addRef}{(logical), if \code{TRUE} the content of \code{ref} will be added to  \code{mat} as additional column}

\item{inclInfo}{(logical) allows returning list with new matrix and additional information}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allows easier tracking of messages produced}
}
\value{
This function returns the input matrix in an adjusted order (plus an optional additional column showing the reference)
 or if \code{inclInfo=TRUE} a list with $mat (adjusted matrix), $byColumn, $newOrder and $method;
 the reference can bee added as additional last column if \code{addRef=TRUE}
}
\description{
This function allows adjusting the order of lines of a matrix \code{mat} to a reference character-vector \code{ref},
even when initial direct matching of character-strings using \code{match} is not possible/successful.
In this case, various variants of using \code{grep} will be used to see if unambiguous matching is possible of characteristic parts of the text.
All columns of \code{mat} will be tested an the column giving the bes resuts will be used.
}
\details{
This function tests all columns of \code{mat} to find perfect matching results to the reference \code{ref}.
In case of multiple results the
In case no direct matching is possible, \code{grep} will be used to find the best partial matching.
The orderof the rows of input \code{mat} will be adjusted according to the matching results.

If \code{addRef=TRUE}, the reference will be included as additional column to the results, too.
}
\examples{
## Note : columns b and e allow non-ambigous match, not all elements of e are present in a
mat0 <- cbind(a=c("mvvk","axxd","bxxd","vv"),b=c("iwwy","iyyu","kvvh","gxx"), c=rep(9,4),
  d=c("hgf","hgf","vxc","nvnn"), e=c("_vv_","_ww_","_xx_","_yy_"))
matchMatrixLinesToRef(mat0[,1:4], ref=mat0[,5])
matchMatrixLinesToRef(mat0[,1:4], ref=mat0[1:3,5], inclInfo=TRUE)

matchMatrixLinesToRef(mat0[,-2], ref=mat0[,2], inclInfo=TRUE)   # needs 'reverse grep'
}
\seealso{
\code{\link[base]{match}},  \code{\link[base]{grep}}, \code{\link{trimRedundText}}, \code{\link{replicateStructure}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getPairwiseSetup.R
\name{getPairwiseSetup}
\alias{getPairwiseSetup}
\title{Extract Pairwise Testing Setup}
\usage{
getPairwiseSetup(
  pwGrpNa,
  grpNa = NULL,
  sep = NULL,
  combPwSep = "combine1",
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{pwGrpNa}{(character) the names of pairwise-testing (ie 'concatenated' \code{pwGrpNa} using the separator \code{sep} )}

\item{grpNa}{(character) the names of the groups of replicates (ie conditions) used to test}

\item{sep}{(NULL or character, length=1) if not \code{NULL} the character-vector (of length=1) given will be used to split \code{grpNa} using \code{stringsplit()}}

\item{combPwSep}{(character) sequence of separators for building combined names; may be combPwSep='combine1' or 'combine2'; will be used in pwSeparatorList()}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a list with $ind with matrix of indexes to (sorted) \code{pwGrpNa} for each pairwise comparison; 
  $pwGrpNames with matrix of \code{pwGrpNa} for each pairwise comparison;  
  $sep for the separator found ; $pwGrpNa which recapitulates (sorted) \code{pwGrpNa} as factor
}
\description{
This function allows extacting, checking and combinig elements defining labels for pairwise comparisons.
}
\details{
This function was designed to support functions exploiting pairwise testing results like plotting functions for Volcano-Plots etc.

Initially the function will check if the object given as argument \code{pwGrpNa} is an MAarrayLM-objects object and/or was created using \code{moderTestXgrp}. 
If yes, the function will try to extract names of groups, pairwise combinations and separator used from $setup, if present.
If no $setup found this function will check through several elements in MAarrayLM-objects to gather all elements needed.

2) The separator \code{sep} is given and exact matches at both sides will be searched.
However, if the character(s) from \code{sep} do appear inside \code{pwGrpNa} no matches will be found.

If some \code{pwGrpNa} are not found in \code{grpNa} this will be marked as NA.  

Finally, if no testing result is given as input, basic parameters group-names, pairwise names and the separator to be used can be combined into a list.
}
\examples{
grp3 <- factor(rep(LETTERS[c(3,1,4)],c(2,3,3)))
set.seed(2017); t8 <- matrix(round(rnorm(208*8,10,0.4),2), ncol=8,
  dimnames=list(paste(letters[],rep(1:8,each=26),sep=""), paste0(grp3, c(1:2,1:3,1:3))))
test1 <- moderTestXgrp(t8, grp3)  
getPairwiseSetup(test1)



}
\seealso{
(for running multiple pair-wise test) \code{\link{moderTestXgrp}}, \code{\link{convPairwiseSetup}}, \code{\link{presenceFilt}}, \code{\link{getPWseparator}}, \code{\link{indexGroupsFromPW}}, 
  (used underneith:) \code{\link[base]{grep}}, \code{\link[base]{strsplit}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkVectLength.R
\name{.vector2Matr}
\alias{.vector2Matr}
\title{Convert numeric vector to matrix}
\usage{
.vector2Matr(x, colNa = NULL, rowsKeep = TRUE)
}
\arguments{
\item{x}{(numeric or character) main input}

\item{colNa}{(integer) design the comumn-name to be given}

\item{rowsKeep}{(logical) is \code{TRUE} make matrix of 1 column, otherwise of 1 row}
}
\value{
matrix
}
\description{
Take (numeric) vector and return matrix, if 'colNa' given will be used as colname
}
\examples{
.vector2Matr(c(3:6))
}
\seealso{
\code{\link[base]{matrix}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/countSameStartEnd.R
\name{countSameStartEnd}
\alias{countSameStartEnd}
\title{Count Same Start- And End- Sites Of Edges (Or Fragments)}
\usage{
countSameStartEnd(
  frag,
  minFreq = 2,
  nDig = 4,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{frag}{(matrix) 1st column \code{beg} start-sites, 2nd column \code{end} end-sites of edges, rownames to precise fragment identities are recommended}

\item{minFreq}{(integer) min number of accumulated sites for taking into account (allows filtering with large datasets)}

\item{nDig}{(integer) rounding: number of digits for columns \code{beg.rat} and \code{end.rat} in output}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allows easier tracking of messages produced#' @return This function returns a matrix of 6 columns: input (beg and end), beg.n, beg.rat, end.n, end.rat}
}
\description{
Suppose a parent sequence/string 'ABCDE' gets cut in various fragments (eg 'ABC','AB' ...).   
\code{countSameStartEnd} counts how many (ie re-occuring) start- and end- sites of edges do occur in the input-data. 
The input is presented as matrix of/indicating start- and end-sites of edges.
The function is used to characterize partially redundant edges and accumulation of cutting/breakage sites.
}
\examples{
frag1 <- cbind(beg=c(2,3,7,13,13,15,7,9,7, 3,3,5), end=c(6,12,8,18,20,20,19,12,12, 4,5,7))
rownames(frag1) <- letters[1:nrow(frag1)]
countSameStartEnd(frag1)
simpleFragFig(frag1)
}
\seealso{
to build initial tree \code{\link{buildTree}}, \code{\link{contribToContigPerFrag}}, \code{\link{simpleFragFig}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mergeMatrices.R
\name{mergeMatrices}
\alias{mergeMatrices}
\title{Merge Multiple Matrices}
\usage{
mergeMatrices(
  ...,
  mode = "intersect",
  useColumn = 1,
  na.rm = TRUE,
  extrRowNames = FALSE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{...}{(matrix or data.frame) multiple matrix or data.frame objects may be entered}

\item{mode}{(character) allows choosing restricting to all common elements (\code{mode='intersect'}) or union  (\code{mode='union'})}

\item{useColumn}{(integer, character or list) the column(s) to consider, may be \code{'all'} to use all, integer to select specific indexes or list of indexes or colnames for cutom-selection per matrix}

\item{na.rm}{(logical) suppress \code{NA}s}

\item{extrRowNames}{(logical) decide whether columns with all values different (ie no replicates or max divergency) should be excluded}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a matrix containing all selected columns of the input matrices to fuse
}
\description{
This function allows merging of multiple matrix-like objects.
The matix-rownames will be used to align common elements, either be returning all common elements \code{mode='intersect'} or containg all elements \code{mode='union'} (the result may contains additional \code{NA}s).
}
\details{
Custom column-names can be given by entering matrices like named arguments (see examples below).
The choice of columns tu use may be adopted to each matrix entered, in this case the argument \code{useColumn} may be a list with matrix-names to use or a list of indexes (see examples below).

Note, that matrices may contain repeated rownames (see examples, \code{mat3}). In this case only the first of repeated rownames will be considered (and lines of repeated names ignored).
}
\examples{
mat1 <- matrix(11:18, ncol=2, dimnames=list(letters[3:6],LETTERS[1:2]))
mat2 <- matrix(21:28, ncol=2, dimnames=list(letters[2:5],LETTERS[3:4]))
mat3 <- matrix(31:38, ncol=2, dimnames=list(letters[c(1,3:4,3)],LETTERS[4:5]))

mergeMatrices(mat1, mat2)
mergeMatrices(mat1, mat2, mat3, mode="union", useCol=2)
## custom names for matrix-origin
mergeMatrices(m1=mat1, m2=mat2, mat3, mode="union", useCol=2)
## flexible/custom selection of columns
mergeMatrices(m1=mat1, m2=mat2, mat3, mode="union", useCol=list(1,1:2,2))
}
\seealso{
\code{\link[base]{merge}},  \code{\link{mergeMatrixList}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reduceTable.R
\name{reduceTable}
\alias{reduceTable}
\title{Reduce table by aggregating smaller groups}
\usage{
reduceTable(tab, separFirst = 4, nGrp = 15)
}
\arguments{
\item{tab}{output of \code{\link[base]{table}}}

\item{separFirst}{(integer or NULL) optinal separartion of n 'separFirst' groups (value <2 or NULL 
will priviledge more uniform size of groups, higher values will cause small inital and larger tailing groups)}

\item{nGrp}{(integer) number of groups expected}
}
\value{
This function returns a numeric vector with number of counts and class-borders as names (like table).
}
\description{
\code{reduceTable} treats/reduces results from \code{\link[base]{table}} to 'nGrp' groups, 
optional indiv resolution of 'separFirst' (numeric or NULL). 
Mainly made for reducing the number of classes for betters plots with \code{\link[graphics]{pie}}
}
\examples{
set.seed(2018); dat <- sample(11:60,200,repl=TRUE)
pie(table(dat))
pie(reduceTable(table(dat), sep=NULL))
pie(reduceTable(table(dat), sep=NULL), init.angle=90,
  clockwise=TRUE, col=rainbow(20)[1:15], cex=0.8)
}
\seealso{
\code{\link[base]{table}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filterList.R
\name{filterList}
\alias{filterList}
\title{Filter for unique elements}
\usage{
filterList(lst, filt, minLineRatio = 0.5, silent = FALSE, callFrom = NULL)
}
\arguments{
\item{lst}{(list) main input, each vector, matrix or data.frame in this list will be filtered if its length or number of lines fits to \code{filt}}

\item{filt}{(logical) vector of \code{FALSE/TRUE} to use for filtering. If this a matrix is given, the value of \code{minLineRatio} will be applied as threshod of min content of \code{TRUE} for each line of \code{filt}}

\item{minLineRatio}{(numeric) in case \code{filt} is a matrix of \code{FALSE/TRUE}, this value will be used as threshold of min content of \code{TRUE} for each line of \code{filt}}

\item{silent}{(logical) suppress messages}

\item{callFrom}{(character) allow easier tracking of message(s) produced}
}
\value{
filtered list
}
\description{
This function aims to apply a given filter-citerium, a matrix or vector of \code{FALSE/TRUE} which is typically combined with a second layer   
which filters for a min content of filer-passing values per line for the first/main criterium.
Then all lines concerned will be removed. This will be done for all list-elements (of appropriate size) of the input-list  
(while maintaining the list-structure in the output) not matching the filtering criteria.
}
\examples{
set.seed(2020); dat1 <- round(runif(80),2)
list1 <- list(m1=matrix(dat1[1:40],ncol=8), m2=matrix(dat1[41:80],ncol=8), other=letters[1:8])
rownames(list1$m1) <- rownames(list1$m2) <- paste0("line",1:5)
filterList(list1, list1$m1[,1] >0.4) 
filterList(list1, list1$m1 >0.4) 
}
\seealso{
\code{\link{correctToUnique}}, \code{\link[base]{unique}}, \code{\link[base]{duplicated}}, \code{\link{extrColsDeX}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makeNRedMatr.R
\name{makeNRedMatr}
\alias{makeNRedMatr}
\title{Make non-redundant matrix}
\usage{
makeNRedMatr(
  dat,
  summarizeRedAs,
  iniID = "iniID",
  retDataFrame = TRUE,
  nEqu = FALSE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{dat}{(matrix or data.frame) main input for making non-redundant}

\item{summarizeRedAs}{(character) summarization method(s), typical choices 'median','mean','min' or 'maxOfRef';
basic usage like \code{summarizeRedAs='mean'} will pick independently the mean for each (numeric) column; 
it is also possible to specify different methods for each of columnw (length of \code{summarizeRedAs} should be equal number of numeric columns);
special methods look at a single reference column to decide which line should be picked and their values reported (not compatible with specifying different methods for different columns),}

\item{iniID}{(character) column-name used as reference for determining groups of redundant lines (default="iniID")}

\item{retDataFrame}{(logical) if \code{TRUE}, check if text-columns may be converted to data.frame with numeric}

\item{nEqu}{(logical) if \code{TRUE}, add additional column indicating the number of equal lines for choice (only with min or max)}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allows easier tracking of messages produced}
}
\value{
This function returns a (numeric) matrix or data.frame with summarized data and add'l col with number of initial redundant lines
}
\description{
This function takes matrix or data.frame 'dat' to summarize redundant lines (column argument \code{iniID}) along method specified in \code{summarizeRedAs} 
to treat all lines with redundant \code{iniID} by same approach (ie for all columns the line where specified column is at eg max = 'maxOfRef' ).
If no name given, the function will take the last numeric (factors may be used - they will be read as levels).
}
\details{
When using for selection of single initial line give the character-string of argument \code{summarizeRedAs} a name (eg  \code{summ=c(X1="minOfRef")}
 so that the function will use ONLY the column specified via the name for determining which line should be used/kept.    

It is possible to base the choice from 'redundant' lines on a single reference-column.  
For example, when \code{summarizeRedAs='maxOfRef'} summarizing of all (numeric) columns will be performed according to one single column (ie the line where the last numeric column is at its max).
Otherwiser, a name can be assigned as reference column to be used (eg see last example using \code{summarizeRedAs=c(x1='maxOfRef')})
}
\examples{
t3 <- data.frame(ref=rep(11:15,3),tx=letters[1:15],
  matrix(round(runif(30,-3,2),1),nc=2),stringsAsFactors=FALSE)
by(t3,t3[,1],function(x) x)
t(sapply(by(t3,t3[,1],function(x) x), summarizeCols, me="maxAbsOfRef"))
# calculate mean for lines concerened of all columns :
(xt3 <- makeNRedMatr(t3, summ="mean", iniID="ref"))
# choose lines based only on content of column 'X1' (here: max):
(xt3 <- makeNRedMatr(t3, summ=c(X1="maxOfRef"), iniID="ref")) 
}
\seealso{
simple/partial functionality in \code{\link{summarizeCols}},  \code{\link{checkSimValueInSer}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/htmlSpecCharConv.R
\name{htmlSpecCharConv}
\alias{htmlSpecCharConv}
\title{Html Special Character Conversion}
\usage{
htmlSpecCharConv(txt, silent = FALSE, callFrom = NULL, debug = FALSE)
}
\arguments{
\item{txt}{character vector, including special characters}

\item{silent}{(logical) suppress messages}

\item{callFrom}{(character) allow easier tracking of messages produced}

\item{debug}{(logical) additional messages for debugging}
}
\value{
This function returns a corrected character vector adopted for html display
}
\description{
Converts 'txt' so that (the most common) special characters (like 'beta','micro','square' etc) will be displayed correctly whe used for display in html (eg at mouse-over).
Note : The package \href{https://CRAN.R-project.org/package=stringi}{stringi} is required for the conversions (the input will get returned if \code{stringi} is not available).
Currently only the 16 most common special characters are implemented.
}
\examples{
## we'll use the package stringi to generate text including the 'micro'-symbol as input
x <- if(requireNamespace("stringi", quietly=TRUE)) {
  stringi::stri_unescape_unicode("\\\\u00b5\\\\u003d\\\\u0061\\\\u0062")} else "\"x=axb\""
htmlSpecCharConv(x)
}
\seealso{
tables on \url{https://www.htmlhelp.com/reference/html40/entities/latin1.html},  
  \url{https://www.degraeve.com/reference/specialcharacters.php}, or \code{https://ascii.cl/htmlcodes.htm}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/withinRefRange.R
\name{withinRefRange}
\alias{withinRefRange}
\title{Check For Values Within Range Of Reference}
\usage{
withinRefRange(
  x,
  fa,
  ref = NULL,
  absRef = TRUE,
  asInd = FALSE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{x}{matrix or data.frame}

\item{fa}{(numeric) absolute or relative tolerance value (numeric, length=1), interpreted according to 'absRef' as absolute or relative to 'x'(ie fa*ref)}

\item{ref}{(numeric) (center) reference value for comparison (numeric, length=1), if not given mean of 'x' (excluding NA or non-finite values) will be used}

\item{absRef}{(logical) return result as absolute or relative to 'x'(ie fa*ref)}

\item{asInd}{(logical) if TRUE return index of which values of 'x' are within range, otherwise return values if 'x' within range}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allows easier tracking of messages produced}
}
\value{
This function returns a numeric vector containing only the values within range of reference
}
\description{
This function checks which values of numeric vector 'x' are within range +/- 'fa' x 'ref' (ie within range of reference).
}
\examples{
## within 2.5 +/- 0.7
withinRefRange(-5:6,fa=0.7,ref=2.5)                
## within 2.5 +/- (0.7*2.5)
withinRefRange(-5:6,fa=0.7,ref=2.5,absRef=FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/normalizeThis.R
\name{.normalize}
\alias{.normalize}
\title{Main Normalization function}
\usage{
.normalize(
  dat,
  meth,
  mode,
  param,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{dat}{matrix or data.frame of data to get normalized}

\item{meth}{(character) may be "mean","median","NULL","none", "standardize","trimMean", "rowNormalize", "slope", "exponent", "twoPointSlope", "vsn"; When \code{NULL} or 'none' is chosen the input will be returned}

\item{mode}{(character) may be "proportional", "additive";
decide if normalizatio factors will be applies as multiplicative (proportional) or additive; for log2-omics data \code{mode="additive"} is suggested}

\item{param}{(list) additional parameters}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allows easier tracking of messages produced}
}
\value{
This function returns a numeric vector
}
\description{
This function aims to normalize a matrix or data.frame by columns. 
It assumes all checks have been done before calling this function.
}
\examples{
aa <- matrix(1:12, ncol=3)
.normalize(aa,"median",mode="proportional",param=NULL)
}
\seealso{
\code{\link{normalizeThis}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/asSepList.R
\name{.asDF2}
\alias{.asDF2}
\title{Convert anything to data.frame}
\usage{
.asDF2(z)
}
\arguments{
\item{z}{(numeric vector, factor, matrix or list) main input}
}
\value{
data.frame
}
\description{
This function allows converting anything to data.frame
}
\examples{
.asDF2(c(3:6))
}
\seealso{
\code{\link[base]{as.data.frame}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/linModelSelect.R
\name{linModelSelect}
\alias{linModelSelect}
\title{Test Multiple Starting Levels For Linear Regression Model, Select Best And Plot}
\usage{
linModelSelect(
  rowNa,
  dat,
  expect,
  logExpect = FALSE,
  startLev = NULL,
  lisNa = c(raw = "raw", annot = "annot", datImp = "datImp"),
  plotGraph = TRUE,
  tit = NULL,
  pch = c(1, 3),
  cexLeg = 0.95,
  cexSub = 0.85,
  xLab = NULL,
  yLab = NULL,
  cexXAxis = 0.85,
  cexYAxis = 0.9,
  xLabLas = 1,
  cexLab = 1.1,
  suppressWarnings = TRUE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{rowNa}{(character or numeric, length=1) if \code{character} rowname for line to be extracted from \code{dat}, if  \code{numeric} used as line-number of \code{dat}}

\item{dat}{(matrix, list or MArrayLM-object from limma) main input of which columns should get re-ordered, may be output from \code{\link{moderTestXgrp}} or \code{\link{moderTest2grp}}.}

\item{expect}{(numeric of character) the expected levels; if character, constant unit-characters will be stripped away to extact the numeric content}

\item{logExpect}{(logical) toggle to \code{TRUE} if the main data are logarithmic but \code{expect} is linear}

\item{startLev}{(integer) specify all starting levels to test for omitting here (multiple start sites for modelling linear regression may be specified to finally pick the best model)}

\item{lisNa}{(character) in case \code{dat} is list or MArrayLM-type object, the list-elements with these names will be used as $raw (for indicating initial \code{NA}-values,
$datImp (the main quantitation data to use) and $annot for displaying the corresponding value from the "Accession"-column.}

\item{plotGraph}{(logical) display figure}

\item{tit}{(character) optional custom title}

\item{pch}{(integer) symbols to use n optional plot; 1st for regular values, 2nd for values not used in regression}

\item{cexLeg}{(numeric) size of text in legend}

\item{cexSub}{(numeric) text-size for line (as subtitle) giving regression details of best linear model)}

\item{xLab}{(character) custom x-axis label}

\item{yLab}{(character) custom y-axis label}

\item{cexXAxis}{(character) \code{cex}-type for size of text for x-axis labels}

\item{cexYAxis}{(character) \code{cex}-type for size of text for y-axis labels}

\item{xLabLas}{(integer) \code{las}-type orientation of x-axis labels (set to 2 for vertical axix-labels)}

\item{cexLab}{(numeric) \code{cex}-type for size of text in x & y axis labels (will be passed to \code{cex.lab} in \code{plot()})}

\item{suppressWarnings}{(logical) suppress warnings from lm() that may appear when NAs get introduced}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a list with $coef (coefficients), $name (as/from input \code{rowNa}), $startLev the best starting level)
}
\description{
The aim of this function is to select the data suiting set of levels on the main input data to construct a linear regression model.
}
\details{
In real world measurements one may be confronted to the case of very low level analytes below the detection limit (LOD) and resulting read-outs fluctuate around around a common baseline (instead of \code{NA}). 
With such data it may be preferable to omit the read-outs for the lowest concentrations/levels of analytes if they are spread around a base-line value.
This function allows trying to omit all starting levels designed in \code{startLev}, then the resulting p-values for the linear regression slopes will be checked and the best p-value chosen. 
The input may also be a MArrayLM-type object from package \href{https://bioconductor.org/packages/release/bioc/html/limma.html}{limma} or from \code{\link{moderTestXgrp}} or \code{\link{moderTest2grp}}.
In the graphical representation all points assocoated to levels omitted are shown in light green.
For the graphical display additional information can be used : If the  \code{dat} is list or MArrayLM-type object, 
 the list-elements $raw (according to argument \code{lisNa} will be used to display points initially given as NA ad imputed lateron in grey.
Logarithmic (ie log-linear) data can be treated by settting argument \code{logExpect=TRUE}. Then the levels will be taken as exponent of 2 for the regression, while the original values will be displayed in the figure.
}
\examples{
## Construct data
li1 <- rep(c(4,3,3:6),each=3) + round(runif(18)/5,2)
names(li1) <- paste0(rep(letters[1:5], each=3), rep(1:3,6))
li2 <- rep(c(6,3:7), each=3) + round(runif(18)/5, 2)
dat2 <- rbind(P1=li1, P2=li2)
exp2 <- rep(c(11:16), each=3)

## Check & plot for linear model 
linModelSelect("P2", dat2, expect=exp2)

## Log-Linear data
## Suppose dat2 is result of measures in log2, but exp4 is not
exp4 <- rep(c(3,10,30,100,300,1000), each=3)
linModelSelect("P2", dat2, expect=exp4, logE=FALSE)    # bad
linModelSelect("P2", dat2, expect=exp4, logE=TRUE)

}
\seealso{
\code{\link{moderTestXgrp}} for single comparisons, \code{\link[base]{order}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/test2factLimma.R
\name{test2factLimma}
\alias{test2factLimma}
\title{2-Factorial Limma-Style t-Test}
\usage{
test2factLimma(
  datMatr,
  fac1,
  fac2,
  testSynerg = FALSE,
  testOrientation = "=",
  addResults = c("lfdr", "FDR", "Mval", "means"),
  addGenes = NULL,
  silent = FALSE,
  callFrom = NULL,
  debug = FALSE
)
}
\arguments{
\item{datMatr}{matrix or data.frame with lines as indenpendent series of measures (eg different genes)}

\item{fac1}{(character or factor) vector describing grouping elements of each line of 'datMatr' for first factor, must be of same langth as fac2}

\item{fac2}{(character or factor) vector describing grouping elements of each line of 'datMatr' for second factor, must be of same langth as fac1}

\item{testSynerg}{(logical) decide if factor-interactions (eg synergy) should be tested in model, otherwise additive factors are supposed}

\item{testOrientation}{(character) default (or any non-recignized input) '=', otherwise either '>','gerater','sup','upper' or '<','inf','lower'}

\item{addResults}{(character) vector defining which types of information should be included to output, may be 'lfdr','FDR' (for BY correction), 'Mval' (M values), 'means' (matrix with mean values for each group of replicates)}

\item{addGenes}{(matrix or data.frame) additional information to add to output}

\item{silent}{(logical) suppress messages}

\item{callFrom}{(character) allow easier tracking of messages produced}

\item{debug}{(logical) additional messages for debugging}
}
\value{
This function returns an object of class "MArrayLM" (from limma) containing/enriched by the testing results
}
\description{
The aim of this function is to provide convenient acces to two-factorial (linear) testing withing the framework of \code{\link{makeMAList}} including the emprical Bayes shrinkage. 
The input data 'datMatr' which should already be organized as limma-type MAList, eg using using \code{\link{makeMAList}}. 
Note: This function uses the Bioconductor package \href{https://bioconductor.org/packages/release/bioc/html/limma.html}{limma} (which must be installed).
}
\examples{
set.seed(2014)
dat0 <- rnorm(30) + rep(c(10,15,19,20),c(9,8,7,6))
fa <- factor(rep(letters[1:4],c(9,8,7,6)))
dat2 <- data.frame(facA=rep(c("-","A","-","A"), c(9,8,7,6)),
  facB= rep(c("-","-","B","B"), c(9,8,7,6)), dat1=dat0, dat2=runif(30))
grpNa <- sub("-","",sub("\\\\.","", apply(dat2[,1:2], 1, paste, collapse="")))    
test2f <- test2factLimma(t(dat2[,3:4]), dat2$facA, dat2$facB)  
test2f     # just the p-values
# Similarly, you can easily summarize results using topTable from limma
if(requireNamespace("limma", quietly=TRUE)) {
  test2g <- test2factLimma(t(dat2[,3:4]), dat2$facA, dat2$facB, addR=FALSE)  
  library(limma)
  topTable(test2g, coef=1, n=5) 
  topTable(test2g, coef=2, n=5) } 
}
\seealso{
\code{\link{makeMAList}}, single line testing \code{\link[limma]{lmFit}} and the \code{eBayes}-family of functions in package \href{https://bioconductor.org/packages/release/bioc/html/limma.html}{limma}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cutAtMultSites.R
\name{.addLetterWoLast}
\alias{.addLetterWoLast}
\title{Add letter to all elements but not last}
\usage{
.addLetterWoLast(x, addChr)
}
\arguments{
\item{x}{(character) main input}

\item{addChr}{(character)}
}
\value{
This function returns a modified character vector
}
\description{
This function allows to add 'addChr' to all entries, without the last entry
}
\examples{
.addLetterWoLast(c("abc","efgh"),"Z")
}
\seealso{
\code{\link[base]{paste}}; used in \code{\link{cutAtMultSites}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get1stOfRepeatedByCol.R
\name{get1stOfRepeatedByCol}
\alias{get1stOfRepeatedByCol}
\title{Get First Of Repeated By Column}
\usage{
get1stOfRepeatedByCol(
  mat,
  sortBy = "seq",
  sortSupl = "ty",
  asFirstLast = c("full", "inter"),
  markIfAmbig = c("ambig", "seqNa"),
  asList = FALSE,
  abmiPref = "_",
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{mat}{(matrix or data.frame) numeric vector to be tested}

\item{sortBy}{(character) column name for which elements should be made unique, numeric or character column; 'sortSupl' .. add'l colname to always select specific 1st)}

\item{sortSupl}{(character) default="ty"}

\item{asFirstLast}{(character,length=2) to force specific strings from coluln 'sortSupl' as first and last when selecting 1st of repeated terms, default=c("full","inter")}

\item{markIfAmbig}{(character,length=2) 1st will be set to 'TRUE' if ambiguous/repeated, 2nd will get (heading) prefix, default=c("ambig","seqNa")}

\item{asList}{(logical) to return list with non-redundant ('unique') and removed lines ('repeats')}

\item{abmiPref}{(character) prefix to note ambiguous entries/terms, default="_"}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns depending on argumnet 'asList' either list with non-redundant ('unique') and removed lines ('repeats')
}
\description{
\code{get1stOfRepeatedByCol} sorts matrix 'mat' and extracts only 1st occurance of values in column 'sortBy'.
Returns then non-redundant matrix (ie for column 'sortBy', if 'markIfAmbig' specifies existing col, mark ambig there).
Note : problem when sortSupl or sortBy not present (or not intended for use)
}
\examples{
aa <- cbind(no=as.character(1:20),seq=sample(LETTERS[1:15],20,repl=TRUE),
  ty=sample(c("full","Nter","inter"),20,repl=TRUE),ambig=rep(NA,20),seqNa=1:20)
get1stOfRepeatedByCol(aa)
}
\seealso{
\code{\link{firstOfRepeated}} for (more basic) treatment of simple vector, \code{\link{nonAmbiguousNum}} for numeric use (much faster !!!)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/listBatchReplace.R
\name{listBatchReplace}
\alias{listBatchReplace}
\title{Replacements in list}
\usage{
listBatchReplace(
  lst,
  searchValue,
  replaceBy,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{lst}{input-list to be used for replacing}

\item{searchValue}{(character, length=1)}

\item{replaceBy}{(character, length=1)}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a corrected list
}
\description{
\code{listBatchReplace} replaces in list \code{lst} all entries with value \code{searchValue} by \code{replaceBy}
}
\examples{
lst1 <- list(aa=1:4, bb=c("abc","efg","abhh","effge"), cc=c("abdc","efg"))
listBatchReplace(lst1, search="efg", repl="EFG", sil=FALSE)
}
\seealso{
basic replacement \code{sub} in \code{\link[base]{grep}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/regrBy1or2point.R
\name{regrBy1or2point}
\alias{regrBy1or2point}
\title{Rescaling According To Reference Data Using Linear Regression.}
\usage{
regrBy1or2point(
  inDat,
  refLst,
  regrTo = c(1, 0.5),
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{inDat}{(matrix or data.frame) data to rescaled}

\item{refLst}{list of names existing in inDat (one group of names for each value in 'regrTo'), to be transformed in values precised in 'regTo'; if no matches to names of 'inDat' found, the 2 lowest and/or highest highest values will be chosen}

\item{regrTo}{(numeric,length=2) range (at scale 0-1) of target-values for mean of elements cited in 'refLst'}

\item{silent}{(logical) suppress messages if \code{TRUE}}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allows easier tracking of messages produced}
}
\value{
This function returns a normalized matrix
}
\description{
This function does rescaling: linear transform simple vector 'inDat' that (mean of) elements of names cited in 'refLst' will end up as values 'regrTo'. 
Regress single vector according to 'refLst' (describing names of inDat).
If 'refLst' contains 2 groups, the 1st group will be set to the 1st value of 'regrTo' (and the 2nd group of 'refLst' to the 2nd 'regtTo')
}
\examples{
set.seed(2016); dat1 <- 1:50 +(1:50)*round(runif(50),1)
names(dat1) <- 1:length(dat1)
reg1 <- regrBy1or2point(dat1, refLst=c("2","49"))
plot(reg1, dat1) 
}
\seealso{
\code{\link{adjBy2ptReg}}, \code{\link{regrMultBy1or2point}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/addBeforFileExtension.R
\name{addBeforFileExtension}
\alias{addBeforFileExtension}
\title{Add text before file-extension}
\usage{
addBeforFileExtension(
  x,
  add,
  sep = "_",
  silent = FALSE,
  callFrom = NULL,
  debug = FALSE
)
}
\arguments{
\item{x}{main character vector}

\item{add}{character vector to be added}

\item{sep}{(character) separator between 'x' & 'add' (character, length 1)}

\item{silent}{(logical) suppress messages}

\item{callFrom}{(character) allow easier tracking of messages produced}

\item{debug}{(logical) additional messages for debugging}
}
\value{
modified character vector
}
\description{
This function helps changing charater srings like file-names and allows adding the character vector 'add'
(length 1) before the extension (defined by last '.') of the input string 'x'. 
Used for easily creating variants/additional filenames but keeping current extension.
}
\examples{
addBeforFileExtension(c("abd.txt","ghg.ijij.txt","kjh"),"new")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rowNormalize.R
\name{.rowNormFact}
\alias{.rowNormFact}
\title{Obtain Normalization Factor (main)}
\usage{
.rowNormFact(
  dat,
  combOfN,
  comUse,
  method = "median",
  refLi = NULL,
  refGrp = NULL,
  proportMode = TRUE,
  minQuant = NULL,
  maxFact = 10,
  omitNonAlignable = FALSE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{dat}{(matrix) .. init data, smay be parse matrix with numerous NA}

\item{combOfN}{(matrix) ..  # matrix of index for all sub-groups (assumed as sorted)}

\item{comUse}{(list) .. index of complete lines for each col of combOfN}

\item{method}{(character) may be "mean","median" (plus "NULL","none"); When NULL or 'none' is chosen the input will be returned as is}

\item{refLi}{(NULL or numeric) allows to consider only specific lines of 'dat' when determining normalization factors (all data will be normalized)}

\item{refGrp}{(integer) Only the columns indicated will be used as reference, default all columns (integer or colnames)}

\item{proportMode}{(logical) decide if normalization should be done by multiplicative or additive factor}

\item{minQuant}{(numeric) optional filter to set all values below given value as \code{NA}}

\item{maxFact}{(numeric, length=2) max normalization factor}

\item{omitNonAlignable}{(logical) allow omitting all columns which can't get aligned due to sparseness}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) This function allows easier tracking of messages produced}
}
\value{
This function returns a matrix of column-indexes complementing (nCombin rows)
}
\description{
This function was designed to obtain normalization factors.
}
\examples{
ma1 <- matrix(11:41, ncol=3)
}
\seealso{
\code{\link{rowNormalize}}; \code{\link{normalizeThis}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/partialDist.R
\name{.raiseColLowest}
\alias{.raiseColLowest}
\title{Raise all values close to lowest value}
\usage{
.raiseColLowest(
  mat,
  raiseTo = NULL,
  minFa = 0.1,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{mat}{(matrix of numeric values) main input}

\item{raiseTo}{(numeric)}

\item{minFa}{(numeric) minimum factor}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) display additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of message(s) produced}
}
\value{
This function returns a numeric vector with  numer of non-numeric characters (ie not '.' or 0-9))
}
\description{
This function aims to raise all values close to lowest value to end up as at value of 'raiseTo'.
This is done independently for each col of mat.
This function sets all data to common raiseTo (which is min among all cols)
}
\examples{
aa <- 11:15
}
\seealso{
\code{\link[base]{nchar}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rmEnumeratorName.R
\name{rmEnumeratorName}
\alias{rmEnumeratorName}
\title{Remove or rename enumerator tag/name (or remove entire enumerator) from tailing enumerators}
\usage{
rmEnumeratorName(
  dat,
  nameEnum = c("Number", "No", "#", "Replicate", "Sample"),
  sepEnum = c(" ", "-", "_", "/"),
  newSep = "",
  incl = c("anyCase", "trim2"),
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{dat}{(character vecor or matrix) main input}

\item{nameEnum}{(character) potential enumerator-names}

\item{sepEnum}{(character)  potential separators for enumerator-names}

\item{newSep}{(character) potential enumerator-names}

\item{incl}{(character) options to include further variants of the enumerator-names, 
use \code{"rmEnum"} for completely removing enumerator tag/name and digits for different options of trimming names/tags from \code{nameEnum};
or one may use \code{anyCase}, 
\code{trim3} (trimming down to max 3 letters),
\code{trim2} (trimming to max 2 letters) or  \code{trim1} (trimming down to single letter); 
\code{trim0} works like \code{trim1} but also includes ' ', ie no enumerator tag/name in front of the digit(s)}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) display additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a corrected vector (or matrix), or a list if \code{incl="rmEnumL"} containing $dat (corrected data),
  $pattern (the combination of separator-symbols and separtor text/words found), and if input is matrix $column (which column of the input was identified and treated)
}
\description{
This function allows indentifying, removing or renaming enumerator tag/name (or remove entire enumerator) from tailing enumerators (eg 'abc_No1' to 'abc_1').
A panel of potential candidates as combination of separator-symbols and separtor text/words will be tested to find if one matches all data.
In case the main input is a matrix, all columns will be tested independently to find the first column where one specific combination of separator-symbols and separtor text/words is found.
Several options exist for the output, the combination of separator-symbols and separtor text/words may be included, too.
}
\details{
In case only digit-enumerators are present (ie, without repetitive text), one has to use \code{incl="rmEnum"} to remove terminal enumerators. 
This will work, only when all items do contain terminal digits.

Please note, that checking a variety of different separator text-word and separator-symbols may give an important number of combinations to check.
In particular, when automatic trimming of separator text-words is added (eg \code{incl="trim2"}), the complexity of associated searches increases quickly.
Thus, with large data-sets restricting the content of the arguments \code{nameEnum}, \code{sepEnum} and (in particular) \code{newSep} to the most probable terms/options
is suggested to help reducing demands on memory and CPU.

In case the input \code{dat} is a matrix and multiple different numerator-types are found, only the first colum (from the left) will be treated.
If you which to remove/subsitute mutiple types of enumerators the function \code{rmEnumeratorName} must be run independently, see last example below.
}
\examples{
xv <- c("hg_1","hjRe2_2","hk-33")
rmEnumeratorName(xv)
rmEnumeratorName(xv, incl="rmEnum")

xx <- c("hg_Re1","hjRe2_Re2","hk-Re3_Re33")
rmEnumeratorName(xx)
rmEnumeratorName(xx, newSep="--")
rmEnumeratorName(xx, incl="anyCase")

xy <- cbind(a=11:13, b=c("11#11","2_No2","333_samp333"), c=xx)
rmEnumeratorName(xy)
rmEnumeratorName(xy,incl=c("anyCase","trim2","rmEnumL"))

xz <- cbind(a=11:13, b=c("23#11","4#2","567#333"), c=xx)
apply(xz, 2, rmEnumeratorName, sepEnum=c("","_"), newSep="_", silent=TRUE)

}
\seealso{
when the exact pattern is known \code{\link[base]{grep}} and \code{sub} may allow direct manipulations much faster
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trimmedMean.R
\name{trimmedMean}
\alias{trimmedMean}
\title{Trimmed Mean}
\usage{
trimmedMean(
  dat,
  trim = c(l = 0.2, u = 0.2),
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{dat}{(numeric) numeric vector}

\item{trim}{(numeric, length=2) specifies how data should get trimmed, lower and upper fraction(s) to exclude have to be assigned separately.
The lower and upper fraction may be named 'l' and 'u'.
The value 0 means that all (sorted) data on a given side will be used.}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allows easier tracking of messages produced}
}
\value{
This function returns a (numeric) vector with the trimmed mean
}
\description{
This function allows more flexible options for calculating a trimmed mean compared to \code{mean} (from the base-package).
For example, the trimming may be asymmetric to the median of the data.
}
\details{
If the second value of trim is <0.5 it is supposed that this indicates a fraction from the upper end the vector-dat as \code{\link[base]{mean}} does.
Otherwise, \code{trim=c(l=0.2,u=0.7)} will be interpreted indication to use the 20th percentile to 70th percentile of \code{dat}.

Please note, that trimmed means - and in particular asymmetric trimmed means - should be used with caution as there is also a risk of introducing bias.
}
\examples{
x <- c(17:11,27:28)
mean(x); mean(x, trim=0.15)
trimmedMean(x, trim=c(l=0, u=0.7))   # asymmetric trim

}
\seealso{
\code{\link[base]{mean}} (symmetric trimming only)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/replNAbyLow.R
\name{replNAbyLow}
\alias{replNAbyLow}
\title{Replace NAs by low values}
\usage{
replNAbyLow(
  x,
  grp,
  quant = 0.8,
  signific = 3,
  unif = TRUE,
  absOnly = FALSE,
  seed = NULL,
  silent = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{x}{(numeric matrix or data.frame) main input}

\item{grp}{(factor) to organize replicate columns of (x)}

\item{quant}{(numeric) quantile form 'neighbour' values to use as upper limit for random values}

\item{signific}{number of signif digits for random values}

\item{unif}{(logical) toggle between uniform and Gaussian random values}

\item{absOnly}{(logical) if TRUE, make negative NA-replacment values positive as absolute values}

\item{seed}{(integer) for use with set.seed for reproducible output}

\item{silent}{(logical) suppress messages}

\item{callFrom}{(character) allow easier tracking of message(s) produced}
}
\value{
numeric matrix (same dim as 'x') with \code{NA} replaced
}
\description{
With several screening techniques used in hight-throughput biology values at/below detection limit are returned as \code{NA}.
However, the resultant \code{NA}-values may be difficult to analyse properly, simply ignoring \code{NA}-values mat not be a good choice.
When (technical) replicate measurements are available, one can look for cases where one gave an \code{NA} while the other did not 
with the aim of investigating such 'NA-neighbours'.  
\code{replNAbyLow} locates and replaces \code{NA} values by (random) values from same line & same group 'grp'. 
The origin of NAs should be predominantly absence of measure (quantitation) due to signal below limit of detection
and not saturation at upper detection limit or other technical problems.  
Note, this approach may be not optimal if the number of NA-neighbours is very low.
Replacamet is done -depending on agrument 'unif'- by Gaussian random model based on neighbour values (within same group),
using their means and sd, or a uniform random model (min and max of neighbour values) .
Then numeric matrix (same dim as 'x') with \code{NA} replaced is returned.
}
\examples{
dat <- matrix(round(rnorm(30),2),ncol=6); grD <- gl(2,3)
dat[sort(sample(1:30,9,repl=FALSE))] <- NA
dat; replNAbyLow(dat,gr=grD)
}
\seealso{
\code{\link{naOmit}}, \code{\link[stats]{na.fail}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/orderMatrToRef.R
\name{orderMatrToRef}
\alias{orderMatrToRef}
\title{Order Lines of Matrix According to Reference (Character) Vector}
\usage{
orderMatrToRef(
  mat,
  ref,
  addRef = TRUE,
  listReturn = TRUE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{mat}{(matrix, data.frame) main input of which rows should get re-ordered according to a (character) reference vector \code{ref}}

\item{ref}{(character) reference imposing new order}

\item{addRef}{(logical) add \code{ref} to output as new column}

\item{listReturn}{(logical) allows retrieving more information in form of list}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) display additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns, depending on \code{listReturn}, either the input-matrix in new order or a list with $mat (the input matrix in new order), $grep (matched matrix) and $col indicating the colum of \code{mat} finally used
}
\description{
This function orders lines of matrix \code{mat} according to a (character) reference vector \code{ref}.
To do so, all columns of \code{mat} will be considered to use the first column from left with the best (partial) matching results.
This function first looks for unambiguous perfect matches, and if not found successive rounds of more elaborte partial matching will be engaged: 
In case of no perfect matches found, grep of \code{ref} on all columns of \code{mat} and/or grep of all columns of \code{mat} on \code{ref} (ie 'reverse grep') will be applied (finally a 'two way grep' approach).
Until a perfect match is found each element of \code{ref} will be tested on \code{mat} and inversely (for each column) each element of \code{mat} will be tested on \code{ref}.
The approach with the best number of (unique) matches will be chosen. In case of one-to-many matches, it will be tried to use most complete lines (see also last example).
}
\examples{
mat1 <- matrix(paste0("__",letters[rep(c(1,1,2,2,3),3) +rep(0:2,each=5)], rep(1:5)), ncol=3)
orderMatrToRef(mat1, paste0(letters[c(3,4,5,3,4)],c(1,3,5,2,4)))

mat2 <- matrix(paste0("__",letters[rep(c(1,1,2,2,3),3) +rep(0:2,each=5)], 
  c(rep(1:5,2),1,1,3:5 )), ncol=3)
orderMatrToRef(mat2, paste0(letters[c(3,4,5,3,4)],c(1,3,5,1,4)))

mat3 <- matrix(paste0(letters[rep(c(1,1,2,2,3),3) +rep(0:2,each=5)], 
  c(rep(1:5,2),1,1,3,3,5 )), ncol=3)
orderMatrToRef(mat3, paste0("__",letters[c(3,4,5,3,4)],c(1,3,5,1,3)))

}
\seealso{
for basic ordering see \code{\link[base]{match}}; \code{\link{checkGrpOrder}} for testing each line for expected order, \code{\link{checkStrictOrder}} to check for strict (ascending or descending) order
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filt3dimArr.R
\name{.filterSw}
\alias{.filterSw}
\title{Filter 3-dim array of numeric data (main)}
\usage{
.filterSw(x, fiTy, checkVa, indexRet = TRUE)
}
\arguments{
\item{x}{array (3-dim) of numeric data}

\item{fiTy}{(character) which type of testing to perform ('eq','inf','infeq','sup','supeq', '>', '<', '>=', '<=', '==')}

\item{checkVa}{(logical) s}

\item{indexRet}{(logical) if \code{TRUE} (default) rather return index numbers than filtered values}
}
\value{
This function returns either index (position within 'x') or concrete (filtered) result
}
\description{
Filtering of matrix or array \code{x} (may be 3-dim array) according to \code{fiTy} and \code{checkVa}
}
\examples{
arr1 <- array(11:34, dim=c(4,3,2), dimnames=list(c(LETTERS[1:4]),
  paste("col",1:3,sep=""),c("ch1","ch2")))
filt3dimArr(arr1,displCrit=c("col1","col2"),filtCrit="col2",filtVal=7)
.filterSw(arr1, fiTy="inf", checkVa=7)
}
\seealso{
\code{\link{filt3dimArr}}; \code{\link{filterList}}; \code{\link{filterLiColDeList}};
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/closeMatchMatrix.R
\name{closeMatchMatrix}
\alias{closeMatchMatrix}
\title{Reorganize results of search for close (similar) values in matrix-view}
\usage{
closeMatchMatrix(
  closeMatch,
  predMatr,
  measMatr,
  prefMatch = c("^x", "^y"),
  colPred = 1,
  colMeas = 1,
  limitToBest = TRUE,
  asDataFrame = FALSE,
  origNa = TRUE,
  silent = FALSE,
  callFrom = NULL,
  debug = FALSE
)
}
\arguments{
\item{closeMatch}{(list) output from \code{\link[wrMisc]{findCloseMatch}}, ie list with hits for each 'x' (1st argument) : named vectors of value & x index in name; run with 'sortMatch'=F}

\item{predMatr}{(vector or matrix) predicted values, the column 'colPred' indicates which column is used for matching from \code{\link[wrMisc]{findCloseMatch}}; if column 'id' present this column will be used as identifier for matching}

\item{measMatr}{(vector or matrix) measured values, the column 'colMeas' indicates which column is used for matching from \code{\link[wrMisc]{findCloseMatch}}; if column 'id' present this column will be used as identifier for matching}

\item{prefMatch}{(character, length=2) prefixes ('^x' and/or '^y') thay may have been added by \code{findCloseMatch}}

\item{colPred}{(integer or text, length=1) column of 'predMatr' with main values of comparison}

\item{colMeas}{(integer or text, length=1) column of 'measMatr' with main measures of comparison}

\item{limitToBest}{(integer) column of 'measMatr' with main measures of comparison}

\item{asDataFrame}{(logical) convert results to data.frame if non-numeric matrix produced (may slightly slow down big results)}

\item{origNa}{(logical) will try to use original names of objects 'predMatr','measMatr', if they are not multi-column and not conflicting other output-names (otherwise 'predMatr','measMatr' will appear)}

\item{silent}{(logical) suppress messages}

\item{callFrom}{(character) allows easier tracking of message(s) produced}

\item{debug}{(logical) for bug-tracking: more/enhanced messages}
}
\value{
results as matrix-view based on initial results from \code{\link[wrMisc]{findCloseMatch}}, including optional columns of suppelemental data for both sets of data for comparison. Returns \code{NULL} when nothing within limits
}
\description{
\code{closeMatchMatrix} reorganizes/refines results from simple search of similar values of 2 sets of data by \code{\link[wrMisc]{findCloseMatch}} (as list for one-to many relations) to more human friendly/readable matrix.
This function returns results combining two sets of data which were initially compared (eg measured and threoretical values) as matrix-view using output of \code{\link[wrMisc]{findCloseMatch}} and both original datastes
Additional information (covariables, annotation, ...) may be included as optional columns for either 'predMatr' or 'measMatr'.
Note : It is important to run \code{\link[wrMisc]{findCloseMatch}} with \code{sortMatch=FALSE} !
Note : Results presented based on view of 'predMatr', so if multiple 'measMatr' are at within tolared distance, lines of 'measMatr' will be repeated;
Note : Distances  'disToMeas' and 'ppmToPred' are oriented : neg value if measured is lower than predicted (and pos values if higher than predicted);
Note : Returns \code{NULL} when nothing within given limits of comparison;
}
\examples{
aA <- c(11:17); bB <- c(12.001,13.999); cC <- c(16.2,8,9,12.5,15.9,13.5,15.7,14.1,5)
(cloMa <- findCloseMatch(aA,cC,com="diff",lim=0.5,sor=FALSE))       
# all matches (of 2d arg) to/within limit for each of 1st arg ('x'); 'y' ..to 2nd arg = cC
(maAa <- closeMatchMatrix(cloMa,aA,cC,lim=TRUE))  #
(maAa <- closeMatchMatrix(cloMa,aA,cC,lim=FALSE,origN=TRUE))  #
(maAa <- closeMatchMatrix(cloMa,cbind(valA=81:87,aA),cbind(valC=91:99,cC),colM=2,
  colP=2,lim=FALSE))
(maAa <- closeMatchMatrix(cloMa,cbind(aA,valA=81:87),cC,lim=FALSE,deb=TRUE))  #
a2 <- aA; names(a2) <- letters[1:length(a2)];  c2 <- cC; names(c2) <- letters[10+1:length(c2)]
(cloM2 <- findCloseMatch(x=a2,y=c2,com="diff",lim=0.5,sor=FALSE)) 
(maA2 <- closeMatchMatrix(cloM2,predM=cbind(valA=81:87,a2),measM=cbind(valC=91:99,c2),
  colM=2,colP=2,lim=FALSE,asData=TRUE)) 
(maA2 <- closeMatchMatrix(cloM2,cbind(id=names(a2),valA=81:87,a2),cbind(id=names(c2),
  valC=91:99,c2),colM=3,colP=3,lim=FALSE,deb=FALSE)) 
}
\seealso{
\code{\link[wrMisc]{findCloseMatch}}, \code{\link[wrMisc]{checkSimValueInSer}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scaleXY.R
\name{scaleXY}
\alias{scaleXY}
\title{Scale data to given minimum and maxiumum}
\usage{
scaleXY(x, min = 0, max = 1)
}
\arguments{
\item{x}{(numeric) vector to rescacle}

\item{min}{(numeric) minimum value in output}

\item{max}{(numeric) maximum value in output}
}
\value{
vector of rescaled data (in dimensions as input)
}
\description{
This is a convenient way to scale data to given minimum and maxiumum without full standarization, ie without deviding by the sd.
}
\examples{
dat <- matrix(2*round(runif(100),2), ncol=4)
range(dat)
dat1 <- scaleXY(dat, 1,100)
range(dat1)
summary(dat1)

## scale for each column individually
dat2 <- apply(dat, 2, scaleXY, 1, 100)
range(dat2)
summary(dat2)
}
\seealso{
\code{\link[base]{scale}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nonAmbiguousNum.R
\name{nonAmbiguousNum}
\alias{nonAmbiguousNum}
\title{Make numeric vector non-ambiguous (ie unique)}
\usage{
nonAmbiguousNum(
  x,
  uniqOnly = FALSE,
  asList = FALSE,
  nameMod = "amb_",
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{x}{(numeric) main input}

\item{uniqOnly}{(logical) if=TRUE return unique only, if =FALSE return unique and single representative of non-unique values (with '' added to name), selection of representative of repeated: first (of sorted) or middle if >2 instances}

\item{asList}{(logical) return list}

\item{nameMod}{(character) text to add in case on ambiguous values, default="amb_"}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allows easier tracking of messages produced}
}
\value{
sorted non-ambigous numeric vector (or list if 'asList'=TRUE and 'uniqOnly'=FALSE)
}
\description{
Thios function transfomrmes a vector of (named) numeric values \code{x} into unique. 
Note: for non-numeric use the function \code{\link{firstOfRepeated}} - but 1000x slower !
Return sorted non-ambigous numeric vector (or list if 'asList'=TRUE and 'uniqOnly'=FASLSE)
}
\examples{
set.seed(2017); aa <- round(rnorm(100),2); names(aa) <- 1:length(aa)
str(nonAmbiguousNum(aa))
str(nonAmbiguousNum(aa,uniq=FALSE,asLi=TRUE))
}
\seealso{
\code{\link{firstOfRepeated}} for non-numeric use (much slower !!!), \code{\link[base]{duplicated}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mergeW2.R
\name{.convertNa}
\alias{.convertNa}
\title{Convert/standardize names of 'query' to standard names from 'ref'}
\usage{
.convertNa(query, ref, partMatch = TRUE)
}
\arguments{
\item{query}{(matrix or data.frame, min 2 columns) main input}

\item{ref}{(list) list of multiple possible names associated to given group, reference name for each group is name of list}

\item{partMatch}{(logical) allows partial matching (ie name of 'ref' must be in head of 'query')}
}
\value{
This function returns a character vector
}
\description{
This function converts/standardizes names of 'query' to standard names from 'ref' (list of possible names (char vect) where names define standardized name).
It takes 'query' as character vector and return character vecor (same length as 'query') with 'converted/corrected' names
}
\examples{
daPa <- matrix(c(1:5,8,2:6,9), ncol=2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkAvSd.R
\name{checkAvSd}
\alias{checkAvSd}
\title{Check how multiple groups of data separate or overlap based on mean +/- sd}
\usage{
checkAvSd(
  daAv,
  daSd,
  nByGr = NULL,
  multSd = 2,
  codeConst = "const",
  extSearch = FALSE,
  outAsLogical = TRUE,
  silent = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{daAv}{matrix or data.frame}

\item{daSd}{matrix or data.frame}

\item{nByGr}{optinal specifying number of Elements per group, allows rather using SEM (adopt to variable n of different groups)}

\item{multSd}{(numeric) the factor specifyin how many sd values should be used as margin}

\item{codeConst}{(character) which term/word to use when specifying 'constant'}

\item{extSearch}{(logical) if TRUE, extend search to one group further (will call result 'nearUp' or 'nearDw')}

\item{outAsLogical}{to switch between 2col-output (separate col for 'up' and 'down') or simple categorical vector ('const','okDw','okUp')}

\item{silent}{(logical) suppress messages}

\item{callFrom}{(character) allow easier tracking of message(s) produced}
}
\value{
vector describing character as 'const' or 'okUp','okDw' (or if extSearch=TRUE 'nearUp','nearDw')
}
\description{
\code{checkAvSd} compares if/how neighbour groups separate/overlap via the 'engineering approach' (+/- 2 standard-deviations is similar to a=0.05 \code{t.test}).
This approach may be used as less elegant alternative to (multi-group) logistic regression.
The function uses 'daAv' as matrix of means (rows are tested for up/down character/progression) which get compared with boundaries taken from daSd (for Sd values of each mean in 'daAv').
}
\examples{
mat1 <- matrix(rep(11:24,3)[1:40],byrow=TRUE,ncol=8)
checkGrpOrderSEM(mat1,grp=gl(3,3)[-1])
checkAvSd(rowGrpMeans(mat1,gl(3,3)[-1]),rowGrpSds(mat1,gl(3,3)[-1]) )
# consider variable n :
checkAvSd(rowGrpMeans(mat1,gl(3,3)[-1]),rowGrpSds(mat1,gl(3,3)[-1]),nByGr=c(2,3,3)) 
}
\seealso{
\code{\link[wrMisc]{rowGrpMeans}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extrSpcText.R
\name{.retain1stPart}
\alias{.retain1stPart}
\title{Trim character string: keep only text before 'sep'}
\usage{
.retain1stPart(chr, sep = " = ", offSet = 1)
}
\arguments{
\item{chr}{character vector to be treated}

\item{sep}{(character) saparator}

\item{offSet}{(integer) off-set}
}
\value{
This function returns a modified character vector
}
\description{
Trim character string: keep only text before 'sep' (length=1 !)
}
\examples{
.retain1stPart("abc = def")
}
\seealso{
\code{\link[base]{substr}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkVectLength.R
\name{.checkArgNa}
\alias{.checkArgNa}
\title{Check argument names}
\usage{
.checkArgNa(x, argNa, lazyEval = TRUE)
}
\arguments{
\item{x}{(character) main input}

\item{argNa}{(character) argument name}

\item{lazyEval}{(logical) decide if argument should be avaluated with abbreviated names, too}
}
\value{
This function returns a elongated character vector
}
\description{
This function allows checking of argument names
}
\examples{
.checkArgNa("Abc",c("ab","Ab","BCD"))
}
\seealso{
\code{\link[base]{chartr}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/minDiff.R
\name{minDiff}
\alias{minDiff}
\title{Minimum distance/difference between values}
\usage{
minDiff(
  x,
  digSig = 3,
  ppm = TRUE,
  initOrder = TRUE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{x}{(numeric) vector to search minimum difference}

\item{digSig}{number of significant digits, used for ratio or ppm column}

\item{ppm}{(logical) display distance as ppm (1e6*diff/refValue, ie normalized difference eg as used in mass spectrometry), otherwise the ratio is given as : value(from 'x') / closestValue (from 'x')}

\item{initOrder}{(logical) return matrix so that 'x' matches exactely 2nd col of output}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a matrix
}
\description{
This function aims to find the min distance (ie closest point) to any other x (numeric value), ie intra 'x' and
 returns matrix with 'index','value','dif','ppm','ncur','nbest','best'.
At equal distance to lower & upper neighbour point, the upper (following) point is chosen (as single best).
In case of multiple ex-aequo distance returns 1st of multiple, may be different at various repeats.
}
\examples{
set.seed(2017); aa <- 100*c(0.1 +round(runif(20),2),0.53,0.53)
minDiff(aa); 
minDiff(aa,initO=TRUE,ppm=FALSE); .minDif(unique(aa))
}
\seealso{
\code{\link[base]{diff}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/renameColumns.R
\name{renameColumns}
\alias{renameColumns}
\title{Rename columns}
\usage{
renameColumns(refMatr, newName, silent = FALSE, debug = FALSE, callFrom = NULL)
}
\arguments{
\item{refMatr}{matrix (or data.frame) where column-names should be changed}

\item{newName}{(matrix of character) giving correspondence of old to new names (number of lines must match number of columns of 'refMatr')}

\item{silent}{(logical) suppres messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allows easier tracking of messages produced}
}
\value{
This function returns a matrix (or data.frame) with renamed columns
}
\description{
This function renames columns of 'refMatr' using 2-column matrix (or data.frame) indicating old and new names (for replacement).
}
\examples{
ma <- matrix(1:8,ncol=4,dimnames=list(1:2,LETTERS[1:4]))
replBy1 <- cbind(new=c("dd","bb","z_"),old=c("D","B","zz"))
replBy2 <- matrix(c("D","B","zz","dd","bb","z_"),ncol=2)
replBy3 <- matrix(c("X","Y","zz","xx","yy","z_"),ncol=2)
renameColumns(ma,replBy1)
renameColumns(ma,replBy2)
renameColumns(ma,replBy3)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summarizeCols.R
\name{summarizeCols}
\alias{summarizeCols}
\title{Summarize columns (as median,mean,min,last or other methods)}
\usage{
summarizeCols(
  matr,
  meth = "median",
  refCol = NULL,
  nEqu = FALSE,
  supl = NULL,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{matr}{data.frame matrix of data to be summarized by comlumn (may do different method for text and numeric comlumns)}

\item{meth}{(character) summarization method, may be 'mean','aver','median','sd','CV', 'min','max','first','last','maxOfRef','minOfRef','maxAbsLast','minAbsLast',
'medianComplete' or 'meanComplete', 'n' (number of non-\code{NA} elements),'n.NA' (number of \code{NA} elements), 'NULL' (returns \code{NULL})}

\item{refCol}{(character or integr) column to be used as reference}

\item{nEqu}{(logical) if \code{TRUE}, add additional column indicating the number of equal lines for choice (only with min or max)}

\item{supl}{(numeric) supplemental parameters for the various summarizing functions (eg used with \code{meth="trimmedMean", supl=c(l=0.1,u=0.2)} to pass arguments to \code{\link{trimmedMean}})}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allows easier tracking of messages produced}
}
\value{
vector with summary for each column
}
\description{
\code{summarizeCols} summarizes all columns of matrix (or data.frame).
In case of text-columns the sorted middle (~median) will be given, unless 'maxAbsLast', 'minAbsLast',
 .. consider only last column of 'matr' : choose from all columns the line where (max of) last col is at min;
 'medianComplete' or 'meanComplete' consideres only lines/rows where no NA occur (NA have influence other columns !)
}
\details{
The argument \code{method} allows options that treat (summarize) all columns independently or to select one line (based on argument \code{refCol})
}
\examples{
t1 <- matrix(round(runif(30,1,9)),nc=3); rownames(t1) <- letters[c(1:5,3:4,6:4)]
summarizeCols(t1, me="median")
t(sapply(by(t1,rownames(t1), function(x) x), summarizeCols,me="maxAbsLast"))
t3 <- data.frame(ref=rep(11:15,3), tx=letters[1:15],
  matrix(round(runif(30,-3,2),1), ncol=2), stringsAsFactors=FALSE)
by(t3,t3[,1], function(x) x)
by(t3,t3[,1], function(x) summarizeCols(x, me="maxAbsLast"))
t(sapply(by(t3, t3[,1], function(x) x), summarizeCols, me="maxAbsLast"))
}
\seealso{
\code{\link[base]{colSums}}; if data has subgroups to be used in a \code{\link[base]{tapply}}-way please see \code{\link{makeNRedMatr}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rowGrpSums.R
\name{rowGrpSums}
\alias{rowGrpSums}
\title{rowSums with destinction of groups (of columns, eg groups of replicates)}
\usage{
rowGrpSums(x, grp, na.rm = TRUE)
}
\arguments{
\item{x}{matrix or data.frame}

\item{grp}{(character or factor) defining which columns should be grouped (considered as replicates)}

\item{na.rm}{(logical) a logical value indicating whether \code{NA}-values should be stripped before the computation proceeds.}
}
\value{
This function a matrix with row/group sum values
}
\description{
This function calculates row-sums for matrix with multiple groups of data, ie similar to \code{rowSums} but one summed value for each line and group of data.
Groups are specified as columns of 'x' in 'grp' (so length of grp should match number of columns of 'x', NAs are allowed).
}
\examples{
set.seed(2016); dat1 <- matrix(c(runif(200) +rep(1:10,20)), ncol=10)
head(rowGrpMeans(dat1, gr=gl(4, 3, labels=LETTERS[1:4])[2:11]))
}
\seealso{
\code{\link{rowGrpMeans}}, \code{\link{rowGrpSds}}, \code{\link{rowSds}}, \code{\link[base]{colSums}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/levIndex.R
\name{levIndex}
\alias{levIndex}
\title{Transform (factor) levels into index}
\usage{
levIndex(
  dat,
  byOccurance = TRUE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{dat}{(numeric or character vector or factor) main input}

\item{byOccurance}{(logical) toogle if lowest index should be based on alphabetical order or on order of input}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
matrix with mean values
}
\description{
This function helps transforming a numeric or character vector into indexes of levels (of its original values).
By default indexes are assigned by order of occurance, ie, the first value of \code{x} will be get the index of 1.
Using the argument \code{byOccurance=FALSE} the resultant indexes will follow the sorted values.
}
\examples{
x1 <- letters[rep(c(5,2:3),1:3)]
levIndex(x1)
levIndex(x1, byOccurance=FALSE)
## with factor 
fa1 <- factor(letters[rep(c(5,2:3),1:3)], levels=letters[1:6])
levIndex(fa1)
levIndex(fa1, byOccurance=FALSE)
}
\seealso{
\code{\link{rowSds}}, \code{\link[base]{colSums}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/findHeadAndTail.R
\name{findHeadAndTail}
\alias{findHeadAndTail}
\title{Find Group-Names In Pairwise Combined And Isolate Separator}
\usage{
findHeadAndTail(
  grpNa,
  pairwNa,
  reportAs = "list",
  sortGrp = TRUE,
  rmAmbig = TRUE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{grpNa}{(character) the names of the groups of replicates (ie conditions) used to test}

\item{pairwNa}{(character) the names of pairwise-testing (ie 'concatenated' \code{sampNa}}

\item{reportAs}{(character) default \code{'reportAs=list'} to return list, otherwise a character vector with separator(s) found will be returned}

\item{sortGrp}{(logical) by default groups will be sorted (as levels of a factor appear sorted)}

\item{rmAmbig}{(logical) remove all ambiguous}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a list containing $sep a character vector of the separator(s) identified
}
\description{
This function allows mapping which group-names are occuring either as head or as tail in pairwise combined terms (by checking from tails).
}
\details{
This function aims identifying the separator used wo create pairwise comparison labels by looking for given group-names at head and tail of combined labels.
If labeles occur inside oyther ones (eg 'AB' in 'ABC') it will always privilidge the longest one.
After stripping off all group-names the remaining charachers are considered as the separator used.
The function will also check if all pairwise labels returned the same separtator and in case of multiple separators found the most frequent may be reytained.
}
\examples{
findHeadAndTail(LETTERS[4:2], c("D-X","D-C"))

grp1 <- c("C","ACC","CC","B.B","B.BA","B","CA")
pwNa1 <- "B.B-CC" 
findHeadAndTail(grp1, pwNa1)

pwNa2 <- c("B.B-CC", "CA-ACC")
findHeadAndTail(grp1, pwNa2)
}
\seealso{
(for running multiple pair-wise test) \code{\link{moderTestXgrp}}, used underneith \code{\link[base]{grep}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/minDiff.R
\name{.minDif}
\alias{.minDif}
\title{find closest neighbour to numeric vector}
\usage{
.minDif(z, initOrder = TRUE, rat = TRUE)
}
\arguments{
\item{z}{(numeric) vector to search minimum difference}

\item{initOrder}{(logical) return matrix so that 'x' matches exactely 2nd col of output}

\item{rat}{(logical) express result as ratio}
}
\value{
This function returns a matrix with index,value,dif,best
}
\description{
This function aims to find closest neighbour to numeric vector
}
\examples{
.minDif(c(11:15,17))
}
\seealso{
\code{\link[stats]{dist}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/renameColumns.R
\name{.keepFiniteCol}
\alias{.keepFiniteCol}
\title{Remove all columns where all data are not finite}
\usage{
.keepFiniteCol(
  dat,
  msgStart = NULL,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{dat}{(matrix or data.frame) main input}

\item{msgStart}{(character)}

\item{silent}{(logical) suppres messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allows easier tracking of messages produced}
}
\value{
This function returns a corrected matrix or data.frame
}
\description{
This function aims to remove all columns where all data are not finite
}
\examples{
ma1 <- matrix(c(1:5, Inf), ncol=2)
.keepFiniteCol(ma1)

}
\seealso{
\code{\link{renameColumns}}; \code{\link[base]{is.finite}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cutArrayInCluLike.R
\name{.arrLstSEM}
\alias{.arrLstSEM}
\title{Summarize along columns of mult arrays in list}
\usage{
.arrLstSEM(
  arrLst,
  arrOutp = FALSE,
  signifDig = 3,
  formatCheck = FALSE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{arrLst}{(list) main input}

\item{arrOutp}{(logical)}

\item{signifDig}{(integer)}

\item{formatCheck}{(logical)}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
array (1st dim will be summary along cols, rows will be layers of 3rd array-dim ie dim(arrLst[[1]])[3])
}
\description{
This function allows summarizing along columns of mult arrays in list
}
\examples{
.datSlope(c(3:6))
}
\seealso{
used in \code{\link{cutArrayInCluLike}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summarizeCols.R
\name{.summarizeCols}
\alias{.summarizeCols}
\title{Summarize columns of matrix (or data.frame) 'x' using apply (main)}
\usage{
.summarizeCols(
  x,
  me = "median",
  nEq = FALSE,
  vectAs1row = TRUE,
  supl = NULL,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{x}{data.frame matrix of data to be summarized by comlumn}

\item{me}{(character, length=1) summarization method (eg 'min','max','mean','mean.trim','median','sd','CV', 'medianComplete' or 'meanComplete' etc, see \code{\link{summarizeCols}})}

\item{nEq}{(logical) if TRUE, add additional column indicating the number of equal lines for choice (only with min or max)}

\item{vectAs1row}{(logical) if TRUE will interprete non-matrix 'x' as matrix with 1 row (correct effect of automatic conversion when extracting 1 line)}

\item{supl}{(numeric) supplemental parameters for the various summarizing functions (currently used with 'me=mean.trim' to assign upper and lower trimming fraction, passed to )}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
vector with summary for each column (unless 'me=="summary"', in this case a matrix or list will be returned )
}
\description{
This function summarizes columns of matrix (or data.frame) 'x' using apply
In case of character entries the 'median' of sorted values will be returned
}
\examples{
m1 <- matrix(c(28,27,11,12,11,12), nrow=2, dimnames=list(1:2,c("y","x","ref")))
.summarizeCols(m1, me="median")
}
\seealso{
\code{\link{summarizeCols}},  \code{\link[base]{table}} \code{\link[base]{table}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readVarColumns.R
\name{readVarColumns}
\alias{readVarColumns}
\title{Read Tabular Content Of Files With Variable Number Of Columns}
\usage{
readVarColumns(
  fiName,
  path = NULL,
  sep = "\\t",
  header = TRUE,
  emptyFields = NA,
  refCo = NULL,
  supNa = NULL,
  silent = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{fiName}{(character) file-name}

\item{path}{(character) optional path}

\item{sep}{(character) separator (between columns)}

\item{header}{(logical) indicating whether the file contains the names of the variables as its first line.}

\item{emptyFields}{(\code{NA} or character) missing headers will be replaced by the content of 'emptyFields', if \code{NA} the last column-name will be re-used and a counter added}

\item{refCo}{(integer) for custom choice of column to be used as row-names (default will use 1st text-column)}

\item{supNa}{(character) base for constructing name for columns wo names (+counter starting at 2), default column-name to left of 1st col wo colname}

\item{silent}{(logical) suppress messages}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a matrix (character or numeric)
}
\description{
Reading the content of files where the number of separators (eg tabulation) is variable poses problems with traditional methods for reding files, like  \code{\link[utils]{read.table}}.
This function reads each line independently and then parses all separators therein. The first line is assumed to be column-headers.
Finally, all data will be returned in a matrix adopted to the line with most separators and if the number of column-headers is insufficient, new (unique) column-headers will be generated.
Thus, the lines may contain different number of elements, empty elements (ie tabular fields) will always get added to right of data read 
and their content will be as defined by argument \code{emptyFields} (default \code{NA}).
}
\details{
Note, this functions assumes one line of header and at least one line of data !
Note, for numeric data the comma is assumed to be US-Style (as '.').
Note, that it is assumed, that any missing fields for the complete tabular view are missing on the right (ie at the end of line) !
}
\examples{
path1 <- system.file("extdata",package="wrMisc")
fiNa <- "Names1.tsv"
datAll <- readVarColumns(fiName=file.path(path1,fiNa))
str(datAll)
}
\seealso{
for regular 'complete' data \code{\link[utils]{read.table}} and its argument \code{flush}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nonAmbiguousMat.R
\name{nonAmbiguousMat}
\alias{nonAmbiguousMat}
\title{Transform matrix to non-ambiguous matrix (in respect to given column)}
\usage{
nonAmbiguousMat(
  mat,
  byCol,
  uniqOnly = FALSE,
  asList = FALSE,
  nameMod = "amb_",
  silent = TRUE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{mat}{numeric or character matrix (or data.frame), column specified by 'byCol' must be/will be used as.numeric, 1st column of 'mat' will be considered like index & used for adding prefix 'nameMod' (unless byCol=1, then 2nd col will be used)}

\item{byCol}{(character or integer-index) column by which ambiguousity will be tested}

\item{uniqOnly}{(logical) if =TRUE return unique only, if =FALSE return unique and single representative of non-unique values (with '' added to name), selection of representative of repeated: first (of sorted) or middle if >2 instances}

\item{asList}{(logical) return result as list}

\item{nameMod}{(character) prefix added to 1st column of 'mat' (expect 'by') for indicating non-unique/ambiguous values}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) for bug-tracking: more/enhanced messages and intermediate objects written in global name-space}

\item{callFrom}{(character) allows easier tracking of message(s) produced}
}
\value{
sorted non-ambigous numeric vector (or list if 'asList'=TRUE and 'uniqOnly'=FALSE)
}
\description{
\code{nonAmbiguousMat} makes values of matrix 'mat' in col 'byCol' unique.
}
\examples{
set.seed(2017); mat2 <- matrix(c(1:100,round(rnorm(200),2)),ncol=3,
  dimnames=list(1:100,LETTERS[1:3]));
head(mat2U <- nonAmbiguousMat(mat2,by="B",na="_",uniqO=FALSE),n=15)
head(get1stOfRepeatedByCol(mat2,sortB="B",sortS="B"))
}
\seealso{
for non-numeric use \code{\link{firstOfRepeated}} - but 1000x much slower !; \code{\link{get1stOfRepeatedByCol}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/firstLineOfDat.R
\name{firstLineOfDat}
\alias{firstLineOfDat}
\title{Filter matrix to keep only first of repeated lines}
\usage{
firstLineOfDat(dat, refCol = 2, silent = FALSE, debug = FALSE, callFrom = NULL)
}
\arguments{
\item{dat}{(matrix or data.frame) main input}

\item{refCol}{(integer) column number of reference-column}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
matrix (same number of columns as input)
}
\description{
This function aims to reduce the complexity of a matrix (or data.frame) in case column 'refCol' has multiple lines with same value. 
In this case, it reduces the input-data to 1st line of redundant entries and returns a matrix (or data.frame) without lines identified as redundant entries for 'refCol').
in sum, this functions works lile useng \code{unique} on a given column, and propagates the same treatment to all other columns.
}
\examples{
(mat1 <- matrix(c(1:6,rep(1:3,1:3)),ncol=2,dimnames=list(letters[1:6],LETTERS[1:2])))
firstLineOfDat(mat1)
}
\seealso{
\code{\link{firstOfRepeated}}, \code{\link[base]{unique}}, \code{\link[base]{duplicated}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/adjustUnitPrefix.R
\name{adjustUnitPrefix}
\alias{adjustUnitPrefix}
\title{Adjust Value With Different Decimal Prefixes To Single Prefix Plus Unit}
\usage{
adjustUnitPrefix(
  x,
  pref = c("z", "a", "f", "p", "n", "u", "m", "", "k", "M", "G"),
  unit = "sec",
  sep = c("_", ".", " ", ""),
  minTrimNChar = 0,
  returnType = c("NAifInvalid", "allText"),
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{x}{(character) vector containing digit uunit-prefix and unit terms}

\item{pref}{(character) multiplicative unit-prefixes, assumes as increasing factors of 1000}

\item{unit}{(character) unit name, the numeric part may be sepatated by one space-character}

\item{sep}{(character) separator characters that may appear between integer numeric value and unit description}

\item{minTrimNChar}{(integer) min number of text characters when trimming adjacent text on left and right of main numeric+prefix+unit}

\item{returnType}{(character) set options for retuning results : 'NAifInvalid' .. return NA for invalid parts,'allText' .. return initial text if problem, 'trim'}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a character vector (same length as input) with adjusted unified decimal prefix and adjusted numeric content, the numeric content only is also giben in the names of the output
}
\description{
This function provides help converting values with with different unit-prefixes to a single prefix-unit type.
This can be used to convert a vector of mixed prefixes like 'p' and 'n'.
Any text to the right of the unit will be ignored.
}
\details{
The aim of this function if to allow adjusting a vector containing '100pMol' and '1nMol' to '100pMol' and '1000pMol' for better downstream analysis.
Please note that the current version recognizes and converts only interger values; decimals or scientific writing won't be recognized properly.
The resultant numeric vector expresses all values as lowest prefix unit level.
In case of invalid entries \code{NA}s will be returned.

Please note that decimal/comma digits will not be recognized properly, since the function will consider (by default) the decimal sign as just another separator.

To avoid special characters (which may not work on all operating-systems) the letter 'u' is used for 'micro'.
}
\examples{
adjustUnitPrefix(c("2.psec abc","20 fsec etc"), unit="sec")

x1 <- c("50_amol", "5_fmol","250_amol","100_amol", NA, "500_amol", "500_amol", "1_fmol")
adjustUnitPrefix(x1, unit="mol")                   

x2 <- c("abCc 500_nmol ABC", "abEe5_umol", "", "abFF_100_nmol_G", "abGg 2_mol", "abH.1 mmol")
rbind( adjustUnitPrefix(x2, unit="mol", returnType="allText") , 
  adjustUnitPrefix(x2, unit="mol", returnType="trim"),
  adjustUnitPrefix(x2, unit="mol", returnType=""))
}
\seealso{
\code{\link{convToNum}}; \code{\link{checkUnitPrefix}}; \code{\link{trimRedundText}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getPWseparator.R
\name{getPWseparator}
\alias{getPWseparator}
\title{Identify Separator In Pairwise Group-Names Or Find Separator For Use With Pairwise Group-Names  
  
This function allows identifing separator used when pairwise groups are presented.}
\usage{
getPWseparator(
  compNames = NULL,
  grp = NULL,
  potSep = c("-", "---", "_", "___", ".", "-vs-", "_vs_", "--vs--", "__vs__", " ", "  ",
    "--", "__"),
  includeGrp = FALSE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{compNames}{(character or integer) names of pairwise combined group-names}

\item{grp}{(character or factor) optional groups (may include group-names that do not occur in \code{compNames})}

\item{potSep}{(character) potential separators to be checked if occuring in \code{useComp}; the first fitting will used}

\item{includeGrp}{(logical) if \code{includeGrp=TRUE} a list including also the group-names is returned}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a character vector (length=1) of the (first) separator fitting to all data, or a list with $sep and $grpNa if \code{includeGrp=TRUE}
}
\description{
Identify Separator In Pairwise Group-Names Or Find Separator For Use With Pairwise Group-Names  
  
This function allows identifing separator used when pairwise groups are presented.
}
\details{
Note : Potential separators \code{potSep} must be at least 1 character long
  
The character '.' may used as separator (argument \code{potSep}), it will be protected internally for obtaining correct results when using grep().
Note : The indexing is relative to the levels of grp, thus to sorted group-names !

Cases of usage : 
1) Identify separator used in concatenated labels, ie concatenated group-names are given, (only argument \code{compNames} is obligatory)

2) Suggets/Find suitable separator for given group-labels (ie a separator not occurring in any of the group-labels) when names 
   of \code{grp} given (thus argument \code{grp} is obligatory), 
   the first instance of \code{potSep} fitting will be chosen
}
\examples{
## Suggest separator
getPWseparator(grp=LETTERS[1:3])
getPWseparator(grp=c("B-C","D","E")) 
getPWseparator(grp=c("B-C","D","E"), includeGrp=TRUE) 

## Identify separator used
getPWseparator(compNames=c("B-C","B-D","C-D")) 
getPWseparator(compNames=c("B-C","B-D","C-D"), includeGrp=TRUE) 
}
\seealso{
\code{\link{indexGroupsFromPW}}, \code{\link{getPWseparator}}, colnames of testing results from \code{\link{moderTestXgrp}}, used by \code{\link{replacePWseparator}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkFilePath.R
\name{checkFilePath}
\alias{checkFilePath}
\title{Verify File-name If Existing (in specified path), If Has Proper Extension Or Select Files With Proper Extension From Given Path}
\usage{
checkFilePath(
  fileName = NULL,
  path = "./",
  expectExt = "",
  mode = "compressedOption",
  compressedOption = NULL,
  strictExtension = NULL,
  stopIfNothing = NULL,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{fileName}{(character) name of file to be tested; may also include an absolute or relative path; 
if \code{NULL} and \code{path} as well as \code{expectExt} will take 1st file in given path and proper extension}

\item{path}{(character, length=1) optional separate entry for path of \code{fileName}}

\item{expectExt}{(character) file extension (will not be considered if \code{""})}

\item{mode}{(character) further details if function should give error or warning if no files found
integrates previous argument \code{compressedOption} to also look for look for .gz compressed files; 
\code{strictExtension} to decide if extension (\code{expectExt}) - if given - should be considered obligatory;
\code{stopIfNothing} to stop with error if no files found}

\item{compressedOption}{deprected  (logical) also look for .gz compressed files}

\item{strictExtension}{deprected  (logical) decide if extesion (\code{expectExt}) - if given - should be considered obligatory}

\item{stopIfNothing}{deprected, please use argument \code{mode} instead !}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a character vector with verified file-name(s) (and path), returns \code{NULL} if nothing found - unless \code{mode="stopIfNothing"}
}
\description{
This function allows tesing if a given file-name corresponds to an existing file (eg for reading lateron).
Indications to the path and file-extensions may be given separately. If no files do match .gz compressed versions may be searched, too.
}
\details{
When the filename given by the user exists but it's file-extension is not matching \code{expectExt} 
the argument \code{strictExtension} allows to decide if the filename will still be returned or not.

When \code{expectExt} is given, initial search will look for perfect matches. 
However, if nothing is found and \code{strictExtension=FALSE}, a more relaxed and non-case-sensitive search will be performed.
}
\examples{
(RhomeFi <- list.files(R.home()))
file.exists(file.path(R.home(), "bin"))
checkFilePath(c("xxx","unins000"), R.home(), expectExt="dat")
}
\seealso{
\code{\link[base]{file.exists}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rowNormalize.R
\name{rowNormalize}
\alias{rowNormalize}
\title{Row Normalize}
\usage{
rowNormalize(
  dat,
  method = "median",
  refLines = NULL,
  refGrp = NULL,
  proportMode = TRUE,
  minQuant = NULL,
  sparseLim = 0.4,
  nCombin = 3,
  omitNonAlignable = FALSE,
  maxFact = 10,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{dat}{matrix or data.frame of data to get normalized}

\item{method}{(character) may be "mean","median" (plus "NULL","none"); When NULL or 'none' is chosen the input will be returned as is}

\item{refLines}{(NULL or numeric) allows to consider only specific lines of 'dat' when determining normalization factors (all data will be normalized)}

\item{refGrp}{(integer) Only the columns indicated will be used as reference, default all columns (integer or colnames)}

\item{proportMode}{(logical) decide if normalization should be done by multiplicative or additive factor}

\item{minQuant}{(numeric) optional filter to set all values below given value as \code{NA}}

\item{sparseLim}{(integer) decide at which min content of  \code{NA} values the function should go in sparse-mode}

\item{nCombin}{(NULL or integer) used only in sparse-mode (ie if content of \code{NA}s higher than content of \code{sparseLim}): Number of groups of smller matrixes with this number of columns to be inspected initailly;
low values (small groups have higher chances of more common elements)}

\item{omitNonAlignable}{(logical) allow omitting all columns which can't get aligned due to sparseness}

\item{maxFact}{(numeric, length=2) max normalization factor}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allows easier tracking of messages produced}
}
\value{
This function returns a matrix of normalized data
}
\description{
This function was designed for normalizing data that is supposed to be particularly similar, like a collection of technical replicates.
Thus, initially for each row an independent normalization factor is calculated and the median or mean across all factors will be finally applied to the data.
This function has a special mode of operation with higher content of \code{NA} values (which may pose problems with other normalization approaches). 
If the \code{NA}-content is higher than the threshold set in \code{sparseLim},
a special procedure for sparse data will be applied (iteratively trating subsets of \code{nCombin} columns that will be combined in a later step).
}
\details{
Arguments were kept similar with function \code{normalizeThis} as much as possible.
In most cases data get normalized by proportional factors. In case of log2-data (very common in omics-data) normalizing by an additive factor is equivalent to a proportional factor.

This function has a special mode of operation for sparse data (ie containing a high content of \code{NA} values).
0-values by themselves will be primarily considered as true measurment outcomes and not as missing.
However, by using the argument \code{minQuant} all values below a given threshold will be set as \code{NA} and this may possibly trigger the sparse mode of normalizing.

Note : Using a small value of \code{nCombin} will give the highest chances of finding sufficient complete combination of columns with sparse data.
However, this will also increase (very much) the computational efforts and time required to produce an output.

When using default proportional mode a potential division by 0 could occur, when the initial normalization factor turns out as 0.
In this case a small value (default the maximum value of \code{dat} / 10 will be added to all data before normalizing.
If this also creates 0-vales in the data this factor will be multiplied by 0.03.
}
\examples{
## sparse matrix  normalization
set.seed(2); AA <- matrix(rbinom(110,10,0.05), nrow=10)
AA[,4:5] <- AA[,4:5] *rep(4:3, each=nrow(AA))
AA[2,c(2,6,7)] <- 1; AA[3,8] <- 1;

(AA1 <- rowNormalize(AA))
(AA2 <- rowNormalize(AA, minQuant=1))   # set all 0 as NAs
(AA3 <- rowNormalize(AA, refLines=1:6, omitNonAlignable=FALSE, minQuant=1))


}
\seealso{
\code{\link{normalizeThis}}, \code{\link{exponNormalize}}, \code{\link{adjBy2ptReg}}, \code{\link[vsn]{justvsn}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/listGroupsByNames.R
\name{listGroupsByNames}
\alias{listGroupsByNames}
\title{Organize Values Into List And Sort By Names}
\usage{
listGroupsByNames(x, sep = ".", silent = FALSE, debug = FALSE, callFrom = NULL)
}
\arguments{
\item{x}{(list) main input}

\item{sep}{(character) separator (note that typcal separators may need to be protected, only automatically added for '.')}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a matrix or data.frame
}
\description{
Sort values of \code{'x'} by its names and organize as list by common names, the names until \code{'sep'} are used for (re)grouping. 
Note that typical spearators occuring the initial names may need protection by '\\' (this is automatically taken care of for the case of the dot ('.') separator).
}
\examples{
listGroupsByNames((1:10)/5)
ser1 <- 1:6; names(ser1) <- c("AA","BB","AA.1","CC","AA.b","BB.e")
listGroupsByNames(ser1)
}
\seealso{
\code{rbind} in \code{\link[base]{cbind}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/countCloseToLimits.R
\name{countCloseToLimits}
\alias{countCloseToLimits}
\title{Count From Two Vectors Number Of Values Close Within Given Limits}
\usage{
countCloseToLimits(
  closeMatch,
  limitIdent = 5,
  prefix = "lim_",
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{closeMatch}{(list) output from findCloseMatch(), ie list indicating which instances of 2 series of data have close matches}

\item{limitIdent}{(numeric) max limit or panel of threshold values to test (if single value, in addtion a panel with values below will be tested)}

\item{prefix}{(character)  prefix for names of output}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns an integer vector with counts for number of list-elements with at least one absolue value below threshold, names
}
\description{
This functions summarizes the serach of similar (or identical) numeric values from 2 initial vectors, it 
evaluates the result from initial search run by findCloseMatch(), whose output is a less convenient list.
\code{countCloseToLimits} checks furthermore how many results within additional (more stringent)
distance-limits may be found and returns the number of distance values within the limits tested.
Designed for checking if threshold used with findCloseMatch() may be set more stringent, eg  when searching reasonable FDR limits ...
}
\examples{
set.seed(2019); aa <- sample(12:15,20,repl=TRUE) +round(runif(20),2)-0.5
bb <- 11:18
match1 <- findCloseMatch(aa,bb,com="diff",lim=0.65)
head(match1)
(tmp3 <- countCloseToLimits(match1,lim=c(0.5,0.35,0.2)))
(tmp4 <- countCloseToLimits(match1,lim=0.7))
}
\seealso{
\code{\link[wrMisc]{findCloseMatch}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/packageDownloadStat.R
\name{packageDownloadStat}
\alias{packageDownloadStat}
\title{Simple Package Download Statistics From CRAN}
\usage{
packageDownloadStat(
  queryPackages = c("wrMisc", "wrProteo", "cif", "bcv", "FinCovRegularization"),
  countUrl = "http://www.datasciencemeta.com/rpackages",
  refQuant = (1:10)/10,
  options = c("naOmit", "sort"),
  figure = TRUE,
  log = "",
  silent = FALSE,
  callFrom = NULL,
  debug = FALSE
)
}
\arguments{
\item{queryPackages}{(character or integer) package names of interest, if \code{integer}, n random packages will be picked by random}

\item{countUrl}{(character) the url where the dayly counts ara available}

\item{refQuant}{(numeric) add reference quantile values to output matrix}

\item{options}{(character) additional seetings : use 'naOmit' to remove NA-lines from output (package-names not found in 'countUrl'); 
'sort' for sorting output by number of downloads}

\item{figure}{(logical) decide of figure should be printed}

\item{log}{(character) set count-axis of figure to linear or log-scale (by setting \code{log="y"})}

\item{silent}{(logical) suppress messages}

\item{callFrom}{(character) allow easier tracking of messages produced}

\item{debug}{(logical) additional messages for debugging}
}
\value{
This function retuns a matrix with download counts (or \code{NULL} if the web-site can't be accessed or the query-packages are not found there)
}
\description{
This function allows accessing the most recent counts of package downloads availabale on http://www.datasciencemeta.com/rpackages,
obtaining rank quantiles and to compare (multiple) given packages to the bulk data, optionally a plot can be drawn.
}
\details{
Detailed articles on this subject have been published on R-Hub (https://blog.r-hub.io/2020/05/11/packagerank-intro/) and on
R-bloggers (https://www.r-bloggers.com/2020/10/a-cran-downloads-experiment/).
The task of checking the number of downloads for a given package has also been addressed by several other packages (eg dlstats, cranlogs, adjustedcranlogs).

This function only allows accessing counts as listed on the website of \code{www.datasciencemeta.com} which get updated dayly.
Please note, that reading all lines from the website may take a few seconds !!
To get a better understanding of the counts read, reference quantiles for download-counts get added by default  (see argument \code{refQuant}).
The (optional) figure can be drawn in linear scale (default, with minor zoom to lower number of counts) or in log (necessary for proper display of the entire range of counts), by setting the argument \code{log="y"}.

The number of downloads counted by RStudio may not be a perfect measure for the actual usage/popularity of a given package,
the articles cited above discuss this in more detail.
For example, multiple downloads from the same IP or subsequent downloads of multiple (older) versions of the same package seem to get counted, too.
}
\examples{
## Let's try a microscopic test-file (NOT representative for true up-to-date counts !!)
pack1 <- c("cif", "bcv", "FinCovRegularization", "wrMisc", "wrProteo")
testFi <- file.path(system.file("extdata", package="wrMisc"), "rpackagesMicro.html")
packageDownloadStat(pack1, countUrl=testFi, log="y", figure=FALSE)
## For real online counting simply use the argument countUrl in default setting

}
\seealso{
packages \href{https://CRAN.R-project.org/package=cranlogs}{cranlogs} and \href{https://CRAN.R-project.org/package=packageRank}{packageRank}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gitDataUrl.R
\name{gitDataUrl}
\alias{gitDataUrl}
\title{Convert GitHub Url-Name For Reading In Raw-Mode}
\usage{
gitDataUrl(
  urlName,
  replTxt = NULL,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{urlName}{(charachter) main url-address}

\item{replTxt}{(NULL or matrix) adjust/ custom-modify search- and replacement items; default settings will exchange the beginning of the site to 'raw.githubusercontent.com' and then exchange 'blob' to 'refs/head'
otherwise should be matrix with 2 columns, the 1st colimn entries will be used as 'search-for' and the 2nd as 'replace by' for each row}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
corrected urlName
}
\description{
This functions converts a given urlName so that (tabular) data from GitHub (https://github.com/) can data be read correctly as tabular data.
}
\details{
Thus, this will exchange the beginning of the site to 'raw.githubusercontent.com' and then exchange 'blob' to 'refs/head' .
Alternatively, the user may custom define all terms to get exchanged (as matrix with 2 columns 'old' and 'new').          

Note : The default spearator from  \code{\link[utils]{read.delim}} may suit better for reading the data.
}
\examples{
url1 <- paste0("https://github.com/bigbio/proteomics-metadata-standard/blob/",
  "main/datasets/PXD001819/PXD001819.sdrf.tsv") 
rbind(ini=url1, new=gitDataUrl(url1))

}
\seealso{
\code{\link[base]{sub}}; \code{\link[utils]{read.delim}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nonredDataFrame.R
\name{nonredDataFrame}
\alias{nonredDataFrame}
\title{Filter for unique elements}
\usage{
nonredDataFrame(
  dataFr,
  useCol = c(pepID = "peptide_id", protID = "accession", seq = "sequence", mod =
    "modifications"),
  sepCollapse = "//",
  callFrom = NULL
)
}
\arguments{
\item{dataFr}{(data.frame) main inpput}

\item{useCol}{(character,length=2) comlumn names of 'dataFr' to use : 1st value designates where redundant values should be gathered; 2nd value designes column of which information should be concatenated}

\item{sepCollapse}{(character) conatenation symbol}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a data.frame of filtered (fewer lines) with additional 2 columns 'nSamePep' (number of redundant entries) and 'concID' (concatenated content)
}
\description{
\code{nonredDataFrame}  filters 'x' (list of char-vectors or char-vector) for elements unique (to 'ref' or if NULL to all 'x') and of character length. 
May be used for different 'accession' for same pep sequence (same 'peptide_id').
Note : made for treating data.frames, may be slightly slower than matrix equivalent
}
\examples{
df1 <- data.frame(cbind(xA=letters[1:5], xB=c("h","h","f","e","f"), xC=LETTERS[1:5])) 
nonredDataFrame(df1, useCol=c("xB","xC")) 
}
\seealso{
\code{\link{combineRedBasedOnCol}}, \code{\link{correctToUnique}}, \code{\link[base]{unique}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rankToContigTab.R
\name{rankToContigTab}
\alias{rankToContigTab}
\title{Contingenty tables for fit of ranking}
\usage{
rankToContigTab(dat, grp)
}
\arguments{
\item{dat}{(matrix or data.frame of integer values) ranking of multiple features (lines), equal ranks may occur}

\item{grp}{(integer) expected ranking}
}
\value{
array (1st dim is rows of dat, 2nd is unique(grp), 3rd dim is ok/bad)
}
\description{
Count the number of instances where the corresponding columns of 'dat' have a value matching the group number as specified by 'grp'.
Counting will be performed/repeated independently for each line of 'dat'.
Returns array  (1st dim is rows of dat, 2nd is unique(grp), 3rd dim is ok/bad), these results may be tested using eg \code{\link[stats]{fisher.test}}.
This function was made for prearing to test the ranking of multiple features (lines in 'mat') including replicates (levels of 'grp').
}
\examples{
# Let's create a matrix with ranks (equal ranks do occur)
ma0 <- matrix(rep(1:3,each=6), ncol=6, dimnames=list(
  c("li1","li2","ref"), letters[1:6]))
ma0[1,6] <- 1                       # create item not matching correctly
ma0[2,] <- c(3:1,2,1,3)             # create items not matching correctly
gr0 <- gl(3,2)                      # the expected ranking (as duplicates)
(count0 <- rankToContigTab(ma0,gr0))
cTab <- t(apply(count0, c(1,3) ,sum))
# Now we can compare the ranking of line1 to ref ...
fisher.test(cTab[,c(3,1)])          # test li1 against ref
fisher.test(cTab[,c(3,2)])          # test li2 against ref
}
\seealso{
\code{\link[stats]{lm}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rowGrpSds.R
\name{rowGrpSds}
\alias{rowGrpSds}
\title{Per line and per group sd-values}
\usage{
rowGrpSds(x, grp)
}
\arguments{
\item{x}{matrix where relplicates are organized into seprate columns}

\item{grp}{(character or factor) defining which columns should be grouped (considered as replicates)}
}
\value{
This function returns a matrix of sd values
}
\description{
\code{rowGrpSds} calculate Sd (standard-deviation) for matrix with multiple groups of data, ie one sd for each group of data. 
Groups are specified as columns of 'x' in 'grp' (so length of grp should match number of columns of 'x', NAs are allowed).
}
\examples{
set.seed(2016); dat1 <- matrix(c(runif(200) +rep(1:10,20)), ncol=10)
head(rowGrpSds(dat1, gr=gl(4,3,labels=LETTERS[1:4])[2:11]))
}
\seealso{
\code{\link{rowGrpMeans}}, \code{\link{rowCVs}}, \code{\link{rowSEMs}},\code{\link[stats]{sd}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkSimValueInSer.R
\name{checkSimValueInSer}
\alias{checkSimValueInSer}
\title{Check for similar values in series}
\usage{
checkSimValueInSer(
  x,
  ppm = 5,
  sortX = TRUE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{x}{numeric vector}

\item{ppm}{(numeric, length=1) ppm-range for considering as similar}

\item{sortX}{(logical) allows speeding up function when set to FALSE, for large data that are already sorted}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a logical vector : \code{TRUE} for each entry of \code{x} where at least one neighbour is inside of ppm distance/range
}
\description{
This function checks all values of 'x' for similar neighbour values within (relative) range of 'ppm' (ie parts per milion as measure of distance).
By default values will be sorted internally, so if a given value of \code{x} has anywhere in  \code{x}  another value close enough, this will be detected.
However, if \code{sortX=FALSE} only the values next to left and right will be considered.
Return logical vector : FALSE for each entry of 'x' if value inside of ppm range to neighbour (of sorted values)
}
\examples{
va1 <- c(4:7,7,7,7,7,8:10)+(1:11)/28600; checkSimValueInSer(va1)
data.frame(va=sort(va1),simil=checkSimValueInSer(va1))
}
\seealso{
similar with more options \code{\link{withinRefRange}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/arrayCV.R
\name{arrayCV}
\alias{arrayCV}
\title{CV of array}
\usage{
arrayCV(arr, byDim = 3, silent = TRUE, callFrom = NULL)
}
\arguments{
\item{arr}{(3-dim) array of numeric data like where replicates are along one dimesion of the array}

\item{byDim}{(integer) over which dimension repliates are found}

\item{silent}{(logical) suppres messages}

\item{callFrom}{(character) allow easier tracking of message produced}
}
\value{
matrix of CV values
}
\description{
\code{arrayCV} gets CVs for replicates in 2 or 3 dim array and returns CVs as matrix.
This function may be used to calculate CVs from replicate microtiter plates (eg 8x12) where replicates are typically done as multiple plates,
ie initial matrixes that are the organized into arrays.
}
\examples{
set.seed(2016); dat1 <- matrix(c(runif(200) +rep(1:10,20)), ncol=10)
head(arrayCV(dat1,byDim=2))
}
\seealso{
\code{\link{rowCVs}}, \code{\link{rowGrpCV}}, \code{\link{replPlateCV}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/searchLinesAtGivenSlope.R
\name{.keepCenter1d}
\alias{.keepCenter1d}
\title{Refine/filter 'dat1' (1dim dataset, eg cluster) with aim of keeping center of data}
\usage{
.keepCenter1d(
  dat1,
  core = NULL,
  keepOnly = TRUE,
  displPlot = FALSE,
  silent = TRUE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{dat1}{simple numeric vector}

\item{core}{numeric vactor (betw 0 and 1) for fraction of data to keep; if null trimmedMean/max hist occurance will be used, limited within 30-70 perent; may also be 'high' or 'low' for forcing low (20-60percent) or high (75-99) percent of data to retain}

\item{keepOnly}{(logical)}

\item{displPlot}{(logical) show plot of hist & boundaries}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns the index of values retained or if 'keepOnly' return list with 'keep' index and 'drop' index
}
\description{
This function allows to refine/filter 'dat1' (1dim dataset, eg cluster) with aim of keeping center of data.
It is done based on most freq class of histogramm keep/filter data if 'core' (% of volume) are within 'core' (%) range
}
\examples{
set.seed(2016); dat1 <- matrix(c(runif(200)+rep(1:10,20)),ncol=10)
}
\seealso{
\code{\link{searchLinesAtGivenSlope}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lmSelClu.R
\name{lmSelClu}
\alias{lmSelClu}
\title{Run lm on segmented data (from clustering)}
\usage{
lmSelClu(
  dat,
  useCol = 1:2,
  clu = "max",
  regTy = "lin",
  filt1 = NULL,
  filt2 = NULL,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{dat}{matrix or data.frame}

\item{useCol}{(integer or charcter) specify which 2 columns of 'dat' to use for linear regression}

\item{clu}{(character) name of cluster to be extracted and treatad}

\item{regTy}{(character) change type used for linear regression :  'lin' for 1st col ~ 2nd col, 'res' for residue ~ 2nd col, 'norRes' for residue/2nd col ~2nd col or 'sqNorRes','inv' for 1st col ~ 1/(2nd col), 'invRes' for residue ~ 1/(2nd col)}

\item{filt1}{(logical or numerical) filter criteria for 1st of 'useCol' , if numeric then select all lines of dat less than max of filt1}

\item{filt2}{(logical or numerical) filter criteria for 2nd of 'useCol' , if numeric then select all lines of dat less than max of filt2}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
lm object (or NULL if no data left)
}
\description{
\code{lmSelClu} runs linear regression on data segmented previously (eg by clustering).  
This functio offers various types of (2-coefficient) linear regression on 2 columns of 'dat' (matrix with 3rd col named 'clu' or 'cluID', numeric elements for cluster-number).
If argument \code{'clu'} is (default) 'max', the column 'clu' will be inspected to take most frequent value of 'clu', otherwise a numeric entry specifying the cluster to extract is expected.
Note: this function was initially made for use with results from diagCheck()
Note: this function lacks means of judging godness of fit of the regression preformed & means for plotting
}
\examples{
set.seed(2016); ran1 <- runif(220)
mat1 <- round(rbind(matrix(c(1:100+ran1[1:100],rep(1,50)),ncol=3), 
  matrix(c(1:60,68:9+ran1[101:160],rep(2,60)),nc=3)),1)
colnames(mat1) <- c("a","BB","clu")
lmSelClu(mat1)
plot(mat1[which(mat1[,3]=="2"),1:2],col=grey(0.6))
abline(lmSelClu(mat1),lty=2,lwd=2)
# 
mat2 <- round(rbind(matrix(c(1:100+ran1[1:100],rep(1,50)),ncol=3), 
  matrix(c(1:60,(2:61+ran1[101:160])^2,rep(2,60)),nc=3)),1)
colnames(mat2) <- c("a","BB","clu")
(reg2 <- lmSelClu(mat2,regTy="sqNor"))
plot(function(x) coef(reg2)[2]+ (coef(reg2)[2]*x^2),xlim=c(1,70))
points(mat2[which(mat2[,3]=="2"),1:2],col=2)
}
\seealso{
\code{\link[stats]{lm}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/adjBy2ptReg.R
\name{.checkRegrArguments}
\alias{.checkRegrArguments}
\title{Check regression arguments}
\usage{
.checkRegrArguments(inData, refList, regreTo, callFrom = NULL)
}
\arguments{
\item{inData}{(numeric vector) main input}

\item{refList}{(list)}

\item{regreTo}{(numeric vector)}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
list
}
\description{
This function allows to check arguments for linear regression. Used as argument checking for \code{regrBy1or2point} and \code{regrMultBy1or2point}
}
\examples{
.datSlope(c(3:6))
}
\seealso{
\code{\link[base]{append}}; \code{\link{lrbind}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/adjBy2ptReg.R
\name{adjBy2ptReg}
\alias{adjBy2ptReg}
\title{Linear rescaling of numeric vertor or matrix}
\usage{
adjBy2ptReg(
  dat,
  lims,
  regrTo = c(0.1, 0.9),
  refLines = NULL,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{dat}{numeric vector, matrix or data.frame}

\item{lims}{(list, length=2) should be list giving limits (list(lo=c(min,max),hi=c(min,max)) in data allowing identifying which points will be used for determining slope & offset}

\item{regrTo}{(numeric, length=2) to which characteristics data should be regressed}

\item{refLines}{(NULL or integer) optional subselection of lines of dat (will be used internal as refDat)}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) display additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a matrix (of same dimensions as inlut matrix) with normalized values
}
\description{
\code{adjBy2ptReg} takes data within window defined by 'lims' and determines linear transformation so that these points get the regression characteristics 'regrTo', 
all other points (ie beyond the limits) will follow the same transformation.
In other words, this function performs 'linear rescaling', by adjusting (normalizing) the vector 'dat' by linear regression so that points falling in 'lims'
(list with upper & lower boundaries) will end up as 'regrTo'.
}
\examples{
set.seed(2016); dat1 <- round(runif(50,0,100),1)
## extreme values will be further away :
adjBy2ptReg(dat1,lims=list(c(5,9), c(60,90)))
plot(dat1, adjBy2ptReg(dat1, lims=list(c(5,9),c(60,90))))
}
\seealso{
\code{\link{normalizeThis}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/readXlsxBatch.R
\name{readXlsxBatch}
\alias{readXlsxBatch}
\title{Read Batch of Excel xlsx-Files}
\usage{
readXlsxBatch(
  fileNames = NULL,
  path = ".",
  fileExtension = "xlsx",
  excludeFiles = NULL,
  sheetInd = 1,
  checkFormat = TRUE,
  returnArray = TRUE,
  columns = c("Plate", "Well", "StainA"),
  simpleNames = 3,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{fileNames}{(character) provide either explicit list of file-names to be read or leave \code{NULL} for reading all files ending with 'xlsx' in path specified with argument \code{path}}

\item{path}{(character) there may be a different path for each file}

\item{fileExtension}{(character) extension of files (default='\code{xlsx}')}

\item{excludeFiles}{(character) names of files to exclude (only used when reading all files of given directory)}

\item{sheetInd}{(character or integer) specify which sheet to extract (as exact name of sheed or sheet-number, eg \code{sheetInd=2} will extract always the 2nd sheet (no matter the name); if given as sheet-name but nor present in file an empty list-elements wil be returned}

\item{checkFormat}{(logical) if \code{TRUE}: check header, remove empty columns, if rownames are increasing integeres it will searh for fisrt column with different entries to use as rownames}

\item{returnArray}{(logical) allows switching from array to list-output}

\item{columns}{(NULL or character) column-headers to be extracted (if specified, otherwise all columns will be extracted)}

\item{simpleNames}{(integer), if \code{NULL} all characters of fileNames will be maintained, otherwise allows truncating names (from beginning) to get to variable part (using .trimLeft()), but keeping at least the number of charcters indicated by this argument}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) display additional messages for debugging}

\item{callFrom}{(character) allows easier tracking of messages produced}
}
\value{
This function returns a list of data.frames
}
\description{
\code{readXlsxBatch} reads data out of multiple xlsx files, the sheet indicated by 'sheetInd' will be considered. 
All files must have the same organization of data, as this is typically the case when high-throughput measurements are automatically saved while experiments progress.
In particular, the first file read is used to structure the output.
}
\details{
By default all columns with text-content may be eliminated to keep the numeric part only, which may then get organized to a 3-dim numeric array 
 (where the additional files will be used as 2nd dimension and multiple columns per file shown as 3rd dimension). 

NOTE : (starting from version wrMisc-1.5.5) requires packages \href{https://CRAN.R-project.org/package=readxl}{readxl} and 
\href{https://CRAN.R-project.org/package=Rcpp}{Rcpp} being installed ! 
(This allows much faster and memory efficient processing than previous use of package '\code{xlsx}')
}
\examples{
path1 <- system.file("extdata", package="wrMisc")
fiNa <- c("pl01_1.xlsx","pl01_2.xlsx","pl02_1.xlsx","pl02_2.xlsx")
datAll <- readXlsxBatch(fiNa, path1)
str(datAll)
## Now let's read all xlsx files of directory
datAll2 <- readXlsxBatch(path=path1, silent=TRUE)
identical(datAll, datAll2)
}
\seealso{
\code{\link[readxl]{read_excel}}; for simple reading of (older) xls-files under 32-bit R one may also see the package \href{https://CRAN.R-project.org/package=RODBC}{RODBC}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/equLenNumber.R
\name{.composeCallName}
\alias{.composeCallName}
\title{Compose Sequence Of (Function-)Calls}
\usage{
.composeCallName(newNa, add2Head = "", add2Tail = " : ", callFrom = NULL)
}
\arguments{
\item{newNa}{(character vector) main input}

\item{add2Head}{(character)}

\item{add2Tail}{(character)}

\item{callFrom}{(character) may also contain multiple separate names (ie length >1), will be concatenated using ' -> '}
}
\value{
This function returns a character vector (history of who called whom)
}
\description{
This function was designed for tracing the hierarchy of function-calls.
It allows to remove any tailing space or ': ' from 'callFrom' (character vector) and return with added 'newNa' (+ 'add2Tail')
}
\examples{
.composeCallName("newFunction", callFrom="initFunction")
}
\seealso{
\code{\link[base]{paste}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makeMAList.R
\name{makeMAList}
\alias{makeMAList}
\title{Make MA-List Object}
\usage{
makeMAList(
  mat,
  MAfac,
  useF = c("R", "G"),
  isLog = TRUE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{mat}{main input matrix}

\item{MAfac}{(factor) factor orgnaizing columns of 'mat' (if \code{useF} contains the default 'R' and 'G', they should also be part of \code{MAfac})}

\item{useF}{(character) two specific factor-leves of \code{MAfac} that will be used/extracted}

\item{isLog}{(logical) tell if data is already log2 (will be considered when computing M and A values)}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
limma-type "MAList" containing M and A values
}
\description{
\code{makeMAList} extracts sets of data-pairs (like R & G series) and makes MA objects as \code{MA-List object} (eg for ratio oriented analysis).
The grouping of columns as sets of replicate-measurements is done according to argumnet \code{MAfac}.
The output is fully compatible to functions of package \href{https://bioconductor.org/packages/release/bioc/html/limma.html}{limma} (Bioconductor).
}
\details{
This function requires Bioconductor package \href{https://bioconductor.org/packages/release/bioc/html/limma.html}{limma} being installed.
}
\examples{
set.seed(2017); t4 <- matrix(round(runif(40,1,9),2), ncol=4,
  dimnames=list(letters[c(1:5,3:4,6:4)], c("AA1","BB1","AA2","BB2")))
makeMAList(t4, gl(2,2,labels=c("R","G")))
}
\seealso{
\code{\link{test2factLimma}}, for creating RG-lists within limma: \code{MA.RG} in \code{\link[limma]{normalizeWithinArrays}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/uniqCountReport.R
\name{.plotCountPie}
\alias{.plotCountPie}
\title{Pie plot for counting results}
\usage{
.plotCountPie(
  count,
  tit = NULL,
  col = NULL,
  radius = 0.9,
  sizeTo = NULL,
  clockwise = FALSE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{count}{(integer vector) counting result}

\item{tit}{(character) optional title in plot}

\item{col}{(character) custom colors in pie}

\item{radius}{(numeric) radius passed to  \code{pie}}

\item{sizeTo}{(numeric or charcter) optional reference group for size-population relative adjusting overall surface of pie}

\item{clockwise}{(logical) argument passed to pie}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
vector with counts of n (total), nUnique (wo any repeated), nHasRepeated (first of repeated), nRedundant), optional figure
}
\description{
This function allows to inspect results of \code{table} or \code{uniqCountReport} on a pie-plot
Note : fairly slow for long vectors !!
}
\examples{
.plotCountPie(table(c(1:5,4:2)))
}
\seealso{
\code{\link{uniqCountReport}}, \code{\link{correctToUnique}}, \code{\link[base]{unique}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkUnitPrefix.R
\name{checkUnitPrefix}
\alias{checkUnitPrefix}
\title{Check For Common Unit-Name in Text}
\usage{
checkUnitPrefix(
  x,
  pref = c("a", "f", "p", "n", "u", "m", "", "k", "M", "G", "T", "P"),
  unit = c("m", "s", "sec", "Mol", "mol", "g", "K", "cd", "A", "W", "Watt", "V", "Volt"),
  sep = c("", " ", ";", ",", "_", "."),
  sep2 = "",
  stringentSearch = FALSE,
  na.rm = FALSE,
  protSpecChar = TRUE,
  inclPat = FALSE,
  callFrom = NULL,
  silent = FALSE,
  debug = FALSE
)
}
\arguments{
\item{x}{(character) vector containing digit uunit-prefix and unit terms}

\item{pref}{(character) multiplicative unit-prefixes, assumes as increasing factors of 1000}

\item{unit}{(character) unit name, the numeric part may be sepatated by one space-character}

\item{sep}{(character) separator character(s) that may appear between integer numeric value and unit-prefix}

\item{sep2}{(character) separator character(s) after \code{unit}, set to \code{sep2=""} for ignoring characters following \code{unit}}

\item{stringentSearch}{(logical) if \code{TRUE} only matches with same separators (sep, sep2) pass, otherwise different elements may contain different separators}

\item{na.rm}{(logical) remove \code{NA} from input}

\item{protSpecChar}{(logical) protect special characters and use as they are instead of regex-meaning}

\item{inclPat}{(logical) return list including pattern of successful search}

\item{callFrom}{(character) allow easier tracking of messages produced}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}
}
\value{
This function returns a charcter vector (length=1) with the common unit name, if \code{inclPat=TRUE} it returns a list with $unit and $pattern
}
\description{
This function aims to find a unit abbreviation or name occurring in all elements of a character-vector \code{x}.
The unit name may be preceeded by different decimal prefixes (eg 'k','M'), as defined by argument \code{pref} and separators (\code{sep}). 
The unit name will be returned (or first of multiple).
}
\details{
Basically this function searches the pattern : digit + separator(\code{sep}) + prefix(\code{pref}) + \code{unit} + optional separator2(\code{sep2})
and returns the first unit-name/abbreviation found in all elements of \code{x}.

If \code{} if()
In case of invalid entries or no common unit-names \code{NULL} will be returned.

Please note the 'u' is used for 'micro' since handeling of special characters may not be portal between different operating systems.
}
\examples{
x1 <- c("10fg WW","xx 10fg 3pW"," 1pg 2.0W")
checkUnitPrefix(x1)
## different separators between digit and prefix:
x2 <- c("10fg WW","xx 8_fg 3pW"," 1 pg-2.0W")
checkUnitPrefix(x2, stringentSearch=TRUE)
checkUnitPrefix(x2, stringentSearch=FALSE)

x4 <- c("CT_mixture_QY_50_amol_CN_UPS1_CV_Standards_Research_Group",
  "CT_mixture_QY_5_fmol_CN_UPS1_CV_Standards_Research_Group")

}
\seealso{
\code{\link{convToNum}}; \code{\link{adjustUnitPrefix}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/renameColumns.R
\name{.removeEmptyCol}
\alias{.removeEmptyCol}
\title{Search for (empty) columns conaining only entries defined in 'searchFields' and remove such columns}
\usage{
.removeEmptyCol(
  dat,
  fromBackOnly = TRUE,
  searchFields = c("", " ", "NA.", NA),
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{dat}{(matrix or data.frame) main input}

\item{fromBackOnly}{(logical)}

\item{searchFields}{(character)}

\item{silent}{(logical) suppres messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allows easier tracking of messages produced}
}
\value{
This function returns a corrected matrix or data.frame
}
\description{
This function aims to search for (empty) columns conaining only entries defined in 'searchFields' and remove such columns.
If 'fromBackOnly' =TRUE .. only tailing empty columns will be removed (other columns with "empty" entries in middle will be kept).
If ''=TRUE columns containing all NAs will be excluded as well 
This function will also remove columns containing (exculsively) mixtures of the various 'searchFields'.
}
\examples{
ma1 <- matrix(c(1:5, NA), ncol=2)
.removeEmptyCol(ma1)
}
\seealso{
\code{\link{renameColumns}};  \code{\link[base]{is.finite}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makeMAList.R
\name{.getAmean}
\alias{.getAmean}
\title{Get A value for each group of replicates}
\usage{
.getAmean(dat, grp)
}
\arguments{
\item{dat}{(matrix or data.frame) main input}

\item{grp}{(factor) grouping of replicates}
}
\value{
This function returns a numeric vector
}
\description{
This function calculates the 'A' value (ie group mean) for each group of replicates (eg for MA-plot)
}
\examples{
.getAmean(matrix(11:18, ncol=4), gl(2,2))
}
\seealso{
\code{\link{makeMAList}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/matchNamesWithReverseParts.R
\name{matchNamesWithReverseParts}
\alias{matchNamesWithReverseParts}
\title{Value Matching with optional reversing of sub-parts of non-matching elements}
\usage{
matchNamesWithReverseParts(
  x,
  y,
  sep = "-",
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{x}{(character) first vector for match}

\item{y}{(character) second vector for match}

\item{sep}{(character) separator between elements}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
index for matching (integer) x to y
}
\description{
This function provides a variant to \code{\link[base]{match}}, where initially non-matching elements of \code{x}  
will be tested by decomposing non-matching elements, reversing the parts in front and after the separator \code{sep} and re-matching.
If separator \code{sep} does not occur, a warning will be issued, if it occurs more than once, 
the parts before and after the first separartor will be used and a warning issued.
}
\examples{
tx1 <- c("a-b","a-c","d-a","d-b","b-c","d-c")
tmp <- triCoord(4)
tx2 <- paste(letters[tmp[,1]],letters[tmp[,2]],sep="-")
## Some matches won't be found, since 'a-d' got reversed to 'd-a', etc... 
match(tx1,tx1)  
matchNamesWithReverseParts(tx1,tx2)
}
\seealso{
\code{\link[base]{match}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ratioAllComb.R
\name{.bringToCtr}
\alias{.bringToCtr}
\title{Bring most extreme to center}
\usage{
.bringToCtr(aa, ctr, ctrFa = 0.75)
}
\arguments{
\item{aa}{(numeric) main input}

\item{ctr}{(numeric) 'control'}

\item{ctrFa}{(numeric <1) modulate amplitude of effect}
}
\value{
This function returns an adjusted numeric vector
}
\description{
This function aims to bring most extreme value to center
}
\examples{
.bringToCtr(11:14, 9)
}
\seealso{
\code{\link[stats]{dist}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ratioAllComb.R
\name{.breakInSer}
\alias{.breakInSer}
\title{Get series of values after last discontinuity}
\usage{
.breakInSer(x, getFrom = "last")
}
\arguments{
\item{x}{(numeric) main input}

\item{getFrom}{(character)}
}
\value{
This function returns a numeric vector of reduced length
}
\description{
This function aims to get series of values after last discontinuity
}
\examples{
.breakInSer(c(11:14,16:18))
}
\seealso{
\code{\link[stats]{dist}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/findCloseMatch.R
\name{findCloseMatch}
\alias{findCloseMatch}
\title{Find Close Numeric Values Between Two Vectors}
\usage{
findCloseMatch(
  x,
  y,
  compTy = "ppm",
  limit = 5,
  asIndex = FALSE,
  maxFitShort = 100,
  sortMatch = FALSE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{x}{numeric vector for comparison}

\item{y}{numeric vector for comparison}

\item{compTy}{(character) may be 'diff' or 'ppm', will be used with threshold from argument 'limit'}

\item{limit}{(numeric) threshold value for retaining values, used with distace-type specified in argument 'compTy'}

\item{asIndex}{(logical) optionally rather report index of retained values}

\item{maxFitShort}{(numeric) limit output to max number of elements (avoid returning high number of results if filtering was not enough stringent)}

\item{sortMatch}{(logical) if TRUE than matching will be preformed as 'match longer (of x & y) to closer', this may process slightly faster (eg 'x' longer: list for each 'y' all 'x' that are close, otherwise list of each 'x'),}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a list with close matches of 'x' to given 'y', the numeric value dependes on 'sortMatch' (if FASLE then always value of 'y' otherwise of longest of x&y)
}
\description{
\code{findCloseMatch} finds close matches (similar values) between two numeric vectors ('x','y') based on method 'compTy' and threshold 'limit'. 
 Return list with close matches of 'x' to given 'y', the numeric value dependes on 'sortMatch' (if FALSE then always value of 'y' otherwise of longest of x&y).  
 Note: Speed & memory improvement if 'sortMatch'=TRUE (but result might be inversed!): adopt search of x->y or y->x to searching matches of each longest to each shorter  (ie flip x &y).
 Otherwise, if length of 'x' & 'y' are very different, it may be advantagous to use a long(er) 'x'  and short(er) 'y' (with 'sortMatch'=FALSE).  
 Note: Names of 'x' & 'y' or (if no names) prefix letters 'x' & 'y' are always added as names to results.
}
\examples{
aa <- 11:14 ; bb <- c(13.1,11.5,14.3,20:21)
findCloseMatch(aa,bb,com="diff",lim=0.6)
findCloseMatch(c(a=5,b=11,c=12,d=18),c(G=2,H=11,I=12,J=13)+0.5, comp="diff", lim=2)
findCloseMatch(c(4,5,11,12,18),c(2,11,12,13,33)+0.5, comp="diff", lim=2)
findCloseMatch(c(4,5,11,12,18),c(2,11,12,13,33)+0.5, comp="diff", lim=2, sort=FALSE)
.compareByDiff(list(c(a=10,b=11,c=12,d=13),c(H=11,I=12,J=13,K=33)+0.5),limit=1) #' return matrix

a2 <- c(11:20); names(a2) <- letters[11:20]
b2 <- c(25:5)+c(rep(0,5),(1:10)/50000,rep(0,6)); names(b2) <- LETTERS[25:5]
which(abs(b2-a2[8]) < a2[8]*1e-6*5)                                     #'  find R=18 : no10
findCloseMatch(a2, b2, com="ppm", lim=5)                                #'  find Q,R,S,T
findCloseMatch(a2, b2, com="ppm", lim=5,asI=TRUE)                       #'  find Q,R,S,T
findCloseMatch(b2, a2, com="ppm", lim=5,asI=TRUE,sort=FALSE)
findCloseMatch(a2, b2, com="ratio", lim=1.000005)                       #'  find Q,R,S,T
findCloseMatch(a2, b2, com="diff", lim=0.00005)                         #'  find S,T
}
\seealso{
\code{\link{checkSimValueInSer}} and (from this package) \code{.compareByDiff}, for convient output \code{\link{countCloseToLimits}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkStrictOrder.R
\name{.scaleSpecGrp}
\alias{.scaleSpecGrp}
\title{Rescale respective to specific group}
\usage{
.scaleSpecGrp(
  x,
  grp1Num,
  grp1Val,
  grp2Num = NULL,
  grp2Val = NULL,
  sumMeth = "mean",
  callFrom = NULL
)
}
\arguments{
\item{x}{(numeric vector) main input}

\item{grp1Num}{(numeric)}

\item{grp1Val}{(numeric)}

\item{grp2Num}{(numeric)}

\item{grp2Val}{(numeric)}

\item{sumMeth}{(character) method for summarizing}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
numeric vector
}
\description{
This function allows to rescale data 'x' so that 2 specific groups get normalized to predefined values (and all other values follow proportionally)
'grp1Num' and 'grp2Num' should be either numeric for positions in 'x' or character for names of 'x';
if 'grp1Num' and/or 'grp2Num' design mulitple locations: perform median or mean summarization, according to 'sumMeth'
}
\examples{
.firstMin(c(4,3:6))
}
\seealso{
\code{\link[base]{which.min}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkStrictOrder.R
\name{checkStrictOrder}
\alias{checkStrictOrder}
\title{Check for strict (ascencing or descending) order}
\usage{
checkStrictOrder(
  dat,
  invertCount = FALSE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{dat}{matrix or data.frame}

\item{invertCount}{(logical) inverse counting (ie return 0 for all elememts in order)}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) display additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
matrix with counts of up pairs, down pairs, equal-pairs, if 'invertCount'=TRUE all non-events are counted, ie a resulting 0 means that all columns are following the described characteristics (with variabale col-numbers easier to count)
}
\description{
\code{checkStrictOrder} tests lines of 'dat' (matrix of data.frame) for strict order (ascending, descending or constant), 
each col of data is tested relative to the col on its left.
}
\examples{
set.seed(2005); mat1 <- rbind(matrix(round(runif(40),1),nc=4), rep(1,4))
checkStrictOrder(mat1); mat1[which(checkStrictOrder(mat1)[,2]==0),]
}
\seealso{
\code{\link[base]{order}}, \code{\link{checkGrpOrder}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cutArrayInCluLike.R
\name{.checkConsistentArrList}
\alias{.checkConsistentArrList}
\title{Check list of arrays for consistent dimensions of all arrays}
\usage{
.checkConsistentArrList(
  arrLst,
  arrNDim = 3,
  fxName = NULL,
  varName = NULL,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{arrLst}{(list) main input}

\item{arrNDim}{(integer) number of dimensions for arrays}

\item{fxName}{(character) this name will be given in message}

\item{varName}{(character)}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
list
}
\description{
This function allows to check list of arrays for consistent dimensions of all arrays
}
\examples{
.datSlope(c(3:6))
}
\seealso{
used in \code{\link{cutArrayInCluLike}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stdErrMedBoot.R
\name{stdErrMedBoot}
\alias{stdErrMedBoot}
\title{Standard Eror Of Median by Boot-Strap}
\usage{
stdErrMedBoot(x, nBoot = 9, silent = FALSE, debug = FALSE, callFrom = NULL)
}
\arguments{
\item{x}{(numeric) vector to estimate median and it's standard error}

\item{nBoot}{(integer) number for iterations}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) display additional messages for debugging}

\item{callFrom}{(character) allows easier tracking of messages produced}
}
\value{
This function returns a (numeric) vector with estimated standard error
}
\description{
\code{stdErrMedBoot} estimate standard eror of median by bootstrap approach. 
Note: requires package \href{https://CRAN.R-project.org/package=boot}{boot}
}
\examples{
set.seed(2014); ra1 <- c(rnorm(9,2,1),runif(8,1,2))
rat1 <- ratioAllComb(ra1[1:9],ra1[10:17])
median(rat1); stdErrMedBoot(rat1)
}
\seealso{
\code{\link{colMedSds}} and \code{\link{rowMedSds}}; based on \code{\link[boot]{boot}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extrNumericFromMatr.R
\name{extrNumericFromMatr}
\alias{extrNumericFromMatr}
\title{Extract numeric part of matrix or data.frame}
\usage{
extrNumericFromMatr(
  dat,
  trimToData = TRUE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{dat}{matrix (or data.frame) for extracting numeric parts}

\item{trimToData}{(logical) default to remove (crop) lines and cols contributing to NA, non-numeric data is transfomed to NA}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of message(s) produced}
}
\value{
This function reurns a matrix of numeric data
}
\description{
This function extracts numeric part of matrix or data.frame, removing remaining non-numeric elements if \code{trimToData} is set to \code{TRUE}.
Note, that cropping entire lines where a (single) text element appeared may quickly reduce the overal content of the input data.
}
\examples{
mat <- matrix(c(letters[1:7],14:16,LETTERS[1:6]),nrow=4,dimnames=list(1:4,letters[1:4]))
mat; extrNumericFromMatr(mat)
mat <- matrix(c(letters[1:4],1,"e",12:19,LETTERS[1:6]),nr=5,dimnames=list(11:15,letters[1:4]))
mat; extrNumericFromMatr(mat)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/findSimilFrom2sets.R
\name{findSimilFrom2sets}
\alias{findSimilFrom2sets}
\title{Find similar numeric values from two vectors/matrixes}
\usage{
findSimilFrom2sets(
  predMatr,
  measMatr,
  colMeas = 1,
  colPre = 1,
  compareTy = "diff",
  limitComp = 0.5,
  bestOnly = FALSE,
  silent = FALSE,
  callFrom = NULL,
  debug = FALSE
)
}
\arguments{
\item{predMatr}{(matrix or numeric vector) dataset number 1, referred to as 'predicted', the colum speified in argument \code{colPre} points to the data to be used}

\item{measMatr}{(matrix or numeric vector) dataset number 2, referred to as 'measured', the colum speified in argument \code{colMeas} points to the data to be used}

\item{colMeas}{(integer) which column number of 'measMatr' to consider}

\item{colPre}{(integer) which column number of 'predMatr' to consider}

\item{compareTy}{(character) 'diff' (difference) 'ppm' (relative difference)}

\item{limitComp}{(numeric) limit used by 'compareTy'}

\item{bestOnly}{(logical) allows to filter only hits with min distance (defined by 'compareTy'), 3rd last col will be 'nBest' - otherwise 3rd last col 'isBest'}

\item{silent}{(logical) suppress messages}

\item{callFrom}{(character) allow easier tracking of messages produced}

\item{debug}{(logical) for bug-tracking: more/enhanced messages}
}
\value{
This function returns a matrix (predMatr) with add'l columns for index to and 'grp' (group of similar values (1-to-many)), 'nGrp' (n of grp), 'isBest' or 'nBest', 'disToMeas' (distance/difference between pair) &  'ppmToPred' (distance in ppm)
}
\description{
\code{findSimilFrom2sets} compares to vectors or matrixes and returns combined view including only all close (by \code{\link[wrMisc]{findCloseMatch}}).
Return matrix (predMatr) with add'l columns for index to and 'grp' (group of similar values (1-to-many)), 'nGrp' (n of grp), 'isBest' or 'nBest', 'disToMeas' 
(distance/difference between pair) &  'ppmToPred' (distance in ppm).
Note: too wide 'limitComp' will result in large window and many 'good' hits will compete (and be mutually exlcuded) if selection 'bestOnly' is selected
}
\examples{
aA <- c(11:17); bB <- c(12.001,13.999); cC <- c(16.2,8,9,12.5,12.6,15.9,14.1)
aZ <-  matrix(c(aA,aA+20),ncol=2,dimnames=list(letters[1:length(aA)],c("aaA","aZ")))
cZ <-  matrix(c(cC,cC+20),ncol=2,dimnames=list(letters[1:length(cC)],c("ccC","cZ")))
findCloseMatch(cC,aA,com="diff",lim=0.5,sor=FALSE)
findSimilFrom2sets(aA,cC)
findSimilFrom2sets(cC,aA)
findSimilFrom2sets(aA,cC,best=FALSE)
findSimilFrom2sets(aA,cC,comp="ppm",lim=5e4,deb=TRUE)
findSimilFrom2sets(aA,cC,comp="ppm",lim=9e4,bestO=FALSE)
# below: find fewer 'best matches' since search window larger (ie more good hits compete !)
findSimilFrom2sets(aA,cC,comp="ppm",lim=9e4,bestO=TRUE)      
}
\seealso{
\code{\link{checkSimValueInSer}} \code{\link{findCloseMatch}} \code{\link{closeMatchMatrix}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/uniqCountReport.R
\name{uniqCountReport}
\alias{uniqCountReport}
\title{Report number of unique and redundant elements (optional figure)}
\usage{
uniqCountReport(
  dat,
  frL = NULL,
  plotDispl = FALSE,
  tit = NULL,
  col = NULL,
  radius = 0.9,
  sizeTo = NULL,
  clockwise = FALSE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{dat}{(charcter or numeric vector) main input where number of unique (and redunant) should be determined}

\item{frL}{(logical) optional (re-)introducing results from \code{duplicated} to shorten time of execution}

\item{plotDispl}{(logical) decide if pie-type plot should be produced}

\item{tit}{(character) optional title in plot}

\item{col}{(character) custom colors in pie}

\item{radius}{(numeric) radius passed to  \code{pie}}

\item{sizeTo}{(numeric or charcter) optional reference group for size-population relative adjusting overall surface of pie}

\item{clockwise}{(logical) argument passed to pie}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
vector with counts of n (total), nUnique (wo any repeated), nHasRepeated (first of repeated), nRedundant), optional figure
}
\description{
Make report about number of unique and redundant elements of vector 'dat'.
Note : fairly slow for long vectors !!
}
\examples{
layout(1:2)
uniqCountReport(rep(1:7,1:7),plot=TRUE)
uniqCountReport(rep(1:3,1:3),plot=TRUE,sizeTo=rep(1:7,1:7))      
}
\seealso{
\code{\link{correctToUnique}}, \code{\link[base]{unique}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/elimCloseCoord.R
\name{elimCloseCoord}
\alias{elimCloseCoord}
\title{Eliminate close (overlapping) points (in x & y space)}
\usage{
elimCloseCoord(
  dat,
  useCol = 1:2,
  elimIdentOnly = FALSE,
  refine = 2,
  nDig = 3,
  callFrom = NULL,
  silent = FALSE
)
}
\arguments{
\item{dat}{matrix (or data.frame) with main numeric input}

\item{useCol}{(numeric) index for numeric columns of 'dat' to use/consider}

\item{elimIdentOnly}{(logical) if TRUE, eliminate real duplicated points only (ie identical values only)}

\item{refine}{(numeric) allows increasing stringency even further  (higher 'refine' .. more lines considered equal)}

\item{nDig}{(integer) number of significant digits used for rounding, if two 'similar' values are identical after this rounding the second will be eliminated.}

\item{callFrom}{(character) allows easier tracking of message(s) produced}

\item{silent}{(logical) suppress messages}
}
\value{
resultant matrix/data.frame
}
\description{
\code{elimCloseCoord} reduces number of rows in 'dat' by eliminating lines where x & y coordinates (columns of matrix '\code{dat}' defined by '\code{useCol}') are identical (overlay points) or very close.
The stringency for 'close' values may be fine-tuned using \code{nDig}), this function uses internally \code{\link{firstOfRepeated}}.
}
\examples{
da1 <- matrix(c(rep(0:4,5),0.01,1.1,2.04,3.07,4.5),nc=2); da1[,1] <- da1[,1]*99; head(da1)
elimCloseCoord(da1)
}
\seealso{
\code{\link{findCloseMatch}}, \code{\link{firstOfRepeated}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/firstOfRepLines.R
\name{.pasteCols}
\alias{.pasteCols}
\title{Paste-concatenate all columns of matrix}
\usage{
.pasteCols(mat, sep = "")
}
\arguments{
\item{mat}{inital matrix}

\item{sep}{(character) separator}
}
\value{
This function returns a simplified/non-redundant vector/matrix (ie fewer lines for matrix), or respective index
}
\description{
This function allows paste columns
}
\examples{
.pasteCols(matrix(11:16,ncol=2), sep="_")
}
\seealso{
\code{\link[base]{unique}}, \code{\link{nonAmbiguousNum}}, faster than \code{\link{firstOfRepeated}} which gives more detail in output (lines/elements/indexes of omitted)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rowGrpCV.R
\name{rowGrpCV}
\alias{rowGrpCV}
\title{Row group CV}
\usage{
rowGrpCV(x, grp, means = NULL, listOutp = FALSE)
}
\arguments{
\item{x}{numeric matrix where relplicates are organized into separate columns}

\item{grp}{(factor) defining which columns should be grouped (considered as replicates)}

\item{means}{(numeric) alternative values instead of means by .rowGrpMeans()}

\item{listOutp}{(logical) if TRUE, provide output as list with $CV, $mean and $n}
}
\value{
This function returns a matrix of CV values
}
\description{
This function calculates CVs for matrix with multiple groups of data, ie one CV for each group of data. 
Groups are specified as columns of 'x' in 'grp' (so length of grp should match number of columns of 'x', NAs are allowed)
}
\examples{
set.seed(2016); dat1 <- matrix(c(runif(200)+rep(1:10,20)),ncol=10)
head(rowGrpCV(dat1, gr=gl(4,3,labels=LETTERS[1:4])[2:11]))
}
\seealso{
\code{\link{rowCVs}}, \code{\link{arrayCV}},  \code{\link{replPlateCV}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filterNetw.R
\name{.filterNetw}
\alias{.filterNetw}
\title{Filter nodes & edges for extracting networks (main)
 
This function allows extracting and filtering network-data based on fixed threshold (\code{limInt}) and add sandwich-nodes (nodes inter-connecting initial nodes) out of node-based queries.}
\usage{
.filterNetw(
  lst,
  remOrphans = TRUE,
  reverseCheck = TRUE,
  filtCol = 2,
  callFrom = NULL,
  silent = FALSE,
  debug = FALSE
)
}
\arguments{
\item{lst}{(list, composed of multiple matrix or data.frames ) main input (each list-element should have same number of columns)}

\item{remOrphans}{(logical) remove networks consisting only of 2 connected edges}

\item{reverseCheck}{(logical)}

\item{filtCol}{(integer, length=1) which column of \code{lst} should be usd to filter using thresholds \code{limInt} and \code{sandwLim}}

\item{callFrom}{(character) allow easier tracking of message(s) produced}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) display additional messages for debugging}
}
\value{
This function returns a matrix or data.frame
}
\description{
Filter nodes & edges for extracting networks (main)
 
This function allows extracting and filtering network-data based on fixed threshold (\code{limInt}) and add sandwich-nodes (nodes inter-connecting initial nodes) out of node-based queries.
}
\examples{
ab <- 1:10  
}
\seealso{
\code{\link{filterNetw}} and other CRAN package dedeicated to networks
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/combineSingleT.R
\name{combineSingleT}
\alias{combineSingleT}
\title{Get All Combinations With TRUE From Each Column}
\usage{
combineSingleT(mat, silent = FALSE, debug = FALSE, callFrom = NULL)
}
\arguments{
\item{mat}{2-column matrix of logical values}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allows easier tracking of messages produced}
}
\value{
This function returns a matrix with indexes of conbinations of \code{TRUE}
}
\description{
This function addresses the case when multiple alternatove ways exit to combine two elements. 
\code{combineSingleT} makes combinatory choices : if multiple \code{TRUE} in given column of 'mat' make all multiple selections with always one \code{TRUE} from each column
The resultant output contains index for first and second input columns elements to be combined.
}
\examples{
## Example: Fist column indicates which boys want to dance and second column  
## which girls want to dance. So if several boys want to dance each of the girls 
## will have the chance to dance with each of them.  
matr <- matrix(c(TRUE,FALSE,TRUE,FALSE,TRUE,FALSE),ncol=2)
combineSingleT(matr)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/searchLinesAtGivenSlope.R
\name{searchLinesAtGivenSlope}
\alias{searchLinesAtGivenSlope}
\title{Search Points Forming Lines At Given Slope}
\usage{
searchLinesAtGivenSlope(
  dat,
  coeff = 1.5,
  filtExtr = c(0, 1),
  minMaxDistThr = NULL,
  lmCompare = TRUE,
  indexPoints = TRUE,
  displHist = FALSE,
  displScat = FALSE,
  bestCluByDistRat = TRUE,
  neighbDiLim = NULL,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{dat}{matrix or data.frame, main input}

\item{coeff}{(numeric) slope to consider}

\item{filtExtr}{(integer) lower & upper quantile values, remove points with extreme deviation to offset=0, (if single value: everything up to or after will be used)}

\item{minMaxDistThr}{(logical) optional minumum and maximum distance threshold}

\item{lmCompare}{(logical) add'l fitting of linear regression to best results, return offset AND slope based on lm fit}

\item{indexPoints}{(logical) return results as list with element 'index' specifying retained points}

\item{displHist}{(logical) display histogram of residues}

\item{displScat}{(logical) display (simple) scatter plot}

\item{bestCluByDistRat}{(logical) initial selection of decent clusters based on ratio overallDist/averNeighbDist (or by CV & cor)}

\item{neighbDiLim}{(numeric) additional threshold for (trimmed mean) neighbour-distance}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) for bug-tracking: more/enhanced messages}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This functions returns a matrix of line-characteristics  (or if indexPoints is \code{TRUE} then list (line-characteristics & index & lm-results)
}
\description{
\code{searchLinesAtGivenSlope} searchs among set of points (2-dim) those forming line(s) with user-defined slope ('coeff'),
 ie search optimal (slope-) offset parameter(s) for (regression) line(s) with given slope ('coef').
 Note: larger data-sets : segment residuals to 'coeff' & select most homogenous
}
\details{
Note: The package MASS is required when using as \code{lmCompare=TRUE}.
For larger data the function will try using the package \code{NbClust} (available from CRAN) if installed.
}
\examples{
set.seed(2016); ra1 <- runif(300)
 dat1 <- cbind(x=round(c(1:100+ra1[1:100]/5,4*ra1[1:50]),1),
  y=round(c(1:100+ra1[101:200]/5, 4*ra1[101:150]), 1))
(li1 <- searchLinesAtGivenSlope(dat1, coeff=1))
}
\seealso{
\code{\link[stats]{lm}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/upperMaCoord.R
\name{upperMaCoord}
\alias{upperMaCoord}
\title{(upper) pairwise x,y combinations}
\usage{
upperMaCoord(n, silent = FALSE, debug = FALSE, callFrom = NULL)
}
\arguments{
\item{n}{(integer) number of elements for making all pair-wise combinations}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a 2-column matrix wiyh indexes for all pairwise combinations of 1:n
}
\description{
\code{upperMaCoord} gets pairwise combinations for 'n' elements; return matrix with x & y coordinates to form all pairwise groups for n elements.
But no distinction of 'upper' or 'lower' possible like in \code{\link{triCoord}}
}
\examples{
upperMaCoord(4)
}
\seealso{
\code{\link[base]{lower.tri}}, more evolved version \code{\link{triCoord}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rmOrphans.R
\name{rmOrphans}
\alias{rmOrphans}
\title{Remove or Reassign Orphan Indexes}
\usage{
rmOrphans(
  ind,
  minN = 1,
  reassign = FALSE,
  side = "both",
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{ind}{(integer) main input of (cluster-) indexes}

\item{minN}{(numeric, length=1) the min frequency to consider as orphans, if less than 1 it will be interpreted as ratio compared to length of \code{index}}

\item{reassign}{(logical) if \code{TRUE} orphan indexes will be replaced by neighbour class indexes (towrds the center)}

\item{side}{(character) may be 'both', 'b', 'upper', 'u', 'lower' or 'l' to decide if lower and/or upper end indexes should be treated.}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allows easier tracking of messages produced}
}
\value{
This function returns an integer vector of adjusted indexes
}
\description{
This function allows detecting terminal orphans of a vector of (cluster-) indexes and removing (ie marking as \code{NA})
  or re-assigning them to the neighbour class towrds the center.
}
\details{
All input of \code{ind} is supposed to be interger values as (cluster-) indexes.
This function will look if the lowest and/or highest (cluster-) indexes appear at very low frequency so that they may be considered orphans. 
The argument \code{minN} assigns the threshold of when the frquency of terminal values may be considered as 'orphan',
either as absolute threshold or if less than 1 as ratio (0.1 => 10%tile of length of \code{ind}) .

The argument \code{side} may be 'both', 'b', 'upper', 'u', 'lower' or 'l', to decide if lower and/or upper end indexes should be treated.
}
\examples{
 x=c(3:1,3:4,4:6,5:3); rmOrphans(x)
rmOrphans(x, minN=0.2) 
## reassign orphans to neighbour center class
cbind(x,  x=x, def=rmOrphans(x, reassign=TRUE), 
  minN=rmOrphans(x, minN=0.2, reassign=TRUE) )
}
\seealso{
\code{\link[base]{table}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makeMAList.R
\name{.allRatioMatr1to2}
\alias{.allRatioMatr1to2}
\title{Calculate ratios for each column to each column of reference-matrix}
\usage{
.allRatioMatr1to2(x, y, asLog2 = TRUE, sumMeth = "mean", callFrom = NULL)
}
\arguments{
\item{x}{(matrix or data.frame) main input1}

\item{y}{(matrix or data.frame) main input2}

\item{asLog2}{(logical)}

\item{sumMeth}{(character) method}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a numeric vector or matrix in dimension of 'x' (so far summarize all ratios from mult division from mult ref cols as mean or median )
}
\description{
This function calculates ratio(s) for each column of matrix 'x' versus all/each column(s) of matrix 'y' (reference)
}
\examples{
.allRatioMatr1to2(matrix(11:14, ncol=2), matrix(21:24, ncol=2))
}
\seealso{
\code{\link{makeMAList}}, \code{\link[base]{grep}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/triCoord.R
\name{triCoord}
\alias{triCoord}
\title{Pairwise x,y Combinations}
\usage{
triCoord(n, side = "upper", silent = FALSE, debug = FALSE, callFrom = NULL)
}
\arguments{
\item{n}{(integer) number of elements for making all pair-wise combinations}

\item{side}{(character) "upper" or "lower"}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a 2-column matrix with indexes for all pairwise combinations of 1:n
}
\description{
This function gets pairwise combinations for 'n' elements; returns matrix with x & y coordinates to form all pairwise groups for 1:n elements
}
\examples{
triCoord(4)
}
\seealso{
\code{\link{upperMaCoord}}, \code{\link{getPWseparator}}, \code{\link{indexGroupsFromPW}}, \code{\link{diffCombin}}; \code{\link[utils]{combn}}, \code{\link[base]{lower.tri}} or \code{upper.tri}, simpler version \code{\link{upperMaCoord}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/completeArrLst.R
\name{completeArrLst}
\alias{completeArrLst}
\title{Complete list of arrays for same dimensions}
\usage{
completeArrLst(arrLst, silent = FALSE, debug = FALSE, callFrom = NULL)
}
\arguments{
\item{arrLst}{(list) list of arrays (typically 1st and 2nd dim for specific genes/objects, 3rd for different measures associated with)}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allows easier tracking of message(s) produced}
}
\value{
This function returns a list of arrays, now with same dimension of arrays
}
\description{
This functions aims to inspect repeating structues of data given as list of arrays and will try to complete 
arrays with fewer lines or columns (as this may appear eg with the very last set of high-thourput sceening data 
if fewer measures remain in the last set). Thus, the dimensions of the arrays are compared and 
cases with fewer (lost) columns (eg fewer experimental replicates) will be adjust/complete  by adding column(s) of NA.
Used eg when at reading mircotiterplate data the last set is not complete.
}
\examples{
arr1 <- array(1:24,dim=c(4,3,2),dimnames=list(c(LETTERS[1:4]),
  paste("col",1:3,sep=""),c("ch1","ch2")))
arr3 <- array(81:96,dim=c(4,2,2),dimnames=list(c(LETTERS[1:4]),
  paste("col",1:2,sep=""),c("ch1","ch2")))
arrL3 <- list(pl1=arr1,pl3=arr3)
completeArrLst(arrL3)
}
\seealso{
\code{\link[wrMisc]{organizeAsListOfRepl}}, \code{\link[wrMisc]{extr1chan}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/buildTree.R
\name{.growTree}
\alias{.growTree}
\title{Grow Tree}
\usage{
.growTree(tm, setX, addToObj = NULL)
}
\arguments{
\item{tm}{(list) main input, $disDat .. matrix with integer start & end sites for fragments; $lo (logical) which fragments may be grown; $start (integer) index for which line of $disDat to start; $it numeric version of $lo; $preN for previous tree objects towards root; $iter for iterator (starting at 1))}

\item{setX}{.. data.tree object (main obj from root)}

\item{addToObj}{.. data.tree object (branch on which to add new branches/nodes)}
}
\value{
This function returns a list
}
\description{
This function allows growing tree-like structures (data.tree objects)
}
\examples{
.datSlope(c(3:6))
}
\seealso{
\code{\link{buildTree}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ratioAllComb.R
\name{ratioAllComb}
\alias{ratioAllComb}
\title{Calculate all ratios between x and y}
\usage{
ratioAllComb(
  x,
  y,
  maxLim = 10000,
  isLog = FALSE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{x}{(numeric) vector, numerator for constructing rations}

\item{y}{(numeric) vector, denominator for constructing rations}

\item{maxLim}{(integer) allows reducing complexity by drawing for very long x or y}

\item{isLog}{(logical) adjust ratio calculation to log-data}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\description{
This function calculates all possible pairwise ratios between all individual calues of x and y, or samples up to a maximum number of combinations.
}
\examples{
set.seed(2014); ra1 <- c(rnorm(9,2,1),runif(8,1,2))
ratioAllComb(ra1[1:9],ra1[10:17])
boxplot(list(norm=ra1[1:9], unif=ra1[10:17], rat=ratioAllComb(ra1[1:9],ra1[10:17])))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/equLenNumber.R
\name{equLenNumber}
\alias{equLenNumber}
\title{Equal character-length number}
\usage{
equLenNumber(x, silent = FALSE, callFrom = NULL, debug = FALSE)
}
\arguments{
\item{x}{(caracter) input vector}

\item{silent}{(logical) suppress messages}

\item{callFrom}{(character) allow easier tracking of messages produced}

\item{debug}{(logical) additional messages for debugging}
}
\value{
This function returns a character vector formated as equal number of characters per value
}
\description{
\code{equLenNumber} convert numeric entry 'x' to text, with all elements getting the same number of characters (ie  by adding preceeding or tailing 0s, if needed).
So far, this function cannot handle scientific annotations.
}
\examples{
equLenNumber(c(12,-3,321))
equLenNumber(c(12,-3.3,321))
}
\seealso{
\code{\link[base]{sprintf}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sortByNRepeated.R
\name{sortByNRepeated}
\alias{sortByNRepeated}
\title{Make a list of common occurances sorted by number of repeats}
\usage{
sortByNRepeated(
  x,
  y = NULL,
  z = NULL,
  filterIntraRep = TRUE,
  silent = TRUE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{x}{(list, character or integer) main input, if list, arguments \code{y} and \code{z} will not be used}

\item{y}{(character or integer) supplemental vector to comare with \code{x}}

\item{z}{(character or integer) supplemental vector to comare with \code{x}}

\item{filterIntraRep}{(logical) allow making vectors \code{x}, \code{y} and \code{z} unique before comparing (defaults to \code{TRUE})}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a list sorted by number of occurances. The names of the list indicate the number of repeats.
}
\description{
The aim of this function is to count the number of occurances of words when comaring separate vectors (\code{x}, \code{y} and \code{z}) or from a list (given as \code{x})
and to give an output sorted by their frequency.
The output lists the various values/words by their frequency, the names of the resulting list-elements indicate number of times the values/words were found repeated.
}
\details{
In order to compare the frquency of values/words between separate vectors or vectors within a list, it is necessary that these have been made unique before calling this function or using \code{filterIntraRep=TRUE}. 

In case the input is given as list (in \code{x}), there is no restriction to the number of vectors to be compared. 
With very long lists, however, the computational effort incerases (like it does when using \code{table})
}
\examples{
sortByNRepeated(x=LETTERS[1:11], y=LETTERS[3:13], z=LETTERS[6:12])
sortByNRepeated(x=LETTERS[1:11], y=LETTERS[c(3:13,5:4)], z=LETTERS[6:12])

}
\seealso{
\code{\link[base]{table}},  \code{\link[wrMisc]{replicateStructure}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rowGrpSums.R
\name{.rowGrpSums}
\alias{.rowGrpSums}
\title{row group rowSums per group (main)}
\usage{
.rowGrpSums(x, grp, na.replVa = NULL, na.rm = TRUE)
}
\arguments{
\item{x}{numeric matrix where relplicates are organized into separate columns}

\item{grp}{(factor) defining which columns should be grouped (considered as replicates)}

\item{na.replVa}{(numeric) value to replace \code{NA} values}

\item{na.rm}{(logical) remove all \code{NA} values}
}
\value{
This function returns a matrix of row-sums for matrix with multiple groups of data
}
\description{
This function calculates row-sums for matrix with multiple groups of data, with multiple groups of data, ie one sd for each group of data.
}
\examples{
set.seed(2016); dat1 <- matrix(c(runif(200)+rep(1:10,20)),ncol=10)
grp1 <- gl(4,3,labels=LETTERS[1:4])[2:11]
head(.rowGrpSums(dat1, grp1)) 
}
\seealso{
\code{\link{rowGrpCV}}, \code{\link{rowCVs}}, \code{\link{arrayCV}}, \code{\link{replPlateCV}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extrSpcText.R
\name{extrSpcText}
\alias{extrSpcText}
\title{Extract specific text}
\usage{
extrSpcText(
  txt,
  cutFrom = " GN=",
  cutTo = " PE=",
  missingAs = NA,
  exclFromTag = TRUE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{txt}{character vector to be treated}

\item{cutFrom}{(character) text where to start cutting}

\item{cutTo}{(character) text where to stop cutting}

\item{missingAs}{(character) specific content of output at line/location of 'exclLi'}

\item{exclFromTag}{(logical) to exclude text given in 'cutFrom' from result}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a modified character vector
}
\description{
This function extracts/cuts text-fragments out of \code{txt} following specific anchors defined by arguments \code{cutFrom} and \code{cutTo}.
}
\details{
In case \code{cutFrom} is not found \code{missingAs} will be returned.
In case \code{cutTo} is not found, text gets extracted with \code{chaMaxEl} characters.
}
\examples{
extrSpcText(c(" ghjg GN=thisText PE=001"," GN=_ PE=", NA, "abcd"))
extrSpcText(c("ABCDEF.3-6","05g","bc.4-5"), cutFr="\\\\.", cutT="-")
}
\seealso{
\code{\link[base]{substr}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mergeW2.R
\name{mergeW2}
\alias{mergeW2}
\title{Extended version of merge for multiple objects (even without rownames)}
\usage{
mergeW2(
  ...,
  nonRedundID = TRUE,
  convertDF = TRUE,
  selMerg = TRUE,
  inputNamesLst = NULL,
  noMatchPursue = TRUE,
  standColNa = FALSE,
  lastOfMultCols = c("p.value", "Lfdr"),
  duplTxtSep = "_",
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{...}{all data (vectors, matrixes or data.frames) intendes for merge}

\item{nonRedundID}{(logical) if TRUE, allways add 1st column with non-redundant IDs (add anyway if non-redundant IDs found )}

\item{convertDF}{(logical) allows converting output in data.frame, add new heading col with non-red rownames & check which cols should be numeric}

\item{selMerg}{(logical) if FALSE toggle to classic merge() (will give more rows in output in case of redundant names}

\item{inputNamesLst}{(list) named list with character vectors (should be unique), search these names in input for extracting/merging elements use for 'lazy matching' when checking names of input, default : 7 groups ('Mvalue', 'Avalue','p.value','mouseInfo','Lfdr','link','filt') with common short versions}

\item{noMatchPursue}{(logical) allows using entries where 0 names match (just as if no names given)}

\item{standColNa}{(logical) if TRUE return standard colnames as defined in 'inputNamesLst' (ie 'chInp'), otherwise colnames as initially provided}

\item{lastOfMultCols}{may specify input groups where only last col will be used/extracted}

\item{duplTxtSep}{(character)  separator for counting/denomiating multiple occurances of same name}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) for bug-tracking: more/enhanced messages and intermediate objects written in global name-space}

\item{callFrom}{(character) allows easier tracking of message(s) produced}
}
\value{
matrix or data.frame of fused data
}
\description{
\code{mergeW2} povides flexible merging out of 'MArrayLM'-object (if found, won't consider any other input-data) or of separate vectors or matrixes.
The main idea was to have somthing not adding add'l lines as merge might do, but to stay within the frame of the 1st argument given, even when IDs are repeated, 
so the output follows the order of the 1st argument, non-redundant IDs are created (orig IDs as new column).
If no 'MArrayLM'-object found: try to combine all elements of input '...', input-names must match predefined variants 'chInp'.
IDs given in 1st argument and not found in later arguments will be displayed as NA in the output matrix of data.frame.  
 Note : (non-data) arguments must be given with full name (so far no lazy evaluation, may conflict with names in 'inputNamesLst'). 
 Note : special characters in colnames bound to give trouble. 
 Note : when no names given, \code{mergeW2} will presume order of elements (names) from 'inputNamesLst'. 
PROBLEM : error after xxMerg3 when several entries have matching (row)names but some entries match only partially  (what to do : replace with NAs ??)
}
\examples{
t1 <- 1:10; names(t1) <- letters[c(1:7,3:4,8)]
t2 <- 20:11;  names(t2) <- letters[c(1:7,3:4,8)]
t3 <- 101:110; names(t3) <- letters[c(11:20)]
t4 <- matrix(100:81,ncol=2,dimnames=list(letters[1:10],c("co1","co2")))
t5 <- cbind(t1=t1,t52=t1+20,t53=t1+30)
  t1; t2; t3;  cbind(t1,t2)
mergeW2(Mval=t1,p.value=t2,debug=FALSE)
}
\seealso{
\code{\link[base]{merge}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkVectLength.R
\name{.convertMatrToNum}
\alias{.convertMatrToNum}
\title{Convert numeric matrix to numeric}
\usage{
.convertMatrToNum(matr, useCol = NULL)
}
\arguments{
\item{matr}{(matrix) main input}

\item{useCol}{(integer) design the comumns to be used}
}
\value{
numeric vector
}
\description{
Take matrix and return vector
}
\examples{
.convertMatrToNum(matrix(1:6, ncol=2))
}
\seealso{
\code{\link[base]{matrix}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cleanReplicates.R
\name{cleanReplicates}
\alias{cleanReplicates}
\title{Replace Most Distant Values by NA}
\usage{
cleanReplicates(
  x,
  centrMeth = "median",
  nOutl = 2,
  retOffPos = FALSE,
  silent = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{x}{matrix (or data.frame)}

\item{centrMeth}{(character) method to summarize (mean or median)}

\item{nOutl}{(integer) determines how many points per line will be set to \code{NA} (with n=1 the worst row of replicates will be 'cleaned')}

\item{retOffPos}{(logical) if \code{TRUE}, replace the most extreme outlyer only}

\item{silent}{(logical) suppres messages}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a matrix of same dimensions as input \code{x},  data-points which were tagged/removed are set to \code{NA}, or if \code{retOffPos=TRUE} the most extreme/outlier positions
}
\description{
This procedures aims to streighten (clean) the most extreme values of noisy replicates by identifying the most distant points 
(among a set of replicates). The input 'x' (matrix or data.frame) is supposed to come from multiple different measures taken 
in replicates (eg weight of different individuals as rows taken as multiple replicate measures in subsequent columns).
}
\details{
With the argument \code{nOutl} the user chooses the total number of most extreme values to replace by \code{NA}.
how many of the most extreme replicates of the whole dataset will replaced by \code{NA}, ie with \code{nOutl=1} 
only the single most extreme outlyer will be replaced by \code{NA}. 
Outlier points are determined as point(s) with highest distance to (row) center (median and mean choice via argument 'centrMeth').
Thus function returns input data with "removed" points set to \code{NA}, or if \code{retOffPos=TRUE} the most extreme/outlier positions.
}
\examples{
mat3 <- matrix(c(19,20,30, 18,19,28, 16,14,35),ncol=3)
cleanReplicates(mat3, nOutl=1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rnormW.R
\name{rnormW}
\alias{rnormW}
\title{Normal random number generation with close fit to expected mean and sd}
\usage{
rnormW(
  n,
  mean = 0,
  sd = 1,
  seed = NULL,
  digits = 8,
  silent = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{n}{(integer, length=1)  number of observations. If \code{length(n) > 1}, the length is taken to be the number required.}

\item{mean}{(numeric, length=1) expected mean}

\item{sd}{(numeric, length=1) expected sd}

\item{seed}{(integer, length=1) seed for generating random numbers}

\item{digits}{(integer, length=1 or \code{NULL}) number of significant digits for output, set to \code{NULL} to get all digits}

\item{silent}{(logical) suppress messages}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a numeric vector of random values
}
\description{
This function allows creating a vector of random values similar to \code{rnorm}, but resulting value get recorrected to fit to expected mean and sd.
When the number of random values to generate is low, the mean and sd of the resultant values may deviate from the expected mean and sd when using the standard \code{rnorm} function.
In such cases the function \code{rnormW} helps getting much closer to the expected mean and sd.
}
\details{
For making result reproducible, a seed for generating random numbers can be set via the argument \code{seed}.
However, with \code{n=2} the resulting values are 'fixed' since no random component is possible at n <3.
}
\examples{
x1 <- (11:16)[-5]
mean(x1); sd(x1)
## the standard way
ra1 <- rnorm(n=length(x1), mean=mean(x1), sd=sd(x1))
## typically the random values deviate (slightly) from expected mean and sd
mean(ra1) -mean(x1) 
sd(ra1) -sd(x1)
## random numbers with close fit to expected mean and sd :
ra2 <- rnormW(length(x1), mean(x1), sd(x1))
mean(ra2) -mean(x1) 
sd(ra2) -sd(x1)          # much closer to expected value
}
\seealso{
\code{\link[stats]{Normal}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/randIndFx.R
\name{randIndFx}
\alias{randIndFx}
\title{Distance of categorical data (Jaccard, Rand and adjusted Rand index)}
\usage{
randIndFx(
  ma,
  method = "ARI",
  adjSense = TRUE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{ma}{(matrix) main input for distance calulation}

\item{method}{(character) name of distance method (eg "ARI","RI","J","FM")}

\item{adjSense}{(logical) allows introducing correlation/anticorrelation (interprete neg distance results as anti)}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a distance matrix
}
\description{
\code{randIndFx} calculates distance of categorical data (as Rand Index, Adjusted Rand Index or Jaccard Index). 
Note: uses/requires package \href{https://CRAN.R-project.org/package=flexclust}{flexclust}
Methods so far available (via flexclust): "ARI" .. adjusted Rand Index, "RI" .. Rand index, "J" .. Jaccard, "FM" .. Fowlkes-Mallows.
}
\examples{
set.seed(2016); tab2 <- matrix(sample(1:2, size=42, replace=TRUE), ncol=7)
if(requireNamespace("flexclust")) { flexclust::comPart(tab2[1,], tab2[2,])
  flexclust::comPart(tab2[1,], tab2[3,])
  flexclust::comPart(tab2[1,], tab2[4,]) }
## via randIndFx():
  randIndFx(tab2, adjSense=FALSE)
  cor(t(tab2))
  randIndFx(tab2, adjSense=TRUE)
}
\seealso{
\code{comPart} in \code{\link[flexclust]{randIndex}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extrSpcText.R
\name{.cutAtSearch}
\alias{.cutAtSearch}
\title{Search character-string and cut either before or after}
\usage{
.cutAtSearch(
  x,
  searchChar,
  after = TRUE,
  silent = TRUE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{x}{character vector to be treated}

\item{searchChar}{(character) text to look for}

\item{after}{(logical)}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a modified character vector
}
\description{
This function extracts/cuts text-fragments out of \code{txt} following specific anchors defined by arguments \code{cutFrom} and \code{cutTo}.
}
\examples{
.cutAtSearch("abcdefg","de")
}
\seealso{
\code{\link[base]{grep}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkGroupOrder.R
\name{checkGrpOrder}
\alias{checkGrpOrder}
\title{checkGrpOrder}
\usage{
checkGrpOrder(
  x,
  rankExp = NULL,
  revRank = TRUE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{x}{matrix or data.frame}

\item{rankExp}{(numeric) expected order for values in columns, default 'rankExp' =1:ncol(x)}

\item{revRank}{(logical) if 'revRank'=TRUE, the initial ranks & reversed ranks will be tested}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) display additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
vector of logical values
}
\description{
\code{checkGrpOrder} tests each line of 'x' if expected order appears. 
Used for comparing groups of measures with expected profile (simply by mataching expected order)
}
\examples{
set.seed(2005); mat1 <- rbind(matrix(round(runif(40),1),nc=4), rep(1,4))
checkGrpOrder(mat1)
checkGrpOrder(mat1,c(1,4,3,2))
}
\seealso{
\code{\link[wrMisc]{checkGrpOrderSEM}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rowGrpSds.R
\name{.rowGrpSds}
\alias{.rowGrpSds}
\title{row group sd (main)}
\usage{
.rowGrpSds(x, grp)
}
\arguments{
\item{x}{numeric matrix where relplicates are organized into separate columns}

\item{grp}{(factor) defining which columns should be grouped (considered as replicates)}
}
\value{
This function returns a matrix of sd values per row and group of replicates
}
\description{
This function calculates sd for matrix with multiple groups of data, ie one sd for each group of data.
}
\examples{
set.seed(2016); dat1 <- matrix(c(runif(200)+rep(1:10,20)),ncol=10)
grp1 <- gl(4,3,labels=LETTERS[1:4])[2:11]
head(.rowGrpSds(dat1, grp1))
}
\seealso{
\code{\link{rowGrpCV}}, \code{\link{rowCVs}}, \code{\link{arrayCV}}, \code{\link{replPlateCV}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/t.testAllInd.R
\name{tTestAllVal}
\alias{tTestAllVal}
\title{t.test on all individual values against all other values}
\usage{
tTestAllVal(
  x,
  alph = 0.05,
  alternative = "two.sided",
  p.adj = NULL,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{x}{matrix or data.frame}

\item{alph}{(numeric) threshold alpha (passed to \code{t.test})}

\item{alternative}{(character) will be passed to \code{t.test} as argument 'alternative', may be "two.sided",..}

\item{p.adj}{(character) multiple test correction : may be NULL (no correction), "BH","BY","holm","hochberg" or "bonferroni"  (but not 'fdr' since this may be confounded with local false discovery rate), see \code{\link[stats]{p.adjust}}}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a numeric vector with p-values or FDR (depending on argument \code{p.adj})
}
\description{
Run t.test on each indiv value of x against all its neighbours (=remaining values of same vector) in order to test if tis value is likely to belong to vector x. 
This represents a repeated leave-one-out testing. Mutiple choices for multiple testing correction are available.
}
\examples{
set.seed(2016); x1 <- rnorm(100)
allTests1 <- tTestAllVal(x1)
hist(allTests1,breaks="FD")
}
\seealso{
\code{\link[stats]{t.test}}, \code{\link[stats]{p.adjust}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/linRegrParamAndPVal.R
\name{linRegrParamAndPVal}
\alias{linRegrParamAndPVal}
\title{Fit linear regression, return parameters and p-values}
\usage{
linRegrParamAndPVal(
  dep,
  y = NULL,
  asVect = TRUE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{dep}{(numeric vector, matrix or data.frame) explanatory or dependent variable, if matrix or data.frame the 1st column will be used, if 'y'=\code{NULL} the 2nd column will be used as 'y'}

\item{y}{(numeric vector) independent variable (the value that should get estimated based on 'dep')}

\item{asVect}{(logical) return numeric vector (Intercept, slope, p.intercept, p.slope) or matrix or results}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
numeric vector (Intercept, slope, p.intercept, p.slope), or if \code{asVect}==\code{TRUE} as matrix (p.values in 2nd column)
}
\description{
This function fits a linear regression and returns the parameters, including p-values from Anova.
Here the vector 'y' (scalar response or dependent variable, ie the value that should get estimated) will be estimated according to 'dep' (explanatory or independent variable).
Alternatively, 'dep' may me a \code{matrix} where 1st column will be used as 'dep and the 2nd column as 'y'.
}
\examples{
linRegrParamAndPVal(c(5,5.1,8,8.2),gl(2,2))
}
\seealso{
\code{\link[stats]{lm}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mergeSelCol.R
\name{mergeSelCol3}
\alias{mergeSelCol3}
\title{mergeSelCol3}
\usage{
mergeSelCol3(
  dat1,
  dat2,
  dat3,
  selCols,
  supCols2 = NULL,
  supCols3 = NULL,
  byC = NULL,
  useAll = FALSE,
  setRownames = TRUE,
  newSuff = c(".x", ".y", ".z"),
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{dat1}{matrix or data.frame for fusing}

\item{dat2}{matrix or data.frame for fusing}

\item{dat3}{matrix or data.frame for fusing}

\item{selCols}{will be used to define columns to be used; optionally may be different for 'dat2' : define in 'supCols2'}

\item{supCols2}{if additional column-names should be extracted form dat2}

\item{supCols3}{if additional column-names should be extracted form dat3}

\item{byC}{(character) 'by' value used in \code{\link[base]{merge}}}

\item{useAll}{(logical) use all lines (will produce NAs when given identifyer not found un 2nd group of data)}

\item{setRownames}{if \code{TRUE}, will use values of col used as 'by' as rownames instead of showing as add'l col in output}

\item{newSuff}{(character) prefix (argument 'suffixes' in \code{merge})}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) display additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a data.frame containing the merged columns
}
\description{
successive merge of selected columns out of 3 matrix or data.frames.
'selCols' will be used to define columns to be used; optionally may be different for 'dat2' : define in 'supCols2'. 
Output-cols will get additions specified in newSuff (default '.x' and '.y')
}
\examples{
mat1 <- matrix(c(1:7,letters[1:7],11:17),ncol=3,dimnames=list(LETTERS[1:7],c("x1","x2","x3")))
mat2 <- matrix(c(1:6,c("b","a","e","f","g","k"),31:36), ncol=3,
  dimnames=list(LETTERS[11:16],c("y1","x2","x3")))
mat3 <- matrix(c(1:6,c("c","a","e","b","g","k"),51:56), ncol=3,
  dimnames=list(LETTERS[11:16],c("z1","x2","x3")))
mergeSelCol3(mat1, mat2, mat3, selC=c("x2","x3"))
}
\seealso{
\code{\link[base]{merge}}, \code{\link{mergeSelCol}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nonRedundLines.R
\name{nonRedundLines}
\alias{nonRedundLines}
\title{Non-Redundant Lines Of Matrix}
\usage{
nonRedundLines(dat, silent = FALSE, debug = FALSE, callFrom = NULL)
}
\arguments{
\item{dat}{(matrix or data.frame) main input}

\item{silent}{(logical) suppress messages if \code{TRUE}}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allows easier tracking of messages produced}
}
\value{
This function returns a matrix (or data.frame) without repeated lines (keep 1st occurance)..
}
\description{
This function reduces complexity of matrix (or data.frame) if multiple consectuive (!) lines with same values.
Return matrix (or data.frame) without repeated lines (keep 1st occurance)
}
\examples{
mat2 <- matrix(rep(c(1,1:3,3,1),2),ncol=2,dimnames=list(letters[1:6],LETTERS[1:2]))
nonRedundLines(mat2)
}
\seealso{
\code{\link{firstLineOfDat}}, \code{\link{firstOfRepLines}}, \code{\link{findRepeated}}, \code{\link{firstOfRepeated}}, \code{\link{get1stOfRepeatedByCol}}, \code{\link{combineRedBasedOnCol}}, \code{\link{correctToUnique}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/colorAccording2.R
\name{.chooseGrpCol}
\alias{.chooseGrpCol}
\title{Automatic choice of colors}
\usage{
.chooseGrpCol(
  nGrp,
  paired = FALSE,
  alph = 0.2,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{nGrp}{(numeric vector) main input}

\item{paired}{(logical)}

\item{alph}{(numeric vector)}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced#'}
}
\value{
This function returns a character vector with color codes
}
\description{
This function allows to do automatic choice of colors: if single-> grey, if few -> RColorBrewer, if many : gradient green -> grey/red
}
\examples{
.chooseGrpCol(4)	
}
\seealso{
\code{\link[grDevices]{rgb}}; \code{\link{colorAccording2}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/unifyEnumerator.R
\name{unifyEnumerator}
\alias{unifyEnumerator}
\title{Unify Enumerators}
\usage{
unifyEnumerator(
  x,
  refSep = "_",
  baseSep = c("\\\\-", "\\\\ ", "\\\\."),
  suplEnu = c("Repl", "Rep", "R", "Number", "No", "Sample", "Samp"),
  stringentMatch = TRUE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{x}{(character) main input}

\item{refSep}{(character) separator for output}

\item{baseSep}{(character) basic seprators to test (you have to protect special characters)}

\item{suplEnu}{(character) additional text}

\item{stringentMatch}{(logical) decide if enumerator text has to be found in all instances or only once}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) display additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a character vector of same length as input \code{x}, with it's content as adjusted enumerators
}
\description{
The aim of this function is to provide help in automatically harmonizing enumerators at the end of sample-names.
When data have same grouped setup/design, many times this is reflected in their names, eg 'A_sample1', 'A_sample2' and 'B_sample1'.
However, human operators may use multiple similar (but not identical) ways of expressing the same meanin, eg writng 'A_Samp_1'.
This function allows testing a panel of different extensions of enumerators and (if recognized) to replace them by a user-defined standard text/enumerator.
Please note that the more recent function \code{\link{rmEnumeratorName}} offers better/more flexible options.
}
\details{
This function has been developed for matching series of the same samples passing in parallel through different evaluation software (see R package wrProteo).
The way human operators may name things may easily leave room for surprises and this function allows testing only a limited number of common ways of writing.
Thus, in any case, the user is advised to inspect the results by eye and - if needed- to adjust the parameters.

Basically enumerator separators can be constructed by combing a base-separator \code{baseSep} (like '-', '_' etc) and an enumerator-abbreviation \code{suplEnu}.
Then, all possible combinations will be tested if they occur in the text \code{x}.
Furthermore, the text searched has to be followd by on or multiple digts at the end of text-entry (decimal comma-separators etc are not allowed).
Thus, if there is other 'free text' following to the right after the enumerator-text this function will not find any enumerators to replace.

The argument \code{stringentMatch} allows defining if this text has to be found in all text-entries of \code{x} or just one of them.
Whe using \code{stringentMatch=FALSE} there is risk that other text not meant to design enumerators may be picked up and modified.

Please note, that with large data-sets (ie many columns) testing/checking a larger panel of enumerator-abreviations may result in slower performance.
In cases of larger data-sets it may be more effective to first study the data and then run simple subsitions using sub targeted for this very case.
}
\examples{
unifyEnumerator(c("ab-1","ab-2","c-3"))
unifyEnumerator(c("ab-R1","ab-R2","c-R3"))
unifyEnumerator(c("ab-1","c3-2","dR3"), strin=FALSE);

}
\seealso{
\code{\link{rmEnumeratorName}} for better/more flexible options; \code{\link[base]{grep}} or \code{sub()}, etc if exact and consistent patterns are known
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sampNoDeMArrayLM.R
\name{sampNoDeMArrayLM}
\alias{sampNoDeMArrayLM}
\title{Locate Sample Index From Index Or Name Of Pair-Wise Comparisons In List Or MArrayLM-Object}
\usage{
sampNoDeMArrayLM(
  MArrayObj,
  useComp = NULL,
  outTy = "index",
  groupSep = NULL,
  combPwSep = "combine1",
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{MArrayObj}{(list or MArray-object) main input, object to examine and extract pairwise question information}

\item{useComp}{(character or integer, length=1) index or name of pairwise-comparison to be addressed}

\item{outTy}{(character) choose if just vector of indexes (with \code{outTy="index0"}), matrix of indexes (organized by pairwise questions, with \code{outTy="index"}), 
or matrix of group-names (with \code{outTy="names"}) or list with $index0 (osrted unique vector or indexes),  $pwGrpNa, $pwIndex, $concat, $sep and $compIndex will get returned}

\item{groupSep}{(NULL or character of length=1) separator for pair of names; if \code{NULL} a suitable separator (among '-','--','_','__','.',' ' and '  ') will be used}

\item{combPwSep}{(character) sequence of separators for building combined names (may be combPwSep='combine1' or 'combine2'); will be used in pwSeparatorList()}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns if \code{outTy="names"} a numeric vector with indexes indicating the sorted (!!) columns of (replicate) mean-values corresponding to the comparison specified in \code{useComp};
  or if \code{outTy="names"} or  \code{outTy="pwGrpNa"} :  the (separated) names of pairwise groups will be returned as matrix (each pairwise question as separate line) 
  or if \code{outTy="list"} a list with $index0 (sorted unique vector or indexes), $pwGrpNa (the separated pairwise group-names), $pwIndex (the ith pairwise comparison), $concat (the concatenated pairwise names), $sep (pairwise separator) and $compIndex
}
\description{
When multiple series of data are tested simultaneaously (eg using \code{moderTestXgrp}), multiple pairwise comparisons get performed. 
This function helps locating the groups of samples, eg respective mean-columns, corresponding to specific pairwise comparisons.
}
\details{
As main input one gives a list or MArrayLM-object containing testing results from pairwise comparisons,
and specific comparisons indicated by \code{useComp} to get located in the elements of mean-columns (\code{lstMeans}).

Note, that that the numeric indexes should be used with caution as they refer to the levels of the groups which are sorted
and thus not necessarily in the order of other elements like the group-means. 

This function allows finding inverted questions (eg 'D-C' even though 'C-D' was actually run in testing), but the result will always be presented in the initial order
}
\examples{
grp3 <- factor(rep(LETTERS[4:2], c(2,3,3)))
set.seed(2017); t8 <- matrix(round(rnorm(208*8,10,0.4), 2), ncol=8,
  dimnames=list(paste(letters[],rep(1:8,each=26),sep=""), paste0(grp3,c(1:2,1:3,1:3))))
if(requireNamespace("limma", quietly=TRUE)) {  # need limma installed...
  test8 <- moderTestXgrp(t8, grp3, useComparison="all") 
  head(test8$p.value)         # all pairwise comparisons available
  sampNoDeMArrayLM(test8, 3)   
  levels(grp3)[sampNoDeMArrayLM(test8, 3)]
  head(test8$means[,sampNoDeMArrayLM(test8, 3, outTy="names")], 3)      
  head(test8$means[,sampNoDeMArrayLM(test8, "D-C", outTy="names")], 3) # inverted question
} 
}
\seealso{
\code{\link{moderTestXgrp}}, this function gets used eg in \code{\link[wrGraph]{MAplotW}} or \code{\link[wrGraph]{VolcanoPlotW}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/diffCombin.R
\name{diffCombin}
\alias{diffCombin}
\title{Compute Matrix Of Differences For All Pairwise Combinations Of Numeric Vector}
\usage{
diffCombin(
  x,
  diagAsNA = FALSE,
  prefix = TRUE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{x}{numeric vector to compute differences for all combinations}

\item{diagAsNA}{(logical) return all self-self combinations as NA (otherwise 0)}

\item{prefix}{(logical) if TRUE, dimnames of output will specify orientation (prefix='from.' and 'to.')}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allows easier tracking of messages produced}
}
\value{
This function returns a numeric matrix of all pairwise differences
}
\description{
\code{diffCombin} returns matrix of differences (eg resulting from subsititution) for all pairwise combinations of numeric vector 'x'.
}
\examples{
diffCombin(c(10,11.1,13.3,16.6))
}
\seealso{
\code{\link[base]{diff}} for simple differences
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/searchLinesAtGivenSlope.R
\name{.predRes}
\alias{.predRes}
\title{Calculate residues of (2-dim) linear model 'lMod'-prediction of/for 'dat'}
\usage{
.predRes(dat, lMod, regTy = "lin", useCol = 1:2)
}
\arguments{
\item{dat}{matrix or data.frame, main input}

\item{lMod}{linear model, only used to extract coefficients offset & slope}

\item{regTy}{(character) type of regression model}

\item{useCol}{(integer) columns to use}
}
\value{
This function returns a numeric vector of residues (for each line of dat)
}
\description{
This function calculates residues of (2-dim) linear model 'lMod'-prediction of/for 'dat' (using 2nd col of 'useCol' )
 (indexing in 'dat', matrix or data.frame with min 2 cols), using 1st col of 'useCol' as 'x'.
It may be used for comparing/identifying data close to regression  (eg re-finding data on autoregression line in FT-ICR)
}
\examples{
set.seed(2016); dat1 <- matrix(c(runif(200)+rep(1:10,20)),ncol=10)
}
\seealso{
\code{\link{searchLinesAtGivenSlope}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chooseGroupNames.R
\name{chooseGroupNames}
\alias{chooseGroupNames}
\title{Choose Column Most Likely For Sample-Names}
\usage{
chooseGroupNames(
  mat,
  useCoNa = NULL,
  method = "median",
  sep = c("_", "-", " ", ".", "=", ";"),
  rmTxt = NULL,
  asUnique = TRUE,
  partEnumerator = FALSE,
  fullReport = FALSE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{mat}{(matrix or data.frame) contains possible choices for sample-names}

\item{useCoNa}{(character) optional custom choice for columns of \code{mat} to check; if \code{NULL} all columns will be used/checked}

\item{method}{(character) decide how to choose number of groups as : min, low, med, high, max or mode
Note arguments  \code{asUnique} and \code{partEnumerator} influence which columns of \code{mat} will be evaluated/checked}

\item{sep}{(character) separators considered when searching and removing common words}

\item{rmTxt}{(character, length=1) optional removing of custom text (eg variable file-extensions); no obligation that \code{rmTxt} occurs in all instances}

\item{asUnique}{(logical) requires all (potential) samples-names to be unique (ie no repeats) to be considered for group-names; also removes all candidate columns with all different names}

\item{partEnumerator}{(logical) when \code{TRUE} allows some instances of (potential) sample-names without numerator: a1, a2, b (ie some wo enumerator)}

\item{fullReport}{(logical) if \code{TRUE} returns list with $group, $sampleNames, $col (iondex of column from mat and name of c)}

\item{silent}{(logical) suppress messages if \code{TRUE}}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allows easier tracking of messages produced}
}
\value{
This function returns a character vector with grouop-names (and sample-names as names of entries) or 
  if \code{fullReport=TRUE} a list with $group, $sampleNames, $col (index of column from \code{mat} and name of column
}
\description{
This function looks at all comumns of  \code{mat} which columns may be likely choices for sample-names and derives then group-names after stripping terminal enumerators.
Ideal sample-names should contain some replicates indicates as terminal enumerators.
}
\details{
The basic idea is that the column containing (good) samples-names contains all different entries and that by stripping terminal enumerators one can understand the grouping of replicates.
Note arguments  \code{asUnique} and \code{partEnumerator} influence which columns of \code{mat} will be evaluated/checked
}
\examples{
 mat <- cbind(a=letters[1:6], b=paste(rep(c("b","B"), each=3), 1:3), c=rep(1,6), 
   d=gl(3,2), e=rep(c("e","E"),3), f=paste(rep(c("F","f","ff"), each=2), 1:2))
chooseGroupNames(mat, method="median")         # col 2 (b/B)
chooseGroupNames(mat, method="median", fullReport=TRUE) 
chooseGroupNames(mat, method="min")            # col 2 (b/B)
chooseGroupNames(mat, method="max")            # col 6 (F/f/ff)
chooseGroupNames(mat, method="max", asUnique=FALSE) # col 1 (a..)
}
\seealso{
\code{\link{rmSharedWords}}, \code{\link{replicateStructure}}, \code{\link{protectSpecChar}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/presenceGrpFilt.R
\name{presenceGrpFilt}
\alias{presenceGrpFilt}
\title{Filter For Each Group Of Columns For Sufficient Data As Non-NA}
\usage{
presenceGrpFilt(
  dat,
  grp,
  presThr = 0.75,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{dat}{matrix or data.frame (abundance or expression-values which may contain some \code{NA}s).}

\item{grp}{factor of min 2 levels describing which column of 'dat' belongs to which group (levels 1 & 2 will be used)}

\item{presThr}{(numeric) min ratio of non- \code{NA} values (per group) for returning a given line & group as  \code{TRUE}}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
logical matrix (with on column for each level of \code{grp})
}
\description{
The aim of this function is to filter for each group of columns for sufficient data as non-NA.
}
\details{
This function allows to identify lines with an \code{NA}-content above the threshold \code{presThr} per group as defined by the levels of factor \code{grp}.  
With different types of projects/questions different threshold  \code{presThr} levels may be useful.
For example, if one would like to keep the degree of threshold  \code{presThr}s per group rather low, one could use a value of 0.75 (ie >= 75% values non-NA.
}
\examples{
 mat <- matrix(NA, nrow=11, ncol=6)
 mat[lower.tri(mat)] <- 1
 mat <- cbind(mat, mat[,1:4])
 colnames(mat) <- c(paste0("re",1:6), paste0("x",1:4))
 mat[6:8,7:10] <- mat[1:3,7:10]  # ref
 mat[9:11,1:6] <- mat[2:4,1:6]

##  accept 1 NA out of 4, 2 NA out of 6   (ie certainly present)
 (filt0a <- presenceGrpFilt(mat, rep(1:2, c(6,4)), pres=0.66))
##  accept 2 NA out of 4, 2 NA out of 6   (ie min 50\% present)
 (filt0b <- presenceGrpFilt(mat, rep(1:2, c(6,4)), pres=0.5))
##  accept 3 NA out of 4, 4 NA out of 6   (ie possibly present)
 (filt0c <- presenceGrpFilt(mat, rep(1:2, c(6,4)), pres=0.19))

}
\seealso{
\code{\link{presenceFilt}}, there are also other packages totaly dedicated to filtering on CRAN and Bioconductor
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/singleLineAnova.R
\name{singleLineAnova}
\alias{singleLineAnova}
\title{2-factorial Anova on single line of data}
\usage{
singleLineAnova(
  dat,
  fac1,
  fac2,
  inclInteraction = TRUE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{dat}{numeric vector}

\item{fac1}{(character or factor) vector describing grouping elements of dat for first factor, must be of same langth as fac2}

\item{fac2}{(character or factor) vector describing grouping elements of dat for second factor, must be of same langth as fac1}

\item{inclInteraction}{(logical) decide if factor-interactions (eg synergy) should be included to model}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns the (uncorrected) p for factor 'Pr(>F)' (see \code{\link[stats]{aov}})
}
\description{
This function runs 2-factorial Anova on a single line of data (using \code{\link[stats]{aov}} from package \code{stats}) 
using a model with two factors (without factor-interaction) and extracts the correpsonding p-value.
}
\examples{
set.seed(2012); dat <- round(runif(8),1)
singleLineAnova(dat, gl(2,4),rep(1:2,4))
}
\seealso{
\code{\link[stats]{aov}}, \code{\link[stats]{anova}}; for repeated tests using the  package \href{https://bioconductor.org/packages/release/bioc/html/limma.html}{limma} including \code{\link[limma]{lmFit}} and \code{eBayes} see \code{\link{test2factLimma}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ratioAllComb.R
\name{.removeCol}
\alias{.removeCol}
\title{Remove columns indicated by col-number}
\usage{
.removeCol(matr, rmCol)
}
\arguments{
\item{matr}{(matrix or data.frame) main input}

\item{rmCol}{(integer) column index for removing}
}
\value{
This function returns an matrix or data.frame
}
\description{
This function aims to remove columns indicated by col-number
}
\examples{
aa <- matrix(1:6, ncol=3)
.removeCol(aa, 2)
}
\seealso{
\code{\link[stats]{dist}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rowGrpNA.R
\name{rowGrpNA}
\alias{rowGrpNA}
\title{Count number of NAs per row and group of columns}
\usage{
rowGrpNA(mat, grp)
}
\arguments{
\item{mat}{(matrix of data.frame) data to count the number of \code{NA}s}

\item{grp}{(character or factor) defining which columns should be grouped (considered as replicates)}
}
\value{
matrix with number of \code{NA}s per group
}
\description{
This functions allows easy counting the number of NAs per row in data organized in multiple sub-groups as columns.
}
\examples{
mat2 <- c(22.2, 22.5, 22.2, 22.2, 21.5, 22.0, 22.1, 21.7, 21.5, 22, 22.2, 22.7,
   NA, NA, NA, NA, NA, NA, NA, 21.2,   NA, NA, NA, NA,
   NA, 22.6, 23.2, 23.2,  22.4, 22.8, 22.8, NA,  23.3, 23.2, NA, 23.7,
   NA, 23.0, 23.1, 23.0,  23.2, 23.2, NA, 23.3,  NA, NA, 23.3, 23.8)
mat2 <- matrix(mat2, ncol=12, byrow=TRUE)
gr4 <- gl(3, 4, labels=LETTERS[1:3])
# overal number of NAs per row
rowSums(is.na(mat2)) 
# number of NAs per row and group
rowGrpNA(mat2, gr4)
}
\seealso{
\code{\link{rowGrpMeans}}, \code{\link{rowSds}}, \code{\link[base]{colSums}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/firstOfRepLines.R
\name{.extrNAneighb}
\alias{.extrNAneighb}
\title{Extract NA-neighbour values}
\usage{
.extrNAneighb(x, grp)
}
\arguments{
\item{x}{initial matrix to treat}

\item{grp}{(factor) grouing of replicates}
}
\value{
This function returns a numeric vector
}
\description{
This function allows extracting NA-neighbour value
}
\examples{
.extrNAneighb(c(11:14,NA), rep(1,5))
}
\seealso{
\code{\link[base]{unique}}, \code{\link{nonAmbiguousNum}}, faster than \code{\link{firstOfRepeated}} which gives more detail in output (lines/elements/indexes of omitted)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fusePairs.R
\name{.neigbDis}
\alias{.neigbDis}
\title{Distances beteenw sorted points of 2-columns}
\usage{
.neigbDis(x, asSum = TRUE)
}
\arguments{
\item{x}{(matrix or data.frame, min 2 columns) main input}

\item{asSum}{(logical) if \code{TRUE} (default) the sum of all distances will be returned, otherwise the individual distances}
}
\value{
This function returns a numeric vector with distances
}
\description{
This function returns distances beteenw sorted points of 2-column matrix 'x'
}
\examples{
daPa <- matrix(c(1:5,8,2:6,9), ncol=2)
.neigbDis(daPa)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nFragments.R
\name{nFragments0}
\alias{nFragments0}
\title{Number Of Fragments After Cut At Specific Character(s)}
\usage{
nFragments0(protSeq, cutAt, silent = FALSE, debug = FALSE, callFrom = NULL)
}
\arguments{
\item{protSeq}{(character) text to be cut}

\item{cutAt}{(integer) position to cut}

\item{silent}{(logical) suppress messages if \code{TRUE}}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allows easier tracking of messages produced}
}
\value{
This function returns a numeric vector with number of fragments for each entry 'protSeq' (names are 'protSeq')
}
\description{
This function tells the number of fragments/entry when cutting after 'cutAt'
}
\examples{
tmp <- "MSVSRTMEDSCELDLVYVTERIIAVSFPSTANEENFRSNLREVAQMLKSKHGGNYLLFNLSERRPDITKLHAKVLEFGWPDLHTPALEKI" 
nFragments0(c(tmp,"ojioRij"),c("R","K"))
}
\seealso{
more elaborate \code{{nFragments}}; \code{\link{cutAtMultSites}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/replicateStructure.R
\name{replicateStructure}
\alias{replicateStructure}
\title{Search and Select Groups of Replicates}
\usage{
replicateStructure(
  x,
  method = "median",
  sep = "__",
  exclNoRepl = TRUE,
  trimNames = FALSE,
  includeOther = FALSE,
  silent = FALSE,
  callFrom = NULL,
  debug = FALSE
)
}
\arguments{
\item{x}{(matrix or data.frame) the annotation to inspect; each column is supposed to describe another set of annoation/metadata for the rows of \code{x} (min 1 row and 1 column),}

\item{method}{(character, length=1) the procedure to choose column(s) with properties of information, may be \code{highest} or \code{max} (max number of levels)
\code{lowest} or \code{min} (min number of levels), \code{median} (median of all options for number of levels),
\code{combAll} (combine all columns of \code{x}) or \code{combNonOrth} (combine only non-orthogonal columns of \code{x}, to avoid avoid n lines with n levels);
lazy evluation of the argument is possible}

\item{sep}{(character) separator used when a method combining multiple columns (eg combAll, combNonOrth)  is chosen (should not appear anywhere in \code{x})}

\item{exclNoRepl}{(logical) decide whether columns with all values different (ie no replicates or max divergency) should be excluded}

\item{trimNames}{(logical) optional trimming of names in \code{x} by removing redundant heading and tailing text}

\item{includeOther}{(logical) include $allCols with pattern of (all) other columns}

\item{silent}{(logical) suppress messages}

\item{callFrom}{(character) allow easier tracking of messages produced}

\item{debug}{(logical) additional messages for debugging}
}
\value{
This function returns a list with $col (column index relativ to \code{x}), $lev (abstract labels of level),
  $meth (note of method finally used) and $allCols with general replicate structure of all columns of  \code{x}
}
\description{
This function was designed for mining annotation information organized in multiple columns to identify the (potential) grouping of multiple samples, ie to determine factor levels.
The argument \code{method} allows further finetuning if high or low number of groups should be preferred, if multiple columns may be combined, or to choose a particular custom column for desiganting factor levels.
}
\details{
Statistical tests require specifying which samples should be considered as replicates of whom.
In some cases, like the Sdrf-format, automatic mining of such annotation to indentify an experiment's underlying structure of replicates
may be challanging, since the key information may not always be found in the same column.
For this reason this function allows inspecting all columns of a matrix of data.frame to identify which colmns may serve describing groups of replicates.

The argument \code{exclNoRepl=TRUE} allows excluding all columns with different content for each line (like line-numbers), ie information without any replicates.
It is set by default to \code{TRUE} to exclude such columns, since statistical tests usually do require some replicates.

When using as \code{method="combAll"}, there is risk all lines (samples) will be be considered different and no replicates remain.
To avoid this situation the argument can be set to \code{method="combNonOrth"}.
Using this mode it will be checked if  more columns will lead to complete loss of replicates, and -if so- concerned columns omitted.
}
\examples{
## a is all different, b is groups of 2,
## c & d  are groups of 2 nut NOT 'same general' pattern as b
strX <- data.frame(a=letters[18:11], b=letters[rep(c(3:1,4), each=2)],
 c=letters[rep(c(5,8:6), each=2)], d=letters[c(1:2,1:3,3:4,4)],
 e=letters[rep(c(4,8,4,7),each=2)], f=rep("z",8) )
strX
replicateStructure(strX[,1:2])
replicateStructure(strX[,1:4], method="combAll")
replicateStructure(strX[,1:4], method="combAll", exclNoRepl=FALSE)
replicateStructure(strX[,1:4], method="combNonOrth", exclNoRepl=TRUE)
replicateStructure(strX, method="lowest")
replicateStructure(strX, method=3, includeOther=TRUE)   # custom choice of 3rd column



}
\seealso{
\code{\link[base]{duplicated}}, uses \code{\link{trimRedundText}}; for chosing single most likely column describing sample-names and groups of replicates \code{\link{chooseGroupNames}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/correctToUnique.R
\name{correctToUnique}
\alias{correctToUnique}
\title{Correct vector to unique}
\usage{
correctToUnique(
  x,
  sep = "_",
  atEnd = TRUE,
  maxIter = 4,
  NAenum = TRUE,
  silent = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{x}{input character vector}

\item{sep}{(character) separator used when  counter}

\item{atEnd}{(logical) decide location of placing the counter (at end or at beginning of initial text)}

\item{maxIter}{(numeric) max number of iterations}

\item{NAenum}{(logical) if \code{TRUE} all \code{NA}s will be enumerated (NA_1,NA_2,...)}

\item{silent}{(logical) suppress messages}

\item{callFrom}{(character) for better tracking of use of functions}
}
\value{
This function returns a character vector
}
\description{
\code{correctToUnique} checks 'x' for unique entries, while maintaining the original length. If necessary a counter will added to non-unique entries.
}
\examples{
correctToUnique(c("li0","n",NA,NA,rep(c("li2","li3"),2),rep("n",4))) 
}
\seealso{
\code{\link[base]{unique}} will simply remove repeated elements, ie length of 'x' won't remain constant, \code{\link{filtSizeUniq}} is more complex and slower, \code{\link{treatTxtDuplicates}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convMatr2df.R
\name{.mayBeNum}
\alias{.mayBeNum}
\title{Check if vector may be numeric content}
\usage{
.mayBeNum(x, pattern = NULL)
}
\arguments{
\item{x}{(numeric vector) main input}

\item{pattern}{(character)  custom pattern to check}
}
\value{
This functions returns a logical/boolean vector for each of the elements of 'x'
}
\description{
This function allows to checking if a given vector may be numeric content
}
\examples{
.mayBeNum(c(3:6))
}
\seealso{
\code{\link[base]{numeric}}; \code{\link{convMatr2df}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/numPairDeColNames.R
\name{numPairDeColNames}
\alias{numPairDeColNames}
\title{Extract pair of numeric values from vector or column-names}
\usage{
numPairDeColNames(
  dat,
  selComp = NULL,
  stripTxt = NULL,
  sep = "-",
  columLabel = "conc",
  sortByAbsRatio = FALSE,
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{dat}{(matrix or data.frame) main input}

\item{selComp}{(character) the column index selected}

\item{stripTxt}{(character, max length=2) text to ignore, if NULL heading letter and punctuation characters will be removed; default will remove all letters (and following spaces)}

\item{sep}{(character, length=1) separator between pair of numeric values to extract}

\item{columLabel}{(character) column labels in output}

\item{sortByAbsRatio}{(logical) optional sorting of output by (absolute) log-ratios (most extreme ratios on top)}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns a matrix
}
\description{
This function extracts a pair of numeric values out of a vector or colnames (from a matrix). 
This is useful when pairwise comparisons are concatenated like '10c-100c', return matrix with 'index'=selComp, log2rat and both numeric.
Additional white space or character text can be removed via the argument \code{stripTxt}.
Of course, the separator \code{sep} needs to be specified and should not be included to 'stripTxt'.
}
\examples{
##  composed column names
mat1 <- matrix(1:8, nrow=2, dimnames=list(NULL, paste0(1:4,"-",6:9)))
numPairDeColNames(mat1)
numPairDeColNames(colnames(mat1))
##  works also with simple numeric column names
mat2 <- matrix(1:8, nrow=2, dimnames=list(NULL, paste0("a",6:9)))
numPairDeColNames(mat2)
}
\seealso{
\code{\link[base]{strsplit}} and help on regex
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multiCharReplace.R
\name{multiCharReplace}
\alias{multiCharReplace}
\title{Multiple replacement of entire character elements in simple vector, matrix or data.frame}
\usage{
multiCharReplace(
  mat,
  repl,
  convTo = NULL,
  silent = FALSE,
  debug = TRUE,
  callFrom = NULL
)
}
\arguments{
\item{mat}{(character vector, matrix or data.frame) main data}

\item{repl}{(matrix or list) tells what to replace by what: If matrix the 1st oolumn will be considered as 'old' and the 2nd as 'replaceBy'; if named list, the names of the list-elements will be consdered as 'replaceBy'}

\item{convTo}{(character) optional conversion of content to 'numeric' or 'logical'}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of messages produced}
}
\value{
This function returns an object of same dimension as input (with replaced content)
}
\description{
This functions allows multiple types of replacements of entire character elements in simple vector, matrix or data.frame.
In addtion, the result may be optionally directly transformed to logical or numeric
}
\examples{
x1 <- c("ab","bc","cd","efg","ghj")
multiCharReplace(x1, cbind(old=c("bc","efg"), new=c("BBCC","EF")))

x2 <- c("High","n/a","High","High","Low")
multiCharReplace(x2, cbind(old=c("n/a","Low","High"), new=c(NA,FALSE,TRUE)),convTo="logical")

# works also to replace numeric content :
x3 <- matrix(11:16, ncol=2)
multiCharReplace(x3, cbind(12:13,112:113))
}
\seealso{
\code{\link[base]{grep}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rowSEMs.R
\name{rowSEMs}
\alias{rowSEMs}
\title{SEM for each row}
\usage{
rowSEMs(dat)
}
\arguments{
\item{dat}{matrix or data.frame}
}
\value{
This function returns a numeric vector with SEM values
}
\description{
This function speed optimized SEM (standard error of the mean) for each row.
The function takes a matrix or data.frame and treats each row as set of data for SEM; NAs are ignored from data.
Note: NaN instances will be transformed to NA
}
\examples{
set.seed(2016); dat1 <- matrix(c(runif(200)+rep(1:10,20)),ncol=10)
head(rowSEMs(dat1))
}
\seealso{
\code{\link{rowSds}},  \code{\link{colSds}}, \code{\link[base]{colSums}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/combineOverlapInfo.R
\name{combineOverlapInfo}
\alias{combineOverlapInfo}
\title{Find And Combine Points Located Very Close In x/y Space}
\usage{
combineOverlapInfo(
  dat,
  suplInfo = NULL,
  disThr = 0.01,
  addNsimil = TRUE,
  txtSepChar = ",",
  combSym = "+",
  maxOverl = 50,
  callFrom = NULL,
  debug = FALSE,
  silent = FALSE
)
}
\arguments{
\item{dat}{(matrix) matrix or data.frame with 2 cols (used ONLY 1st & 2nd column !), used as x & y coordinates}

\item{suplInfo}{(NULL or character) when points are considered overlapping the text from 'suplInfo' will be reduced to fragment before 'txtSepChar' and combined (with others from overlapping text) using 'combSym', if NULL $combInf will appear with row-numbers}

\item{disThr}{(numeric) distance-thrshold for considering as similar via searchDataPairs()}

\item{addNsimil}{(logical) include number of fused points}

\item{txtSepChar}{(character) for use with .retain1stPart(): where to cut (& keep 1st part) text from 'suplInfo' to return in out$CombInf; only 1st element used !}

\item{combSym}{(character) concatenation symbol (character, length=1) for points considered overlaying, see also 'suplInfo'}

\item{maxOverl}{(integer) if NULL no limit or max limit of group/clu size (avoid condensing too many neighbour points to single cloud)}

\item{callFrom}{(character) allow easier tracking of messages produced}

\item{debug}{(logical) additional messages for debugging}

\item{silent}{(logical) suppres messages}
}
\value{
This function returns a matrix with fused (condensed) information for cluster of overapping points
}
\description{
Search points in x,y space that are located very close and thus likely to overlap.
In case of points close enough, various options for joining names (and shortening longer descriptions) are available.
}
\examples{
set.seed(2013)
datT2 <- matrix(round(rnorm(200)+3,1),ncol=2,dimnames=list(paste("li",1:100,sep=""),
  letters[23:24]))
# (mimick) some short and longer names for each line
inf2 <- cbind(sh=paste(rep(letters[1:4],each=26),rep(letters,4),1:(26*4),sep=""),
 lo=paste(rep(LETTERS[1:4],each=26),rep(LETTERS,4),1:(26*4),",",rep(letters[sample.int(26)],4),
  rep(letters[sample.int(26)],4),sep=""))[1:100,] 
head(datT2,n=10)
head(combineOverlapInfo(datT2,disThr=0.03),n=10)
head(combineOverlapInfo(datT2,suplI=inf2[,2],disThr=0.03),n=10)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multiMatch.R
\name{multiMatch}
\alias{multiMatch}
\title{Simple Multi-to-Multi Matching of (Concatenated) Terms}
\usage{
multiMatch(
  x,
  y,
  sep = "; ",
  sep2 = NULL,
  method = "byX",
  silent = FALSE,
  debug = FALSE,
  callFrom = NULL
)
}
\arguments{
\item{x}{(vector or list) first object to compare; if vector, the (partially) concatenated identifyers (will be split using separator \code{sep}), or list of items to be matched (ie already split)}

\item{y}{(vector or list) second object to compare; if vector, the (partially) concatenated identifyers (will be split using separator \code{sep}), or list of items to be matched (ie already split)}

\item{sep}{(character, length=1) separator used to split concatenated identifyers (if \code{x} or  \code{y} is vector)}

\item{sep2}{(character, length=1) optional separator used when \code{method="matched"} to concatenate all indexes of \code{y} for column \code{y.allInd}}

\item{method}{(character) mode of operation: 'asIndex' to return index of y (those hwo have matches) with names of x (which x are the correpsonding match)}

\item{silent}{(logical) suppress messages}

\item{debug}{(logical) display additional messages for debugging}

\item{callFrom}{(character) allow easier tracking of message(s) produced}
}
\value{
matrix, data.frame or list with matching results depending on \code{method} chosen
}
\description{
This function allows convenient matching of multi-to-multi relationships between two objects/vectors. 
It was designed for finding common elements in multiple to multiple matching situations (eg when comparing \code{c("aa; bb", "cc")}  to \code{c("bb; ab","dd")}, 
ie to find 'bb' as matching between both objects).
}
\details{
\code{method='byX'} .. returns data.frame with view oriented towards entries of \code{x}: character column \code{x} for entire content of \code{x}; integer column \code{x.Ind} for index of \code{x}; 
 character column \code{TagBest} for most frequent matching isolated tag/ID; integer column \code{y.IndBest} index of most frequent matching \code{y};
 character column \code{y.IndAll} index for all \code{y} matching any of the tags;
 character column \code{y.Match} for entire content of best matching \code{y}; 
 character column \code{y.Adj} for \code{y} adjusted to best matching \code{y} for easier subsequent perfect matching.
                                                                                                                                                      
\code{method=c("byX","filter")} .. combinded argument to keep only lines with any matches 
                                                                                                                                                      
\code{method='byTag'} .. returns matrix (of integers) from view of isolated tags from \code{x} (a separate line for each tag from  \code{x} matching to \code{y});

\code{method=c("byTag","filter")} ..if combined as arguments, this will return a data.frame for all unique tags with any matches between \code{x} and \code{y}, with 
  additional colunms \code{x.AllInd} for all matching \code{x}-indexes,  \code{y.IndBest} best matching \code{y} index; \code{x.n} for number of different \code{x} conatining this tag;
\code{y.AllInd} for all matching \code{y}-indexes 

\code{method='adjustXtoY'} .. returns vector with \code{x} adjusted to \code{y}, ie those elements of \code{x} matching are replace by the exact corresponding term of \code{y}.
                                                                                                                                                      
\code{method=NULL} .. If no term matching the options shown above is given, another version of 'asIndex' is returned, but indexes to \code{y} _after_ spliting by \code{sep}. 
Again, this method can be filtered by using \code{method="filter"} to focus on the best matches to \code{x}.
}
\examples{
aa <- c("m","k", "j; aa", "m; aa; bb; o; ee", "n; dd; cc", "aa", "cc")
bb <- c("dd; r", "aa", "ee; bb; q; cc", "p; cc")
(match1 <- multiMatch(aa, bb, method=NULL))      # match bb to aa
(match2 <- multiMatch(aa, bb, method="byX"))     # match bb to aa
(match3 <- multiMatch(aa, bb, method="byTag"))   # match bb to aa
(match4 <- multiMatch(aa, bb, method=c("byTag","filter")))   # match bb to aa

}
\seealso{
\code{\link[base]{match}}; \code{\link[base]{strsplit}}
}
